#include <misc.h>

subroutine bandi(plon, dlam    ,lamp    ,iband   , nlon, beglat, endlat, &
                 nlonex,   fullgrid  )

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Calculate longitude and latitude indices that identify the
! intervals on the extended grid that contain the departure points.
! Upon entry, all dep. points should be within jintmx intervals of the
! Northern- and Southern-most model latitudes. Note: the algorithm
! relies on certain relationships of the intervals in the Gaussian grid.
! 
! Method: 
!  dlam    Length of increment in equally spaced longitude grid (rad.)
!  phib    Latitude values for the extended grid.
!  lamp    Longitude coordinates of the points.  It is assumed that
!                     0.0 .le. lamp(i) .lt. 2*pi .
!  phip    Latitude coordinates of the points.
!  iband   Longitude index of the points.  This index points into
!          the extended arrays, e.g.,
!                   lam(iband(i)) .le. lamp(i) .lt. lam(iband(i)+1) .
!  jband   Latitude index of the points.  This index points into
!          the extended arrays, e.g.,
!                   phib(jband(i)) .le. phip(i) .lt. phib(jband(i)+1) .
! 
! Author: J. Olson
! 
!-----------------------------------------------------------------------
!
! $Id: bandi.F90,v 1.1.1.1 2014/08/26 01:56:31 wangzzh Exp $
! $Author: wangzzh $
!
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  implicit none

!------------------------------Arguments--------------------------------
  integer , intent(in)  :: plon, beglat, endlat     ! number of longitudes
  integer , intent(in)  :: nlonex(beglat:endlat)    ! number of longitudes
  real(r8), intent(in)  :: dlam(beglat:endlat)        ! longitude increment
  real(r8), intent(in)  :: lamp(plon)    ! longitude coordinates of dep. points
  integer , intent(in)  :: nlon               ! number of longitudes
  logical fullgrid          ! true => no grid reduction towards poles
  integer , intent(out) :: iband(plon,beglat:endlat) ! longitude index of dep. points
!-----------------------------------------------------------------------
!
!---------------------------Local workspace-----------------------------
!
  integer i,j,k             ! indices
  real(r8) rdlam(beglat:endlat)     ! reciprocal of longitude increment
!
!-----------------------------------------------------------------------
!
  do j = beglat,endlat
     rdlam(j) = 1./dlam(j)
  end do
!
! Loop over level and longitude

! Latitude indices.
!
! do j = jcen-ng, jcen+ng
  do j = beglat,endlat
!
     if (fullgrid.or.nlonex(j)==nlon) then
        do i = 1,nlon
           iband(i,j) = i
        end do
     else
! Longitude indices.
!
        do i = 1,nlon
           iband(i,j) = 1 + int( lamp(i)*rdlam(j))
!        if(i==nlon-1.or.i==nlon) then
!         write(*,*)'wzz: iband=',iband(i,j), lamp(i), rdlam(j), dlam(j),j
!        endif
        end do
     end if
  end do

  return
end subroutine bandi
