   module dynamics_vars

   use shr_kind_mod,       only: r8 => shr_kind_r8, r4 => shr_kind_r4

   implicit none

   public T_FVDYCORE_VARS, T_FVDYCORE_GRID, T_FVDYCORE_STATE

! T_FVDYCORE_VARS contains the prognostic variables for FVdycore
  type T_FVDYCORE_VARS
       real(r8), dimension(:,:,:  ), pointer     :: U      ! U winds (D-grid)
       real(r8), dimension(:,:,:  ), pointer     :: V      ! V winds (D-grid)
       real(r8), dimension(:,:,:  ), pointer     :: PT     ! scaled virtual pot. temp.
       real(r8), dimension(:,:,:  ), pointer     :: PE     ! Pressure at layer edges
       real(r8), dimension(:,:,:  ), pointer     :: PKZ    ! P^kappa mean
       real(r8), dimension(:,:,:,:), pointer     :: tracer ! Tracers
  end type T_FVDYCORE_VARS

! T_FVDYCORE_GRID contains information about the horizontal and vertical
! discretization, unlike in ARIES where these data are split into HORZ_GRID
! and VERT_GRID.  The reason for this: currently all of this information is
! initialized in one call to FVCAM dynamics_init.
 
  type T_FVDYCORE_GRID

!
! PILGRIM communication information (was in spmd_dyn)
!
    integer :: twod_decomp = 0  ! 1 for multi-2D decompositions, 0 otherwise

    integer :: npes_xy= 1    ! number of PEs for XY decomposition
    integer :: npes_yz= 1    ! number of PEs for YZ decomposition
    integer :: myid_y = 0    ! subdomain index (0-based) in latitude (y)
    integer :: myid_z = 0    ! subdomain index (0 based) in level (z)
    integer :: npr_y  = 1    ! number of subdomains in y
    integer :: npr_z  = 1    ! number of subdomains in z

    integer :: myidxy_x = 0  ! subdomain index (0-based) in longitude (x) (second. decomp.)
    integer :: myidxy_y = 0  ! subdomain index (0 based) in latitude (y) (second. decomp.)
    integer :: nprxy_x = 1   ! number of subdomains in x (second. decomp.)
    integer :: nprxy_y = 1   ! number of subdomains in y (second. decomp.)
    integer :: iam = 0       ! 

    integer :: mod_method = 0  ! 1 for mpi derived types with transposes, 0 for contiguous buffers
    integer :: mod_geopk = 0   ! 1 for mpi derived types with transposes, 0 for contiguous buffers
    integer :: mod_gatscat = 0 ! 1 for mpi derived types with transposes, 0 for contiguous buffers


    integer :: commdyn           ! communicator for all dynamics
    integer :: commxy            ! communicator for XY decomposition
    integer :: commyz            ! communicator for YZ decomposition
    integer :: commnyz           ! communicator for multiple YZ decomposition

    integer :: comm_y            ! communicator in latitude
    integer :: comm_z            ! communicator in vertical
    integer :: commxy_x          ! communicator in longitude (xy second. decomp.)
    integer :: commxy_y          ! communicator in latitude (xy second. decomp.)
    logical :: geopkdist         ! use distributed method for geopotential calculation 
                                 !  with 2D decomp.
    logical :: geopk16byte       ! use Z-parallel distributed method for geopotential 
                                 !  calculation with 2D decomp.; otherwise use Z-serial
                                 !  pipeline algorithm when using distributed algoritm
    integer :: geopkblocks       ! number of stages to use in Z-serial pipeline 
                                 !  (non-transpose) geopotential algorithm
    integer :: modc_dynrun(4)    ! 1: mod_comm irregular underlying communication method for dyn_run/misc
                                 ! 2: mod_comm irregular communication handshaking for dyn_run/misc
                                 ! 3: mod_comm irregular communication send protocol for dyn_run/misc
                                 ! 4: mod_comm irregular communication nonblocking request throttle for dyn_run/misc
    integer :: modc_cdcore(4)    ! 1: mod_comm irregular underlying communication method for cd_core/geopk
                                 ! 2: mod_comm irregular communication handshaking for cd_core/geopk
                                 ! 3: geopk_d and mod_comm irregular communication send protocol for cd_core/geopk
                                 ! 4: mod_comm irregular communication nonblocking request throttle for cd_core/geopk
    integer :: modc_gather(4)    ! 1: mod_comm irregular underlying communication method for gather
                                 ! 2: mod_comm irregular communication handshaking for gather
                                 ! 3: mod_comm irregular communication send protocol for gather
                                 ! 4: mod_comm irregular communication nonblocking request throttle for gather
    integer :: modc_scatter(4)   ! 1: mod_comm irregular underlying communication method for scatter
                                 ! 2: mod_comm irregular communication handshaking for scatter
                                 ! 3: mod_comm irregular communication send protocol for scatter
                                 ! 4: mod_comm irregular communication nonblocking request throttle for scatter 
    integer :: modc_tracer(4)    ! 1: mod_comm irregular underlying communication method for multiple tracers
                                 ! 2: mod_comm irregular communication handshaking for multiple tracers
                                 ! 3: mod_comm irregular communication send protocol for multiple tracers
                                 ! 4: mod_comm irregular communication nonblocking request throttle for multiple tracers 
    integer :: modc_onetwo       ! one or two simultaneous mod_comm irregular communications (excl. tracers)
    integer :: modc_tracers      ! max number of tracers for simultaneous mod_comm irregular communications

!
! END PILGRIM communication information
!
    integer                         :: IFIRST
    integer                         :: ILAST
! zy adds above

    integer                         :: JFIRST           ! Start latitude (exclusive)
    integer                         :: JLAST            ! End latitude (exclusive)
 
!
    integer                         :: NG_C             ! Ccore ghosting
    integer                         :: NG_D             ! Dcore ghosting
    integer                         :: NG_S             ! Staggered grid ghosting for
                                                        ! certain arrays, max(ng_c+1,ng_d)
!
! For 2D decomposition (currently not used)
!
    integer                         :: IFIRSTXY         ! Start longitude (exclusive)
    integer                         :: ILASTXY          ! End longitude (exclusive)
    integer                         :: JFIRSTXY         ! Start latitude (exclusive)
    integer                         :: JLASTXY          ! End latitude (exclusive)
!
    integer                         :: IM               ! Full longitude dim
    integer                         :: JM               ! Full latitude dim (including poles)
!
    real(r8)                        :: DL
    real(r8)                        :: DP
    real(r8)                        :: ACAP
    real(r8)                        :: RCAP
!
    real(r8), dimension(:), pointer :: COSP             ! Cosine of lat angle -- volume mean
    real(r8), dimension(:), pointer :: SINP             ! Sine of lat angle -- volume mean
    real(r8), dimension(:), pointer :: COSE             ! Cosine at finite volume edge
    real(r8), dimension(:), pointer :: SINE             ! Sine at finite volume edge
    real(r8), dimension(:), pointer :: ACOSP            ! Reciprocal of cosine of lat angle
!
    real(r8), dimension(:), pointer :: ACOSU            ! Reciprocal of cosine of lat angle (staggered)
!
    real(r8), dimension(:), pointer :: COSLON           ! Cosine of longitudes - volume center
    real(r8), dimension(:), pointer :: SINLON           ! Sine of longitudes - volume center
    real(r8), dimension(:), pointer :: COSL5            ! Cosine of longitudes - volume center
    real(r8), dimension(:), pointer :: SINL5            ! Sine of longitudes - volume center
 
!
!   Variables which are used repeatedly in CD_CORE
!

      integer ::       js2g0
      integer ::       jn2g0
      integer ::       jn1g1

      real(r8), pointer :: trigs(:)
      real(r8), pointer :: fc(:), f0(:)
      real(r8), pointer :: dc(:,:), de(:,:), sc(:), se(:)
      real(r8), pointer :: cdx(:,:), cdy(:,:)
      real(r8), pointer :: cdx4(:,:), cdy4(:,:) !for div4 damping
      real(r8), pointer :: cdxde(:,:), cdxdp(:,:),cdyde(:,:),cdydp(:,:)      !for del2 damping
      real(r8), pointer :: cdxdiv(:,:), cdydiv(:,:), cdtau4(:,:) !for del2 damping

      real(r8), pointer :: dcdiv4(:,:), dediv4(:,:), scdiv4(:), sediv4(:)    !for div4 damping

      real(r8), pointer :: dtdx(:), dtdxe(:), txe5(:), dtxe5(:)
      real(r8), pointer :: dyce(:),   dx(:) ,  rdx(:),    cy(:)
      real(r8), pointer :: dtdx2(:), dtdx4(:),  dxdt(:), dxe(:)
      real(r8), pointer :: cye(:),    dycp(:),  rdxe(:), rdy(:), dtdy(:), dtdy5(:)

!     real(r8) :: rdy, dtdy, dydt, dtdy5, tdy5
      real(r8) :: dydt, tdy5
      real(r8) :: dt0 = 0.

      integer  :: ifax(13)

      real(r8) ::  zt_c
      real(r8) ::  zt_d

!
! This part refers to the vertical grid
!
    integer                         :: KM              ! Numer of levels
    integer                         :: KMAX            ! KM+1 (?)
!
! For 2D decomposition (currently not used)
!
    integer                         :: KFIRST          ! Start level (exclusive)
    integer                         :: KLAST           ! End level (exclusive)
    integer                         :: KLASTP          ! klast+1, except km+1 when klastp=km+1
!
!
    integer                         :: KORD            ! monotonicity order for mapping (te_map)
    integer                         :: KS              ! Number of true pressure levels (out of KM+1)
    real(r8)                        :: PTOP            ! pressure at top (ak(1))
    real(r8)                        :: PINT            ! initial pressure (ak(km+1))
    real(r8), dimension(:), pointer :: AK              ! Sigma mapping
    real(r8), dimension(:), pointer :: BK              ! Sigma mapping

!
! Tracers
!
    integer                         :: NQ              ! Number of advected tracers
    integer                         :: NTOTQ           ! Total number of tracers (NQ <= NC)

! Extra subdomain bounds for cd_core/trac2d overlap and trac2d decomposition
! Relevant for secondary yz decomposition only; refers back to primary yz decomposition
    integer                         :: JFIRSTCT          ! jfirst
    integer                         :: JLASTCT           ! jlast
    integer                         :: KFIRSTCT          ! kfirst
    integer                         :: KLASTCT           ! klast

! Bounds for tracer decomposition
    integer, dimension(:), pointer  :: ktloa             ! lower tracer index (global map)
    integer, dimension(:), pointer  :: kthia             ! upper tracer index (global map)
    integer                         :: ktlo              ! lower tracer index (local)
    integer                         :: kthi              ! upper tracer index (local)

  end type T_FVDYCORE_GRID

! Constants used by fvcore
  type T_FVDYCORE_CONSTANTS
    real(r8)                             :: pi
    real(r8)                             :: omega    ! angular velocity of earth's rotation  
    real(r8)                             :: cp       ! heat capacity of air at constant pressure
    real(r8)                             :: ae       ! radius of the earth (m)
    real(r8)                             :: rair     ! Gas constant of the air
    real(r8)                             :: cappa    ! Cappa?
    real(r8)                             :: zvir     ! RWV/RAIR-1
  end type T_FVDYCORE_CONSTANTS

  integer, parameter :: NUM_FVDYCORE_ALARMS        = 3
  integer, parameter :: NUM_TIMES      = 8

  type T_FVDYCORE_STATE
!!!    private
    type (T_FVDYCORE_VARS)               :: VARS
    type (T_FVDYCORE_GRID )              :: GRID
    type (T_FVDYCORE_CONSTANTS)          :: CONSTANTS
#if defined( GEOS_MODE )
    type (ESMF_Clock), pointer           :: CLOCK
    type (ESMF_Alarm)                    :: ALARMS(NUM_FVDYCORE_ALARMS)
#endif
    integer(kind=8)                      :: RUN_TIMES(4,NUM_TIMES)
    logical                              :: DOTIME, DODYN
    real(r8)                             :: DT          ! Large time step
    real(r8)                             :: CHECK_DT    ! Time step to check maxmin
    integer                              :: ICD, JCD    ! Algorithm orders (C Grid)
    integer                              :: IORD, JORD  ! Algorithm orders (D Grid)
    integer                              :: KORD        ! Vertical order
    integer                              :: TE_METHOD   ! method for total energy mapping (te_map)
    logical                              :: CONSV       ! dycore conserves tot. en.
    integer                              :: NSPLIT
    integer                              :: NSPLTRAC
    integer                              :: NSPLTVRM
    integer                              :: NUM_CALLS
    integer                              :: FILTCW      ! filter c-grid winds if positive
  end type T_FVDYCORE_STATE

!
! !DESCRIPTION:
!
!      This module provides variables which are specific to the Lin-Rood
!      dynamical core.  Most of them were previously SAVE variables in 
!      different routines and were set with an "if (first)" statement.
!
!      \begin{tabular}{|l|l|} \hline \hline
!        lr\_init    &  Initialize the Lin-Rood variables  \\ \hline
!        lr\_clean   &  Deallocate all internal data structures \\ \hline 
!                                \hline
!      \end{tabular}
!
! !REVISION HISTORY:
!   01.06.06   Sawyer     Consolidated from various code snippets
!   01.07.12   Sawyer     Removed CCM common blocks comtim.h and commap.h
!   03.06.25   Sawyer     Cleaned up, used ParPatternCopy (Create)
!   03.07.23   Sawyer     Removed dependencies on params.h, constituents
!   03.08.05   Sawyer     Removed rayf_init and hswf_init, related vars
!   03.09.17   Sawyer     Removed unneeded ghost definitions
!   03.10.22   Sawyer     pmgrid removed (now spmd_dyn)
!   03.11.18   Sawyer     Removed set_eta (ak, bk, now read from restart)
!   03.12.04   Sawyer     Moved T_FVDYCORE_GRID here (removed some vars)
!   04.08.25   Sawyer     Removed all module data members, now GRID only
!   04.10.06   Sawyer     Added spmd_dyn vars here; ESMF transpose vars
!   05.04.12   Sawyer     Added support for r4/r8 tracers
!   05.05.24   Sawyer     CAM/GEOS5 merge (removed GEOS_mod dependencies)
!   05.06.10   Sawyer     Scaled down version for CAM (no ESMF)
!   05.11.10   Sawyer     Removed dyn_interface (now in dyn_comp)
!   06.03.01   Sawyer     Removed m_ttrans, q_to_qxy, qxy_to_q, etc.
!   06.05.09   Sawyer     Added CONSV to dyn_state (conserve energy)
!   06.08.27   Sawyer     Removed unused ESMF code for RouteHandle
!
!EOP
!-----------------------------------------------------------------------
   real(r8), parameter ::  D0_0                    =   0.0_r8
   real(r8), parameter ::  D0_5                    =   0.5_r8
   real(r8), parameter ::  D1_0                    =   1.0_r8
   real(r8), parameter ::  D2_0                    =   2.0_r8
   real(r8), parameter ::  D4_0                    =   4.0_r8
   real(r8), parameter ::  D10_0                   =  10.0_r8
   real(r8), parameter ::  D180_0                  = 180.0_r8
   real(r8), parameter ::  ratmax                  =  0.81_r8

end module dynamics_vars
