#include <misc.h>
#include <params.h>

subroutine ffsl_bndexch

!----------------------------------------------------------------------- 
! 
! Purpose: Pack and Exchange initial prognostic information among all the 
!          processors
!
! Method: 
! 
! Author: 
! 
!-----------------------------------------------------------------------
! $Id: ffsl_bndexch.F90,v 1.1.1.1 2014/08/26 01:56:31 wangzzh Exp $
! $Author: wangzzh $
!
!----------------------------Parameters---------------------------------

#ifdef SPMD
  use pmgrid,   only: iam
  use spmd_dyn, only: npes, cut, cutex, neighs, neighs_proc, &
                      neighn, neighn_proc

  implicit none
!
! Local workspace
!
  integer ns, nn
  integer inreg( 2 )
  integer outreg( 2 )
  integer others,othern   ! Other node
!
! Return if number of processors is less than 2
!
  if (npes .lt. 2) return
!
! For each partition (south and north) communicate boundaries
! on each side of partition among however many neighbors necessary
!
! send south, receive north
!
! write(*,*)'bnd:neighs', neighs, neighn
  ns = 1
  nn = 1
  do while (ns .le. neighs .or. nn .le. neighn)
     if (ns .le. neighs) then
        others = neighs_proc(ns)
!
! Intersection of my cuts and neighbor processor's extended
! cuts tells if this node needs to send data to neighbor 
!
        call intersct(cut(1,iam),cutex(1,others),outreg)  
        ns = ns + 1
     else
        others = -1
        outreg(1) = 0
        outreg(2) = 0
     end if

     if (nn .le. neighn) then
        othern = neighn_proc(nn)
!
! Intersection of neighbor cuts and this node's extended
! cut tells if this node receives data from neighbor 
!
        call intersct(cut(1,othern),cutex(1,iam),inreg)  
        nn = nn + 1
     else
        othern = -1
        inreg(1) = 0
        inreg(2) = 0
     end if

!    write(*,*)'ss=', others, outreg,othern,inreg, ns, nn
     call ffsl_bndexch_mpi(others,outreg,othern,inreg)
  end do

!
! send north, receive south
!
! write(*,*)'bnd:neighn', neighn
  ns = 1
  nn = 1
  do while (ns .le. neighs .or. nn .le. neighn)
     if (nn .le. neighn) then
        othern = neighn_proc(nn)
!
! Intersection of my cuts and neighbor processor's extended
! cuts tells if this node needs to send data to neighbor 
!
        call intersct(cut(1,iam),cutex(1,othern),outreg)  
        nn = nn + 1
     else
        othern = -1
        outreg(1) = 0
        outreg(2) = 0
     end if

     if (ns .le. neighs) then
        others = neighs_proc(ns)
!
! Intersection of neighbor cuts and this node's extended
! cut tells if this node receives data from neighbor 
!
        call intersct(cut(1,others),cutex(1,iam),inreg)  
        ns = ns + 1
     else
        others = -1
        inreg(1) = 0
        inreg(2) = 0
     end if

     call ffsl_bndexch_mpi(othern,outreg,others,inreg)
  end do
#endif
  return
end subroutine ffsl_bndexch

#ifdef SPMD
subroutine ffsl_bndexch_mpi(othero,outreg,otheri,inreg)
!-----------------------------------------------------------------------
! Send initial prognostic information to my peer process
!-----------------------------------------------------------------------
  use pmgrid,       only: plat, plndlv, j1, plond
  use constituents, only: pcnst
  use prognostics,  only: u3, v3, qminus, n3m1, n3m2, psex
  use mpishorthand, only: mpir8, mpicom, MPI_STATUS_SIZE

  implicit none

  integer, parameter :: msgtype = 6000
  integer, parameter :: j1m = j1 - 1
  integer, parameter :: siz = (2 + pcnst)*plndlv

  integer othero,outreg(2),otheri,inreg(2)
  integer num
  integer msg

  integer reqs(4*(plat+1))
  integer stats(MPI_STATUS_SIZE, 4*(plat+1))

  integer reqr(4*(plat+1))
  integer statr(MPI_STATUS_SIZE, 4*(plat+1))

  integer i,j
  integer reqs_i,reqr_i
  integer nn

  reqs_i = 0
! nn = n3m1
  nn = n3m2
  if (othero .ne. -1) then
     do i = outreg(1), outreg(2)
        j = 4*(i-outreg(1))

        msg = msgtype + j
        reqs_i = reqs_i + 1
        call mpiisend (u3(1,1,j1m+i,nn),plndlv,mpir8, othero,msg,mpicom,reqs(reqs_i))

        msg = msgtype + j + 1
        reqs_i = reqs_i + 1
        call mpiisend (v3(1,1,j1m+i,nn),plndlv,mpir8, othero,msg,mpicom,reqs(reqs_i))

        msg = msgtype + j + 2
        reqs_i = reqs_i + 1
        num = pcnst*plndlv
        call mpiisend (qminus(1,1,1,j1m+i),num,mpir8, othero,msg,mpicom,reqs(reqs_i))
             
        msg = msgtype + j + 3
        reqs_i = reqs_i + 1
        num = plond
        call mpiisend (psex(1,j1m+i),num,mpir8, othero,msg,mpicom,reqs(reqs_i))
             
     end do
  end if

  reqr_i = 0
  if (otheri .ne. -1) then
     do i = inreg(1), inreg(2)
        j = 4*(i-inreg(1))
        msg = msgtype + j
        reqr_i = reqr_i + 1
        call mpiirecv (u3(1,1,j1m+i,nn),plndlv,mpir8, otheri,msg,mpicom,reqr(reqr_i))

        msg = msgtype + j + 1
        reqr_i = reqr_i + 1
        call mpiirecv (v3(1,1,j1m+i,nn),plndlv,mpir8, otheri,msg,mpicom,reqr(reqr_i))

        msg = msgtype + j + 2
        reqr_i = reqr_i + 1
        num = pcnst*plndlv
        call mpiirecv (qminus(1,1,1,j1m+i),num,mpir8, otheri,msg,mpicom,reqr(reqr_i))
             
        msg = msgtype + j + 3
        reqr_i = reqr_i + 1
        num = plond
        call mpiirecv (psex(1,j1m+i),num,mpir8, otheri,msg,mpicom,reqr(reqr_i))
             
     end do
  end if

  if (reqs_i .ne. 0) then
     call mpiwaitall(reqs_i,reqs,stats)
  end if

  if (reqr_i .ne. 0) then
     call mpiwaitall(reqr_i,reqr,statr)
  end if

  return
end subroutine ffsl_bndexch_mpi

subroutine mp_bndexch_q(q, im, jm, nq, jfirst, jlast, ng)
  use shr_kind_mod,   only: r8=>shr_kind_r8
  use pmgrid,         only: iam, beglat, endlat
  use spmd_dyn, only: npes, cut, cutex, neighs, neighs_proc, &
                      neighn, neighn_proc

!
  implicit none
  integer :: im, jm, nq
  integer :: jfirst, jlast, ng
  real(r8), intent(inout) :: q(im, nq, jfirst-ng:jlast+ng)
!
! Local workspace
!
  integer ns, nn
  integer inreg( 2 )
  integer outreg( 2 )
  integer others,othern   ! Other node
  integer i, j, k, nlon2
  real(r8) zave
!
! Return if number of processors is less than 2
!
  if (npes .lt. 2) return
!
! For each partition (south and north) communicate boundaries
! on each side of partition among however many neighbors necessary
!
! send south, receive north
!
! write(*,*)'bnd:neighs', beglat, endlat, jfirst, jlast, ng
! write(*,*)'bnd:neighs', neighs, neighn
  ns = 1
  nn = 1
  do while (ns .le. neighs .or. nn .le. neighn)
     if (ns .le. neighs) then
        others = neighs_proc(ns)
!
! Intersection of my cuts and neighbor processor's extended
! cuts tells if this node needs to send data to neighbor 
!
        call intersct(cut(1,iam),cutex(1,others),outreg)  
        ns = ns + 1
     else
        others = -1
        outreg(1) = 0
        outreg(2) = 0
     end if

     if (nn .le. neighn) then
        othern = neighn_proc(nn)
!
! Intersection of neighbor cuts and this node's extended
! cut tells if this node receives data from neighbor 
!
        call intersct(cut(1,othern),cutex(1,iam),inreg)  
        nn = nn + 1
     else
        othern = -1
        inreg(1) = 0
        inreg(2) = 0
     end if

!    write(*,*)'ss=', others, outreg,othern,inreg, ns, nn
     call ffsl_bndexch_mpi_q(others,outreg,othern,inreg, &
                             q, im, nq, jfirst, jlast, ng)
  end do

!
! send north, receive south
!
! write(*,*)'bnd:neighn', neighn
  ns = 1
  nn = 1
  do while (ns .le. neighs .or. nn .le. neighn)
     if (nn .le. neighn) then
        othern = neighn_proc(nn)
!
! Intersection of my cuts and neighbor processor's extended
! cuts tells if this node needs to send data to neighbor 
!
        call intersct(cut(1,iam),cutex(1,othern),outreg)  
        nn = nn + 1
     else
        othern = -1
        outreg(1) = 0
        outreg(2) = 0
     end if

     if (ns .le. neighs) then
        others = neighs_proc(ns)
!
! Intersection of neighbor cuts and this node's extended
! cut tells if this node receives data from neighbor 
!
        call intersct(cut(1,others),cutex(1,iam),inreg)  
        ns = ns + 1
     else
        others = -1
        inreg(1) = 0
        inreg(2) = 0
     end if

     call ffsl_bndexch_mpi_q(othern,outreg,others,inreg, &
                             q, im, nq, jfirst, jlast, ng)
  end do
!
! Fill latitude extensions of a scalar extended array beyond the poles
!
! Fill north pole line.
!
  if (jm+1<=jlast+ng) then
     do k=1,nq
        zave = 0.0
        do i=1,im
           zave = zave + q(i,k,jm  )
        end do
        zave = zave/im
        do i=1,im
           q(i,k,jm+1) = zave
        end do
     end do
  end if
!
! Fill northern lines beyond pole line.
!
  if( jm+2 <= jlast+ng )then
     do j=jm+2,jlast+ng
!       if (j<=endlatex) then
           nlon2 = im/2
           do k=1,nq
              do i=1,nlon2
                 q(      i,k,j) = q(nlon2+i,k,2*jm+2-j)
                 q(nlon2+i,k,j) = q(      i,k,2*jm+2-j)
              end do
           end do
!       end if
     end do
  end if

! Fill south pole line.
!
  if (0>=jfirst-ng) then
     do k=1,nq
        zave = 0.0
        do i = 1, im
           zave = zave + q(i,k,1  )
        end do
        zave = zave/im
        do i=1, im
           q(i,k,0) = zave
        end do
     end do
  end if
!
! Fill southern lines beyond pole line.
!
  if( -1 >= jfirst-ng )then
     do j=jfirst-ng,-1
!       if (j>=beglatex) then
           nlon2 = im/2
           do k=1,nq
              do i=1,1+nlon2-1
                 q(      i,k,j) = q(nlon2+i,k,-j)
                 q(nlon2+i,k,j) = q(      i,k,-j)
              end do
           end do
!       end if
     end do
  end if

  return
end subroutine mp_bndexch_q

subroutine ffsl_bndexch_mpi_q(othero,outreg,otheri,inreg, &
                              q, im, nq, jfirst, jlast, ng)
!-----------------------------------------------------------------------
! Send initial prognostic information to my peer process
!-----------------------------------------------------------------------
  use shr_kind_mod, only: r8=>shr_kind_r8
  use pmgrid,       only: plat, j1
  use mpishorthand, only: mpir8, mpicom, MPI_STATUS_SIZE

  implicit none

  integer, parameter :: msgtype = 6000
! integer, parameter :: j1m = j1 - 1
! integer, parameter :: siz = (2 + pcnst)*plndlv

  integer othero,outreg(2),otheri,inreg(2)
  integer num
  integer msg
  integer im, nq
  integer :: jfirst, jlast, ng
  real(r8), intent(inout) :: q(im, nq, jfirst-ng:jlast+ng)

  integer reqs(4*(plat+1))
  integer stats(MPI_STATUS_SIZE, 4*(plat+1))

  integer reqr(4*(plat+1))
  integer statr(MPI_STATUS_SIZE, 4*(plat+1))

  integer i,j
  integer reqs_i,reqr_i

  reqs_i = 0
  if (othero .ne. -1) then
     do i = outreg(1), outreg(2)
        j = i-outreg(1)

        msg = msgtype + j
        reqs_i = reqs_i + 1
        num = nq*im
!   write(*,*)'wzz: outreg=', i, j1m, j1m+i
        call mpiisend (q(1,1,i),num,mpir8, othero,msg,mpicom,reqs(reqs_i))
             
     end do
  end if

  reqr_i = 0
  if (otheri .ne. -1) then
     do i = inreg(1), inreg(2)
        j = i-inreg(1)
        msg = msgtype + j
        reqr_i = reqr_i + 1
        num = nq*im
!   write(*,*)'wzz: inreg=', i, j1m, j1m+i
        call mpiirecv (q(1,1,i),num,mpir8, otheri,msg,mpicom,reqr(reqr_i))
             
     end do
  end if

  if (reqs_i .ne. 0) then
     call mpiwaitall(reqs_i,reqs,stats)
  end if

  if (reqr_i .ne. 0) then
     call mpiwaitall(reqr_i,reqr,statr)
  end if

  return
end subroutine ffsl_bndexch_mpi_q
#endif
