#include <misc.h>

  module ffsl_mod
  
  use shr_kind_mod,   only: r8=>shr_kind_r8, r4=>shr_kind_r4
  use shr_sys_mod,    only: shr_sys_flush
  use dynamics_vars,  only: T_FVDYCORE_GRID

  implicit none

  public                 :: ffsl_run
  private                :: scan_fxcrq, fx_cr_q

  real(r8), parameter    :: ZERO                 = 0.0_r8
  real(r8), parameter    :: D0_5                 = 0.5_r8
  real(r8), parameter    :: D1_0                 = 1.0_r8
  real(r8), parameter    :: THREE_QUARTERS       = 0.75_r8
  real(r4), parameter    :: ONE_R4               = 1.0_r4
  real(r8), parameter    :: SECS_IN_SIX_HOURS    = 21600.0_r8

  CONTAINS
!---------------------------------------------------------
! origins: dyn_run in dyn_comp.F90
! purpose: run part of offline FFSL transport
!          only to derive tracer value
! revised: zy
! date   : 2011.06.15
!---------------------------------------------------------
  subroutine ffsl_run( grid, dt, uc, vc, delp, pe, pi, ae, tracer,  &
                       lam, dlam, nlon, fullgrid, qout)

  use fv_control_mod, only: iord, jord, kord
  use mapz_module,    only: mapn_ppm_tracer

  type(T_FVDYCORE_GRID), intent(inout)   :: GRID      !initialized all grid info
  real(r8), intent(in)    :: dt, pi, ae
  real(r8), intent(in)    :: dlam(grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8), intent(in)    :: lam(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8), intent(in)    :: delp(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%kfirst:grid%klast)
  real(r8), intent(in)    :: tracer(grid%im,grid%kfirst:grid%klast,grid%ntotq,grid%jfirstxy:grid%jlastxy)
  integer,  intent(in)    :: nlon(grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  logical fullgrid          ! true => no grid reduction towards poles

  real(r8), intent(inout) :: uc(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%kfirst:grid%klast) 
  real(r8), intent(inout) :: vc(grid%im,grid%jfirst-grid%ng_d:grid%jlast+grid%ng_d,grid%kfirst:grid%klast)   
  real(r8), intent(out)   :: qout(grid%im,grid%kfirst:grid%klast,grid%ntotq,grid%jfirst:grid%jlast)
  real(r8)                :: pe(grid%ifirst:grid%ilast,grid%km+1)
!
!  Local
!
  real(r8)                :: delpf(grid%im, grid%jfirst:grid%jlast, grid%kfirst:grid%klast)
  real(r8)                :: mfx(grid%im, grid%jfirst:grid%jlast, grid%kfirst:grid%klast)
  real(r8)                :: mfy(grid%im, grid%jfirst:grid%jlast+1, grid%kfirst:grid%klast)
  real(r8)                :: cx(grid%im, grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c, grid%kfirst:grid%klast)
  real(r8)                :: cy(grid%im, grid%jfirst:grid%jlast+1, grid%kfirst:grid%klast)
  real(r8)                :: pe2(grid%ifirst:grid%ilast,grid%km+1)
  integer                 :: iord_c, jord_c, krd
!
! Magic Number
!
  integer                 :: i, j, k, i1, i2, iv
  integer                 :: im, km, nq
  integer                 :: ifirst, ilast         ! starting & ending longitude index
  integer                 :: jfirst, jlast
  integer                 :: kfirst, klast
  integer                 :: ng_c

!
!  for convienence
!

  im     = grid%im
  km     = grid%km
  nq     = grid%nq
  ifirst = grid%ifirst
  ilast  = grid%ilast
  jfirst = grid%jfirst
  jlast  = grid%jlast
  kfirst = grid%kfirst
  klast  = grid%klast
  ng_c   = grid%ng_c

!
  mfx(:,:,:) = ZERO
  mfy(:,:,:) = ZERO
  cx(:,:,:)  = ZERO
  cy(:,:,:)  = ZERO

!---------------original version------------------
!  if( iord <= 2 ) then
!      iord_c  =  1
!  else
!      iord_c  = -2
!  endif

!  if( jord <= 2 ) then
!      jord_c  =  1
!  else
!      jord_c  =  -2
!  endif
!--------------------------------------------------

! zy forces
 
  iord_c = iord
  jord_c = jord
! if(jfirst==1)then
! write(*,*)'wzz: iord_c, jord_c in ffsl_run 1=', iord_c, jord_c, jlast, tracer(1,jlast,11,1), pe(1,jlast,11)
! write(*,*)'wzz: iord_c, jord_c in ffsl_run 1=', iord_c, jord_c,  &
!             tracer(39,8,4,3), grid%jfirstxy, grid%jlastxy, vc(8,9,6)
! endif
! call t_startf ('scan_fx')

  call scan_fxcrq  ( grid,   delp,   dt,               &
                     pi,     ae,  iord_c, jord_c,      &  ! this routine internally cycles
                     cx,     cy,  mfx,    mfy,         &
                     delpf,  uc,  vc,     tracer, lam, dlam, nlon, fullgrid, qout  )
! call t_stopf ('scan_fx')
!
! call t_startf ('remap_tr')

   if(kord == 8) then
      krd = 8
   else
      krd = 7
   endif
!
   i1 = ifirst
#ifdef OUTER_OMP
!$OMP PARALLEL DO PRIVATE (J, I, K, I2, iv, pe, pe2)
#endif
   do j = jfirst, jlast
      i2 = i1-1+nlon(j)
!
! New pe for remap
      pe(:,1) = grid%ak(1)
      do k=1,km
         do i= i1, i2
           pe(i,k+1)  = pe(i,k) + delpf(i,j,k)
         enddo
      enddo
!
! pe2 = updated  coordinate from top to bottom
!$OMP PARALLEL DO PRIVATE (K, I)
      do k=1,km+1
         do i= i1, i2
           pe2(i,k) = grid%ak(k) + grid%bk(k)*pe(i,km+1)  ! Remapped PLE based on Updated PS
         enddo
      enddo

      call mapn_ppm_tracer( km,   pe,   delpf(ifirst:ilast,j,1:km), &
                            qout(ifirst:ilast,1:km,1:nq,j), nq,     &
                            km,   pe2,   i1,   i2,                  &
                            j,    ifirst, ilast, jfirst, jlast, 0, krd)
   end do
! call t_stopf ('remap_tr')
! if(jfirst==1)then
!    write(*,*)'wzz: remap_tracer 1', tracer(1,jlast,9:11,1)
! endif

  return
  end subroutine ffsl_run

!-----------------------------------------------------------
! origins: cd_core
! purpose: setting environment for 
!          calculating flux and courant and tracer 
! revised: zy 
! date   : 2011.06.15 
!-----------------------------------------------------------
 subroutine scan_fxcrq(grid,   delp,   dt,                 &
                       pi,     ae,  iord_c, jord_c,        &
                       cx3,    cy3, mfx,    mfy,           &
                       delpf,  uc,  vc,     q, lam, dlam, nlon, fullgrid, qout   )
! !USES:

! !INPUT PARAMETERS:
  type (T_FVDYCORE_GRID), intent(inout) :: grid! grid (for YZ decomp)
!  integer, intent(in) :: ipe                  ! ipe=1:  end of cd_core()
                                               ! ipe=-1,-2: start of cd_core()
                                               ! ipe=-2,2: second to last call to cd_core()
                                               ! ipe=0 :
  integer, intent(in) :: iord_c, jord_c        ! scheme order on C grid in X and Y dir.
  integer, intent(in) :: nlon(grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  logical fullgrid          ! true => no grid reduction towards poles

! ct_overlap data
  real(r8), intent(in) :: pi
  real(r8), intent(in) :: ae                   ! Radius of the Earth (m)
  real(r8), intent(in) :: dt                   ! small time step in seconds
  real(r8), intent(in) :: dlam(grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8), intent(in) :: lam(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8), intent(in) ::   & 
    delp(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%kfirst:grid%klast)  ! Delta pressure (pascal)

! Input time independent arrays:

! !INPUT/OUTPUT PARAMETERS:
  real(r8), intent(inout) ::   &
    uc(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%kfirst:grid%klast)   ! u-Winds on C-grid
  real(r8), intent(inout) ::   &
    vc(grid%im,grid%jfirst-grid%ng_d:grid%jlast+grid%ng_d,grid%kfirst:grid%klast)   ! v-Winds on C-grid

! Input/output: accumulated winds & mass fluxes on c-grid for large-
!               time-step transport
  real(r8), intent(inout) ::   &
    cx3(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%kfirst:grid%klast)! Accum. Courant no. in X
  real(r8), intent(inout) ::   &
    cy3(grid%im,grid%jfirst:grid%jlast+1,grid%kfirst:grid%klast)        ! Accumulated Courant no. in Y
  real(r8), intent(inout) ::   &
    mfx(grid%im,grid%jfirst:grid%jlast,grid%kfirst:grid%klast)          ! Mass flux in X  (unghosted)
  real(r8), intent(inout) ::   &
    mfy(grid%im,grid%jfirst:grid%jlast+1,grid%kfirst:grid%klast)        ! Mass flux in Y
  real(r8), intent(in) ::   &
    q(grid%im, grid%kfirst:grid%klast, grid%ntotq,grid%jfirstxy:grid%jlastxy)
! Input/output work arrays:
  real(r8), intent(out) ::   &
    delpf(grid%im,grid%jfirst:grid%jlast,grid%kfirst:grid%klast)  ! filtered delp
  real(r8), intent(out) :: qout(grid%im,grid%kfirst:grid%klast,grid%ntotq,grid%jfirst:grid%jlast)
! outout 
!
! Work arrays
   real(r8)   ::  xfx(grid%im,grid%jfirst:grid%jlast)
   real(r8)   ::  yfx(grid%im,grid%jfirst:grid%jlast+1)
   real(r8)   ::  crx(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
   real(r8)   ::  cry(grid%im,grid%jfirst-1:grid%jlast+1)
   real(r8)   ::  qwork(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%ntotq)
! Local 2D arrays:
      real(r8) ::  wk(grid%im+2,grid%jfirst:  grid%jlast+2)
      real(r8) ::  wk1(grid%im,grid%jfirst-1:grid%jlast+1)
      real(r8) ::  wk2(grid%im+1,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
      real(r8) ::  wk3(grid%im,grid%jfirst-1:grid%jlast+1)
      real(r8) :: p1d(grid%im)
! fvitt    cell centered u- and v-Winds (m/s)
      real(r8) ::  u_cen(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%kfirst:grid%klast)
      real(r8) ::  ua(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%kfirst:grid%klast)
! Local scalars
      real(r8), parameter ::  D0_1                    =   0.1_r8
      real(r8), parameter ::  D4_0                    =   4.0_r8
      real(r8), parameter ::  D8_0                    =   8.0_r8
      real(r8), parameter ::  D128_0                  = 128.0_r8
      real(r8), parameter ::  D180_0                  = 180.0_r8
      real(r8), parameter ::  D1E5                    = 1.0e5_r8
      real(r8), parameter ::  tiny                    = 1.0e-10_r8
      real(r8) ::  dt5

      integer :: im, jm, km, iq     ! problem dimensions
      integer :: nq                 ! # of tracers to be advected by trac2d
      integer :: ifirstxy,ilastxy   ! xy-decomp. longitude ranges
      integer :: jfirstxy,jlastxy   ! xy-decomp. latitude ranges
      integer :: ng_c               ! ghost latitudes on C grid
      integer :: ng_d               ! 
!     integer :: ng_s               ! max(ng_c+1,ng_c) significant if ng_c = ng_c

      integer :: jfirst
      integer :: jlast
      integer :: kfirst
      integer :: klast
      integer :: klastp            ! klast, except km+1 when klast=km

      integer :: iam
      integer :: npr_y
      integer :: npes_xy
      integer :: npes_yz

      integer i, j, k, ml
      integer js1g1, js2g0, js2g1, jn2g1
      integer jn2g0, jn1g1
      integer iord, jord
      integer ktot, ktotp

! logical :: reset_winds = .false.

! C.-C. Chen, omega calculation
! zy set it to local

     real(r8) ::   &
       cx_om(grid%im,grid%jfirst:grid%jlast,grid%kfirst:grid%klast)   ! Courant in X
     real(r8) ::   &
       cy_om(grid%im,grid%jfirst:grid%jlast+1,grid%kfirst:grid%klast) ! Courant in Y

!--------END OF STATEMENTS-----------------------------------------

      iam      = grid%iam
      npr_y    = grid%npr_y
      npes_xy  = grid%npes_xy
      npes_yz  = grid%npes_yz

      im       = grid%im
      jm       = grid%jm
      km       = grid%km
      nq       = grid%nq

      ng_c     = grid%ng_c
      ng_d     = grid%ng_d
!     ng_s     = grid%ng_s

      jfirst   = grid%jfirst
      jlast    = grid%jlast
      kfirst   = grid%kfirst
      klast    = grid%klast
      klastp   = grid%klastp
      if (jfirst>jlast) return   ! wzz, 2017.2.17

      ifirstxy = grid%ifirstxy
      ilastxy  = grid%ilastxy
      jfirstxy = grid%jfirstxy
      jlastxy  = grid%jlastxy

      ktot  = klast - kfirst + 1
      ktotp = ktot + 1

! if (iam .lt. npes_yz) then

! Set general loop limits
! jfirst >= 1; jlast <= jm
      js1g1  = max(1,jfirst-1)
      js2g0  = max(2,jfirst)
      js2g1  = max(2,jfirst-1)
      jn2g0  = min(jm-1,jlast)
      jn1g1  = min(jm,jlast+1)
      jn2g1  = min(jm-1,jlast+1)

!     if( abs(grid%dt0-dt) > D0_1 ) then

      grid%dt0 = dt
      dt5 = D0_5*dt

      grid%dtdy(:)  = dt *grid%rdy(:)
      grid%dtdy5(:) = dt5*grid%rdy(:)
      grid%dydt  = (ae*grid%dp) / dt

      do j=jfirstxy,jlastxy
         grid%dtdx(j)  = dt /grid%dx(j)
         grid%dxdt(j)  = grid%dx(j) / dt
         grid%dtdx2(j) = D0_5*grid%dtdx(j)
         grid%dtdx4(j) = D0_5*grid%dtdx2(j)
      enddo

      do j=jfirstxy+1,jlastxy
         grid%dtdxe(j) = dt / grid%dxe(j)
         grid%dtxe5(j) = D0_5*grid%dtdxe(j)
         grid%txe5(j)  = D0_5/grid%dtdxe(j)
      enddo

! Fill C-grid advecting winds Halo regions
! vc only needs to be ghosted at jlast+1
 
!$OMP PARALLEL DO PRIVATE (K, I, J, ML,IORD, JORD, QWORK, XFX, YFX, CRX, CRY)
   do  k=kfirst,klast     ! This is the main parallel loop.
!---------------------original version-----------------
       if ( k <= km/8 ) then
            iord = 1
            jord = 1
       else
            iord = iord_c
            jord = jord_c
       endif
!------------------------------------------------------
! zy forces
!       iord = iord_c
!       jord = iord_c
!-----------------------------------------------------------------
!  call routine to calculate flux and couranti and q
!-----------------------------------------------------------------
!      qwork(:,jfirst-ng_c:jlast+ng_c,:)  = q(:,jfirst-ng_c:jlast+ng_c,k, :)
       do ml = 1, nq
       do j = jfirst-ng_c, jlast+ng_c
       do i = 1, im
       qwork(i,j,ml)  = q(i,k, ml,j)
       end do
       end do
       end do

       call fx_cr_q( grid,   delp(1,jfirst-ng_c,k), uc(1,jfirst-ng_c,k),  &
                     vc(1,jfirst-ng_d,k),   delpf(1,jfirst,k),  &
                     tiny,   iord,    jord,  xfx,    yfx,   crx,    cry, qwork, k, &
                     lam, dlam, nlon, fullgrid)
!      mfx(:,:,k)     = xfx(:,:)  
!      mfy(:,:,k)     = yfx(:,:)  
!      cx3(:,:,k)     = crx(:,:)  
!      cy3(:,jfirst:jlast+1,k)  = cry(:,jfirst:jlast+1)  
       do ml = 1, nq
       do j = jfirst, jlast
       do i = 1, im
       qout  (i,k, ml, j) = qwork(i,j,ml)
       end do
       end do
       end do
   enddo
!
 return
 end subroutine scan_fxcrq

!------------------------------------------------------
! origins: c_sw
! purpose: compute flux and courant 
!          this is done for each k level 
! revised :zy
! date    :2011-06-16
!------------------------------------------------------

 subroutine  fx_cr_q(grid,   delp,    uc,     vc,   delpf,  &
                     tiny,   iord,    jord,         &
                     xfx,    yfx,     crx,     cry, q, k, lam, dlam, nlon, fullgrid)
! !USES:
! modified for nsplit steps, wzz, 2012.12.8
!
  use tp_core,       only: tp2c
  use fill_module,   only: fillxy
  use mpishorthand
  use commap,        only: w

! !INPUT PARAMETERS:
  type (T_FVDYCORE_GRID), intent(in) :: grid
  integer, intent(in)    :: iord
  integer, intent(in)    :: jord
  integer, intent(in)    :: k
  real(r8), intent(in)   :: dlam(grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8), intent(in)   :: lam(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)

! Prognostic variables:
  real(r8), intent(in)   :: delp(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8), intent(in)   :: tiny
  integer,  intent(in)   :: nlon(grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  logical fullgrid          ! true => no grid reduction towards poles
! !INPUT/OUTPUT PARAMETERS:
  real(r8), intent(inout):: uc(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8), intent(inout):: vc(grid%im,grid%jfirst-grid%ng_d:grid%jlast+grid%ng_d )
  real(r8), intent(inout):: q(grid%im, grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%ntotq)
! !OUTPUT PARAMETERS:
  real(r8), intent(out)  :: delpf(grid%im,grid%jfirst:grid%jlast)
  real(r8), intent(out)  :: xfx(grid%im,grid%jfirst:grid%jlast)
  real(r8), intent(out)  :: yfx(grid%im,grid%jfirst:grid%jlast+1)
  real(r8), intent(out)  :: crx(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8), intent(out)  :: cry(grid%im,grid%jfirst-1:grid%jlast+1)
!
! Magic Number
!  
  real(r8), parameter ::  D0_125                  =  0.125_r8
  real(r8), parameter ::  D0_25                   =  0.25_r8
  real(r8), parameter ::  D2_0                    =  2.0_r8
  real(r8), parameter ::  D1E30                   =  1.0e30_r8
!--------------------------------------------------------------
! Local 
  real(r8) :: zt_c
  real(r8) :: dtdy
  real(r8) :: rcap
  real(r8), pointer:: sc(:)
  real(r8), pointer:: dc(:,:)
  real(r8), pointer:: cosp(:)
  real(r8), pointer:: acosp(:)
  real(r8), pointer:: cose(:)
  real(r8), pointer:: dxdt(:)
  real(r8), pointer:: dxe(:)
  real(r8), pointer:: rdxe(:)
  real(r8), pointer:: dtdx(:)
  real(r8), pointer:: dtdx4(:)
  real(r8), pointer:: dtxe5(:)
  real(r8), pointer:: dycp(:)
  real(r8), pointer:: cye(:)
  real(r8), pointer:: fc(:)
  real(r8), pointer:: sinlon(:)
  real(r8), pointer:: coslon(:)
  real(r8), pointer:: sinl5(:)
  real(r8), pointer:: cosl5(:)

  real(r8) :: flx(grid%im,grid%jfirst:grid%jlast)
  real(r8) :: fly(grid%im,grid%jfirst:grid%jlast+1)
  real(r8) :: dp1(grid%im,grid%jfirst:grid%jlast)
  real(r8) :: dp2(grid%im,grid%jfirst:grid%jlast)
  real(r8) :: q1(grid%im,grid%nq, grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
!
  real(r8) :: fx(grid%im,grid%jfirst:grid%jlast)
  real(r8) :: tm2(grid%im,grid%jfirst:grid%jlast,grid%ntotq)
  real(r8) :: va(grid%im,grid%jfirst:grid%jlast)
  real(r8) :: wk1(grid%im,grid%jfirst-1:grid%jlast+1)
  real(r8) :: fy(grid%im,grid%jfirst:grid%jlast+1)
  real(r8) :: ymass(grid%im,grid%jfirst: grid%jlast+1) 
  real(r8) :: vort_u(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: fxjv(grid%im,grid%jfirst-1:grid%jn2g0)
  real(r8) :: p1dv(grid%im,grid%jfirst-1:grid%jn2g0)
  real(r8) :: cx1v(grid%im,grid%jfirst-1:grid%jn2g0)
  real(r8) :: al(-grid%im/3:grid%im+grid%im/3)
  real(r8) :: ar(-grid%im/3:grid%im+grid%im/3)
  real(r8) :: a6(-grid%im/3:grid%im+grid%im/3)
  real(r8) :: us, vs, un, vn
  real(r8) :: cmax, cymax, frac, tmassf
  real(r8) :: uanp(grid%im), uasp(grid%im), vanp(grid%im), vasp(grid%im)
!
  real(r8) :: wkdp(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: wku(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: wkv(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: wkq(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%ntotq)
  real(r8) :: dpxl(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: dpxr(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: uxl(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: uxr(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: vxl(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: vxr(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: qxl(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%ntotq)
  real(r8) :: qxr(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c,grid%ntotq)
  real(r8) :: wkcrx(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  real(r8) :: wkcry(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  integer  :: idp(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)

! logical  :: ffsl(grid%jm)
  logical  :: ffsl(grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  logical  :: ffsly(grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c)
  logical  :: sldv(grid%jfirst-1:grid%jn2g0)
  integer  :: i, j , iq
  integer  :: js0, jn0, jb 
  integer  :: js1g1, js2gc1, jn2gc, jn1g1, js2g0, js2gc, jn1gc
  integer  :: im, jm, nq, jfirst, jlast, jn2g0, ng_s, ng_c
  integer  :: nsplt, it
  integer  :: ierr
!
!   For convenience
!
  im     = grid%im
  jm     = grid%jm
  nq     = grid%nq

  jfirst = grid%jfirst
  jlast  = grid%jlast

  ng_c   = grid%ng_c
  ng_c   = grid%ng_c
  ng_s   = grid%ng_s
  rcap   = grid%rcap

  zt_c   =  grid%zt_c

  sc     => grid%sc
  dc     => grid%dc

  cosp   => grid%cosp
  acosp  => grid%acosp
  cose   => grid%cose

  dxdt   => grid%dxdt
  dxe    => grid%dxe
  rdxe   => grid%rdxe
  dtdx   => grid%dtdx          ! zy dtdx2->dtdx
  dtdx4  => grid%dtdx4
  dtxe5  => grid%dtxe5
  dycp   => grid%dycp
  cye    => grid%cye
  fc     => grid%fc

  sinlon => grid%sinlon
  coslon => grid%coslon
  sinl5  => grid%sinl5
  cosl5  => grid%cosl5

! Set loop limits

! js2g0  = max(2,jfirst)
! jn1g1  = min(jm,jlast+1)
! js2gc  = max(2,jfirst-ng_c)     ! NG lats on S   ( starting at 2  )
! js2g1  = max(2,jfirst-1)        ! for va calculation
! jn2gc  = min(jm-1,jlast+ng_c)   ! NG latitudes on N (ending at jm-1)
  js2g0  = max(1,jfirst)
  jn1g1  = min(jm+1,jlast+1)
  js2gc  = max(1,jfirst-ng_c)     ! NG lats on S   ( starting at 2  )
  jn2gc  = min(jm,jlast+ng_c)   ! NG latitudes on N (ending at jm-1)
  jn2g0  = grid%jn2g0
! js2g1  = max(1,jfirst-1)        ! for va calculation

  if ( jfirst /= 1 ) then
     do i=1,im
        dtdy   =  grid%dtdy(jfirst-1)  ! zy dtdy5->dtdy
        cry(i,jfirst-1)   = dtdy*vc(i,jfirst-1)
!       ymass(i,jfirst-1) = cry(i,jfirst-1)*cose(jfirst-1)   ! zy adds
     enddo
  endif

!  if(jfirst==17)then
! write(*,*)'wzz: js2g0,jn1g1 in cr_q 0=', js2g0,jn1g1, delp(2,19),vc(2,19), grid%dtdy(19), &
!      grid%dtdy(18)
!  endif
!!!$OMP PARALLEL DO PRIVATE (J, I, DTDY)
  do j=js2g0,jn1g1                     ! ymass needed on NS
     do i=1,nlon(j)
        dtdy   =  grid%dtdy(j)      ! zy dtdy5->dtdy
        cry(i,j)   = dtdy*vc(i,j)
!       ymass(i,j) = cry(i,j)*cose(j)
     enddo
  enddo

! SJL: Check if FFSL integer fluxes need to be computed

! write(*,*)'wzz: js2gc,jn2gc in cr_q 2=', js2gc,jn2gc
! do j=js2gc,jn2gc                ! ffsl needed on N*sg S*sg
! do j=jfirst-ng_c,jlast+ng_c                ! ffsl needed on N*sg S*sg
!    do i=1,im
!       crx(i,j) = uc(i,j)*dtdx(j)
!    enddo
!    if(j==0.or.j==jm+1) then    ! for poles
!       crx(:,j) = ZERO
!    endif
! enddo
!!!$OMP PARALLEL DO PRIVATE (J, I)
  do j=js2gc,jn2gc                ! ffsl needed on N*sg S*sg
     do i=1,nlon(j)
        crx(i,j) = uc(i,j)*dtdx(j)
     enddo
     ffsl(j) = .false.
! comment the following line, wzz
!    if( cosp(j) < zt_c ) then
        do i=1,nlon(j)
         if( abs(crx(i,j)) > D1_0 ) then
             ffsl(j) = .true. 
!            write(*,*)'wzz: ffsl=T ', j
!if ( !defined UNICOSMP ) || ( !defined NEC_SX )
                exit
!endif
           endif
        enddo
!    endif
  enddo

!!!$OMP PARALLEL DO PRIVATE (J, I)
  do j=js2g0,jn1g1                ! ffsl needed on N*sg S*sg
     ffsly(j) = .false.
!    if( cosp(j) < zt_c ) then
        do i=1, nlon(j)
         if( abs(cry(i,j)) > D1_0) then
             ffsly(j) = .true.
!            write(*,*)'wzz: ffsly=T ', i,j,  cry(i,j), jordk, vc(i,j),grid%dtdy(j)
!if ( !defined UNICOSMP ) || ( !defined NEC_SX )
                exit
!endif
           endif
        enddo
!    endif
  enddo

!--------------------------------------------------------------
! 2D transport of polar filtered delp (for computing fluxes!)
! Update is done on the unfiltered delp
!--------------------------------------------------------------
   if ( .not.fullgrid ) then
     do j = jfirst-ng_c, jlast+ng_c
        call cubxdrw (nlon(j),  dlam(j), uc(1,j), uxl(1,j), uxr(1,j))
        call cubxdrw (nlon(j),  dlam(j), vc(1,j), vxl(1,j), vxr(1,j))
     enddo
   endif
!
   if ( fullgrid ) then
! New va definition
!!!$OMP PARALLEL DO PRIVATE (J, I)
   do j=js2g0,jn2g0                     ! va needed on S (for YCC, iv==1)
     do i=1,nlon(j)
        va(i,j) = D0_5*(cry(i,j)+cry(i,j+1))  ! zy remove 0.5
        if(cry(i,j)*cry(i,j+1) > ZERO) then
          if(cry(i,j) > ZERO) then
            va(i,j) = cry(i,j)
          else
            va(i,j) = cry(i,j+1)       ! cy ghosted on N
          endif
        else
            va(i,j) = ZERO
        endif
     enddo
   enddo
!
!!!$OMP PARALLEL DO PRIVATE (J, I)
   do j=js2g0,jn1g1                ! ffsl needed on N*sg S*sg
      if( ffsly(j) ) then
         do i=1, nlon(j)
           ymass(i,j) = cose(j)
         enddo
       else
         do i=1, nlon(j)
           ymass(i,j) = cry(i,j)*cose(j)
         enddo
      endif
   enddo

   call tp2c( delpf,  va,  delp,    &
              crx, cry(1:im,jfirst:jlast+1),             &
              im, jm, 1, iord, jord, ng_c, xfx,              &
              yfx, ffsl, ffsly, rcap, acosp(jfirst:jlast),   &
              crx(1,jfirst), ymass, cosp(jfirst-ng_c:jlast+ng_c),          &
              0, jfirst, jlast, im)
   else
     do j = jfirst-ng_c, jlast+ng_c
        call cubxdrw (nlon(j),  dlam(j), delp(1,j), dpxl(1,j), dpxr(1,j))
     enddo
!
     js0 = jfirst
     jn0 = js0
     do while (js0.le.jlast)
        do j = js0+1, jlast
           jn0 = j
           if(nlon(j)/=nlon(js0)) then
              jn0 = j-1
              exit
           endif
        enddo

       call bandi(im, dlam(js0-ng_c), lam(1,js0),idp(1,js0-ng_c),nlon(js0), js0-ng_c, jn0+ng_c, &
                  nlon(js0-ng_c), fullgrid)
!
       call herxinw(1   ,1   ,uc(1,js0-ng_c)      ,uxl(1,js0-ng_c)   ,uxr(1,js0-ng_c)  , &
                 lam(1,js0-ng_c)    ,idp(1,js0-ng_c)     ,wku(1,js0-ng_c), nlon(js0-ng_c), &
                 nlon(js0)   , im,js0-ng_c, jn0+ng_c, 1, js0  )
       call herxinw(1   ,1   ,vc(1,js0-ng_c)      ,vxl(1,js0-ng_c)   ,vxr(1,js0-ng_c)  , &
                 lam(1,js0-ng_c)    ,idp(1,js0-ng_c)     ,wkv(1,js0-ng_c), nlon(js0-ng_c), &
                 nlon(js0)   , im,js0-ng_c, jn0+ng_c, 1, js0  )
       call herxinw(1   ,1   ,delp(1,js0-ng_c)      ,dpxl(1,js0-ng_c)   ,dpxr(1,js0-ng_c)  , &
                 lam(1,js0-ng_c)    ,idp(1,js0-ng_c)     ,wkdp(1,js0-ng_c), nlon(js0-ng_c), &
                 nlon(js0)   , im,js0-ng_c, jn0+ng_c, 1, js0  )
  do jb=js0,jn0+1                     ! ??! ymass needed on NS
     dtdy   =  grid%dtdy(jb)      ! zy dtdy5->dtdy
     do i=1, nlon(js0)
        wkcry(i,jb)   = dtdy*(wkv(i,jb)+wkv(i,jb-1))*0.5
!       if(i==16.and.jb==2) then
!       write(*,*)'cry at :', wkcry(i,jb), dtdy, wkv(i,jb), wkv(i,jb-1),i,jb,i
!       endif
     enddo
  enddo
  do jb=js0-ng_c, jn0+ng_c
     dtdy   =  dtdx(jb)*dlam(jb)/dlam(js0)
     wkcrx(1,jb) = 0.5*(wku(1,jb)+wku(nlon(js0),jb))*dtdy
     do i=2, nlon(js0)
        wkcrx(i,jb) = 0.5*(wku(i,jb)+wku(i-1,jb))*dtdy
     enddo
  enddo
   do jb=js0,jn0+1      ! ffsl needed on N*sg S*sg
      if( ffsly(jb) ) then
         do i=1, nlon(js0)
           ymass(i,jb) = cose(jb)
         enddo
       else
         do i=1, nlon(js0)
           ymass(i,jb) = wkcry(i,jb)*cose(jb)
         enddo
      endif
   enddo
! New va definition
   do jb=js0, jn0                               ! va needed on S (for YCC, iv==1)
     do i=1, nlon(js0)
        if(wkcry(i,jb)*wkcry(i,jb+1) > ZERO) then
          if(wkcry(i,jb) > ZERO) then
            va(i,jb) = wkcry(i,jb)
          else
            va(i,jb) = wkcry(i,jb+1)       ! cy ghosted on N
          endif
        else
            va(i,jb) = ZERO
        endif
     enddo
   enddo
!
       call tp2c( delpf(:,js0:jn0),  va(1,js0),  wkdp(1,js0-ng_c),    &
                wkcrx(1,js0-ng_c), wkcry(1,js0),     &
                im, jm, 1, iord, jord, ng_c, xfx(1,js0),              &
                yfx(1,js0), ffsl(js0-ng_c), ffsly(js0-ng_c), rcap, acosp(js0:jn0),    &
                wkcrx(1,js0), ymass(1,js0), cosp(js0-ng_c:jn0+ng_c),          &
                0, js0, jn0, nlon(js0))
!
        js0 = jn0+1
        jn0 = js0
      enddo
   endif
!
!  Keep the total mass conserve, so the total delpf should be 0. wzz, 2013.7.8
!
!  tmassf = 0.
!  do j=jfirst,jlast
!     frac = 0.
!     do i=1,nlon(j)
!       frac  = frac + delpf(i,j)
!     enddo
!     tmassf = tmassf + w(j)*frac/nlon(j)
!  enddo
!  if(jfirst==1.and.(k==9.or.k==10)) then
!  write(*,*)'wzz: delpf1=', delpf(1,jlast), frac, jfirst
!  end if
#if (defined SPMD)
!  frac = tmassf
!  call mpiallsum( frac, tmassf, 1, mpir8, mpicom )
!  call mpi_allreduce( frac, tmassf, 1, mpir8, mpi_sum, mpicom, ierr )
#endif
!  tmassf = 0.5*tmassf     ! global mean mass.
!  if(jfirst==1.and.(k==9.or.k==10)) then
!  write(*,*)'wzz: frac=', frac, tmassf, k, jfirst, delpf(1,jlast), delp(1, jlast)
!  end if

!
!!!$OMP PARALLEL DO PRIVATE (J, I)
   do j=jfirst,jlast
      do i=1,nlon(j)
        delpf(i,j) = delp(i,j) + delpf(i,j)
!       delpf(i,j) = delp(i,j) + delpf(i,j) - tmassf   ! wzz, 2013.7.10
        dp1(i,j) = delp(i,j)   ! wzz
      enddo
   enddo
!  if(jfirst==1.and.(k==9.or.k==10)) then
!  write(*,*)'wzz: delpf 1=', delpf(1,jlast), jfirst
!  end if

!        cymax = 0.
!        do j=js2g0,jn1g1
!           cmax = 0.
!           do i=1,nlon(j)
!              cmax = max( abs(cry(i,j)), cmax)
!           enddo
!           if( j==1 .or. j==jm+1 ) then
! Do not allow N-S CFL number at poles to be greater than 0.5
! The limit is 1.0 elsewhere
!               cymax = max(cymax, 2.*cmax)
!!            if(2.*cmax>=1.)then
!!              write(*,*)'wzz: cmax=', cmax, j
!!            endif
!           else
!               cymax = max(cymax, cmax)
!!            if(cmax>=1.)then
!!              write(*,*)'wzz: cmax=', cmax, j
!!            endif
!           endif
!        enddo
!
   if ( fullgrid ) then
!     nsplt = 2     !  set for test here. wzz, 2012.12.8
      nsplt = 1     !  set for test here. wzz, 2012.12.8
   else
      nsplt = 1 
   endif
      frac  = 1. / float(nsplt)
!
      if( nsplt /= 1 ) then
!!!     do j=2,jm-1
             do j=js2gc,jn2gc 
                do i=1,nlon(j)
                   crx(i,j) =  crx(i,j) * frac      ! crx ghosted on N*ng S*ng
                enddo
             enddo

!!!     do j=2,jm-1
             do j=js2g0,jn2g0
                do i=1,nlon(j)
                   xfx(i,j) = xfx(i,j) * frac
                enddo
             enddo

!!!     do j=2,jm
             do j=js2g0,jn1g1
                do i=1,nlon(j)
                   cry(i,j) =  cry(i,j) * frac    ! cry ghosted on N
                   yfx(i,j) = yfx(i,j) * frac    ! mfy ghosted on N
                enddo
             enddo

!!!$OMP PARALLEL DO PRIVATE (J, I)
  do j=js2g0,jn2g0                     ! va needed on S (for YCC, iv==1)
     do i=1,im
        if(cry(i,j)*cry(i,j+1) > ZERO) then
          if(cry(i,j) > ZERO) then
            va(i,j) = cry(i,j)
          else
            va(i,j) = cry(i,j+1)       ! cy ghosted on N
          endif
        else
            va(i,j) = ZERO
        endif
     enddo
  enddo

!!!$OMP PARALLEL DO PRIVATE (J, I)
  do j=js2gc,jn2gc                ! ffsl needed on N*sg S*sg
     ffsl(j) = .false.
! comment the following line, wzz
!    if( cosp(j) < zt_c ) then
        do i=1,nlon(j)
         if( abs(crx(i,j)) > D1_0 ) then
             ffsl(j) = .true. 
!            write(*,*)'wzz: ffsl=T ', j
!if ( !defined UNICOSMP ) || ( !defined NEC_SX )
                exit
!endif
           endif
        enddo
!    endif
  enddo

!!!$OMP PARALLEL DO PRIVATE (J, I)
  do j=js2g0,jn1g1                ! ?????needed on N*sg S*sg
     ffsly(j) = .false.
!    if( cosp(j) < zt_c ) then
        do i=1, nlon(j)
         if( abs(cry(i,j)) > D1_0) then
             ffsly(j) = .true.
!            write(*,*)'wzz: ffsly=T ', i,j,  cry(i,j), jordk, vc(i,j),grid%dtdy(j)
!if ( !defined UNICOSMP ) || ( !defined NEC_SX )
                exit
!endif
           endif
        enddo
!    endif
  enddo

      end if  !if( nsplt /= 1 ) 

!  if(jfirst==1) write(*,*)'wzz: fullgrid', fullgrid
   if ( fullgrid ) then
!!!$OMP PARALLEL DO PRIVATE (J, I)
   do j=js2g0,jn2g0                      ! xfx not ghosted
      if( ffsl(j) ) then
         do i=1,nlon(j)
!          if(i==10) then
!            write(*,*)'wzz: xfx 0=', xfx(i,j),  &
!             sign(max(abs(crx(i,j)),tiny),crx(i,j)), crx(i,j), j
!          end if
           flx(i,j) = xfx(i,j)/sign(max(abs(crx(i,j)),tiny),crx(i,j))
!          if(i==10) then
!            write(*,*)'wzz: xfx 1=', xfx(i,j)
!          end if
         enddo
      else
         do i = 1, nlon(j)
            flx(i,j) = xfx(i,j)
         enddo
      endif
   enddo

!!!$OMP PARALLEL DO PRIVATE (J, I)
   do j=js2g0,jn1g1                ! ?????needed on N*sg S*sg
      if( ffsly(j) ) then
         do i=1,nlon(j)
!          if(i==10) then
!            write(*,*)'wzz: xfx 0=', xfx(i,j),  &
!             sign(max(abs(crx(i,j)),tiny),crx(i,j)), crx(i,j), j
!          end if
           fly(i,j) = yfx(i,j)/sign(max(abs(cry(i,j)),tiny),cry(i,j))
!          if(i==10) then
!            write(*,*)'wzz: xfx 1=', xfx(i,j)
!          end if
         enddo
      else
         do i = 1, nlon(j)
            fly(i,j) = yfx(i,j)
         enddo
      endif
   enddo
   endif  !if ( fullgrid )
!
  do it = 1, nsplt
   if ( fullgrid ) then
!!!$OMP PARALLEL DO PRIVATE (J, I)
     do j=jfirst,jlast
        do i=1,nlon(j)-1
           dp2(i,j) =  xfx(i,j) - xfx(i+1,j) +  &
                     (yfx(i,j) - yfx(i,j+1)) * acosp(j)
        enddo
        dp2(nlon(j),j) =  xfx(nlon(j),j) - xfx(1,j) +  &
                      (yfx(nlon(j),j) - yfx(nlon(j),j+1)) * acosp(j)
     enddo
!!!$OMP PARALLEL DO PRIVATE (J, I)
     do j=jfirst,jlast
        do i=1,nlon(j)
           dp2(i,j) =  dp1(i,j) + dp2(i,j)
        enddo
     enddo

!----------------------------------------------------------
! q -advection using pre-computed flux : xfx, xfy
!----------------------------------------------------------

   call tp2c( tm2 ,va(1,jfirst),  q(1,jfirst-ng_c,1),     &
              crx(1,jfirst-ng_c), cry(1,jfirst),          &
              im, jm,  nq, iord, jord, ng_c, fx,          &
              fy, ffsl, ffsly, rcap, acosp(jfirst:jlast),   &
              flx, fly, cosp(jfirst-ng_c:jlast+ng_c), 1, jfirst, jlast, im)
   else
! wzz, 2012.12.8
    do iq=1,nq
      do j=jfirst-ng_c,jlast+ng_c
         do i=1,nlon(j)
           q(i,j,iq)   = q(i,j,iq)*delp(i,j)
!          if(q(i,j,iq)<0. .and.j==400.and.iq==3.and.k==23.and.i==250) then
!          if(q(i,j,iq)<0. .and.j==400.and.iq==3.and.k==23) then
!            write(*,*)'wzz: q00<0:', q(i,j,iq), i, j, iq, delp(i,j)
!          endif
         enddo
      enddo
    end do
!
     do iq = 1, nq
        do j = jfirst-ng_c, jlast+ng_c
        call cubxdrw (nlon(j),  dlam(j), q(1,j,iq), qxl(1,j,iq), qxr(1,j,iq))
        enddo
     enddo
!
     js0 = jfirst
     jn0 = js0
     do while (js0.le.jlast)
        do j = js0+1, jlast
           jn0 = j
           if(nlon(j)/=nlon(js0)) then
              jn0 = j-1
              exit
           endif
        enddo

       call bandi(im, dlam(js0-ng_c), lam(1,js0),idp(1,js0-ng_c),nlon(js0), js0-ng_c, jn0+ng_c, &
                  nlon(js0-ng_c), fullgrid)
!
       call herxinw(1   ,1   ,uc(1,js0-ng_c)      ,uxl(1,js0-ng_c)   ,uxr(1,js0-ng_c)  , &
                 lam(1,js0-ng_c)    ,idp(1,js0-ng_c)     ,wku(1,js0-ng_c), nlon(js0-ng_c), &
                 nlon(js0)   , im,js0-ng_c, jn0+ng_c, 1, js0  )
       call herxinw(1   ,1   ,vc(1,js0-ng_c)      ,vxl(1,js0-ng_c)   ,vxr(1,js0-ng_c)  , &
                 lam(1,js0-ng_c)    ,idp(1,js0-ng_c)     ,wkv(1,js0-ng_c), nlon(js0-ng_c), &
                 nlon(js0)   , im,js0-ng_c, jn0+ng_c, 1, js0  )
       call herxinw(nq  ,nq  ,q(:,js0-ng_c:jn0+ng_c,:)      ,  &
            qxl(:,js0-ng_c:jn0+ng_c,:)   ,qxr(:,js0-ng_c:jn0+ng_c,:)  , &
            lam(1,js0-ng_c), idp(1,js0-ng_c), wkq(:,js0-ng_c:jn0+ng_c,:), nlon(js0-ng_c), &
            nlon(js0)   , im,js0-ng_c, jn0+ng_c, 1, js0  )
  do jb=js0,jn0+1                     ! ??! ymass needed on NS
     dtdy   =  grid%dtdy(jb)      ! zy dtdy5->dtdy
     do i=1, nlon(js0)
        wkcry(i,jb)   = dtdy*(wkv(i,jb)+wkv(i,jb-1))*0.5
!       if(i==16.and.jb==2) then
!       write(*,*)'cry at :', wkcry(i,jb), dtdy, wkv(i,jb), wkv(i,jb-1),i,jb,i
!       endif
     enddo
  enddo
  do jb=js0-ng_c, jn0+ng_c
     dtdy   =  dtdx(jb)*dlam(jb)/dlam(js0)
     wkcrx(1,jb) = 0.5*(wku(1,jb)+wku(nlon(js0),jb))*dtdy
     do i=2, nlon(js0)
        wkcrx(i,jb) = 0.5*(wku(i,jb)+wku(i-1,jb))*dtdy
     enddo
  enddo
   do jb=js0,jn0+1      ! ffsl needed on N*sg S*sg
      if( ffsly(jb) ) then
         do i=1, nlon(js0)
           ymass(i,jb) = cose(jb)
         enddo
       else
         do i=1, nlon(js0)
           ymass(i,jb) = wkcry(i,jb)*cose(jb)
         enddo
      endif
   enddo
! New va definition
   do jb=js0, jn0                               ! va needed on S (for YCC, iv==1)
     do i=1, nlon(js0)
        if(wkcry(i,jb)*wkcry(i,jb+1) > ZERO) then
          if(wkcry(i,jb) > ZERO) then
            va(i,jb) = wkcry(i,jb)
          else
            va(i,jb) = wkcry(i,jb+1)       ! cy ghosted on N
          endif
        else
            va(i,jb) = ZERO
        endif
     enddo
   enddo

       call tp2c( tm2(:,js0:jn0,:),  va(1,js0),  wkq(:,js0-ng_c:jn0+ng_c,:),    &
                wkcrx(1,js0-ng_c), wkcry(1,js0),     &
                im, jm, nq, iord, jord, ng_c, xfx(1,js0),              &
                yfx(1,js0), ffsl(js0-ng_c), ffsly(js0-ng_c), rcap, acosp(js0:jn0),    &
                wkcrx(1,js0), ymass(1,js0), cosp(js0-ng_c:jn0+ng_c),          &
                0, js0, jn0, nlon(js0))
!
        js0 = jn0+1
        jn0 = js0
     enddo

   endif

!-------------------------------
! use wk4, crx as work arrays
! filter--optional
!-------------------------------

!     call pft2d(ptk(1,js2g0), sc,   &
!                dc, im, jn2g0-js2g0+1,  &
!                wk4, crx )
!     call pft2d(tm2(1,js2g0), sc,   &
!                dc, im, jn2g0-js2g0+1,  &
!                wk4, crx )

   do iq=1,nq
!     if(jfirst==17)then
!        write(*,*)'wzz: q=', q(2,18,iq),delp(2,18),tm2(2,18,iq),iq
!     endif
    if ( fullgrid ) then
      do j=jfirst,jlast
         do i=1,nlon(j)
!          q(i,j,iq)   = q(i,j,iq) + tm2(i,j,iq)
!          q(i,j,iq)   = q(i,j,iq)*delp(i,j) + tm2(i,j,iq)
           q(i,j,iq)   = q(i,j,iq)*dp1(i,j) + tm2(i,j,iq)
!          if(q(i,j,iq)<0. .and.j==400.and.iq==3.and.k==23.and.i==250) then
!            write(*,*)'wzz: q0<0:', q(i,j,iq), i, j, iq, tm2(i,j,iq)
!          endif
         enddo
      enddo
    else
      do j=jfirst,jlast
         do i=1,nlon(j)
           q(i,j,iq)   = q(i,j,iq) + tm2(i,j,iq)
!          if(q(i,j,iq)<0. .and.j==250.and.iq==2.and.k==25.and.i==59) then
!          if(q(i,j,iq)/=q(i,j,iq) .and.iq==3.and.k==1.and.j==5) then
!          if(q(i,j,iq)/=q(i,j,iq) .and.j==5) then
!            write(*,*)'wzz: q0<0:', q(i,j,iq), i, j, iq, tm2(i,j,iq),k,delp(i,j),nlon(j)
!          endif
         enddo
      enddo
    endif

      call fillxy ( q(1,jfirst,iq), im, jm, jfirst, &
                    jlast, grid%acap, grid%cosp(jfirst:jlast), grid%acosp(jfirst:jlast), nlon(jfirst:jlast))
   
    if ( fullgrid ) then
!!!$OMP PARALLEL DO PRIVATE (J, I)
      do j=jfirst,jlast
         do i=1,nlon(j)
!          q(i,j,iq)   = q(i,j,iq) / delpf(i,j)
           q(i,j,iq)   = q(i,j,iq) / dp2(i,j)
!          if(q(i,j,iq)<0. .and.j==400.and.iq==3.and.k==23.and.i==250) then
!            write(*,*)'wzz: q1<0:', q(i,j,iq), i, delpf(i,j)
!          endif
         enddo
      enddo
    else
!$OMP PARALLEL DO PRIVATE (J, I)
      do j=jfirst,jlast
         do i=1,nlon(j)
           q(i,j,iq)   = q(i,j,iq) / delpf(i,j)
!          if(q(i,j,iq)<0. .and.j==250.and.iq==2.and.k==25.and.i==59) then
!          if(q(i,j,iq)/=q(i,j,iq).or.q(i,j,iq)<0.) then
!            write(*,*)'wzz: q1<0:', q(i,j,iq), i, j,k,iq,delpf(i,j)
!          endif
         enddo
      enddo
    endif !if ( fullgrid )
   enddo
!
    if(nsplt>1.and.it<nsplt) then
#ifdef SPMD
! exchange the boundary values
   do iq=1,nq
      do j=jfirst,jlast
         do i=1,im
           q1(i,iq,j) = q(i,j,iq)
         enddo
      enddo
   enddo
      call mp_bndexch_q(q1,im,jm, nq, jfirst,jlast, ng_c)
   do iq=1,nq
      do j=jfirst-ng_c,jlast+ng_c
         do i=1,im
           q(i,j,iq) = q1(i,iq,j)
         enddo
      enddo
   enddo
#endif
      do j=jfirst,jlast
         do i=1,im
            dp1(i,j) = dp2(i,j)
         enddo
      enddo
    endif
!
   enddo   ! it = 1, nsplt
!
   return
 end subroutine  fx_cr_q 
!--------------------------------------------------------------------------
 end module ffsl_mod 
