#include <misc.h>
!------------------------------------------------------
! origins: dynamics_init and dynpkg_init
! purpose: initilize grid info
! revised: zy
! date   : 2011-06-16
!------------------------------------------------------
  module grid_mod

    use shr_kind_mod,   only: r8 => shr_kind_r8
    use dynamics_vars,  only: T_FVDYCORE_GRID
    use dynamics_vars,  only: D0_0, D0_5, D1_0, D2_0, D4_0, D10_0, D180_0, ratmax
    use pmgrid,         only: iam
    use rgrid,          only: nlon

    implicit none

    public  :: grid_init, grid_clean

CONTAINS
  subroutine   grid_init( ks, ak, bk, phi, dphi,  &
                          lam, dlam, grid )

! !USES:
      use shr_const_mod,  only: pi=>shr_const_pi, ae=>shr_const_rearth, om=>shr_const_omega
      use fv_control_mod, only: trac_decomp, jord
      use pmgrid,         only: plon, plat, plev, plevp, beglat, endlat, &
                                beglatex, endlatex, j1, plond, platd
      use constituents,   only: pcnst
!
! !INPUT PARAMETERS:

      integer,  intent(in)    :: ks                    !ok!  True # pressure levels

      real(r8), intent(in)    :: ak(plevp)             !ok! hybrid coordiante
      real(r8), intent(in)    :: bk(plevp)             !ok! hybrid coordiante
      real(r8), intent(in)    :: phi(platd)            ! latitude  coords of extended grid
      real(r8), intent(in)    :: dphi(platd)           ! latitude intervals (radians)
      real(r8), intent(in)    :: lam(plond,platd)      ! longitude coords of extended grid
      real(r8), intent(in)    :: dlam(platd)           ! longitudinal grid interval (radians)
!
! !INPUT/OUTPUT PARAMETERS:
      type(T_FVDYCORE_GRID), intent(inout)  :: grid     ! Resulting grid

! !DESCRIPTION:
!
!-----------------------------------------------------------------------
!
!
      grid%iam      = iam
!
!  Set the basic grid variables

      grid%im       = plon
      grid%jm       = plat
      grid%km       = plev
      grid%kmax     = plevp
      grid%nq       = pcnst
      grid%ntotq    = pcnst
      grid%ks       = ks  
  
!
! zy adds for later use
!
      grid%ifirst   = 1
      grid%ilast    = plon
!
      grid%jfirst   = beglat
      grid%jlast    = endlat
! wzz, 2012.7.28
      grid%jfirstxy = beglatex-j1+1
      grid%jlastxy  = endlatex-j1+1
!
      grid%kfirst   = 1
      grid%klast    = plev
!     if ( klast == km ) then
!       grid%klastp = km+1
!     else
!       grid%klastp = klast
!     endif
      grid%klastp = grid%klast + 1

!     write(*,*)'wzz: platd=', platd, j1
!     write(*,*)'wzz: phi=', phi(j1-1:)
!     write(*,*)'wzz: dphi=', dphi(j1-1:)
!     write(*,*)'wzz: dlam=', dlam(j1-1:)
      call dynpkg_init( pi, ae, om, jord, phi(beglatex:endlatex), dphi(beglatex:endlatex), &
                        dlam(beglatex:endlatex), grid )

!
! Level-dependent variables  (was in vert_init, now removed)
!
      ALLOCATE(GRID%AK(plevp))
      ALLOCATE(GRID%BK(plevp))
      GRID%AK   = AK
      GRID%BK   = BK
      GRID%PTOP = GRID%AK(1)
      GRID%PINT = GRID%AK(ks)
!
! Tracer decomposition limits
!
      allocate(grid%ktloa(trac_decomp))
      allocate(grid%kthia(trac_decomp))
      grid%ktloa(:) = 1
      grid%kthia(:) = pcnst
      grid%ktlo = 1
      grid%kthi = pcnst

      return
  end subroutine grid_init

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: grid_clean -- clean up Lin-Rood-specific variables
!
! !INTERFACE:
  subroutine   grid_clean( grid )
! !INPUT/OUTPUT PARAMETERS:
      type(T_FVDYCORE_GRID), intent(inout)  :: grid     ! Resulting grid
!

    call dynpkg_clean( grid )

    if(associated(GRID%AK              )) deallocate(GRID%AK)
    if(associated(GRID%BK              )) deallocate(GRID%BK)
    if(associated(GRID%ktloa           )) deallocate(GRID%ktloa)
    if(associated(GRID%kthia           )) deallocate(GRID%kthia)

  end subroutine   grid_clean

!----------------------------------------------------------------------- 
!BOP
! !ROUTINE:  dynpkg_init --- Initialization for dynamics package
!
! !INTERFACE:
  subroutine dynpkg_init( pi, ae, om, jord, phi, dphi, dlam, grid )

! !USES:
   use pmgrid,     only: platd, plat, j1
   use rgrid,      only: nlon
   use abortutils, only: endrun
!
   implicit none

! !INPUT PARAMETERS:
      real(r8),  intent(in) :: pi
      real(r8),  intent(in) :: ae
      real(r8),  intent(in) :: om
      type( T_FVDYCORE_GRID ), intent(inout) :: grid
      real(r8),  intent(in) :: phi(grid%jfirstxy:grid%jlastxy)         ! latitude  coords of extended grid
      real(r8),  intent(in) :: dphi(grid%jfirstxy:grid%jlastxy)        ! latitude intervals (radians)
      real(r8),  intent(in) :: dlam(grid%jfirstxy:grid%jlastxy)        ! longitudinal grid interval (radians)

      integer, intent(in)   :: jord

! !INPUT/OUTPUT PARAMETERS:
!     type( T_FVDYCORE_GRID ), intent(inout) :: grid


! !DESCRIPTION:
! 
!   {\bf Purpose:} Initialization of the FV specific GRID vars
! 
! !REVISION HISTORY: 
!   00.01.10    Grant        Creation using code from SJ Lin
!   01.03.26    Sawyer       Added ProTeX documentation
!   01.06.06    Sawyer       Modified for dynamics_vars
!   04.08.25    Sawyer       Now updates GRID
!   05.06.30    Sawyer       Added initializations from cd_core
!   06.09.15    Sawyer       PI now passed as argument
!
!EOP
!-----------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
      integer  :: im, jm
      integer  :: i, j, imh, js2g0, jn2g0, jn1g1, js2gc, jn1gc
      integer  :: js2gs, jn2gd, jn1gs
      integer  :: jfirstxy, jlastxy
      real(r8) :: zam5, zamda
      real(r8) :: ph5      ! This is to ensure 64-bit for any choice of r8

      real(r8), pointer :: coslon(:), sinlon(:), cosl5(:), sinl5(:)
      real(r8), pointer :: cosp(:), sinp(:), cose(:), sine(:), acosp(:), acosu(:)

!
! Local variables from cd_core
!

      integer  :: icffta
      real(r8) :: rcffta

      real(r8) :: rat, ycrit

      im = grid%im
      jm = grid%jm
      jfirstxy = grid%jfirstxy
      jlastxy  = grid%jlastxy
!
! Start initialization
!
      grid%dl  = (pi+pi)/im   !  ??? for reduced grids
      grid%dp  = pi/(jm+1.)
!     write(*,*)'wzz: dl, dp=', grid%dl,  grid%dp

      allocate(grid%cosp(jfirstxy:jlastxy))
      allocate(grid%sinp(jfirstxy:jlastxy))
      allocate(grid%cose(jfirstxy:jlastxy))
      allocate(grid%sine(jfirstxy:jlastxy+1))
!     allocate(grid%sine(jfirstxy:jlastxy))
      allocate(grid%acosp(jfirstxy:jlastxy))
      allocate(grid%acosu(jfirstxy:jlastxy))
      allocate(grid%dx(jfirstxy:jlastxy))
      allocate(grid%rdx(jfirstxy:jlastxy))

      allocate(grid%coslon(im))
      allocate(grid%sinlon(im))
      allocate(grid%cosl5(im))
      allocate(grid%sinl5(im))

      cosp => grid%cosp
      sinp => grid%sinp
      cose => grid%cose
      sine => grid%sine
      acosp  => grid%acosp
      acosu  => grid%acosu

      coslon => grid%coslon
      sinlon => grid%sinlon
      cosl5  => grid%cosl5
      sinl5  => grid%sinl5

!     do j=jfirstxy+1, jlastxy
      do j=jfirstxy, jlastxy+1
         ph5  = -D0_5*pi + (j-D0_5)*grid%dp
!        ph5  = -D0_5*pi + ((j-1)-D0_5)*(pi/(jm-1))
!        ph5  = D0_5*(phi(j)+phi(j-1))
!        write(*,*)'wzz: ph5 1=', ph5, j
         sine(j) = sin(ph5)
      enddo

      !
      ! cos(theta) at cell center distretized as
      !
      ! cos(theta) = d(sin(theta))/d(theta)
      !
      do j=jfirstxy, jlastxy
!        cosp(j) = cos(phi(j))
         cosp(j) = (sine(j+1) - sine(j))/grid%dp

!        grid%dx(j)    = dlam(j)*ae*cosp(j)
         grid%dx(j)    = dlam(j)*ae*abs(cosp(j))
         grid%rdx(j)   = D1_0 / grid%dx(j)
!        if(j==0.or.j==jm+1) then   ! for poles
!        write(*,*)'wzz: dx 0 =', grid%dx(j), cosp(j), j, dlam(j), grid%rdx(j)
!        end if
      enddo
!     cosp( 1) =  D0_0
!     cosp(jm) =  D0_0

! Define cosine at edges..

      do j=jfirstxy+1, jlastxy
         cose(j) = D0_5 * (cosp(j-1)+cosp(j))
!        cose(j) = cos(D0_5 * (phi(j-1) + phi(j)))
      enddo
!     cose(1) = cose(2)

      do j=jfirstxy+1, jlastxy-1
         acosu(j) = D2_0 / (cose(j) + cose(j+1))
      enddo

      do j=jfirstxy, jlastxy
         sinp(j) = sin(phi(j))
      enddo
!     sinp( 1) = -D1_0
!     sinp(jm) =  D1_0

!
! Pole cap area and inverse
!     grid%acap = im*(D1_0+sine(2)) / grid%dp
!     grid%acap = im*cosp(1)
!     grid%acap = im*(D1_0+sine(2)) / (dphi(0)+dphi(1))*0.5
      if(grid%jfirst==1) then
!        grid%acap = im*cose(1)*0.5
         grid%acap = nlon(1)*(D1_0+sine(1)) / grid%dp
      else if(grid%jlast==jm) then
!        grid%acap = im*cose(jm+1)*0.5
         grid%acap = nlon(jm)*(D1_0-sine(jm+1)) / grid%dp
      else
         grid%acap = im*0.5   ! set defaults.
      end if
      grid%rcap = D1_0 / grid%acap
!     write(*,*)'wzz: acap =', grid%acap, grid%dp, grid%rcap
 
      do j=jfirstxy, jlastxy
         acosp(j) = D1_0 / cosp(j)
      enddo
      if(grid%jfirst==1) then
      acosp( 0) = grid%rcap * nlon(1)   ! south pole
!     acosp( 0) = grid%rcap * im   ! south pole
      cosp(0) = D1_0 /acosp(0)
      write(*,*)'wzz: acosp=', acosp
      end if
      if(grid%jlast==jm) then
      acosp(jm+1) = grid%rcap * nlon(jm)   ! north pole
!     acosp(jm+1) = grid%rcap * im   ! north pole
      cosp(jm+1) = D1_0 /acosp(jm+1)
      write(*,*)'wzz: acosp=', acosp
      end if

      imh = im/2
      if(im .ne. 2*imh) then
         call endrun('im must be an even integer')
      endif
 
! Define logitude at the center of the volume
! i=1, Zamda = -pi
 
      do i=1,imh
         zam5          = ((i-1)-D0_5) * grid%dl
         cosl5(i)      =  cos(zam5)
         cosl5(i+imh)  = -cosl5(i)
         sinl5(i)      =  sin(zam5)
         sinl5(i+imh)  = -sinl5(i)
         zamda         = (i-1)*grid%dl
         coslon(i)     =  cos(zamda)
         coslon(i+imh) = -coslon(i)
         sinlon(i)     =  sin(zamda)
         sinlon(i+imh) = -sinlon(i)
      enddo

!
! calculate the ghost region sizes
!
      grid%ng_c = j1-1                 ! Avoid the case where ng_c = 1
      grid%ng_d = min( abs(jord), j1-1)   ! SJL: number of max ghost latitudes
      grid%ng_d = max( grid%ng_d, j1-2)
      grid%ng_s = max( grid%ng_c+1, grid%ng_d )
!     grid%ng_c = 3
!     grid%ng_s = 3
!     grid%ng_c = 0
!     grid%ng_d = 0                   ! No ghosting necessary for pure SMP runs
!     grid%ng_s = 0
!
! cd_core initializations
!

      allocate(grid%dtdx(jfirstxy:jlastxy))
      allocate(grid%dtdx2(jfirstxy:jlastxy))
      allocate(grid%dtdx4(jfirstxy:jlastxy))
      allocate(grid%dtdxe(jfirstxy:jlastxy))
      allocate(grid%dxdt(jfirstxy:jlastxy))
      allocate(grid%dxe(jfirstxy:jlastxy))
      allocate(grid%cye(jfirstxy:jlastxy))
      allocate(grid%dycp(jfirstxy:jlastxy))
      allocate(grid%rdxe(jfirstxy:jlastxy))
      allocate(grid%txe5(jfirstxy:jlastxy))
      allocate(grid%dtxe5(jfirstxy:jlastxy))
      allocate(grid%dyce(jfirstxy:jlastxy))
      allocate(grid%cy(jfirstxy:jlastxy))
      allocate(grid%rdy(jfirstxy:jlastxy))
      allocate(grid%dtdy(jfirstxy:jlastxy))
      allocate(grid%dtdy5(jfirstxy:jlastxy))

!     js2g0  = max(2,grid%jfirst)
!     jn2g0  = min(jm-1,grid%jlast)
!     jn1g1  = min(jm,grid%jlast+1)
      js2g0  = max(1,grid%jfirst)
      jn2g0  = min(jm,grid%jlast)
      jn1g1  = min(jm+1,grid%jlast+1)
      js2gc  = max(2,grid%jfirst-grid%ng_c) ! NG lats on S (starting at 2)
      jn1gc  = min(jm,grid%jlast+grid%ng_c) ! ng_c lats on N (ending at jm)

      grid%js2g0  = js2g0
      grid%jn2g0  = jn2g0
      grid%jn1g1  = jn1g1

      js2gs = max(2,grid%jfirst-grid%ng_s)
      jn2gd = min(jm-1,grid%jlast+grid%ng_d)
      jn1gs = min(jm,grid%jlast+grid%ng_s)

      allocate(grid%sc(js2g0:jn2g0))
      allocate(grid%se(js2g0:jn1g1))
      allocate(grid%dc(im,js2g0:jn2g0))
      allocate(grid%de(im,js2g0:jn1g1))

      grid%dt0 = D0_0
!     do j=jfirstxy+1,jlastxy
      do j=jfirstxy,jlastxy
         grid%rdy(j)   = D1_0/(ae*grid%dp)
!        grid%rdy(j)   = D1_0/(ae*dphi(j-1))
      enddo

      do j=jfirstxy+1,jlastxy-1
         grid%dycp(j)  = ae*dphi(j-1)/grid%cosp(j)
         grid%cy(j)    = grid%rdy(j) * grid%acosp(j)
      enddo

      do j=jfirstxy+1,jlastxy
!        grid%dxe(j)   = ae*grid%dl*grid%cose(j)   ! ??? For reduced grids?
         grid%dxe(j)   = ae*dlam(j)*grid%cose(j)   ! ??? For reduced grids?
         grid%rdxe(j)  = D1_0 / grid%dxe(j)
         grid%cye(j)   = D1_0 / (ae*grid%cose(j)*dphi(j-1))
         grid%dyce(j)  = ae*dphi(j-1)/grid%cose(j)
      enddo

!     allocate(grid%scdiv4(js2gs:jn2gd))   !for filtering of u and v in div4 damping 
!     allocate(grid%sediv4(js2gs:jn1gs))   !for filtering of u and v in div4 damping 
!     allocate(grid%dcdiv4(im,js2gs:jn2gd))!for filtering of u and v in div4 damping 
!     allocate(grid%dediv4(im,js2gs:jn1gs))!for filtering of u and v in div4 damping 

!      call pftinit(im)

! Determine ycrit such that effective DX >= DY
      rat = real(im,r8)/real(2*(jm-1),r8)
      ycrit = acos( min(ratmax, rat) ) * (D180_0/pi)

!      call pft_cf(im, jm, js2g0, jn2g0, jn1g1, &
!                  grid%sc, grid%se, grid%dc, grid%de,  &
!                  grid%cosp, grid%cose, ycrit)

      !for filtering of u and v in div4 damping 
      !(needs larger halo than cam3.5 code)
!      call pft_cf(im, jm, js2gs, jn2gd, jn1gs,                             & 
!                  grid%scdiv4, grid%sediv4, grid%dcdiv4, grid%dediv4,      & 
!                  grid%cosp, grid%cose, ycrit)                               

! 000304 bug fix: ng_s not ng_d
      allocate( grid%f0(grid%jfirst-grid%ng_s-1:grid%jlast+grid%ng_d) )
      allocate( grid%fc(js2gc:jn1gc) )

! 000304 bug fix
      do j=max(1,grid%jfirst-grid%ng_s-1),min(jm,grid%jlast+grid%ng_d)
        grid%f0(j) = (om+om)*grid%sinp(j)
      enddo

! Compute coriolis parameter at cell corners.
      do j=js2gc, jn1gc                    ! Not the issue with ng_c = ng_d ! ????
         grid%fc(j) = D0_5*(grid%f0(j) + grid%f0(j-1))
      enddo

! C-grid
      grid%zt_c = cos( D10_0 * pi / D180_0 )

  end subroutine dynpkg_init

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: dynpkg_clean -- clean up Lin-Rood-specific variables
!
! !INTERFACE:
  subroutine   dynpkg_clean( grid )
! !INPUT/OUTPUT PARAMETERS:
      type(T_FVDYCORE_GRID), intent(inout)  :: grid     ! Resulting grid
!
      if(associated(grid%cosp     )) deallocate(grid%cosp)
      if(associated(grid%sinp     )) deallocate(grid%sinp)
      if(associated(grid%cose     )) deallocate(grid%cose)
      if(associated(grid%sine     )) deallocate(grid%sine)
      if(associated(grid%acosp    )) deallocate(grid%acosp)
      if(associated(grid%acosu    )) deallocate(grid%acosu)

      if(associated(grid%coslon   )) deallocate(grid%coslon)
      if(associated(grid%sinlon   )) deallocate(grid%sinlon)
      if(associated(grid%cosl5    )) deallocate(grid%cosl5)
      if(associated(grid%sinl5    )) deallocate(grid%sinl5)

      if(associated(grid%dx       )) deallocate(grid%dx)
      if(associated(grid%rdx      )) deallocate(grid%rdx)

      if(associated(grid%dtdx    )) deallocate(grid%dtdx)
      if(associated(grid%dtdx2   )) deallocate(grid%dtdx2)
      if(associated(grid%dtdx4   )) deallocate(grid%dtdx4)
      if(associated(grid%dtdxe   )) deallocate(grid%dtdxe)
      if(associated(grid%dxdt    )) deallocate(grid%dxdt)
      if(associated(grid%dxe     )) deallocate(grid%dxe)
      if(associated(grid%cye     )) deallocate(grid%cye)
      if(associated(grid%dycp    )) deallocate(grid%dycp)
      if(associated(grid%rdxe    )) deallocate(grid%rdxe)
      if(associated(grid%txe5    )) deallocate(grid%txe5)
      if(associated(grid%dtxe5   )) deallocate(grid%dtxe5)
      if(associated(grid%dyce    )) deallocate(grid%dyce)
      if(associated(grid%cy      )) deallocate(grid%cy)

      if(associated(grid%sc            )) deallocate(grid%sc)
      if(associated(grid%se            )) deallocate(grid%se)
      if(associated(grid%dc            )) deallocate(grid%dc)
      if(associated(grid%de            )) deallocate(grid%de)
!
      if(associated(GRID%f0              )) deallocate(GRID%f0)
      if(associated(GRID%fc              )) deallocate(GRID%fc)

  end subroutine   dynpkg_clean

  end module grid_mod
