
module caltime

!  SOME OPERATIONS ABOUT DATE ARE IN THIS MODULE.

   use shr_kind_mod, only: r8 => shr_kind_r8
   use calstring, only: caps_string

   implicit none
   public :: catimc, dayas
   private

!=========================================================================================
contains
!=========================================================================================

   SUBROUTINE CATIMC(ITIME,ITIM0,IDT,CUN)
!**  ---------- SUBROUTINE CATIMC(ITIME,ITIME0,IDT,CUN)
!  PURPOSE:
!    CALCULATING THE CHANGED DATE
!  INPUT:
!    ITIME:  THE INITIAL DATE
!    IDT:  THE VARIATE TIME
!  RETURN:
!    ITIM0:  THE CHANGED DATE
!  COMMANDS:
!    THE FORMAT OF ITIME & ITIM0: YYMMDDHH(I8),
!    AND THE RETURN TIME:
!      ITIM0=ITIME+IDT
!      THE UNIT OF IDT:  CUN
!  BY WWW.
!  THIS SUBROUTINE IS MODIFIED FOR THE CENTURY-WORM PROBLEM.
!  1999.3.19,WZZ.
!  To solve the century-worm problem completely, now the input
!     ITIME is the full length of time. It means that ITIME do not
!     need the limitation of (I8) format.
!    CUN:  THE UNIT OF INCREASING TIME 'IDT'
!          YR:   year
!          MO:   month
!          DY:   day
!          HR:   hour
!          The default value is 'HR'
!  2000.3.2, WZZ
!  *******************************************************************
   integer, intent(in) :: ITIME, IDT
   CHARACTER CUN*2
   integer, intent(out) :: ITIM0
!
! local variables
   integer IYEAR, MONTH, IDAY, IHOUR, IDAYM
   integer MON(12)
   DATA MON/31,28,31,30,31,30,31,31,30,31,30,31/
   CHARACTER CH1*2
!
   CALL DEITIME(ITIME,IYEAR,MONTH,IDAY,IHOUR)
   MON(2) = MON2(IYEAR)
!  set the default vaule of CH1
   CH1=CUN
   CALL CAPs_string(CH1)
!
   select case (CH1)
!
      case ('YR')
! unit is year
      IYEAR=IYEAR+IDT
!
      case ('MO')
! unit is month
      MONTH=MONTH+IDT
      do while (MONTH<=0.or.MONTH>12)
         if (MONTH<=0) then
            MONTH=MONTH+12
            IYEAR=IYEAR-1
         else
            MONTH=MONTH-12
            IYEAR=IYEAR+1
         endif
      enddo
!
      case ('DY')
! unit is day
      IDAY=IDAY+IDT
      IDAYM=MON(MONTH)
      do while (IDAY<=0.or.IDAY>IDAYM)
        IF(IDAY<=0) THEN
           MONTH=MONTH-1
           IF(MONTH.LE.0) THEN
              MONTH=12
              IYEAR=IYEAR-1
              MON(2) = MON2(IYEAR)
           ENDIF
           IDAYM=MON(MONTH)
           IDAY=IDAY+IDAYM
        ELSE
           MONTH=MONTH+1
           IF(MONTH.GT.12) THEN
              MONTH=1
              IYEAR=IYEAR+1
              MON(2) = MON2(IYEAR)
           ENDIF
           IDAY=IDAY-IDAYM
           IDAYM=MON(MONTH)
        ENDIF
      enddo
!
      case default
! unit is hour
      IHOUR=IHOUR+IDT
      do while (IHOUR<0 .or. IHOUR>=24)
         IF(IHOUR.LT.0)THEN
            IHOUR=IHOUR+24
            IDAY=IDAY-1
            IF(IDAY.LE.0)THEN
               MONTH=MONTH-1
               IF(MONTH.LE.0)THEN
                  MONTH=12+MONTH
                  IYEAR=IYEAR-1
                  MON(2) = MON2(IYEAR)
               ENDIF
              IDAY=MON(MONTH)
            ENDIF
         ELSE IF(IHOUR.GE.24)THEN
            IHOUR=IHOUR-24
            IDAY=IDAY+1
            IF(IDAY.GT.MON(MONTH))THEN
               MONTH=MONTH+1
               IDAY=1
               IF(MONTH.GT.12)THEN
                  MONTH=MONTH-12
                  IYEAR=IYEAR+1
                  MON(2) = MON2(IYEAR)
               ENDIF
            ENDIF
         ENDIF
      enddo
!
   end select
!
   ITIM0=ITCOC(IYEAR,MONTH,IDAY,IHOUR)
!
   END SUBROUTINE CATIMC
!=========================================================================================

   SUBROUTINE IDTIME(ITIM,ITIM0,IDT)
!**  ---------- SUBROUTINE IDTIME(ITIM,ITIME0,IDT)
!  PURPOSE:
!    CALCULATING THE DATE DIFFERENCE.
!  INPUT:
!    ITIME:  THE INITIAL DATE
!    ITIM0:  THE CHANGED DATE
!  RETURN:
!    IDT:  THE DATE DIFFERENCE.
!  COMMANDS:
!    THE FORMAT OF ITIME & ITIM0: YYMMDDHH(I8),
!    AND THE RETURN TIME:
!      IDT = ITIM0 - ITIME
!      THE UNIT OF IDT:  HR.
!  BY WWW,1997-06-02.
!  THIS SUBROUTINE IS MODIFIED FOR THE CENTRAY-WORM PROBLEM.
!  1999.3.19,WZZ.
!  *******************************************************************
!
   integer, intent(in) :: ITIM, ITIM0
   integer, intent(out) :: IDT
!
! local variables
   integer ISIGN, IY, IY0, IYMD0, ID00
   integer ITIME, ITIM1, IYMD1
!
   IF(ITIM0.GT.ITIM) THEN
      ISIGN=1
   ELSE IF(ITIM0.LT.ITIM) THEN
      ISIGN=-1
   ENDIF
   IY=ITIM/1000000
   IY0=ITIM0/1000000
   IF(ABS(IY-IY0).GT.70) ISIGN=-1*ISIGN
!
   IDT=0
   IF(ITIM0.EQ.ITIM) RETURN
   IYMD0=ITIM0/100
   ID00=24*ISIGN
   ITIM1=ITIM
   GOTO 40
!
  35  IDT=IDT+ID00
   ITIME=ITIM1
   call CATIMC(ITIME,ITIM1,ID00, 'HR')
  40  IYMD1=ITIM1/100
   IF(IYMD1.NE.IYMD0) GOTO 35
   IDT=IDT+MOD(ITIM0,100)-MOD(ITIM1,100)
!   
   END SUBROUTINE IDTIME
!=========================================================================================

   SUBROUTINE DEITIME(ITIME,IY,IM,ID,IH)
!
!  DECODE ITIME TO IY,IM,ID,IH
!  ITIME: YYMMDDHH.
!  WZZ, 1998-06-09.
!
   integer, intent(in) :: ITIME
   integer, intent(out) :: IY, IM, ID, IH
!
   IY=ITIME/1000000
   IM=MOD(ITIME,1000000)/10000
   ID=MOD(ITIME,10000)/100
   IH=MOD(ITIME,100)
!
   END SUBROUTINE DEITIME
!=========================================================================================

   integer FUNCTION ITCOC(IY,IM,ID,IH)
!
!  FORM ITIME USING IY,IM,ID,IH.
!  The returned time is th full date, which means that
!    it do not need the (I8) format.
!  WZZ, 2000-03-02.
!
! Return value
!
   integer, intent(in) :: IY, IM, ID, IH
!
   ITCOC=IH+100*ID+10000*IM+1000000*IY
!
   END FUNCTION ITCOC
!=========================================================================================

   integer FUNCTION ITCO4(IY,IM,ID,IH)
!
!  FORM ITIME USING IY,IM,ID,IH.
!  The returned time is (I8) format.
!  WZZ, 1998-06-09.
!
! Return value
!
   integer, intent(in) :: IY, IM, ID, IH
!
   ITCO4=IH+100*ID+10000*IM+1000000*MOD(IY,100)
!
   END FUNCTION ITCO4
!=========================================================================================

   integer FUNCTION MON2(IYEAR)
!
!  Purpose:
!    To determine the days of Feb in year 'IYEAR'
!  WZZ, 2000-03-02.
!
! Return value
!
   integer, intent(in) :: IYEAR
!
! local variables
      integer IY1, IY2, IY3
!
   IY1=MOD(IYEAR,4)
   IY2=MOD(IYEAR,100)
   IY3=MOD(IYEAR,400)
   MON2=28
   IF(IY1.EQ.0.AND.IY2.NE.0.OR.IY3.EQ.0) MON2=29
!
   END FUNCTION MON2
!=========================================================================================

   subroutine dayas(daytime, atttext, timesiz)

   use calstring, only: lena

   integer , intent(in) :: timesiz
   real(r8), intent(inout) :: daytime(timesiz)
   character*(*), intent(inout) :: atttext
!  local variables
   integer n
   integer len
   integer i, i1, i2
   integer yr, mon, day
   integer iyear0, iyear

     if (atttext(1:4).eq.'hour') then
! change time from 'hours since' to 'days since':
        do n = 1, timesiz
           daytime(n) = daytime(n)/24
        enddo
        atttext(1:4) = 'days'
     endif
     if (atttext(1:4).eq.'days') then
! change time from 'days since' to 'day as':
! extract the start year information first
        len = len_trim(atttext)
        i1 = lena(atttext, index(atttext, 'since')+5)

! start year:
        do while (i1 <= len .and. (atttext(i1:i1)<'0'.or. &
                  atttext(i1:i1)>'9') )
          i1 = i1+1
        enddo
        i2 = i1
        do while (i2 <= len .and. (atttext(i2:i2)>='0'.and. &
                  atttext(i2:i2)<='9') )
          i2 = i2+1
        enddo
        yr = 0
        do i = i1, i2-1
           yr = yr*10+ichar(atttext(i:i))-ichar('0')
        enddo

! start month:
        i1 = i2
        do while (i1 <= len .and. (atttext(i1:i1)<'0'.or. &
                  atttext(i1:i1)>'9') )
          i1 = i1+1
        enddo
        i2 = i1
        do while (i2 <= len .and. (atttext(i2:i2)>='0'.and. &
                  atttext(i2:i2)<='9') )
          i2 = i2+1
        enddo
        mon = 0
        do i = i1, i2-1
           mon = mon*10+ichar(atttext(i:i))-ichar('0')
        enddo

! start day:
        i1 = i2
        do while (i1 <= len .and. (atttext(i1:i1)<'0'.or. &
                  atttext(i1:i1)>'9') )
          i1 = i1+1
        enddo
        i2 = i1
        do while (i2 <= len .and. (atttext(i2:i2)>='0'.and. &
                  atttext(i2:i2)<='9') )
          i2 = i2+1
        enddo
        day = 0
        do i = i1, i2-1
           day = day*10+ichar(atttext(i:i))-ichar('0')
        enddo

        iyear0 = yr*1000000 + mon*10000 + day*100
        do n = 1, timesiz
           i = int(daytime(n))
           if (i < 0) i = i-1
           call catimc(iyear0, iyear, i, 'DY')
           daytime(n) = daytime(n)-i+iyear/100
        enddo
     endif
   end subroutine dayas

end module caltime
