#include <misc.h>
#include <params.h>

subroutine dlpsbar(lpsbar, lpsbardl, lpsbardm, lpsbard2, lpsbard4)

!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid
   use commap,    only: clat

   implicit none
!---------------------------------------------------------------------
!
! Arguments
!
   real(r8), intent(inout) :: lpsbar(plond,plat)      ! Pa
   real(r8), intent(out) :: lpsbardl(plond,plat)  
   real(r8), intent(out) :: lpsbardm(plond,plat)  

   real(r8), intent(out) :: lpsbard2(plond,plat)  
   real(r8), intent(out) :: lpsbard4(plond,plat)  
!
! local variables
!
   integer  lat, i
   real(r8) lpsbardl2 (plond,plat)
   real(r8) lpsbardm2 (plond,plat)
   real(r8) lpsd2dl   (plond,plat)
   real(r8) lpsd2dm   (plond,plat)
   real(r8) lpsd2dl2  (plond,plat)
   real(r8) lpsd2dm2  (plond,plat)
   real(r8) coslat
   real(r8) coslat2
!-------------------------------------------------------
!
   call getlpsdl(lpsbar, lpsbardl)
   call getlpsdm(lpsbar, lpsbardm)
!
! compute lpsd2
!
   call getlpsdl(lpsbardl, lpsbardl2)
   call getlpsdm(lpsbardm, lpsbardm2)

   do lat=1, plat
      coslat = cos(clat(lat))
	  coslat2= coslat*coslat
      do i=1, plond
        lpsbard2(i, lat) = (lpsbardl2(i,lat) + lpsbardm2(i,lat))/coslat2
      enddo
   enddo
!
! compute lpsd4
!
   call getlpsdl(lpsbard2, lpsd2dl)
   call getlpsdm(lpsbard2, lpsd2dm)

   call getlpsdl(lpsd2dl, lpsd2dl2)
   call getlpsdm(lpsd2dm, lpsd2dm2)

   do lat=1, plat
      coslat = cos(clat(lat))
      coslat2= coslat*coslat
      do i=1, plond
        lpsbard4(i, lat) = (lpsd2dl2(i,lat) + lpsd2dm2(i,lat))/coslat2
      enddo
   enddo
! 
   return
end subroutine dlpsbar

!=====================================
!
   subroutine getlpsdl(lps, lpsdl)
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid
   use commap,       only: clat
   implicit none
!
!  input/output variables
!
   real(r8), intent(inout) :: lps(plond,plat)      ! surface pressure (Pa)
   real(r8), intent(out) :: lpsdl(plond,plat)      ! surface pressure (Pa)
!
! local variables
!
   integer lat, i, iopp
   real(r8) AE
   real(r8) pi
   real(r8) dx, dy2, dy01, dy02

   data AE/6.37122E+06/ 
   pi= 4.*atan(1.0)
!-------------------------
!
! compute lpsdl
! 
    do lat=1, plat
	   dx = 2.*pi/float(plon)*AE
       do i=1, plon
	     if(i.eq.1) then
             lpsdl(i,lat)= 2./3.*(lps(2,lat)-lps(plon,lat)) &
                            -1./12.*(lps(3,lat)-lps(plon-1,lat))
 	     elseif(i.eq.2) then
             lpsdl(i,lat)= 2./3.*(lps(3,lat)-lps(1,lat))  &
                            -1./12.*(lps(4,lat)-lps(plon,lat))

	     elseif( i.eq. (plon-1) ) then
             lpsdl(i,lat)= 2./3.*(lps(plon,lat)-lps(plon-2,lat))  &
                            -1./12.*(lps(1,lat)-lps(plon-3,lat))
  	     elseif( i.eq. plon ) then
             lpsdl(i,lat)= 2./3.*(lps(1,lat)-lps(plon-1,lat))  &
                            -1./12.*(lps(2,lat)-lps(plon-2,lat))
         else
             lpsdl(i,lat)= 2./3.*(lps(i+1,lat) - lps(i-1,lat))  &
                            -1./12.*(lps(i+2,lat)-lps(i-2,lat))
         endif	
         lpsdl(i,lat)= lpsdl(i,lat)/dx
	   enddo  

	   do i=plon+1, plond
         lpsdl(i,lat)= lpsdl(i-plon,lat)
  	   enddo
    enddo
!
  return
end subroutine getlpsdl


   subroutine getlpsdm(lps, lpsdm)
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid
   use commap,       only: clat
   implicit none
!
!  input/output variables
!
   real(r8) lps  (plond,plat)      ! surface pressure (Pa)
   real(r8) lpsdm(plond,plat)      ! surface pressure (Pa)
!
! local variables
!
   integer lat, i, iopp
   real(r8) AE
   real(r8) pi
   real(r8) dx, dy2, dy01, dy02, dydx

   data AE/6.37122E+06/ 
   pi= 4.*atan(1.0)
!-------------------------
!
! compute lpsdm(i,lat) = (1.-u2).d(lps)/du = cos(fai).d(lps)/AE*d(fai)
!
   do lat=1, plat
      do i=1, plon
	    iopp = i + plon/2
        if(iopp.gt.plon) iopp = plon-iopp
!
 	    if( lat.eq.1 .or. lat.eq.plat ) then
          dy01 = abs(pi-clat(1)) 
          if(dy01 .gt. 2*pi) dy01= 2*pi - dy01
          dy02 = abs(pi-clat(2))
          if(dy02 .gt. 2*pi) dy02= 2*pi - dy02
          dy2= dy01+dy02           
        else
		  dy2 = abs(clat(lat+1)-clat(lat-1))
        endif
!
        if(lat.eq.1) then
          lpsdm(i,1) = (lps(i,2)- lps(iopp,1))/dy2
 	    elseif( lat.eq.plat ) then
          lpsdm(i,plat) = (lps(iopp,plat)- lps(i,plat-1))/dy2
        else           
!--------------
!          lpsdm(i,lat) = (lps(i, lat+1)- lps(i,lat-1))/dy2
!!!        
          call integral(clat(lat-1),lps(i,lat-1),clat(lat),lps(i,lat), &    
                        clat(lat+1),lps(i,lat+1),clat(lat),dydx)
          lpsdm(i,lat) = dydx
!-------------
        endif
!
        lpsdm(i,lat) = lpsdm(i,lat)*cos(clat(lat))/AE
!
	 enddo

	 do i=plon+1, plond
        lpsdm(i,lat) = lpsdm(i-plon,lat)
	 enddo

   enddo
!
   return
end subroutine getlpsdm

   subroutine integral(x1,y1,x2,y2,x3,y3, x0, dydx0)
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
   real(r8) x1,y1,x2,y2,x3,y3, x0, dydx0
   real a1, a2, a0
!
!     y= a0+a1*x+a2*x*x
!
    a2= ((y3-y1)*(x2-x1) - (y2-y1)*(x3-x1) )
	a2 = a2/( (x3*x3-x1*x1)*(x2-x1) - (x2*x2-x1*x1)*(x3-x1) )
	a1 = ((y2-y1) - a2*(x2*x2-x1*x1))/(x2-x1) 
	a0 = y1 - a1*x1 - a2*x1*x1
!
!
    dydx0 = a1 + 2*a2*x0
    return
end subroutine integral
