#include <misc.h>
#include <params.h>

subroutine grmult(rcoslat ,d       ,qm1     ,tm1     ,um1     ,&
                  vm1     ,z       ,tm2     ,phis    ,dpsl    ,&
                  dpsm    ,omga    ,pdel    ,pbot    ,logpsm2 ,&
                  logpsm1 ,rpmid   ,rpdel   ,fu      ,fv      ,&
                  t2      ,ut      ,vt      ,drhs    ,pmid    ,&
                  etadot  ,etamid  ,engy    ,ddpn    ,vpdsn   ,&
                  dpslon  ,dpslat  ,nlon    ,&
                  lat     ,pmidm2  ,lpsbar  ,coef    )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Non-linear dynamics calculations in grid point space
! 
! Method: 
! 
! Author: 
! Original version:  CCM1
! Standardized:      J. Rosinski, June 1992
! Reviewed:          B. Boville, D. Williamson, J. Hack, August 1992
! Reviewed:          B. Boville, D. Williamson, April 1996
!
!-----------------------------------------------------------------------
!
! $Id: grmult.F90,v 1.1.1.1 2015/06/03 01:45:33 jiewh Exp $
! $Author: jiewh $
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use dynconst, only: omega     !  wzz, 2008.7.7
   use pmgrid
   use pspect
   use commap
   use physconst, only: rair, cappa, cpvir, zvir
   use dycore,       only: get_resolution
   use time_manager, only: get_step_size
   implicit none

#include <comhyb.h>
!
! Input arguments
!
   real(r8), intent(in) :: rcoslat               ! 1./cosine(latitude)
   real(r8), intent(in) :: d(plond,plev)         ! divergence
   real(r8), intent(in) :: qm1(plond,plev)         ! specific humidity
   real(r8), intent(in) :: tm1(plond,plev)         ! temperature
   real(r8), intent(in) :: um1(plond,plev)         ! zonal wind * cos(lat)
   real(r8), intent(in) :: vm1(plond,plev)         ! meridional wind * cos(lat)
   real(r8), intent(in) :: z(plond,plev)         ! vorticity
   real(r8), intent(in) :: phis(plond)           ! surface geopotential
   real(r8), intent(in) :: dpsl(plond)           ! longitudinal component of grad ln(ps)
   real(r8), intent(in) :: dpsm(plond)           ! latitudinal component of grad ln(ps)
   real(r8), intent(in) :: omga(plond,plev)      ! vertical pressure velocity
   real(r8), intent(in) :: pdel(plond,plev)      ! layer thicknesses (pressure)
   real(r8), intent(in) :: pbot(plond)           ! bottom interface pressure
   real(r8), intent(in) :: logpsm2(plond)        ! log(psm2)
   real(r8), intent(in) :: logpsm1(plond)          ! log(ps)
   real(r8), intent(in) :: rpmid(plond,plev)     ! 1./pmid
   real(r8), intent(in) :: rpdel(plond,plev)     ! 1./pdel
   real(r8), intent(in) :: tm2(plond,plev)       ! temperature at previous time step
   integer,  intent(in) :: nlon
!
! Input/Output arguments
!
   real(r8), intent(inout) :: fu(plond,plev)        ! nonlinear term - u momentum eqn
   real(r8), intent(inout) :: fv(plond,plev)        ! nonlinear term - v momentum eqn
   real(r8), intent(inout) :: t2(plond,plev)        ! nonlinear term - temperature
   real(r8), intent(inout) :: ut(plond,plev)        ! (u*TM1) - heat flux - zonal 
   real(r8), intent(inout) :: vt(plond,plev)        ! (u*TM1) - heat flux - meridional
   real(r8), intent(inout) :: drhs(plond,plev)      ! RHS of divergence eqn (del^2 term)
   real(r8), intent(inout) :: pmid(plond,plev)      ! pressure at full levels
   real(r8), intent(inout) :: etadot(plon,plevp)    ! vertical velocity in eta coordinates
   real(r8), intent(in)    :: etamid(plev)          ! midpoint values of eta (a+b)
   real(r8), intent(inout) :: engy(plond,plev)      ! kinetic energy
!
! wtw 2002.05
!
   integer, intent(in) :: lat             ! latitude index

   real(r8), intent(inout) :: pmidm2(plond,plev)  ! pressure at model levels (time n)
   real(r8), intent(in) :: lpsbar  (plond)     ! Pa
   real(r8), intent(in) :: coef
!
! Output arguments
!
   real(r8), intent(out) :: ddpn(plond)           ! complete sum of d*delta p
   real(r8), intent(out) :: vpdsn(plond)          ! complete sum V dot grad(ln(ps)) delta b
   real(r8), intent(out) :: dpslat(plond,plev)    ! ln(ps) component of lon press gradient
   real(r8), intent(out) :: dpslon(plond,plev)    ! ln(ps) component of lat press gradient

!
!---------------------------Local workspace-----------------------------
!
! Wu Tongwen May, 2004
!
   real(r8) pi, AE, c0, v2(plev), c2(plev), pmidref(plev)
   real(r8) dtime

!----------------------
   real(r8) tbarpsbar(plond)
   real(r8) logpsbar(plond)
   real(r8) dphis(plond)
   real(r8) dphis1(plond)
   real(r8) dphis2(plond)
   real(r8) psm1(plond)
   real(r8) tref(plond,plev)
   real(r8) trefm2(plond,plev)
   real(r8) trefdp(plond,plev)
   real(r8) fcor

   real(r8) qref(plond,plev) 
   real(r8) qrefdp(plond,plev) 
   real(r8) treffun, trefdpfun, phireffun, qreffun, qrefdpfun
   external treffun, trefdpfun, phireffun, qreffun, qrefdpfun
!
   real(r8) tv(plond,plev)        ! virtual temperature
   real(r8) ddpk(plond)           ! partial sum of d*delta p
   real(r8) vkdp                  ! V dot grad(ln(ps))
   real(r8) vpdsk(plond)          ! partial sum  V dot grad(ln(ps)) delta b
   real(r8) tk0(plond)            ! tm1 at phony level 0
   real(r8) uk0(plond)            ! u at phony level 0
   real(r8) vk0(plond)            ! v at phone level 0
   real(r8) rtv(plond,plev)       ! rair*tv
   real(r8) pterm(plond,plev)     ! intermediate term for hydrostatic eqn
   real(r8) tterm(plond,plev)     ! intermediate term for hydrostatic eqn
   real(r8) tmp                   ! temporary workspace
   real(r8) tmpk                  ! temporary workspace
   real(r8) tmpkp1                ! temporary workspace
   real(r8) edotdpde(plond,plevp) ! etadot*dp/deta
   real(r8) udel(plond,0:plev-1)  ! vertical u difference
   real(r8) vdel(plond,0:plev-1)  ! vertical v difference
   real(r8) tdel(plond,0:plev-1)  ! vertical TM1 difference

   integer i,k,kk             ! longitude, level indices
!
! Initialize arrays which represent vertical sums (ddpk, ddpn, vpdsk,
! vpdsn).  Set upper boundary condition arrays (k=0: tk0, uk0, vk0).
!
!-----------------------
   dtime = get_step_size()
   pi    = 4.*atan(1.0)
   AE    = 6.371e+6
   if( get_resolution() == 'T266' ) then
       c0 = 8.e-3    ! test-t266
   elseif ( get_resolution() == 'T106' ) then
       c0 = 0.0e-3
   else
       c0 = 0.0 
   endif

!----------------------

      ddpk(:)  = 0.0
      ddpn(:)  = 0.0
      vpdsk(:) = 0.0
      vpdsn(:) = 0.0
      tk0(:)  = 0.0
      uk0(:)  = 0.0
      vk0(:)  = 0.0
!
! Wu Tongwen May, 2004
!
#ifdef WU_REFATM
!$OMP PARALLEL DO PRIVATE (I)
   do i=1,nlon
      psm1(i)  = exp(logpsm1(i))                       ! wtw, 2004.05 (in order to compute dphis)
      dphis(i) = phis(i) -  phireffun(psm1(i))         ! wtw, 2004.05
   enddo

   if (lat > plat/2) then          ! northern hemisphere
      fcor= 2.*omega*sqrt(1.-(1/(rcoslat*rcoslat)))
   else                            ! southern hemisphere
      fcor= -2.*omega*sqrt(1.-(1/(rcoslat*rcoslat)))
   endif
!
!$OMP PARALLEL DO PRIVATE (K, I)
   do k=1,plev
      do i=1,nlon
         pmid(i,k)   = 1./rpmid(i,k)
         tref(i,k)   = treffun(pmid(i,k))
         trefdp(i,k) = trefdpfun(pmid(i,k)) 
	 trefm2(i,k) = treffun(pmidm2(i,k))
!
         qref(i,k)   = qreffun(pmid(i,k))
	 qrefdp(i,k) = qrefdpfun(pmid(i,k))
      enddo
   enddo
#endif
!
! Virtual temperature
!
   call virtem(nlon,plond,plev,tm1,qm1,zvir,tv)
!
! sum(plev)(d(k)*dp(k))
!
!$OMP PARALLEL DO PRIVATE (K, I)
   do k=1,plev
      do i=1,nlon
#ifdef WU_REFATM
         rtv(i,k) = rair*(tv(i,k)-tref(i,k))
#else
         rtv(i,k) = rair*tv(i,k)
#endif
      end do
   end do
!$OMP PARALLEL DO PRIVATE (I, K, VKDP)
   do i=1,nlon
      do k=1,plev
         ddpn(i) = ddpn(i) + d(i,k)*pdel(i,k)
      end do
!
! sum(plev)(v(k)*grad(lnps)*db(k))
!
      do k=nprlev,plev
         vkdp = rcoslat*(um1(i,k)*dpsl(i) + vm1(i,k)*dpsm(i))*pbot(i)
         vpdsn(i) = vpdsn(i) + vkdp*hybd(k)
      end do
!
! Compute etadot (dp/deta) (k+1/2).  Note: sum(k)(d(j)*dp(j)) required in
! pressure region. sum(k)(d(j)*dp(j)) and sum(k)(v(j)*grad(ps)*db(j)) 
! required in hybrid region
!
      edotdpde(i,1) = 0.
      do k=1,nprlev-1
         ddpk(i) = ddpk(i) + d(i,k)*pdel(i,k)
         edotdpde(i,k+1) = -ddpk(i)
      end do
      do k=nprlev,plev-1
         ddpk(i) = ddpk(i) + d(i,k)*pdel(i,k)
         vkdp = rcoslat*(um1(i,k)*dpsl(i) + vm1(i,k)*dpsm(i))*pbot(i)
         vpdsk(i) = vpdsk(i) + vkdp*hybd(k)
         edotdpde(i,k+1) = -ddpk(i) - vpdsk(i) + hybi(k+1)*(ddpn(i)+vpdsn(i))
      end do
      edotdpde(i,plevp) = 0.
   end do
!
! Nonlinear advection terms.  u*tm1, v*tm1, kinetic energy first
!
!$OMP PARALLEL DO PRIVATE (K, I)
   do k=1,plev
      do i=1,nlon
#ifdef WU_REFATM
         ut(i,k) = um1(i,k)*(tm1(i,k)-tref(i,k))
         vt(i,k) = vm1(i,k)*(tm1(i,k)-tref(i,k))
#else
         ut(i,k) = um1(i,k)*tm1(i,k)
         vt(i,k) = vm1(i,k)*tm1(i,k)
#endif
         engy(i,k) = 0.5*(um1(i,k)*um1(i,k) + vm1(i,k)*vm1(i,k))
      end do
   end do
!
! Compute workspace arrays for delta-u, delta-v, delta-tm1 (k)
!
!$OMP PARALLEL DO PRIVATE (K, I)
   do k=0,plev-1
     if(k==0) then
      do i=1,nlon
      udel(i,0) = um1(i,1) - uk0(i)
      vdel(i,0) = vm1(i,1) - vk0(i)

#ifdef WU_REFATM
      tdel(i,0) = (tm1(i,1)-tref(i,1)) - tk0(i)
#else
      tdel(i,0) = tm1(i,1) - tk0(i)
#endif
      end do
     else
      do i=1,nlon
         udel(i,k) = um1(i,k+1) - um1(i,k)
         vdel(i,k) = vm1(i,k+1) - vm1(i,k)
#ifdef WU_REFATM
         tdel(i,k) = (tm1(i,k+1)-tref(i,k+1)) - (tm1(i,k)-tref(i,k))
#else
         tdel(i,k) = tm1(i,k+1) - tm1(i,k)
#endif
      end do
     endif
   end do
!
! Horizontal advection: u*z, v*z, energy conversion term (omega/p),
! vertical advection for interface above.  Pure pressure region first.
!
!$OMP PARALLEL DO PRIVATE (K, I, TMPK, TMPKP1, TMP)
   do k=1,plev
     if(k<nprlev) then
      do i=1,nlon
         dpslat(i,k) = 0.
         dpslon(i,k) = 0.
         tmpk   = 0.5*rpdel(i,k)*edotdpde(i,k  )
         tmpkp1 = 0.5*rpdel(i,k)*edotdpde(i,k+1)
         fu(i,k) = fu(i,k) + vm1(i,k)*z(i,k) - udel(i,k-1)*tmpk - udel(i,k  )*tmpkp1
         fv(i,k) = fv(i,k) - um1(i,k)*z(i,k) - vdel(i,k-1)*tmpk - vdel(i,k  )*tmpkp1

#ifdef WU_REFATM
#ifdef HADVTEST
         t2(i,k) = t2(i,k) + d(i,k)*(tm1(i,k)-tref(i,k))
#else
         t2(i,k) = t2(i,k) + d(i,k)*(tm1(i,k)-tref(i,k)) - tdel(i,k-1)*tmpk + &
            cappa*tv(i,k)/(1. + cpvir*qm1(i,k))* &
            omga(i,k)*rpmid(i,k) - tdel(i,k)*tmpkp1 &
			-trefdp(i,k)*omga(i,k)
#endif
#else
#ifdef HADVTEST
!
!jr Modify so TM1 only has horizontal advection
!
         t2(i,k) = t2(i,k) + d(i,k)*tm1(i,k)
#else
         t2(i,k) = t2(i,k) + d(i,k)*tm1(i,k) - tdel(i,k-1)*tmpk + &
            cappa*tv(i,k)/(1. + cpvir*qm1(i,k))* &
            omga(i,k)*rpmid(i,k) - tdel(i,k)*tmpkp1
#endif
#endif
      end do
!
! Hybrid region above bottom level: Computations are the same as in pure
! pressure region, except that pressure gradient terms are added to 
! momentum tendencies.
!
     else if(k<plev) then
      do i=1,nlon
         tmpk   = 0.5*rpdel(i,k)*edotdpde(i,k  )
         tmpkp1 = 0.5*rpdel(i,k)*edotdpde(i,k+1)
         tmp = rtv(i,k)*hybm(k)*rpmid(i,k)*pbot(i)
         dpslon(i,k) = rcoslat*tmp*dpsl(i)
         dpslat(i,k) = rcoslat*tmp*dpsm(i)
         fu(i,k) = fu(i,k) + vm1(i,k)*z(i,k) - udel(i,k-1)*tmpk - &
            udel(i,k  )*tmpkp1 - dpslon(i,k)
         fv(i,k) = fv(i,k) - um1(i,k)*z(i,k) - vdel(i,k-1)*tmpk - &
            vdel(i,k  )*tmpkp1 - dpslat(i,k)
#ifdef HADVTEST
!
!jr Modify so TM1 only has horizontal advection
!
#ifdef WU_REFATM
         t2(i,k) = t2(i,k) + d(i,k)*(tm1(i,k)-tref(i,k))
#else
         t2(i,k) = t2(i,k) + d(i,k)*tm1(i,k)
#endif
#else
#ifdef WU_REFATM
         t2(i,k) = t2(i,k) + d(i,k)*(tm1(i,k)-tref(i,k)) - tdel(i,k-1)*tmpk + &
            cappa*tv(i,k)/(1. + cpvir*qm1(i,k))* &
            omga(i,k)*rpmid(i,k) - tdel(i,k)*tmpkp1  &
			- trefdp(i,k)*omga(i,k)
#else
         t2(i,k) = t2(i,k) + d(i,k)*tm1(i,k) - tdel(i,k-1)*tmpk + &
            cappa*tv(i,k)/(1. + cpvir*qm1(i,k))* &
            omga(i,k)*rpmid(i,k) - tdel(i,k)*tmpkp1
#endif
#endif
      end do
     else
!
! Bottom level
!
   do i=1,nlon
      tmpk = 0.5*rpdel(i,plev)*edotdpde(i,plev  )
      tmp  = rtv(i,plev)*hybm(plev)*rpmid(i,plev)*pbot(i)
      dpslon(i,plev) = rcoslat*tmp*dpsl(i)
      dpslat(i,plev) = rcoslat*tmp*dpsm(i)
      fu(i,plev) = fu(i,plev) + vm1(i,plev)*z(i,plev) - &
         udel(i,plev-1)*tmpk - dpslon(i,plev)
      fv(i,plev) = fv(i,plev) - um1(i,plev)*z(i,plev) - &
         vdel(i,plev-1)*tmpk - dpslat(i,plev)
#ifdef HADVTEST
!
!jr Modify so TM1 only has horizontal advection
!
#ifdef WU_REFATM
      t2(i,plev) = t2(i,plev) + d(i,plev)*(tm1(i,plev)-tref(i,plev))
#else
      t2(i,plev) = t2(i,plev) + d(i,plev)*tm1(i,plev)
#endif
#else
#ifdef WU_REFATM
      t2(i,plev) = t2(i,plev) + d(i,plev)*(tm1(i,plev)-tref(i,plev)) - &
         tdel(i,plev-1)*tmpk + &
         cappa*tv(i,plev)/(1. + cpvir*qm1(i,plev))* &
         omga(i,plev)*rpmid(i,plev)  &
		 - trefdp(i,plev)*omga(i,plev)
#else
      t2(i,plev) = t2(i,plev) + d(i,plev)*tm1(i,plev) - &
         tdel(i,plev-1)*tmpk + &
         cappa*tv(i,plev)/(1. + cpvir*qm1(i,plev))* &
         omga(i,plev)*rpmid(i,plev)
#endif
#endif
   end do
     end if
   end do
!
! Convert eta-dot(dp/deta) to eta-dot (top and bottom = 0.)
!
      etadot(:,1) = 0.
      etadot(:,plevp) = 0.
!$OMP PARALLEL DO PRIVATE (K, TMP, I)
   do k=2,plev
      tmp = etamid(k) - etamid(k-1)
      do i=1,nlon
#ifdef HADVTEST
!
!jr Set etadot to zero for horizontal advection test
!
         etadot(i,k) = 0.
#else
         etadot(i,k) = edotdpde(i,k)*tmp/(pmid(i,k) - pmid(i,k-1))
#endif
      end do
   end do
!
!-----------------------------------------------------------------------
!
! Divergence and hydrostatic equations
! Store some temporary terms
!
!$OMP PARALLEL DO PRIVATE (K, I)
   do k=1,plev
      do i=1,nlon
#ifdef WU_REFATM
         tterm(i,k) = 0.5*(tm2(i,k)-trefm2(i,k)) - (tm1(i,k)-tref(i,k))
#else
         tterm(i,k) = 0.5*tm2(i,k) - tm1(i,k)
#endif
         pterm(i,k) = rtv(i,k)*rpmid(i,k)*pdel(i,k)
      end do
   end do
!
! Del squared part of RHS of divergence equation.
! Kinetic energy and diagonal term of hydrostatic equation.
! Total temperature as opposed to  perturbation temperature is acceptable
! since del-square operator will operate on this term.
!
!$OMP PARALLEL DO PRIVATE (K, I)
   do k=1,plev-1
      do i=1,nlon
#ifdef WU_REFATM
         drhs(i,k) = dphis(i) + engy(i,k) + rtv(i,k)*0.5* &
            rpmid(i,k)*pdel(i,k) + href(k,k)*tterm(i,k) + &
            bps(k)*(0.5* logpsm2(i) - logpsm1(i) )
#ifdef WU_PSATM
         drhs(i,k) = dphis(i) + engy(i,k) + rtv(i,k)*0.5* &
            rpmid(i,k)*pdel(i,k) + href(k,k)*tterm(i,k) + &
            bps(k)*(0.5* (logpsm2(i)-coef*lpsbar(i)) - (logpsm1(i)-coef*lpsbar(i)) )
#endif

!---------------------
! 2012.6 wutw
! Include the divergence damping
!
         pmidref(k) = hyam(k)*ps0 + hybm(k)*1.0e+5

        if( plev == 26 ) then
             c2(k) = c0*max(1.0,8.*(1.0+tanh(log(pmid(i,1)/pmidref(k)))))
         else
        !    c2(k) = c0*max(1.0,8.*(1.0+tanh(log(pmid(i,1)/pmidref(k)))))  
             c2(k) = c0         
         endif

         v2(k) = c2(k)*(AE*(2.*pi/float(plon))*AE*w(lat)/2.0*pi)/dtime

         drhs(i,k) = drhs(i,k) - v2(k)*d(i,k)
!--
#else
         drhs(i,k) = phis(i) + engy(i,k) + rtv(i,k)*0.5* &
            rpmid(i,k)*pdel(i,k) + href(k,k)*tterm(i,k) + &
            bps(k)*(0.5*logpsm2(i) - logpsm1(i))
#endif
      end do
   end do
   do i=1,nlon
#ifdef WU_REFATM
      drhs(i,plev) = dphis(i) + engy(i,plev) + rtv(i,plev)*0.5* &
         rpmid(i,plev)*pdel(i,plev) + &
         href(plev,plev)*tterm(i,plev) + &
         bps(plev)*(0.5*logpsm2(i) - logpsm1(i) )
#ifdef WU_PSATM
      drhs(i,plev) = dphis(i) + engy(i,plev) + rtv(i,plev)*0.5* &
         rpmid(i,plev)*pdel(i,plev) + &
         href(plev,plev)*tterm(i,plev) + &
         bps(plev)*(0.5*(logpsm2(i)-coef*lpsbar(i)) - (logpsm1(i)-coef*lpsbar(i)) )
#endif
#else
      drhs(i,plev) = phis(i) + engy(i,plev) + rtv(i,plev)*0.5* &
         rpmid(i,plev)*pdel(i,plev) + &
         href(plev,plev)*tterm(i,plev) + &
         bps(plev)*(0.5*logpsm2(i) - logpsm1(i))
#endif
   end do
!
! Bottom level term of hydrostatic equation
!
!$OMP PARALLEL DO PRIVATE (K, I)
   do k=1,plev-1
      do i=1,nlon
         drhs(i,k) = drhs(i,k) + rtv(i,plev)* &
            rpmid(i,plev)*pdel(i,plev) + &
            href(plev,k)*tterm(i,plev)
      end do
   end do
!
! Interior terms of hydrostatic equation
!
!$OMP PARALLEL DO PRIVATE (K, KK, I)
   do k=1,plev-2
      do kk=k+1,plev-1
         do i=1,nlon
            drhs(i,k) = drhs(i,k) + pterm(i,kk) + href(kk,k)*tterm(i,kk)
         end do
      end do
   end do
!    
   return
end subroutine grmult

