#include <misc.h>
#include <params.h>

real function treffun(pre)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Define the temperature at "pre" pressure level in the reference atmosphere
! 
! Method: 
! 
! Author: Wu Tongwen
! 
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  implicit none

  real(r8), parameter :: a_ref = 0.09923
  real(r8), parameter :: b_ref = 247.7874
  real(r8), parameter :: c_ref = -1.0385

!-----------------------------------------------------------------------
  real(r8), intent(in)  :: pre          ! pressure (pascals)

!---------------------------Local workspace-----------------------------
  integer  i,k             ! Longitude, level indices
  real(r8) premb

!-----------------------------------------------------------------------

  premb = pre*0.01 
  treffun = premb*(a_ref + b_ref*exp(c_ref*log(premb)))   ! K
!
  return
end function treffun


real function trefdpfun(pre)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Define the temperature at "pre" pressure level in the reference atmosphere
! 
! Method: 
! 
! Author: Wu Tongwen
! 
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  implicit none

  real(r8), parameter :: a_ref = 0.09923
  real(r8), parameter :: b_ref = 247.7874
  real(r8), parameter :: c_ref = -1.0385

!-----------------------------------------------------------------------
  real(r8), intent(in)  :: pre          ! pressure (pascals)

!---------------------------Local workspace-----------------------------
  integer  i,k             ! Longitude, level indices
  real(r8) premb

!-----------------------------------------------------------------------

  premb = pre*0.01 
  trefdpfun = a_ref + (b_ref + b_ref*c_ref)*exp(c_ref*log(premb))   ! K/hPa
  trefdpfun = trefdpfun*0.01                ! K/Pa
!
  return
end function trefdpfun


real function phireffun(pre)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Define the temperature at "pre" pressure level in the reference atmosphere
! 
! Method: 
! 
! Author: Wu Tongwen
! 
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  implicit none

  real(r8), parameter :: a_ref = 0.09923
  real(r8), parameter :: b_ref = 247.7874
  real(r8), parameter :: c_ref = -1.0385
  real(r8), parameter :: p0_ref = 1013.25     ! hPa
  real(r8), parameter :: rair   = 287.04      ! J.Kg-1.K-1
!-----------------------------------------------------------------------
  real(r8), intent(in)  :: pre          ! pressure (pascals)

!---------------------------Local workspace-----------------------------
  integer  i,k             ! Longitude, level indices
  real(r8) premb           

!-----------------------------------------------------------------------

  premb = pre*0.01                                                ! hPa
  phireffun = a_ref*(premb-p0_ref) +                              &
              b_ref/(1.0+c_ref)*( exp((1.0+c_ref)*log(premb)) -   &
			                      exp((1.0+c_ref)*log(p0_ref)) )   
  phireffun = -rair*phireffun                                     ! m2/s2
!
  return
end function phireffun


real function qreffun(pre)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Define the temperature at "pre" pressure level in the reference atmosphere
! 
! Method: 
! 
! Author: Wu Tongwen
! 
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  implicit none

  real(r8), parameter :: a_ref = 2.57282E-04
  real(r8), parameter :: b_ref = 2.68E+04                 ! Pa

!-----------------------------------------------------------------------
  real(r8), intent(in)  :: pre          ! pressure (pascals)

!-----------------------------------------------------------------------
!   qreffun = a_ref*(exp(pre/b_ref) - 1.0)
   qreffun = 0.0
!
  return
end function qreffun


real function qrefdpfun(pre)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Define the temperature at "pre" pressure level in the reference atmosphere
! 
! Method: 
! 
! Author: Wu Tongwen
! 
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  implicit none

  real(r8), parameter :: a_ref = 2.57282E-04
  real(r8), parameter :: b_ref = 2.68E+04                 ! Pa

!-----------------------------------------------------------------------
  real(r8), intent(in)  :: pre          ! pressure (pascals)

!-----------------------------------------------------------------------

!  qrefdpfun = exp(pre/b_ref)*a_ref/b_ref            ! unit: 1/Pa
  qrefdpfun = 0.0
!
  return
end function qrefdpfun

