#include <misc.h>
#include <params.h>

#ifdef TDK_CU
subroutine tfilt_massfix (ztodt,         lat,    u3m1,   u3,     &
                          v3m1,   v3,    t3m1,   t3,     q3m1,   &
                          q3,     psm1,  ps,             alpha,  &
                          etamid, qfcst, vort,   div,    vortm2, &
                          divm2,         qminus, psm2,   um2,    &
                          vm2,    tm2,   qm2,    vortm1, divm1,  &
                          omga,   dpsl,  dpsm,   beta,   nlon,&
						  pstend ,utend, vtend, ttend, qtend,  & 
						  tatend,  qatend, t2phy   )
#else
#ifdef WU_CONV
subroutine tfilt_massfix (ztodt,         lat,    u3m1,   u3,     u3obs,  &
                          v3m1,   v3,    v3obs,  t3m1,   t3,     t3obs,  q3m1,   &   !jwh
                          q3,     q3obs, psm1,  ps,             alpha,  &
                          etamid, qfcst, vort,   vt3obs, div,    dv3obs, vortm2, &   !jwh
                          divm2,         qminus, psm2,   um2,    &
                          vm2,    tm2,   qm2,    vortm1, divm1,  &
                          omga,   dpsl,  dpsm,   beta,   nlon,&
                          t32,    q32,   pstend   )
#else
subroutine tfilt_massfix (ztodt,         lat,    u3m1,   u3,     &
                          v3m1,   v3,    t3m1,   t3,     q3m1,   &
                          q3,     psm1,  ps,             alpha,  &
                          etamid, qfcst, vort,   div,    vortm2, &
                          divm2,         qminus, psm2,   um2,    &
                          vm2,    tm2,   qm2,    vortm1, divm1,  &
                          omga,   dpsl,  dpsm,   beta,   nlon)
#endif
#endif
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Time filter (second half of filter for vorticity and divergence only)
! 
! Method: 
! 
! Author: 
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use history, only: outfld
   use pmgrid
   use pspect
   use commap
   use constituents, only: pcnst, pnats, qmin
   use constituents, only: cnst_get_type_byind
   use time_manager, only: get_nstep
#ifdef OBS-FORCE
   use time_manager, only: get_curr_date, get_step_size, force_start,force_stop, &
                           start_ymd, obs_ncep, obs_fnl, obs_ecmwf, obs_rain   !jwh
   use atm_force_t639,  only: vinterp          !jwh
#endif
   use physconst, only: cpair, gravit
#if ( defined SCAM)
   use scamMod
#endif
#if ( defined BFB_CAM_SCAM_IOP )
   use iop
   use constituents, only: ppcnst,cnst_get_ind,cnst_name
#endif
   use constituents, only: ppcnst     !zf 2009.07.09
   implicit none

#include <comctl.h>
#include <comlun.h>
#include <comqfl.h>
#include <comtfc.h>
!
! Input arguments
!
   real(r8), intent(in) :: ztodt                  ! two delta t (unless nstep<2)

   real(r8), intent(in) :: qfcst(plond,plev,pcnst)! slt moisture forecast
   real(r8), intent(inout) :: vort(plond,plev)    
   real(r8), intent(inout) :: div(plond,plev)     
   real(r8), intent(inout) :: vortm2(plond,plev)
   real(r8), intent(inout) :: divm2(plond,plev)
   real(r8), intent(in) :: qminus(plond,plev,pcnst)
   real(r8), intent(inout) :: psm2(plond)
   real(r8), intent(inout) :: um2(plond,plev)
   real(r8), intent(inout) :: vm2(plond,plev)
   real(r8), intent(inout) :: tm2(plond,plev)
   real(r8), intent(inout) :: qm2(plond,plev,pcnst+pnats)
   real(r8), intent(inout) :: omga(plond,plev)
   real(r8), intent(in) :: dpsl(plond)
   real(r8), intent(in) :: dpsm(plond)
   real(r8), intent(in) :: beta                   ! energy fixer coefficient
   real(r8), intent(in) :: alpha(pcnst)
   real(r8), intent(in) :: etamid(plev)           ! vertical coords at midpoints 
   real(r8), intent(inout) :: u3(plond,plev)
   real(r8), intent(inout) :: v3(plond,plev)      
   real(r8), intent(inout) :: t3(plond,plev)     

   integer, intent(in) :: lat
   integer, intent(in) :: nlon

! Input/Output arguments

   real(r8), intent(inout) :: q3(plond,plev,pcnst+pnats)
   real(r8), intent(inout) :: ps(plond)
   real(r8), intent(inout) :: vortm1(plond,plev)
   real(r8), intent(inout) :: psm1(plond)
   real(r8), intent(inout) :: u3m1(plond,plev)
   real(r8), intent(inout) :: v3m1(plond,plev)
   real(r8), intent(inout) :: t3m1(plond,plev)
   real(r8), intent(inout) :: divm1(plond,plev)
   real(r8), intent(inout) :: q3m1(plond,plev,pcnst+pnats)
#ifdef WU_CONV
   real(r8), intent(out) :: t32(plond,plev)
   real(r8), intent(out) :: q32(plond,plev)
   real(r8), intent(inout) :: t3obs(plond,plev)    !jwh
   real(r8), intent(inout) :: q3obs(plond,plev)    !jwh
   real(r8), intent(inout) :: u3obs(plond,plev)    !jwh
   real(r8), intent(inout) :: v3obs(plond,plev)    !jwh
   real(r8), intent(inout) :: vt3obs(plond,plev)   !jwh
   real(r8), intent(inout) :: dv3obs(plond,plev)   !jwh
!======================================================================================
   real(r8)  tobs_sigma(plev) !jwh
   real(r8)  qobs_sigma(plev) !jwh
   real(r8)  pmid1d(plev)     !jwh
!-------NCEP_17----------
   real(r8)  workt3(17)   !jwh NCEP
   real(r8)  workq3(8)    !jwh   real(r8)  levpa(17)    !jwh
   real(r8)  levpa(17),levpa8(8)
   data levpa/1000.0,2000.0,3000.0,5000.0,7000.0,10000.0,15000.0,20000.0,25000.0,&
              30000.0,40000.0,50000.0,60000.0,70000.0,85000.0,92500.0,100000.0/  !17 levs
  data levpa8/30000.0,40000.0,50000.0,60000.0,70000.0,85000.0,92500.0,100000.0/  !8 levs
!------T639_32-------
   real(r8) workt3_t639(32)
   real(r8) levpa_t639(32)
   data levpa_t639 /100, 200, 300, 400, 500, 700, 1000, 2000, 3000, &
          5000, 7000, 10000, 15000, 20000, 25000, 30000, 35000,&
          40000, 45000, 50000, 55000, 60000, 65000, 70000, 75000, &
          80000, 85000, 90000, 92500, 95000, 97500, 100000/
!------FNL_26--------
   real(r8) workt3_fnl(26)
   real(r8)  lev26(26)  !jwh 2013
   data lev26/1000, 2000, 3000, 5000, 7000, 10000, 15000, 20000, 25000, 30000, 35000, 40000, 45000, 50000, &
             55000, 60000, 65000, 70000, 75000, 80000., 85000., 90000., 92500., 95000., 97500., 100000./ !26 levs
!--------------------
   real(r8) coefk(plev)                              !jwh
   integer :: yr, mon, day        ! components of a date
   integer :: ncsec               ! current time of day [seconds]

   real(r8) ptmp0
   real(r8) ptmp1
   real(r8) ptmp2
   real(r8) ptmp3

   real(r8) dtime                 ! timestep size
   real(r8) coeft
   real(r8) delay_step, srtstep, endstep, mstep
   integer  delay_day, yrsrt, yrsrt_force, yrend_force
   real(r8) daysrt, daysrt_force, dayend_force
   integer nday(12)   ! Starting day number for each month
   save nday
   data nday/0,31,59,90,120,151,181,212,243,273,304,334/  !jwh
!-----------------------------------------------------------------   
!======================================================================================
!  Nudging coefficient (by wj 2016)
  REAL(r8)   nudg_dv,nudg_vt,nudg_u,nudg_v,nudg_t,nudg_q !for fcs
  REAL(r8), PARAMETER :: nfact=1.e-5  ! rescaling factor for nudging coefficients
  REAL(r8)   nudg_dvo,nudg_vto,nudg_uo,nudg_vo,nudg_to,nudg_qo !for obs
  LOGICAL :: lnudgimp = .TRUE.
  ! true  --> implicit nudging (default)
  ! false --> explicit nudging
!======================================================================================

#endif

#ifdef TDK_CU
   real(r8), intent(inout) :: tatend(plond,plev)  ! adv dT/dt
   real(r8), intent(in)    :: t2phy(plond,plev)   ! phy dT/dt
   real(r8), intent(inout) :: qatend(plond,plev,pcnst)
#endif
!
! Local workspace
!
   integer ifcnt                   ! Counter
   integer :: nstep                ! current timestep number

   real(r8) qfcst1(plond,plev,pcnst) ! Workspace for qfcst
   real(r8) engycorr(plond,plev)   ! energy equivalent to T correction
   real(r8) rpmid(plond,plev)      ! 1./pmid
   real(r8) pdel(plond,plev)       ! pdel(k)   = pint  (k+1)-pint  (k)
   real(r8) pint(plond,plevp)      ! pressure at model interfaces (n  )
   real(r8) pmid(plond,plev)       ! pressure at model levels (time n)
   real(r8) utend(plond,plev)      ! du/dt
   real(r8) vtend(plond,plev)      ! dv/dt
   real(r8) ttend(plond,plev)      ! dT/dt
#ifdef TDK_CU
   real(r8) qtend(plond,plev,pcnst+pnats)! dq/dt
#else
   real(r8) qtend(plond,plev,pcnst)! dq/dt
#endif
   real(r8) pstend(plond)          ! d(ps)/dt
   real(r8) pintm1(plond,plevp)    ! pressure at model interfaces (n-1)
   real(r8) pmidm1(plond,plev)     ! pressure at model levels (time n-1)
   real(r8) pdelm1(plond,plev)     ! pdelm1(k) = pintm1(k+1)-pintm1(k)
   real(r8) om2eps
   real(r8) corm
   real(r8) wm
   real(r8) absf
   real(r8) worst
   logical lfixlim               ! flag to turn on fixer limiter

   real(r8) ta(plond,plev,pcnst)   ! total advection of constituents
   real(r8) dqfx3(plond,plev,pcnst)! q tendency due to mass adjustment
   real(r8) coslat                 ! cosine(latitude)
   real(r8) rcoslat(plond)         ! 1./cosine(latitude)
!   real(r8) engt                   ! Thermal   energy integral
!   real(r8) engk                   ! Kinetic   energy integral
!   real(r8) engp                   ! Potential energy integral
   integer i, k, m,j,ixcldliq,ixcldice
#if ( defined BFB_CAM_SCAM_IOP )
   real(r8) :: t3forecast(plond,plev),delta_t3(plond,plev)
   real(r8) :: q3forecast(plond,plev,ppcnst),delta_q3(plond,plev,ppcnst)
   real(r8) divt3d(plond,plev)      ! 1./pmid
   real(r8) divq3d(plond,plev,ppcnst)      ! 1./pmid
   integer j,ixcldliq,ixcldice
#endif
   real(r8) divq3dout(plond,plev,ppcnst)      ! 1./pmid   zf 2009.07.06

!-----------------------------------------------------------------------
   qfcst1(1:nlon,:,:) = qfcst(i1:nlon+i1-1,:,:)

!zf 2009.07.06
   do i=1,nlon
     do k=1,plev
       divq3dout(i,k,1)= (qfcst1(i,k,1)-qminus(i,k,1))/ztodt
     end do
   end do
   call outfld ('QTENDA  ',divq3dout(:,:,1),plond,lat)
!zf 2009.07.06

   nstep = get_nstep()
#if ( defined BFB_CAM_SCAM_IOP )
!
! Calculate 3d dynamics term
!
   do k=1,plev
      do i=1,nlon
         divt3d(i,k)=(t3(i,k)-tm2(i,k))/ztodt -t2sav(i,k,lat)
         t3forecast(i,k)=tm2(i,k)+ztodt*t2sav(i,k,lat)+ztodt*divt3d(i,k)
      end do
   end do
   do i=1,nlon
      do m=1,ppcnst
         do k=1,plev
            divq3d(i,k,m)= (qfcst1(i,k,m)-qminus(i,k,m))/ztodt
            q3forecast(i,k,m)=qminus(i,k,m)+divq3d(i,k,m)*ztodt
         end do
      end do
   end do


   q3(:nlon,:,:)=q3forecast(:nlon,:,:)
   t3(:nlon,:)=t3forecast(:nlon,:)
   qfcst1(:nlon,:,:)=q3(:nlon,:,:)

!
! outflds for iop history tape - to get bit for bit with scam
! the n-1 values are put out.  After the fields are written out
! the current time level of info will be buffered for output next
! timestep
!
   call outfld('t',t3sav(1,1,lat)    ,plond   ,lat     )
   call outfld('q',q3sav(1,1,1,lat)  ,plond   ,lat     )
   call outfld('Ps',pssav(1,lat)    ,plond   ,lat     )
   call outfld('u',u3sav(1,1,lat)    ,plond   ,lat     )
   call outfld('v',v3sav(1,1,lat)    ,plond   ,lat     )
!
! read single values into plon arrays for output to history tape
! it would be nice if history tape supported 1 dimensional array variables
!
   fixmas_plond(:)=fixmassav(lat)
   beta_plond(:)=betasav(lat)
   clat_plond=clat(lat)

   call outfld('fixmas',fixmas_plond,plond   ,lat     )
   call outfld('beta',beta_plond  ,plond   ,lat     )
   call outfld('CLAT    ',clat_plond  ,plond   ,lat     )
   call outfld('divT3d',divt3dsav(1,1,lat)  ,plond   ,lat     )
   call outfld('divq3d',divq3dsav(1,1,1,lat)  ,plond   ,lat     )
   call cnst_get_ind('CLDLIQ', ixcldliq)
   call cnst_get_ind('CLDICE', ixcldice)

   if (nstep.eq.0) then
     call outfld('dcldliq',q3sav(1,1,ixcldliq,lat),plond   ,lat     )
     call outfld('dcldice',q3sav(1,1,ixcldice,lat),plond   ,lat     )
   else
     call outfld('dcldliq',divq3dsav(1,1,ixcldliq,lat),plond   ,lat     )
     call outfld('dcldice',divq3dsav(1,1,ixcldice,lat),plond   ,lat     )
   endif 
!
! buffer fields for output at next timestep
!
   divq3dsav(:nlon,:,:,lat)= divq3d(:nlon,:,:)
   divt3dsav(:nlon,:,lat)= divt3d(:nlon,:)
   t3sav(:nlon,:,lat) = t3(:nlon,:)
   u3sav(:nlon,:,lat) = u3(:nlon,:)
   v3sav(:nlon,:,lat) = v3(:nlon,:)
   q3sav(:nlon,:,:,lat) = q3(:nlon,:,:)
   pssav(:nlon,lat) = ps(:nlon)
   betasav(lat)=beta
   fixmassav(lat)=fixmas
#endif


   coslat = cos(clat(lat))
   do i=1,nlon
     rcoslat(i) = 1./coslat
   enddo
   lfixlim = .true.


!
! Set average dry mass to specified constant preserving horizontal
! gradients of ln(ps). Proportionality factor was calculated in STEPON
! for nstep=0 or SCAN2 otherwise from integrals calculated in INIDAT
! and SCAN2 respectively.
! Set p*.
!
   do i=1,nlon
      ps(i) = ps(i)*fixmas
   end do
!
! Set current time pressure arrays for model levels etc.
!
   call plevs0(nlon    ,plond   ,plev    ,ps      ,pint    ,pmid    ,pdel)
!
   rpmid(:nlon,:plev) = 1./pmid(:nlon,:plev)
!------------------------------------------
#ifdef TDK_CU   
   do m=1, pcnst
      do k=1, plev
	     do i=1, nlon
		    qatend(i,k,m) = (qfcst1(i,k,m) - qminus(i,k,m))/ztodt
		 enddo
	  enddo
   enddo
#endif
!-----------------------------------------------
!
! Add temperature correction for energy conservation
!
   do k=1,plev
      do i=1,nlon
         engycorr(i,k) = (cpair/gravit)*beta*pdel(i,k)/ztodt
         t3      (i,k) = t3(i,k) + beta
      end do
   end do
!
! Output Energy correction term
!
   call outfld('ENGYCORR',engycorr ,plond   ,lat     )
!
! Compute q tendency due to mass adjustment
! If LFIXLIM = .T., then:
! Check to see if fixer is exceeding a desired fractional limit of the
! constituent mixing ratio ("corm").  If so, then limit the fixer to
! that specified limit.
!
   do m=1,pcnst
      if (cnst_get_type_byind(m).eq.'dry' ) then
         corm    = 1.e36
      else
         corm    = 0.1
      end if

      do k=1,plev
         do i=1,nlon
#if ( defined SCAM )
            dqfx3(i,k,m) = dqfxcam(i,k,m)
#else
            dqfx3(i,k,m) = alpha(m)*etamid(k)*abs(qfcst1(i,k,m) - qminus(i,k,m))
#if ( defined BFB_CAM_SCAM_IOP )
            dqfx3sav(i,k,m) = dqfx3(i,k,m)
#endif
#endif
         end do
         if (lfixlim) then
            ifcnt = 0
            worst = 0.
            wm    = 0.
            do i = 1,nlon
               absf = abs(dqfx3(i,k,m))
               if (absf.gt.corm) then
                  ifcnt = ifcnt + 1
                  worst = max(absf,worst)
                  wm = wm + absf
                  dqfx3(i,k,m) = sign(corm,dqfx3(i,k,m))
               endif
            end do
            if (ifcnt.gt.0) then
               wm = wm/float(ifcnt)
#ifndef HADVTEST
! TBH:  Commented out as of CAM CRB meeting on 6/20/03
!               write (6,1000) m,corm,ifcnt,k,lat,wm,worst
#endif
            endif
         endif
         do i=1,nlon
            dqfx3(i,k,m) = qfcst1(i,k,m)*dqfx3(i,k,m)/ztodt
#ifdef HADVTEST
            q3(i,k,m) = qfcst1(i,k,m)
#else
            q3(i,k,m) = qfcst1(i,k,m) + ztodt*dqfx3(i,k,m)
#endif
            ta(i,k,m) = (q3(i,k,m) - qminus(i,k,m))/ztodt
         end do
      end do
   end do

#if ( !defined SCAM )
#if ( defined BFB_CAM_SCAM_IOP )
   do m=1,pcnst
      alpha_plond(:,m)= alphasav(m,lat)
      call outfld(alphanam(m),alpha_plond(1,m) ,plond   ,lat     )
      call outfld(dqfxnam(m),dqfx3savm1(1,1,m,lat) ,plond   ,lat     )
      alphasav(m,lat)= alpha(m)
      dqfx3savm1(:,:,m,lat)= dqfx3sav(:,:,m)
   end do
#endif
#endif
!
! Check for and correct invalid constituents
!
   call qneg3 ('TFILT_MASSFIX',lat   ,nlon    ,plond   ,plev    , &
               pcnst+pnats,qmin ,q3(1,1,1))
!
! Send slt tendencies to the history tape
!
!   do m=1,pcnst
!      call outfld(tottnam(m),ta(1,1,m),plond   ,lat     )
!   end do
#if ( !defined SCAM )
!
! Calculate vertical motion field
!
   call omcalc (rcoslat ,div     ,u3      ,v3      ,dpsl    ,  &
                dpsm    ,pmid    ,pdel    ,rpmid   ,pint(1,plevp), &
                omga    ,nlon    )

#endif

!   write(6,*)'tfilt: lat=',lat
!   write(6,*)'omga=',omga
!
! Time filter (second half of filter for vorticity and divergence only)
!
!   if(lat.eq.2) then
!      write(6,*)'tfilt: ps=',psm2(13),psm1(13),ps(13)
!      write(6,*)'tfilt: u=',um2(13,18),u3m1(13,18),u3(13,18)
!      write(6,*)'tfilt: t=',tm2(13,18),t3m1(13,18),t3(13,18)
!      write(6,*)'tfilt: water=',qm2(13,18,1),q3m1(13,18,1),q3(13,18,1)
!      write(6,*)'tfilt: cwat=',qm2(13,18,2),q3m1(13,18,2),q3(13,18,2)
!      write(6,*)'tfilt: vort=',vortm2(13,18),vortm1(13,18),vort(13,18)
!      write(6,*)'tfilt: div=',divm2(13,18),divm1(13,18),div(13,18)
!   end if

#ifdef WU_CONV
#ifdef OBS-FORCE
   do i=1,nlon
      if (obs_ncep) then          ! use NCEP 17 levels obs data
          do k=1, 17
             workt3(k) = t3obs(i,k)
          end do
          do k=1, 8
             workq3(k) = q3obs(i,k)
          end do
          do k=1, plev
             pmid1d(k) = pmid(i,k)
          end do
          call vinterp(workt3, levpa, 17, tobs_sigma , pmid1d, plev, 1)   ! 1 to log
          call vinterp(workq3, levpa8, 8, qobs_sigma , pmid1d, plev, 0)  ! 0 to line
          do k=1, plev
             t3obs(i,k) = tobs_sigma(k)
             q3obs(i,k) = qobs_sigma(k)
          end do
      else if (obs_fnl) then  ! jwh 2013
          do k=1, 26 
             workt3_fnl(k) = t3obs(i,k)
          end do
          do k=1, plev
             pmid1d(k) = pmid(i,k)
          end do
!         call vinterp(workt3_fnl, lev26, plev, tobs_sigma , pmid1d, plev, 1)
          call vinterp(workt3_fnl, lev26, 26, tobs_sigma , pmid1d, plev, 1)
          do k=1, plev
             t3obs(i,k) =  tobs_sigma(k)
          end do
!!!!!    q begin   !! wj 2016
          do k=1, 26
             workt3_fnl(k) = q3obs(i,k)
          end do
          do k=1, plev
             pmid1d(k) = pmid(i,k)
          end do
          call vinterp(workt3_fnl, lev26, 26, qobs_sigma , pmid1d, plev, 1)
          do k=1, plev
                q3obs(i,k) = qobs_sigma(k)
          end do
!!!!!!   q  end !!!!!!!! wj 2016
      else if (obs_ecmwf) then  ! jwh 
!           if (masterproc) write(*,*) "Interpolating in OBSFORCE Mode"
      else     ! use T639
          do k=1, 32 
             workt3_t639(k) = t3obs(i,k)
          end do
          do k=1, plev
             pmid1d(k) = pmid(i,k)
          end do
          call vinterp(workt3_t639, levpa_t639, 32, tobs_sigma , pmid1d, plev, 1)
          do k=1, plev
             t3obs(i,k) = tobs_sigma(k)
          end do
      end if
   end do

   call outfld('T3OBS   ',t3obs   ,plond   ,lat)                 !(jwh)
   call outfld('U3OBS   ',u3obs   ,plond   ,lat)                 !(jwh)
   call outfld('V3OBS   ',v3obs   ,plond   ,lat)                 !(jwh)
   call outfld('Q3OBS   ',q3obs   ,plond   ,lat)                 !(jwh)
!------------------------------------------------
  call get_curr_date(yr, mon, day, ncsec)
  dtime = get_step_size() 
  delay_day = 1 
  !delay_day = 10   ! for slow coordination  
  delay_step =  (86400/dtime)*delay_day

  call bnday(start_ymd,daysrt,yrsrt)

!wtw  call bnday(force_start,daysrt_force,yrsrt_force)
  call bnday_hour(force_start,daysrt_force,yrsrt_force)

!wtw  call bnday(force_stop,dayend_force,yrend_force)
  call bnday_hour(force_stop,dayend_force,yrend_force)
  
  if (yrend_force > yrsrt) then
    srtstep = (86400/dtime)*daysrt_force + (86400/dtime)*(365*(yrsrt_force - yrsrt) - daysrt) 
    endstep = (86400/dtime)*dayend_force + (86400/dtime)*(365*(yrend_force - yrsrt) - daysrt) 
  else
    srtstep = (86400/dtime)*(daysrt_force - daysrt) 
    endstep = (86400/dtime)*(dayend_force - daysrt) 
  end if
  if (nstep <= endstep .and. nstep >= srtstep .and. (yr >= yrsrt_force .and. yr <= yrend_force)) then 
 ! if (nstep <= endstep .and. nstep >= srtstep) then
 
!  if (masterproc) print*,"using obs data ",nstep,srtstep,endstep 

!  if (nstep .eq. endstep-1) then
!---------------------------
     do k=1,plev
       do i=1,nlon
          if (obs_ecmwf) then  
           !ptmp0 = 1.0*100    !1.0*100    ! Pa    ! 2019-2021 use
           !ptmp1 = 10.0*100   !10.0*100   ! Pa    ! 2019-2021 use
            ptmp0 = -1.0*100   !0.0*100    ! Pa    ! 2022 use
            ptmp1 = 0.0*100    !0.0*100    ! Pa    ! 2022 use
          else
            ptmp0 = 10.0*100   !10.0*100   ! Pa
            ptmp1 = 30.0*100   !30.0*100   ! Pa
          end if
          ptmp2 = 925.*100*(pmid(i,plev)/101300) !10.0*100   ! Pa
          ptmp3 = 1000.*100*(pmid(i,plev)/101300) !10.0*100   ! Pa

          if( pmid(i,k) < ptmp0) then
            coefk(k) = 0.0
          else if ( pmid(i,k) >= ptmp0 .and. pmid(i,k) < ptmp1 ) then
            coefk(k) = ( pmid(i,k) - ptmp0 )/ (ptmp1 - ptmp0)
          else if (pmid(i,k) >= ptmp1 .and. pmid(i,k) < ptmp2 ) then
            coefk(k) = 1.0
          else if (pmid(i,k) >= ptmp2 .and. pmid(i,k) < ptmp3 ) then
            coefk(k) = ( pmid(i,k) - ptmp3 )/ (ptmp2 - ptmp3)
          else
            coefk(k) = 0.0
          endif

!---------------------------------------------------------------------         
     if (obs_fnl) then  
!        if (nstep <= delay_step + srtstep) then          
!           coeft = 1.0*((nstep-srtstep)/delay_step)        
!        else
            coeft= 1.0 
!        end if
     else
         coeft= 1.0 
     end if

         t3(i,k)=t3(i,k)*(1.0-coefk(k)*coeft) + t3obs(i,k)*coefk(k)*coeft         
!         q3(i,k,1)=q3(i,k,1)*(1.0-coefk(k)*coeft) + q3obs(i,k)*coefk(k)*coeft     
         u3(i,k)=u3(i,k)*(1.0-coefk(k)*coeft) + u3obs(i,k)*coefk(k)*coeft        
         v3(i,k)=v3(i,k)*(1.0-coefk(k)*coeft) + v3obs(i,k)*coefk(k)*coeft          
         vort(i,k)=vort(i,k)*(1.0-coefk(k)*coeft) + vt3obs(i,k)*coefk(k)*coeft   
         div(i,k)=div(i,k)*(1.0-coefk(k)*coeft) + dv3obs(i,k)*coefk(k)*coeft     

!************************** wj 2016 modify ****************************** 

          nudg_dv = 10.*coefk(k)  ! 2.78 hours
          nudg_vt = 10.*coefk(k)  ! 2.78 hours
          nudg_u =  10.*coefk(k)  ! 2.78 hours
          nudg_v =  10.*coefk(k)  ! 2.78 hours
          nudg_t =  10.*coefk(k)  ! 2.78 hours
          nudg_q =  1*coefk(k)    ! 27.8 hours

          nudg_dvo = nudg_dv * nfact * ztodt
          nudg_vto = nudg_vt * nfact * ztodt
          nudg_uo = nudg_u * nfact * ztodt
          nudg_vo = nudg_v * nfact * ztodt
          nudg_to = nudg_t * nfact * ztodt
          nudg_qo = nudg_q * nfact * ztodt

          nudg_dv  = 1.-nudg_dvo
          nudg_vt  = 1.-nudg_vto
          nudg_u   = 1.-nudg_uo
          nudg_v   = 1.-nudg_vo
          nudg_t   = 1.-nudg_to
          nudg_q   = 1.-nudg_qo

          ! convert coefficients for implicit nudging
          IF (lnudgimp) THEN
             ! using implicit nudging reset weights
             nudg_dv  = 1./(1+nudg_dvo)
             nudg_vt  = 1./(1+nudg_vto)
             nudg_u   = 1./(1+nudg_uo)
             nudg_v   = 1./(1+nudg_vo)
             nudg_t   = 1./(1+nudg_to)
             nudg_q   = 1./(1+nudg_qo)
             ! reset weights for observations
             nudg_dvo = nudg_dvo * nudg_dv
             nudg_vto = nudg_vto * nudg_vt
             nudg_uo  = nudg_uo  * nudg_u
             nudg_vo  = nudg_vo  * nudg_v
             nudg_to  = nudg_to  * nudg_t
             nudg_qo  = nudg_qo  * nudg_q
          ENDIF
    !     write(*,*) q3(i,k,1),q3obs(i,k),k,pmid(i,k)
    !     t3(i,k)=t3(i,k)*nudg_t + t3obs(i,k)*nudg_to
          q3(i,k,1)=q3(i,k,1)*nudg_q + q3obs(i,k)*nudg_qo
    !     u3(i,k)=u3(i,k)*nudg_u + u3obs(i,k)*nudg_uo
    !     v3(i,k)=v3(i,k)*nudg_v + v3obs(i,k)*nudg_vo
    !     vort(i,k)=vort(i,k)*nudg_vt + vt3obs(i,k)*nudg_vto
    !     div(i,k)=div(i,k)*nudg_dv + dv3obs(i,k)*nudg_dvo
!************************** wj modify end ****************************** 
 
       end do
     end do

!-----------
  elseif (nstep > endstep) then
      do k=1,plev
        do i=1,nlon
           t3obs(i,k) = t3(i,k)
           q3obs(i,k) = q3(i,k,1)
           u3obs(i,k) = u3(i,k)
           v3obs(i,k) = v3(i,k)
           vt3obs(i,k) = vort(i,k)
           dv3obs(i,k) = div(i,k)
        enddo
      enddo
!--
  end if

   do k=1,plev
      do i=1,nlon
         t32(i,k)=t3(i,k)
         q32(i,k)=q3(i,k,1)
      enddo
   enddo
#endif
#endif

   om2eps = 1. - 2.*eps
#if ( defined SCAM )
!
! for scam I moved the time advance and readiop before dynamics to get forcasted
! values for divt3d and divq3d
!
   if (nstep.ge.3) then
#else
   if (nstep.ge.2) then
#endif
      do k=1,plev
         do i=1,nlon
            u3m1(i,k) = om2eps*u3m1(i,k) + eps*um2(i,k) + eps*u3(i,k)
            v3m1(i,k) = om2eps*v3m1(i,k) + eps*vm2(i,k) + eps*v3(i,k)
            t3m1(i,k) = om2eps*t3m1(i,k) + eps*tm2(i,k) + eps*t3(i,k)
            q3m1(i,k,1) = om2eps*q3m1(i,k,1) + eps*qm2(i,k,1) + eps*q3(i,k,1)
            vortm1(i,k) = om2eps*vortm1(i,k) + eps*vortm2(i,k) + eps*vort(i,k)
            divm1(i,k) = om2eps*divm1(i,k) + eps*divm2(i,k) + eps*div(i,k)
! jwh (replace first step)
!            if (nstep .eq. endstep-1) then
!               u3m1(i,k) = u3obs(i,k)
!               v3m1(i,k) = v3obs(i,k)
!               t3m1(i,k) = t3obs(i,k)
!               vortm1(i,k) = vt3obs(i,k)
!               divm1(i,k) = dv3obs(i,k)
!            end if
! 2010
         end do
!
         do m=2,pcnst+pnats
            do i=1,nlon
               q3m1(i,k,m) = om2eps*q3m1(i,k,m) + eps*qm2(i,k,m) + eps*q3(i,k,m)
            end do
         end do
      end do
      do i=1,nlon
         psm1(i) = om2eps*psm1(i) + eps*psm2(i) + eps*ps(i)
      end do
   end if

   call plevs0 (nlon    ,plond   ,plev    ,psm1    ,pintm1  ,pmidm1  ,pdelm1)

!
! Compute time tendencies:comment out since currently not on h-t
!
   do k=1,plev
      do i=1,nlon
         ttend(i,k) = (t3(i,k)-tm2(i,k))/ztodt
         utend(i,k) = (u3(i,k)-um2(i,k))/ztodt
         vtend(i,k) = (v3(i,k)-vm2(i,k))/ztodt
#ifdef TDK_CU
         tatend(i,k) = ttend(i,k)- t2phy(i,k)
#endif
      end do
   end do
#ifdef TDK_CU
   do m=1,pcnst+pnats
#else
   do m=1,pcnst
#endif
      do k=1,plev
         do i=1,nlon
            qtend(i,k,m) = (q3(i,k,m) - qm2(i,k,m))/ztodt
         end do
      end do
   end do

   do i=1,nlon
      pstend(i) = (ps(i) - psm2(i))/ztodt
   end do
!
!   do m=1,pcnst
!      call outfld (tendnam(m),qtend(1,1,m),plond,lat)
!   end do

   call outfld ('UTEND   ',utend,plond,lat)
   call outfld ('VTEND   ',vtend,plond,lat)
   call outfld ('TTEND   ',ttend,plond,lat)
   call outfld ('LPSTEN  ',pstend,plond,lat)
   call outfld ('QTEND   ',qtend(:,:,1),plond,lat)    !zf 2009.07.06

   return
1000 format(' TIMEFILTER: WARNING: fixer for tracer ',i3,' exceeded ', &
      f8.5,' for ',i5,' points at k,lat = ',2i4, &
      ' Avg/Worst = ',1p2e10.2)
end subroutine  tfilt_massfix
!-----------------------------------------------------------------------
 subroutine bnday (ncdate, day, yr)
!----------------------------------------------------------------------- 
! 
! Purpose: Convert date and seconds of day to floating point calendar day, for
!          boundary dataset handling
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
!
! Arguments
!
   integer, intent(in) :: ncdate      ! Current date as yymmdd or yyyymmdd

   real(r8), intent(out) :: day       ! Day of year

   integer, intent(out) :: yr       ! Day of year
!
! Local Variables
!
   integer myr         ! year number
   integer mnth        ! Month number
   integer mday        ! Day number of month
   integer mhour       ! hour number of day
   integer jdcon(12)   ! Starting day number for each month
   save jdcon
   data jdcon/0,31,59,90,120,151,181,212,243,273,304,334/
!
! Decode month and day
!
   myr = ncdate/10000
   mnth = mod(ncdate,10000)/100
   mday = mod(ncdate,100)
   day = jdcon(mnth) + mday - 1
   yr = myr   
   return
end subroutine bnday


 subroutine bnday_hour (ncdate, day, yr)
!-----------------------------------------------------------------------
!
! Purpose: Convert date and seconds of day to floating point calendar day, for
!          boundary dataset handling
!
! Method: Use table of days per month to do conversion
!
! Author: CCM Core Group
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
!
! Arguments
!
   integer, intent(in) :: ncdate      ! Current date as yymmddhh or yyyymmddhh

   real(r8), intent(out) :: day       ! Day of year

   integer, intent(out) :: yr       ! Day of year
!
! Local Variables
!
   integer myr         ! year number
   integer mnth        ! Month number
   integer mday        ! Day number of month
   integer mhour       ! hour number of day
   integer jdcon(12)   ! Starting day number for each month
   save jdcon
   data jdcon/0,31,59,90,120,151,181,212,243,273,304,334/
!
! Decode month and day
!
   myr = ncdate/1000000
   mnth = mod(ncdate,1000000)/10000
   mday = mod(ncdate,10000)/100
   mhour= mod(ncdate,100)
   day = jdcon(mnth) + mday - 1 + mhour/24.0
   yr = myr
  
   return
end subroutine bnday_hour

