
!-------------------------------------------------------------------------!
!  There are some modifications for implementation on gaussian grid       !
!  ZhangYi                                                                !
!  date: Dec. 2012                                                        !
!-------------------------------------------------------------------------!

 module tspas

   use shr_kind_mod,    only: r8=> shr_kind_r8
   use shr_const_mod,   only: ae=> shr_const_rearth
   use pmgrid,          only: masterproc,plon,plat,plev,plevp
   use constituents,    only: pcnst

   private
   save
   public :: tspas_init, tspas_run, tspas_final

      !global info
      real(r8), parameter               :: pi        = 4._r8*atan(1._r8)
      integer,  parameter               :: ng_c      = 3              ! equals to numbnd in scanslt
      integer,  parameter               :: platd     = plat+2*ng_c    ! tmp
      !
      real(r8), allocatable             :: sinu  (:)
      real(r8), allocatable             :: sinv  (:)
      real(r8), allocatable             :: cu    (:)
      real(r8), allocatable             :: cv    (:)
      real(r8), allocatable             :: dx    (:)
      real(r8), allocatable             :: tuuex (:,:) 
      real(r8), allocatable             :: qex   (:,:) 
      real(r8), allocatable             :: aex   (:,:) 
      real(r8), allocatable             :: fxex  (:,:) 
      real(r8), allocatable             :: mfxex (:,:) 
      integer,  allocatable             :: kk    (:)                   ! increment for grid in high latitude
      real(r8)                          :: dt
      integer                           :: jfirst
      integer                           :: jlast
      integer                           :: js
      integer                           :: jn
      integer                           :: incre_begin_south
      integer                           :: incre_begin_north
      integer                           :: kmax
      real(r8)                          :: dphi  (platd)          ! lat increment in radians 
      real(r8)                          :: dphi2 (platd)          ! only for calculating FY at js and jn-1

 contains

 subroutine tspas_init(dt_in,jfirst_in,jlast_in)

 !---------------------------------------------------!
 !          initialize global info                   !
 !---------------------------------------------------!
      implicit none

      real(r8), intent(in)  :: dt_in
      integer,  intent(in)  :: jfirst_in,jlast_in
     
      integer j
      real(r8) :: kkc,n

      jfirst  = jfirst_in
      jlast   = jlast_in
      dt      = dt_in

      allocate(sinu(platd))        ! sin of colatitude at u
      allocate(sinv(platd))        ! sin of colatitude at v
      allocate(cu(platd))          !
      allocate(cv(platd))          !
      allocate(dx(platd))          !
      allocate(kk(platd))          !

      call gridxy(sinu,sinv,cu,cv,js,jn,dx,dphi,dphi2) ! acquire grid info

      kk(:) = 1
      ! calculate kk
      n    = 2._r8       ! tuning para
      kkc  = 1._r8

      do j = platd/2+1, jn-1
         if(dx(platd/2)/(kkc*dx(j)) .gt. n)then
            kkc = check(n*kkc)
         end if
         kk(j)  = kkc
      end do
      kk(js+1:platd/2) = kk(jn-1:platd/2+1:-1)
     
         kmax = kk(1)
      do j = 1, platd
         if(kk(j).gt.kmax)then
            kmax = kk(j)
         end if
      end do

      if(masterproc) print*,"kmax",kmax

      do j = platd/2,1,-1           !south pole
         if(kk(j).ne.1)then
            incre_begin_south = j
            goto 10 
         end if
      end do
 
  10    do j = platd/2+1,platd      !north pole
         if(kk(j).ne.1)then
            incre_begin_north = j
            goto 20 
         end if
      end do
   
  20  if(masterproc) print*,"incre begin north and south",incre_begin_north,incre_begin_south

      ! allocate extended array
      allocate(tuuex(1-kmax:plon+kmax,jfirst-ng_c:jlast+ng_c))
      allocate(  qex(1-kmax:plon+kmax,jfirst-ng_c:jlast+ng_c))
      allocate( fxex(1-kmax:plon+kmax,jfirst-ng_c:jlast+ng_c))
      allocate(mfxex(1-kmax:plon+kmax,jfirst-ng_c:jlast+ng_c))
      allocate(  aex(1-kmax:plon+kmax,jfirst-ng_c:jlast+ng_c))

      return

 end subroutine tspas_init

!---------------------------------------------------------------------------------------------
!
! V-wind at mid-point of meridional-circle,since the q ranges from js 
! to jn, so v needs to have js to jn-1. 
!
! The C-grid used: from south(low) to north(high)
!
!        (north) jlast
!                 
!                vj 
!                |
! (west)  ui----Qij----ui+1  (east)
!                |
!                vj-1 
!
!        (south) jfirst
!
! the south pole is source, the north pole is sink
!
! q/u(j)[v(j)] ranges from js[js-1] to jn, namely the southernmost and northernmost
!
! uc: jfirst-1 to jlast+1
! vc: jfirst-2 to jlast+1
! if poles occur, using specific operations
!
!---------------------------------------------------------------------------------------------

  subroutine tspas_run(q,u,v,w,detai,detam,NSPLIT)

      use shr_kind_mod,  only: r8=>shr_kind_r8
      use shr_const_mod, only: ae=>SHR_CONST_REARTH

      implicit none

      !-----------------------------------------------------------------------
      !iostream
      real(r8), intent(inout) :: q(plon,jfirst-ng_c:jlast+ng_c,plev,pcnst)
      real(r8), intent(in)    :: u(plon,jfirst-ng_c:jlast+ng_c,plev)
      real(r8), intent(in)    :: v(plon,jfirst-ng_c:jlast+ng_c,plev)
      real(r8), intent(inout) :: w(plon,jfirst-ng_c:jlast-ng_c,plevp) !out value for nothing
      real(r8), intent(in)    :: detai(plevp-1)
      real(r8), intent(in)    :: detam(plev-1)
      integer,  intent(in)    :: NSPLIT
      ! ---------------------------------LOCAL--------------------------------
      real(r8) :: uex(0:plon,jfirst-ng_c:jlast+ng_c)
      real(r8) :: uc(plon,jfirst-ng_c:jlast+ng_c)
      real(r8) :: vc(plon,jfirst-ng_c:jlast+ng_c)
      real(r8) :: tuu(plon,jfirst-ng_c:jlast+ng_c)
      real(r8) :: tvv(plon,jfirst-ng_c:jlast+ng_c)
      real(r8) ::  fx(plon,jfirst-ng_c:jlast+ng_c)  !for qhstar
      real(r8) ::  fy(plon,jfirst-ng_c:jlast+ng_c)
      real(r8) :: mfx(plon,jfirst-ng_c:jlast+ng_c)  !for q
      real(r8) :: mfy(plon,jfirst-ng_c:jlast+ng_c)
      real(r8) ::      a(plon,jfirst-ng_c:jlast+ng_c)
      real(r8) :: qhstar(plon,jfirst-ng_c:jlast+ng_c)
      real(r8) ::   beta(plon,jfirst-ng_c:jlast+ng_c)
      real(r8) ::  cx
      real(r8) ::  cy
      real(r8) ::  cxstar
      real(r8) ::  cystar
      !-------------------------------------------------------------------------
      real(r8)  qmin
      real(r8)  qmax
      real(r8)  gama
      real(r8)  temp1
      real(r8)  temp2
      real(r8)  temp3
      real(r8)  temp4
      real(r8)  qw(plev)
      real(r8)  ww(plevp)
      real(r8)  tmp_sum
      real(r8)  qh_pole_1_south
      real(r8)  qh_pole_1_north
      real(r8)  zero, half, fourth
      data      zero, half, fourth /0.0_r8,0.5_r8,0.25_r8/
      integer   it, i, j, k, iv

      !-----------------------------------------------------------------------
      ! horizontal advection, k-loop outside
      DO K = 1, PLEV

      ! fill uex
      uex(1:plon,:) = u(1:plon,:,k)
      uex(0,:)      = u(plon,:,k)
      ! transform to C-grid
      do i = 1, plon
        do j = jfirst-1, jlast+1
           uc(i,j)  = 0.5_r8*(uex(i-1,j)+uex(i,j))
           tuu(i,j) = cu(j)*uc(i,j)/kk(j)

           vc(i,j)  = 0.5_r8*(v(i,j,k)+v(i,j+1,k))
           tvv(i,j) = cv(j)*vc(i,j)
        end do
           j = jfirst-2
           vc(i,j)  = 0.5_r8*(v(i,j,k)+v(i,j+1,k))
           tvv(i,j) = cv(j)*vc(i,j)
      end do

      ! fill tuuex
      tuuex(1:plon,:)           = tuu(1:plon,:)
      tuuex(1-kmax:0,:)         = tuu(plon+1-kmax:plon,:)
      tuuex(plon+1:plon+kmax,:) = tuu(1:kmax,:)

      do j = jfirst-1, jlast+1
        do i = 1, plon   ! this is the index at Q !
           temp1 = abs(tuuex(i-(kk(j)-1)/2,j)/sinu(j))*(1-abs(tuuex(i-(kk(j)-1)/2,j)/sinu(j)))
           temp2 = abs(tuuex(i+(kk(j)+1)/2,j)/sinu(j))*(1-abs(tuuex(i+(kk(j)+1)/2,j)/sinu(j)))
           temp3 = abs(tvv(i,j-1)/(dphi(j)*sinv(j-1)))*(1-abs(tvv(i,j-1)/(dphi(j)*sinv(j-1))))
           temp4 = abs(tvv(i,j)/(dphi(j)*sinv(j)))*(1-abs(tvv(i,j)/(dphi(j)*sinv(j))))
           gama  = max(temp1, temp2, temp3, temp4)
           beta(i,j) = 1.0_r8/(1.0_r8-gama)*1.2_r8
        end do
      end do

!======================       ADVECTION BEGINS       =======================
      ! For each time interval, each cnst and each level
      DO IV = 1, PCNST

      if(jfirst.eq.js)then   !south pole, ensure single value at pole
            q(:,js,k,iv)   = sum(q(1:plon,js,k,iv)/plon)
            w(:,js-ng_c,k) = sum(w(1:plon,js-ng_c,k)/plon)
      endif
     
      if(jlast.eq.jn)then    !north pole, ensure single value at pole
            q(:,jn,k,iv)   = sum(q(1:plon,jn,k,iv)/plon)
            w(:,jn-ng_c,k) = sum(w(1:plon,jn-ng_c,k)/plon)
      endif

      DO IT = 1, NSPLIT

      ! fill new qex
      qex(1:plon,:)           = q(1:plon,:,k,iv)
      qex(1-kmax:0,:)         = q(plon+1-kmax:plon,:,k,iv)
      qex(plon+1:plon+kmax,:) = q(1:kmax,:,k,iv)

      ! -----------------------------------------------------------------------
      ! lax-wendroff scheme
      ! =======================================================================
      do i = 1, plon
        do j = jfirst-1, jlast+1
           fx(i,j) = half*tuu(i,j)*(qex(i+(kk(j)-1)/2,j)+qex(i-(kk(j)+1)/2,j))&
          -half*tuu(i,j)*tuu(i,j)*(qex(i+(kk(j)-1)/2,j)-qex(i-(kk(j)+1)/2,j))/sinu(j)
           fy(i,j) = half*tvv(i,j)*(q(i,j+1,k,iv)+q(i,j,k,iv))&
          -half*tvv(i,j)*tvv(i,j)/dphi(j)*(q(i,j+1,k,iv)-q(i,j,k,iv))/sinv(j)
        end do
           j  = jfirst-2
           fy(i,j) = half*tvv(i,j)*(q(i,j+1,k,iv)+q(i,j,k,iv))&
          -half*tvv(i,j)*tvv(i,j)/dphi(j)*(q(i,j+1,k,iv)-q(i,j,k,iv))/sinv(j)
      end do

      ! fill fxex
      fxex(1:plon,:)           = fx(1:plon,:)
      fxex(1-kmax:0,:)         = fx(plon+1-kmax:plon,:)
      fxex(plon+1:plon+kmax,:) = fx(1:kmax,:)

      ! =======================================================================
      ! calculate intermediate qhstar
      ! fx jfirst-1 to jlast+1, fy jfirst-2 to jlast+1
      do j = jfirst-1, jlast+1
        do i = 1, plon
           qhstar(i,j) = q(i,j,k,iv)-beta(i,j)*&
                 (fxex(i+(kk(j)+1)/2,j)-fxex(i-(kk(j)-1)/2,j)+((fy(i,j)-fy(i,j-1))/dphi2(j)))/sinu(j)
        end do
      end do
      ! =======================================================================
      ! handle poles
      if (jfirst .eq. js) then
          tmp_sum = zero
          do i = 1, plon 
             tmp_sum       = tmp_sum+beta(i,js+1)*fy(i,js)
          end do
             qhstar(:,js)  = q(1,js,k,iv)-2._r8*tmp_sum/(sinv(js)*dphi2(js)*plon)
      end if

      if (jlast .eq. jn) then
          tmp_sum = zero
          do i = 1, plon
             tmp_sum           = tmp_sum+beta(i,jn-1)*fy(i,jn-1)
          end do
             qhstar(:,jn) = q(1,jn,k,iv)+2._r8*tmp_sum/(sinv(jn-1)*dphi2(jn)*plon)
      end if

      ! -----------------------------------------------------------------------
      ! tspr judgement (a <= 0 is good)
      ! to ensure a from jfirst-1 to jlast+1, q from jfirst-2 and jlast+2
      do j = jfirst-1, jlast+1
         if(j.eq.js) then
            do i = 1, plon
               qmin   =  1.0e15
               qmax   = -1.0e15
               qmin   = min(q(i,j,k,iv), q(i,j+1,k,iv), qmin)
               qmax   = max(q(i,j,k,iv), q(i,j+1,k,iv), qmax)
               a(i,j) = (qhstar(i,j)-qmax)*(qhstar(i,j)-qmin)
            enddo
         else if (j .eq. jn) then
            do i = 1, plon
               qmin   =  1.0e15
               qmax   = -1.0e15
               qmin   = min(q(i,j,k,iv), q(i,j-1,k,iv), qmin)
               qmax   = max(q(i,j,k,iv), q(i,j-1,k,iv), qmax)
               a(i,j) = (qhstar(i,j)-qmax)*(qhstar(i,j)-qmin)
            enddo
         else
            do i = 1, plon
               qmin   =  1.0e15
               qmax   = -1.0e15
               qmin   = min(qex(i,j),qex(i+kk(j),j),qex(i-kk(j),j),qex(i,j+1),qex(i,j-1),qmin)
               qmax   = max(qex(i,j),qex(i+kk(j),j),qex(i-kk(j),j),qex(i,j+1),qex(i,j-1),qmax)
               a(i,j) = (qhstar(i,j)-qmax)*(qhstar(i,j)-qmin)
            enddo
         end if
      end do

      ! fill aex
      aex(1:plon,:)           = a(1:plon,:)
      aex(1-kmax:0,:)         = a(plon+1-kmax:plon,:)
      aex(plon+1:plon+kmax,:) = a(1:kmax,:)

      ! ------------------------------------------------------------------------
      ! upwind scheme
      ! =======================================================================
      ! calculate fluxes
       do i = 1, plon !index at U
         ! mfx from jfirst to jlast
         do j = jfirst, jlast

              if(aex(i+(kk(j)-1)/2,j).le.0.0_r8 .and. aex(i-(kk(j)+1)/2,j).le.0.0_r8)then 
                  cx  = abs(tuu(i,j)/sinu(j))
              else
                  cx  = 1.0_r8
              end if

                 mfx(i,j) = half*tuu(i,j)*(qex(i+(kk(j)-1)/2,j)+qex(i-(kk(j)+1)/2,j))&
                -half*abs(cx*tuu(i,j))*(qex(i+(kk(j)-1)/2,j)-qex(i-(kk(j)+1)/2,j))

              if(a(i,j).le.0.0_r8 .and. a(i,j+1).le.0.0_r8)then
                 cy  = abs(tvv(i,j)/sinv(j))/dphi(j)
              else
                 cy  = 1.0_r8
              end if
                 mfy(i,j) = half*tvv(i,j)*&
                            (q(i,j+1,k,iv)+q(i,j,k,iv))-half*abs(cy*tvv(i,j))*(q(i,j+1,k,iv)-q(i,j,k,iv))
         end do
            !q and a needs from jfirst-1 to jlast +1 to ensure mfy form jfirst-1 to jlast
               j   = jfirst-1

               if(a(i,j).le.0.0_r8 .and. a(i,j+1).le.0.0_r8)then
                 cy  = abs(tvv(i,j)/sinv(j))/dphi(j)
               else
                 cy  = 1.0_r8
               end if

                 mfy(i,j) = half*tvv(i,j)*&
                            (q(i,j+1,k,iv)+q(i,j,k,iv))-half*abs(cy*tvv(i,j))*(q(i,j+1,k,iv)-q(i,j,k,iv))
       end do

      ! fill mfxex
      mfxex(1:plon,:)           = mfx(1:plon,:)
      mfxex(1-kmax:0,:)         = mfx(plon+1-kmax:plon,:)
      mfxex(plon+1:plon+kmax,:) = mfx(1:kmax,:)

       ! -----------------------------------------------------------------------
       ! handle poles
      if (jfirst .eq. js) then
          qh_pole_1_south = q(1,js,k,iv)
      end if

      if (jlast .eq. jn) then
          qh_pole_1_north = q(1,jn,k,iv)
      end if

      ! for q from jfirst to jlast, mfx from jfirst to jlast, mfy from jfirst-1
      ! to jlast
      do j = jfirst, jlast
        do i = 1, plon
           q(i,j,k,iv) = q(i,j,k,iv)-(mfxex(i+(kk(j)+1)/2,j)-mfxex(i-(kk(j)-1)/2,j)+((mfy(i,j)-mfy(i,j-1))/dphi2(j)))/sinu(j)
        end do
      end do

      !update poles
      if(jfirst.eq.js)then
          q(:,js,k,iv)     = qh_pole_1_south-2._r8*sum(mfy(1:plon,js))/(sinv(js)*dphi2(js)*plon)
      end if

      if(jlast.eq.jn)then
          q(:,jn,k,iv)     = qh_pole_1_north+2._r8*sum(mfy(1:plon,jn-1))/(sinv(jn-1)*dphi2(jn)*plon)
      end if

      END DO ! nsplit
      END DO ! pcnst
      END DO ! plev

! vertical advection still uses large step

      do iv= 1, pcnst
        do i = 1, plon
          do j = jfirst-ng_c, jlast-ng_c ! follow the index of ww
             qw(:) = q(i,j+ng_c,:,iv)
             ww(:) = w(i,j,:)
             call tspasw(qw,ww,detam,detai,nsplit)
             q(i,j+ng_c,:,iv) = qw(:)
          enddo
        enddo
      enddo

    return

 end subroutine tspas_run 

 subroutine tspas_final
  
      deallocate(sinu)
      deallocate(sinv)
      deallocate(cu)
      deallocate(cv)
      deallocate(dx)
      deallocate(kk)
      deallocate(tuuex)
      deallocate(  qex)
      deallocate( fxex)
      deallocate(mfxex)
      deallocate(  aex)

 end subroutine tspas_final

!-----------------------------------------------------------
 subroutine tspasw(q,w,detam,detai,nsplit)
  
  !---------------------------------------------------------
  ! purpose: conduct vertical advection
  !
      implicit none
     
      !iostream
      real(r8), intent(inout) :: q(plev)            !q at mid-point
      real(r8), intent(in)    :: w(plevp)           !vertical motion at interface
      real(r8), intent(in)    :: detam(plev-1)      !delta pressure at mid-point
      real(r8), intent(in)    :: detai(plevp-1)     !delta pressure at int-point
      integer,  intent(in)    :: nsplit
      !local
      real(r8)                :: qstar(plev)
      real(r8)                ::    aw(plev)
      real(r8)                ::  qmin(plev), qmax(plev)
      real(r8)                :: detamdt(plev-1),dtdetam(plev-1)
      real(r8)                :: detaidt(plev), dtdetai(plev)
      real(r8)                :: fw(plevp)
      real(r8)                :: temp1,temp2,temp3,temp4
      real(r8)                :: gama
      real(r8)                :: beta(plev)
      real(r8)                :: cwstar,wstar,cw
      integer                 :: k


      do k=1,plev
         qmin(k)        = 1.0e15_r8
         qmax(k)        =-1.0e15_r8
         detaidt(k)     = detai(k)/(dt*nsplit)
         dtdetai(k)     = nsplit*dt/detai(k)
      enddo

      do k=1,plev-1
         detamdt(k)     = detam(k)/(dt*nsplit)
         dtdetam(k)     = nsplit*dt/detam(k)
      enddo

      !Lax-Wendroff
      do k=2,plevp-1
         fw(k)          = 0.5_r8*w(k)*(q(k)+q(k-1))-0.5_r8*w(k)*w(k)*(q(k)-q(k-1))/detamdt(k-1)
      enddo

      do k=2,plev-1
         temp1          = abs(w(k)/detamdt(k-1))*(1-abs(w(k)/detamdt(k-1)))
         temp2          = abs(w(k+1)/detamdt(k))*(1-abs(w(k+1)/detamdt(k)))
         gama           = max(temp1,temp2)
         beta(k)        = 2.0_r8/(2.0_r8-gama)
         qstar(k)       = q(k)-beta(k)*(fw(k+1)-fw(k))*dtdetai(k)
      enddo
         qstar(1)       = q(1)-beta(2)*fw(2)*dtdetai(1)
         qstar(plev)    = q(plev)+beta(plev-1)*fw(plev)*dtdetai(plev)

      !for aw, at the same point as q
      do k=2,plev-1
         qmin(k)     = min(q(k),q(k-1),q(k+1),qmin(k))
         qmax(k)     = max(q(k),q(k-1),q(k+1),qmax(k))
         aw(k)       = (qstar(k)-qmin(k))*(qstar(k)-qmax(k))
      enddo
         qmin(1)     = min(q(1),q(2),qmin(1))
         qmax(1)     = max(q(1),q(2),qmax(1))
         aw(1)       = (qstar(1)-qmin(1))*(qstar(1)-qmax(1))

         qmin(plev)  = min(q(plev),q(plev-1),qmin(plev))
         qmax(plev)  = max(q(plev),q(plev-1),qmax(plev))
         aw(plev)    = (qstar(plev)-qmin(plev))*(qstar(plev)-qmax(plev))

      !for fw, at the same point as w
      do k=2,plevp-1
         cw       = 1.0_r8
         if(aw(k-1).le.0.0_r8 .and. aw(k).le.0.0_r8) cw = abs(w(k)/detamdt(k-1)) 
         wstar       = cw*w(k)
         fw(k)       = 0.5_r8*w(k)*(q(k)+q(k-1))-0.5_r8*abs(wstar)*(q(k)-q(k-1))
      enddo
         fw(1)       = 0.0_r8
         fw(plevp)   = 0.0_r8

      do k=1,plev
         q(k)        = q(k)-(fw(k+1)-fw(k))*dtdetai(k)
      enddo

      return

 end subroutine tspasw

!-----------------------------------------------------------

 subroutine gridxy(sinu,sinv,cu,cv,js,jn,dx,dphi,dphi2)

 !----------------------------------------------------------
 ! 
 ! Purpose: 
 ! Define the "extended" grid used in the semi-Lagrangian transport
 ! scheme.  The longitudes are equally spaced and the latitudes are
 ! Gaussian.  The global grid is extended to include "wraparound" points
 ! on all sides.
 ! 
 ! Method: 
 ! 
 ! Author: J. Olson
 ! raw: grdxy in scanslt.F90, modified for tspas
 !----------------------------------------------------------

   use gauaw_mod, only: gauaw
      implicit none
      integer, parameter         :: nxpt   = 1             ! no. of pts outside active domain of interpolant
      integer, parameter         :: jintmx = 2             ! number of extra latitudes in polar region
      integer, parameter         :: istart = 1             ! index for first model long.
      integer, parameter         :: jstart = nxpt+jintmx+1 ! index for first model lat.
      integer, parameter         :: jstop  = jstart-1+plat ! index for last  model lat.
      !iostream
      real(r8), intent(out)      :: sinu(platd)       ! at gaussian
      real(r8), intent(out)      :: sinv(platd)
      real(r8), intent(out)      ::   cu(platd)
      real(r8), intent(out)      ::   cv(platd)
      real(r8), intent(out)      ::   dx(platd)
      integer,  intent(out)      :: js
      integer,  intent(out)      :: jn
      real(r8), intent(out)      :: dphi  (platd)          ! lat increment in radians 
      real(r8), intent(out)      :: dphi2 (platd)          ! only for calculating FY at js and jn-1
      !local
      real(r8)                   :: dlam  (platd)          ! lon increment in radians
      real(r8)                   :: lam   (plon,platd)     ! lon coords    in extended grid
      integer                    :: nlonex(platd)
      !
      real(r8)                   :: sinlam(plon,platd)     ! sin(lam)
      real(r8)                   :: coslam(plon,platd)     ! cos(lam)
      real(r8)                   :: wrk(platd)             ! work space
      real(r8)                   :: phi_half(platd)        ! latitude at V-point 
      real(r8)                   :: phi(platd)        ! latitude at V-point 
      real(r8)                   :: w(plat)        ! latitude at V-point 
      real(r8)                   :: cophi                  ! co-latitude, keep it positive
      integer                    :: i,j,ig                 ! indices
      integer                    :: nlond                  ! extended long dim
      real(r8)                   :: lam0                   ! lamda = 0

      lam0 = 0.0_r8
      !zy:since there is no reduced grid, nlonex = plon
      nlonex(:) = plon
      !
      !Interval length in equally spaced longitude grid.
      !
      do j=1,platd
         dlam(j) = 2._r8*pi/real(nlonex(j),r8)
         do i = 1,nlonex(j)
            lam(i,j) = real(i-istart,r8)*dlam(j) + lam0
         end do
      end do
      !
      ! Compute Gauss latitudes and weights.  On return; phi contains the
      ! sine of the latitudes starting closest to the north pole and going
      ! toward the south; w contains the corresponding Gauss weights.
      !
      call gauaw(phi,w,plat)
      !
      ! Reorder and compute latitude values.
      !
      do j = jstart,jstop
         wrk(j) = asin( phi(jstop-j+1) )
      end do
      phi(jstart:jstop) = wrk(jstart:jstop)
      !
      ! North and south poles.
      !
      phi(jstart-1) = -pi/2.0_r8
      phi(jstop +1) =  pi/2.0_r8
      !
      ! Extend Gauss latitudes below south pole so that the spacing above
      ! the pole is symmetric, and phi is decreasing, i.e., phi < -pi/2
      !
      if( jstart > 2 )then
        do j = 1,jstart-2
           phi(j) = -pi - phi(2*jstart-2-j)
        end do
      end if
      !
      ! Analogously for Northern Hemisphere
      !
      if( platd > jstop+1 )then
         do j = jstop+2,platd
            phi(j) = pi - phi(2*jstop+2-j)
         end do
      end if
      !
      ! Sine and cosine of longitude.
      !
      do j=1,platd
         ig = 0
         !do i = istart,nlonex(j)+nxpt
         do i = istart,nlonex(j)
            ig = ig + 1
            sinlam(ig,j) = sin( lam(i,j) )
            coslam(ig,j) = cos( lam(i,j) )
         end do
      end do

      !------------------------------------------------------
      !  Below stuff is exclusively added for TSPAS                
      !                                          
      !  for dphi and dy, since the dy only means to v
      !  dphi at j is the distance between vc(j-1) to vc(j)
      !  namely:  
      !                  |   dy   |
      !            vc(j-1)---qj---vc(j)
      !  since here, each j is at gaussian point (qj),
      !  so some averaging needed
      !------------------------------------------------------

      ! for dy, cv, at half grid, phi is on int grid
      ! dphi and dphi2 only differ at pole
      ! set dphi  the rad between integer grid
      ! set dphi2 the rad between half(V) grid

      do j = jstart-1,jstop
         dphi(j) = abs(phi(j+1)-phi(j)) ! delta rad between gaussian lat
      end do

      do j = jstart-1,jstop ! half grid
         phi_half(j)  = 0.5_r8*(phi(j+1)+phi(j))       ! lat at half grid
         cophi        = pi/2._r8-abs(phi_half(j))
         sinv(j)      = sin(cophi)
         cv(j)        = dt*sinv(j)/ae
      enddo

      do j = jstart+1,jstop-1
         dphi2(j) = abs(phi_half(j)-phi_half(j-1))     ! delta rad between V lat
      end do
         dphi2(jstart) = pi/2._r8-abs(phi_half(jstart))
         dphi2(jstop)  = pi/2._r8-abs(phi_half(jstop-1))

      !for dx, cu, at integer grid
      do j = jstart,jstop
         cophi   = pi/2._r8-abs(phi(j))
         dx(j)   = ae*sin(cophi)*dlam(j)  !when pole, dx=0
         sinu(j) = sin(cophi)
         cu(j)   = dt*sinu(j)/(ae*sinu(j)*dlam(j))
      enddo

      js       = jstart
      jn       = jstop

      return
 end subroutine gridxy

 real(r8) function check(num)

      real(r8), intent(in) :: num

      if(mod(num,2.0_r8).eq.0.)then           ! even
         check = num+1._r8
      else if(mod(num,2.0_r8).eq.1.)then      ! odd
         check = num
      else                                    ! fraction, take the closet int number larger than that

        if(mod(int(num)+1,2).eq.0)then        ! even
          check = int(num)+1._r8+1
        else if(mod(int(num)+1,2).eq.1)then   ! odd
          check = int(num)+1._r8
        end if
      end if 

 return
 end function check

 end module tspas
