#include <misc.h>
#include <params.h>

subroutine windequ(nstep,   nlon,   ztodt,   fcor,                     &
                   u3m2,    u3m1,   u3,      v3m2,   v3m1,        v3,  &
		           windRu3m2,       windRu3m1,       windRu3,          &
			       windRv3m2,       windRv3m1,       windRv3,          &
			       windmu,          windmv)

!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid

   implicit none
!---------------------------------------------------------------------
!
! Arguments
!
   integer, intent(in) :: nstep
   integer, intent(in) :: nlon            ! number of longitudes
   real(r8), intent(in) :: ztodt          ! twice the timestep unless nstep=0
   real(r8), intent(in) :: fcor

   real(r8), intent(in) :: u3m2(plond,plev)  
   real(r8), intent(in) :: u3m1(plond,plev)  

   real(r8), intent(in) :: v3m2(plond,plev)  
   real(r8), intent(in) :: v3m1(plond,plev)  

   real(r8), intent(in) :: windRu3m2(plond,plev)
   real(r8), intent(in) :: windRu3m1(plond,plev)
   real(r8), intent(in) :: windRu3  (plond,plev)

   real(r8), intent(in) :: windRv3m2(plond,plev)
   real(r8), intent(in) :: windRv3m1(plond,plev)
   real(r8), intent(in) :: windRv3  (plond,plev)

   real(r8), intent(in) :: windmu(plond,plev)
   real(r8), intent(in) :: windmv(plond,plev)

   real(r8), intent(out) :: u3  (plond,plev)  
   real(r8), intent(out) :: v3  (plond,plev)  
!
!---------------------------Local workspace-----------------------------
!
   integer i,k               ! longitude, level
   real(r8)  a
   real(r8)  bu
   real(r8)  bv
!
!-----------------------------------------------------------------------
   if(nstep.eq.0) then

     do k=1,plev
        do i=1,nlon
           u3(i,k) = u3m1(i,k) + ztodt*( v3m1(i,k)*fcor               &
		                          + windRu3m1(i,k) + windmu(i,k) )
           v3(i,k) = v3m1(i,k) + ztodt*( -u3m1(i,k)*fcor              &
		                          + windRv3m1(i,k) + windmv(i,k) )
        end do
     end do
!
   else
!
     a  = fcor*ztodt*0.25
     do k=1,plev
        do i=1,nlon
           bu = u3m2(i,k) + ztodt*( (2.0*v3m1(i,k) +v3m2(i,k))*fcor*0.25             &
		                  + (windRu3m2(i,k) +2.0*windRu3m1(i,k) +windRu3(i,k))*0.25  &
				  + windmu(i,k) )
           bv = v3m2(i,k) + ztodt*( -(2.0*u3m1(i,k) +u3m2(i,k))*fcor*0.25            &
		                  + (windRv3m2(i,k) +2.0*windRv3m1(i,k) +windRv3(i,k))*0.25  &
             			  + windmv(i,k) )
           u3(i,k) = (bu +a*bv)/(1.0 + a*a)
           v3(i,k) = -a*u3(i,k) + bv 
        end do
     end do
!
   endif

end subroutine windequ


