! --------------------------------------------------------------------
MODULE BCC_RAD_init_mod
! --------------------------------------------------------------------

! --------------------------------------------------------------------
! Initializes the fields to write to file and initializes anything 
! that is related to the McICA codes that can be done up front.  For 
! exaample, the selective spectral sampling approach
! --------------------------------------------------------------------

  USE shr_kind_mod,  ONLY: r8=>shr_kind_r8
  USE ppgrid,        ONLY: pcols, pver
  USE constituents,  ONLY: cnst_get_ind
  
  USE physconst,     ONLY: gravit, latvap, zvir

  IMPLICIT NONE

CONTAINS

  SUBROUTINE BCC_RAD_init()

    USE history,       ONLY: addfld, add_default, phys_decomp

    IMPLICIT NONE

    !!!!2-D radiation variables -->
     ! all sky fluxes
      call addfld ('GFLWDS  ','W/m2    ',1, 'A','Mean GEN downward longwave flux at surface (McICA)' ,phys_decomp) 
      call addfld ('GFLNS   ','W/m2    ',1, 'A','Mean GEN net longwave flux at surface (McICA)'      ,phys_decomp)
      call addfld ('GFLNT   ','W/m2    ',1, 'A','Mean GEN net longwave flux at top (McICA)'          ,phys_decomp) 
      call addfld ('GFLUT   ','W/m2    ',1, 'A','Mean GEN upward longwave flux at TOM (McICA)'       ,phys_decomp)
      call addfld ('GLWCF   ','W/m2    ',1, 'A','Mean GEN longwave cloud forcing at TOM (McICA)'     ,phys_decomp) 
      call addfld ('GFSNS   ','W/m2    ',1, 'A','Mean GEN net solar flux at surface (McICA)'         ,phys_decomp)
      call addfld ('GFSNT   ','W/m2    ',1, 'A','Mean GEN net solar flux at TOM (McICA)'             ,phys_decomp) 
      call addfld ('GFSNTOA ','W/m2    ',1, 'A','Mean GEN net solar flx at TOA (McICA)'              ,phys_decomp)
      call addfld ('GFSDS   ','W/m2    ',1, 'A','Mean GEN downward shortwave flux at surface (McICA)',phys_decomp) 
      call addfld ('GFSUT   ','W/m2    ',1, 'A','Mean GEN upward shortwave flux at TOM (McICA)'      ,phys_decomp)
      call addfld ('GFSUTOA ','W/m2    ',1, 'A','Mean GEN upward shortwave flux at TOA (McICA)'      ,phys_decomp)
      call addfld ('GSWCF   ','W/m2    ',1, 'A','Mean GEN shortwave cloud forcing at TOM (McICA)'    ,phys_decomp) 
      call addfld ('GSOLS   ','W/m2    ',1, 'A','Mean GEN shortwave vis. direct beam flux (McICA)'   ,phys_decomp)
      call addfld ('GSOLL   ','W/m2    ',1, 'A','Mean GEN shortwave NIR direct beam flux (McICA)'    ,phys_decomp)
      call addfld ('GSOLSD  ','W/m2    ',1, 'A','Mean GEN shortwave vis. diffuse beam flux (McICA)'  ,phys_decomp)
      call addfld ('GSOLLD  ','W/m2    ',1, 'A','Mean GEN shortwave NIR diffuse beam flux (McICA)'   ,phys_decomp)
      call addfld ('GSOLIN  ','W/m2    ',1, 'A','Mean GEN shortwave flux incident at TOA (McICA)'    ,phys_decomp)

   !  ! clear sky fluxes
      call addfld ('GFLWDSC ','W/m2    ',1, 'A','Mean clear-sky downward longwave flux at surface (McICA)' ,phys_decomp) 
      call addfld ('GFLNSC  ','W/m2    ',1, 'A','Mean clear-sky net longwave flux at surface (McICA)'      ,phys_decomp)
      call addfld ('GFLNTC  ','W/m2    ',1, 'A','Mean clear-sky net longwave flux at top (McICA)'          ,phys_decomp) 
      call addfld ('GFLUTC  ','W/m2    ',1, 'A','Mean clear-sky upward longwave flux at TOM (McICA)'       ,phys_decomp)
      call addfld ('GFSNSC  ','W/m2    ',1, 'A','Mean clear-sky net solar flux at surface (McICA)'         ,phys_decomp)
      call addfld ('GFSNTC  ','W/m2    ',1, 'A','Mean clear-sky net solar flux at TOM (McICA)'             ,phys_decomp) 
      call addfld ('GFSNTOAC','W/m2    ',1, 'A','Mean clear-sky net solar flx at TOA (McICA)'              ,phys_decomp)
      call addfld ('GFSDSC  ','W/m2    ',1, 'A','Mean clear-sky downward shortwave flux at surface (McICA)',phys_decomp) 
      call addfld ('GFSUTC  ','W/m2    ',1, 'A','Mean clear-sky upward shortwave flux at TOM (McICA)'      ,phys_decomp)
      call addfld ('GFSUTOAC','W/m2    ',1, 'A','Mean clear-sky upward shortwave flux at TOA (McICA)'      ,phys_decomp)

   ! !!!!3-D radiation variables -->

   !   call addfld ('GFSWU   ','K/s     ',pver, 'A','Mean GEN shortwave upward flux at each layer (McICA)',phys_decomp)
   !   call addfld ('GFSWD   ','K/s     ',pver, 'A','Mean GEN shortwave downward flux at each layer (McICA)',phys_decomp)
   !   call addfld ('GFSWUC  ','K/s     ',pver, 'A','Mean GEN shortwave clear-sky upward flux at each layer (McICA)',phys_decomp)
   !   call addfld ('GFSWDC  ','K/s     ',pver, 'A','Mean GEN shortwave clear-sky downward flux at each layer (McICA)',phys_decomp)

   !   call addfld ('GFLWU   ','K/s     ',pver, 'A','Mean GEN longwave upward flux at each layer (McICA)',phys_decomp)
   !   call addfld ('GFLWD   ','K/s     ',pver, 'A','Mean GEN longwave downward flux at each layer (McICA)',phys_decomp)
   !   call addfld ('GFLWUC  ','K/s     ',pver, 'A','Mean GEN longwave clear-sky upward flux at each layer (McICA)',phys_decomp)
   !   call addfld ('GFLWDC  ','K/s     ',pver, 'A','Mean GEN longwave clear-sky downward flux at each layer (McICA)',phys_decomp)

      call addfld ('GQRL    ','K/s     ',pver, 'A','Mean GEN longwave heating rate (McICA)',phys_decomp)
      call addfld ('GQRS    ','K/s     ',pver, 'A','Mean GEN shortwave heating rate (McICA)',phys_decomp)
      call addfld ('GQRLC   ','K/s     ',pver, 'A','clear-sky longwave heating rate (McICA)',phys_decomp)
      call addfld ('GQRSC   ','K/s     ',pver, 'A','clear-sky shortwave heating rate(McICA)',phys_decomp)

   ! !!!!variables for old radiation scheme -->

      call addfld ('FSUTOA  ','W/m2    ',1, 'A','Mean upward shortwave flux at TOM (McICA)'      ,phys_decomp)
      call addfld ('FSUTOAC ','W/m2    ',1, 'A','Mean clear-sky upward shortwave flux at TOM (McICA)'      ,phys_decomp)
      call addfld ('FLWDS   ','W/m2    ',1, 'A','Mean downward longwave flux at surface (McICA)' ,phys_decomp) 
      call addfld ('FLWDSC  ','W/m2    ',1, 'A','Mean clear-sky downward longwave flux at surface (McICA)' ,phys_decomp) 
      call addfld ('QRLC    ','K/s     ',pver, 'A','clear-sky longwave heating rate (McICA)',phys_decomp)
      call addfld ('QRSC    ','K/s     ',pver, 'A','clear-sky shortwave heating rate(McICA)',phys_decomp)


    !!!!!generated cloud fractions -->

      call addfld ('GENCFTOT','        ',1   , 'A','Mean total cloud fraction from MSC cld. gen. (GEN)',phys_decomp)
      call addfld ('GENCFLOW','        ',1   , 'A','Mean low cloud fraction from MSC cld. gen. (GEN)',phys_decomp)
      call addfld ('GENCFMED','        ',1   , 'A','Mean middle cloud fraction from MSC cld. gen. (GEN)',phys_decomp)
      call addfld ('GENCFHGH','        ',1   , 'A','Mean high cloud fraction from MSC cld. gen. (GEN)',phys_decomp)
      call addfld ('GENCFLAY','        ',pver, 'A','Mean layer cloud fraction from MSC cld. gen. (GEN)',phys_decomp)


    !!!!!! trace gases-->
      call addfld ('N2OMMR ','         ',pver, 'A','N2O mass mixing ratio',phys_decomp)
      call addfld ('CH4MMR ','         ',pver, 'A','CH4 mass mixing ratio',phys_decomp)
      call addfld ('CF11MMR','         ',pver, 'A','CFC11 mass mixing ratio',phys_decomp)
      call addfld ('CF12MMR','         ',pver, 'A','CFC12 mass mixing ratio',phys_decomp)

   ! aerosol optical depth-->
     call addfld ('TAUBC','        ',1   , 'A','AOD of BC',phys_decomp)
     call addfld ('TAUOC','        ',1   , 'A','AOD of OC',phys_decomp)
     call addfld ('TAUDST','       ',1   , 'A','AOD of Dust',phys_decomp)
     call addfld ('TAUSSLT','      ',1   , 'A','AOD of sea salt',phys_decomp)
     call addfld ('TAUSUL','       ',1   , 'A','AOD of sulfate',phys_decomp)

     call add_default('TAUBC', 1, ' ')
     call add_default('TAUOC', 1, ' ')
     call add_default('TAUDST', 1, ' ')
     call add_default('TAUSSLT', 1, ' ')
     call add_default('TAUSUL', 1, ' ')
 
     call addfld ('TAUTOT  ','K/s     ',pver, 'A','clear-sky longwave heating rate (McICA)',phys_decomp)
     call addfld ('WATOT   ','K/s     ',pver, 'A','clear-sky longwave heating rate (McICA)',phys_decomp)
     call addfld ('GATOT   ','K/s     ',pver, 'A','clear-sky longwave heating rate (McICA)',phys_decomp)
     call addfld ('FATOT   ','K/s     ',pver, 'A','clear-sky longwave heating rate (McICA)',phys_decomp)
     call add_default('TAUTOT', 1, ' ')
     call add_default('WATOT', 1, ' ')
     call add_default('GATOT', 1, ' ')
     call add_default('FATOT', 1, ' ')
 RETURN
    END SUBROUTINE BCC_RAD_init
  END MODULE BCC_RAD_init_mod
