! --------------------------------------------------
MODULE CONVERT_O3_O2_MOD
! --------------------------------------------------

  IMPLICIT NONE

CONTAINS

! --------------------------------------------------
  SUBROUTINE convert_o3_o2(t,p,interp_p,interp_z,old_o3,airdens,new_o3,o2)
! --------------------------------------------------

! --------------------------------------------------
! Compute vertical profiles of number density of air and
! of ozone.
! --------------------------------------------------

    USE shr_kind_mod,only: r8 => shr_kind_r8
    USE ppgrid

    IMPLICIT NONE
    INCLUDE 'rad_PSU.inc'
    
!
! INPUT DATA
!
    REAL(r8), INTENT(IN) :: &
    t(1,2,NZP2)              ! Clear and cloudy temperature profiles, mid-level  (K) 
    
    REAL(r8), INTENT(IN) ::   &
    p(1,2,NZP2),              & ! Clear and cloudy pressure profiles, mid-level     (hPa)
    interp_p(1,2,NZP2+1),     & ! Clear and cloudy pressure profiles, full-level    (hPa)
    interp_z(NZP2+1),         & ! Heights at full-level                             (m)
    old_o3(NZP2),             & ! Vertical ozone profile                            (g/g)
    airdens(NZP2)               ! Vertical profile of air density                   (kgm^-3) 

!
! OUTPUT DATA
!

    REAL(r8), INTENT(OUT) ::  & 
    new_o3(NZP2),             & ! Ozone number concentration profile (#m^-3)
    o2(NZP2)                    ! Air number concentration profile   (#m^-3)

!
! LOCAL VARIABLES
!
    INTEGER :: &
    kk            ! Z direction layer number

    REAL(r8) ::    &
    dz,            & ! Vertical layer thickness  (m)
    massdensity,   & ! Mass density of layer     (gm^-3)
    airdensity,    & ! Number density of air     (#m^-3)
    numberdensity    ! Number density ozone      (?)
    
    DO kk = 1,NZP2

! Compute number density of o2.

       o2(kk) = O2_MIXING_RATIO*airdens(kk)
       
       dz = interp_z(kk+1)-interp_z(kk)

! Mass density in g per meter^3.

       massdensity = 100.0_r8*(interp_p(1,1,kk)-interp_p(1,1,kk+1))/G
       massdensity = massdensity * (1.0_r8/dz)
       massdensity = massdensity * (1.0E+03)

! Number density in Particles per meter^3.

       airdensity = (massdensity/GAIR)*A0

! Ozone number density.

       numberdensity = ((GAIR/GOZONE)*airdensity*old_o3(kk))
       new_o3(kk) = (((numberdensity*1.0E-06)*LOSCMHIDT_VOLUME) / A0)
    END DO

    RETURN
  END SUBROUTINE convert_o3_o2
  
END MODULE CONVERT_O3_O2_MOD
