! --------------------------------------------------------------------
MODULE McICA_CLD_DIAG_MOD
! --------------------------------------------------------------------

  IMPLICIT NONE
  
CONTAINS
  
  SUBROUTINE McICA_CLD_DIAG(qc, qi, pint_in, tot_cf, low_cf, mid_cf, hgh_cf, lay_cf) !lay_cf added by JXW

! --------------------------------------------------------------------
! Compute the vertically projected cloud fraction as computed by the 
! stochastic cloud generator.
! --------------------------------------------------------------------
    
    USE shr_kind_mod,  ONLY: r8=>shr_kind_r8
    USE ppgrid,        ONLY: pcols, pver
    USE rad_PSU_dims_include_mod ! McICA dimensions 
       
! INPUT DATA
    REAL(r8), INTENT(IN) :: &
    qc(NX,NY,NZ),           & ! Cloud water
    qi(NX,NY,NZ),           & ! Cloud ice
    pint_in(NZ)               ! Pressure at interfaces (top to bottom)

! OUTPUT DATA
    REAL(r8), INTENT(OUT) :: &
    tot_cf,                  & ! Total vertically projected cloud fraction
    low_cf,                  & ! Vertically projected low cloud fraction
    mid_cf,                  & ! Vertically projected mid-level cloud fraction
    hgh_cf,                  & ! Vertically projected high cloud fraction  
    lay_cf(NZ)                 ! cloud fraction at each layer 

   
! LOCAL DATA
    
    REAL(r8), PARAMETER :: &
    low_cld_pint = 68000.0,  & ! Cloud at pressures greater than this are low clouds
    hgh_cld_pint = 44000.0     ! Cloud at pressures less than this are high clouds

    REAL(r8) ::       &
    pint(NZ),         & ! Pressure at interfaces (bottom to top)
    count_tot(NX,NY), & ! Total vertically projected cloud field
    count_low(NX,NY), & ! Vertically projected low cloud field 
    count_mid(NX,NY), & ! Vertically projected mid-level cloud field
    count_hgh(NX,NY)    ! Vertically projected high cloud field
   
    REAL(r8) ::   count_lay(NX,NY,NZ)  ! layer cloud field   ! added by JXW, 2010.12.21

    INTEGER :: &
    ix,        &
    iy,        &
    iz
    
    count_tot = 0.0
    count_low = 0.0
    count_mid = 0.0
    count_hgh = 0.0
    count_lay(:,:,:) = 0.0
    lay_cf(:) = 0.0

! Flip the interface pressure field
   
    pint(1:NZ) = pint_in(NZ:1:-1)

! Project the cloud field    
    DO iz = 1, NZ
       DO iy = 1, NY
          DO ix = 1, NX
             IF ((qc(ix,iy,iz)+qi(ix,iy,iz)) .GT. 0.0) THEN ! Cloudy cell
                IF (pint(iz) .GE. low_cld_pint) THEN ! Low cloud
                   count_low(ix,iy) = 1.0
                   count_tot(ix,iy) = 1.0
                END IF

                IF (pint(iz) .LE. hgh_cld_pint) THEN ! High cloud
                   count_hgh(ix,iy) = 1.0
                   count_tot(ix,iy) = 1.0
                END IF

                IF (pint(iz) .GT. hgh_cld_pint .AND. pint(iz) .LT. low_cld_pint) THEN ! Middle cloud
                   count_mid(ix,iy) = 1.0
                   count_tot(ix,iy) = 1.0
                END IF
                
                count_lay(ix,iy,iz) = 1.0   ! added by JXW,2010.12.21

             END IF
          END DO ! ix
       END DO ! iy
    END DO ! iz

! Compute total cloud fraction
    tot_cf = SUM(count_tot)/REAL(NX*NY,r8)
    low_cf = SUM(count_low)/REAL(NX*NY,r8)
    mid_cf = SUM(count_mid)/REAL(NX*NY,r8)
    hgh_cf = SUM(count_hgh)/REAL(NX*NY,r8)

! added by JXW, 2010.12.21 -->
    do iz =1,nz
      lay_cf(nz-iz+1) = SUM(count_lay(:,:,iz))/REAL(NX*NY,r8)
   !  if (lay_cf(nz-1+iz)>1.0) write(6,*) 'lay_cf=',lay_cf(nz-1+iz)
    end do

    RETURN

  END SUBROUTINE McICA_CLD_DIAG

END MODULE McICA_CLD_DIAG_MOD
