!--------------------------------------------------
MODULE RAD_DRIVER_McICA_ZHANG_MOD
! --------------------------------------------------

  IMPLICIT NONE
  
CONTAINS

! --------------------------------------------------
  SUBROUTINE RAD_DRIVER_McICA_ZHANG(                                                          &
                          ! input
                              tabs_in, tabs_s, qv_in, rh, pmid_GCM, pint_GCM, pdel_GCM, pres_s,   &
                              o3, zmid_GCM, zint_GCM,           &    
                              qc_in, qi_in, smu, eccf,          &
                              n2o,  ch4,  cfc11, cfc12,         &
                              ocnfrac, landfrac, icefrac, rel_GCM, rei_GCM,             &
                              aldir, aldif, asdir, asdif,                               &
                              tauaer_in,waaer_in,gaaer_in,faaer_in,                     &
                          ! output 
                              qrs_crm, qrl_crm, qrs_clr_crm, qrl_clr_crm,               &
                              flwds_crm, flwdsc_crm, fsns_crm, fsnt_crm, flns_crm,      &
                              flnt_crm, flntc_crm, sols_crm, soll_crm, solsd_crm,       &
                              solld_crm, fsntoa_crm, fsntoac_crm, fsds_crm, fsdsc_crm,  &
                              fsntc_crm, fsutoa_crm, fsutoac_crm, fsnsc_crm, flut_crm,  &
                              flutc_crm, flnsc_crm, solin_crm,                          &
                          ! input
                              lat, clat)      !by JXW,2010.08.03

! --------------------------------------------------

! --------------------------------------------------
! This subroutine computes longwave and shortwave fluxes and heating rates
! --------------------------------------------------

    USE AVERAGE_FIELDS_MOD
    USE CONVERT_O3_O2_MOD
    USE COMPUTE_EXTRA_LAYERS_ZHANG_MOD
   ! USE COMPUTE_SPECTRAL_MODEL_IND_MOD
    USE shr_kind_mod,only: r8 => shr_kind_r8
    USE shr_orb_mod
    USE time_manager, only: get_curr_calday
    USE ppgrid
    USE physconst, only: cpair,gravit,mwdry,mwco2,mwch4,mwn2o,mwf11,mwf12,mwh2o  
    USE ghg_surfvals, only: co2vmr
    USE abortutils, only: endrun

!---add by JXW for Monte Carlo sample--------- 
    USE RAN_NUMBERS_MOD

!---add by JXW for subroutines----------
    USE RAD_SUBROUTINES_ZHANG_MOD
    USE RAD_SPECSAMPL_ZHANG_MOD          ! Number of samples to take when using spectral sampling
  
    IMPLICIT NONE
    INCLUDE 'paras.inc'   ! add by JXW.
    INCLUDE 'rad_PSU.inc' ! add by JXW
    INCLUDE 'comsol.h'

!
! INPUT DATA
!
  
    REAL(r8), INTENT(IN) ::  &
    tabs_in(NX,NY,pver),       & ! Domain temperature                         (K)
    tabs_s,                  & ! Surface temperature                        (K) 
    qv_in(NX,NY,pver),         & ! Domain water vapour                        (g/kg)
    pmid_GCM(pver),            & ! Domain pressure at midpoints               (Pa)
    pint_GCM(pver+1),          & ! Domain pressure at interfaces              (Pa)
    pdel_GCM(pver),            & ! Layer pressure thickness                   (Pa)
    pres_s,                  & ! Surface pressure                           (Pa)
    zmid_GCM(pver),            & ! Mid-level height                           (m)
    zint_GCM(pver+1),          & ! Interface-level height                     (m)
    o3(pver),                  & ! Vertical ozone volumn mixing ratio                   
    smu,                     & ! Cosine of the solar zenith angle
    eccf                       ! Earth orbit eccentricity factor

    REAL(r8)  ::  smu2          ! Cosine of the solar zenith angle (zero out negetive)
    
    REAL(r8), INTENT(IN) ::  &
    qc_in(NX,NY,pver),         & ! Domain LWC                             (kg/kg)
    qi_in(NX,NY,pver)            ! Domain IWC                             (kg/kg)

    
    REAL(r8), INTENT(IN) ::  &
    ocnfrac,                 & ! Fraction of gridbox that is ocean
    landfrac,                & ! Fraction of gridbox that is land
    icefrac                    ! Fraction of gridbox that is ice

    REAL(r8), INTENT(IN) ::  &
    rel_GCM(pver),             & ! Effective radius of liquid drops from GCM parametrization  (microns)
    rei_GCM(pver)                ! Effective radius of ice particles from GCM parametrization (microns)

    REAL(r8), INTENT(IN) ::  &
    aldir,                   & ! Direct surface SW albedo  (Near IR)
    aldif,                   & ! Diffuse surface SW albedo (Near IR)
    asdir,                   & ! Direct surface SW albedo  (Vis)
    asdif                      ! Diffuse surface SW albedo (Vis)

    REAL(r8),INTENT(in) :: rh(pver)       ! relative humidity

    INTEGER, INTENT(in) :: lat
    
    REAL(r8), INTENT(in) :: clat  ! current latitude (radians)

    REAL(r8) :: dlat ! current latitude (degrees)

! trace gases from GCM, JXW 2010.09.13
    REAL(r8), INTENT(IN) :: &
    n2o(pver),                &  !n2o mass mixing ration
    ch4(pver),                &  !ch4 mass mixing ration
    cfc11(pver),              &  !cfc11 mass mixing ration
    cfc12(pver)                  !cfc12 mass mixing ration

! aerosol optical properties from CUACE, JXW 2011.07.18
    REAL(r8), INTENT(IN) ::  &
    tauaer_in(0:pver,nbnd),  &
    waaer_in(0:pver,nbnd),   &
    gaaer_in(0:pver,nbnd),   &
    faaer_in(0:pver,nbnd)

!
! OUTPUT DATA
!

    REAL(r8), INTENT(OUT) :: &
    qrs_crm(pver),             & ! Shortwave heating rate                 (Ks^-1)
    qrl_crm(pver),             & ! Longwave  heating rate                 (Ks^-1)
    qrs_clr_crm(pver),         & ! Shortwave heating rate                 (Ks^-1)
    qrl_clr_crm(pver),         & ! Shortwave heating rate                 (Ks^-1)
    flwds_crm,               & ! Surface longwave down flux             (Wm^-2)
    flwdsc_crm,              & ! Clear-sky surface longwave down flux   (Wm^-2)
    fsns_crm,                & ! Surface solar absorbed flux            (Wm^-2)
    fsnt_crm,                & ! Net column abs solar flux at model top (Wm^-2)
    flns_crm,                & ! Srf longwave cooling (up-down) flux    (Wm^-2)
    flnt_crm,                & ! Net outgoing lw flux at model top      (Wm^-2)
    sols_crm,                & ! Direct beam solar rad. onto srf (sw)   (Wm^-2)
    soll_crm,                & ! Direct beam solar rad. onto srf (lw)   (Wm^-2)
    solsd_crm,               & ! Diffuse solar radiation onto srf (sw)  (Wm^-2)
    solld_crm,               & ! Diffuse solar radiation onto srf (lw)  (Wm^-2)
    fsntoa_crm,              & ! Net solar flux at TOA                  (Wm^-2)
    fsntoac_crm,             & ! Clear sky net solar flux at TOA        (Wm^-2)
    fsds_crm,                & ! Flux shortwave downwelling surface     (Wm^-2)
    fsdsc_crm,               & ! Clear sky flux SW Downwelling Surface  (Wm^-2)
    fsntc_crm,               & ! Clear sky total column abs solar flux  (Wm^-2)
    fsutoa_crm,              & ! Flux Shortwave Upwelling TOA           (Wm^-2)
    fsutoac_crm,             & ! Clear sky Flux Shortwave Upwelling TOA (Wm^-2)
    fsnsc_crm,               & ! Clear sky surface abs solar flux       (Wm^-2)
    flut_crm,                & ! Upward flux at top of model            (Wm^-2)
    flutc_crm,               & ! Upward clear sky flux at top of model  (Wm^-2)
    flntc_crm,               & ! Clear sky lw flux at model top         (Wm^-2)
    flnsc_crm,               & ! Clear sky lw flux at srf (up-down)     (Wm^-2)
    solin_crm                  ! Solar incident flux                    (Wm^-2)

!
! LOCAL VARIABLES
!
    ! note: NZP2=pver+3
    REAL(r8) ::            &
    ave_p(1,2,NZP2),       & ! Clear and cloudy pressure profiles, mid-level                   (hPa)
    interp_p(1,2,NZP2+1),  & ! Clear and cloudy pressure profiles, full-level                  (hPa)
    interp_z(NZP2+1),      & ! Heights at full-level                                           (m)
    ave_t(1,2,NZP2),       & ! Clear and cloudy temperature profiles, mid-level                (K)
    interp_t(1,2,NZP2+1),  & ! Clear and cloudy temperature profiles, full-level               (K)
    ave_qv(1,2,NZP2),      & ! Clear and cloudy water vapour mixing ratio profiles, mid-level  (g/kg)
    o3_temp(NZP2),         & ! Ozone mixing ratio profile                                      (g/g)
    airdens(NZP2),         & ! Density of air                                                  (#m^-3) 
    airdensity               ! Density of air                                                  (kgm^-3)
    

    REAL(r8) ::     &
    num_conc_liq,   & ! Cube root of liquid particle num. conc (g/m^3)^0.333
    num_conc_ice      ! Cube root of ice particle num. conc    (g/m^3)^0.333

    REAL(r8) ::     &
    new_o3(NZP2),   & ! o3 volumn mix ratio (ppmv)
    new_o2(NZP2)      ! O2 volumn mix ratio (ppmv)

    INTEGER ::            &
    cloud_index(NX,NY),   &   ! Clear and cloudy columns
    ii,                   &   ! Counter
    jj,                   &   ! Counter
    ix,                   &   ! Counter
    iy,                   &   ! Counter
    iz                        ! Counter

    INTEGER ::  ib,ic,k,imol,iw,ig,isampl,nsampl,ipcl,l,nplk1,ia,kk  ! Counter.     add by JXW

    REAL(r8) ::           &            
    pmid(pver),             & ! Domain pressure at midpoints    (hPa)
    pint(pver+1),           & ! Domain pressure at interfaces   (hPa)
    pdel(pver),             & ! Layer pressure thickness        (hPa)
    zdel(pver),             &
    zmid(pver),             & ! Mid-level height                (m)
    zint(pver+1),           & ! Interface-level height          (m)
    qv(NX,NY,pver),         & ! subcolumn water vapor
    tabs(NX,NY,pver)          

    REAL(r8)  ::          &
    operat1(pver),          &
    operat2(pver) 

    REAL(r8)  ::          &
    qc(NX,NY,NZP2),       & ! subcolumn LWC  (g/m^3)
    qi(NX,NY,NZP2)          ! subcolumn IWC  (g/m^3)

    REAL(r8) ::           &
    rel(NZP2),            & ! Effective radius of liquid drops used locally  (microns)
    rei(NZP2)               ! Effective radius of ice particles used locally (microns)

    REAL(r8) ::           &
    rei_j(nx,ny,NZP2),    & ! Effective radius of ice particles defpending on IWC and T (microns), JXW
    B                       ! intermediate from IWC AND T to REI_J, JXW
!
! McICA local variables
! -->
    INTEGER  ::           &
    cfint 
    REAL(r8) ::           & 
    cf                     ! Domain averaged cloud fraction
	
    REAL(r8) ::           &
    cloud_frac(nx,ny,NZP2)          ! cell cloud fraction

    INTEGER ::            &
    cld_x(NX),            & ! Index of cloudy columns in the x direction
    cld_y(NY),            & ! Index of cloudy columns in the y direction
    cld_nx,               & ! Number of cloudy columns in the x direction
    cld_ny,               & ! Number of cloudy columns in the y direction
    flag,                 & ! Cloudy flag
    counter                 ! Counter

    REAL(r8) ::           &
    sols,                 & ! Direct beam solar rad. onto srf (sw)   (Wm^-2)
    soll,                 & ! Direct beam solar rad. onto srf (lw)   (Wm^-2)
    solsd,                & ! Diffuse solar radiation onto srf (sw)  (Wm^-2)
    solld                   ! Diffuse solar radiation onto srf (lw)  (Wm^-2)

    REAL(r8) ::           &
    sols_clr_tem,         & ! Direct beam solar rad. onto srf (sw) (clear sky)  (Wm^-2)
    soll_clr_tem,         & ! Direct beam solar rad. onto srf (lw) (clear sky)  (Wm^-2)
    solsd_clr_tem,        & ! Diffuse solar radiation onto srf (sw) (clear sky) (Wm^-2)
    solld_clr_tem           ! Diffuse solar radiation onto srf (lw) (clear sky) (Wm^-2)

    REAL(r8) ::           &
    sols_cld_tem,         & ! Direct beam solar rad. onto srf (sw) (cloudy sky)  (Wm^-2)
    soll_cld_tem,         & ! Direct beam solar rad. onto srf (lw) (cloudy sky)  (Wm^-2)
    solsd_cld_tem,        & ! Diffuse solar radiation onto srf (sw) (cloudy sky) (Wm^-2)
    solld_cld_tem           ! Diffuse solar radiation onto srf (lw) (cloudy sky) (Wm^-2)

    REAL(r8) ::           &
    sols_clr,             & ! Direct beam solar rad. onto srf (sw) (clear sky)  (Wm^-2)
    soll_clr,             & ! Direct beam solar rad. onto srf (lw) (clear sky)  (Wm^-2)
    solsd_clr,            & ! Diffuse solar radiation onto srf (sw) (clear sky) (Wm^-2)
    solld_clr               ! Diffuse solar radiation onto srf (lw) (clear sky) (Wm^-2)

    REAL(r8) ::           &
    sols_cld,             & ! Direct beam solar rad. onto srf (sw) (cloudy sky)  (Wm^-2)
    soll_cld,             & ! Direct beam solar rad. onto srf (lw) (cloudy sky)  (Wm^-2)
    solsd_cld,            & ! Diffuse solar radiation onto srf (sw) (cloudy sky) (Wm^-2)
    solld_cld               ! Diffuse solar radiation onto srf (lw) (cloudy sky) (Wm^-2)

    REAL(r8) ::           &
    fddir,                &
    fddif

    REAL(r8) ::                 &
    radswup_McICA(NZP2+1),      & ! Upward SW flux (McICA)                  (Wm^-2)
    radswdn_McICA(NZP2+1),      & ! Downward SW flux (McICA)                (Wm^-2)
    radlwup_McICA(NZP2+1),      & ! Upward LW flux (McICA)                  (Wm^-2)
    radlwdn_McICA(NZP2+1),      & ! Downward LW flux (McICA)                (Wm^-2)
    radqrsw_McICA(NZP2),        & ! SW heating rate (McICA)                 (Ks^-1)
    radqrlw_McICA(NZP2),        & ! LW heating rate (McICA)                 (Ks^-1)
    radqrsw_McICA_clr(NZP2),    & ! SW heating rate (McICA)                 (Ks^-1)
    radqrlw_McICA_clr(NZP2)       ! LW heating rate (McICA)                 (Ks^-1)

    REAL(r8) ::                  &
    radswup_McICA_cld(NZP2+1),   & ! Upward SW flux cloudy (McICA)        (Wm^-2)
    radswdn_McICA_cld(NZP2+1),   & ! Downward SW flux cloudy (McICA)      (Wm^-2)
    radlwup_McICA_cld(NZP2+1),   & ! Upward LW flux cloudy (McICA)        (Wm^-2)
    radlwdn_McICA_cld(NZP2+1),   & ! Downward LW flux cloudy (McICA)      (Wm^-2)
    radswup_McICA_clr(NZP2+1),   & ! Upward SW flux clear (McICA)         (Wm^-2)
    radswdn_McICA_clr(NZP2+1),   & ! Downward SW flux clear (McICA)       (Wm^-2)
    radlwup_McICA_clr(NZP2+1),   & ! Upward LW flux clear (McICA)         (Wm^-2)
    radlwdn_McICA_clr(NZP2+1)      ! Downward LW flux clear (McICA)       (Wm^-2)

   ! qrad_McICA_cld(NX,NY,NZ),    & ! Net radiative heating cloudy (McICA) (Ks^-1)
   ! radqrsw_McICA_cld(NZP2),     & ! SW heating rate cloudy (McICA)       (Ks^-1)
   ! radqrlw_McICA_cld(NZP2)        ! LW heating rate cloudy (McICA)       (Ks^-1)

!
! radiation local variables
! -->

! coordinate related
      REAL(r8) ::   new_pdel(1,2,NZP2) ! pressure depth of each layer
      REAL(r8) ::   new_zdel(NZP2)     ! z depth of each layer

! atmosphere related
      REAL(r8) ::   new_rh(NZP2)       ! relative humidity 
      REAL(r8) ::   rhoair(NZP2)       ! satuated relative humidity
! ground related 
      REAL(r8) ::   sr(nbnd,nsfc) ! surface properties
      REAL(r8) ::   galb(2)   ! ground albedo (for clear and cloudy sky)
      INTEGER  ::   idslf(nbnd) ! index of surface type

! band related
      REAL(r8) ::   wv(nbnd1)   ! band boundaries
      REAL(r8) ::   pwgt(nbnd,ng) ! weight of subintervals
      INTEGER  ::   mg(nbnd)    ! number of subinterval

! rayleigh scattering related 
      REAL(r8) ::   ry(nbnd)  ! rayleigh scattering coefficient
      REAL(r8) ::   qmol(nmax2) ! moments

! tau related variables
      REAL(r8) ::   taur               ! Rayleigh scattering optical depth
      REAL(r8) ::   tau_r(NZP2,nmax2,2)     ! transition variable as optic depth, rayleigh
      REAL(r8) ::   tau_aer(NZP2,nmax2,2)   ! transition variable as optic depth, aerosol
      REAL(r8) ::   tau_cld(NZP2,nmax2,2)   ! transition variable as optic depth, aerosol
      REAL(r8) ::   tau_ext(NZP2,2)  ! combinded extinction optic depth
      REAL(r8) ::   tau_ext_clr(NZP2,2)  ! combinded extinction optic depth for clear sky
   !  REAL(r8) ::   tau_col(NX,NY,2)  ! combinded extinction optic depth in a whole sub-colomn
      REAL(r8) ::   tau_tot(NZP2,2)


! phase function related
      REAL(r8) ::   scap(NZP2,2)
      REAL(r8) ::   g_phase(3,NZP2,2)  ! phase function 
      INTEGER  ::   nda2         ! = nda*2
      REAL(r8) ::   omg(NZP2,2)
	        
! plank function related
      REAL(r8) ::   wl
   !  REAL(r8) ::   bbplk                 ! function 
      REAL(r8) ::   aplnk(nbnd,naplk)     
      REAL(r8) ::   bp(NZP2+1)
      REAL(r8) ::   bl(NZP2) 
      REAL(r8) ::   akt(nt,NZP2)   ! p-t table
      REAL(r8) ::   knu(NZP2)
      REAL(r8) ::   sku(NZP2)
      REAL(r8) ::   tkd(NZP2)
      REAL(r8) ::   ddp(NZP2) 
      REAL(r8) ::   dbp1,dbp2,cplk(nplnk,NZP2,2),cplk_clr(nplnk,NZP2,2)
      REAL(r8) ::   bgnd(2),bgnd0


! gas/CKD related 
      REAL(r8) ::   dz(NZP2)   
      REAL(r8) ::   c0          
      REAL(r8) ::   amtpb(nmol,NZP2)     
      REAL(r8) ::   wbrd(NZP2)
      REAL(r8) ::   kpt(nbnd,kabs,np,nt,ng) ! table 
      REAL(r8) ::   kpt_test(nbnd,kabs,np,nt,ng) ! table 
      REAL(r8) ::   slfk(nbnd,np,nt,ng) ! table
      INTEGER  ::   nabs(nbnd)  ! number of absorption gases
      INTEGER  ::   idabs(nbnd,kabs)  ! index of absorption gases at each band
      REAL(r8) ::   cgas(NZP2,nmol)   ! gas concentration  (ppmv)
                    !   1: h2o, 2: co2, 3: o3, 4: n2o, 5: co, 6: ch4, 7: o2	    
      INTEGER  ::   mp(NZP2)      ! the first of the 22 layers in pres table that is under each mid-layer

! particle related 
      REAL(r8) ::   r_aer(NZP2)          ! cloud or aerosol mode radius
      REAL(r8) ::   r0(npcl,kvar)        ! reference mode radius 
      INTEGER  ::   nvar(npcl,2)         ! index for hydrophilic or hydrophobic
      INTEGER  ::   mvar
      REAL(r8) ::   aa,bb,cc
      REAL(r8) ::   IWC
      REAL(r8) ::   cz   
      REAL(r8) ::   q(nbnd,npcl,kvar,nmax2)  ! moments (???)
      REAL(r8) ::   q_interp             ! same as q, but are interpolated from q. 
      REAL(r8) ::   cpcl(NZP2,npcl,3)    ! parameters of particles (i,j,k)
          !             i: layer number (from top to bottom)
          !         j = 1: water cloud               2: ice cloud
          !             3: dust-like                 4: soot
          !             5: volcanic-ash              6: h2so4
          !             7: rural                     8: sea salt
          !             9: urban                    10: tropo.
          !            11: yellow dust
          !         k = 1: concentration in ppmv
          !                = aerosol volume(cm3/cm3)*t*p0/t0/p*1.0e6
          !                  with t0=273, p0=1atm.
          !               type 7,8,9,10 = dry concentration(parapc.voldp)
          !                             = total concentration(parapc.volp)
          !         k = 2: mode radius(cm)
          !         k = 3: undecided for future use.
 
! random number related
      REAL(r8) ::   ran_ix,ran_iy
! two streem related
      REAL(r8) ::   calday
      REAL(r8) ::   fsol(nbnd)  ! incident solar flux
      REAL(r8) ::   fsol_temp(nbnd)  ! incident solar flux, temporary when using expanded shortwave. 
      REAL(r8) ::   flxd(NZP2+1),flxu(NZP2+1),flxdc(NZP2+1),flxuc(NZP2+1)
      REAL(r8) ::   flxd_dir(NZP2+1),flxu_dir(NZP2+1)
      REAL(r8) ::   flxd_dif(NZP2+1),flxu_dif(NZP2+1)
      REAL(r8) ::   mult
! heating rate related variables

      REAL(r8) ::  net_fluxl(NZP2),net_fluxs(NZP2)
      REAL(r8) ::  delt_p(NZP2)
      REAL(r8) ::  amair
      REAL(r8),PARAMETER :: am(1:nmol)=(/18.0e-3,44.0e-3,48.0e-3,44.0e-3,16.0e-3,   &
                                28.0e-3,32.0e-3,1.0,1.0,1.0,1.0/)


   INCLUDE 'input_17b_low.h'   ! no ice optical properties. low spectral resolution
   INCLUDE 'qice_17b.h'        ! only ice optical properties, units is m^2/g
!
! Zero out arrays
! -->
    radswup_McICA(:)      = 0.0_r8
    radswdn_McICA(:)      = 0.0_r8
    radlwup_McICA(:)      = 0.0_r8
    radlwdn_McICA(:)      = 0.0_r8

    radqrlw_McICA(:)      = 0.0_r8
    radqrsw_McICA(:)      = 0.0_r8

    radswup_McICA_cld(:)  = 0.0_r8
    radswdn_McICA_cld(:)  = 0.0_r8  
    radlwup_McICA_cld(:)  = 0.0_r8
    radlwdn_McICA_cld(:)  = 0.0_r8

    radswup_McICA_clr(:)  = 0.0_r8
    radswdn_McICA_clr(:)  = 0.0_r8
    radlwup_McICA_clr(:)  = 0.0_r8
    radlwdn_McICA_clr(:)  = 0.0_r8


    tau_r(:,:,:)           = 0.0_r8 
    tau_aer(:,:,:)         = 0.0_r8 
    tau_cld(:,:,:)         = 0.0_r8 
    tau_ext(:,:)           = 0.0_r8 
    tau_tot(:,:)           = 0.0_r8 
  
    ave_p(:,:,:)           = 0.0_r8
    ave_t(:,:,:)           = 0.0_r8
    new_pdel(:,:,:)        = 0.0_r8
    new_zdel(:)            = 0.0_r8
    interp_p(:,:,:)        = 0.0_r8
    interp_z(:)            = 0.0_r8
    interp_t(:,:,:)        = 0.0_r8
     
    qc(:,:,:)              = 0.0_r8
    qi(:,:,:)              = 0.0_r8
    rel(:)                 = 0.0_r8
    rei(:)                 = 0.0_r8
    new_rh(:)              = 0.0_r8

    o3_temp(:)             = 0.0_r8
    new_o3(:)              = 0.0_r8
    new_o2(:)              = 0.0_r8
    ave_qv(:,:,:)          = 0.0_r8

    sols_cld              = 0.0_r8
    soll_cld              = 0.0_r8
    solsd_cld             = 0.0_r8
    solld_cld             = 0.0_r8

    sols_clr              = 0.0_r8
    soll_clr              = 0.0_r8
    solsd_clr             = 0.0_r8
    solld_clr             = 0.0_r8

    sols                  = 0.0_r8
    soll                  = 0.0_r8
    solsd                 = 0.0_r8
    solld                 = 0.0_r8

! zero out the output arrays 
    qrs_crm(:)         = 0.0_r8       
    qrl_crm(:)         = 0.0_r8
    qrs_clr_crm(:)         = 0.0_r8
    qrl_clr_crm(:)         = 0.0_r8
    flwds_crm          = 0.0_r8
    flwdsc_crm         = 0.0_r8
    fsns_crm           = 0.0_r8
    fsnt_crm           = 0.0_r8
    flns_crm           = 0.0_r8
    flnt_crm           = 0.0_r8
    sols_crm           = 0.0_r8
    soll_crm           = 0.0_r8
    solsd_crm          = 0.0_r8
    solld_crm          = 0.0_r8
    fsntoa_crm         = 0.0_r8
    fsntoac_crm        = 0.0_r8
    fsds_crm           = 0.0_r8
    fsdsc_crm          = 0.0_r8
    fsntc_crm          = 0.0_r8
    fsutoa_crm         = 0.0_r8
    fsutoac_crm        = 0.0_r8
    fsnsc_crm          = 0.0_r8
    flut_crm           = 0.0_r8
    flutc_crm          = 0.0_r8
    flntc_crm          = 0.0_r8
    flnsc_crm          = 0.0_r8
    solin_crm          = 0.0_r8 

!zero out others    
    
    cld_x(:)           = 0.0_r8
    cld_y(:)           = 0.0_r8
    cpcl(:,:,:)        = 0.0_r8
    cgas(:,:)          = 0.0_r8
   ! tabs(:,:,:)        = 0.0_r8

!
! directly assigne variables in base to top order
! -->
    qv(1:nx,1:ny,1:pver) = qv_in(1:nx,1:ny,1:pver)
    qc(1:nx,1:ny,1:pver) = qc_in(1:nx,1:ny,1:pver)
    qi(1:nx,1:ny,1:pver) = qi_in(1:nx,1:ny,1:pver)
    tabs(1:nx,1:ny,1:pver) = tabs_in(1:nx,1:ny,1:pver)
    
!
! Base to top order is needed for computing extra layers, so we flip the opposite order here. 
! -->
    pmid(1:pver)=pmid_GCM(pver:1:-1)/100.0_r8
    pdel(1:pver)=pdel_GCM(pver:1:-1)/100.0_r8
    zmid(1:pver)=zmid_GCM(pver:1:-1)
    rel(1:pver)=rel_GCM(pver:1:-1)/10000.0_r8
    rei(1:pver)=rei_GCM(pver:1:-1)/10000.0_r8
    new_rh(1:pver)=rh(pver:1:-1)
    new_o3(1:pver)=o3(pver:1:-1)*1000000.0_r8 

    pint(1:pver+1)=pint_GCM(pver+1:1:-1)/100.0_r8
    zint(1:pver+1)=zint_GCM(pver+1:1:-1)
    do iz=1,pver
      zdel(iz)=abs(zint(iz)-zint(iz+1))
    end do

! Find cloudy sells and cloudy columns  by jxw
    where (qc(:,:,:)<0.) qc=0.0_r8
    where (qi(:,:,:)<0.) qi=0.0_r8
   
 !cloud_index(:,:)=0
    cloud_frac(:,:,:)=0.0_r8
    where (qc(:,:,:)+qi(:,:,:) .gt. 0.0) cloud_frac =1.0_r8
  !  DO ix = 1, NX
  !     DO iy = 1, NY
  !        DO iz = 1, NZP2
  !           IF ((qc(ix,iy,iz)+qi(ix,iy,iz)) .GT. 0.0) THEN ! Cloudy cell
  !             cloud_frac(ix,iy,iz)=1.0_r8
  !           END IF
  !        END DO ! ix
  !     END DO ! iy
  !  END DO ! iz

! Find cloudy columns, by JXW, 2010.07.16
    cloud_index(:,:)=0
    DO ix=1,NX
      DO iy=1,NY
       ! DO iz=1,pver
       !   if (cloud_frac(ix,iy,iz)==1.0)  then
       !      cloud_index(ix,iy)=1
       !      exit
       !   else
       !      cloud_index(ix,iy)=0
       !   endif
       ! ENDDO
        if (any(cloud_frac(ix,iy,1:pver)==1.0)) cloud_index(ix,iy)=1
      ENDDO
    ENDDO

! Find cloudy columns and compute radiation cloud fraction

  ! X direction
    
    cld_nx = 0
    counter = 1
    DO ix = 1,NX
       flag = 0
       DO iy = 1,NY
          IF (cloud_index(ix,iy) .GT. 0) THEN
             flag = 1
	     exit
          END IF
       END DO ! iy
       IF (flag .EQ. 1) THEN
          cld_x(counter) = ix
          cld_nx         = cld_nx + 1
          counter        = counter + 1
          flag           = 0
       END IF
    END DO ! ix
    
  ! Y direction

    cld_ny = 0
    counter = 1
    DO iy = 1,NY
       flag = 0
       DO ix = 1,NX
          IF (cloud_index(ix,iy) .GT. 0) THEN
             flag = 1
	  exit
          END IF
       END DO ! ix
       IF (flag .EQ. 1) THEN
          cld_y(counter) = iy
          cld_ny         = cld_ny + 1
          counter        = counter + 1
          flag           = 0
       END IF
    END DO ! iy

   cfint=sum(cloud_index(:,:))
   cf=REAL(cfint,r8)/REAL(NX*NY,r8)

! Average fields.  To keep things speedy we average the temperature,
! and water vapour fields.  We have one average profile for cloudy 
! columns and one for clear columns.

! First convert the liquid and ice cloud fields from kg/kg to g/m^3
     qc(:,:,:)=qc(:,:,:)*1000.0_r8  !kg/kg->g/kg
     qi(:,:,:)=qi(:,:,:)*1000.0_r8  !kg/kg->g/kg

     operat1(1:pver)=pdel(1:pver)/(zdel(1:pver)*pmid(1:pver))*p0*3.73776e-4
     operat2(1:pver)=pdel(1:pver)*100.0/9.8
   
    !DO iz = 1,pver
       DO iy = 1,NY
          DO ix = 1,NX

! Make sure that the liquid and ice water contents are positive or zero
           
           !  IF (qi(ix,iy,iz) .LT. 0.0_r8) THEN              
           !     qi(ix,iy,iz) = 0.0_r8
           !  ELSE
           !     qi(ix,iy,iz) = qi(ix,iy,iz)*1000.0_r8 ! kg/kg => g/kg
           !  ENDIF
           !  
           !  IF (qc(ix,iy,iz) .LT. 0.0_r8) THEN
           !     qc(ix,iy,iz) = 0.0_r8
           !  ELSE
           !     qc(ix,iy,iz) = qc(ix,iy,iz)*1000.0_r8 ! kg/kg => g/kg
           !  ENDIF
  ! Bellow are two methods to translate kg/kg to ppmv. As has been tested, qc is almost identical,
  ! while qi from the simple way is systematically larger than that generated from the complex way.          
   !### a simple way to translate kg/kg to ppmv:
       ! always set qc to ppmv
          ! qc(ix,iy,iz) = qc(ix,iy,iz)*pdel(iz)*100.0/9.8/abs((zint(iz)-zint(iz+1)))  &
          !          *tabs(ix,iy,iz)*p0/(273.0*pmid(iz)*100.0)  
           qc(ix,iy,1:pver) = qc(ix,iy,1:pver)*tabs(ix,iy,1:pver)*operat1(1:pver)
       ! ## set qi to ppmv when input.h is used
       !    qi(ix,iy,iz) = qi(ix,iy,iz)*pdel(iz)*100.0/9.8/abs((zint(iz)-zint(iz+1)))/0.917  &
       !                   *tabs(ix,iy,iz)*p0/(273.0*pmid(iz)*100.0)  
       ! ## set qi to g/m^2  when input_jxw.h and q_jxw.h is used
          ! qi(ix,iy,iz) = qi(ix,iy,iz)*pdel(iz)*100.0/9.8
           qi(ix,iy,1:pver) = qi(ix,iy,1:pver)*operat2

   !### a complex way to translate kg/kg to ppmv: 
       !    airdensity  = (pmid(iz)*100.0/tabs(ix,iy,iz))/(1000.0_r8*RGAS/GAIR) ! PV=(m/M)*RT (M: molecular weight)
       !      qc(ix,iy,iz) = qc(ix,iy,iz)*airdensity ! g/kg => g/m^3
       !      qi(ix,iy,iz) = qi(ix,iy,iz)*airdensity ! g/kg => g/m^3  
       !      qc(ix,iy,iz)=(qc(ix,iy,iz)*1.0E-3/1000.0*1.0E+6)    &
       !                   *tabs(ix,iy,iz)*p0/(273.0*pmid(iz)*100.0)    ! water density=1000.0kg/m^3
       !      qi(ix,iy,iz)=(qi(ix,iy,iz)*1.0E-3/917.0*1.0E+6)  &
       !                   *tabs(ix,iy,iz)*p0/(273.0*pmid(iz)*100.0)    ! ice density  =917.0kg/m^3
          END DO ! ix
       END DO ! iy
    !END DO ! iz

! compute REI_J depending on IWC and T, JXW, according to Wyser (1998)
  !!!!!! IWC = qi(:,:,:)/abs(zint(iz)-zint(iz+1)
  !!!!!! IWC0 = 50.0 (g/m^3)
   rei_j(:,:,:) = 0.0_r8
   do ix=1,nx
     do iy=1,ny
       do iz=1,pver
        if (qi(ix,iy,iz)>0.0e-10) then
         B = -2.0 + 10.0**(-3.0)*(273.0-tabs(ix,iy,iz))**1.5*log10(qi(ix,iy,iz)/abs(zint(iz)-zint(iz+1))/50.0)
         rei_j(ix,iy,iz)=377.4+203.3*B+37.91*B**2.0+2.3696*B**3.0
        end if
      ! note: re ice should not be too large or too small (1-80 micron)
         rei_j(ix,iy,iz)=max(rei_j(ix,iy,iz),1.0_r8)
         rei_j(ix,iy,iz)=min(rei_j(ix,iy,iz),80.0_r8)
        !end if
       end do ! iz
     end do ! iy
   end do ! ix

! compute REI_J depending on IWC , JXW, according to Liou et al.(2008)
! if (1==2) then
!   if (lat>=22 .and. lat<=43) then  ! tropical
!       aa=5.4199_r8
!       bb=0.35211_r8
!       cc=0.012680_r8
!
!      do ix=1,nx
!        do iy=1,ny
!          do iz=1,pver 
!            IWC=qi(ix,iy,iz)/abs(zint(iz)-zint(iz+1))
!            rei_j(ix,iy,iz)= exp(aa+bb*log(IWC)+cc*(log(IWC))**2)*0.5_r8
!          end do
!        end do
!      end do
!
!   else if (lat<=10 .or. lat >=55)  then  !high latitude
!       aa=4.8510_r8
!       bb=0.33159_r8
!       cc=0.026189_r8
!
!      do ix=1,nx
!        do iy=1,ny
!          do iz=1,pver
!            IWC=qi(ix,iy,iz)/abs(zint(iz)-zint(iz+1))
!            rei_j(ix,iy,iz)= exp(aa+bb*log(IWC)+cc*(log(IWC))**2)*0.5_r8
!          end do
!        end do
!      end do
!
!   else 
!
!      do ix=1,nx
!        do iy=1,ny
!          do iz=1,pver 
!            if (tabs(ix,iy,iz)>233.0) then 
!              aa=5.2375_r8
!              bb=0.13142_r8
!              cc=0.0_r8
!              IWC=qi(ix,iy,iz)/abs(zint(iz)-zint(iz+1))
!              rei_j(ix,iy,iz)= exp(aa+bb*log(IWC)+cc*(log(IWC))**2)*0.5_r8
!            else 
!              aa=4.3257_r8
!              bb=0.26535_r8
!              cc=0.021864_r8
!              IWC=qi(ix,iy,iz)/abs(zint(iz)-zint(iz+1))
!              rei_j(ix,iy,iz)= exp(aa+bb*log(IWC)+cc*(log(IWC))**2)*0.5_r8
!            end if
!          end do
!        end do
!      end do
!   end if 
!
!end if  !(1==2)



! Compute average pressure, temperature and water vapour for clear and cloudy column
      
  CALL average_fields(tabs, qv(:,:,1:pver), qc(:,:,1:pver), qi(:,:,1:pver), ave_t, ave_qv, cloud_index)

    ave_p(1,1,1:pver)=pmid(1:pver)
    ave_p(1,2,1:pver)=pmid(1:pver)
   ! DO jj = 1,2
   !    DO ii = 1,1
   !       ave_p(ii,jj,1:pver) = pmid(1:pver)
   !    END DO
   ! END DO

! Interpolate height, pressures and temperatures to full levels.
   interp_z(1:pver+1)=zint(1:pver+1)
   interp_p(1,1,1:pver+1)=pint(1:pver+1)
   interp_p(1,2,1:pver+1)=pint(1:pver+1)

      DO kk=2,pver+1           
        interp_t(1,1:2,kk)=(tabs(1,1,kk-1)+tabs(1,1,kk))*0.5_r8     
      END DO
        interp_t(1,1:2,pver+1)=2.0_r8*tabs(1,1,pver)-interp_t(1,1,pver)
        interp_t(1,1:2,1)  = tabs_s !2.0_r8*tabs(ii,jj,1)-interp_t(ii,jj,2)

!   DO ii=1,1
!    DO jj=1,2
!      DO kk=2,pver+1           
!        interp_t(ii,jj,kk)=(tabs(1,1,kk-1)+tabs(1,1,kk))*0.5_r8     
!      END DO
!        interp_t(ii,jj,pver+1)=2.0_r8*tabs(1,1,pver)-interp_t(ii,jj,pver)
!        interp_t(ii,jj,1)  = tabs_s !2.0_r8*tabs(ii,jj,1)-interp_t(ii,jj,2)
!   END DO
!  END DO

! Add four extra vertical layers to take the atmosphere up to 70 km
! NOTE!!!! I assume the top of the model is below 50 km.  If it is 
! near 50 km or above 50 km the results will be garbage.  The best thing
! to do in that case is set NZP2 equal to NZ in rad_PSU.inc and comment out
! this suroutine

    CALL compute_extra_layers_zhang(interp_z, ave_p, interp_p, ave_t, &
                            interp_t, ave_qv, new_o3)

! compute new_pdel and new_zdel

  !DO ii=1,NZ
  !  new_pdel(:,:,ii)=pdel(ii)
  !ENDDO

DO ii=1,NZP2
   new_pdel(:,:,ii)=abs(interp_p(:,:,ii+1)-interp_p(:,:,ii))
   new_zdel(ii)=abs(interp_z(ii+1)-interp_z(ii))
ENDDO ! ii

! convert Mixing ratio to ppmv for new_o2 --->
    new_o2(:)=O2_MIXING_RATIO*1000000.0_r8  ! 

! convert g/kg to ppmv for ave_qv --->
  !  ave_qv(:,:,:)=ave_qv(:,:,:)/1000.0_r8*(mwdry/mwh2o)*1000000.0_r8
  !  ave_qv(:,:,:)=ave_qv(:,:,:)/1000.0_r8*(28.966/18.016)*1000000.0_r8
    ave_qv(:,:,:)=ave_qv(:,:,:)*1607.793

! Flip all variables(base to top-->top to base) to fit radiation calculation.
  ave_p(1,1:2,1:NZP2)     =   ave_p(1,1:2,NZP2:1:-1)
  new_pdel(1,1:2,1:NZP2)  =   new_pdel(1,1:2,NZP2:1:-1)
  ave_t(1,1:2,1:NZP2)     =   ave_t(1,1:2,NZP2:1:-1)
  ave_qv(1,1:2,1:NZP2)    =   ave_qv(1,1:2,NZP2:1:-1)
  new_zdel(1:NZP2)        =   new_zdel(NZP2:1:-1)
  new_o2(1:NZP2)          =   new_o2(NZP2:1:-1)
  new_o3(1:NZP2)          =   new_o3(NZP2:1:-1)
  rel(1:NZP2)             =   rel(NZP2:1:-1)
  rei(1:NZP2)             =   rei(NZP2:1:-1)
  rei_j(1:nx,1:ny,1:NZP2)  =   rei_j(1:nx,1:ny,NZP2:1:-1)
  new_rh(1:NZP2)          =   new_rh(NZP2:1:-1)
  qc(1:nx,1:ny,1:NZP2)    =   qc(1:nx,1:ny,NZP2:1:-1)
  qi(1:nx,1:ny,1:NZP2)    =   qi(1:nx,1:ny,NZP2:1:-1)
  cloud_frac(1:nx,1:ny,1:NZP2)  =   cloud_frac(1:nx,1:ny,NZP2:1:-1)

  interp_p(1,1:2,1:NZP2+1) =  interp_p(1,1:2,NZP2+1:1:-1)
  interp_t(1,1:2,1:NZP2+1) =  interp_t(1,1:2,NZP2+1:1:-1)

!=========== Bellow is Zhang's code to abtain opticle depth. by JXW=========

! cpcl(NZP2,npcl,3)    ! parameters of particles (i,j,k)
!             i: layer number (from top to bottom)
!         j = 1: water cloud               2: ice cloud
!             3: dust-like                 4: soot
!             5: volcanic-ash              6: h2so4
!             7: rural                     8: sea salt
!             9: urban                    10: tropo.
!            11: yellow dust
!         k = 1: concentration in ppmv
!                = aerosol volume(cm3/cm3)*t*p0/t0/p*1.0e6
!                  with t0=273, p0=1atm.
!               type 7,8,9,10 = dry concentration(parapc.voldp)
!                             = total concentration(parapc.volp)
!         k = 2: mode radius(cm)
!         k = 3: undecided for future use.

! assign mode radius of cloud water and ice. 
     cpcl(1:nzp2,1,2)=rel(1:nzp2)    
  !  cpcl(1:nzp2,2,2) =rei(1:nzp2)

! assign gas concentrations
!  1: h2o, 2: co2, 3: o3, 4: n2o, 5: co, 6: ch4, 7: o2	
     cgas(1:NZP2,1)= ave_qv(1,1,1:NZP2)  !water vapor
     cgas(1:NZP2,2)= co2vmr*1000000.0_r8        !CO2
     cgas(1:NZP2,3)= new_o3(1:NZP2)             !ozone
    ! cgas(1:NZP2,4)= 0.31_r8                   !N2O
    ! cgas(1:NZP2,6)= 1.7_r8                    !CH4
     cgas(1:NZP2,7)= new_o2(1:NZP2)             ! O2 

     cgas(NZP2-pver+1:NZP2,4)= n2o(pver:1:-1)*1000000.0_r8    !N2O
     cgas(NZP2-pver+1:NZP2,6)= ch4(pver:1:-1)*1000000.0_r8    !CH4
!
! calculation of some transition voriables.
! -->
  c0=rgas/(g*gair)
  DO iz=1,NZP2   
     rhoair(iz)=ave_p(1,1,iz)*tstd/(pstd*ave_t(1,1,iz))
     dz(iz)=c0*tstd*abs(new_pdel(1,1,iz))/pstd
     do imol=1,nmol
       amtpb(imol,iz)=cgas(iz,imol)*dz(iz)*(1.0E-1)*(2.69E19) 
     enddo ! imol
     wbrd(iz)=dz(iz)*1.0E5*2.69E19*0.922804_r8
  END DO ! iz 

  call sdp(NZP2,ave_p(1,1,:),ddp,mp) ! get ddp and mp

 smu2=max(smu,1.0e-10); ! zero out negative values. 
 nda2=2*nda
!
! begine loop of band
! -->

DO iw=1,nbnd
    
    if(wv(iw)>=2680.0_r8 .and. smu2<=1.0e-10)  then  ! shortwave and nighttime
      exit
    endif

  !  if(wv(iw)<1430.0)  then 
  !     ib=1 ! longwave
  !  else if (wv(iw)>=2680.) then
  !     ib=2 ! shortwave
  !  else
  !     ib=3 ! both longwave and shortwave
  !  endif
    if(wv(iw)<2680.0)  then 
       ib=1 ! longwave
    else !if (wv(iw)>=2680.) then
       ib=2 ! shortwave
    end if

! assign ground albedo
    if(wv(iw)>=2680.0.and.wv(iw)<12000.0)  then  ! NIR
      galb(1)=aldir
      galb(2)=aldif
    else if (wv(iw)>=12000.0) then      ! VIS
      galb(1)=asdir
      galb(2)=asdif
    else   ! longwave
      galb(1)=0.0_r8
      galb(2)=0.0_r8
       ! galb(1:2)= 0.04_r8*landfrac+0.02_r8*ocnfrac+0.03_r8*icefrac
    endif
!
! rayleigh scattering
! -->
    DO iz=1,NZP2
      taur=ry(iw)*abs(new_pdel(1,1,iz))/pstd
      DO k=1,nda2+2
       tau_r(iz,k,1:2)=taur*qmol(k)
      END DO ! 
    END DO ! iz
!
! aerosol optics.
! -->
    do k=1,2
      tau_aer(3:nzp2,1,k)=tauaer_in(:,iw)
      tau_aer(3:nzp2,2,k)=tauaer_in(:,iw)*(1-waaer_in(:,iw))
      tau_aer(3:nzp2,3,k)=tauaer_in(:,iw)*waaer_in(:,iw)*gaaer_in(:,iw)
      tau_aer(3:nzp2,4,k)=tauaer_in(:,iw)*waaer_in(:,iw)*faaer_in(:,iw)
    end do

!
! planck functions at sublayer interfaces and at layers
! -->
    if (wv(iw)>=2680.0) then  ! shortwave
      nplk1=0
      bgnd(1:2)=0.0_r8
    else                     ! longwave
      nplk1=3
      wl=1.0E4/sqrt(wv(iw)*wv(iw+1))
      do iz=1,NZP2
         bp(iz)=bbplk(wl,interp_t(1,1,iz),naplk,aplnk(iw,1:naplk)) ! at interface level
         bl(iz)=bbplk(wl,ave_t(1,1,iz),naplk,aplnk(iw,1:naplk))    ! at mid-level
      enddo !iz

      bp(NZP2+1)=bbplk(wl,interp_t(1,1,NZP2+1),naplk,aplnk(iw,1:naplk))
      bgnd0=bbplk(wl,tabs_s,naplk,aplnk(iw,1:naplk))
      bgnd(1:2)=(1.0_r8-galb(1:2))*bgnd0  
    endif 
!=========================================================================================
!
! start loop of subintervals
! -->

    DO ig=1,mg(iw)  ! sub-intervals under band iw  

! gas absorption
        tkd(1:NZP2)=0.0_r8
        knu(:)=0.0_r8
        DO ia=1,nabs(iw)
           DO iz=1,NZP2

              ! if(iw<=8 .and. idabs(iw,ia)==3) then
              !   kpt_test(iw,ia,1:17,:,:)=kpt(iw,ia,1:17,:,:)*0.96
              !   kpt_test(iw,ia,18:22,:,:)=kpt(iw,ia,18:22,:,:)*1.0
              ! else
              !   kpt_test(iw,ia,:,:,:)=kpt(iw,ia,:,:,:)
              ! end if 

                 akt(1:nt,iz)=kpt(iw,ia,mp(iz)-1,1:nt,ig)+ &
                             (kpt(iw,ia,mp(iz),1:nt,ig)-kpt(iw,ia,mp(iz)-1,1:nt,ig))*ddp(iz)
                 akt(1:nt,iz)=10**akt(1:nt,iz)
            ENDDO ! iz

            call tdok1(NZP2,akt(1:nt,1:NZP2),ave_t(1,1,1:NZP2),knu(1:NZP2)) 
                 tkd(1:NZP2)=tkd(1:NZP2)+knu(1:NZP2)*amtpb(idabs(iw,ia),1:NZP2)
        ENDDO !ia

      !   if(wv(iw)<2680.0) then  ! longwave 
        if (idslf(iw)/=0) then
          do iz=1,NZP2
             akt(1:nt,iz)=slfk(iw,mp(iz)-1,1:nt,ig)+(slfk(iw,mp(iz) &
                ,1:nt,ig)-slfk(iw,mp(iz)-1,1:nt,ig))*ddp(iz)
             akt(1:nt,iz)=10**akt(1:nt,iz)
          enddo !l
          call tdok1(NZP2,akt(1:nt,1:NZP2),ave_t(1,1,1:NZP2),sku(1:NZP2))
               tkd(1:NZP2)=sku(1:NZP2)+tkd(1:NZP2) 
          tkd(1:NZP2)=tkd(1:NZP2)+sku(1:NZP2)*amtpb(1,1:NZP2)**2	 &
                /(amtpb(1,1:NZP2)+wbrd(1:NZP2))
        endif
         
!================
! radiation in cloudy sub-colomns
! Loop through the extra samples for this interval
! -->
        DO isampl=1,ncol_samp(ig,iw) !max(cld_nx,1),ncol_samp(ig,iw) ! cld_nx to use ICA
   ! Using Knuth's old RNG
         if (cf>0.0) then  
                ran_ix = rand_()
                ran_iy = rand_()
             
                ix = cld_x(1+int(ran_ix*cld_nx))
                iy = cld_y(1+int(ran_iy*cld_ny)) 

                IF (ix > NX) THEN
                   ix = NX
                END IF
                
                IF (iy > NY) THEN
                   iy = NY
                END IF
          else 
                ix=1
                iy=1
          endif

    ! added by JXW to use ICA
       ! if (cf>0.0) then
       !   ix=cld_x(isampl) 
       !   iy=cld_y(1)
       ! end if
       !   ix=min(ix,nx)
       !   ix=max(ix,1)
       !   iy=min(iy,1)
       !   iy=max(iy,1)

! assigne concentration of cloud water and ice

	   cpcl(1:NZP2,1,1)=qc(ix,iy,1:NZP2)   ! by jxw
	   cpcl(1:NZP2,2,1)=qi(ix,iy,1:NZP2)   ! by jxw
           cpcl(1:nzp2,2,2)=rei_j(ix,iy,1:nzp2)*1.0e-4

       tau_cld(:,:,:) = 0.0_r8
       g_phase = 0.0_r8
       cplk(1:3,1:nzp2,1:2)=0.0_r8
       mvar=nvar(1,2)

! cloud optics and combined optics in sub-colomns
      DO iz=1, NZP2
        !! firstly, caculate cloud optical depth 
            cz=0.1_r8*rhoair(iz)*dz(iz)

          ! DO K=1,nda2+2
          !! liquid cloud
          !! -->
            if (cpcl(iz,1,1)>0.0) then 
               ! mvar=nvar(1,2)
            
               DO K=1,nda2+2
                 cpcl(iz,1,2)=min(cpcl(iz,1,2),40.0e-4)
                 cpcl(iz,1,2)=max(cpcl(iz,1,2),1.0e-4)
                 call lint2(mvar,r0(1,1:mvar),q(iw,1,1:mvar,k),cpcl(iz,1,2) ,q_interp)
                 tau_cld(iz,k,1)= tau_cld(iz,k,1)+q_interp*cpcl(iz,1,1)*cz
                 tau_cld(iz,k,1)= max(tau_cld(iz,k,1),0.0_r8)
               END DO
                 !if (tau_cld(iz,k,1)<0.0) then
                 !   tau_cld(iz,k,1)=0.0_r8
                 !end if	
             end if
           !! ice cloud     
           !! -->
             if (cpcl(iz,2,1)>0.0) then 
              ! mvar=nvar(1,2) 
             !! note: re ice should not be too large or too small
              !  if (cpcl(iz,2,2)>80.0e-4) then
              !    cpcl(iz,2,2)=80.0e-4
              !  end if

              !  if (cpcl(iz,2,2)<1.0e-4) then
              !    cpcl(iz,2,2)=1.0e-4
              !  end if
              !   cpcl(iz,2,2)=min(cpcl(iz,2,2),80.0e-4)
              !   cpcl(iz,2,2)=max(cpcl(iz,2,2),1.0e-4)

               DO K=1,nda2+2
                 call lint2(mvar,1.0/r0(2,mvar:1:-1),q(iw,2,mvar:1:-1,k),1.0/cpcl(iz,2,2) ,q_interp)  !jxw reverse rei
              !  tau_cld(iz,k,1)=tau_cld(iz,k,1)+q_interp*cpcl(iz,2,1)*cz   ! if input.h is used
                 tau_cld(iz,k,1)=tau_cld(iz,k,1)+q_interp*cpcl(iz,2,1) ! if input_jxw.h and q_jxw.h is used
                 tau_cld(iz,k,1)=max(tau_cld(iz,k,1),0.0_r8)
               END DO

             !! to make sure there is no negative tau_cld caused by interpolation, by JXW
               ! if (tau_cld(iz,k,1)<0.0) then 
               !    tau_cld(iz,k,1)=0.0_r8
               ! endif
              end if

           ! ENDDO ! K
        
        !! secondly, conbine all optical depth  (except for gas absorption)

          DO ic=1,2 ! ic=1: cloudy; ic=2: clear
           !! combined optical depth and g except for gas aborption  
           !! -->
            tau_ext(iz,ic)=tau_r(iz,1,ic)+tau_aer(iz,1,ic)+tau_cld(iz,1,ic)
           ! tau_col(ix,iy,ic)=tau_col(ix,iy,ic)+tau_ext(ix,iy,iz,ic)
            scap(iz,ic)=tau_ext(iz,ic)-(tau_r(iz,2,ic)+tau_aer(iz,2,ic)+tau_cld(iz,2,ic))
            g_phase(1,iz,ic)=1.0_r8
            g_phase(2:nda2+1,iz,ic)=(tau_r(iz,3:nda+2,ic)+tau_aer(iz,3:nda+2,ic)+      &
                                        tau_cld(iz,3:nda+2,ic))/scap(iz,ic)    
           !! total optical depth(tau) and single scattering albedo(omg) including gas absorption
           !! -->
             tau_tot(iz,ic)=tkd(iz)+tau_ext(iz,ic)
              
           !-----
             if(tau_tot(iz,ic)<=0.0) then
                 tau_tot(iz,ic)=0.0_r8; omg(iz,ic)=1.0_r8
             else
                 omg(iz,ic)=scap(iz,ic)/tau_tot(iz,ic)
             endif
          enddo !ic
       !! get plank expansion coefficients  (after rei/gas/particle tau have all been assigned)
         ! cplk(1:3,iz,1:2)=0.0_r8

          if(wv(iw)<2680.0) then  ! longwave 
                dbp1=4.0_r8*bl(iz)-bp(iz+1)-3.0_r8*bp(iz)
                dbp2=bp(iz+1)+bp(iz)-2.0_r8*bl(iz)
               do ic=1,2
                 cplk(1,iz,ic)=bp(iz)
                 if(tau_tot(iz,ic)>0.0)then
                    cplk(2,iz,ic)=dbp1/tau_tot(iz,ic)
                    cplk(3,iz,ic)=dbp2/(tau_tot(iz,ic)**2)*2.0_r8
                 endif
               enddo  ! ic
          endif
     ENDDO  ! iz


! two-stream transfer

!enddo 		
        
   ! flxd(:)=0.0_r8
   ! flxu(:)=0.0_r8
   ! flxdc(:)=0.0_r8
   ! flxuc(:)=0.0_r8

   ! flxd_dir(:)=0.0
   ! flxd_dif(:)=0.0
   ! flxu_dir(:)=0.0
   ! flxu_dif(:)=0.0

   ! sols_cld_tem=0.0_r8
   ! soll_cld_tem=0.0_r8
   ! solsd_cld_tem=0.0_r8
   ! solld_cld_tem=0.0_r8

   ! sols_clr_tem=0.0_r8
   ! soll_clr_tem=0.0_r8
   ! solsd_clr_tem=0.0_r8
   ! solld_clr_tem=0.0_r8
   
    mult=pwgt(iw,ig)/(max(REAL(ncol_samp(ig,iw),r8),1.0_r8)) 
   !mult=pwgt(iw,ig)/(max(REAL(cld_nx,r8),1.0_r8))   ! to use ICA

  if (ib==1) then  ! longwave
    fsol_temp(iw)=0.0_r8
    call twst2cld(nda,smu2,tau_tot(1:NZP2,1:2),omg(1:NZP2,1:2),cloud_frac(ix,iy,1:NZP2),   &
             g_phase(1:3,1:NZP2,1:2),galb,fsol_temp(iw),nplk1,cplk(1:nplk,1:NZP2,1:2),          &
             bgnd(1:2),ib,flxd(1:NZP2+1),flxu(1:NZP2+1),                                   &
             sols_cld_tem,soll_cld_tem,solsd_cld_tem,solld_cld_tem)

    radlwup_McICA_cld(1:NZP2+1)=radlwup_McICA_cld(1:NZP2+1)+flxu(1:NZP2+1)*mult 
    radlwdn_McICA_cld(1:NZP2+1)=radlwdn_McICA_cld(1:NZP2+1)+flxd(1:NZP2+1)*mult

  end if

  !if (ib==2 .and. smu2>1.0e-10) then  ! shortwave  
  !if (ib==2) then  ! shortwave  
  if (wv(iw)>1430.0 .and. smu2>1.0e-10) then  ! expanded shortwave, account for contributions from 1430-2680 cm^-1
     if(wv(iw)<12000.0) then   ! infrared 
        call SWTRANedd(smu2,tau_tot(1:NZP2,1),omg(1:NZP2,1),aldir,aldif,fsol(iw),          &
             flxd(1:NZP2),flxu(1:NZP2),flxd_dir(1:NZP2),flxd_dif(1:NZP2),flxu_dir(1:NZP2),flxu_dif(1:NZP2), &
             fddir,fddif,g_phase(1:3,1:NZP2,1))
     radswup_McICA_cld(1:NZP2+1)=radswup_McICA_cld(1:NZP2+1)+flxu(1:NZP2+1)*mult*eccf
     radswdn_McICA_cld(1:NZP2+1)=radswdn_McICA_cld(1:NZP2+1)+flxd(1:NZP2+1)*mult*eccf  
        soll_cld   =  soll_cld  + fddir*mult*eccf
        solld_cld  =  solld_cld + fddif*mult*eccf
     else if (wv(iw)>12000.0)  then
        call SWTRANedd(smu2,tau_tot(1:NZP2,1),omg(1:NZP2,1),asdir,asdif,fsol(iw),          &
             flxd(1:NZP2),flxu(1:NZP2),flxd_dir(1:NZP2),flxd_dif(1:NZP2),flxu_dir(1:NZP2),flxu_dif(1:NZP2), &
             fddir,fddif,g_phase(1:3,1:NZP2,1))
     radswup_McICA_cld(1:NZP2+1)=radswup_McICA_cld(1:NZP2+1)+flxu(1:NZP2+1)*mult*eccf
     radswdn_McICA_cld(1:NZP2+1)=radswdn_McICA_cld(1:NZP2+1)+flxd(1:NZP2+1)*mult*eccf  
        sols_cld   =  sols_cld  + fddir*mult*eccf
        solsd_cld  =  solsd_cld + fddif*mult*eccf
     else 
        call SWTRANedd(smu2,tau_tot(1:NZP2,1),omg(1:NZP2,1),asdir,asdif,fsol(iw),          &
             flxd(1:NZP2),flxu(1:NZP2),flxd_dir(1:NZP2),flxd_dif(1:NZP2),flxu_dir(1:NZP2),flxu_dif(1:NZP2), &
             fddir,fddif,g_phase(1:3,1:NZP2,1))
        !---------------------------
        ! add to account for energy in 0.7-0.83 micron.
        ! 2013.1.12
         ! if (wv(iw)==12000.0) then
             flxu=flxu- (flxu_dir*0.2723*(asdir-aldir) +  &
                  flxu_dif*0.2723*(asdif-aldif) )
         ! end if
        !---------------------------
     radswup_McICA_cld(1:NZP2+1)=radswup_McICA_cld(1:NZP2+1)+flxu(1:NZP2+1)*mult*eccf
     radswdn_McICA_cld(1:NZP2+1)=radswdn_McICA_cld(1:NZP2+1)+flxd(1:NZP2+1)*mult*eccf  
        soll_cld   =  soll_cld  + fddir*0.2723 *mult*eccf
        solld_cld  =  solld_cld + fddif*0.2723 *mult*eccf
        sols_cld   =  sols_cld  + fddir*0.7277 *mult*eccf
        solsd_cld  =  solsd_cld + fddif*0.7277 *mult*eccf

     end if


    ! radswup_McICA_cld(1:NZP2+1)=radswup_McICA_cld(1:NZP2+1)+flxu(1:NZP2+1)*mult*eccf
    ! radswdn_McICA_cld(1:NZP2+1)=radswdn_McICA_cld(1:NZP2+1)+flxd(1:NZP2+1)*mult*eccf  

    ! if(wv(iw)<12000.0) then   ! infrared 
    !    soll_cld   =  soll_cld  + fddir*mult*eccf
    !    solld_cld  =  solld_cld + fddif*mult*eccf
    ! !---------------------------
    ! ! add to account for energy in 0.7-0.83 micron.
    ! ! 2013.1.12
    ! else if (wv(iw)==12000.0) then
    !    soll_cld   =  soll_cld  + fddir*0.2723 *mult*eccf
    !    solld_cld  =  solld_cld + fddif*0.2723 *mult*eccf
    !    sols_cld   =  sols_cld  + fddir*0.7277 *mult*eccf
    !    solsd_cld  =  solsd_cld + fddif*0.7277 *mult*eccf
    ! !---------------------------
    ! else
    !    sols_cld   =  sols_cld  + fddir*mult*eccf
    !    solsd_cld  =  solsd_cld + fddif*mult*eccf
    ! endif 
  endif

   
   !
   ! the results of flxd and flxu are from top to bottom, flip when accumulate through bands
   ! fluxes in cloudy columns
   ! -->
   !      if (ib==1) then  ! longwave
   !        
   !        radlwup_McICA_cld(1:NZP2+1)=radlwup_McICA_cld(1:NZP2+1)+flxu(1:NZP2+1)*mult 
   !        radlwdn_McICA_cld(1:NZP2+1)=radlwdn_McICA_cld(1:NZP2+1)+flxd(1:NZP2+1)*mult
   !      endif
   !    !  elseif (ib==2) then ! shortwave
   !        radswup_McICA_cld(1:NZP2+1)=radswup_McICA_cld(1:NZP2+1)+flxu(1:NZP2+1)*mult*eccf
   !        radswdn_McICA_cld(1:NZP2+1)=radswdn_McICA_cld(1:NZP2+1)+flxd(1:NZP2+1)*mult*eccf  
   !
   !      end if 
   !     
   !     if (ib==2) then
   !       if(wv(iw)<12000.0) then   ! infrared 
   !           soll_cld   =  soll_cld  + fddir*mult*eccf
   !           solld_cld  =  solld_cld + fddif*mult*eccf
   !       else 
   !           sols_cld   =  sols_cld  + fddir*mult*eccf
   !           solsd_cld  =  solsd_cld + fddif*mult*eccf
   !       endif 
   !     endif 

      END DO ! isampl
   !
   ! fluxes in clear columns
   ! -->  
   !  if (isampl==1) then 
       call t_startf('TWST2CLR')
 
       if (ib==1) then  ! Longwave
          fsol_temp(iw)=0.0_r8
          call twst2clr(nda,smu2,tau_tot(1:NZP2,1:2),omg(1:NZP2,1:2),cloud_frac(ix,iy,1:NZP2),    &
              g_phase(1:3,1:NZP2,1:2),galb,fsol_temp(iw),nplk1,cplk(1:nplk,1:NZP2,1:2), &
              bgnd(1:2),ib,flxdc(1:NZP2+1),flxuc(1:NZP2+1),    &
              sols_clr_tem,soll_clr_tem,solsd_clr_tem,solld_clr_tem)

           radlwup_McICA_clr(1:NZP2+1)=radlwup_McICA_clr(1:NZP2+1)+flxuc(1:NZP2+1)*pwgt(iw,ig)
           radlwdn_McICA_clr(1:NZP2+1)=radlwdn_McICA_clr(1:NZP2+1)+flxdc(1:NZP2+1)*pwgt(iw,ig)

       end if

       !if (ib>=2 .and. smu2>1.0e-10) then
       !if (ib==2 ) then
       if (wv(iw)>1430.0 .and. smu2>1.0e-10)  then  ! expanded shortwave, account for contributions from 1430-2680 cm^-1
         if (wv(iw)<12000.0) then  ! infrared
          !call SWTRANts(smu2,tau_tot(1:NZP2,2),omg(1:NZP2,2),aldir,aldif,fsol(iw),     &
          !    flxdc(1:NZP2),flxuc(1:NZP2),fddir,fddif,g_phase(1:3,1:NZP2,2))
          call SWTRANedd(smu2,tau_tot(1:NZP2,2),omg(1:NZP2,2),aldir,aldif,fsol(iw),          &
              flxdc(1:NZP2),flxuc(1:NZP2),flxd_dir(1:NZP2),flxd_dif(1:NZP2),flxu_dir(1:NZP2),flxu_dif(1:NZP2), &
              fddir,fddif,g_phase(1:3,1:NZP2,2))
         radswup_McICA_clr(1:NZP2+1)=radswup_McICA_clr(1:NZP2+1)+flxuc(1:NZP2+1)*pwgt(iw,ig)*eccf
         radswdn_McICA_clr(1:NZP2+1)=radswdn_McICA_clr(1:NZP2+1)+flxdc(1:NZP2+1)*pwgt(iw,ig)*eccf 
            soll_clr  =  soll_clr + fddir*pwgt(iw,ig)*eccf
            solld_clr =  solld_clr + fddif*pwgt(iw,ig)*eccf

         else if (wv(iw)>12000.0) then ! visable
          !call SWTRANts(smu2,tau_tot(1:NZP2,2),omg(1:NZP2,2),asdir,asdif,fsol(iw),     &
          !     flxdc(1:NZP2),flxuc(1:NZP2),fddir,fddif,g_phase(1:3,1:NZP2,2))
          call SWTRANedd(smu2,tau_tot(1:NZP2,2),omg(1:NZP2,2),asdir,asdif,fsol(iw),          &
              flxdc(1:NZP2),flxuc(1:NZP2),flxd_dir(1:NZP2),flxd_dif(1:NZP2),flxu_dir(1:NZP2),flxu_dif(1:NZP2), &
              fddir,fddif,g_phase(1:3,1:NZP2,2))
         radswup_McICA_clr(1:NZP2+1)=radswup_McICA_clr(1:NZP2+1)+flxuc(1:NZP2+1)*pwgt(iw,ig)*eccf
         radswdn_McICA_clr(1:NZP2+1)=radswdn_McICA_clr(1:NZP2+1)+flxdc(1:NZP2+1)*pwgt(iw,ig)*eccf 
            sols_clr  =  sols_clr + fddir*pwgt(iw,ig)*eccf
            solsd_clr =  solsd_clr + fddif*pwgt(iw,ig)*eccf
         else 
          call SWTRANedd(smu2,tau_tot(1:NZP2,2),omg(1:NZP2,2),asdir,asdif,fsol(iw),          &
              flxdc(1:NZP2),flxuc(1:NZP2),flxd_dir(1:NZP2),flxd_dif(1:NZP2),flxu_dir(1:NZP2),flxu_dif(1:NZP2), &
              fddir,fddif,g_phase(1:3,1:NZP2,2))
       !---------------------------
       ! add to account for energy in 0.7-0.83 micron.
       ! 2013.1.12
         !if (wv(iw)==12000.0) then
            flxuc=flxuc- (flxu_dir*0.2723*(asdir-aldir) +  &
                 flxu_dif*0.2723*(asdif-aldif) )
         !end if
       !---------------------------
         radswup_McICA_clr(1:NZP2+1)=radswup_McICA_clr(1:NZP2+1)+flxuc(1:NZP2+1)*pwgt(iw,ig)*eccf
         radswdn_McICA_clr(1:NZP2+1)=radswdn_McICA_clr(1:NZP2+1)+flxdc(1:NZP2+1)*pwgt(iw,ig)*eccf 
            soll_clr   =  soll_clr  + fddir*0.2723 *pwgt(iw,ig)*eccf
            solld_clr  =  solld_clr + fddif*0.2723 *pwgt(iw,ig)*eccf
            sols_clr   =  sols_clr  + fddir*0.7277 *pwgt(iw,ig)*eccf
            solsd_clr  =  solsd_clr + fddif*0.7277 *pwgt(iw,ig)*eccf
               
         end if

       !---------------------------
       ! add to account for energy in 0.7-0.83 micron.
       ! 2013.1.12
        ! if (wv(iw)==12000.0) then
        !    flxuc=flxuc- (flxu_dir*0.2723*(asdir-aldir) +  &
        !         flxu_dif*0.2723*(asdif-aldif) )
        ! end if
       !---------------------------

        ! radswup_McICA_clr(1:NZP2+1)=radswup_McICA_clr(1:NZP2+1)+flxuc(1:NZP2+1)*pwgt(iw,ig)*eccf
        ! radswdn_McICA_clr(1:NZP2+1)=radswdn_McICA_clr(1:NZP2+1)+flxdc(1:NZP2+1)*pwgt(iw,ig)*eccf 

        ! if(wv(iw)<12000.0) then   ! infrared 
        !    soll_clr  =  soll_clr + fddir*pwgt(iw,ig)*eccf
        !    solld_clr =  solld_clr + fddif*pwgt(iw,ig)*eccf
  !---------------------------
  ! add to account for energy in 0.7-0.83 micron.
  ! 2013.1.12
        ! else if (wv(iw)==12000.0) then
        !    soll_clr   =  soll_clr  + fddir*0.2723 *pwgt(iw,ig)*eccf
        !    solld_clr  =  solld_clr + fddif*0.2723 *pwgt(iw,ig)*eccf
        !    sols_clr   =  sols_clr  + fddir*0.7277 *pwgt(iw,ig)*eccf
        !    solsd_clr  =  solsd_clr + fddif*0.7277 *pwgt(iw,ig)*eccf
    !---------------------------
        ! else 
        !    sols_clr  =  sols_clr + fddir*pwgt(iw,ig)*eccf
        !    solsd_clr =  solsd_clr + fddif*pwgt(iw,ig)*eccf
        ! endif
       end if

       call t_stopf('TWST2CLR')

       ! if (ib==1) then 
       !    radlwup_McICA_clr(1:NZP2+1)=radlwup_McICA_clr(1:NZP2+1)+flxuc(1:NZP2+1)*pwgt(iw,ig)
       !    radlwdn_McICA_clr(1:NZP2+1)=radlwdn_McICA_clr(1:NZP2+1)+flxdc(1:NZP2+1)*pwgt(iw,ig)
       !
       !  else if (ib==2) then
       !    radswup_McICA_clr(1:NZP2+1)=radswup_McICA_clr(1:NZP2+1)+flxuc(1:NZP2+1)*pwgt(iw,ig)*eccf
       !    radswdn_McICA_clr(1:NZP2+1)=radswdn_McICA_clr(1:NZP2+1)+flxdc(1:NZP2+1)*pwgt(iw,ig)*eccf 
       ! endif
       ! 
       ! if (ib==2) then
       !   if(wv(iw)<12000.0) then   ! infrared 
       !       soll_clr  =  soll_clr + fddir*pwgt(iw,ig)*eccf
       !       solld_clr  =  solld_clr + fddif*pwgt(iw,ig)*eccf
       !   else 
       !       sols_clr  =  sols_clr + fddir*pwgt(iw,ig)*eccf
       !       solsd_clr  =  solsd_clr + fddif*pwgt(iw,ig)*eccf
       !   endif
       ! endif
    ! end if !isample
 !===============
      ! END DO ! isampl
     ENDDO ! ig
   ENDDO ! iw 
!
! all sky radiation fields
! -->   
   radlwup_McICA(1:NZP2+1)=radlwup_McICA_clr(1:NZP2+1)*(1.0_r8-cf)+radlwup_McICA_cld(1:NZP2+1)*cf
   radlwdn_McICA(1:NZP2+1)=radlwdn_McICA_clr(1:NZP2+1)*(1.0_r8-cf)+radlwdn_McICA_cld(1:NZP2+1)*cf
   radswup_McICA(1:NZP2+1)=radswup_McICA_clr(1:NZP2+1)*(1.0_r8-cf)+radswup_McICA_cld(1:NZP2+1)*cf
   radswdn_McICA(1:NZP2+1)=radswdn_McICA_clr(1:NZP2+1)*(1.0_r8-cf)+radswdn_McICA_cld(1:NZP2+1)*cf

   sols=sols_clr*(1.0_r8-cf)+sols_cld*cf
   soll=soll_clr*(1.0_r8-cf)+soll_cld*cf
   solsd=solsd_clr*(1.0_r8-cf)+solsd_cld*cf
   solld=solld_clr*(1.0_r8-cf)+solld_cld*cf


  do iz=1,NZP2

    delt_p(iz)= new_pdel(1,1,iz)
!
! all_sky heating rate 
! --> 
    net_fluxl(iz)=radlwdn_McICA(iz)-radlwup_McICA(iz)- &
                  (radlwdn_McICA(iz+1)-radlwup_McICA(iz+1))

    net_fluxs(iz)=radswdn_McICA(iz)-radswup_McICA(iz) - &
                   (radswdn_McICA(iz+1)-radswup_McICA(iz+1))

 ! method 1 -->
     !amair=GAIR*0.001_r8+sum(cgas(NZP2-pver+iz,1:6)*1.0E-6*(am(1:6)-GAIR*0.001_r8))
     !radqrlw_McICA(iz)=g*net_fluxl(iz)*amair/(3.5_r8*RGAS*delt_p(iz)*100.0_r8)
     !radqrsw_McICA(iz)=g*net_fluxs(iz)*amair/(3.5_r8*RGAS*delt_p(iz)*100.0_r8)
 ! method 2 -->
     radqrlw_McICA(iz)= gravit*net_fluxl(iz)/(new_pdel(1,1,iz)*100.0_r8)/cpair !*(1-cgas(iz,3)/15.0_r8)  
     radqrsw_McICA(iz)= gravit*net_fluxs(iz)/(new_pdel(1,1,iz)*100.0_r8)/cpair !*(1+cgas(iz,3)/15.0_r8)
end do

  do iz=1,NZP2
!
! clear_sky heating rate
! -->

     net_fluxl(iz)=radlwdn_McICA_clr(iz)-radlwup_McICA_clr(iz) - &
                   (radlwdn_McICA_clr(iz+1)-radlwup_McICA_clr(iz+1))
 
     net_fluxs(iz)=radswdn_McICA_clr(iz)-radswup_McICA_clr(iz) - &
                   (radswdn_McICA_clr(iz+1)-radswup_McICA_clr(iz+1))
 
   ! method 1 -->
     !amair=GAIR*0.001_r8+sum(cgas(NZP2-pver+iz,1:6)*(1.e-6)*(am(1:6)-GAIR*0.001_r8))
     !radqrlw_McICA_clr(iz)=g*net_fluxl(iz)*amair/(3.5_r8*RGAS*delt_p(iz)*100.0_r8)
     !radqrsw_McICA_clr(iz)=g*net_fluxs(iz)*amair/(3.5_r8*RGAS*delt_p(iz)*100.0_r8)
   ! method 2 -->
     radqrlw_McICA_clr(iz)= gravit*net_fluxl(iz)/(new_pdel(1,1,iz)*100.0_r8)/cpair !*(1-cgas(iz,3)/15.0_r8)  
     radqrsw_McICA_clr(iz)= gravit*net_fluxs(iz)/(new_pdel(1,1,iz)*100.0_r8)/cpair !*(1+cgas(iz,3)/15.0_r8)

  enddo !iz

!
! clear_sky heating rate
! -->

!  do iz=1,NZP2
!     net_fluxl(iz)=radlwdn_McICA_clr(iz)-radlwup_McICA_clr(iz) - &
!                   (radlwdn_McICA_clr(iz+1)-radlwup_McICA_clr(iz+1))
! 
!     net_fluxs(iz)=radswdn_McICA_clr(iz)-radswup_McICA_clr(iz) - &
!                   (radswdn_McICA_clr(iz+1)-radswup_McICA_clr(iz+1))
! 
!     delt_p(iz)= new_pdel(1,1,iz)
!   ! method 1 -->
!     !amair=GAIR*0.001_r8+sum(cgas(NZP2-pver+iz,1:6)*(1.e-6)*(am(1:6)-GAIR*0.001_r8))
!     !radqrlw_McICA_clr(iz)=g*net_fluxl(iz)*amair/(3.5_r8*RGAS*delt_p(iz)*100.0_r8)
!     !radqrsw_McICA_clr(iz)=g*net_fluxs(iz)*amair/(3.5_r8*RGAS*delt_p(iz)*100.0_r8)
!   ! method 2 -->
!     radqrlw_McICA_clr(iz)= gravit*net_fluxl(iz)/(delt_p(iz)*100.0_r8)/cpair 
!     radqrsw_McICA_clr(iz)= gravit*net_fluxs(iz)/(delt_p(iz)*100.0_r8)/cpair
!
!  enddo !iz

! 
! assign output variables
! -->

! the first layer is TOA, and the last layer is above surface
    qrs_crm(1:pver)  = radqrsw_McICA(NZP2-pver+1:NZP2)
    qrl_crm(1:pver)  = radqrlw_McICA(NZP2-pver+1:NZP2)
    qrs_clr_crm(1:pver)  = radqrsw_McICA_clr(NZP2-pver+1:NZP2)
    qrl_clr_crm(1:pver)  = radqrlw_McICA_clr(NZP2-pver+1:NZP2)
    !qrl_clr_crm(1:pver)  = ave_p(1,1,NZP2-pver+1:NZP2)
    flwds_crm   = radlwdn_McICA(NZP2+1)
    flwdsc_crm  = radlwdn_McICA_clr(NZP2+1)

    fsns_crm    = radswdn_McICA(NZP2+1) - radswup_McICA(NZP2+1)
    fsnt_crm    = radswdn_McICA(NZP2-pver+1) - radswup_McICA(NZP2-pver+1)
    flns_crm    = radlwup_McICA(NZP2+1)-radlwdn_McICA(NZP2+1)
    flnt_crm    = radlwup_McICA(NZP2-pver+1)-radlwdn_McICA(NZP2-pver+1)
    sols_crm    = sols
    solsd_crm   = solsd
    soll_crm    = soll
    solld_crm   = solld
    fsntoa_crm  = radswdn_McICA(1) - radswup_McICA(1) 
    fsntoac_crm = radswdn_McICA_clr(1) - radswup_McICA_clr(1)
    fsds_crm    = radswdn_McICA(NZP2+1)
    fsdsc_crm   = radswdn_McICA_clr(NZP2+1)
    fsntc_crm   = radswdn_McICA_clr(NZP2-pver+1) - radswup_McICA_clr(NZP2-pver+1)
    fsutoa_crm  = radswup_McICA(1)
    fsutoac_crm = radswup_McICA_clr(1)
    fsnsc_crm   = radswdn_McICA_clr(NZP2+1) - radswup_McICA_clr(NZP2+1)
    flut_crm    = radlwup_McICA(NZP2-pver+1)
    flutc_crm   = radlwup_McICA_clr(NZP2-pver+1)
    flntc_crm   = radlwup_McICA_clr(NZP2-pver+1)-radlwdn_McICA_clr(NZP2-pver+1)
    flnsc_crm   = radlwup_McICA_clr(NZP2+1)-radlwdn_McICA_clr(NZP2+1)
    solin_crm   = radswdn_McICA(1)

    RETURN
  END SUBROUTINE RAD_DRIVER_McICA_ZHANG
END MODULE RAD_DRIVER_McICA_ZHANG_MOD
