! --------------------------------------------------
MODULE RAD_SPECSAMPL_ZHANG_MOD
! --------------------------------------------------

  IMPLICIT NONE

! --------------------------------------------------
! Define the number of samples to use for the CKD 
! integration points to reduce the noise in the McICA.
! This module contains the numbers for Cole's SW CKD
! and uses three different methods to define the best 
! sampling methods.  Over ocean, the error in the atmospheric
! heating was reduced, over the the surface heating was reduced
! and over snow, ice, a 50/50 weighting was used.
! The method of optimization is described in Raisanen et al, 2003
! --------------------------------------------------

! Set that minimized error in atmospheric heating over ocean, 82 samples

! number of subinterval,high spectral resolution
! data mg(1:17)/ 12, 16,16, 5, 9, 14, 8, 15, 16, 7, 3, 1, 2, 2, 2, 4, 2/

 ! INTEGER, PARAMETER ::  ncol_samp(16,17) = RESHAPE &
 !                                         ((/1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, & !band1
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, & !band2
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, & !band3
 !                                            1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band4
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, & !band5
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, & !band6
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, & !band7
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, & !band8
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, & !band9
 !                                            1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band10
 !                                            1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band11
 !                                            1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band12
 !                                            1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band13
 !                                            1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band14
 !                                            1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band15
 !                                            1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band16
 !                                            1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0/),(/16,17/)) !band17
  
! number of subinterval, middle spectral resolution
! data mg(1:17)/ 12, 15,16, 5, 9, 12, 5, 14, 6, 7, 3, 1, 2, 2, 2, 4, 2/

 !  INTEGER, PARAMETER ::  ncol_samp(16,17) = RESHAPE &
 !                                         ((/1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, & !band1
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, & !band2
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, & !band3
 !                                            1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band4
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, & !band5
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, & !band6
 !                                            1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band7
 !                                            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, & !band8
 !                                            1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band9
 !                                            1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band10
 !                                            1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band11
 !                                            1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band12
 !                                            1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band13
 !                                            1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band14
 !                                            1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band15
 !                                            1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, & !band16
 !                                            1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0/),(/16,17/)) !band17

! number of subinterval, low spectral resolution
! data mg(1:17)/ 4, 4 ,11 ,5 ,5 ,5 ,5 ,2 ,5 ,5 ,3 ,1 , 2, 2, 2, 4, 2/

  INTEGER, PARAMETER ::  ncol_samp(11,17) = RESHAPE &
                                          ((/1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,  & !band1
                                             1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,  & !band2
                                             1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band3
                                             1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,  & !band4
                                             1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,  & !band5
                                             1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,  & !band6
                                             1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,  & !band7
                                             1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band8
                                             1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,  & !band9
                                             1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,  & !band10
                                             1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,  & !band11
                                             1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band12
                                             1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band13
                                             1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band14
                                             1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band15
                                             1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,  & !band16
                                             1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0/),(/11,17/)) !band17
 ! 

 ! double sample numbers-->
  !INTEGER, PARAMETER ::  ncol_samp(11,17) = RESHAPE &
  !                                        ((/2, 2, 2, 1, 0, 0, 0, 0, 0, 0, 0,  & !band1
  !                                           1, 1, 3, 5, 0, 0, 0, 0, 0, 0, 0,  & !band2
  !                                           1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1,  & !band3
  !                                           1, 2, 4, 4, 3, 0, 0, 0, 0, 0, 0,  & !band4
  !                                           1, 2, 3, 3, 2, 0, 0, 0, 0, 0, 0,  & !band5
  !                                           1, 3, 2, 2, 2, 0, 0, 0, 0, 0, 0,  & !band6
  !                                           1, 1, 2, 1, 2, 0, 0, 0, 0, 0, 0,  & !band7
  !                                           1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band8
  !                                           2, 1, 2, 3, 2, 0, 0, 0, 0, 0, 0,  & !band9
  !                                           2, 1, 5, 4, 3, 0, 0, 0, 0, 0, 0,  & !band10
  !                                           1, 4, 1, 0, 0, 0, 0, 0, 0, 0, 0,  & !band11
  !                                           2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band12
  !                                           2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band13
  !                                           2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band14
  !                                           3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band15
  !                                           1, 1, 2, 2, 0, 0, 0, 0, 0, 0, 0,  & !band16
  !                                           3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0/),(/11,17/)) !band17
 ! tiple sample numbers --> 
 ! NTEGER, PARAMETER ::  ncol_samp(11,17) = RESHAPE &
 !                                         ((/2, 4, 4, 1, 0, 0, 0, 0, 0, 0, 0,  & !band1
 !                                            2, 1, 4, 5, 0, 0, 0, 0, 0, 0, 0,  & !band2
 !                                            2, 2, 2, 1, 2, 3, 1, 3, 3, 3, 2,  & !band3
 !                                            3, 3, 5, 6, 6, 0, 0, 0, 0, 0, 0,  & !band4
 !                                            1, 3, 5, 4, 2, 0, 0, 0, 0, 0, 0,  & !band5
 !                                            1, 4, 3, 4, 4, 0, 0, 0, 0, 0, 0,  & !band6
 !                                            4, 1, 5, 1, 4, 0, 0, 0, 0, 0, 0,  & !band7
 !                                            1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band8
 !                                            3, 2, 2, 6, 3, 0, 0, 0, 0, 0, 0,  & !band9
 !                                            4, 1, 7, 6, 7, 0, 0, 0, 0, 0, 0,  & !band10
 !                                            1, 7, 2, 0, 0, 0, 0, 0, 0, 0, 0,  & !band11
 !                                            2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band12
 !                                            2, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band13
 !                                            4, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band14
 !                                            3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band15
 !                                            2, 1, 3, 2, 0, 0, 0, 0, 0, 0, 0,  & !band16
 !                                            3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0/),(/11,17/)) !band17

! number of subinterval, 27 Bands
! data mg(1:27)/ 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 
!                16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
!                16,  1,  2,  2, 2, 4, 2/

!  INTEGER, PARAMETER ::  ncol_samp(16,27) = RESHAPE &
!                 ((/1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band1
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band2
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band3
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band4
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band5
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band6
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band7
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band8
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band9
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band10
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band11
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band12   
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band13
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band14
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band15
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band16
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band17
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band18
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band19
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band20
 !                   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  & !band21
 !                   1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band22
 !                   1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band23
 !                   1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band24
 !                   1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band25
 !                   1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  & !band26
 !                   1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0/),  & !band27
 !                                                         (/16,27/)) 
  
END MODULE RAD_SPECSAMPL_ZHANG_MOD
