!---------------------------
MODULE RAD_SUBROUTINES_ZHANG_MOD
!---------------------------

  IMPLICIT NONE

CONTAINS 
!---------------------------
  SUBROUTINE sdp(nln,p,dp,np)
! derive absorption coefficient from pt table and fitting.
!--history
! 02.02.27  modified knupt in ckdng.f (Zhang Hua)
! 02.11.25  modified from knpt1.      (M.Sekiguchi)
!
 use shr_kind_mod,    only: r8 => shr_kind_r8  ! add by JXW

      implicit none
      integer,intent(in)::nln
      real(r8),intent(in)::p(nln)
      integer,intent(out)::np(nln)
      real(r8),intent(out)::dp(nln)
! works
      integer,parameter::kp=22
      real(r8)::pres(1:kp)=(/1.00000e-02, 1.5799999e-02, 2.1500001e-02&
          , 2.5100000e-02, 4.6399999e-02, 0.1000000, 0.1585000		  &
          , 0.2154000, 0.3981000, 0.4641000, 1.000000, 2.154400	   &
          , 4.641600, 10.00000, 21.54400, 46.41600, 100.0000, 220.0000	&
          , 340.0000, 460.0000, 700.0000, 1013.000/)
      integer::l,ip

      do l=1,nln
         do ip = 1, kp
            if (p(l)<=pres(ip)) exit
         enddo
         np(l)=ip
         if(np(l)==1) np(l)=2
         if(np(l)==kp+1) np(l)=kp   ! jxw
         dp(l)=log10(p(l)/pres(np(l)-1))/log10(pres(np(l))/pres(np(l)-1))
      enddo
   END SUBROUTINE sdp

!---------------------------------
   SUBROUTINE tdok1(nln,akt,t,knu)
!----------------------------------------------------------------------c
!                using shi's formula: (t/to)**(a+bt)
!----------------------------------------------------------------------c
   use shr_kind_mod,    only: r8 => shr_kind_r8  ! add by JXW
      implicit none
      integer,intent(in)::nln
      real(r8),intent(in)::akt(3,nln),t(nln)
      real(r8),intent(out)::knu(nln)
      integer::l
      real(r8)::a,b,c,at1,at2,bt1,bt2
!
      at1 = log(200.0/260.0); at2 = log(320.0/260.0)

      do l=1,nln
         bt1 = log(akt(1,l)/akt(2,l))
         bt2 = log(akt(3,l)/akt(2,l))

         b   = (bt2/at2-bt1/at1)/120.0
         a   = bt2/at2 - b*320.0
       
         knu(l)=akt(2,l)*(t(l)/260.0)**(a+b*t(l))
      enddo
      return
    END SUBROUTINE tdok1

!---------------------------
   SUBROUTINE LINT2(K,X,Y,A,B)
     use shr_kind_mod,    only: r8 => shr_kind_r8  ! add by JXW
     IMPLICIT NONE
      INTEGER,INTENT(IN)::K
      REAL(8),INTENT(IN)::X(K),Y(K),A
      real(r8),INTENT(OUT)::B
      INTEGER::J,RP  !reference point
      real(r8)::WGT
!
!zh      MIN=COUNT(A<X(1))
!zh      MAX=COUNT(A>X(K))
!zh      DO I=1,N
           
      DO J=2,K
         IF(A<X(1)) THEN
            WGT=(X(1)-A)/(X(2)-X(1))
            B=Y(1)-(Y(2)-Y(1))*WGT
           ELSEIF(A<=X(J)) THEN
                  WGT=(A-X(J-1))/(X(J)-X(J-1))
                  B=Y(J-1)+(Y(J)-Y(J-1))*WGT
				  EXIT
           ELSEIF(A>X(K)) THEN 
            WGT=(A-X(K))/(X(K)-X(K-1))
           !  WGT=0.5
            B=Y(K)+(Y(K)-Y(K-1))*WGT
          ENDIF
      ENDDO
   END SUBROUTINE  LINT2

!---------------------------
      subroutine twst2cld(nda,am0,thk,omg,gdcfrc,g,galb,fsol,nplk1	 &
          ,cplk,bgnd,nb,flxd,flxu,sols,soll,solsd,solld)
! solve the radiative transfer in atmosphere system for
! dynamical models.  flux at interfaces. two stream method
! by Teruyuki Nakajima
!
!--- history
! 92  2.20 created from rtrn1.
!     6.25 bug for if
! 93. 5.29 single scattering approx. for thin layer.
! 94. 12.2 partial cloudiness approx. by semi-random method.(Y.Tsushima)
! 95. 5.30 introduce sumwm to improve ground reflection.(Y.Tsushima)
! 02.12.05 integrated twst12 and twst32 (M.Sekiguchi)
!--- input
! nda        I        # of streams
! am0        R        consine of solar zenith angles .gt. 0.
! nln        I        number of atmospheric sublayers.
! thk   R(knln,2)     optical thickness of sublayers from top to bottom
! omg   R(knln,2)     single scattering albedo
! gdcfrc  R(knln)     cloud cover rate of sublayer.
! g    R(3,knln,2)    legendre moments of phase function
!                      g(i), i=1,nlgn1;  nlgn1=2*nda+1=3
! galb     R(2)       ground albedo (lambertian)
! fsol       R        solar irradiance at the system top.
! nplk1      I        number of order to approximate plank + 1.
!                      if 0 then no thermal.
! cplk R(kplk1,knln,2) thermal emission
!                      = sum(k1=1,nplk1) cplk(k1,l) * tau**(k1-1)
!                      tau is the optical depth measured from the top
!                      of the sublayer (same unit as fsol).
! bgnd     R(2)        thermal emission from the ground
!                      = (1-galb)*plank function
!                      (same unit as fsol). if 0 then no emission
! nb         I        flag of solar or thermal region
!--- output
! flxd     R(knln1)   downward flux at interfaces.
! flxu     R(knln1)   upward   flux at interfaces.
! err      c*64       error indicater. if " " then normal end.

      use shr_kind_mod,    only: r8 => shr_kind_r8 
      use ppgrid
      
      implicit none
!      include 'rad_PSU_dim.inc'
! parameters
      integer,parameter :: nplnk=3,    &   ! plank function
	                   nln=pver+3,     &   ! # of layers
		           nln1=nln+1, & 
			   nln2=nln+2  
	  real(r8),parameter :: pi=3.141592654e0,rad=pi/180.0e0
! input variables--> 
      integer,intent(in) :: nda,     & !
	                        nplk1,   & ! plank function index
						    nb         ! band index, =1:longwave, =2 shortwave
      real(r8),intent(in) :: am0,                & ! cosine solar zenith angle
	                        thk(nln,2),         & ! optical depth
						    omg(nln,2),         & ! single scattering albedo
						    gdcfrc(nln),	      & ! layer cloud fraction
                            g(3,nln,2),         & ! phase function
						    galb(2),            & ! surface albedo
						    fsol,               & ! insident solar flux at TOA
						    cplk(nplnk,nln,2),  & ! layer thermal emission and plank expansion
						    bgnd(2)               ! surface thermal emission
! output variables-->
      real(r8),intent(out) :: flxd(nln1),        & ! downward flux at boundaries
	                     flxu(nln1),        & ! upward flux at boundaries
                             sols,              & ! direct solar beam radiation onto surface. (visible)
                             soll,              & ! direct solar beam radiation onto surface. (infrared)
                             solsd,             & ! diffuse solar beam radiation onto surface. (visible)
                             solld                ! diffuse solar beam radiation onto surface. (infrared)
! local variables
     
   ! two-stream
      real(r8)::amua,wa,wmp,wmm,eps,sumwm,pla(2),pl0(2),pa(2),p0(2)
   ! delta-m method 
      real(r8)::expdm(nln1),exptm,thkt(nln,2)
      real(r8)::re(nln1),te(nln1),ser(nln1),set(nln1),rl(nln1),sl(nln1)
      real(r8)::dptt(nln1),g0,ff(nln),gt(nln),omgt(nln),cplkt(nplnk,nln)
   ! sublayer
      integer::ipk,iw0
      real(r8)::t1,t2,tau,ww,gg,cpl(nplnk),pt,pr,pt0,pr0,sum,r,t,sum1,er	 &
          ,et,x,y,q,qi,zeig,zeig2,xi,dp(nplnk),dm(nplnk),cplk1,sp,sm,g1 &
          ,gam,trns0,sum2,vp,vm,e0,exp2,exp4,cf,sf,slf,sl1f,sum3,sum4	 &
          ,ap,am,bp,bm,c11,c22,dp1,dm1,dp0,dm0,vp0,vm1,rn(nln1),rr,tt	 &
          ,r2d,su,bb,sd,rd,rup(nln2),rdn(nln2),s2u,r1d,s1d,rt,st,fu1	 &
          ,fd1
   ! work
      integer::l,ic,k1,i,j,nlt,nlt1,l1

! zero out the output files
  sols     = 0.0_r8
  soll     = 0.0_r8
  solsd    = 0.0_r8
  solld    = 0.0_r8
  flxd(1:nln1)     = 0.0_r8
  flxu(1:nln1)     = 0.0_r8

      if(nb==1) then
! amua=1/1.66
         amua=0.602409638e0; wa=1.0e0
! wmp=sqrt(wa*amua); wmm=sqrt(wa/amua)
         wmp =0.776150525e0; wmm=1.288409873e0
         eps=1.d-10
      elseif(nb==2) then
         amua=0.577350269e0; wa=1.0e0
         wmp =0.759835686e0; wmm=1.316074013e0
         eps=1.d-5
      else
         return
      endif
      sumwm=amua*wa
! set legendre polynomials.
      pla(1)=wmm ; pla(2)=wmm*amua
      pl0(1)=1.e0; pl0(2)=am0
      pa(1)=0.5e0*pla(1)**2; pa(2)=1.5e0*pla(2)**2
      p0(1)=0.5e0*pla(1)*pl0(1); p0(2)=1.5e0*pla(2)*pl0(2)
! delta-m moments
      expdm(1)=1.e0
      do l=1,nln
         thkt(l,1:2)=(1.e0-g(3,l,1:2)/g(1,l,1:2)*omg(l,1:2))*thk(l,1:2)
         exptm=gdcfrc(l)*exp(-thkt(l,1)/am0)		  &
             +(1.e0-gdcfrc(l))*exp(-thkt(l,2)/am0)
         expdm(l+1)=expdm(l)*exptm
!print *,thkt(l,1:2),l
        if (expdm(l+1)>1.0)  then
          write(6,*) 'exptm=',exptm
          write(6,*) 'am0=',am0
          write(6,*) 'thkt=',thkt(:,1)  
          write(6,*) 'gdcfrc=',gdcfrc(:)
          write(6,*) 'thk=',thk(:,1)  
          write(6,*) 'g3_1=', g(3,:,1)
          write(6,*) 'g3_2=', g(3,:,2)
          write(6,*) 'g1_1=', g(1,:,1)
          write(6,*) 'g1_1=', g(1,:,2)
          write(6,*) 'omg1=',omg(:,1)
          write(6,*) 'omg2=',omg(:,2)

        end if   
  
      enddo
!zhanghua correct a bug here 2008.7

         re(1:nln1)=0.e0; te(1:nln1)=0.e0
         set(1:nln1)=0.e0; ser(1:nln1)=0.e0
         rl(1:nln1)=0.e0; sl(1:nln1)=0.e0
!zhanghua correction end------------
      do ic=1,2 !2
 ! 
         dptt(1)=0.e0
         if(ic==1) rn(1:nln)=gdcfrc(1:nln)
         if(ic==2) rn(1:nln)=1.e0-gdcfrc(1:nln)
         do l=1,nln
            g0=g(1,l,ic)
            ff(l)=g(3,l,ic)/g0
            gt(l)=(g(2,l,ic)/g0-ff(l))/(1-ff(l))
            omgt(l)=(1-ff(l))*omg(l,ic)/(1-ff(l)*omg(l,ic))
            dptt(l+1)=dptt(l)+thkt(l,ic)
         enddo
! scaling coefficients for thermal emission
         if(nplk1>0) then
            do k1=1,nplk1
               cplkt(k1,1:nln)=cplk(k1,1:nln,ic)/(1.e0-omg(1:nln,ic) &
                   *ff(1:nln))**(k1-1)
            enddo
         endif
! solve the eigenvalue problem of atmospheric sublayers.
! loop for sublayers
         do l=1,nln
            t1=dptt(l); t2=dptt(l+1); tau=t2-t1
            ww=omgt(l); gg=gt(l); ipk=0
            if(1.e0-ww .le. eps) then
               iw0=1
            else
               iw0=0
               if(nplk1>0) ipk=1
            endif
            if(nplk1>0) then
               cpl(1:nplk1)=2*pi*(1.e0-ww)*cplkt(1:nplk1,l)
            endif
!c scattering media
!c pt, pr
            pt =pa(1)+gg*pa(2); pr =pa(1)-gg*pa(2)
            pt0=p0(1)+gg*p0(2); pr0=p0(1)-gg*p0(2)
            if(tau<=1.0d-4) then
               sum=0.e0
               if(nplk1>0) then
                  do i=1,nplk1
                     sum=sum+cpl(i)*tau**(i-1)
                  enddo
                  sum=wmm*(sum+cpl(1))*0.5e0*tau
               endif
               r=ww*pr*tau
               t=1-tau/amua+ww*pt*tau
               sum1=ww*tau*expdm(l)*exp(-thkt(l,ic)*0.5e0/am0)*fsol
               er=pr0*sum1+sum; et=pt0*sum1+sum
            else
!c eigenvalue problem
! x, y matrices
               x=1.e0/amua-ww*(pt-pr); y=1.e0/amua-ww*(pt+pr)
               q=sqrt(x); qi=1/q
               zeig2=x*y; zeig=sqrt(zeig2)
               xi=1.e0/x
! thermal source
               if(ipk==0.e0) then
                  if(nplk1>0) then
                     dp(1:nplk1)=0.e0; dm(1:nplk1)=0.e0
                  endif
               else
                  do j=nplk1,1,-1
                     if(j+2>nplk1) then
                        cplk1=0
                     else
                        cplk1=dp(j+2)
                     endif
                     dp(j)=((j+1)*j*cplk1+q*wmm*cpl(j))/zeig2
                     dm(j)=q*dp(j)
                  enddo
                  do j=1,nplk1
                     sum=0.e0
                     if(j+1<=nplk1) sum=qi*dp(j+1)
                     dp(j)=dm(j)-j*sum; dm(j)=dm(j)+j*sum
                  enddo
               endif
               sp=ww*(pt0+pr0); sm=ww*(pt0-pr0)
               g1=-x*sp-sm/am0; gam=qi*g1/(1.0/am0**2-zeig2)
               trns0=expdm(l)*fsol
               sum1=q*gam; sum2=qi*gam/am0+xi*sm
               vp=(sum1+sum2)/2.0*trns0; vm=(sum1-sum2)/2.0*trns0
               e0=exp(-tau/am0)

! base function  cf(tau) and sf(tau).
               exp2=zeig*tau; exp4=exp(-exp2)
               cf  =(1+exp4)*0.5e0; sf  =(1-exp4)*0.5e0
               slf =zeig*sf
               if(abs(exp2)<=eps) then
                  sl1f =tau*0.5e0
               else
                  sl1f =sf/zeig
               endif
! a+-, b+-
               sum1=q*cf; sum2=qi*slf; sum3=q*sl1f; sum4=qi*cf
               ap=sum1-sum2; am=sum1+sum2
               bp=sum3-sum4; bm=sum3+sum4
! c11 and c22 -> their inversion.
               c11=1.0e0/(2*am); c22=1.0e0/(2*bm)
! r, t matrices
               sum1=ap*c11; sum2=bp*c22
               r=sum1+sum2; t=sum1-sum2
! er, et matrices
               dp1=0.e0; dm1=0.e0
               if(ipk==1) then
                  do j=1,nplk1
                     dp1=dp1+dp(j)*tau**(j-1)
                     dm1=dm1+dm(j)*tau**(j-1)
                  enddo
                  dp0=dp(1); dm0=dm(1)
               else
                  dp0=0.e0; dm0=0.e0
               endif
               vp0=vp   +dp0; vm1=vm*e0+dm1
               sum1=r*vp0+t*vm1; sum2=t*vp0+r*vm1
               er=vm   +dm0-sum1; et=vp*e0+dp1-sum2
            endif
!c set to buffer
            re (l)=re(l)+rn(l)*r; te (l)=te(l)+rn(l)*t
            ser(l)=ser(l)+rn(l)*er; set(l)=set(l)+rn(l)*et
         enddo ! nln
!
! lambert surface
         if(galb(ic)<=0.e0 .and. bgnd(ic)<=0.e0) then
            nlt=nln
         else
            nlt=nln1
            t=0.e0; r=galb(ic)*wmp**2/sumwm
            x=galb(ic)*am0*expdm(nlt)*fsol/sumwm+2*pi*bgnd(ic)
            et=0; er=wmp*x
            rn(nln1)=rn(nln)
            re (nlt)=re(nlt)+rn(nlt)*r; te (nlt)=te(nlt)+rn(nlt)*t
            ser(nlt)=ser(nlt)+rn(nlt)*er;set(nlt)=set(nlt)+rn(nlt)*et
         endif
      enddo !ic
!ic loop end
      nlt1=nlt+1
! upward adding
      rl(nlt)=re(nlt)
      sl(nlt)=ser(nlt)
      if(nlt >= 2) then
        do l=nlt-1,1,-1
           l1=l+1
           rr=re(l); tt=te(l)
           r2d=rl(l1); su=r2d*set(l)+sl(l1)
           bb=1/(1-r2d*rr)
           sd=bb*su; su=tt*sd+ser(l)
           rd=rr+tt*bb*r2d*tt
           rl(l)=rd; sl(l)=su
!	write(6,'(2e15.4)') rl(l),sl(l)
        enddo
      endif
! field at the top of the system
      rup(1)=sl(1);  rdn(1)=0.e0
! downward adding
      rt=re(1); st=set(1)
      if(nlt>=2) then
        do l=2,nlt
! internal field
           s2u=sl(l); r2d=rl(l)
           sd=st+rt*s2u
           r1d=1.e0/(1.e0-rt*r2d); s1d=r1d*sd
           rdn(l)=s1d; rup(l)=r2d*s1d+s2u
!
           rr=re(l); tt=te(l)
           bb=1/(1-rt*rr); sd=bb*(rt*ser(l)+st)
           st=tt*sd+set(l); rt=rr+tt*bb*rt*tt
        enddo
      endif
! field at the bottom of the system
      rdn(nlt1)=st; rup(nlt1)=0.e0
! flux
      do l=1,nln1
         fu1=wmp*rup(l); fd1=wmp*rdn(l)
!         if(nb==1) then
            fu1=fu1*(nb-1)+fu1/amua*0.5e0*(2-nb)
            fd1=fd1*(nb-1)+fd1/amua*0.5e0*(2-nb)
!         endif
         flxu(l)=fu1; flxd(l)=fd1+am0*expdm(l)*fsol 

 ! JXW ADDED, 2010.08.24 
        if (l==nln1.and.nb==2) then 
           sols=am0*expdm(l)*fsol
           soll=am0*expdm(l)*fsol
           solsd=fd1
           solld=fd1
        endif 
      enddo
! test jxw
    if (flxd(nln1)>1000) then
      write(6,*) '*********** error in twstcld *************'
      write(6,*) 'soll_twst=',soll
      write(6,*) 'solld_twst=',solld
      write(6,*) 'sols_twst=',sols
      write(6,*) 'solsd_twst=',solsd
      write(6,*) 'expdm=',expdm(:)
      write(6,*) 'fsol=', fsol
      write(6,*) 'rdn=', rdn
      write(6,*) 'rup=', rup
      write(6,*) '************************'
    end if 
! end test
! end do !ic
      return
      end subroutine twst2cld

!-------------------------------

!---------------------------
      subroutine twst2clr(nda,am0,thk,omg,gdcfrc,g,galb,fsol,nplk1	 &
          ,cplk,bgnd,nb,flxdc,flxuc,solsc,sollc,solsdc,solldc)
! solve the radiative transfer in atmosphere system for
! dynamical models.  flux at interfaces. two stream method
! by Teruyuki Nakajima
!
!--- history
! 92  2.20 created from rtrn1.
!     6.25 bug for if
! 93. 5.29 single scattering approx. for thin layer.
! 94. 12.2 partial cloudiness approx. by semi-random method.(Y.Tsushima)
! 95. 5.30 introduce sumwm to improve ground reflection.(Y.Tsushima)
! 02.12.05 integrated twst12 and twst32 (M.Sekiguchi)
!--- input
! nda        I        # of streams
! am0        R        consine of solar zenith angles .gt. 0.
! nln        I        number of atmospheric sublayers.
! thk   R(knln,2)     optical thickness of sublayers from top to bottom
! omg   R(knln,2)     single scattering albedo
! gdcfrc  R(knln)     cloud cover rate of sublayer.
! g    R(3,knln,2)    legendre moments of phase function
!                      g(i), i=1,nlgn1;  nlgn1=2*nda+1=3
! galb     R(2)       ground albedo (lambertian)
! fsol       R        solar irradiance at the system top.
! nplk1      I        number of order to approximate plank + 1.
!                      if 0 then no thermal.
! cplk R(kplk1,knln,2) thermal emission
!                      = sum(k1=1,nplk1) cplk(k1,l) * tau**(k1-1)
!                      tau is the optical depth measured from the top
!                      of the sublayer (same unit as fsol).
! bgnd     R(2)        thermal emission from the ground
!                      = (1-galb)*plank function
!                      (same unit as fsol). if 0 then no emission
! nb         I        flag of solar or thermal region
!--- output
! flxd     R(knln1)   downward flux at interfaces.
! flxu     R(knln1)   upward   flux at interfaces.
! err      c*64       error indicater. if " " then normal end.

      use shr_kind_mod,    only: r8 => shr_kind_r8 
      use ppgrid

      implicit none
! parameters
      integer,parameter :: nplnk=3,    &   ! plank function
	                   nln=pver+3,     &   ! # of layers
			   nln1=nln+1, & 
			   nln2=nln+2  
	  real(r8),parameter :: pi=3.141592654e0, rad=pi/180.0e0
! input variables--> 
      integer,intent(in) :: nda,     & !
	                        nplk1,   & ! plank function index
						    nb         ! band index, =1:longwave, =2 shortwave
      real(r8),intent(in) :: am0,                & ! cosine solar zenith angle
	                        thk(nln,2),         & ! optical depth
						    omg(nln,2),         & ! single scattering albedo
						    gdcfrc(nln),	      & ! layer cloud fraction
                            g(3,nln,2),         & ! phase function
						    galb(2),            & ! surface albedo
						    fsol,               & ! insident solar flux at TOA
						    cplk(nplnk,nln,2),  & ! layer thermal emission and plank expansion
						    bgnd(2)               ! surface thermal emission
! output variables-->
      real(r8),intent(out) :: flxdc(nln1),        & ! downward flux at boundaries
	                         flxuc(nln1),        & ! upward flux at boundaries
                             solsc,              & ! direct solar beam radiation onto surface. (visible)
                             sollc,              & ! direct solar beam radiation onto surface. (infrared)
                             solsdc,             & ! diffuse solar beam radiation onto surface. (visible)
                             solldc                ! diffuse solar beam radiation onto surface. (infrared)
! local variables
     
   ! two-stream
      real(r8)::amua,wa,wmp,wmm,eps,sumwm,pla(2),pl0(2),pa(2),p0(2)
   ! delta-m method 
      real(r8)::expdm(nln1),exptm,thkt(nln,2)
      real(r8)::re(nln1),te(nln1),ser(nln1),set(nln1),rl(nln1),sl(nln1)
      real(r8)::dptt(nln1),g0,ff(nln),gt(nln),omgt(nln),cplkt(nplnk,nln)
   ! sublayer
      integer::ipk,iw0
      real(r8)::t1,t2,tau,ww,gg,cpl(nplnk),pt,pr,pt0,pr0,sum,r,t,sum1,er	 &
          ,et,x,y,q,qi,zeig,zeig2,xi,dp(nplnk),dm(nplnk),cplk1,sp,sm,g1 &
          ,gam,trns0,sum2,vp,vm,e0,exp2,exp4,cf,sf,slf,sl1f,sum3,sum4	 &
          ,ap,am,bp,bm,c11,c22,dp1,dm1,dp0,dm0,vp0,vm1,rn(nln1),rr,tt	 &
          ,r2d,su,bb,sd,rd,rup(nln2),rdn(nln2),s2u,r1d,s1d,rt,st,fu1	 &
          ,fd1
   ! work
      integer::l,ic,k1,i,j,nlt,nlt1,l1

! zero out the output files
  solsc     = 0.0_r8
  sollc     = 0.0_r8
  solsdc    = 0.0_r8
  solldc    = 0.0_r8
  flxdc(1:nln1)     = 0.0_r8
  flxuc(1:nln1)     = 0.0_r8

      if(nb==1) then
! amua=1/1.66
         amua=0.602409638e0; wa=1.0e0
! wmp=sqrt(wa*amua); wmm=sqrt(wa/amua)
         wmp =0.776150525e0; wmm=1.288409873e0
         eps=1.d-10
      elseif(nb==2) then
         amua=0.577350269e0; wa=1.0e0
         wmp =0.759835686e0; wmm=1.316074013e0
         eps=1.d-5
      else
         return
      endif
      sumwm=amua*wa
! set legendre polynomials.
      pla(1)=wmm ; pla(2)=wmm*amua
      pl0(1)=1.e0; pl0(2)=am0
      pa(1)=0.5e0*pla(1)**2; pa(2)=1.5e0*pla(2)**2
      p0(1)=0.5e0*pla(1)*pl0(1); p0(2)=1.5e0*pla(2)*pl0(2)
! delta-m moments
      expdm(1)=1.e0
      do l=1,nln
         thkt(l,1:2)=(1.e0-g(3,l,1:2)/g(1,l,1:2)*omg(l,1:2))*thk(l,1:2)
         exptm=exp(-thkt(l,2)/am0)
         expdm(l+1)=expdm(l)*exptm
!print *,thkt(l,1:2),l
      enddo
!zhanghua correct a bug here 2008.7

         re(1:nln1)=0.e0; te(1:nln1)=0.e0
         set(1:nln1)=0.e0; ser(1:nln1)=0.e0
         rl(1:nln1)=0.e0; sl(1:nln1)=0.e0
!zhanghua correction end------------
       ic=2 !2
 ! 
         dptt(1)=0.e0
         rn(1:nln)=1.e0
         do l=1,nln
            g0=g(1,l,ic)
            ff(l)=g(3,l,ic)/g0
            gt(l)=(g(2,l,ic)/g0-ff(l))/(1-ff(l))
            omgt(l)=(1-ff(l))*omg(l,ic)/(1-ff(l)*omg(l,ic))
            dptt(l+1)=dptt(l)+thkt(l,ic)
         enddo
! scaling coefficients for thermal emission
         if(nplk1>0) then
            do k1=1,nplk1
               cplkt(k1,1:nln)=cplk(k1,1:nln,ic)/(1.e0-omg(1:nln,ic) &
                   *ff(1:nln))**(k1-1)
            enddo
         endif
! solve the eigenvalue problem of atmospheric sublayers.
! loop for sublayers
         do l=1,nln
            t1=dptt(l); t2=dptt(l+1); tau=t2-t1
            ww=omgt(l); gg=gt(l); ipk=0
            if(1.e0-ww .le. eps) then
               iw0=1
            else
               iw0=0
               if(nplk1>0) ipk=1
            endif
            if(nplk1>0) then
               cpl(1:nplk1)=2*pi*(1.e0-ww)*cplkt(1:nplk1,l)
            endif
!c scattering media
!c pt, pr
            pt =pa(1)+gg*pa(2); pr =pa(1)-gg*pa(2)
            pt0=p0(1)+gg*p0(2); pr0=p0(1)-gg*p0(2)
            if(tau<=1.0d-4) then
               sum=0.e0
               if(nplk1>0) then
                  do i=1,nplk1
                     sum=sum+cpl(i)*tau**(i-1)
                  enddo
                  sum=wmm*(sum+cpl(1))*0.5e0*tau
               endif
               r=ww*pr*tau
               t=1-tau/amua+ww*pt*tau
               sum1=ww*tau*expdm(l)*exp(-thkt(l,ic)*0.5e0/am0)*fsol
               er=pr0*sum1+sum; et=pt0*sum1+sum
            else
!c eigenvalue problem
! x, y matrices
               x=1.e0/amua-ww*(pt-pr); y=1.e0/amua-ww*(pt+pr)
               q=sqrt(x); qi=1/q
               zeig2=x*y; zeig=sqrt(zeig2)
               xi=1.e0/x
! thermal source
               if(ipk==0.e0) then
                  if(nplk1>0) then
                     dp(1:nplk1)=0.e0; dm(1:nplk1)=0.e0
                  endif
               else
                  do j=nplk1,1,-1
                     if(j+2>nplk1) then
                        cplk1=0
                     else
                        cplk1=dp(j+2)
                     endif
                     dp(j)=((j+1)*j*cplk1+q*wmm*cpl(j))/zeig2
                     dm(j)=q*dp(j)
                  enddo
                  do j=1,nplk1
                     sum=0.e0
                     if(j+1<=nplk1) sum=qi*dp(j+1)
                     dp(j)=dm(j)-j*sum; dm(j)=dm(j)+j*sum
                  enddo
               endif
               sp=ww*(pt0+pr0); sm=ww*(pt0-pr0)
               g1=-x*sp-sm/am0; gam=qi*g1/(1.0/am0**2-zeig2)
               trns0=expdm(l)*fsol
               sum1=q*gam; sum2=qi*gam/am0+xi*sm
               vp=(sum1+sum2)/2.0*trns0; vm=(sum1-sum2)/2.0*trns0
               e0=exp(-tau/am0)

! base function  cf(tau) and sf(tau).
               exp2=zeig*tau; exp4=exp(-exp2)
               cf  =(1+exp4)*0.5e0; sf  =(1-exp4)*0.5e0
               slf =zeig*sf
               if(abs(exp2)<=eps) then
                  sl1f =tau*0.5e0
               else
                  sl1f =sf/zeig
               endif
! a+-, b+-
               sum1=q*cf; sum2=qi*slf; sum3=q*sl1f; sum4=qi*cf
               ap=sum1-sum2; am=sum1+sum2
               bp=sum3-sum4; bm=sum3+sum4
! c11 and c22 -> their inversion.
               c11=1.0e0/(2*am); c22=1.0e0/(2*bm)
! r, t matrices
               sum1=ap*c11; sum2=bp*c22
               r=sum1+sum2; t=sum1-sum2
! er, et matrices
               dp1=0.e0; dm1=0.e0
               if(ipk==1) then
                  do j=1,nplk1
                     dp1=dp1+dp(j)*tau**(j-1)
                     dm1=dm1+dm(j)*tau**(j-1)
                  enddo
                  dp0=dp(1); dm0=dm(1)
               else
                  dp0=0.e0; dm0=0.e0
               endif
               vp0=vp   +dp0; vm1=vm*e0+dm1
               sum1=r*vp0+t*vm1; sum2=t*vp0+r*vm1
               er=vm   +dm0-sum1; et=vp*e0+dp1-sum2
            endif
!c set to buffer
            re (l)=re(l)+rn(l)*r; te (l)=te(l)+rn(l)*t
            ser(l)=ser(l)+rn(l)*er; set(l)=set(l)+rn(l)*et
         enddo ! nln
!
! lambert surface
         if(galb(ic)<=0.e0 .and. bgnd(ic)<=0.e0) then
            nlt=nln
         else
            nlt=nln1
            t=0.e0; r=galb(ic)*wmp**2/sumwm
            x=galb(ic)*am0*expdm(nlt)*fsol/sumwm+2*pi*bgnd(ic)
            et=0; er=wmp*x
            rn(nln1)=rn(nln)
            re (nlt)=re(nlt)+rn(nlt)*r; te (nlt)=te(nlt)+rn(nlt)*t
            ser(nlt)=ser(nlt)+rn(nlt)*er;set(nlt)=set(nlt)+rn(nlt)*et
         endif
!ic loop end
      nlt1=nlt+1
! upward adding
      rl(nlt)=re(nlt)
      sl(nlt)=ser(nlt)
      if(nlt >= 2) then
        do l=nlt-1,1,-1
           l1=l+1
           rr=re(l); tt=te(l)
           r2d=rl(l1); su=r2d*set(l)+sl(l1)
           bb=1/(1-r2d*rr)
           sd=bb*su; su=tt*sd+ser(l)
           rd=rr+tt*bb*r2d*tt
           rl(l)=rd; sl(l)=su
!	write(6,'(2e15.4)') rl(l),sl(l)
        enddo
      endif
! field at the top of the system
      rup(1)=sl(1);  rdn(1)=0.e0
! downward adding
      rt=re(1); st=set(1)
      if(nlt>=2) then
        do l=2,nlt
! internal field
           s2u=sl(l); r2d=rl(l)
           sd=st+rt*s2u
           r1d=1.e0/(1.e0-rt*r2d); s1d=r1d*sd
           rdn(l)=s1d; rup(l)=r2d*s1d+s2u
!
           rr=re(l); tt=te(l)
           bb=1/(1-rt*rr); sd=bb*(rt*ser(l)+st)
           st=tt*sd+set(l); rt=rr+tt*bb*rt*tt
        enddo
      endif
! field at the bottom of the system
      rdn(nlt1)=st; rup(nlt1)=0.e0
! flux
      do l=1,nln1
         fu1=wmp*rup(l); fd1=wmp*rdn(l)
!         if(nb==1) then
            fu1=fu1*(nb-1)+fu1/amua*0.5e0*(2-nb)
            fd1=fd1*(nb-1)+fd1/amua*0.5e0*(2-nb)
!         endif
         flxuc(l)=fu1; flxdc(l)=fd1+am0*expdm(l)*fsol 
 ! JXW ADDED, 2010.08.24 
        if (l==nln1.and.nb==2) then 
           solsc=am0*expdm(l)*fsol
           sollc=am0*expdm(l)*fsol
           solsdc=fd1
           solldc=fd1
        endif 
      enddo

      return
      end subroutine twst2clr

!-------------------------------



   FUNCTION bbplk(wl,tmp,naplk,aplk)
      use shr_kind_mod,    only: r8 => shr_kind_r8  ! add by JXW  
      implicit none
      integer::naplk,j
      real(r8)::wl,tmp,aplk(1:naplk),b,xx,bbplk
      
      xx=1.0/(wl*tmp)
      b=aplk(1)
      do j=2,naplk
         b=b+aplk(j)*xx**(j-1)
      enddo
      bbplk=1.0/(exp(b)*wl**3*xx)
   END FUNCTION bbplk 

 END MODULE RAD_SUBROUTINES_ZHANG_MOD
