      SUBROUTINE SWTRANedd(ams,tau,omg,albdir,albdif,fsol,FLXD,FLXU, &
         !---------------
         ! added to account for the energy between 0.7 and 0.83 micron.
         ! 2013.1.12
                   FLXD_DIR, FLXD_DIF,FLXU_DIR,FLXU_DIF,             &
         !---------------
                   FDDIR,FDDIF,g)  
!
!     * MAY 14,2004 - J.LI,L.SOLHEIM..
!     * DEC 10,2010 - X. W. Jing. albdir, albdif, FDDIR, FDDIF^M
!     * OCT 10,2013 - F, Zhang.  TDTR
!     * JUL 13,2014 - X. W. Jing. BUG fix: initial value for TCUMDTR.
!----------------------------------------------------------------------
!     DELTA-EDDINGTON APPROXIMATION AND ADDING PROCESS FOR CLEAR AND   
!     ALL SKY, THE ADDING METHOD BY COAKLEY ET AL (1983). THIS CODE    
!     CAN DEAL WITH SOLAR RADIATIVE TRANSFER THROUGH ATMOSPHERE WITH   
!     PROPER TREATMENT OF CLOUD OVERLAP (RANDOM + MAXIMUM OR RANDOM +  
!     SLANTWISE) AND CLOUD SUB-GRID VARIABILITY. THE THEORY FOR ADDING,
!     CLOUD OVERLAP LI AND DOBBIE (2003). CLOUD SUB-GRID VARIABILITY   
!     SIMILAR TO WITH ADJUSTMENT OF CLOUD OPTICAL DEPTH                
!                                                                      
!     REFL:    REFLECTIVITY (1) CLEAR SKY; (2) ALL SKY                 
!     TRAN:    TRANSMITIVITY                                           
!     CUMDTR:  DIRECT TRANSMISSION FOR MULT-LAYERS                     
!     TAUA:    AEROSOL OPTICAL DEPTH                                   
!     TAUR:    RAYLEIGH OPTICAL DEPTH                                  
!     TAUG:    GASEOUS OPTICAL DEPTH                                   
!     TAUOMA:  AEROSOL OPTICAL DEPTH TIMES AEROSOL SINGLE SCATTERING   
!              ALBEDO                                                  
!     TAUOMGA: TAUOMA TIMES AEROSOL ASYMMETRY FACTOR                   
!     F1:      SQUARE OF AEROSOL ASYMMETRY FACTOR                      
!     F2:      SQUARE OF CLOUD ASYMMETRY FACTOR                        
!     TAUCS:   CLOUD OPTICAL DEPTH                                     
!     TAUOMC:  CLOUD OPTICAL DEPTH TIMES CLOUD SINGLE SCATTERING ALBEDO
!     TAUOMGC: TAUOMC TIMES CLOUD ASYMMETRY FACTOR                     
!     CLD:     CLOUD FRACTION                                          
!     CLDM:    MAXIMUM PORTION IN EACH CLOUD BLOCK, IN WHICH THE EXACT 
!              SOLUTION FOR SUBGRID VARIABILITY IS APPLIED             
!     A1:      VARIOUS RELATION FOR CLOUD OVERLAP                      
!     RMU:     COS OF SOLAR ZENITH ANGLE                               
!     C1 @ C2: TWO FACTORS NOT DEPENDENT ON IB AND IG CALCULATED       
!              OUTSIDE FOR EFFICIENCY                                  
!     ALBSUR:  SURFACE ALBEDO                                          
!     CSALB:   CLEAR-SKY SURFACE ALBEDO                                
!     NBLK:    NUMBER OF CLOUD BLOCKS ACCOUNTED FROM SURFACE           
!     NCT:     THE HIGHEST CLOUD TOP LEVEL FOR THE LONGITUDE AND       
!              LATITUDE LOOP (ILG)                                     
!     RDF:     LAYER DIFFUSE REFLECTION                                
!     TDF:     LAYER DIFFUSE TRANSMISSION                              
!     RDR:     LAYER DIRECT REFLECTION                                 
!     TDR:     LAYER DIRECT TRANSMISSION                               
!     DTR:     DIRECT TRANSMISSION                                     
!     RMDF:    BLOCK DIFFUSE REFLECTION FROM MODEL TOP LEVEL           
!     TMDR:    BLOCK DIRECT TRANSMISSION FROM MODEL TOP LEVEL          
!     RMUR:    BLOCK DIRECT REFLECTION FROM MODEL BOTTOM LEVEL         
!     RMUF:    BLOCK DIFFUSE REFLECTION FROM MODEL BOTTOM LEVEL        
!----------------------------------------------------------------------
      use shr_kind_mod, only: r8 => shr_kind_r8
      use ppgrid
      IMPLICIT REAL (A-H,O-Z),INTEGER (I-N)
        INTEGER,PARAMETER::LEV1=1,LAY=pver+3,LEV=LAY+1
        real(r8),intent(in)::TAU(LAY), OMG(LAY),AMS,FSOL,g(3,lay)
        real(r8),intent(in)::albdir, albdif
        real(r8),intent(out)::flxd(LEV),flxu(LEV)
        real(r8),intent(out)::FDDIR, FDDIF
      !---------------
      ! added to account for the energy between 0.7 and 0.83 micron.
      ! 2013.1.12
        real(r8),intent(out)::flxd_dir(LEV),flxu_dir(LEV),  &
                              flxd_dif(LEV),flxu_dif(LEV)
        REAL(r8) REFL_DIR(LEV)
        REAL(r8) REFL_DIF(LEV)
      !--------------

      REAL(r8) REFL(2,LEV), TRAN(2,LEV), CUMDTR(4,LEV),TRAN0  
      REAL(r8) TAUA(LAY), TAUR(LAY), TAUG(LAY), TAUOMA(LAY), &
          TAUOMGA(LAY), F1(LAY), F2(LAY), TAUCS(LAY),   &
          TAUOMC(LAY), TAUOMGC(LAY), CLD(LAY),          &
          ALBSUR, CSALB
      REAL(r8) RDF(4,LAY), TDF(4,LAY), RDR(4,LAY),  &
          TDR(4,LAY), DTR(4,LAY), RMDF(4,LEV),   &
          TMDR(4,LEV), RMUR(4,LEV), RMUF(4,LEV) 
      REAL(r8) TDTR(LEV)     ! Zhang F.
      REAL(r8) TCUMDTR(LEV)  ! Jing X. W.
!
!----------------------------------------------------------------------
!     COMBINE THE OPTICAL PROPERTIES FOR SOLAR,                        
!     1, AEROSOL + RAYLEIGH + GAS; 2, CLOUD + AEROSOL + RAYLEIGH + GAS 
!     CALCULATE THE DIRECT AND DIFFUSE REFLECTION AND TRANSMISSION IN  
!     THE SCATTERING LAYERS USING THE DELTA-EDDINGTON METHOD.          
!----------------------------------------------------------------------
!
      DO  K = LEV1, LAY
        AMU0 = AMS
        OG = G(2,K)
        OMEGA = OMG(K)
        SF = G(3,K) 
        OTAU=TAU(K)
        TAU1     =  OTAU * (1.0 - SF * OMEGA)

        OM1      =  OMEGA * (1. - SF)/(1. - SF * OMEGA)
        COW      =  1.0 - OM1 

        SSGAS1   = (OG - SF) / (1.0 - SF)
        COWG     =  1.0 - OM1 * SSGAS1

        DTR(1,K)      =  EXP( - TAU1 / AMU0)
        TDTR(K)      =  EXP( - OTAU / AMU0)
        ALAMD    =  SQRT(3.0 * COW * COWG+1e-6)
!                write(*,*)k

        U        =  1.50 * COWG / ALAMD
        U2       =  U + U
        UU       =  U * U
        EFUN     =  EXP(- ALAMD * TAU1)
        EFUN2    =  EFUN * EFUN
        X        = (UU - U2 + 1.0) * EFUN2
        RN       =  1.0 / (UU + U2 + 1.0 - X)
        RDF(1,K)      = (UU - 1.0) * (1.0  - EFUN2) * RN
        TDF(1,K)      = (U2 + U2) * EFUN * RN
        X        =  ALAMD * AMU0
        Y        =  1.0 - X * X
        C1       =  0.75 * AMU0
        C2       =  2.0 * C1 * AMU0
        DM       =  OM1 / Y
        GSCW     =  SSGAS1 * COW
        APPGM    = (C1 + 0.50 + GSCW * (C1 + C2)) * DM
        APMGM    = (C1 - 0.50 + GSCW * (C1 - C2)) * DM
        RDR(1,K)      = APPGM * RDF(1,K)+APMGM*(TDF(1,K)*DTR(1,K) - 1.0)
        TDR(1,K)=APPGM * TDF(1,K) + (APMGM*RDF(1,K)-APPGM +1.0)*DTR(1,K)

       end do

!
!        PAUSE
!
!----------------------------------------------------------------------
!     INITIALIZATION FOR THE FIRST LEVEL (LEV1).                       
!----------------------------------------------------------------------
!
        TRAN0                   =  1.0
        ATRAN0                  =  1.0 - TRAN0
        TMDR(1,LEV1)            =  TRAN0
        RMDF(1,LEV1)            =  ATRAN0
        CUMDTR(1,LEV1)          =  TRAN0
        TMDR(2,LEV1)            =  TRAN0
        RMDF(2,LEV1)            =  ATRAN0
        CUMDTR(2,LEV1)          =  TRAN0
        TMDR(3,LEV1)            =  TRAN0
        RMDF(3,LEV1)            =  ATRAN0
        CUMDTR(3,LEV1)          =  TRAN0
        TMDR(4,LEV1)            =  TRAN0
        RMDF(4,LEV1)            =  ATRAN0
        CUMDTR(4,LEV1)          =  TRAN0
        TCUMDTR(LEV1)            = TRAN0 !Jing X. W.
!----------------------------------------------------------------------
!     INITIALIZATION FOR THE GROUND LAYER.                             
!----------------------------------------------------------------------
!
        CSALB                   =  0.0
        ALBSUR                  =  0.0
        RMUR(1,LEV)             =  ALBDIR  !Jing X. W.
        RMUF(1,LEV)             =  ALBDIF  !Jing X. W.
        RMUR(2,LEV)             =  ALBSUR
        RMUF(2,LEV)             =  ALBSUR
        RMUR(3,LEV)             =  ALBSUR
        RMUF(3,LEV)             =  ALBSUR
        RMUR(4,LEV)             =  ALBSUR
        RMUF(4,LEV)             =  ALBSUR

        CUMDTR(2,LEV)           =  0.0
        CUMDTR(3,LEV)           =  0.0
        CUMDTR(4,LEV)           =  0.0

        TCUMDTR(LEV)           =  0.0 ! Jing X. W.
!
!----------------------------------------------------------------------
!     ADD THE LAYERS DOWNWARD FROM THE SECOND LAYER TO THE SURFACE.    
!----------------------------------------------------------------------
!
      DO  K = LEV1 + 1, LEV  
        KM1 = K - 1
        L = LEV - K + LEV1
        LP1 = L + 1

          DMM                     =  TDF(1,KM1) /   &
                                   (1.0 - RDF(1,KM1) * RMDF(1,KM1))
          FMM                     =  RMDF(1,KM1) * DMM
          TMDR(1,K)             =  CUMDTR(1,KM1) * (TDR(1,KM1) + &
                                    RDR(1,KM1) * FMM) +   &
                                   (TMDR(1,KM1) - CUMDTR(1, KM1)) * & 
                                    DMM
          RMDF(1,K)             =  RDF(1,KM1) + TDF(1,KM1) * FMM  
          CUMDTR(1,K)           =  CUMDTR(1,KM1) * DTR(1,KM1)

          TCUMDTR(K)           =  TCUMDTR(KM1) * TDTR(KM1)


!----------------------------------------------------------------------
!     ADD THE LAYERS UPWARD FROM ONE LAYER ABOVE SURFACE TO THE LEV1.  
!----------------------------------------------------------------------
!
          UMM                     =  TDF(1,L) /  &
                                   (1.0 - RDF(1,L) * RMUF(1,LP1))  
          FMM                     =  RMUF(1,LP1) * UMM
          RMUR(1,L)             =  RDR(1,L) + DTR(1,L) *    &
                                    RMUR(1,LP1) * UMM + (TDR(1,L) -  &
                                    DTR(1,L)) * FMM
          RMUF(1,L)             =  RDF(1,L) + TDF(1,L) * FMM

       end do
!
!----------------------------------------------------------------------
!     ADD DOWNWARD TO CALCULATE THE RESULTANT REFLECTANCE AND          
!     TRANSMITTANCE AT FLUX LEVELS.                                    
!----------------------------------------------------------------------
!
      DO  K = LEV1, LEV
        KM1 = K - 1

          DMM                     =  1.0 /  &
                                   (1.0 - RMUF(1,K) * RMDF(1,K))
          X                       =  CUMDTR(1,K) * RMUR(1,K)
          Y                       =  TMDR(1,K) - CUMDTR(1,K)
          TRAN(1,K)             =  CUMDTR(1,K) +  &
                                   (X * RMDF(1,K) + Y) * DMM
          REFL(1,K)             = (X + Y * RMUF(1,K)) * DMM

     !---------------------------
     ! added to account for the energy between 0.7 and 0.83 micron.
     ! 2013.1.12
           REFL_DIR(K)             =  X * DMM
           REFL_DIF(K)             =  Y * RMUF(1,K) * DMM
     !---------------------------
      end do
  
       do k=1,LEV                          
          FLXU(K)                                 = REFL(1,K)*AMS*FSOL
          FLXD(K)                                 = TRAN(1,K)*AMS*FSOL
       enddo
     !----------------------------changed by zhangfeng  2013. 10.9
     !   FDDIR                 = CUMDTR(1,LEV)*AMS*FSOL^M
         FDDIR                 = TCUMDTR(LEV)*AMS*FSOL
         FDDIF                 = FLXD(LEV)- FDDIR      
     !---------------------------
     ! added to account for the energy between 0.7 and 0.83
     ! 2013.1.12
       DO k=1,LEV                      
         FLXU_DIR(K)             = REFL_DIR(K)*AMS*FSOL
         FLXU_DIF(K)             = REFL_DIF(K)*AMS*FSOL
     !___________________changded by Zhang feng 2013. 10. 9
     !   FLXD_DIR(K)            = CUMDTR(1,LEV)*AMS*FSOL^M
         FLXD_DIR(K)            = TCUMDTR(k)*AMS*FSOL
     !------------------------------------------------------------
         FLXD_DIF(K)           = FLXD(K)-FLXD_DIR(K)
       ENDDO
     !---------------------------
      RETURN
      END
