#include <misc.h>
#include <params.h>

subroutine aero_rad_interf_climo(lchnk   ,ncol    , E_pint    ,E_pmid   ,  E_rh, &
                           E_aermmr  ,E_aer_tr  ,E_zi   ,E_t  ,E_coszrs,   &
                           aertau  ,aerssa  ,aerasm  ,aerfwd, tauxar_new,  &
                           wa_new, ga_new, fa_new)
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid
   use prescribed_aerosols, only: idxBG, idxSUL, idxSSLT, idxOCPHO, idxBCPHO, idxOCPHI, idxBCPHI, &
     idxDUSTfirst, numDUST, idxVOLC, naer_all
   use aer_optics, only: nrh, ndstsz, ksul, wsul, gsul, &
     ksslt, wsslt, gsslt, kcphil, wcphil, gcphil, kcphob, wcphob, gcphob, &
     kcb, wcb, gcb, kdst, wdst, gdst, kbg, wbg, gbg, kvolc, wvolc, gvolc
   use abortutils, only: endrun
   use cmparray_mod, only: CmpDayNite, ExpDayNite
   use quicksort, only: quick_sort
   use constituents, only: ppcnst  
   use history, only: outfld

   implicit none

   integer nspint            ! Num of spctrl intervals across solar spectrum
   integer naer_groups       ! Num of aerosol groups for optical diagnostics
   integer sbin

   parameter ( nspint = 17 )
   parameter ( naer_groups = 7 )    ! current groupings are sul, sslt, all carbons, all dust, and all aerosols
   parameter ( sbin = 1 )          ! aerosol size bin 
! Input arguments
! 
   integer, intent(in) :: lchnk             ! chunk identifier
   integer, intent(in) :: ncol              ! number of atmospheric columns


   real(r8), intent(in) :: E_pmid(pcols,pver) ! Level pressure
   real(r8), intent(in) :: E_pint(pcols,pverp) ! Interface pressure
   real(r8), intent(in) :: E_aermmr(pcols,pver,naer_all) ! Aerosol mass mixing ratio
   real(r8), intent(in) :: E_aer_tr(pcols,pver,ppcnst)  ! by wzl
   real(r8), intent(in) :: E_zi(pcols,pverp) ! by wzl
   real(r8), intent(in) :: E_t(pcols,pver) ! by wzl
   real(r8), intent(in) :: E_rh(pcols,pver)   ! Relative humidity (fraction)
! 
   real(r8), intent(in) :: E_coszrs(pcols)    ! Cosine solar zenith angle
! 
! Output arguments
   real(r8), intent(out) :: tauxar_new(pcols,0:pver,nspint)    ! aerosol optical depth
   real(r8), intent(out) :: wa_new(pcols,0:pver,nspint)    ! aerosol single scattering albedo
   real(r8), intent(out) :: ga_new(pcols,0:pver,nspint)    ! aerosol asymmetry parameter
   real(r8), intent(out) :: fa_new(pcols,0:pver,nspint)    ! aerosol forward scattering
! 
!   real(r8) , intent(out) :: frc_day(pcols) ! = 1 for daylight, =0 for night columns
  real(r8), intent(out) :: aertau(pcols,nspint,naer_groups) ! Aerosol column optical depth
  real(r8), intent(out) :: aerssa(pcols,nspint,naer_groups) ! Aerosol column averaged single scattering albedo
  real(r8), intent(out) :: aerasm(pcols,nspint,naer_groups) ! Aerosol column averaged asymmetry parameter
  real(r8), intent(out) :: aerfwd(pcols,nspint,naer_groups) ! Aerosol column averaged forward scattering
!
!---------------------------Local variables-----------------------------
!
! Local and reordered copies of the intent(in) variables
!
   real(r8) :: pmid(pcols,pver) ! Level pressure
   real(r8) :: pint(pcols,pverp) ! Interface pressure
   real(r8) :: aermmr(pcols,pver,naer_all) ! Aerosol mass mixing ratio
   real(r8) :: aermmr_test(pcols,pver,naer_all) ! Aerosol mass mixing ratio
   real(r8) :: aer_tr(pcols,pver,ppcnst)    ! by 11
   real(r8) :: zi(pcols,pverp) ! by 11
   real(r8) :: t(pcols,pver)  ! by wzl
   real(r8) :: rh(pcols,pver)   ! Relative humidity (fraction)
   real(r8) :: coszrs(pcols)    ! Cosine solar zenith angle
   real(r8) :: frc_day(pcols)
!
! Other
! 
   integer ns                ! Spectral loop index
   integer i                 ! Longitude loop index
   integer k                 ! Level loop index
   integer n                 ! Loop index for daylight
   integer ksz               ! dust size bin index
   integer krh               ! relative humidity bin index
   integer kaer              ! aerosol group index
   real(r8) wrh              ! weight for linear interpolation between lut points
   real(r8) :: rhtrunc       ! rh, truncated for the purposes of extrapolating
                             ! aerosol optical properties
!
!----------------------------------------------------------------------------------
! new aerosol/wangzhili
   real(r8) gravit
   parameter ( gravit=980.615999999999985)

   integer IRH
   real(r8) ss_ex(sbin,nspint), oc_ex(sbin,nspint), sf_ex(sbin,nspint)
   real(r8) ss_w(sbin,nspint),  oc_w(sbin,nspint), sf_w(sbin,nspint)
   real(r8) ss_g(sbin,nspint),  oc_g(sbin,nspint), sf_g(sbin,nspint)
   real(r8) ss_f(sbin), bc_f, oc_f(sbin), sd_f, sf_f(sbin)
   real(r8) u_ss(pcols,sbin,pver), u_bc(pcols,sbin,pver)
   real(r8) u_oc(pcols,sbin,pver), u_sd(pcols,sbin,pver), u_sf(pcols,sbin,pver)
   real(r8) tv_tau_ss, tv_tau_bc, tv_tau_cphil, tv_tau_cphob, tv_tau_sd, tv_tau_sf
   real(r8) Tv_tau_w_ss,Tv_tau_w_bc,Tv_tau_w_cphil,TV_tau_w_cphob, Tv_tau_w_sd,Tv_tau_w_sf
   real(r8) Tv_tau_w_g_ss,Tv_tau_w_g_bc,Tv_tau_w_g_cphil,TV_tau_w_g_cphob, Tv_tau_w_g_sd,Tv_tau_w_g_sf
   real(r8) Tv_tau_w_f_ss,Tv_tau_w_f_bc,Tv_tau_w_f_cphil,TV_tau_w_f_cphob, Tv_tau_w_f_sd,Tv_tau_w_f_sf
   real(r8) tau_ss_tot,	tau_bc_tot,tau_oc_tot,tau_sd_tot,tau_sf_tot
   real(r8) tau_w_ss_tot,tau_w_bc_tot,tau_w_oc_tot, tau_w_sd_tot,tau_w_sf_tot
   real(r8) tau_w_g_ss_tot,tau_w_g_bc_tot,tau_w_g_oc_tot,tau_w_g_sd_tot,tau_w_g_sf_tot
   real(r8) tau_w_f_ss_tot,tau_w_f_bc_tot,tau_w_f_oc_tot,tau_w_f_sd_tot,tau_w_f_sf_tot

   real(r8) tau_sd_tmp,tau_w_sd_tmp, tau_w_g_sd_tmp, tau_w_f_sd_tmp

   real(r8) tau_tot_new
   real(r8) tau_w_tot_new
   real(r8) tau_w_g_tot_new
   real(r8) tau_w_f_tot_new
   real(r8) w_tot_new
   real(r8) g_tot_new
   real(r8) f_tot_new
   integer iband, idst

   real(r8) v_tau_ss(pcols,sbin)             ! optical depth, dust, by size category
   real(r8) v_tau_w_ss(pcols,sbin)          ! optical depth * single scattering albedo, dust, by size
   real(r8) v_tau_w_g_ss(pcols,sbin)         ! optical depth * single scattering albedo * asymmetry parameter, dust, by size
   real(r8) v_tau_w_f_ss(pcols,sbin)   ! optical depth * forward scattering fraction * single scattering albedo, dust, by size
   real(r8) v_tau_ss_tot(pcols)         ! optical depth, total dust
   real(r8) v_tau_w_ss_tot(pcols)       ! optical depth * single scattering albedo, total dust
   real(r8) v_tau_w_g_ss_tot(pcols)     ! optical depth * single scattering albedo * asymmetry parameter, total dust
   real(r8) v_tau_w_f_ss_tot(pcols)     ! optical depth * forward scattering fraction * single scattering albedo, total dust

   real(r8) v_tau_sf(pcols,sbin)             ! optical depth, dust, by size category
   real(r8) v_tau_w_sf(pcols,sbin)          ! optical depth * single scattering albedo, dust, by size
   real(r8) v_tau_w_g_sf(pcols,sbin)         ! optical depth * single scattering albedo * asymmetry parameter, dust, by size
   real(r8) v_tau_w_f_sf(pcols,sbin)   ! optical depth * forward scattering fraction * single scattering albedo, dust, by size
   real(r8) v_tau_sf_tot(pcols)         ! optical depth, total dust
   real(r8) v_tau_w_sf_tot(pcols)       ! optical depth * single scattering albedo, total dust
   real(r8) v_tau_w_g_sf_tot(pcols)     ! optical depth * single scattering albedo * asymmetry parameter, total dust
   real(r8) v_tau_w_f_sf_tot(pcols)     ! optical depth * forward scattering fraction * single scattering albedo, total dust
  
   real(r8) v_tau_bc(pcols,sbin)             ! optical depth, dust, by size category
   real(r8) v_tau_w_bc(pcols,sbin)          ! optical depth * single scattering albedo, dust, by size
   real(r8) v_tau_w_g_bc(pcols,sbin)         ! optical depth * single scattering albedo * asymmetry parameter, dust, by size
   real(r8) v_tau_w_f_bc(pcols,sbin)   ! optical depth * forward scattering fraction * single scattering albedo, dust, by size
   real(r8) v_tau_bc_tot(pcols)         ! optical depth, total dust
   real(r8) v_tau_w_bc_tot(pcols)       ! optical depth * single scattering albedo, total dust
   real(r8) v_tau_w_g_bc_tot(pcols)     ! optical depth * single scattering albedo * asymmetry parameter, total dust
   real(r8) v_tau_w_f_bc_tot(pcols)     ! optical depth * forward scattering fraction * single scattering albedo, total dust

   real(r8) v_tau_oc(pcols,sbin)             ! optical depth, dust, by size category
   real(r8) v_tau_w_oc(pcols,sbin)          ! optical depth * single scattering albedo, dust, by size
   real(r8) v_tau_w_g_oc(pcols,sbin)         ! optical depth * single scattering albedo * asymmetry parameter, dust, by size
   real(r8) v_tau_w_f_oc(pcols,sbin)   ! optical depth * forward scattering fraction * single scattering albedo, dust, by size
   real(r8) v_tau_oc_tot(pcols)         ! optical depth, total dust
   real(r8) v_tau_w_oc_tot(pcols)       ! optical depth * single scattering albedo, total dust
   real(r8) v_tau_w_g_oc_tot(pcols)     ! optical depth * single scattering albedo * asymmetry parameter, total dust
   real(r8) v_tau_w_f_oc_tot(pcols)     ! optical depth * forward scattering fraction * single scattering albedo, total dust

   real(r8) v_tau_sd(pcols,sbin)             ! optical depth, dust, by size category
   real(r8) v_tau_w_sd(pcols,sbin)          ! optical depth * single scattering albedo, dust, by size
   real(r8) v_tau_w_g_sd(pcols,sbin)         ! optical depth * single scattering albedo * asymmetry parameter, dust, by size
   real(r8) v_tau_w_f_sd(pcols,sbin)   ! optical depth * forward scattering fraction * single scattering albedo, dust, by size
   real(r8) v_tau_sd_tot(pcols)         ! optical depth, total dust
   real(r8) v_tau_w_sd_tot(pcols)       ! optical depth * single scattering albedo, total dust
   real(r8) v_tau_w_g_sd_tot(pcols)     ! optical depth * single scattering albedo * asymmetry parameter, total dust
   real(r8) v_tau_w_f_sd_tot(pcols)     ! optical depth * forward scattering fraction * single scattering albedo, total dust
!------------------------------------------------------------------
! Aerosol mass paths by species
!
   real(r8) usul(pcols,pver)   ! sulfate (SO4)
   real(r8) ubg(pcols,pver)    ! background aerosol
   real(r8) usslt(pcols,pver)  ! sea-salt (SSLT)
   real(r8) ucphil(pcols,pver) ! hydrophilic organic carbon (OCPHI)
   real(r8) ucphob(pcols,pver) ! hydrophobic organic carbon (OCPHO)
   real(r8) ucb(pcols,pver)    ! black carbon (BCPHI + BCPHO)
   real(r8) uvolc(pcols,pver) ! volcanic mass
   real(r8) udst(pcols,ndstsz,pver) ! dust

!
! local variables used for the external mixing of aerosol species
!
  real(r8) tau_sul             ! optical depth, sulfate
  real(r8) tau_bg              ! optical depth, background aerosol
  real(r8) tau_sslt            ! optical depth, sea-salt
  real(r8) tau_cphil           ! optical depth, hydrophilic carbon
  real(r8) tau_cphob           ! optical depth, hydrophobic carbon
  real(r8) tau_cb              ! optical depth, black carbon
  real(r8) tau_volc            ! optical depth, volcanic
  real(r8) tau_dst(ndstsz)     ! optical depth, dust, by size category
  real(r8) tau_dst_tot         ! optical depth, total dust
  real(r8) tau_tot             ! optical depth, total aerosol

  real(r8) tau_w_sul           ! optical depth * single scattering albedo, sulfate
  real(r8) tau_w_bg            ! optical depth * single scattering albedo, background aerosol
  real(r8) tau_w_sslt          ! optical depth * single scattering albedo, sea-salt
  real(r8) tau_w_cphil         ! optical depth * single scattering albedo, hydrophilic carbon
  real(r8) tau_w_cphob         ! optical depth * single scattering albedo, hydrophobic carbon
  real(r8) tau_w_cb            ! optical depth * single scattering albedo, black carbon
  real(r8) tau_w_volc          ! optical depth * single scattering albedo, volcanic
  real(r8) tau_w_dst(ndstsz)   ! optical depth * single scattering albedo, dust, by size
  real(r8) tau_w_dst_tot       ! optical depth * single scattering albedo, total dust
  real(r8) tau_w_tot           ! optical depth * single scattering albedo, total aerosol

  real(r8) tau_w_g_sul         ! optical depth * single scattering albedo * asymmetry parameter, sulfate
  real(r8) tau_w_g_bg          ! optical depth * single scattering albedo * asymmetry parameter, background aerosol
  real(r8) tau_w_g_sslt        ! optical depth * single scattering albedo * asymmetry parameter, sea-salt
  real(r8) tau_w_g_cphil       ! optical depth * single scattering albedo * asymmetry parameter, hydrophilic carbon
  real(r8) tau_w_g_cphob       ! optical depth * single scattering albedo * asymmetry parameter, hydrophobic carbon
  real(r8) tau_w_g_cb          ! optical depth * single scattering albedo * asymmetry parameter, black carbon
  real(r8) tau_w_g_volc        ! optical depth * single scattering albedo * asymmetry parameter, volcanic
  real(r8) tau_w_g_dst(ndstsz) ! optical depth * single scattering albedo * asymmetry parameter, dust, by size
  real(r8) tau_w_g_dst_tot     ! optical depth * single scattering albedo * asymmetry parameter, total dust
  real(r8) tau_w_g_tot         ! optical depth * single scattering albedo * asymmetry parameter, total aerosol

  real(r8) f_sul               ! forward scattering fraction, sulfate
  real(r8) f_bg                ! forward scattering fraction, background aerosol
  real(r8) f_sslt              ! forward scattering fraction, sea-salt
  real(r8) f_cphil             ! forward scattering fraction, hydrophilic carbon
  real(r8) f_cphob             ! forward scattering fraction, hydrophobic carbon
  real(r8) f_cb                ! forward scattering fraction, black carbon
  real(r8) f_volc              ! forward scattering fraction, volcanic
  real(r8) f_dst(ndstsz)       ! forward scattering fraction, dust, by size
  real(r8) f_dst_tot           ! forward scattering fraction, total dust
  real(r8) f_tot               ! forward scattering fraction, total aerosol

  real(r8) tau_w_f_sul         ! optical depth * forward scattering fraction * single scattering albedo, sulfate
  real(r8) tau_w_f_bg          ! optical depth * forward scattering fraction * single scattering albedo, background
  real(r8) tau_w_f_sslt        ! optical depth * forward scattering fraction * single scattering albedo, sea-salt
  real(r8) tau_w_f_cphil       ! optical depth * forward scattering fraction * single scattering albedo, hydrophilic C
  real(r8) tau_w_f_cphob       ! optical depth * forward scattering fraction * single scattering albedo, hydrophobic C
  real(r8) tau_w_f_cb          ! optical depth * forward scattering fraction * single scattering albedo, black C
  real(r8) tau_w_f_volc        ! optical depth * forward scattering fraction * single scattering albedo, volcanic
  real(r8) tau_w_f_dst(ndstsz) ! optical depth * forward scattering fraction * single scattering albedo, dust, by size
  real(r8) tau_w_f_dst_tot     ! optical depth * forward scattering fraction * single scattering albedo, total dust
  real(r8) tau_w_f_tot         ! optical depth * forward scattering fraction * single scattering albedo, total aerosol
  real(r8) w_dst_tot           ! single scattering albedo, total dust
  real(r8) w_tot               ! single scattering albedo, total aerosol
  real(r8) g_dst_tot           ! asymmetry parameter, total dust
  real(r8) g_tot               ! asymmetry parameter, total aerosol
  real(r8) ksuli               ! specific extinction interpolated between rh look-up-table points, sulfate
  real(r8) ksslti              ! specific extinction interpolated between rh look-up-table points, sea-salt
  real(r8) kcphili             ! specific extinction interpolated between rh look-up-table points, hydrophilic carbon
  real(r8) wsuli               ! single scattering albedo interpolated between rh look-up-table points, sulfate
  real(r8) wsslti              ! single scattering albedo interpolated between rh look-up-table points, sea-salt
  real(r8) wcphili             ! single scattering albedo interpolated between rh look-up-table points, hydrophilic carbon
  real(r8) gsuli               ! asymmetry parameter interpolated between rh look-up-table points, sulfate
  real(r8) gsslti              ! asymmetry parameter interpolated between rh look-up-table points, sea-salt
  real(r8) gcphili             ! asymmetry parameter interpolated between rh look-up-table points, hydrophilic carbon
! 
! Aerosol radiative property arrays
! 
   real(r8) tauxar(pcols,0:pver) ! aerosol extinction optical depth
   real(r8) wa(pcols,0:pver) ! aerosol single scattering albedo
   real(r8) ga(pcols,0:pver) ! aerosol assymetry parameter
   real(r8) fa(pcols,0:pver) ! aerosol forward scattered fraction
! 
! Various arrays and other constants:
! 
   real(r8) pflx(pcols,0:pverp) ! Interface press, including extra layer
   real(r8) pdel             ! Pressure difference across layer
   real(r8) path             ! Mass path of layer
   real(r8) ptop             ! Lower interface pressure of extra layer
! 
! Layer absorber amounts; note that 0 refers to the extra layer added
! above the top model layer
! 
   real(r8) uaer(pcols,0:pver) ! Layer aerosol amount 
!
  integer :: kk
  integer :: Nday                      ! Number of daylight columns
  integer :: Nnite                     ! Number of night columns
  integer, dimension(pcols) :: IdxDay  ! Indicies of daylight coumns
  integer, dimension(pcols) :: IdxNite ! Indicies of night coumns
!
! Arrays used in vectorization
!
  real(r8), dimension(pcols,ndstsz) :: v_tau_dst             ! optical depth, dust, by size category
  real(r8), dimension(pcols,ndstsz) :: v_tau_w_dst           ! optical depth * single scattering albedo, dust, by size
  real(r8), dimension(pcols,ndstsz) :: v_tau_w_g_dst         ! optical depth * single scattering albedo * asymmetry parameter, dust, by size
  real(r8), dimension(pcols,ndstsz) :: v_tau_w_f_dst         ! optical depth * forward scattering fraction * single scattering albedo, dust, by size
  real(r8), dimension(pcols)        :: v_tau_dst_tot         ! optical depth, total dust
  real(r8), dimension(pcols)        :: v_tau_w_dst_tot       ! optical depth * single scattering albedo, total dust
  real(r8), dimension(pcols)        :: v_tau_w_g_dst_tot     ! optical depth * single scattering albedo * asymmetry parameter, total dust
  real(r8), dimension(pcols)        :: v_tau_w_f_dst_tot     ! optical depth * forward scattering fraction * single scattering albedo, total dust

   real(r8) :: Tv_tau_dst
   real(r8) :: Tv_tau_w_dst
   real(r8) :: Tv_tau_w_g_dst
   real(r8) :: Tv_tau_w_f_dst

   logical :: lrhtrunc_lt0(pver)      ! Logical rhtrunc < 0.0_r8
   logical :: lg_tot_gt1(pver)        ! Logical g_tot > 1.0_r8
   logical :: lg_tot_ltm1(pver)       ! Logical g_tot < -1.0_r8
   logical :: lf_tot_gt1(pver)        ! Logical f_tot > 1.0_r8
   logical :: lf_tot_lt0(pver)        ! Logical f_tot < 0.0_r8

!-------wangzhili-------
#include <aero_opt_17b_climo.h>
! 
!-----------------------------------------------------------------------
! START OF CALCULATION
!-----------------------------------------------------------------------
   
   frc_day(1:ncol) = 0.0_r8

! zero out output fields.

   aertau(1:ncol,1:nspint,1:naer_groups) = 0.0_r8
   aerssa(1:ncol,1:nspint,1:naer_groups) = 0.0_r8
   aerasm(1:ncol,1:nspint,1:naer_groups) = 0.0_r8
   aerfwd(1:ncol,1:nspint,1:naer_groups) = 0.0_r8

   tauxar_new(1:ncol,0:pver,1:nspint) = 0.0_r8
   wa_new(1:ncol,0:pver,1:nspint) = 0.0_r8
   ga_new(1:ncol,0:pver,1:nspint) = 0.0_r8
   fa_new(1:ncol,0:pver,1:nspint) = 0.0_r8
    
! 
! Compute starting, ending daytime loop indices:
!  *** Note this logic assumes day and night points are contiguous so
!  *** will not work in general with chunked data structure.
! 
   Nday = 0
   Nnite = 0

   do i = 1, ncol
      if ( E_coszrs(i) > 0.0_r8 ) then
         Nday = Nday + 1
         IdxDay(Nday) = i
      else
         Nnite = Nnite + 1
         IdxNite(Nnite) = i
      end if
   end do
! 
! If night everywhere, return:
! 
   if ( Nday == 0 ) return
!
! Rearrange input arrays
!
   call CmpDayNite(E_pmid, pmid,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pver)
   call CmpDayNite(E_pint, pint,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pverp)
   call CmpDayNite(E_aermmr, aermmr,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pver, 1, naer_all)
   call CmpDayNite(E_aer_tr, aer_tr,    Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pver, 1, ppcnst)
   call CmpDayNite(E_zi, zi,            Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pverp)
   call CmpDayNite(E_t, t,              Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pver)
   call CmpDayNite(E_rh, rh,		Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pver)
   call CmpDayNite(E_coszrs, coszrs,	Nday, IdxDay, Nnite, IdxNite, 1, pcols)

! 
! Perform other initializations
! 
   do k=1,pverp
      do i=1,Nday
         pflx(i,k) = pint(i,k)
      end do
   end do

   do i=1,Nday
! 
! Define solar incident radiation and interface pressures:
! 
         pflx(i,0) = 0._r8
! 
! Compute optical paths:
! 
         ptop      = pflx(i,1)
         uaer(i,0) = 0.0_r8

   end do ! End  do i=1,Nday

   do k=1,pver
!cdir nodep
      do i=1,Nday
         pdel      = pflx(i,k+1) - pflx(i,k)
         path      = pdel / gravit

         usul(i,k) = aermmr(i,k,idxSUL) * path
         ubg(i,k) = aermmr(i,k,idxBG) * path
         usslt(i,k) = aermmr(i,k,idxSSLT) * path

         !aermmr_test(i,k,idxSUL) = aermmr(i,k,idxSUL) * path
         !aermmr_test(i,k,idxSSLT) = aermmr(i,k,idxSSLT) * path

         if (usslt(i,k) .lt. 0.0) then                    ! usslt is sometimes small and negative, will be fixed
           usslt(i,k) = 0.0
         end if
         ucphil(i,k) = aermmr(i,k,idxOCPHI) * path
         ucphob(i,k) = aermmr(i,k,idxOCPHO) * path
         ucb(i,k) = ( aermmr(i,k,idxBCPHO) + aermmr(i,k,idxBCPHI) ) * path
         uvolc(i,k) =  aermmr(i,k,idxVOLC)
     
     !cdir expand=ndstsz
              do ksz = 1, ndstsz
                udst(i,ksz,k) = aermmr(i,k,idxDUSTfirst-1+ksz) * path
              end do


!new aerosol//wangzhili
        ! do iband=1,sbin
        !   u_ss(i,iband,k)=aer_tr(i,k,3+iband)*path
        !   u_bc(i,iband,k)=aer_tr(i,k,15+iband)*path
        !   u_oc(i,iband,k)=aer_tr(i,k,27+iband)*path
        !   u_sd(i,iband,k)=aer_tr(i,k,39+iband)*path
        !   u_sf(i,iband,k)=aer_tr(i,k,51+iband)*path
        ! enddo
      end do ! end  do i=1,Nday
    end do ! end  k=1,pver
! 
! Set cloud properties for top (0) layer; so long as tauxcl is zero,
! there is no cloud above top of model; the other cloud properties
! are arbitrary:
! 
      do i=1,Nday
! aerosol
         tauxar_new(i,0,:)  = 0._r8
         wa_new(i,0,:)      = 0.925_r8
         ga_new(i,0,:)      = 0.850_r8
         fa_new(i,0,:)      = 0.7225_r8
      end do  ! end  do i=1,Nday
! 
! Begin spectral loop
! 
   do ns=1,nspint
!
! Compute tau_dst_tot, tau_w_dst_tot, tau_w_g_dst_tot, and, tau_w_f_dst_tot
!
      do i=1,Nday
         frc_day(i) = 1.0_r8
         do kaer = 1, naer_groups
            aertau(i,ns,kaer) = 0.0
            aerssa(i,ns,kaer) = 0.0
            aerasm(i,ns,kaer) = 0.0
            aerfwd(i,ns,kaer) = 0.0
         end do
      end do ! end do i=1,Nday

!new aerosol//wangzhili
     ! bc_f  = bc_g(ns)*bc_g(ns)
     ! sd_f  = sd_g(ns)*sd_g(ns)

      f_volc   = gvolc(ns) * gvolc(ns)  ! CAM3 OLD
      f_bg     = gbg(ns) * gbg(ns)      ! CAM3 OLD

      do k=1,pver
!
         lrhtrunc_lt0(k) = .false.
         lg_tot_gt1(k)   = .false.
         lg_tot_ltm1(k)  = .false.
         lf_tot_gt1(k)   = .false.
         lf_tot_lt0(k)   = .false.

!-----------------------------------------------
!new aerosol//wangzhili
  		 v_tau_ss_tot(1:Nday)     = 0.0_r8
		 v_tau_w_ss_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_ss_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_ss_tot(1:Nday) = 0.0_r8

		 v_tau_bc_tot(1:Nday)     = 0.0_r8
		 v_tau_w_bc_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_bc_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_bc_tot(1:Nday) = 0.0_r8

		 v_tau_oc_tot(1:Nday)     = 0.0_r8
		 v_tau_w_oc_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_oc_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_oc_tot(1:Nday) = 0.0_r8

		 v_tau_sd_tot(1:Nday)     = 0.0_r8
		 v_tau_w_sd_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_sd_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_sd_tot(1:Nday) = 0.0_r8

		 v_tau_sf_tot(1:Nday)     = 0.0_r8
		 v_tau_w_sf_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_sf_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_sf_tot(1:Nday) = 0.0_r8

!new aerosol//wangzhili
         do i=1,Nday
            if (rh(i,k) .LT. BRH(1)) then
                  IRH = 0                                   ! RH  0-20%
           else if (rh(i,k) .GE. BRH(1) .and. rh(i,k) .LT. BRH(2)) then
                  IRH = 1                                   !     20-50
           else if (rh(i,k) .GE. BRH(2) .and. rh(i,k) .LT. BRH(3)) then
                  IRH = 2                                   !     50-70
           else if (rh(i,k) .GE. BRH(3) .and. rh(i,k) .LT. BRH(4)) then
                  IRH = 3                                   !     70-90
           else if (rh(i,k) .GE. BRH(4) .and. rh(i,k) .LT. BRH(5)) then
                  IRH = 4                                   !     90-95
           else if (rh(i,k) .GE. BRH(5) .and. rh(i,k) .LT. BRH(6)) then
                  IRH = 5                                   !     95-99
	   else
	          IRH = 6                                   !      > 99
           endif

           do kk=1,sbin


             if (IRH .EQ. 0) then
    	        ss_ex(kk,ns) = qexss(1,ns)
	        ss_w(kk,ns)  = wss(1,ns)
	        ss_g(kk,ns)  = gss(1,ns)
                oc_ex(kk,ns) = qexoc(1,ns)
                oc_w(kk,ns)  = woc(1,ns)
	        oc_g(kk,ns)  = goc(1,ns)
	        sf_ex(kk,ns) = qexsf(1,ns)
                sf_w(kk,ns)  = wsf(1,ns)
	        sf_g(kk,ns)  = gsf(1,ns)
             else if (IRH .EQ. 6) then
    	        ss_ex(kk,ns) = qexss(IRH,ns)
	        ss_w(kk,ns)  = wss(IRH,ns)
	        ss_g(kk,ns)  = gss(IRH,ns)
                oc_ex(kk,ns) = qexoc(IRH,ns)
                oc_w(kk,ns)  = woc(IRH,ns)
	        oc_g(kk,ns)  = goc(IRH,ns)
	        sf_ex(kk,ns) = qexsf(IRH,ns)
                sf_w(kk,ns)  = wsf(IRH,ns)
	        sf_g(kk,ns)  = gsf(IRH,ns)
                
              else
                ss_ex(kk,ns) = ((rh(i,k)-BRH(IRH))*qexss(IRH+1,ns)+(BRH(IRH+1)-rh(i,k)) &
                                *qexss(IRH,ns))/(BRH(IRH+1)-BRH(IRH))
                ss_w(kk,ns)  = ((rh(i,k)-BRH(IRH))*wss(IRH+1,ns)+(BRH(IRH+1)-rh(i,k)) &
                                *wss(IRH,ns))/(BRH(IRH+1)-BRH(IRH))				
 	        ss_g(kk,ns)  = ((rh(i,k)-BRH(IRH))*gss(IRH+1,ns)+(BRH(IRH+1)-rh(i,k)) &
                                *gss(IRH,ns))/(BRH(IRH+1)-BRH(IRH))
                oc_ex(kk,ns) = ((rh(i,k)-BRH(IRH))*qexoc(IRH+1,ns)+(BRH(IRH+1)-rh(i,k)) &
                                *qexoc(IRH,ns))/(BRH(IRH+1)-BRH(IRH))
                oc_w(kk,ns)  = ((rh(i,k)-BRH(IRH))*woc(IRH+1,ns)+(BRH(IRH+1)-rh(i,k)) &
                                *woc(IRH,ns))/(BRH(IRH+1)-BRH(IRH))				
	        oc_g(kk,ns)  = ((rh(i,k)-BRH(IRH))*goc(IRH+1,ns)+(BRH(IRH+1)-rh(i,k)) &
                                *goc(IRH,ns))/(BRH(IRH+1)-BRH(IRH))
	        sf_ex(kk,ns) = ((rh(i,k)-BRH(IRH))*qexsf(IRH+1,ns)+(BRH(IRH+1)-rh(i,k)) &
                                *qexsf(IRH,ns))/(BRH(IRH+1)-BRH(IRH))
                sf_w(kk,ns)  = ((rh(i,k)-BRH(IRH))*wsf(IRH+1,ns)+(BRH(IRH+1)-rh(i,k)) &
                                *wsf(IRH,ns))/(BRH(IRH+1)-BRH(IRH))				
	        sf_g(kk,ns)  = ((rh(i,k)-BRH(IRH))*gsf(IRH+1,ns)+(BRH(IRH+1)-rh(i,k)) &
                                *gsf(IRH,ns))/(BRH(IRH+1)-BRH(IRH))
	       end if

               ss_f(kk) = ss_g(kk,ns)*ss_g(kk,ns)
               oc_f(kk) = oc_g(kk,ns)*oc_g(kk,ns)
               sf_f(kk) = sf_g(kk,ns)*sf_g(kk,ns)
           !!!!!!
               tv_tau_ss     = 1.e4*1.e-4*usslt(i,k)*ss_ex(kk,ns)/2.2
               tv_tau_w_ss   = tv_tau_ss*ss_w(kk,ns)
	       tv_tau_w_g_ss = tv_tau_ss*ss_w(kk,ns)*ss_g(kk,ns)
	       tv_tau_w_f_ss = tv_tau_ss*ss_w(kk,ns)*ss_f(kk)
		 
               tv_tau_bc     = 1.e4*1.e-4*ucb(i,k)*bc_ex(ns)/1.0
               tv_tau_w_bc   = tv_tau_bc*bc_w(ns)
	       tv_tau_w_g_bc = tv_tau_bc*bc_w(ns)*bc_g(ns)
	       tv_tau_w_f_bc = tv_tau_bc*bc_w(ns)*(bc_g(ns)**2)

               tv_tau_cphil     = 1.e4*1.e-4*ucphil(i,k)*oc_ex(kk,ns)/1.3
	       tv_tau_w_cphil   = tv_tau_cphil*oc_w(kk,ns)
	       tv_tau_w_g_cphil = tv_tau_cphil*oc_w(kk,ns)*oc_g(kk,ns)
	       tv_tau_w_f_cphil = tv_tau_cphil*oc_w(kk,ns)*oc_f(kk)

               tv_tau_cphob     = 1.e4*1.e-4*ucphob(i,k)*qexoc(1,ns)/1.3
	       tv_tau_w_cphob   = tv_tau_cphob*woc(1,ns)
	       tv_tau_w_g_cphob = tv_tau_cphob*woc(1,ns)*goc(1,ns)
	       tv_tau_w_f_cphob = tv_tau_cphob*woc(1,ns)*(goc(1,ns)**2)

               tv_tau_sf     = 1.e4*1.e-4*usul(i,k)*sf_ex(kk,ns)/1.7
	       tv_tau_w_sf   = tv_tau_sf*sf_w(kk,ns)
	       tv_tau_w_g_sf = tv_tau_sf*sf_w(kk,ns)*sf_g(kk,ns)
	       tv_tau_w_f_sf = tv_tau_sf*sf_w(kk,ns)*sf_f(kk)

              ! add by zhaoshy-->
               tv_tau_sd = 0.0_r8
               tv_tau_w_sd = 0.0_r8
               tv_tau_w_g_sd = 0.0_r8
               tv_tau_w_f_sd = 0.0_r8
               tau_sd_tmp=0.0_r8
               tau_w_sd_tmp=0.0_r8
               tau_w_g_sd_tmp=0.0_r8
               tau_w_f_sd_tmp=0.0_r8


               do idst=1,ndstsz
                 tau_sd_tmp     = 1.e4*1.e-4*sd_ex(idst,ns)*udst(i,idst,k)/2.6
	         tau_w_sd_tmp   = tau_sd_tmp*sd_w(idst,ns)
	         tau_w_g_sd_tmp = tau_sd_tmp*sd_w(idst,ns)*sd_g(idst,ns)
	         tau_w_f_sd_tmp = tau_sd_tmp*sd_w(idst,ns)*(sd_g(idst,ns)**2)

                 tv_tau_sd = tv_tau_sd + tau_sd_tmp
                 tv_tau_w_sd = tv_tau_w_sd + tau_w_sd_tmp
                 tv_tau_w_g_sd = tv_tau_w_g_sd + tau_w_g_sd_tmp
                 tv_tau_w_f_sd = tv_tau_w_f_sd + tau_w_f_sd_tmp
  
               end do
           !!!!!!
	       v_tau_ss_tot(i)     = v_tau_ss_tot(i)+ tv_tau_ss
	       v_tau_w_ss_tot(i)   = v_tau_w_ss_tot(i)+ tv_tau_w_ss
	       v_tau_w_g_ss_tot(i) = v_tau_w_g_ss_tot(i)+ tv_tau_w_g_ss
	       v_tau_w_f_ss_tot(i) = v_tau_w_f_ss_tot(i)+ tv_tau_w_f_ss
 
	       v_tau_bc_tot(i)     = v_tau_bc_tot(i)+ tv_tau_bc
	       v_tau_w_bc_tot(i)   = v_tau_w_bc_tot(i)+ tv_tau_w_bc
	       v_tau_w_g_bc_tot(i) = v_tau_w_g_bc_tot(i)+ tv_tau_w_g_bc
	       v_tau_w_f_bc_tot(i) = v_tau_w_f_bc_tot(i)+ tv_tau_w_f_bc

	       v_tau_oc_tot(i)     = v_tau_oc_tot(i)+ tv_tau_cphil + tv_tau_cphob
	       v_tau_w_oc_tot(i)   = v_tau_w_oc_tot(i)+ tv_tau_w_cphil + tv_tau_w_cphob
	       v_tau_w_g_oc_tot(i) = v_tau_w_g_oc_tot(i)+ tv_tau_w_g_cphil + tv_tau_w_g_cphob
	       v_tau_w_f_oc_tot(i) = v_tau_w_f_oc_tot(i)+ tv_tau_w_f_cphil + tv_tau_w_f_cphob

	       v_tau_sd_tot(i)     = v_tau_sd_tot(i)+ tv_tau_sd
	       v_tau_w_sd_tot(i)   = v_tau_w_sd_tot(i)+ tv_tau_w_sd
    	       v_tau_w_g_sd_tot(i) = v_tau_w_g_sd_tot(i)+ tv_tau_w_g_sd
    	       v_tau_w_f_sd_tot(i) = v_tau_w_f_sd_tot(i)+ tv_tau_w_f_sd

               v_tau_sf_tot(i)     = v_tau_sf_tot(i)+ tv_tau_sf
    	       v_tau_w_sf_tot(i)   = v_tau_w_sf_tot(i)+ tv_tau_w_sf
    	       v_tau_w_g_sf_tot(i) = v_tau_w_g_sf_tot(i)+ tv_tau_w_g_sf
    	       v_tau_w_f_sf_tot(i) = v_tau_w_f_sf_tot(i)+ tv_tau_w_f_sf
              
	    enddo  ! do kk=1, sbin
         enddo ! do i=1,Nday
!--------------------------------------------------------------------------

         do i=1,Nday

             !  tau_volc = 1.e3 * kvolc(ns) * uvolc(i,k)    ! CAM3 OLD
             !  tau_bg = 1.e4 * kbg(ns) * ubg(i,k)          ! CAM3 OLD
	     !  tau_w_volc = tau_volc * wvolc(ns)           ! CAM3 OLD
             !  tau_w_bg = tau_bg * wbg(ns)                 ! CAM3 OLD
             !  tau_w_g_volc = tau_w_volc * gvolc(ns)       ! CAM3 OLD
             !  tau_w_g_bg = tau_w_bg * gbg(ns)             ! CAM3 OLD
             !  tau_w_f_volc = tau_w_volc * f_volc          ! CAM3 OLD
             !  tau_w_f_bg = tau_w_bg * f_bg                ! CAM3 OLD
!
               tau_volc = qexsf(1,ns) * uvolc(i,k)/1.7    
               tau_bg =  qexsf(1,ns) * ubg(i,k)/1.7          
	       tau_w_volc = tau_volc * wsf(1,ns)             
               tau_w_bg = tau_bg * wsf(1,ns)                 
               tau_w_g_volc = tau_w_volc * gsf(1,ns)         
               tau_w_g_bg = tau_w_bg * gsf(1,ns)             
               tau_w_f_volc = tau_w_volc * (gsf(1,ns)**2)      
               tau_w_f_bg = tau_w_bg * (gsf(1,ns)**2)        
!
! mix dust aerosol size bins
!  !new aerosol//wangzhili 
               tau_ss_tot    =v_tau_ss_tot(i)
    	       tau_w_ss_tot  =v_tau_w_ss_tot(i)
	       tau_w_g_ss_tot=v_tau_w_g_ss_tot(i)
	       tau_w_f_ss_tot=v_tau_w_g_ss_tot(i)

	       tau_bc_tot    =v_tau_bc_tot(i)
	       tau_w_bc_tot  =v_tau_w_bc_tot(i)
	       tau_w_g_bc_tot=v_tau_w_g_bc_tot(i)
	       tau_w_f_bc_tot=v_tau_w_f_bc_tot(i)

	       tau_oc_tot    =v_tau_oc_tot(i)
	       tau_w_oc_tot  =v_tau_w_oc_tot(i)
	       tau_w_g_oc_tot=v_tau_w_g_oc_tot(i)
	       tau_w_f_oc_tot=v_tau_w_f_oc_tot(i)

	       tau_sd_tot    =v_tau_sd_tot(i)
	       tau_w_sd_tot  =v_tau_w_sd_tot(i)
	       tau_w_g_sd_tot=v_tau_w_g_sd_tot(i)
	       tau_w_f_sd_tot=v_tau_w_f_sd_tot(i)

               tau_sf_tot    =v_tau_sf_tot(i)
	       tau_w_sf_tot  =v_tau_w_sf_tot(i)
	       tau_w_g_sf_tot=v_tau_w_g_sf_tot(i)
               tau_w_f_sf_tot=v_tau_w_f_sf_tot(i)
!
! mix aerosols
!
!new  aersol//wangzhili
!--------------------------------------------------------------------------------
           tau_tot_new     = tau_sf_tot+tau_ss_tot+tau_bc_tot+tau_oc_tot +tau_sd_tot
           tau_tot_new     = tau_tot_new  + tau_bg + tau_volc
           tau_w_tot_new   = tau_w_sf_tot+tau_w_ss_tot+tau_w_bc_tot+tau_w_oc_tot +tau_w_sd_tot
           tau_w_tot_new   = tau_w_tot_new  + tau_w_bg + tau_w_volc
           tau_w_g_tot_new = tau_w_g_sf_tot+tau_w_g_ss_tot+tau_w_g_bc_tot+tau_w_g_oc_tot +tau_w_g_sd_tot
           tau_w_g_tot_new = tau_w_g_tot_new  + tau_w_g_bg + tau_w_g_volc
           tau_w_f_tot_new = tau_w_f_sf_tot+tau_w_f_ss_tot+tau_w_f_bc_tot+tau_w_f_oc_tot +tau_w_f_sd_tot
           tau_w_f_tot_new = tau_w_f_tot_new   + tau_w_f_bg + tau_w_f_volc
     
!new aerosol//wangzhili
!-----------------------------------------------------------------------
           if (tau_tot_new .gt. 0.0) then
             w_tot_new = tau_w_tot_new / tau_tot_new
           else
             w_tot_new = 0.0
           endif

           if (tau_w_tot_new .gt. 0.0) then
             g_tot_new = tau_w_g_tot_new / tau_w_tot_new
             f_tot_new = tau_w_f_tot_new / tau_w_tot_new
           else
             g_tot_new = 0.0
             f_tot_new = 0.0
           endif

           tauxar_new(i,k,ns) = max(tau_tot_new,0.0)
           wa_new(i,k,ns)     = min(w_tot_new,  0.999999_r8)
           ga_new(i,k,ns)     = min(g_tot_new,	0.999999_r8)
           fa_new(i,k,ns)     = min(f_tot_new,	0.999999_r8)
!-------------------------------------------------------------------------
           aertau(i,ns,1) = aertau(i,ns,1) + tau_sf_tot
           aertau(i,ns,2) = aertau(i,ns,2) + tau_ss_tot
           aertau(i,ns,3) = aertau(i,ns,3) + tau_oc_tot+ tau_bc_tot 
           aertau(i,ns,4) = aertau(i,ns,4) + tau_sd_tot
           aertau(i,ns,5) = aertau(i,ns,5) + tau_bg
           aertau(i,ns,6) = aertau(i,ns,6) + tau_volc
           aertau(i,ns,7) = aertau(i,ns,7) + tau_tot_new
         ! end if
           aerssa(i,ns,1) = aerssa(i,ns,1) + tau_w_sf_tot
           aerssa(i,ns,2) = aerssa(i,ns,2) + tau_w_ss_tot
           aerssa(i,ns,3) = aerssa(i,ns,3) + tau_w_oc_tot+ tau_w_bc_tot 
           aerssa(i,ns,4) = aerssa(i,ns,4) + tau_w_sd_tot
           aerssa(i,ns,5) = aerssa(i,ns,5) + tau_w_bg
           aerssa(i,ns,6) = aerssa(i,ns,6) + tau_w_volc
           aerssa(i,ns,7) = aerssa(i,ns,7) + tau_w_tot_new

           aerasm(i,ns,1) = aerasm(i,ns,1) + tau_w_g_sf_tot
           aerasm(i,ns,2) = aerasm(i,ns,2) + tau_w_g_ss_tot
           aerasm(i,ns,3) = aerasm(i,ns,3) + tau_w_g_oc_tot + tau_w_g_bc_tot
           aerasm(i,ns,4) = aerasm(i,ns,4) + tau_w_g_sd_tot
           aerasm(i,ns,5) = aerasm(i,ns,5) + tau_w_g_bg
           aerasm(i,ns,6) = aerasm(i,ns,6) + tau_w_g_volc
           aerasm(i,ns,7) = aerasm(i,ns,7) + tau_w_g_tot_new

           aerfwd(i,ns,1) = aerfwd(i,ns,1) + tau_w_f_sf_tot
           aerfwd(i,ns,2) = aerfwd(i,ns,2) + tau_w_f_ss_tot
           aerfwd(i,ns,3) = aerfwd(i,ns,3) + tau_w_f_oc_tot+ tau_w_f_bc_tot
           aerfwd(i,ns,4) = aerfwd(i,ns,4) + tau_w_f_sd_tot
           aerfwd(i,ns,5) = aerfwd(i,ns,5) + tau_w_f_bg
           aerfwd(i,ns,6) = aerfwd(i,ns,6) + tau_w_f_volc
           aerfwd(i,ns,7) = aerfwd(i,ns,7) + tau_w_f_tot_new

         end do ! end do i=1,Nday
      end do ! end do k=1,pver

!CSD$ END PARALLEL 

      if (any( lrhtrunc_lt0(:) )) then
         write(6,*) "rh < 0.0"
         call endrun('AERO_RAD_INTERFACE')
      end if
      if (any( lg_tot_gt1(:) )) then
         write(6,*) "g_tot > 1.0"
         call endrun('AERO_RAD_INTERFACE')
      end if
      if (any( lg_tot_ltm1(:) )) then
         write(6,*) "g_tot < -1.0"
         call endrun('AERO_RAD_INTERFACE')
      end if
      if (any( lf_tot_gt1(:) )) then
         write(6,*) "f_tot > 1.0"
         call endrun('AERO_RAD_INTERFACE')
      end if
      if (any( lf_tot_lt0(:) )) then
         write(6,*) "f_tot < 0.0"
         call endrun('AERO_RAD_INTERFACE')
      end if

   end do
!
! intent(out)
!
 ! call ExpDayNite(frc_day,	Nday, IdxDay, Nnite, IdxNite, 1, pcols)
   call ExpDayNite(aertau,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, nspint, 1, naer_groups)
   call ExpDayNite(aerssa,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, nspint, 1, naer_groups)
   call ExpDayNite(aerasm,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, nspint, 1, naer_groups)
   call ExpDayNite(aerfwd,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, nspint, 1, naer_groups)
   call ExpDayNite(tauxar_new,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 0, pver, 1, nspint)
   call ExpDayNite(wa_new,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 0, pver, 1, nspint)
   call ExpDayNite(ga_new,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 0, pver, 1, nspint)
   call ExpDayNite(fa_new,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 0, pver, 1, nspint)

   call ExpDayNite(aermmr,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pver, 1, naer_all)
 !  call ExpDayNite(aermmr_test,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pver, 1, naer_all)
 !  call ExpDayNite(usul,	Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pver)
!-----------------------------------------------------------------------------------------------------
  ! call outfld('TAUTOT  ', tauxar_new(:,1:pver,11), pcols, lchnk)
  ! call outfld('WATOT   ', wa_new(:,1:pver,11), pcols, lchnk)
  ! call outfld('GATOT   ', ga_new(:,1:pver,11), pcols, lchnk)
  ! call outfld('FATOT   ', fa_new(:,1:pver,11), pcols, lchnk)

  ! call outfld('TAUSUL  ', aertau(:,11,1), pcols, lchnk)
  ! call outfld('TAUSSLT ', aertau(:,11,2), pcols, lchnk)
  ! call outfld('TAUBC   ', aertau(:,11,3), pcols, lchnk)
  ! call outfld('TAUDST  ', aertau(:,11,4), pcols, lchnk)
  ! call outfld('TAUOC   ', aertau(:,11,5), pcols, lchnk)
  ! call outfld('TAUVOL  ', aertau(:,11,6), pcols, lchnk)
  ! call outfld('TAUBG   ', aertau(:,11,7), pcols, lchnk)

  ! call outfld('TAUTOT  ', aermmr(:,1:pver,idxSUL), pcols, lchnk)
  ! call outfld('WATOT   ', aermmr_test(:,1:pver,idxSUL), pcols, lchnk)
  ! call outfld('GATOT   ', aermmr_test(:,1:pver,idxSSLT), pcols, lchnk)
  ! call outfld('FATOT   ', aermmr_test(:,1:pver,idxDUSTfirst), pcols, lchnk)

!-----------------------------------------------------------------------------------------------------
   return
end subroutine aero_rad_interf_climo
