include 'rad_PSU_dims.inc'

REAL, PARAMETER :: SMALL_NUMBER_r8 = 1.0E-12 ! A small number
REAL, PARAMETER :: MIN_CLD = 1.0/REAL(NX*NY) ! At least one cloudly column present
REAL, PARAMETER :: P0 = 101325.0 ! Standard surface pressure in Pa

REAL, PARAMETER :: LAND_LIQ_DROP_CONC  = 250.0 ! Drops per cm^3
REAL, PARAMETER :: LAND_ICE_DROP_CONC  = 1.0   ! Particles per cm^3
REAL, PARAMETER :: OCEAN_LIQ_DROP_CONC = 100.0 ! Drops per cm^3
REAL, PARAMETER :: OCEAN_ICE_DROP_CONC = 1.0   ! Particles per cm^3
REAL, PARAMETER :: CONST_REW = 71.67 ! Liquid, 0.5*((6/pi)^0.33)*exp(sigma^2)*100.0, sigma = 0.38
REAL, PARAMETER :: CONST_REI = 63.45 ! Ice,    0.5*((6/pi)^0.33)*exp(sigma^2)*100.0, sigma = 0.15
REAL, PARAMETER :: LAND_CLD_EPS_L = 1.0E-6 ! 1.4E-2 ! Minimum amount of LWC for a land cloud
REAL, PARAMETER :: LAND_CLD_EPS_I = 1.0E-6 ! 4.7E-4 ! Minimum amount of IWC for a land cloud
REAL, PARAMETER :: OCEAN_CLD_EPS_L = 1.0E-6 ! 5.9E-3 ! Minimum amount of LWC for a ocean cloud
REAL, PARAMETER :: OCEAN_CLD_EPS_I = 1.0E-6 ! 4.7E-4 ! Minimum amount of IWC for a ocean cloud
REAL, PARAMETER :: MIN_REL = 2.8 ! Minimum liquid drop size
REAL, PARAMETER :: MAX_REL = 39.8 ! Maximum liquid drop size
REAL, PARAMETER :: MIN_REI = 23.0 ! Minimum ice particle size
REAL, PARAMETER :: MAX_REI = 140.0 ! Maximum ice particle size

REAL, PARAMETER :: PI = 3.141593 ! Pi
REAL, PARAMETER :: ONEOVER245 = 0.00408163 ! 1/245 for Planck function
 
REAL, PARAMETER :: CONST1 = 7.242901E+24 ! Constant to compute air number density
REAL, PARAMETER :: CONST2 = 1.607682E-3 ! Constant to compute water molecule number density

REAL, PARAMETER :: O2_MIXING_RATIO = 2.0946E-01 ! Oxygen mixing ratio
REAL, PARAMETER :: LOSCMHIDT_VOLUME = 2.24E+04 ! LOSCMHIDT_VOLUME for ozone 
REAL, PARAMETER :: GAIR = 28.964 ! Molecular weight dry air
REAL, PARAMETER :: GOZONE = 47.9982 ! Molecular weight ozone
REAL, PARAMETER :: RGAS = 8.31441 ! Gas constant
REAL, PARAMETER :: G = 9.80616 !9.80665 ! Acceleration due to gravity

REAL, PARAMETER :: A0 = 6.022045E+23 ! Avagarrdos Number
REAL, PARAMETER :: DEG2RAD = PI/180.0 ! Convert degrees to radians

REAL, PARAMETER :: W0_CRIT = 0.995 !0.99999995 ! Cutoff for conservative solution
REAL, PARAMETER :: U1 = 0.606530659 ! 1.0/exp(0.5), mu1
REAL, PARAMETER :: RU = 1.648721271 ! exp(0.5), 1/mu1
REAL, PARAMETER :: UU3 = 1.103638324 !3*U1^2
REAL, PARAMETER :: SMALL_W0 = 1.0e-6 ! Minimum SSA to use

REAL, PARAMETER :: UMCO2=360.0, UMCH4=1.70, UMN2O=0.31 ! CO2, CH4, N2O concentrations
INTEGER, PARAMETER :: MAXNB=18, MAXNG=12, NB=18 ! Band and subintervals for Function LW CKD

!REAL, PARAMETER :: hr_coef = 9.984839E-3 ! Convert flux divergence to heating rates K/s
! For SP-GCM the division by Cp_air is done outside of the radiation code so set hr_coef
! equal to gravity
REAL, PARAMETER :: hr_coef = G ! Convert flux divergence to heating rates K/s
