subroutine radctl_zh (                                                                  &
            ! input
                      lchnk, ncol, lwup, coszrs, calday, clat, clon, cld, rel, rei, state,    & 
                      aldir, aldif, asdir, asdif, ocnfrac, landfrac, icefrac,       &
            ! output
                      qrs, qrl, flwds, fsns, fsnt, flns, flnt, sols, soll, solsd, solld, fsds) 
  
  use shr_kind_mod,    only: r8=>shr_kind_r8
  use ppgrid 
  use physics_types,   only: physics_state 
  use phys_grid,       only: get_rlat_all_p, get_rlon_all_p, get_lon_all_p, get_lat_all_p 
  use constituents,    only: cnst_get_ind 
  use physconst,       only: cpair, mwdry, mwco2, mwch4, mwn2o, mwf11, mwf12, epsilo, stebol !epsilo  
  use time_manager,    only: get_curr_calday, get_curr_date 
  use shr_kind_mod,    only: r8 => shr_kind_r8 
  use wv_saturation,   only: aqsat          ! wangzhili 2011.6.23 
  use aerosol_intr,    only: set_aerosol_from_prognostics !wangzhili 
  use prescribed_aerosols, only: get_aerosol, naer_all, get_rf_scales  , get_int_scales, idxVOLC, &
                                 radforce, aerosol_diagnostics , aerosol_indirect
  use pmgrid,          only: PLON 
  use history,         only: outfld

  use MSC_CLD_GEN_DRIVER_MRO_MOD        ! Cloud generator 
  use rad_PSU_dims_include_mod          ! McICA dimensions 
  use RAN_NUMBERS_MOD                   ! Random number generator 
  use McICA_CLD_DIAG_MOD                ! Diagnose the vertically projected cloud fractions 
! ZHANG RAD CODE 
  use RAD_DRIVER_McICA_ZHANG_MOD        ! use ZHANG radiative code 
  
   implicit none
 
!#include <comctl.h> 
 
!!!! input 
   INTEGER, INTENT(IN)  :: lchnk                ! lchnk 
   INTEGER, INTENT(IN)  :: ncol                 ! ncol 
   REAL(r8), INTENT(IN) :: lwup(pcols)          ! lwup 
   REAL(r8), INTENT(IN) :: coszrs(pcols)        ! coszrs 
   REAL(r8), INTENT(IN) :: calday               ! calday 
   REAL(r8), INTENT(IN) :: clat(pcols)          ! clat 
   REAL(r8), INTENT(IN) :: clon(pcols)          ! clon 
   real(r8), INTENT(in) :: cld(pcols,pver)      ! cld  
   real(r8), intent(in) :: rel(pcols,pver)      ! rel 
   real(r8), intent(in) :: rei(pcols,pver)      ! rei 
 
   type(physics_state), intent(in) :: state     ! state 
 
   real(r8), intent(in) :: aldir(pcols)         ! aldir 
   real(r8), intent(in) :: aldif(pcols)         ! aldif 
   real(r8), intent(in) :: asdir(pcols)         ! asdir 
   real(r8), intent(in) :: asdif(pcols)         ! asdif	   
   real(r8), intent(in) :: ocnfrac(pcols)       ! ocnfrac 
   real(r8), intent(in) :: landfrac(pcols)      ! landfrac 
   real(r8), intent(in) :: icefrac(pcols)       ! icefrac 
 
!!!!output 
   real(r8), intent(out) :: qrs(pcols,pver)     !qrs 
   real(r8), intent(out) :: qrl(pcols,pver)     !qrl 
   real(r8), intent(out) :: flwds(pcols)        !flwds 
   real(r8), intent(out) :: fsns(pcols)         !fsns 
   real(r8), intent(out) :: fsnt(pcols)         !fsnt 
   real(r8), intent(out) :: flns(pcols)         !flns 
   real(r8), intent(out) :: flnt(pcols)         !flnt 
   real(r8), intent(out) :: sols(pcols)         !sols 
   real(r8), intent(out) :: soll(pcols)         !soll 
   real(r8), intent(out) :: solsd(pcols)        !solsd 
   real(r8), intent(out) :: solld(pcols)        !solld 
   real(r8), intent(out) :: fsds(pcols)         !fsds 
 
!!!!local 
 
  ! Related to random number generation 
   integer tod       ! Current UTC time of day 
   integer yr        ! Current UTC year 
   integer mon       ! Current UTC month 
   integer day       ! Current UTC day 
   integer hour      ! Current UTC hour 
   integer minute    ! Current UTC minute 
   integer sec       ! Current UTC second 
   integer ran_seed  ! Random number seed 
    
   integer lat(pcols)        ! current latitudes(indices) 
   integer lon(pcols)        ! current longtitudes(indices) 
 
   real(r8) :: ts(pcols)            ! ts

  ! McICA input fields 
   real(r8) t_rad_McICA(NX,  NY, pver)     ! rad temperature 
   real(r8) qv_rad_McICA(NX, NY, pver)     ! rad vapor 
   real(r8) qc_rad_McICA(NX, NY, pver)     ! rad cloud water 
   real(r8) qi_rad_McICA(NX, NY, pver)     ! rad cloud ice  
   real(r8) o3vmr_McICA(pcols,pver)      ! ozone volume mixing ratio  
   real(r8) coszrs_McICA(pcols)          ! Cosine of the solar zenith angle 
   real(r8) eccf_McICA                   ! Earth orbit eccentricity factor 
 
  ! In-cloud water content 
   real(r8) qc_cld(pcols,pver)           ! In cloud liquid water content 
   real(r8) qi_cld(pcols,pver)           ! In cloud ice water content 
   integer :: ixcldice, ixcldliq         ! constituent indices for cloud liquid and ice water. 
  ! McICA radiation fields 
   real(r8)  qrs_McICA(pcols,pver)       ! Shortwave heating rate (McICA) 
   real(r8)  qrl_McICA(pcols,pver)       ! Longwave  heating rate (McICA) 
   real(r8)  qrs_clr_McICA(pcols,pver)   ! Shortwave heating rate (McICA) 
   real(r8)  qrl_clr_McICA(pcols,pver)   ! Shortwave heating rate (McICA) 
   real(r8)  flwds_McICA(pcols)          ! Surface longwave down flux (McICA) 
   real(r8)  fsns_McICA(pcols)           ! Surface solar absorbed flux (McICA) 
   real(r8)  fsnt_McICA(pcols)           ! Net column abs solar flux at model top (McICA)  
   real(r8)  flns_McICA(pcols)           ! Srf longwave cooling (up-down) flux (McICA) 
   real(r8)  flnt_McICA(pcols)           ! Net outgoing lw flux at model top (McICA) 
   real(r8)  sols_McICA(pcols)           ! Direct beam solar rad. onto srf (sw) (McICA) 
   real(r8)  soll_McICA(pcols)           ! Direct beam solar rad. onto srf (lw) (McICA) 
   real(r8)  solsd_McICA(pcols)          ! Diffuse solar radiation onto srf (sw) (McICA) 
   real(r8)  solld_McICA(pcols)          ! Diffuse solar radiation onto srf (lw) (McICA) 
   real(r8)  fsntoa_McICA(pcols)         ! Net solar flux at TOA (McICA) 
   real(r8)  fsds_McICA(pcols)           ! Flux Shortwave Downwelling Surface (McICA) 
   real(r8)  fsutoa_McICA(pcols)         ! Flux Shortwave Upwelling TOA (McICA) 
   real(r8)  flut_McICA(pcols)           ! Upward flux at top of model (McICA) 
    
   real(r8) flwdsc_McICA(pcols)          ! Surface longwave down flux (McICA)                            (Clear-sky) 
   real(r8) flntc_McICA(pcols)           ! Net outgoing lw flux at model top (McICA)                     (Clear-sky) 
   real(r8) fsntoac_McICA(pcols)         ! Net solar flux at TOA (McICA)                                 (Clear-sky) 
   real(r8) fsdsc_McICA(pcols)           ! Flux Shortwave Downwelling Surface (McICA)                    (Clear-sky) 
   real(r8) fsntc_McICA(pcols)           ! Net column abs solar flux at model top (McICA)                (Clear-sky) 
   real(r8) fsutoac_McICA(pcols)         ! Flux Shortwave Upwelling TOA (McICA)                          (Clear-sky) 
   real(r8) fsnsc_McICA(pcols)           ! Surface solar absorbed flux (McICA)                           (Clear-sky) 
   real(r8) flutc_McICA(pcols)           ! Upward flux at top of model (McICA)                           (Clear-sky) 
   real(r8) flnsc_McICA(pcols)           ! Srf longwave cooling (up-down) flux (McICA)                   (Clear-sky) 
   real(r8) solin_McICA(pcols)           ! Incident downward solar flux at the top of atmosphere (McICA)   
  ! McICA radiation fields 
   real(r8)  qrs_rf(pcols,pver)       ! Shortwave heating rate (rf) 
   real(r8)  qrl_rf(pcols,pver)       ! Longwave  heating rate (rf) 
   real(r8)  qrs_clr_rf(pcols,pver)   ! Shortwave heating rate (rf) 
   real(r8)  qrl_clr_rf(pcols,pver)   ! Shortwave heating rate (rf) 
   real(r8)  flwds_rf(pcols)          ! Surface longwave down flux (rf) 
   real(r8)  fsns_rf(pcols)           ! Surface solar absorbed flux (rf) 
   real(r8)  fsnt_rf(pcols)           ! Net column abs solar flux at model top (rf)  
   real(r8)  flns_rf(pcols)           ! Srf longwave cooling (up-down) flux (rf) 
   real(r8)  flnt_rf(pcols)           ! Net outgoing lw flux at model top (rf) 
   real(r8)  sols_rf(pcols)           ! Direct beam solar rad. onto srf (sw) (rf) 
   real(r8)  soll_rf(pcols)           ! Direct beam solar rad. onto srf (lw) (rf) 
   real(r8)  solsd_rf(pcols)          ! Diffuse solar radiation onto srf (sw) (rf) 
   real(r8)  solld_rf(pcols)          ! Diffuse solar radiation onto srf (lw) (rf) 
   real(r8)  fsntoa_rf(pcols)         ! Net solar flux at TOA (rf) 
   real(r8)  fsds_rf(pcols)           ! Flux Shortwave Downwelling Surface (rf) 
   real(r8)  fsutoa_rf(pcols)         ! Flux Shortwave Upwelling TOA (rf) 
   real(r8)  flut_rf(pcols)           ! Upward flux at top of model (rf) 
    
   real(r8) flwdsc_rf(pcols)          ! Surface longwave down flux (rf)                            (Clear-sky) 
   real(r8) flntc_rf(pcols)           ! Net outgoing lw flux at model top (rf)                     (Clear-sky) 
   real(r8) fsntoac_rf(pcols)         ! Net solar flux at TOA (rf)                                 (Clear-sky) 
   real(r8) fsdsc_rf(pcols)           ! Flux Shortwave Downwelling Surface (rf)                    (Clear-sky) 
   real(r8) fsntc_rf(pcols)           ! Net column abs solar flux at model top (rf)                (Clear-sky) 
   real(r8) fsutoac_rf(pcols)         ! Flux Shortwave Upwelling TOA (rf)                          (Clear-sky) 
   real(r8) fsnsc_rf(pcols)           ! Surface solar absorbed flux (rf)                           (Clear-sky) 
   real(r8) flutc_rf(pcols)           ! Upward flux at top of model (rf)                           (Clear-sky) 
   real(r8) flnsc_rf(pcols)           ! Srf longwave cooling (up-down) flux (rf)                   (Clear-sky) 
   real(r8) solin_rf(pcols)           ! Incident downward solar flux at the top of atmosphere (rf)   

  ! generated cloud fractions. 
   real(r8)  tot_cf(pcols)               ! Total vertically projected cloud fraction from generator 
   real(r8)  low_cf(pcols)               ! Vertically projected low cloud fraction from generator  
   real(r8)  mid_cf(pcols)               ! Vertically projected mid-level cloud fraction from generator   
   real(r8)  hgh_cf(pcols)               ! Vertically projected high cloud fraction from generator  
   real(r8)  lay_cf(pcols,pver)          ! Vertically projected high cloud fraction from generator  , JXW ADD 
 
   integer i, k, ii, kk 
 
 ! trace gases  
  real(r8) :: n2o(pcols,pver) 
  real(r8) :: ch4(pcols,pver) 
  real(r8) :: cfc11(pcols,pver) 
  real(r8) :: cfc12(pcols,pver) 
 
 ! wangzhili 2011.6.23, for aerosols. 
  integer nspint            ! Num of spctrl intervals across solar spectrum 
  integer naer_groups       ! Num of aerosol groups for optical diagnostics 
  integer sbin              ! Num of size intervals by zhaoshy 
  parameter ( nspint = 17 ) 
  parameter ( naer_groups = 7 )    ! current groupings are sul, sslt, all carbons, all dust, background, volc, and all aerosols 
  real(r8) o3vmr(pcols,pver)   ! Ozone volume mixing ratio 
  real(r8) o3mmr(pcols,pver)   ! Ozone mass mixing ratio 
  real(r8) pbr(pcols,pver)     ! Model mid-level pressures (dynes/cm2) 
  real(r8) pnm(pcols,pver+1)    ! Model interface pressures (dynes/cm2) 
  real(r8) eccf                 ! Earth/sun distance factor 
  real(r8) esat(pcols,pver)        ! saturation vapor pressure 
  real(r8) qsat(pcols,pver)        ! saturation specific humidity 
  real(r8) rh(pcols,pver)          ! level relative humidity (fraction) 
  real(r8) aerosol(pcols, pver, naer_all) ! aerosol mass mixing ratios 
  real(r8) scales(naer_all)               ! scaling factors for aerosols 
  real(r8) tauaer(pcols,0:pver,nspint)    ! aerosol optical depth 
  real(r8) waaer(pcols,0:pver,nspint)     ! aerosol single scattering albedo 
  real(r8) gaaer(pcols,0:pver,nspint)     ! aerosol asymmetry parameter 
  real(r8) faaer(pcols,0:pver,nspint)     ! aerosol forward scattering 
 
  real(r8) tauaer_rf(pcols,0:pver,nspint)    ! aerosol optical depth 
  real(r8) waaer_rf(pcols,0:pver,nspint)     ! aerosol single scattering albedo 
  real(r8) gaaer_rf(pcols,0:pver,nspint)     ! aerosol asymmetry parameter 
  real(r8) faaer_rf(pcols,0:pver,nspint)     ! aerosol forward scattering 
 
  real(r8) aertau(pcols,nspint,naer_groups) ! Aerosol column optical depth 
  real(r8) aerssa(pcols,nspint,naer_groups) ! Aerosol column averaged single scattering albedo 
  real(r8) aerasm(pcols,nspint,naer_groups) ! Aerosol column averaged asymmetry parameter 
  real(r8) aerfwd(pcols,nspint,naer_groups) ! Aerosol column averaged forward scattering 

  real(r8) :: B
  real(r8) :: qi_cld_2
  real(r8) :: rei_j(pcols,pver)
! 
!---------------------------
!zero out output variables.
!
  qrs(:,:) = 0.0_r8
  qrl(:,:) = 0.0_r8
  flwds(:) = 0.0_r8
  fsns(:)  = 0.0_r8
  fsnt(:)  = 0.0_r8
  flns(:)  = 0.0_r8
  flnt(:)  = 0.0_r8
  sols(:)  = 0.0_r8
  soll(:)  = 0.0_r8
  solsd(:) = 0.0_r8
  solld(:) = 0.0_r8
  fsds(:)  = 0.0_r8

  call radinp(lchnk  ,ncol  ,state%pmid  ,state%pint  ,o3vmr  ,pbr  ,pnm  ,eccf  ,o3mmr) 

  call aqsat(state%t, state%pmid, esat, qsat, pcols, ncol, pver, 1, pver)  
  rh(1:ncol,1:pver) = state%q(1:ncol,1:pver,1) / qsat(1:ncol,1:pver) * & 
                      ((1.0 - epsilo) * qsat(1:ncol,1:pver) + epsilo) / & 
                      ((1.0 - epsilo) * state%q(1:ncol,1:pver,1) + epsilo) 

! 
!------------------------------------------------------------------------------ 
! Calculate the aerosol tau, ssa, asm, fwd from CUACE by wangzhili, 2011.6.23 
! 
 ! call get_rf_scales(scales) 
 ! call get_aerosol(lchnk, state%pint, aerosol, scales) 
 ! call set_aerosol_from_prognostics (state, aerosol, scales) 
 ! call aqsat(state%t, state%pmid, esat, qsat, pcols, ncol, pver, 1, pver)  
 ! rh(1:ncol,1:pver) = state%q(1:ncol,1:pver,1) / qsat(1:ncol,1:pver) * & 
 !                     ((1.0 - epsilo) * qsat(1:ncol,1:pver) + epsilo) / & 
 !                     ((1.0 - epsilo) * state%q(1:ncol,1:pver,1) + epsilo) 
 ! call radozn(lchnk  ,ncol  ,state%pmid  ,o3vmr) 
 ! call aero_rad_interf_cuace(lchnk   ,ncol , pnm, pbr, rh, aerosol, state%q, state%zi, state%t, & 
 !               coszrs, aertau, aerssa, aerasm, aerfwd, tauaer, waaer, gaaer, faaer)  
!------------------------------------------------------------------------------ 


  if (radforce) then

 !------------------------------------------------------------------------------ 
 ! Calculate the aerosol tau, ssa, asm, fwd for climatology aerosol data.

     call get_rf_scales(scales) 
     call get_aerosol(lchnk, state%pint, aerosol, scales, state) 
     call set_aerosol_from_prognostics (state, aerosol) 
     call aero_rad_interf_climo(lchnk   ,ncol , pnm, pbr, rh, aerosol, state%q, state%zi, state%t, &
                   coszrs, aertau, aerssa, aerasm, aerfwd, tauaer_rf, waaer_rf, gaaer_rf, faaer_rf)
     call aerosol_indirect(ncol, lchnk, landfrac, state%pmid, state%t, state%q, cld, state%zm, rel) 
  
    ! tauaer_rf(:,:,:)=0.0
    ! waaer_rf(:,:,:)=0.0
    ! gaaer_rf(:,:,:)=0.0
    ! faaer_rf(:,:,:)=0.0
    !  call outfld('TAUSUL  ', aertau(:,11,1), pcols, lchnk)
    !  call outfld('TAUSSLT ', aertau(:,11,2), pcols, lchnk)
    !  call outfld('TAUBC   ', aertau(:,11,3), pcols, lchnk)
    !  call outfld('TAUDST  ', aertau(:,11,4), pcols, lchnk)
    !  call outfld('TAUOC   ', aertau(:,11,5), pcols, lchnk)
 !------------------------------------------------------------------------------ 

   end if 

    call get_int_scales(scales) 
    call get_aerosol(lchnk, state%pint, aerosol, scales, state) 
    call set_aerosol_from_prognostics (state, aerosol) 
    call aero_rad_interf_climo(lchnk   ,ncol , pnm, pbr, rh, aerosol, state%q, state%zi, state%t, &
                  coszrs, aertau, aerssa, aerasm, aerfwd, tauaer, waaer, gaaer, faaer)
    call aerosol_indirect(ncol, lchnk, landfrac, state%pmid, state%t, state%q, cld, state%zm, rel) 

    call outfld('SULOD_v ', aertau(:,11,1) ,pcols,lchnk)
    call outfld('SSLTOD_v', aertau(:,11,2) ,pcols,lchnk)
    call outfld('CAROD_v ', aertau(:,11,3) ,pcols,lchnk)
    call outfld('DUSTOD_v', aertau(:,11,4) ,pcols,lchnk)
    call outfld('BGOD_v  ', aertau(:,11,5) ,pcols,lchnk)
    call outfld('VOLCOD_v', aertau(:,11,6) ,pcols,lchnk)
    call outfld('AEROD_v ', aertau(:,11,7) ,pcols,lchnk)
    call outfld('AERSSA_v', aerssa(:,11,7) ,pcols,lchnk)
    call outfld('AERASM_v', aerasm(:,11,7) ,pcols,lchnk)
    call outfld('AERFWD_v', aerfwd(:,11,7) ,pcols,lchnk)
    call aerosol_diagnostics(state, aerosol) 
   ! tauaer=0.0_r8
   ! waaer=0.0_r8
   ! gaaer=0.0_r8
   ! faaer=0.0_r8
!--------------------------------------------------------------------------- 
! 
! 
! calculate McICA needed fields. 
! 
! Need the ozone profile for this GCM column. 
 
      CALL radozn(lchnk, ncol, state%pmid, o3vmr_McICA) 
      !double o3
      !   o3vmr_McICA(:,:) = o3vmr_McICA(:,:)*2.0_r8
       
! Compute the Earth orbit eccentricity factor and the cosine of the  
! solar zenith angle 
 
      CALL zenith_McICA(calday, clat, clon, coszrs_McICA, eccf_McICA, ncol) 
       
! Get the indices for cloud liquid and ice water 
       
      call cnst_get_ind('CLDLIQ', ixcldliq) 
      call cnst_get_ind('CLDICE', ixcldice) 
 
! Get the latitude and longitude indices 
      call get_lat_all_p(lchnk, ncol, lat) 
      call get_lon_all_p(lchnk, ncol, lon) 
 
! Compute vertical profile of in-cloud liquid and ice water content 
! state%q(i,:,ixcldliq) and state%q(i,:,ixcldice) are the gridbox-mean  
! liquid and ice water contents 
      DO ii = 1,ncol 
         DO kk = 1, pver 
            qc_cld(ii,kk) = state%q(ii,kk,ixcldliq)/max(0.01_r8,cld(ii,kk))  
            qi_cld(ii,kk) = state%q(ii,kk,ixcldice)/max(0.01_r8,cld(ii,kk))  
         END DO 
      END DO 
 
! Get trace gases 
! 
      call trcmix(lchnk   ,ncol   , & 
                  state%pmid    ,n2o,   ch4   ,cfc11  ,cfc12) 
    
      call outfld('N2OMMR ',n2o  ,pcols ,lchnk ) 
      call outfld('CH4MMR ',ch4  ,pcols ,lchnk ) 
      call outfld('CF11MMR',cfc11  ,pcols ,lchnk ) 
      call outfld('CF12MMR',cfc12  ,pcols ,lchnk ) 

! convert mass mixing ratio to volume mixing ratio 
     n2o(:,:) = n2o(:,:) * (mwdry/mwn2o)  
     ch4(:,:) = ch4(:,:) * (mwdry/mwch4)  
     cfc11(:,:) = cfc11(:,:) * (mwdry/mwf11)  
     cfc12(:,:) = cfc12(:,:) * (mwdry/mwf12)  

! JNSC 
! Following the suggestion of Robert Pincus, reseed the the random number generator for each latitude and 
! longitude at each timestep.  Therefore the code will be restartable and get the same results.  Also  
! will remove the correlation between GCM columns due to using the same random number sequence. 
! One thing that bothers me is the use of short random number sequences that are constantly reseeded 
! Currently using RAN3 but should upgrade to Twister in the future 
 
DO i = 1,ncol 
 
! Reseed the random number generator so that the McICA calculation will use the same sequence of random  
! numbers as if the diagnostic calculations was not called 
 
         CALL get_curr_date(yr,mon,day,tod) 
 
         hour   = INT(tod/(60.0*60.0)) 
         minute = INT((tod-hour*60.0*60.0)/60.0) 
         sec    = INT(tod-hour*60.0*60.0-minute*60.0) 
          
         ran_seed = lat(i)*PLON+lon(i)+yr+mon+day+hour+minute+sec 
         ran_seed = -ran_seed 
         CALL ranset(ran_seed) 
! Call cloud generator 
         call t_startf ('MSCCLD') 
         CALL MSC_CLD_GEN_DRIVER_MRO(cld(i,:), state%t(i,:), state%q(i,:,1), qi_cld(i,:), & 
                                     qc_cld(i,:), state%zm(i,:), t_rad_McICA, qv_rad_McICA, & 
                                     qi_rad_McICA, qc_rad_McICA) 
         call t_stopf ('MSCCLD') 
 
! Call cloud diagnostics 
         call McICA_CLD_DIAG(qi_rad_McICA, qc_rad_McICA, state%pint(i,:), tot_cf(i), low_cf(i), & 
                                                                           mid_cf(i), hgh_cf(i), & 
                                                                           lay_cf(i,:))
                                                                           
! compute REI_J according to Wyser(1998), with IWC and T
  !!!!!! IWC = qi(:,:,:)/abs(zint(iz)-zint(iz+1)
  !!!!!! IWC0 = 50.0 (g/m^3)
   do k=1,pver
     if (qi_cld(i,k)>1.0e-8) then
       qi_cld_2=qi_cld(i,k)*state%pdel(i,k)/9.8
       B = -2.0 + 10.0**(-3.0)*(273.0-state%t(i,k))**1.5*log10(qi_cld_2/abs(state%zi(i,k)-state%zi(i,k+1))/50.0)
       rei_j(i,k)=377.4+203.3*B+37.91*B**2.0+2.3696*B**3.0
       if (rei_j(i,k)<0.0) rei_j(i,k)=0.0_r8
     end if
   end do ! k
! compute surface temperature from input lwup.

  ts(i)=sqrt(sqrt(lwup(i)/stebol))

! call radiation process 
   
  !------------------------------------------------------------------------------ 
  if (radforce) then
     ran_seed = lat(i)*PLON+lon(i)+yr+mon+day+hour+minute+sec 
     ran_seed = -ran_seed 
     CALL ranset(ran_seed) 

     call RAD_DRIVER_McICA_ZHANG (t_rad_McICA, ts(i), qv_rad_McICA,rh(i,:), state%pmid(i,:),          & 
                            state%pint(i,:), state%pdel(i,:), state%ps(i),              & 
                            o3vmr_McICA(i,:), state%zm(i,:), state%zi(i,:),             & 
                            qc_rad_McICA, qi_rad_McICA, coszrs_McICA(i), eccf_McICA,    & 
                            n2o(i,:),  ch4(i,:),  cfc11(i,:),  cfc12(i,:),              &              
                            ocnfrac(i), landfrac(i), icefrac(i), rel(i,:), rei(i,:),    & 
                            aldir(i), aldif(i), asdir(i), asdif(i),     & 
                            tauaer_rf(i,:,:),waaer_rf(i,:,:),gaaer_rf(i,:,:),faaer_rf(i,:,:),           & 
                            qrs_rf(i,:), qrl_rf(i,:), qrs_clr_rf(i,:), qrl_clr_rf(i,:), & 
                            flwds_rf(i), flwdsc_rf(i),  & 
                            fsns_rf(i), fsnt_rf(i), flns_rf(i), flnt_rf(i),       & 
                            flntc_rf(i), sols_rf(i), soll_rf(i), solsd_rf(i),     & 
                            solld_rf(i), fsntoa_MCICA(i), fsntoac_rf(i), fsds_rf(i), & 
                            fsdsc_rf(i), fsntc_rf(i), fsutoa_rf(i), fsutoac_rf(i),& 
                            fsnsc_rf(i), flut_rf(i), flutc_rf(i), flnsc_rf(i),    & 
                            solin_rf(i),lat(i), clat(i) )       

   end if
   !--------------------------------------------------------------------------- 

   ! Calculate the aerosol tau, ssa, asm, fwd for climatology aerosol data.
         ran_seed = lat(i)*PLON+lon(i)+yr+mon+day+hour+minute+sec 
         ran_seed = -ran_seed 
         CALL ranset(ran_seed) 

         call t_startf ('RADMcICA') 
         call RAD_DRIVER_McICA_ZHANG (t_rad_McICA, ts(i), qv_rad_McICA,rh(i,:), state%pmid(i,:),          & 
                                state%pint(i,:), state%pdel(i,:), state%ps(i),              & 
                                o3vmr_McICA(i,:), state%zm(i,:), state%zi(i,:),             & 
                                qc_rad_McICA, qi_rad_McICA, coszrs_McICA(i), eccf_McICA,    & 
                                n2o(i,:),  ch4(i,:),  cfc11(i,:),  cfc12(i,:),              &              
                                ocnfrac(i), landfrac(i), icefrac(i), rel(i,:), rei(i,:),    & 
                                aldir(i), aldif(i), asdir(i), asdif(i),     & 
                                tauaer(i,:,:),waaer(i,:,:),gaaer(i,:,:),faaer(i,:,:),           & 
                                qrs_McICA(i,:), qrl_McICA(i,:), qrs_clr_McICA(i,:), qrl_clr_McICA(i,:), & 
                                flwds_McICA(i), flwdsc_McICA(i),  & 
                                fsns_McICA(i), fsnt_McICA(i), flns_McICA(i), flnt_McICA(i),       & 
                                flntc_McICA(i), sols_McICA(i), soll_McICA(i), solsd_McICA(i),     & 
                                solld_McICA(i), fsntoa_MCICA(i), fsntoac_McICA(i), fsds_McICA(i), & 
                                fsdsc_McICA(i), fsntc_McICA(i), fsutoa_McICA(i), fsutoac_McICA(i),& 
                                fsnsc_McICA(i), flut_McICA(i), flutc_McICA(i), flnsc_McICA(i),    & 
                                solin_McICA(i),lat(i), clat(i) )       
         call t_stopf ('RADMcICA')  

!! *cpair to make the units of heating rates consistent with the GCM's requirment. 
       qrl_McICA(i,:)=qrl_McICA(i,:)*cpair 
       qrs_McICA(i,:)=qrs_McICA(i,:)*cpair 
       qrl_clr_McICA(i,:)=qrl_clr_McICA(i,:)*cpair 
       qrs_clr_McICA(i,:)=qrs_clr_McICA(i,:)*cpair 
 
  END DO ! i 

      qrs(:,:)  = qrs_McICA(:,:) 
      qrl(:,:)  = qrl_McICA(:,:) 
      flwds(:)  = flwds_McICA(:) 
      fsns(:)   = fsns_McICA(:) 
      fsnt(:)   = fsnt_McICA(:) 
      flns(:)   = flns_McICA(:) 
      flnt(:)   = flnt_McICA(:) 
      sols(:)   = sols_McICA(:) 
      soll(:)   = soll_McICA(:) 
      solsd(:)  = solsd_McICA(:) 
      solld(:)  = solld_McICA(:) 
      fsds(:)   = fsds_McICA(:) 

! output fields to history files. 
     ! call outfld('GQRL   ',qrl_McICA/cpair     , pcols,lchnk) 
     ! call outfld('GQRS   ',qrs_McICA/cpair     , pcols,lchnk) 
     ! call outfld('QRLC  ',qrl_clr_McICA/cpair     , pcols,lchnk) 
     ! call outfld('QRSC  ',qrs_clr_McICA/cpair     , pcols,lchnk) 
     ! call outfld('GFSDS  ',fsds_McICA  ,pcols,lchnk) 
     ! call outfld('GFSNT  ',fsnt_McICA  ,pcols,lchnk) 
     ! call outfld('GFSNS  ',fsns_McICA  ,pcols,lchnk) 
     ! call outfld('GFSNTOA',fsntoa_McICA,pcols,lchnk) 
     ! call outfld('GSOLS  ',sols_McICA  ,pcols,lchnk) 
     ! call outfld('GSOLL  ',soll_McICA  ,pcols,lchnk) 
     ! call outfld('GSOLSD ',solsd_McICA ,pcols,lchnk) 
     ! call outfld('GSOLLD ',solld_McICA ,pcols,lchnk) 
     ! call outfld('GFLNT  ',flnt_McICA  ,pcols,lchnk) 
     ! call outfld('GFLUT  ',flut_McICA  ,pcols,lchnk) 
     ! call outfld('GFLNS  ',flns_McICA  ,pcols,lchnk) 
     ! call outfld('GSWCF  ',fsntoa_McICA-fsntoac_McICA ,pcols,lchnk) 
     ! call outfld('GLWCF  ',flutc_McICA-flut_McICA ,pcols,lchnk) 
     ! call outfld('GFSDSC  ',fsdsc_McICA  ,pcols,lchnk) 
     ! call outfld('GFSNTC  ',fsntc_McICA  ,pcols,lchnk) 
     ! call outfld('GFSNSC  ',fsnsc_McICA  ,pcols,lchnk) 
     ! call outfld('GFSNTOAC',fsntoac_McICA,pcols,lchnk) 
     ! call outfld('GFLNTC  ',flntc_McICA  ,pcols,lchnk) 
     ! call outfld('GFLUTC  ',flutc_McICA  ,pcols,lchnk) 
     ! call outfld('GFLWDSC ',flwdsc_McICA ,pcols,lchnk)     ! NEW VARIABLE BY McICA 
     ! call outfld('GFLWDS  ',flwds_McICA ,pcols,lchnk)      ! NEW VARIABLE BY McICA 
     ! call outfld('GFLNSC  ',flnsc_McICA  ,pcols,lchnk) 
     ! call outfld('GSOLIN  ',solin_McICA  , pcols,lchnk) 

     ! call outfld('GENCFTOT ',tot_cf ,pcols ,lchnk )
     ! call outfld('GENCFLOW ',low_cf ,pcols ,lchnk )
     ! call outfld('GENCFMED ',mid_cf ,pcols ,lchnk )
     ! call outfld('GENCFHGH ',hgh_cf ,pcols ,lchnk )
     ! call outfld('GENCFLAY  ',lay_cf ,pcols ,lchnk )

      call outfld('QRL   ',qrl_McICA/cpair     , pcols,lchnk) 
      call outfld('QRS   ',qrs_McICA/cpair     , pcols,lchnk) 
      call outfld('QRLC  ',qrl_clr_McICA/cpair     , pcols,lchnk) 
      call outfld('QRSC  ',qrs_clr_McICA/cpair     , pcols,lchnk) 
      call outfld('FSDS  ',fsds_McICA  ,pcols,lchnk) 
      call outfld('FSNT  ',fsnt_McICA  ,pcols,lchnk) 
      call outfld('FSNS  ',fsns_McICA  ,pcols,lchnk) 
      call outfld('FSNTOA',fsntoa_McICA,pcols,lchnk) 
      call outfld('SOLS  ',sols_McICA  ,pcols,lchnk) 
      call outfld('SOLL  ',soll_McICA  ,pcols,lchnk) 
      call outfld('SOLSD ',solsd_McICA ,pcols,lchnk) 
      call outfld('SOLLD ',solld_McICA ,pcols,lchnk) 
      call outfld('FLNT  ',flnt_McICA  ,pcols,lchnk) 
      call outfld('FLUT  ',flut_McICA  ,pcols,lchnk) 
      call outfld('FLNS  ',flns_McICA  ,pcols,lchnk) 
      call outfld('SWCF  ',fsntoa_McICA-fsntoac_McICA ,pcols,lchnk) 
      call outfld('LWCF  ',flutc_McICA-flut_McICA ,pcols,lchnk) 
      call outfld('FSDSC  ',fsdsc_McICA  ,pcols,lchnk) 
      call outfld('FSNTC  ',fsntc_McICA  ,pcols,lchnk) 
      call outfld('FSNSC  ',fsnsc_McICA  ,pcols,lchnk) 
      call outfld('FSNTOAC',fsntoac_McICA,pcols,lchnk) 
      call outfld('FLNTC  ',flntc_McICA  ,pcols,lchnk) 
      call outfld('FLUTC  ',flutc_McICA  ,pcols,lchnk) 
      call outfld('FLWDSC ',flwdsc_McICA ,pcols,lchnk)     ! NEW VARIABLE BY McICA 
      call outfld('FLWDS  ',flwds_McICA ,pcols,lchnk)      ! NEW VARIABLE BY McICA 
      call outfld('FLNSC  ',flnsc_McICA  ,pcols,lchnk) 
      call outfld('SOLIN  ',solin_McICA  , pcols,lchnk) 

      call outfld('CLDTOT ',tot_cf ,pcols ,lchnk )
      call outfld('CLDLOW ',low_cf ,pcols ,lchnk )
      call outfld('CLDMED ',mid_cf ,pcols ,lchnk )
      call outfld('CLDHGH ',hgh_cf ,pcols ,lchnk )
      call outfld('CLOUD  ',lay_cf ,pcols ,lchnk )
    if (radforce) then
      call outfld('QRS_RF ',qrs_rf  ,pcols,lchnk) 
      call outfld('FSNT_RF ',fsnt_rf  ,pcols,lchnk) 
      call outfld('FSNS_RF ',fsns_rf  ,pcols,lchnk) 
      call outfld('FSNTC_RF',fsntc_rf  ,pcols,lchnk) 
      call outfld('FSNSC_RF',fsnsc_rf  ,pcols,lchnk) 
    end if
       

end subroutine
