	SUBROUTINE SWTRANts(ams,tau,omg,albdir,albdif,fsol,FLXD,FLXU,    &
                FDDIR,FDDIF,g)


!
!     * MAY 14,2004 - J.LI,L.SOLHEIM..
!     * DEC 10,2010 - X. W. Jing. albdir, albdif, FDDIR, FDDIF
!----------------------------------------------------------------------!
!     DELTA-EDDINGTON APPROXIMATION AND ADDING PROCESS FOR CLEAR AND   !
!     ALL SKY, THE ADDING METHOD BY COAKLEY ET AL (1983). THIS CODE    !
!     CAN DEAL WITH SOLAR RADIATIVE TRANSFER THROUGH ATMOSPHERE WITH   !
!     PROPER TREATMENT OF CLOUD OVERLAP (RANDOM + MAXIMUM OR RANDOM +  !
!     SLANTWISE) AND CLOUD SUB-GRID VARIABILITY. THE THEORY FOR ADDING,!
!     CLOUD OVERLAP LI AND DOBBIE (2003). CLOUD SUB-GRID VARIABILITY   !
!     SIMILAR TO WITH ADJUSTMENT OF CLOUD OPTICAL DEPTH                !
!                                                                      !
!     REFL:    REFLECTIVITY (1) CLEAR SKY; (2) ALL SKY                 !
!     TRAN:    TRANSMITIVITY                                           !
!     CUMDTR:  DIRECT TRANSMISSION FOR MULT-LAYERS                     !
!     TAUA:    AEROSOL OPTICAL DEPTH                                   !
!     TAUR:    RAYLEIGH OPTICAL DEPTH                                  !
!     TAUG:    GASEOUS OPTICAL DEPTH                                   !
!     TAUOMA:  AEROSOL OPTICAL DEPTH TIMES AEROSOL SINGLE SCATTERING   !
!              ALBEDO                                                  !
!     TAUOMGA: TAUOMA TIMES AEROSOL ASYMMETRY FACTOR                   !
!     F1:      SQUARE OF AEROSOL ASYMMETRY FACTOR                      !
!     F2:      SQUARE OF CLOUD ASYMMETRY FACTOR                        !
!     TAUCS:   CLOUD OPTICAL DEPTH                                     !
!     TAUOMC:  CLOUD OPTICAL DEPTH TIMES CLOUD SINGLE SCATTERING ALBEDO!
!     TAUOMGC: TAUOMC TIMES CLOUD ASYMMETRY FACTOR                     !
!     CLD:     CLOUD FRACTION                                          !
!     CLDM:    MAXIMUM PORTION IN EACH CLOUD BLOCK, IN WHICH THE EXACT !
!              SOLUTION FOR SUBGRID VARIABILITY IS APPLIED             !
!     A1:      VARIOUS RELATION FOR CLOUD OVERLAP                      !
!     RMU:     COS OF SOLAR ZENITH ANGLE                               !
!     C1 @ C2: TWO FACTORS NOT DEPENDENT ON IB AND IG CALCULATED       !
!              OUTSIDE FOR EFFICIENCY                                  !
!     ALBSUR:  SURFACE ALBEDO                                          !
!     CSALB:   CLEAR-SKY SURFACE ALBEDO                                !
!     NBLK:    NUMBER OF CLOUD BLOCKS ACCOUNTED FROM SURFACE           !
!     NCT:     THE HIGHEST CLOUD TOP LEVEL FOR THE LONGITUDE AND       !
!              LATITUDE LOOP (ILG)                                     !
!     RDF:     LAYER DIFFUSE REFLECTION                                !
!     TDF:     LAYER DIFFUSE TRANSMISSION                              !
!     RDR:     LAYER DIRECT REFLECTION                                 !
!     TDR:     LAYER DIRECT TRANSMISSION                               !
!     DTR:     DIRECT TRANSMISSION                                     !
!     RMDF:    BLOCK DIFFUSE REFLECTION FROM MODEL TOP LEVEL           !
!     TMDR:    BLOCK DIRECT TRANSMISSION FROM MODEL TOP LEVEL          !
!     RMUR:    BLOCK DIRECT REFLECTION FROM MODEL BOTTOM LEVEL         !
!     RMUF:    BLOCK DIFFUSE REFLECTION FROM MODEL BOTTOM LEVEL        !
!----------------------------------------------------------------------!
      use shr_kind_mod, only: r8 => shr_kind_r8
      use ppgrid
      IMPLICIT REAL (A-H,O-Z),   &
      INTEGER (I-N)
       ! IMPLICIT NONE
	INTEGER,PARAMETER::LEV1=1,LAY=pver+3,LEV=LAY+1
	real(r8),intent(in)::TAU(LAY), OMG(LAY),AMS,GALB,FSOL,     &
       g(3,lay),albdir,albdif
	real(r8),intent(out)::flxd(LEV),flxu(LEV),FDDIR,FDDIF


      REAL(r8) REFL(2,LEV), TRAN(2,LEV), CUMDTR(4,LEV),              &
          TRAN0
      REAL(r8) TAUA(LAY), TAUR(LAY), TAUG(LAY), TAUOMA(LAY),         &
          TAUOMGA(LAY), F1(LAY), F2(LAY), TAUCS(LAY),                &
          TAUOMC(LAY), TAUOMGC(LAY), CLD(LAY),                       &
          ALBSUR, CSALB
      REAL(r8) RDF(4,LAY), TDF(4,LAY), RDR(4,LAY),                   &
          TDR(4,LAY), DTR(4,LAY), RMDF(4,LEV),                       &
          TMDR(4,LEV), RMUR(4,LEV), RMUF(4,LEV)
!      INTEGER NBLK(ILG, LAY), NCT(ILG)
!
!----------------------------------------------------------------------!
!     COMBINE THE OPTICAL PROPERTIES FOR SOLAR,                        !
!     1, AEROSOL + RAYLEIGH + GAS; 2, CLOUD + AEROSOL + RAYLEIGH + GAS !
!     CALCULATE THE DIRECT AND DIFFUSE REFLECTION AND TRANSMISSION IN  !
!     THE SCATTERING LAYERS USING THE DELTA-EDDINGTON METHOD.          !
!----------------------------------------------------------------------!
!
!	DO K=1,100
!      WRITE(*,*)G(2,K,2),K
!	ENDDO
!	PAUSE
      DO K = LEV1, LAY
	OOM = OMG(K)
      OG = G(2,K)
      F = G(3,K)
      AMU1 = 0.5773503
      AMU11 = AMU1 * AMU1

      AMU0 = AMS

      AMU00 = AMU0 * AMU0
      AMU01 = AMU0 * AMU1
      OTAU   =  TAU(K)

      OM     =  OOM * (1.0 - F) / (1. - OOM * F)
      TAU1    =  OTAU * (1.0 - OOM * F)
      G1      = (OG - F) / (1.0 - F) 

!	OM=OOM
!	TAU1=OTAU
!	G1=OG

      COM    =  1.0 - OM +1E-10 
      COMG   =  1.0 - OM * G1

      AK     =  SQRT(COM * COMG) / AMU1
      TR0    =  EXP(- TAU1 / AMU0)
	TR11   =  EXP(-AK * TAU1)
      SP     =  2.0 * OM
      SM     =  - 6.0 * OM * G1 * AMU01
      Z1     =  SM / AMU01 - COMG * SP / AMU11
      Z2     =  SP / AMU01 - COM * SM / AMU11
      DEM    =  AMU00 / (1.0 - AMU00 * AK * AK)
      EPSI   =  0.5 * (Z1 + Z2) * DEM
      GAMA   =  0.5 * (Z1 - Z2) * DEM
	DTR(1,K)                  =  EXP( - TAU1 / AMS)

      AMUK   =  2.0 * AMU1 * AK
!      DIM    = (COMG + COM + AMUK) * TR1 - (COMG + COM - AMUK) / TR1
      RDF(1,K)    =  OM * (1.0 - G1) * (1 - TR11*TR11) /              &
      ((COMG + COM + AMUK) - (COMG + COM - AMUK) *TR11*TR11)
 
      TDF(1,K)    =  2.0 * AMUK*TR11 / ((COMG + COM + AMUK)  -        &
      (COMG + COM - AMUK) *TR11* TR11 )


      RDR(1,K)    =  0.5 * AMU1 * (EPSI * (1.0 - TDF(1,K) * TR0) -    &
               GAMA * RDF(1,K)) / AMU0
      TDR(1,K)    =  0.5 * AMU1 *                                      &
           (GAMA * (TR0 - TDF(1,K)) - EPSI * RDF(1,K) * TR0) / AMU0    & 
	     +DTR(1,K)



!	WRITE(*,*)RDF(1,K),TDF(1,K),RDR(1,K),TDR(1,K),K


      END DO
!
!	PAUSE
!
!----------------------------------------------------------------------!
!     INITIALIZATION FOR THE FIRST LEVEL (LEV1).                       !
!----------------------------------------------------------------------!
!
        TRAN0                	=  1.0
	    ATRAN0                  =  1.0 - TRAN0
        TMDR(1,LEV1)            =  TRAN0
        RMDF(1,LEV1)            =  ATRAN0
        CUMDTR(1,LEV1)          =  TRAN0
        TMDR(2,LEV1)            =  TRAN0
        RMDF(2,LEV1)            =  ATRAN0
        CUMDTR(2,LEV1)          =  TRAN0
        TMDR(3,LEV1)            =  TRAN0
        RMDF(3,LEV1)            =  ATRAN0
        CUMDTR(3,LEV1)          =  TRAN0
        TMDR(4,LEV1)            =  TRAN0
        RMDF(4,LEV1)            =  ATRAN0
        CUMDTR(4,LEV1)          =  TRAN0
!
!----------------------------------------------------------------------!
!     INITIALIZATION FOR THE GROUND LAYER.                             !
!----------------------------------------------------------------------!
!
        CSALB                   =  0.0
	    ALBSUR                  =  0.0
	    RMUR(1,LEV)             =  ALBDIR
        RMUF(1,LEV)             =  ALBDIF
        RMUR(2,LEV)             =  ALBSUR
        RMUF(2,LEV)             =  ALBSUR
        RMUR(3,LEV)             =  ALBSUR
        RMUF(3,LEV)             =  ALBSUR
        RMUR(4,LEV)             =  ALBSUR
        RMUF(4,LEV)             =  ALBSUR

        CUMDTR(2,LEV)           =  0.0
        CUMDTR(3,LEV)           =  0.0
        CUMDTR(4,LEV)           =  0.0

!
!----------------------------------------------------------------------!
!     ADD THE LAYERS DOWNWARD FROM THE SECOND LAYER TO THE SURFACE.    !
!----------------------------------------------------------------------!
!
      DO K = LEV1 + 1, LEV  
        KM1 = K - 1
        L = LEV - K + LEV1
        LP1 = L + 1

          DMM                     =  TDF(1,KM1) /                     &
                                   (1.0 - RDF(1,KM1) * RMDF(1,KM1))
          FMM                     =  RMDF(1,KM1) * DMM                
          TMDR(1,K)             =  CUMDTR(1,KM1) * (TDR(1,KM1) +     &
                                    RDR(1,KM1) * FMM) +              &
                                   (TMDR(1,KM1) - CUMDTR(1,KM1)) *   &
                                    DMM
          RMDF(1,K)             =  RDF(1,KM1) + TDF(1,KM1) * FMM      
          CUMDTR(1,K)           =  CUMDTR(1,KM1) * DTR(1,KM1)




!
!----------------------------------------------------------------------!
!     ADD THE LAYERS UPWARD FROM ONE LAYER ABOVE SURFACE TO THE LEV1.  !
!----------------------------------------------------------------------!
!
          UMM                     =  TDF(1,L) /                    &
                                   (1.0 - RDF(1,L) * RMUF(1,LP1))
          FMM                     =  RMUF(1,LP1) * UMM
          RMUR(1,L)             =  RDR(1,L) + DTR(1,L) *             &
                                   RMUR(1,LP1) * UMM + (TDR(1,L) -   &
                                   DTR(1,L)) * FMM
          RMUF(1,L)             =  RDF(1,L) + TDF(1,L) * FMM


      END DO
!
!----------------------------------------------------------------------!
!     ADD DOWNWARD TO CALCULATE THE RESULTANT REFLECTANCE AND          !
!     TRANSMITTANCE AT FLUX LEVELS.                                    !
!----------------------------------------------------------------------!
!
      DO K = LEV1, LEV
        KM1 = K - 1

          DMM                     =  1.0 /                       &
                                   (1.0 - RMUF(1,K) * RMDF(1,K)) 
          X                       =  CUMDTR(1,K) * RMUR(1,K)
          Y                       =  TMDR(1,K) - CUMDTR(1,K)
          TRAN(1,K)             =  CUMDTR(1,K) +                 &
                                   (X * RMDF(1,K) + Y) * DMM
          REFL(1,K)             =  (X + Y * RMUF(1,K)) * DMM

      END DO
  
  
  	DO k=1,LEV			  
          FLXU(K)				  = REFL(1,K)*AMS*FSOL
          FLXD(K)                 = TRAN(1,K)*AMS*FSOL
	ENDDO
          FDDIR                = CUMDTR(1,LEV)*AMS*FSOL
          FDDIF                = FLXD(LEV)- FDDIR	
      
	RETURN

      END SUBROUTINE
