#include <misc.h>
#include <params.h>
module bcc_aerosols_intr

  use shr_kind_mod,only: r8 => shr_kind_r8
  use pmgrid,      only: masterproc
  use pmgrid,      only: plev, plevp
  use ppgrid,      only: pcols,pver
  use constituents,only: ppcnst,cnst_name, cnst_get_ind
  use abortutils,  only: endrun

  implicit none

  private          ! Make default type private to the module

  save

  !
  ! Public interfaces
  !

  public :: mz_aero_initialize                           
  public :: mz_aero_wet_intr                             ! interface to wet deposition

!---------
  integer ::              num_mz_aerosols
  integer, allocatable :: mz_aerosol_ids(:)    ! (/ id_cb2, id_oc2, id_so4, id_sa1, id_sa2, id_sa3, id_sa4 /)

  logical ::          use_cam_sulfchem = .true.
  logical, public  :: do_cam_sulfchem
  integer, pointer :: id_so2, id_so4, id_dms, id_o3, id_h2o2, id_oh, id_no3, id_ho2
  integer, target  :: spc_ids(8)

contains

  !===============================================================================
  subroutine mz_aero_initialize( )
    use bcc_sulchem,          only : inisulchem
    use bcc_chem_mods,        only : gas_wetdep_list, gas_wetdep_cnt
    use bcc_chem_mods,        only : aer_wetdep_list, aer_wetdep_cnt
!
    implicit none

    integer :: m                                
    integer :: mm  
	integer :: id                             

    !-----------------------------------------------------------------------

    num_mz_aerosols = 0

!---
    do m = 1,aer_wetdep_cnt
       do mm=1,gas_wetdep_cnt
          if ( trim(gas_wetdep_list(mm)) == trim(aer_wetdep_list(m)) ) then
             call endrun('mz_aero_initialize: '//trim(aer_wetdep_list(m))//' is in both het_lst and aer_wetdep_list')
          endif
       enddo

       call cnst_get_ind ( aer_wetdep_list(m), id )

       if ( id < 1 ) then 
          write(*,*) 'mz_aero_initialize: '//trim(aer_wetdep_list(m))//' does not exit in simulation'
          call endrun('mz_aero_initialize: invalid washout species')
       else
          num_mz_aerosols = num_mz_aerosols + 1
       endif
    enddo
!--
    if ( num_mz_aerosols .ne. aer_wetdep_cnt ) then 
          write(*,*) 'mz_aero_initialize: aer_wetdep_cnt =/= num_mz_aerosols, STOP'
          call endrun
    endif
!--
    allocate( mz_aerosol_ids(num_mz_aerosols) )

    do m = 1, num_mz_aerosols    ! num_mz_aerosols = aer_wetdep_cnt
       call cnst_get_ind ( aer_wetdep_list(m), mz_aerosol_ids(m) )
    enddo

!------------------------

    id_so2  => spc_ids(1)
    id_so4  => spc_ids(2)
    id_o3   => spc_ids(3)
    id_h2o2 => spc_ids(4)
    id_oh   => spc_ids(5)
    id_no3  => spc_ids(6)
    id_ho2  => spc_ids(7)
    id_dms  => spc_ids(8)

    call cnst_get_ind ( 'SO2',  id_so2 )
    call cnst_get_ind ( 'SO4',  id_so4 )
    call cnst_get_ind ( 'O3',   id_o3  )
    call cnst_get_ind ( 'H2O2', id_h2o2 )
    call cnst_get_ind ( 'OH',   id_oh  )
    call cnst_get_ind ( 'NO3',  id_no3 )
    call cnst_get_ind ( 'HO2',  id_ho2 )
    call cnst_get_ind ( 'DMS',  id_dms )

    use_cam_sulfchem = .true.
    do_cam_sulfchem  =  use_cam_sulfchem .and. all(spc_ids > 0)    ! do_cam_sulfchem =.true.
    
    if ( do_cam_sulfchem ) then
       call inisulchem()
    endif

  end subroutine mz_aero_initialize

  !===============================================================================
  subroutine mz_aero_wet_intr ( lchnk, ncol, t, pmid, pdel, pmiddry, q, totcond, &
                                lat,   lon,  zm,   nstep, dt, cme, prain,    &
                                evapr, cldv, cldvcu, cldvst, cldc, cldn, fracis, &
				calday, cmfdqr, evapc, conicw, rainmr, dlf)
    !----------------------------------------------------------------------- 
    !-----------------------------------------------------------------------
    use history,          only: outfld
    use physics_types,    only: physics_state, physics_ptend
    use bcc_wetdep,       only: wetdepa_v1, wetdepa_v2
    use bcc_sulchem,      only: sulf_chemdr, cldychmini, dicor
    use physconst,        only: gravit
    use constituents,     only: cnst_mw
    use physconst,        only: amass => mwdry         ! molecular weight dry air ~ kg/kmole
    use physconst,        only: boltz                  ! J/K/molecule
    use bcc_chem_mods,    only: dust_names
    use bcc_chem_mods,    only: progseasalts_names
    use bcc_chem_mods,    only: fix_mass

    !-----------------------------------------------------------------------
    implicit none
    !-----------------------------------------------------------------------
    !
    ! Arguments:
    !
    real(r8),            intent(in)  :: dt             ! time step
    integer,  intent(in) :: nstep
    integer  :: lchnk                              ! chunk identifier
    integer  :: ncol                               ! number of atmospheric columns
    real(r8), intent(inout)    :: q(pcols,pver, ppcnst)        ! temperature (K)
    real(r8), intent(in) :: t(pcols,pver)        ! temperature (K)
    real(r8), intent(in) :: pmid(pcols,pver)     ! pressure at layer midpoints
    real(r8), intent(in) :: pmiddry(pcols,pver)     ! pressure at layer midpoints
    real(r8), intent(in) :: pdel(pcols,pver)     ! pressure difference across layers
    real(r8), intent(in) :: totcond(pcols, pver) ! total condensate
    real(r8), intent(in) :: lat(pcols)                  ! latitude 
    real(r8), intent(in) :: lon(pcols)                  ! latitude 
    real(r8), intent(in) :: zm(pcols,pver)            ! local condensation of cloud water

    real(r8), intent(in) :: cme(pcols,pver)            ! local condensation of cloud water
    real(r8), intent(in) :: prain(pcols,pver)          ! production of rain
    real(r8), intent(in) :: evapr(pcols,pver)          ! evaporation of rain
    real(r8), intent(in) :: cldn(pcols,pver)           ! cloud fraction
    real(r8), intent(in) :: cldc(pcols,pver)           ! convective cloud fraction
    real(r8), intent(in) :: cldv(pcols,pver)           ! cloudy volume undergoing scavenging
    real(r8), intent(in) :: cldvcu(pcols,pver)         ! Convective precipitation area at the top interface of current layer
    real(r8), intent(in) :: cldvst(pcols,pver)         ! Stratiform precipitation area at the top interface of current layer
    real(r8), intent(in) :: conicw(pcols, pver)
    real(r8), intent(in) :: cmfdqr(pcols, pver)
    real(r8), intent(in) :: evapc(pcols, pver)         ! Evaporation rate of convective precipitation  
    real(r8), intent(in) :: rainmr(pcols, pver)        ! rain mixing ratio
    real(r8), intent(in) :: dlf(pcols,pver)            ! Detrainment of convective condensate
    real(r8), intent(inout) :: fracis(pcols,pver,ppcnst) ! fraction of transported species that are insoluble
    real(r8), intent(in) :: calday        ! current calendar day

    !
    ! Local variables
    !
    integer  :: m                                  ! tracer index
    integer  :: ixcldice, ixcldliq
    real(r8) :: obuf(1)
    real(r8) :: iscavt(pcols, pver)
    integer  :: yr, mon, day, ncsec
    integer  :: ncdate
    integer  :: mm
    integer  :: nphob
    real(r8), dimension(pcols,pver) :: h2o23d,o3,oh,no3,ho2,so2,so4,dms,h2o2,ekso2,ekh2o2

    real(r8), dimension(pcols,pver) ::&
         ph,          &! ph of drops
         hion          ! Hydrogen ion concentration in drops

    integer ::&
         indcp(pcols,pver),    &! Indices of cloudy grid points
         ncldypts(pver)         ! Number of cloudy grid points

    real(r8) :: icwmr2(pcols,pver) ! in cloud water mixing ratio for hack scheme
    real(r8) :: icscavt(pcols, pver)
    real(r8) :: isscavt(pcols, pver)
    real(r8) :: bcscavt(pcols, pver)
    real(r8) :: bsscavt(pcols, pver)
    real(r8) :: sol_factb, sol_facti
    real(r8) :: sol_factic(pcols,pver)
    real(r8) :: sol_factbi, sol_factii, sol_factiic
    real(r8) :: xhnm(pcols, pver)
    real(r8) :: sflx(pcols)            ! deposition flux
    real(r8) :: dicorfac(pcols)        ! factors to increase HO2 diurnal averages
    integer  :: i,k
    real(r8) :: scavcoef(pcols,pver) ! Dana and Hales coefficient (/mm) (0.1)

    real(r8) :: qtend(pcols,pver,ppcnst)
    !-----------------------------------------------------------------------

    call cnst_get_ind('CLDICE', ixcldice)
    call cnst_get_ind('CLDLIQ', ixcldliq)
!   totcond(:ncol, :) = state%q(:ncol,:,ixcldliq) + state%q(:ncol,:,ixcldice)

    if ( num_mz_aerosols > 0 ) then

       ! Wet deposition of aerosol species: except dust_names and progseasalts_names
	   !                                    CB2, OC2, SO4

       scavcoef(:ncol,:)= 0.1_r8
       qtend(:,:,:)     = 0.0 

       do m = 1, num_mz_aerosols
          mm = mz_aerosol_ids(m)

           if (mm > 0 .and. (.not. any(dust_names==cnst_name(mm)))  .and. (.not. any(progseasalts_names==cnst_name(mm))) ) then

             sol_factb = 0.3_r8
             sol_facti = 0.3_r8

!             call wetdepa_v1( t, pmid, q, pdel,  &
!                  cldn, cldc, cmfdqr, conicw, prain, cme,                     &
!                  evapr, totcond, q(:,:,mm), dt,            &
!                  qtend(:,:,mm), iscavt, cldv, fracis(:,:,mm), sol_factb, ncol, &
!                  scavcoef, icscavt=icscavt, isscavt=isscavt, bcscavt=bcscavt, bsscavt=bsscavt, &
!                  sol_facti_in=sol_facti)

             call wetdepa_v1( t, pmid, q, pdel,  &
                  cldn, cldc, cmfdqr, conicw, prain, cme,                     &
                  evapr, totcond, q(:,:,mm), dt,            &
                  qtend(:,:,mm), iscavt, cldv, fracis(:,:,mm), sol_factb, ncol, &
                  scavcoef, icscavt=icscavt, isscavt=isscavt, bcscavt=bcscavt, bsscavt=bsscavt, &
                  sol_facti_in=sol_facti)

             sflx(:)=0._r8
             do k=1,pver
                do i=1,ncol
	           q(i,k,mm) = q(i,k,mm) + qtend(i,k,mm)*dt 
	           q(i,k,mm) = max(q(i,k,mm),1.e-20)
                   sflx(i)   = sflx(i)   + qtend(i,k,mm)*pdel(i,k)/gravit
                enddo
             enddo

             ! export deposition fluxes to coupler, the total amount from wet deposition to ground
!wtw             select case (trim(cnst_name(mm)))
!wtw             case('CB2')
!wtw                do i = 1, ncol
!wtw                    srf_wetdep(i) = max(-sflx(i), 0._r8)
!wtw                 end do
!wtw             case('OC2')
!wtw                do i = 1, ncol
!wtw                    srf_wetdep(i) = max(-sflx(i), 0._r8)
!wtw                end do
!wtw           end select

          endif
       end do
    endif

   if ( do_cam_sulfchem ) then
       so4(:ncol,:)  = q(:ncol,:,id_so4)              ! updated above
       so2(:ncol,:)  = q(:ncol,:,id_so2) 
       dms(:ncol,:)  = q(:ncol,:,id_dms)
       h2o2(:ncol,:) = q(:ncol,:,id_h2o2)

       ! need oh,ho2,no3 in molec/cm3, o3 in kg/kg

!cam5  o3(:ncol,:) = q(:ncol,:,id_o3)*amass/fix_mass(id_o3) ! vmr
       o3(:ncol,:) = q(:ncol,:,id_o3)*amass/cnst_mw(id_o3)  ! vmr

       ! vmr -> molec/cm3
       xhnm(:ncol,:) = 1.e-6_r8 * pmiddry(:ncol,:) / (boltz*t(:ncol,:))

       oh(:ncol,:)  = q(:ncol,:,id_oh)* (amass/cnst_mw(id_oh )) * xhnm(:ncol,:)
       no3(:ncol,:) = q(:ncol,:,id_no3)* (amass/cnst_mw(id_no3)) * xhnm(:ncol,:)
       ho2(:ncol,:) = q(:ncol,:,id_ho2)* (amass/cnst_mw(id_ho2)) * xhnm(:ncol,:)  !  molec/cm3 

       ! alter HO2 from diurnal average to instantaneous value

       call dicor( calday, lat, ncol, dicorfac)

       do k = 1, pver
          do i = 1, ncol
             ho2(i,k) = ho2(i,k)*dicorfac(i)
          end do
       end do

       icwmr2(:,:) = 0._r8

       call cldychmini( t,  pmid, totcond, rainmr, cldn, &
            cldv, conicw, icwmr2, so2, so4, &
            ph, hion, ekso2, ekh2o2, indcp, &
            ncldypts, ncol )

       call sulf_chemdr( t, pmid, totcond,    rainmr,    cldn,     cldv,           &
                         conicw,  icwmr2,     indcp,     ncldypts, hion,                      &
                         so2,     so4,        dms,       h2o2,     ekso2,    ekh2o2,         &
                         o3,      h2o23d,     oh,        no3,      ho2,      q(:,:,1), &
                         zm,  dt, calday,     lat,       lon,      ncol,     lchnk  )

!      qtend(:ncol,:,id_so2) = (so2(:ncol,:)-q(:ncol,:,id_so2))/dt
!      qtend(:ncol,:,id_so4) = (so4(:ncol,:)-q(:ncol,:,id_so4))/dt
!      qtend(:ncol,:,id_dms) = (dms(:ncol,:)-q(:ncol,:,id_dms))/dt
!      qtend(:ncol,:,id_h2o2) = (h2o2(:ncol,:)-q(:ncol,:,id_h2o2))/dt
!      
!---------
! wtw
       q(:ncol,:,id_so2) = max(so2(:ncol,:), 1.e-20)
       q(:ncol,:,id_so4) = max(so4(:ncol,:), 1.e-20)
       q(:ncol,:,id_dms) = max(dms(:ncol,:), 1.e-20)
!      q(:ncol,:,id_h2o2)= max(h2o2(:ncol,:), 1.e-20)
!
! Cancel the change of q(:ncol,:,id_h2o2) is a test to aviod the impact of aero on h2o2
!---
    endif
    return
    end subroutine mz_aero_wet_intr

end module bcc_aerosols_intr
