#include <misc.h>
#include <params.h>

module bcc_airplane
!--------------------------------------------------------------------
!	... airplane insitu emission sources
!--------------------------------------------------------------------
      use shr_kind_mod,      only : r8 => shr_kind_r8
      use bcc_netcdf
      use bcc_const_mozart,  only : d2r, rearth, pi
      use bcc_chem_utls,     only : get_spc_ndx

      use bcc_sim_chm,       only : latwts, sim_chm_data    !zf test
      use bcc_chem_mods,     only : adv_mass

!add by zf 2008.03.11
      use pmgrid,         only: masterproc, plon, plat, iam
      use ioFileMod,      only: getfil
      use abortutils,     only: endrun
      use error_messages, only: handle_ncerr
      use ppgrid,         only: pcols, begchunk, endchunk
      use phys_grid,      only: scatter_field_to_chunk

#if ( defined SPMD )
  use mpishorthand      , only: mpicom, mpiint, mpir8
#endif

!end zf

      implicit none

      private 
      save

      public airpl_src
	  public airpl_set
	  public pnobdy,    &
			 pcobdy,    &
			 pch4bdy,   &
			 air_altitude    !add by zf 2008.03.11

      character(len=256), public  :: bndairplane         ! full pathname for Mozart aircraft emissions 

      real, allocatable :: &
                 pnobdy(:,:,:), &
                 pcobdy(:,:,:), &
                 pch4bdy(:,:,:), &
                 air_altitude(:)
      integer   ::   nlat, nlon, nlev

      contains


  subroutine airpl_set( lchnk, ncol, no_ndx, co_ndx, xno_ndx, cldtop, zint_abs, extfrc) 

    use ppgrid,           only : pver, pcols
    use history,          only : outfld
    use bcc_chem_mods,    only : extcnt

    implicit none
    integer, intent(in)  :: lchnk, ncol, no_ndx, co_ndx, xno_ndx
    real(r8), intent(in) :: cldtop(pcols), zint_abs(pcols,pver+1)
    real(r8), intent(inout) :: extfrc(pcols,pver,extcnt)
!
! Local Variables
!
    real(r8), dimension(pcols,pver) :: no_air, co_air

    real(r8) :: ztab_top, ztab_bot, zdel, zdeli, frac, zlev_top, zlev_bot
    integer  :: nlev
    integer  :: cldind, kk, i, k
    logical  :: has_airpl_src

    no_air(:,:) = 0._r8
    co_air(:,:) = 0._r8

    has_airpl_src = .true.

    if(has_airpl_src) then
       !---------------------------------------------------------------------
       !     ... Add the airplane emissions; must do vertical interpolation
       !---------------------------------------------------------------------
       ztab_top = maxval( air_altitude )
       ztab_bot = minval( air_altitude )
       nlev     = size(air_altitude) - 1

       !---------------------------------------------------------------------
       !     ... add the airplane emissions; must do vertical interpolation
       !         Note: the interpolation code is conserving and assumes the
       !               aircraft emission vertical grid is uniform with a
       !               one kilometer spacing
       !---------------------------------------------------------------------
       level_loop : do k = 1,pver
          long_loop : do i = 1,ncol
             zlev_top = zint_abs(i,k)                 ! altitude at top of model level (km)
             zlev_bot = zint_abs(i,k+1)               ! altitude at bottom of model level (km)
             zdel = (zlev_top - zlev_bot) * 1.e5_r8  ! model level thickness (cm)
             zdeli = 1./zdel
             if( zlev_bot <= ztab_top .and. zlev_top >= ztab_bot ) then
                do kk = 1,nlev
                   if( zlev_bot <= air_altitude(kk+1) .and. zlev_top >= air_altitude(kk) ) then
                      frac = (min( zlev_top, air_altitude(kk+1) ) - max( zlev_bot, air_altitude(kk) )) &
                           /(air_altitude(kk+1) - air_altitude(kk)) ! *del_alti(kk)
                      if( no_ndx > 0 ) then
                            extfrc(i,k,no_ndx) = extfrc(i,k,no_ndx) + frac * pnobdy(i,kk,lchnk) * zdeli
                            no_air(i,k) = frac * pnobdy(i,kk,lchnk) * zdeli
                      end if

                      if( xno_ndx > 0 ) then
                            extfrc(i,k,xno_ndx) = extfrc(i,k,xno_ndx) + frac * pnobdy(i,kk,lchnk) * zdeli
                      end if

                      if( co_ndx > 0 ) then
                         extfrc(i,k,co_ndx) = extfrc(i,k,co_ndx) + frac * pcobdy(i,kk,lchnk) * zdeli
                         co_air(i,k) = frac * pcobdy(i,kk,lchnk) * zdeli
                      end if
!--
! wtw add
!
!                      if( ch4_ndx > 0 ) then
!                          extfrc(i,k,ch4_ndx) = extfrc(i,k,ch4_ndx) + frac * pch4bdy(i,kk,lchnk) * zdeli
!                      end if
!
!                      if( no2_ndx > 0 ) then
!                         if( idx3_NO2 > 0 ) then
!                            extfrc(i,k,no2_ndx) = extfrc(i,k,no2_ndx) + em3data(i,k,idx3_NO2) ! mol.cm-3.s-1
!						 endif
!                      end if
!
!                      if( cb2_ndx > 0 ) then
!                         if( idx3_CB2 > 0 ) then
!                            extfrc(i,k,cb2_ndx) = extfrc(i,k,cb2_ndx) + em3data(i,k,idx3_CB2)  ! mol.cm-3.s-1
!                         endif
!                      end if
!
!                      if( so2_ndx > 0 ) then
!                         if( idx3_SO2 > 0 ) then
!                            extfrc(i,k,so2_ndx) = extfrc(i,k,so2_ndx) + em3data(i,k,idx3_SO2) ! mol.cm-3.s-1
!                         endif
!                      end if
!
!                      if( so4_ndx > 0 ) then
!                         if( idx3_SO4 > 0 ) then
!                            extfrc(i,k,so4_ndx) = extfrc(i,k,so4_ndx) + em3data(i,k,idx3_SO4) ! mol.cm-3.s-1
!                         endif
!                      end if
!
! wtw
!-------------------------
                   end if
                end do
             end if
             if( k == pver ) then
                do kk = 1,nlev
                   if( zlev_bot > air_altitude(kk) ) then
                      frac = (min( zlev_bot, air_altitude(kk+1) ) - air_altitude(kk)) &
                           /(air_altitude(kk+1) - air_altitude(kk)) ! *del_alti(kk)
                      if( no_ndx > 0 ) then
                             extfrc(i,k,no_ndx) = extfrc(i,k,no_ndx) + frac * pnobdy(i,kk,lchnk) * zdeli
                             no_air(i,k) = frac * pnobdy(i,kk,lchnk) * zdeli
                      end if

                      if( xno_ndx > 0 ) then
                             extfrc(i,k,xno_ndx) = extfrc(i,k,xno_ndx) + frac * pnobdy(i,kk,lchnk) * zdeli
                      end if

                      if( co_ndx > 0 ) then
                         extfrc(i,k,co_ndx) = extfrc(i,k,co_ndx) + frac * pcobdy(i,kk,lchnk) * zdeli
                         co_air(i,k) = frac * pcobdy(i,kk,lchnk) * zdeli
                      end if

!--------
! wtw
!                      if( ch4_ndx > 0 ) then
!                         extfrc(i,k,ch4_ndx) = extfrc(i,k,ch4_ndx) + frac * pch4bdy(i,kk,lchnk) * zdeli
!                      end if
!
!                      if( no2_ndx > 0 ) then
!                         if( idx3_NO2 > 0 ) then
!                            extfrc(i,k,no2_ndx) = extfrc(i,k,no2_ndx) + em3data(i,k,idx3_NO2) ! mol.cm-3.s-1
!			 endif
!                      end if
!
!                      if( cb2_ndx > 0 ) then
!                         if( idx3_CB2 > 0 ) then
!                           extfrc(i,k,cb2_ndx) = extfrc(i,k,cb2_ndx) + em3data(i,k,idx3_cb2) ! mol.cm-3.s-1
!			 endif
!                      end if
!
!                      if( so2_ndx > 0 ) then
!                         if( idx3_SO2 > 0 ) then
!                            extfrc(i,k,so2_ndx) = extfrc(i,k,so2_ndx) + em3data(i,k,idx3_SO2) ! mol.cm-3.s-1
!			 endif
!                      end if
!
!                      if( so4_ndx > 0 ) then
!                         if( idx3_SO4 > 0 ) then
!                            extfrc(i,k,so4_ndx) = extfrc(i,k,so4_ndx) + em3data(i,k,idx3_SO4) ! mol.cm-3.s-1
!                         endif
!                      end if
! wtw
!----------
                   else
                      exit
                   end if
                end do
             end if
          end do long_loop
       end do level_loop
      
    end if
  end subroutine airpl_set

      subroutine airpl_src( bndairplane, latwts_file )
 
!-----------------------------------------------------------------------
! 	... initialize airplane emissions
!	    note: the emissions are read in in units of molecules/cm**2/s
!	          on a vertically resolved grid.
!		  conversion to units of molec/cm**3/s is done in setext
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!	... local variables
!-----------------------------------------------------------------------
      integer :: i, k, j
      integer :: ndims
      integer :: ncid, vid, ierr
      integer :: dimid_lat, dimid_lon, dimid_lev
      integer :: co_ndx, ch4_ndx
      integer :: dimid(3)
      real, allocatable :: lat(:), lon(:)
      real    :: total(3) = 0., total_wrk
      real    :: seq, sf(plat), factor

      real, allocatable :: &
                 pno_in(:,:,:), &
                 pco_in(:,:,:), &
                 pch4_in(:,:,:)

      real, allocatable :: &
                 pno(:,:,:), &
                 pco(:,:,:), &
                 pch4(:,:,:)

      character(len=256) :: locfn    ! netcdf local filename to open   add by zf 2008.03.11
	  character(len=*)   :: bndairplane
	  character(len=*)   :: latwts_file
!-----------------------------------------------------------------------
!	... open netcdf file
!-----------------------------------------------------------------------

!add by zf 2008.03.11

    if (masterproc) then
      call getfil (bndairplane, locfn)
      call wrap_open(locfn,0,ncid)

!end zf

!-----------------------------------------------------------------------
!       ... get grid dimensions from file
!-----------------------------------------------------------------------
      call handle_ncerr( nf_inq_dimid( ncid, 'lat', dimid_lat ), &
                         'airpl_src: failed to find dimension lat' )
      call handle_ncerr( nf_inq_dimlen( ncid, dimid_lat, nlat ), &
                         'airpl_src: failed to get length of dimension lat' )
      allocate( lat(nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: lat allocation error = ',ierr
         call endrun
      end if
      call handle_ncerr( nf_inq_varid( ncid, 'lat', vid ), &
                         'airpl_src: failed to find variable lat' )
      call handle_ncerr( nf_get_var_double( ncid, vid, lat ), &
                         'airpl_src: failed to read variable lat' )
      lat(:nlat) = lat(:nlat) * d2r
 
      call handle_ncerr( nf_inq_dimid( ncid, 'lon', dimid_lon ), &
                         'airpl_src: failed to find dimension lon' )
      call handle_ncerr( nf_inq_dimlen( ncid, dimid_lon, nlon ), &
                         'airpl_src: failed to get length of dimension lon' )
      allocate( lon(nlon), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: lon allocation error = ',ierr
         call endrun
      end if
      call handle_ncerr( nf_inq_varid( ncid, 'lon', vid ), &
                         'airpl_src: failed to find variable lon' )
      call handle_ncerr( nf_get_var_double( ncid, vid, lon ), &
                         'airpl_src: failed to read variable lon' )
      lon(:nlon) = lon(:nlon) * d2r
 
      call handle_ncerr( nf_inq_dimid( ncid, 'altitude', dimid_lev ), &
                         'airpl_src: failed to find dimension altitude' )
      call handle_ncerr( nf_inq_dimlen( ncid, dimid_lev, nlev ), &
                         'airpl_src: failed to get length of dimension altitude' )
    end if

#if (defined SPMD )
    call mpibcast( nlev, 1, mpiint, 0, mpicom )
#endif

      allocate( air_altitude( nlev+1 ), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: air_altitude allocation error = ',ierr
         call endrun
      end if

    if (masterproc) then
      call handle_ncerr( nf_inq_varid( ncid, 'altitude', vid ), &
                         'airpl_src: failed to find variable altitude' )
      call handle_ncerr( nf_get_var_double( ncid, vid, air_altitude(1:nlev) ), &
                         'airpl_src: failed to read variable altitude' )
      air_altitude(nlev+1) = air_altitude(nlev) + (air_altitude(nlev) - air_altitude(nlev-1))

    end if

#if (defined SPMD )
    call mpibcast( nlon, 1, mpiint, 0, mpicom )
    call mpibcast( nlat, 1, mpiint, 0, mpicom )
    do i = 1, nlev+1
      call mpibcast( air_altitude(i), 1, mpir8, 0, mpicom )
    end do
#endif

      allocate( pno(nlon,nlat,nlev), stat=ierr )
      allocate( pco(nlon,nlat,nlev), stat=ierr )
      allocate( pch4(nlon,nlat,nlev), stat=ierr )

      allocate( pno_in(nlon,nlev,nlat), stat=ierr )
      allocate( pco_in(nlon,nlev,nlat), stat=ierr )
      allocate( pch4_in(nlon,nlev,nlat), stat=ierr )

      allocate( pnobdy(pcols,nlev,begchunk:endchunk), stat=ierr )
      allocate( pcobdy(pcols,nlev,begchunk:endchunk), stat=ierr )
      allocate( pch4bdy(pcols,nlev,begchunk:endchunk), stat=ierr )

!----
! compute latwtsbdy
!
!     call sim_chm_data( latwts_file )
!-----------------------------------------------------------------------
!	... read emissions
!-----------------------------------------------------------------------
    if (masterproc) then
      call handle_ncerr( nf_inq_varid( ncid, 'nox', vid ), &
                         'airpl_src: failed to get id for variable nox' )
      call handle_ncerr( nf_inq_varndims( ncid, vid, ndims ), &
                         'airpl_src: failed to get number of dimensions for variable nox' )
      if ( ndims /= 3 ) then
         write(*,*) 'airpl_src: variable nox has ndims = ',ndims,', expecting 3'
         call endrun
      end if
      call handle_ncerr( nf_inq_vardimid( ncid, vid, dimid ), &
                         'airpl_src: failed to get dimension ids for variable nox' )
      if( dimid(1) /= dimid_lon  .or. dimid(2) /= dimid_lat .or.  dimid(3) /= dimid_lev ) then
            write(*,*) 'airpl_src: dimensions in wrong order for variable nox'
            write(*,*) '...      expecting (lon, lat, lev)'
            call endrun
      end if
      call handle_ncerr( nf_get_vara_double( ncid, vid, &
                                             (/ 1, 1, 1/), &                   ! start
                                             (/ nlon, nlat, nlev /), &   ! count
                                             pno ), &
                         'airpl_src: failed to read variable nox' )

      call handle_ncerr( nf_inq_varid( ncid, 'co', vid ), &
                         'airpl_src: failed to get id for variable co' )
      call handle_ncerr( nf_inq_varndims( ncid, vid, ndims ), &
                         'airpl_src: failed to get number of dimensions for variable co' )
      if ( ndims /= 3 ) then
         write(*,*) 'read_sflx: variable co has ndims = ',ndims,', expecting 3'
         call endrun
      end if
      call handle_ncerr( nf_inq_vardimid( ncid, vid, dimid ), &
                         'airpl_src: failed to get dimension ids for variable co' )
      if ( dimid(1) /= dimid_lon .or. dimid(2) /= dimid_lat .or. dimid(3) /= dimid_lev ) then
            write(*,*) 'airpl_src: dimensions in wrong order for variable co'
            write(*,*) '...      expecting (lon, lat, lev)'
            call endrun
      end if
      call handle_ncerr( nf_get_vara_double( ncid, vid, &
                                             (/ 1, 1, 1/), &                   ! start
                                             (/ nlon, nlat, nlev /), &  ! count
                                             pco ), &
                         'airpl_src: failed to read variable co' )
       pch4(:,:,:) = 0.

      call handle_ncerr( nf_close( ncid ), 'airpl_src: failed to close netcdf file' )

!-----------------------------------------------------------------------
!       ... get global emission from this source
!-----------------------------------------------------------------------
!
!wtw  pi = 4.*ATAN( 1. )
!
      seq = 2.*pi*1.e4*rearth**2/real(plon)
      do j = 1,plat
         sf(j) = seq*latwts(j)
      end do

      factor = 86400. * 365. &   ! sec / year
             / 6.022e23 &        ! molec / mole
             * 1.e-12            ! tg / g

      total(:) = 0.
      do j = 1,plat
         total(1) = total(1) &
                  + sum( pno( :plon, j, :nlev ) ) * sf(j)
         total(2) = total(2) &
                  + sum( pco( :plon, j, :nlev ) ) * sf(j)
         total(3) = total(3) &
                  + sum( pch4( :plon, j, :nlev ) ) * sf(j)
      end do
!-----------------------------------------------------------------------
!       ... convert totals from molec cm^-2 s^-1 to tg y^-1
!-----------------------------------------------------------------------
      total(1) = total(1) * 14.00674 * factor
      co_ndx = get_spc_ndx( 'CO' )
      if( co_ndx > 0 ) then
         total(2) = total(2) * adv_mass(co_ndx) * factor
      end if
      ch4_ndx = get_spc_ndx( 'CH4' )
      if( ch4_ndx > 0 ) then
         total(3) = total(3) * adv_mass(ch4_ndx) * factor
      end if

#ifdef USE_MPI
      do j = 1,3
         call mpi_allreduce( total(j), total_wrk, 1, mpi_double_precision, mpi_sum, mpi_comm_comp, ierr )
         if( ierr /= mpi_success ) then
            write(*,*) 'airpl_src: mpi_allreduce for total failed; error = ',ierr
            call endrun
         end if
         total(j) = total_wrk
      end do
#endif
         write(*,'('' airpl_src aircraft emissions: '',a6,'' = '',f10.3,1x,a6)') 'no',total(1),'tgn/y'
	 if( co_ndx > 0 ) then
            write(*,'('' airpl_src aircraft emissions: '',a6,'' = '',f10.3,1x,a6)') 'co',total(2),'tg/y'
	 end if
	 if( ch4_ndx > 0 ) then
            write(*,'('' airpl_src aircraft emissions: '',a6,'' = '',f10.3,1x,a6)') 'ch4',total(3),'tg/y'
	 end if

         do i = 1, nlon 
         do j = 1, nlat 
         do k = 1, nlev 
           pno_in(i,k,j) = pno(i,j,k)
           pco_in(i,k,j) = pco(i,j,k)
           pch4_in(i,k,j) = pch4(i,j,k)
         end do
         end do
         end do

        call scatter_field_to_chunk(1,nlev,1,nlon,pno_in,pnobdy)
        call scatter_field_to_chunk(1,nlev,1,nlon,pco_in,pcobdy)
        call scatter_field_to_chunk(1,nlev,1,nlon,pch4_in,pch4bdy)

#if (defined SPMD )
      else

        call scatter_field_to_chunk(1,nlev,1,nlon,pno_in,pnobdy)
        call scatter_field_to_chunk(1,nlev,1,nlon,pco_in,pcobdy)
        call scatter_field_to_chunk(1,nlev,1,nlon,pch4_in,pch4bdy)
#endif

      end if

      deallocate(pno)
      deallocate(pco)
      deallocate(pch4)
      deallocate(pno_in)
      deallocate(pco_in)
      deallocate(pch4_in)

      end subroutine airpl_src

end module bcc_airplane
