#include <misc.h>
#include <params.h>

      module bcc_chem_mods
!--------------------------------------------------------------
! ... Basic chemistry parameters and arrays
!--------------------------------------------------------------
      use shr_kind_mod, only : r8 => shr_kind_r8
      use constituents, only : ppcnst
      implicit none

      public :: set_sim_dat

      save
      integer, parameter :: gas_pcnst = 59+16,       & ! number of "gas phase" species
	                    hetcnt    = 59+16,       & ! = gas_pcnst,  number of heterogeneous processes
                            phtcnt = 73,         & ! number of photolysis reactions
                            rxntot = 232, &  ! 232+7,      & ! number of total reactions
                            gascnt = 159, &        ! number of gas phase reactions
                            nabscol = 2, &         ! number of absorbing column densities
                            nfs = 2, &             ! number of "fixed" species
                            relcnt = 0, &          ! number of relationship species
                            grpcnt = 0, & ! number of group members
                            nzcnt = 496, & ! number of non-zero matrix entries
                            extcnt  = 11 + 4, & ! number of species with external forcing
                            clscnt1 = 17, &   ! 17 + 16, & ! number of species in explicit class
                            clscnt2 = 0, & ! number of species in hov class
                            clscnt3 = 0, & ! number of species in ebi class
                            clscnt4 = 42, & ! number of species in implicit class
                            clscnt5 = 0, & ! number of species in rodas class
                            indexm = 1, & ! index of total atm density in invariant array
                            indexh2o = 0, & ! index of water vapor density
                            clsze = 1, & ! loop length for implicit chemistry
                            nslvd = 0
      character(len=8) :: slvd_lst(max(1,nslvd))
!--
! wtw
      integer            :: imozart

      integer            :: map2chm(gas_pcnst)          ! index map to/from chemistry/constituents list
 
      logical            :: cnst_fixed_ubc(gas_pcnst) = .false.  ! included the water vapor, cloud water

      integer, parameter :: drydep_cnt = 15
      integer            :: drydep_mapping(drydep_cnt) 

      character(len=8), dimension(drydep_cnt), parameter :: & ! constituent names
!         drydep_list    = (/'CH2O    ','CH3OOH  ','CO      ','H2O2    ','HNO3    ', &
!                            'HO2NO2  ','NH3     ','NO      ','NO2     ','O3      ', &
!		      	    'CH4     ',  &
!		            'CB1     ','CB2     ','OC1     ','OC2     ','DST01   ', &
!		            'DST02   ','DST03   ','DST04   '/)

         drydep_list    = (/'CH2O    ','CH3OOH  ','CO      ','H2O2    ','HNO3    ', &
                            'HO2NO2  ','NH3     ','NO      ','NO2     ','O3      ', &
                            'CH4     ',  &
                            'CB1     ','CB2     ','OC1     ','OC2     '/)

       integer, parameter :: gas_wetdep_cnt = 12
       integer            :: gas_wetdep_mapping (gas_wetdep_cnt) 

       character(len=8), dimension(gas_wetdep_cnt), parameter :: & ! constituent names
          gas_wetdep_list    = (/'H2O2    ','HNO3    ','CH2O    ','CH3OOH  ','HO2NO2  ', &
                                 'CLONO2  ','BRONO2  ','HCL     ','N2O5    ','HOCL    ', &
		                 'HOBR    ','HBR     '/)

       integer, parameter :: aer_wetdep_cnt = 11
       integer            :: aer_wetdep_mapping (aer_wetdep_cnt) 

       character(len=8), dimension(aer_wetdep_cnt), parameter :: & ! constituent names
          aer_wetdep_list    = (/'CB2     ','OC2     ','SO4     ','DST01   ','DST02   ', &
		                 'DST03   ','DST04   ','SSLT01  ','SSLT02  ','SSLT03  ', &
				 'SSLT04  '/)

       integer, parameter :: fstrat_cnt = 9
       integer            :: fstrat_mapping(fstrat_cnt)
       character(len=8), dimension(fstrat_cnt), parameter :: & ! constituent names
       fstrat_list = (/'O3      ','NO      ','NO2     ','HNO3    ','CO      ', &
	               'CH4     ','N2O     ','N2O5    ','H2O     '/)

       integer, parameter :: srfems_cnt = 15
       character(len=8), dimension(srfems_cnt), parameter :: & ! constituent names
       srfems_lst    = (/'CH2O    ','CH4     ','CO      ','H2      ','N2O     ', &
	                 'NO      ','CH3COOH ','NH3     ','DMS     ','SO2     ', &
			 'SO4     ','CB1     ','CB2     ','OC1     ','OC2     '/)
!--
    character(len=8), dimension(gas_pcnst), parameter :: & ! constituent names
          solsym = (/'O3      ','O       ','O1D     ','O2      ','O2_1S   ', &
                     'O2_1D   ','N2O     ','N       ','NO      ','NO2     ', &
                     'NO3     ','HNO3    ','HO2NO2  ','N2O5    ','CH4     ', &
                     'CH3O2   ','CH3OOH  ','CH2O    ','CO      ','H2      ', &
                     'H       ','OH      ','HO2     ','H2O2    ','CLY     ', &
                     'BRY     ','CL      ','CL2     ','CLO     ','OCLO    ', &
                     'CL2O2   ','HCL     ','HOCL    ','CLONO2  ','BRCL    ', &
                     'BR      ','BRO     ','HBR     ','HOBR    ','BRONO2  ', &
                     'CH3CL   ','CH3BR   ','CFC11   ','CFC12   ','CFC113  ', &
                     'HCFC22  ','CCL4    ','CH3CCL3 ','CF3BR   ','CF2CLBR ', &
                     'CO2     ','N2p     ','O2p     ','Np      ','Op      ', &
                     'NOp     ','e       ','N2D     ','H2O     ',            &
                     'SO2     ','SO4     ','DMS     ','OC1     ','OC2     ', &
	   	     'CB1     ','CB2     ','SSLT01  ','SSLT02  ','SSLT03  ', &
	   	     'SSLT04  ','DST01   ','DST02   ','DST03   ','DST04   ', &
	       	     'NH3     '/)
!
   real(r8), dimension(gas_pcnst), parameter :: &             ! 59+16
           adv_mass  = (/ 47.99819946_r8, 15.99940014_r8, 15.99940014_r8, 31.99880028_r8, 31.99880028_r8, &
                          31.99880028_r8, 44.01287842_r8, 14.00673962_r8, 30.00613976_r8, 46.00553894_r8, &
                          62.00494003_r8, 63.01234055_r8, 79.01174164_r8, 108.0104828_r8, 16.04059982_r8, &
                          47.03200150_r8, 48.03939819_r8, 30.02519989_r8, 28.01040077_r8, 2.014800072_r8, &
                          1.007400036_r8, 17.00679970_r8, 33.00619888_r8, 34.01359940_r8, 100.9168472_r8, &
                          99.71685028_r8, 35.45270157_r8, 70.90540314_r8, 51.45209885_r8, 67.45149994_r8, &
                          102.9041977_r8, 36.46009827_r8, 52.45949936_r8, 97.45764160_r8, 115.3566971_r8, &
                          79.90399933_r8, 95.90339661_r8, 80.91139984_r8, 96.91079712_r8, 141.9089355_r8, &
                          50.48590088_r8, 94.93720245_r8, 137.3675079_r8, 120.9132080_r8, 187.3753052_r8, &
                          86.46790314_r8, 153.8217926_r8, 133.4022980_r8, 148.9102020_r8, 165.3645020_r8, &
                          44.00979996_r8, 28.01347923_r8, 31.99880028_r8, 14.00673962_r8, 15.99940014_r8, &
                          30.00613976_r8, 0.5485669826E-03_r8, 14.00673962_r8, 18.01420021_r8 ,           &
                          64.06479645_r8, 96.06359863_r8, 62.13240051_r8, 12.01099968_r8, 12.01099968_r8, &
			  12.01099968_r8, 12.01099968_r8, 58.44246674_r8, 58.44246674_r8, 58.44246674_r8, &
			  58.44246674_r8, 135.0640411_r8, 135.0640411_r8, 135.0640411_r8, 135.0640411_r8, &
			  17.02894000_r8/)

      character(len=8), dimension(hetcnt), parameter :: &
           het_lst  = (/ 'O3      ','O       ','O1D     ','O2      ','O2_1S   ', &
                         'O2_1D   ','N2O     ','N       ','NO      ','NO2     ', &
                         'NO3     ','HNO3    ','HO2NO2  ','N2O5    ','CH4     ', &
                         'CH3O2   ','CH3OOH  ','CH2O    ','CO      ','H2      ', &
                         'H       ','OH      ','HO2     ','H2O2    ','CLY     ', &
                         'BRY     ','CL      ','CL2     ','CLO     ','OCLO    ', &
                         'CL2O2   ','HCL     ','HOCL    ','CLONO2  ','BRCL    ', &
                         'BR      ','BRO     ','HBR     ','HOBR    ','BRONO2  ', &
                         'CH3CL   ','CH3BR   ','CFC11   ','CFC12   ','CFC113  ', &
                         'HCFC22  ','CCL4    ','CH3CCL3 ','CF3BR   ','CF2CLBR ', &
                         'CO2     ','N2p     ','O2p     ','Np      ','Op      ', &
                         'NOp     ','e       ','N2D     ','H2O     ',            &
                         'SO2     ','SO4     ','DMS     ','OC1     ','OC2     ', &
  		         'CB1     ','CB2     ','SSLT01  ','SSLT02  ','SSLT03  ', &
  		         'SSLT04  ','DST01   ','DST02   ','DST03   ','DST04   ', &
 	                 'NH3     '/)

   character(len=8), dimension(nfs), parameter :: &
       inv_lst  = (/ 'M       ', 'N2      '/)

!wtw   integer, parameter :: rxt_tag_cnt = 142 + 7   ! 142 in WACCM
   integer, parameter :: rxt_tag_cnt = 142   ! 142 in WACCM

   character(len=16), dimension(rxt_tag_cnt), parameter :: &  
                    rxt_tag_lst = (/ 'jo2_a           ', 'jo2_b           ', 'jo3_a           ', 'jo3_b           ', &
                                     'jn2o            ', 'jno             ', 'jno_i           ', 'jno2            ', &
                                     'jn2o5_a         ', 'jn2o5_b         ', 'jhno3           ', 'jno3_a          ', &
                                     'jno3_b          ', 'jho2no2_a       ', 'jho2no2_b       ', 'jch3ooh         ', &
                                     'jch2o_a         ', 'jch2o_b         ', 'jh2o_a          ', 'jh2o_b          ', &
                                     'jh2o_c          ', 'jh2o2           ', 'jcl2            ', 'jclo            ', &
                                     'joclo           ', 'jcl2o2          ', 'jhocl           ', 'jhcl            ', &
                                     'jclono2_a       ', 'jclono2_b       ', 'jbrcl           ', 'jbro            ', &
                                     'jhobr           ', 'jbrono2_a       ', 'jbrono2_b       ', 'jch3cl          ', &
                                     'jccl4           ', 'jch3ccl3        ', 'jcfcl3          ', 'jcf2cl2         ', &
                                     'jcfc113         ', 'jhcfc22         ', 'jch3br          ', 'jcf3br          ', &
                                     'jcf2clbr        ', 'jco2            ', 'jch4_a          ', 'jch4_b          ', &
                                     'jeuv_1          ', 'jeuv_2          ', 'jeuv_3          ', 'jeuv_4          ', &
                                     'jeuv_5          ', 'jeuv_6          ', 'jeuv_7          ', 'jeuv_8          ', &
                                     'jeuv_9          ', 'jeuv_10         ', 'jeuv_11         ', 'jeuv_12         ', &
                                     'jeuv_13         ', 'jeuv_14         ', 'jeuv_15         ', 'jeuv_16         ', &
                                     'jeuv_17         ', 'jeuv_18         ', 'jeuv_19         ', 'jeuv_20         ', &
                                     'jeuv_21         ', 'jeuv_22         ', 'jeuv_23         ', 'jeuv_24         ', &
                                     'jeuv_25         ', 'usr_O_O2        ', 'cph1            ', 'usr_O_O         ', &
                                     'cph18           ', 'cph19           ', 'cph20           ', 'cph21           ', &
                                     'ag2             ', 'cph22           ', 'cph23           ', 'cph24           ', &
                                     'ag1             ', 'cph17           ', 'cph16           ', 'cph29           ', &
                                     'cph25           ', 'cph26           ', 'cph27           ', 'cph28           ', &
                                     'cph8            ', 'cph12           ', 'cph13           ', 'tag_NO2_NO3     ', &
                                     'usr_N2O5_M      ', 'usr_HNO3_OH     ', 'tag_NO2_HO2     ', 'usr_HO2NO2_M    ', &
                                     'usr_CO_OH_b     ', 'cph5            ', 'cph7            ', 'cph15           ', &
                                     'cph3            ', 'cph11           ', 'cph14           ', 'cph4            ', &
                                     'cph9            ', 'usr_HO2_HO2     ', 'tag_CLO_CLO     ', 'usr_CL2O2_M     ', &
                                     'het1            ', 'het2            ', 'het3            ', 'het4            ', &
                                     'het5            ', 'het6            ', 'het7            ', 'het8            ', &
                                     'het9            ', 'het10           ', 'het11           ', 'het12           ', &
                                     'het13           ', 'het14           ', 'het15           ', 'het16           ', &
                                     'het17           ', 'ion1            ', 'ion2            ', 'ion3            ', &
                                     'ion4            ', 'ion5            ', 'ion6            ', 'ion7            ', &
                                     'ion8            ', 'ion9            ', 'ion11           ', 'elec1           ', &
                                     'elec2           ', 'elec3           ' /)
!--- BCC
!                                     'usr_cb1         ', 'usr_oc1         ', 'usr_nh3         ', 'usr_dms1        ', &
!	   			                     'usr_dms2        ', 'usr_DMS_OH      ', 'usr_SO2_OH      ' /)
!--- BCC

  integer, dimension(rxt_tag_cnt), parameter :: &  
      rxt_tag_map                  = (/ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, &
                                       11, 12, 13, 14, 15, 16, 17, 18, 19, 20, &
                                       21, 22, 23, 24, 25, 26, 27, 28, 29, 30, &
                                       31, 32, 33, 34, 35, 36, 37, 38, 39, 40, &
                                       41, 42, 43, 44, 45, 46, 47, 48, 49, 50, &
                                       51, 52, 53, 54, 55, 56, 57, 58, 59, 60, &
                                       61, 62, 63, 64, 65, 66, 67, 68, 69, 70, &
                                       71, 72, 73, 74, 75, 76, 77, 78, 79, 80, &
                                       82, 83, 84, 85, 86, 87, 88, 89, 108, 109, &
                                      110, 111, 114, 115, 116, 119, 120, 122, 127, 129, &
                                      138, 139, 140, 142, 144, 145, 146, 151, 152, 153, &
                                      171, 172, 202, 203, 204, 205, 206, 207, 208, 209, &
                                      210, 211, 212, 213, 214, 215, 216, 217, 218, 219, &
                                      220, 221, 222, 223, 224, 225, 226, 227, 229, 230, &
                                      231, 232 /)
!--- BCC
!                                      233, 234, 235, 236, 237, 238, 239 /)
!--- BCC

!--
      character(len=8), dimension(extcnt), parameter :: &
          extfrc_lst       = (/ 'NO      ','NO2     ','CO      ','Op      ','O2p     ', &
                                'Np      ','N2p     ','N2D     ','N       ','e       ', &
                                'OH      ',                                          &
				'SO2     ','CB2     ','CH4     ','SO4     ' /)    ! wtw

!      logical, dimension(extcnt), parameter :: &
!          frc_from_dataset = (/ .true., .true., .true., .false., .false., &
!                                     .false., .false., .false., .false., .false., &
!                                     .false., .true. /)

      integer, dimension(5), parameter :: & 
                  clscnt = (/ 17 , 0 , 0 , 42 , 0 /)

!wtw      integer, dimension(5), parameter :: & 
!wtw              clscnt = (/ 17+16 , 0 , 0 , 42 , 0 /)

      real(r8), dimension(nfs), parameter :: &
                  fix_mass = (/ 0.00000000_r8, 28.0134792_r8 /)

      integer, dimension(clscnt4), parameter :: &
               diag_map = (/ 1, 4, 7, 9, 12, 14, 17, 23, 29, 36, &
                            43, 49, 54, 62, 71, 80, 86, 92, 100, 109, &
                           120, 131, 138, 148, 158, 167, 175, 187, 205, 221, &
                           244, 279, 302, 321, 345, 364, 389, 415, 433, 455, &
                           478, 496 /)

      integer, parameter :: ndst  = 4, &
							nsst  = 4

      character(len=8), dimension(ndst), parameter :: &
	  	       dust_names  = (/'DST01', 'DST02', 'DST03', 'DST04'/) 

      character(len=8), dimension(nsst), parameter :: &
	       progseasalts_names = (/'SSLT01', 'SSLT02', 'SSLT03', 'SSLT04'/)

      integer            :: cls_rxt_cnt(4,5) 
      integer            :: clsmap(gas_pcnst,5) 
      integer            :: permute(gas_pcnst,5)
      character(len=16)  :: pht_alias_lst (phtcnt,2)
      real(r8)           :: pht_alias_mult(phtcnt,2)

contains

   subroutine set_sim_dat
      pht_alias_lst(1:phtcnt,1) = &
	                       (/ 'userdefined     ', 'userdefined     ', '                ', '                ', &
                              '                ', 'userdefined     ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     '/)
      pht_alias_lst(1:phtcnt,2) = &
	                       (/ '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              '                ', '                ', '                ', '                ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     ', 'userdefined     ', 'userdefined     ', 'userdefined     ', &
                              'userdefined     '/)
      pht_alias_mult(1:phtcnt,1) = (/ 1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8 /)
      pht_alias_mult(1:phtcnt,2) = (/ 1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8, 1._r8, 1._r8, &
                          1._r8, 1._r8, 1._r8 /)
!---
! not used
!
        cls_rxt_cnt(1:4,1) = (/ 10 , 39 , 0 , 17 /)
!----
        cls_rxt_cnt(1:4,4) = (/ 22 , 96 , 114 , 42 /)

!---
! not used
!
!        clsmap(1:17+16,1) = (/ 15, 7, 19, 20, 41, 42, 43, 44, 45, 46, &
!                               47, 48, 49, 50, 51, 25, 26, & 
!                               60, 61, 62, 63, 64, 65, 66, 67, 68, 69, &
!                               70, 71, 72, 73, 74, 75 /)

        clsmap(1:17,1) = (/ 15, 7, 19, 20, 41, 42, 43, 44, 45, 46, &
                               47, 48, 49, 50, 51, 25, 26 /)
!------                            

         clsmap(1:42,4) = (/ 1, 2, 3, 4, 5, 6, 8, 9, 10, 22, &
                            11, 12, 13, 14, 16, 17, 18, 21, 23, 24, &
                            59, 27, 28, 29, 30, 31, 32, 33, 34, 35, &
                            36, 37, 38, 39, 40, 52, 53, 54, 55, 56, &
                            58, 57 /)


        permute(1:42,4) = (/ 42, 32, 36, 29, 3, 2, 22, 33, 35, 38, &
                             39, 17, 9, 7, 23, 8, 27, 28, 37, 14, &
                             26, 40, 5, 31, 4, 1, 41, 24, 25, 6, &
                             34, 30, 16, 18, 13, 15, 19, 10, 11, 20, &
                             12, 21 /)
   end subroutine set_sim_dat

end module bcc_chem_mods
