#include <misc.h>
#include <params.h>

module bcc_chem_utls
 
      use shr_kind_mod, only : r8 => shr_kind_r8

      private
      public :: get_spc_ndx, get_het_ndx, get_extfrc_ndx, get_rxt_ndx, get_inv_ndx
	  public :: flt_date, rebin, has_srfems, moz_findplb
	  public :: diurnal_geom, slvd_index

      save
    
contains

  integer function get_spc_ndx( spc_name )
!-----------------------------------------------------------------------
!     ... return overall species index associated with spc_name
!-----------------------------------------------------------------------

      use bcc_chem_mods,  only : gas_pcnst
      use bcc_chem_mods,  only : tracnam => solsym

      implicit none

!-----------------------------------------------------------------------
!     ... dummy arguments
!-----------------------------------------------------------------------
      character(len=*), intent(in) :: spc_name

!-----------------------------------------------------------------------
!     ... local variables
!-----------------------------------------------------------------------
      integer :: m

      get_spc_ndx = -1
      do m = 1, gas_pcnst
         if( trim( spc_name ) == trim( tracnam(m) ) ) then
            get_spc_ndx = m
            exit
         end if
      end do

  end function get_spc_ndx

  integer function get_inv_ndx( invariant )
    !-----------------------------------------------------------------------
    !     ... return overall external frcing index associated with spc_name
    !-----------------------------------------------------------------------

    use bcc_chem_mods,  only : nfs, inv_lst

    implicit none

    !-----------------------------------------------------------------------
    !     ... dummy arguments
    !-----------------------------------------------------------------------
    character(len=*), intent(in) :: invariant

    !-----------------------------------------------------------------------
    !     ... local variables
    !-----------------------------------------------------------------------
    integer :: m

    get_inv_ndx = -1
    do m = 1,nfs
       if( trim( invariant ) == trim( inv_lst(m) ) ) then
          get_inv_ndx = m
          exit
       end if
    end do

  end function get_inv_ndx

  integer function get_het_ndx( het_name )
    !-----------------------------------------------------------------------
    !     ... return overall het process index associated with spc_name
    !-----------------------------------------------------------------------

    use bcc_chem_mods, only : gas_wetdep_list, gas_wetdep_cnt

    implicit none

    !-----------------------------------------------------------------------
    !     ... dummy arguments
    !-----------------------------------------------------------------------
    character(len=*), intent(in) :: het_name

    !-----------------------------------------------------------------------
    !     ... local variables
    !-----------------------------------------------------------------------
    integer :: m

    get_het_ndx=-1

    do m=1,gas_wetdep_cnt

       if( trim( het_name ) == trim( gas_wetdep_list(m) ) ) then
          get_het_ndx = get_spc_ndx( gas_wetdep_list(m) )
          return
       endif
  
    enddo

  end function get_het_ndx

  integer function get_extfrc_ndx( frc_name )
    !-----------------------------------------------------------------------
    !     ... return overall external frcing index associated with spc_name
    !-----------------------------------------------------------------------

    use bcc_chem_mods,  only : extcnt, extfrc_lst

    implicit none

    !-----------------------------------------------------------------------
    !     ... dummy arguments
    !-----------------------------------------------------------------------
    character(len=*), intent(in) :: frc_name

    !-----------------------------------------------------------------------
    !     ... local variables
    !-----------------------------------------------------------------------
    integer :: m

    get_extfrc_ndx = -1
    if( extcnt > 0 ) then
       do m = 1,max(1,extcnt)
          if( trim( frc_name ) == trim( extfrc_lst(m) ) ) then
             get_extfrc_ndx = m
             exit
          end if
       end do
    end if

  end function get_extfrc_ndx

  integer function get_rxt_ndx( rxt_tag )
    !-----------------------------------------------------------------------
    !     ... return overall external frcing index associated with spc_name
    !-----------------------------------------------------------------------

    use bcc_chem_mods,  only : rxt_tag_cnt, rxt_tag_lst, rxt_tag_map

    implicit none

    !-----------------------------------------------------------------------
    !     ... dummy arguments
    !-----------------------------------------------------------------------
    character(len=*), intent(in) :: rxt_tag

    !-----------------------------------------------------------------------
    !     ... local variables
    !-----------------------------------------------------------------------
    integer :: m

    get_rxt_ndx = -1
    do m = 1,rxt_tag_cnt
       if( trim( rxt_tag ) == trim( rxt_tag_lst(m) ) ) then
          get_rxt_ndx = rxt_tag_map(m)
          exit
       end if
    end do

  end function get_rxt_ndx

  real(r8) function flt_date( ncdate, ncsec )
    !----------------------------------------------------------------------- 
    ! Purpose: Convert date and seconds of day to floating point days since
    !          0001/01/01
    !-----------------------------------------------------------------------
    use time_manager, only : timemgr_datediff
    implicit none

    !-----------------------------------------------------------------------
    !	... dummy arguments
    !-----------------------------------------------------------------------
    integer, intent(in)   :: ncdate      ! Current date as yyyymmdd
    integer, intent(in)   :: ncsec       ! Seconds of day for current date

    integer :: refymd = 00010101
    integer :: reftod = 0

    call timemgr_datediff(refymd, reftod, ncdate, ncsec, flt_date)

  end function flt_date

  subroutine rebin( nsrc, ntrg, src_x, trg_x, src, trg )
    !---------------------------------------------------------------
    !	... rebin src to trg
    !---------------------------------------------------------------

    implicit none

    !---------------------------------------------------------------
    !	... dummy arguments
    !---------------------------------------------------------------
    integer, intent(in)   :: nsrc                  ! dimension source array
    integer, intent(in)   :: ntrg                  ! dimension target array
    real(r8), intent(in)      :: src_x(nsrc+1)         ! source coordinates
    real(r8), intent(in)      :: trg_x(ntrg+1)         ! target coordinates
    real(r8), intent(in)      :: src(nsrc)             ! source array
    real(r8), intent(out)     :: trg(ntrg)             ! target array

    !---------------------------------------------------------------
    !	... local variables
    !---------------------------------------------------------------
    integer  :: i, l
    integer  :: si, si1
    integer  :: sil, siu
    real(r8)     :: y
    real(r8)     :: sl, su
    real(r8)     :: tl, tu

    !---------------------------------------------------------------
    !	... check interval overlap
    !---------------------------------------------------------------
    !     if( trg_x(1) < src_x(1) .or. trg_x(ntrg+1) > src_x(nsrc+1) ) then
    !        write(iulog,*) 'rebin: target grid is outside source grid'
    !        write(iulog,*) '       target grid from ',trg_x(1),' to ',trg_x(ntrg+1)
    !        write(iulog,*) '       source grid from ',src_x(1),' to ',src_x(nsrc+1)
    !        call endrun
    !     end if

    do i = 1,ntrg
       tl = trg_x(i)
       if( tl < src_x(nsrc+1) ) then
          do sil = 1,nsrc+1
             if( tl <= src_x(sil) ) then
                exit
             end if
          end do
          tu = trg_x(i+1)
          do siu = 1,nsrc+1
             if( tu <= src_x(siu) ) then
                exit
             end if
          end do
          y   = 0._r8
          sil = max( sil,2 )
          siu = min( siu,nsrc+1 )
          do si = sil,siu
             si1 = si - 1
             sl  = max( tl,src_x(si1) )
             su  = min( tu,src_x(si) )
             y   = y + (su - sl)*src(si1)
          end do
          trg(i) = y/(trg_x(i+1) - trg_x(i))
       else
          trg(i) = 0._r8
       end if
    end do

  end subroutine rebin

      logical function has_srfems( spc_name )
!-----------------------------------------------------------------------
!     ... return logical for species surface emission
!-----------------------------------------------------------------------

      use bcc_chem_mods,    only : srfems_cnt, srfems_lst

      implicit none

!-----------------------------------------------------------------------
!     ... dummy arguments
!-----------------------------------------------------------------------
      character(len=*), intent(in) :: spc_name
!-----------------------------------------------------------------------
!     ... local variables
!-----------------------------------------------------------------------
      integer :: m

      has_srfems = .false.
      do m = 1,srfems_cnt
         if( trim( spc_name ) == trim( srfems_lst(m) ) ) then
            has_srfems = .true.
            exit
         end if
      end do

      end function has_srfems

      subroutine moz_findplb( x, nx, xval, index )
!-----------------------------------------------------------------------
! 	... find periodic lower bound
! 	search the input array for the lower bound of the interval that
! 	contains the input value.  the returned index satifies:
! 	x(index) .le. xval .lt. x(index+1)
! 	assume the array represents values in one cycle of a periodic coordinate.
! 	so, if xval .lt. x(1), then index=nx.
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!	... dummy args
!-----------------------------------------------------------------------
      integer, intent(in) :: &
        nx
      integer, intent(out) :: &
        index
      real, intent(in) :: &
        x(nx), &                         ! strictly increasing array
        xval

!-----------------------------------------------------------------------
!	... local variables
!-----------------------------------------------------------------------
      integer :: i

      if( xval < x(1) .or. xval >= x(nx) ) then
         index = nx
         return
      end if

      do i = 2,nx
         if( xval < x(i) ) then
            index = i - 1
            exit
         end if
      end do

      end subroutine moz_findplb

      subroutine diurnal_geom( longitude, latitude, time_of_year, polar_night, polar_day, &
                               sunon, sunoff, loc_angle, zen_angle )
!------------------------------------------------------------------
!       ... diurnal geometry factors
!------------------------------------------------------------------

      use bcc_const_mozart, only : pi, twopi, pid2, dayspy, d2r

      implicit none

!------------------------------------------------------------------
!       ... dummy arguments
!------------------------------------------------------------------
      real,    intent(in)  ::     latitude           ! latitude 
      real,    intent(in)  ::     longitude          ! latitude 
      real, intent(in)     ::     time_of_year       ! time of year
      real, intent(out)    ::     sunon           ! sunrise angle in radians
      real, intent(out)    ::     sunoff          ! sunset angle in radians
      real, intent(out)    ::     zen_angle       ! solar zenith angle
      real, intent(out)    ::     loc_angle       ! "local" time angle
      logical, intent(out) ::     polar_day       ! continuous daylight flag
      logical, intent(out) ::     polar_night     ! continuous night flag

!------------------------------------------------------------------
!        ... local variables
!------------------------------------------------------------------
      real    ::  dec_max
      real    ::  declination
      real    ::  doy_loc            ! day of year
      real    ::  tod                ! time of day
      real    ::  sin_dec, cos_dec   ! sin, cos declination
      real    ::  cosphi             ! cos latitude
      real    ::  sinphi             ! sin latitude

      dec_max     = 23.45 * d2r
      sinphi      = sin( latitude )
      cosphi      = cos( latitude )
      polar_day   = .false.
      polar_night = .false.
!------------------------------------------------------------------
!        note: this formula assumes a 365 day year !
!------------------------------------------------------------------
      doy_loc     = aint( time_of_year )
      declination = dec_max * cos((doy_loc - 172.)*twopi/dayspy)
!------------------------------------------------------------------
!        determine if in polar day or night
!        if not in polar day or night then
!        calculate terminator longitudes
!------------------------------------------------------------------
      if( abs(latitude) >= (pid2 - abs(declination)) ) then
         if( sign(1.,declination) == sign(1.,latitude) ) then
            polar_day = .true.
            sunoff    = 2.*twopi
            sunon     = -twopi
         else
            polar_night  = .true.
            zen_angle = -1.0
            return
         end if
      else
         sunoff = acos( -tan(declination)*tan(latitude) )
         sunon  = twopi - sunoff
      end if

      sin_dec = sin( declination )
      cos_dec = cos( declination )
!------------------------------------------------------------------
!       ... compute base for zenith angle
!------------------------------------------------------------------
      tod = (time_of_year - doy_loc) + .5
!-------------------------------------------------------------------
!        note: longitude 0 (greenwich) at 0:00 hrs
!              maps to local angle = pi
!-------------------------------------------------------------------
      loc_angle = (tod + longitude/2.8125*180./pi/128.)*twopi
      loc_angle = mod( loc_angle,twopi )

      if( polar_day ) then
         zen_angle = acos( sinphi*sin_dec + cosphi*cos_dec*cos(loc_angle) )
      else
         if( loc_angle <= sunoff .or. loc_angle >= sunon ) then
            zen_angle = acos( sinphi*sin_dec + cosphi*cos_dec*cos(loc_angle) )
         else
            zen_angle = -1.
         endif
      end if

      end subroutine diurnal_geom

!---------------------------------------------------------------------
!---------------------------------------------------------------------
  function slvd_index( name )
    use bcc_chem_mods,    only : slvd_lst, nslvd
    implicit none

    character(len=*) :: name
    integer :: slvd_index

    integer :: m

    slvd_index = -1

    if ( nslvd < 1 ) return

    do m=1,nslvd
       if ( name == slvd_lst(m) ) then
          slvd_index = m
          return 
       endif
    enddo

  endfunction slvd_index

end module bcc_chem_utls

