#include <misc.h>
#include <params.h>

module bcc_chemdr

      implicit none

      private
      public :: chemdr
      public :: chemdr_inti
      public :: qh2o_reset
      public :: bcc_chemdr_ub

      save

      integer :: o3_ndx, so4_ndx, h2o_ndx, o2_ndx, o_ndx, hno3_ndx
      integer :: idx_cldice
      integer :: so2_extfrc_ndx, cb2_extfrc_ndx, ch4_extfrc_ndx, so4_extfrc_ndx

      real :: esfact = 1.           ! earth sun distance factor

contains

  subroutine chemdr_inti
    use bcc_chem_utls,     only : get_spc_ndx, get_extfrc_ndx
    use constituents,      only : cnst_get_ind

    implicit none

    !-----------------------------------------------------------------------

    o3_ndx  = get_spc_ndx('O3')
    o_ndx   = get_spc_ndx('O')
    o2_ndx  = get_spc_ndx('O2')
    so4_ndx = get_spc_ndx('SO4')
    h2o_ndx = get_spc_ndx('H2O')
    hno3_ndx= get_spc_ndx('HNO3')

    so2_extfrc_ndx = get_extfrc_ndx( 'SO2' )
    cb2_extfrc_ndx = get_extfrc_ndx( 'CB2' )
    ch4_extfrc_ndx = get_extfrc_ndx( 'CH4' )
    so4_extfrc_ndx = get_extfrc_ndx( 'SO4' )

    call cnst_get_ind( 'CLDICE', idx_cldice)

    end subroutine chemdr_inti

!---------------------------------------------------------------------------------
      subroutine chemdr( lchnk  , ncol   , q      , qtend ,   &
                         ps     , pmid   , pdel   , ocnfrac,  &
                         icefrac, zm     , zi     , phis   ,  &
                         cldfr  , cldtop , cmfdqr , prain ,   &
                         nevapr , cwat   , tfld   , ts    ,   &
                         troplev, em3data, mbar   , sulfate0 )
						  
!-----------------------------------------------------------------------
!     ... Chem_solver advances the volumetric mixing ratio
!         forward one time step via a combination of explicit,
!         ebi, hov, fully implicit, and/or rodas algorithms.
!-----------------------------------------------------------------------
      use ppgrid,               only : pcols
      use ppgrid,               only : pver, pverp
      use shr_kind_mod,         only : r8 => shr_kind_r8
      use constituents,         only : ppcnst, cnst_get_ind
      use history,              only : outfld
      use physconst,            only : rga, gravit
      use bcc_const_mozart,     only : pi
      use time_manager,         only : get_curr_calday, is_perpetual, get_perp_date, get_curr_date
      use bcc_chem_mods,        only : gas_pcnst, solsym, imozart
      use bcc_chem_mods,        only : indexm,   phtcnt, rxntot,  &
                                       ncol_abs => nabscol, grpcnt, nfs, extcnt, hetcnt
      use bcc_chem_mods,        only : map2chm, adv_mass
      use bcc_mean_mass,        only : set_mean_mass
      use bcc_photo,            only : set_ub_col, setcol
      use bcc_chem_utls,        only : diurnal_geom 
      use bcc_mass_xforms,      only : mmr2vmr, vmr2mmr, mmr2vmri
      use bcc_setinv,           only : setinv
      use bcc_adjrxt,           only : adjrxt
      use bcc_setrxt,           only : setrxt
      use bcc_usrrxt,           only : usrrxt
      use bcc_phtadj,           only : phtadj
      use bcc_sethet,           only : sethet
      use bcc_setext,           only : setext
      use bcc_exp_sol,          only : exp_sol
      use bcc_imp_sol,          only : imp_sol
      use phys_grid,            only : get_rlat_all_p, get_rlon_all_p
      use phys_grid,            only : get_lat_all_p,  get_lon_all_p
      use bcc_setsox,           only : setsox
      use bcc_drydep,           only : drydep
      use bcc_aerosols_intr,    only : do_cam_sulfchem
      use mo_strato_aerosol,    only : aero_set

      use bcc_sulf,             only : sulf_interp
      use time_manager,         only : get_step_size
      use bcc_charge_neutrality,only:  charge_balance
      use wv_saturation,        only:  aqsat
      use bcc_chem_mods,        only:  ndst
      use bcc_neu_wetdep,       only:  neu_wetdep_tend
      use bcc_strato_sad,       only:  strato_sad_set
      use constituents,         only : cnst_mw
      use bcc_sad,              only : sad_strat_calc
      use bcc_strato_rates,     only : ratecon_sfstrat
      use bcc_strato_rates,     only : has_strato_chem
      use shr_orb_mod,          only : shr_orb_decl
      use bcc_photo,            only : table_photo
      use bcc_aero_settling,    only : strat_aer_settling
      use physconst,            only : rair 
      use bcc_SETSOX,           only : has_sox            ! .true.    
      use bcc_negtrc,           only : negtrc
      use bcc_ub_vals,          only : set_ub_vals
      use prescribed_em_d3,     only : nem3
      use constituents,         only : cnst_name
      use time_manager,         only : get_nstep
      implicit none

#include <comsol.h>               !  lambm0, eccen, mvelpp, obliqr

!-----------------------------------------------------------------------
!        ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in)   ::  lchnk
      integer, intent(in)   ::  ncol 
      real(r8), intent(inout) :: q(pcols,pver,ppcnst)              ! species concentrations (kg/kg)
      real(r8), intent(inout) :: qtend(pcols,pver,ppcnst)          ! species tendencies (kg/kg/s)

      real(r8), intent(in)  ::  ps(pcols)               ! surface press ( pascals )
      real(r8), intent(in)  ::  phis(pcols)             ! surf geopot
      real(r8), intent(in)  ::  cldtop(pcols)           ! cloud top level ( 1 ... pver )
      real(r8), intent(in)  ::  pmid(pcols,pver)        ! midpoint press ( pascals )
      real(r8), intent(in)  ::  pdel(pcols,pver)        ! delta press across midpoints
      real(r8), intent(in)  ::  zm(pcols,pver)          ! midpoint geopotential height above the surface (m)
      real(r8), intent(in)  ::  zi(pcols,pver+1)        ! interface geopotential height above the surface (m)
      real(r8), intent(in)  ::  icefrac(pcols)          ! sea-ice areal fraction
      real(r8), intent(in)  ::  ocnfrac(pcols)          ! ocean areal fraction
      real(r8), intent(in)  ::  ts(pcols)               ! sfc temp (merged w/ocean if coupled)
      real(r8), intent(in)  ::  cldfr(pcols,pver)       ! cloud fraction
      real(r8), intent(in)  ::  nevapr(pcols,pver)      ! evap precip ( 1/s )
      real(r8), intent(in)  ::  cwat(pcols,pver)        ! total cloud water (kg/kg)
      real(r8), intent(in)  ::  tfld(pcols,pver)        ! midpoint temperature
      real(r8), intent(in)  ::  cmfdqr(pcols,pver)      ! dq/dt due to convective rainout 
      real(r8), intent(in)  ::  prain(pcols,pver)       ! rate of conversion of condensate to precipitation (kg/kg/s)
      real(r8), intent(in)  ::  em3data(pcols,pver,nem3)       
      real(r8), intent(in)  ::  sulfate0(pcols,pver)       

      real(r8), intent(out) ::  mbar(pcols,pver)      ! mean wet atmospheric mass ( amu )

      real(r8), parameter   ::  rad2deg = 180._r8/pi                ! radians to degrees conversion factor
      real(r8), parameter   ::  m2km  = 1.e-3_r8
      real(r8), parameter   ::  Pa2mb = 1.e-2_r8

      integer    :: troplev(pcols)            ! tropopause level index   
!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer  ::  ncdate, yr, mon, day, ncsec
      integer  ::  i, k, m, n
      integer  ::  nstep
      integer  ::  idx_H2O
      integer  ::  ixcldice
      integer  ::  latndx(pcols)                        ! chunk lat indicies
      integer  ::  lonndx(pcols)                        ! chunk lon indicies
      integer  ::  ltrop_sol(pcols)                     ! tropopause vertical index used in chem solvers

      real(r8) ::  vmr_sol(pcols,pver,gas_pcnst)            ! xported species ( mmr )
      real(r8) ::  mmr(pcols,pver,gas_pcnst)            ! xported species ( mmr )
      real(r8) ::  mmr_new(pcols,pver,gas_pcnst)        ! chem working concentrations (kg/kg)
      real(r8) ::  mmr_tend(pcols,pver,gas_pcnst)       ! chemistry species tendencies (kg/kg/s)
      real(r8) ::  depvel(pcols,gas_pcnst)              ! dry deposition velocity (cm/s)
      real(r8) ::  vmr(pcols,pver,gas_pcnst)            ! xported species ( vmr )
      real(r8) ::  dflx(pcols, gas_pcnst)               ! constituent surface flux (kg/m^2/s)
      real     ::  o3vmr(pcols,pver)                    ! CMIP5 O3 vmr

      real(r8) ::  delt                                 ! timestep in seconds
      real(r8) ::  sunon(pcols)    
      real(r8) ::  sunoff(pcols)    
      real(r8) ::  invariants(pcols,pver,max(1,nfs))
      real(r8) ::  col_dens(pcols,pver,max(1,ncol_abs))    ! column densities (molecules/cm^2)
      real(r8) ::  col_delta(pcols,0:pver,max(1,ncol_abs)) ! layer column densities (molecules/cm^2)
      real(r8) ::  het_rates(pcols,pver,max(1,hetcnt))
      real(r8) ::  extfrc(pcols,pver,max(1,extcnt))
      real(r8) ::  strato_sad(pcols,pver)           ! stratospheric sad (1/cm)

      real(r8) ::  reaction_rates(pcols,pver,rxntot)
      real(r8) ::  nas(pcols,pver,max(1,grpcnt))           ! non-advected species( mmr )
      real(r8) ::  h2ovmr(pcols,pver)    ! water vapor volume mixing ratio
      real(r8) ::  zmid(pcols,pver)      ! midpoint geopotential in km
      real(r8) ::  zmidr(pcols,pver)     ! midpoint geopotential in km realitive to surf
      real(r8) ::  sulfate(pcols,pver)   ! sulfate aerosols
      real(r8) ::  zen_angle(pcols)
      real(r8) ::  loc_angle(pcols)

      logical  ::  polar_night(pcols), &
                   polar_day(pcols)

      real(r8) :: hno3_cond(pcols,pver,2)         ! hno3 condensed phase concentration (mol/mol)
      real(r8) :: h2o_cond(pcols,pver)            ! h2o condensed phase concentration (mol/mol)
      real(r8) :: h2o_gas(pcols,pver)             ! h2o gas phase concentration (mol/mol)
      real(r8) :: hno3_gas(pcols,pver)            ! hno3 gas phase concentration (mol/mol)
      real(r8) :: wrk(pcols,pver) 
      real(r8) :: cldice(pcols,pver) 
      real(r8) :: pmb(pcols,pver)   
      real(r8) :: radius_strat(pcols,pver,3)      ! radius of sulfate, nat, & ice ( cm )
      real(r8) :: sad_strat(pcols,pver,3)         ! surf area density of sulfate, nat, & ice ( cm^2/cm^3 )
      real(r8) :: asdir(pcols)                    ! albedo: shortwave, direct
      real(r8) :: o2mmr(pcols,pver)               ! o2 concentration (kg/kg)
      real(r8) :: ommr(pcols,pver)                ! o concentration (kg/kg)

      real(r8) :: zsurf(pcols)                    ! surface height (m)
      real(r8) :: rlats(pcols), rlons(pcols)      ! chunk latitudes and longitudes (radians)
      real(r8) :: zintr(pcols,pver+1)             ! interface geopotential in km realitive to surf
      real(r8) :: zint(pcols,pver+1)              ! interface geopotential in km
      real(r8) :: relhum(pcols,pver)                          ! relative humidity
      real(r8) :: satv(pcols,pver)                            ! wrk array for relative humidity
      real(r8) :: satq(pcols,pver)                            ! wrk array for relative humidity
      real(r8) :: sad_total(pcols,pver)                  ! total trop. sad (cm^2/cm^3)
      real(r8) :: tvs(pcols)
      real(r8) :: qh2o(pcols,pver)               ! specific humidity (kg/kg)
      real(r8) :: delt_inverse
      real(r8) :: calday
      real(r8) :: sza(pcols)                       ! solar zenith angles (degrees)

      real(r8) :: delta    ! Solar declination angle  in radians
      real(r8) :: eccf     ! Earth orbit eccentricity factor

      real(r8) :: tau
      real(r8) :: coefadj
      logical  :: do_neu_wetdep

      real(r8) :: ftem(pcols,pver)
!----------------------------------------------------------------------------
    nstep        = get_nstep()
    delt         = get_step_size()  
    delt_inverse = 1._r8 / delt
    calday       = get_curr_calday()

    if ( is_perpetual() ) then
      call get_perp_date(yr, mon, day, ncsec)
    else
      call get_curr_date(yr, mon, day, ncsec)
    end if
    ncdate = yr*10000 + mon*100 + day

!-----------------------------------------------------------------------
!        ... Calculate parameters for diurnal geometry
!-----------------------------------------------------------------------

    call get_lat_all_p( lchnk, ncol, latndx )
    call get_lon_all_p( lchnk, ncol, lonndx )
    call get_rlat_all_p( lchnk, ncol, rlats )
    call get_rlon_all_p( lchnk, ncol, rlons )

    do i = 1, pcols
       call diurnal_geom( rlons(i), rlats(i), calday, polar_night(i), polar_day(i), &
                          sunon(i), sunoff(i), loc_angle(i),  zen_angle(i) )
    enddo

    !-----------------------------------------------------------------------      
    !        ... Xform geopotential height from m to km 
    !            and pressure from Pa to mb
    !-----------------------------------------------------------------------      
    sza(:)       = zen_angle(:) * rad2deg
    zsurf(:ncol) = rga * phis(:ncol)
    do k = 1,pver
       zintr(:ncol,k) = m2km * zi(:ncol,k)
       zmidr(:ncol,k) = m2km * zm(:ncol,k)
       zmid(:ncol,k)  = m2km * (zm(:ncol,k) + zsurf(:ncol))
       zint(:ncol,k)  = m2km * (zi(:ncol,k) + zsurf(:ncol))
       pmb(:ncol,k)   = Pa2mb * pmid(:ncol,k)
    end do
    zint(:ncol,pver+1) = m2km * (zi(:ncol,pver+1) + zsurf(:ncol))
    zintr(:ncol,pver+1)= m2km *  zi(:ncol,pver+1)

    !--------------------------------------------------------
    !-----------------------------------------------------------------------      
    !        ... map incoming concentrations to working array
    !-----------------------------------------------------------------------  
    
    call radozn(lchnk   ,ncol    ,pmid    ,o3vmr   )

    if( nstep <= 1 ) then
       do m = 1,ppcnst
          n = map2chm(m)
          if( n == o3_ndx ) then
              q(:ncol,:pver,m) = o3vmr(:ncol,:pver)*48.0/28.966
          endif
       end do
    endif

    mmr(:,:,:) = 0.0    
    do m = 1,ppcnst
       n = map2chm(m)
       if( n > 0 ) then
           mmr(:ncol,:,n)   =  q(:ncol,:,m)
       end if
    end do

    !-----------------------------------------------------------------------
    !        ... Set atmosphere mean mass
    !-----------------------------------------------------------------------
    call set_mean_mass( ncol, mmr, mbar )

    !-----------------------------------------------------------------------
    !        ... Xform from mmr to vmr
    !-----------------------------------------------------------------------
    call mmr2vmr( mmr, vmr, mbar, ncol )

    !-----------------------------------------------------------------------      
    !        ... store water vapor in wrk variable
    !-----------------------------------------------------------------------      
    
    qh2o(:ncol,:)   = mmr(:ncol,:,h2o_ndx)
    h2ovmr(:ncol,:) = vmr(:ncol,:,h2o_ndx)

    !-----------------------------------------------------------------------      
    !        ... force ion/electron balance
    !-----------------------------------------------------------------------      
    call charge_balance( ncol, vmr(:,:,:) )

    !-----------------------------------------------------------------------      
    !        ... Set the "invariants"
    !-----------------------------------------------------------------------  
    call setinv( invariants, tfld, h2ovmr, vmr, pmid, ncol, lchnk )

    !-----------------------------------------------------------------------      
    !        ... interpolate SAGEII data for surface area
    !-----------------------------------------------------------------------      
!wtw    call strato_sad_set( pmid, strato_sad, ncol, lchnk)   ! output: strato_sad

    call aero_set( lchnk, ncol, strato_sad )

    if ( has_strato_chem ) then    ! .true.
       !-----------------------------------------------------------------------      
       !        ... initialize condensed and gas phases; all hno3 to gas
       !-----------------------------------------------------------------------      
       do k = 1,pver
          hno3_gas(:,k)   = vmr(:,k,hno3_ndx)
          h2o_gas(:,k)    = h2ovmr(:,k)
          wrk(:,k)        = h2ovmr(:,k)
          cldice(:ncol,k) = q(:ncol,k,idx_cldice)
       end do
       do m = 1,2
          do k = 1,pver
             hno3_cond(:,k,m) = 0._r8
          end do
       end do

       ! output: h2o_cond
       call mmr2vmri( cldice, h2o_cond, mbar, cnst_mw(idx_cldice), ncol )

       !-----------------------------------------------------------------------      
       !        ... call SAD routine
       !----------------------------------------------------------------------- 
	        
       call sad_strat_calc( lchnk, invariants(:ncol,:,indexm), pmb, tfld, hno3_gas, &
                            hno3_cond, h2o_gas, h2o_cond, strato_sad(:ncol,:), radius_strat, &
                            sad_strat, latndx, lonndx, ncol )

       do k = 1,pver
          vmr(:,k,hno3_ndx) = hno3_gas(:,k)
          h2ovmr(:,k)       = h2o_gas(:,k)
          vmr(:,k,h2o_ndx)  = h2o_gas(:,k)
          wrk(:,k)          = (h2ovmr(:,k) - wrk(:,k))*delt_inverse
       end do

       !-----------------------------------------------------------------------      
       !        ... call aerosol reaction rates
       !-----------------------------------------------------------------------      
       call ratecon_sfstrat( invariants(:,:,indexm), pmid, tfld, &
                             radius_strat(:,:,1), sad_strat(:,:,1), sad_strat(:,:,2), &
                             sad_strat(:,:,3), h2ovmr, vmr, reaction_rates, ncol )

    endif

    !-----------------------------------------------------------------------      
    !        ... Set the column densities at the upper boundary
    !-----------------------------------------------------------------------     
 
    call set_ub_col( col_delta, vmr, invariants, ps, pdel, ncol, lchnk)

    !-----------------------------------------------------------------------      
    !       ...  Set rates for "tabular" and user specified reactions
    !-----------------------------------------------------------------------      
    call setrxt( reaction_rates, tfld, invariants(1,1,indexm), ncol )

    !-------------------------------------------------------
    ! .... set sulf  ...........
    !-----------------------------------------------------
!    call sulf_interp( pmid, calday, sulfate, ncol, lchnk )
!    sulfate(:,:) = vmr(:,:,so4_ndx)
    sulfate(:,:)  = sulfate0(:,:)    

    !-----------------------------------------------------------------
    !	... compute the relative humidity
    !-----------------------------------------------------------------
    call aqsat( tfld(:ncol,:), pmid(:ncol,:), satv, satq, ncol, &
                ncol, pver, 1, pver )

    do k = 1,pver
       relhum(:,k) = .622_r8 * h2ovmr(:,k) / satq(:,k)
       relhum(:,k) = max( 0._r8,min( 1._r8,relhum(:,k) ) )
    end do

    call usrrxt( reaction_rates, tfld, tfld, tfld, invariants, h2ovmr, ps, &
                 pmid, invariants(:,:,indexm), sulfate, mmr, relhum, strato_sad, &
                 troplev, ncol, sad_total, cwat, mbar )

    call adjrxt( reaction_rates, invariants, invariants(1,1,indexm), ncol )

    !-----------------------------------------------------------------------
    !        ... Compute the photolysis rates at time = t(n+1)
    !-----------------------------------------------------------------------      
    !-----------------------------------------------------------------------      
    !     	... Set the column densities
    !-----------------------------------------------------------------------      
    call setcol( col_delta, col_dens, vmr, pdel,  ncol )

    !-----------------------------------------------------------------------      
    !     	... Calculate the photodissociation rates
    !-----------------------------------------------------------------------      

    call shr_orb_decl( calday, eccen, mvelpp, lambm0, obliqr  , &
                       delta, esfact )

    !-----------------------------------------------------------------
    !	... lookup the photolysis rates from table
    !-----------------------------------------------------------------
     call table_photo( reaction_rates, pmid, pdel, tfld, zmid, zint, &
                       col_dens, zen_angle, asdir, cwat, cldfr, &
                       esfact, vmr, invariants, ncol, lchnk )

    !-----------------------------------------------------------------------      
    !     	... Adjust the photodissociation rates
    !-----------------------------------------------------------------------  

    call phtadj( reaction_rates, invariants, invariants(:,:,indexm), ncol )

    !-----------------------------------------------------------------------
    !        ... Compute the extraneous frcing at time = t(n+1)
    !-----------------------------------------------------------------------      
    if ( o2_ndx > 0 .and. o_ndx > 0 ) then
       do k = 1,pver
          o2mmr(:ncol,k) = mmr(:ncol,k,o2_ndx)
          ommr(:ncol,k)  = mmr(:ncol,k,o_ndx)
       end do
    endif
    !-----------------------------------------------------------------------
    !        ... Compute the extraneous frcing at time = t(n+1)
    !-----------------------------------------------------------------------  
	   
    call setext( lonndx, latndx, extfrc, zint, zintr, cldtop, &
                 zmid, lchnk, tfld, o2mmr, ommr, &
                 pmid, mbar,  rlats, calday, ncol, rlons , &
                 em3data )

    do m = 1,extcnt
          do k = 1,pver
             extfrc(:ncol,k,m) = extfrc(:ncol,k,m) / invariants(:ncol,k,indexm)  
 	                                   ! (mol.cm-3.s-1) => (1/cm3/s)
          end do
    end do

    !-----------------------------------------------------------------------
    !        ... Form the washout rates
    !-----------------------------------------------------------------------  
    if ( do_neu_wetdep ) then
        het_rates = 0._r8
    else
        call sethet( het_rates, pmid, zmid, phis, tfld, &
                     cmfdqr, prain, nevapr, delt, invariants(:,:,indexm), &
                     vmr, ncol, lchnk )
    end if

    ltrop_sol(:ncol) = 0  ! apply solver to all levels

    !=======================================================================
    !        ... Call the class solution algorithms
    !=======================================================================
    !-----------------------------------------------------------------------
    !	... Solve for "Explicit" species
    !-----------------------------------------------------------------------
    call exp_sol( vmr, reaction_rates, het_rates, extfrc, delt, invariants(1,1,indexm), ncol, lchnk, ltrop_sol )

    do m=1, gas_pcnst
       do i=1, ncol
          do k=1, pver
             if( vmr(i,k,m) < 1.e-18) then
                 vmr(i,k,m) = 1.e-18
             endif
          enddo
       enddo
    enddo

    !-----------------------------------------------------------------------
    !	... Solve for "Implicit" species
    !-----------------------------------------------------------------------

    if ( has_strato_chem ) wrk(:,:) = vmr(:,:,h2o_ndx)
   
    call t_startf('imp_sol')

!    call imp_sol( vmr, vmr_sol, reaction_rates, het_rates, extfrc, delt, &
!                  invariants(1,1,indexm), ncol, lchnk, ltrop_sol )
!
!    vmr(:,:,:) = vmr_sol(:,:,:) 

    call t_stopf('imp_sol')

    !-------------------------------------------------------------------------
    !   ... Add the exfrc for SO2, CB2, CH4, SO4
    !-----------------------------------------------------------------------

!    do n = 1, gas_pcnst
!       if( trim(solsym(n)) .eq. 'SO2' .and. so2_extfrc_ndx > 0) then
!           vmr(:ncol,:,n) = vmr(:ncol,:,n) + extfrc(:ncol,:,so2_extfrc_ndx)*delt   ! extfrc (1/cm3/s)
!       end if
!       if( trim(solsym(n)) .eq. 'CB2' .and. cb2_extfrc_ndx > 0) then
!           vmr(:ncol,:,n) = vmr(:ncol,:,n) + extfrc(:ncol,:,cb2_extfrc_ndx)*delt   ! extfrc (1/cm3/s)
!       end if
!       if( trim(solsym(n)) .eq. 'CH4' .and. ch4_extfrc_ndx > 0) then
!           vmr(:ncol,:,n) = vmr(:ncol,:,n) + extfrc(:ncol,:,ch4_extfrc_ndx)*delt   ! extfrc (1/cm3/s)
!       end if
!       if( trim(solsym(n)) .eq. 'SO4' .and. so4_extfrc_ndx > 0) then
!           vmr(:ncol,:,n) = vmr(:ncol,:,n) + extfrc(:ncol,:,so4_extfrc_ndx)*delt   ! extfrc (1/cm3/s)
!       end if
!    end do

    !-------------------------------------------------------------------
    ! ....  aerosol packages
    !------------------------------------------------------------------------
    if( has_sox .and. (.not. do_cam_sulfchem) ) then  ! .false.
       call setsox( ncol,   &
            pmid,   &
            delt,   &
            tfld,   &
            qh2o,   &
            cwat,   &
            invariants(1,1,indexm), &
            vmr, invariants )
    endif

    if ( has_strato_chem ) then   !.true.

       !-----------------------------------------------------------------------      
       !         ... aerosol settling
       !             first settle hno3(2) using radius ice
       !             secnd settle hno3(3) using radius large nat
       !-----------------------------------------------------------------------      
       wrk(:,:) = vmr(:,:,h2o_ndx)

       call strat_aer_settling( invariants(1,1,indexm), pmid, delt, zmid, tfld, &
                                hno3_cond(1,1,2), radius_strat(1,1,2), ncol, lchnk, 2 )

       !-----------------------------------------------------------------------      
       !	... reform total hno3 = gas + all condensed
       !-----------------------------------------------------------------------      
       do k = 1,pver
          vmr(:,k,hno3_ndx) = vmr(:,k,hno3_ndx) + hno3_cond(:,k,1) &
                             + hno3_cond(:,k,2) 
       end do

    endif

    !-----------------------------------------------------------------------      
    !         ... Check for negative values and reset to zero
    !-----------------------------------------------------------------------      
    call negtrc( 'After chemistry ', vmr, ncol )

    !-----------------------------------------------------------------------      
    !         ... Set upper boundary mmr values
    !-----------------------------------------------------------------------      
!
!cam5    call set_fstrat_vals( latndx, vmr, pmid, pint, troplev, calday, ncol,lchnk )
!

     call set_ub_vals( delt, lchnk, vmr, pmid, zmid, tfld, calday, ncol ,o3vmr)

    !-----------------------------------------------------------------------      
    !         ... Set fixed lower boundary mmr values
    !-----------------------------------------------------------------------  
!	
!	flbc_file = atm/waccm/lb/LBC_1850-2100_1.9x2.5_REF2_za_c080114.nc
!	    
!wtw    call flbc_set( vmr, ncol, lchnk, map2chm )
!
!wtw    if ( ghg_chem ) then   !  ghg_chem = .false.
!wtw       call ghg_chem_set_flbc( vmr, ncol )
!wtw    endif
!
    !--------------------------------------------------------------
    !   output the VMR
    !-----------------------------------------------------

!    do m = 1, gas_pcnst
!       if( trim(solsym(m)).eq.'CO2' ) then
!           call outfld ( trim(solsym(m)), vmr(1,1,m), pcols ,lchnk )
!       endif    
!    end do     

    !-----------------------------------------------------------------------      
    !         ... Xform from vmr to mmr
    !-----------------------------------------------------------------------      
    call vmr2mmr( vmr, mmr_tend, mbar, ncol )
!
!wtw    call set_short_lived_species( mmr_tend, lchnk, ncol )
!

    !-----------------------------------------------------------------------      
    !         ... Form the tendencies
    !----------------------------------------------------------------------- 
    do m = 1,gas_pcnst 
       mmr_new(:ncol,:,m)  = mmr_tend(:ncol,:,m)
       mmr_tend(:ncol,:,m) = (mmr_tend(:ncol,:,m) - mmr(:ncol,:,m))*delt_inverse
    enddo

    do m = 1,ppcnst
       n = map2chm(m)
       if( n > 0 ) then
          qtend(:ncol,:,m) = qtend(:ncol,:,m) + mmr_tend(:ncol,:,n) 
       end if
    end do

  end subroutine chemdr

!----------------------------------------------------------------------
        subroutine qh2o_reset( lchnk  , ncol   , q      )
						  
!-----------------------------------------------------------------------
!     ... Chem_solver advances the volumetric mixing ratio
!         forward one time step via a combination of explicit,
!         ebi, hov, fully implicit, and/or rodas algorithms.
!-----------------------------------------------------------------------
      use shr_kind_mod,         only : r8 => shr_kind_r8
      use ppgrid,               only : pcols, pver
      use constituents,         only : ppcnst
      use bcc_chem_mods,        only : map2chm
      implicit none

!-----------------------------------------------------------------------
!        ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in)   ::  lchnk
      integer, intent(in)   ::  ncol 
      real(r8), intent(inout) :: q(pcols,pver,ppcnst)              ! species concentrations (kg/kg)
!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer  ::  m, n

    !-----------------------------------------------------------------------      
    !        ... map incoming concentrations to working array
    !-----------------------------------------------------------------------      
    do m = 1,ppcnst
       n = map2chm(m)
       if( n > 0 ) then
           if( m .eq. h2o_ndx ) then 
		  q(:ncol,:,m)   = q(:ncol,:,1)   
       	   endif
       end if
    end do
    end subroutine qh2o_reset

    subroutine bcc_chemdr_ub( delt, lchnk  , ncol,   q,  tfld, pmid,  zm, phis)
!-----------------------------------------------------------------------
!     ... Chem_solver advances the volumetric mixing ratio
!         forward one time step via a combination of explicit,
!         ebi, hov, fully implicit, and/or rodas algorithms.
!-----------------------------------------------------------------------
      use ppgrid,               only : pcols
      use ppgrid,               only : pver
      use shr_kind_mod,         only : r8 => shr_kind_r8
      use constituents,         only : ppcnst
      use physconst,            only : rga
      use time_manager,         only : get_curr_calday
      use bcc_chem_mods,        only : gas_pcnst
      use bcc_chem_mods,        only : map2chm
      use bcc_mean_mass,        only : set_mean_mass
      use bcc_mass_xforms,      only : mmr2vmr, vmr2mmr
      use time_manager,         only : get_step_size
      use bcc_ub_vals,          only : set_ub_vals

      implicit none

!-----------------------------------------------------------------------
!        ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in)   ::  lchnk
      integer, intent(in)   ::  ncol
      real(r8), intent(inout) :: q(pcols,pver,ppcnst)              ! species concentrations (kg/kg)
      real(r8), intent(in)  ::  pmid(pcols,pver)        ! midpoint press ( pascals )
      real(r8), intent(in)  ::  zm(pcols,pver)          ! midpoint geopotential height above the surface (m)
      real(r8), intent(in)  ::  tfld(pcols,pver)        ! midpoint temperature
      real(r8), intent(in)  ::  phis(pcols)

      real(r8)              ::  mbar(pcols,pver)      ! mean wet atmospheric mass ( amu )

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      real(r8), parameter   ::  m2km  = 1.e-3_r8
      integer  ::  i, k, m, n
      integer  ::  nstep
      real(r8) ::  mmr(pcols,pver,gas_pcnst)            ! xported species ( mmr )
      real(r8) ::  vmr(pcols,pver,gas_pcnst)            ! xported species ( vmr )
      real(r8) ::  o3vmr(pcols,pver)                    ! CMIP5 O3 vmr
      real(r8) ::  delt                                 ! timestep in seconds
      real(r8) ::  zmid(pcols,pver)      ! midpoint geopotential in km
      real(r8) ::  calday
      real(r8) ::  zsurf(pcols)                    ! surface height (m)
!----------------------------------------------------------------------------

    calday       = get_curr_calday()

    mmr(:,:,:) = 0.0
    do m = 1,ppcnst
       n = map2chm(m)
       if( n > 0 ) then
           mmr(:ncol,:,n) = q(:ncol,:,m)
       end if
    end do

    !-----------------------------------------------------------------------
    !        ... Set atmosphere mean mass
    !-----------------------------------------------------------------------
    call set_mean_mass( ncol, mmr, mbar )

    !-----------------------------------------------------------------------
    !        ... Xform from mmr to vmr
    !-----------------------------------------------------------------------
    call mmr2vmr( mmr, vmr, mbar, ncol )

    !-----------------------------------------------------------------------
    !         ... Set upper boundary mmr values
    !-----------------------------------------------------------------------
    call radozn(lchnk   ,ncol    ,pmid    ,o3vmr   )

    zsurf(:ncol) = rga * phis(:ncol)
    do k = 1,pver
       zmid(:ncol,k)  = m2km * (zm(:ncol,k) + zsurf(:ncol))
    end do

    call set_ub_vals( delt, lchnk, vmr, pmid, zmid, tfld, calday, ncol ,o3vmr)

    !-----------------------------------------------------------------------
    !         ... Xform from vmr to mmr
    !-----------------------------------------------------------------------
    call vmr2mmr( vmr, mmr, mbar, ncol )

    do m = 1,ppcnst
       n = map2chm(m)
       if( n > 0 ) then
           q(:ncol,:,m) = mmr(:ncol,:,n)
       end if
    end do
    end subroutine bcc_chemdr_ub

end module bcc_chemdr
