#include <misc.h>
#include <params.h>


module bcc_drydep


!---------------------------------------------------------------------
!       ... Dry deposition velocity input data and code for netcdf input
!---------------------------------------------------------------------
      use shr_kind_mod,  only : r8 => shr_kind_r8
      use bcc_chem_mods, only : gas_pcnst
      use bcc_chem_mods, only : drydep_cnt, drydep_list, drydep_mapping
      use ppgrid,        only : pcols, begchunk, endchunk, pver


      implicit none
 
      save


      private
      public   :: drydep_inti , interpdvel, has_drydep, drydep


      real(r8), allocatable :: dvel(:,:,:,:)
      real(r8), allocatable :: dvel_interp(:,:,:)
      real(r8), allocatable :: sea_ice(:,:)


      integer ::  ch2o_ndx,   &
                  ch3ooh_ndx, &
                  co_ndx,     &
                  h2o2_ndx,   &
                  hno3_ndx,   &     
                  ho2no2_ndx, &   
                  nh3_ndx,    &     
                  no_ndx,     &     
                  no2_ndx,    &    
                  o3_ndx,     &         
                  ch4_ndx,    &        
                  cb1_ndx,    &        
                  cb2_ndx,    &        
                  oc1_ndx,    &        
                  oc2_ndx,    &        
                  dst01_ndx,  &      
                  dst02_ndx,  &      
                  dst03_ndx,  &      
                  dst04_ndx,  &      
                  h2_ndx,     &
		  so2_ndx,    &
		  so4_ndx

contains


      subroutine drydep_inti( bnddvel)
!---------------------------------------------------------------------------
!       ... Initialize module, depvel arrays, and a few other variables.
!           The depvel fields will be linearly interpolated to the correct time
!---------------------------------------------------------------------------


      use bcc_netcdf
      use bcc_const_mozart, only : d2r
      use pmgrid,           only : plon, plat
      use bcc_chem_mods,    only : solsym
      use bcc_chem_utls,    only : get_spc_ndx
      use bcc_charutl,      only : glc


!add by zf 2008.08.13
      use ppgrid,           only: pcols, begchunk, endchunk
      use pmgrid,           only: masterproc
      use ioFileMod,        only: getfil
      use abortutils,       only: endrun
      use error_messages,   only: handle_ncerr
      use phys_grid,        only: scatter_field_to_chunk


#if ( defined SPMD )
      use mpishorthand,         only: mpicom, mpiint
#endif


!end zf
      implicit none


      character(len=*), intent(in) :: bnddvel
!---------------------------------------------------------------------------
!       ... Local variables
!---------------------------------------------------------------------------
      integer :: ncid, vid, vid_dvel, nlat, nlon, nmonth, ndims
      integer :: dimid_lat, dimid_lon, dimid_species, dimid_time
      integer :: dimid(4), count(4), start(4)
      integer :: jlim(2), jl, ju
      integer :: gndx, m, nchar, ierr, i
      integer :: k,j, n                                    !zf
      real    :: scale_factor
      real, dimension(plon,plat) :: wrk2d
      real, allocatable :: dvel_lats(:), dvel_lons(:)
      real, allocatable :: dvel_in(:,:,:,:)                          ! input depvel array
      character(len=50) :: units
      character(len=80) :: filename, lpath, rpath
      real  :: dvel_old(plon,plat,12) 


      integer varid
      character(len=256) :: locfn    ! netcdf local filename to open   2008.08.13
!---------------------------------------------------------------------------
!       ... Setup species maps
!---------------------------------------------------------------------------
      ch2o_ndx     = get_spc_ndx( 'CH2O')
      ch3ooh_ndx   = get_spc_ndx( 'CH3OOH')
      co_ndx       = get_spc_ndx( 'CO')
      h2o2_ndx     = get_spc_ndx( 'H2O2')
      hno3_ndx     = get_spc_ndx( 'HNO3')
      ho2no2_ndx   = get_spc_ndx( 'HO2NO2')
      nh3_ndx      = get_spc_ndx( 'NH3')
      no_ndx       = get_spc_ndx( 'NO')
      no2_ndx      = get_spc_ndx( 'NO2')
      o3_ndx       = get_spc_ndx( 'O3')
      ch4_ndx      = get_spc_ndx( 'CH4')
      cb1_ndx      = get_spc_ndx( 'CB1')
      cb2_ndx      = get_spc_ndx( 'CB2')
      oc1_ndx      = get_spc_ndx( 'OC1')
      oc2_ndx      = get_spc_ndx( 'OC2')
      dst01_ndx    = get_spc_ndx( 'DST01')
      dst02_ndx    = get_spc_ndx( 'DST02')
      dst03_ndx    = get_spc_ndx( 'DST03')
      dst04_ndx    = get_spc_ndx( 'DST04')
      h2_ndx       = get_spc_ndx( 'H2')

      so2_ndx      = get_spc_ndx( 'SO2')
      so4_ndx      = get_spc_ndx( 'SO4')

!---------------------------------------------------------------------------
!       ... Open NetCDF file
!---------------------------------------------------------------------------
      if( masterproc ) then


      call getfil (bnddvel, locfn)
      call wrap_open(locfn,0,ncid)


!---------------------------------------------------------------------------
!       ... Inquire about dimensions
!---------------------------------------------------------------------------
      call handle_ncerr( NF_INQ_DIMID( ncid, 'lon', dimid_lon ), 'dvel_inti: getting lon dimension ID' )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lon, nlon ), 'dvel_inti: getting nlon' )


      call handle_ncerr( NF_INQ_DIMID( ncid, 'lat', dimid_lat ), 'dvel_inti: getting lat dimension ID' )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lat, nlat ), 'dvel_inti: getting nlat' )


      call handle_ncerr( NF_INQ_DIMID( ncid, 'time', dimid_time ), 'dvel_inti: getting time dimension ID' )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_time, nmonth ), 'dvel_inti: getting nmonth' )


      write(*,*) 'dvel_inti: dimensions (nlon,nlat,nmonth) = ',nlon,nlat,nmonth


!---------------------------------------------------------------------------
!       ... Allocate depvel lats, lons and read
!---------------------------------------------------------------------------
      allocate( dvel_lats(nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'dvel_inti: Failed to allocate dvel_lats vector'
         call endrun
      end if
      allocate( dvel_lons(nlon), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'dvel_inti: Failed to allocate dvel_lons vector'
         call endrun
      end if
      call handle_ncerr( NF_INQ_VARID( ncid, 'lat', vid ), 'dvel_inti: lat dim not in dvel file' )
      call handle_ncerr( NF_GET_VAR_DOUBLE( ncid, vid, dvel_lats ), 'dvel_inti: getting dvel lats' )
      call handle_ncerr( NF_INQ_VARID( ncid, 'lon', vid ), 'dvel_inti: lon dim not in dvel file' )
      call handle_ncerr( NF_GET_VAR_DOUBLE( ncid, vid, dvel_lons ), 'dvel_inti: getting dvel lons' )


!---------------------------------------------------------------------------
!       ... Set the transform from inputs lats to simulation lats
!---------------------------------------------------------------------------


      dvel_lats(:nlat) = d2r * dvel_lats(:nlat)
      dvel_lons(:nlon) = d2r * dvel_lons(:nlon)


      deallocate( dvel_lats, dvel_lons )


      end if             !masterproc


!---------------------------------------------------------------------------
!     	... Allocate dvel and read data from file
!---------------------------------------------------------------------------
#if (defined SPMD )
    call mpibcast( nlon, 1, mpiint, 0, mpicom )
    call mpibcast( nlat, 1, mpiint, 0, mpicom )
    call mpibcast( nmonth, 1, mpiint, 0, mpicom )
!    call mpibcast( nspecies, 1, mpiint, 0, mpicom )
#endif


      allocate ( dvel(pcols,begchunk:endchunk,drydep_cnt,12) )
      allocate ( dvel_interp(pcols,begchunk:endchunk,drydep_cnt))
      allocate ( sea_ice(pcols,begchunk:endchunk))


      allocate( dvel_in(nlon,nlat,drydep_cnt,nmonth), stat=ierr )


      if( ierr /= 0 ) then
         write(*,*) 'dvel_inti: Failed to allocate dvel_in'
         call endrun
      end if


      if( masterproc ) then


        do n = 1, drydep_cnt


          call wrap_inq_varid(ncid, trim(drydep_list(n)), varid)
          call wrap_get_var_realx(ncid, varid, dvel_old)


          units = ' '
          call handle_ncerr( NF_GET_ATT_TEXT( ncid, varid, 'units', units ), 'dvel_inti: dvel units not found' )


          if( units(:glc(units)) == 'm/s' ) then
             scale_factor = 100.
          elseif( units(:glc(units)) == 'cm/s' ) then
             scale_factor = 1.
          else
             scale_factor = 1.
          end if


!---------------------------------------------------------------------------
!     	... Check units of deposition velocity. If necessary, convert to cm/s.
!---------------------------------------------------------------------------


	      do i = 1, nlon
	        do j = 1, nlat
	          do k = 1, nmonth
 	             dvel_in(i,j,n,k) = dvel_old(i,j,k) * scale_factor 
	          enddo
	        enddo
	      enddo
   	    enddo


      end if             !masterproc


!---------------------------------------------------------------------------
!     	... Regrid deposition velocities
!---------------------------------------------------------------------------


      do i = 1, nmonth
        call scatter_field_to_chunk(1,1,drydep_cnt,nlon,dvel_in(:,:,:,i),dvel(:,:,:,i))
      enddo


!---------------------------------------------------------------------------
!     	... Allocate dvel_interp array
!---------------------------------------------------------------------------
      deallocate( dvel_in )      
      end subroutine drydep_inti


      subroutine interpdvel( ncol, calday, lchnk )
!---------------------------------------------------------------------------
!       ... Interpolate the fields whose values are required at the
!           begining of a timestep.
!---------------------------------------------------------------------------


      use bcc_calendar, only : caldayr
      use bcc_surf,     only : seaicebdy
      use bcc_srf_emis, only : nppbdy


      implicit none


!---------------------------------------------------------------------------
!       ... Dummy arguments
!---------------------------------------------------------------------------
      integer, intent(in) :: ncol            ! latitude tile dimension
      integer, intent(in) :: lchnk           ! local latitude index
      real, intent(in)    :: calday          ! Interpolate the input data to calday


!---------------------------------------------------------------------------
!       ... Local variables
!---------------------------------------------------------------------------
      integer  ::  m, last, next, i
      integer  ::  dates(12) = (/ 116, 214, 316, 415,  516,  615, &
                                  716, 816, 915, 1016, 1115, 1216 /)
      real :: calday_loc, last_days, next_days
      real, save ::  days(12)
      logical, save  ::  entered = .false.


      if( .not. entered ) then
         do m = 1,12
            days(m) = caldayr( dates(m), 0 )
         end do
         entered = .true.
      end if


      if( calday < days(1) ) then
         next = 1
         last = 12
      else if( calday >= days(12) ) then
         next = 1
         last = 12
      else
         do m = 11,1,-1
            if( calday >= days(m) ) then
               exit
            end if
         end do
         last = m
         next = m + 1
      end if


      last_days  = days( last )
      next_days  = days( next )
      calday_loc = calday


      if( next_days < last_days ) then
         next_days = next_days + 365.
      end if
      if( calday_loc < last_days ) then
         calday_loc = calday_loc + 365.
      end if
      
	  
      do m = 1,drydep_cnt
         call intp2d( last_days, next_days, calday_loc, &
                      dvel(1:ncol,lchnk,m,last), &
                      dvel(1:ncol,lchnk,m,next), &
                      dvel_interp(1:ncol,lchnk,m), ncol )
      end do


      call intp2d( last_days, next_days, calday_loc, &
                   seaicebdy(1:ncol,lchnk,last), &
                   seaicebdy(1:ncol,lchnk,next), &
                   sea_ice(1:ncol,lchnk),ncol )
      end subroutine interpdvel


      subroutine intp2d( t1, t2, tint, f1, f2, fint, plonl )
!-----------------------------------------------------------------------
!       ... Linearly interpolate between f1(t1) and f2(t2) to fint(tint).
!-----------------------------------------------------------------------


      implicit none


!-----------------------------------------------------------------------
!       ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in)  :: plonl                                   


      real, intent(in)  :: t1            ! time level of f1
      real, intent(in)  :: t2            ! time level of f2
      real, intent(in)  :: tint          ! interpolant time
      real, intent(in)  :: f1(plonl)     ! field at time t1
      real, intent(in)  :: f2(plonl)     ! field at time t2


      real, intent(out) :: fint(plonl)   ! field at time tint


!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      real :: factor


      factor = (tint - t1)/(t2 - t1)


      fint(:) = f1(:) + (f2(:) - f1(:))*factor


      end subroutine intp2d


      subroutine drydep( lchnk, calday, tsurf, zen_angle, &
                         depvel, dflx, rair, q, p, tv, ncol , ocnfrac, &
                         vlc_trb_SO4, vlc_trb_NH4, vlc_trb_CB2, vlc_trb_SO2 )
!--------------------------------------------------------
!       ... Form the deposition velocities for this
!           lchnkitude slice
!--------------------------------------------------------


      use bcc_chem_utls, only : get_spc_ndx
      use bcc_surf,      only : oceanbdy
      use bcc_chem_mods, only : gas_pcnst
      use pmgrid,        only : masterproc, plev


      implicit none


!--------------------------------------------------------
!       ... Dummy arguments
!--------------------------------------------------------
      integer, intent(in) ::  ncol               ! longitude tile dimension
      integer, intent(in) ::  lchnk                 ! incoming lchnkitude index
      real, intent(in)    ::  rair                ! gas constant of dry air in J/deg/kg
      real, intent(in)    ::  q(pcols,plev,gas_pcnst) ! tracer mmr (kg/kg)
      real, intent(in)    ::  p(pcols)            ! midpoint pressure in surface layer (Pa)
      real, intent(in)    ::  calday              ! time of year in days
      real, intent(in)    ::  tsurf(pcols)        ! surface temperature (K)
      real, intent(in)    ::  zen_angle(pcols)    ! zenith angle (radians)
      real, intent(in)    ::  tv(pcols)           ! virtual temperature in surface layer (K)
      real, intent(inout) ::  dflx(pcols,gas_pcnst)   ! flux due to dry deposition (kg/m^2/sec)
      real, intent(out)   ::  depvel(pcols,gas_pcnst) ! deposition vel (cm/s)

      real(r8),intent(in) :: vlc_trb_SO4(pcols)       ! dry deposite velcity (m/s)
      real(r8),intent(in) :: vlc_trb_NH4(pcols)       ! dry deposite velcity (m/s)
      real(r8),intent(in) :: vlc_trb_CB2(pcols)       ! dry deposite velcity (m/s)
      real(r8),intent(in) :: vlc_trb_SO2(pcols)       ! dry deposite velcity (m/s)
!--
! wtw
      real(r8), intent(in) :: ocnfrac(pcols)          ! ocean areal fraction
!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer :: m, spc_ndx, tmp_ndx, i
      real    :: vel(pcols)
      real    :: glace(pcols)
      real    :: open_ocean(pcols)
      real    :: temp_fac(pcols)
      real    :: wrk(pcols)
      real    :: tmp(pcols)
      real    :: o3_tab_dvel(pcols)

!-----------------------------------------------------------------------
!       ... Note the factor 1.e-2 in the wrk array calculchnkion is
!           to transform the incoming dep vel from cm/s to m/s
!-----------------------------------------------------------------------
      wrk(:) =  1.e-2 * p(:) / (rair * tv(:))


!--------------------------------------------------------
!       ... Time interpolchnke primary depvel array
!           (also seaice and npp)
!--------------------------------------------------------
      call interpdvel( ncol, calday, lchnk )
!--------------------------------------------------------
!       ... Initialize all deposition velocities to zero
!--------------------------------------------------------
      depvel(:,:) = 0.
      do i = 1,drydep_cnt
         m = drydep_mapping(i) 
         if( m > 0 ) then
            depvel(:,m) = dvel_interp(:,lchnk,i)
         end if
      end do


!--------------------------------------------------------
!       ... Set some variables needed for some dvel calculchnkions
!--------------------------------------------------------


      temp_fac(:)   = min( 1., max( 0., (tsurf(:) - 268.) / 5. ) )
      open_ocean(:) = max( 0.,oceanbdy(:,lchnk) - sea_ice(:,lchnk) )
      glace(:)      = sea_ice(:,lchnk) + (1. - oceanbdy(:,lchnk)) * (1. - temp_fac(:))
      glace(:)      = min( 1.,glace(:) )


!--------------------------------------------------------
!       ... Set ch2o dvel
!--------------------------------------------------------
      ch2o_ndx     = get_spc_ndx( 'CH2O')
      dflx(:,ch2o_ndx)   = wrk(:) * depvel(:,ch2o_ndx) * q(:,plev,ch2o_ndx)
!--------------------------------------------------------
!       ... Set ch3ooh dvel
!--------------------------------------------------------
      ch3ooh_ndx     = get_spc_ndx( 'CH3OOH')
      dflx(:,ch3ooh_ndx)   = wrk(:) * depvel(:,ch3ooh_ndx) * q(:,plev,ch3ooh_ndx)
!--------------------------------------------------------
!       ... Set co dvel
!--------------------------------------------------------
      co_ndx     = get_spc_ndx( 'CO')
      dflx(:,co_ndx)   = wrk(:) * depvel(:,co_ndx) * q(:,plev,co_ndx)
!--------------------------------------------------------
!       ... Set h2o2 dvel
!--------------------------------------------------------
      h2o2_ndx     = get_spc_ndx( 'H2O2')
      dflx(:,h2o2_ndx)   = wrk(:) * depvel(:,h2o2_ndx) * q(:,plev,h2o2_ndx)
!--------------------------------------------------------
!       ... Set hno3 dvel
!--------------------------------------------------------
      hno3_ndx     = get_spc_ndx( 'HNO3')
      dflx(:,hno3_ndx)   = wrk(:) * depvel(:,hno3_ndx) * q(:,plev,hno3_ndx)
!--------------------------------------------------------
!       ... Set ho2no2 dvel
!--------------------------------------------------------
      ho2no2_ndx     = get_spc_ndx( 'HO2NO2')
      dflx(:,ho2no2_ndx)   = wrk(:) * depvel(:,ho2no2_ndx) * q(:,plev,ho2no2_ndx)
!--------------------------------------------------------
!       ... Set nh3 dvel
!--------------------------------------------------------
      nh3_ndx     = get_spc_ndx( 'NH3')
      dflx(:,nh3_ndx)   = wrk(:) * depvel(:,nh3_ndx) * q(:,plev,nh3_ndx)
!--------------------------------------------------------
!       ... Set no dvel
!--------------------------------------------------------
      no_ndx     = get_spc_ndx( 'NO')
      dflx(:,no_ndx)   = wrk(:) * depvel(:,no_ndx) * q(:,plev,no_ndx)
!--------------------------------------------------------
!       ... Set no2 dvel
!--------------------------------------------------------
      no2_ndx     = get_spc_ndx( 'NO2')
      dflx(:,no2_ndx)   = wrk(:) * depvel(:,no2_ndx) * q(:,plev,no2_ndx)


!--------------------------------------------------------
!       ... Set o3 dvel
!       ... Set diurnal dependence for OX dvel
!--------------------------------------------------------

      o3_ndx     = get_spc_ndx( 'O3')
      tmp(:) = max( 1.,sqrt( (depvel(:,o3_ndx) - .2)**3/.27 + 4.*depvel(:,o3_ndx) + .67 ) )
      vel(:) = depvel(:,o3_ndx)
      where( zen_angle(:) <= 0. )
             vel(:) = vel(:) / tmp(:)
      elsewhere
             vel(:) = vel(:) * tmp(:)
      endwhere
      dflx(:,o3_ndx)   = wrk(:) * vel(:) * q(:,plev,o3_ndx)


!--------------------------------------------------------
!       ... Set ch4 dvel
!--------------------------------------------------------
      ch4_ndx     = get_spc_ndx( 'CH4')
      dflx(:,ch4_ndx)   = wrk(:) * depvel(:,ch4_ndx) * q(:,plev,ch4_ndx)

!--------------------------------------------------------
!       ... Set cb1 dvel   wtw add
!--------------------------------------------------------
       cb1_ndx     = get_spc_ndx( 'CB1')
       dflx(:,cb1_ndx)   = wrk(:) * vlc_trb_CB2(:) * q(:,plev,cb1_ndx)     

!--------------------------------------------------------
!       ... Set cb2 dvel   wtw add
!--------------------------------------------------------
       cb2_ndx     = get_spc_ndx( 'CB2')
       dflx(:,cb2_ndx)   = wrk(:) * vlc_trb_CB2(:) * q(:,plev,cb2_ndx) 

!--------------------------------------------------------
!       ... Set oc1 dvel   wtw add
!--------------------------------------------------------
       oc1_ndx     = get_spc_ndx( 'OC1')
       dflx(:,oc1_ndx)   = wrk(:) * vlc_trb_CB2(:) * q(:,plev,oc1_ndx) 

!--------------------------------------------------------
!       ... Set oc2 dvel   wtw add
!--------------------------------------------------------
       oc2_ndx     = get_spc_ndx( 'OC2')
       dflx(:,oc2_ndx)   = wrk(:) * vlc_trb_CB2(:) * q(:,plev,oc2_ndx) 

!--------------------------------------------------------
!       ... Set so4 dvel   wtw add
!--------------------------------------------------------
       so4_ndx     = get_spc_ndx( 'SO4')
       dflx(:,so4_ndx)   = wrk(:) * vlc_trb_SO4(:) * q(:,plev,so4_ndx)  

!--------------------------------------------------------
!       ... Set so2 dvel   wtw add
!--------------------------------------------------------
       so2_ndx     = get_spc_ndx( 'SO2')
       dflx(:,so2_ndx)   = wrk(:) * vlc_trb_SO2(:) * q(:,plev,so2_ndx)   

!--------------------------------------------------------
!       ... Set h2 and relchnked species dep vel
!--------------------------------------------------------
      h2_ndx     = get_spc_ndx( 'H2')
      tmp(:)     = depvel(:,co_ndx) * 1.5
      dflx(:,h2_ndx)   = wrk(:) * tmp(:) * q(:,plev,h2_ndx)

      end subroutine drydep

      logical function has_drydep( spc_name )
!-----------------------------------------------------------------------
!     ... return logical for species dry deposition
!-----------------------------------------------------------------------


      use bcc_chem_mods,  only : drydep_cnt, drydep_list


      implicit none


!-----------------------------------------------------------------------
!     ... dummy arguments
!-----------------------------------------------------------------------
      character(len=*), intent(in) :: spc_name


!-----------------------------------------------------------------------
!     ... local variables
!-----------------------------------------------------------------------
      integer :: m


      has_drydep = .false.
      do m = 1,drydep_cnt
         if( trim( spc_name ) == trim( drydep_list(m) ) ) then
            has_drydep = .true.
            exit
         end if
      end do


      end function has_drydep


end module bcc_drydep






