#include <misc.h>
#include <params.h>

module bcc_exp_sol

  private
  public :: exp_sol
  public :: exp_sol_inti

contains

  subroutine exp_sol_inti

    use bcc_chem_mods,   only : solsym
    use bcc_chem_mods,   only : clscnt1, clsmap
    use ppgrid,          only : pver
    use history,         only : addfld, add_default, phys_decomp

    implicit none

    integer :: i,j

    do i = 1,clscnt1

       j = clsmap(i,1)
!       call addfld( trim(solsym(j))//'_CHMP', '/cm3/s ', pver, 'I', 'chemical production rate', phys_decomp )
!       call addfld( trim(solsym(j))//'_CHML', '/cm3/s ', pver, 'I', 'chemical loss rate',       phys_decomp )

    enddo
  end subroutine exp_sol_inti


  subroutine exp_sol( base_sol, reaction_rates, het_rates, extfrc, delt, xhnm, ncol, lchnk, ltrop )
    !-----------------------------------------------------------------------
    !      	... Exp_sol advances the volumetric mixing ratio
    !           forward one time step via the fully explicit
    !           Euler scheme
    !-----------------------------------------------------------------------

    use bcc_chem_mods,     only : clscnt1, extcnt, gas_pcnst, clsmap, rxntot, hetcnt
    use ppgrid,            only : pcols, pver
    use bcc_prod_loss,     only : exp_prod_loss
    use bcc_indprd,        only : indprd
    use shr_kind_mod,      only : r8 => shr_kind_r8
    use history,           only : outfld
    use bcc_chem_mods,     only : solsym
    use bcc_chem_utls,     only : get_spc_ndx

    implicit none
    !-----------------------------------------------------------------------
    !     	... Dummy arguments
    !-----------------------------------------------------------------------
    integer,  intent(in)    ::  ncol                                ! columns in chunck
    integer,  intent(in)    ::  lchnk                               ! chunk id
    real(r8), intent(in)    ::  delt                                ! time step (s)
    real(r8), intent(in)    ::  het_rates(pcols,pver,max(1,hetcnt))  ! het rates (1/cm^3/s)
    real(r8), intent(in)    ::  reaction_rates(pcols,pver,rxntot)    ! rxt rates (1/cm^3/s)
    real(r8), intent(in)    ::  extfrc(pcols,pver,extcnt)            ! "external insitu forcing" (1/cm^3/s)
    real(r8), intent(in)    ::  xhnm(pcols,pver)
    integer,  intent(in)    ::  ltrop(pcols)                        ! chemistry troposphere boundary (index)
    real(r8), intent(inout) ::  base_sol(pcols,pver,gas_pcnst)       ! working mixing ratios (vmr)

    !-----------------------------------------------------------------------
    !     	... Local variables
    !-----------------------------------------------------------------------
    integer  ::  oc1_ndx, oc2_ndx, cb1_ndx, cb2_ndx
    integer  ::  i, k, l, m
    real(r8), dimension(pcols,pver,clscnt1) :: &
         prod, &
         loss, &
         ind_prd

    real(r8) :: wrk    (pcols,pver) 
	real(r8) :: sol_oc1(pcols,pver)
	real(r8) :: sol_cb1(pcols,pver)

    !-----------------------------------------------------------------------      
    !        ... Put "independent" production in the forcing
    !-----------------------------------------------------------------------      
    call indprd( 1, ind_prd, clscnt1, base_sol, extfrc, &
         reaction_rates, ncol )

    !-----------------------------------------------------------------------      
    !      	... Form F(y)
    !-----------------------------------------------------------------------      
    call exp_prod_loss( prod, loss, base_sol, reaction_rates, het_rates )

    !-----------------------------------------------------------------------      
    !    	... Solve for the mixing ratio at t(n+1)
    !-----------------------------------------------------------------------   
	oc1_ndx = get_spc_ndx( 'OC1' )
	oc2_ndx = get_spc_ndx( 'OC2' )
	cb1_ndx = get_spc_ndx( 'CB1' )
	cb2_ndx = get_spc_ndx( 'CB2' )

    sol_oc1(:,:) = 0._r8
    sol_cb1(:,:) = 0._r8
	   
    do m = 1,clscnt1
       l = clsmap(m,1)
       do i = 1,ncol
          do k = ltrop(i)+1,pver
             if( l == oc1_ndx ) then
                 sol_oc1(i,k)     = delt * (prod(i,k,m) + ind_prd(i,k,m) - loss(i,k,m))
				 if( (sol_oc1(i,k) + base_sol(i,k,l)) .lt. 1.e-18 )  sol_oc1(i,k) = 1.e-18 - base_sol(i,k,l) 
                 base_sol(i,k,l)  = base_sol(i,k,l) + sol_oc1(i,k)

             elseif( l == oc2_ndx ) then
                 base_sol(i,k,l)  = base_sol(i,k,l) -  sol_oc1(i,k) 

             elseif( l == cb1_ndx ) then
                 sol_cb1(i,k)     = delt * (prod(i,k,m) + ind_prd(i,k,m) - loss(i,k,m))
				 if( (sol_cb1(i,k) + base_sol(i,k,l)) .lt. 1.e-18 )  sol_cb1(i,k) = 1.e-18 - base_sol(i,k,l) 
                 base_sol(i,k,l)  = base_sol(i,k,l) + sol_cb1(i,k)

             elseif( l == cb2_ndx ) then
                 base_sol(i,k,l)  = base_sol(i,k,l) -  sol_cb1(i,k) 
            
			 else
                 base_sol(i,k,l)  = base_sol(i,k,l) + delt * (prod(i,k,m) + ind_prd(i,k,m) - loss(i,k,m))
		     endif
          end do
       end do
    end do

  end subroutine exp_sol

end module bcc_exp_sol
