#include <misc.h>
#include <params.h>

module bcc_grid_area

   implicit none
   save
   private
   public :: get_area_all_p
   real, public, allocatable, dimension(:,:) :: area_p

contains

   subroutine get_area_all_p

!----------------------------------------------------------------------- 
! 
!                          
! Purpose: Return latitude and longitude (in radians), column surface
!          area (in radians squared) and surface integration weights
!          for global column indices that will be passed to/from physics
! 
!        sum(area) = 4.0*pie
!
! Method: 
! 
! Author: wtw
! 
!-----------------------------------------------------------------------
   use shr_kind_mod,  only: r8 => shr_kind_r8
   use pmgrid,        only: plat, plon, masterproc
   use ppgrid,        only: pcols, begchunk, endchunk   
   use phys_grid,     only: scatter_field_to_chunk
   use physconst,     only: pi
   use commap,        only: latdeg, londeg

   implicit none

!---------------------------Local workspace-----------------------------
!
    integer i,j                      ! loop indices
    real(r8) :: area_all(plon,plat)
    real(r8) :: ns_vert(2,plon)      ! latitude grid vertices
    real(r8) :: ew_vert(2,plon)      ! longitude grid vertices
    real(r8) :: del_theta            ! difference in latitude at a grid point
    real(r8) :: del_phi              ! difference in longitude at a grid point
    real(r8), parameter :: degtorad=pi/180_r8
!-----------------------------------------------------------------------
    if(masterproc) then
      do i=1, plon
        do j=1, plat

          if (j .eq. 1) then
              ns_vert(1,i) = -90.0_r8
          else
              ns_vert(1,i) = (latdeg(j) + latdeg(j-1) )*0.5_r8
          endif
          
          if (j .eq. plat) then
              ns_vert(2,i) =  90.0_r8
          else
              ns_vert(2,i) = (latdeg(j) + latdeg(j+1) )*0.5_r8
          endif

          ! Longitude vertices
          ew_vert(1,1)        = (londeg(1,j) - 360.0_r8 + londeg(plon,j))*0.5_r8
          ew_vert(1,2:plon)   = (londeg(1:plon-1,j) + londeg(2:plon,j))*0.5_r8
          ew_vert(2,1:plon-1) = ew_vert(1,2:plon)
          ew_vert(2,plon)     = (londeg(plon,j) + (360.0_r8 + londeg(1,j)))*0.5_r8
          
          del_phi   = sin( ns_vert(2,i)*degtorad ) - sin( ns_vert(1,i)*degtorad )
          del_theta = ( ew_vert(2,i) - ew_vert(1,i) )*degtorad
          area_all(i,j)   = del_theta*del_phi
       enddo
  	 enddo
    endif ! masterproc
!----------

    allocate (area_p(pcols,begchunk:endchunk))

    call scatter_field_to_chunk(1,1,1,plon,area_all,area_p)
    return
    end subroutine get_area_all_p

end module bcc_grid_area