#include <misc.h>
#include <params.h>

module bcc_inirun

  use shr_kind_mod, only : r8 => shr_kind_r8
  use pmgrid,       only : masterproc, plev, plevp

  implicit none

  private
  public :: inirun

contains

  subroutine inirun

    !-----------------------------------------------------------------------
    ! 	... Chemistry module intialization
    !-----------------------------------------------------------------------
    use bcc_tracinp,        only : bndic
    use bcc_mean_mass,      only : init_mean_mass
    use bcc_airplane,       only : airpl_src
    use bcc_photo,          only : photo_inti
    use bcc_lightning,      only : lightning_inti
    use bcc_drydep,         only : drydep_inti
    use bcc_imp_sol,        only : imp_slv_inti
    use bcc_exp_sol,        only : exp_sol_inti
    use bcc_ub_vals,        only : ub_inti
    use bcc_setsox,         only : sox_inti
    use bcc_sethet,         only : sethet_inti
    use bcc_usrrxt,         only : usrrxt_inti
    use bcc_setext,         only : setext_inti
    use bcc_setinv,         only : setinv_inti
    use bcc_airglow,        only : init_airglow
    use bcc_spedata,        only : spedata_ini
    use bcc_iondrag,        only : iondrag_init
    use bcc_qbo,            only : qbo_init
    use bcc_surf,           only : surf_inti
    use bcc_srf_emis,       only : srf_emis_inti
    use bcc_aerosols_intr,  only : mz_aero_initialize
    use bcc_dust_intr,      only : dust_initialize
    use mo_strato_aerosol,  only : aero_init

    use bcc_sulf,           only : sulf_inti
    use bcc_mass_xforms,    only : init_mass_xforms
    use bcc_strato_rates,   only : init_strato_rates
    use bcc_cph,            only : init_cph
    use bcc_strato_sad,     only : strato_sad_inti
    use bcc_sad,            only : sad_inti
    use bcc_solar_parms,    only : solar_parms_init, get_solar_parms
    use bcc_euvac,          only : euvac_init, euvac_set_etf
    use bcc_heatnirco2,     only : heatnirco2_init
    use bcc_waccm_hrates,   only : init_hrates
    use bcc_aurora,         only : aurora_inti
    use bcc_clybry_fam,     only : clybry_fam_init
    use bcc_neu_wetdep,     only : neu_wetdep_init 

	use bcc_chemdr,         only : chemdr_inti
    implicit none

#include <comhyb.h>             ! define hypm
!-------------------------------------------------------------
    character(len=150) :: solar_parms_file
    character(len=150) :: euvac_file
    character(len=150) :: euvacdat_file
    character(len=150) :: photon_file
    character(len=150) :: electron_file

    character(len=150) :: airpl_emis_file
    character(len=150) :: sulf_file
    character(len=150) :: sad_file
    character(len=150) :: depvel_file
    character(len=150) :: xs_coef_file
    character(len=150) :: xs_short_file
    character(len=150) :: xs_long_file
    character(len=150) :: rsf_file
    character(len=150) :: fstrat_file

    character(len=150) :: exo_coldens_file
    character(len=256) :: efield_lflux_file
    character(len=256) :: efield_hflux_file 
    character(len=256) :: efield_wei96_file 
    character(len=256) :: airplane_emiss_file
    character(len=256) :: latwts_file
    character(len=256) :: spe_data_file
    character(len=256) :: solar_data_file
    character(len=256) :: soil_erod_file
    character(len=256) :: bndemis_file
    character(len=256) :: qbo_forcing_file
	logical            :: qbo_use_forcing
	
    real(r8)  ::   f107
    real(r8)  ::   f107a

    real(r8)  ::   lght_no_prd_factor = 1._r8
	integer   ::   i
    character(len=10)  time_type    !  'SERIAL', or 'CYCLICAL'
    integer            cycle_yr     !  for time_type = 'CYCLICAL'

    character(len=256) emisspath
!---------------------------------------------------------------

    do i = len_trim(bndic), 1, -1
       if(bndic(i:i).eq.'/') exit
    enddo
    emisspath = bndic(1:i)

!-----------------------------------------------------------------
   efield_lflux_file = trim(emisspath)//'coeff_lflux.dat'
   efield_hflux_file = trim(emisspath)//'coeff_hflux.dat'
   efield_wei96_file = trim(emisspath)//'wei96.cofcnts'
      
   call iondrag_init( hypm, efield_lflux_file, efield_hflux_file, efield_wei96_file )

!----------------------------------------------------------
   qbo_use_forcing  = .false.
   qbo_forcing_file = ''

!---
!  qbo_use_forcing  = .true.
!  qbo_forcing_file = trim(emisspath)//'/qbocoefficients_c091230.nc'
!---
   call qbo_init( qbo_use_forcing, qbo_forcing_file )

!-----------------------------------------
    call chemdr_inti

    call init_mean_mass

    call init_mass_xforms

    call setinv_inti

    call sethet_inti

    call usrrxt_inti

    call init_hrates

    call init_airglow

    call init_strato_rates

    call init_cph

    !-----------------------------------------------------------------------
    ! 	... Read time-dependent surface variables dataset
    !-----------------------------------------------------------------------

      call surf_inti

    !-----------------------------------------------------------------------
    ! 	... read time-independent airplane emissions
    !-----------------------------------------------------------------------

    airplane_emiss_file = trim(emisspath)//'emissions.aircraft.T42LR.nc'
    latwts_file         = trim(emisspath)//'latwts.dat'

    call airpl_src( airplane_emiss_file, latwts_file )      

    !-----------------------------------------------------------------------
    ! 	... read time-dependent surface flux dataset
    !-----------------------------------------------------------------------

    bndemis_file = trim(emisspath)//'emis.mozart2.c130311.nc'

    call srf_emis_inti( bndemis_file)

    !-----------------------------------------------------------------------
    ! 	... initialize external forcings module
    !       extfrc_lst(: 12) = (/ 'NO      ','NO2     ','CO   ','CH4    ')     
    !-----------------------------------------------------------------------

    call setext_inti

    !-----------------------------------------------------------------------
    ! 	... read the stratospheric sad dataset
    !   ... prescribed strat aero surf area density module
    !-----------------------------------------------------------------------
    !      real(r8), intent(inout) :: sad(pcols, pver) ! stratospheric aerosol surface area density (1/cm)
    !

!    sad_file = trim(emisspath)//'SAD_GS2D_1950-2005_L66.nc.t42.nc'
!    time_type = 'CYCLICAL'        ! 'SERIAL', or 'CYCLICAL'
!    if( time_type .eq. 'CYCLICAL' ) then
!        cycle_yr  =  1971             !  only for time_type = 'CYCLICAL'
!    endif
!
!    call strato_sad_inti (sad_file, time_type, cycle_yr )    ! must be modified 

      sad_file = trim(emisspath)//'WACCM3.0/data/atm/waccm/sulf'// &
                 '/SAD_SAGE_4x5_t42L26.nc'
!      sad_file = '/gfs3/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/sulf'// &
!                '/SAD_SAGE_4x5_t42L26.nc'

     cycle_yr    = 19710101
     time_type   = 'FIXED'                ! 'FIXED' or 'RAMPED'
     call aero_init ( time_type, cycle_yr, sad_file)

    !----------------------------------------------------------------------
    !   ... read sulfate
    !----------------------------------------------------    
    	sulf_file = trim(emisspath)//'sulfate.M1.nc'    ! atm/waccm/sulf/sulfate.M1.nc
        call sulf_inti(sulf_file)                       ! must be modified 

    !-----------------------------------------------------------------------
    !	... initialize the sad module
    !-----------------------------------------------------------------------

    call sad_inti

    !-----------------------------------------------------------------------
    !	... initialize the lightning module
    !-----------------------------------------------------------------------

    call lightning_inti( lght_no_prd_factor )

    !-----------------------------------------------------------------------
    !	... initialize the dry deposition module
    !-----------------------------------------------------------------------

!   bnddvel is the same as  depvel_file
!   depvel_file = atm/cam/chem/trop_mozart/dvel/depvel_monthly.nc
!
    depvel_file = trim(emisspath)//'depvel_monthly_t42_c130307.nc'

    call drydep_inti (depvel_file)

    !-----------------------------------------------------------------------
    !	... Initialize the upper boundary module
    !-----------------------------------------------------------------------
!
!	fstrat_file = atm/cam/chem/trop_mozart/ub/ubvals_b40.20th.track1_1996-2005_c110315.nc
!
    fstrat_file = trim(emisspath)//'ubvals_b40.20th.track1_1996-2005_t42.nc'

    call ub_inti( fstrat_file )

    !-----------------------------------------------------------------------
    ! 	... initialize the co2 nir heating module
    !-----------------------------------------------------------------------

    call heatnirco2_init

    !-----------------------------------------------------------------------
    ! 	... initialize photorate module
    !-----------------------------------------------------------------------
!
!      solar_parms_file = trim(emisspath)//'wasolar_ext.nc'
       solar_parms_file = trim(emisspath)//'proxy_solar_Solomon_Richmond_1845-2008_daily_noleap_c110526.nc'

       call solar_parms_init(solar_parms_file)

       !-----------------------------------------------------------------------
       ! 	... initialize the solar parameters module
       !-----------------------------------------------------------------------
       call get_solar_parms( f107_s = f107, f107a_s = f107a )
!
       !-----------------------------------------------------------------------
       ! 	... initialize the euvac etf module
       !-----------------------------------------------------------------------
!
!      euvac_file = /space/gpfs07/model/cesm_inputs/atm/waccm/phot/euvac_v1.2.nc
!
       euvac_file = trim(emisspath)//'euvac_v1.2.nc'

       call euvac_init(euvac_file)

       call euvac_set_etf( f107, f107a )
       !-----------------------------------------------------------------------
       ! 	... initialize the solar proton event module
       !-----------------------------------------------------------------------
!
!      spe_data_file= atm/waccm/solar/spes.1963-2009.c100804.nc
!
       spe_data_file= trim(emisspath)//'spes_1963-2011_c120302.nc'

       call spedata_ini (spe_data_file)
!
!       solar_data_file= atm/cam/solar/spectral_irradiance_Lean_1610-2009_ann_c100405.nc
!       xs_long_file   = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/temp_prs_GT200nm_jpl06_c080930.nc
!       xs_coef_file   = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/effxstex.txt
!       xs_short_file  = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/xs_short_jpl06_c080930.nc
!       rsf_file       = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/RSF_GT200nm_v3.0_c080811.nc
!       euvacdat_file  = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/euvac.dat
!       photon_file    = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/photon.dat
!       electron_file  = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/electron.dat
!
       solar_data_file= trim(emisspath)//'spectral_irradiance_Lean_1610-2009_ann_c100405.nc'    ! missing 
       xs_long_file   = trim(emisspath)//'temp_prs_GT200nm_jpl06_c080930.nc'
       xs_coef_file   = trim(emisspath)//'effxstex.txt'
       xs_short_file  = trim(emisspath)//'xs_short_jpl06_c080930.nc'
       rsf_file       = trim(emisspath)//'RSF_GT200nm_v3.0_c080811.nc'
       euvacdat_file  = trim(emisspath)//'euvac_v1.2.dat'
       photon_file    = trim(emisspath)//'photon.dat'
       electron_file  = trim(emisspath)//'electron.dat'
	   exo_coldens_file = trim(emisspath)//'exo_coldens_c20130311.nc'
  
       call photo_inti( solar_data_file, &
                        xs_coef_file,  xs_short_file, xs_long_file, rsf_file,         &
                        euvacdat_file, photon_file,  electron_file, exo_coldens_file  ) 

    !-----------------------------------------------------------------------
    !	... initialize the implicit solver
    !-----------------------------------------------------------------------
    call imp_slv_inti()

    call exp_sol_inti()

    !-----------------------------------------------------------------------
    !       ... initialize the stratospheric ozone source
    !-----------------------------------------------------------------------
    !
    ! initialize the various aerosols components
    !

    call sox_inti()

    !-----------------------------------------------------------------------
    !	... initialize ion production
    !-----------------------------------------------------------------------
    call aurora_inti

    call neu_wetdep_init()

    call clybry_fam_init()

!---------------------------------------------------------------------
!  ... initialize dust erodibility
!--------------------------------------------------------------------
! soil_erod_file: atm/cam/dst/dst_64x128_c090203.nc
!
    soil_erod_file = trim(emisspath)//'dst_64x128_t42_c20130311.nc'
    call dust_initialize(soil_erod_file)

    call mz_aero_initialize

  end subroutine inirun

end module bcc_inirun
