#include <misc.h>
#include <params.h>

module bcc_srf_emis

!---------------------------------------------------------------
! 	... Interpolation surface emissions
!---------------------------------------------------------------

      use pmgrid,         only: plon, plat, masterproc

      implicit none

      public  :: srf_emis_inti, srf_emis 
	  public  :: nppbdy

      private

      save

      real, allocatable, dimension(:,:,:) :: &
           ch2oembdy,  &     
           ch4embdy,   &     
           coembdy,    &     
           h2embdy,    &     
           n2oembdy,   &    
           noxembdy,   &    
           ch3coohembdy, &        
           nh3embdy,     &     
           dmsembdy,     &   
           so2embdy,     &  
           cb1embdy,     &  
           cb2embdy,     &  
           oc1embdy,     &  
           oc2embdy,     &
           nppbdy

      real    :: sf(plat)
      real    :: factor
      integer ::        & 
           ch2o_ndx,    &
           ch4_ndx,     &
           co_ndx,      &
           h2_ndx,      &
           n2o_ndx,     &
           no_ndx,      &
           ch3cooh_ndx, &
           nh3_ndx,     &
           dms_ndx,     &
           so2_ndx,     &
           cb1_ndx,     &
           cb2_ndx,     &
           oc1_ndx,     &
           oc2_ndx   
      logical ::        &
           ch2o_em,     &
           ch4_em,      &
           co_em,       &
           h2_em,       &
           n2o_em,      &
           no_em,       &
           ch3cooh_em,  &
           nh3_em,      &
           dms_em,      &
           so2_em,      &
           cb1_em,      &
           cb2_em,      &
           oc1_em,      &
           oc2_em     

contains

      subroutine srf_emis_inti(bndemis_file)
!-----------------------------------------------------------------------
! 	... Read the primary emission flux datasets 
!-----------------------------------------------------------------------

      use bcc_chem_utls,     only : get_spc_ndx, has_srfems
      use bcc_sim_chm,       only : latwts  
      use bcc_chem_mods,     only : adv_mass
      use bcc_const_mozart,  only : d2r, pi, rearth
      use bcc_tracinp,       only : bndic

      use ppgrid,         only:  pcols, begchunk, endchunk   
      use phys_grid,      only:  scatter_field_to_chunk    
      use ioFileMod,      only:  getfil
      use abortutils,     only:  endrun

      implicit none

!-----
!
      character(len=*), intent(in) :: bndemis_file    ! full pathname for Mozart

!-----------------------------------------------------------------------
! 	... dummy arguments

      integer  :: pplon = 1

!-----------------------------------------------------------------------
! 	... Local variables
!-----------------------------------------------------------------------
      real, allocatable, dimension(:,:,:) :: &
           ch2oem,              &
           ch4em,               &
           coem,                &
           h2em,                &
           n2oem,               &
           noxem,               &
           ch3coohem,           &
           nh3em,               &
           dmsem,               &
           so2em,               &
           cb1em,               &
           cb2em,               &
           oc1em,               &
           oc2em,               &
		   npp

      real, parameter :: amufac = 1.65979e-23         ! 1.e4* kg / amu
      integer :: astat
      integer :: i, j, ncid, did, vid                            ! Indices
      real    :: mw
      real    :: seq
      character(len=10)  :: species
      character(len=256) :: locfn    ! netcdf local filename to open   add by zf 2008.03.13
	  integer  ::  nlon, nlat

!      bndch2o     = trim(emisspath)//'/emissions.ch2o.surface.T42LR.nc'
!      bndch4      = trim(emisspath)//'/emissions.ch4.surface.T42LR.nc'
!      bndco       = trim(emisspath)//'/emissions.co.surface.T42LR.nc'
!      bndh2       = trim(emisspath)//'/emissions.h2.surface.T42LR.nc'
!      bndn2o      = trim(emisspath)//'/emissions.n2o.surface.T42LR.nc'
!      bndnox      = trim(emisspath)//'/emissions.nox.surface.T42LR.nc'
!      bndnpp      = trim(emisspath)//'/emissions.npp.surface.T42LR.nc'
!      bndch3cooh  = trim(emisspath)//'/emissions.CH3COOH.surface.1.9x2.5_c110105.nc'
!      bndnh3      = trim(emisspath)//'/emissions.NH3.surface.1.9x2.5_c110105.nc'
!      bnddms      = trim(emisspath)//'/emissions.DMS.surface.T42LR.nc'
!      bndso2      = trim(emisspath)//'/emissions.SO2.surface.T42LR.nc'
!      bndso4      = trim(emisspath)//'/aerocom_SO4_surface_2000.c080807.nc'
!      bndcb1      = trim(emisspath)//'/aerocom_CB1_2000.c080807.nc'
!      bndcb2      = trim(emisspath)//'/emissions.CB2.surface.1.9x2.5_c110426.nc'
!      bndoc1      = trim(emisspath)//'/aerocom_OC1_2000.nosoa.c080807.nc'
!      bndoc2      = trim(emisspath)//'/emissions.OC2.surface.1.9x2.5_c110105.nc'
!-------------
!      bndch2o     = '/home/wutw/BCC_AGCM2.0.2/inputdata/chem_data/emissions.ch2o.surface.T42LR.nc'
!      bndch4      = '/home/wutw/BCC_AGCM2.0.2/inputdata/chem_data/emissions.ch4.surface.T42LR.nc'
!      bndco       = '/home/wutw/BCC_AGCM2.0.2/inputdata/chem_data/emissions.co.surface.T42LR.nc'
!      bndh2       = '/home/wutw/BCC_AGCM2.0.2/inputdata/chem_data/emissions.h2.surface.T42LR.nc'
!      bndn2o      = '/home/wutw/BCC_AGCM2.0.2/inputdata/chem_data/emissions.n2o.surface.T42LR.nc'
!      bndnox      = '/home/wutw/BCC_AGCM2.0.2/inputdata/chem_data/emissions.nox.surface.T42LR.nc'
!      bndnpp      = '/home/wutw/BCC_AGCM2.0.2/inputdata/chem_data/emissions.npp.surface.T42LR.nc'
!      no_emis_file          = atm/cam/chem/emis/poet_reas_gfed2_ex/emissions.NO.surface.1.9x2.5_c110105.nc
!      ch3cooh_emis_file     = atm/cam/chem/emis/poet_reas_gfed2_ex/emissions.CH3COOH.surface.1.9x2.5_c110105.nc
!      nh3_emis_file         = atm/cam/chem/emis/poet_reas_gfed2_ex/emissions.NH3.surface.1.9x2.5_c110105.nc
!      dms_emis_file        =atm/cam/chem/trop_mozart/emis/emissions.DMS.surface.T42LR.nc
!      so2_emis_file        =atm/cam/chem/trop_mozart/emis/emissions.SO2.surface.T42LR.nc
!      so4_emis_file	      =atm/cam/chem/trop_mozart_aero/emis/aerocom_SO4_surface_2000.c080807.nc
!      cb1_emis_file	      =atm/cam/chem/trop_mozart_aero/emis/aerocom_CB1_2000.c080807.nc
!      cb2_emis_file       =atm/cam/chem/emis/1992-2010/emissions.CB2.surface.1.9x2.5_c110426.nc
!      oc1_emis_file	      =atm/cam/chem/trop_mozart_aero/emis/aerocom_OC1_2000.nosoa.c080807.nc
!      oc2_emis_file         = atm/cam/chem/emis/poet_reas_gfed2_ex/emissions.OC2.surface.1.9x2.5_c110105.nc
!
!-----------------------------------------------------------------------
! 	... Setup the indicies
!-----------------------------------------------------------------------
      ch2o_ndx     = get_spc_ndx( 'CH2O' )
      ch4_ndx      = get_spc_ndx( 'CH4' )
      co_ndx       = get_spc_ndx( 'CO' )
      h2_ndx       = get_spc_ndx( 'H2' )
      n2o_ndx      = get_spc_ndx( 'N2O' )
      no_ndx       = get_spc_ndx( 'NO' )
      ch3cooh_ndx  = get_spc_ndx( 'CH3COOH' )
      nh3_ndx      = get_spc_ndx( 'NH3' )
      dms_ndx      = get_spc_ndx( 'DMS' )
      so2_ndx      = get_spc_ndx( 'SO2' )
      cb1_ndx      = get_spc_ndx( 'CB1' )
      cb2_ndx      = get_spc_ndx( 'CB2' )
      oc1_ndx      = get_spc_ndx( 'OC1' )
      oc2_ndx      = get_spc_ndx( 'OC2' )

      ch2o_em     = has_srfems( 'CH2O' )
      ch4_em      = has_srfems( 'CH4' )
      co_em       = has_srfems( 'CO' )
      h2_em       = has_srfems( 'H2' )
      n2o_em      = has_srfems( 'N2O' )
      no_em       = has_srfems( 'NO' )
      ch3cooh_em  = has_srfems( 'CH3COOH' )
      nh3_em      = has_srfems( 'NH3' )
      dms_em      = has_srfems( 'DMS' )
      so2_em      = has_srfems( 'SO2' )
      cb1_em      = has_srfems( 'CB1' )
      cb2_em      = has_srfems( 'CB2' )
      oc1_em      = has_srfems( 'OC1' )
      oc2_em      = has_srfems( 'OC2' )

      if (masterproc) then
          write(*,*) 'srf_emis_inti : diagnostic'
      endif

!-----------------------------------------------------------------------
! 	... allocate working arrays
!-----------------------------------------------------------------------
      allocate( &
           ch2oem(plon,plat,12),&
           ch4em(plon,plat,12),&
           coem(plon,plat,12),&
           h2em(plon,plat,12),&
           n2oem(plon,plat,12),&
           noxem(plon,plat,12),&
           ch3coohem(plon,plat,12),&
           nh3em(plon,plat,12),&
           dmsem(plon,plat,12),&
           so2em(plon,plat,12),&
           cb1em(plon,plat,12),&
           cb2em(plon,plat,12),&
           oc1em(plon,plat,12),&
           oc2em(plon,plat,12),&
		   npp(plon,plat,12), &
  	            stat=astat )

      if( astat/= 0 ) then
  	      write(*,*) 'srf_emis_inti: failed to allocate wrking arrays; error = ',astat
	      call endrun
      end if

!-----------------------------------------------------------------------
! 	... Diagnostics setup
!-----------------------------------------------------------------------
      seq = 2.*pi*1.e4*rearth**2/REAL(plon)
      sf(:plat) = seq*latwts(:plat)
      factor = 86400. * 365. &   ! sec / year
             / 6.022e23 &        ! molec / mole 
             * 1.e-12            ! Tg / g

     if (masterproc) then

       call getfil (bndemis_file, locfn)
       call wrap_open(locfn,0,ncid)

       call wrap_inq_dimid( ncid, 'lon', did )
       call wrap_inq_dimlen( ncid, did, nlon )

       call wrap_inq_dimid( ncid, 'lat', did )
       call wrap_inq_dimlen( ncid, did, nlat )

       if ( nlon .ne. plon .or. nlat .ne. plat) then
	      write(6,*) 'bcc_srf_emis.F90 :  nlon .ne. plon .or. nlat .ne. plat'
          call endrun
       end if
!-----------------------------------------------------------------------
! 	... CH2O
!-----------------------------------------------------------------------
      if( ch2o_em ) then
         species  = 'CH2O'
         mw       = adv_mass(ch2o_ndx)               ! g / mole

		 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, ch2oem )
         ch2oem(:,:,:)  = amufac * adv_mass(ch2o_ndx) * ch2oem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... CH4
!-----------------------------------------------------------------------
      if( ch4_em ) then
         species  = 'CH4'
         mw       = adv_mass(ch4_ndx)               ! g / mole

		 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, ch4em )
         ch4em(:,:,:)   = amufac * adv_mass(ch4_ndx) * ch4em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... CO
!-----------------------------------------------------------------------
      if( co_em ) then
         species  = 'CO'
         mw       = adv_mass(co_ndx)               ! g / mole

		 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, coem )
         coem(:,:,:) = amufac * adv_mass(co_ndx) * coem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... H2
!-----------------------------------------------------------------------
      if( h2_em ) then
         species  = 'H2'
         mw       = adv_mass(h2_ndx)               ! g / mole

		 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, h2em )
         h2em(:,:,:) = amufac * adv_mass(h2_ndx) * h2em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... N2O
!-----------------------------------------------------------------------
      if( n2o_em ) then
         species  = 'N2O'
         mw       = adv_mass(n2o_ndx)               ! g / mole

		 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, n2oem )
         n2oem(:,:,:) = amufac * adv_mass(n2o_ndx) * n2oem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... NO
!-----------------------------------------------------------------------
      if( no_em ) then
         species  = 'NO'
         mw       = 14.00674                      ! g N / mole

	 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, noxem )
         noxem(:,:,:)   = amufac * adv_mass(no_ndx) * noxem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... CH3COOH
!-----------------------------------------------------------------------
      if( ch3cooh_em ) then
         species  = 'CH3COOH'
         mw       = adv_mass(ch3cooh_ndx)               ! g / mole

	 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, ch3coohem )
         ch3coohem(:,:,:) = amufac * adv_mass(ch3cooh_ndx) * ch3coohem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... NH3
!-----------------------------------------------------------------------
      if( nh3_em ) then
         species  = 'NH3'
         mw       = adv_mass(nh3_ndx)               ! g / mole

		 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, nh3em )
         nh3em(:,:,:) = amufac * adv_mass(nh3_ndx) * nh3em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... DMS
!-----------------------------------------------------------------------
      if( dms_em ) then
         species  = 'DMS'
         mw       = adv_mass(dms_ndx)               ! g / mole

		 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, dmsem )
         dmsem(:,:,:) = amufac * adv_mass(dms_ndx) * dmsem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... SO2
!-----------------------------------------------------------------------
      if( so2_em ) then
         species  = 'SO2'
         mw       = adv_mass(so2_ndx)               ! g / mole

		 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, so2em )
         so2em(:,:,:) = amufac * adv_mass(so2_ndx) * so2em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... CB1
!-----------------------------------------------------------------------
      if( cb1_em ) then
         species  = 'CB1'
         mw       = adv_mass(cb1_ndx)               ! g / mole

	 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, cb1em )
         cb1em(:,:,:) = amufac * adv_mass(cb1_ndx) * cb1em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... CB2
!-----------------------------------------------------------------------
      if( cb2_em ) then
         species  = 'CB2'
         mw       = adv_mass(cb2_ndx)               ! g / mole

	 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, cb2em )
         cb2em(:,:,:) = amufac * adv_mass(cb2_ndx) * cb2em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... OC1
!-----------------------------------------------------------------------
      if( oc1_em ) then
         species  = 'OC1'
         mw       = adv_mass(oc1_ndx)               ! g / mole

	 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, oc1em )
         oc1em(:,:,:) = amufac * adv_mass(oc1_ndx) * oc1em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... OC2
!-----------------------------------------------------------------------
      if( oc2_em ) then
         species  = 'OC2'
         mw       = adv_mass(oc2_ndx)               ! g / mole

	 call wrap_inq_varid( ncid, trim(species), vid )
         call wrap_get_var_realx( ncid, vid, oc2em )
         oc2em(:,:,:) = amufac * adv_mass(oc2_ndx) * oc2em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... NPP
!-----------------------------------------------------------------------
      species  = 'NPP'
      mw       = 1.               ! g / mole

      call wrap_inq_varid( ncid, trim(species), vid )
      call wrap_get_var_realx( ncid, vid, npp )

   endif   ! END masterproc

   allocate( &
           ch2oembdy(pcols,begchunk:endchunk,12),&
           ch4embdy(pcols,begchunk:endchunk,12),&
           coembdy(pcols,begchunk:endchunk,12),&
           h2embdy(pcols,begchunk:endchunk,12),&
           n2oembdy(pcols,begchunk:endchunk,12),&
           noxembdy(pcols,begchunk:endchunk,12),&
           ch3coohembdy(pcols,begchunk:endchunk,12),&
           nh3embdy(pcols,begchunk:endchunk,12),&
           dmsembdy(pcols,begchunk:endchunk,12),&
           so2embdy(pcols,begchunk:endchunk,12),&
           cb1embdy(pcols,begchunk:endchunk,12),&
           cb2embdy(pcols,begchunk:endchunk,12),&
           oc1embdy(pcols,begchunk:endchunk,12),&
           oc2embdy(pcols,begchunk:endchunk,12),&
		   nppbdy(plon,plat,12), &
		stat=astat )

      if( astat/= 0 ) then
         write(*,*) 'srf_emis_inti: failed to allocate wrking arrays; error = ',astat
	     call endrun
      end if

      call scatter_field_to_chunk(1,1,12,plon,ch2oem,ch2oembdy)
      call scatter_field_to_chunk(1,1,12,plon,ch4em,ch4embdy)
      call scatter_field_to_chunk(1,1,12,plon,coem,coembdy)
      call scatter_field_to_chunk(1,1,12,plon,h2em,h2embdy)
      call scatter_field_to_chunk(1,1,12,plon,n2oem,n2oembdy)
      call scatter_field_to_chunk(1,1,12,plon,noxem,noxembdy)
      call scatter_field_to_chunk(1,1,12,plon,ch3coohem,ch3coohembdy)
      call scatter_field_to_chunk(1,1,12,plon,nh3em,nh3embdy)
      call scatter_field_to_chunk(1,1,12,plon,dmsem,dmsembdy)
      call scatter_field_to_chunk(1,1,12,plon,so2em,so2embdy)
      call scatter_field_to_chunk(1,1,12,plon,cb1em,cb1embdy)
      call scatter_field_to_chunk(1,1,12,plon,cb2em,cb2embdy)
      call scatter_field_to_chunk(1,1,12,plon,oc1em,oc1embdy)
      call scatter_field_to_chunk(1,1,12,plon,oc2em,oc2embdy)
      call scatter_field_to_chunk(1,1,12,plon,npp,nppbdy)

      end subroutine srf_emis_inti

      subroutine read_emis( plonl, platl, pplon, species, mw, units, ncid, emis )
!-----------------------------------------------------------------------
!       ... Read surface emissions from NetCDF file
!-----------------------------------------------------------------------

      use bcc_netcdf
      use bcc_const_mozart,  only: phi, lam, d2r
      use abortutils,        only: endrun
      use error_messages,    only: handle_ncerr

      implicit none
!-----------------------------------------------------------------------
!       ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) :: plonl, platl, pplon
      integer, intent(in) :: ncid
      real, intent(in)    :: mw
      real, dimension(plonl,platl,12), intent(out) :: emis
      character(len=*), intent(in) :: species, units

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer :: j, jl, ju, m               ! Indices
      integer :: jlim(2)
      integer :: ierr
      integer :: vid, nlat, nlon, nmonth, nvars, ndims, attlen
      integer :: dimid_lat, dimid_lon, dimid_month
      integer, allocatable :: dimid(:)
      integer :: gndx
      real, allocatable :: lat(:), lon(:)
      real :: total, total_wrk, scale_factor
      real :: wrk(plonl,platl,12)
      character(len=NF_MAX_NAME) :: varname
      character(len=100) :: var_longname

!-----------------------------------------------------------------------
!       ... Get grid dimensions from file
!-----------------------------------------------------------------------
      call handle_ncerr( NF_INQ_DIMID( ncid, 'lat', dimid_lat ), &
                         'read_emis: Failed to find dimension lat for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lat, nlat ), &
                         'read_emis: Failed to get length of dimension lat for species ' // &
                         trim(species) )
      allocate( lat(nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'read_emis: lat allocation error = ',ierr
         call endrun
      end if
      call handle_ncerr( NF_INQ_VARID( ncid, 'lat', vid ), &
                         'read_emis: Failed to find variable lat for species ' // trim(species) )
      call handle_ncerr( NF_GET_VAR_DOUBLE( ncid, vid, lat ), &
                         'read_emis: Failed to read variable lat for species ' // trim(species) )
      lat(:nlat) = lat(:nlat) * d2r
 
      call handle_ncerr( NF_INQ_DIMID( ncid, 'lon', dimid_lon ), &
                         'read_emis: Failed to find dimension lon for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lon, nlon ), &
                         'read_emis: Failed to find dimension lon for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lon, nlon ), &
                         'read_emis: Failed to get length of dimension lon for species ' // &
                         trim(species) )
      allocate( lon(nlon), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'read_emis: lon allocation error = ',ierr
         call endrun
      end if
      call handle_ncerr( NF_INQ_VARID( ncid, 'lon', vid ), &
                         'read_emis: Failed to find variable lon for species ' // trim(species) )
      call handle_ncerr( NF_GET_VAR_DOUBLE( ncid, vid, lon ), &
                         'read_emis: Failed to read variable lon for species ' // trim(species) )
      lon(:nlon) = lon(:nlon) * d2r
 
      call handle_ncerr( NF_INQ_DIMID( ncid, 'month', dimid_month ), &
                         'read_emis: Failed to find dimension month for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_month, nmonth ), &
                         'read_emis: Failed to get length of dimension month for species ' // &
                         trim(species) )
      if( nmonth /= 12 ) then
         write(*,*) 'read_emis: Error, number of months = ',nmonth,' for species ' // trim(species)
         call endrun
      end if

      call handle_ncerr( NF_INQ_NVARS( ncid, nvars ), &
                         'read_emis: Failed to get number of variables for species ' // &
                         trim(species) )

!-----------------------------------------------------------------------
!       ... Set total emissions to zero
!-----------------------------------------------------------------------
      emis(:,:,:) = 0.
!-----------------------------------------------------------------------
!       ... Loop over file variables
!-----------------------------------------------------------------------
      write(*,*) 'Annual average ',trim(species),' emissions ',trim(units)
      do vid = 1,nvars
         ierr = NF_INQ_VARNAME( ncid, vid, varname )
         if( ierr /= 0 ) then
            write(*,*) 'read_emis: Failed to get name of variable # ',vid, &
                '; species=' // trim(species)
            call endrun
         end if
         call handle_ncerr( NF_INQ_VARNDIMS( ncid, vid, ndims ), &
                            'read_emis: Failed to get number of dimensions for ' // &
                            'variable ' // trim(varname) // ', species=' // trim(species) )
         if( ndims < 3 ) then
            cycle
         elseif( ndims > 3 ) then
            write(*,*) 'read_emis: Skipping variable ', trim(varname),', ndims = ',ndims, &
                       ' , species=',trim(species)
            cycle
         end if
!-----------------------------------------------------------------------
!       ... Check order of dimensions. Must be (lon, lat, month).
!-----------------------------------------------------------------------
         allocate( dimid( ndims ), stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'read_emis: dimid allocation error = ',ierr
            call endrun
         end if
         call handle_ncerr( NF_INQ_VARDIMID( ncid, vid, dimid ), &
                            'read_emis: Failed to get dimension IDs for variable ' // &
                            trim(varname)  // ', species=' // trim(species) )
         if( dimid(1) /= dimid_lon .or. dimid(2) /= dimid_lat .or. dimid(3) /= dimid_month ) then
            write(*,*) 'read_emis: Dimensions in wrong order for variable ',trim(varname)
            write(*,*) '...      Expecting (lon, lat, month)'
            call endrun
         end if
         deallocate( dimid, stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'read_emis: Failed to deallocate dimid, ierr = ',ierr
            call endrun
         end if
!-----------------------------------------------------------------------
!       ... Check for long_name of variable
!-----------------------------------------------------------------------
         var_longname = ' '
         ierr = NF_GET_ATT_TEXT( ncid, vid, 'long_name', var_longname )
         if( ierr /= 0 ) then
            var_longname = trim( varname )
         end if
!-----------------------------------------------------------------------
!       ... Read data from this variable
!-----------------------------------------------------------------------
         call handle_ncerr( NF_GET_VARA_DOUBLE( ncid, vid, &
                                                (/ 1, 1, 1/), &                 ! start
                                                (/ nlon, nlat, 12 /), &  ! count
                                                wrk), &
                            'read_emis: Failed to read variable ' // trim( varname ) // &
                            ', species=' // trim(species) )
!-----------------------------------------------------------------------
!       ... Check for scale_factor
!-----------------------------------------------------------------------
         ierr = NF_GET_ATT_DOUBLE( ncid, vid, 'scale_factor', scale_factor )
         if( ierr /= NF_NOERR  ) then
            scale_factor = 1.
         end if

         total = 0.
         do m = 1,12
           do j = 1,plat
             total = total + sf(j) * SUM( wrk( :plon, j, m ) )
           enddo
           emis (:,:,m) = emis(:,:,m) + wrk(:,:,m)
         enddo
!-----------------------------------------------------------------------
!       ... Get global emission from this source
!-----------------------------------------------------------------------
         total = total * factor * mw / 12. ! convert from molec/s to Tg/y, divide by 12 (months)
 
            write(*,'(2a10,1x,a50,1x,f10.3)') trim(species), trim(varname), trim(var_longname), total
      end do  ! (vid)
!-----------------------------------------------------------------------
!       ... Close NetCDF file
!-----------------------------------------------------------------------
      call handle_ncerr( NF_CLOSE( ncid ), &
                         'read_emis: Failed to close NetCDF file, species=' // trim(species) )

!-----------------------------------------------------------------------
!       ... Get total global emission for this species
!-----------------------------------------------------------------------
      total = 0.
      do j = 1,platl
         total = total + sf(j) * SUM( emis( :plonl, j, :12 ) )
      end do
      total = total * factor * mw / 12. ! convert from molec/s to Tg/y, divide by 12 (months)
 
      write(*,'(2a10,1x,a50,1x,f10.3)') trim(species), 'TOTAL', 'TOTAL', total

      end subroutine read_emis

      subroutine srf_emis( lchnk, phir, calday, sflx, oro, &
                           local_angle, polar_night, polar_day, sunon, sunoff, plonl, &
                           nem2, em2data )
!--------------------------------------------------------
!       ... Form the surface fluxes for this
!           latitude slice
!--------------------------------------------------------
      use shr_kind_mod, only: r8 => shr_kind_r8
      use bcc_const_mozart, only : pi, d2r, lat60, lat70
      use bcc_chem_mods,    only : gas_pcnst
      use bcc_calendar,     only : caldayr
      use bcc_chem_mods,    only : adv_mass

      use prescribed_em_d2, only:  &
          idxCH2O , &
          idxCH4  , &
          idxCO   , &
          idxH2   , &
          idxN2O  , &
          idxNO   , &
          idxDMS  , &
	  idxSO2  , &
	  idxCB1  , &
	  idxCB2  , &
          idxNH3

      implicit none

      integer, parameter :: idxCH3COOH = 0
	  integer, parameter :: idxOC1     = 0
 	  integer, parameter :: idxOC2     = 0

!--------------------------------------------------------
!       ... Dummy arguments
!--------------------------------------------------------
      integer, intent(in) ::   plonl, lchnk
      integer, intent(in) ::   oro(plonl)              ! incoming land surface type
      real(r8), intent(in)    ::   sunon(plonl)            ! sunrise angle in radians
      real(r8), intent(in)    ::   sunoff(plonl)           ! sundown angle in radians
      real(r8), intent(in)    ::   local_angle(plonl)      ! local "time" angle
      real(r8), intent(inout) ::   sflx(plonl,gas_pcnst)   ! surface emissions ( kg/m^2/s )
      real, intent(in)    ::   calday                  ! time of year in days
      real, intent(in)    ::   phir(plonl)             ! latitude                
      logical, intent(in) ::   polar_night(plonl)      ! polar night flag
      logical, intent(in) ::   polar_day(plonl)        ! polar day flag

!---
! wtw 2012.10
!
      integer, intent(in) :: nem2
      real(r8), intent(in)    :: em2data(plonl,nem2)
      real(r8), parameter     :: amufac = 1.65979e-23         ! 1.e4* kg / amu
!--------------------------------------------------------
!       ... Local variables
!--------------------------------------------------------
      integer, parameter :: land = 1, ocean = 0
      integer  ::  surf_type
      integer  ::  i, m, k, last, next
      integer  ::  dates(12) = (/ 116, 214, 316, 415,  516,  615, &
                                  716, 816, 915, 1016, 1115, 1216 /)
      real     ::  factor, dels
      real     ::  dayfrac            ! fration of day in light
      real     ::  iso_off            ! time iso flux turns off
      real     ::  iso_on             ! time iso flux turns on
      real, save ::  days(12)
      logical, save  ::  entered = .false.

!--------------------------------------------------------
!       ... Initialize the monthly day of year times
!--------------------------------------------------------
      if( .not. entered ) then
         do m = 1,12
            days(m) = caldayr( dates(m), 0 )
         end do
         entered = .true.
      end if

!--------------------------------------------------------
!       ... Setup the time interpolation
!--------------------------------------------------------
      if( calday < days(1) ) then
         next = 1
         last = 12
         dels = (365. + calday - days(12)) / (365. + days(1) - days(12))
      else if( calday >= days(12) ) then
         next = 1
         last = 12
         dels = (calday - days(12)) / (365. + days(1) - days(12))
      else
         do m = 11,1,-1
            if( calday >= days(m) ) then
               exit
            end if
         end do
         last = m
         next = m + 1
         dels = (calday - days(m)) / (days(m+1) - days(m))
      end if
      dels = max( min( 1.,dels ),0. )

!--------------------------------------------------------
!       ... Set non-zero emissions
!--------------------------------------------------------

      if( ch2o_em ) then
        if( idxCH2O < 1 ) then
            sflx(:,ch2o_ndx) = ch2oembdy(:,lchnk,last) &
                          + dels * (ch2oembdy(:,lchnk,next) - ch2oembdy(:,lchnk,last))
		else
            sflx(:,ch2o_ndx) = amufac * adv_mass(ch2o_ndx) * em2data(:,idxCH2O)   ! wtw
        endif
      end if

      if( ch4_em ) then
        if( idxCH4 < 1 ) then
            sflx(:,ch4_ndx) = ch4embdy(:,lchnk,last) &
                           + dels * (ch4embdy(:,lchnk,next) - ch4embdy(:,lchnk,last))
        else
            sflx(:,ch4_ndx) = amufac * adv_mass(ch4_ndx) * em2data(:,idxCH4)   ! wtw
		endif
      end if

      if( co_em ) then
	    if(idxCO < 1) then
            sflx(:,co_ndx) = coembdy(:,lchnk,last) &
                       + dels * (coembdy(:,lchnk,next) - coembdy(:,lchnk,last))
        else
            sflx(:,co_ndx) = amufac * adv_mass(co_ndx) * em2data(:,idxCO)   ! wtw
        endif
      end if

      if( h2_em ) then
	     if(idxH2 < 1) then
            sflx(:,h2_ndx) = h2embdy(:,lchnk,last) &
                        + dels * (h2embdy(:,lchnk,next) - h2embdy(:,lchnk,last))
		 else
            sflx(:,h2_ndx) = amufac * adv_mass(h2_ndx) * em2data(:,idxH2)   ! wtw
         endif
      end if

      if( n2o_em ) then
         if(idxN2O < 1) then
            sflx(:,n2o_ndx) = n2oembdy(:,lchnk,last) &
                        + dels * (n2oembdy(:,lchnk,next) - n2oembdy(:,lchnk,last))
         else
            sflx(:,n2o_ndx) = amufac * adv_mass(n2o_ndx) * em2data(:,idxN2O)   ! wtw
         endif
      end if

      if( no_em ) then
         if(idxNO < 1) then
            sflx(:,no_ndx) = noxembdy(:,lchnk ,last) &
                       + dels * (noxembdy(:,lchnk ,next) - noxembdy(:,lchnk,last))
         else
            sflx(:,no_ndx) = amufac * adv_mass(no_ndx) * em2data(:,idxNO)   ! wtw
         endif
      end if

      if( ch3cooh_em ) then
	 if(idxCH3COOH < 1) then
            sflx(:,ch3cooh_ndx) = ch3coohembdy(:,lchnk,last) &
                         + dels * (ch3coohembdy(:,lchnk,next) - ch3coohembdy(:,lchnk,last))
         else
            sflx(:,ch3cooh_ndx) = amufac * adv_mass(ch3cooh_ndx) * em2data(:,idxCH3COOH)   ! wtw
         endif
      end if

      if( nh3_em ) then
	     if(idxNH3 < 1) then
            sflx(:,nh3_ndx) = nh3embdy(:,lchnk,last) &
                          + dels * (nh3embdy(:,lchnk,next) - nh3embdy(:,lchnk,last))
         else
            sflx(:,nh3_ndx) = amufac * adv_mass(nh3_ndx) * em2data(:,idxNH3)   ! wtw
         endif
      end if

      if( dms_em ) then
         if(idxDMS < 1) then
            sflx(:,dms_ndx) = dmsembdy(:,lchnk,last) &
                          + dels * (dmsembdy(:,lchnk,next) - dmsembdy(:,lchnk,last))
         else
            sflx(:,dms_ndx) = amufac * adv_mass(dms_ndx) * em2data(:,idxDMS)   ! wtw
         endif
      end if

      if( so2_em ) then
         if(idxSO2 < 1) then
            sflx(:,so2_ndx) = so2embdy(:,lchnk,last) &
                          + dels * (so2embdy(:,lchnk,next) - so2embdy(:,lchnk,last))
         else
            sflx(:,so2_ndx) = amufac * adv_mass(so2_ndx) * em2data(:,idxSO2)   ! wtw
         endif
      end if

      if( cb1_em ) then
         if(idxCB1 < 1) then
            sflx(:,cb1_ndx) = cb1embdy(:,lchnk,last) &
                         + dels * (cb1embdy(:,lchnk,next) - cb1embdy(:,lchnk,last))
         else
            sflx(:,cb1_ndx) = amufac * adv_mass(cb1_ndx) * em2data(:,idxCB1)   ! wtw
         endif
      end if

      if( cb2_em ) then
         if(idxCB2 < 1) then
            sflx(:,cb2_ndx) = cb2embdy(:,lchnk,last) &
                          + dels * (cb2embdy(:,lchnk,next) - cb2embdy(:,lchnk,last))
         else
            sflx(:,cb2_ndx) = amufac * adv_mass(cb2_ndx) * em2data(:,idxCB2)   ! wtw
         endif
      end if

      if( oc1_em ) then
         if(idxOC1 < 1) then
           sflx(:,oc1_ndx) = oc1embdy(:,lchnk,last) &
                          + dels * (oc1embdy(:,lchnk,next) - oc1embdy(:,lchnk,last))
         else
           sflx(:,oc1_ndx) = amufac * adv_mass(oc1_ndx) * em2data(:,idxOC1)   ! wtw
         endif
      end if

      if( oc2_em ) then
         if(idxOC2 < 1) then
           sflx(:,oc2_ndx) = oc2embdy(:,lchnk,last) &
                          + dels * (oc2embdy(:,lchnk,next) - oc2embdy(:,lchnk,last))
         else
           sflx(:,oc2_ndx) = amufac * adv_mass(oc2_ndx) * em2data(:,idxOC2)   ! wtw
         endif
      end if

      end subroutine srf_emis

end module bcc_srf_emis

