
#include <misc.h>
      module bcc_SULF
!---------------------------------------------------------------
!	... Annual cycle for sulfur
!---------------------------------------------------------------

#if ( defined SPMD )
  use mpishorthand      , only: mpicom, mpiint, mpir8
#endif
      implicit none

      private
      public  :: SULF_INTI , READ_SULF , SULF_INTERP
      save
       
      character(len=256), public  ::  bndsulf 

      integer :: ncid = 0, &
                 time_cnt = 0, &
                 lotim = 0, &
                 hitim = 0, &
                 loin = 1, &
                 hiin = 2, &
                 jlim_in(2), &
                 sulf_nlevs, &
                 nlon, &
                 nlat
      real :: sulf_P0
      real, allocatable :: times(:)
      real, allocatable :: psi(:,:,:), &
                           sulfatei(:,:,:,:), &
                           sulf_hyam(:), &
                           sulf_hybm(:), &
                           psi_in(:,:), &
                           sulfatei_in(:,:,:)

      real, allocatable :: psibdy(:,:,:), &
                           sulfateibdy(:,:,:,:)
      logical :: cyclical = .true.

      CONTAINS 

      subroutine SULF_INTI(sulf_file)
!-----------------------------------------------------------------------
! 	... Open netCDF file containing annual sulfur data.  Initialize
!           arrays with the data to be interpolated to the current time.
!
!           It is assumed that the time coordinate is increasing
!           and represents calendar days; range = [1.,366.).
!-----------------------------------------------------------------------

      use bcc_netcdf
      use bcc_const_mozart, only : d2r
      use pmgrid,           only : plon, plat

!add by zf 2008.05.28
      use pmgrid,          only: masterproc
      use ioFileMod,       only: getfil
      use abortutils,      only: endrun
      use error_messages,  only: handle_ncerr

!end zf

      implicit none

!-----------------------------------------------------------------------
!	... Local variables
!-----------------------------------------------------------------------
      character(len=*) sulf_file

      integer :: dimid_lat
      integer :: dimid_lon
      integer :: dimid_lev

      integer :: vid
      integer :: ierr
      integer :: times_id

      integer :: k

      real, allocatable :: lat(:)
      real, allocatable :: lon(:)

      character(len=256) :: locfn    ! netcdf local filename to open   add by zf

      logical :: there

!-----------------------------------------------------------------------
!     	... Open netcdf file
!-----------------------------------------------------------------------
      bndsulf = sulf_file 

      if (masterproc) then

      call getfil (bndsulf, locfn)
      call wrap_open(locfn,0,ncid)
!end zf

!-----------------------------------------------------------------------
!     	... Inquire about file
!-----------------------------------------------------------------------
      call HANDLE_NCERR( NF_INQ_UNLIMDIM( ncid, times_id ), &
                         'SULF_INTI: File inquiry' )
      call HANDLE_NCERR( NF_INQ_DIMLEN( ncid, times_id, time_cnt ), &
                         'SULF_INTI: Getting time point count' )

      write(*,*) '============== time_cnt ===========', time_cnt 
 
    end if   !(masterproc)
!-----------------------------------------------------------------------
!     	... Allocate space for time coordinate data
!-----------------------------------------------------------------------
#if (defined SPMD )
    call mpibcast( time_cnt  , 1, mpiint, 0, mpicom )
#endif

      ALLOCATE( times(time_cnt), stat = ierr )
      if( ierr /= 0 ) then
	 write(*,*) ' SULF_INTI : Failed to allocate times array; error = ',ierr
	 call ENDRUN
      end if
!-----------------------------------------------------------------------
!     	... Get time coordinate
!-----------------------------------------------------------------------
    if (masterproc) then
      call HANDLE_NCERR( NF_INQ_VARID( ncid, 'time', vid ), 'SULF_INTI: Getting time id' )
      call HANDLE_NCERR( NF_GET_VAR_DOUBLE( ncid, vid, times ), 'SULF_INTI: Getting times' )
      write(*,*) 'SULF_INTI: times' 
      write(*,'(1p,5e21.13)') times

!-----------------------------------------------------------------------
!     	... Get vertical coordinate
!-----------------------------------------------------------------------
      call HANDLE_NCERR( NF_INQ_DIMID( ncid, 'lev', dimid_lev ), 'SULF_INTI: Failed to find dimension lev' )
      call HANDLE_NCERR( NF_INQ_DIMLEN( ncid, dimid_lev, sulf_nlevs ), &
                         'SULF_INTI: Failed to get length of dimension lev' )
    end if    ! (masterproc)    
 
#if (defined SPMD )
    call mpibcast( sulf_nlevs, 1, mpiint, 0, mpicom )
#endif

      ALLOCATE( sulf_hyam(sulf_nlevs), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'SULF_INTI: hyam allocation error = ',ierr
         call ENDRUN
      end if
      ALLOCATE( sulf_hybm(sulf_nlevs), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'SULF_INTI: hybm allocation error = ',ierr
         call ENDRUN
      end if

    if (masterproc) then
      call HANDLE_NCERR( NF_INQ_VARID( ncid, 'hyam', vid ), 'SULF_INTI: Getting hyam id' )
      call HANDLE_NCERR( NF_GET_VAR_DOUBLE( ncid, vid, sulf_hyam ), 'SULF_INTI: Getting hyam' )
      call HANDLE_NCERR( NF_INQ_VARID( ncid, 'hybm', vid ), 'SULF_INTI: Getting hybm id' )
      call HANDLE_NCERR( NF_GET_VAR_DOUBLE( ncid, vid, sulf_hybm ), 'SULF_INTI: Getting hybm' )
      call HANDLE_NCERR( NF_INQ_VARID( ncid, 'P0', vid ), 'SULF_INTI: Getting P0 id' )
      call HANDLE_NCERR( NF_GET_VAR_DOUBLE( ncid, vid, sulf_P0 ), 'SULF_INTI: Getting P0' )
      write(*,*) 'SULF_INTI: P0 = ',sulf_P0

!-----------------------------------------------------------------------
!     	... Make sure the time coordinate looks like calendar day, and is increasing.
!-----------------------------------------------------------------------
      call MOZ_CHKTIME( times, time_cnt )

!-----------------------------------------------------------------------
!     	... Get latitude and longitude
!-----------------------------------------------------------------------
      call HANDLE_NCERR( NF_INQ_DIMID( ncid, 'lat', dimid_lat ), &
                         'SULF_INTI: Failed to find dimension lat' )
      call HANDLE_NCERR( NF_INQ_DIMLEN( ncid, dimid_lat, nlat ), &
                         'SULF_INTI: Failed to get length of dimension lat' )
      ALLOCATE( lat(nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'SULF_INTI: lat allocation error = ',ierr
         call ENDRUN
      end if
      call HANDLE_NCERR( NF_INQ_VARID( ncid, 'lat', vid ), &
                         'SULF_INTI: Failed to find variable lat' )
      call HANDLE_NCERR( NF_GET_VAR_DOUBLE( ncid, vid, lat ), &
                         'SULF_INTI: Failed to read variable lat' )
      lat(:nlat) = lat(:nlat) * d2r
 
      call HANDLE_NCERR( NF_INQ_DIMID( ncid, 'lon', dimid_lon ), &
                         'SULF_INTI: Failed to find dimension lon' )
      call HANDLE_NCERR( NF_INQ_DIMLEN( ncid, dimid_lon, nlon ), &
                         'SULF_INTI: Failed to get length of dimension lon' )
      ALLOCATE( lon(nlon), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'SULF_INTI: lon allocation error = ',ierr
         call ENDRUN
      end if
      call HANDLE_NCERR( NF_INQ_VARID( ncid, 'lon', vid ), &
                         'SULF_INTI: Failed to find variable lon' )
      call HANDLE_NCERR( NF_GET_VAR_DOUBLE( ncid, vid, lon ), &
                         'SULF_INTI: Failed to read variable lon' )
      lon(:nlon) = lon(:nlon) * d2r

      end if              !(masterproc)
!-----------------------------------------------------------------------

#if (defined SPMD )
    call mpibcast( nlon, 1, mpiint, 0, mpicom )
    call mpibcast( nlat, 1, mpiint, 0, mpicom )
    call mpibcast( sulf_P0, 1, mpir8, 0, mpicom )
    do k = 1, sulf_nlevs
      call mpibcast( sulf_hyam(k), 1, mpir8, 0, mpicom )
      call mpibcast( sulf_hybm(k), 1, mpir8, 0, mpicom )
    end do
    do k = 1, time_cnt
      call mpibcast( times(k), 1, mpir8, 0, mpicom )
    end do
#endif
      
    if (masterproc) then
      allocate( psi_in( nlon,nlat), stat=ierr)
      if( ierr /= 0 ) then
         write(*,*) 'SULF_INTI: psi_in allocation error = ',ierr
         call ENDRUN
      end if
      ALLOCATE( sulfatei_in( nlon,sulf_nlevs,nlat), stat=ierr)
      if( ierr /= 0 ) then
         write(*,*) 'SULF_INTI: sulfatei_in allocation error = ',ierr
         call ENDRUN
      end if
    end if              !(masterproc)

!---------------

      ALLOCATE( sulfatei( nlon,sulf_nlevs,nlat,2 ), stat=ierr)
      if( ierr /= 0 ) then
         write(*,*) 'SULF_INTI: sulfatei allocation error = ',ierr
         call ENDRUN
      end if

      ALLOCATE( psi( nlon,nlat,2 ), stat=ierr)
      if( ierr /= 0 ) then
         write(*,*) 'SULF_INTI: psi allocation error = ',ierr
         call ENDRUN
      end if
      end subroutine SULF_INTI

      subroutine READ_SULF( )
!-----------------------------------------------------------------------
!       ... Interpolate annual cycle sulfur data to the current time.
!           Read in new time samples of the input data as necessary.
!-----------------------------------------------------------------------

      use bcc_netcdf

!add by zf 2008.05.28
      use pmgrid,         only:  masterproc
      use time_manager,   only:  get_curr_calday
      use ppgrid,         only:  pcols, begchunk, endchunk
      use phys_grid,      only:  scatter_field_to_chunk
      use pmgrid,         only : plon, plat   !zf test

!end zf

      implicit none

!-----------------------------------------------------------------------
!       ... Dummy args
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer :: ierr
      integer :: i, j, k, jl, ju,oldlotim, oldhitim,vid, ret
      integer, dimension(4) :: start, count
      real :: cday

!add by zf 2008.05.28

    if (masterproc) then

      cday = get_curr_calday()  ! convert calday to single precision if necessary

!-----------------------------------------------------------------------
!       ... Check to see if model time is still bounded by dataset times.
!-----------------------------------------------------------------------
      oldlotim = lotim
      oldhitim = hitim
      call MOZ_FINDPLB( times, time_cnt, cday, lotim )
      if( cyclical ) then
         write(*,*) '=========================== time_cnt in READ_SULF ====', time_cnt
         hitim = MOD( lotim,time_cnt ) + 1
      else
         hitim = lotim + 1
      end if

      if( hitim /= oldhitim ) then
!-----------------------------------------------------------------------
!       ... Read in new hitim data.  Replace old lotim data.
!-----------------------------------------------------------------------
         loin = hiin
         hiin = MOD( loin,2 ) + 1
         start(:3) = (/ 1, 1, hitim /)
         count(:3) = (/ nlon, nlat, 1 /)
         call HANDLE_NCERR( NF_INQ_VARID( ncid, 'PS', vid ), &
                            'READ_SULF: Getting PS id' )
         call HANDLE_NCERR( NF_GET_VARA_DOUBLE( ncid, vid, start, count, psi_in ), &
                            'READ_SULF: Getting PS' )
!-----------------------------------------------------------------------
!       ... Regrid maccm input surface presssure
!-----------------------------------------------------------------------
         do i = 1, nlon
         do j = 1, nlat
           psi(i,j,hiin) = psi_in(i,j)
         enddo
         enddo

         start(:4) = (/ 1, 1, 1, hitim /)
         count(:4) = (/ nlon,sulf_nlevs, nlat, 1 /)
         call HANDLE_NCERR( NF_INQ_VARID( ncid, 'SULFATE', vid ), &
                            'READ_SULF: Getting SULFATE id' )
         call HANDLE_NCERR( NF_GET_VARA_DOUBLE( ncid, vid, start, count, sulfatei_in ), &
                            'READ_SULF: Getting SULFATE' )
!-----------------------------------------------------------------------
!       ... Regrid maccm input sulfate
!-----------------------------------------------------------------------
        do i = 1, nlon
        do j = 1, nlat
           do k = 1,sulf_nlevs
             sulfatei(i,k,j,hiin) = sulfatei_in(i,k,j)
           end do
        end do
        end do

         write(*,*) ' '
         write(*,*) 'READ_SULF: read data for day ',times(hitim)
         if( lotim /= oldhitim ) then
!-----------------------------------------------------------------------
!       ... Read in new lotim data.  Replace old hitim data.
!-----------------------------------------------------------------------
            start(:3) = (/ 1, 1, lotim /)
            count(:3) = (/ nlon, nlat, 1 /)
            call HANDLE_NCERR( NF_INQ_VARID( ncid, 'PS', vid ), 'READ_SULF: Getting PS id' )
            call HANDLE_NCERR( NF_GET_VARA_DOUBLE( ncid, vid, start, count, psi_in ), 'READ_SULF: Getting PS' )
!-----------------------------------------------------------------------
!       ... Regrid maccm input surface presssure
!-----------------------------------------------------------------------
         do i = 1, nlon
         do j = 1, nlat
           psi(i,j,loin) = psi_in(i,j)
         enddo
         enddo


            start(:4) = (/ 1, 1, 1, lotim /)
            count(:4) = (/ nlon, sulf_nlevs, nlat, 1 /)
            call HANDLE_NCERR( NF_INQ_VARID( ncid, 'SULFATE', vid ), &
                               'READ_SULF: Getting SULFATE id' )
            call HANDLE_NCERR( NF_GET_VARA_DOUBLE( ncid, vid, start, count, sulfatei_in ), &
                               'READ_SULF: Getting SULFATE' )
!-----------------------------------------------------------------------
!       ... Regrid maccm input sulfate
!-----------------------------------------------------------------------
        do i = 1, nlon
        do j = 1, nlat
           do k = 1,sulf_nlevs
             sulfatei(i,k,j,loin) = sulfatei_in(i,k,j)
           end do
        end do
        end do
            write(*,*) 'READ_SULF: read data for day ',times(lotim)
        end if
      end if

    end if       !(masterproc)

    allocate( sulfateibdy(pcols,sulf_nlevs,begchunk:endchunk,2), stat=ierr)
    allocate( psibdy(pcols,begchunk:endchunk,2), stat=ierr)

    call scatter_field_to_chunk(1,1,2,nlon,psi,psibdy)
    call scatter_field_to_chunk(1,sulf_nlevs,2,nlon,sulfatei,sulfateibdy)

!zf    DEALLOCATE(psi)            !zftest
!zf    DEALLOCATE(sulfatei)       !zftest

#if (defined SPMD )
    call mpibcast( hitim, 1, mpiint, 0, mpicom )
    call mpibcast( lotim, 1, mpiint, 0, mpicom )
#endif

      end subroutine READ_SULF

      subroutine SULF_INTERP( pmid, calday, ccm_sulf, plonl, lchnk )
!-----------------------------------------------------------------------
!       ... Time interpolate sulfatei to current time
!-----------------------------------------------------------------------

      use pmgrid, only : plev

      implicit none

!-----------------------------------------------------------------------
!       ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) :: plonl
      integer, intent(in) :: lchnk
      real, intent(in)    :: calday              ! day of year
      real, intent(in)    :: pmid(plonl,plev)    ! midpoint pressure ( pascals )
      real, intent(out)   :: ccm_sulf(plonl,plev)    ! output sulfate

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      real    ::  tint, dt, dt1      ! interpolation time
      real    ::  delp               ! pressure delta
      real    ::  pinterp            ! interpolation pressure
      real    ::  sulf_press(plonl,sulf_nlevs,2)
      real    ::  zint_sulf(plonl,plev,2)
      integer ::  i, k, ku, kl      ! working indicies

!-----------------------------------------------------------------------
!       ... Form MOZ1 pressures at enclosing months
!-----------------------------------------------------------------------
      do k = 1,sulf_nlevs
        do i = 1,plonl
         sulf_press(i,k,loin) = sulf_hyam(k)*sulf_P0 + sulf_hybm(k)*psibdy(i,lchnk,loin)
         sulf_press(i,k,hiin) = sulf_hyam(k)*sulf_P0 + sulf_hybm(k)*psibdy(i,lchnk,hiin)
        end do
      end do

!-----------------------------------------------------------------------
!       ... Vertical interpolation of images data to model levels
!           Note: images data only up to 50mb
!-----------------------------------------------------------------------
      do i = 1,plonl
         do k = 1,plev
            pinterp = pmid(i,k)
            if( pinterp < MAXVAL( sulf_press(i,1,:2) ) ) then
               zint_sulf(i,k,loin) = 0.
               zint_sulf(i,k,hiin) = 0.
            else
               if( pinterp > sulf_press(i,sulf_nlevs,loin) ) then
                  zint_sulf(i,k,loin) = sulfateibdy(i,sulf_nlevs,lchnk,loin)
               else
                  do ku = 2,sulf_nlevs
                     if( pinterp <= sulf_press(i,ku,loin) ) then
                        kl = ku - 1
                        delp =  LOG( pinterp/sulf_press(i,kl,loin) ) &
                             /  LOG( sulf_press(i,ku,loin)/sulf_press(i,kl,loin) )
                        zint_sulf(i,k,loin) = sulfateibdy(i,kl,lchnk,loin) &
                             + delp * (sulfateibdy(i,ku,lchnk,loin) - sulfateibdy(i,kl,lchnk,loin))
                        exit
                     end if
                  end do
               end if
               if( pinterp > sulf_press(i,sulf_nlevs,hiin) ) then
                  zint_sulf(i,k,hiin) = sulfateibdy(i,sulf_nlevs,lchnk,hiin)
                  cycle
               else
                  do ku = 2,sulf_nlevs
                     if( pinterp <= sulf_press(i,ku,hiin) ) then
                        kl = ku - 1
                        delp = LOG( pinterp/sulf_press(i,kl,hiin) ) &
                             / LOG( sulf_press(i,ku,hiin)/sulf_press(i,kl,hiin) )
                        zint_sulf(i,k,hiin) = sulfateibdy(i,kl,lchnk,hiin) &
                             + delp * (sulfateibdy(i,ku,lchnk,hiin) - sulfateibdy(i,kl,lchnk,hiin))
                        exit
                     end if
                  end do
               end if
            end if
         end do
      end do

!-----------------------------------------------------------------------
!       ... Linear interpolation
!           Start by computing the number of days between
!           the lower and upper bounds, and days between
!           the model time and lower bound.
!-----------------------------------------------------------------------
      if( times(hitim) < times(lotim) ) then
         dt = 365. - times(lotim) + times(hitim)
         if( calday <= times(hitim) )then
            dt1 = 365. - times(lotim) + calday
         else
            dt1 = calday - times(lotim)
         end if
      else
         dt = times(hitim) - times(lotim)
         dt1 = calday - times(lotim)
      end if

      tint = dt1 / dt
      call MOZ_LININTP( plonl*plev, 0., 1., tint, &
                        zint_sulf(1,1,loin), &
                        zint_sulf(1,1,hiin), &
                        ccm_sulf )

      end subroutine SULF_INTERP

      end module bcc_SULF
