#include <misc.h>
#include <params.h>
module bcc_surface

      implicit none

      public  ::  sflxdr 
      save

contains

      subroutine sflxdr( lchnk   ,ncol    ,landfrac ,ocnfrac    , &
	                     pmid    ,ts      ,t        ,q          , & 
                         sflx    ,nem2    ,em2data  ,             &
						 vlc_trb_SO4, vlc_trb_NH4, vlc_trb_CB2, vlc_trb_SO2 )

!-----------------------------------------------------------------------
! 	... set surface fluxes or make adjustment to field due to
!           surface flux. dry deposition routines are called from
!           here because they work by setting surface fluxes.
!-----------------------------------------------------------------------
      use shr_kind_mod,     only : r8 => shr_kind_r8
      use ppgrid,           only : pcols, pver
      use bcc_srf_emis,     only : srf_emis
	  use bcc_chem_mods,    only : gas_pcnst, map2chm
      use bcc_chem_utls,    only : diurnal_geom
      use phys_grid,        only : get_rlat_all_p, get_rlon_all_p       !2008.08.11
      use physconst,        only : zvir, rair
      use constituents,     only : ppcnst
      use bcc_drydep,       only : drydep
      use time_manager,     only:  get_curr_calday

      implicit none

!-----------------------------------------------------------------------
! 	... dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) :: lchnk
      integer, intent(in) :: ncol 

      real(r8),intent(in) :: landfrac(pcols)                ! landfrac 
      real(r8),intent(in) :: ocnfrac(pcols)                 ! landfrac 
      integer, intent(in) :: nem2
      real(r8),intent(in) :: em2data(pcols,nem2) 
      real(r8),intent(in) :: pmid(pcols,pver)

	  real(r8),intent(in) :: t(pcols,pver)
	  real(r8),intent(in) :: ts(pcols)                     ! sfc temp
      real(r8),intent(in) :: q(pcols,pver,ppcnst)       ! species concentrations (kg/kg)
      real(r8),intent(inout) :: sflx(pcols,ppcnst)         ! surface flux for advected species

      real(r8),intent(in) :: vlc_trb_SO4(pcols)       ! dry deposite velcity (m/s)
      real(r8),intent(in) :: vlc_trb_NH4(pcols)       ! dry deposite velcity (m/s)
      real(r8),intent(in) :: vlc_trb_CB2(pcols)       ! dry deposite velcity (m/s)
      real(r8),intent(in) :: vlc_trb_SO2(pcols)       ! dry deposite velcity (m/s)      
!-----------------------------------------------------------------------
! 	... local variables
!-----------------------------------------------------------------------
      real(r8)    :: calday                         ! julian day + fraction (greenwich)
      integer     :: ioro(pcols)                       ! orography 

      integer     :: i, m, n
      real(r8)    :: sunon(pcols)         ! sunrise longitude (radians)
      real(r8)    :: sunoff(pcols)        ! sunset longitude (radians)
      real(r8)    :: zen_angle(pcols)     ! zenith angle as function of longitude (radians)
      real(r8)    :: loc_angle(pcols)     ! time angle as function of longitude (radians)
      real(r8)    :: clon(pcols)
      real(r8)    :: clat(pcols)
      real(r8)    :: tvs(pcols)

      logical     :: polar_night(pcols)          ! wrk flag for diurnal_geom
      logical     :: polar_day(pcols)            ! wrk flag for diurnal_geom

      real(r8)    :: mmr(pcols,pver,gas_pcnst)   
      real(r8)    :: cflx(pcols,gas_pcnst)        ! surface flux for advected species
      real(r8)    :: dflx(pcols,gas_pcnst)  
      real(r8)    ::  depvel(pcols,gas_pcnst)              ! dry deposition velocity (cm/s)
!------------------------------------------------------------------------------------------
      do i = 1, pcols
        if(landfrac(i) .gt. 0.5) then
          ioro(i) = 1
        else
          ioro(i) = 0
        endif
      enddo

      do m = 1,gas_pcnst
        cflx(:,m) = 0.
      enddo

      do m = 1,ppcnst
         n = map2chm(m)
         if( n > 0) then
		     mmr(:ncol,:pver,n) = q(:ncol,:pver,m)
	     endif
      end do

!-----------------------------------------------------------------------
!	... diurnal geometry
!-----------------------------------------------------------------------
      calday = get_curr_calday()

      call get_rlat_all_p(lchnk, ncol, clat)    !add by zf 2008.08.11
      call get_rlon_all_p(lchnk, ncol, clon)    !add by zf 2008.08.11

      do i = 1, pcols
        call diurnal_geom( clon(i), clat(i), calday, polar_night(i), polar_day(i), &
                           sunon(i), sunoff(i), loc_angle(i), zen_angle(i) )
      enddo
!-----------------------------------------------------------------------
!	... the surface emissions
!-----------------------------------------------------------------------
      call srf_emis( lchnk, clat, calday, cflx, ioro, &
                     loc_angle, polar_night, polar_day, sunon, sunoff, ncol, &
                     nem2, em2data )
!-----------------------------------------------------------------------
!	... the dry deposition
!-----------------------------------------------------------------------
    tvs(:ncol) = t(:ncol,pver) * (1._r8 + zvir * q(:ncol,pver,1))
    
    dflx(:,:)  = 0.0
    call drydep( lchnk,  calday, ts, zen_angle, &
                 depvel, dflx, rair, mmr, pmid(:,pver), &
                 tvs,    ncol, ocnfrac, &
                 vlc_trb_SO4, vlc_trb_NH4, &
                 vlc_trb_CB2, vlc_trb_SO2 )                    ! dflx: output, downward (+)
                                                               ! depvel
!----------------------------------------------------------------------
!	... form surface flux
!-----------------------------------------------------------------------

     do m = 1,ppcnst
        n = map2chm(m)
        if( n > 0 ) then
           sflx(:ncol,m) = sflx(:ncol,m) + cflx(:ncol,n) - dflx(:ncol,n)
        end if
     end do

     end subroutine sflxdr

end module bcc_surface
