#include <misc.h>
#include <params.h>

module bcc_timestep_init

      implicit none
      private

      public :: bccchem_timestep_init

      save
    
contains

  subroutine bccchem_timestep_init (phys_state)

    use shr_kind_mod,     only : r8 => shr_kind_r8
    use ppgrid,           only : begchunk, endchunk
    use physics_types,    only : physics_state
    use time_manager,     only : get_curr_calday
    use bcc_solar_parms,  only : solar_parms_timestep_init
    use bcc_jshort,       only : jshort_timestep_init
    use bcc_jlong,        only : jlong_timestep_init
    use bcc_aurora,       only : aurora_timestep_init
    use bcc_photo,        only : photo_timestep_init
    use bcc_strato_sad,   only : strato_sad_timestep_init
    use mo_strato_aerosol,only : aero_timestep_init
    use bcc_cfc11star,    only : update_cfc11star
    use bcc_iondrag,      only : do_waccm_ions
    use bcc_qbo,          only : qbo_use_forcing
    use bcc_efield,       only : get_efield
    use bcc_qbo,          only : qbo_timestep_init
    use bcc_sulf,         only : read_sulf

    implicit none

    type(physics_state), intent(inout) :: phys_state(begchunk:endchunk)                 
!
! Local variables
!
    real(r8) :: calday

    !-----------------------------------------------------------------------
    ! get current calendar day of year
    !-----------------------------------------------------------------------
    calday = get_curr_calday( )

    !---------------------------------
    ! Compute the electric field
    !------------------------------------

    if (do_waccm_ions) then
        call get_efield
    endif

    !----------------------------------------------------------------------
    ! update QBO data for this time step
    !----------------------------------------------------------------------

	if(qbo_use_forcing) then
        call qbo_timestep_init
    endif

    !-----------------------------------------------------------------------
    ! Advance stratospheric aerosol densities...
    !-----------------------------------------------------------------------
!    call strato_sad_timestep_init()
    call aero_timestep_init

    !-----------------------------
    ! read sulf
    !------------------------
!    call read_sulf

    !-----------------------------------------------------------------------
    ! Set solar parameters
    !-----------------------------------------------------------------------
    call solar_parms_timestep_init

    !-----------------------------------------------------------------------------
    !	... setup the time interpolation for mo_photo
    !-----------------------------------------------------------------------------
    call photo_timestep_init( calday )

    !-----------------------------------------------------------------------
    ! Set jlong etf
    !-----------------------------------------------------------------------
    call jlong_timestep_init

    !-----------------------------------------------------------------------
    ! Set jshort etf
    !-----------------------------------------------------------------------
    call jshort_timestep_init

    !-----------------------------------------------------------------------
    ! Set up aurora
    !-----------------------------------------------------------------------
    call aurora_timestep_init

    call update_cfc11star( phys_state )

  end subroutine bccchem_timestep_init

end module bcc_timestep_init
