#include <misc.h>
#include <params.h>

module bcc_ub_vals
!---------------------------------------------------------------
!	... variables for the upper boundary values
!---------------------------------------------------------------
      use shr_kind_mod, only:  r8 => shr_kind_r8
      use ppgrid,       only:  pcols
      implicit none

      private
      public  :: ub_inti
      public  :: set_ub_vals

      save

      integer :: table_nox_ndx = -1
      integer :: table_h2o_ndx = -1
      integer :: no_ndx, no2_ndx, ox_ndx, co_ndx, ch4_ndx, o3s_ndx, o3inert_ndx
      integer :: h2o_ndx

      integer :: jlim_in(2)
      real, allocatable       :: ub_plevs(:)         ! pascals
      integer                 :: ub_nlevs, &         ! # of levs in ubc file
                                 ub_nlat, &          ! # of lats in ubc file
                                 ub_nspecies, &      ! # of species in ubc file
                                 ub_nmonth           ! # of months in ubc file
      real,    allocatable       :: mr_ub_bdy(:,:,:,:,:)      ! vmr
      integer, allocatable    :: map(:)              ! species indices for ubc species
      logical :: sim_has_nox

      contains

      subroutine ub_inti( fstrat_file )
!------------------------------------------------------------------
!	... initialize upper boundary values
!------------------------------------------------------------------

      use bcc_netcdf
      use bcc_chem_mods,     only : gas_pcnst
      use bcc_chem_mods,     only : fstrat_list, fstrat_cnt
      use bcc_const_mozart,  only : d2r
      use pmgrid,            only : plon, plat
      use bcc_chem_mods,     only : solsym
      use bcc_chem_utls,     only : get_spc_ndx

      use pmgrid,         only: masterproc
      use ioFileMod,      only: getfil
      use abortutils,     only: endrun
      use error_messages, only: handle_ncerr
      use ppgrid,         only: pcols, begchunk, endchunk
      use phys_grid,      only: scatter_field_to_chunk

#if ( defined SPMD )
  use mpishorthand, only: mpicom, mpiint, mpir8
#endif

!end zf

      implicit none

      character (len=*), intent(in)  :: fstrat_file

!------------------------------------------------------------------
!	... local variables
!------------------------------------------------------------------
	  logical  :: has_fstrat(gas_pcnst)

      integer :: jl, ju
      integer :: spcno, lev, month, ierr
      real, allocatable :: lat(:)
      real, allocatable       :: mr_ub(:,:,:,:)      ! vmr
      real, allocatable       :: mr_ub_temp(:,:,:,:,:)      ! vmr
      integer :: ncid, vid
      integer :: dimid_lat, dimid_lev, dimid_species, dimid_month, &
                 dimid(4), ndims
      character(len=80) :: attribute
      character(len=25), allocatable :: ub_species_names(:)
      integer, dimension(4) :: start, count
      integer :: i, j, k, l, m, nchar

      character(len=256) :: locfn    ! netcdf local filename to open   add by zf
	  character(len=8)   :: wrk_name
!-----------------------------------------------------------------------
!       ... get species indicies
!-----------------------------------------------------------------------
      no_ndx  = get_spc_ndx( 'NO' )
      no2_ndx = get_spc_ndx( 'NO2' )
      sim_has_nox = no_ndx > 0 .or. no2_ndx > 0
      ox_ndx = get_spc_ndx( 'OX' )
      if( ox_ndx < 1 ) then
         ox_ndx = get_spc_ndx( 'O3' )
      end if
      co_ndx = get_spc_ndx( 'CO' )
      ch4_ndx = get_spc_ndx( 'CH4' )
      o3s_ndx = get_spc_ndx( 'O3S' )
      o3inert_ndx = get_spc_ndx( 'O3INERT' )
!-----------
! wtw
!
    h2o_ndx     = get_spc_ndx( 'H2O' )

    has_fstrat(:) = .false.
    do i = 1, fstrat_cnt
       if ( len_trim(fstrat_list(i))==0 ) exit
       j = get_spc_ndx(fstrat_list(i))
       if ( j > 0 ) then
          has_fstrat(j) = .true.
       else
          write(*,*) 'fstrat_inti: '//trim(fstrat_list(i))//' is not included in species set'
          call endrun('fstrat_inti: invalid fixed stratosphere species')
       endif
    enddo
    if (.not. any(has_fstrat(:))) return

!wtw
!-----------------------------------------------------------------------
!       ... open netcdf file
!-----------------------------------------------------------------------

    if (masterproc) then
      call getfil (fstrat_file, locfn)
      call wrap_open(locfn,0,ncid)
!-----------------------------------------------------------------------
!       ... get latitude
!-----------------------------------------------------------------------
      call handle_ncerr( nf_inq_dimid( ncid, 'lat', dimid_lat ), &
                         'ub_inti: failed to find dimension lat' )
      call handle_ncerr( nf_inq_dimlen( ncid, dimid_lat, ub_nlat ), &
                         'ub_inti: failed to get length of dimension lat' )
      allocate( lat(ub_nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'ub_inti: lat allocation error = ',ierr
         call endrun
      end if
      call handle_ncerr( nf_inq_varid( ncid, 'lat', vid ), &
                         'ub_inti: failed to find variable lat' )
      call handle_ncerr( nf_get_var_double( ncid, vid, lat ), &
                         'ub_inti: failed to read variable lat' )
      lat(:ub_nlat) = lat(:ub_nlat) * d2r

!-----------------------------------------------------------------------
!       ... get vertical coordinate (if necessary, convert units to pa)
!-----------------------------------------------------------------------
      call handle_ncerr( nf_inq_dimid( ncid, 'lev', dimid_lev ), &
                         'ub_inti: failed to find dimension lev' )
      call handle_ncerr( nf_inq_dimlen( ncid, dimid_lev, ub_nlevs ), &
                         'ub_inti: failed to get length of dimension lev' )
    end if

#if (defined SPMD )
    call mpibcast( ub_nlevs, 1, mpiint, 0, mpicom )
#endif
      allocate( ub_plevs(ub_nlevs), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'ub_inti: ub_plevs allocation error = ',ierr
         call endrun
      end if

    if (masterproc) then
      call handle_ncerr( nf_inq_varid( ncid, 'lev', vid ), &
                         'ub_inti: getting lev id' )
      call handle_ncerr( nf_get_var_double( ncid, vid, ub_plevs ), &
                         'ub_inti: getting ub_plevs' )
      attribute(:) = ' '
      ierr= nf_get_att_text( ncid, vid, 'units', attribute )
      if( ierr == nf_noerr )then
         if( trim(attribute) == 'mb' .or. trim(attribute) == 'hpa' )then
            write(*,*) 'ub_inti: units for lev = ',trim(attribute),'... converting to pa'
            ub_plevs(:) = 100. * ub_plevs(:)
         else if( trim(attribute) /= 'pa' .and. trim(attribute) /= 'pa' )then
            write(*,*) 'ub_inti: unknown units for lev, units=*',trim(attribute),'*'
            write(*,*) 'ub_inti: ',attribute=='mb',trim(attribute)=='mb',attribute(1:2)=='mb'
            call endrun
         end if
      else
            write(*,*) 'ub_inti: warning! units attribute for lev missing, assuming mb'
            ub_plevs(:) = 100. * ub_plevs(:)
      end if
!-----------------------------------------------------------------------
!       ... get time and species dimensions
!-----------------------------------------------------------------------
      call handle_ncerr( nf_inq_dimid( ncid, 'month', dimid_month ), &
                         'ub_inti: failed to find dimension month' )
      call handle_ncerr( nf_inq_dimlen( ncid, dimid_month, ub_nmonth ), &
                         'ub_inti: failed to get length of dimension month' )
      if( ub_nmonth /= 12 )then
         write(*,*) 'ub_inti: error! number of months = ',ub_nmonth,', expecting 12'
         call endrun
      end if
      call handle_ncerr( nf_inq_dimid( ncid, 'species', dimid_species ), &
                         'ub_inti: failed to find dimension species' )
      call handle_ncerr( nf_inq_dimlen( ncid, dimid_species, ub_nspecies ), &
                         'ub_inti: failed to get length of dimension species' )

      allocate( mr_ub(ub_nlat,ub_nspecies,ub_nmonth,ub_nlevs), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'ub_inti: mr_ub allocation error = ',ierr
         call endrun
      end if

    endif

#if (defined SPMD )
    call mpibcast( ub_nmonth, 1, mpiint, 0, mpicom )
    call mpibcast( ub_nspecies, 1, mpiint, 0, mpicom )
    call mpibcast( ub_nlat, 1, mpiint, 0, mpicom )
    call mpibcast( ub_plevs, ub_nlevs, mpir8, 0, mpicom )
#endif

      allocate( mr_ub_temp(plon,ub_nlevs,ub_nlat,ub_nmonth,ub_nspecies), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'ub_inti: mr_ub_temp allocation error = ',ierr
         call endrun
      end if

      allocate( map(ub_nspecies), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'ub_inti: map allocation error = ',ierr
         call endrun
      end if
!------------------------------------------------------------------
!	... read in the species names
!------------------------------------------------------------------
    if (masterproc) then
      call handle_ncerr( nf_inq_varid( ncid, 'specname', vid ), &
                         'ub_inti: getting specname id' )
      call handle_ncerr( nf_inq_varndims( ncid, vid, ndims ), &
                         'ub_inti: getting number of dimensions for specname' )
      call handle_ncerr( nf_inq_vardimid( ncid, vid, dimid ), &
                         'ub_inti: getting dimensions for vmr' )
      call handle_ncerr( nf_inq_dimlen( ncid, dimid(1), nchar ), &
                         'ub_inti: getting dimension length' )
      allocate( ub_species_names(ub_nspecies), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'ub_inti: ub_species_names allocation error = ',ierr
         call endrun
      end if

      do i = 1,ub_nspecies
         start(:2) = (/ 1, i /)
         count(:2) = (/ nchar, 1 /)
         ub_species_names(i)(:) = ' '
         call handle_ncerr( nf_get_vara_text( ncid, vid, start(:2), count(:2), ub_species_names(i) ), &
                            'ub_inti: getting species names' )
	 if( trim(ub_species_names(i)) == 'NOX' ) then
	        table_nox_ndx = i
     else if( trim(ub_species_names(i)) == 'H2O' ) then
            table_h2o_ndx = i
	 end if
         map(i) = 0
         do j = 1,gas_pcnst
            if( trim(ub_species_names(i)) == trim(solsym(j)) ) then
               map(i) = j
               exit
            end if
         end do

        if( map(i) == 0 ) then
          if( trim(ub_species_names(i)) == 'OX' ) then
             wrk_name = 'O3'
             do j = 1,gas_pcnst
                if( trim(wrk_name) == trim(solsym(j)) ) then
                   if( has_fstrat(j) ) then 
                      if( masterproc ) write(*,*) 'fstrat_inti: '//trim(solsym(j))//' is fixed in stratosphere'
                      map(i) = j
                      exit
                   end if
                end if
             end do
             if( map(i) == 0 ) then
                write(*,*) 'fstrat_inti: ubc table species ',trim(ub_species_names(i)), ' not used'
             end if
          else if( (trim(ub_species_names(i)) /= 'NOX') .and. (trim(ub_species_names(i)) /= 'H2O') ) then
              write(*,*) 'fstrat_inti: ubc table species ',trim(ub_species_names(i)), ' not used'
          end if
         end if

      end do

!------------------------------------------------------------------
!	... check dimensions for vmr variable
!------------------------------------------------------------------
      call handle_ncerr( nf_inq_varid( ncid, 'vmr', vid ), &
                         'ub_inti: getting vmr id' )
      call handle_ncerr( nf_inq_varndims( ncid, vid, ndims ), &
                         'ub_inti: getting number of dimensions for vmr' )
      if( ndims /= 4 ) then
         write(*,*) 'ub_inti: error! variable vmr has ndims = ',ndims,', expecting 4'
         call endrun
      end if
      call handle_ncerr( nf_inq_vardimid( ncid, vid, dimid ), &
                         'ub_inti: getting dimensions for vmr' )
      if( dimid(1) /= dimid_lat .or. dimid(2) /= dimid_species .or. &
          dimid(3) /= dimid_month .or. dimid(4) /= dimid_lev )then
         write(*,*) 'ub_inti: error! dimensions in wrong order for variable vmr,'// &
                    'expecting (lat,species,month,lev)'
         call endrun
      end if

!------------------------------------------------------------------
!	... read in the ub mixing ratio values
!------------------------------------------------------------------
      start = (/ 1, 1, 1, 1 /)
      count = (/ ub_nlat, ub_nspecies, 12, ub_nlevs /)

      call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, mr_ub ), &
                         'ub_inti: getting vmr' )
      call handle_ncerr( nf_close( ncid ), 'ub_inti: failed to close file ' // trim(fstrat_file) )

      do i = 1, plon
      do j = 1, ub_nlat
      do k = 1, ub_nlevs
      do l = 1, ub_nmonth
      do m = 1, ub_nspecies
        mr_ub_temp(i,k,j,l,m) = mr_ub(j,m,l,k)
      enddo
      enddo
      enddo
      enddo
      enddo

!      do m = 1, ub_nspecies
!      do l = 1, ub_nmonth
!        call scatter_field_to_chunk(1,ub_nlevs,1,plon,mr_ub_temp(:,:,:,l,m),mr_ub_bdy(:,:,:,l,m))
!      enddo
!      enddo
!
      deallocate(mr_ub)

!#if (defined SPMD )
!      else
!      do m = 1, ub_nspecies
!      do l = 1, ub_nmonth
!zftest        call scatter_field_to_chunk(1,ub_nlevs,1,plon,mr_ub_temp(:,:,:,l,m),mr_ub_bdy(:,:,:,l,m))
!      enddo
!      enddo

!#endif
      endif   !(masterproc)

#if (defined SPMD )
    call mpibcast( map, ub_nspecies, mpiint, 0, mpicom )
#endif

      allocate( mr_ub_bdy(pcols,ub_nlevs,begchunk:endchunk,ub_nmonth,ub_nspecies), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'ub_inti: mr_ub_bdy allocation error = ',ierr
         call endrun
      end if

      do m = 1, ub_nspecies
      do l = 1, ub_nmonth
        call scatter_field_to_chunk(1,ub_nlevs,1,plon,mr_ub_temp(:,:,:,l,m),mr_ub_bdy(:,:,:,l,m))
      enddo
      enddo

      deallocate(mr_ub_temp)

      end subroutine ub_inti

      subroutine set_ub_vals( delt, lchnk, vmr, pmid, zmid, &
                              tfld, calday, ncol, o3vmr )
!--------------------------------------------------------------------
!       ... set the upper boundary values for :
!           ox, nox, hno3, ch4, co, n2o, n2o5 & stratospheric o3
!--------------------------------------------------------------------
      use ppgrid,            only : pver
      use bcc_chem_mods,     only : gas_pcnst
      use bcc_calendar,      only : caldayr

      implicit none

!--------------------------------------------------------------------
!       ... dummy args
!--------------------------------------------------------------------
      integer, intent(in) :: lchnk             ! lat index
      integer, intent(in) :: ncol             ! lon tile dim
      real, intent(in)    :: calday            ! day of year including fraction
      real, intent(in)    :: delt              ! 
      real, intent(in)    :: tfld(pcols,pver)  ! (k)
      real, intent(in)    :: zmid(pcols,pver)  ! midpoint height (km)
      real, intent(in)    :: pmid(pcols,pver)  ! midpoint pressure (pa)
      real, intent(inout) :: vmr(pcols,pver,gas_pcnst) ! species concentrations (vmr)
      real, intent(in)    :: o3vmr(pcols,pver)  ! CMIP5 O3 vmr

!--------------------------------------------------------------------
!       ... local variables
!--------------------------------------------------------------------
      integer, parameter :: winter = 1, spring = 2
      integer, parameter :: summer = 3, fall = 4
      integer, parameter :: zlower = pver
!
!wtw     integer, parameter :: kmax   = 2                 ! upper bndy level to which all species are set
!
      integer  ::  kmax(pcols)
!------------
      real, parameter    :: ztrop_low  = 6.            ! lowest tropopause level allowed (km)
      real, parameter    :: ztrop_high = 18.           ! highest tropopause level allowed (km)
      real, parameter    :: max_dtdz   = 2.            ! max dt/dz for tropopause level (degrees k/km)
      integer  ::  m, last, next, i, k, k1
      integer  ::  season_last, season_next
      integer  ::  ltrop
      integer  ::  levrelax
      integer, dimension(pcols,zlower) :: kl, ku
      integer  ::  dates(12) = (/ 116, 214, 316, 415,  516,  615, 716, 816, 915, 1016, 1115, 1216 /)

      real     ::  taurelax, facrelax, vmrrelax
      real     ::  dz, dtdz, pinterp
      real     ::  nox_ubc, xno, xno2, rno
      real     ::  dels
      real, dimension(pcols,zlower) ::  delp
      real     ::  pint_vals(2)
      real     ::  days(12)

      logical  ::  north, south
      logical  ::  found_trop

!--------------------------------------------------------
!       ... initialize the monthly day of year times
!--------------------------------------------------------
         do m = 1,12
            days(m) = caldayr( dates(m), 0 )
         end do

!--------------------------------------------------------
!       ... setup the time interpolation
!--------------------------------------------------------
      if( calday < days(1) ) then
         next = 1
         last = 12
         dels = (365. + calday - days(12)) / (365. + days(1) - days(12))
      else if( calday >= days(12) ) then
         next = 1
         last = 12
         dels = (calday - days(12)) / (365. + days(1) - days(12))
      else
         do m = 11,1,-1
            if( calday >= days(m) ) then
               exit
            end if
         end do
         last = m
         next = m + 1
         dels = (calday - days(m)) / (days(m+1) - days(m))
      end if
!--------------------------------------------------------
!       ... set season of interpolation months
!--------------------------------------------------------
      dels = max( min( 1.,dels ),0. )
      if( next <= 5 .and. next >= 3 ) then
         season_next = spring
      else if( next <= 11 .and. next >= 9 ) then
         season_next = fall
      else if( next <= 8 .and. next >= 6 ) then
         season_next = summer
      else
         season_next = winter
      end if
      if( last <= 5 .and. last >= 3 ) then
         season_last = spring
      else if( last <= 11 .and. last >= 9 ) then
         season_last = fall
      else if( last <= 8 .and. last >= 6 ) then
         season_last = summer
      else
         season_last = winter
      end if
!--------------------------------------------------------
!       ... setup the pressure interpolation
!--------------------------------------------------------
      do k = 1,zlower
         do i = 1,ncol
            if( pmid(i,k) <= ub_plevs(1) ) then
               kl(i,k) = 1
               ku(i,k) = 1
               delp(i,k) = 0.
            else if( pmid(i,k) >= ub_plevs(ub_nlevs) ) then
               kl(i,k) = ub_nlevs
               ku(i,k) = ub_nlevs
               delp(i,k) = 0.
            else
               pinterp = pmid(i,k)
               do k1 = 2,ub_nlevs
                  if( pinterp <= ub_plevs(k1) ) then
                     ku(i,k) = k1
                     kl(i,k) = k1 - 1
                     delp(i,k) = log( pinterp/ub_plevs(kl(i,k)) ) &
                               / log( ub_plevs(ku(i,k))/ub_plevs(kl(i,k)) )
                     exit
                  end if
               end do
            end if
         end do
      end do

!--------------------------------------------------------
!       ... set the mixing ratios at upper boundary
!--------------------------------------------------------
! wtw
!
! CAM5 
!	... find max level less than 50 mb
!           fix UB vals from top of model to this level
!
    do i = 1,ncol
       do k = 2,pver
          if( pmid(i,k) > 50.e2_r8 ) then
             kmax(i) = k
             exit
          end if
       end do
    end do
!-------------------------------------------
    do m = 1,ub_nspecies
       do i = 1,ncol
          do k = 1,kmax(i)
               pint_vals(1) = mr_ub_bdy(i,kl(i,k),lchnk,last,m) &
                              + delp(i,k) &
                             * (mr_ub_bdy(i,ku(i,k),lchnk,last,m) &
                                - mr_ub_bdy(i,kl(i,k),lchnk,last,m))
               pint_vals(2) = mr_ub_bdy(i,kl(i,k),lchnk,next,m) &
                              + delp(i,k) &
                             * (mr_ub_bdy(i,ku(i,k),lchnk,next,m) &
                                - mr_ub_bdy(i,kl(i,k),lchnk,next,m))
               if( m /= table_nox_ndx ) then
!-------------
                  if( map(m) == ox_ndx ) then
                     vmr(i,k,map(m)) = o3vmr(i,k)            ! o3vmr: CMIP5 o3 vmr
                  else
                     vmr(i,k,map(m)) = pint_vals(1) &
                                  + dels * (pint_vals(2) - pint_vals(1))
                  endif

               else if( sim_has_nox ) then
                  nox_ubc = pint_vals(1) + dels * (pint_vals(2) - pint_vals(1))
                  if( no_ndx > 0 ) then
                     xno  = vmr(i,k,no_ndx)
                  else
                     xno  = 0.
                  end if
                  if( no2_ndx > 0 ) then
                     xno2 = vmr(i,k,no2_ndx)
                  else
                     xno2 = 0.
                  end if
                  rno  = xno / (xno + xno2)
                  if( no_ndx > 0 ) then
                     vmr(i,k,no_ndx)  = rno * nox_ubc
                  end if
                  if( no2_ndx > 0 ) then
                     vmr(i,k,no2_ndx) = (1. - rno) * nox_ubc
                  end if
               end if
            end do
         end do
      end do

!--------------------------------------------------------
!       ... set up the relaxation for lower stratosphere
!--------------------------------------------------------
! taurelax = relaxation timescale (in sec)
! facrelax = fractional relaxation towards ubc
!          = 1 => use ubc
!          = 0 => ignore ubc, use model concentrations
!--------------------------------------------------------
      taurelax = 864000. ! (10 days)
!--------------------------------------------------------
! for delt/taurelax << 1, could approximate as facrelax = delt/taurelax
!--------------------------------------------------------
      facrelax = 1. - exp( -delt / taurelax )

!--------------------------------------------------------
!       ... find the tropopause location
!--------------------------------------------------------
      do i = 1,ncol
        found_trop = .false.
        do k = pver-1,2,-1
           if( zmid(i,k) < ztrop_low ) then
              cycle
           else if( zmid(i,k) > ztrop_high ) then
              exit
           end if
           dtdz = (tfld(i,k) - tfld(i,k-1)) /(zmid(i,k-1) - zmid(i,k))
           ltrop = k
           if( dtdz <= max_dtdz ) then
              found_trop = .true.
              exit
           end if
        end do
        if( .not. found_trop ) then
           write(*,*) 'set_ub_vals: failed to find tropopause at lchnk ind=',lchnk,', lon ind=',i
        end if
!--------------------------------------------------------
!       ... relax lower stratosphere to extended ubc
!           check to make sure ubc is not being imposed too low
!           levrelax = lowest model level (highest pressure)
!                      in which to relax to ubc
!--------------------------------------------------------

        levrelax = ltrop
        do while( pmid(i,levrelax) > ub_plevs(ub_nlevs) )
           levrelax = levrelax - 1
        end do
        do k = kmax(i)+1,levrelax
           if( sim_has_nox ) then
              if( no_ndx > 0 ) then
                 xno  = vmr(i,k,no_ndx)
              else
                 xno  = 0.
              end if
              if( no2_ndx > 0 ) then
                 xno2 = vmr(i,k,no2_ndx)
              else
                 xno2 = 0.
              end if
              rno     = xno / (xno + xno2)
              nox_ubc = xno + xno2
           end if
           do m = 1,ub_nspecies
              if( map(m) == co_ndx .or. map(m) == ch4_ndx ) then
                 cycle
              end if
              pint_vals(1) = mr_ub_bdy(i,kl(i,k),lchnk,last,m) &
                             + delp(i,k) &
                               * (mr_ub_bdy(i,ku(i,k),lchnk,last,m) &
                                  - mr_ub_bdy(i,kl(i,k),lchnk,last,m))
              pint_vals(2) = mr_ub_bdy(i,kl(i,k),lchnk,next,m) &
                             + delp(i,k) &
                              * (mr_ub_bdy(i,ku(i,k),lchnk,next,m) &
                                 - mr_ub_bdy(i,kl(i,k),lchnk,next,m))
              vmrrelax = pint_vals(1) &
                         + dels * (pint_vals(2) - pint_vals(1))
!----
! h2o is not restored 
!
              if( m /= table_nox_ndx .and. m /= table_h2o_ndx  ) then
!mozart2      if( m /= table_nox_ndx ) then

!---------------------------
                  if( map(m) == ox_ndx ) then
                      vmr(i,k,map(m)) = vmr(i,k,map(m)) &              ! o3vmr: CMIP5 o3 vmr
                                   + ( o3vmr(i,k) - vmr(i,k,map(m))) * facrelax
                  else
                      vmr(i,k,map(m)) = vmr(i,k,map(m)) &
                                      + (vmrrelax - vmr(i,k,map(m))) * facrelax
                 endif
!---------------------------
              else if( sim_has_nox) then
                 nox_ubc = nox_ubc + (vmrrelax - nox_ubc) * facrelax
              end if
           end do
           if( sim_has_nox ) then
              if( no_ndx > 0 ) then
                  vmr(i,k,no_ndx)  = rno * nox_ubc
              end if
              if( no2_ndx > 0 ) then
                  vmr(i,k,no2_ndx) = (1. - rno) * nox_ubc
              end if
           end if
        end do
      end do

      end subroutine set_ub_vals

end module bcc_ub_vals
