#include <misc.h>
#include <params.h>

subroutine advnce
!-----------------------------------------------------------------------
!
! Purpose: 
! Advance time information
!
! Method: 
!
! Author: CCM1, CMS Contact: J. Truesdale
!
!-----------------------------------------------------------------------
  use shr_kind_mod, only: r8 => shr_kind_r8
  use chemistry,    only: trace_gas
  use aerosol_intr, only: aerosol_time_interp
#ifdef GHG_2D
  use ghg_surfvals_2d, only: ghg_surfvals_ramp_2d,ghg_ramp_read_2d
  use ghg_surfvals_2d, only: co2_surfvals_ramp_2d,co2_ramp_time_2d
#else
  use ghg_surfvals, only: ghg_surfvals_ramp, ghg_surfvals_set
#endif
  use tracers,      only: tracers_flag, tracers_timestep_init
  use time_manager, only: get_nstep
#ifdef VOL_DENS
  use volcanicdens, only: read_volcanic_dens
#else
  use volcanicmass, only: read_volcanic_mass
#endif

#ifdef MAC_SP
  use volcanicradi, only: read_volcanic_radi
#endif
#ifdef MAC_SP
  use prescribed_aerosols,      only: aerint, strat_volcanic, rampyear_prescribed_aeros
  use prescribed_aerosols_cdnc, only: aerint_cdnc
#else
  use prescribed_aerosols, only: aerint, strat_volcanic, rampyear_prescribed_aeros
#endif

  use ramp_scon, only: ramp_sconst
  use ozn_data,  only: oznint

  implicit none

#include <comctl.h>
#include <comlun.h>
!
!-----------------------------------------------------------------------
!
! Local workspace
!
  integer :: nstep             ! current timestep number
!-----------------------------------------------------------------------

  nstep = get_nstep()
!
! Determine whether it is time for a shortwave or longwave radiation 
! calculation
!
  dosw = nstep.eq.0 .or. iradsw.eq.1 .or. (mod(nstep-1,iradsw).eq.0 .and. nstep.ne.1)
  dolw = nstep.eq.0 .or. iradlw.eq.1 .or. (mod(nstep-1,iradlw).eq.0 .and. nstep.ne.1)
!
! Determine whether it is time for an absorptivity/emissivity calculation
!
  doabsems = nstep.eq.0 .or. iradae.eq.1 .or. (mod(nstep-1,iradae).eq.0 .and. nstep.ne.1)
  aeres = (mod(nstep,iradae).ne.0)
!
! Update ozone and aerosol data on shortwave or longwave time step. 
! Note that the ozone data is not needed on a longwave time step unless the
! absorptivities are being updated ("doabsems").
!
  if (dosw .or. dolw) then
     call oznint ()

#ifdef MAC_SP
     call aerint ()
     call aerint_cdnc ()
#else
     call aerint ()
#endif

     if(strat_volcanic) then
#ifdef VOL_DENS
       call read_volcanic_dens(rampyear_prescribed_aeros)
#else
       call read_volcanic_mass(rampyear_prescribed_aeros)
#endif
     end if 

#ifdef MAC_SP
         call read_volcanic_radi(rampyear_prescribed_aeros)
#endif

  end if
!
! Ramping ghg if appropriate
!
#ifdef GHG_2D
   if (ghg_surfvals_ramp_2d()) then
      call  ghg_ramp_read_2d()
   else if (co2_surfvals_ramp_2d()) then 
      call  co2_ramp_time_2d()
   else
    write(6,*) 'Error in ghg_surfvals_init_2d,ghg_surfvals_ramp_2d'
   end if
#else  
   if (ghg_surfvals_ramp()) call ghg_surfvals_set()
#endif
!
! Ramp solar constant if appropraite
!
  if (doRamp_scon) call ramp_sconst
!
! Time interpolate for chemistry, if appropriate
!
!wtw  if (trace_gas) call chem_timestep_init
!
! get the aerosol surface fluxes for this time step
  call aerosol_time_interp
!
! Time interpolate for tracers, if appropriate
!
  if (tracers_flag) call tracers_timestep_init

end subroutine advnce
