#include <misc.h>
#include <params.h>

subroutine cldnrh_lmdz(lchnk,  ncol,                    &
                        pmid,  pint, ztodt, dlf, t, q,             &
                        qi,    ql ,  dpp, cld) 

!-----------------------------------------------------------------------
!
! Purpose:
! Interface to compute cloud
!
!
! Author: ZF
!
!-----------------------------------------------------------------------
    use shr_kind_mod,    only: r8 => shr_kind_r8
    use physconst,       only: cpair, latvap, latice
    use ppgrid,          only: pcols, pver

    implicit none

    real(r8), parameter ::   RG        = 9.80665
!------------------------------Arguments--------------------------------
!
! Input arguments
!
   integer, intent(in) :: lchnk                  ! chunk identifier
   integer, intent(in) :: ncol                   ! number of atmospheric columns

   real(r8), intent(in) :: ztodt 
   real(r8), intent(in) :: pint(pcols,pver+ 1)   ! interface pressures
!   real(r8), intent(in) :: ps(pcols)             ! surface pressures (Pa) 
   real(r8), intent(in) :: pmid(pcols,pver)      ! midpoint pressures
   real(r8), intent(in) :: t(pcols,pver)         ! temperature
   real(r8), intent(in) :: q(pcols,pver)         ! specific humidity
   real(r8), intent(in) :: qi(pcols,pver)        ! cloud ice content
   real(r8), intent(in) :: ql(pcols,pver)        ! cloud liquid content
   real(r8),intent(in)  :: dlf(pcols,pver)       ! 
   real(r8), intent(in) :: dpp(pcols,pver)       ! 
!
! Output arguments
!
   real(r8)             :: cldn(pcols,pver)      ! cloud fraction
   real(r8)             :: cldf(pcols,pver)      ! convective cloud

   real(r8), intent(out) :: cld(pcols,pver)       ! cloud fraction
!
!---------------------------Local workspace-----------------------------
!
   integer           :: i, k

   real(r8),allocatable  :: d_t(:,:)              ! temperature variation (K)
   real(r8),allocatable  :: d_q(:,:)              ! water vapour variation
   real(r8),allocatable  :: d_ql(:,:)             ! liquid water variation 
   real(r8),allocatable  :: radliq(:,:)           ! liquid water for radiation 
   real(r8),allocatable  :: rhcl(:,:)             ! relative humidity for clear sky
   real(r8),allocatable  :: rain(:)               ! rainfall rate (mm/s)
   real(r8),allocatable  :: snow(:)               ! snowfall rate (mm/s)
   real(r8),allocatable  :: prfl(:,:)             ! rainfall flux (kg/m2/s)
   real(r8),allocatable  :: psfl(:,:)             ! snowfall flux (kg/m2/s)
   real(r8),allocatable  :: qs(:,:)               ! saturated water vapour
 ! real(r8),allocatable  :: ptconv(:,:)           ! convection point
 ! zhangli--change----according to subroutines---
   LOGICAL, allocatable  :: ptconv(:,:)   
   
   real(r8),allocatable  :: ratqss(:,:)       
   real(r8),allocatable  :: ratqsc(:,:)        
   real(r8),allocatable  :: ratqs(:,:)       
   real(r8),allocatable  :: cond(:,:)           

   real(r8),allocatable  :: pint1(:,:)             ! interface pressures
   real(r8),allocatable  :: pmid1(:,:)             ! midpoint pressures
   real(r8),allocatable  :: t1(:,:)                ! temperature
   real(r8),allocatable  :: q1(:,:)                ! specific humidity
   real(r8),allocatable  :: dpp1(:,:)              ! 
   real(r8),allocatable  :: cldn1(:,:)             ! cloud fraction
   real(r8),allocatable  :: cldf1(:,:)             ! cloud fraction
!--------------------------------------------------------------------------------

   allocate(pint1(pcols,pver+ 1)) 
   allocate(pmid1(pcols,pver))
   allocate(t1(pcols,pver))
   allocate(q1(pcols,pver))
   allocate(dpp1(pcols,pver))
   allocate(cldn1(pcols,pver))
   allocate(cldf1(pcols,pver))

   allocate(d_t(pcols,pver))   
   allocate(d_q(pcols,pver))  
   allocate(d_ql(pcols,pver))
   allocate(radliq(pcols,pver)) 
   allocate(rhcl(pcols,pver))  
   allocate(rain(pcols))      
   allocate(snow(pcols))     
   allocate(prfl(pcols,pver+1)) 
   allocate(psfl(pcols,pver+1))
   allocate(qs(pcols,pver))   
   allocate(ptconv(pcols,pver)) 
   allocate(ratqss(pcols,pver))
   allocate(ratqsc(pcols,pver))
   allocate(ratqs(pcols,pver))
   allocate(cond(pcols,pver))

  do k = 1, pver
     pint1(:,k) = pint(:,pver+1-k+1)
     pmid1(:,k) = pmid(:,pver-k+1)

!    t1(:,k)    = t(:,pver-k+1) - (latice * qi(:,pver-k+1) + latvap * ql(:,pver-k+1))/cpair 
!    t1(:,k)    = t(:,pver-k+1) 
!    q1(:,k)    = q(:,pver-k+1) + qi(:,pver-k+1) + ql(:,pver-k+1)

     t1(:,k)    = t(:,pver-k+1)
     q1(:,k)    = q(:,pver-k+1)

     cond(:,k)  = dlf(:,pver-k+1)
     dpp1(:,k)  = dpp(:,pver-k+1)
  enddo
  pint1(:,pver+1) = pint(:,1)

!
! calculate the saturated water vapour:
!
  call calcul_qsat(ncol,pver,t1,pmid1,qs)

!
! Calculate cloud fraction by using the convective condensation:
!
  call clouds_gno(ncol,pver,q1,qs,cond,ptconv,ratqsc,cldf1)
!
! Calculate the values of ratqs (water vapour distribution):
!

  call calcul_ratqs(ncol, pver,t1, pint1, pmid1, ratqss)
  ratqs(:,:) = SQRT(ratqsc(:,:)**2+ratqss(:,:)**2)   !org  !zhangli add from org
!
! Large-scale cloud scheme:
!
!  cldn1(:,:) = 0.0 

  call pdfcloud_lsc(ncol,pver,pmid1,t1,q1,ratqs, cldn1)

  do k = 1, pver
     cldn(:,k) = cldn1(:,pver-k+1)
     cldf(:,k) = cldf1(:,pver-k+1) 
     cld(:,k) = max(cldn(:,k), cldf(:,k)) !org
  enddo

  deallocate(pint1)
  deallocate(pmid1)
  deallocate(t1)
  deallocate(q1)
  deallocate(dpp1)
  deallocate(cldn1)
  deallocate(cldf1)

  deallocate(d_t)      
  deallocate(d_q)     
  deallocate(d_ql)
  deallocate(radliq)
  deallocate(rhcl) 
  deallocate(rain)         
  deallocate(snow)        
  deallocate(prfl)
  deallocate(psfl)
  deallocate(qs)  
  deallocate(ptconv) 
  deallocate(ratqss)
  deallocate(ratqsc)
  deallocate(ratqs)
  deallocate(cond)

end subroutine cldnrh_lmdz

!-------------------------------------------------------------------------

SUBROUTINE calcul_qsat(klon,klev,t,pplay,qs)

  use shr_kind_mod,    only: r8 => shr_kind_r8
  IMPLICIT none

  real(r8), parameter ::   RTT       = 273.16
  real(r8), parameter ::   R3LES     = 17.269
  real(r8), parameter ::   R3IES     = 21.875
  real(r8), parameter ::   R4LES     = 35.86
  real(r8), parameter ::   R4IES     = 7.66
  real(r8), parameter ::   RMV       = 18.0153
  real(r8), parameter ::   RMD       = 28.9644
  real(r8), parameter ::   RKBOL     = 1.380658e-23
  real(r8), parameter ::   RNAVO     = 6.0221367e23
  real(r8), parameter ::   R         = RNAVO*RKBOL
  real(r8), parameter ::   RD        = 1000.*R/RMD
  real(r8), parameter ::   RV        = 1000.*R/RMV
  real(r8), parameter ::   RESTT     = 611.14
  real(r8), parameter ::   R2ES      = RESTT*RD/RV
  real(r8), parameter ::   RETV      = RV/RD -1


  INTEGER, INTENT(IN) :: klon, klev
  REAL, DIMENSION(klon,klev), INTENT(IN) :: t, pplay
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: qs
!
  REAL :: zdelta, zFOEEW, zcor
  INTEGER :: i, k
!
  DO k = 1, klev
  DO i = 1, klon
     zdelta=MAX(0.,SIGN(1.,RTT-t(i,k)))
     zFOEEW = EXP ( (R3LES*(1.-zdelta)+R3IES*zdelta) * (t(i,k)-RTT)        &
                  / (t(i,k)-(R4LES*(1.-zdelta)+R4IES*zdelta)) )
     qs(i,k)= R2ES*zFOEEW/pplay(i,k)
     qs(i,k)=MIN(0.5,qs(i,k))
     zcor=1./(1.-RETV*qs(i,k))
     qs(i,k)=qs(i,k)*zcor
  ENDDO
  ENDDO
!
END SUBROUTINE calcul_qsat

!---------------------------------------------------------------------------
SUBROUTINE clouds_gno(klon,klev,R,RS,QSUB,ptconv,ratqsc,cldf)
  IMPLICIT NONE
!
! Cloud fraction parameterization, based on a PDF approach of Bony and Emanuel (2001, JAS)
! (Programme adapted from LMDZ, put into f90 format by Laurent Li 2008-12-30)
!
! ***          There is no subgrid-scale condensation;        ***
! ***   the scheme becomes equivalent to an "all-or-nothing"  ***
! ***             large-scale condensation scheme.            ***
!
! ***     Some condensation is produced at the subgrid-scale       ***
! ***                                                              ***
! ***       PDF = generalized log-normal distribution (GNO)        ***
! ***   (k<0 because a lower bound is considered for the PDF)      ***
! ***                                                              ***
! ***  -> Determine x (the parameter k of the GNO PDF) such        ***
! ***  that the contribution of subgrid-scale processes to         ***
! ***  the in-cloud water content is equal to QSUB(k)              ***
! ***  (equations (13), (14), (15) + Appendix B of the paper)      ***
! ***                                                              ***
! ***    Here, an iterative method is used for this purpose        ***
! ***    (other numerical methods might be more efficient)         ***
! ***                                                              ***
! ***          NB: the "error function" is called ERF              ***
! ***                 (ERF in double precision)                   ***
!
  INTEGER, INTENT(IN) :: klon, klev ! horizontal and vertical dimensions
  REAL, DIMENSION(klon,klev), INTENT(IN) :: R
        ! Domain-averaged mixing ratio of total water
  REAL, DIMENSION(klon,klev), INTENT(IN) :: RS
        ! Mean saturation humidity mixing ratio within the gridbox
  REAL, DIMENSION(klon,klev), INTENT(IN) :: QSUB
        ! Mixing ratio of condensed water within clouds associated
        ! with SUBGRID-SCALE condensation processes (here, it is
        ! predicted by the convection scheme)
!
  LOGICAL, DIMENSION(klon,klev), INTENT(OUT) :: ptconv ! Point convectif = TRUE
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: ratqsc ! normalized distribution width
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: cldf ! cloud fraction
!
! -- parameters controlling the iteration:
! --    nmax    : maximum nb of iterations (hopefully never reached)
! --    epsilon : accuracy of the numerical resolution
! --    vmax    : v-value above which we use an asymptotic expression for ERF(v)
!
  INTEGER, PARAMETER :: nmax=10
  REAL, PARAMETER :: epsilon=0.02, vmax0=2.0
  REAL, PARAMETER :: min_mu=1.e-12, min_Q=1.e-12
!
  REAL, ALLOCATABLE, DIMENSION(:) :: vmax
  REAL, ALLOCATABLE, DIMENSION(:) :: mu, delta, beta
  REAL, ALLOCATABLE, DIMENSION(:) :: xx
  LOGICAL, ALLOCATABLE, DIMENSION(:) :: lconv ! .T. if the calculation converges
                                              ! (also true if qsub < min_q)
  REAL :: zu2, zv2, u, v, erfcu, erfcv, dist, fprime, det, block, coeff, xx1, xx2, aux
!
  INTEGER :: i,k, n, m
  REAL :: pi, sqrtpi, sqrt2, zx1, zx2, exdel
!
  REAL :: ERF ! error function (is it an intrinsic function ?)
!
  ALLOCATE(vmax(klon))
  ALLOCATE(mu(klon))
  ALLOCATE(delta(klon))
  ALLOCATE(beta(klon))
  ALLOCATE(xx(klon))
  ALLOCATE(lconv(klon))
!
  pi = ACOS(-1.)
  sqrtpi=SQRT(pi)
  sqrt2=SQRT(2.)
!
  ptconv(:,:)=.FALSE.
  ratqsc(:,:)=0.
  cldf(:,:)=0.0
!
  DO 500 k = 1, klev
   DO i = 1, klon
      mu(i) = R(i,k)
      mu(i) = MAX(mu(i),min_mu)
      delta(i) = LOG(mu(i)/MAX(RS(i,k),min_mu))
   ENDDO
!
   DO i = 1, klon
   IF (QSUB(i,k) .LT. min_Q) THEN ! not enough cloud water, pass
      ptconv(i,k)=.FALSE.
      ratqsc(i,k)=0.
      lconv(i)  = .TRUE.
   ELSE
      lconv(i)  = .FALSE.
      vmax(i) = vmax0
      beta(i) = QSUB(i,k)/mu(i) + EXP( -MIN(0.0,delta(i)) )
! --  roots of equation v > vmax:
      det = delta(i) + vmax(i)*vmax(i)
      IF (det.LE.0.0) vmax(i) = vmax0 + 1.0
      det = delta(i) + vmax(i)*vmax(i)
      IF (det.LE.0.) THEN
          xx(i) = -0.0001
      ELSE
         zx1=-sqrt2*vmax(i)
         zx2=SQRT(1.0+delta(i)/(vmax(i)*vmax(i)))
         xx1=zx1*(1.0-zx2)
         xx2=zx1*(1.0+zx2)
         xx(i) = 1.01 * xx1
         IF ( xx1 .GE. 0.0 ) xx(i) = 0.5*xx2
      ENDIF
      IF (delta(i).LT.0.) xx(i) = -0.5*SQRT(log(2.))
   ENDIF
   ENDDO
!----------------------------------------------------------------------
   DO n = 1, nmax ! Iterative procedure to find the solution
   DO i = 1, klon ! vector
   IF (.NOT.lconv(i)) THEN
!----------------------------------------------------------------------
      u = delta(i)/(xx(i)*sqrt2) + xx(i)/(2.*sqrt2)
      v = delta(i)/(xx(i)*sqrt2) - xx(i)/(2.*sqrt2)
      IF ( v .GT. vmax(i) ) THEN
         IF (ABS(u).GT.vmax(i) .AND.  delta(i).LT.0.) THEN
! -- use asymptotic expression of erf for u and v large:
! ( -> analytic solution for xx )
             exdel=beta(i)*EXP(delta(i))
             aux = 2.0*delta(i)*(1.-exdel)/(1.+exdel)
             IF (aux.lt.0.) THEN
!                print*,'AUX(',i,',',k,')<0',aux,delta(i),beta(i)
                aux=0.
             ENDIF
             xx(i) = -SQRT(aux)
             block = EXP(-v*v) / v / sqrtpi
             dist = 0.0
             fprime = 1.0
         ELSE
! -- erfv -> 1.0, use an asymptotic expression of erfv for v large:
             erfcu = 1.0-ERF(u)
!  !!! ATTENTION : rajout d'un seuil pour l'exponentiel
             aux = sqrtpi*erfcu*EXP(min(v*v,100.))
             coeff = 1.0 - 1./2./(v*v) + 3./4./(v*v*v*v)
             block = coeff * EXP(-v*v) / v / sqrtpi
             dist = v * aux / coeff - beta(i)
             fprime = 2.0 / xx(i) * (v*v) &
                * ( coeff*EXP(-delta(i)) - u * aux ) /coeff/coeff
         ENDIF ! ABS(u)
      ELSE
! -- general case:
         erfcu = 1.0-ERF(u)
         erfcv = 1.0-ERF(v)
         block = erfcv
         dist = erfcu / erfcv - beta(i)
         zu2=u*u
         zv2=v*v
         IF (zu2.GT.20. .OR. zv2.GT.20.) THEN
            zu2=20.
            zv2=20.
            fprime = 0.
         ELSE
            fprime = 2. /sqrtpi /xx(i) /(erfcv*erfcv) &
                * ( erfcv*v*EXP(-zu2) - erfcu*u*EXP(-zv2) )
         ENDIF
      ENDIF ! x
!
! -- test numerical convergence:
      IF (beta(i).LT.1.e-10) THEN
         print*,'avant test ',i,k,lconv(i),u,v,beta(i)
         CALL abort
      ENDIF
      IF (ABS(fprime).LT.1.e-11) THEN
         print*,'avant test fprime<.e-11 ',i,k,lconv(i),u,v,beta(i),fprime
         print*,'i,k,R,RS,QSUB', i,k,R(i,k),rs(i,k),qsub(i,k)
         fprime=SIGN(1.e-11,fprime)
      ENDIF
!
      IF (ABS(dist/beta(i)) .LT. epsilon) THEN
         ptconv(i,k) = .TRUE.
         lconv(i)=.TRUE.
         ratqsc(i,k)=MIN(2.*(v-u)*(v-u),20.) !  borne pour l'exponentielle
         ratqsc(i,k)=SQRT(EXP(ratqsc(i,k))-1.)
         cldf(i,k) = 0.5 * block
      ELSE
         xx(i) = xx(i) - dist/fprime
      ENDIF
!----------------------------------------------------------------------
   ENDIF ! lconv
   ENDDO ! vector
   ENDDO ! End of Iterative procedure to find the solution
!----------------------------------------------------------------------

  500 CONTINUE  ! k
!
  DEALLOCATE(vmax)
  DEALLOCATE(mu)
  DEALLOCATE(delta)
  DEALLOCATE(beta)
  DEALLOCATE(xx)
  DEALLOCATE(lconv)
!
END SUBROUTINE clouds_gno

!---------------------------------------------------------------------------
SUBROUTINE calcul_ratqs(klon, klev,t, ps, pplay, ratqs)
IMPLICIT none
!
! ratqs associated to large-scale condensation
!
  INTEGER, INTENT(IN) :: klon, klev
  REAL, DIMENSION(klon,klev),INTENT(IN) :: t
  REAL, DIMENSION(klon), INTENT(IN) :: ps
  REAL, DIMENSION(klon,klev), INTENT(IN) :: pplay
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: ratqs
  REAL :: alfa  
!
 REAL, SAVE :: ratqsbas=0.005, ratqshaut=0.33   !org PDF 
! REAL, SAVE :: ratqsbas=0.33, ratqshaut=0.005   !pdf.1
! REAL, SAVE :: ratqsbas=0.5, ratqshaut=0.005   !pdf.2
! REAL, SAVE :: ratqsbas=0.5, ratqshaut=0.100   !pdf.3
! REAL, SAVE :: ratqsbas=0.1, ratqshaut=0.005   !pdf.4
! REAL, SAVE :: ratqsbas=0.1, ratqshaut=0.001   !pdf.5
! REAL, SAVE :: ratqsbas=0.009, ratqshaut=0.0005   !pdf.6
!  REAL, SAVE :: ratqsbas=0.009, ratqshaut=0.0001   !pdf.7

!
  INTEGER :: i, k
!
  alfa = 1.0
  DO k = 1, klev
  DO i = 1, klon
!----------org-------------------
!     ratqs(i,k)=ratqsbas+(ratqshaut-ratqsbas)* &            ! org
!                 MIN((ps(i)-pplay(i,k))/(ps(i)-30000.),1.)  ! org,pdf,pdf.1-pdf.4
!                 MIN((ps(i)-pplay(i,k))/(ps(i)-10000.),1.)  ! pdf.5-PDF.7
!----------zhangli---adding tuning parameter refer to temperature---
     if(t(i,k).lt.273.16)then
          alfa  = 0.5*(abs(t(i,k)-273.16))   !alfa-t
!          alfa = 10.0   !alfa10
!         alfa = 20.0   ! alfa20
!         alfa = 30.0   ! alfa30
     else
         alfa = 1.0
     end if
     ratqs(i,k)=ratqsbas+alfa * (ratqshaut-ratqsbas)* &
                MIN((ps(i)-pplay(i,k))/(ps(i)-30000.),1.)  !pdf.4-alfa20,30,pdf.alfa10        
!---------------------------------------------------------------------

  ENDDO
  ENDDO

! ratqs(:,:) = 0.1
!
END SUBROUTINE calcul_ratqs

!----------------------------------------------------------------------------

SUBROUTINE fisrtilp(klon,klev,dtime,paprs,pplay,t,q,ptconv,ratqs, &
                    d_t, d_q, d_ql, rneb, radliq, rain, snow, &
                    prfl, psfl, rhcl)
    use shr_kind_mod,    only: r8 => shr_kind_r8
IMPLICIT none
!======================================================================
! Programmed by Z.X. Li (LMD/CNRS) date: 19950320
! Object: Large-scale condensation and precipitation. Cloud scheme
!======================================================================
! main physical constants
!#include "YOMCST.h"
! thermodynamics constants
!#include "YOETHF.h"
    real(r8), parameter ::   RTT       = 273.16
    real(r8), parameter ::   RMV       = 18.0153
    real(r8), parameter ::   RMD       = 28.9644
    real(r8), parameter ::   RKBOL     = 1.380658e-23
    real(r8), parameter ::   RNAVO     = 6.0221367e23
    real(r8), parameter ::   R         = RNAVO*RKBOL
    real(r8), parameter ::   RD        = 1000.*R/RMD
    real(r8), parameter ::   RV        = 1000.*R/RMV
    real(r8), parameter ::   RESTT     = 611.14
    real(r8), parameter ::   R2ES      = RESTT*RD/RV
    real(r8), parameter ::   R3LES     = 17.269
    real(r8), parameter ::   R3IES     = 21.875
    real(r8), parameter ::   R4LES     = 35.86
    real(r8), parameter ::   R4IES     = 7.66
    real(r8), parameter ::   R5LES     = R3LES*(RTT-R4LES)
    real(r8), parameter ::   R5IES     = R3IES*(RTT-R4IES)
    real(r8), parameter ::   RCPV      = 4.*RV
    real(r8), parameter ::   RCPD      = 3.5*RD
    real(r8), parameter ::   RVTMP2    = RCPV/RCPD - 1
    real(r8), parameter ::   RG        = 9.80665
    real(r8), parameter ::   RETV      = RV/RD -1
    real(r8), parameter ::   RLVTT     = 2.5008e6
    real(r8), parameter ::   RLSTT     = 2.8345e6

!
! Arguments:
!
  INTEGER, INTENT(IN) :: klon, klev ! horizontal and vertical dimensions
  REAL, INTENT(IN) :: dtime ! time step (s)
  REAL, DIMENSION(klon,klev+1), INTENT(IN) :: paprs ! interface pressure (Pa)
  REAL, DIMENSION(klon,klev), INTENT(IN) :: pplay ! mid-layer pressure (Pa)
  REAL, DIMENSION(klon,klev), INTENT(IN) :: t ! temperature (K)
  REAL, DIMENSION(klon,klev), INTENT(IN) :: q ! specific humidity (kg/kg)
  LOGICAL, DIMENSION(klon,klev), INTENT(IN) :: ptconv ! convection point
  REAL, DIMENSION(klon,klev), INTENT(IN) :: ratqs ! water vapour distribution width
!
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: d_t ! temperature variation (K)
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: d_q ! water vapour variation
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: d_ql ! liquid water variation
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: rneb ! cloud fraction
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: radliq ! liquid water for radiation
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: rhcl ! relative humidity for clear sky  
  REAL, DIMENSION(klon), INTENT(OUT) :: rain ! rainfall rate (mm/s)
  REAL, DIMENSION(klon), INTENT(OUT) :: snow ! snowfall rate (mm/s)
  REAL, DIMENSION(klon,klev+1), INTENT(OUT) :: prfl ! rainfall flux (kg/m2/s)
  REAL, DIMENSION(klon,klev+1), INTENT(OUT) :: psfl ! snowfall flux (kg/m2/s)
!
! Some parameters of the programme:
!
  REAL, PARAMETER :: cld_lc_lsc = 2.6e-4 ! conversion constant from cloud to rain
  REAL, PARAMETER :: cld_lc_con = 2.6e-4 ! conversion constant from cloud to rain
  REAL, PARAMETER :: cld_tau_lsc = 3600.0 ! time scale of cloud-to-rain conversion
  REAL, PARAMETER :: cld_tau_con = 3600.0 ! time scale of cloud-to-rain conversion
  REAL, PARAMETER :: ffallv_lsc = 1.0 ! constant to adjust ice-falling velocity
  REAL, PARAMETER :: ffallv_con = 1.0 ! constant to adjust ice-falling velocity
  REAL, PARAMETER :: coef_eva = 2.0e-5 ! constant for rain evaporation
  LOGICAL, PARAMETER :: reevap_ice = .TRUE.
  INTEGER, PARAMETER :: iflag_pdf = 1 ! assumption for water vapour distribution
  REAL, PARAMETER :: seuil_neb=0.001 ! threshold for a cloud to be considered as existent
  INTEGER, PARAMETER :: ninter=5 ! sub temporal intervals for the precipitation
  LOGICAL, PARAMETER :: evap_prec=.TRUE. ! take into account the evaporation of rain
  LOGICAL, PARAMETER :: cpartiel=.TRUE. ! partial condensation (opposite of all-or-nothing)
  REAL, PARAMETER :: t_coup=234.0 ! temperature threshold for ice
  INTEGER, PARAMETER :: nexpo = 6 ! controls the slope of the transition between ice/water
!
! Local Variables:
!
  LOGICAL, SAVE :: appel1er=.TRUE.
!
  REAL :: zdelta, zcor, zcvm5, zqev, zqevt, zdelq, ztglace
  REAL :: zmair, zcpair, zcpeau, zm_solid
  REAL, ALLOCATABLE, DIMENSION(:) :: zct, zcl
  REAL, ALLOCATABLE, DIMENSION(:) :: zpdf_sig,zpdf_k,zpdf_delta,Zpdf_a,zpdf_b,zpdf_e1,zpdf_e2
  REAL, ALLOCATABLE, DIMENSION(:) :: zqs, zdqs, zrfl, zrfln, zoliq, zcond, zq, zqn, zt
  REAL, ALLOCATABLE, DIMENSION(:) :: zdz, zrho, ztot, zrhol, zchau, zfroi, zfice, zneb
  REAL, ALLOCATABLE, DIMENSION(:) :: zprec_cond, zlh_solid
!
  INTEGER :: i, k, n, kk
! changed by zhangli 20130115
!  REAL, EXTERNAL ::  ERF ! error function (is it an intrinsic function ?)
! changed by zhangli
!
  REAL :: fallv ! Final falling velocity of ice cristals
  REAL :: zFOEEW ! saturated water vapour
  REAL :: zFOEDE ! derivative of the saturated water vapour
!
! Allocate temporal arrays:
!
  ALLOCATE (zct(klon))
  ALLOCATE (zcl(klon))
  ALLOCATE (zpdf_sig(klon))
  ALLOCATE (zpdf_k(klon))
  ALLOCATE (zpdf_delta(klon))
  ALLOCATE (Zpdf_a(klon))
  ALLOCATE (zpdf_b(klon))
  ALLOCATE (zpdf_e1(klon))
  ALLOCATE (zpdf_e2(klon))
  ALLOCATE (zqs(klon))
  ALLOCATE (zdqs(klon))
  ALLOCATE (zrfl(klon))
  ALLOCATE (zrfln(klon))
  ALLOCATE (zoliq(klon))
  ALLOCATE (zcond(klon))
  ALLOCATE (zq(klon))
  ALLOCATE (zqn(klon))
  ALLOCATE (zt(klon))
  ALLOCATE (zdz(klon))
  ALLOCATE (zrho(klon))
  ALLOCATE (ztot(klon))
  ALLOCATE (zrhol(klon))
  ALLOCATE (zchau(klon))
  ALLOCATE (zfroi(klon))
  ALLOCATE (zfice(klon))
  ALLOCATE (zneb(klon))
  ALLOCATE (zprec_cond(klon))
  ALLOCATE (zlh_solid(klon))
!
  IF (appel1er) THEN
     PRINT*, 'fisrtilp, ninter:', ninter
     PRINT*, 'fisrtilp, evap_prec:', evap_prec
     PRINT*, 'fisrtilp, cpartiel:', cpartiel
     appel1er = .FALSE.
  ENDIF ! test sur appel1er
!
  DO i = 1, klon
     zoliq(i)=0.
  ENDDO
 ztglace = RTT - 15.0 ! Determiner les nuages froids par leur temperature

  zdelq=0.0
!
! Initialize the outputs
!
  DO k = 1, klev+1
  DO i = 1, klon
     prfl(i,k) = 0.0
     psfl(i,k) = 0.0
  ENDDO
  ENDDO
  DO k = 1, klev
  DO i = 1, klon
     d_t(i,k) = 0.0
     d_q(i,k) = 0.0
     d_ql(i,k) = 0.0
     rneb(i,k) = 0.0
     radliq(i,k) = 0.0
  ENDDO
  ENDDO
  DO i = 1, klon
     rain(i) = 0.0
     snow(i) = 0.0
  ENDDO
!
! Initialize the precipitation flux to zero
!
  DO i = 1, klon
     zrfl(i) = 0.0
     zneb(i) = seuil_neb
  ENDDO
!
! Loop on vertical (from top to bottom):
!
  DO k = klev, 1, -1
!
   DO i = 1, klon
      zt(i)=t(i,k)
      zq(i)=q(i,k)
   ENDDO
!
! Calculate the air temperature variation due to the sensible heat
! transported by the rain flux. In the future, it may also be
! necessary to include this effect to the surface flux (temperature
! difference between the first level and the surface)
!
   DO i = 1, klon
   IF (k.LE.klev-1) THEN
      zmair=(paprs(i,k)-paprs(i,k+1))/RG
      zcpair=RCPD*(1.0+RVTMP2*zq(i))
      zcpeau=RCPD*RVTMP2
      zt(i) = ( (t(i,k+1)+d_t(i,k+1))*zrfl(i)*dtime*zcpeau + zmair*zcpair*zt(i) ) &
            / (zmair*zcpair + zrfl(i)*dtime*zcpeau)
   ENDIF
   ENDDO
!
! Calculate the evaporation of the precipitation
!
   IF (evap_prec) THEN
      DO i = 1, klon
      IF (zrfl(i) .GT.0.) THEN

         zdelta=MAX(0.,SIGN(1.,RTT-zt(i)))
         zFOEEW = EXP ( (R3LES*(1.-zdelta)+R3IES*zdelta) * (zt(i)-RTT)        &
                      / (zt(i)-(R4LES*(1.-zdelta)+R4IES*zdelta)) )
         zqs(i)= R2ES*zFOEEW/pplay(i,k)
         zqs(i)=MIN(0.5,zqs(i))
         zcor=1./(1.-RETV*zqs(i))
         zqs(i)=zqs(i)*zcor

         zqev = MAX (0.0, (zqs(i)-zq(i))*zneb(i) )
         zqevt = coef_eva * (1.0-zq(i)/zqs(i)) * SQRT(zrfl(i)) &
              * (paprs(i,k)-paprs(i,k+1))/pplay(i,k)*zt(i)*RD/RG
         zqevt = MAX(0.0,MIN(zqevt,zrfl(i))) &
              * RG*dtime/(paprs(i,k)-paprs(i,k+1))
         zqev = MIN (zqev, zqevt)
         zrfln(i) = zrfl(i) - zqev*(paprs(i,k)-paprs(i,k+1)) /RG/dtime

! pour la glace, on r???vapore toute la pr???ip dans la couche du dessous
! la glace venant de la couche du dessus est simplement dans la couche
! du dessous.

         IF (zt(i) .LT. t_coup.and.reevap_ice) zrfln(i)=0.

         zq(i) = zq(i) - (zrfln(i)-zrfl(i)) &
                  * (RG/(paprs(i,k)-paprs(i,k+1)))*dtime
         zt(i) = zt(i) + (zrfln(i)-zrfl(i)) &
                  * (RG/(paprs(i,k)-paprs(i,k+1)))*dtime &
                  * RLVTT/RCPD/(1.0+RVTMP2*zq(i))
         zrfl(i) = zrfln(i)
      ENDIF
      ENDDO
   ENDIF
!
! Calculate Qs (water vapour saturation)
! and L/Cp*dQs/dT (derivative to temperature).
! This linearization of the Qs allows a more accurate calculation of the
! condensation: the relase of latent heat during the condensation can
! increase the temperature and the saturated water vapour, so decrease
! a little the initially-obtained condensation amount
!
   DO i = 1, klon
      zdelta = MAX(0.,SIGN(1.,RTT-zt(i)))
      zcvm5 = R5LES*RLVTT*(1.-zdelta) + R5IES*RLSTT*zdelta
      zcvm5 = zcvm5 /RCPD/(1.0+RVTMP2*zq(i))
      zFOEEW = EXP ( (R3LES*(1.-zdelta)+R3IES*zdelta) * (zt(i)-RTT)        &
                   / (zt(i)-(R4LES*(1.-zdelta)+R4IES*zdelta)) )
      zqs(i) = R2ES*zFOEEW/pplay(i,k)
      zqs(i) = MIN(0.5,zqs(i))
      zcor = 1./(1.-RETV*zqs(i))
      zqs(i) = zqs(i)*zcor
      zFOEDE = zqs(i)*zcor*zcvm5 / (zt(i)-(R4LES*(1.-zdelta)+R4IES*zdelta))**2
      zdqs(i) = zFOEDE
      zdqs(i) = zFOEDE
   ENDDO
!
! Determine the partial condensation and calculate the amount of condensed water
! rneb - cloud cover;
! zqn - total water amount in cloud;
! zcond - averaged condensed water in the grid
!
! Two options are available: The first one is based on a simple uniform
! distribution of water vapour, with ratqs as the distribution width.
! The second option is based on a more compilicated PDF introduced by
! Sandrine Bony.
!
   IF (cpartiel) THEN
!
      IF (iflag_pdf.EQ.0) THEN ! Option of a uniform distribution
         DO i=1,klon
            zdelq = MIN(ratqs(i,k),0.99) * zq(i)
            rneb(i,k) = (zq(i)+zdelq-zqs(i)) / (2.0*zdelq)
            zqn(i) = (zq(i)+zdelq+zqs(i))/2.0
            rhcl(i,k)=(zqs(i)+zq(i)-zdelq)/2./zqs(i)
         ENDDO
      ELSE ! Option with new PDF
         DO i=1,klon
         IF (zq(i).LT.1.e-15) THEN
            print*,'ZQ(',i,',',k,')=',zq(i)
            zq(i)=1.e-15
         ENDIF
         ENDDO
         DO i=1,klon
            zpdf_sig(i)=ratqs(i,k)*zq(i)
            zpdf_k(i)=-SQRT(LOG(1.+(zpdf_sig(i)/zq(i))**2))
            zpdf_delta(i)=LOG(zq(i)/zqs(i))
            zpdf_a(i)=zpdf_delta(i)/(zpdf_k(i)*SQRT(2.))
            zpdf_b(i)=zpdf_k(i)/(2.*SQRT(2.))
            zpdf_e1(i)=zpdf_a(i)-zpdf_b(i)
            zpdf_e1(i)=SIGN(MIN(ABS(zpdf_e1(i)),5.),zpdf_e1(i))
            zpdf_e1(i)=1.-ERF(zpdf_e1(i))
            zpdf_e2(i)=zpdf_a(i)+zpdf_b(i)
            zpdf_e2(i)=SIGN(MIN(ABS(zpdf_e2(i)),5.),zpdf_e2(i))
            zpdf_e2(i)=1.-ERF(zpdf_e2(i))
            IF (zpdf_e1(i).LT.1.e-10) THEN
               rneb(i,k)=0.
               zqn(i)=zqs(i)
            ELSE
               rneb(i,k)=0.5*zpdf_e1(i)
               zqn(i)=zq(i)*zpdf_e2(i)/zpdf_e1(i)
            ENDIF
         ENDDO
      ENDIF ! iflag_pdf

      DO i=1,klon
         IF (rneb(i,k) .LE. 0.0) zqn(i) = 0.0
         IF (rneb(i,k) .GE. 1.0) zqn(i) = zq(i)
         rneb(i,k) = MAX(0.0,MIN(1.0,rneb(i,k)))
         zcond(i) = MAX(0.0,zqn(i)-zqs(i))*rneb(i,k)/(1.+zdqs(i))
!  On ne divise pas par 1+zdqs pour forcer a avoir l'eau predite par
!  la convection.  ATTENTION !!! Il va falloir verifier tout ca.
!         zcond(i) = MAX(0.0,zqn(i)-zqs(i))*rneb(i,k)
      ENDDO
   ELSE ! cpartiel
      DO i = 1, klon
         IF (zq(i).GT.zqs(i)) THEN
            rneb(i,k) = 1.0
         ELSE
            rneb(i,k) = 0.0
         ENDIF
         zcond(i) = MAX(0.0,zq(i)-zqs(i))/(1.+zdqs(i))
      ENDDO
   ENDIF ! cpartiel
!
   DO i = 1, klon
      zq(i) = zq(i) - zcond(i)
      zt(i) = zt(i) + zcond(i) * RLVTT/RCPD/(1.0+RVTMP2*zq(i))
      rhcl(i,k)=MAX(0.,MIN(1.0,zq(i)/zqs(i)))
   ENDDO
!
! Split the condensate into precipitation and cloud water content
!
   DO i = 1, klon
   IF (rneb(i,k).GT.0.0) THEN
      zoliq(i) = zcond(i)
      zrho(i) = pplay(i,k) / zt(i) / RD
      zdz(i) = (paprs(i,k)-paprs(i,k+1)) / (zrho(i)*RG)
 !org     zfice(i) = 1.0 - (zt(i)-ztglace) / (273.13-ztglace)
 ! zhangli---
       zfice(i) = 1.0 - (zt(i)-ztglace) / (RTT-ztglace)
 !---zhangli  
     zfice(i) = MIN(MAX(zfice(i),0.0),1.0)
      zfice(i) = zfice(i)**nexpo
      zneb(i) = MAX(rneb(i,k), seuil_neb)
      radliq(i,k) = zoliq(i)/FLOAT(ninter+1)
   ENDIF
   ENDDO
!
   DO n = 1, ninter
   DO i = 1, klon
   IF (rneb(i,k).GT.0.0) THEN
!
!     Water amount to precipitate
!
      if (ptconv(i,k)) then
         zcl(i)=cld_lc_con
         zct(i)=1./cld_tau_con
      else
         zcl(i)=cld_lc_lsc
         zct(i)=1./cld_tau_lsc
      endif
      zchau(i) = zct(i)*dtime/FLOAT(ninter) * zoliq(i) &
           *(1.0-EXP(-(zoliq(i)/zneb(i)/zcl(i))**2)) *(1.-zfice(i))
!
!     Ice amount to precipitate
!
      zrhol(i) = zrho(i) * zoliq(i) / zneb(i)
      if (ptconv(i,k)) then
         fallv = 3.29/2.0 * ((zrhol(i))**0.16) * ffallv_con
      else
         fallv = 3.29/2.0 * ((zrhol(i))**0.16) * ffallv_lsc
      endif
      zfroi(i) = dtime/FLOAT(ninter)/zdz(i)*zoliq(i) *fallv * zfice(i)
!
      ztot(i) = zchau(i) + zfroi(i)
      IF (zneb(i).EQ.seuil_neb) ztot(i) = 0.0
      ztot(i) = MIN(MAX(ztot(i),0.0),zoliq(i))
      zoliq(i) = MAX(zoliq(i)-ztot(i), 0.0)
      radliq(i,k) = radliq(i,k) + zoliq(i)/FLOAT(ninter+1)
   ENDIF
   ENDDO
   ENDDO
!
   DO i = 1, klon
   IF (rneb(i,k).GT.0.0) THEN
      d_ql(i,k) = zoliq(i)
      zrfl(i) = zrfl(i)+ MAX(zcond(i)-zoliq(i),0.0) &
                         * (paprs(i,k)-paprs(i,k+1))/(RG*dtime)
   ENDIF
   IF (zt(i).LT.RTT) THEN
      psfl(i,k)=zrfl(i)
   ELSE
      prfl(i,k)=zrfl(i)
   ENDIF
   ENDDO
!
! Calculate the tendancies of q and t:
!
   DO i = 1, klon
      d_q(i,k) = zq(i) - q(i,k)
      d_t(i,k) = zt(i) - t(i,k)
   ENDDO
!
  enddo  ! End of Loop on vertical
!
! Rain or snow at the surface following the first-layer temperature
!
  DO i = 1, klon
  IF ((t(i,1)+d_t(i,1)) .LT. RTT) THEN
     snow(i) = zrfl(i)
     zlh_solid(i) = RLSTT-RLVTT
  ELSE
     rain(i) = zrfl(i)
     zlh_solid(i) = 0.
  ENDIF
  ENDDO
!
! For energy conservation : when snow is present, the solification
! latent heat is considered.
!
  DO k = 1, klev
  DO i = 1, klon
     zcpair=RCPD*(1.0+RVTMP2*(q(i,k)+d_q(i,k)))
     zmair=(paprs(i,k)-paprs(i,k+1))/RG
     zm_solid = (prfl(i,k)-prfl(i,k+1)+psfl(i,k)-psfl(i,k+1))*dtime
     d_t(i,k) = d_t(i,k) + zlh_solid(i) *zm_solid / (zcpair*zmair)
  ENDDO
  ENDDO
!
  DEALLOCATE (zct)
  DEALLOCATE (zcl)
  DEALLOCATE (zpdf_sig)
  DEALLOCATE (zpdf_k)
  DEALLOCATE (zpdf_delta)
  DEALLOCATE (Zpdf_a)
  DEALLOCATE (zpdf_b)
  DEALLOCATE (zpdf_e1)
  DEALLOCATE (zpdf_e2)
  DEALLOCATE (zqs)
  DEALLOCATE (zdqs)
  DEALLOCATE (zrfl)
  DEALLOCATE (zrfln)
  DEALLOCATE (zoliq)
  DEALLOCATE (zcond)
  DEALLOCATE (zq)
  DEALLOCATE (zqn)
  DEALLOCATE (zt)
  DEALLOCATE (zdz)
  DEALLOCATE (zrho)
  DEALLOCATE (ztot)
  DEALLOCATE (zrhol)
  DEALLOCATE (zchau)
  DEALLOCATE (zfroi)
  DEALLOCATE (zfice)
  DEALLOCATE (zneb)
  DEALLOCATE (zprec_cond)
  DEALLOCATE (zlh_solid)
!
END SUBROUTINE fisrtilp
SUBROUTINE pdfcloud_lsc(klon,klev,pplay,t,q,ratqs, rneb)
    use shr_kind_mod,    only: r8 => shr_kind_r8
IMPLICIT none
!======================================================================
! Programmed by Z.X. Li (LMD/CNRS) date: 2012-12-27
! Object: Large-scale Cloud scheme based on a PDF of water vapour
!======================================================================
! main physical constants
!#include "YOMCST.h"
! thermodynamics constants
!#include "YOETHF.h"
    real(r8), parameter ::   RTT       = 273.16
    real(r8), parameter ::   RMV       = 18.0153
    real(r8), parameter ::   RMD       = 28.9644
    real(r8), parameter ::   RKBOL     = 1.380658e-23
    real(r8), parameter ::   RNAVO     = 6.0221367e23
    real(r8), parameter ::   R         = RNAVO*RKBOL
    real(r8), parameter ::   RD        = 1000.*R/RMD
    real(r8), parameter ::   RV        = 1000.*R/RMV
    real(r8), parameter ::   RESTT     = 611.14
    real(r8), parameter ::   R2ES      = RESTT*RD/RV
    real(r8), parameter ::   R3LES     = 17.269
    real(r8), parameter ::   R3IES     = 21.875
    real(r8), parameter ::   R4LES     = 35.86
    real(r8), parameter ::   R4IES     = 7.66
    real(r8), parameter ::   R5LES     = R3LES*(RTT-R4LES)
    real(r8), parameter ::   R5IES     = R3IES*(RTT-R4IES)
    real(r8), parameter ::   RCPV      = 4.*RV
    real(r8), parameter ::   RCPD      = 3.5*RD
    real(r8), parameter ::   RVTMP2    = RCPV/RCPD - 1
    real(r8), parameter ::   RG        = 9.80665
    real(r8), parameter ::   RETV      = RV/RD -1
    real(r8), parameter ::   RLVTT     = 2.5008e6
    real(r8), parameter ::   RLSTT     = 2.8345e6
!
! Arguments:
!
  INTEGER, INTENT(IN) :: klon, klev ! horizontal and vertical dimensions
  REAL, DIMENSION(klon,klev), INTENT(IN) :: pplay ! mid-layer pressure (Pa)
  REAL, DIMENSION(klon,klev), INTENT(IN) :: t ! temperature (K)
  REAL, DIMENSION(klon,klev), INTENT(IN) :: q ! specific humidity (kg/kg)
  REAL, DIMENSION(klon,klev), INTENT(IN) :: ratqs ! water vapour distribution width
!
  REAL, DIMENSION(klon,klev), INTENT(OUT) :: rneb ! cloud fraction
!
! Some parameters of the programme:
!
  LOGICAL, PARAMETER :: cpartiel=.TRUE. ! partial condensation (opposite of all-or-nothing)
  INTEGER, PARAMETER :: iflag_pdf = 1 ! assumption for water vapour distribution
  REAL, PARAMETER :: seuil_neb=0.001 ! threshold for a cloud to be considered as existent
!
! Local Variables:
!
  REAL :: zdelta, zcor, zcvm5, zdelq
  REAL, ALLOCATABLE, DIMENSION(:) :: zpdf_sig,zpdf_k,zpdf_delta,Zpdf_a,zpdf_b,zpdf_e1,zpdf_e2
  REAL, ALLOCATABLE, DIMENSION(:) :: zqs, zdqs, zcond, zq, zqn, zt
!
  INTEGER :: i, k, n
! changed by zhangli 20130115
!  REAL, EXTERNAL ::  ERF ! error function (is it an intrinsic function ?)
!  changed by zhangli
!
  REAL :: zFOEEW ! saturated water vapour
  REAL :: zFOEDE ! derivative of the saturated water vapour
!
! Allocate temporal arrays:
!
  ALLOCATE (zpdf_sig(klon))
  ALLOCATE (zpdf_k(klon))
  ALLOCATE (zpdf_delta(klon))
  ALLOCATE (Zpdf_a(klon))
  ALLOCATE (zpdf_b(klon))
  ALLOCATE (zpdf_e1(klon))
  ALLOCATE (zpdf_e2(klon))
  ALLOCATE (zqs(klon))
  ALLOCATE (zdqs(klon))
  ALLOCATE (zcond(klon))
  ALLOCATE (zq(klon))
  ALLOCATE (zqn(klon))
  ALLOCATE (zt(klon))
!
! Initialize the outputs
!
  DO k = 1, klev
  DO i = 1, klon
     rneb(i,k) = 0.0
  ENDDO
  ENDDO
!
! Loop on vertical (from top to bottom):
!
  DO k = klev, 1, -1
!
   DO i = 1, klon
      zt(i)=t(i,k)
      zq(i)=q(i,k)
   ENDDO
!
! Calculate Qs (water vapour saturation)
! and L/Cp*dQs/dT (derivative to temperature).
! This linearization of the Qs allows a more accurate calculation of the
! condensation: the relase of latent heat during the condensation can
! increase the temperature and the saturated water vapour, so decrease
! a little the initially-obtained condensation amount
!
   DO i = 1, klon
      zdelta = MAX(0.,SIGN(1.,RTT-zt(i)))
      zcvm5 = R5LES*RLVTT*(1.-zdelta) + R5IES*RLSTT*zdelta
      zcvm5 = zcvm5 /RCPD/(1.0+RVTMP2*zq(i))
      zFOEEW = EXP ( (R3LES*(1.-zdelta)+R3IES*zdelta) * (zt(i)-RTT)        &
                   / (zt(i)-(R4LES*(1.-zdelta)+R4IES*zdelta)) )
      zqs(i) = R2ES*zFOEEW/pplay(i,k)
      zqs(i) = MIN(0.5,zqs(i))
      zcor = 1./(1.-RETV*zqs(i))
      zqs(i) = zqs(i)*zcor
      zFOEDE = zqs(i)*zcor*zcvm5 / (zt(i)-(R4LES*(1.-zdelta)+R4IES*zdelta))**2
      zdqs(i) = zFOEDE
      zdqs(i) = zFOEDE
   ENDDO
!
! Determine the partial condensation and calculate the amount of condensed water
! rneb - cloud cover;
! zqn - total water amount in cloud;
! zcond - averaged condensed water in the grid
!
! Two options are available: The first one is based on a simple uniform
! distribution of water vapour, with ratqs as the distribution width.
! The second option is based on a more compilicated PDF introduced by
! Sandrine Bony.
!
   IF (cpartiel) THEN
!
      IF (iflag_pdf.EQ.0) THEN ! Option of a uniform distribution
         DO i=1,klon
            zdelq = MIN(ratqs(i,k),0.99) * zq(i)
            rneb(i,k) = (zq(i)+zdelq-zqs(i)) / (2.0*zdelq)
            zqn(i) = (zq(i)+zdelq+zqs(i))/2.0
         ENDDO
      ELSE ! Option with new PDF
         DO i=1,klon
            zq(i)=MAX(zq(i),1.e-15)
         ENDDO
         DO i=1,klon
            zpdf_sig(i)=ratqs(i,k)*zq(i)
            zpdf_k(i)=-SQRT(LOG(1.+(zpdf_sig(i)/zq(i))**2))
            zpdf_delta(i)=LOG(zq(i)/zqs(i))
            zpdf_a(i)=zpdf_delta(i)/(zpdf_k(i)*SQRT(2.))
            zpdf_b(i)=zpdf_k(i)/(2.*SQRT(2.))
            zpdf_e1(i)=zpdf_a(i)-zpdf_b(i)
            zpdf_e1(i)=SIGN(MIN(ABS(zpdf_e1(i)),5.),zpdf_e1(i))
            zpdf_e1(i)=1.-ERF(zpdf_e1(i))
            zpdf_e2(i)=zpdf_a(i)+zpdf_b(i)
            zpdf_e2(i)=SIGN(MIN(ABS(zpdf_e2(i)),5.),zpdf_e2(i))
            zpdf_e2(i)=1.-ERF(zpdf_e2(i))
            IF (zpdf_e1(i).LT.1.e-10) THEN
               rneb(i,k)=0.
               zqn(i)=zqs(i)
            ELSE
               rneb(i,k)=0.5*zpdf_e1(i)
               zqn(i)=zq(i)*zpdf_e2(i)/zpdf_e1(i)
            ENDIF
         ENDDO
      ENDIF ! iflag_pdf

      DO i=1,klon
         IF (rneb(i,k) .LE. 0.0) zqn(i) = 0.0
         IF (rneb(i,k) .GE. 1.0) zqn(i) = zq(i)
         rneb(i,k) = MAX(0.0,MIN(1.0,rneb(i,k)))
         zcond(i) = MAX(0.0,zqn(i)-zqs(i))*rneb(i,k)/(1.+zdqs(i))
      ENDDO
   ELSE ! cpartiel
      DO i = 1, klon
         IF (zq(i).GT.zqs(i)) THEN
            rneb(i,k) = 1.0
         ELSE
            rneb(i,k) = 0.0
         ENDIF
         zcond(i) = MAX(0.0,zq(i)-zqs(i))/(1.+zdqs(i))
      ENDDO
   ENDIF ! cpartiel
!
   DO i = 1, klon
      zq(i) = zq(i) - zcond(i)
      zt(i) = zt(i) + zcond(i) * RLVTT/RCPD/(1.0+RVTMP2*zq(i))
   ENDDO
!
!
  enddo ! End of Loop on vertical
!
  DEALLOCATE (zpdf_sig)
  DEALLOCATE (zpdf_k)
  DEALLOCATE (zpdf_delta)
  DEALLOCATE (Zpdf_a)
  DEALLOCATE (zpdf_b)
  DEALLOCATE (zpdf_e1)
  DEALLOCATE (zpdf_e2)
  DEALLOCATE (zqs)
  DEALLOCATE (zdqs)
  DEALLOCATE (zcond)
  DEALLOCATE (zq)
  DEALLOCATE (zqn)
  DEALLOCATE (zt)
!
END SUBROUTINE pdfcloud_lsc
