!+ updates t and q tendencies, precipitation rates does global diagnostics
!+ $Id: cudtdq.F90,v 1.1.1.1 2005/12/22 13:43:11 wangzzh Exp $

SUBROUTINE cudtdq(klp2,klon,klev,klevp1,ktopm2,paphp1,ldcum,pten,ptte,pqte, &
&      pxtec,pmfus,pmfds,pmfuq,pmfdq,pmful,pdmfup, &
&      pdmfdp,plude,pdpmel,prain,prfl,psfl,prsfc, &
&      pssfc,paprc,paprs,ztodt, PLGLAC)

  ! Description:
  !
  ! Updates t and q tendencies, precipitation rates does global diagnostics
  !
  ! Method:
  !
  ! *cudtdq* is called from *cumastr*
  !
  ! Authors:
  !
  ! M. Tiedtke, ECMWF, in July 1986 and December 1989, original source
  ! L. Kornblueh, MPI, May 1998, f90 rewrite
  ! U. Schulzweida, MPI, May 1998, f90 rewrite
  !
  ! for more details see file AUTHORS
  !

  USE mo_constants, ONLY: rhoh2o, &! density of liquid water
                          tmelt,  &! temperature of fusion of ice
                          alv,    &! latent heat for vaporisation
                          g,      &! gravity acceleration
                          alf,    &! latent heat for fusion
                          als,    &! latent heat for sublimation
                          rcpd,   &! rcpd=1./cpd
                          R2ES     ,R3LES    ,R3IES    ,R4LES   ,   &
                 R4IES    ,R5LES    ,R5IES   ,R5ALVCP  ,R5ALSCP  ,           &   
                 RALVDCP  ,RALSDCP  ,RTWAT   ,RTICE,  RLVTT   ,RLSTT   ,RTT

  IMPLICIT NONE

  !  Scalar arguments with intent(In):
  INTEGER, INTENT (IN) :: klev, klevp1, klon, klp2, ktopm2

  !  Array arguments with intent(In):
  REAL, INTENT (IN) :: paphp1(klp2,klevp1), pdmfdp(klp2,klev), &
&      pdmfup(klp2,klev), pdpmel(klp2,klev), plude(klp2,klev), &
&      pmfdq(klp2,klev), pmfds(klp2,klev), &
&      pmful(klp2,klev), pmfuq(klp2,klev), pmfus(klp2,klev), &
&       prain(klp2), prfl(klp2), &
&      psfl(klp2), pten(klp2,klev),ztodt
  LOGICAL, INTENT (IN) :: ldcum(klp2)

  REAL :: PLGLAC(klp2,klev)

  !  Scalar arguments with intent(InOut):

  !  Array arguments with intent(InOut):
  REAL, INTENT (INOUT) :: paprc(klp2), paprs(klp2), pqte(klp2,klev), &
&                         prsfc(klp2), pssfc(klp2), ptte(klp2,klev), &
&                         pxtec(klp2,klev)

  !  Local scalars: 
  REAL :: zalv, zdiagt, zdiagw, zdqdt, zdtdt, zdxtdt
  INTEGER :: jk, jl, jt
  LOGICAL :: llo1

#include "fcttre.h"          !sxl add

  !  Intrinsic functions 
  INTRINSIC MERGE


  !  Executable statements 

!-- 1. Specify parameters

! zdiagt = 0.5*ztodt
  zdiagt = ztodt
  zdiagw = zdiagt/rhoh2o

!sxl add init--------------------------
  do jk= 1, klev
    do jl= 1,klon
      pxtec(jl,jk) = 0.0
    end do
  end do
!sxl add end---------------------------

!-- 2. Incrementation of t and q tendencies
!  write(6,*) '****** ktopm2= ', ktopm2
  DO jk = ktopm2, klev

    IF (jk<klev) THEN
      DO jl = 1, klon
        IF (ldcum(jl)) THEN
          llo1 = (pten(jl,jk)-tmelt) > 0.
!sxl          zalv = MERGE(alv,als,llo1)
          zalv=FOELHM(pten(jl,jk))
          zdtdt = (g/(paphp1(jl,jk+1)-paphp1(jl,jk)))*rcpd*(pmfus(jl,jk+1)- &
&              pmfus(jl,jk)+pmfds(jl,jk+1)-pmfds(jl,jk)-alf*pdpmel(jl,jk)   &
&              +alf*PLGLAC(jl,jk)-zalv*(pmful(jl,jk+1)-pmful(jl,jk)-plude(jl, &
&              jk)-(pdmfup(jl,jk)+pdmfdp(jl,jk))))
          ptte(jl,jk) = ptte(jl,jk) + zdtdt
          zdqdt = (g/(paphp1(jl,jk+1)-paphp1(jl,jk)))*(pmfuq(jl,jk+1)-pmfuq( &
&              jl,jk)+pmfdq(jl,jk+1)-pmfdq(jl,jk)+pmful(jl,jk+1)-pmful(jl,jk)- &
&              plude(jl,jk)-(pdmfup(jl,jk)+pdmfdp(jl,jk)))
          pqte(jl,jk) = pqte(jl,jk) + zdqdt
          pxtec(jl,jk) = (g/(paphp1(jl,jk+1)-paphp1(jl,jk)))*plude(jl,jk)
        END IF
      END DO

    ELSE
      DO jl = 1, klon
        IF (ldcum(jl)) THEN
          llo1 = (pten(jl,jk)-tmelt) > 0.
!sxl          zalv = MERGE(alv,als,llo1)
          zalv=FOELHM(pten(jl,jk))
          zdtdt = -(g/(paphp1(jl,jk+1)-paphp1(jl,jk)))*rcpd*(pmfus(jl,jk)+ &
&              pmfds(jl,jk)+alf*pdpmel(jl,jk)-zalv*(pmful(jl,jk)+pdmfup(jl, &
&              jk)+pdmfdp(jl,jk)+plude(jl,jk)))
          ptte(jl,jk) = ptte(jl,jk) + zdtdt
          zdqdt = -(g/(paphp1(jl,jk+1)-paphp1(jl,jk)))*(pmfuq(jl,jk)+pmfdq(jl &
&              ,jk)+plude(jl,jk)+(pmful(jl,jk)+pdmfup(jl,jk)+pdmfdp(jl,jk)))
          pqte(jl,jk) = pqte(jl,jk) + zdqdt
          pxtec(jl,jk) = (g/(paphp1(jl,jk+1)-paphp1(jl,jk)))*plude(jl,jk)
        END IF
      END DO

    END IF
  END DO
!-- 3. Update surface fields and do global budgets
  DO jl = 1, klon
    prsfc(jl) = prfl(jl)
    pssfc(jl) = psfl(jl)
    paprc(jl) = paprc(jl) + zdiagw*(prfl(jl)+psfl(jl))
    paprs(jl) = paprs(jl) + zdiagw*psfl(jl)
  END DO

  RETURN
END SUBROUTINE cudtdq
