!+ does the final calculation of convective fluxes

SUBROUTINE cuflx(klp2,klon,klev,klevp1,pqen,pqsen,ptenh,pqenh,&
&   paphp1,pgeoh,kcbot,kctop,kdtop,ktype,lddraf,ldcum,pmfu,   &
                 pmfd,pmfus,pmfds,pmfuq,pmfdq,pmful,plude,pdmfup,pdmfdp,prfl,prain,pten, &
                 psfl,pdpmel,ktopm2,ztodt,papp1,PLGLAC)

  ! Description:
  !
  ! Does the final calculation of convective fluxes.
  !
  ! Method:
  !
  ! This routine does the final calculation of convective
  ! fluxes in the cloud layer and in the subcloud layer
  !
  ! This routine is called from *cumastr*.
  !
  ! Authors:
  !
  ! M. Tiedtke, ECMWF, July 1989, original source
  ! M. Tiedtke, ECMWF, December 1989, changed
  ! L. Kornblueh, MPI, May 1998, f90 rewrite
  ! U. Schulzweida, MPI, May 1998, f90 rewrite
  ! 
  ! for more details see file AUTHORS
  !

  USE mo_constants,     ONLY: cpd,   &! specific heat at constant pressure
                              alf,   &! latent heat for vaporisation
                              g,     &! gravity acceleration
                              tmelt, &! temperature of fusion of ice
                              R2ES   ,R3LES  ,R3IES   ,R4LES   ,                &
                              R4IES  ,R5LES  ,R5IES   ,R5ALVCP  ,R5ALSCP  ,     &   
                        RALVDCP  ,RALSDCP  ,RTWAT  ,RTICE,  RLVTT  ,RLSTT  ,RTT

  USE time_manager,     ONLY: is_first_step,         &! initial run
                              is_first_restart_step, &! restart run
                              get_step_size           ! step size
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid,             only: plev, plevp !Needed for hypm passed to vd_inti

  IMPLICIT NONE

#include <comhyb.h>

  !  Scalar arguments with intent(In):
  INTEGER, INTENT (IN) :: klev, klevp1, klon, klp2

  !  Array arguments with intent(In):
  REAL, INTENT (IN) :: paphp1(klp2,klevp1), pgeoh(klp2,klev), pqen(klp2,klev), &
                       pqenh(klp2,klev),  pten(klp2,klev),    &
                       ptenh(klp2,klev),ztodt
  INTEGER, INTENT (IN) :: kcbot(klp2), kctop(klp2), kdtop(klp2)
  LOGICAL, INTENT (IN) :: ldcum(klp2)

  !  Array arguments with intent(InOut):
  REAL, INTENT (INOUT) :: pdmfdp(klp2,klev), pdmfup(klp2,klev), pdpmel(klp2,klev), &
                          plude(klp2,klev), pmfd(klp2,klev), pmfdq(klp2,klev),     &
                          pmfds(klp2,klev), &
                          pmfu(klp2,klev), pmful(klp2,klev), pmfuq(klp2,klev),     &
                          pmfus(klp2,klev), &
!pmfuxt(klon,klev,ktrac),               &
                          prain(klp2), prfl(klp2), psfl(klp2)
  INTEGER, INTENT (INOUT) :: ktype(klp2)
  LOGICAL, INTENT (INOUT) :: lddraf(klp2)

  !  Scalar arguments with intent(Out):
  INTEGER, INTENT (OUT) :: ktopm2

  !  Local scalars: 
  REAL :: zcons1, zcons2, zcucov, zdpevap, zdrfl, zfac, zrfl, zrfln, zrmin, &
          zrnew, zrsum, zsnmlt, ztmelp2, ztmst, zzp  ! zrsum new for AMIP2
  INTEGER :: ikb, jk, jl, jt, ik
!sxl add---------------------------------------
  REAL :: ZTAUMEL, ZCONS1A, ZCPECONS, ZALFAW, ZDRFL1, ZDENOM, ZPDR, ZPDS
  REAL :: PLGLAC(klp2,klev) , papp1(klp2,klev),  pqsen(klp2,klev)
  REAL :: PMFLXR(klp2,klevp1),  PMFLXS(klp2,klevp1)

!sxl add---------------------------------------
  !  Local arrays: 
  REAL :: zpsubcl(klp2)
  REAL :: cevapcu(klev)

#include "fcttre.h"          !sxl add

  !  Intrinsic functions 
  INTRINSIC MAX, MERGE, MIN, SQRT


  !  Executable statements

  !  eta level for CAM
  cevapcu(:)=1.93E-6*261.*SQRT(1.E3/(38.3*0.293)*SQRT(hyam(:)+hybm(:)))*0.5/g 

  !  Specify constants
  ztmst = ztodt
!sxl  IF (is_first_step()) ztmst = 0.5*ztodt
  zcons1 = cpd/(alf*g*ztmst)
  zcons2 = 1./(g*ztmst)
  zcucov = 0.05
  ztmelp2 = tmelt + 2.
!sxl add---------------------
  ZTAUMEL=5.*3.6E3
  ZCONS1A= cpd/(alf*g*ZTAUMEL) 
  ZCPECONS=5.44E-4/g
  DO jk = 1, klev+1
    DO jl = 1, klon
       PMFLXR(jl,jk)=0.0
       PMFLXS(jl,jk)=0.0
    END DO
  END DO 

!sxl add---------------------

!-- 1. Determine final convective fluxes

!  itop = klev
  DO jl = 1, klon
!    itop = MIN(itop,kctop(jl))
    IF ( .NOT. ldcum(jl) .OR. kdtop(jl)<kctop(jl)) lddraf(jl) = .FALSE.
    IF ( .NOT. ldcum(jl)) ktype(jl) = 0
  END DO
!  ktopm2 = itop - 2
  ktopm2 = 2
  DO jk = ktopm2, klev
!DIR$ IVDEP
!OCL NOVREC
    DO jl = 1, klon
      IF (ldcum(jl) .AND. jk>=kctop(jl)-1) THEN
        pmfus(jl,jk) = pmfus(jl,jk) - pmfu(jl,jk)*(cpd*ptenh(jl,jk)+pgeoh(jl,jk))
        pmfuq(jl,jk) = pmfuq(jl,jk) - pmfu(jl,jk)*pqenh(jl,jk)
        PLGLAC(jl,jk) = pmfu(jl,jk)*PLGLAC(jl,jk)
        IF (lddraf(jl) .AND. jk>=kdtop(jl)) THEN
          pmfds(jl,jk) = pmfds(jl,jk) - pmfd(jl,jk)*(cpd*ptenh(jl,jk)+pgeoh(jl,jk))
          pmfdq(jl,jk) = pmfdq(jl,jk) - pmfd(jl,jk)*pqenh(jl,jk)
        ELSE
          pmfd(jl,jk) = 0.
          pmfds(jl,jk) = 0.
          pmfdq(jl,jk) = 0.
          pdmfdp(jl,jk-1) = 0.
        END IF
      ELSE
        pmfu(jl,jk) = 0.
        pmfd(jl,jk) = 0.
        pmfus(jl,jk) = 0.
        pmfds(jl,jk) = 0.
        pmfuq(jl,jk) = 0.
        pmfdq(jl,jk) = 0.
        pmful(jl,jk) = 0.
        PLGLAC(jl,jk)= 0.
        pdmfup(jl,jk-1) = 0.
        pdmfdp(jl,jk-1) = 0.
        plude(jl,jk-1) = 0.
      END IF
    END DO

  END DO

!sxl  DO jk = ktopm2, klev
!DIR$ IVDEP
!OCL NOVREC
!sxl    DO jl = 1, klon
!sxl      IF (ldcum(jl) .AND. jk>kcbot(jl)) THEN
!sxl        ikb = kcbot(jl)
!sxl        zzp = ((paphp1(jl,klevp1)-paphp1(jl,jk))/(paphp1(jl,klevp1)-paphp1(jl,ikb)))
!sxl        zzp = MERGE(zzp**2,zzp,ktype(jl)==3)
!sxl        pmfu(jl,jk) = pmfu(jl,ikb)*zzp
!sxl        pmfus(jl,jk) = pmfus(jl,ikb)*zzp
!sxl        pmfuq(jl,jk) = pmfuq(jl,ikb)*zzp
!sxl        pmful(jl,jk) = pmful(jl,ikb)*zzp
!sxl      END IF
!sxl    END DO

!sxl  END DO
!sxl add-----------------------------------------------------
      DO jl=1,klon
        IF(ldcum(jl)) THEN
          ikb=kcbot(jl)
          ik=ikb+1
          zzp=((paphp1(jl,klev+1)-paphp1(jl,ik))/              &
           (paphp1(jl,klev+1)-paphp1(jl,ikb)))
          IF(ktype(jl).EQ.3) THEN
            zzp=zzp**2
          ENDIF
          pmfu(jl,ik)=pmfu(jl,ikb)*zzp
          pmfus(jl,ik)=(pmfus(jl,ikb)-                         &  
            FOELHM(ptenh(jl,ikb))*pmful(jl,ikb))*zzp
          pmfuq(jl,ik)=(pmfuq(jl,ikb)+pmful(jl,ikb))*zzp
          pmful(jl,ik)=0.
        ENDIF
      ENDDO
      DO jk=ktopm2,klev
!DIR$ IVDEP
!OCL NOVREC
        DO jl=1,klon
          IF(ldcum(jl).AND. jk.GT.kcbot(jl)+1) THEN
            ikb=kcbot(jl)+1
            zzp=((paphp1(jl,klev+1)-paphp1(jl,jk))/           &
             (paphp1(jl,klev+1)-paphp1(jl,ikb)))
            IF(ktype(jl).EQ.3) THEN
              zzp=zzp**2
            ENDIF
            pmfu(jl,jk)=pmfu(jl,ikb)*zzp
            pmfus(jl,jk)=pmfus(jl,ikb)*zzp
            pmfuq(jl,jk)=pmfuq(jl,ikb)*zzp
            pmful(jl,jk)=0.
            IF(lddraf(jl).AND.pmfd(jl,ikb+1).EQ.0.) THEN
              pmfd(jl,jk)=pmfd(jl,ikb)*zzp
              pmfds(jl,jk)=pmfds(jl,ikb)*zzp
              pmfdq(jl,jk)=pmfdq(jl,ikb)*zzp
            ENDIF 
          ENDIF
        ENDDO
      ENDDO

!sxl add end-------------------------------------------------

!-- 2. Calculate rain/snow fall rates
!      Calculate melting of snow
!      Calculate evaporation of precip

  DO jl = 1, klon
    prfl(jl) = 0.
    psfl(jl) = 0.
    prain(jl) = 0.
  END DO
!sxl  DO jk = ktopm2, klev
!sxl    DO jl = 1, klon
!sxl      IF (ldcum(jl)) THEN
!sxl        prain(jl) = prain(jl) + pdmfup(jl,jk)
!sxl        IF (pten(jl,jk)>tmelt) THEN
!sxl          prfl(jl) = prfl(jl) + pdmfup(jl,jk) + pdmfdp(jl,jk)
!sxl          IF (psfl(jl)>0. .AND. pten(jl,jk)>ztmelp2) THEN
!sxl            zfac = zcons1*(paphp1(jl,jk+1)-paphp1(jl,jk))
!sxl            zsnmlt = MIN(psfl(jl),zfac*(pten(jl,jk)-ztmelp2))
!sxl            pdpmel(jl,jk) = zsnmlt
!sxl            psfl(jl) = psfl(jl) - zsnmlt
!sxl            prfl(jl) = prfl(jl) + zsnmlt
!sxl          END IF
!sxl        ELSE
!sxl          psfl(jl) = psfl(jl) + pdmfup(jl,jk) + pdmfdp(jl,jk)
!sxl        END IF
!sxl      END IF
!sxl    END DO
!sxl  END DO
!sxl  DO jl = 1, klon
!sxl    prfl(jl) = MAX(prfl(jl),0.)
!sxl    psfl(jl) = MAX(psfl(jl),0.)
!sxl    zpsubcl(jl) = prfl(jl) + psfl(jl)
!sxl  END DO
!sxl  DO jk = ktopm2, klev
!sxl    DO jl = 1, klon
!sxl      IF (ldcum(jl) .AND. jk>=kcbot(jl) .AND. zpsubcl(jl)>1.E-20) THEN
!sxl        zrfl = zpsubcl(jl)
!sxl        zrnew = (MAX(0.,SQRT(zrfl/zcucov)-cevapcu(jk)*(paphp1(jl,jk+1)- &
!sxl                paphp1(jl,jk))*MAX(0.,pqsen(jl,jk)-pqen(jl,jk))))**2*zcucov
!sxl        zrmin = zrfl - zcucov*MAX(0.,0.8*pqsen(jl,jk)-pqen(jl,jk))*zcons2* &
!sxl                (paphp1(jl,jk+1)-paphp1(jl,jk))
!sxl        zrnew = MAX(zrnew,zrmin)
!sxl        zrfln = MAX(zrnew,0.)
!sxl        zdrfl = MIN(0.,zrfln-zrfl)
!sxl        pdmfup(jl,jk) = pdmfup(jl,jk) + zdrfl
!sxl        zpsubcl(jl) = zrfln
!sxl      END IF
!sxl    END DO
!sxl  END DO
!sxl add--------------------------------------------------
      DO jk=ktopm2,klev
        DO jl=1,klon
          IF(ldcum(jl)) THEN
            prain(jl)=prain(jl)+pdmfup(jl,jk)
            IF(PMFLXS(jl,jk).GT.0..AND.pten(jl,jk).GT.RTT) THEN
              zcons1=ZCONS1A*(1.+0.5*(pten(jl,jk)-RTT))
              zfac=zcons1*(paphp1(jl,jk+1)-paphp1(jl,jk))
              zsnmlt=MIN(PMFLXS(jl,jk),zfac*(pten(jl,jk)-RTT))
              pdpmel(jl,jk)=zsnmlt                         
              pqsen(jl,jk)=FOEEWM(pten(jl,jk)-zsnmlt/zfac)/papp1(jl,jk)
            ENDIF
            ZALFAW=FOEALFA(pten(jl,jk))
            PMFLXR(jl,jk+1)=PMFLXR(jl,jk)+ZALFAW*                    &
              (pdmfup(jl,jk)+pdmfdp(jl,jk))+pdpmel(jl,jk)
            PMFLXS(jl,jk+1)=PMFLXS(jl,jk)+(1.-ZALFAW)*               &
              (pdmfup(jl,jk)+pdmfdp(jl,jk))-pdpmel(jl,jk)
            IF(PMFLXR(jl,jk+1)+PMFLXS(jl,jk+1).LT.0.0) THEN
              pdmfdp(jl,jk)=-(PMFLXR(jl,jk)+PMFLXS(jl,jk)+pdmfup(jl,jk))
              PMFLXR(jl,jk+1)=0.0
              PMFLXS(jl,jk+1)=0.0
              pdpmel(jl,jk)  =0.0
            ENDIF   
          ENDIF
        ENDDO
      ENDDO
      DO jk=ktopm2,klev
        DO jl=1,klon
          IF(ldcum(jl).AND.jk.GE.kcbot(jl).AND.               &
           pqen(jl,jk).LT.0.8*pqsen(jl,jk)) THEN 
            zrfl=PMFLXR(jl,jk)+PMFLXS(jl,jk)
            IF(zrfl.GT.1.E-20) THEN
              ZDRFL1=ZCPECONS*MAX(0.,pqsen(jl,jk)-pqen(jl,jk))*zcucov*         &  
               (SQRT(paphp1(jl,jk)/paphp1(jl,klev+1))/5.09E-3*                 &
               zrfl/zcucov)**0.5777*                                           &
               (paphp1(jl,jk+1)-paphp1(jl,jk))
              zrnew=zrfl-ZDRFL1
              zrmin=zrfl-zcucov*MAX(0.,0.8*pqsen(jl,jk)-pqen(jl,jk))           &
               *zcons2*(paphp1(jl,jk+1)-paphp1(jl,jk))
              zrnew=MAX(zrnew,zrmin)
              zrfln=MAX(zrnew,0.)
              zdrfl=MIN(0.,zrfln-zrfl)
              ZDENOM=1./MAX(1.E-20,PMFLXR(jl,jk)+PMFLXS(jl,jk))
              ZALFAW=FOEALFA(pten(jl,jk))
              ZPDR=ZALFAW*pdmfdp(jl,jk)
              ZPDS=(1.-ZALFAW)*pdmfdp(jl,jk)
              PMFLXR(jl,jk+1)=PMFLXR(jl,jk)+ZPDR                              & 
               +pdpmel(jl,jk)+zdrfl*PMFLXR(jl,jk)*ZDENOM
              PMFLXS(jl,jk+1)=PMFLXS(jl,jk)+ZPDS                              &
               -pdpmel(jl,jk)+zdrfl*PMFLXS(jl,jk)*ZDENOM
              pdmfup(jl,jk)=pdmfup(jl,jk)+zdrfl
            ELSE
              PMFLXR(jl,jk+1)=0.0
              PMFLXS(jl,jk+1)=0.0 
              pdmfdp(jl,jk)=0.0
              pdpmel(jl,jk)=0.0
            ENDIF   
          ENDIF
        ENDDO
      ENDDO

      DO jl = 1, klon
         prfl(jl) = PMFLXR(jl,klev+1) 
         psfl(jl) = PMFLXS(jl,klev+1)
      END DO
!sxl add end----------------------------------------------
! ------ CORRECTION OF PRECIPITATION EVAPORATION ERROR (AMIP2) --------
!                (Found by M. Werner)
!sxl    DO jl = 1, klon
!sxl      zrsum    = prfl(jl)+psfl(jl)
!sxl      zdpevap  = zpsubcl(jl) - zrsum
!sxl      prfl(jl) = prfl(jl) + zdpevap*prfl(jl)*(1./MAX(1.E-20,zrsum))
!sxl      psfl(jl) = psfl(jl) + zdpevap*psfl(jl)*(1./MAX(1.E-20,zrsum))
!sxl    END DO

  RETURN
END SUBROUTINE cuflx
