#include <misc.h>
#include <params.h>

subroutine dadadj_new (lchnk   ,ncol    , &
                       pmid    ,pint    ,pdel    ,t     , q)
!----------------------------------------------------------------------- 
! 
! Purpose: dry adiabatic adjustment
! 
! Author: wu tongwen 
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid      , only: pver, pverp, pcols
   implicit none
!
! Arguments
!
   integer, intent(in) :: lchnk               ! chunk identifier
   integer, intent(in) :: ncol                ! number of atmospheric columns

   real(r8), intent(in) :: pmid(pcols,pver)   ! pressure at model levels (Pa)
   real(r8), intent(in) :: pint(pcols,pverp)  ! pressure at model interfaces (Pa)
   real(r8), intent(in) :: pdel(pcols,pver)   ! vertical delta-p (Pa)
!                    
! Input/output arguments
!
   real(r8), intent(inout) :: t(pcols,pver)      ! temperature (K)
   real(r8), intent(inout) :: q(pcols,pver)      ! specific humidity
!
!---------------------------Local workspace-----------------------------
!
   integer i, k
   integer kb0, ke0, kb, ke
   integer ks, ka, kk
   integer N

   real SS
   real GR
   real CGAMA
   real CPR
   real tmean
   real ng
   real nts

   real tz  (pver)
   real plog(pver)
   real g   (pver)
   real pmid0(pver)
   real pint0(pverp)
   real pdel0(pver)
   real t0(pver)
   real q0(pver)

   integer niter    ! number of iterations
   parameter (niter = 15)

   DATA CGAMA/0.0/,GR/29.2857/,CPR/0.2857/     
   SS=GR*CGAMA+CPR                                                   

!---------------------------------------------	 
do i=1,ncol
   DO K=1,pver                                                   
	  pmid0(k) = pmid(i,k)*0.01 
	  pdel0(k) = pdel(i,k)*0.01
	  t0(k)    = t(i,k)
	  q0(k)    = q(i,k)
   enddo
   do k=1, pverp
      pint0(k) = pint(i,k)*0.01   ! Pa --> hPa
   enddo
!!
   DO K=1,pver                                                   
      TZ(K)= t0(i)
      plog(k) = alog(pmid0(k))   
   enddo
   DO K=1, pver-1                                                   
      G(k) = (TZ(K+1)-TZ(K))/(PLOG(K+1)-PLOG(K))
   enddo                      
!
!  SEARCH THE UNSTABLE LAYERS FROM BOTTOM TO TOP.                    
!
   N= 0

   KB0=0                                                             
   KE0=0                                                             
 1 KS=2                                                              
 7 DO K=KS,pver                                                    
     KA=pver-K+1    !   KA RANGES FROM pver-1 TO 1                                        
     TMEAN=(TZ(KA+1)+TZ(KA))/2.0                                       
     NG=G(KA)                                                      
     NTS=TMEAN*SS
     IF( (NG-NTS) .gt. 0.0 ) then
!  
!  AN UNSTABLE LAYER(KLV) HAS BEEN FOUND                             
!  
       KE=KA+1                                                           
       KB=KA                                                             
!  
!  KE IS THE BOTTOM LAYER, KB IS THE TOP LAYER FOR A UNSTABLE KLV    
!  
     3 IF(KB.EQ.1) GO TO 4                                               
!  
!  CHECK TO SEE IF MULTIPLE LAYER ADJUSTMENT IS NEEDED               
!  
       TMEAN=(TZ(KB)+TZ(KB-1))/2.0                                       
       NG=G(KB-1)                                                    
       NTS=TMEAN*SS 
       IF(NG.LE.NTS) GO TO 4                                             
       KB=KB-1                                                           
       GO TO 3 
     ENDIF
   ENDDO
!
!  NO UNSTABLE LAYER                                                 
!
   goto 80
!
! CHECK WHETHER READJUST THE SAME LAYER TO PREVENT DEAD-LOOP          
!
 4 IF(KE.EQ.KE0.AND.KB.EQ.KB0) then
      KS=pver-KB+2                                                      
      GO TO 7 
   ELSE
      KK=KE-KB                                                          
!
      CALL TADJST(G, TZ, q0, pmid0, plog, pdel0, KE, KB, KK )     
!
      KB0=KB                                                            
      KE0=KE                                                            
      DO K=KB,KE                                                    
         T0(k) =TZ(K)  
      ENDDO

      N = N +1
	  if(N.gt.niter) goto 80 
!
!   BACK TO VERY BEGINNING TO TRY AGAIN                               
!
      KS=pver-KE+1                                                      
      IF( (KS-2).lt.0) then
         goto 1
      ELSE
         goto 7
      ENDIF
    ENDIF
80  t(i,k) = t0(k)
    q(i,k) = q0(k)
enddo
return
end subroutine dadadj_new

SUBROUTINE TADJST(GAMA,T,Q,PLA,PLOG,pdel0,KBTM,KTOP,KTL)    
!---------------------------------------------------------------------- 
! purpose: ADJUST UNSTABLE LAYER-TEMPERATURE (ACTIVE ADJUSTMENT)      
!    NOTE: THE CONSERVATION CONDITION ARE AS FOLLOWING:                
!                 LMETHD=1 THETA-MEAN CONSERVED,                        
!                 LMETHD=2 T-MEAN CONSERVED                             
!          *INPUT:GAMA: D(T)/D(LN(P))(1-D)                              
!                    T: TEMPERATURE(1-D)                                
!                    P: PRESSURE OF MEAN SIGMA LAYER(1-D)               
!                 KBTM: VERTICAL INDEX OF BOTTOM UNSTABL LAYER          
!                 KTOP: VERTICAL INDEX OF TOP UNSTABL LAYER             
!                  KTL: DIMENSION OF T AND P                            
!                  I,J: HORIZONTAL POSITION INDEXES                     
!          OUTPUT:GAMA: ADJUSTED GAMA AFTER CALLING                     
!                    T: ADJUSTED T AFTER CALLING                        
!          *GAUSS,UVQADJ                                                
!----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid      , only: pver, pverp, pcols

   implicit none
!
! Arguments
!
   integer KTL
   integer KBTM
   integer KTOP

   real pla(pver)   ! pressure at model levels
   real plog(pver)  ! log(pla)
   real pdel0(pver)  
!                    
! Input/output arguments
!
   real t   (pver)      ! temperature (K)
   real q   (pver)
   real gama(pver)      ! 
!
!---------------------------Local workspace-----------------------------
!
   integer k, ki
   integer na, na2
   integer LMETHD

   real ss, ss2
   real exner(pver)
   real theta(pver)
   real x(pver)
   real a(pver)
   real b(pver)
   real c(pver)
   real d(pver)
   real dt(pver)
   real p0, cpr, cgama, gr
   DATA P0/1000.0/,CPR/0.2857/,CGAMA/-5E-4/,GR/29.2857/              
   data LMETHD/2/

   logical LME2

   LME2=LMETHD.EQ.2                                                  
!----------------------------------------------------------------------
!
!  EXNER=(P/P0)**(R/CP),DT:D(THETA) TO BE ADJUSTED                   
!
      SS=GR*CGAMA+CPR                                                   
      SS2=SS/2.0                                                        
      NA=KTL+1                                                          
      DO K=1,NA                                                      
         KI=KTOP+K-1                                                       
         EXNER(K)=(PLA(KI)/P0)**CPR                                        
         THETA(K)=T(KI)/EXNER(K)                                           
	  ENDDO
!
!   FILL COEF.VECTORS FOR CALLING SUB. GAUSS                          
!
      DO K=1,KTL                                                     
         KI=KTOP+K-1                                                       
         X(K)=SS2*(PLOG(KI+1)-PLOG(KI))                                    
!
! T--MEAN CONSERVED                                                   
!
         A(K)=-(1+X(K))                                                    
         B(K)=(1-X(K))                                                     
         IF( .not. LME2) then
!		 
! THETA--MEAN CONSERVED                                               
!
              A(K)=A(K)*EXNER(K)                                                
              B(K)=B(K)*EXNER(K+1)                                              
         ENDIF
      ENDDO
      DO K=1,NA                                                      
         KI=KTOP+K-1                                                       
         C(K)=pdel0(KI)                                        
!
!  IF(LME2) C(K)=C(K)/EXNER(K)
!
      ENDDO
!
!   CALCULATE ELEMENTS OF X COLUMN VECTOR                             
!
      DO K=1,KTL                                                     
         KI=KTOP+K-1                                                       
         X(K)=(X(K)+1.0)*T(KI)+(X(K)-1.0)*T(KI+1)                          
      ENDDO                                                          
      X(NA)=0.0                                                         
!
!   GET D(THETA) IN X AFTER CALLING SUB. GAUSS                        
!
      CALL GAUSS_DRY(A,B,C,D,X,NA)                                          
!
!   ADJUST U AND V BY CALLING SUB. UVQADJ                             
!
      DO K=1,NA                                                      
         KI=KTOP+K-1                                                       
         IF(LME2) A(K)=T(KI)                                               
         IF(.NOT.LME2) A(K)=THETA(K)
	  ENDDO                                      
!
! ADJUST THE MOISTURE0
!
      CALL UVQADJ(X, Q, NA, KTOP, A, pdel0, 1.0, LME2)         
!
!   CORRECT TEMPERATURE                                               
!
      DO K=1,NA                                                      
         KI=KTOP+K-1                                                       
         IF(LME2) THEN
            T(KI)=T(KI)+X(K)                                                  
		 ELSE
            DT(K)=X(K)*EXNER(K)                                               
            T(KI)=T(KI)+DT(K)                                                 
         ENDIF
      ENDDO
      NA2=NA+1                                                          
      DO K=1,NA2                                                     
         KI=KTOP+K-2                                                       
         IF( .NOT. (KI.LE.0.OR.KI.GE.pver) ) THEN
              GAMA(KI)=(T(KI+1)-T(KI))/(PLOG(KI+1)-PLOG(KI))                    
		 ENDIF
      ENDDO
      RETURN                                                            
end subroutine TADJST

SUBROUTINE UVQADJ(DA, P, NA, KTOP, A, pdel0, ALFA, LG)      
!-----------------------------------------------------------------------
! purpose:  *ADJUST THE PASSIVE QUANTITIES (U,V,Q) IN DRY ADJUSTMENT    
!           *CALL UVQADJ(DA,P,NLO,NLA,KLX,NA,KTOP,A,I,J,PS,ALFA,LG)     
!           *INPUT:         DA: ACTIVE ADJUST PART OF A                 
!                            P: A QUANTITY TO BE PASSIVE ADJUSTED       
!                  NLO,NLA,KLX: DIMENSION OF A                          
!                           NA: NUMBER OF ADJUSTED LAYERS               
!                         KTOP: VERTICAL INDEX OF TOP ADJUSTED LAYER    
!                         ALFA: ADJUST-COEFFICIENT FOR STAGGERED GRIDS  
!                           LG: ADJUST SCHEME SELECTOR                  
!            *OUTPUT:        P: ADJUSTED QUANTITY AFTER CALLING         
!     ................................................................. 
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid      , only: pver, pverp, pcols

   implicit none
!
! Arguments
!
   integer NA
   integer KTOP

   real DA(pver)
   real A(pver)
   real ALFA
   real pdel0(pver)  
 
   logical LG
!                    
! Input/output arguments
!
   real P(pver)
!
!---------------------------Local workspace-----------------------------
!
   integer k, ki

   real DEL(pver)
   real dp (pver)
   real DB(pver)
   real aa
   real x
   real pp
!-------------------------------------------------------------------
   AA=0.0                                                            
   IF(.NOT.LG) then
      PP=0.0                                                            
      DO K=1,NA                                                     
         KI=KTOP+K-1      
         DP(K)=pdel0(KI)
         AA=AA+DP(K)*P(KI)                                             
         PP=PP+DP(K)
	  enddo                                                       
      AA=AA/PP                                                          
      DO K=1,NA                                                     
        KI=KTOP+K-1                                                       
        P(KI)=AA 
      enddo
   ELSE
      DEL(1)=0.0                                                        
      DEL(NA+1)=0.0                                                     
      DO K=1,NA                                                     
         KI=KTOP+K-1                                                       
         DP(K)= pdel0(KI)
         IF(K.ne.NA) then
	        AA=AA+DP(K)*DA(K)       
            DEL(K+1)=AA/(A(K+1)-A(K))                                         
         endif
	  enddo
      DO K=1,NA                                                     
         KI=KTOP+K-1                                                       
         X=0.0                                                             
         IF(DEL(K).NE.0) X=X+DEL(K)*(P(KI-1)-P(KI))                
         IF(DEL(K+1).NE.0) X=X+DEL(K+1)*(P(KI+1)-P(KI))            
         DB(K)=X/DP(K)*ALFA                                                
      ENDDO
      DO  K=1,NA                                                     
         KI=KTOP+K-1                                                       
         P(KI)=P(KI)+DB(K)                                         
      enddo
    ENDIF
    RETURN                                                            
END SUBROUTINE UVQADJ      

SUBROUTINE GAUSS_DRY(A,B,C,D,Y,LM)                                    
!----------------------------------------------------------------------
! purpose: *GAUSS ELIMINATION METHOD TO SOLVE LINEAR EQNS IN TADJST     
!                 FOR DRY ADJUSTMENT                                    
!          *CALL GAUSS(A,B,C,Y,LM)                                      
!          *INPUT: A: MAIN DIAGONAL ARRAY                               
!                  B: UPPER OFF-DIAGONAL COEF.ARRAY                     
!                  C: BOTTOM ROW COEF ARRAY                             
!                  Y: RHS ON INPUT                                      
!                 LM: ORDER OF MATRIX                                   
!          *OUTPUT:Y: SOLUTION ON OUTPUT                                
!                  D: WORK ARRAY                                        
!     ................................................................. 
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid      , only: pver, pverp, pcols

   implicit none
!
! Arguments
!
   integer LM
   real a(pver)
   real b(pver)
   real c(pver)
   real d(pver)
!                    
! Input/output arguments
!
   real y(pver)
!
!---------------------------Local workspace-----------------------------
!
   integer k, kk
   integer LMM1
   real F

!----------------------------------------------------------
   LMM1=LM-1                                                         
   F=C(1)                                                            
   DO K=1,LMM1                                                     
      D(K)=B(K)/A(K)                                                    
      Y(K)=Y(K)/A(K)                                                    
      Y(LM)=Y(LM)-F*Y(K)                                                
      F=C(K+1)-F*D(K)                                                   
   ENDDo
   Y(LM)=Y(LM)/F                                                     
   DO KK=1,LMM1                                                    
      K=LM-KK                                                           
      Y(K)=Y(K)-D(K)*Y(K+1)                                             
   ENDDO
   RETURN                                                            
END SUBROUTINE GAUSS_DRY
                                                                       
                                                        

