      subroutine find_tropopaus( lchnk  , ncol, zmid, tfld, ltrop )
      use shr_kind_mod,  only: r8=>shr_kind_r8
      use ppgrid,        only: pcols, pver
      implicit none
!----
      integer, intent(in) ::  lchnk
      integer, intent(in) ::  ncol 
      real(r8),intent(in) ::  zmid  (pcols, pver)      ! height (m)
      real(r8),intent(in) ::  tfld(pcols, pver)        ! temperature (K)
      integer ,intent(out) :: ltrop(pcols)             ! tropopaus level 
!----------
      real, parameter    :: ztrop_low  = 6.            ! lowest tropopause level allowed (km)
      real, parameter    :: ztrop_high = 18.           ! highest tropopause level allowed (km)
      real, parameter    :: max_dtdz   = 2. 

      real(r8)           :: zmidkm(pcols,pver)           
      real(r8)           :: dtdz
      integer i, k
	  logical found_trop 
!--------------------------------------------------------
!       ... find the tropopause location
!--------------------------------------------------------
      zmidkm(:,:) = zmid(:,:)*1.e-3
      do i = 1, ncol
        found_trop = .false.
        do k = pver-1,2,-1
           if( zmidkm(i,k) < ztrop_low ) then
              cycle
           else if( zmidkm(i,k) > ztrop_high ) then
              exit
           end if
           dtdz = (tfld(i,k) - tfld(i,k-1)) /(zmidkm(i,k-1) - zmidkm(i,k))
           ltrop(i) = k
           if( dtdz <= max_dtdz ) then
              found_trop = .true.
              exit
           end if
        end do
!        if( .not. found_trop ) then
!           write(*,*) ': failed to find tropopause at lchnk ind=',lchnk,', lon ind=',i
!        end if
      enddo
      return
	  end subroutine find_tropopaus
