#include <misc.h>
#include <params.h>


#ifdef MAC_SP
subroutine mac_sp_driver(clon, clat, phis, wavmid, zm, zint, aod_prof, ssa_prof, asy_prof,fa_prof)

    use shr_kind_mod, only: r8 => shr_kind_r8
    use time_manager, only: get_curr_date
    use physconst, only: gravit
    use ppgrid 
    use mo_simple_plumes, only: sp_aop_profile
    use volcanicradi, only: aerocyc
    use prescribed_aerosols, only: rampyear_prescribed_aeros

implicit none

    real(r8), intent(in) :: clon          !units: radians
    real(r8), intent(in) :: clat          !units: radians
    real(r8), intent(in) :: phis          !units: m^2/s^2
    real(r8), intent(in) :: wavmid        !units: nm
    real(r8), intent(in) :: zm(pver)      !units: m (top to surface)
    real(r8), intent(in) :: zint(pver+1)  !units: m (top to surface)

    integer, parameter :: nmon=12
    integer, parameter :: ng=1   !number of columns
    integer :: iyear, imon, iday, tod
    integer :: ig, iz
    real(r8) :: lon(ng) 
    real(r8) :: lat(ng) 
    real(r8) :: oro(ng)       !units:m
    real(r8) :: year_fr 
    real(r8) :: zdelt(ng,pver)  !units: m (top to surface)
    real(r8) :: zmid(ng,pver)  !units: m (top to surface)

    real(r8),intent(out) :: aod_prof(ng,pver)
    real(r8),intent(out) :: ssa_prof(ng,pver)
    real(r8),intent(out) :: asy_prof(ng,pver)
    real(r8),intent(out) :: fa_prof(ng,pver)
    real(r8) :: dNovrN(ng)
    
    lon=clon*57.2958  !radians to degrees
    lat=clat*57.2958  !radians to degrees
    oro=phis/gravit

    call get_curr_date(iyear,imon,iday,tod)

    if(aerocyc)then
       year_fr = rampyear_prescribed_aeros + (imon-0.5) / nmon
    else
       year_fr = iyear + (imon-0.5) / nmon
    endif

    do ig=1,ng 
     do iz=1,pver
       zdelt(ig,iz)=zint(iz)-zint(iz+1)
       zmid(ig,iz)=zm(iz)
       if(zdelt(ig,iz) <0.0) stop '-----negative zdelt in mac_sp_driver----'
     end do
    end do

    aod_prof(:,:)=0.0_r8
    ssa_prof(:,:)=0.0_r8
    asy_prof(:,:)=0.0_r8
    fa_prof(:,:)=0.0_r8

    call sp_aop_profile                       ( &
           pver         ,1       ,1       ,wavmid  ,oro     ,lon      ,lat     , &
           year_fr      ,zmid(1:ng,pver:1:-1)     ,zdelt(1:ng,pver:1:-1)    ,dNovrN(1:ng)   ,    &
           aod_prof(1:ng,pver:1:-1)     ,ssa_prof(1:ng,pver:1:-1)   , asy_prof(1:ng,pver:1:-1)  )
    do ig=1,ng
     do iz=1,pver
        fa_prof(ig,iz)=asy_prof(ig,iz)*asy_prof(ig,iz)
     end do
    end do

end subroutine
#endif
