MODULE mo_constants

! wzz, 2008.7.7
  use shr_const_mod
!
  IMPLICIT NONE

  ! ----------------------------------------------------------------
  !
  ! module *mo_constants* basic universal constants  and derived constants.
  !
  ! ----------------------------------------------------------------

  REAL, PARAMETER :: dayl = 86400.  ! length of the day (in seconds).

  REAL :: api     ! 2.*arcsin(1.).
  REAL :: a       ! radius of the earth.
  REAL :: omega   ! solid rotation velocity of the earth.
  REAL :: g       ! gravity acceleration.
  REAL :: cpd     ! specific heat at constant pressure (dry air).
  REAL :: cpv     !            idem               (water vapour).
  REAL :: rd      ! gas constant for dry air.
  REAL :: rv      !    idem      for water vapour.
  REAL :: rcpd    ! rcpd=1./cpd.
  REAL :: vtmpc1  ! vtmpc1=rv/rd-1.
  REAL :: vtmpc2  ! vtmpc2=cpv/cpd-1.
  REAL :: rhoh2o  ! density of liquid water.
  REAL :: alv     ! latent heat for vaporisation.
  REAL :: als     ! latent heat for sublimation.
  REAL :: alf     ! latent heat for fusion.
  REAL :: clw     ! specific heat for liquid water.
  REAL :: tmelt   ! temperature of fusion of ice.
  REAL :: solc    ! solar constant.
  REAL :: stbo    ! stephan boltzmann constant.
  REAL :: yearl   ! length of the year (in days).

  ! constants used for computation of saturation mixing ratio
  !   over liquid water(*c_les*) or ice(*c_ies*).

  REAL :: c1es    ! 610.78
  REAL :: c2es    ! 1es*rd/rv
  REAL :: c3les   ! 17.269
  REAL :: c3ies   ! 21.875
  REAL :: c4les   ! 35.86
  REAL :: c4ies   !  7.66
  REAL :: c5les   ! c3les*(tmelt-c4les)
  REAL :: c5ies   ! c3ies*(tmelt-c4ies)
  REAL :: c5alvcp ! c5les*alv/cpd
  REAL :: c5alscp ! c5ies*als/cpd
  REAL :: alvdcp  ! alv/cpd
  REAL :: alsdcp  ! als/cpd

!sxl add for mixed phases----------
  REAL ::  RLVTT 
  REAL ::  RLSTT 
  REAL ::  RTT
  REAL ::  CPDEC
  REAL ::  R2ES 
  REAL ::  R3LES 
  REAL ::  R3IES 
  REAL ::  R4LES 
  REAL ::  R4IES 
  REAL ::  R5LES 
  REAL ::  R5IES   
  REAL ::  R5ALVCP 
  REAL ::  R5ALSCP 
  REAL ::  RALVDCP 
  REAL ::  RALSDCP
  REAL ::  RLMLT
  REAL ::  RALFDCP 
  REAL ::  RTWAT 
  REAL ::  RTBER
  REAL ::  RTICE
!sxl add end-----------------------
CONTAINS

  SUBROUTINE inicon

    ! Description:
    ! Preset constants in mo_constants.
    !
    ! Method:
    !
    ! *inicon* is called from *setdyn*.
    !
    ! Authors:
    !
    ! M. Jarraud, ECMWF, December 1982, original source
    ! L. Kornblueh, MPI, May 1998, f90 rewrite
    ! U. Schulzweida, MPI, May 1998, f90 rewrite
    ! H.-S. Bauer, MPI, Jul 1998, changed
    ! A. Rhodin, MPI, Jan 1999, subroutine inicon put into module mo_constants
    !
    ! for more details see file AUTHORS
    !
    !  Intrinsic functions 
!    INTRINSIC ASIN

    !  Executable statements 

    !-- 1. Preset constants

    api    = SHR_CONST_PI
    a      = SHR_CONST_REARTH
    omega  = SHR_CONST_OMEGA
    g      = SHR_CONST_G
    cpd    = SHR_CONST_CPDAIR
    cpv    = SHR_CONST_CPWV
!    cpv    = 1869.46
    rd     = SHR_CONST_RDAIR
    rv     = SHR_CONST_RWV

    rcpd   = 1./cpd
    vtmpc1 = rv/rd - 1.
    vtmpc2 = cpv/cpd - 1.

    rhoh2o = SHR_CONST_RHOFW
    alv    = 2.5008E6
    als    = 2.8345E6
    alf    = als - alv

    clw    = 4186.84
    tmelt  = SHR_CONST_TKFRZ

    solc   = 1365.
    stbo   = SHR_CONST_STEBOL

    c1es    = 610.78
    c2es    = c1es*rd/rv
    c3les   = 17.269
    c3ies   = 21.875
    c4les   = 35.86
    c4ies   =  7.66
    c5les   = c3les*(tmelt-c4les)
    c5ies   = c3ies*(tmelt-c4ies)
    c5alvcp = c5les*alv/cpd
    c5alscp = c5ies*als/cpd
    alvdcp  = alv/cpd
    alsdcp  = als/cpd

!sxl add for mixed phases------------------
      RTT=273.16
      RLVTT=2.5008E+6
      RLSTT=2.8345E+6
      CPDEC=1004.71
      R2ES=611.21*rd/rv
      R3LES=17.502
      R3IES=22.587
      R4LES=32.19
      R4IES=-0.7
      R5LES=R3LES*(RTT-R4LES)
      R5IES=R3IES*(RTT-R4IES)
      R5ALVCP=R5LES*RLVTT/CPDEC
      R5ALSCP=R5IES*RLSTT/CPDEC
      RALVDCP=RLVTT/CPDEC
      RALSDCP=RLSTT/CPDEC
      RLMLT=RLSTT-RLVTT
      RALFDCP=RLMLT/CPDEC
      RTWAT=RTT
      RTBER=RTT-5.      
      RTICE=RTT-23.

!sxl add end-------------------------------

  END SUBROUTINE inicon

END MODULE mo_constants
