#include <misc.h> 
#include <params.h>
#if (defined MOZART2) || (defined BCCCHEM) || (defined GEOSCHEM)  || (defined WACCM)
module prescribed_em_d2
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid,       only: pcols, pver, pverp, begchunk, endchunk
   use pmgrid,       only: plon, plat, plev, plevp, masterproc
   use phys_grid,    only: get_ncols_p, scatter_field_to_chunk
   use time_manager, only: get_curr_calday, get_curr_date
   use infnan,       only: inf, bigint
   use abortutils,   only: endrun
   use filenames,    only: bndtvaer
#ifdef BCCCHEM
   use bcc_tracinp,  only: bndic
#endif
#if (defined WACCM) || (defined MOZART2)
   use mo_tracinp,   only: bndic
#endif

!---------------------------zf 2010.11.08------------------------------
   use rgrid,          only: nlon
   use commap,         only: latdeg, londeg
   use pmgrid,         only: plond
!-----------------------------------------------------------------------
#if ( defined SCAM )
   use getnetcdfdata
#endif

   implicit none

#if (defined BCCCHEM) || (defined WACCM)
   integer    ,public,parameter :: nem2    = 11
   character*8,public,parameter :: varname2d (nem2) = (/ &
                       'CH2O    ','CH4     ','CO      ',   &
                       'H2      ','N2O     ','NO      ',   &
                       'DMS     ','SO2     ',              & 
                       'CB1     ','CB2     ','NH3     '/)

   integer, public, parameter :: &   ! note: set the value = 0, if there is not emission data
          idxCH2O     =  1, &
          idxCH4      =  2, &
          idxCO       =  3, &
          idxH2       =  4, &
          idxN2O      =  5, &
          idxNO       =  6, &
          idxDMS      =  7, &
          idxSO2      =  8, &
 	  idxCB1      =  9, &
          idxCB2      = 10, &
	  idxNH3      = 11
   
#elif (defined MOZART2) 
   integer    ,public,parameter :: nem2    = 18
   character*8,public :: varname2d (nem2) 
     data varname2d / 'CH3COCH3', 'C2H4    ',             'C2H6    ', 'C3H6    ', &
                      'C3H8    ', 'CH2O    ', 'CH3CHO  ', 'CH3OH   ', 'N2O     ', &
                      'CO      ', 'H2      ', 'ISOP    ', 'NO      ', 'C4H10   ', &
                      'CH4     ',                                                 &
                      'DMS     ', 'SO2     ',                                     &
                      'NH3     ' /
   integer, public, parameter :: &
          idxCH3COCH3 =  1, &
          idxC2H4     =  2, &
          idxC2H6     =  3, &
          idxC3H6     =  4, &
          idxC3H8     =  5, &
          idxCH2O     =  6, &
          idxCH3CHO   =  7, &
          idxCH3OH    =  8, &
          idxN2O      =  9, &
          idxCO       = 10, &
          idxH2       = 11, &
          idxISOP     = 12, &
          idxNO       = 13, &
          idxC4H10    = 14, &
          idxCH4      = 15, &
          idxDMS      = 16, &
          idxSO2      = 17, &
          idxNH3      = 18, &
          idxC2H5OH   = -1, &
          idxCB1      = -1, &
          idxCB2      = -1, &
          idxOC1      = -1, &
          idxOC2      = -1, &
          idxC10H16   = -1, &
          idxCH3COCHO = -1

!********************************************************************************************************************
!    CH3COCH3-------------  anthro   bb   --------   biogenic_mozart   ocean   ----   --------------   -------
!    C2H4    -------------  anthro   bb   biogenic   biogenic_mozart   ocean   ----   --------------   -------
!    C2H5OH  -------------  anthro   bb   --------   ---------------   -----   ----   --------------   -------
!    C2H6    -------------  anthro   bb   biogenic   biogenic_mozart   ocean   ----   --------------   -------
!    C3H6    -------------  anthro   bb   biogenic   biogenic_mozart   ocean   ----   --------------   -------
!    C3H8    -------------  anthro   bb   biogenic   biogenic_mozart   ocean   ----   --------------   -------
!    CH2O    -------------  anthro   bb   --------   ---------------   -----   ----   --------------   -------
!    CH3CHO  -------------  anthro   bb   --------   ---------------   -----   ----   --------------   -------  
!    CH3OH   -------------  anthro   bb   biogenic   biogenic_mozart   -----   ----   --------------   -------
!    N2O     -------------  anthro   bb   --------   ---------------   ocean   soil   --------------   -------
!    CO      -------------  anthro   bb   biogenic   ---------------   ocean   soil   --------------   -------
!    H2      -------------  anthro   bb   --------   ---------------   ocean   soil   --------------   -------
!    ISOP    -------------  ------   bb   biogenic   biogenic_mozart   -----   ----   --------------   -------
!    NO      -------------  anthro   bb   --------   ---------------   -----   soil   --------------   -------
!    C4H10   -------------  anthro   bb   --------   biogenic_mozart   ocean   ----   --------------   ------- 
!    CH4     -------------  anthro   bb   --------   biogenic_mozart   ocean   ----   rice+ruminants   ------- 
!    DMS     -------------  ------   --   --------   ---------------   ocean   ----   --------------   -------
!    SO2     -------------  anthro   bb   --------   ---------------   -----   ----   --------------   volcano
!    CB1     -------------  anthro   bb   --------   ---------------   -----   ----   --------------   -------
!    CB2     -------------  anthro   bb   --------   ---------------   -----   ----   --------------   -------
!    OC1     -------------  anthro   bb   --------   ---------------   -----   ----   --------------   -------
!    OC2     -------------  anthro   bb   --------   ---------------   -----   ----   --------------   -------
!    NH3     -------------  anthro   bb   --------   ---------------   ocean   soil   --------------   -------
!    C10H16  -------------  ------   --   biogenic   biogenic_mozart   -----   ----   --------------   ------- 
!********************************************************************************************************************

#endif

!
! compute carbons scaling from population

  integer, public           :: rampyear_prescribed_em2   =  bigint ! this is the only acceptable value
  character(len=16), public :: scenario_prescribed_em2   = 'FIXED' ! 'FIXED' or 'RAMPED'

  private
  save

  integer  :: em2nid          ! netcdf id for em2ol file (init to invalid)

  real(r8) :: cdaym           ! calendar day of prv month
  real(r8) :: cdayp           ! calendar day of next month
  
  public  em2ini_IPCC
  public  em2int_IPCC
  public  getem2_IPCC
!
! for IPCC data 
!
   integer,private   :: lonem2iz     ! size of longitude dimension on em2ol dataset
   integer,private   :: latem2iz     ! size of latitude dimension on em2ol dataset

   real(r8),private, allocatable :: em2lon(:)
   real(r8),private, allocatable :: em2lat(:)
   real(r8),private,allocatable,dimension(:,:,:,:) ::  em2i
                      ! input sulfate bio mixing ratios (pcols,begchunk:endchunk,2,nem2)
   real(r8),private,allocatable,dimension(:,:,:)   ::  em2
                      ! time interpolated sulfate bio mixing ratios (pcols,begchunk:endchunk,nem2)

   real(r8), private :: cdayem2m        ! calendar day for prv. month sulfate values read in
   real(r8), private :: cdayem2p        ! calendar day for nxt. month sulfate values read in

   real(r8),private :: bndry_m          ! calendar day for the lower time-boundary of inputdata
   real(r8),private :: bndry_p          ! calendar day for the upper time-boundary of inputdata
   logical, private :: outofbndry       ! whether current model time is out of the time-boundary

   integer,parameter :: nfile = 19    ! number of input data files
   integer,parameter :: yr_em2(nfile) = (/ &
                      1840, 1850, 1860, 1870, 1880, 1890, &
         	      1900, 1910, 1920, 1930, 1940, &
                      1950, 1960, 1970, 1980, 1990, &
                      2000, 2010, 2020  /)

   integer, private :: nmsul              ! index to prv month in array. init to 1 and toggle between 1 and 2
   integer, private :: npsul              ! index to nxt month in array. init to 2 and toggle between 1 and 2

   character*256,private :: file_em2(nfile)

   integer, private,parameter :: nsample = 12   ! number of samples in each input data file

   real(r8),private           :: sample_em2(nsample) =      (/&
                1296000.0,  3888000.0,  6393600.0,  9072000.0, &
               11664000.0, 14342400.0, 16934400.0, 19612800.0, &  
               22291200.0, 24883200.0, 27561600.0, 30153600.0 /)
!
! Integer data
!
   integer, private :: sp_m, sp_p
   integer, private :: yrm,  yrp
   integer, private :: yr_am, yr_bm, yr_ap, yr_bp, indxbp
!
   integer, private :: ncid_em2_ap, ncid_em2_bp  ! sulfate dataset id

   character*80,  private ::  em2path=''        ! full pathname for sulfate dataset
   character*256, private ::  em2pathfile_bp    ! full pathname for sulfate dataset

!------------------zf 2010.11.15---------------------------------------------------
   interface

   subroutine bilin (arrin, xin, yin, nlondin, nlonin, &
                     nlevdin, nlev, nlatin, arrout, xout, &
                     yout, nlondout, nlonout, nlevdout, nlatout, fillvalue)

   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun

   integer, intent(in) :: nlondin                        ! longitude dimension of input grid
   integer, intent(in) :: nlonin                         ! number of real longitudes (input)
   integer, intent(in) :: nlevdin                        ! vertical dimension of input grid
   integer, intent(in) :: nlev                           ! number of vertical levels
   integer, intent(in) :: nlatin                         ! number of input latitudes
   integer, intent(in) :: nlatout                        ! number of output latitudes
   integer, intent(in) :: nlondout                       ! longitude dimension of output grid
   integer, intent(in) :: nlonout(nlatout)               ! number of output longitudes per lat
   integer, intent(in) :: nlevdout                       ! vertical dimension of output grid

   real(r8), intent(in) :: arrin(nlondin,nlevdin,nlatin) ! input array of values to interpolate
   real(r8), intent(in) :: xin(nlondin)                  ! input x mesh
   real(r8), intent(in) :: yin(nlatin)                   ! input y mesh
   real(r8), intent(in) :: xout(nlondout,nlatout)        ! output x mesh
   real(r8), intent(in) :: yout(nlatout)                 ! output y mesh
   real(r8), optional, intent(in) :: fillvalue           ! undef. value, wzz, 2009.8.31

   real(r8), intent(out) :: arrout(nlondout,nlevdout,nlatout) ! interpolated array

   end subroutine bilin

   end interface
!----------------------------------------------------------------------------------

contains

subroutine em2ini_IPCC
!----------------------------------------------------------------------- 
! 
! Purpose: Do initial read of time-variant em2ol dataset, containing
!          em2ol mixing ratios as a function of time.  It is currently
!          required that the em2ol dataset have the *SAME* horizontal
!          and vertical resolution as the model. Therefore, ONLY a time
!          interpolation of the dataset is currently performed.
! 
!-----------------------------------------------------------------------
   use ioFileMod,      only: getfil
   use error_messages, only: alloc_err, handle_ncerr
   use time_manager,   only: get_curr_date, get_perp_date, get_curr_calday, &
                             is_perpetual
   use dycore,         only: get_resolution
   implicit none
!
   integer :: ncid_em2_am=-1, ncid_em2_bm  ! sulfate dataset id
   integer :: em2_id_ap,   em2_id_bp
   integer :: em2_id_am,   em2_id_bm 
 
   integer :: lonsiz     ! size of longitude dimension on sulfate dataset
   integer :: latsiz     ! size of latitude dimension on sulfate dataset
   integer :: timsiz     ! size of time dimension on sulfate dataset

   integer :: idx

   integer   :: nchar, nspecies, ispecies, em2_sp_id          !zf 2016.02.19
   character :: species_name*25                               !zf 2016.02.19

   character*256 ::  em2pathfile_am ! full pathname for sulfate dataset
   character*256 ::  em2pathfile_bm ! full pathname for sulfate dataset
   character*256 ::  em2pathfile_ap ! full pathname for sulfate dataset

#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
   character*80, parameter ::  p1t_2d='IPCC6/emis.surface_'      !historical
!  character*80, parameter ::  p1t_2d='IPCC-var2d.1870/var2d_'      !1870 
#endif
   character*80, parameter ::  p2='.nc'     ! part of the name of sulfate data file

   character(len=256) :: locfn          ! netcdf local filename to open   
!-----------------------------------------------------------------------
   include 'netcdf.inc'
!
! Local workspace
!
   character(len=256) :: locfn_am, locfn_bm, locfn_ap, locfn_bp     ! local filenames

   integer londimid              ! netcdf id for longitude dimension
   integer latdimid              ! netcdf id for latitude dimension
   integer lonid                 ! netcdf id for longitude variable
   integer latid                 ! netcdf id for latitude variable
   integer timeid                ! netcdf id for time variable
   integer cnt3(3)               ! array of counts for each dimension
   integer strt3(3)              ! array of starting indices
   integer i, k, lat, n          ! longitude, level, latitude, time indices
   integer istat                 ! error return
   integer dimids(nf_max_var_dims) ! netcdf variable shape

   integer yr, mon, day, ncsec ! components of a date
   real(r8) calday                   ! current calendar day
   real(r8) caldayloc                ! calendar day (includes yr if no cycling)
   real(r8) fact1, fact2, deltat

   character*4 yrtmp

   real(r8) xem2i_am(plond,plat)  ! input sulfate bio mixing ratios
   real(r8) xem2i_bm(plond,plat)  ! input sulfate ant mixing ratios
   real(r8) xem2i_ap(plond,plat)  ! input sulfate bio mixing ratios
   real(r8) xem2i_bp(plond,plat)  ! input sulfate ant mixing ratios

   real(r8) xem2i(plon,plat,2)   ! input aerosol ant mixing ratios
   
   real(r8), allocatable :: xem2i_tem2pa(:,:) 
   real(r8), allocatable :: xem2i_tem2pb(:,:)
   real(r8), allocatable :: xem2i_tem2pc(:,:)   !zf2016.02.22
!
!-----------------------------------------------------------------------
!
! Allocate space for data.
!
  allocate( em2i(pcols,begchunk:endchunk,2,nem2), stat=istat )
  call alloc_err( istat, 'em2ini', 'em2i', pcols*(endchunk-begchunk+1)*2*nem2 )

  allocate( em2(pcols,begchunk:endchunk,nem2), stat=istat )
  call alloc_err( istat, 'em2ini', 'em2',  pcols*(endchunk-begchunk+1)*nem2 )

  nmsul = 1
  npsul = 2
!
! unit shift from second to day
!
   do n=1,nsample
      sample_em2(n) = sample_em2(n)/86400.0+1.0
   enddo
!
! the names of rawdata files
!
   yrtmp = '1950'
   do n=1,nfile
      write(yrtmp,'(i4.4)') yr_em2(n)
      file_em2(n) = trim(p1t_2d)//yrtmp//trim(p2)
   enddo



! set the time_boundaries of input dataset
!
   bndry_m = yr_em2(1    )*365.0 + sample_em2(1) 
   bndry_p = yr_em2(nfile)*365.0 + sample_em2(nsample) 

#if (defined BCCCHEM) || (defined WACCM) || (defined MOZART2)
   do i = len_trim(bndic), 1, -1
       if(bndic(i:i).eq.'/') exit
   enddo
   em2path = bndic(1:i)
#endif
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
     calday = get_curr_calday()
     call get_curr_date(yr, mon, day, ncsec)
     if ( rampyear_prescribed_em2 .ne. bigint ) then
        yr = rampyear_prescribed_em2        ! wzz, 2009.10.15
     endif
     caldayloc = calday + yr*365.

!----------------------------------------------------------------
! locate the model time among the samples of sulfate data file      
!---------------------------------------------------------------
     if ( caldayloc.lt.bndry_m ) then

        outofbndry = .true.
        cdayem2p  = bndry_m
        cdayp     = sample_em2(1)    !???
        sp_p   = 1
        yrp    = yr_em2(1)
        yr_ap  = yr_em2(1)
        yr_bp  = yr_em2(2)
        indxbp = 2
        em2pathfile_ap = trim(em2path)//file_em2(1)
        em2pathfile_bp = trim(em2path)//file_em2(2)

     elseif ( caldayloc.ge.bndry_p ) then

        outofbndry = .true.
        sp_p   = nsample 
        yrp    = yr_em2(nfile)
        yr_ap  = yr_em2(nfile)
        yr_bp  = yr_em2(nfile)
        em2pathfile_ap = trim(em2path)//file_em2(nfile)
        em2pathfile_bp = trim(em2path)//file_em2(nfile)

     else
        outofbndry = .false.
!
! month
!
        if ( calday.lt.sample_em2(1) ) then
          yrm  = yr-1
          yrp  = yr
          sp_m = nsample 
          sp_p = 1
          goto 100
        elseif (calday.ge.sample_em2(nsample) ) then
          yrm = yr
          yrp = yr+1
          sp_m = nsample
          sp_p = 1 
          goto 100
        else 
          do n=1,nsample-1
            if ( calday.ge.sample_em2(n) .and. calday.lt.sample_em2(n+1) ) then
               yrm  = yr
               yrp  = yr
               sp_m = n
               sp_p = n+1
               goto 100
            endif
          enddo
        endif
        write(*,*) 'em2INI: Failed to find seconds bracketing model_time =',calday
        call endrun
100     continue
        if ( rampyear_prescribed_em2 .ne. bigint ) then
          yrm = rampyear_prescribed_em2        ! wzz, 2009.10.15
          yrp = rampyear_prescribed_em2        ! wzz, 2009.10.15
        endif
        cdayem2m = yrm*365.0+sample_em2(sp_m)
        cdayem2p = yrp*365.0+sample_em2(sp_p)
        cdaym = sample_em2(sp_m)
        cdayp = sample_em2(sp_p)
!
! year ( both yrm & yrp )
!
        do n=1,nfile-1
          yr_am = yr_em2(n)
          yr_bm = yr_em2(n+1)
          if ( yrm.ge.yr_am .and. yrm.lt.yr_bm ) goto 200
        enddo
        write(*,*) 'em2INI: Failed to find years bracketing year  yrm=',yrm
        call endrun
200     continue
        em2pathfile_am = trim(em2path)//trim(file_em2(n))
        em2pathfile_bm = trim(em2path)//trim(file_em2(n+1))
 
        write(*,*) ' em2pathfile_am =', em2pathfile_am
        write(*,*) ' em2pathfile_bm =', em2pathfile_bm

        do n=1,nfile-1
          yr_ap = yr_em2(n)
          yr_bp = yr_em2(n+1)
          if ( yrp.ge.yr_ap .and. yrp.lt.yr_bp ) goto 300
        enddo
        write(*,*) 'em2INI: Failed to find years bracketing year  yrp=',yrp
        call endrun
300     indxbp = n+1 
        em2pathfile_ap = trim(em2path)//trim(file_em2(n))
        em2pathfile_bp = trim(em2path)//trim(file_em2(n+1))

        write(*,*) ' em2pathfile_ap =', em2pathfile_ap
        write(*,*) ' em2pathfile_bp =', em2pathfile_bp

     endif

  if (masterproc) then
!--------------------------------------------------------------------------
! open the right data files and read in raw data
!--------------------------------------------------------------------------
     if ( outofbndry ) then

        call getfil(em2pathfile_ap, locfn_ap)
        call wrap_open(locfn_ap, 0, ncid_em2_ap)
      
        write(6,*) 'em2pathfile_ap =', em2pathfile_ap
        write(6,*) 'Now open em2 file (locfn_ap):', locfn_ap
!
! Obtain dimension id's
!
        call wrap_inq_dimid( ncid_em2_ap, 'lon', londimid)
        call wrap_inq_dimid( ncid_em2_ap, 'lat', latdimid)
        call wrap_inq_dimid( ncid_em2_ap, 'time',timeid  )
!
! Obtain size of dimensions.
! Check that horizontal and vertical dimensions are same as model's
! (check one file only and assume that all the other files are of the same setting)
!
        call wrap_inq_dimlen( ncid_em2_ap, londimid, lonem2iz   ) !zf 2010.11.10
        call wrap_inq_dimlen( ncid_em2_ap, latdimid, latem2iz   )  !zf 2010.11.10

       allocate( em2lon(lonem2iz), stat=istat )  
       allocate( em2lat(latem2iz), stat=istat )  

      call wrap_inq_varid( ncid_em2_ap, 'lon', lonid   )
      call wrap_inq_varid( ncid_em2_ap, 'lat', latid   )

      call wrap_get_var_realx( ncid_em2_ap, lonid, em2lon )
      call wrap_get_var_realx( ncid_em2_ap, latid, em2lat )

!---------------------------------------------------------------

      call wrap_inq_dimlen( ncid_em2_ap, timeid, timsiz   )
      if (timsiz /= nsample) then
         write(6,*)'em2INI: timsiz=',timsiz,' must = ', nsample 
         call endrun
      end if
!
! the variable
!
      call wrap_inq_varid( ncid_em2_ap, trim(varname2d(1))//'_anthro', em2_id_ap )   !zf 2016.02.22

      call wrap_inq_vardimid (ncid_em2_ap, em2_id_ap, dimids)
      if (dimids(1) /= londimid .and. dimids(2) /= latdimid) then
         write(6,*)'em2INI: Data must be ordered lon, lat, lev, time'
         call endrun
      end if
!
!---------------------------------------------------------------
! if not out of boundary
!---------------------------------------------------------------
    else   
  
      call getfil(em2pathfile_am, locfn_am)
      call wrap_open(locfn_am, 0, ncid_em2_am)

      write(6,*) 'Now open em2 file (locfn_am):', locfn_am

      call getfil(em2pathfile_bm, locfn_bm)
      call wrap_open(locfn_bm, 0, ncid_em2_bm)

      write(*,*)'wzz: locfn_ap'
      call getfil(em2pathfile_ap, locfn_ap)
      call wrap_open(locfn_ap, 0, ncid_em2_ap)

      write(*,*)'wzz: locfn_bp'
      call getfil(em2pathfile_bp, locfn_bp)
      call wrap_open(locfn_bp, 0, ncid_em2_bp)
!
      call wrap_inq_dimid( ncid_em2_am, 'lon', londimid)
      call wrap_inq_dimid( ncid_em2_am, 'lat', latdimid)
      call wrap_inq_dimid( ncid_em2_am, 'time',timeid  )

      call wrap_inq_dimlen( ncid_em2_am, londimid, lonem2iz   )
      call wrap_inq_dimlen( ncid_em2_am, latdimid, latem2iz   )

      call wrap_inq_dimlen( ncid_em2_am, timeid, timsiz   )
      if (timsiz /= nsample) then
         write(6,*)'em2INI: timsiz=',timsiz,' must = ', nsample 
         call endrun
      end if
!------------------zf 2010.11.08-----------------------------------

      allocate( em2lon(lonem2iz), stat=istat )   
      allocate( em2lat(latem2iz), stat=istat )

      call wrap_inq_varid( ncid_em2_am, 'lon', lonid   )
      call wrap_inq_varid( ncid_em2_am, 'lat', latid   )

      call wrap_get_var_realx( ncid_em2_am, lonid, em2lon )
      call wrap_get_var_realx( ncid_em2_am, latid, em2lat )

!--------------------------------------------------------------------
!
! Obtain sulfate mixing ratio id
!
      call wrap_inq_varid( ncid_em2_am, trim(varname2d(1))//'_anthro' , em2_id_am )

      call wrap_inq_vardimid (ncid_em2_am, em2_id_am, dimids)
      if (dimids(1) /= londimid .and. dimids(2) /= latdimid) then
         write(6,*)'em2INI: Data must be ordered lon, lat, lev, time'
         call endrun
      end if
    endif    ! end if (.not.outofbndry)

!------------------------------------------------------------------------------
! master does the work above
!------------------------------------------------------------------------------
   endif
!
   allocate( xem2i_tem2pa(lonem2iz,latem2iz), stat=istat )   !zf2010.11.08
   allocate( xem2i_tem2pb(lonem2iz,latem2iz), stat=istat )   !zf2010.11.08
   allocate( xem2i_tem2pc(lonem2iz,latem2iz), stat=istat )   !zf2016.02.22

   do idx=1,nem2
   if (masterproc) then
!--------------------------------------------------------------------------
! open the right data files and read in raw data
!--------------------------------------------------------------------------
    
    if ( outofbndry ) then
!
! Set up hyperslab corners
!
      strt3(1) = 1
      strt3(2) = 1
      cnt3(1)  = lonem2iz  !zf
      cnt3(2)  = latem2iz  !zf
      cnt3(3)  = 1
!
! need to read in only 1 sample of rawdata
!
      strt3(3)  = sp_p 
      xem2i_ap = 0.0
      xem2i_tem2pa = 0.0   !zf
      xem2i_tem2pb = 0.0   !zf
      xem2i_tem2pc = 0.0   !zf

      call wrap_inq_varid( ncid_em2_ap, trim(varname2d(idx)) , em2_id_ap )

!----------------zf 2016.02.19---------------------
      call wrap_inq_vardimid (ncid_em2_ap, em2_id_ap, dimids)
      call wrap_inq_dimlen (ncid_em2_ap, dimids(1), nchar)    !zf 2016.02.19
      call wrap_inq_dimlen (ncid_em2_ap, dimids(2), nspecies)    !zf 2016.02.19
      do ispecies = 1, nspecies
        call wrap_get_vara_text (ncid_em2_ap, em2_id_ap, (/1,ispecies/), (/nchar,1/), species_name)  !zf 2016.02.19
        call wrap_inq_varid( ncid_em2_ap, trim(species_name) , em2_sp_id)
        call wrap_get_vara_realx (ncid_em2_ap, em2_sp_id, strt3, cnt3, xem2i_tem2pc(:,:))  !zf 2016.02.19

#ifdef HAMOCC 

       ! note: at the present, the N2o and DMS flux from HAMOCC are not used 
       !       and they still use the prescribved data
 
!        if ( trim(species_name) .eq. 'DMS_ocean' .or. trim(species_name) .eq. 'N2O_ocean') then
!             xem2i_tem2pc(:,:) = 0.
!        end if

#endif
        xem2i_tem2pa(:,:) = xem2i_tem2pa(:,:) + xem2i_tem2pc(:,:)
      end do
!zf2016      call wrap_get_vara_realx (ncid_em2_ap, em2_id_ap,strt3,cnt3, &
!zf2016                                xem2i_tem2pa(:,:))                  !zf

      do i = 1, lonem2iz
      do n = 1, latem2iz
         if( xem2i_tem2pa(i,n) .gt. 1.e+30 ) xem2i_tem2pa(i,n) = 0.0
         xem2i_tem2pb(i, n) = xem2i_tem2pa(i, n)
      enddo
      enddo

      if(lonem2iz .eq. plon .and. latem2iz .eq. plat) then
         xem2i_ap = xem2i_tem2pa
      else
         call bilin(xem2i_tem2pb,em2lon,em2lat,lonem2iz,lonem2iz,1,1,   &
                 latem2iz,xem2i_ap,londeg,latdeg,plond,nlon,1,plat)
      end if

      do n    = 1,2
       do lat = 1,plat
         do i = 1,plon
           xem2i(i,lat,n)= xem2i_ap(i,lat)
         enddo
        enddo
      enddo
!---------------------------------------------------------
     else
!
! Set up hyperslab corners
!
      strt3(1) = 1
      strt3(2) = 1
      cnt3(1)  = lonem2iz  !zf
      cnt3(2)  = latem2iz   !zf
      cnt3(3)  = 1
!
! read in the 4 sample of rawdata
!
      strt3(3) = sp_m 
      xem2i_am = 0.0

      call wrap_inq_varid( ncid_em2_am, trim(varname2d(idx)) , em2_id_am )
      call wrap_inq_varid( ncid_em2_bm, trim(varname2d(idx)) , em2_id_bm )
      call wrap_inq_varid( ncid_em2_ap, trim(varname2d(idx)) , em2_id_ap )
      call wrap_inq_varid( ncid_em2_bp, trim(varname2d(idx)) , em2_id_bp )

      xem2i_tem2pa = 0.0   !zf
      xem2i_tem2pb = 0.0   !zf
      xem2i_tem2pc = 0.0   !zf

!----------------zf 2016.02.19---------------------
      call wrap_inq_vardimid (ncid_em2_am, em2_id_am, dimids)
      call wrap_inq_dimlen (ncid_em2_am, dimids(1), nchar)    !zf 2016.02.19
      call wrap_inq_dimlen (ncid_em2_am, dimids(2), nspecies)    !zf 2016.02.19

      write(6,*) 'zftest em2_id_am = ', em2_id_am, nchar, nspecies   !zftest

      do ispecies = 1, nspecies
        call wrap_get_vara_text (ncid_em2_am, em2_id_am, (/1,ispecies/), (/nchar,1/), species_name)  !zf 2016.02.19
        call wrap_inq_varid( ncid_em2_am, trim(species_name) , em2_sp_id )
        call wrap_get_vara_realx (ncid_em2_am, em2_sp_id, strt3, cnt3, xem2i_tem2pc(:,:))  !zf 2016.02.19
#ifdef HAMOCC 

       ! note: at the present, the N2o and DMS flux from HAMOCC are not used
       !       and they still use the prescribved data

!        if ( trim(species_name) .eq. 'DMS_ocean' .or. trim(species_name) .eq. 'N2O_ocean') then
!             xem2i_tem2pc(:,:) = 0.
!        end if

#endif
        xem2i_tem2pa(:,:) = xem2i_tem2pa(:,:) + xem2i_tem2pc(:,:)
        write(6,*) varname2d(idx), '   ', species_name    !zftest
      end do
!-----------------------------------------------
!zf2016      call wrap_get_vara_realx (ncid_em2_am,em2_id_am,strt3,cnt3, &
!zf2016                                xem2i_tem2pa(:,:))  !zf

!-------------zf 2010.11.10-----------------
      do i = 1, lonem2iz
      do n = 1, latem2iz
           if( xem2i_tem2pa(i,n) .gt. 1.e+30 ) xem2i_tem2pa(i,n) = 0.0
           xem2i_tem2pb(i, n) = xem2i_tem2pa(i, n)
      enddo
      enddo
       
      if(lonem2iz .eq. plon .and. latem2iz .eq. plat) then
         xem2i_am = xem2i_tem2pb
      else
         call bilin(xem2i_tem2pb,em2lon,em2lat,lonem2iz,lonem2iz,1,1,   &
                 latem2iz,xem2i_am,londeg,latdeg,plond,nlon,1,plat)
      endif
!-----------------------------------------------

      xem2i_bm = 0.0
      xem2i_tem2pa = 0.0   !zf
      xem2i_tem2pb = 0.0   !zf
      xem2i_tem2pc = 0.0   !zf

!----------------zf 2016.02.19---------------------
      call wrap_inq_vardimid (ncid_em2_bm, em2_id_bm, dimids)
      call wrap_inq_dimlen (ncid_em2_bm, dimids(2), nspecies)    !zf 2016.02.19
      do ispecies = 1, nspecies
        call wrap_get_vara_text (ncid_em2_bm, em2_id_bm, (/1,ispecies/), (/nchar,1/), species_name)  !zf 2016.02.19
        call wrap_inq_varid( ncid_em2_bm, trim(species_name) , em2_sp_id )
        call wrap_get_vara_realx (ncid_em2_bm, em2_sp_id, strt3, cnt3, xem2i_tem2pc(:,:))  !zf 2016.02.19
#ifdef HAMOCC  

       ! note: at the present, the N2o and DMS flux from HAMOCC are not used
       !       and they still use the prescribved data

!        if ( trim(species_name) .eq. 'DMS_ocean' .or. trim(species_name) .eq. 'N2O_ocean') then
!             xem2i_tem2pc(:,:) = 0.
!        end if
#endif
        xem2i_tem2pa(:,:) = xem2i_tem2pa(:,:) + xem2i_tem2pc(:,:)
      end do
!-----------------------------------------------
!zf2016      call wrap_get_vara_realx (ncid_em2_bm,em2_id_bm,strt3,cnt3, &
!zf2016                                xem2i_tem2pa(:,:))  !zf
      do i = 1, lonem2iz
      do n = 1, latem2iz
         if( xem2i_tem2pa(i,n) .gt. 1.e+30 ) xem2i_tem2pa(i,n) = 0.0
         xem2i_tem2pb(i, n) = xem2i_tem2pa(i, n)
      enddo
      enddo
       
     if( lonem2iz .eq. plon .and. latem2iz .eq. plat) then
        xem2i_bm = xem2i_tem2pb
     else
         call bilin(xem2i_tem2pb,em2lon,em2lat,lonem2iz,lonem2iz,1,1,   &
                    latem2iz,xem2i_bm,londeg,latdeg,plond,nlon,1,plat)
     endif
!-----------------------------------------------
!
      strt3(3) = sp_p 
      xem2i_ap = 0.0
      xem2i_tem2pa = 0.0   !zf
      xem2i_tem2pb = 0.0   !zf
      xem2i_tem2pc = 0.0   !zf

!----------------zf 2016.02.19---------------------
      call wrap_inq_vardimid (ncid_em2_ap, em2_id_ap, dimids)
      call wrap_inq_dimlen (ncid_em2_ap, dimids(2), nspecies)    !zf 2016.02.19
      do ispecies = 1, nspecies
        call wrap_get_vara_text (ncid_em2_ap, em2_id_ap, (/1,ispecies/), (/nchar,1/), species_name)  !zf 2016.02.19
        call wrap_inq_varid( ncid_em2_ap, trim(species_name) , em2_sp_id )
        call wrap_get_vara_realx (ncid_em2_ap, em2_sp_id, strt3, cnt3, xem2i_tem2pc(:,:))  !zf 2016.02.19
#ifdef HAMOCC 

       ! note: at the present, the N2o and DMS flux from HAMOCC are not used
       !       and they still use the prescribved data

!        if ( trim(species_name) .eq. 'DMS_ocean' .or. trim(species_name) .eq. 'N2O_ocean') then
!             xem2i_tem2pc(:,:) = 0.
!        end if
#endif
        xem2i_tem2pa(:,:) = xem2i_tem2pa(:,:) + xem2i_tem2pc(:,:)
      end do
!-----------------------------------------------
!zf2016      call wrap_get_vara_realx (ncid_em2_ap,em2_id_ap,strt3,cnt3, &
!zf2016                                xem2i_tem2pa(:,:))  !zf

      do i = 1, lonem2iz
      do n = 1, latem2iz
         if( xem2i_tem2pa(i,n) .gt. 1.e+30 ) xem2i_tem2pa(i,n) = 0.0
         xem2i_tem2pb(i, n) = xem2i_tem2pa(i, n)
      enddo
      enddo
       
      if(lonem2iz .eq. plon .and. latem2iz .eq. plat) then
         xem2i_ap = xem2i_tem2pb
      else
         call bilin(xem2i_tem2pb,em2lon,em2lat,lonem2iz,lonem2iz,1,1,  &
                 latem2iz,xem2i_ap,londeg,latdeg,plond,nlon,1,plat)
      endif
!-----------------------------------------------

      xem2i_bp = 0.0
      xem2i_tem2pa = 0.0   !zf
      xem2i_tem2pb = 0.0   !zf
      xem2i_tem2pc = 0.0   !zf

!----------------zf 2016.02.19---------------------
      call wrap_inq_vardimid (ncid_em2_bp, em2_id_bp, dimids)
      call wrap_inq_dimlen (ncid_em2_bp, dimids(2), nspecies)    !zf 2016.02.19
      do ispecies = 1, nspecies
        call wrap_get_vara_text (ncid_em2_bp, em2_id_bp, (/1,ispecies/), (/nchar,1/), species_name)  !zf 2016.02.19
        call wrap_inq_varid( ncid_em2_bp, trim(species_name) , em2_sp_id )
        call wrap_get_vara_realx (ncid_em2_bp, em2_sp_id, strt3, cnt3, xem2i_tem2pc(:,:))  !zf 2016.02.19
#ifdef HAMOCC 
       ! note: at the present, the N2o and DMS flux from HAMOCC are not used
       !       and they still use the prescribved data

!        if ( trim(species_name) .eq. 'DMS_ocean' .or. trim(species_name) .eq. 'N2O_ocean') then
!             xem2i_tem2pc(:,:) = 0.
!        end if
#endif
        xem2i_tem2pa(:,:) = xem2i_tem2pa(:,:) + xem2i_tem2pc(:,:)
      end do
!-----------------------------------------------
!zf2016      call wrap_get_vara_realx (ncid_em2_bp,em2_id_bp,strt3,cnt3, &
!zf2016                                xem2i_tem2pa(:,:))  !zf

!-------------zf 2010.11.10-----------------
      do i = 1, lonem2iz
      do n = 1, latem2iz
         if( xem2i_tem2pa(i,n) .gt. 1.e+30 ) xem2i_tem2pa(i,n) = 0.0
         xem2i_tem2pb(i, n) = xem2i_tem2pa(i, n)
      enddo
      enddo
       
     if(lonem2iz .eq. plon .and. latem2iz .eq. plat) then
        xem2i_bp = xem2i_tem2pb
     else
        call bilin(xem2i_tem2pb,em2lon,em2lat,lonem2iz,lonem2iz,1,1,   &
                   latem2iz,xem2i_bp,londeg,latdeg,plond,nlon,1,plat)
     endif
!-----------------------------------------------
!
! Normal interpolation between consecutive time slices.
!
      write(*,*) "yr_bm=",yr_bm,"yr_am=",yr_am, "yrm=",yrm
      deltat = dble( yr_bm - yr_am )
      fact1  = dble( yr_bm - yrm  )/deltat
      fact2  = dble( yrm   - yr_am)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'em2INI--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
        do i = 1,plon
           xem2i(i,lat,nmsul)= fact1*xem2i_am(i,lat)+fact2*xem2i_bm(i,lat)
        enddo
      enddo
!
      deltat = dble( yr_bp - yr_ap )
      fact1  = dble( yr_bp - yrp  )/deltat
      fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'em2INI--2: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
        do i = 1,plon
           xem2i(i,lat,npsul)= fact1*xem2i_ap(i,lat)+fact2*xem2i_bp(i,lat)
        enddo
      enddo

    endif    ! end if (.not.outofbndry)
!------------------------------------------------------------------------------
! master does the work above
!------------------------------------------------------------------------------
   endif    ! if( masterproc) 
!------------------------------------------------------------------------------

   call scatter_field_to_chunk(1,1,1,plon,xem2i(1,1,nmsul),em2i(1,begchunk,nmsul,idx))
   call scatter_field_to_chunk(1,1,1,plon,xem2i(1,1,npsul),em2i(1,begchunk,npsul,idx))

   if (outofbndry) then
       call scatter_field_to_chunk(1,1,1,plon,xem2i_ap,em2(:,:,idx))  !??
   endif

!-------------------------------
   enddo

   deallocate (xem2i_tem2pa)   !zf2010.11.08
   deallocate (xem2i_tem2pb)   !zf2010.11.08
   deallocate (xem2i_tem2pc)   !zf2010.11.08

   return
end subroutine em2ini_IPCC

!###############################################################################
!
subroutine em2int_IPCC
!----------------------------------------------------------------------- 
!
! Purpose: Interpolate sulfate mixing ratios to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
!-----------------------------------------------------------------------
   use ioFileMod,      only: getfil
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use timeinterp,   only: getfactors

   implicit none

!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
!
! Local workspace
!
   integer cnt3(3)            ! array of counts for each dimension
   integer strt3(3)           ! array of starting indices
   integer i, k, lchnk,lat    ! column, level, chunk, latitude indices
   integer ncol               ! number of columns in current chunk
   integer ntmp               ! tem2porary
   integer yr, mon, day       ! components of a date
   integer ncsec              ! current time of day [seconds]
   integer idx

   character(len=256)  locfn_bp     ! local filename

   real(r8) calday            ! current calendar day
   real(r8) caldayloc         ! calendar day (includes yr if no cycling)
   real(r8) deltat            ! time (days) between interpolating sulfate data
   real(r8) fact1, fact2      ! time interpolation factors

   real(r8) xem2i   (plon,plat)  ! input sulfate bio mixing ratios

   integer dimids(nf_max_var_dims) ! netcdf variable shape  zf2016.02.22
   integer   :: nchar, nspecies, ispecies, em2_sp_id          !zf 2016.02.19
   character :: species_name*25                               !zf 2016.02.19
!--------------------------------------zf---------------------------------
   real(r8) xem2i_ap(plond,plat)  ! input sulfate bio mixing ratios
   real(r8) xem2i_bp(plond,plat)  ! input sulfate ant mixing ratios
   real(r8), allocatable :: xem2i_tem2pa(:,:)   
   real(r8), allocatable :: xem2i_tem2pb(:,:) 
   real(r8), allocatable :: xem2i_tem2pc(:,:)    !zf2016.02.22 
   integer istat                        !  error return
!--------------------------------------------------------------------------

   integer :: sul_id_ap, sul_id_bp
!-----------------------------------------------------------------------
!
! Use year information only if a multiyear dataset
!
   calday = get_curr_calday()
   call get_curr_date(yr, mon, day, ncsec)

   if ( rampyear_prescribed_em2 .ne. bigint ) then
      yr = rampyear_prescribed_em2        ! wzz, 2009.10.15
   endif
   caldayloc = calday + yr*365.

   if ((caldayloc.lt.bndry_m).or.(caldayloc.ge.bndry_p)) then
        outofbndry = .true.
   else
        outofbndry = .false.
   endif
   allocate( xem2i_tem2pa(lonem2iz,latem2iz), stat=istat )   !zf2010.11.08
   allocate( xem2i_tem2pb(lonem2iz,latem2iz), stat=istat )   !zf2010.11.08
   allocate( xem2i_tem2pc(lonem2iz,latem2iz), stat=istat )   !zf2016.22.22

!-------------------------------------------------------------------------------
!  there is no need to do anything when current model time is
!   out of the time-boundary of input data files
!-------------------------------------------------------------------------------
 if (.not.outofbndry) then
!
! Initialize hyperslab corners
!
   if (masterproc) then

      strt3(1) = 1
      strt3(2) = 1
      cnt3(1)  = lonem2iz  !zf
      cnt3(2)  = latem2iz  !zf
      cnt3(3)  = 1

   endif
!
! If model time is past current forward sulfate timeslice, read in the next
! timeslice for time interpolation. 

   if ( (caldayloc .gt. cdayem2p .and. rampyear_prescribed_em2 == bigint) .or. &
        (caldayloc .gt. cdayem2p .and. .not. (sp_p == 1 .and. caldayloc > cdayem2m))) then

      sp_m = sp_p
      sp_p = sp_p+1

      if ( sp_p.gt.nsample ) then     ! a new year
          sp_p = 1
          yrp  = yrp+1
          if ( rampyear_prescribed_em2 .ne. bigint ) then
             yrp = rampyear_prescribed_em2        ! wzz, 2009.10.15
          endif

          if (masterproc) then

            if ( yrp.gt.yr_bp ) then    ! need to locate the new year
             indxbp = indxbp+1
             yr_ap = yr_bp
             yr_bp = yr_em2(indxbp)

             ncid_em2_ap = ncid_em2_bp
             sul_id_ap = sul_id_bp

             em2pathfile_bp = trim(em2path)//trim(file_em2(indxbp))
             call getfil(em2pathfile_bp, locfn_bp)
             call wrap_open(locfn_bp, 0, ncid_em2_bp)
             write(6,*) 'Now open em2 file (locfn_bp):', locfn_bp 

             call wrap_inq_varid( ncid_em2_bp, trim(varname2d(1)), sul_id_bp )
            endif

          endif

      endif
      
      cdayem2m = cdayem2p
      cdayem2p = yrp*365.0+sample_em2(sp_p)

      cdaym = cdayp
      cdayp = sample_em2(sp_p)

      ntmp    = nmsul
      nmsul   = npsul
      npsul   = ntmp

    do idx = 1, nem2 

!------------
      if (masterproc) then
        strt3(3) = sp_p 
        xem2i_ap = 0.0
        xem2i_tem2pa = 0.0  !zf
        xem2i_tem2pb = 0.0  !zf
        xem2i_tem2pc = 0.0  !zf

        call wrap_inq_varid( ncid_em2_ap, trim(varname2d(idx)) , sul_id_ap )
!----------------zf 2016.02.19---------------------
      call wrap_inq_vardimid (ncid_em2_ap, sul_id_ap, dimids)
      call wrap_inq_dimlen (ncid_em2_ap, dimids(1), nchar)    !zf 2016.02.19
      call wrap_inq_dimlen (ncid_em2_ap, dimids(2), nspecies)    !zf 2016.02.19
      do ispecies = 1, nspecies
        call wrap_get_vara_text (ncid_em2_ap, sul_id_ap, (/1,ispecies/), (/nchar,1/), species_name)  !zf 2016.02.19
        call wrap_inq_varid( ncid_em2_ap, trim(species_name) , em2_sp_id )
        call wrap_get_vara_realx (ncid_em2_ap, em2_sp_id, strt3, cnt3, xem2i_tem2pc(:,:))  !zf 2016.02.19
#ifdef HAMOCC 
       ! note: at the present, the N2o and DMS flux from HAMOCC are not used
       !       and they still use the prescribved data

!        if ( trim(species_name) .eq. 'DMS_ocean' .or. trim(species_name) .eq. 'N2O_ocean') then
!             xem2i_tem2pc(:,:) = 0.
!        end if
#endif
        xem2i_tem2pa(:,:) = xem2i_tem2pa(:,:) + xem2i_tem2pc(:,:)
      end do
!-----------------------------------------------
!zf2016        call wrap_get_vara_realx (ncid_em2_ap,sul_id_ap,strt3,cnt3, &
!zf2016                                  xem2i_tem2pa(:,:))  !zf

        do i = 1, lonem2iz
        do lat = 1, latem2iz
          if( xem2i_tem2pa(i,lat) .gt. 1.e+30 ) xem2i_tem2pa(i,lat) = 0.0
          xem2i_tem2pb(i, lat) = xem2i_tem2pa(i, lat)
        enddo
        enddo
       
        if(lonem2iz .eq. plon .and. latem2iz .eq. plat) then
           xem2i_ap = xem2i_tem2pb
        else
           call bilin(xem2i_tem2pb,em2lon,em2lat,lonem2iz,lonem2iz,1,1,  &
                     latem2iz,xem2i_ap,londeg,latdeg,plond,nlon,1,plat)
        endif
!-----------------------------------------------
!
        xem2i_bp = 0.0
        xem2i_tem2pa = 0.0  !zf
        xem2i_tem2pb = 0.0  !zf
        xem2i_tem2pc = 0.0  !zf
        call wrap_inq_varid( ncid_em2_bp, trim(varname2d(idx)) , sul_id_bp )

!----------------zf 2016.02.19---------------------
      call wrap_inq_vardimid (ncid_em2_bp, sul_id_bp, dimids)
      call wrap_inq_dimlen (ncid_em2_bp, dimids(2), nspecies)    !zf 2016.02.19
      do ispecies = 1, nspecies
        call wrap_get_vara_text (ncid_em2_bp, sul_id_bp, (/1,ispecies/), (/nchar,1/), species_name)  !zf 2016.02.19
        call wrap_inq_varid( ncid_em2_bp, trim(species_name) , em2_sp_id )
        call wrap_get_vara_realx (ncid_em2_bp, em2_sp_id, strt3, cnt3, xem2i_tem2pc(:,:))  !zf 2016.02.19
#ifdef HAMOCC 
       ! note: at the present, the N2o and DMS flux from HAMOCC are not used
       !       and they still use the prescribved data

!        if ( trim(species_name) .eq. 'DMS_ocean' .or. trim(species_name) .eq. 'N2O_ocean') then
!             xem2i_tem2pc(:,:) = 0.
!        end if
#endif
        xem2i_tem2pa(:,:) = xem2i_tem2pa(:,:) + xem2i_tem2pc(:,:)
      end do
!-----------------------------------------------
!zf2016        call wrap_get_vara_realx (ncid_em2_bp,sul_id_bp,strt3,cnt3, &
!zf2016                                  xem2i_tem2pa(:,:))  !zf

        do i = 1, lonem2iz
        do lat = 1, latem2iz
           if( xem2i_tem2pa(i,lat) .gt. 1.e+30 ) xem2i_tem2pa(i,lat) = 0.0
           xem2i_tem2pb(i, lat) = xem2i_tem2pa(i, lat)
        enddo
        enddo
       
        if(lonem2iz .eq. plon .and. latem2iz .eq. plat) then
           xem2i_bp = xem2i_tem2pb
        else
           call bilin(xem2i_tem2pb,em2lon,em2lat,lonem2iz,lonem2iz,1,1, &
                      latem2iz,xem2i_bp,londeg,latdeg,plond,nlon,1,plat)
        endif
!-----------------------------------------------
!
! Normal interpolation between consecutive time slices.
!
        write(6,*) "yr_bp,yr_ap,yrp" , yr_bp,yr_ap,yrp
        deltat = dble( yr_bp - yr_ap )
        fact1  = dble( yr_bp - yrp  )/deltat
        fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'em2INT--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
        do lat = 1,plat
          do i = 1,plon
             xem2i(i,lat)= fact1*xem2i_ap(i,lat)+fact2*xem2i_bp(i,lat)
          enddo
        enddo
!
! master does the work above
!
      endif  ! end masterproc

      call scatter_field_to_chunk(1,1,1,plon,xem2i,em2i(1,begchunk,npsul,idx))

    enddo

   end if  ! 
 
!--------------------------------------------------------------------------
! Determine time interpolation factor.   
!
   if ( rampyear_prescribed_em2 .ne. bigint ) then   ! wzz, 2009.10.20
        call getfactors (.true., sp_p, sample_em2(sp_m), sample_em2(sp_p), calday, &
                          fact1, fact2, 'em2int_IPCC:')
   else
        call getfactors (.false., sp_p, cdayem2m, cdayem2p, caldayloc, &
                          fact1, fact2, 'em2int_IPCC:')
   endif
!
! Time interpolation.
!
   do lchnk=begchunk,endchunk
      ncol = get_ncols_p(lchnk)
         do i=1,ncol
            do idx = 1, nem2
                em2(i,lchnk,idx) = em2i(i,lchnk,nmsul,idx)*fact1 +  &
                                   em2i(i,lchnk,npsul,idx)*fact2
            enddo
         end do
   end do
!
!----------------------------------------------------------------------------
 endif
!-----------------------------------------------------------
   deallocate (xem2i_tem2pa)   !zf2010.11.08
   deallocate (xem2i_tem2pb)   !zf2010.11.08
   deallocate (xem2i_tem2pc)   !zf2016.02.22

   return
end subroutine em2int_IPCC

subroutine getem2_IPCC( lchnk, ncol, emiss2 )

!-----------------------------------------------------------------------
!
! Purpose: Return slice of time interpolated sulfate aerosol.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
!
! Arguments
!
  integer , intent(in)   :: lchnk            ! chunk identifier
  integer , intent(in)   :: ncol             ! number of atmospheric columns
  real(r8), intent(out)  :: emiss2(pcols,nem2)  ! biogenic sulfate
!
! Local variables.
!
   integer :: i, k, n                 ! longitude, level indices
   integer :: L, L1, L2
!-----------------------------------------------------------------------
   do n = 1, nem2
      do i = 1, ncol
         emiss2(i,n) = em2(i,lchnk,n)
      end do
   enddo

   return
end subroutine getem2_IPCC

!###############################################################################
end module prescribed_em_d2
#endif

