#include <misc.h>
#include <params.h>

#ifdef MOZART2

module prescribed_em_d2_anthro

!----------------------------------------------------------------------- 
! Purpose:
!
! Interpolation of datasets.
!-----------------------------------------------------------------------

   use shr_kind_mod,   only: r8 => shr_kind_r8
   use pmgrid,         only: plon, plat, masterproc
   use ppgrid,         only: begchunk, endchunk, pcols, pver
   use phys_grid,      only: scatter_field_to_chunk
   use time_manager,   only: get_curr_date, get_curr_calday
   use abortutils,     only: endrun

#if ( defined SPMD )
   use mpishorthand
#endif

   implicit none

   public em2ini_IPCC_anthro
   public em2int_IPCC_anthro
   public getem2_IPCC_anthro
 
   logical, public  :: emis2dcyc = .false.

   integer, public,parameter :: nem2_anthro    = 21
   character*8,public :: varname2d (nem2_anthro)
   data varname2d / 'CH3COCHO', 'C2H4    ', 'C2H5OH  ', 'C2H6    ', 'C3H6    ', &
                    'C3H8    ', 'CH2O    ', 'CH3CHO  ', 'CH3OH   ', 'N2O     ', &
                    'CO      ',                         'NO      ', 'C4H10   ', &
                    'CH4     ',                                                 &
                                'SO2     ', 'CB1     ', 'CB2     ', 'OC1     ', &
                    'OC2     ', 'NH3     ', 'Terpenes' /
   integer, public, parameter :: &
          idxCH3COCHO_anthro =  1, &
          idxC2H4_anthro     =  2, &
          idxC2H5OH_anthro   =  3, &
          idxC2H6_anthro     =  4, &
          idxC3H6_anthro     =  5, &
          idxC3H8_anthro     =  6, &
          idxCH2O_anthro     =  7, &
          idxCH3CHO_anthro   =  8, &
          idxCH3OH_anthro    =  9, &
          idxN2O_anthro      = 10, &
          idxCO_anthro       = 11, &
          idxNO_anthro       = 12, &
          idxC4H10_anthro    = 13, &
          idxCH4_anthro      = 14, &
          idxSO2_anthro      = 15, &
          idxCB1_anthro      = 16, &
          idxCB2_anthro      = 17, &
          idxOC1_anthro      = 18, &
          idxOC2_anthro      = 19, &
          idxNH3_anthro      = 20, &
          idxC10H16_anthro   = 21, &
          idxH2_anthro       = -1, &
          idxISOP_anthro     = -1, &
          idxDMS_anthro      = -1, &
          idxCH3COCH3_anthro = -1

   save

! Private data

  real(r8), private :: cdaym     ! dataset calendar day previous month
  real(r8), private :: cdayp     ! dataset calendar day next month

  integer  :: np1           ! current forward time index of ozone dataset
  integer  :: ncid_anthro   ! netcdf ID for anthro emission
  integer  :: ncid_bb       ! netcdf ID for bb emission
  integer  :: em2d_id       ! netcdf id for emission variable
  integer  :: anthro_id     ! netcdf id for emission variable
  integer  :: bb_id         ! netcdf id for emission variable
  integer  :: timesiz   ! size of time dimension on ozone dataset

  real(r8), private, allocatable :: emis_anth(:, :, :)
  real(r8), private, allocatable :: time_emis(:)   ! Date on dataset (days since 0000 yr) 

!------
!
  real(r8), private, allocatable  :: emis_2d(:,:,:,:)    ! (pcols,begchunk:endchunk,2,nem2_anthro)

!===============================================================================
contains
!===============================================================================

   subroutine em2ini_IPCC_anthro
   use mo_tracinp,        only : bndic
   use iofilemod,          only : getfil
!
! Local workspace
!
   integer londimid                        ! netcdf id for longitude dimension
   integer latdimid                        ! netcdf id for latitude dimension
   integer timeid                          ! netcdf id for time variable
   integer dimids(3)                       ! variable shape
   integer cnt3(3)                         ! array of counts for each dimension
   integer strt3(3)                        ! array of starting indices
   integer i, j, k, lat, n                    ! longitude, level, latitude, time indices
   integer  :: yr, mon, day                ! components of a date
   integer  :: ncsec                       ! current time of day [seconds]
   integer  :: istat
   real(r8) :: calday                      ! current calendar day
   real(r8) :: caldayloc                     ! calendar day (includes yr if no cycling)

   character(len=256) :: locfn             ! netcdf local filename to open 
   character(len=256) :: filename
   character(len=25)  :: species_name
   character(len=50)  :: units
  
   integer latsiz, lonsiz, idx
   integer nchar, nspecies, ispecies, varlen

   real(r8), allocatable  ::  anthro(:,:), bb(:,:)
   real(r8), allocatable  ::  emis_2dbdym(:,:), emis_2dbdyp(:,:)
!-----------------------------------------------------------------------
!
! Allocate space for data.
!
  allocate( emis_anth(pcols,begchunk:endchunk,nem2_anthro), stat=istat )
!
! SPMD: Master does all the work. Sends needed info to slaves
!
   units(:) = ' '
   if (masterproc) then

! need declaration for locfn

      do i = len_trim(bndic), 1, -1
         if(bndic(i:i).eq.'/') exit
      enddo

      filename = trim(bndic(1:i))//'IPCC6/CMIP6_surface_emission_bb_185001-201512_c20170106.nc'
!      filename = '/cmb/g4/zhangf/VERSIONS-20160901/emissions/CMIP6_surface_emission_bb_185001-201512_c20170106.nc'
      call getfil (filename, locfn, 0)
      call wrap_open (locfn, 0, ncid_bb)

      filename = trim(bndic(1:i))//'IPCC6/CMIP6_surface_emission_anthro_185001-201412_c20170106.nc'
!      filename = '/cmb/g4/zhangf/VERSIONS-20160901/emissions/CMIP6_surface_emission_anthro_185001-201412_c20170106.nc'
      call getfil (filename, locfn, 0)
      call wrap_open (locfn, 0, ncid_anthro)
!
! Use year information only if not cycling ozone dataset
!
      calday = get_curr_calday()
      call get_curr_date(yr, mon, day, ncsec)

!-------zf 2016.06.12
      if (emis2dcyc) then
        caldayloc = calday
      else
        caldayloc = calday + yr*365.  
      end if
!
! Get and check dimension info
!
      CALL WRAP_INQ_DIMID( ncid_bb, 'lon', londimid   )
      CALL WRAP_INQ_DIMID( ncid_bb, 'lat', latdimid   )
      CALL WRAP_INQ_DIMID( ncid_bb, 'time', timeid  )

      CALL WRAP_INQ_DIMLEN( ncid_bb, londimid, lonsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_bb, latdimid, latsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_bb, timeid, timesiz   )

      CALL WRAP_INQ_VARID( ncid_bb, trim(varname2d(1))//'_bb', em2d_id   )

      CALL WRAP_INQ_VARDIMID (ncid_bb, em2d_id, dimids)
      if (dimids(1) /= londimid .and. dimids(2) /= latdimid) then
         write(6,*)'em2ini_IPCC_anthro: Data must be ordered lon, lat, time'
         call endrun
      end if
      if (lonsiz .ne. plon .or. latsiz .ne. plat ) then
         call endrun ('em2ini_IPCC_anthro:  The lonsiz and latsiz are not the same as the model resolution')
      end if

      CALL WRAP_INQ_VARID( ncid_bb, 'time', timeid   )
      CALL WRAP_GET_ATT_TEXT(ncid_bb, timeid, 'units', units )
      if ( trim(units) /= 'days since 1850-01-01 0:0:0' ) then
         call endrun ('em2ini_IPCC_anthro:  The time units are not days since 1850-01-01')
      end if
   end if   ! masterproc

#if (defined SPMD )
   call mpibcast (timesiz, 1, mpiint, 0, mpicom)
#endif
!
! Dynamically allocated memory for module data
!
   allocate (bb(plon, plat), stat=istat)
   allocate (anthro(plon, plat), stat=istat)
   allocate (emis_2dbdym(plon, plat), stat=istat)
   allocate (emis_2dbdyp(plon, plat), stat=istat)
   allocate( time_emis(timesiz),                     &
             emis_2d(pcols,begchunk:endchunk,2,nem2_anthro), stat=istat )

   if (masterproc) then
!
! Retrieve entire date and sec variables.
!
      CALL WRAP_GET_VAR_REALX (ncid_bb,timeid,time_emis)

      strt3(1) = 1
      strt3(2) = 1
      cnt3(1)  = lonsiz
      cnt3(2)  = latsiz
      cnt3(3)  = 1
   endif   ! materproc

!
! Special code for interpolation between December and January
!
   do idx = 1, nem2_anthro
   if (masterproc) then
      call wrap_inq_varid( ncid_bb, trim(varname2d(idx)), em2d_id   )
      call wrap_inq_vardimid (ncid_bb, em2d_id, dimids)
      call wrap_inq_dimlen (ncid_bb, dimids(1), nchar)    
      call wrap_inq_dimlen (ncid_bb, dimids(2), nspecies) 

      bb = 0.
      anthro = 0.
      varlen = len_trim(varname2d(idx))

      if (emis2dcyc) then
        n = 12
        np1 = 1
         cdaym = time_emis(n)
         cdayp = time_emis(np1)
         if (caldayloc <= cdayp .or. caldayloc > cdaym) then
            strt3(3) = n
            do ispecies = 1, nspecies
               call wrap_get_vara_text (ncid_bb, em2d_id, (/1,ispecies/), (/nchar,1/), species_name)  

               if (species_name(varlen+2:varlen+3) .eq. 'bb') then
                  call wrap_inq_varid( ncid_bb, trim(species_name) , bb_id)
                  call wrap_get_vara_realx (ncid_bb, bb_id, strt3, cnt3, bb)  
               else
                  call wrap_inq_varid( ncid_anthro, trim(species_name) , anthro_id)
                  call wrap_get_vara_realx (ncid_anthro, anthro_id, strt3, cnt3, anthro)
               endif
            end do

            emis_2dbdym = anthro + bb
            anthro = 0.
            bb = 0.
 
            strt3(3) = np1
            do ispecies = 1, nspecies
               call wrap_get_vara_text (ncid_bb, em2d_id, (/1,ispecies/), (/nchar,1/), species_name)  

               if (species_name(varlen+2:varlen+3) .eq. 'bb') then
                  call wrap_inq_varid( ncid_bb, trim(species_name) , bb_id)
                  call wrap_get_vara_realx (ncid_bb, bb_id, strt3, cnt3, bb)  
               else
                  call wrap_inq_varid( ncid_anthro, trim(species_name) , anthro_id)
                  call wrap_get_vara_realx (ncid_anthro, anthro_id, strt3, cnt3, anthro)
               endif
            end do

            emis_2dbdyp = anthro + bb
            anthro = 0.
            bb = 0.
 
            goto 10
         end if
      end if
!
! Normal interpolation between consecutive time slices.
!
      do n=1,timesiz-1
         np1 = n + 1
         cdaym = time_emis(n)
         cdayp = time_emis(n+1)
 
!---------zf 2016.06.13
         if (.not.emis2dcyc ) then
           cdaym = cdaym + 365.0*1850.0 
           cdayp = cdayp + 365.0*1850.0 
         end if
!---------------
 
         if (caldayloc >= cdaym .and. caldayloc < cdayp) then
            strt3(3) = n
            do ispecies = 1, nspecies
               call wrap_get_vara_text (ncid_bb, em2d_id, (/1,ispecies/), (/nchar,1/), species_name)  

               if (species_name(varlen+2:varlen+3) .eq. 'bb') then
                  call wrap_inq_varid( ncid_bb, trim(species_name) , bb_id)
                  call wrap_get_vara_realx (ncid_bb, bb_id, strt3, cnt3, bb)  
               else
                  call wrap_inq_varid( ncid_anthro, trim(species_name) , anthro_id)
                  call wrap_get_vara_realx (ncid_anthro, anthro_id, strt3, cnt3, anthro)
               endif
            end do

            emis_2dbdym = anthro + bb
            anthro = 0.
            bb = 0.
 
            strt3(3) = np1
            do ispecies = 1, nspecies
               call wrap_get_vara_text (ncid_bb, em2d_id, (/1,ispecies/), (/nchar,1/), species_name)  

               if (species_name(varlen+2:varlen+3) .eq. 'bb') then
                  call wrap_inq_varid( ncid_bb, trim(species_name) , bb_id)
                  call wrap_get_vara_realx (ncid_bb, bb_id, strt3, cnt3, bb)  
               else
                  call wrap_inq_varid( ncid_anthro, trim(species_name) , anthro_id)
                  call wrap_get_vara_realx (ncid_anthro, anthro_id, strt3, cnt3, anthro)
               endif
            end do

            emis_2dbdyp = anthro + bb
            anthro = 0.
            bb = 0.
 
            goto 10
         end if
      end do

      write(6,*)'em2ini_IPCC_anthro: Failed to find dates bracketing , year, month, day',  yr, mon, day
      call endrun
!------------------------------------------------
10    continue
 
   endif   ! masterproc

   call scatter_field_to_chunk(1,1,1,plon,emis_2dbdym,emis_2d(1,begchunk,1,idx))
   call scatter_field_to_chunk(1,1,1,plon,emis_2dbdyp,emis_2d(1,begchunk,2,idx))

   end do  ! idx 

#if (defined SPMD )
   call mpibcast (np1, 1, mpiint, 0, mpicom)
   call mpibcast (time_emis, timesiz, mpir8, 0, mpicom )
   call mpibcast (cdaym, 1, mpir8, 0, mpicom)
   call mpibcast (cdayp, 1, mpir8, 0, mpicom)
#endif

   deallocate (bb)
   deallocate (anthro)
   deallocate (emis_2dbdym)
   deallocate (emis_2dbdyp)

end subroutine em2ini_IPCC_anthro

!===============================================================================

subroutine em2int_IPCC_anthro
!----------------------------------------------------------------------- 
! 
! Purpose: Interpolate ozone mixing ratios to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
! Method: Find next month of ozone data to interpolate.  Linearly interpolate 
!         vertically and horizontally
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use timeinterp,   only: getfactors
   use phys_grid,    only: get_ncols_p
   use time_manager, only: get_curr_date
!
! Local workspace
!
   integer cnt3(3)                ! array of counts for each dimension
   integer strt3(3)               ! array of starting indices
   integer i                      ! longitude indices
   integer idx                    
   integer c, ncol                ! chunk index, number of columns in chunk
   integer :: yr, mon, day        ! components of a date
   integer :: ncsec               ! current time of day [seconds]

   real(r8) fact1, fact2          ! time interpolation factors
   real(r8) :: calday             ! current calendar day
   real(r8) caldayloc             ! calendar day (includes yr if no cycling)

   integer  :: nchar, nspecies, ispecies, varlen
   integer  :: istat
   integer  :: dimids(3)                       ! variable shape

   real(r8), allocatable  ::  anthro(:,:), bb(:,:)
   real(r8), allocatable  ::  emis_2dbdyp(:,:)

   character(len=25)  :: species_name
!-----------------------------------------------------------------------
!
! Use year information only if a multiyear dataset
!
   calday    = get_curr_calday()
   call get_curr_date(yr, mon, day, ncsec)

!----------zf2016.06.13
   if (emis2dcyc) then
     caldayloc = calday
   else
     caldayloc = calday + yr*365.
   end if
!
! If model time is past current forward ozone timeslice, then
! masterproc reads in the next timeslice for time interpolation.  Messy logic is 
! for emis2dcyc = .true. interpolation between December and January (np1 == 1).  Note 
! that np1 is never 1 when emis2dcyc is .false.
!
   if (caldayloc > cdayp .and. .not. (np1 == 1 .and. caldayloc > cdaym)) then

      do idx = 1, nem2_anthro
        emis_2d(:, :, 1, idx) =  emis_2d(:, :, 2, idx) 
      end do

      if (masterproc) then
         if (emis2dcyc) then
           np1 = mod(np1,12) + 1           !zf2016.05.13
         else
           np1 = np1 + 1
         end if

         if (np1 > timesiz) then
            call endrun ('em2int_IPCC_anthro: Attempt to read past end of dataset')
         end if

         cdaym = cdayp
         cdayp = time_emis(np1)
         if ( .not.emis2dcyc) then
           cdayp = cdayp + 365.0*1850.0           !zf2016.05.13
         end if

         if (.not. (np1 == 1 .or. caldayloc <= cdayp)) then
            if (masterproc) then
               write(6,*)'em2int_IPCC_anthro: Input emis for date',time_emis(np1), &
                        'does not exceed model date',' Stopping.'
            endif
            call endrun ()
         end if

         strt3(1) = 1
         strt3(2) = 1
         strt3(3) = np1
         cnt3(1)  = plon
         cnt3(2)  = plat
         cnt3(3)  = 1
      end if  ! masterproc

      allocate (bb(plon, plat), stat=istat)
      allocate (anthro(plon, plat), stat=istat)
      allocate (emis_2dbdyp(plon, plat), stat=istat)

      do idx = 1, nem2_anthro
      if (masterproc) then
!
! Allocate memory for more dynamic local workspace
!
      call wrap_inq_varid( ncid_bb, trim(varname2d(idx)), em2d_id   )
      call wrap_inq_vardimid (ncid_bb, em2d_id, dimids)
      call wrap_inq_dimlen (ncid_bb, dimids(1), nchar)    
      call wrap_inq_dimlen (ncid_bb, dimids(2), nspecies) 

      bb = 0.
      anthro = 0.
      varlen = len_trim(varname2d(idx))

         do ispecies = 1, nspecies
            call wrap_get_vara_text (ncid_bb, em2d_id, (/1,ispecies/), (/nchar,1/), species_name)  

            if (species_name(varlen+2:varlen+3) .eq. 'bb') then
               call wrap_inq_varid( ncid_bb, trim(species_name) , bb_id)
               call wrap_get_vara_realx (ncid_bb, bb_id, strt3, cnt3, bb)  
            else
               call wrap_inq_varid( ncid_anthro, trim(species_name) , anthro_id)
               call wrap_get_vara_realx (ncid_anthro, anthro_id, strt3, cnt3, anthro)
            endif
         end do

         emis_2dbdyp = anthro + bb
         anthro = 0.
         bb = 0.
 
         write(6,*)'em2int_IPCC_anthro: Read surface emission for date (days) ', time_emis(np1)
      end if  ! masterproc

#if (defined SPMD )
     call mpibcast (np1, 1, mpiint, 0, mpicom)
     call mpibcast (cdaym, 1, mpir8, 0, mpicom)
     call mpibcast (cdayp, 1, mpir8, 0, mpicom)
#endif

      call scatter_field_to_chunk(1,1,1,plon,emis_2dbdyp,emis_2d(1,begchunk,2,idx) )
    end do   ! idx

   deallocate (bb)
   deallocate (anthro)
   deallocate (emis_2dbdyp)

   end if

!
! Determine time interpolation factors.
!
   call getfactors (emis2dcyc, np1, cdaym, cdayp, caldayloc, &
                    fact1, fact2, 'em2d_data_TIMESTEP_INIT:')
!
! Time interpolation.
!
   do c=begchunk,endchunk
      ncol = get_ncols_p(c)
      do i=1,ncol
        do idx=1,nem2_anthro
          emis_anth(i,c,idx) = emis_2d(i,c,1,idx)*fact1 + emis_2d(i,c,2,idx)*fact2
        end do
      end do
   end do

end subroutine em2int_IPCC_anthro

!*********************************
 subroutine getem2_IPCC_anthro( lchnk, ncol, emis2_anthro )

!-----------------------------------------------------------------------
!
! Purpose: Return slice of time interpolated emissions.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
!
! Arguments
!
  integer , intent(in)   :: lchnk            ! chunk identifier
  integer , intent(in)   :: ncol             ! number of atmospheric columns
  real(r8), intent(out)  :: emis2_anthro(pcols,nem2_anthro)  ! emissions
!
! Local variables.
!
   integer :: i, k, n                 ! longitude, level indices
   integer :: L, L1, L2
!-----------------------------------------------------------------------
   do n = 1, nem2_anthro
      do i = 1, ncol
         emis2_anthro(i,n) = emis_anth(i,lchnk,n)
      end do
   enddo

   return
 end subroutine getem2_IPCC_anthro

end module prescribed_em_d2_anthro

#endif
