#include <misc.h> 
#include <params.h>

#if (defined MOZART2) 
module prescribed_em_d3_aero
!
!-----------------------------------------------------------------------

   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid,       only: pcols, pver, pverp, begchunk, endchunk
   use pmgrid,       only: plon, plat, plev, plevp, masterproc
   use phys_grid,    only: get_ncols_p, scatter_field_to_chunk
   use time_manager, only: get_curr_calday, get_curr_date
   use infnan,       only: inf, bigint
   use abortutils,   only: endrun
   use filenames,    only: bndtvaer

#if (defined MOZART2)
   use mo_tracinp,  only: bndic
#endif

!---------------------------zf 2010.11.08------------------------------
   use rgrid,          only: nlon
   use commap,         only: latdeg, londeg
   use pmgrid,         only: plond
!-----------------------------------------------------------------------
#if ( defined SPMD )
   use mpishorthand, only: mpicom, mpir8
#endif

#if ( defined SCAM )
   use getnetcdfdata
#endif

   implicit none

#if (defined MOZART2)
!
!     so2_vrt_emis_file    =atm/cam/chem/trop_mozart_aero/emis/SO2_vrt_emis_1.9x2.5_1850-2000_c070330.nc
!     so4_vrt_emis_file    =atm/cam/chem/trop_mozart_aero/emis/SO4_vrt_emis_1.9x2.5_1850-2000_c070330.nc
!     no2_aircraft_emis_file = atm/cam/chem/1850-2000_emis/IPCC_emissions_aircraft_NO2_1850-2000_1.9x2.5_c090729.nc
!     bc_aircraft_emis_file  = atm/cam/chem/1850-2000_emis/IPCC_emissions_aircraft_BC_1850-2000_1.9x2.5.c090729.nc
!
   integer    ,public,parameter :: levsiz  = 25     ! emiss data vertical layers
   real(r8)   ,public           :: em3lev(levsiz)

   integer    ,public,parameter :: nem3    = 4
   character*9,public,parameter :: varname3d (nem3) = (/ &
                        'NO2', 'SO2', 'CB2', 'SO4' /)
   integer, public, parameter :: &
          idx3_NO2   =  1, &
          idx3_SO2   =  2, &
          idx3_CB2   =  3, &
          idx3_SO4   =  4
   integer, public     :: nem3_aero = nem3
   character*9, public :: varname3d_aero (nem3) 

!   varname3d_aero = varname3d 
!
#endif

! compute carbons scaling from population

  integer, public           :: rampyear_prescribed_em3   =  bigint ! this is the only acceptable value
  character(len=16), public :: scenario_prescribed_em3   = 'FIXED' ! 'FIXED' or 'RAMPED'

  private
  save

  integer  :: em3nid = -1           ! netcdf id for em3ol file (init to invalid)

  real(r8) :: cdaym = inf          ! calendar day of prv month
  real(r8) :: cdayp = inf          ! calendar day of next month
  
  public  em3ini_IPCC_aero
  public  em3int_IPCC_aero
  public  getem3_IPCC_aero
!
! for IPCC data 
!
   integer,private   :: lonem3iz     ! size of longitude dimension on em3ol dataset
   integer,private   :: latem3iz     ! size of latitude dimension on em3ol dataset
   integer,private   :: levem3iz     ! size of vertical dimension on em3ol dataset

   real(r8),private, allocatable :: em3lon(:)
   real(r8),private, allocatable :: em3lat(:)
   real(r8),private,allocatable,dimension(:,:,:,:,:) ::  em3i
                      ! input sulfate bio mixing ratios (pcols,levsiz,begchunk:endchunk,2,nem3)
   real(r8),private,allocatable,dimension(:,:,:,:)   ::  em3
                      ! time interpolated sulfate bio mixing ratios (pcols,levsiz,begchunk:endchunk,nem3)

   real(r8), private :: cdayem3m        ! calendar day for prv. month sulfate values read in
   real(r8), private :: cdayem3p        ! calendar day for nxt. month sulfate values read in

   real(r8),private :: bndry_m          ! calendar day for the lower time-boundary of inputdata
   real(r8),private :: bndry_p          ! calendar day for the upper time-boundary of inputdata
   logical, private :: outofbndry       ! whether current model time is out of the time-boundary

#if (defined MOZART2)
   integer,parameter :: nfile = 20    ! number of input data files
   integer,parameter :: yr_em3(nfile) = (/ 1850, &
                      1910, 1915, 1920, 1925, 1930, 1935, 1940, 1945, 1950, 1955, &
                      1960, 1965, 1970, 1975, 1980, 1985, 1990, 1995, 2000  /)
#endif

   integer, private :: nmsul              ! index to prv month in array. init to 1 and toggle between 1 and 2
   integer, private :: npsul              ! index to nxt month in array. init to 2 and toggle between 1 and 2

   character*256,private :: file_em3(nfile)

   integer, private,parameter :: nsample = 12   ! number of samples in each input data file

   real(r8),private           :: sample_em3(nsample) =      (/&
                1296000.0,  3888000.0,  6393600.0,  9072000.0, &
               11664000.0, 14342400.0, 16934400.0, 19612800.0, &  
               22291200.0, 24883200.0, 27561600.0, 30153600.0 /)
!
! Integer data
!
   integer, private :: sp_m, sp_p
   integer, private :: yrm,  yrp
   integer, private :: yr_am, yr_bm, yr_ap, yr_bp, indxbp
!
   integer, private :: ncid_em3_ap, ncid_em3_bp  ! sulfate dataset id

   character*80,  private ::  em3datapath=''        ! full pathname for sulfate dataset
   character*256, private ::  em3datapathfile_bp    ! full pathname for sulfate dataset

contains

subroutine em3ini_IPCC_aero
!----------------------------------------------------------------------- 
! 
! Purpose: Do initial read of time-variant em3ol dataset, containing
!          em3ol mixing ratios as a function of time.  It is currently
!          required that the em3ol dataset have the *SAME* horizontal
!          and vertical resolution as the model. Therefore, ONLY a time
!          interpolation of the dataset is currently performed.
! 
!-----------------------------------------------------------------------
   use ioFileMod,      only: getfil
   use error_messages, only: alloc_err, handle_ncerr
   use time_manager,   only: get_curr_date, get_perp_date, get_curr_calday, &
                             is_perpetual
   use dycore,         only: get_resolution

   implicit none

   include 'netcdf.inc'
!
   integer :: ncid_em3_am=-1, ncid_em3_bm  ! sulfate dataset id
   integer :: em3_id_ap,   em3_id_bp
   integer :: em3_id_am,   em3_id_bm 
 
   integer :: lonsiz     ! size of longitude dimension on sulfate dataset
   integer :: latsiz     ! size of latitude dimension on sulfate dataset
   integer :: timsiz     ! size of time dimension on sulfate dataset

   integer :: idx

   character*256 ::  em3datapathfile_am ! full pathname for sulfate dataset
   character*256 ::  em3datapathfile_bm ! full pathname for sulfate dataset
   character*256 ::  em3datapathfile_ap ! full pathname for sulfate dataset

#if (defined MOZART2) 
   character*80, parameter ::  p1t42='IPCC-var3d/var3d_' 
   character*80, parameter ::  p2='.nc'     ! part of the name of sulfate data file
#endif

   character(len=256) :: locfn          ! netcdf local filename to open   
!-----------------------------------------------------------------------
!
! Local workspace
!
   character(len=256) :: locfn_am, locfn_bm, locfn_ap, locfn_bp     ! local filenames

   integer londimid              ! netcdf id for longitude dimension
   integer latdimid              ! netcdf id for latitude dimension
   integer levdimid              ! netcdf id for level dimension
   integer lonid                 ! netcdf id for longitude variable
   integer latid                 ! netcdf id for latitude variable
   integer levid                 ! netcdf id for level variable
   integer timeid                ! netcdf id for time variable
   integer cnt4(4)               ! array of counts for each dimension
   integer strt4(4)              ! array of starting indices
   integer i, k, lat, n          ! longitude, level, latitude, time indices
   integer istat                 ! error return
   integer dimids(nf_max_var_dims) ! netcdf variable shape

   integer yr, mon, day, ncsec ! components of a date
   real(r8) calday                   ! current calendar day
   real(r8) caldayloc                ! calendar day (includes yr if no cycling)
   real(r8) fact1, fact2, deltat

   character*4 yrtmp

   real(r8) xem3i_am(plond,levsiz,plat)
   real(r8) xem3i_bm(plond,levsiz,plat)
   real(r8) xem3i_ap(plond,levsiz,plat)
   real(r8) xem3i_bp(plond,levsiz,plat)
   
   real(r8)    xem3i(plon,levsiz,plat,2)

   real(r8), allocatable :: xem3i_tem3pa(:,:,:) 
   real(r8), allocatable :: xem3i_tem3pb(:,:,:)
!
!-----------------------------------------------------------------------
   varname3d_aero = varname3d
!
! Allocate space for data.
!
  allocate( em3i(pcols,levsiz,begchunk:endchunk,2,nem3), stat=istat )
  call alloc_err( istat, 'em3ini', 'em3i', pcols*levsiz*(endchunk-begchunk+1)*2*nem3 )

  allocate( em3(pcols,levsiz,begchunk:endchunk,nem3), stat=istat )
  call alloc_err( istat, 'em3ini', 'em3',  pcols*levsiz*(endchunk-begchunk+1)*nem3 )

  nmsul = 1
  npsul = 2
!
! unit shift from second to day
!
   do n=1,nsample
      sample_em3(n) = sample_em3(n)/86400.0+1.0
   enddo
!
! the names of rawdata files
!
   yrtmp = '1950'
   do n=1,nfile
      write(yrtmp,'(i4.4)') yr_em3(n)
      file_em3(n) = trim(p1t42)//yrtmp//trim(p2)
   enddo

! set the time_boundaries of input dataset
!
   bndry_m = yr_em3(1    )*365.0 + sample_em3(1) 
   bndry_p = yr_em3(nfile)*365.0 + sample_em3(nsample) 

#if (defined MOZART2) 
   do i = len_trim(bndic), 1, -1
       if(bndic(i:i).eq.'/') exit
   enddo
   em3datapath = bndic(1:i)
#endif
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
     calday = get_curr_calday()
     call get_curr_date(yr, mon, day, ncsec)
     if ( rampyear_prescribed_em3 .ne. bigint ) then
        yr = rampyear_prescribed_em3        ! wzz, 2009.10.15
     endif
     caldayloc = calday + yr*365.

!----------------------------------------------------------------
! locate the model time among the samples of sulfate data file      
!---------------------------------------------------------------
     if ( caldayloc.lt.bndry_m ) then

        outofbndry = .true.
        cdayem3p  = bndry_m
        cdayp     = sample_em3(1)    !???
        sp_p   = 1
        yrp    = yr_em3(1)
        yr_ap  = yr_em3(1)
        yr_bp  = yr_em3(2)
        indxbp = 2
        em3datapathfile_ap = trim(em3datapath)//file_em3(1)
        em3datapathfile_bp = trim(em3datapath)//file_em3(2)

     elseif ( caldayloc.ge.bndry_p ) then

        outofbndry = .true.
        sp_p   = nsample 
        yrp    = yr_em3(nfile)
        yr_ap  = yr_em3(nfile)
        yr_bp  = yr_em3(nfile)
        em3datapathfile_ap = trim(em3datapath)//file_em3(nfile)

     else
        outofbndry = .false.
!
! month
!
        if ( calday.lt.sample_em3(1) ) then
          yrm  = yr-1
          yrp  = yr
          sp_m = nsample 
          sp_p = 1
          goto 100
        elseif (calday.ge.sample_em3(nsample) ) then
          yrm = yr
          yrp = yr+1
          sp_m = nsample
          sp_p = 1 
          goto 100
        else 
          do n=1,nsample-1
            if ( calday.ge.sample_em3(n) .and. calday.lt.sample_em3(n+1) ) then
               yrm  = yr
               yrp  = yr
               sp_m = n
               sp_p = n+1
               goto 100
            endif
          enddo
        endif
        write(*,*) 'em3INI: Failed to find seconds bracketing model_time =',calday
        call endrun
100     continue
        if ( rampyear_prescribed_em3 .ne. bigint ) then
          yrm = rampyear_prescribed_em3        ! wzz, 2009.10.15
          yrp = rampyear_prescribed_em3        ! wzz, 2009.10.15
        endif
        cdayem3m = yrm*365.0+sample_em3(sp_m)
        cdayem3p = yrp*365.0+sample_em3(sp_p)
        cdaym = sample_em3(sp_m)
        cdayp = sample_em3(sp_p)
!
! year ( both yrm & yrp )
!
        do n=1,nfile-1
          yr_am = yr_em3(n)
          yr_bm = yr_em3(n+1)
          if ( yrm.ge.yr_am .and. yrm.lt.yr_bm ) goto 200
        enddo
        write(*,*) 'em3INI: Failed to find years bracketing year  yrm=',yrm
        call endrun
200     continue
        em3datapathfile_am = trim(em3datapath)//trim(file_em3(n))
        em3datapathfile_bm = trim(em3datapath)//trim(file_em3(n+1))
 
        do n=1,nfile-1
          yr_ap = yr_em3(n)
          yr_bp = yr_em3(n+1)
          if ( yrp.ge.yr_ap .and. yrp.lt.yr_bp ) goto 300
        enddo
        write(*,*) 'em3INI: Failed to find years bracketing year  yrp=',yrp
        call endrun
300     indxbp = n+1 
        em3datapathfile_ap = trim(em3datapath)//trim(file_em3(n))
        em3datapathfile_bp = trim(em3datapath)//trim(file_em3(n+1))

     endif

  if (masterproc) then
!--------------------------------------------------------------------------
! open the right data files and read in raw data
!--------------------------------------------------------------------------
     if ( outofbndry ) then

        call getfil(em3datapathfile_ap, locfn_ap)
        call wrap_open(locfn_ap, 0, ncid_em3_ap)
      
        write(6,*) 'em3datapathfile_ap =', em3datapathfile_ap
        write(6,*) 'Now open em3 file (locfn_ap):', locfn_ap
!
! Obtain dimension id's
!
        call wrap_inq_dimid( ncid_em3_ap, 'lon', londimid)
        call wrap_inq_dimid( ncid_em3_ap, 'lat', latdimid)
        call wrap_inq_dimid( ncid_em3_ap, 'lev', levdimid)
        call wrap_inq_dimid( ncid_em3_ap, 'time',timeid  )
!
! Obtain size of dimensions.
! Check that horizontal and vertical dimensions are same as model's
! (check one file only and assume that all the other files are of the same setting)
!
        call wrap_inq_dimlen( ncid_em3_ap, londimid, lonem3iz   ) !zf 2010.11.10
        call wrap_inq_dimlen( ncid_em3_ap, latdimid, latem3iz   )  !zf 2010.11.10
        call wrap_inq_dimlen( ncid_em3_ap, levdimid, levem3iz   )

       if (levem3iz .ne. levsiz) then
          write(6,*)'em3INI: levsiz=',levsiz,' .ne. equal to levem3iz',levem3iz
          call endrun
       end if

       allocate( em3lon(lonem3iz), stat=istat )  
       allocate( em3lat(latem3iz), stat=istat )  

      call wrap_inq_varid( ncid_em3_ap, 'lon', lonid   )
      call wrap_inq_varid( ncid_em3_ap, 'lat', latid   )
      call wrap_inq_varid( ncid_em3_ap, 'lev', levid   )

      call wrap_get_var_realx( ncid_em3_ap, lonid, em3lon )
      call wrap_get_var_realx( ncid_em3_ap, latid, em3lat )
      call wrap_get_var_realx( ncid_em3_ap, levid, em3lev )

      em3lev(:) = em3lev(:) * 1000.0    ! from km to m
!---------------------------------------------------------------

      call wrap_inq_dimlen( ncid_em3_ap, timeid, timsiz   )
      if (timsiz /= nsample) then
         write(6,*)'em3INI: timsiz=',timsiz,' must = ', nsample 
         call endrun
      end if
!
! the variable
!
      call wrap_inq_varid( ncid_em3_ap, trim(varname3d(1)), em3_id_ap )

      call wrap_inq_vardimid (ncid_em3_ap, em3_id_ap, dimids)
      if (dimids(1) /= londimid .and. dimids(3) /= levdimid .and. dimids(2) /= latdimid) then
         write(6,*)'em3INI: Data must be ordered lon, lat, lev, time'
         call endrun
      end if
!
!---------------------------------------------------------------
! if not out of boundary
!---------------------------------------------------------------
    else   
  
      call getfil(em3datapathfile_am, locfn_am)
      call wrap_open(locfn_am, 0, ncid_em3_am)

      write(6,*) 'em3datapathfile_am =', em3datapathfile_am
      write(6,*) 'Now open em3 file (locfn_am):', locfn_am

      call getfil(em3datapathfile_bm, locfn_bm)
      call wrap_open(locfn_bm, 0, ncid_em3_bm)
      write(6,*) 'Now open em3 file (locfn_bm):', locfn_bm

      write(*,*)'wzz: locfn_ap'
      call getfil(em3datapathfile_ap, locfn_ap)
      call wrap_open(locfn_ap, 0, ncid_em3_ap)
      write(6,*) 'Now open em3 file (locfn_ap):', locfn_ap

      write(*,*)'wzz: locfn_bp'
      call getfil(em3datapathfile_bp, locfn_bp)
      call wrap_open(locfn_bp, 0, ncid_em3_bp)
      write(6,*) 'Now open em3 file (locfn_bp):', locfn_bp
!
      call wrap_inq_dimid( ncid_em3_am, 'lon', londimid)
      call wrap_inq_dimid( ncid_em3_am, 'lat', latdimid)
      call wrap_inq_dimid( ncid_em3_am, 'lev', levdimid)
      call wrap_inq_dimid( ncid_em3_am, 'time',timeid  )

      call wrap_inq_dimlen( ncid_em3_am, londimid, lonem3iz   )
      call wrap_inq_dimlen( ncid_em3_am, latdimid, latem3iz   )
      call wrap_inq_dimlen( ncid_em3_am, levdimid, levem3iz   )

      if (levem3iz .ne. levsiz) then
         write(6,*)'em3INI: levsiz=',levsiz,' .ne. equal to levem3iz=',levem3iz
         call endrun
      end if

      call wrap_inq_dimlen( ncid_em3_am, timeid, timsiz   )
      if (timsiz /= nsample) then
         write(6,*)'em3INI: timsiz=',timsiz,' must = ', nsample 
         call endrun
      end if
!------------------zf 2010.11.08-----------------------------------

      allocate( em3lon(lonem3iz), stat=istat )   
      allocate( em3lat(latem3iz), stat=istat )

      call wrap_inq_varid( ncid_em3_am, 'lon', lonid   )
      call wrap_inq_varid( ncid_em3_am, 'lat', latid   )
      call wrap_inq_varid( ncid_em3_am, 'lev', levid   )

      call wrap_get_var_realx( ncid_em3_am, lonid, em3lon )
      call wrap_get_var_realx( ncid_em3_am, latid, em3lat )
      call wrap_get_var_realx( ncid_em3_am, levid, em3lev )

      em3lev(:) = em3lev(:) * 1000.0    ! from km to m  
!--------------------------------------------------------------------
!
! Obtain sulfate mixing ratio id
!
      call wrap_inq_varid( ncid_em3_am, trim(varname3d(1)), em3_id_am )

      call wrap_inq_vardimid (ncid_em3_am, em3_id_am, dimids)
      if (dimids(1) /= londimid .and. dimids(3) /= levdimid .and. dimids(2) /= latdimid) then
         write(6,*)'em3INI: Data must be ordered lon, lat, lev, time'
         call endrun
      end if
    endif    ! end if (.not.outofbndry)
!------------------------------------------------------------------------------
! master does the work above
!------------------------------------------------------------------------------
   endif

#if ( defined SPMD )
   call mpibcast (em3lev, levsiz, mpir8, 0, mpicom)
#endif
!
   allocate( xem3i_tem3pa(lonem3iz,latem3iz,levsiz), stat=istat )   !zf2010.11.08
   allocate( xem3i_tem3pb(lonem3iz,levsiz,latem3iz), stat=istat )   !zf2010.11.08

   do idx=1,nem3

   if (masterproc) then
!--------------------------------------------------------------------------
! open the right data files and read in raw data
!--------------------------------------------------------------------------
    
    if ( outofbndry ) then
!
! Set up hyperslab corners
!
      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
      cnt4(1)  = lonem3iz  !zf
      cnt4(2)  = latem3iz  !zf
      cnt4(3)  = levsiz
      cnt4(4)  = 1
!
! need to read in only 1 sample of rawdata
!
      strt4(4)  = sp_p 
      xem3i_ap = 0.0
      xem3i_tem3pa = 0.0   !zf
      xem3i_tem3pb = 0.0   !zf

      call wrap_inq_varid( ncid_em3_ap, trim(varname3d(idx)) , em3_id_ap )
      call wrap_get_vara_realx (ncid_em3_ap, em3_id_ap,strt4,cnt4, &
                                xem3i_tem3pa(:,:,1:levsiz))                  !zf

      do i = 1, lonem3iz
      do n = 1, latem3iz
      do k = 1, levsiz
         if( xem3i_tem3pa(i,n,k) .gt. 1.e+14) xem3i_tem3pa(i,n,k) = 0.0
         xem3i_tem3pb(i, k, n) = xem3i_tem3pa(i, n, k)
      enddo
      enddo
      enddo
       
      if(lonem3iz .eq. plon .and. latem3iz .eq. plat) then
         xem3i_ap = xem3i_tem3pa
      else
         call bilin(xem3i_tem3pb,em3lon,em3lat,lonem3iz,lonem3iz,levsiz,levsiz,   &
                 latem3iz,xem3i_ap,londeg,latdeg,plond,nlon,levsiz,plat)
      end if
!-----------------------------------------------
      do n    = 1,2
       do lat = 1,plat
        do k  = 1,levsiz
         do i = 1,plon
            xem3i(i,k,lat,n)= xem3i_ap(i,k,lat)
         enddo
        enddo
       enddo
      enddo

     else
!----------------------------------------------------------------
! Set up hyperslab corners
!
      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
      cnt4(1)  = lonem3iz  !zf
      cnt4(2)  = latem3iz   !zf
      cnt4(3)  = levsiz
      cnt4(4)  = 1
!
! read in the 4 sample of rawdata
!
      strt4(4) = sp_m 
      xem3i_am = 0.0

      call wrap_inq_varid( ncid_em3_am, trim(varname3d(idx)) , em3_id_am )
      call wrap_inq_varid( ncid_em3_bm, trim(varname3d(idx)) , em3_id_bm )
      call wrap_inq_varid( ncid_em3_ap, trim(varname3d(idx)) , em3_id_ap )
      call wrap_inq_varid( ncid_em3_bp, trim(varname3d(idx)) , em3_id_bp )

      xem3i_tem3pa = 0.0   !zf
      xem3i_tem3pb = 0.0   !zf

      call wrap_get_vara_realx (ncid_em3_am,em3_id_am,strt4,cnt4, &
                                xem3i_tem3pa(:,:,1:levsiz))  !zf

!-------------zf 2010.11.10-----------------
      do i = 1, lonem3iz
      do n = 1, latem3iz
      do k = 1, levsiz
           if( xem3i_tem3pa(i,n,k) .gt. 1.e+14) xem3i_tem3pa(i,n,k) = 0.0
           xem3i_tem3pb(i, k, n) = xem3i_tem3pa(i, n, k)
      enddo
      enddo
      enddo
       
      if(lonem3iz .eq. plon .and. latem3iz .eq. plat) then
         xem3i_am = xem3i_tem3pb
      else
         call bilin(xem3i_tem3pb,em3lon,em3lat,lonem3iz,lonem3iz,levsiz,levsiz,   &
                    latem3iz,xem3i_am,londeg,latdeg,plond,nlon,levsiz,plat)
      endif
!-----------------------------------------------

      xem3i_bm = 0.0
      xem3i_tem3pa = 0.0   !zf
      xem3i_tem3pb = 0.0   !zf
      call wrap_get_vara_realx (ncid_em3_bm,em3_id_bm,strt4,cnt4, &
                                xem3i_tem3pa(:,:,1:levsiz))  !zf
      do i = 1, lonem3iz
      do n = 1, latem3iz
      do k = 1, levsiz
         if( xem3i_tem3pa(i,n,k) .gt. 1.e+14) xem3i_tem3pa(i,n,k) = 0.0 
         xem3i_tem3pb(i, k, n) = xem3i_tem3pa(i, n, k)
      enddo
      enddo
      enddo
       
     if( lonem3iz .eq. plon .and. latem3iz .eq. plat) then
         xem3i_bm = xem3i_tem3pb
     else
         call bilin(xem3i_tem3pb,em3lon,em3lat,lonem3iz,lonem3iz,levsiz,levsiz,   &
                    latem3iz,xem3i_bm,londeg,latdeg,plond,nlon,levsiz,plat)
     endif
!-----------------------------------------------
!
      strt4(4) = sp_p 
      xem3i_ap = 0.0
      xem3i_tem3pa = 0.0   !zf
      xem3i_tem3pb = 0.0   !zf
      call wrap_get_vara_realx (ncid_em3_ap,em3_id_ap,strt4,cnt4, &
                                xem3i_tem3pa(:,:,1:levsiz))  !zf

      do i = 1, lonem3iz
      do n = 1, latem3iz
      do k = 1, levsiz
         if( xem3i_tem3pa(i,n,k) .gt. 1.e+14) xem3i_tem3pa(i,n,k) = 0.0
         xem3i_tem3pb(i, k, n) = xem3i_tem3pa(i, n, k)
      enddo
      enddo
      enddo
       
      if(lonem3iz .eq. plon .and. latem3iz .eq. plat) then
         xem3i_ap = xem3i_tem3pb
      else
         call bilin(xem3i_tem3pb,em3lon,em3lat,lonem3iz,lonem3iz,levsiz,levsiz,   &
                 latem3iz,xem3i_ap,londeg,latdeg,plond,nlon,levsiz,plat)
      endif
!-----------------------------------------------

      xem3i_bp = 0.0
      xem3i_tem3pa = 0.0   !zf
      xem3i_tem3pb = 0.0   !zf
      call wrap_get_vara_realx (ncid_em3_bp,em3_id_bp,strt4,cnt4, &
                                xem3i_tem3pa(:,:,1:levsiz))  !zf

!-------------zf 2010.11.10-----------------
      do i = 1, lonem3iz
      do n = 1, latem3iz
      do k = 1, levsiz
         if( xem3i_tem3pa(i,n,k) .gt. 1.e+14) xem3i_tem3pa(i,n,k) = 0.0
         xem3i_tem3pb(i, k, n) = xem3i_tem3pa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonem3iz .eq. plon .and. latem3iz .eq. plat) then
        xem3i_bp = xem3i_tem3pb
     else
        call bilin(xem3i_tem3pb,em3lon,em3lat,lonem3iz,lonem3iz,levsiz,levsiz,   &
                   latem3iz,xem3i_bp,londeg,latdeg,plond,nlon,levsiz,plat)
     endif
!-----------------------------------------------
!
! Normal interpolation between consecutive time slices.
!
      write(6,*) "yr_bm=",yr_bm,"yr_am=",yr_am, "yrm=",yrm
      deltat = dble( yr_bm - yr_am )
      fact1  = dble( yr_bm - yrm  )/deltat
      fact2  = dble( yrm   - yr_am)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'em3INI--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
       do k  = 1,levsiz
        do i = 1,plon
           xem3i(i,k,lat,nmsul)= fact1*xem3i_am(i,k,lat)+fact2*xem3i_bm(i,k,lat)
        enddo
       enddo
      enddo
!
      deltat = dble( yr_bp - yr_ap )
      fact1  = dble( yr_bp - yrp  )/deltat
      fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'em3INI--2: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
       do k  = 1,levsiz
        do i = 1,plon
           xem3i(i,k,lat,npsul)= fact1*xem3i_ap(i,k,lat)+fact2*xem3i_bp(i,k,lat)
        enddo
       enddo
      enddo

    endif    ! end if (.not.outofbndry)
!------------------------------------------------------------------------------
! master does the work above
!------------------------------------------------------------------------------
   endif    ! if( masterproc) 
!------------------------------------------------------------------------------

   call scatter_field_to_chunk(1,levsiz,1,plon,xem3i(1,1,1,nmsul),em3i(1,1,begchunk,nmsul,idx))
   call scatter_field_to_chunk(1,levsiz,1,plon,xem3i(1,1,1,npsul),em3i(1,1,begchunk,npsul,idx))

   if (outofbndry) then
       call scatter_field_to_chunk(1,levsiz,1,plon,xem3i_ap,em3(:,:,:,idx))  !??
   endif

   enddo  ! loop idx=1, nem3

   deallocate (xem3i_tem3pa)   !zf2010.11.08
   deallocate (xem3i_tem3pb)   !zf2010.11.08

   return
end subroutine em3ini_IPCC_aero

!###############################################################################
!
subroutine em3int_IPCC_aero
!----------------------------------------------------------------------- 
!
! Purpose: Interpolate sulfate mixing ratios to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
!-----------------------------------------------------------------------
   use ioFileMod,      only: getfil
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use timeinterp,   only: getfactors

   implicit none

!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
!
! Local workspace
!
   integer cnt4(4)            ! array of counts for each dimension
   integer strt4(4)           ! array of starting indices
   integer i, k, lchnk,lat    ! column, level, chunk, latitude indices
   integer ncol               ! number of columns in current chunk
   integer ntmp               ! tem3porary
   integer yr, mon, day       ! components of a date
   integer ncsec              ! current time of day [seconds]
   integer idx

   character(len=256)  locfn_bp     ! local filename

   real(r8) calday            ! current calendar day
   real(r8) caldayloc         ! calendar day (includes yr if no cycling)
   real(r8) deltat            ! time (days) between interpolating sulfate data
   real(r8) fact1, fact2      ! time interpolation factors

   real(r8) xem3i   (plon,levsiz,plat)  ! input sulfate bio mixing ratios

!--------------------------------------zf---------------------------------
   real(r8) xem3i_ap(plond,levsiz,plat)  ! input sulfate bio mixing ratios
   real(r8) xem3i_bp(plond,levsiz,plat)  ! input sulfate ant mixing ratios
   real(r8), allocatable :: xem3i_tem3pa(:,:,:)   
   real(r8), allocatable :: xem3i_tem3pb(:,:,:) 
   integer istat                        !  error return
!--------------------------------------------------------------------------

   integer :: sul_id_ap, sul_id_bp
!-----------------------------------------------------------------------
!
! Use year information only if a multiyear dataset
!
   calday = get_curr_calday()
   call get_curr_date(yr, mon, day, ncsec)

   if ( rampyear_prescribed_em3 .ne. bigint ) then
      yr = rampyear_prescribed_em3        ! wzz, 2009.10.15
   endif
   caldayloc = calday + yr*365.

   if ((caldayloc.lt.bndry_m).or.(caldayloc.ge.bndry_p)) then
        outofbndry = .true.
   else
        outofbndry = .false.
   endif
   allocate( xem3i_tem3pa(lonem3iz,latem3iz,pver), stat=istat )   !zf2010.11.08
   allocate( xem3i_tem3pb(lonem3iz,levsiz,latem3iz), stat=istat )   !zf2010.11.08
!-------------------------------------------------------------------------------
!  there is no need to do anything when current model time is
!   out of the time-boundary of input data files
!-------------------------------------------------------------------------------
 if (.not.outofbndry) then
!
! Initialize hyperslab corners
!
   if (masterproc) then

      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
      cnt4(1)  = lonem3iz  !zf
      cnt4(2)  = latem3iz  !zf
      cnt4(3)  = levsiz
      cnt4(4)  = 1

   endif
!
! If model time is past current forward sulfate timeslice, read in the next
! timeslice for time interpolation. 

   if ( (caldayloc .gt. cdayem3p .and. rampyear_prescribed_em3 == bigint) .or. &
        (caldayloc .gt. cdayem3p .and. .not. (sp_p == 1 .and. caldayloc > cdayem3m))) then
      sp_m = sp_p
      sp_p = sp_p+1

      if ( sp_p.gt.nsample ) then     ! a new year
          sp_p = 1
          yrp  = yrp+1
          if ( rampyear_prescribed_em3 .ne. bigint ) then
             yrp = rampyear_prescribed_em3        ! wzz, 2009.10.15
          endif

          if (masterproc) then

            if ( yrp.gt.yr_bp ) then    ! need to locate the new year
             indxbp = indxbp+1
             yr_ap = yr_bp
             yr_bp = yr_em3(indxbp)

             ncid_em3_ap = ncid_em3_bp
             sul_id_ap = sul_id_bp

             em3datapathfile_bp = trim(em3datapath)//trim(file_em3(indxbp))
             call getfil(em3datapathfile_bp, locfn_bp)
             call wrap_open(locfn_bp, 0, ncid_em3_bp)
             write(6,*) 'Now open em3 file (locfn_bp):', locfn_bp 

             call wrap_inq_varid( ncid_em3_bp, trim(varname3d(1)), sul_id_bp )
            endif

          endif

      endif
      
      cdayem3m = cdayem3p
      cdayem3p = yrp*365.0+sample_em3(sp_p)

      cdaym = cdayp
      cdayp = sample_em3(sp_p)

      ntmp    = nmsul
      nmsul   = npsul
      npsul   = ntmp

    do idx = 1, nem3
      if (masterproc) then
        strt4(4) = sp_p 
        xem3i_ap = 0.0
        xem3i_tem3pa = 0.0  !zf
        xem3i_tem3pb = 0.0  !zf
        call wrap_inq_varid( ncid_em3_ap, trim(varname3d(idx)) , sul_id_ap )
        call wrap_get_vara_realx (ncid_em3_ap,sul_id_ap,strt4,cnt4, &
                                  xem3i_tem3pa(:,:,1:pver))  !zf

        do i = 1, lonem3iz
        do lat = 1, latem3iz
        do k = 1, levem3iz
          if( xem3i_tem3pa(i,lat,k) .gt. 1.e+14) xem3i_tem3pa(i,lat,k) = 0.0
          xem3i_tem3pb(i, k, lat) = xem3i_tem3pa(i, lat, k)
        enddo
        enddo
        enddo
       
        if(lonem3iz .eq. plon .and. latem3iz .eq. plat) then
           xem3i_ap = xem3i_tem3pb
        else
           call bilin(xem3i_tem3pb,em3lon,em3lat,lonem3iz,lonem3iz,levem3iz,levem3iz,  &
                     latem3iz,xem3i_ap,londeg,latdeg,plond,nlon,levem3iz,plat)
        endif
!-----------------------------------------------
!
        xem3i_bp = 0.0
        xem3i_tem3pa = 0.0  !zf
        xem3i_tem3pb = 0.0  !zf
        call wrap_inq_varid( ncid_em3_bp, trim(varname3d(idx)) , sul_id_bp )
        call wrap_get_vara_realx (ncid_em3_bp,sul_id_bp,strt4,cnt4, &
                                  xem3i_tem3pa(:,:,1:pver))  !zf

        do i = 1, lonem3iz
        do lat = 1, latem3iz
        do k = 1, levem3iz
          if( xem3i_tem3pa(i,lat,k) .gt. 1.e+14) xem3i_tem3pa(i,lat,k) = 0.0
           xem3i_tem3pb(i, k, lat) = xem3i_tem3pa(i, lat, k)
        enddo
        enddo
        enddo
       
        if(lonem3iz .eq. plon .and. latem3iz .eq. plat) then
           xem3i_bp = xem3i_tem3pb
        else
           call bilin(xem3i_tem3pb,em3lon,em3lat,lonem3iz,lonem3iz,levem3iz,levem3iz,  &
                      latem3iz,xem3i_bp,londeg,latdeg,plond,nlon,levem3iz,plat)
        endif
!-----------------------------------------------
!
! Normal interpolation between consecutive time slices.
!
        write(6,*) "yr_bp,yr_ap,yrp" , yr_bp,yr_ap,yrp
        deltat = dble( yr_bp - yr_ap )
        fact1  = dble( yr_bp - yrp  )/deltat
        fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'em3INT--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
        do lat = 1,plat
         do k  = 1,levsiz
          do i = 1,plon
             xem3i(i,k,lat)= fact1*xem3i_ap(i,k,lat)+fact2*xem3i_bp(i,k,lat)
          enddo
         enddo
        enddo
!
! master does the work above
!
      endif  ! end masterproc

      call scatter_field_to_chunk(1,levsiz,1,plon,xem3i,em3i(1,1,begchunk,npsul,idx))

    enddo

   end if  ! 

!--------------------------------------------------------------------------
! Determine time interpolation factor.   
!
   if ( rampyear_prescribed_em3 .ne. bigint ) then   
        call getfactors (.true., sp_p, sample_em3(sp_m), sample_em3(sp_p), calday, &
                          fact1, fact2, 'em3int_IPCC:')
   else
        call getfactors (.false., sp_p, cdayem3m, cdayem3p, caldayloc, &
                          fact1, fact2, 'em3int_IPCC:')
   endif
!
! Time interpolation.
!
   do lchnk=begchunk,endchunk
      ncol = get_ncols_p(lchnk)
      do k=1,levsiz
         do i=1,ncol
           do idx = 1, nem3
            em3(i,k,lchnk,idx) = em3i(i,k,lchnk,nmsul,idx)*fact1 +  &
                                 em3i(i,k,lchnk,npsul,idx)*fact2
           enddo 
         end do
      end do
   end do
!
!----------------------------------------------------------------------------
  endif

   deallocate (xem3i_tem3pa)   !zf2010.11.08
   deallocate (xem3i_tem3pb)   !zf2010.11.08

   return
end subroutine em3int_IPCC_aero

subroutine getem3_IPCC_aero( lchnk, ncol, zm,  em3data )

!-----------------------------------------------------------------------
!
! Purpose: Return slice of time interpolated sulfate aerosol.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
!
! Arguments
!
  integer , intent(in)   :: lchnk            ! chunk identifier
  integer , intent(in)   :: ncol             ! number of atmospheric columns
  real(r8), intent(in)   :: zm(pcols,pver)    ! geopot height above surface at midpoints (m)  
  real(r8), intent(out)  :: em3data(pcols,pver,nem3)  ! biogenic sulfate
!
! Local variables.
!
   integer :: i, k, n                 ! longitude, level indices
   integer :: L, L1, L2
!-----------------------------------------------------------------------
   do n = 1, nem3
   do k = 1, pver
      do i = 1, ncol
            em3data(i,k,n) = 0.0
            do L=1, levsiz-1
               if ( zm(i,k) .gt. em3lev(L)  .and. zm(i,k) .le. em3lev(L+1) ) then
                   em3data(i,k,n)  = em3(i,L,lchnk,n)+(em3(i,L+1,lchnk,n) - em3(i,L,lchnk,n))* &
                                    (zm(i,k)-em3lev(L))/(em3lev(L+1)-em3lev(L))
               endif
            enddo
      end do
   end do
   enddo

   return
end subroutine getem3_IPCC_aero

!###############################################################################
end module prescribed_em_d3_aero
#endif

