#include <misc.h>
#include <params.h>

#ifdef MOZART2

module prescribed_emis_3d

!----------------------------------------------------------------------- 
! Purpose:
!
! Interpolation of datasets.
!-----------------------------------------------------------------------

   use shr_kind_mod,   only: r8 => shr_kind_r8
   use pmgrid,         only: plon, plat, masterproc
   use ppgrid,         only: begchunk, endchunk, pcols, pver, pverp
   use phys_grid,      only: scatter_field_to_chunk
   use time_manager,   only: get_curr_date, get_curr_calday
   use abortutils,     only: endrun
   use iofilemod,      only : getfil
   use mo_tracinp,     only : bndic

#if ( defined SPMD )
   use mpishorthand
#endif

   implicit none

   public em3d_ini_IPCC
   public em3d_int_IPCC
   public getem3_IPCC6
 
   logical, public  :: emis3dcyc = .false.

   integer, public,parameter :: nem3_IPCC6    = 8
   character*8,public :: varname3d (nem3_IPCC6)
   data varname3d / 'CB1     ', 'CB2     ', 'OC1     ', 'OC2     ',   &
                    'CO      ', 'NO      ', 'SO2     ', 'NH3     ' /
   integer, public, parameter :: &
          idxCB1_IPCC6 =  1, &
          idxCB2_IPCC6 =  2, &
          idxOC1_IPCC6 =  3, &
          idxOC2_IPCC6 =  4, &
          idxCO_IPCC6  =  5, &
          idxNO_IPCC6  =  6, &
          idxSO2_IPCC6 =  7, &
          idxNH3_IPCC6 =  8

   integer  :: levsiz

   save

! Private data

  real(r8), private :: cdaym     ! dataset calendar day previous month
  real(r8), private :: cdayp     ! dataset calendar day next month

  integer  :: np1           ! current forward time index of ozone dataset
  integer  :: ncid_air       ! netcdf ID for bb emission
  integer  :: air_id     ! netcdf id for emission variable
  integer  :: timesiz   ! size of time dimension on ozone dataset
  integer  :: latsiz

  real(r8), private, allocatable :: time_emis(:)   ! Date on dataset (days since 0000 yr) 

!------
!
  real(r8), private, allocatable  :: emis_3d(:,:,:,:,:)    ! (pcols,begchunk:endchunk,2,nem3_IPCC6)
  real(r8), private, allocatable  :: emis_anth(:,:,:,:)    ! (pcols,levsiz,begchunk:endchunk,nem3_IPCC6)
  real(r8), private, allocatable  :: altitude(:)           ! (levsiz)

!===============================================================================
contains
!===============================================================================

   subroutine em3d_ini_IPCC

   implicit none
!
! Local workspace
!
   integer londimid                        ! netcdf id for longitude dimension
   integer latdimid                        ! netcdf id for latitude dimension
   integer levdimid                        ! netcdf id for level dimension
   integer timeid                          ! netcdf id for time variable
   integer levelid                         ! netcdf id for level variable
   integer dimids(4)                       ! variable shape
   integer cnt4(4)                         ! array of counts for each dimension
   integer strt4(4)                        ! array of starting indices
   integer i, j, k, lat, n                    ! longitude, level, latitude, time indices
   integer  :: yr, mon, day                ! components of a date
   integer  :: ncsec                       ! current time of day [seconds]
   integer  :: istat
   real(r8) :: calday                      ! current calendar day
   real(r8) :: caldayloc                     ! calendar day (includes yr if no cycling)

   character(len=256) :: locfn             ! netcdf local filename to open 
   character(len=256) :: filename
   character(len=50)  :: units
  
   integer  :: latsiz, lonsiz, idx

   real(r8), allocatable  ::  emis_3dbdym(:,:,:), emis_3dbdyp(:,:,:)
!-----------------------------------------------------------------------
!
! SPMD: Master does all the work. Sends needed info to slaves
!
   units(:) = ' '
   if (masterproc) then

! need declaration for locfn

      do i = len_trim(bndic), 1, -1
         if(bndic(i:i).eq.'/') exit
      enddo

      filename = trim(bndic(1:i))//'IPCC6/'//trim(varname3d(1))//'_CMIP6_AIR_anthro_185001-201412_c20170213.nc'
      call getfil (filename, locfn, 0)
      call wrap_open (locfn, 0, ncid_air)
!
! Use year information only if not cycling ozone dataset
!
      calday = get_curr_calday()
      call get_curr_date(yr, mon, day, ncsec)

!-------zf 2016.06.12
      if (emis3dcyc) then
        caldayloc = calday
      else
        caldayloc = calday + yr*365.  
      end if
!
! Get and check dimension info
!
      CALL WRAP_INQ_DIMID( ncid_air, 'lon', londimid   )
      CALL WRAP_INQ_DIMID( ncid_air, 'lat', latdimid   )
      CALL WRAP_INQ_DIMID( ncid_air, 'level', levdimid  )
      CALL WRAP_INQ_DIMID( ncid_air, 'time', timeid  )

      CALL WRAP_INQ_DIMLEN( ncid_air, londimid, lonsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_air, latdimid, latsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_air, levdimid, levsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_air, timeid, timesiz   )

      CALL WRAP_INQ_VARID( ncid_air, trim(varname3d(1)), air_id   )

      CALL WRAP_INQ_VARDIMID (ncid_air, air_id, dimids)
      if (dimids(1) /= londimid .and. dimids(2) /= levdimid .and. dimids(3) /= latdimid) then
         write(6,*)'em3d_ini_IPCC: Data must be ordered lon, lat, lev, time'
         call endrun
      end if
      if (lonsiz .ne. plon .or. latsiz .ne. plat ) then
         call endrun ('em3d_ini_IPCC:  The lonsiz and latsiz are not the same as the model resolution')
      end if

      CALL WRAP_INQ_VARID( ncid_air, 'time', timeid   )
      CALL WRAP_GET_ATT_TEXT(ncid_air, timeid, 'units', units )
      if ( trim(units) /= 'days since 1750-01-01 00:00:00' ) then
         call endrun ('em3d_ini_IPCC:  The time units are not days since 1750-01-01')
      end if
   end if   ! masterproc

#if (defined SPMD )
   call mpibcast (timesiz, 1, mpiint, 0, mpicom)
   call mpibcast (levsiz,  1, mpiint, 0, mpicom)
#endif
!
! Dynamically allocated memory for module data
!
   allocate (altitude(levsiz+1), stat=istat)
   allocate (emis_3dbdym(plon, levsiz, plat), stat=istat)
   allocate (emis_3dbdyp(plon, levsiz, plat), stat=istat)
   allocate (time_emis(timesiz),                     &
             emis_3d(pcols,levsiz,begchunk:endchunk,2,nem3_IPCC6), stat=istat )
   allocate (emis_anth(pcols, levsiz, begchunk:endchunk, nem3_IPCC6), stat=istat )

   if (masterproc) then
!
! Retrieve entire date and sec variables.
!
      CALL WRAP_GET_VAR_REALX (ncid_air,timeid,time_emis)

      CALL WRAP_INQ_VARID( ncid_air, 'level', levelid  )
      CALL WRAP_GET_VAR_REALX (ncid_air, levelid, altitude(1:levsiz) )
      altitude(levsiz + 1) = altitude(levsiz) + (altitude(levsiz) - altitude(levsiz-1))

      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
      cnt4(1)  = lonsiz
      cnt4(2)  = levsiz
      cnt4(3)  = latsiz
      cnt4(4)  = 1 

      call wrap_close(ncid_air)

   endif   ! materproc

!
! Special code for interpolation between December and January
!
   do idx = 1, nem3_IPCC6
   if (masterproc) then
      filename = trim(bndic(1:i))//'IPCC6/'//trim(varname3d(idx))//'_CMIP6_AIR_anthro_185001-201412_c20170213.nc'
      call getfil (filename, locfn, 0)
      call wrap_open (locfn, 0, ncid_air)

      if (emis3dcyc) then
        n = 12
        np1 = 1
         cdaym = time_emis(n) - 100*365.
         cdayp = time_emis(np1) - 100*365.
         if (caldayloc <= cdayp .or. caldayloc > cdaym) then
            strt4(4) = n
            call wrap_inq_varid( ncid_air, trim(varname3d(idx)) , air_id)
            call wrap_get_vara_realx (ncid_air, air_id, strt4, cnt4, emis_3dbdym)

            strt4(4) = np1
            call wrap_inq_varid( ncid_air, trim(varname3d(idx)) , air_id)
            call wrap_get_vara_realx (ncid_air, air_id, strt4, cnt4, emis_3dbdyp)

            goto 10
         end if
      end if
!
! Normal interpolation between consecutive time slices.
!
      do n=1,timesiz-1
         np1 = n + 1
         cdaym = time_emis(n) - 100*365.
         cdayp = time_emis(n+1) - 100*365.
 
!---------zf 2016.06.13
         if (.not.emis3dcyc ) then
           cdaym = cdaym + 365.0*1850.0 
           cdayp = cdayp + 365.0*1850.0 
         end if
!---------------
 
         if (caldayloc >= cdaym .and. caldayloc < cdayp) then
            strt4(4) = n
            call wrap_inq_varid( ncid_air, trim(varname3d(idx)) , air_id)
            call wrap_get_vara_realx (ncid_air, air_id, strt4, cnt4, emis_3dbdym)

            strt4(4) = np1
            call wrap_inq_varid( ncid_air, trim(varname3d(idx)) , air_id)
            call wrap_get_vara_realx (ncid_air, air_id, strt4, cnt4, emis_3dbdyp)

            goto 10
         end if
      end do

      write(6,*)'em3d_ini_IPCC: Failed to find dates bracketing , year, month, day',  yr, mon, day
      call endrun
!------------------------------------------------
10    continue
 
      call wrap_close(ncid_air)

   endif   ! materproc

   call scatter_field_to_chunk(1,levsiz,1,plon,emis_3dbdym,emis_3d(1,1,begchunk,1,idx))
   call scatter_field_to_chunk(1,levsiz,1,plon,emis_3dbdyp,emis_3d(1,1,begchunk,2,idx))

   end do  ! idx 

#if (defined SPMD )
   call mpibcast (np1, 1, mpiint, 0, mpicom)
   call mpibcast (time_emis, timesiz, mpir8, 0, mpicom )
   call mpibcast (cdaym, 1, mpir8, 0, mpicom)
   call mpibcast (cdayp, 1, mpir8, 0, mpicom)
   call mpibcast (altitude, levsiz+1, mpir8, 0, mpicom )
#endif

   deallocate (emis_3dbdym)
   deallocate (emis_3dbdyp)

end subroutine em3d_ini_IPCC

!===============================================================================

subroutine em3d_int_IPCC 
!----------------------------------------------------------------------- 
! 
! Purpose: Interpolate ozone mixing ratios to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
! Method: Find next month of ozone data to interpolate.  Linearly interpolate 
!         vertically and horizontally
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use timeinterp,   only: getfactors
   use phys_grid,    only: get_ncols_p
   use time_manager, only: get_curr_date

   implicit none
!
! Local workspace
!
   integer cnt4(4)                ! array of counts for each dimension
   integer strt4(4)               ! array of starting indices
   integer i,k                    ! longitude indices
   integer idx                    
   integer c, ncol                ! chunk index, number of columns in chunk
   integer :: yr, mon, day        ! components of a date
   integer :: ncsec               ! current time of day [seconds]

   real(r8) fact1, fact2          ! time interpolation factors
   real(r8) :: calday             ! current calendar day
   real(r8) caldayloc             ! calendar day (includes yr if no cycling)

   integer  :: istat
   integer  :: dimids(3)                       ! variable shape

   real(r8), allocatable  ::  emis_3dbdyp(:,:,:)

   character(len=256) :: locfn             ! netcdf local filename to open 
   character(len=256) :: filename
!-----------------------------------------------------------------------
!
! Use year information only if a multiyear dataset
!
   calday    = get_curr_calday()
   call get_curr_date(yr, mon, day, ncsec)

!----------zf2016.06.13
   if (emis3dcyc) then
     caldayloc = calday
   else
     caldayloc = calday + yr*365.
   end if
!
! If model time is past current forward ozone timeslice, then
! masterproc reads in the next timeslice for time interpolation.  Messy logic is 
! for emis3dcyc = .true. interpolation between December and January (np1 == 1).  Note 
! that np1 is never 1 when emis3dcyc is .false.
!
   if (caldayloc > cdayp .and. .not. (np1 == 1 .and. caldayloc > cdaym)) then

      do idx = 1, nem3_IPCC6
        emis_3d(:, :, :, 1, idx) =  emis_3d(:, :, :, 2, idx) 
      end do

      if (masterproc) then
         if (emis3dcyc) then
           np1 = mod(np1,12) + 1           !zf2016.05.13
         else
           np1 = np1 + 1
         end if

         if (np1 > timesiz) then
            call endrun ('em3d_int_IPCC: Attempt to read past end of dataset')
         end if

         cdaym = cdayp
         cdayp = time_emis(np1) - 100*365.
         if ( .not.emis3dcyc) then
           cdayp = cdayp + 365.0*1850.0           !zf2016.05.13
         end if

         if (.not. (np1 == 1 .or. caldayloc <= cdayp)) then
            if (masterproc) then
               write(6,*)'em3d_int_IPCC: Input emis for date',time_emis(np1), &
                        'does not exceed model date',' Stopping.'
            endif
            call endrun ()
         end if

         strt4(1) = 1
         strt4(2) = 1
         strt4(3) = 1
         strt4(4) = np1
         cnt4(1)  = plon
         cnt4(2)  = levsiz
         cnt4(3)  = plat
         cnt4(4)  = 1
      end if  ! masterproc

      allocate (emis_3dbdyp(plon, levsiz, plat), stat=istat)

      do idx = 1, nem3_IPCC6
      if (masterproc) then
!
! Allocate memory for more dynamic local workspace
!
      do i = len_trim(bndic), 1, -1
         if(bndic(i:i).eq.'/') exit
      enddo

      filename = trim(bndic(1:i))//'IPCC6/'//trim(varname3d(idx))//'_CMIP6_AIR_anthro_185001-201412_c20170213.nc'
         call getfil (filename, locfn, 0)
         call wrap_open (locfn, 0, ncid_air)

         call wrap_inq_varid( ncid_air, trim(varname3d(idx)), air_id   )
         call wrap_get_vara_realx (ncid_air, air_id, strt4, cnt4, emis_3dbdyp)  
 
         write(6,*)'em3d_int_IPCC: Read surface emission for date (days) ', time_emis(np1)
         call wrap_close(ncid_air)
      end if  ! masterproc

#if (defined SPMD )
     call mpibcast (np1, 1, mpiint, 0, mpicom)
     call mpibcast (cdaym, 1, mpir8, 0, mpicom)
     call mpibcast (cdayp, 1, mpir8, 0, mpicom)
#endif

      call scatter_field_to_chunk(1,levsiz,1,plon,emis_3dbdyp,emis_3d(1,1,begchunk,2,idx) )
    end do   ! idx

   deallocate (emis_3dbdyp)

   end if

!
! Determine time interpolation factors.
!
   call getfactors (emis3dcyc, np1, cdaym, cdayp, caldayloc, &
                    fact1, fact2, 'em3d_data_TIMESTEP_INIT:')
!
! Time interpolation.
!
   do c=begchunk,endchunk
      ncol = get_ncols_p(c)
      do k=1,levsiz
      do i=1,ncol
        do idx=1,nem3_IPCC6
          emis_anth(i,k,c,idx) = emis_3d(i,k,c,1,idx)*fact1 + emis_3d(i,k,c,2,idx)*fact2
          !fco2(i,c) = fco2(i,c)*1.e+3    ! kg CO2/m2/s -> g CO2/m2/s
        end do
      end do
      end do
   end do

end subroutine em3d_int_IPCC

!*********************************
 subroutine getem3_IPCC6( lchnk, ncol, zi, em3_IPCC6 )

!-----------------------------------------------------------------------
!
! Purpose: Return slice of interpolated emissions.
!
!-----------------------------------------------------------------------
   use mo_chem_mods,  only :  adv_mass
   use mo_chem_utls,  only :  get_spc_ndx

   implicit none
!-----------------------------------------------------------------------
!
! Arguments
!
  integer , intent(in)   :: lchnk            ! chunk identifier
  integer , intent(in)   :: ncol             ! number of atmospheric columns
  real    , intent(in)   :: zi(pcols, pverp)                  ! interface geopot height ( m )
  real(r8), intent(out)  :: em3_IPCC6(pcols, pver, nem3_IPCC6)  ! emissions
!
! Local variables.
!
   integer :: n, k, ndx
   real                  :: zint(pcols, pverp)                  ! interface geopot height ( km )
    
!-----------------------------------------------------------------------
   em3_IPCC6 = 0.
   zint(:,:) = zi(:,:)/1000. 

   do n = 1, nem3_IPCC6
      ndx = get_spc_ndx( trim(varname3d(n)) )
      emis_anth(:, :, lchnk, n) = emis_anth(:, :, lchnk, n)*1.e3*6.02*1.e23/adv_mass(ndx)/1.e4
      call vertical( lchnk, ncol, zint, emis_anth(1:ncol, 1:levsiz, lchnk, n), em3_IPCC6(1:ncol, 1:pver, n))
   enddo

 end subroutine getem3_IPCC6

 subroutine vertical( lchnk, ncol, zint, var_in, var_out)

!-----------------------------------------------------------------------
!
! Purpose: Return slice of interpolated emissions.
!
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
!
! Arguments
!
  integer , intent(in)   :: lchnk            ! chunk identifier
  integer , intent(in)   :: ncol             ! number of atmospheric columns
  real(r8), intent(in)   :: zint(pcols, pverp)                  ! interface geopot height ( km )
  real(r8), intent(in)   :: var_in(pcols, levsiz) 
  real(r8), intent(out)  :: var_out(pcols, pver) 
!
! Local variables.
!
   integer      :: i, k, kk                ! longitude, level indices
   real(r8)     :: zlev_top, zlev_bot
   real(r8)     :: zdel, zdeli, frac
   real(r8)     :: altitude1(levsiz+1)
!-----------------------------------------------------------------------

   do k = 1, levsiz + 1
!     altitude1(k) = altitude(levsiz + 2 - k)
     altitude1(k) = altitude(k)
   end do

   var_out = 0.

   do k = 1, pver 
      do i = 1, ncol
         zlev_top = zint(i,k)                 ! altitude at top of model level (km)
         zlev_bot = zint(i,k+1)               ! altitude at bottom of model level (km)
         zdel = (zlev_top - zlev_bot) * 1.e5  ! model level thickness (cm)

         zdeli = 1./zdel
         if( zlev_bot <= maxval(altitude1) .and. zlev_top >= minval(altitude1) ) then
            do kk = 1, size(altitude) - 1
               if( zlev_bot <= altitude1(kk+1) .and. zlev_top >= altitude1(kk) ) then
                   frac = (min( zlev_top, altitude1(kk+1) ) - max( zlev_bot, altitude1(kk) )) &
                         /(altitude1(kk+1) - altitude1(kk))
                   var_out(i,k) = var_out(i,k) + frac * var_in(i,kk) * zdeli
               end if
            end do
         end if
      end do
   end do

   return
 end subroutine vertical

end module prescribed_emis_3d

#endif
