#include <misc.h>
#include <params.h>

subroutine tphysac (ztodt,   pblh,    kbfs,    qpert,   tpert,  shf,  &
                    taux,    tauy,    cflx,    sgh,    lhf,  &
                    landfrac,snowh,   tref,    precc,  precl,&
                    precsc, precsl,   state,   tend,    pbuf,&
                    ocnfrac, fsds, icefrac, fv, ram1, ts, &
#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
                    chemflx,  &
                    flx_mss_vrt_dst01, flx_mss_vrt_dst02, &
                    flx_mss_vrt_dst03, flx_mss_vrt_dst04, &
                    sflx_carbon, co2emis, fco2,           &
                    vlc_trb_SO4, vlc_trb_NH4, vlc_trb_CB2, vlc_trb_SO2, &
                    u10, v10,                             &   !zf 2015-05-19
                    lnd_ram1,  lnd_fv,                    &
#endif
#ifdef CO2
                    sflx_carbon, co2emis, fco2,           &
#endif
#ifdef NSTEP_GW
                    utgw, vtgw, stgw, qtgw, &
#endif
#ifdef UWMT
                    kvm, kvh, &
#endif
                    fhs )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Tendency physics after coupling to land, sea, and ice models.
! Computes the following:
!   o Radon surface flux and decay (optional)
!   o Vertical diffusion and planetary boundary layer
!   o Multiple gravity wave drag
! 
! Method: 
! <Describe the algorithm(s) used in the routine.> 
! <Also include any applicable external references.> 
! 
! Author: CCM1, CMS Contact: J. Truesdale
! 
!-----------------------------------------------------------------------
   use shr_kind_mod,       only: r8 => shr_kind_r8
   use ppgrid,             only: pcols, pver, pverp
   use chemistry,          only: trace_gas, chem_timestep_tend
   use gw_drag,            only: gw_intr
   use vertical_diffusion, only: vd_intr
   use rayleigh_friction,  only: do_rayleigh, rayleigh_friction_tend ! add by luyx20170119
   use physics_types,      only: physics_state, physics_tend, physics_ptend, physics_update,    &
                                 physics_ptend_init, physics_dme_adjust
   use phys_buffer,        only: pbuf_size_max, pbuf_fld, pbuf_old_tim_idx, pbuf_get_fld_idx
   use constituents,       only: ppcnst, qmin, iaero
   use tracers,            only: tracers_timestep_tend, set_state_pdry, set_dry_to_wet
   use physconst,          only: zvir, gravit, rhoh2o, latvap,latice, cpair, rair
   use aerosol_intr,       only: aerosol_emis_intr, aerosol_drydep_intr, aerosol_srcsnk_intr
   use check_energy,       only: check_energy_chng
   use check_energy,       only: check_tracers_data, check_tracers_init, check_tracers_chng
   use time_manager,       only: get_nstep
   use abortutils,         only: endrun
   use dycore,             only: dycore_is
   use cuace_driver,       only: intrfc             ! wzz, 2008.9.8
#ifdef BCCCHEM
   use bcc_progseasalts_intr,  only: progseasalts_drydep_intr, bcc_progseasalts_emis_intr
   use bcc_dust_intr,          only: dust_drydep_intr, bcc_dust_emis_intr
   use time_manager,           only: is_perpetual
   use time_manager,           only: get_perp_date,  get_curr_date
   use phys_grid,              only: get_rlat_all_p, get_lat_all_p
   use bcc_chem_mods,          only: gas_pcnst
   use bcc_surf_nature_emis,   only: surf_nature_emis
   use constituents,           only: cnst_get_ind
   use history,                only: outfld
   use prescribed_em_d2,       only: getem2_IPCC, nem2, varname2d
   use bcc_surface,            only: sflxdr
#endif

#ifdef MOZART2
   use chemistry,       only: ixchm
   use prescribed_em_d2,       only: getem2_IPCC, nem2, varname2d
   use time_manager,           only: is_perpetual
   use time_manager,           only: get_perp_date, get_curr_date
   use phys_grid,              only: get_rlat_all_p, get_lat_all_p
   use constituents,           only: cnst_get_ind
   use bcc_dust_intr,          only: dust_drydep_intr, bcc_dust_emis_intr
   use bcc_progseasalts_intr,  only: progseasalts_drydep_intr, bcc_progseasalts_emis_intr
   use bcc_nature_emis_drydep, only: nature_emis_drydep
   use history,                only: outfld
   use prescribed_fco2_data, only: fco2cyc
#endif

#if ( defined CO2)
   use constituents,           only: cnst_get_ind
   use prescribed_fco2_data, only: fco2cyc
#endif

#if ( defined WACCM)
   use charge_neutrality,  only: charge_fix
   use iondrag,            only: iondrag_calc
   use prescribed_em_d2,   only: getem2_IPCC, nem2, varname2d
#endif

   implicit none

#include <comctl.h>
!
! Arguments
!
   real(r8), intent(in) :: ztodt                  ! Two times model timestep (2 delta-t)
   real(r8), intent(in) :: landfrac(pcols)        ! Land fraction
   real(r8), intent(in) :: icefrac(pcols)         ! ice fraction
   real(r8), intent(in) :: ocnfrac(pcols)         ! Land fraction
   real(r8), intent(in) :: snowh(pcols)           ! snow depth (liquid water equivalent)
   real(r8), intent(in) :: fv(pcols)              ! for dry deposition velocities for dust from land model
   real(r8), intent(in) :: ram1(pcols)            ! for dry deposition velocities for dust from land model
   real(r8), intent(in) :: tref(pcols)            ! 2m air temperature
   real(r8), intent(in) :: precc(pcols)           ! convective precipitation
   real(r8), intent(in) :: precl(pcols)           ! large-scale precipitation
   real(r8), intent(in) :: fsds(pcols)            ! down solar flux
   real(r8), intent(in) :: precsc(pcols)          ! convective snow
   real(r8), intent(in) :: precsl(pcols)          ! large-scale snow
   real(r8), intent(out) :: pblh(pcols)           ! Planetary boundary layer height

   real(r8), intent(out) :: kbfs(pcols)           ! surface buoyancy flux

   real(r8), intent(out) :: qpert(pcols,ppcnst)   ! Moisture/constit. perturbation (PBL)
   real(r8), intent(out) :: tpert(pcols)          ! Temperature perturbation (PBL)
   real(r8), intent(inout) :: shf(pcols)          ! Sensible heat flux (w/m^2)
   real(r8), intent(in) :: taux(pcols)            ! X surface stress (zonal)
   real(r8), intent(in) :: tauy(pcols)            ! Y surface stress (meridional)
   real(r8), intent(inout) :: cflx(pcols,ppcnst)  ! Surface constituent flux (kg/m^2/s)
   real(r8), intent(in) :: sgh(pcols)             ! Std. deviation of orography for gwd
   real(r8), intent(in) :: ts(pcols)              ! srf temperature
   real(r8), intent(inout) :: lhf(pcols)          ! Latent heat flux (w/m^2)
   real(r8), intent(in)    :: fhs(pcols)    ! soil moisture content, wzz, 2008.3.12

#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
   integer  :: ncsec
   integer  :: lat(pcols)                  ! latitude index for S->N storage
   real(r8) :: clat(pcols)                 ! latitude
   integer  :: idx_CO2

   real(r8),intent(in)    :: chemflx(pcols,ppcnst)    
   real(r8),intent(in)    :: flx_mss_vrt_dst01(pcols)    !surface dust emission (kg/m**2/s) [ + = to atm]
   real(r8),intent(in)    :: flx_mss_vrt_dst02(pcols)    !surface dust emission (kg/m**2/s) [ + = to atm]
   real(r8),intent(in)    :: flx_mss_vrt_dst03(pcols)    !surface dust emission (kg/m**2/s) [ + = to atm]
   real(r8),intent(in)    :: flx_mss_vrt_dst04(pcols)    !surface dust emission (kg/m**2/s) [ + = to atm]
   real(r8),intent(in)    :: sflx_carbon(pcols)           !   gC/m**2/s ,+ up to atm
   real(r8),intent(in)    :: co2emis(pcols)               !   gC/m**2/s ,+ up to atm
   real(r8),intent(in)    :: fco2(pcols)
   real(r8)               :: sflx(pcols, ppcnst)

   real(r8),intent(in)    :: vlc_trb_SO4(pcols)       ! dry deposite velcity (m/s)
   real(r8),intent(in)    :: vlc_trb_NH4(pcols)       ! dry deposite velcity (m/s)
   real(r8),intent(in)    :: vlc_trb_CB2(pcols)       ! dry deposite velcity (m/s)
   real(r8),intent(in)    :: vlc_trb_SO2(pcols)       ! dry deposite velcity (m/s)

   real(r8)    :: lnd_ram1(pcols)
   real(r8)    :: lnd_fv(pcols)

   real(r8),intent(in)    :: u10(pcols)               ! zf 2015-05-19
   real(r8),intent(in)    :: v10(pcols)               ! zf 2015-05-19
#endif
  
#if (defined BCCCHEM) || (defined WACCM)
   integer  :: ncsec
   integer  :: lat(pcols)                  ! latitude index for S->N storage
   real(r8) :: clat(pcols)                 ! latitude
   integer  :: idx_CO2

   real(r8),intent(in)    :: flx_mss_vrt_dst01(pcols)    !surface dust emission (kg/m**2/s) [ + = to atm]
   real(r8),intent(in)    :: flx_mss_vrt_dst02(pcols)    !surface dust emission (kg/m**2/s) [ + = to atm]
   real(r8),intent(in)    :: flx_mss_vrt_dst03(pcols)    !surface dust emission (kg/m**2/s) [ + = to atm]
   real(r8),intent(in)    :: flx_mss_vrt_dst04(pcols)    !surface dust emission (kg/m**2/s) [ + = to atm]
   real(r8),intent(in)    :: sflx_carbon(pcols)           !   gC/m**2/s ,+ up to atm
   real(r8),intent(in)    :: co2emis(pcols)               !   gC/m**2/s ,+ up to atm
   real(r8),intent(in)    :: fco2(pcols)                  
   real(r8)               :: sflx(pcols, ppcnst)

   real(r8),intent(in)    :: vlc_trb_SO4(pcols)       ! dry deposite velcity (m/s)
   real(r8),intent(in)    :: vlc_trb_NH4(pcols)       ! dry deposite velcity (m/s)
   real(r8),intent(in)    :: vlc_trb_CB2(pcols)       ! dry deposite velcity (m/s)
   real(r8),intent(in)    :: vlc_trb_SO2(pcols)       ! dry deposite velcity (m/s)
#endif

#ifdef NSTEP_GW
   real(r8), intent(inout) :: utgw(pcols,pver)
   real(r8), intent(inout) :: vtgw(pcols,pver)
   real(r8), intent(inout) :: stgw(pcols,pver)
   real(r8), intent(inout) :: qtgw(pcols,pver,ppcnst)
   integer :: igw
   logical :: dogw
#endif

#ifdef CO2
   integer  :: idx_CO2
   real(r8),intent(in)    :: sflx_carbon(pcols)           !   gC/m**2/s ,+ up to atm
   real(r8),intent(in)    :: co2emis(pcols)               !   gC/m**2/s ,+ up to atm
   real(r8),intent(in)    :: fco2(pcols)
#endif

   type(physics_state), intent(inout) :: state
   type(physics_tend ), intent(inout) :: tend
   type(pbuf_fld),      intent(inout), dimension(pbuf_size_max) :: pbuf

   type(check_tracers_data):: tracerint             ! tracer mass integrals and cummulative boundary fluxes

#ifdef UWMT
   real(r8),intent(inout) :: kvm(pcols,pverp)
   real(r8),intent(inout) :: kvh(pcols,pverp)
#endif

!
!---------------------------Local workspace-----------------------------
!
   type(physics_ptend)     :: ptend               ! indivdual parameterization tendencies

#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
   real(r8) :: em2data (pcols, nem2)
   real(r8) :: density_rho(pcols)
#endif

   integer  :: nstep                              ! current timestep number
   real(r8) :: zero(pcols)                        ! array of zeros

   integer :: lchnk                                ! chunk identifier
   integer :: ncol                                 ! number of atmospheric columns
   integer i,k,m                 ! Longitude, level indices
   integer :: yr, mon, day, tod       ! components of a date

   logical :: labort                            ! abort flag

   real(r8) tvm(pcols,pver)           ! virtual temperature
   real(r8) prect(pcols)              ! total precipitation
   real(r8) surfric(pcols)              ! surface friction velocity
   real(r8) obklen(pcols)             ! Obukhov length
   real(r8) :: fh2o(pcols)            ! h2o flux to balance source from methane chemistry
!
   integer modx      ! for aerosol process selection, wzz, 2008.2.19
   real(r8) pretrow(pcols,pver,2)     ! precipitation(kg/m2 s), wzz, 2008.2.18
   real(r8) fkt(pcols,pverp), grcld_cs(pcols,pverp), grcld_ck(pcols,pverp), grcld_fn(pcols,pverp)
   real(r8), pointer, dimension(:,:,:) :: fracis         ! fraction of transported species that are insoluble
!  real(r8) fracis(pcols,pver,ppcnst)         ! fraction of transported species that are insoluble

! physics buffer fields for total energy and mass adjustment
   integer itim, ifld
   real(r8), pointer, dimension(:  ) :: teout
   real(r8), pointer, dimension(:,:) :: qini
   real(r8), pointer, dimension(:,:) :: tini
   real(r8), pointer, dimension(:,:) :: cld
   real(r8), pointer, dimension(:,:) :: concld   ! wzz for convective cloud cover
   real(r8), pointer, dimension(:,:) :: cstcld   ! wzz for layered cloud cover
   real(r8), pointer, dimension(:,:) :: pr_cn   
! for convective precipitation for aero-input, wzz, 2008.2.29
   real(r8), pointer, dimension(:,:) :: pr_cs   
! for large-scale precipitation for aero-input, wzz, 2008.2.29
   real(r8) :: ccflx(pcols,ppcnst)  ! Surface constituent flux (kg/m^2/s)
!
!-----------------------------------------------------------------------
   lchnk = state%lchnk
   ncol  = state%ncol
!   if(lchnk.eq.193) then
!   write(*,*)'wzz: lchnk in tphysac =', lchnk
!      write(0,*)'wzz: aero 0 in a:', state%q(2,pver,15)
!   endif

! Associate pointers with physics buffer fields
   itim = pbuf_old_tim_idx()
   ifld = pbuf_get_fld_idx('TEOUT')
   teout => pbuf(ifld)%fld_ptr(1,1:pcols,1,lchnk,itim)
   ifld = pbuf_get_fld_idx('QINI')
   qini  => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
   ifld = pbuf_get_fld_idx('TINI')
   tini  => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
   ifld = pbuf_get_fld_idx('CLD')
   cld => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)
   ifld = pbuf_get_fld_idx('CONCLD')
   concld  => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)
   ifld = pbuf_get_fld_idx('CSTCLD')
   cstcld  => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)
   ifld = pbuf_get_fld_idx('CONPRE')
   pr_cn   => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)
   ifld = pbuf_get_fld_idx('CSTPRE')
   pr_cs   => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)
   ifld = pbuf_get_fld_idx('FRACS')
   fracis  => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk, 1:ppcnst)  ! wzz, 2008.8.28

!
! wzz, 2008.2.27
    grcld_ck(:, 1)       = concld(:,1)
    grcld_ck(:, 2:pverp) = concld(:,:)
    grcld_cs(:, 1)       = cstcld(:,1)
    grcld_cs(:, 2:pverp) = cstcld(:,:)
    grcld_fn(:, 1)       = cld(:,1)
    grcld_fn(:, 2:pverp) = cld(:,:)
    pretrow(:,:,1)       = pr_cs(:,:)
    pretrow(:,:,2)       = pr_cn(:,:)
!
! accumulate fluxes into net flux array
! jrm Include latent heat of fusion for snow
!
   do i=1,ncol
      tend%flx_net(i) = tend%flx_net(i) + shf(i) + (precc(i) + precl(i))*latvap*rhoh2o &
           + (precsc(i) + precsl(i))*latice*rhoh2o
   end do

! Initialize parameterization tendency structure

   call physics_ptend_init(ptend)

   call t_startf ('aerosol_em')
!
   fracis(:,:,:) = 1.0
!
!   if(lchnk.eq.193) then
!      write(0,*)'wzz: aero 01 in a:', state%q(2,pver,15)
!   endif
   select case ( iaero )
   case ( 0 )
      call aerosol_emis_intr (state, ptend, cflx, ztodt)
   case ( 1 )
      modx = 0
      call intrfc(ztodt,nstep,state, ptend, ts,fsds, snowh, &
              landfrac, icefrac, ocnfrac, grcld_ck, grcld_cs, grcld_fn, &
              fkt, fhs, pretrow,ccflx,fracis, modx)
      cflx(:,4:ppcnst) = 0.
   case default
      call endrun('aerosol_emis:  unknown scheme')
   end select
!
   call physics_update (state, tend, ptend, ztodt)
!
   call t_stopf ('aerosol_em')

! get nstep and zero array for energy checker
   zero = 0.
   nstep = get_nstep()
   call check_tracers_init(state, tracerint)

! Check if latent heat flux exceeds the total moisture content of the
! lowest model layer, thereby creating negative moisture.

   call qneg4('TPHYSAC '       ,lchnk               ,ncol  ,ztodt ,          &
              state%q(1,pver,1),state%rpdel(1,pver) ,shf ,lhf ,cflx(1,1) )

!===================================================
! Source/sink terms for advected tracers.
!===================================================
! Test tracers

      call tracers_timestep_tend(state, ptend, cflx, landfrac, ztodt)      
      call physics_update (state, tend, ptend, ztodt)
      call check_tracers_chng(state, tracerint, "tracers_timestep_tend", nstep, ztodt, cflx)

! Advected greenhouse trace gases:

   if (trace_gas) then
      call chem_timestep_tend(state, ptend, cflx, ztodt, fh2o)
      call physics_update (state, tend, ptend, ztodt)
! Check energy integrals
!
      call check_energy_chng(state, tend, "chem", nstep, ztodt, fh2o, zero, zero, zero)
!
! Check tracer integrals
      call check_tracers_chng(state, tracerint, "chem_timestep_tend", nstep, ztodt, cflx)
   end if

!==========================================================================

#if (defined MOZART2)

    if ( is_perpetual() ) then
       call get_perp_date(yr, mon, day, ncsec)
    else
       call get_curr_date(yr, mon, day, ncsec)
    end if

    lchnk     = state%lchnk
    ncol      = state%ncol

    call get_lat_all_p (lchnk, ncol, lat)
    call get_rlat_all_p(lchnk, ncol, clat)

!   note that tendencies are not only in sfc layer (because of sedimentation)
!   and that ptend is updated within each subroutine for different species
!
!--------------------------------------------------------------------------
!
!  update cflx
!
    cflx(:,ixchm:ppcnst) = 0.0 + chemflx(:,ixchm:ppcnst)

    call bcc_dust_emis_intr (lchnk, ncol,  ztodt, cflx,  landfrac, state%pdel,     &    ! inout: cflx
                             flx_mss_vrt_dst01, flx_mss_vrt_dst02, flx_mss_vrt_dst03, flx_mss_vrt_dst04)

    call bcc_progseasalts_emis_intr (lchnk, ncol,  state%u, state%v,  state%zm, state%pdel,   &
                                     ztodt, cflx, ocnfrac, icefrac, u10, v10)

!----------------------------------------------
                                                                        ! inout: cflx
    call cnst_get_ind('CO2', idx_CO2)

#ifdef COUP_CSM

    if( fco2cyc ) then       !  for control run  (no emission)
        cflx(:,idx_CO2) =  cflx(:,idx_CO2) + ( sflx_carbon(:)*44.0/12.0 ) * 1.e-3
                                       ! from surface to atmosphere (gC/m2/s-> kgCO2/m2/s)
    else
        cflx(:,idx_CO2) =  cflx(:,idx_CO2) + ( sflx_carbon(:)*44.0/12.0 + co2emis(:) ) * 1.e-3
                                       ! from surface to atmosphere (gC/m2/s-> kgCO2/m2/s)
    endif

#else

    cflx(:,idx_CO2) =  cflx(:,idx_CO2) + ( sflx_carbon(:)*44.0/12.0*landfrac(:)  & 
                                          + fco2(:)*(1.0-landfrac(:)) + co2emis(:) ) *1.e-3 
                                       ! from surface to atmosphere (gC/m2/s-> kgCO2/m2/s)
#endif
!-----------------------
#endif

!---------------------------------------------------------------------------------------------
#ifdef CO2 
!
!  update cflx
!

    call cnst_get_ind('CO2', idx_CO2)

#ifdef COUP_CSM

    if( fco2cyc ) then       !  for control run  (no emission)
        cflx(:,idx_CO2) =  ( sflx_carbon(:)*44.0/12.0 ) * 1.e-3
                                       ! from surface to atmosphere (gC/m2/s-> kgCO2/m2/s)
    else
        cflx(:,idx_CO2) =  ( sflx_carbon(:)*44.0/12.0 + co2emis(:) ) * 1.e-3
                                       ! from surface to atmosphere (gC/m2/s-> kgCO2/m2/s)
    endif

#else

    cflx(:,idx_CO2) =  ( sflx_carbon(:)*44.0/12.0*landfrac(:)  & 
                                          + fco2(:)*(1.0-landfrac(:)) + co2emis(:) ) *1.e-3 
                                       ! from surface to atmosphere (gC/m2/s-> kgCO2/m2/s)
#endif

#endif
!===================================================
! Vertical diffusion/pbl calculation
! Call vertical diffusion code (pbl, free atmosphere and molecular)
!===================================================

!#ifdef MICROP-MG
!   call vd_intr (ztodt    ,state    ,taux     ,tauy     , shf    ,&
!                 cflx     ,pblh     ,kbfs     ,tpert    ,qpert    , surfric  ,&
!                 obklen   ,ptend    ,cld      ,ocnfrac  , landfrac, sgh , pbuf)
#ifdef UWMT
   call vd_intr (ztodt    ,state    ,taux     ,tauy     , shf    ,&
                 cflx     ,pblh     ,kbfs     ,tpert    ,qpert    , surfric  ,&
                 obklen   ,ptend    ,cld      ,cstcld   , ocnfrac  , landfrac, sgh, kvm, kvh )
#else
   call vd_intr (ztodt    ,state    ,taux     ,tauy     , shf    ,&
                 cflx     ,pblh     ,kbfs     ,tpert    ,qpert    , surfric  ,&
                 obklen   ,ptend    ,cld      ,ocnfrac  , landfrac, sgh )
#endif

   call physics_update (state, tend, ptend, ztodt)

!===================================================
! Rayleigh friction calculation, add by luyx20170119
!===================================================
   if (do_rayleigh) then
      call rayleigh_friction_tend( ztodt, state, ptend)
      call physics_update (state, tend, ptend, ztodt)
   endif

! Check energy integrals
!-------------------
!
   call check_energy_chng(state, tend, "vdiff", nstep, ztodt, cflx(:,1), zero, zero, shf)
!----------
   call check_tracers_chng(state, tracerint, "vdiff", nstep, ztodt, cflx)

!  aerosol dry deposition processes
!
!   if(lchnk.eq.193) then
!      write(0,*)'wzz: aero 1 in a:', state%q(2,pver,15)
!   endif
   call t_startf ('aerosol_dry')
   select case ( iaero )
   case ( 0 )
      call aerosol_drydep_intr (state, ptend, cflx(:,1), ztodt, &
       fsds, obklen, tref, surfric, prect, snowh, pblh, shf, landfrac, &
       icefrac, ocnfrac, fv, ram1)
   case ( 1 )
      modx = 2
      call intrfc(ztodt,nstep,state, ptend, ts, fsds, snowh, &
           landfrac, icefrac, ocnfrac, grcld_ck, grcld_cs, grcld_fn,    &
           fkt, fhs, pretrow, ccflx,fracis, modx)
   case default
      call endrun('aerosol_drydep:  unknown scheme')
   end select

!   if(lchnk.eq.193) then
!      write(0,*)'wzz: aero 2 in a:', state%q(2,pver,15)
!   endif

   call physics_update (state, tend, ptend, ztodt)

!   if(lchnk.eq.193) then
!      write(0,*)'wzz: aero 3 in a:', state%q(2,pver,15)
!   endif
!
   call t_stopf ('aerosol_dry')

!==========================================================================

#ifdef MOZART2
!
!  3-Dimension drydep    update: ptend%
!
!   CB1, CB2, OC1, OC2, SO4 other_aerosols_drydep
!
 
    call other_aerosols_drydep (lchnk, ncol, state%q, state%pdel, state%t, state%pmid, ptend%lq, ptend%q, ztodt, &
                                obklen, surfric, pblh,  &
                                landfrac, icefrac, ocnfrac, lnd_fv, lnd_ram1 ,vlc_trb_SO2  )

    call dust_drydep_intr (lchnk, ncol,   state%q, state%pdel, state%t, state%pmid, ptend%lq, ptend%q, ztodt, lat,   clat,    &
                           fsds,  obklen, ts,    surfric, pblh,                 &
                           mon,   landfrac,      icefrac, ocnfrac,      lnd_fv,    lnd_ram1 )

    call progseasalts_drydep_intr ( lchnk,   ncol,    state%q, state%pdel, state%t, state%pmid, ptend%lq, ptend%q, &
                                    ztodt,    lat,     clat,                     &
                                    fsds,    obklen,  ts,       surfric, pblh, &
                                    mon,     landfrac,icefrac,  ocnfrac, lnd_fv,    lnd_ram1)

    call physics_update (state, tend, ptend, ztodt)

#endif

!================================================
!	... enforce charge neutrality
!================================================

#if ( defined WACCM )
    call charge_fix( ncol, state%q(:,:,:) )
#endif

!===================================================
! Gravity wave drag
!===================================================

#ifdef NSTEP_GW

    dogw = nstep.eq.0 .or. igw.eq.1 .or. (mod(nstep-1,igw).eq.0 .and. nstep.ne.1)

    if ( dogw ) then
       call gw_intr (state   ,sgh     ,pblh    ,ztodt   , ptend , landfrac)
       utgw(:ncol,:pver)         = ptend%u(:ncol,:pver)
       vtgw(:ncol,:pver)         = ptend%v(:ncol,:pver)
       stgw(:ncol,:pver)         = ptend%s(:ncol,:pver)
       qtgw(:ncol,:pver,:ppcnst) = ptend%q(:ncol,:pver,:ppcnst)
    else
       ptend%u(:ncol,:pver)         = utgw(:ncol,:pver)
       ptend%v(:ncol,:pver)         = vtgw(:ncol,:pver)
       ptend%s(:ncol,:pver)         = stgw(:ncol,:pver)
       ptend%q(:ncol,:pver,:ppcnst) = qtgw(:ncol,:pver,:ppcnst)
       ptend%lu          = .TRUE.
       ptend%lv          = .TRUE.
       ptend%ls          = .TRUE.
       ptend%lq(:ppcnst) = .TRUE.
    endif

#else

    call gw_intr (state   ,sgh     ,pblh    ,ztodt   , ptend , landfrac)

!----
#endif

    call physics_update (state, tend, ptend, ztodt)

!==============================================================
! Ion drag calculation
!===============================================================

#if ( defined WACCM )

   call t_startf ( 'iondrag' )
   call iondrag_calc( lchnk, ncol, state, ptend, pbuf, ztodt )
   call physics_update (state, tend, ptend, ztodt)

   call check_energy_chng(state, tend, "iondrag", nstep, ztodt, zero, zero, zero, zero)
   call t_stopf  ( 'iondrag' )

#endif
!
! Check energy integrals
!----
!
   call check_energy_chng(state, tend, "gwdrag", nstep, ztodt, zero, zero, zero, zero)
!
! conversion of aerosols from one category to another by non-wet processes
   if ( iaero == 0 ) then   ! wzz, 2007.1.11
   call aerosol_srcsnk_intr (state, ptend, ztodt, ocnfrac)
   else if ( iaero == 1 ) then
   end if
   call physics_update (state, tend, ptend, ztodt)

!-------------- Energy budget checks vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
   teout(:ncol) = state%te_cur(:ncol)

!*** BAB's FV heating kludge *** apply the heating as temperature tendency.
!*** BAB's FV heating kludge *** modify the temperature in the state structure
   state%t(:ncol,:pver) = tini(:ncol,:pver) + ztodt*tend%dtdt(:ncol,:pver)

!
! FV: convert dry-type mixing ratios to moist here because physics_dme_adjust
!     assumes moist. This is done in p_d_coupling for other dynamics. Bundy, Feb 2004.
!
   if ( dycore_is('LR') ) call set_dry_to_wet(state)    ! Physics had dry, dynamics wants moist


! Scale dry mass and energy (does nothing if dycore is EUL or SLD)
   call physics_dme_adjust(state, tend, qini, ztodt)
!!!   REMOVE THIS CALL, SINCE ONLY Q IS BEING ADJUSTED. WON'T BALANCE ENERGY. TE IS SAVED BEFORE THIS
!!!   call check_energy_chng(state, tend, "drymass", nstep, ztodt, zero, zero, zero, zero)
!-------------- Energy budget checks ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

   if (aqua_planet) then
      labort = .false.
      do i=1,ncol
         if (ocnfrac(i) /= 1.) labort = .true.
      end do
      if (labort) then
         call endrun ('TPHYSAC error:  grid contains non-ocean point')
      endif
   endif

!   if(lchnk.eq.193) then
!      write(0,*)'wzz: aero 4 in a:', state%q(2,pver,15)
!   endif
end subroutine tphysac
