#include <misc.h>
#include <params.h>

#if (defined MOZART2)

subroutine tphysbc_chem (ztodt,     ts,         state,     tend,          &
                         pbuf,      landfrac,   ocnfrac,   icefrac,       &
                         lcwat,     cld,        concld,    fracis,        &
                         ixcldice,  ixcldliq,      &
                         nevapr,    prain,      cmfmc,           &
                         cmfdqr,    cnt,        cnb,             &
                         ptend,     nstep,      calday,    asdir  , &
                         flux_ISOP, flux_ACET,  flux_C3H6,                &
                         flux_C2H4, flux_OC2,   flux_C10H16,              &
                         flux_CO2,  flux_N2O,   flux_DMS,                 &
                         sflx,      troplev,    mbar )
 
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Tendency physics BEFORE coupling to land, sea, and ice models.
! 
! Method: 
! Call physics subroutines and compute the following:
!     o cloud calculations (cloud fraction, emissivity, etc.)
!     o radiation calculations
! Pass surface fields for separate surface flux calculations
! Dump appropriate fields to history file.
! 
! Author: CCM1, CMS Contact: J. Truesdale
! 
!-----------------------------------------------------------------------

   use shr_kind_mod,    only: r8 => shr_kind_r8
   use ppgrid
   use phys_grid,       only: get_rlat_all_p, get_rlon_all_p, get_lat_all_p
   use phys_buffer,     only: pbuf_size_max, pbuf_fld
   use physics_types,   only: physics_state, physics_tend, physics_ptend, physics_update
   use history,         only: outfld
   use time_manager,    only: get_step_size

   use constituents,    only: pcnst, pnats, ppcnst, qmin, iaero
   use check_energy,    only: check_energy_chng

#ifdef MOZART2
   use constituents,    only: sflxnam
   use mo_hook,         only: moz_hook    !2008.07.10
   use mo_surface,      only: sflxdr      !2008.08.11
   use mo_chemdr,       only: chemdr
   use chemistry,       only: ixchm, ncnst
   use mo_chem_mods,           only: grpcnt
   use mo_chem_utls,  only : get_grp_mem_ndx
   use constituents,           only: cnst_get_ind
   use bcc_progseasalts_intr,  only: bcc_progseasalts_wet
   use bcc_dust_intr,          only: bcc_dust_wet
   use bcc_aerosols_intr,      only: mz_aero_wet_intr
   use bcc_wetdep,             only: bcc_clddiag

   use acbnd,            only: acbndget
   use prescribed_em_d3, only: getem3_IPCC, nem3
   use prescribed_emis_3d, only: getem3_IPCC6, nem3_IPCC6
   use prescribed_em_d2, only: getem2_IPCC, nem2, varname2d
   use prescribed_em_d2_anthro, only: getem2_IPCC_anthro, nem2_anthro
   use prescribed_em_d2, only: idxCO, idxCH4, idxNO, idxC2H4
   use prescribed_aerosols, only: get_aerosol, naer_all, idxSUL, get_int_scales
#ifdef MAC_SP
   use prescribed_aerosols_cdnc, only: get_aerosol_cdnc
#endif

#endif


!#ifdef WACCM 
!   use prescribed_em_d3,       only: getem3_IPCC, nem3, varname3d
!   use prescribed_em_d2,       only: getem2_IPCC, nem2, varname2d
!   use prescribed_aerosols,    only: get_aerosol, naer_all, idxSUL
!   use physconst,              only : gravit
!   use chem_mods,              only : imozart
!   use m_spc_id,               only : id_H2O, id_O3, id_HNO3
!   use mo_gas_phase_chemdr,    only: gas_phase_chemdr
!#endif

!#ifdef BCCCHEM
!   use bcc_charge_neutrality,  only: charge_fix
!   use bcc_iondrag,            only: iondrag_calc, do_waccm_ions
!   use bcc_qbo,                only: qbo_relax
!   use bcc_surface,            only: sflxdr
!   use bcc_neu_wetdep,         only: do_neu_wetdep
!   use bcc_qbo,                only: qbo_use_forcing
!   use constituents,           only: sflxnam
!   use bcc_tropopause,         only: tropopause_twmo
!   use bcc_chemdr,             only: chemdr, qh2o_reset
!   use bcc_neu_wetdep,         only: neu_wetdep_tend
!   use bcc_progseasalts_intr,  only: bcc_progseasalts_wet
!   use bcc_dust_intr,          only: bcc_dust_wet
!   use physconst,              only: latvap
!   use bcc_aerosols_intr,      only: mz_aero_wet_intr
!   use bcc_wetdep,             only: bcc_clddiag
!   use bcc_chem_mods,          only: adv_mass
!   use time_manager,           only: get_curr_date, get_perp_date
!   use prescribed_aerosols,    only: get_aerosol, naer_all, idxSUL
!   use chemistry,              only: ixchm
!   use bcc_chem_mods,          only: gas_pcnst
!   use acbnd,                  only: acbndget
!   use prescribed_em_d3,       only: getem3_IPCC, nem3, varname3d
!   use prescribed_em_d2,       only: getem2_IPCC, nem2, varname2d
!   use prescribed_aerosols,    only: get_aerosol, naer_all, idxSUL
!   use time_manager,           only: is_perpetual
!   use time_manager,           only: get_nstep
!   use bcc_chem_mods,          only : map2chm
!   use bcc_chem_utls,          only : get_spc_ndx
!   use bcc_chem_mods,          only : solsym
!   use constituents,           only: cnst_get_ind
!#endif

!#ifdef GEOSCHEM
!   use wv_saturation,   only: aqsat
!   use gc_emis_edgar,   only: coembdy
!   use chemistry,       only: ncnst, ixchm
!   use gc_chemdr,       only: do_gc_chem, molar_mass
!#endif

   implicit none

#include <comctl.h>
!
! Arguments
!
   real(r8), intent(in) :: ztodt                          ! 2 delta t (model time increment)
   real(r8), intent(in) :: ts(pcols)                      ! surface temperature
   real(r8), intent(in) :: landfrac(pcols)                ! land fraction

   real(r8), intent(in) :: ocnfrac(pcols)                 ! land fraction
   real(r8), intent(in) :: icefrac(pcols)                 ! land fraction

   type(physics_state), intent(inout) :: state
   type(physics_tend ), intent(inout) :: tend
   type(pbuf_fld),      intent(inout), dimension(pbuf_size_max) :: pbuf
!
#ifdef MOZART2
    real(r8),intent(inout) :: sflx(pcols, ppcnst)
    integer                :: m
    real(r8),intent(out)   :: mbar(pcols,pver)      ! mean wet atmospheric mass ( amu )
    integer, intent(in)    :: troplev (pcols)

    real(r8),intent(in)    :: flux_ISOP(pcols)
    real(r8),intent(in)    :: flux_ACET(pcols)
    real(r8),intent(in)    :: flux_C3H6(pcols)
    real(r8),intent(in)    :: flux_C2H4(pcols)
    real(r8),intent(in)    :: flux_OC2(pcols)
    real(r8),intent(in)    :: flux_C10H16(pcols)

    real(r8),intent(in)    :: flux_CO2(pcols)
    real(r8),intent(in)    :: flux_N2O(pcols)
    real(r8),intent(in)    :: flux_DMS(pcols)

    real(r8):: totcond(pcols,pver)
    real(r8):: evapc(pcols,pver)           ! Evaporation rate of convective precipitation   zf????????????
    real(r8):: rainmr(pcols,pver)  ! mixing ratio of rain within cloud volume
    real(r8):: cldv(pcols,pver)     ! cloudy volume undergoing wet chem and scavenging
    real(r8):: cldvcu(pcols,pver)   ! Convective precipitation area at the top interface of current layer
    real(r8):: cldvst(pcols,pver)   ! Stratiform precipitation area at the top interface of current layer
#endif


!#ifdef BCCCHEM
!   integer id3_NO
!
!   real(r8),intent(out)   :: mbar(pcols,pver)      ! mean wet atmospheric mass ( amu )
!
!   real(r8),intent(inout) :: sflx(pcols, ppcnst)
!   integer, intent(in)    :: troplev (pcols)
!
!   integer                :: n, m
!   integer                :: yr, mon, day, ncsec
!
!    real(r8):: totcond(pcols,pver)
!    real(r8):: evapc(pcols,pver)           ! Evaporation rate of convective precipitation   zf????????????
!    real(r8):: rainmr(pcols,pver)  ! mixing ratio of rain within cloud volume
!    real(r8):: cldv(pcols,pver)     ! cloudy volume undergoing wet chem and scavenging
!    real(r8):: cldvcu(pcols,pver)   ! Convective precipitation area at the top interface of current layer
!    real(r8):: cldvst(pcols,pver)   ! Stratiform precipitation area at the top interface of current layer
!#endif

#if (defined MOZART2)
   real(r8) :: h2o23d(pcols,pver)
   real(r8) :: o3(pcols,pver)
   real(r8) :: ho2im(pcols,pver)
   real(r8) :: no3(pcols,pver)
   real(r8) :: oh(pcols,pver)


   real(r8) :: em3data (pcols, pver, nem3)
   real(r8) :: em2data (pcols, nem2)
   real(r8) :: em2data_anthro (pcols, nem2_anthro)
   real(r8) :: em3_IPCC6(pcols, pver, nem3_IPCC6)
   integer idx

   real(r8) :: asdir(pcols)

   real(r8) aerosol(pcols, pver, naer_all) ! aerosol mass mixing ratios
   real(r8) scales(naer_all)               ! scaling factors for aerosols

#endif

!#ifdef GEOSCHEM
!   integer coidx

!   real(r8) :: srfflx(pcols)
!   real(r8), parameter :: amufac   =  1.65979e-23         ! 1.e4* kg / amu

!   real(r8),intent(inout) :: sflx(pcols, ppcnst)
!   real(r8)               :: albedo(pcols)
!   real(r8)               :: coslat(pcols)
!   real(r8)               :: ptrop(pcols)
!   real(r8)               :: trefht(pcols)
!   real(r8)               :: shflx(pcols)
!   real(r8)               :: u_10m(pcols)
!   real(r8)               :: v_10m(pcols)
!   real(r8)               :: z0(pcols)
!   real(r8)               :: ustar(pcols)
!   real(r8)               :: es(pcols, pver)
!   real(r8)               :: qs(pcols, pver)
!   real(r8)               :: relhum(pcols, pver)
!   real(r8)               :: dz(pcols, pver)
!   real(r8)               :: tend_ciw(pcols, pver)
!   real(r8)               :: tend_clw(pcols, pver)
!   real(r8)               :: tend_w(pcols, pver)
!   real(r8)               :: tend_dynq(pcols, pver)   ! the tendency of q due to dynamics
!   real(r8)               :: rho(pcols, pver)
!   real(r8)               :: tauoptd(pcols,0:pver)
!   real(r8)               :: uv_albedo(pcols)
!   real(r8)               :: gwettop(pcols)
!   integer                :: tmp
!   logical :: FIRST = .true.
!   real(r8)               :: convert(pcols,pver) ! Unit conversion variable
!   real(r8)               :: emis_tmp(pcols, pver), tmp_q(pcols,pver)
!#endif

!#ifdef WACCM
!   real(r8)               :: cldw(pcols, pver)
!   real(r8),intent(inout) :: sflx(pcols, ppcnst)
!   real(r8)               :: fh2o(pcols)
!   real(r8),intent(out)   :: mbar(pcols,pver)      ! mean wet atmospheric mass ( amu )
!   integer, intent(in)    :: troplev (pcols)
!
!   real(r8)               :: qtemp(pcols,pver,ppcnst)
!#endif
!
!---------------------------Local workspace-----------------------------
!
   type(physics_ptend),intent(inout)  :: ptend                  ! indivdual parameterization tendencies

   integer,intent(in)  :: nstep                         ! current timestep number
   integer      lat(pcols)                   ! current latitudes(indices)

   real(r8),intent(in)  :: calday                        ! current calendar day
   real(r8) :: clat(pcols)                   ! current latitudes(radians)
   real(r8) :: clon(pcols)                   ! current longitudes(radians)

   real(r8),intent(in)  :: cmfmc(pcols,pverp)            ! Convective mass flux--m sub c

   real(r8),intent(in)  :: cnt(pcols)                        ! Top level of convective activity
   real(r8),intent(in)  :: cnb(pcols)                        ! Lowest level of convective activity

   integer lchnk                              ! chunk identifier
   integer ncol                               ! number of atmospheric columns


   integer  i,k                                          ! Longitude, level, constituent indices
   integer,intent(in) :: ixcldice, ixcldliq              ! constituent indices for cloud liquid and ice water.
   
   integer idx_H2O, idx_so4
   integer idx_O3, idx_O, idx_O1D
   integer o3_nadx, o_nadx, o1d_nadx

   real(r8) nas(pcols,pver,max(1,grpcnt))           ! non-advected species( mmr )

!---------------------------------
!
! physics buffer fields to compute tendencies for cloud condensation package
   real(r8),intent(in)   :: lcwat(pcols,pver)
   real(r8),intent(in)   :: cld(pcols,pver)
   real(r8),intent(in)   :: concld(pcols,pver)   ! wzz for convective cloud cover

   real(r8) :: prod_no(pcols,pver)
!                                          
! Used for OUTFLD only                     
!                                          
   real(r8),intent(inout) ::  fracis(pcols,pver,ppcnst)  ! fraction of transported species that are insoluble
!
   real(r8),intent(in)    ::  cmfdqr(pcols,pver)               ! dq/dt due to moist convective rainout

! energy checking variables
   real(r8) :: zero(pcols)                    ! array of zeros

   real(r8),intent(in)    ::  nevapr(pcols,pver)
   real(r8),intent(in)    ::  prain(pcols,pver)

!-----------------------------------------------------------------------

  zero = 0.
  lchnk = state%lchnk
  ncol  = state%ncol

#if (defined MOZART2)
!
! Get surface emission (em3data and em2data)
!
   em3data(:,:,:) = 0.0_r8
   em2data(:,:)   = 0.0
   em2data_anthro(:,:)   = 0.0
   em3_IPCC6(:,:,:) = 0.0_r8

   !
   ! out: em3data
   !
   call  getem3_IPCC( lchnk, ncol, state%zm, em3data )
   call  getem3_IPCC6( lchnk, ncol, state%zi, em3_IPCC6 )

   !
   ! out: em2data
   !
   call  getem2_IPCC( lchnk, ncol, em2data )
   call  getem2_IPCC_anthro( lchnk, ncol, em2data_anthro )
!         
!   do idx=1, nem3
!       call outfld(varname3d(idx),  em3data(1,1,idx)  ,pcols   ,lchnk   )
!   enddo
!
!   do idx=1, nem2
!       call outfld(varname2d(idx),  em2data(1,idx)  ,pcols   ,lchnk   )
!   enddo
!
!---------------------------------------------------------

   call get_lat_all_p (lchnk, ncol, lat )
   call get_rlat_all_p(lchnk, ncol, clat)
   call get_rlon_all_p(lchnk, ncol, clon)
   
   call moz_hook(ncol, prod_no, cnt, cnb, state%zm, state%zi, state%t, &
                 landfrac, ocnfrac, icefrac, pcols, clat, lchnk )

   !
   ! update sflx ,but do not update q(:,:,:) 
   !
   call sflxdr(ztodt, calday         , state%q(1,1,ixchm) , state%pmid  ,state%pdel,  ts    ,     &
               state%q(1,1,1) , state%t        , pcols ,    sflx(1,ixchm),        &
               lchnk          , ncol           , landfrac , &
               flux_ISOP      , flux_ACET      ,  flux_C3H6,                 &
               flux_C2H4      , flux_OC2       ,  flux_C10H16,               &
               flux_CO2       , flux_N2O       ,  flux_DMS,                  &
               nem2_anthro    , em2data_anthro ,                             &
               nem2, em2data(1,1) )
   !
   ! out: aerosol 
   !

!#ifdef MAC_SP
!   call get_int_scales(scales)
!   call get_aerosol_cdnc(lchnk, state%pint, aerosol, scales, state)
!#else
!   call get_int_scales(scales)
!   call get_aerosol(lchnk, state%pint, aerosol, scales, state)
!#endif

    call cnst_get_ind('SO4', idx_so4)
    do i=1, ncol 
       do k=1, pver
          aerosol(i,k,idxSUL) = state%q(i,k,idx_so4)   ! mmr
       enddo
    enddo
   !
   ! inout: state%q 
   !
   call chemdr(ztodt,   state%q(1,1,ixchm), state%ps, state%pmid, state%pdel     ,     &
               ts            , state%zm, state%zi  , state%phis     ,     &
               cld           , cnt     , cmfdqr    , prain          ,     &
               nevapr        , lcwat   , state%t   , state%q(1,1,1) ,     &
               lchnk         , ncol    , pcols     , nas,                 & 
               em3data(1,1,1)  , nem3, aerosol(1,1,idxSUL) , &
               em3_IPCC6     , nem3_IPCC6,                                &
               asdir, troplev , prod_no )

   call cnst_get_ind('O3', idx_O3)
   call cnst_get_ind('O1D', idx_O1D)
   call cnst_get_ind('O', idx_O)

   o3_nadx  = get_grp_mem_ndx( 'O3' )
   o1d_nadx = get_grp_mem_ndx( 'O1D' )
   o_nadx   = get_grp_mem_ndx( 'O' )

   state%q(:ncol,:pver,idx_O3)  =  nas(:ncol,:pver,o3_nadx)
   state%q(:ncol,:pver,idx_O1D) =  nas(:ncol,:pver,o1d_nadx)
   state%q(:ncol,:pver,idx_O)   =  nas(:ncol,:pver,o_nadx)
 
#endif

   return
end subroutine tphysbc_chem
#endif
