#include <misc.h>
#include <params.h>


subroutine tphysbc_moist (ztodt,   ts,      sst,     snowh,   landm,   &
                    landfrac,ocnfrac, icefrac, fhs,     dtcond,  &
                    dqcond,  pblht,   kbfs  ,  tpert,   qpert,   precl,   &
                    precc,   precsl,  precsc,  prcsnw,  fsds,    &
                    cmfmc,   cldst,   cnt,     cnb,     &
                    dlf,     mc,      prain,   nevapr,  cmfdqr, &
                    du2,     conicw,  ixcldice,ixcldliq,tracerint, &
                    state,   tend,    pbuf,    ptend,   dyn_tend,  &
                    fracis,  qcwat,   tcwat,   lcwat,   cld,       &
                    q0_conv, t0_conv, date0_conv,                 &
                    concld,  cstcld,  pr_cn,   pr_cs )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Tendency physics BEFORE coupling to land, sea, and ice models.
! 
! Method: 
! Call physics subroutines and compute the following:
!     o cloud calculations (cloud fraction, emissivity, etc.)
!     o radiation calculations
! Pass surface fields for separate surface flux calculations
! Dump appropriate fields to history file.
! 
! Author: CCM1, CMS Contact: J. Truesdale
! 
!-----------------------------------------------------------------------


   use shr_kind_mod,    only: r8 => shr_kind_r8
   use ppgrid
   use phys_grid,       only: get_rlat_all_p, get_rlon_all_p, get_lon_all_p, get_lat_all_p
   use phys_buffer,     only: pbuf_size_max, pbuf_fld, pbuf_old_tim_idx, pbuf_get_fld_idx
   use cldcond,         only: cldcond_tend, cldcond_zmconv_detrain, cldcond_sediment
   use physics_types,   only: physics_state, physics_tend, physics_ptend, physics_update, &
                              dynamics_tend
   use history,         only: outfld
   use physconst,       only: gravit, cpair
   use wv_saturation,   only: aqsat
   use constituents,    only: pcnst, pnats, ppcnst, iaero
   use physconst,       only: epsilo, latvap
   use moistconvection, only: rgas

   use constituents,    only: dcconnam, cnst_get_ind
   use wu_conv,         only: wu_conv_evap, wu_convr
   use time_manager,    only: is_first_step, get_nstep, get_curr_calday, get_curr_date
   use moistconvection, only: cmfmca
   use check_energy,    only: check_energy_chng
   use check_energy,    only: check_tracers_data, check_tracers_chng
   use aerosol_intr,    only: aerosol_wet_intr
   use cuace_driver,    only: intrfc    !   wzz, 2008.9.8
   use abortutils,      only: endrun
   use cldnrhmod,       only: cldnrh

#ifdef MICROP-MG
  use macrop_driver,   only: macrop_driver_tend
  use microp_driver,   only: microp_driver_tend
  use aerosol_intr,    only: microp_aerosol_wet_intr
  use param_cldoptics, only: microp_param_cldoptics_calc
  use wu_conv,         only :microp_wu_conv_evap
#endif

!++sxj
#ifdef MG08
  use pmgrid,          only: masterproc,iam
  use cldcond,         only: stratiform_tend
!#else
!  use cldcond,         only: cldcond_tend, cldcond_zmconv_detrain, cldcond_sediment
#endif
!--sxj

#ifdef BCC_DRAG
   use phys_buffer,    only : pbuf_get_fld_idx
#endif

   implicit none


#include <comctl.h>
!
! Arguments
!
   real(r8), intent(in) :: ztodt                          ! 2 delta t (model time increment)
   real(r8), intent(in) :: ts(pcols)                      ! surface temperature
   real(r8), intent(in) :: sst(pcols)                     ! sea surface temperature
   real(r8), intent(in) :: snowh(pcols)                  ! Snow depth (liquid water equivalent)
   real(r8), intent(in) :: landm(pcols)                   ! land fraction ramp
   real(r8), intent(in) :: landfrac(pcols)                ! land fraction
   real(r8), intent(in) :: ocnfrac(pcols)                 ! land fraction
   real(r8), intent(in) :: icefrac(pcols)                 ! land fraction
   real(r8), intent(in) :: fhs(pcols)    ! soil moisture content, wzz, 2008.3.12
   real(r8), intent(inout) :: dtcond(pcols,pver)            ! dT/dt due to moist processes
   real(r8), intent(inout) :: dqcond(pcols,pver)            ! dq/dt due to moist processes
   real(r8), intent(inout) :: date0_conv

   real(r8), intent(in)    :: kbfs(pcols)                 ! surface turbulence flux
   real(r8), intent(inout) :: pblht(pcols)                ! Planetary boundary layer height
   real(r8), intent(inout) :: tpert(pcols)                ! Thermal temperature excess
   real(r8), intent(inout) :: qpert(pcols,ppcnst)         ! Thermal humidity & constituent excess
   real(r8), intent(out) :: precl(pcols)                  ! Large-scale precipitation rate
   real(r8), intent(out) :: precc(pcols)                  ! Convective-scale preciptn rate
   real(r8), intent(out) :: precsl(pcols)                 ! L.S. snowfall rate
   real(r8), intent(out) :: precsc(pcols)                 ! C.S. snowfall rate
   real(r8), intent(out) :: prcsnw(pcols)                 ! snowfall rate (precsl + precsc)
   real(r8), intent(out) :: fsds(pcols)                   ! Surface solar down flux  zf?????????????---in
   real(r8), intent(out) :: cmfmc(pcols,pverp)            ! Convective mass flux--m sub c
   real(r8), intent(out) :: cldst(pcols,pver)             ! stratus cloud cover over ocean
   real(r8), intent(out) :: cnt(pcols)                        ! Top level of convective activity
   real(r8), intent(out) :: cnb(pcols)                        ! Lowest level of convective activity
   real(r8), intent(out) :: dlf(pcols,pver)                   ! Detraining cld H20 from convection
   real(r8), intent(out) :: mc(pcols,pver)
   real(r8), intent(out) :: prain(pcols,pver)
   real(r8), intent(out) :: nevapr(pcols,pver)
   real(r8), intent(out) :: du2(pcols,pver)
   real(r8), intent(out) :: conicw(pcols,pver)
   real(r8), intent(inout) :: q0_conv(pcols,pver)           ! dq/dt due to moist processes
   real(r8), intent(inout) :: t0_conv(pcols,pver)            ! dt/dt due to moist processes

   integer,  intent(out) :: ixcldice, ixcldliq              ! constituent indices for cloud liquid and ice water.

   real(r8) cmfdqr(pcols,pver)               ! dq/dt due to moist convective rainout
   
   type(check_tracers_data), intent(inout)  :: tracerint             ! energy integrals and cummulative boundary fluxes
   type(physics_state),      intent(inout)  :: state
   type(physics_tend ),      intent(inout)  :: tend
   type(pbuf_fld),           intent(inout), dimension(pbuf_size_max) :: pbuf
   type(physics_ptend),      intent(inout)  :: ptend                  ! indivdual parameterization tendencies
   type(dynamics_tend),      intent(inout)  :: dyn_tend
   real(r8),                 intent(inout)  :: qcwat(pcols,pver)
   real(r8),                 intent(inout)  :: tcwat(pcols,pver)
   real(r8),                 intent(inout)  :: lcwat(pcols,pver)
   real(r8),                 intent(inout)  :: cld(pcols,pver)
   real(r8),                 intent(inout)  :: fracis(pcols,pver,ppcnst)  ! fraction of transported species that are insoluble zf???
   real(r8),                 intent(inout)  :: concld(pcols,pver)   ! wzz for convective cloud cover
   real(r8),                 intent(inout)  :: cstcld(pcols,pver)   ! wzz for layered cloud cover
   real(r8),                 intent(inout)  :: pr_cn(pcols,pver)   ! for convective precipitation for aero-input, wzz, 2008.2.29
   real(r8),                 intent(inout)  :: pr_cs(pcols,pver)   ! for large-scale precipitation for aero-input, wzz, 2008.2.29

#ifdef GEOSCHEM
   real(r8),intent(out)   :: tend_dynq(pcols, pver)   ! the tendency of q due to dynamics
#endif

!
!---------------------------Local workspace-----------------------------
!
   real(r8) qbarb(pcols,pver)
   real(r8) tbarb(pcols,pver)
   real(r8) qsbar(pcols,pver)
   real(r8) esbar(pcols,pver)
   real(r8) dqsdtp(pcols,pver)

   real(r8) tdeepcld(pcols,pver)
   real(r8) tshallowcld(pcols,pver)

   real(r8) qdeepcld(pcols,pver)
   real(r8) qshallowcld(pcols,pver)

   real(r8) deepcld(pcols,pver)
   real(r8) shallowcld(pcols,pver)
!---
   real(r8) :: rhdfda(pcols,pver)            ! dRh/dcloud, old 
   real(r8) :: rhu00 (pcols,pver)            ! Rh threshold for cloud, old

#ifdef MICROP-MG
   type(physics_state)   :: state_eq         ! equilibrium state variables
#endif

   integer  :: nstep                         ! current timestep number
   integer      lat(pcols)                   ! current latitudes(indices)
   integer      lon(pcols)                   ! current longtitudes(indices)

#ifdef BCC_DRAG
   integer  :: idx_zmdt
   real(r8), pointer, dimension(:,:) :: zmdt
#endif

#ifdef UWMT
   real(r8), pointer, dimension(:,:) :: wsedl
#endif

   real(r8) :: calday                        ! current calendar day
   real(r8) :: clat(pcols)                   ! current latitudes(radians)
   real(r8) :: zdu(pcols,pver)               ! detraining mass flux from deep convection
   real(r8) :: ftem(pcols,pver)              ! Temporary workspace for outfld variables
   real(r8) :: zmrprd(pcols,pver)            ! rain production in ZM convection
   real(r8) :: cmfsl(pcols,pver)             ! Moist convection lw stat energy flux
   real(r8) :: cmflq(pcols,pver)             ! Moist convection total water flux

   real(r8) cmfcme(pcols,pver)                ! cmf condensation - evaporation
   real(r8) cmfdqr2(pcols,pver)               ! dq/dt due to moist convective rainout
   real(r8) cmfmc2(pcols,pverp)               ! Moist convection cloud mass flux
   real(r8) cmfsl2(pcols,pver)                ! Moist convection lw stat energy flux
   real(r8) cmflq2(pcols,pver)                ! Moist convection total water flux
   real(r8) cnt2(pcols)                       ! Top level of convective activity
   real(r8) cnb2(pcols)                       ! Bottom level of convective activity
   real(r8) pflx(pcols,pverp)                 ! Conv rain flux thru out btm of lev
   real(r8) dlf2(pcols,pver)                   ! dq/dt due to rainout terms
   real(r8) qpert2(pcols,ppcnst)              ! Perturbation q
   real(r8) rtdt                              ! 1./ztodt
   real(r8) tpert2(pcols)                     ! Perturbation T
   real(r8) cape(pcols)       

   integer lchnk                              ! chunk identifier
   integer ncol                               ! number of atmospheric columns


   integer  i,k,m                               ! Longitude, level, constituent indices
   integer :: lcl(pcols) 
!---------------------------------
#ifdef MICROP-MG
   integer ::  cldtop_idx
   integer ::  cldbot_idx
   integer ::  rprdtot_idx

   integer ::  rprdsh_idx
   integer ::  icwmrsh_idx
   integer ::  nevapr_shcu_idx
   integer ::  sh_flxprc_idx
   integer ::  sh_flxsnw_idx
   integer ::  sh_cldliq_idx
   integer ::  sh_cldice_idx

   integer ::  rprddp_idx
   integer ::  icwmrdp_idx
   integer ::  nevapr_dpcu_idx
   integer ::  dp_flxprc_idx
   integer ::  dp_flxsnw_idx
   integer ::  dp_cldliq_idx
   integer ::  dp_cldice_idx

   real(r8), pointer, dimension(:) ::   jctop    ! o row of top-of-deep-convection indices passed out.
   real(r8), pointer, dimension(:) ::   jcbot    ! o row of base of cloud indices passed out.

   real(r8), pointer, dimension(:,:) :: rprdsh
   real(r8), pointer, dimension(:,:) :: icwmrsh
   real(r8), pointer, dimension(:,:) :: evapcsh      ! Evaporation of deep convective precipitation
   real(r8), pointer, dimension(:,:) :: sh_flxprc      ! Convective-scale flux of precip at interfaces (kg/m2/s)
   real(r8), pointer, dimension(:,:) :: sh_flxsnw      ! Convective-scale flux of snow   at interfaces (kg/m2/s)
   real(r8), pointer, dimension(:,:) :: sh_cldliq
   real(r8), pointer, dimension(:,:) :: sh_cldice

   real(r8), pointer, dimension(:,:) :: rprddp         ! rain production rate
   real(r8), pointer, dimension(:,:) :: ql             ! wg grid slice of cloud liquid water.
   real(r8), pointer, dimension(:,:) :: evapcdp      ! Evaporation of deep convective precipitation
   real(r8), pointer, dimension(:,:) :: dp_flxprc      ! Convective-scale flux of precip at interfaces (kg/m2/s)
   real(r8), pointer, dimension(:,:) :: dp_flxsnw      ! Convective-scale flux of snow   at interfaces (kg/m2/s)
   real(r8), pointer, dimension(:,:) :: dp_cldliq
   real(r8), pointer, dimension(:,:) :: dp_cldice

   integer :: ixnumice, ixnumliq              ! constituent indices for cloud liquid and ice water.

   real(r8) :: prec_str(pcols)          ! [Total] Sfc flux of precip from stratiform [ m/s ]
   real(r8) :: snow_str(pcols)          ! [Total] Sfc flux of snow from stratiform   [ m/s ]
   real(r8) :: cmeliq(pcols,pver)    ! local variables

#endif

#ifdef UWMT
   integer ::  wsedl_idx
#endif
                                           
   real(r8) clc(pcols)                        ! Total convective cloud (cloud scheme)

   integer modx     ! for aerosol process selection, wzz, 2008.9.1
   real(r8) pretrow(pcols,pver,2)     ! precipitation(kg/m2 s), wzz, 2008.2.18
   real(r8) fkt(pcols,pverp), grcld_cs(pcols,pverp), grcld_ck(pcols,pverp), grcld_fn(pcols,pverp)
   real(r8) :: ccflx(pcols,ppcnst)  ! Surface constituent flux (kg/m^2/s)

! Used for OUTFLD only                     
!                                          
   real(r8) icwmr1(pcols,pver)                ! in cloud water mixing ration for zhang scheme
   real(r8) icwmr2(pcols,pver)                ! in cloud water mixing ration for hack scheme
!
!     Variables for doing deep convective transport outside of wu_convr
!
   real(r8) :: mb(pcols),eps0(pcols),zfg(pcols,pverp)
   integer  :: jd(pcols)
!
   real(r8) mu2(pcols,pver)
   real(r8) eu2(pcols,pver)
   real(r8) md2(pcols,pver)
   real(r8) ed2(pcols,pver)
   real(r8) dp(pcols,pver)
   real(r8) dpdry(pcols,pver)
   real(r8) dsubcld(pcols)


! stratiform precipitation variables
   real(r8) :: prec_pcw(pcols)                ! total precip from prognostic cloud scheme
   real(r8) :: snow_pcw(pcols)                ! snow from prognostic cloud scheme
   real(r8) :: prec_sed(pcols)                ! total precip from cloud sedimentation
   real(r8) :: snow_sed(pcols)                ! snow from cloud ice sedimentation


! convective precipitation variables
   real(r8) :: prec_zmc(pcols)                ! total precipitation from ZM convection
   real(r8) :: snow_zmc(pcols)                ! snow from ZM convection
   real(r8) :: prec_cmf(pcols)                ! total precipitation from Hack convection
   real(r8) :: snow_cmf(pcols)                ! snow from Hack convection


! energy checking variables
   real(r8) :: zero(pcols)                    ! array of zeros
   real(r8) :: rliq(pcols)                    ! vertical integral of liquid not yet in q(ixcldliq)
   real(r8) :: rliq2(pcols)                   ! vertical integral of liquid from shallow scheme
   real(r8) :: flx_cnd(pcols)


   integer jt(pcols)
   integer maxg(pcols)
   integer ideep(pcols)
   integer lengath
   real(r8) cldc(pcols,pver)
   real(r8) qme(pcols,pver)
   real(r8) wrscav(pcols,pver)
   real(r8) zfprec(pcols,pver)
   real(r8) wpevpa(pcols,pver)

   real(r8) cflx(pcols,ppcnst)

   real(r8) zmevapr(pcols,pver)
   real(r8) cmfevapr(pcols,pver)

!zf 2009.06.04 ----------------------------------------
   real(r8) cind(pcols)
   real(r8) sci(pcols)
   real(r8) dci(pcols)


   real(r8) totcond3(pcols,pver) 
   real(r8) clwvi(pcols)        
   real(r8) clivi(pcols)       
   real(r8) ccb(pcols)       
   real(r8) cct(pcols)       

!--------wtw 200506 -----------------

   real(r8) :: q_box(pcols,pver)
   real(r8) :: t_box(pcols,pver) 
   real(r8) :: cld_box(pcols,pver) 
   real(r8) :: q_env(pcols,pver)
   real(r8) :: t_env(pcols,pver)   
   real(r8) :: cld_env(pcols,pver) 

#ifdef WU_CONV
   integer  ni
   integer  ntracer
   real(r8), allocatable :: tracer(:,:,:)

   real(r8) ptendq(pcols,pver)
   real(r8) ptendt(pcols,pver)
   real(r8) ptendps(pcols)      

!#endif
!#ifdef WU_REFATM
   real(r8) snowls(pcols)         ! large scale precipitation (m/s)    zf????????
   real(r8) rainls(pcols)         ! large scale snowfall (m/s)
#endif

   integer :: yr,mon,day                ! components of date
   integer :: ncdate                    ! formated date [yyyymmdd]
   integer :: ncsec  

!-------------------------------------------------------------
!
   zero = 0.
!
   lchnk = state%lchnk
   ncol  = state%ncol

   rtdt = 1./ztodt


   nstep = get_nstep()
   calday = get_curr_calday()

#ifdef MICROP-MG
   cldtop_idx      = pbuf_get_fld_idx('CLDTOP')
   cldbot_idx      = pbuf_get_fld_idx('CLDBOT')
   rprdtot_idx     = pbuf_get_fld_idx('RPRDTOT')
   
   
   rprdsh_idx      = pbuf_get_fld_idx('RPRDSH')
   icwmrsh_idx     = pbuf_get_fld_idx('ICWMRSH')
   nevapr_shcu_idx = pbuf_get_fld_idx('NEVAPR_SHCU')
   sh_flxprc_idx   = pbuf_get_fld_idx('SH_FLXPRC')
   sh_flxsnw_idx   = pbuf_get_fld_idx('SH_FLXSNW')
   sh_cldliq_idx   = pbuf_get_fld_idx('SH_CLDLIQ')
   sh_cldice_idx   = pbuf_get_fld_idx('SH_CLDICE')

   
   rprddp_idx      = pbuf_get_fld_idx('RPRDDP')
   icwmrdp_idx     = pbuf_get_fld_idx('ICWMRDP')
   nevapr_dpcu_idx = pbuf_get_fld_idx('NEVAPR_DPCU')
   dp_flxprc_idx   = pbuf_get_fld_idx('DP_FLXPRC')
   dp_flxsnw_idx   = pbuf_get_fld_idx('DP_FLXSNW')
   dp_cldliq_idx   = pbuf_get_fld_idx('DP_CLDLIQ')
   dp_cldice_idx   = pbuf_get_fld_idx('DP_CLDICE')
#endif

#ifdef UWMT
   wsedl_idx = pbuf_get_fld_idx('WSEDL')
   wsedl    => pbuf(wsedl_idx)%fld_ptr(1,:,:,lchnk,1)
#endif

!------  wtw 2005.05 ----------

   snowls(:) = 0.0
   rainls(:) = 0.0
!
!===================================================
! Moist convection
!===================================================
!
! Since the PBL doesn't pass constituent perturbations, they
! are zeroed here for input to the moist convection routine
!
   qpert(:ncol,2:ppcnst) = 0.0

!---------
!
! Begin with WU (2011) convection parameterization
!
         call t_startf ('wu_convr')

         call get_curr_date(yr, mon, day, ncsec)
         ncdate = (yr*10000 + mon*100 + day)*86400 + ncsec

!        ptendq(:ncol,:pver)=(state%q(:ncol,:pver,1)-state%q1(:ncol,:pver))/ztodt  ! 200805
!        ptendt(:ncol,:pver)=(state%t(:ncol,:pver)  -state%t1(:ncol,:pver))/ztodt  ! 200805
         if( nstep == 0 ) then
             ptendq(:ncol,:pver) = 0.0
             ptendt(:ncol,:pver) = 0.0
         else

#ifdef NSTEP_MOIST
             ptendq(:ncol,:pver)=(state%q(:ncol,:pver,1) - q0_conv(:ncol,:pver))/(ncdate - date0_conv) &
                           - (state%q1(:ncol,:pver)  - q0_conv(:ncol,:pver))/ztodt
             ptendt(:ncol,:pver)=(state%t(:ncol,:pver)   - t0_conv(:ncol,:pver))/(ncdate - date0_conv) &
                           - (state%t1(:ncol,:pver)  - t0_conv(:ncol,:pver))/ztodt
#else
             ptendq(:ncol,:pver)=(state%q(:ncol,:pver,1)-state%q1(:ncol,:pver))/ztodt
             ptendt(:ncol,:pver)=(state%t(:ncol,:pver)  -state%t1(:ncol,:pver))/ztodt
#endif

         endif

         t0_conv(:ncol,:pver) = state%t(:ncol,:pver)
         q0_conv(:ncol,:pver) = state%q(:ncol,:pver,1)
         date0_conv =  ncdate        

         ptendps(:ncol) = dyn_tend%pstend(:ncol)

!wtw   call outfld('ZMDT0   ',ptendt         ,pcols   ,lchnk   )
!wtw   call outfld('ZMDQ0   ',ptendq         ,pcols   ,lchnk   )

!--------------------------------------------
#ifdef MOZART2
   allocate (tracer(pcols,pver,ppcnst-1-3)) 
   ni = 0
#ifdef MG08
   do m=2, 5
#else
   do m=2, 3
#endif
      ni = ni + 1
      tracer(:ncol,:pver,ni) = state%q(:ncol,:pver,m)
   enddo
   ntracer = ni

!---------------------------
#elif (defined CO2)
   allocate (tracer(pcols,pver,ppcnst-1))
   ni = 0
   do m=2, 3
      ni = ni + 1
      tracer(:ncol,:pver,ni) = state%q(:ncol,:pver,m)
   enddo
   ntracer = ni

!--------------------------------
#else
   allocate (tracer(pcols,pver,ppcnst-1))
   ni = 0
   do m=2, ppcnst    
      ni = ni + 1
      tracer(:ncol,:pver,ni) = state%q(:ncol,:pver,m)
   enddo
   ntracer = ni
#endif

   do i=1, ncol
      do k=1, pver
         qbarb(i,k) = state%q(i,k,1)
         tbarb(i,k) = state%t(i,k)
      enddo
   enddo

   cnt(:ncol) = pver
   cnb(:ncol) = pver
   dlf(:ncol,:pver) = 0.0
!-----------------------------------------------------------------
   call wu_convr( lchnk,    ncol, &
                  state%t,  state%q,    prec_zmc,   cnt,     cnb,      &
                  pblht,    kbfs     ,state%zm, state%phis,    state%zi,   ptend%q(:,:,1), &
                  ptend%s,  state%pmid,   state%pint,  state%pdel,   &
                  .5*ztodt,cmfmc,    cmfcme, cape,                  &
                  tpert,    dlf,      pflx,    zdu,     zmrprd,  &
                  mu2,      md2,     du2,     eu2,     ed2,      &
                  mb,       mc,      zfg,     eps0,    jd,       &  !  wzz
                  dp,       dsubcld, jt,      maxg,    ideep,    &
                  lengath, icwmr1,   rliq, ptendq, ptendt   ,state%omega, &
                  ptendps, landfrac, state%u,   state%v ,&
                  ptend%u, ptend%v , tracer, ntracer, lcl )

#ifdef MICROP-MG
    jctop      => pbuf(cldtop_idx)%fld_ptr(1,1:pcols,1,state%lchnk,1)
    jcbot      => pbuf(cldbot_idx)%fld_ptr(1,1:pcols,1,state%lchnk,1)
    ql         => pbuf(icwmrdp_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
    rprddp     => pbuf(rprddp_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)


    jctop(:ncol)        = cnt(:ncol)
    jcbot(:ncol)        = cnb(:ncol)
    ql(:ncol,:pver)     = icwmr1(:ncol,:pver)
    rprddp(:ncol,:pver) = zmrprd(:ncol,:pver)
#endif
!--------
! update all tracer except water vapor caused by deep convection
!
#ifdef MOZART2
   ni = 0
!  do m=2, ppcnst-3  
#ifdef MG08
   do m=2, 5
#else
   do m=2, 3
#endif
      ni = ni + 1
      state%q(:ncol,:pver,m) = tracer(:ncol,:pver,ni)
      do i=1, ncol
         do k=1, pver
            if( state%q(i,k,m) < 1.e-12) state%q(i,k,m) = 1.e-12 
         enddo 
      enddo
   enddo
!---------------------
#elif (defined CO2)
   ni = 0
   do m=2, 3
      ni = ni + 1
      state%q(:ncol,:pver,m) = tracer(:ncol,:pver,ni)
      do i=1, ncol
         do k=1, pver
            if( state%q(i,k,m) < 1.e-12) state%q(i,k,m) = 1.e-12
         enddo
      enddo
   enddo

!----------------
#else
   ni = 0
   do m=2, ppcnst 
      ni = ni + 1
      state%q(:ncol,:pver,m) = tracer(:ncol,:pver,ni)
      do i=1, ncol
         do k=1, pver
            if( state%q(i,k,m) < 1.e-12) state%q(i,k,m) = 1.e-12 
         enddo
      enddo
   enddo
#endif
!---
   ptend%name  = 'wu_convr'
   ptend%ls    = .TRUE.
   ptend%lq(1) = .TRUE.
   ptend%lu    = .TRUE.
   ptend%lv    = .TRUE.


   cmfsl (:ncol,:) = 0. ! This is not returned from zm, hence it is zeroed.
   cmflq (:ncol,:) = 0. ! This is not returned from zm, hence it is zeroed.

#ifdef BCC_DRAG
   idx_zmdt = pbuf_get_fld_idx('ZMDT')
   zmdt  => pbuf(idx_zmdt) %fld_ptr(1,1:pcols,1:pver,lchnk,1)
#endif

   ftem(:ncol,:pver) = ptend%s(:ncol,:pver)/cpair

#ifdef BCC_DRAG
   zmdt(:ncol,:pver) = ftem(:ncol,:pver)
   do i=1, ncol
      do k=1, pver
         if( zmdt(i,k) < 0.0_r8) zmdt(i,k) =0.0_r8
      enddo
   enddo
#endif

!---
   ftem(:ncol,:pver) = ptend%s(:ncol,:pver)/cpair
   call outfld('ZMDT    ',ftem           ,pcols   ,lchnk   )
   call outfld('ZMDQ    ',ptend%q(1,1,1) ,pcols   ,lchnk   )
   call t_stopf('wu_convr')

   call physics_update(state, tend, ptend, ztodt)

!
! Deep convection cloud fraction
!
   call aqsat(tbarb   ,state%pmid,   esbar      ,qsbar      ,pcols   , &
              ncol    ,pver    ,1       ,pver    )

   do i=1, ncol
      do k=1, pver
         deepcld(i,k)  = 0.0
         qdeepcld(i,k) = qbarb(i,k)
         tdeepcld(i,k) = tbarb(i,k)
         if ( k.ge.nint(cnt(i)).and.k.le.nint(cnb(i)) .and. nint(cnt(i)).ne.nint(cnb(i)) .and. &
              dlf(i,k) > 0. ) then
            dqsdtp(i,k) = qsbar(i,k)* (1.+qsbar(i,k)/epsilo)*epsilo*latvap/(rgas*tbarb(i,k)**2)
            if( qsbar(i,k)-qbarb(i,k) > 0.0 ) then
               deepcld(i,k)  = ( (state%q(i,k,1)-qbarb(i,k)) - dqsdtp(i,k)*(state%t(i,k)-tbarb(i,k)) )/&
                                  max((qsbar(i,k)-qbarb(i,k)),1.e-10)
            else
               deepcld(i,k)  =  0.0
            endif
            deepcld(i,k)  = max(min(deepcld(i,k),1.0), 0.)
            if( deepcld(i,k) > 0.0) then 
                tdeepcld(i,k) = ( state%t(i,k)-tbarb(i,k)) /max(deepcld(i,k),0.005) + tbarb(i,k)
                if( tdeepcld(i,k) - tbarb(i,k) > 10.0 ) then
                    tdeepcld(i,k) = tbarb(i,k) + 10.0
                    deepcld(i,k)  = max(min( (state%t(i,k)-tbarb(i,k))/10.0, 1.0), 0.)
                endif
                if( tdeepcld(i,k) - tbarb(i,k) < -8.0) then
                    tdeepcld(i,k) = tbarb(i,k) - 8.0
                    deepcld(i,k)  = max(min( (state%t(i,k)-tbarb(i,k))/(-8.0), 1.0), 0.)
                endif 
                call aqsat(tdeepcld(i,k), state%pmid(i,k),   esbar(i,k)  ,qdeepcld(i,k) , 1  , &
                           1    ,1    ,1       ,1   )
            else
                qdeepcld(i,k) = qbarb(i,k)
                tdeepcld(i,k) = tbarb(i,k)
            endif
          endif
      enddo
   enddo

   call aqsat(tdeepcld   ,state%pmid,   esbar      ,qdeepcld      ,pcols   , &
              ncol    ,pver    ,1       ,pver    )
!------------------------
#ifdef WU_CONV
   state%t1(:ncol,:)=state%t(:ncol,:)
   state%q1(:ncol,:)=state%q(:ncol,:,1)
!  dyn_tend%pstend(:ncol) = state%pint(:ncol,pverp)
#endif
!-------------------
! Determine the phase of the precipitation produced and add latent heat of fusion
! Evaporate some of the precip directly into the environment (Sundqvist)
!
#ifdef MICROP-MG
    dp_flxprc     => pbuf(dp_flxprc_idx)%fld_ptr(1,1:pcols,1:pverp,lchnk,1)
    dp_flxsnw     => pbuf(dp_flxsnw_idx)%fld_ptr(1,1:pcols,1:pverp,lchnk,1)
    dp_cldliq     => pbuf(dp_cldliq_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
    dp_cldice     => pbuf(dp_cldice_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
    evapcdp       => pbuf(nevapr_dpcu_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
    dp_cldliq(:ncol,:)   = 0._r8
    dp_cldice(:ncol,:)   = 0._r8
    call microp_wu_conv_evap(state, ptend, zmrprd, cld, ztodt, prec_zmc, snow_zmc, .false., &
                                 dp_flxprc, dp_flxsnw, zmevapr)  ! output : zmevapr
    evapcdp(:ncol,:pver) = ptend%q(:ncol,:pver,1)      !  Evaporation of deep convective precipitation
#else
   call wu_conv_evap(state, ptend, zmrprd, cld, ztodt, prec_zmc, snow_zmc, .false., zmevapr)
#endif
   call physics_update(state, tend, ptend, ztodt)


! Check energy integrals, including "reserved liquid"


   flx_cnd(:ncol) = prec_zmc(:ncol) + rliq(:ncol)
   call check_energy_chng(state, tend, "zm_evap", nstep, ztodt, zero, flx_cnd, snow_zmc, zero)


! Transport cloud water and ice only
!
   call cnst_get_ind('CLDLIQ', ixcldliq)
   call cnst_get_ind('CLDICE', ixcldice)

!--
! for ncnst =0 in wu_conv
!
!   ptend%name = 'convtran1'      
!#ifdef MOZART2
!   ptend%lq(2:ppcnst-3) = .true.
!#else
!   ptend%lq(2:ppcnst) = .true.
!#endif
!
!   call convtran (lchnk,                                        &
!                  ptend%lq,state%q, ppcnst,  mu2,     md2,   &
!                  du2,     eu2,     ed2,     dp,      dsubcld,  &
!                  jt,      maxg,    ideep,   1,       lengath,  &
!                  nstep,   fracis,  ptend%q, dpdry  )
!   call physics_update (state, tend, ptend, ztodt)
!
!-------------------------------------
! Convert mass flux from reported mb/s to kg/m^2/s
!
   cmfmc(:ncol,:pver) = cmfmc(:ncol,:pver) * 100./gravit

!------------
!#ifdef WU_CONV
!   state%t1(:ncol,:)=state%t(:ncol,:)
!   state%q1(:ncol,:)=state%q(:ncol,:,1)
!  dyn_tend%pstend(:ncol) = state%pint(:ncol,pverp)
!#endif
!------------------------------------------------------------------------------------------------
!
! Call Hack (1994) convection scheme to deal with shallow/mid-level convection
!
   do i=1, ncol
      do k=1, pver
         qbarb(i,k) = state%q(i,k,1)
         tbarb(i,k) = state%t(i,k)
      enddo
   enddo


   call t_startf('cmfmca')
   tpert2(:ncol  ) =0.
   qpert2(:ncol,:) = qpert(:ncol,:)  ! BAB Why is this not zero, if tpert2=0???
   call cmfmca (lchnk,   ncol, &
                nstep,   ztodt,   state%pmid,  state%pdel,   &
                state%rpdel,   state%zm,      tpert2,  qpert2,  state%phis,     &
                pblht,   state%t,   state%q,   ptend%s,   ptend%q,      &
                cmfmc2,  cmfdqr2, cmfsl2,  cmflq2,  prec_cmf,   &
                dlf2,     cnt2,    cnb2,    icwmr2   , rliq2, & 
                state%pmiddry, state%pdeldry, state%rpdeldry, &
                totcond3 )         !2009.06.04
#ifdef MICROP-MG
   rprdsh     => pbuf(rprdsh_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
   icwmrsh    => pbuf(icwmrsh_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
   icwmrsh(:ncol,:pver)= icwmr2 (:ncol,:pver)
   rprdsh (:ncol,:pver)= cmfdqr2(:ncol,:pver)


   pbuf(rprdtot_idx)%fld_ptr(1,1:ncol,1:pver,lchnk,1) = rprdsh(:ncol,:pver) + rprddp(:ncol,:pver)
#endif
   ptend%name  = 'cmfmca'
   ptend%ls    = .TRUE.
   ptend%lq(:) = .TRUE.


! Add shallow cloud water detrainment to cloud water detrained from ZM
   dlf(:ncol,:pver) = dlf(:ncol,:pver) + dlf2(:ncol,:pver)
   rliq(:ncol) = rliq(:ncol) + rliq2(:ncol)
   
   ftem(:ncol,:pver) = ptend%s(:ncol,:pver)/cpair
   call outfld('CMFDT   ',ftem          ,pcols   ,lchnk   )
   call outfld('CMFDQ   ',ptend%q(1,1,1),pcols   ,lchnk   )
   call t_stopf('cmfmca')
   call physics_update (state, tend, ptend, ztodt)

!----
! Shallow convection cloud fraction
!
   call aqsat(tbarb   ,state%pmid,   esbar      ,qsbar      ,pcols   , &
               ncol    ,pver    ,1       ,pver    )

   do i=1, ncol
      do k=1, pver
         qshallowcld(i,k) = qbarb(i,k)
         shallowcld(i,k)  = 0.0
         tshallowcld(i,k) = tbarb(i,k)
         if ( k.ge.cnt2(i).and.k.le.cnb2(i) .and. cnt2(i).ne.cnb2(i).and. dlf2(i,k) > 0. ) then
            dqsdtp(i,k) = qsbar(i,k)* (1.+qsbar(i,k)/epsilo)*epsilo*latvap/(rgas*tbarb(i,k)**2)
            if( qsbar(i,k)-qbarb(i,k) > 0.0 ) then
               shallowcld(i,k)  = ( (state%q(i,k,1)-qbarb(i,k)) - dqsdtp(i,k)*(state%t(i,k)-tbarb(i,k)) )/&
                                  max((qsbar(i,k)-qbarb(i,k)),1.e-10)
            else
               shallowcld(i,k)  =  0.0
            endif
            shallowcld(i,k)  = max(min(shallowcld(i,k),1.0), 0.)

            if( shallowcld(i,k) > 0.0) then
                tshallowcld(i,k) = ( state%t(i,k)-tbarb(i,k)) /max(shallowcld(i,k),0.005) + tbarb(i,k)
                if( tshallowcld(i,k) - tbarb(i,k) > 10.0 ) then
                    tshallowcld(i,k) = tbarb(i,k) + 10.0
                    shallowcld(i,k)  = max(min( (state%t(i,k)-tbarb(i,k))/10.0, 1.0), 0.)
                endif
                if( tshallowcld(i,k) - tbarb(i,k) < -8.0) then
                    tshallowcld(i,k) = tbarb(i,k) - 8.0
                    shallowcld(i,k)  = max(min( (state%t(i,k)-tbarb(i,k))/(-8.0), 1.0), 0.)
                endif
                call aqsat(tshallowcld(i,k), state%pmid(i,k),   esbar(i,k)  ,qshallowcld(i,k) , 1  , &
                           1    ,1    ,1       ,1   )
            else
                qshallowcld(i,k) = qbarb(i,k)
                tshallowcld(i,k) = tbarb(i,k)
            endif
         endif
      enddo
   enddo

   call aqsat(tshallowcld   ,state%pmid,   esbar      ,qshallowcld      ,pcols   , &
              ncol    ,pver    ,1       ,pver    )
!----------
! Determine the phase of the precipitation produced and add latent heat of fusion


#ifdef MICROP-MG
    sh_flxprc     => pbuf(sh_flxprc_idx)%fld_ptr(1,1:pcols,1:pverp,lchnk,1)
    sh_flxsnw     => pbuf(sh_flxsnw_idx)%fld_ptr(1,1:pcols,1:pverp,lchnk,1)
    sh_cldliq     => pbuf(sh_cldliq_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
    sh_cldice     => pbuf(sh_cldice_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
    evapcsh       => pbuf(nevapr_shcu_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
    !! clouds have no water... :)
    sh_cldliq(:ncol,:) = 0._r8
    sh_cldice(:ncol,:) = 0._r8
    call microp_wu_conv_evap(state, ptend, cmfdqr2, cld, ztodt, prec_cmf, snow_cmf, .true., &
                          sh_flxprc, sh_flxsnw, cmfevapr)
    evapcsh(:ncol,:pver) = ptend%q(:ncol,:pver,1)
#else
   call wu_conv_evap(state, ptend, cmfdqr2, cld, ztodt, prec_cmf, snow_cmf, .true., cmfevapr)
#endif
   call physics_update(state, tend, ptend, ztodt)

   flx_cnd(:ncol) = prec_cmf(:ncol) + rliq2(:ncol)
   call check_energy_chng(state, tend, "hk_evap", nstep, ztodt, zero, flx_cnd, snow_cmf, zero)

!---------------------------------------------------------------------------------------------
!
! Merge shallow/mid-level output with prior results from Zhang-McFarlane
!
!zf 2009.07.14
   do i=1,ncol
     dci(i) = 0.
     if(cnt(i) < cnb(i)) dci(i) = 1.
   end do
!zf 2009.07.14


   do i=1,ncol
      if (cnt2(i) < cnt(i)) cnt(i) = cnt2(i)
      if (cnb2(i) > cnb(i)) cnb(i) = cnb2(i)
   end do


!zf 2009.06.04 ----------------------------------------------
   do i = 1, pcols
     clwvi(i) = 0.
     clivi(i) = 0.
     sci(i) = 0.
     cind(i) = 0.
     do k = 1, pver
       clwvi(i) = clwvi(i) + state%q(i,k,ixcldliq)*(state%pint(i,k+1)-state%pint(i,k))
       clivi(i) = clivi(i) + state%q(i,k,ixcldice)*(state%pint(i,k+1)-state%pint(i,k))
     end do 
   end do 


   do i=1,ncol
     ccb(i) = 0. 
     cct(i) = 0.
     if(cnt(i) < cnb(i)) then
       ccb(i) = state%pmid(i,cnb(i))
       cct(i) = state%pmid(i,cnt(i))
     end if
     if(cnt2(i) < cnb2(i)) sci(i) = 1.
     if(sci(i) .eq. 1 .or. dci(i) .eq. 1) cind(i) = 1.
   end do
   call outfld('CLWVI   ',clwvi         ,pcols   ,lchnk   )
   call outfld('CLIVI   ',clivi         ,pcols   ,lchnk   )
   call outfld('CCB     ',ccb           ,pcols   ,lchnk   )
   call outfld('CCT     ',cct           ,pcols   ,lchnk   )
   call outfld('CIND    ',cind          ,pcols   ,lchnk   )
   call outfld('SCI     ',sci           ,pcols   ,lchnk   )
   call outfld('DCI     ',dci           ,pcols   ,lchnk   )
   call outfld('CMFMC1  ',cmfmc         ,pcols   ,lchnk   )
   call outfld('CMFMC2  ',cmfmc2        ,pcols   ,lchnk   )
   call outfld('MU2     ',mu2           ,pcols   ,lchnk   )
   call outfld('MD2     ',md2           ,pcols   ,lchnk   )
!zf-----------------------------------------------------------


!
   cmfmc (:ncol,:pver) = cmfmc (:ncol,:pver) + cmfmc2 (:ncol,:pver)
   cmfsl (:ncol,:pver) = cmfsl (:ncol,:pver) + cmfsl2 (:ncol,:pver)
   cmflq (:ncol,:pver) = cmflq (:ncol,:pver) + cmflq2 (:ncol,:pver)
   call outfld('CMFMC' , cmfmc  , pcols, lchnk)
!  output new partition of cloud condensate variables, as well as precipitation 
   call outfld('QC      ',dlf2           ,pcols   ,lchnk   )
   call outfld('PRECSH  ',prec_cmf      ,pcols   ,lchnk       )
   call outfld('CMFDQR', cmfdqr2, pcols, lchnk)
   call outfld('CMFSL' , cmfsl  , pcols, lchnk)
!wtw   call outfld('CMFLQ' , cmflq  , pcols, lchnk)
   call outfld('DQP'   , dlf2    , pcols, lchnk)


#ifdef MICROP-MG
   !
   ! Calculate macrophysical tendencey (sedimentation, detrain, cloud fraction)
   !
      call t_startf('macrop_tend')


      call macrop_driver_tend(state, ptend, ztodt, &
                              landfrac,  ocnfrac, snowh, &
                              dlf, dlf2, lcl   ,  & ! detrain
                              cmfmc,   cmfmc2,                 &
                              ts,   sst,  zdu,  &
                              pbuf,  state_eq,  cmeliq  )            ! cmeliq: output


!      call physics_update (state, tend, ptend, ztodt)
!      call check_energy_chng(state, tend, "macrop_tend", nstep, ztodt, zero, prec_str, snow_str, zero)


      call t_stopf('macrop_tend')


   !
   ! Calculate cloud microphysics
   !


      call t_startf('microp_tend')


      call microp_driver_tend(state, ptend, ztodt, &
#ifdef MODAL_AERO
           cflx, & ! constituent sources
#endif
           rliq  , & ! check energy after detrain
           prec_str, snow_str, prec_sed, snow_sed, prec_pcw, snow_pcw, &
           pbuf, state_eq, cmeliq)


      call physics_update (state, tend, ptend, ztodt)
      call check_energy_chng(state, tend, "microp_tend", nstep, ztodt, zero, prec_str, snow_str, zero)
!
      call t_stopf('microp_tend')


!-------------------------------------------------------------
#else


!++sxj!!!!!!!!!!!!!!!!!!!!!!!!!!!!!cloud microphysics!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#ifdef MG08

       flx_cnd(:ncol) = -rliq(:ncol)
       call check_energy_chng(state, tend, "cldwat_detrain", nstep, ztodt, zero, flx_cnd, zero, zero)

       call stratiform_tend(state, ptend, ztodt, &
                icefrac, landfrac, ocnfrac, &
		snowh,  & ! sediment
                dlf,    & ! detrain 
                cmfmc, cmfmc2, &
		tdeepcld, qdeepcld, deepcld, tshallowcld, qshallowcld, shallowcld,  &
                cld_env  ,t_env   ,q_env     , &
                cnt, cnb, lcl,  cstcld, &  ! sxj
		cld, concld, cldst, prain, nevapr, qme, &  ! sxj  
		ts,    sst ,    zdu,  &  ! sxj
                prec_sed, snow_sed, prec_pcw, snow_pcw, &
                tcwat, qcwat, lcwat )    ! sxj

		
        call physics_update (state, tend, ptend, ztodt)

        call cldnrh(lchnk,   ncol, tdeepcld, qdeepcld, deepcld, tshallowcld,  &
               qshallowcld, shallowcld,                    &
               state%pmid,  state%pint, ztodt, dlf,  state%t,   state%q(1,1,1),   state%q(1,1,ixcldice), &
               state%q(1,1,ixcldliq),    state%omega, &
               cnt,     cnb,     lcl,    cld,    clc,     state%pdel,   &
               cmfmc,   cmfmc2, landfrac,snowh,   concld,  cldst,    &
               cld_env  ,t_env   ,q_env     , &
               ts,      sst, state%pint(1,pverp),       zdu,  ocnfrac, &
               rhdfda,   rhu00 , state%phis, state%zm,  cstcld)

        call outfld('CONCLD  ',concld, pcols,lchnk)
        call outfld('CLDST   ',cldst,  pcols,lchnk)
        call outfld('CNVCLD  ',clc,    pcols,lchnk)

       
       call check_energy_chng(state, tend, "cldwat2m_tend", nstep, ztodt, zero, prec_pcw, snow_pcw, zero)   


       !if (masterproc)  write(6,*) "SXJ tphysbc_moist.F90 stratiform_tend done"
       !write(6,*) "SXJ tphysbc_moist.F90 stratiform_tend done",iam,lchnk,ncol

#else  

!----------------RK scheme begin ----------------
! Allow the cloud liquid drops and ice particles to sediment
! Occurs before adding convectively detrained cloud water, because the phase of the
! of the detrained water is unknown.
   call t_startf('cldwat_sediment')
   call cldcond_sediment(state, ptend, ztodt,cld, icefrac, landfrac, ocnfrac, prec_sed, &
#ifdef UWMT
                         snow_sed, landm, snowh, pr_cs, wsedl)
#else
                         snow_sed, landm, snowh, pr_cs)
#endif
   call physics_update(state, tend, ptend, ztodt)
   call t_stopf('cldwat_sediment')
   

! check energy integrals
   call check_energy_chng(state, tend, "cldwat_sediment", nstep, ztodt, zero, prec_sed, snow_sed, zero)


! Put the detraining cloud water from convection into the cloud and environment. 


   call cldcond_zmconv_detrain(dlf, cld, state, ptend)
   call physics_update(state, tend, ptend, ztodt)


! check energy integrals, reserved liquid has now been used
   flx_cnd(:ncol) = -rliq(:ncol)
   call check_energy_chng(state, tend, "cldwat_detrain", nstep, ztodt, zero, flx_cnd, zero, zero)
!
! cloud fraction after transport and convection,
! derive the relationship between rh and cld from 
! the employed cloud scheme
!

        call cldnrh(lchnk,   ncol, tdeepcld, qdeepcld, deepcld, tshallowcld,  &
               qshallowcld, shallowcld,                    &
               state%pmid,  state%pint, ztodt, dlf,  state%t,   state%q(1,1,1),   state%q(1,1,ixcldice), &
               state%q(1,1,ixcldliq),    state%omega, &
               cnt,     cnb,     lcl,    cld,    clc,     state%pdel,   &
               cmfmc,   cmfmc2, landfrac,snowh,   concld,  cldst,    &
               cld_env  ,t_env   ,q_env     , &
               ts,      sst, state%pint(1,pverp),       zdu,  ocnfrac, &
               rhdfda,   rhu00 , state%phis, state%zm,  cstcld)

   call outfld('CONCLD  ',concld, pcols,lchnk)
   call outfld('CLDST   ',cldst,  pcols,lchnk)
   call outfld('CNVCLD  ',clc,    pcols,lchnk)

! cloud water and ice parameterizations

!   q_box(:,:)  = state%q(:,:,1)
!   t_box(:,:)  = state%t(:,:)
!   cld_box(:,:)= cld(:,:) 
!   state%q(:,:,1) = q_env(:,:)
!   state%t(:,:)   = t_env(:,:)
!   cld(:,:)       = cld_env(:,:) 
  
   call t_startf('cldwat_tend')
   call cldcond_tend(state, ptend, ztodt, &
                     tcwat, qcwat, lcwat, prec_pcw, snow_pcw, icefrac, & 
                     rhdfda, rhu00, cld, nevapr, prain, qme, snowh)

!   state%q(:,:,1) = q_box(:,:)
!   state%t(:,:)   = t_box(:,:)   
!   cld(:,:)       = cld_box(:,:)

   call physics_update (state, tend, ptend, ztodt)

   call t_stopf('cldwat_tend')


! check energy integrals
   call check_energy_chng(state, tend, "cldwat_tend", nstep, ztodt, zero, prec_pcw, snow_pcw, zero)

!----------------RK scheme end ----------------

#endif  
!--sxj

! Save off q and t after cloud water
   do k=1,pver
      qcwat(:ncol,k) = state%q(:ncol,k,1)
      tcwat(:ncol,k) = state%t(:ncol,k)
      lcwat(:ncol,k) = state%q(:ncol,k,ixcldice) + state%q(:ncol,k,ixcldliq)
   end do
!
   call get_lat_all_p(lchnk, ncol, lat)
   call get_lon_all_p(lchnk, ncol, lon)
   call get_rlat_all_p(lchnk, ncol, clat)
   conicw(:ncol,:) = icwmr1(:ncol,:) + icwmr2(:ncol,:)
   cmfdqr(:ncol,:) = zmrprd(:ncol,:) + cmfdqr2(:ncol,:)

!----------------------------------------------------------------------------
#endif


!end microphysics conditional.
!===============================================================================================
!
!  aerosol wet chemistry determines scavenging fractions, and transformations
   call t_startf ('aerosol_wet')

   select case ( iaero )
   case ( 0 )

#ifdef MICROP-MG
      call microp_aerosol_wet_intr (state, ptend, ztodt, pbuf, dlf, cflx, fracis )
#else
   call aerosol_wet_intr (state, ptend, cflx, nstep, ztodt, lat, clat, lon,&
        qme, prain, &
       nevapr, cldc, cld, fracis, calday, cmfdqr, conicw)
#endif
   case ( 1 )
      modx = 1
! wzz for Aerosol model, 2008.9.1
   pr_cn(:ncol,1) = (prain(:ncol,1)-nevapr(:ncol,1))*state%pdel(:ncol,1)/gravit
   do k=2,pver
   pr_cn(:ncol,k) = (prain(:ncol,k)-nevapr(:ncol,k))*state%pdel(:ncol,k)/gravit
   enddo
   pr_cs(:ncol,:) = pr_cs(:ncol,:) + pr_cn(:ncol,:)
!
   pr_cn(:ncol,1) = (cmfdqr(:ncol,1)-zmevapr(:ncol,1)-cmfevapr(:ncol,1))*state%pdel(:ncol,1)/gravit
   do k = 2, pver
   pr_cn(:ncol,k) =     &
                   (cmfdqr(:ncol,k)-zmevapr(:ncol,k)-cmfevapr(:ncol,k))*state%pdel(:ncol,k)/gravit
   enddo
!
    grcld_ck(:, 1)       = concld(:,1)
    grcld_ck(:, 2:pverp) = concld(:,:)
    grcld_cs(:, 1)       = cstcld(:,1)
    grcld_cs(:, 2:pverp) = cstcld(:,:)
    grcld_fn(:, 1)       = cld(:,1)
    grcld_fn(:, 2:pverp) = cld(:,:)
    pretrow(:,:,1)       = pr_cs(:,:)
    pretrow(:,:,2)       = pr_cn(:,:)
    do k = 1, pverp    ! wzz, 2008.9.28
       zfg(:,k) = zfg(:,k) - zfg(:,pverp)
    enddo
!   wrscav(:,:) = prain(:,:)-nevapr(:,:)+cmfdqrt(:,:)-zmevapr(:,:)-cmfevapr(:,:)     ! wzz, 2008.10.23
    wrscav(:,:) = prain(:,:)+cmfdqr(:,:)     ! wzz, 2009.05.14
    zfprec(:,:) = prain(:,:)-nevapr(:,:)+cmfdqr(:,:)-zmevapr(:,:)-cmfevapr(:,:)
    wpevpa(:,:) = nevapr(:,:)+zmevapr(:,:)+cmfevapr(:,:)
!  if ( lchnk.eq.185) then
!   write(0,*)'wzz: concld in b:', concld(13,20),cstcld(13,20), mu2(2,pver)
!  endif


      call intrfc(ztodt,nstep,state, ptend, ts, fsds, snowh, &
           landfrac, icefrac, ocnfrac, grcld_ck, grcld_cs, grcld_fn,  &
           zfprec, wpevpa, wrscav, mu2,eu2,du2,md2,mc,ed2,             &
!          prain, nevapr, wrscav, mu2,eu2,du2,md2,mc,ed2,             &
           dp,mb,eps0,zfg,dsubcld,ideep,jt,maxg,jd,lengath,  &
           fkt, fhs, pretrow, ccflx,fracis)
!
   case default
      call endrun('aerosol_wetdep:  unknown scheme')
   end select
!  if ( lchnk.eq.249) then
!   write(0,*)'wzz: aero 2 in b:', state%q(2,pver,15)
!  endif
   call physics_update (state, tend, ptend, ztodt)


   call t_stopf ('aerosol_wet')
!
   dpdry = 0
   do i = 1,lengath
      dpdry(i,:) = state%pdeldry(ideep(i),:)*0.01
   end do


! check tracer integrals
   call check_tracers_chng(state, tracerint, "cmfmca", nstep, ztodt, cflx)

#if (defined MOZART2)
!
!  aerosol wet chemistry determines scavenging fractions, and transformations

   call t_startf ('aerosol_wet')

   !
   ! inout: state, tend, pbuf
   !
       call tphysbc_moist_chem (   ztodt,     ts,         state,     tend,          &
                             pbuf,      landfrac,   ocnfrac,   icefrac,       &
                             lcwat,     cld,        concld,    fracis,        &
                             conicw,    ixcldice,  ixcldliq,      &
                             nevapr,    prain,      cmfmc,     cldst,         &
                             cmfdqr,    cnt,        cnb,       dlf,           &
                             ptend,     nstep,      calday )


   call physics_update (state, tend, ptend, ztodt)
   call t_stopf ('aerosol_wet')

#endif
!
! Compute rates of temperature and constituent change due to moist processes
!
   dtcond(:ncol,:) = (state%s(:ncol,:) - dtcond(:ncol,:))*rtdt / cpair
   dqcond(:ncol,:) = (state%q(:ncol,:,1) - dqcond(:ncol,:))*rtdt
!  dqcond(:ncol,:,:) = (state%q(:ncol,:,:) - dqcond(:ncol,:,:))*rtdt


   call outfld('DTCOND  ',dtcond, pcols   ,lchnk   )
   call outfld('CMFLQ' , dqcond  , pcols, lchnk)
!  do m=1,ppcnst
!     call outfld(dcconnam(m),dqcond(1,1,m),pcols   ,lchnk )
!  end do


! Compute total convective and stratiform precipitation and snow rates
   do i=1,ncol
      precc (i) = prec_zmc(i) + prec_cmf(i) 
      precl (i) = prec_sed(i) + prec_pcw(i)
      precsc(i) = snow_zmc(i) + snow_cmf(i)
      precsl(i) = snow_sed(i) + snow_pcw(i)


#ifdef WU_REFATM
      precl (i) = precl(i)  +  rainls(i)   ! m/s
      precsl(i) = precsl(i) +  snowls(i)   ! m/s
#endif


! jrm These checks should not be necessary if they exist in the parameterizations
      if(precc(i).lt.0.) precc(i)=0.
      if(precl(i).lt.0.) precl(i)=0.
      if(precsc(i).lt.0.) precsc(i)=0.
      if(precsl(i).lt.0.) precsl(i)=0.
      if(precsc(i).gt.precc(i)) precsc(i)=precc(i)
      if(precsl(i).gt.precl(i)) precsl(i)=precl(i)
! end jrm
   end do
   prcsnw(:ncol) = precsc(:ncol) + precsl(:ncol)   ! total snowfall rate: needed by slab ocean model
!
!===================================================
! Moist physical parameteriztions complete: 
! send dynamical variables, and derived variables to history file
!===================================================

#ifdef WU_CONV
   deallocate (tracer)
#endif

   return
end subroutine tphysbc_moist
