#include <misc.h>
#include <params.h>

#if (defined MOZART2)

subroutine tphysbc_moist_chem (ztodt,     ts,         state,     tend,          &
                         pbuf,      landfrac,   ocnfrac,   icefrac,       &
                         lcwat,     cld,        concld,    fracis,        &
                         conicw,    ixcldice,   ixcldliq,      &
                         nevapr,    prain,      cmfmc,     cldst,         &
                         cmfdqr,    cnt,        cnb,       dlf,           &
                         ptend,     nstep,      calday )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Tendency physics BEFORE coupling to land, sea, and ice models.
! 
! Method: 
! Call physics subroutines and compute the following:
!     o cloud calculations (cloud fraction, emissivity, etc.)
!     o radiation calculations
! Pass surface fields for separate surface flux calculations
! Dump appropriate fields to history file.
! 
! Author: CCM1, CMS Contact: J. Truesdale
! 
!-----------------------------------------------------------------------


   use shr_kind_mod,    only: r8 => shr_kind_r8
   use ppgrid
   use phys_grid,       only: get_rlat_all_p, get_rlon_all_p, get_lat_all_p
   use phys_buffer,     only: pbuf_size_max, pbuf_fld
   use physics_types,   only: physics_state, physics_tend, physics_ptend, physics_update
   use history,         only: outfld
   use time_manager,    only: get_step_size

   use constituents,    only: pcnst, pnats, ppcnst, qmin, iaero
   use check_energy,    only: check_energy_chng

#ifdef MOZART2
   use constituents,    only: sflxnam
   use mo_surface,      only: sflxdr      !2008.08.11
   use mo_chemdr,       only: chemdr
   use chemistry,       only: ixchm, ncnst
   use mo_chem_mods,           only: grpcnt
   use mo_chem_utls,  only : get_grp_mem_ndx
   use constituents,           only: cnst_get_ind
   use bcc_progseasalts_intr,  only: bcc_progseasalts_wet
   use bcc_dust_intr,          only: bcc_dust_wet
   use bcc_aerosols_intr,      only: mz_aero_wet_intr
   use bcc_wetdep,             only: bcc_clddiag

   use acbnd,            only: acbndget
   use prescribed_em_d3, only: getem3_IPCC, nem3
   use prescribed_em_d2, only: getem2_IPCC, nem2, varname2d
   use prescribed_em_d2, only: idxCO, idxCH4, idxNO, idxC2H4

#endif


!#ifdef WACCM 
!   use prescribed_em_d3,       only: getem3_IPCC, nem3, varname3d
!   use prescribed_em_d2,       only: getem2_IPCC, nem2, varname2d
!   use prescribed_aerosols,    only: get_aerosol, naer_all, idxSUL
!   use physconst,              only : gravit
!   use chem_mods,              only : imozart
!   use m_spc_id,               only : id_H2O, id_O3, id_HNO3
!   use mo_gas_phase_chemdr,    only: gas_phase_chemdr
!#endif
!
!#ifdef BCCCHEM
!   use bcc_charge_neutrality,  only: charge_fix
!   use bcc_iondrag,            only: iondrag_calc, do_waccm_ions
!   use bcc_qbo,                only: qbo_relax
!   use bcc_surface,            only: sflxdr
!   use bcc_neu_wetdep,         only: do_neu_wetdep
!   use bcc_qbo,                only: qbo_use_forcing
!   use constituents,           only: sflxnam
!   use bcc_tropopause,         only: tropopause_twmo
!   use bcc_chemdr,             only: chemdr, qh2o_reset
!   use bcc_neu_wetdep,         only: neu_wetdep_tend
!   use bcc_progseasalts_intr,  only: bcc_progseasalts_wet
!   use bcc_dust_intr,          only: bcc_dust_wet
!   use physconst,              only: latvap
!   use bcc_aerosols_intr,      only: mz_aero_wet_intr
!   use bcc_wetdep,             only: bcc_clddiag
!   use bcc_chem_mods,          only: adv_mass
!   use time_manager,           only: get_curr_date, get_perp_date
!   use prescribed_aerosols,    only: get_aerosol, naer_all, idxSUL
!   use chemistry,              only: ixchm
!   use bcc_chem_mods,          only: gas_pcnst
!   use acbnd,                  only: acbndget
!   use prescribed_em_d3,       only: getem3_IPCC, nem3, varname3d
!   use prescribed_em_d2,       only: getem2_IPCC, nem2, varname2d
!   use prescribed_aerosols,    only: get_aerosol, naer_all, idxSUL
!   use time_manager,           only: is_perpetual
!   use time_manager,           only: get_nstep
!   use bcc_chem_mods,          only : map2chm
!   use bcc_chem_utls,          only : get_spc_ndx
!   use bcc_chem_mods,          only : solsym
!   use constituents,           only: cnst_get_ind
!#endif

!#ifdef GEOSCHEM
!   use wv_saturation,   only: aqsat
!   use gc_emis_edgar,   only: coembdy
!   use chemistry,       only: ncnst, ixchm
!   use gc_chemdr,       only: do_gc_chem, molar_mass
!#endif

   implicit none

#include <comctl.h>
!
! Arguments
!
   real(r8), intent(in) :: ztodt                          ! 2 delta t (model time increment)
   real(r8), intent(in) :: ts(pcols)                      ! surface temperature
   real(r8), intent(in) :: landfrac(pcols)                ! land fraction

   real(r8), intent(in) :: ocnfrac(pcols)                 ! land fraction
   real(r8), intent(in) :: icefrac(pcols)                 ! land fraction

   type(physics_state), intent(inout) :: state
   type(physics_tend ), intent(inout) :: tend
   type(pbuf_fld),      intent(inout), dimension(pbuf_size_max) :: pbuf
!
#ifdef MOZART2
   integer                :: m
    real(r8):: totcond(pcols,pver)
    real(r8):: evapc(pcols,pver)           ! Evaporation rate of convective precipitation   zf????????????
    real(r8):: rainmr(pcols,pver)  ! mixing ratio of rain within cloud volume
    real(r8):: cldv(pcols,pver)     ! cloudy volume undergoing wet chem and scavenging
    real(r8):: cldvcu(pcols,pver)   ! Convective precipitation area at the top interface of current layer
    real(r8):: cldvst(pcols,pver)   ! Stratiform precipitation area at the top interface of current layer
#endif


!#ifdef BCCCHEM
!   integer id3_NO
!
!   real(r8),intent(out)   :: mbar(pcols,pver)      ! mean wet atmospheric mass ( amu )

!   real(r8),intent(inout) :: sflx(pcols, ppcnst)
!   integer, intent(in)    :: troplev (pcols)
!
!   integer                :: n, m
!   integer                :: yr, mon, day, ncsec

!    real(r8):: totcond(pcols,pver)
!    real(r8):: evapc(pcols,pver)           ! Evaporation rate of convective precipitation   zf????????????
!    real(r8):: rainmr(pcols,pver)  ! mixing ratio of rain within cloud volume
!    real(r8):: cldv(pcols,pver)     ! cloudy volume undergoing wet chem and scavenging
!    real(r8):: cldvcu(pcols,pver)   ! Convective precipitation area at the top interface of current layer
!    real(r8):: cldvst(pcols,pver)   ! Stratiform precipitation area at the top interface of current layer
!#endif


#if (defined MOZART2)
   real(r8) :: h2o23d(pcols,pver)
   real(r8) :: o3(pcols,pver)
   real(r8) :: ho2im(pcols,pver)
   real(r8) :: no3(pcols,pver)
   real(r8) :: oh(pcols,pver)

   real(r8) :: em3data (pcols, pver, nem3)
   real(r8) :: em2data (pcols, nem2)
   integer idx

   real(r8) :: asdir(pcols)
#endif


!#ifdef GEOSCHEM
!   integer coidx

!   real(r8) :: srfflx(pcols)
!   real(r8), parameter :: amufac   =  1.65979e-23         ! 1.e4* kg / amu

!   real(r8),intent(inout) :: sflx(pcols, ppcnst)
!   real(r8)               :: albedo(pcols)
!   real(r8)               :: coslat(pcols)
!   real(r8)               :: ptrop(pcols)
!   real(r8)               :: trefht(pcols)
!   real(r8)               :: shflx(pcols)
!   real(r8)               :: u_10m(pcols)
!   real(r8)               :: v_10m(pcols)
!   real(r8)               :: z0(pcols)
!   real(r8)               :: ustar(pcols)
!   real(r8)               :: es(pcols, pver)
!   real(r8)               :: qs(pcols, pver)
!   real(r8)               :: relhum(pcols, pver)
!   real(r8)               :: dz(pcols, pver)
!   real(r8)               :: tend_ciw(pcols, pver)
!   real(r8)               :: tend_clw(pcols, pver)
!   real(r8)               :: tend_w(pcols, pver)
!   real(r8)               :: tend_dynq(pcols, pver)   ! the tendency of q due to dynamics
!   real(r8)               :: rho(pcols, pver)
!   real(r8)               :: tauoptd(pcols,0:pver)
!   real(r8)               :: uv_albedo(pcols)
!   real(r8)               :: gwettop(pcols)
!   integer                :: tmp
!   logical :: FIRST = .true.
!   real(r8)               :: convert(pcols,pver) ! Unit conversion variable
!   real(r8)               :: emis_tmp(pcols, pver), tmp_q(pcols,pver)
!#endif

!#ifdef WACCM
!   real(r8)               :: cldw(pcols, pver)
!   real(r8),intent(inout) :: sflx(pcols, ppcnst)
!   real(r8)               :: fh2o(pcols)
!   real(r8),intent(out)   :: mbar(pcols,pver)      ! mean wet atmospheric mass ( amu )
!   integer, intent(in)    :: troplev (pcols)

!   real(r8)               :: qtemp(pcols,pver,ppcnst)
!#endif
!
!---------------------------Local workspace-----------------------------
!
   type(physics_ptend),intent(inout)  :: ptend                  ! indivdual parameterization tendencies

   integer,intent(in)  :: nstep                         ! current timestep number
   integer      lat(pcols)                   ! current latitudes(indices)

   real(r8),intent(in)  :: calday                        ! current calendar day
   real(r8) :: clat(pcols)                   ! current latitudes(radians)
   real(r8) :: clon(pcols)                   ! current longitudes(radians)

   real(r8),intent(in)  :: cmfmc(pcols,pverp)            ! Convective mass flux--m sub c

   real(r8),intent(in)  :: cldst(pcols,pver)                 ! stratus cloud cover over ocean
   real(r8),intent(in)  :: cnt(pcols)                        ! Top level of convective activity
   real(r8),intent(in)  :: cnb(pcols)                        ! Lowest level of convective activity
   real(r8),intent(in)  :: dlf(pcols,pver)                   ! Detraining cld H20 from convection

   integer lchnk                              ! chunk identifier
   integer ncol                               ! number of atmospheric columns


   integer  i,k                                          ! Longitude, level, constituent indices
   integer,intent(in) :: ixcldice, ixcldliq              ! constituent indices for cloud liquid and ice water.
   
   integer idx_H2O
   integer idx_O3, idx_O, idx_O1D
   integer o3_nadx, o_nadx, o1d_nadx

   real(r8) nas(pcols,pver,max(1,grpcnt))           ! non-advected species( mmr )

!---------------------------------
!
! physics buffer fields to compute tendencies for cloud condensation package
   real(r8),intent(in)   :: lcwat(pcols,pver)
   real(r8),intent(in)   :: cld(pcols,pver)
   real(r8),intent(in)   :: concld(pcols,pver)   ! wzz for convective cloud cover
!                                          
! Used for OUTFLD only                     
!                                          
   real(r8),intent(inout) ::  fracis(pcols,pver,ppcnst)  ! fraction of transported species that are insoluble
!
   real(r8),intent(in)    ::  conicw(pcols,pver)
   real(r8),intent(in)    ::  cmfdqr(pcols,pver)               ! dq/dt due to moist convective rainout

! energy checking variables
   real(r8) :: zero(pcols)                    ! array of zeros

   real(r8),intent(in)    ::  nevapr(pcols,pver)
   real(r8),intent(in)    ::  prain(pcols,pver)

!-----------------------------------------------------------------------

  zero = 0.
  lchnk = state%lchnk
  ncol  = state%ncol

!---------------------------------------------------------

#ifdef MOZART2

   call get_lat_all_p (lchnk, ncol, lat )
   call get_rlat_all_p(lchnk, ncol, clat)
   call get_rlon_all_p(lchnk, ncol, clon)
   

   ! cldc: Cumulus cloud fraction, clds: Stratus cloud fraction
   ! output:  cldv, cldvcu, cldvst, rainmr

    call bcc_clddiag( state%t, state%pmid, state%pdel, cmfdqr, evapc, &
                      cld,  concld, cldst, cmfmc, nevapr, prain, &
                      cldv, cldvcu, cldvst, rainmr, ncol )

    totcond(:ncol,:) = state%q(:ncol,:,ixcldliq) + state%q(:ncol,:,ixcldice)

   !
   ! inout: state%q
   !
    call bcc_dust_wet (lchnk, ncol, state%t, state%pmid, state%pdel, state%q, &  
                       nstep, ztodt, lat, clat, &
                       totcond, cmfmc, prain, &
                       nevapr, cldv, concld, cld, fracis, calday,  cmfdqr, conicw, rainmr)
   !
   ! inout: state%q
   !
    call bcc_progseasalts_wet (lchnk, ncol, state%t, state%pmid, state%pdel, state%q, &   
                               nstep, ztodt, lat, clat, &
                               totcond, cmfmc, prain, &
                               nevapr, cldv, concld, cld, fracis, calday,  cmfdqr, conicw, rainmr)

   !
   ! inout: state%q
   !
    call mz_aero_wet_intr (lchnk, ncol, state%t, state%pmid, state%pdel, state%pmiddry, & 
                           state%q, totcond, state%lat, state%lon, state%zm, &
                           nstep, ztodt, cmfmc, prain, &
                           nevapr,   cldv, cldvcu, cldvst, concld, cld, fracis, calday, &
                           cmfdqr,  evapc, conicw, rainmr, dlf)
#endif

   return
end subroutine tphysbc_moist_chem

#endif
