#include <misc.h>
#include <params.h>

#define PCWDETRAIN

#ifdef TDK_CU
subroutine tphysbc_tdk (ztodt,   pblht,   tpert,   ts,      sst,     &
                    qpert,   precl,   precc,   precsl,  precsc,  &
                    asdir,   asdif,   aldir,   aldif,   snowh,   &
                    qrs,     qrl,     flwds,   fsns,    fsnt,    &
                    flns,    flnt,    lwup,    srfrad,  sols,    &
                    soll,    solsd,   solld,   state,   tend,    &
                    pbuf,    prcsnw,  fsds ,   landm,   landfrac,&
  		            ocnfrac, icefrac,   dyn_tend,     cflxn  )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Tendency physics BEFORE coupling to land, sea, and ice models.
! 
! Method: 
! Call physics subroutines and compute the following:
!     o cloud calculations (cloud fraction, emissivity, etc.)
!     o radiation calculations
! Pass surface fields for separate surface flux calculations
! Dump appropriate fields to history file.
! 
!-----------------------------------------------------------------------

   use shr_kind_mod,    only: r8 => shr_kind_r8
   use ppgrid
   use phys_buffer,     only: pbuf_size_max, pbuf_fld, pbuf_old_tim_idx, pbuf_get_fld_idx
   use physics_types,   only: physics_state, physics_tend, physics_ptend, physics_update, &
                              dynamics_tend, physics_ptend_init
   use diagnostics,     only: diag_dynvar
   use history,         only: outfld
   use physconst,       only: gravit, latvap, cpair, tmelt, cappa, zvir, rair, rga, latice
   use constituents,    only: pcnst, pnats, ppcnst, qmin, ixcldw
   use constituents,    only: dcconnam, cnst_get_ind
   use check_energy,    only: check_tracers_data, check_tracers_init, check_tracers_chng
   use check_energy,    only: check_energy_chng
   use radheat,         only: radheat_net
   use time_manager,    only: get_nstep, get_curr_calday
   use time_manager,    only: is_first_step

!----
   use cldcond,         only: cldcond_tend, cldcond_zmconv_detrain, cldcond_sediment
   use zm_conv,         only: zm_conv_evap, zm_convr
   use moistconvection, only: cmfmca
   use cloudsimulator,  only: doisccp, cloudsimulator_run
   use geopotential,    only: geopotential_t
   use cldwat,          only: pcond_tdk, cldwat_fice
   use param_cldoptics, only: param_cldoptics_calc
!----------------------
   use phys_grid,       only: get_rlat_all_p, get_rlon_all_p, get_lon_all_p, get_lat_all_p
   use cldcond,         only: cldcond_tend, cldcond_zmconv_detrain, cldcond_sediment
   use zm_conv,         only: zm_conv_evap, zm_convr
   use moistconvection, only: cmfmca
   use aerosol_intr,    only: aerosol_wet_intr

!----------

   implicit none
!
! Arguments
!
   real(r8), intent(in) :: ztodt                          ! 2 delta t (model time increment)
   real(r8), intent(in) :: ts(pcols)                      ! surface temperature
   real(r8), intent(in) :: sst(pcols)                     ! sea surface temperature
   real(r8), intent(inout) :: pblht(pcols)                ! Planetary boundary layer height
   real(r8), intent(inout) :: tpert(pcols)                ! Thermal temperature excess
   real(r8), intent(inout) :: qpert(pcols,ppcnst)         ! Thermal humidity & constituent excess
   real(r8), intent(in) :: asdir(pcols)                  ! Albedo: shortwave, direct
   real(r8), intent(in) :: asdif(pcols)                  ! Albedo: shortwave, diffuse
   real(r8), intent(in) :: aldir(pcols)                  ! Albedo: longwave, direct
   real(r8), intent(in) :: aldif(pcols)                  ! Albedo: longwave, diffuse
   real(r8), intent(in) :: snowh(pcols)                  ! Snow depth (liquid water equivalent)
   real(r8), intent(inout) :: qrs(pcols,pver)            ! Shortwave heating rate
   real(r8), intent(inout) :: qrl(pcols,pver)            ! Longwave  heating rate
   real(r8), intent(inout) :: flwds(pcols)               ! Surface longwave down flux
   real(r8), intent(inout) :: fsns(pcols)                   ! Surface solar absorbed flux
   real(r8), intent(inout) :: fsnt(pcols)                   ! Net column abs solar flux at model top
   real(r8), intent(inout) :: flns(pcols)                   ! Srf longwave cooling (up-down) flux
   real(r8), intent(inout) :: flnt(pcols)                   ! Net outgoing lw flux at model top
   real(r8), intent(in)  :: lwup(pcols)                    ! Surface longwave up flux
   real(r8), intent(out) :: srfrad(pcols)                 ! Net surface radiative flux (watts/m**2)
   real(r8), intent(inout) :: sols(pcols)                   ! Direct beam solar rad. onto srf (sw)
   real(r8), intent(inout) :: soll(pcols)                   ! Direct beam solar rad. onto srf (lw)
   real(r8), intent(inout) :: solsd(pcols)                  ! Diffuse solar radiation onto srf (sw)
   real(r8), intent(inout) :: solld(pcols)                  ! Diffuse solar radiation onto srf (lw)
   real(r8), intent(out) :: precl(pcols)                  ! Large-scale precipitation rate
   real(r8), intent(out) :: precc(pcols)                  ! Convective-scale preciptn rate
   real(r8), intent(out) :: precsl(pcols)                 ! L.S. snowfall rate
   real(r8), intent(out) :: precsc(pcols)                 ! C.S. snowfall rate
   real(r8), intent(out) :: prcsnw(pcols)                 ! snowfall rate (precsl + precsc)
   real(r8), intent(out) :: fsds(pcols)                   ! Surface solar down flux
   real(r8), intent(in) :: landm(pcols)                   ! land fraction ramp
   real(r8), intent(in) :: landfrac(pcols)                ! land fraction
   real(r8), intent(in) :: ocnfrac(pcols)                ! land fraction
   real(r8), intent(in) :: icefrac(pcols)                ! land fraction

   type(physics_state), intent(inout) :: state
   type(physics_tend ), intent(inout) :: tend
   type(dynamics_tend), intent(inout) :: dyn_tend
   type(pbuf_fld),      intent(inout), dimension(pbuf_size_max) :: pbuf

   real(r8), intent(in)  :: cflxn(pcols,ppcnst)            ! sxl add 
!
!---------------------------Local workspace-----------------------------
!
   type(check_tracers_data):: tracerint             ! energy integrals and cummulative boundary fluxes
   type(physics_ptend)   :: ptend                  ! indivdual parameterization tendencies


   integer :: nstep                         ! current timestep number
   integer :: lchnk                              ! chunk identifier
   integer :: ncol                               ! number of atmospheric columns
   integer :: i,k,m                             ! Longitude, level, constituent indices

   real(r8) :: calday                        ! current calendar day
   real(r8) :: timestep(pcols)
   real(r8) :: concld(pcols,pver)             
   real(r8) :: zero(pcols)                    ! array of zeros
   logical  :: conserve_energy = .true.       ! flag to carry (QRS,QRL)*dp across time steps

!-------------
!
! physics buffer fields to compute tendencies for cloud condensation package
!
   real(r8) cflxn0(pcols,ppcnst)        

   integer itim, ifld
   real(r8), pointer, dimension(:,:) :: qcwat, tcwat, lcwat, cld
   real(r8), pointer, dimension(:  ) :: teout
   real(r8), pointer, dimension(:,:) :: qini
   real(r8), pointer, dimension(:,:) :: tini

   real(r8) :: preclint(pcols)               ! Large-scale precipitation rate (>0.1mm/hr)
   real(r8) :: preccint(pcols)               ! Convective-scale preciptn rate (>0.1mm/hr)
   real(r8) :: preclfrq(pcols)               ! Large-scale precipitation frequency (>0.1mm/hr)
   real(r8) :: preccfrq(pcols)               ! Convective-scale preciptn frequency (>0.1mm/hr)
   real(r8) :: prect(pcols)                      ! total (conv+large scale) precip rate
   real(r8) :: preclp(pcols)                     ! sfc flux of precip from pcond
   real(r8) :: qc(pcols,pver)                    ! dq/dt due to rainout terms
   real(r8) :: nevapr(pcols,pver)                ! local evaporation of precipitation
   real(r8) :: prain(pcols,pver)                 ! local formation of precipitation

   real(r8) :: rhdfda(pcols,pver)            ! dRh/dcloud, old 
   real(r8) :: rhu00 (pcols,pver)            ! Rh threshold for cloud, old
   real(r8) :: zdu(pcols,pver)               ! detraining mass flux from deep convection
   real(r8) :: ftem(pcols,pver)              ! Temporary workspace for outfld variables
   real(r8) :: cmfdqr(pcols,pver)            ! dq/dt due to moist convective rainout
   real(r8) :: cmfmc(pcols,pverp)            ! Convective mass flux--m sub c
   real(r8) :: cmfmc2(pcols,pverp)            ! Convective mass flux--m sub c
   real(r8) :: cmfsl(pcols,pver)             ! Moist convection lw stat energy flux
   real(r8) :: cmflq(pcols,pver)             ! Moist convection total water flux
   real(r8) :: dtcond(pcols,pver)            ! dT/dt due to moist processes
   real(r8) :: dqcond(pcols,pver,ppcnst)     ! dq/dt due to moist processes
   real(r8) :: cldst(pcols,pver)
   real(r8) :: cnt(pcols)                        ! Top level of convective activity
   real(r8) :: cnb(pcols)                        ! Lowest level of convective activity
   real(r8) :: dlf(pcols,pver)                   ! Detraining cld H20 from convection
   real(r8) :: fwaut(pcols,pver)              
   real(r8) :: fsaut(pcols,pver)              
   real(r8) :: fracw(pcols,pver)              
   real(r8) :: fsacw(pcols,pver)              
   real(r8) :: fsaci(pcols,pver)              
   real(r8) :: qme(pcols,pver)                   ! local condensation of cloud water
   real(r8) :: clc(pcols)                        ! Total convective cloud (cloud scheme)
   real(r8) :: qtend(pcols,pver)                 ! moisture tendencies
   real(r8) :: ttend(pcols,pver)                 ! temp tendencies
   real(r8) :: lctend(pcols,pver)                ! cloud liquid water tendencies
   real(r8) :: rmelt(pcols,pver)                 ! heating rate due to phase change of precip
   real(r8) :: zero_tdk(pcols,pverp)                 ! a dummy array

   real(r8) :: dellow(pcols)                     ! delta p for bottom three levels of model
   real(r8) :: tavg(pcols)                       ! mass weighted average temperature for 
   real(r8) :: fracis(pcols,pver,ppcnst)         ! fraction of transported species that are insoluble
   real(r8) :: rtdt                              ! 1./ztodt

   real(r8) :: mu2(pcols,pver)
   real(r8) :: eu2(pcols,pver)
   real(r8) :: du2(pcols,pver)
   real(r8) :: md2(pcols,pver)
   real(r8) :: ed2(pcols,pver)
   real(r8) :: geom1(pcols,pver)
   real(r8) :: papp1(pcols,pver)
   real(r8) :: paphp1(pcols,pver+1)
   real(r8) :: dpp  (pcols,pver)
   real(r8) :: psp1 (pcols) 

   logical  ldland(pcols)

    real(r8) :: fice    (pcols,pver)          ! Fractional ice content within cloud
    real(r8) :: fsnow   (pcols,pver)          ! Fractional snow production
    real(r8) :: totcw   (pcols,pver)          ! total cloud water mixing ratio
    real(r8) :: repartht(pcols,pver)          ! heating rate due to phase repartition of input precip
    integer :: ixcldice, ixcldliq              ! constituent indices for cloud liquid and ice water.

    real(r8) cflx(pcols,ppcnst)

!------------
   real(r8) :: clat(pcols)                   ! current latitudes(radians)
   real(r8) :: clon(pcols)                   ! current longitudes(radians)
   real(r8) :: coszrs(pcols)                     ! Cosine solar zenith angle

   real(r8) :: cicewp(pcols,pver)             ! in-cloud cloud ice water path
   real(r8) :: cliqwp(pcols,pver)             ! in-cloud cloud liquid water path
   real(r8) :: rel(pcols,pver)                   ! Liquid cloud particle effective radius
   real(r8) :: rei(pcols,pver)                   ! Ice effective drop size (microns)
   real(r8) :: emis(pcols,pver)                  ! Cloud longwave emissivity
   real(r8) :: pmxrgn(pcols,pverp)               ! Maximum values of pressure for each
   real(r8) :: cltot(pcols)                      ! Diagnostic total cloud cover
   real(r8) :: cllow(pcols)                      !       "     low  cloud cover
   real(r8) :: clmed(pcols)                      !       "     mid  cloud cover
   real(r8) :: clhgh(pcols)                      !       "     hgh  cloud cover

   integer nmxrgn(pcols)                      ! Number of maximally overlapped regions

#include <comctl.h>
!-----------------------------------------------------------------------
   zero = 0.

   lchnk = state%lchnk
   ncol  = state%ncol

   nstep = get_nstep()
   calday = get_curr_calday()
!
! Output NSTEP for debugging
!
   timestep(:ncol) = nstep
   call outfld ('NSTEP   ',timestep, pcols, lchnk)

!  dry surface pressure
   call outfld ('PSDRY',  state%psdry, pcols, lchnk)

!
! Convert mixing ratio of non-water tracers to mass fraction of total
! atmospheric mass (Overwrite non-water portions of q3m1).
!
!   if (ppcnst > 1) then
!      call mr2mf (lchnk, ncol, state%q)
!   end if

!===================================================
! Moist convection
!===================================================
!
! Associate pointers with physics buffer fields
!
   itim = pbuf_old_tim_idx()
   ifld = pbuf_get_fld_idx('QCWAT')
   qcwat => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)
   ifld = pbuf_get_fld_idx('TCWAT')
   tcwat => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)
   ifld = pbuf_get_fld_idx('LCWAT')
   lcwat => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)
   ifld = pbuf_get_fld_idx('CLD')
   cld => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

   ifld = pbuf_get_fld_idx('TEOUT')
   teout => pbuf(ifld)%fld_ptr(1,1:pcols,1,lchnk,itim)
   ifld = pbuf_get_fld_idx('QINI')
   qini  => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)
   ifld = pbuf_get_fld_idx('TINI')
   tini  => pbuf(ifld)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

!
! Set reciprocal of layer thickness
! Set physics tendencies to 0
!
   state%rpdel(:ncol,:pver) = 1./state%pdel(:ncol,:pver)

   tend %dTdt(:ncol,:pver)  = 0.
   tend %dudt(:ncol,:pver)  = 0.
   tend %dvdt(:ncol,:pver)  = 0.
!
! Compute initial geopotential heights and dry static energies
!
   call geopotential_t (state%lnpint, state%lnpmid  , state%pint,  &
                        state%pmid  , state%pdel    , state%rpdel, &
                        state%t     , state%q(1,1,1), rair , gravit , zvir   , &
                        state%zi    , state%zm      , ncol      )

   state%s(:ncol,:pver) = cpair*state%t(:ncol,:pver) + gravit*state%zm(:ncol,:pver)
   geom1  (:ncol,:pver) = gravit*state%zm(:ncol,:pver)
!
   call physics_ptend_init (ptend) ! Initialize parameterization tendency structure
!
! Make sure that input tracers are all positive (probably unnecessary)
!
   call qneg3('TPHYSBCb',lchnk  ,ncol    ,pcols   ,pver    , &
              ppcnst, qmin  ,state%q )
!
! Setup q and t accumulation fields
!

   fracis (:ncol,:,1:ppcnst) = 1.

! compute mass integrals of input tracers state
!
   call check_tracers_init(state, tracerint)

!===================================================
! Global mean total energy fixer
!===================================================

   tini(:ncol,:pver) = state%t(:ncol,:pver)
   qini(:ncol,:pver) = state%q(:ncol,:pver,1)

   call outfld('TEOUT', teout       , pcols, lchnk   )
   call outfld('TEINP', state%te_ini, pcols, lchnk   )
   call outfld('TEFIX', state%te_cur, pcols, lchnk   )
   call outfld('ENEK',  state%ke,     pcols, lchnk   )

   dqcond(:ncol,:,:) = state%q(:ncol,:,:)
   dtcond(:ncol,:)   = state%s(:ncol,:)
!
!===================================================
! Dry adjustment
!===================================================

! Copy state info for input to dadadj
! This is a kludge, so that dadadj does not have to be correctly reformulated in dry static energy

   ptend%s(:ncol,:pver)   = state%t(:ncol,:pver)
   ptend%q(:ncol,:pver,1) = state%q(:ncol,:pver,1)

   call dadadj (lchnk, ncol, state%pmid,  state%pint,  state%pdel,  &
                ptend%s, ptend%q(1,1,1))
   ptend%name  = 'dadadj'   
   ptend%ls    = .TRUE.
   ptend%lq(1) = .TRUE.
   ptend%s(:ncol,:)   = (ptend%s(:ncol,:)   - state%t(:ncol,:)  )/ztodt * cpair
   ptend%q(:ncol,:,1) = (ptend%q(:ncol,:,1) - state%q(:ncol,:,1))/ztodt

   call physics_update (state, tend, ptend, ztodt)

!
! Zero out precip and convective fields before accumulating terms
!
   precl (:ncol)   = 0.
   preclp(:ncol)   = 0.
   precc (:ncol)   = 0.
   precsl(:ncol)   = 0.
   precsc(:ncol)   = 0.
   qc    (:ncol,:) = 0.
   cmfdqr(:ncol,:) = 0.
   cmfmc (:ncol,:) = 0.
   cmfsl (:ncol,:) = 0.
   cmflq (:ncol,:) = 0.

   dqcond(ncol+1:pcols,:,:) = 0.
   dtcond(ncol+1:pcols,:)   = 0.
!
!===================================================
! Moist convection
!===================================================
!
! Since the PBL doesn't pass constituent perturbations, they
! are zeroed here for input to the moist convection routine
!
   qpert(:ncol,2:ppcnst) = 0.0
!
! JR Set some arrays to zero @ nstep=0. Otherwise random junk off the heap
! or stack will be used in zm_convr
!
   if (is_first_step()) then
      pblht(:ncol)  = 0.
      tpert(:ncol)  = 0.

      do i=1,ncol
         ldland(i)=nint(landfrac(i)) == 1            !sxl add
      end do
   end if

   psp1(:ncol) = state%ps(:ncol) + dyn_tend%pstend(:ncol)*ztodt
!
   call plevs0(ncol, pcols, pver, psp1, paphp1, papp1, dpp)
!
! Begin with Zhang-McFarlane (1996) convection parameterization
!
   call t_startf ('zm_convr')

   mu2(:ncol,:pver)=0.
   md2(:ncol,:pver)=0.
   du2(:ncol,:pver)=0.
   eu2(:ncol,:pver)=0.
   ed2(:ncol,:pver)=0.
   zdu(:ncol,:pver)=0.

   call cucall(ncol, ncol, ncol, pver, pver+1, pver-1, ztodt,  &
                   state%t, state%q(:,:,1), state%u, state%v, state%q(:,:,ixcldw),           &
                   dyn_tend%ttend, dyn_tend%qtend(:,:,1), dyn_tend%utend, dyn_tend%vtend,    &
                   dyn_tend%qtend(:,:,ixcldw), state%omega,  -cflxn(:,1), papp1, paphp1,       &
                   geom1, ldland, ptend%s, ptend%u, ptend%v, ptend%q(:,:,1), precc,          &
                   dlf, cnt, cnb, zdu, cmfmc, mu2, md2, du2, eu2, ed2, dyn_tend%tatend,      &
                   dyn_tend%qatend(:,:,1) )

   ptend%name  = 'zm_convr'    
   ptend%ls    = .TRUE.
   ptend%lq(1) = .TRUE.
   ptend%lu    = .TRUE.
   ptend%lv    = .TRUE.

   ftem(:ncol,:pver) = ptend%s(:ncol,:pver)/cpair
   call outfld('ZMDT    ',ftem           ,pcols   ,lchnk   )
   call outfld('ZMDQ    ',ptend%q(1,1,1) ,pcols   ,lchnk   )

   call t_stopf('zm_convr')

   call physics_update(state, tend, ptend, ztodt)
!
! At PJR's instruction, deleted kludge to get past a once in a lifetime
! problem in cmfmca's transport of liq water due to reliance on m=2 being
! hard-wired to cloud water--JR.  Put back in after run bombed.
!
   where (abs(state%q(:ncol,:pver,ixcldw)) < 1.e-36)
      state%q(:ncol,:pver,ixcldw) = 0.
   end where

#ifndef PCWDETRAIN
!
! put the detraining cloud water into precip to conserve
! mass
!
   do k = 1,pver
      do i = 1,ncol
         precc(i) = precc(i) + dlf(i,k)*state%pdel(i,k)/(gravit*1000.)
      end do
   end do
#else
!
! put the detraining cloud water into the cloud and environment in
! proportion to the cloud fraction
!
   do k = 1,pver
      do i = 1,ncol
         ptend%q(i,k,1)      = dlf(i,k)*(1.-cld(i,k))
         ptend%s(i,k)        =-dlf(i,k)*(1.-cld(i,k))*latvap
         ptend%q(i,k,ixcldw) = dlf(i,k)*cld(i,k)
      end do
   end do
   ptend%name  ='pcwdetrain'     
!   ptend%ls    = .TRUE.
!   ptend%lq(1) = .TRUE.
   ptend%lq(ixcldw) = .TRUE.

   call physics_update(state, tend, ptend, ztodt)

#endif

   call t_startf('cldnrh')
   call cldnrh_tdk(lchnk,   ncol,                                &
               state%pmid,    state%t,   state%q(1,1,1),   state%omega, &
               cnt,     cnb,     cld,    clc,     state%pdel,   &
               cmfmc,   landfrac,snowh,  concld,  cldst,    &
               ts,      state%pint(1,pverp),      zdu,  ocnfrac, &
               rhdfda,   rhu00 )
   call t_stopf('cldnrh')

!
! calculate the tendencies for moisture, temperature and cloud fraction
!
   rtdt = 1./ztodt
   qtend(:ncol,:pver) = (state%q(:ncol,:pver,1)       - qcwat(:ncol,:pver))*rtdt
   ttend(:ncol,:pver) = (state%t(:ncol,:pver)         - tcwat(:ncol,:pver))*rtdt
   lctend(:ncol,:pver) = (state%q(:ncol,:pver,ixcldw) - lcwat(:ncol,:pver))*rtdt

!----
   call t_startf('pcond')

   zero_tdk(:ncol,:pverp) = 0.

   call pcond_tdk (lchnk,   ncol, &
               state%t,   ttend,   state%q(1,1,1), qtend,       state%omega,     &
               state%q(1,1,ixcldw),state%pmid,     state%pdel,  cld,     &
               qme,     nevapr,    prain,          rmelt,    &
               ztodt,   zero_tdk,      fwaut,          fsaut,       fracw,    &
               fsacw,   fsaci,     lctend,         rhdfda,      rhu00, icefrac)
   call t_stopf('pcond')
!
   call outfld('FWAUT',fwaut, pcols,lchnk)
   call outfld('FSAUT',fsaut, pcols,lchnk)
   call outfld('FRACW',fracw, pcols,lchnk)
   call outfld('FSACW',fsacw, pcols,lchnk)
   call outfld('FSACI',fsaci, pcols,lchnk)
!
! make it interactive
!
   do k = 1,pver
      do i = 1,ncol
         ptend%s(i,k)        = (qme(i,k) - nevapr(i,k))*latvap + rmelt(i,k)
         ptend%q(i,k,1)      =-(qme(i,k) - nevapr(i,k))
         ptend%q(i,k,ixcldw) = (qme(i,k) - prain(i,k))
         preclp(i) = preclp(i) + (prain(i,k)-nevapr(i,k))*state%pdel(i,k)/gravit
      end do
   end do
   ptend%name  = 'pcond'    
   ptend%ls    = .TRUE.
   ptend%lq(1) = .TRUE.
   ptend%lq(ixcldw) = .TRUE.
   call physics_update (state, tend, ptend, ztodt)
!
! save off q and t after cloud water
!
   do k=1,pver
      qcwat(:ncol,k) = state%q(:ncol,k,1)
      tcwat(:ncol,k) = state%t(:ncol,k)
      lcwat(:ncol,k) = state%q(:ncol,k,ixcldw)
   end do
!
!     Convective transport of all trace species except water vapor and
!     cloud water done here because we need to do the scavenging first
!     to determine the interstitial fraction.

   call outfld('CLDST   ',cldst,  pcols,lchnk)
   call outfld('CNVCLD  ',clc,    pcols,lchnk)
   call outfld('CONCLD  ',concld, pcols,lchnk)
   call outfld('CME     ',qme,    pcols,lchnk)
!
! and add the precip of pcond and cond together. Note
! since preclp is in kg/m2/s,  We need to renormalize to m/s
! like other precips
!
   precl(:ncol) = precl(:ncol) + preclp(:ncol)/1000.
!
! Compute rate of temperature change due to moist processes
!
   dtcond(:ncol,:) = (state%s(:ncol,:) - dtcond(:ncol,:))*rtdt
   call outfld('DTCOND  ',dtcond / cpair  ,pcols   ,lchnk   )
!
! Compute rate of constituent change due to moist processes
!
   dqcond(:ncol,:,:) = (state%q(:ncol,:,:) - dqcond(:ncol,:,:))*rtdt
   do m=1,ppcnst
      call outfld(dcconnam(m),dqcond(1,1,m),pcols   ,lchnk )
   end do

!----------------------------------
!
! Define fractional amount of cloud condensate in ice phase
!
    call cldwat_fice(ncol, state%t, fice, fsnow)

! Repartition ice and liquid
!
   call cnst_get_ind('CLDLIQ', ixcldliq)
   call cnst_get_ind('CLDICE', ixcldice)

   do k = 1,pver
      do i = 1,ncol
        state%q(i,k,ixcldice) = state%q(i,k,ixcldw) * fice(i,k)
        state%q(i,k,ixcldliq) = state%q(i,k,ixcldw) * (1.0_r8 - fice(i,k))
      enddo
   enddo
!
!===================================================
! Moist physical parameteriztions complete: 
!===================================================
!
! determine whether precipitation, prec, is frozen (snow) or not
! by taking the mass-weighted temperature of the bottom most layers.
!
!  determine mass weighted average temperature in the bottom three 
!  levels of the model (approximately the lowest 900 meters)
!
   dellow(:ncol) = 0.0
   tavg (:ncol) = 0.0
!
   do k=pver-2, pver
      dellow(:ncol) = dellow(:ncol) + state%pdel(:ncol,k)   
      tavg(:ncol) = tavg(:ncol) + state%t(:ncol,k )*state%pdel(:ncol,k)
   end do
!
   tavg (:ncol) = tavg(:ncol)/dellow(:ncol)
!
   where (tavg(:ncol) > (tmelt-2.0) )
      precsc(:ncol) = 0.
      precsl(:ncol) = 0.
   elsewhere
      precsc(:ncol) = precc(:ncol)
      precsl(:ncol) = precl(:ncol)
   end where
   prcsnw(:ncol) = precsc(:ncol) + precsl(:ncol)   ! total snowfall rate: needed by slab ocean model
!
! Precipitation intensity and frequency (where precip exceeds given threshold)
!
  where( precc(:ncol) >= precc_thresh )
     preccfrq(:ncol) = 1.0
     preccint(:ncol) = precc(:ncol)
  elsewhere
     preccint(:ncol) = 0.0
     preccfrq(:ncol) = 0.0
  end where
  where( precl(:ncol) >= precl_thresh )
     preclfrq(:ncol) = 1.0
     preclint(:ncol) = precl(:ncol)
  elsewhere
     preclint(:ncol) = 0.0
     preclfrq(:ncol) = 0.0
  end where
  preclint(:ncol) = preclint(:ncol)*(3600.*1000.) ! convert from m/sec to mm/hr
  preccint(:ncol) = preccint(:ncol)*(3600.*1000.) ! convert from m/sec to mm/hr

!---------------------------------------------------------------------------------------
! Save history variables. These should move to the appropriate parameterization interface
!---------------------------------------------------------------------------------------

   call outfld('CMFMC   ',cmfmc   ,pcols   ,lchnk   )
   call outfld('CMFSL   ',cmfsl   ,pcols   ,lchnk   )
   call outfld('CMFLQ   ',cmflq   ,pcols   ,lchnk   )
   call outfld('PRECL   ',precl   ,pcols   ,lchnk       )
   call outfld('PRECC   ',precc   ,pcols   ,lchnk       )
   call outfld('PRECLINT',preclint,pcols   ,lchnk       )
   call outfld('PRECCINT',preccint,pcols   ,lchnk       )
   call outfld('PRECLFRQ',preclfrq,pcols   ,lchnk       )
   call outfld('PRECCFRQ',preccfrq,pcols   ,lchnk       )
   call outfld('PRECSL  ',precsl  ,pcols   ,lchnk       )
   call outfld('PRECSC  ',precsc  ,pcols   ,lchnk       )
   
   prect(:ncol) = precc(:ncol) + precl(:ncol)
   call outfld('PRECT   ',prect   ,pcols   ,lchnk       )
   call outfld('PRECTMX ',prect   ,pcols   ,lchnk       )

#if ( defined COUP_CSM )
   call outfld('PRECLav ',precl   ,pcols   ,lchnk   )
   call outfld('PRECCav ',precc   ,pcols   ,lchnk   )
#endif

!
!===================================================
! Moist physical parameteriztions complete: 
! send dynamical variables, and derived variables to history file
!===================================================
!
   call diag_dynvar (lchnk, ncol, state)
!
!===================================================
! Radiation computations
!===================================================
!
! Cosine solar zenith angle for current time step
!
   call get_rlat_all_p(lchnk, ncol, clat)
   call get_rlon_all_p(lchnk, ncol, clon)
   call zenith (calday, clat, clon, coszrs, ncol)

   if (dosw .or. dolw) then

! Compute cloud water/ice paths and optical properties for input to radiation
!
      call t_startf('cldoptics')
      call param_cldoptics_calc(state, cld, landfrac, landm,icefrac, &
                                cicewp, cliqwp, emis, rel, rei, pmxrgn, nmxrgn, snowh)
      call t_stopf('cldoptics')
!
! Complete radiation calculations
!
      call t_startf ('radctl')
      call radctl (lchnk, ncol, lwup, emis, state%pmid,             &
                   state%pint, state%lnpmid, state%lnpint, state%t, state%q,   &
                   cld, cicewp, cliqwp, coszrs, asdir, asdif,               &
                   aldir, aldif, pmxrgn, nmxrgn, fsns, fsnt    ,flns    ,flnt    , &
                   qrs, qrl, flwds, rel, rei,                       &
                   sols, soll, solsd, solld,                  &
                   landfrac, state%zm, state, fsds)
      call t_stopf ('radctl')
!
! Cloud cover diagnostics
! radctl can change pmxrgn and nmxrgn so cldsav needs to follow 
! radctl.
!
      call cldsav (lchnk, ncol, cld, state%pmid, cltot, &
                   cllow, clmed, clhgh, nmxrgn, pmxrgn)
!
! Dump cloud field information to history tape buffer (diagnostics)
!
      call outfld('CLDTOT  ',cltot  ,pcols,lchnk)
      call outfld('CLDLOW  ',cllow  ,pcols,lchnk)
      call outfld('CLDMED  ',clmed  ,pcols,lchnk)
      call outfld('CLDHGH  ',clhgh  ,pcols,lchnk)
      call outfld('CLOUD   ',cld    ,pcols,lchnk)

      if (doisccp) then
         call cloudsimulator_run(state, ts, concld, cld, cliqwp, &
                                 cicewp, rel, rei, emis, coszrs  )
      end if
   else

! convert radiative heating rates from Q*dp to Q for energy conservation
      if (conserve_energy) then
         do k =1 , pver
            do i = 1, ncol
               qrs(i,k) = qrs(i,k)/state%pdel(i,k)
               qrl(i,k) = qrl(i,k)/state%pdel(i,k)
            end do
         end do
      end if
         
   end if


!-------------------------------------------------------------------------------
! Compute net flux
! Since fsns, fsnt, flns, and flnt are in the buffer, array values will be carried across
! timesteps when the radiation code is not invoked.
!
   do i=1,ncol
      tend%flx_net(i) = fsnt(i) - fsns(i) - flnt(i) + flns(i)
   end do
!
! Compute net radiative heating
!
   call radheat_net (state, ptend, qrl, qrs)
!
! Add radiation tendencies to cummulative model tendencies and update profiles
!
   call physics_update(state, tend, ptend, ztodt)

! check energy integrals
!
   call check_energy_chng(state, tend, "radheat", nstep, ztodt, zero, zero, zero, tend%flx_net)
!
! Compute net surface radiative flux for use by surface temperature code.
! Note that units have already been converted to mks in RADCTL.  Since
! fsns and flwds are in the buffer, array values will be carried across
! timesteps when the radiation code is not invoked.
!
   srfrad(:ncol) = fsns(:ncol) + flwds(:ncol)

   call outfld('SRFRAD  ',srfrad,pcols,lchnk)
!
! Save atmospheric fields to force surface models
!
   call srfxfer (lchnk, ncol, state%ps, state%u(1,pver), state%v(1,pver),    &
                 state%t(1,pver), state%q(1,pver,1), state%exner(1,pver), state%zm(1,pver), &
                    state%pmid,      &
                 state%rpdel(1,pver))

!---------------------------------------------------------------------------------------
! Save history variables. These should move to the appropriate parameterization interface
!---------------------------------------------------------------------------------------
!     
! Compute heating rate for dtheta/dt 
!
   do k=1,pver
      do i=1,ncol
         ftem(i,k) = (qrs(i,k) + qrl(i,k))/cpair * (1.e5/state%pmid(i,k))**cappa
      end do
   end do
   call outfld('HR      ',ftem    ,pcols   ,lchnk   )

! convert radiative heating rates to Q*dp for energy conservation
   if (conserve_energy) then
      do k =1 , pver
         do i = 1, ncol
            qrs(i,k) = qrs(i,k)*state%pdel(i,k)
            qrl(i,k) = qrl(i,k)*state%pdel(i,k)
         end do
      end do
   end if

!
! Convert mass fractions of non-water tracers back to mixing ratios.
! (Overwrite non-water portions of q3m1).
!
!   if (ppcnst > 1) then
!      call mf2mr (lchnk, ncol, state%q)
!   end if


   return
end subroutine tphysbc_tdk
#endif
