!
! Tranfer the grapes binary initial data to NetCdf format
! The description of the variables should be completed
! wzz, 2007.07.13
!
  implicit none
!
#include <netcdf.inc>
!
  integer, parameter :: ids=1, ide=128
  integer, parameter :: jds=1, jde=64
  integer, parameter :: kms=1, kme=32
  integer, parameter :: num_soil_layers=5   ! aerosol types
!
  real, allocatable, dimension(:,:,:) :: ain
  real, allocatable, dimension(:,:)   :: ain2d
  real, allocatable, dimension(:)     :: ain1d
  real, allocatable, dimension(:)     :: alon, alat
!
  integer, dimension(5)               :: mdate
  integer date(12)
  integer                             :: fid
  integer :: i, j, k, l, jdf
  character :: filname*256, unit*256, name*256
  logical :: global_opt = .true.
!
  integer :: nlon, nlat, nlev, luc, nsoil, nsand, nclay
  integer :: status, ncid
  integer :: LONID, LATID, LEVID, TIMEID, NSOILID, NLUCID, &
             NSLID, NSDID, NCYID, LEVAID
  integer :: dim4_id(4)
  integer :: start2w(4), count2w(4)
  integer :: varid_LON, varid_LAT, varid_LEV, varid_TIME, varid_NS
  integer :: varid
  data date/18700115,18700214,18700315,18700415,18700515,18700615,  &
            18700715,18700815,18700915,18701015,18701115,18701215/

   jdf = jde
!   IF (global_opt) jdf = jde-1

   nlon = ide-ids+1
   nlat = jdf-jds+1
   nlev = kme-kms+1
   luc  = 15; nsoil=12; nsand=6; nclay=2
   allocate (alon(nlon) )
   allocate (alat(nlat) )
!
   mdate(1) = 1985
   mdate(2) = 1
   mdate(3) = 1
   mdate(4) = 0
   mdate(5) = 0
!   READ(fid) mdate
   write(*,*)'mdate = ', mdate
!
   filname = 'SOx.nc'
   status = NF_OPEN(filname, NF_NOWRITE,ncid)
   status = nf_inq_varid (ncid, 'lon', varid)
   status = nf_get_var_real (ncid, varid, alon)
   status = nf_inq_varid (ncid, 'lat', varid)
   status = nf_get_var_real (ncid, varid, alat)
   status = nf_close(ncid)
!
   filname = 'chem.dat'
   status=nf_create(trim(filname)//'.nc', nf_clobber, ncid)
   if (status .NE. nf_noerr) call handerr(status)
   write(*,*) 'A NetCDF file is created, and its ID is => ', ncid
!... begin to define your dimentions
   status=nf_def_dim(ncid, 'lon', nlon, LONID)
   if (status .NE. nf_noerr) call handerr(status)

   status=nf_def_dim(ncid, 'lat', nlat, LATID)
   if (status .NE. nf_noerr) call handerr(status)

   status=nf_def_dim(ncid, 'lev',nlev, LEVID)
   if (status .NE. nf_noerr) call handerr(status)

   status=nf_def_dim(ncid, 'NTP',num_soil_layers, NSOILID)
   if (status .NE. nf_noerr) call handerr(status)

   status=nf_def_dim(ncid, 'LUC',luc, NLUCID)
   if (status .NE. nf_noerr) call handerr(status)

   status=nf_def_dim(ncid, 'NSL',nsoil, NSLID)
   if (status .NE. nf_noerr) call handerr(status)

   status=nf_def_dim(ncid, 'NCY',nclay, NCYID)
   if (status .NE. nf_noerr) call handerr(status)

   status=nf_def_dim(ncid, 'NSD',nsand, NSDID)
   if (status .NE. nf_noerr) call handerr(status)

   status=nf_def_dim(ncid, 'time',nf_unlimited, TIMEID)
   if (status .NE. nf_noerr) call handerr(status)

   write(*,*) 'define the dimentions lon,lat,lev,   &
   &      and time, the ID is => ',LONID,LATID,LEVID,TIMEID

!... Put the attribute of your variables
      name = 'longitude'
      unit = 'degrees_east'
      call wrap_def_var (ncid, 'lon',nf_real, 1,LONID, varid, &
     &                   name, unit)
!
      name = 'latitude'
      unit = 'degrees_north'
      call wrap_def_var (ncid, 'lat',nf_real, 1,LATID, varid, &
     &                   name, unit)
!
      name = 'height'
      unit = 'm'
      call wrap_def_var (ncid, 'lev',nf_real, 1,LEVID, varid, &
     &                   name, unit)
      status = nf_put_att_text (ncid,varid,'positive',4,'down')
!
      name = 'aerosol flux types'
      unit = '-'
      call wrap_def_var (ncid, 'NTP',nf_real, 1,NSOILID, varid, &
     &                   name, unit)
!
      name = 'surface types for BATS'
      unit = '-'
      call wrap_def_var (ncid, 'LUC',nf_real, 1,NLUCID, varid, &
     &                   name, unit)
!
      name = 'time'
      unit = 'days since 0001-01-01 00:00:00'
      write(unit(12:15),'(i4.4)') mdate(1)
      write(unit(17:18),'(i2.2)') mdate(2)
      write(unit(20:21),'(i2.2)') mdate(3)
      write(unit(23:24),'(i2.2)') mdate(4)
      call wrap_def_var (ncid, 'time',nf_double, 1,TIMEID, varid, &
     &                   name, unit)
!
      status=nf_put_att_text(ncid, nf_global, 'long_name', &
      30, 'BCC_AGCM aerosol emission data')
      status=nf_put_att_text(ncid, nf_global, 'host', &
      7, 'wangzzh')
      status=nf_put_att_text(ncid, nf_global, 'source', &
      56, 'transfered from the unformatted data: geia_emission.ccrn')
      status=nf_put_att_text(ncid, nf_global, 'title', &
      9, 'BCC_AGCM2')
      if (status .NE. nf_noerr) call handerr(status)
!
      name = 'mid-season date as 8 digit integer (YYYYMMDD)'
      unit = '-'
      call wrap_def_var (ncid, 'date',nf_int, 1,TIMEID, varid, &
     &                   name, unit)
!
! define atm 3-d data first
      dim4_id(1) = lonid
      dim4_id(2) = latid
      dim4_id(3) = nlucid
      dim4_id(4) = timeid
!
      name = 'fraction of surface types'
      unit = '-'
      call wrap_def_var (ncid, 'FLUC',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      dim4_id(3) = nslid
      name = 'soil types'
      unit = '-'
      call wrap_def_var (ncid, 'SAND2',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      dim4_id(3) = ncyid
      name = 'clay types'
      unit = '-'
      call wrap_def_var (ncid, 'CLAY2',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      dim4_id(3) = nsdid
      name = 'sand classification'
      unit = '-'
      call wrap_def_var (ncid, 'CHDS6',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
! define atm 3-d data
      dim4_id(1) = lonid
      dim4_id(2) = latid
      dim4_id(3) = LEVID
      dim4_id(4) = timeid
!
!     name = 'NH3 monthly flux' 
!     unit = '-'
!     call wrap_def_var (ncid, 'NH3',nf_real, 4,dim4_id, varid, &
!    &                   name, unit)
!
!     name = 'NH4 monthly flux' 
!     unit = '-'
!     call wrap_def_var (ncid, 'NH4',nf_real, 4,dim4_id, varid, &
!    &                   name, unit)
!
! define lnd 3-d data
      dim4_id(1) = lonid
      dim4_id(2) = latid
      dim4_id(3) = NSOILID
      dim4_id(4) = timeid
!
      name = 'BC monthly flux' 
      unit = '-'
      call wrap_def_var (ncid, 'BCFLX',nf_real, 4,dim4_id, varid, &
     &                   name, unit)
!
      name = 'BC INJECTION HEIGHTS'
      unit = '-'
      call wrap_def_var (ncid, 'BCHIT',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'OM monthly flux' 
      unit = '-'
      call wrap_def_var (ncid, 'OCFLX',nf_real, 4,dim4_id, varid, &
     &                   name, unit)
!
      name = 'OM INJECTION HEIGHTS'
      unit = '-'
      call wrap_def_var (ncid, 'OCHIT',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
! define lnd 2-d data
      dim4_id(1) = lonid
      dim4_id(2) = latid
      dim4_id(3) = timeid
!
      name = 'OD monthly flux in 1998'
      unit = '-'
      call wrap_def_var (ncid, 'ODFLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'H2S seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'H2SFLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'LDM seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'LDMFLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'PSO2 seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'PSO2FLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'ASO2 seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'ASO2FLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'PSO4 seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'PSO4FLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'ASO4 seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'ASO4FLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'PNO2 seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'PNO2FLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'ANO2 seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'ANO2FLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'PNO seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'PNOFLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
      name = 'ANO seasonly flux in 1985'
      unit = '-'
      call wrap_def_var (ncid, 'ANOFLX',nf_real, 3,dim4_id, varid, &
     &                   name, unit)
!
!... end of define mode (this function is must be used)
      status=nf_enddef(ncid)
      if (status .NE. nf_noerr) call handerr(status)
      print *, 'end_define'
!
      do k = 1, 12
      status = nf_inq_varid (ncid, 'time',varid)
      status = nf_put_vara_double (ncid, varid, k, 1 , dble(k*30-15) )
!
      status = nf_inq_varid (ncid, 'date',varid)
      status = nf_put_vara_int (ncid, varid, k, 1 , date(k) )
      enddo
   status = nf_inq_varid (ncid, 'lon', varid)
   status = nf_put_var_real (ncid, varid, alon)
   status = nf_inq_varid (ncid, 'lat', varid)
   status = nf_put_var_real (ncid, varid, alat)

! now write the data
   start2w = 1
   count2w = 1
!
   open(20, file='ZHLAND.DAT', status='old')
   allocate (ain(nlon, nlat, luc) )
   do i = 2, nlon
      do j = 1, nlat
      read(20,'(2f11.3,15f8.4)') alon(i), alat(j), (ain(i,j,k),k=1,luc)
      enddo
   write(*,*)'lon=', alon(i)
   enddo
      i = 1
      do j = 1, nlat
      read(20,'(2f11.3,15f8.4)') alon(i), alat(j), (ain(i,j,k),k=1,luc)
      enddo
   write(*,*)'lon=', alon(i)
   close(20)
!
   status = nf_inq_varid (ncid, 'FLUC',varid)
   status = nf_put_var_real (ncid, varid, ain(:, 1:nlat, :))
   if (status.ne.NF_NOERR) call handerr (status)
   deallocate ( ain )

   filname = 'chem.dat'
   fid = 10
   open(fid, file=trim(filname), status='old', form='unformatted', &
         action='read')
!
   allocate (ain2d(nlon+1,nlat))              ! atm 3-d data
   count2w(1)=nlon; count2w(2)=nlat
   do k = 1, 12
   read(fid)ain2d
      status = nf_inq_varid (ncid, 'ODFLX',varid)
      start2w(3)=k
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))
   enddo
   deallocate (ain2d)

   allocate (ain(nlon+1,nlat,5) )
   start2w = 1
   count2w(1)=nlon; count2w(2)=nlat; count2w(3) = 5; count2w(4) = 1
   do k = 1, 12
   do j = 1, 5
   read(fid)ain(:,:,j)
   enddo
      status = nf_inq_varid (ncid, 'BCFLX',varid)
      start2w(4)=k
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain(1:nlon, :,:))
   enddo
   do j = 1, 5
   read(fid)ain(:,:,j)
   enddo
   status = nf_inq_varid (ncid, 'BCHIT',varid)
   status = nf_put_var_real (ncid, varid, ain(1:nlon, 1:nlat, :))
!
   do k = 1, 12
   do j = 1, 5
   read(fid)ain(:,:,j)
   enddo
      status = nf_inq_varid (ncid, 'OCFLX',varid)
      start2w(4)=k
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain(1:nlon, :,:))
   enddo
   do j = 1, 5
   read(fid)ain(:,:,j)
   enddo
   status = nf_inq_varid (ncid, 'OCHIT',varid)
   status = nf_put_var_real (ncid, varid, ain(1:nlon, 1:nlat, :))
   deallocate (ain)
!
   allocate (ain2d(nlon+1,nlat))              ! atm 3-d data
   start2w = 1
   count2w(1)=nlon; count2w(2)=nlat; count2w(3) = 1; count2w(4) = 1
!
   do k = 1, 12, 3
      start2w(3)=k

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'H2SFLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'LDMFLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'PSO2FLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'ASO2FLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'PSO4FLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'ASO4FLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'PNO2FLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'ANO2FLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'PNOFLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))

      read(fid)ain2d
      status = nf_inq_varid (ncid, 'ANOFLX',varid)
      call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain2d(1:nlon, :))
   enddo
   deallocate (ain2d)
!
   allocate (ain(nlon+1,nlat,32) )
   start2w = 1
   count2w(1)=nlon; count2w(2)=nlat; count2w(3) = 32; count2w(4) = 1
   do k = 1, 12
      start2w(4)=k
      do j = 1, 32
      read(fid)ain(:,:,j)
      enddo
!     status = nf_inq_varid (ncid, 'NH3',varid)
!     call wrap_put_vara_gcm (ncid, varid, start2w, count2w, ain(1:nlon, :,:))
   enddo
   deallocate (ain)
!
   allocate (ain(nlon+1,nlat,12) )
   do j = 1, 12
   read(fid)ain(:,:,j)
   enddo
   status = nf_inq_varid (ncid, 'SAND2',varid)
   status = nf_put_var_real (ncid, varid, ain(1:nlon, 1:nlat, :))
   deallocate (ain)
!
   allocate (ain(nlon+1,nlat,2) )
   do j = 1, 2
   read(fid)ain(:,:,j)
   enddo
   status = nf_inq_varid (ncid, 'CLAY2',varid)
   status = nf_put_var_real (ncid, varid, ain(1:nlon, 1:nlat, :))
   deallocate (ain)
!
   allocate (ain(nlon+1,nlat,6) )
   do j = 1, 6
   read(fid)ain(:,:,j)
   enddo
   status = nf_inq_varid (ncid, 'CHDS6',varid)
   status = nf_put_var_real (ncid, varid, ain(1:nlon, 1:nlat, :))
   deallocate (ain)
!
   close(fid)
!... close the nc file..
   status=nf_close(ncid)
   if (status .NE. nf_noerr) call handerr(status)
   stop
999 continue
!
   end
!
      subroutine wrap_def_var (nfid, var, dtype, ndims, dimid, &
     &                         varid, name, unit)
     implicit none
#include <netcdf.inc>

      integer nfid, dtype, ndims
      integer dimid(*)
      character*(*) var, name, unit
      integer varid
      integer ret, lenth
!      integer lenstring
!
      ret = nf_def_var (nfid, var, dtype, ndims, dimid, varid)
      if (ret.ne.NF_NOERR) call handerr (ret)
!
      lenth = len_trim(name)
      if ( lenth.gt.0 ) then
         ret = nf_put_att_text (nfid,varid,'long_name', lenth, name)
         if (ret.ne.NF_NOERR) call handerr (ret)
      endif
      lenth = len_trim(unit)
      if ( lenth.gt.0 ) then
         ret = nf_put_att_text (nfid,varid,'units' ,lenth, unit)
         if (ret.ne.NF_NOERR) call handerr (ret)
      endif
!
      end
!
      subroutine wrap_put_vara_gcm (nfid, varid, start,count,arr)
      implicit none
#include <netcdf.inc>

      integer nfid, varid
      integer start(*), count(*)
      real arr(*)

      integer ret
!#if (REALTYP == 0 )
      ret = nf_put_vara_real (nfid, varid,start,count, arr)
!#endif
!#if (REALTYP == 1 )
!      ret = nf_put_vara_double (nfid, varid, start,count,arr)
!#endif
      if (ret.ne.NF_NOERR) call handerr (ret)
      end
!

!... Make the error control program!
        subroutine handerr(status)

        implicit none
!... include the usefully NetCDF include file
#include <netcdf.inc>

        integer status

        if (status .NE. nf_noerr) then
        write(*,*) NF_STRERROR(status), status
        stop 'program stoped, and you must find   &
     & the error in your source code.'
        endif
        end

