      SUBROUTINE THCAL3B(TSH,TSHB, TG,GT, SH,SHT,SHXKJ, & 
     &                   ILEV,LEV, ILG,IS,IF,IRAD)
!
!     * JUL 10/03 - M.LAZARE. TASKLOCAL->THREADPRIVATE (NEW VERSION BASED
!     *                       ON THCAL3).
!     * MAY 20/97 - M.LAZARE. PREVIOUS VERSION THCAL3:
!     *                       "*PDIR TASKLOCAL" MULTITASKING DIRECTIVE
!     *                       ADDED FOR COMMON BLOCK "LWCONS" WHICH
!     *                       HAS SOME ELEMENTS NOT INITIALIZED BY DATA
!     *                       STATEMENTS.    
!     * FEB 19/91 - M.LAZARE. - LOWEST INTERFACE OF TSHB SET TO MINIMUM 
!     *                         OF GT,TSROW IN ORDER TO "MODEL" SUPER-
!     *                         ADIABATIC LAPSE RATE PROFILE WITH JUMP
!     *                         DISCONTINUITY UNDER UNSTABLE CONDITIONS.
!     *                       - PCARDI REMOVED FROM "LWCONS" COMMON 
!     *                         BLOCK (NOT USED IN NEWER LONGWAVE). 
  
!     * AUG 28/89 - M.LAZARE. - LIKE PREVIOUS VERSION THCAL2 EXCEPT:  
!     *                       A. LWCONS COMMON BLOCK MODIFIED TO BE 
!     *                          CONSISTENT WITH OTHER LOCATIONS IN 
!     *                          CODE.
!     *                       B. PASSED "IRAD" MODEL OPTION PARAMETER 
!     *                          USED TO DECIDE MOON LAYER TEMPERATURE
!     *                          CALCULATION (INSTEAD OF PREVIOUS 
!     *                          PASSED LOGICAL SWITCH "OLDRAD"). 
!     * SEP 23/88 - M.LAZARE. - PREVIOUS MODEL VERSION GCM6.
! 
!     * SET TEMPERATURE ARRAYS FOR PHYSICS ON THERMODYNAMICS LEVELS 
!     * FOR HYBRID VERSION OF MODEL.
!     * SHB ARE THERMODYNAMICS LAYER INTERFACES,
!     * SH  ARE THERMODYNAMICS MID LAYER POSITIONS. 
!     * LEV = ILEV+1. 
!     * 
!     *        THERMODYNAMICS 
!     *     INTERFACE   MID LAYER  TG(MODEL) TSH(TH)     TSHB(TF) 
!     * 
!     *        SIGMA=0. ////////////////////////////////////////////////
!     *                  MOON LAYER          TSH(1) 
!     * SHT(1)=THERMO.TOP ------------------------------ TSHB(1)
!     *                  SH(1)     TG(1)     TSH(2) 
!     * SHT(2)=SHB(1) ---------------------------------- TSHB(2)
!     *                  SH(2)     TG(2)     TSH(3) 
!     * SHT(3)=SHB(2) ---------------------------------- TSHB(3)
!     *                  SH(3)     TG(3)     TSH(4) 
!     * SHT(4)=SHB(3) ---------------------------------- TSHB(4)
!     *                  SH(4)     TG(4)     TSH(5) 
!     * SHT(5)=SHB(4) ---------------------------------- TSHB(5)
!     *                  SH(5)     TG(5)     TSH(6) 
!     * SHT(6)=SHB(5)=1. /////////////////////////////// TSHB(6)
  
      IMPLICIT REAL (A-H,O-Z),  &
     &INTEGER (I-N)

      REAL TG  (ILG,ILEV),   GT (ILG) 
      REAL TSH (ILG, LEV), TSHB (ILG, LEV)
      REAL  SH (ILG,ILEV),  SHT (ILG, LEV), SHXKJ(ILG,ILEV) 

      COMMON /LWCONS/ MAXC,CO2,SIG1,CDAY,X10E,RGRAV 
!$OMP THREADPRIVATE (/LWCONS/)  
!-----------------------------------------------------------------------
!     * MID LAYER TEMPERATURES, WITH EXTRA ONE AT TOP.
! 
      DO 100 L=1,ILEV 
      DO 100 I=IS,IF
      TSH(I,L+1) = TG(I,L)
  100 CONTINUE
! 
!     * LAYER INTERFACE TEMPERATURES, WITH EXTRA ONE AT TOP.
!     * TEMPERATURE DISCONTINUITY AT THE GROUND BASED ON ADIABATIC
!     * EXTRAPOLATION TO SURFACE UNDER UNSTABLE CONDITIONS ONLY, I.E. 
!     * FOR THESE CASES MOST OF THE SUPERADIABATIC LAPSE RATE IS
!     * CONTAINED VERY CLOSE TO THE SURFACE.
! 
      DO 200 I=IS,IF
  200 TSHB(I,LEV)=MIN( TSH(I,LEV)/SHXKJ(I,ILEV), GT(I) )
! 
      DO 300 L=1,ILEV-1 
      DO 300 I=IS,IF
      TSHB(I,L+1) = (TSH(I,L+1)*LOG(SH (I,L+1)/SHT(I,L+1)) & 
     &              +TSH(I,L+2)*LOG(SHT(I,L+1)/SH (I  ,L)))&
     &                         /LOG(SH (I,L+1)/SH (I  ,L)) 
  300 CONTINUE
! 
!     * MOON LAYER TEMPERATURE. 
!     * USE SIG1 FROM GCM6 TO DETERMINE CONDITION TO APPLY, IN ORDER TO 
!     * AVOID LONGITUDINAL GRADIENTS. 
!     * THE CALCULATION IS THE SAME AS PREVIOUS MODEL VERSIONS FOR LOW
!     * MODEL LIDS OR THE OLD RADIATION SCHEME. 
!     * FOR THE NEW RADATION, INTERPOLATION IN "SIGMA" (I.E. MASS) IS 
!     * DONE UP TO THE MODEL TOP (AS DEFINE BY THE DYNAMICS) TO PRODUCE 
!     * TSHB(1) AND A SIMILAR INTERPOLATION IS PERFORMED TO CALCULATE 
!     * THE MOON LAYER TEMPERATURE, WHICH IS ASSUMED TO LIE AT THE
!     * HALF-MASS POINT ABOVE IT. 
!     * HOWEVER, IF THE MODEL TOP LIES AT OR ABOVE THE STRATOPAUSE
!     * (P=1 MB) THEN AN ISOTHERMAL TEMPERATURE STRUCTURE ABOVE THE 
!     * MODEL TOP IS ASSUMED TO EXIST.
! 
      IF(SIG1.GT.0.020) THEN
         DO 400 I=IS,IF 
            TSHB(I,1)=TSH(I,2)
             TSH(I,1)=TSH(I,2)
  400    CONTINUE 
      ELSE
         IF(IRAD.EQ.0) THEN 
            DO 450 I=IS,IF
               TSHB(I,1)=TSH(I,2) 
                TSH(I,1)=MAX(220.,TSH(I,2)+20.) 
  450       CONTINUE
         ELSE 
            DO 500 I=IS,IF
               SLOPE=(TSH(I,2)-TSHB(I,2))/LOG(SH(I,1)/SHT(I,2))
               TSHB(I,1)=TSH(I,2)+SLOPE*LOG(SHT(I,1)/SH(I,1))
               TSH(I,1)=TSHB(I,1) 
               IF(SIG1.GT.0.001) THEN 
                  TSH(I,1)=TSH(I,2)+20. 
               ENDIF
  500       CONTINUE
         ENDIF
      ENDIF 
! 
      RETURN
!-----------------------------------------------------------------------
      END 
