      SUBROUTINE AEROACT (   ILEV,   ILG,    IL1,      IL2,    IIWC, &
     &                      RCOEX,RADCLD, TOTNUM,   RHSIZE,    ILWC, &
     &                        NTR, ISIZE,   RHOP,     ICDN,    WROW, &
     &                       XROW,  WSUB,  RCRIT,     NSUB, AERONUM, &
     &                      ZMLWC,ROAROW, CLSIZE,     CAMR,    MAE)
!-----------------------------------------------------------------------
!
!     HISTORY:
!     --------
!     * JUN 14/2001 - S.L. GONG   THE AEROSOL NUMBER SIZE RANGE WAS 
!     *                           IMPOSED TO CALCULATE THE CLOUD
!     *                           DROPLET NUMBER. 
!     *
!     * DEC 02/1998 - S.L. GONG   THE CLOUD NUMBER DENSITY IS NOW 
!     *                           COMPUTED FROM JONES ET AL 1994
!     *                           PARAMETERIZATION WHICH RELATES THE
!     *                           TOTAL AEROSOL NUMBER TO CLOUD DROPLET
!     *                           NUMBER.
!     *
!     * JAN 12/1998 - S.L. GONG   A SIMPLIFIED VERSION OF GHAN 1993
!     *                           ACTIVATION PARAMETERIZATION SCHEME
!     *                           WITH CONSTANT d PARAMETER.
!     *
!     *
!     METHOD:
!     -------
!
!     VARIABLES:
!     ----------
!
!-----------------------------------------------------------------------
      REAL RADCLD (ILG,ILEV), TOTNUM(ILG,ILEV,ISIZE)
      REAL RHSIZE (ILG,ILEV,ISIZE), RHOP(ILG, ILEV,ISIZE)
      REAL CLSIZE (ILG,ILEV,ISIZE)
      REAL XROW (ILG,ILEV+1,NTR),WSUB(ILG,ILEV,NSUB), RCRIT(ILG,ILEV)
      REAL RCOEX (ILG,ILEV), WROW(ILG,ILEV), ROAROW(ILG,ILEV)
      REAL AERONUM(ILG,ILEV,ISIZE), ZMLWC(ILG,ILEV,2)
      real CAMR
!
!     ACCUMULATE NUMBER OF AEROSOLS[/M^3] FOR EACH GRID
!     THE STARTING SIZE IS r=1.5 um.Bin = 9
!
!  wzz, 2008.10.10, ???
      CALL PUTZERO(TOTNUM, ILG*ILEV*ISIZE)
      CALL PUTZERO(RCOEX, ILG*ILEV)
      DO N=9,1,-1
         DO N1=N,9
            DO L=1+MAE,ILEV
               DO I=IL1,IL2
                  TOTNUM(I,L,N) = TOTNUM(I,L,N)+AERONUM(I,L,N1)  &
     &                                               *ROAROW(I,L)
                  RCRIT(I,L)=FLOAT(ISIZE+1)
                  IF ((CAMR .GT. 1.0 .AND. CAMR .LT. 10.) .OR.  &
     &                (CAMR .GT. 10. .AND. CAMR .LT. 90.))   &
     &                XROW(I,L+1,ICDN)=0.0
               END DO
            END DO
         END DO
      END DO
!
!     COMPUTE THE CLOUD DROPLET NUMBER [1/M^3 AIR]
!     FOR THE CURRENT VERSION, d [M^4 S^-1] IS ASSUMED CONSTANT
!     --- A SUBROUTINE WILL BE INSERTED HERE LATER TO COMPUTE
!         d AS A FUNCTION OF AEROSOL SIZE DISTRIBUTION,
!         COMPOSITION AND OTHER PAPAMETERS.
!
      d = 3.4E-11
      CUB=1./3.
      IF (CAMR .EQ. 1.0 .OR. CAMR .EQ. 10.0) GOTO 100
      DO J=1,NSUB
         DO L=1+MAE,ILEV
            DO I=IL1,IL2
               WTOT=WSUB(I,L,J)+WROW(I,L)
               IF (WTOT .LT. 0.0) THEN
                  XROW(I,L+1,ICDN)=XROW(I,L+1,ICDN)+TOTNUM(I,L,1)/NSUB*  &
     &                            ABS(WTOT)/(ABS(WTOT)+ d*TOTNUM(I,L,1))
               END IF
            END DO
         END DO
      END DO
      GOTO 200
!
!     FIND THE ACTIVATED BIN NUMBER
!
!        DEFINE: RCRIT = BIN NUMBER + % UN-ACTIVED
!        E.G.    RCRIT = 4.3 MEANS BIN 4 AND OVER ARE ACTIVATED WITH
!                        30% UN-ACTIVATED IN BIN 4.
!
  100 CONTINUE
!
!    CLOUD DROPLET NUMBER DENSITY [1/M3] FROM JONES ET AL 1994
!    AEROSOL SIZE RANGE ARE IN r=0.05-1.5 um
!    For 12 size bin configuration, Bin 4 is the cut.
!
       DO L=1+MAE,ILEV
          DO I=IL1,IL2
             TONA=TOTNUM(I,L,4)*1.E-6
             XROW(I,L+1,ICDN)=375.E6*(1.-EXP(-2.5E-3*TONA))
          END DO
       END DO
  200 CONTINUE
      DO N=ISIZE,2, -1
         DO L=1+MAE,ILEV
            DO I=IL1,IL2
               IF (XROW(I,L+1,ICDN) .GT. TOTNUM(I,L,N) .AND.  &
     &             XROW(I,L+1,ICDN) .LE. TOTNUM(I,L,N-1))        THEN
                  RLEFT=(TOTNUM(I,L,N-1)-XROW(I,L+1,ICDN))/  &
     &                          (AERONUM(I,L,N-1)*ROAROW(I,L))
                  RCRIT(I,L)=FLOAT(N-1)+AMIN1(1.0,RLEFT)
               END IF
               IF (XROW(I,L+1,ICDN) .GT. TOTNUM(I,L,1))  &
     &                                     RCRIT(I,L)=1.0
            END DO
         END DO
      END DO
!
!     DISTRIBUTE CLOUD LIQUID WATER [KG/KG] INTO ACTIVATED BINS
!     ACCORDING TO THE CLOUD NUMBER DENSITY IN EACH ACTIVATED BIN
!     AND ESTIMATE CLOUD DROPLET RADIUS.
!
!     CLOUD DROPLET RADIUS AT THE INTERRUPTED BIN WHERE AEROSOL
!     AND CLOUD DROPLET CO-EXIST IS SAVED IN RCOEX(I,L).
!     FOR FULLY ACTVIATED BINS, THE RHSIZE IS UPDATED.
!
      RHOPW=1000.00
      DO N=1,ISIZE
         DO L=1+MAE,ILEV
            DO I=IL1,IL2
                IF (N .GE. INT(RCRIT(I,L))  &
     &                    .AND. XROW(I,L+1,ICDN) .GT. 0.0 .AND.  &
     &                    ZMLWC(I,L,1)+ZMLWC(I,L,2) .GT. 0.0)  THEN
                   H2O=(ZMLWC(I,L,1)+ZMLWC(I,L,2))/  &
     &                                (XROW(I,L+1,ICDN)/ROAROW(I,L))
                   RWI=RHSIZE(I,L,N)
                   IF (N .EQ. INT(RCRIT(I,L))) THEN
                      RCOEX(I,L)=RHSIZE(I,L,N)
                      CLSIZE(I,L,N)=RHSIZE(I,L,N)
                   ELSE
                      CLSIZE(I,L,N)=((RWI*RWI*RWI*RHOP(I,L,N)+  &
     &                                       H2O/4.189)/RHOPW)**CUB
                   END IF
                END IF
            END DO
         END DO
      END DO
      DO L=1+MAE,ILEV
         DO I=IL1,IL2
             IF (XROW(I,L+1,ICDN) .GT. 0.0 .AND.  &
     &                ZMLWC(I,L,1)+ZMLWC(I,L,2) .GT. 0.0) THEN
                H2O=(ZMLWC(I,L,1)+ZMLWC(I,L,2))/  &
     &                                (XROW(I,L+1,ICDN)/ROAROW(I,L))
                RADCLD(I,L) = ((H2O/4.189)/RHOPW)**CUB*1.0E6
             END IF
         END DO
      END DO
      RETURN
      END
