      SUBROUTINE SULFATE (   LEV,     ILEV,      ILG,      IL1,    IL2,  &
     &                      IH2S,     IDMS,     ISO2,     ISO4,  RGRID,  & 
     &                   AERONUM,     XROW,      NTR,    ICOMP, ROAROW,  &
     &                    PRESSG,        T,       RH,      SHJ,  TOTNU,  &
     &                     OHROW,   NO3ROW,       NN,    KOUNT,  TOTCD,  &
     &                    RHSIZE, AEROSIZE,  TCSZROW,   CSZROW,   JLAT,  &
     &                     ISIZE,     DELT,   RTNUCL,    RHOP0,    LON,  &
     &                    RTCOND,    RTSO2,    RTDMS,    RTH2S, TOTCDN,  &
     &                     CLDCV,   ACOEFC,   TOTMAS,      RTH,   IAE1,  &
     &                      PVOL,     TMIN,    PCOND,     CAMR,    MAE,  &
     &                    CONDNU)                                        !SLG
!------------------------------------------------------------------------------
!     PURPOSE: 
!     --------
!     SULFATE AEROSOL FORMATION ROUTINE
!       (1) CALCULATE OH CONCENTRATION
!       (2) OXIDATION OF SO2 BY OH
!       (3) NUCLEATION, I.E. GENERATION OF NEW SULPHATE AEROSOLS
!
!     HISTORY:                                                                  
!     -------- 
!
!     * NOV 03/01 - S.L. GONG  THE MASS BALANCE OF SULPHUR WAS DONE TO
!     *                        RE-DISTRIBUTE THE NUCELATION AND CONDENSATION
!     *                        TENDENCIES. SO2 CONCENTRATION AND PRODUCTION
!     *                        RATES ARE NOW COMPUTED EVERY SUB TIME STEP.
!     *
!     * MAY 09/00 - S.L. GONG  COMBINE H2SO4 PRODUCTION, CONDENSATION
!     *                        AND NUCLEATION INTO AN APPROXIMATE
!     *                        SOLUTION WITH A NEW STATEMENT FUNCTION. 
!     *                                                            
!     * DEC 15/98 - S.L. GONG  NEW NUCLEATION SCHEME OF KULMALA [1998]
!     *         K, VON SALZEN  WAS INTRODUCED INTO THE COMBINED ANALYTICAL
!     *                        EQUATION.
!     *
!     * June 4/98 - S.L. GONG  COMBINE NUCLEATION AND CONDENSATION RATES
!     *                        INTO ONE DIFFERENTIAL EQUATION AND
!     *                        SOLVED ANALYTICALLY.
!     *                                                                            
!     * MAY 26/97 - S.L. GONG  VECTORIZATION AND ADD CHEMISTRY           
!     *                        CHEMICAL FIELDS OF OH, H2O2, O3 AND 
!     *                        NO3 FROM NCAR IMAGES MODEL ARE USED [DAILY
!     *                        AVERAGED]
!     *
!     * AUG 14/96 - S.L. GONG  FIRST VERSION
!
!     SUBROUTINE CALLED FROM CAM_V01
!  -----------------------------------------------------------------
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES
      COMMON /PARAMS/ RGASV,CPRESV                             
      COMMON /NARCM/  AVO, RGASi, AIRMW, BOLTZK,  PI,  AR
      COMMON /NARCM/  A0,RSN0,RCUT0,RCG0,P

      COMMON /PARAM1/ PAI, RVORD, TFREZ,     HS,  HV, DAYLNT

      integer IH2S, IDMS, ISO2, ISO4
      integer NN
!
!     FOR FLOATING-1 MACHINE.

      REAL*8 EFACT, AFAK, B, D,EXPPP
      REAL*8 ACOEFC(ILG,ILEV,2)

!      REAL ACOEFC(ILG,ILEV,2)
      real CAMR

      REAL TCSZROW(ILG),CSZROW(ILG), TOTNU(ILG), TOTCD(ILG,ILEV)
      REAL RH(ILG,LEV), TOTCDN(ILG,ILEV),PCOND(ILG,ILEV,ISIZE) 
      REAL T(ILG,LEV),SHJ(ILG,ILEV), PRESSG(ILG)
      REAL OHROW(ILG, ILEV), NO3ROW(ILG,ILEV),ROAROW(ILG,ILEV)
      REAL RGRID(ILG, ILEV, NTR),CLDCV(ILG,ILEV,2)                     !SLG
      REAL RTSO2(ILG, ILEV,2),RTDMS(ILG, ILEV), RTH(ILG,ILEV,ISIZE)   
      REAL RTH2S(ILG, ILEV), RTNUCL(ILG, ILEV),CONDNU(15)
      REAL XROW (ILG, LEV, NTR)
      REAL RHSIZE(ILG,ILEV,ISIZE), AEROSIZE(2,ISIZE)
      REAL KN, RTCOND(ILG, ILEV,ISIZE),TOTMAS(ILG,ILEV,ISIZE)
      REAL AERONUM(ILG,ILEV,ISIZE), RHOP0(ICOMP), PVOL(ISIZE)
!
      real ACE, XIAO
      DATA ACE/0.02/, XIAO/1.0E3/
!
      real Y4PID3, SO4MW, AKB, ARA
      PARAMETER (Y4PID3 = 4.18879)
      PARAMETER (SO4MW = 98.1E-03)
      PARAMETER (AKB = 1.380662E-23)
      PARAMETER (ARA = 1.)
!
!     STATEMENT FUNCTION OT COMPUTE SO4 CONCENTRATION DUE TO COMBINED
!     NUCLEATION AND CONDENSATION PROCESSES AS A FUNCTION OF TIME.
!      
      OMEGA(A,B,C,D)=A+B*C**(D-1.)
      EXPP(A,B,C,D,T0)=EXP(-OMEGA(A,B,C,D)*T0)
      SO4(A,B,C,D,PP,T0)=((1.-EXPP(A,B,C,D,T0))*PP+EXPP(A,B,C,D,T0)*  &
     &                     OMEGA(A,B,C,D)*C)/OMEGA(A,B,C,D)
!
!     STATEMENT FUNCTION FOR ATOMIC DIFFUSION VOLUME [CM3]
!     MAKAR ET AL, 1998
!
      ADV(W)=0.369*W+6.29
!
!     ZERO WORKING ARRAYS
!
      CALL PUTZERO(TOTCD, ILG*ILEV)
      CALL PUTZERO(TOTCDN,ILG*ILEV)
      CALL PUTZERO(RTNUCL,ILG*ILEV)
      CALL PUTZERO(PCOND, ILG*ILEV*ISIZE)
      CALL PUTZERO(RTCOND,ILG*ILEV*ISIZE)

      ADT2=2.*DELT
      CUB=1./3.
      ZC=0.000
!
      DO L = 1+MAE, ILEV
         DO I=IL1,IL2
!           print *, '0 XROW(I,L+1,ISO2)=',XROW(I,L+1,ISO2),I,L+1
           IF (CAMR .LT. 10.0)   THEN     !FOR GCM OR RCM
!===============================================================
!
!       * CLEAR AIR CHEMISTRY
!
!          RK  - RATE FOR SO2+OH
!          RK1 - DMS+OH VIA H ABSTRACTION
!          RK2 - DMS+OH VIA OH ADDITION
!          RK3 - DMS+NO3
!          RK4 - H2S+OH
!
!===============================================================
!
!      TOTAL CLEAR SKY FRACTION
!
            TCLR=1.-AMIN1(1.0,AMAX1(CLDCV(I,L,1),CLDCV(I,L,2)))        !SLG
!
!     * A FACTOR TO CONVERT DAILY AVERAGE FILED TO DIUNAL CHANGED
!
!           TCSZROW(I)=445.   ! wzz, 2008.10.16
!             TCSZROW(I)=700
!           DICH=MAX(CSZROW(I),ZC)/TCSZROW(I)*DAYLNT/DELT
            DICH=MAX(CSZROW(I),ZC)/TCSZROW(I)
!           ADICH=(1.0-MAX(CSZROW(I),ZC))/((DAYLNT/DELT)-TCSZROW(I))
!    1                           *DAYLNT/DELT
            ADICH=(1.0-MAX(CSZROW(I),ZC))/(1.0-TCSZROW(I))
!           if(i.eq.10.and.l.eq.ILEV-5) 
!    *      write(0,*)'wzz: cszrow=',CSZROW(I),ZC,TCSZROW(I),
!    +         DAYLNT,DELT, DICH,ADICH, I,L
!
!     * FOR SO2. THE REACTION RATE CONSTANT DeMore et al. [1992]
! 
!       (1) SO2 + OH --->
!           dC[SO2]/dt = -k C[OH] C[SO2]
!           C[SO2,t] = C[SO2,0]*exp(-k C[OH] t) 
!
            RK0 = 3.0E-31*(300.0/T(I,L+1))**3.3
            RKINF = 1.5E-12
            ROCON = AVO/AIRMW*1.0E-3          ! AIR KG/M3 ->MOL/CM3
            RK0 = RK0*ROAROW(I,L)*ROCON
            RK = (RK0/(1.0+RK0/RKINF))*0.6**(1./(1.0+   &
     &                         (LOG10(RK0/RKINF))**2))
!
!     QOH SHOULD BE THE CONCENTRATION OF OH IN THE UNIT OF
!         MOLECULES/CM3
!
!         MOZART OH IS IN THE UNIT OF VOLUME MIXING RATIO [CM3/CM3]
!         TO CONVERT TO MOLEC CM-3
!
!            USE: 273.15/T* P/1.0133E5*1.E-3  / 22.4136 * AVO
!                 --------------+------------   ---+---   ---
!                   TO STD STATE VOL LIT       TO G-MOLE  TO MOLECULES
!    
! ---  tianliang radical is in the unit of ppbv, so the follwoing should change into  
! ---     zhouch 2007.06.29-------------------------------------------------------
            ROCON=AVO*273.15/T(I,L+1)*PRESSG(I)*SHJ(I,L)/1.0133E5  &
     &                  *1.E-3/22.4136
! -- to restore agian to the orignal version  2007.08.20
!            ROCON=AVO*273.15/T(I,L+1)*PRESSG(I)*SHJ(I,L)/1.0133E5
!     1                  *1.E-3/22.4136*1.0e-9

            QOH = OHROW(I,L)*DICH*ROCON
! zch need to be correct in the near future
!            QOH=3.0E+07*DICH/(6.02E20)
!
!       SO2 OXIDATION RATE
!
            RTSO2(I,L,1)=RK*QOH
!
!     * FOR DMS. THE REACTION RATE CONSTANT
! 
!       [1] H ABSTRACTION      Atkinson et al. [1989]
!
!       [2] OH ADDITION   e.g. Pham et al [1995]
!
!       [3] NO3 NIGHT REACTION  Pham et al [1995]
!
!          k =  1.9e-13 exp(500/T) cm**3/(mol * s) 
!
!
            RK1=9.6E-12*EXP(-234./T(I,L+1))
            O2 =ROAROW(I,L)*AVO/AIRMW*1.0E-3*0.21       ! AIR KG/M3 ->MOL/CM3
            RK2=1.7E-42*EXP(7810./T(I,L+1))*O2/   &
     &                   (1.+5.5E-31*EXP(7460./T(I,L+1))*O2)
            RK3=1.9E-13*EXP(500./T(I,L+1))
            QNO3=NO3ROW(I,L)*ADICH*ROCON
             
            RTO=(EXP(-((RK1+RK2)*QOH+RK3*QNO3)*ADT2)-1.)/ADT2
            RTDMS(I,L)=XROW(I,L+1,IDMS)*RTO                            !SLG
!           if(i.eq.14.and.l.eq.ilev) then
!           write(0,*)'wzz: XROW(IDMS=', XROW(I,L+1,IDMS),RTDMS(I,L),ADT2,TCLR
!    &       ,RTO,RK1,RK2,RK3, DICH, ADICH, ROCON, AVO
!           endif
!       if( abs(RTDMS(I,L)).gt.1.0e-8)  print *, XROW(I,L+1,IDMS),RTO
            XROW(I,L+1,IDMS)=AMAX1(TMIN,   &
     &                          XROW(I,L+1,IDMS)+RTDMS(I,L)*ADT2*TCLR)
!      
!     * FOR H2S. THE REACTION RATE CONSTANT
!
            RK4=6.3E-12*EXP(-80./T(I,L+1))
            RTH2S(I,L)=XROW(I,L+1,IH2S)*(EXP(-RK4*QOH*ADT2)-1.0)  &    !SLG
     &                                                   /ADT2         !SLG
            XROW(I,L+1,IH2S)=AMAX1(TMIN,    &
     &                          XROW(I,L+1,IH2S)+RTH2S(I,L)*ADT2*TCLR)

            SO2PRO=-64.6*(RTH2S(I,L)/34.06+RTDMS(I,L)/62.13)*TCLR
!             if(abs(RTH2S(I,L)).gt.0)  print *, RTH2S(I,L),SO2PRO
!             if(SO2PRO.GT.1.e-90) print *, RTDMS(I,L),SO2PRO,TCLR,ADT2
!     +                                  ,RTH2S(I,L)
!           if(i.eq.14.and.l.eq.ilev) then
!             if(SO2PRO.gt.0)write(0,*)'wzz: SO2PRO=',RTDMS(I,L),SO2PRO,TCLR,ADT2
!             write(0,*)'wzz: SO2PRO=',SO2PRO,TCLR,XROW(I,L+1,IH2S),XROW(I,L+1,IDMS)
!           endif
            XROW(I,L+1,ISO2)=AMAX1(TMIN,   &
     &                             XROW(I,L+1,ISO2)+SO2PRO*ADT2)
           END IF
!            print *,'before nuclea'
!           print *, 'XROW(I,L+1,ISO2)=',XROW(I,L+1,ISO2)
!===============================================================
!
!      * NUCLEATION
!
!        SCHEME BY KULMALA ET AL [1998]
!        THIS PORTION OF CODE WAS CONTRIBUTED BY KNUT VON SALZEN
!
!          FOR ALL MODELS - GCM, RCM OR AURAMS
!
            RH01 = MAX(MIN(RH(I,L+1),1.0),0.1)
            ATEMPE = MAX(MIN(T(I,L+1),298.),233.)
!
!---        water vapour concentration in 1/cm^3
            ANW = RH01*1.E-06 * EXP ( 77.34491296 - 7235.42451/ATEMPE   &
     &         - 8.2*LOG (ATEMPE) + 5.7113E-03*ATEMPE )    &
     &          / ( AKB * ATEMPE )
!
            ADELTA = ATEMPE / 273.15
            APARA = 25.1289 - 4890.8/ATEMPE - 2.2479*ADELTA*RH01
            APARB = 7643.4/ATEMPE - 1.9712*ADELTA/RH01
            APARC = -1743.3/ATEMPE
            APARD = 1.2233 - 0.0154*ARA / ( ARA + RH01 )   &
     &            - 0.0415*LOG(ANW) + 0.0016*ATEMPE
            APARE = 0.0102
!
!---        calculation of threshold H2SO4 concentration
            ACONCR = EXP (-14.5125 + 0.1335 * ATEMPE - 10.5462 * RH01  &
     &             + 1958.4 * RH01 / ATEMPE )
!
            AEXP = APARA + APARB * APARE
            ATUNER = 1.
!
!      H2SO4 in kg/kg in combined equation and J in per cm^3/s
!
            EXPPP = AEXP*ALOG(AVO*ROAROW(I,L)/SO4MW*1.E-06) -  &
     &                                         APARA*ALOG(ACONCR)
!           EFACT=DEXP(EXPPP)
            EFACT=EXP(EXPPP)
            AFAK = EXP (APARB * APARD + APARC) *EFACT *ATUNER

            AVESIZE = AEROSIZE(1,1)
!      *  to kg/kg/s
            AFAK = AFAK * Y4PID3 * RHOP0(NN) *AVESIZE*AVESIZE*AVESIZE  &
     &                * 1.E+06 / ROAROW(I,L)                            !SLG
!
!---        nucleation coefficients 
            ACOEFC(I,L,1) = AFAK
            ACOEFC(I,L,2) = AEXP
         END DO
      END DO
!         print *, 'before conden'
!===============================================================
!
!
!      * CONDENSATION OF H2SO4 [G] TO EXSITING PARTICLES.
!        LOOP FOR SIZE SPECTRUM 
!
      DO N=1,ISIZE
         DO L=1+MAE,ILEV
           DO I=IL1,IL2
             IF( AERONUM(I,L,N)*ROAROW(I,L) .GT. XIAO) THEN
!
!   DIFFUSION COEFFICIENT OF H2SO4 [GAS] [PERRY AND GREEN, 1984]
!   [M2 S-1]
!   WHERE 0.21145 = [(Ma+Mg)/MaMg]^1/2, Ma=28.97, Mg=98.1 [H2SO4]
!
               DFSO4=1.E-7*T(I,L+1)**1.75*0.21145/(PRESSG(I)*SHJ(I,L)/  &
     &            1.01325E5*(ADV(AIRMW)**CUB+ADV(SO4MW*1.E3)**CUB)**2)
!
!  THE MEAN FREE PATH OF VAPOUR SULPHURIC ACID
!   
               FRP = 3.0*SQRT(PI*98.1/(8.E3*RGASi*T(I,L+1)))*DFSO4
!
!            ACE --> ACCOMMODATION COEFFICIENT ~0.02
!
               KN = FRP/RHSIZE(I,L,N)
               FN = (1.+KN)/(1.+1.71*KN+1.33*KN*KN)
               AN = 1./(1.+1.33*KN*FN*(1./ACE-1.))
!
!     * THE CONDENSATION COEFFICIENTS FOR COMBINED COMPUTATION NEXT.
!       THE AERONUM*ROAROW ACCOUNTS FOR ALL THE PARTICLES IN BIN N. 
!
               RTCOND(I,L,N)=4.*PI*RHSIZE(I,L,N)*DFSO4*FN*AN*  &
     &                                    AERONUM(I,L,N)*ROAROW(I,L)
               TOTCDN(I,L)=TOTCDN(I,L)+RTCOND(I,L,N)
             END IF
           END DO
         END DO
      END DO
!
!    FRACTIONAL CONDENSATION TO EACH BIN FOR USE IN SOA
!
      DO N=1,ISIZE
         DO L=1+MAE,ILEV
            DO I=IL1,IL2
                COND=TOTCDN(I,L)
                IF (COND .GT. 0. .AND. AERONUM(I,L,N)*ROAROW(I,L).GT.  &
     &                                                     XIAO) THEN
                   PCOND (I,L,N)=AMAX1(AMIN1(RTCOND(I,L,N)/COND,1.),0.)
                END IF 
            END DO
         END DO
      END DO

!
!       USE 10 TIME INTERVALS TO COMPUTE THE TENDENCIES
!
!         print *,'beofre gas to part'
      DT=ADT2*0.10
      D2=DT*0.5
      DO L=1+MAE,ILEV
         DO I=IL1,IL2
             SO4T0=MAX(1.E-18,XROW(I,L+1,ISO4))
             SO2T0=MAX(1.E-18,XROW(I,L+1,ISO2))
!       print *, 'SO4T0 =', XROW(I,L+1,ISO4),XROW(I,L+1,ISO2),ISO4
!          if(TOTCDN(I,L).gt.0.) print *, 'TOTCDN(I,L)=',TOTCDN(I,L),I,L
             IF (SO4T0 .GE. 1.E-18 .AND. TOTCDN(I,L) .GT. 0.   &
     &           .AND. XROW(I,L+1,ISO2) .GE. 1.E-18) THEN
!
!           if(i.eq.14.and.l.eq.ilev) then
!             write(0,*)'wzz: SO2T0=',SO4T0,TOTCDN(I,L),XROW(I,L+1,ISO2)
!           endif
!      TOTAL CLEAR SKY FRACTION
!
               TCLR=1.-AMIN1(1.0,AMAX1(CLDCV(I,L,1),CLDCV(I,L,2))) 
               A=AMAX1(AMIN1(TOTCDN(I,L),0.025),0.0)
               B=ACOEFC(I,L,1)
               D=ACOEFC(I,L,2)

               W=-98.1/64.6
!
!      PP IS THE HSO4(GAS) PRODUCTION RATE FROM SO2 OXIDATION
!      IN THE TIME INTERVAL CONDNU( I)
!      SO2 CONCENTRASTION SO2T0 IS UPDATED IN EVERY CONDNU( I)
!
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU( 1))-1.0)/CONDNU( 1)
               SO2T0=SO2T0+PP*CONDNU( 1)/W
               SO4T1  =MAX(AKB,SO4(A,B,SO4T0, D,PP,CONDNU( 1)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU( 2))-1.0)/CONDNU( 2)
               SO2T0=SO2T0+PP*CONDNU( 2)/W
               SO4T2  =MAX(AKB,SO4(A,B,SO4T1, D,PP,CONDNU( 2)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU( 3))-1.0)/CONDNU( 3)
               SO2T0=SO2T0+PP*CONDNU( 3)/W
               SO4T3  =MAX(AKB,SO4(A,B,SO4T2, D,PP,CONDNU( 3)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU( 4))-1.0)/CONDNU( 4)
               SO2T0=SO2T0+PP*CONDNU( 4)/W
               SO4T4  =MAX(AKB,SO4(A,B,SO4T3, D,PP,CONDNU( 4)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU( 5))-1.0)/CONDNU( 5)
               SO2T0=SO2T0+PP*CONDNU( 5)/W
               SO4T5  =MAX(AKB,SO4(A,B,SO4T4, D,PP,CONDNU( 5)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU( 6))-1.0)/CONDNU( 6)
               SO2T0=SO2T0+PP*CONDNU( 6)/W
               SO4T6  =MAX(AKB,SO4(A,B,SO4T5, D,PP,CONDNU( 6)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU( 7))-1.0)/CONDNU( 7)
               SO2T0=SO2T0+PP*CONDNU( 7)/W
               SO4T7  =MAX(AKB,SO4(A,B,SO4T6, D,PP,CONDNU( 7)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU( 8))-1.0)/CONDNU( 8)
               SO2T0=SO2T0+PP*CONDNU( 8)/W
               SO4T8  =MAX(AKB,SO4(A,B,SO4T7, D,PP,CONDNU( 8)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU( 9))-1.0)/CONDNU( 9)
               SO2T0=SO2T0+PP*CONDNU( 9)/W
               SO4T9  =MAX(AKB,SO4(A,B,SO4T8, D,PP,CONDNU( 9)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU(10))-1.0)/CONDNU(10)
               SO2T0=SO2T0+PP*CONDNU(10)/W
               SO4T10 =MAX(AKB,SO4(A,B,SO4T9, D,PP,CONDNU(10)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU(11))-1.0)/CONDNU(11)
               SO2T0=SO2T0+PP*CONDNU(11)/W
               SO4T11 =MAX(AKB,SO4(A,B,SO4T10,D,PP,CONDNU(11)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU(12))-1.0)/CONDNU(12)
               SO2T0=SO2T0+PP*CONDNU(12)/W
               SO4T12 =MAX(AKB,SO4(A,B,SO4T11,D,PP,CONDNU(12)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU(13))-1.0)/CONDNU(13)
               SO2T0=SO2T0+PP*CONDNU(13)/W
               SO4T13 =MAX(AKB,SO4(A,B,SO4T12,D,PP,CONDNU(13)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU(14))-1.0)/CONDNU(14)
               SO2T0=SO2T0+PP*CONDNU(14)/W
               SO4T14 =MAX(AKB,SO4(A,B,SO4T13,D,PP,CONDNU(14)))
!--------------
               PP=W*SO2T0*(EXP(-RTSO2(I,L,1)*CONDNU(15))-1.0)/CONDNU(15)
               SO2T0=SO2T0+PP*CONDNU(15)/W
               SO4T15 =MAX(AKB,SO4(A,B,SO4T14,D,PP,CONDNU(15)))
!
!      TOTAL SO2 CONVERTED INTO SO4
!
               XSO2=XROW(I,L+1,ISO2)-SO2T0
               RTSO2(I,L,1)=XSO2/ADT2    !OXIDATION RATE
               XROW(I,L+1,ISO2)=SO2T0
!
!      GASEOUS SO4 CHANGE
!
               XSO4=(SO4T15-XROW(I,L+1,ISO4))/ADT2
!
!      RESULTING SO4
!                               IF GASEOUS SO4 IS REDUCED.
               TOTSO4= -XSO2*W + MAX(0.0,-XSO4*ADT2)  

               TOTNU(I)=0.0
!
!        FOR NUCLEATION 
!              
               IF (B .GT. 0.0) THEN
                 BLG=LOG(B) 
                 TOTNU(I)=1.0/ADT2 * (   &
     &                    CONDNU( 1)*EXP(BLG + D*LOG(SO4T0 )) +  &
     &                    CONDNU( 2)*EXP(BLG + D*LOG(SO4T1 )) +  &
     &                    CONDNU( 3)*EXP(BLG + D*LOG(SO4T2 )) +  &
     &                    CONDNU( 4)*EXP(BLG + D*LOG(SO4T3 )) +  &
     &                    CONDNU( 5)*EXP(BLG + D*LOG(SO4T4 )) +  &
     &                    CONDNU( 6)*EXP(BLG + D*LOG(SO4T5 )) +  &
     &                    CONDNU( 7)*EXP(BLG + D*LOG(SO4T6 )) +  &
     &                    CONDNU( 8)*EXP(BLG + D*LOG(SO4T7 )) +  &
     &                    CONDNU( 9)*EXP(BLG + D*LOG(SO4T8 )) +  &
     &                    CONDNU(10)*EXP(BLG + D*LOG(SO4T9)) +  &
     &                    CONDNU(11)*EXP(BLG + D*LOG(SO4T10)) +  &
     &                    CONDNU(12)*EXP(BLG + D*LOG(SO4T11)) +  &
     &                    CONDNU(13)*EXP(BLG + D*LOG(SO4T12)) +  &
     &                    CONDNU(14)*EXP(BLG + D*LOG(SO4T13)) +  &
     &                    CONDNU(15)*EXP(BLG + D*LOG(SO4T14)) )
               END IF

!
!     FOR CONDENSATION
!
               TOTCD(I,L)=A/ADT2* (  &
     &                    CONDNU( 1)*SO4T0 + CONDNU( 2)*SO4T1 +  &
     &                    CONDNU( 3)*SO4T2 + CONDNU( 4)*SO4T3 +  &
     &                    CONDNU( 5)*SO4T4 + CONDNU( 6)*SO4T5 +  &
     &                    CONDNU( 7)*SO4T6 + CONDNU( 8)*SO4T7 +  &
     &                    CONDNU( 9)*SO4T8 + CONDNU(10)*SO4T9 +  &
     &                    CONDNU(11)*SO4T10+ CONDNU(12)*SO4T11+  &
     &                    CONDNU(13)*SO4T12+ CONDNU(14)*SO4T13+  &
     &                    CONDNU(15)*SO4T14 )
               RTNUCL(I,L)=TOTNU(I)
!
!      TOTAL SO4 PRODUCTION RATE FROM THE SIPLIFIED SCHEME. IT IS POSSIBLE
!      THAT THE MASS MAY NOT BE BALANCED COMPARED TO THE TOTAL SO2
!      OXIDATED. THEREFORE, THE BALANCED RATE WAS SCALED ACCORDING TO
!      THE INIVIDUAL RATE AND THE CONVERTED AMMOUNT OF SO4: TOTSO4
!
               TOTPAR=TOTCD(I,L)+RTNUCL(I,L)

               TOTCD(I,L)=TOTCD(I,L)/TOTPAR*TOTSO4/ADT2
               RTNUCL(I,L)=RTNUCL(I,L)/TOTPAR*TOTSO4/ADT2

               XROW(I,L+1,ISO4)=SO4T15
!       print *, '1 SO4T0 =', XROW(I,L+1,ISO4),XROW(I,L+1,ISO2),I,L
!
!       * PRODUCTION OF NEW PARTICLES BY NUCLEATION
!
               INEW=ISIZE*(NN-1)+1+(IAE1-1)
               XROW(I,L+1,INEW)=XROW(I,L+1,INEW)+RTNUCL(I,L)*ADT2
!      if(jlat.eq.185.and.i.eq.14.and.l.eq.ilev) then
!      write(0,*)'wzz: INEW=', INEW, XROW(I,L+1,INEW), RTNUCL(I,L), ADT2
!      endif
!                print *, 'RTNUCL(I,L)=',RTNUCL(I,L),'INEW=',INEW       
             END IF
         END DO
      END DO
!
!     FOR CONDENSATION
!
      DO N=1,ISIZE
         DO L=1+MAE,ILEV
            DO I=IL1,IL2
                COND=TOTCDN(I,L)
                IF (COND .GT. 0. .AND. AERONUM(I,L,N)*ROAROW(I,L).GT.  &
     &                                                     XIAO) THEN
!
!      * PERCENTAGE OF SO4 CONDENSED TO EACH BIN
!
                   CONDSO4=AMAX1(AMIN1(RTCOND(I,L,N)/COND,1.),0.)
                   RTCOND(I,L,N)=CONDSO4*TOTCD(I,L)
                ELSE
                   RTCOND(I,L,N)=0.0
                END IF 
            END DO
         END DO
      END DO
!
!       * CALL TO COMPUTE THE INTERSECTIONAL TRANSPORT DUE TO
!         CONDENSATION PROCESS.
!
!      if(jlat.eq.185) then
!      write(0,*)'wzz: in SULFATE 1=', xrow(14,lev,53),xrow(14,lev,49)
!      endif
!        print *, 'before intrsec'
      CALL INTRSEC (   LEV,     ILEV,      ILG,      IL1,    IL2,  &
     &                XROW,    RGRID,      NTR,    ICOMP,   IAE1,  &
     &            AEROSIZE,      RTH,       NN,     ADT2,  RHOP0,XIAO,  &
     &               ISIZE,   RTCOND,  AERONUM,   ROAROW,   PVOL,MAE)
!         print *, '2 SO4T0 =', XROW(I,L,ISO4),XROW(I,L,ISO2), I,L
!      if(jlat.eq.185) then
!      write(0,*)'wzz: in SULFATE 2=', xrow(14,lev,53),xrow(14,lev,49)
!      endif
      RETURN
      END
