      SUBROUTINE COAGD (     ILG,    IL1,    IL2,     ILEV,  THROW,  &
     &                     ISIZE, ROAROW,  RTCOA,   RHSIZE, TOTMAS, &
     &                   AERONUM,    NTR,    NTP,        F,   IAE1, &
     &                     DBSUM,  SGSUM,BINLOSS,   AEROP1,  KOUNT, &
     &                      BETA,   ADT2,  ICOAG,    IREST,   XROW, &
     &                         V,    VKP,   JLAT,     ICOB,   TMIN, &
     &                     PDEPV,  PDIFF, CBAR12,       GX,    MAE,  &
     &                       IGF,  IGFIJ,   RHOP, AEROSIZE,  RHOPD, &
     &                    PRESSG,   DSHJ,  GDREM)

!-----------------------------------------------------------------------------
!     * MAY 12/00   S.L. GONG INTRDUCTION OF IGF AND IGFIJ TO SPPED
!     *                       UP THE CODE. THE PERCENTAGE OF COAGULATION
!     *                       IN GCMIII WEND DOWN FROM 17% TO 1% OF
!     *                       TOTAL CPU TIME.
!     * NOV 06/98   S.L. GONG ADAPT THE JACOBSON'S SCHEME TO
!     *                       CONSERVE VOLUME [MASS] OF EACH SPECIES
!     *                       AFTER COAGULATION.
!     *                       [JACOBSON ET.AL. ATM. ENV. 1994]
!     *
!     * OCT 01/97   S.L. GONG MODIFIED FOR MULTI-COMPONENT AEROSOLS.
!     *
!     *
!     * SEP 23/97 - S.L. GONG RECODE THE COAGULATION MODULE EXCEPT
!     *             E. GIRARD ADAPTING COAGULATION COEFFICIENTS FUNTION
!     *                       BETA FROM PREVIOUS CODE. FEATURING: 
!     *                         
!     *                       A. A UNIQUE DO LOOP TO HANDEL ALL THE
!     *                          COAGULATION PROCESSES AMONG SIZE BINS
!     *                          AND ELLIMINATE THE DUPLICATION OF    
!     *                          COMPUTING BOTH K(1,2) AND K(2,1) WHICH ARE
!     *                          EQUAL.  
!     *                       B. VECTORIZATION OF THE CODE FOR BEST 
!     *                          EFFICIENCY. 
!     *                       C. THE COEFFICIENTS ARE CALCULATED BASED ON
!     *                          THE REAL SIZE AND DENSITY OF PARTICLES. 
!     *                       D. SIMPLIFY THE INTEGRATION PROCEDURE TO
!     *                          BE RUN FOR A 3-D CLIMATE MODEL.
!     *                       E. TENDENCY CALCULATION IS BASED ON THE 
!     *                          ORIGINAL DYNAMICAL COAGULATION EQUATIONS
!     *                          WHERE PARTICLE NUMBER CONCENTRATIONS ARE
!     *                          PROGNOSTIC VARIABLES.
!     *                       F. PHYSICAL PROPERTIES SUCH AS MEAN FREE 
!     *                          PATH, VISCOSITY AND SETLLING VELOCITY ARE
!     *                          COMPUTED IN A CONSISTANT WAY AS IN OTHER
!     *                          AEROSOL ROUTINES.
!     *                       G. COAGC CAN NOW RUN EVERY TIME STEP FOR
!     *                          MUCH SMALLER OVERHEAD.
!     *                                                
!     * MAR 01/95 - E.GIRARD. PREVIOUS VERSION BASED ON GELBARD ET AL [1980]  
!     *
!     *                          SCHEME OF COAGULATION FROM GELBARD ET AL. 
!     *                          WITH PRE-CALCULATED COEFFICIENTS
!     *                          OF COAGULATION WAS USED.
!     *
!     *                          BROWNIAN, TURBULENT AND GRAVITATIONAL 
!     *                          COAGULATION ARE COMPUTED IN THIS MODULE.  
! GDREM  | working array for total ground level | R | ILG*NTR  |  L  |, wzz
!-----------------------------------------------------------------------------
      IMPLICIT NONE
!
      INTEGER ILG ,ILEV,NTR, ISIZE, MAE,IREST,JLAT,KOUNT, N
      INTEGER ICOB,L,IL, IL1, IL2,I,J, K, NN, NTP ,IJ
      INTEGER IK, IAE1, IP, ICOAG ,NT,NO

      REAL DX, V1,DL3,GX0,DIFFX, DIFFY, DSUM,CBAR, GMEAN,AMU
      REAL STICK, XIAO, ADT2,OLDNUM,RTLOSS,CPRESV,RGASV,TURB1  
      REAL CPRES, RGOASQ, RGOCP, RGAS, G, ASQ, RAYON,TW,WW   
      REAL AVNO,    RGASi,     AM,   BOLTZK,  PI,  A,TURBDS

      REAL L1,TMIN, RWI
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES
      COMMON /PARAMS/ RGASV,CPRESV
      COMMON /NARCM/ AVNO,    RGASi,     AM,   BOLTZK,  PI,  A

      REAL THROW(ILG,ILEV+1),ROAROW(ILG,ILEV),AEROSIZE (2,ISIZE)
      REAL RTCOA(ILG,ILEV,NTR),RHSIZE(ILG,ILEV,ISIZE)
      REAL V(ISIZE), BINLOSS(ILG,ILEV,ISIZE),RHOP(ILG,ILEV,ISIZE)
      REAL VKP(ILG,ILEV), BETA (ILG,ILEV,ISIZE,ISIZE)
      REAL F(ISIZE,ISIZE,ISIZE), SGSUM(ILG,ILEV), DBSUM(ILG,ILEV)
      REAL TOTMAS(ILG,ILEV,ISIZE), AERONUM(ILG,ILEV,ISIZE)
      REAL RHOPD(ILG,ILEV,ISIZE)
      REAL AEROP1(ILG,ILEV,ISIZE), XROW(ILG,ILEV+1,NTR)
      REAL PDIFF(ILG,ILEV,ISIZE),PDEPV(ILG,ILEV,ISIZE)
      REAL CBAR12(ILG,ILEV,ISIZE),GX(ILG,ILEV,ISIZE)

      INTEGER IGF(ISIZE), IGFIJ(ISIZE,ISIZE*ISIZE,2)
      real PRESSG(ILG), DSHJ(ILG,ILEV), GDREM(ILG,NTR)   ! wzz, 2008.8.20
      real R3D    !  wzz, 2008.2.15

      DATA STICK/1.0/, TURBDS/0.002/
      DATA XIAO /1.E6/
!
!     NON START-RUN BEGINS 
!
      CALL PUTZERO(RTCOA,ILG*ILEV*NTR)      
      CALL PUTZERO(TOTMAS, ILG*ILEV*ISIZE)
!
!     UPDATE THE CURRENT AEROSOL NUMBER
!
      DO NT=1,NTP
         DO N=1,ISIZE
            NO= N+ISIZE*(NT-1)+(IAE1-1)
            DO L=1+MAE,ILEV
              DO I=IL1,IL2
                 TOTMAS(I,L,N)=TOTMAS(I,L,N)+XROW(I,L+1,NO)
!     if(jlat.eq.249.and.i.eq.2.and.l.eq.ilev.and.n.eq.1) then
!       write(0,*)'wzz: aeronu 0=', TOTMAS(i,l,n),xrow(i,l+1,no),no,nt
!     endif
              END DO
            END DO
         END DO
      END DO

      DO N=1,ISIZE
!         RWI=(AEROSIZE(1,N)+AEROSIZE(2,N))*0.5
!         R3D = 4.189*RWI*RWI*RWI
         R3D = V(N)
         DO L=1+MAE,ILEV
           DO I=IL1,IL2
              AERONUM(I,L,N)=TOTMAS(I,L,N)/(R3D*RHOPD(I,L,N))
           END DO
         END DO
      END DO
!     if(jlat.eq.249) then
!       write(0,*)'wzz: aeronu=', aeronum(2,ilev,1),TOTMAS(2,ilev,1),RHOPD(2,ilev,1)
!     endif

      IF (MOD(KOUNT,ICOAG) .EQ. 0 .OR. IREST .EQ. 1) THEN        !END IF 888

!     IF (JLAT .EQ. 152) WRITE (*,555) KOUNT

      CALL PUTZERO(BETA,ILG*ILEV*ISIZE*ISIZE)
      DO N=1,ICOB
!         RWI=(AEROSIZE(1,N)+AEROSIZE(2,N))*0.5
!         R3D = 4.189*RWI*RWI*RWI
         R3D = V(N)
         DO L=1+MAE,ILEV
            DO IL=IL1,IL2
              DX=2.*RHSIZE(IL,L,N)
              V1=R3D*RHOP(IL,L,N)
!              V1=4.189*RWI*RWI*RWI*RHOP(IL,L,N)
              CBAR12(IL,L,N)=3.51568E-23*THROW(IL,L+1)/V1
              L1=2.5465*PDIFF(IL,L,N)/SQRT(CBAR12(IL,L,N))
              DL3=(DX+L1)*(DX+L1)*(DX+L1)
              GX0=(DL3-(DX*DX+L1*L1)**1.5)/(3.*DX*L1)-DX
              GX(IL,L,N)=GX0*GX0
            END DO
         END DO
      END DO 
         

      DO I=1,ICOB
        DO J=I,ICOB
          DO L=1+MAE,ILEV
            DO IL=IL1,IL2
!
!     * DIFFUSION COEFFICIENTS
!
              DIFFX=PDIFF(IL,L,I)
              DIFFY=PDIFF(IL,L,J)
!
!     * AIR'S DYNAMIC VISCOSITY
!
              AMU=145.8*1.E-8*THROW(IL,L+1)**1.5/   &
     &                             (THROW(IL,L+1)+110.4)              
              DSUM=2.*(RHSIZE(IL,L,I)+RHSIZE(IL,L,J))
!
!       BROWNIAN COAGULATION COEFFICIENT        [V1, V2, VR - PARTICLE MASS, KG] 
!
              CBAR =SQRT(CBAR12(IL,L,I)+CBAR12(IL,L,J))
              GMEAN=SQRT(GX(IL,L,I)+GX(IL,L,J))

              BETA(IL,L,I,J)=6.2832*(DIFFX+DIFFY)*DSUM/(DSUM/(DSUM  &
     &                        +2.*GMEAN)+8.*(DIFFX+DIFFY)/  &
     &                                  (CBAR*DSUM*STICK))
!
!       ADD GRAVITATIONAL COAGULATION
!
              BETA(IL,L,I,J)=BETA(IL,L,I,J)+0.7854*DSUM*DSUM  &
     &                                 *ABS(PDEPV(IL,L,I)-PDEPV(IL,L,J))
              BETA(IL,L,J,I)=BETA(IL,L,I,J)

            END DO
          END DO
        END DO
      END DO
 888  END IF


      CALL PUTZERO(AEROP1,ILG*ILEV*ISIZE)
      DO K=1,ICOB
        CALL PUTZERO(DBSUM,ILG*ILEV)
        CALL PUTZERO(SGSUM,ILG*ILEV)
        CALL PUTZERO(BINLOSS,ILG*ILEV*ISIZE)

        DO J=1,ICOB
          DO L=1+MAE,ILEV
            DO IL=IL1,IL2
!
!       NUMBER LOSS OF K DUE TO COLLISION WITH J [1-ISIZE]
!              
                 OLDNUM=AERONUM(IL,L,J)*ROAROW(IL,L)
                 IF (OLDNUM .GT. XIAO) THEN
                    BINLOSS(IL,L,J)=(1.-F(K,J,K))*BETA(IL,L,J,K)*OLDNUM
!                   BINLOSS(IL,L,J)=(1.-F(J,K,K))*BETA(IL,L,J,K)*OLDNUM
                    SGSUM(IL,L)=SGSUM(IL,L)+BINLOSS(IL,L,J)
                 END IF
            END DO
          END DO
        END DO


        DO IJ=1,IGF(K)     !GETHERED POINTS FOR COAGFR
           I= IGFIJ(K,IJ,1)
           J= IGFIJ(K,IJ,2)
           DO L=1+MAE,ILEV
              DO IL=IL1,IL2
!
!       VOLUME GAIN OF K DUE TO COLLISION OF I AND J [=<K]-- [M3 S-1]
!
                  DBSUM(IL,L)=DBSUM(IL,L)+F(I,J,K)*BETA(IL,L,I,J)*V(I)  &
     &               *AEROP1(IL,L,I)*AERONUM(IL,L,J)*ROAROW(IL,L)
!     if(jlat.eq.249.and.il.eq.2.and.l.eq.ilev) then
!       write(0,*)'wzz: dbsub0=',DBSUM(2,ilev),f(i,j,k),beta(il,l,i,j),v(i),
!    &      aerop1(il,l,i),aeronum(il,l,j),roarow(il,l),i,j,k
!     endif
              END DO
          END DO
        END DO
!
!      TOTAL NUMBER OF K AFTER COAGULATION [# M-3]
!
        DO L=1+MAE,ILEV
          DO IL=IL1,IL2
            AEROP1(IL,L,K)=(ROAROW(IL,L)*AERONUM(IL,L,K)+ADT2/V(K)  &
     &                           *DBSUM(IL,L))/(1.+ADT2*SGSUM(IL,L))
          END DO
        END DO
!
!     MASS BALANCE FOR EACH SPECIES OF K
!       LOSS OF K IS THE SUM OF MASS GAINED BY ALL J
!       SGSUM  - DIMESIONLESS
!       AEROP1 - # M-3
!
        DO NN=1,NTP
           IK=(NN-1)*ISIZE+K+(IAE1-1)
           CALL PUTZERO(VKP, ILG*ILEV)
           DO L=1+MAE,ILEV
             DO IL=IL1,IL2
!
!        LOST TENDENCY OF K
!        AEROP1(IL,L,K) * SGSUM(IL,L) * V(K) IS THE VOLUME LOSS RATE OF
!        BIN K DUE TO THE COAGULATION. BY /(OLDNUM*V(K)) *XROW(IL,L+1,IK)
!        THE LOSS TENDENCY OF IK IS OBTAINED.
!         
!
                 OLDNUM=AERONUM(IL,L,K)*ROAROW(IL,L)
                 IF (OLDNUM .GT. XIAO) THEN
                    VKP(IL,L)=AEROP1(IL,L,K)/OLDNUM*XROW(IL,L+1,IK)
                    RTCOA(IL,L,IK)=RTCOA(IL,L,IK)-VKP(IL,L)*SGSUM(IL,L)
                 END IF
             END DO
           END DO
             
!
!        GAIN TENDENCY OF I DUE TO LOSS OF [K,J]
!
           DO J=1,ICOB
             DO I=MAX(J,K),ICOB
               IF (F(K,J,I) .GT. 0.) THEN
!              IF (F(J,K,I) .GT. 0.) THEN
                 IP=(NN-1)*ISIZE+I+(IAE1-1)
                 DO L=1+MAE,ILEV   
                    DO IL=IL1,IL2
                       RTLOSS=VKP(IL,L)*BINLOSS(IL,L,J)*F(K,J,I)
!                      RTLOSS=VKP(IL,L)*BINLOSS(IL,L,J)*F(J,K,I)
                       RTCOA(IL,L,IP)=RTCOA(IL,L,IP)+RTLOSS 
                    END DO
                 END DO
               END IF
             END DO
           END DO
        END DO                                           !END OF NN LOOP
      END DO                                             !END OF K  LOOP
!
!      UPDATE TRACER DUE TO COAGULATIONS
!
!     if(jlat.eq.249) then
!       write(0,*)'wzz: coag 0=',xrow(2,ilev+1,12),rtcoa(2,ilev,12),tmin, DBSUM(2,ilev)
!     endif
      DO NN=1,NTP
        DO K=1,ISIZE                 !THE TENDENCY FOR K=ISIZE IS ADDED
                                     !DRY DEPOSITION
           IK=(NN-1)*ISIZE+K+(IAE1-1)
           DO L=1+MAE,ILEV
              DO IL=IL1,IL2
                 XROW(IL,L+1,IK)=AMAX1(TMIN,   &
     &                            XROW(IL,L+1,IK)+RTCOA(IL,L,IK)*ADT2)
! wzz, 2008.8.20
!                GDREM(IL,IK) = GDREM(IL,IK) + RTCOA(IL,L,IK)*
!    1                           DSHJ(IL,L)*PRESSG(IL)/G
              END DO
           END DO
        END DO
      END DO
!     if(jlat.eq.249) then
!       write(0,*)'wzz: coag 1=',xrow(2,ilev+1,12),rtcoa(2,ilev,12),tmin
!     endif

 555  FORMAT (' COAGD -> BETA COMPUTED @ ',I5)
      RETURN
      END
