      SUBROUTINE DRYDEPO(ILEV,      LEV,      ILG,      IL1,     IL2, &
     &                    VDG,   SURFWD,    GTROW,  SFCTROW,     MAE, &
     &                 FSGROL,   DEGLAT,    SEASN,       RA,     USI, &
     &                SICNROW,  FSNOROW,    THROW,    ISIZE,     NTP, &
     &                    NTR,     ADT2,   PRESSG,     SHJ,     TMIN, &
     &                  THLEV,   ROAROW,     RHOP,   RHSIZE,    ISO2, &
     &                   IDMS,    FLAND,     XROW,   SURFDG,   PDEPV, &
     &                  RTDRY,    PXNEW,     DSHJ,    GDREM,    IAE1, &
     &                  CLDCV,    RHROW,     IDAY,     JLAT,   PDIFF, &
     &                    LUC,     CAMR,       NG,       NS,      ML, &
     &                   PLLP,       Z0,      SCC,    RCW, AC, CSPEC, &
     &                    LAI,      RM0,      ZPD,     FCAP, WP, RGW, &
     &                   AEST,    GAMMA)
!-----------------------------------------------------------------------
!     HISTORY:
!     --------
!     * AUG 01/1999 - S.L. GONG   PASS ALL CONTROL VARIABLES FOR 15
!                               LAND-USE FROM THE CONTROL FILE.
!
!     * JUN 29/1999 - S.L. GONG   INTRODUCED CAM SWITCH TO BE USED FOR:
!     *                             CAM < 10.0 - FOR RCM OR GCM
!     *                             CAM > 10.0 - FOR AURAMS
!     *
!     * JAN 15/1998 - L.M. ZHANG  REFINED PARAMETERIZATION FOR PARTICLES
!     *                           COMPARED WITH EMPIRICAL MODEL RESUTLS.
!     *
!     * JAN 02/1998 - L.M. ZHANG  ADD ADOM GAS DRY DEPOSITION
!     *               S.L.  GONG  COMBINE BOTH PARTICLE AND GAS
!     *                           DRYDEPOSITION INTO A UNIFIED INTEREFACE.
!     *
!     * DEC 19/1996 - S.L. GONG   VECTORIZED THE WHOLE PROGRAM AND ADD
!     *                           WORKING SPACES.
!     *                           FVEG, CD012, CVOCDL, AEXP
!     *
!     * AUG  4/1994 - S.L. GONG   FIRST VERSION
!     *
!
!     METHOD
!     ---------                 
!     - PARAMETERIZATION OF DRY DEPOSITION SCHEME BY FILIPPO GIORGI
!       [JGR, 91(D9), 1986] WAS USED.
!     - TWO LAYER MODEL.
!
!----------------------------------------------------------------------
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES
      COMMON /PARAMS/ RGASV,CPRESV

      integer JLAT
      integer ILG, NG, ILEV, NTR, LEV
      integer IDMS, ISO2
      REAL DEGLAT(ILG), VDG(ILG,NG)
      REAL THLEV(ILG, ILEV), GDREM(ILG, NTR, 3)
      REAL THROW(ILG,LEV),RHROW(ILG,LEV),SHJ(ILG,ILEV)
      REAL FSNOROW(ILG), CLDCV(ILG,ILEV,2)

      REAL SURFDG(ILG),SURFWD(ILG),FLAND(ILG,LUC)
      REAL RHSIZE(ILG,ILEV,ISIZE),PDEPV(ILG,ILEV,ISIZE)
      REAL RHOP(ILG,ILEV,ISIZE),PDIFF(ILG,ILEV,ISIZE)
      REAL RTDRY(ILG,ILEV,NTR)

      REAL PRESSG(ILG), DSHJ(ILG,ILEV), XROW(ILG,LEV,NTR)

      REAL PXNEW(ILG,ILEV,NTR)
      REAL ROAROW(ILG,ILEV), GTROW(ILG) , SFCTROW(ILG)
      REAL FSGROL(ILG), SICNROW(ILG)

      REAL SEASN(ILG),RA(ILG,LUC),USI(ILG,LUC)
      REAL PLLP(LUC,NS),Z0(LUC,NS),SCC(NG), RCW(NG), AC(NG)
      REAL LAI(LUC,NS),RM0(NG), ZPD(LUC,NS) 
      REAL FCAP(ML),WP(ML),RGW(NG)
      REAL AEST(LUC), GAMMA(LUC)
      real CAMR

      CHARACTER*4 CSPEC(NG)

      CALL PUTZERO (RTDRY,ILG*ILEV*NTR)
!
!     CALL TO VECTORIZED ADOM DRY GAS DEPOSITION SCHEME
!     NG0=1 for SO2 IN CAM ONLY. FOR ALL 12 SPECIES, NG0=NG
!
      IF (CAMR .LT. 10.0) THEN 
         NG0=1
      ELSE
         NG0=NG
      END IF
!      write(0,*)'wzz: CAM=', CAMR, NG0, IDAY, ISO2, DEGLAT(1), DEGLAT(2)
!     if(jlat.eq.185) then
!     write(0,*)'wzz: in DRYDEPO 0', xrow(14,lev,53),GDREM(14,53,1)
!     endif

      STDHi=10.0
      CALL DRYGAS (ILG,   FLAND,    VDG,  SURFWD, STDHi, ILEV,  &
     &           GTROW, SFCTROW,  FSGROL,PXNEW(1,1,1), LUC,   &
     &          DEGLAT, SEASN,RHROW(1,LEV),CLDCV,PXNEW(1,1,2),  &
     &            IDAY,     IL1,    IL2, SICNROW,    FSNOROW,  &
     &              RA,     USI,   JLAT,     NG,    NS,   NG0,  &
     &              Z0,      SCC,    RCW,    AC,   &
     &              LAI,      RM0,      ZPD,   RGW)
!
!     * SO2 DRY DEPOSITION TENDENCY
!

      DO I = IL1, IL2
          QCC=-ABS(VDG(I,1))/THLEV(I,ILEV)*ADT2
          RTDRY(I,ILEV,ISO2) = XROW(I,LEV,ISO2)*(EXP(QCC)-1.0)/ADT2
          XROW(I,ILEV+1,ISO2)=AMAX1(TMIN,XROW(I,ILEV+1,ISO2)   &
     &                                    +RTDRY(I,ILEV,ISO2)*ADT2)
          GDREM(I,ISO2,1)=GDREM(I,ISO2,1)+ RTDRY(I,ILEV,ISO2)*     &
     &                               DSHJ(I,ILEV)*PRESSG(I)/G

      END DO
  200 CONTINUE
!
!     PARTICLE DRY DEPOSITION
!
!     if(jlat.eq.185) then
!     write(0,*)'wzz: in DRYDEPO 1', xrow(14,lev,53),GDREM(14,53,1)
!     endif
      CALL DRYPAR  (THROW,   LEV,    ILEV,     ILG,  ISIZE,  NTP,   &
     &                IL1,    IL2,    NTR,    ADT2, IAE1,   MAE,   &
     &             PRESSG,SHJ,  THLEV,ROAROW,PXNEW(1,1,1),PDIFF,   &
     &              RHOP, RHSIZE,  FLAND,     XROW, PXNEW(1,1,2),LUC,   &
     &             SURFDG, SURFWD,  PDEPV,   RTDRY, DSHJ,PXNEW(1,1,3),   &
     &               JLAT,  SEASN,     RA,     USI, PLLP, AEST,   &
     &                 NS,  GAMMA)

!     if(jlat.eq.185) then
!     write(0,*)'wzz: in DRYDEPO 2', xrow(14,lev,53),GDREM(14,53,1)
!     endif
      DO NN=1,NTP
        DO N = 1,ISIZE
          NP=ISIZE*(NN-1)+N+(IAE1-1)
          DO  L = ILEV, 1+MAE, -1
            DO I = IL1, IL2
                 XROW(I,L+1,NP)=AMAX1(XROW(I,L+1,NP)+RTDRY(I,L,NP)*ADT2, &
     &                                 TMIN)
                 GDREM(I,NP,1) = GDREM(I,NP,1) + RTDRY(I,L,NP)* &
     &                           DSHJ(I,L)*PRESSG(I)/G
            END DO
          END DO
        END DO
      END DO

!     if(jlat.eq.185) then
!     write(0,*)'wzz: in DRYDEPO 3', xrow(14,lev,53),GDREM(14,53,1)
!     endif
      RETURN
      END
