      SUBROUTINE DRYGAS(ILG,      F,    VDG,    U2I,      Z2,  ILEV,  &
     &                   TS,     T2,   SRAD,   ZZ0,     LUC,   &
     &                 GLAT,  SEASN,     RH,  CLDCV,    FCLD,  &
     &                 IDAY,    IL1,    IL2,  PICE,  PSNOW,  &
     &                   RA,    USI,   JLAT,    NG,       NS, NG0,  &
     &                   Z0,     SC,    RCW,   AC,  &
     &                  LAI,    RM0,    ZPD,    RGW)

      integer JLAT
      integer ILG, LUC, NG
      REAL F(ILG, LUC),VDG(ILG, NG)
      REAL ICECOV,K,KUI
      REAL LOGRATIO
      REAL Z0(LUC,NS),SC(NG), RCW(NG), AC(NG),A0
      REAL LAI(LUC,NS),RM0(NG),RGW(NG),ZPD(LUC,NS)
      REAL U2I(ILG),TS(ILG),T2(ILG),PSNOW(ILG),PICE(ILG)
      REAL SRAD(ILG),GLAT(ILG),RH(ILG),FCLD(ILG)
      REAL ZZ0(ILG)
      REAL CLDCV(ILG,ILEV,2), SEASN(ILG),RA(ILG,LUC),USI(ILG,LUC)

      COMMON /NARCM/ AVNO,    RGASi,     AM,   BOLTZK,  PI,  A
      COMMON /NARCM/ A0,RSN0,RCUT0,RCG0,P

      DATA K/0.4/,VI/0.15E-4/,G/9.81/,RAD/0.0174533/
! --- 10 MICRONS (RUTTER 1975)
      DATA BMAX /2.5E-6/
! --- 0.1 MICRONS
      DATA BMIN /0.1E-6/
! --- W/M**2  SOLAR CONSTANT
      DATA SCON /1360./
!      DATA SCON /1380./
!
! --- COEFFICIENTS FOR CALCULATION OF SATURATION VAPOR PRESSURE
! --- PKK: 7/16/85
      DATA AA0/6.107799961/,AA1/4.436518521E-1/,AA2/1.428945805E-2/,  &
     &     AA3/2.650648471E-4/,AA4/3.031240396E-6/,AA5/2.034080948E-8/,  &
     &     AA6/6.136820929E-11/
! --- SEA LEVEL PRESSURE (MB)
      DATA PMB /1000./
!
!****************** HOURLY PARAMETER DEFINITIONS ********************
!     EL     - MONIN OBUKHOV LENGTH (M) - CALCULATED FOR EACH LAND
!                                         USE CATEGORY
!     USTAR  - FRICTION VELOCITY - CALCULATED FOR EACH LAND USE
!                                  CATEGORY
!     U2I    - WIND SPEED AT 10M. (M/S)
!     T2     - TEMPERATURE AT 10M. (DEG K)
!     Z2     - 10 M.
!     GLAT   - CENTER LATITUDE OF GRID CELL (DEG)
!     SRAD   - SOLAR IRRADIANCE AT THE GROUND (W/M**2)
!     FCLD   - CLOUD FRACTION (0-1)
!     TSW    - WATER SURFACE TEMPERATURE (DEG K)
!     TS     - SURFACE TEMPERATURE (DEG K)
!     PSNOW  - SNOW COVER (0.0-1.0)
!     PICE   - ICE COVER OF ENTIRE GRID CELL (0.0-1.0)
!     F      - FRACTION OF EACH LUC   (0.0-1.0)
!     PCWC   - CANOPY WATER CONTENT (0.0-1.0)
!     RH     - RELATIVE HUMIDITY OF AIR AT 10M. (0.0-1.0)
!********************************************************************
!
!      LAND USE CATEGORIES
!       1  EVERGREEN NEEDLELEAF FOREST
!       2  EVERGREEN BROADLEAF FOREST
!       3  DECIDUOUS NEEDLELEAF FOREST
!       4  DECIDUOUS BROADLEAF FOREST
!       5  MIXED FOREST 
!       6  GRASSLAND
!       7  CROPS, MIXED FARMING
!       8  DESERT
!       9  TUNDRA 
!       10 DWART TREES, SHRUBS WITH GROUND COVER 
!       11 WET LAND WITH PLANTS
!       12 ICE CAPS AND GLACIERS
!       13 INLAND WATER
!       14 OCEAN
!       15 URBAN
!------------------------------------------------------------

      ES(TEMP) = 6.108*EXP(17.27*(TEMP - 273.16)/(TEMP - 35.86))
! --- INITIALIZE DEPOSITION VELOCITY AND TOTAL RESISTANCE ARRAYS
      CALL PUTZERO (VDG,  ILG*NG)
      CALL PUTZERO (FCLD, ILG)
!
!     CLOUD COVER
!
      DO L=1,ILEV
         DO I=IL1,IL2
             FCLD(I)=FCLD(I)+ CLDCV(I,L,1)+CLDCV(I,L,2)
         END DO
      END DO
      DO I=IL1,IL2
         FCLD(I)=AMIN1(FCLD(I),1.0)
      END DO
!
!     define season category according to Julian Day and Latitude.
!
      DO 100 IL=IL1,IL2
!
!     FOR NOTHER HEMISPHERE
!
      IF (GLAT(IL) .GE. 0.0) THEN
      IF (IDAY.LE.59 .OR. IDAY.GE.335) THEN           !Jan. Feb. Dec.
         IF (GLAT(IL).LE.30.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.30. .AND. GLAT(IL).LE.35.) SEASN(IL)=5.
         IF (GLAT(IL).GT.35.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.60.AND. IDAY.LE.90)  THEN           !Mar.
         IF (GLAT(IL).LE.35.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.35. .AND. GLAT(IL).LE.40.) SEASN(IL)=5.
         IF (GLAT(IL).GT.40.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.91.AND. IDAY.LE.120) THEN           !Apr.
         IF (GLAT(IL).LE.35.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.35. .AND. GLAT(IL).LE.45.) SEASN(IL)=5.
         IF (GLAT(IL).GT.45.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.121.AND. IDAY.LE.151) THEN           !May.
         IF (GLAT(IL).LE.35.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.35. .AND. GLAT(IL).LE.60.) SEASN(IL)=5.
         IF (GLAT(IL).GT.60.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.152.AND. IDAY.LE.181) THEN           !Jun.
         IF (GLAT(IL).LE.55.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.55. .AND. GLAT(IL).LE.70.) SEASN(IL)=5.
         IF (GLAT(IL).GT.70.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.182.AND. IDAY.LE.212) THEN           !Jul.
         IF (GLAT(IL).LE.70.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.70. .AND. GLAT(IL).LE.80.) SEASN(IL)=5.
         IF (GLAT(IL).GT.80.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.213.AND. IDAY.LE.243) THEN           !Aug.
         IF (GLAT(IL).LE.45.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.45. .AND. GLAT(IL).LE.80.) SEASN(IL)=2.
         IF (GLAT(IL).GT.80.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.244.AND. IDAY.LE.273) THEN           !Sep.
         IF (GLAT(IL).LE.35.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.35. .AND. GLAT(IL).LE.65.) SEASN(IL)=2.
         IF (GLAT(IL).GT.65. .AND. GLAT(IL).LE.80.) SEASN(IL)=3.
         IF (GLAT(IL).GT.80.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.274.AND. IDAY.LE.304) THEN           !Oct.
         IF (GLAT(IL).LE.35.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.35. .AND. GLAT(IL).LE.45.) SEASN(IL)=2.
         IF (GLAT(IL).GT.45. .AND. GLAT(IL).LE.65.) SEASN(IL)=3.
         IF (GLAT(IL).GT.65.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.305.AND. IDAY.LE.334) THEN           !Nov.
         IF (GLAT(IL).LE.35.)                       SEASN(IL)=1.
         IF (GLAT(IL).GT.35. .AND. GLAT(IL).LE.40.) SEASN(IL)=2.
         IF (GLAT(IL).GT.40. .AND. GLAT(IL).LE.50.) SEASN(IL)=3.
         IF (GLAT(IL).GT.50.)                       SEASN(IL)=4.
      END IF

      ELSE
!
!     Southern hemisphere
!
      IF (IDAY.GE.152.AND. IDAY.LE.243) THEN           !Jun. Jul. Aug.
       IF (ABS(GLAT(IL)).LE.30.)                       SEASN(IL)=1. 
       IF (ABS(GLAT(IL)).GT.30. .AND. ABS(GLAT(IL)).LE.35.) SEASN(IL)=5.
       IF (ABS(GLAT(IL)).GT.35.)                       SEASN(IL)=4.
      END IF
       
      IF (IDAY.GE.244.AND. IDAY.LE.273) THEN           !SEp.
       IF (ABS(GLAT(IL)).LE.35.)                       SEASN(IL)=1. 
       IF (ABS(GLAT(IL)).GT.35. .AND. ABS(GLAT(IL)).LE.40.) SEASN(IL)=5.
       IF (ABS(GLAT(IL)).GT.40.)                       SEASN(IL)=4.
      END IF
       
      IF (IDAY.GE.274.AND. IDAY.LE.304) THEN           !Oct.
       IF (ABS(GLAT(IL)).LE.35.)                       SEASN(IL)=1. 
       IF (ABS(GLAT(IL)).GT.35. .AND. ABS(GLAT(IL)).LE.45.) SEASN(IL)=5.
       IF (ABS(GLAT(IL)).GT.45.)                       SEASN(IL)=4.
      END IF
       
      IF (IDAY.GE.305.AND. IDAY.LE.334) THEN           !Nov.
       IF (ABS(GLAT(IL)).LE.35.)                       SEASN(IL)=1. 
       IF (ABS(GLAT(IL)).GT.35. .AND. ABS(GLAT(IL)).LE.60.) SEASN(IL)=5.
       IF (ABS(GLAT(IL)).GT.60.)                       SEASN(IL)=4.
      END IF
      IF (IDAY.GE.335) THEN                            !Dec.
       IF (ABS(GLAT(IL)).LE.55.)                       SEASN(IL)=1. 
       IF (ABS(GLAT(IL)).GT.55. .AND. ABS(GLAT(IL)).LE.70.) SEASN(IL)=5. 
       IF (ABS(GLAT(IL)).GT.70.)                       SEASN(IL)=4.
      END IF
      
      IF (IDAY.LE.31) THEN                            !Jan.
       IF (ABS(GLAT(IL)).LE.70.)                       SEASN(IL)=1. 
       IF (ABS(GLAT(IL)).GT.70. .AND. ABS(GLAT(IL)).LE.80.) SEASN(IL)=5.
       IF (ABS(GLAT(IL)).GT.80.)                       SEASN(IL)=4.  
      END IF
      
      IF (IDAY.GE.32.AND. IDAY.LE.59) THEN           !Feb   
       IF (ABS(GLAT(IL)).LE.45.)                       SEASN(IL)=1.
       IF (ABS(GLAT(IL)).GT.45. .AND. ABS(GLAT(IL)).LE.80.) SEASN(IL)=2.
       IF (ABS(GLAT(IL)).GT.80.)                       SEASN(IL)=4. 
      END IF
      
      IF (IDAY.GE.60.AND. IDAY.LE.90)  THEN            !Mar.
       IF (ABS(GLAT(IL)).LE.35.)                       SEASN(IL)=1.
       IF (ABS(GLAT(IL)).GT.35. .AND. ABS(GLAT(IL)).LE.65.) SEASN(IL)=2.
       IF (ABS(GLAT(IL)).GT.65. .AND. ABS(GLAT(IL)).LE.80.) SEASN(IL)=3.
         IF (ABS(GLAT(IL)).GT.80.)                       SEASN(IL)=4.
      END IF
      
      IF (IDAY.GE.91.AND. IDAY.LE.120) THEN           !Apr.
       IF (ABS(GLAT(IL)).LE.35.)                       SEASN(IL)=1.
       IF (ABS(GLAT(IL)).GT.35. .AND. ABS(GLAT(IL)).LE.45.) SEASN(IL)=2.
       IF (ABS(GLAT(IL)).GT.45. .AND. ABS(GLAT(IL)).LE.65.) SEASN(IL)=3.
         IF (ABS(GLAT(IL)).GT.65.)                       SEASN(IL)=4.
      END IF

      IF (IDAY.GE.121.AND. IDAY.LE.151) THEN           !May.
       IF (ABS(GLAT(IL)).LE.35.)                       SEASN(IL)=1.
       IF (ABS(GLAT(IL)).GT.35. .AND. ABS(GLAT(IL)).LE.40.) SEASN(IL)=2.
       IF (ABS(GLAT(IL)).GT.40. .AND. ABS(GLAT(IL)).LE.50.) SEASN(IL)=3.
         IF (ABS(GLAT(IL)).GT.50.)                       SEASN(IL)=4.
      END IF

      END IF

      IF(SEASN(IL).EQ.3.AND.PSNOW(IL).GT.0.1) SEASN(IL)=4.
      IF(SEASN(IL).EQ.4.AND.PSNOW(IL).LT.0.1) SEASN(IL)=3.

100   CONTINUE

!
! J for species, I for LUC, IL for grids.
!
      DO 900 J=1,NG0
        DO 800 I=1,LUC
          DO 700 IL=IL1,IL2
             RA(IL,I)=0.0999
             ISEA = INT(SEASN(IL))
!
! --- CONSIDER ONLY LAND USE CATEGORIES .GT. 0.005
!
             IF (F(IL,I).LT.0.005) GO TO 700
! --- SET MIN. VALUES
             U2 = AMAX1(U2I(IL),1.0)
             ZZ0(IL)=Z0(I,ISEA)
!
! --- PERCENTAGE OF GRID AREA OVER WATER [SPECIFIC!!, CHANGE WITH LANDUSE]
!
             FW = F(IL,11) + F(IL,13) + F(IL,14)
!
! --- PERCENTAGE OF GRID AREA OVER LAND
!
             PLAND = 1. - FW
!
             PCWC=0.
             IF (RH(IL).GE.0.9) PCWC=0.3
!
! --- CELL SNOW .LE. CELL W/LAND
!
             SNCOV = PSNOW(IL)
             IF (PSNOW(IL).GT.PLAND) SNCOV = PLAND
             ICECOV=PICE(IL)
! --- CELL ICE .LE. CELL W/WATER
             IF (PICE(IL).GT.FW) ICECOV= FW
!
! --- STOMATAL APERTURE AS FUNCTION OF SOLAR IRRADIANCE AND
! --- SOIL MOISTURE
!
             SMAX=SCON*COS((GLAT(IL)-23.45)*RAD)
             B = BMAX * SRAD(IL) * (1. - 0.8*FCLD(IL)) / SMAX + BMIN
             IW=0
             IF(I.EQ.11.OR.I.EQ.13.OR.I.EQ.14) IW=1
!
! --- DETERMINE ICE FRACTION OF WATER CATEGORIES
! --- (NOTE THAT ICECOVER DATA IS ASSUMED TO BE IN FRACTIONS OF ENTIRE
! --- GRID CELL AREA AND NOT FRACTION OF WATER SURFACE AREA.  CHECK
! --- DATA FOR COMPATIBILITY)
! --- FICE IS FRACTION OF CELL WATER WITH ICE
! --- FSNOW IS FRACTION OF CELL LAND THAT IS SNOW
!
             IF(I.EQ.11.OR.I.EQ.13.OR.I.EQ.14)THEN
               FICE=ICECOV/FW
               FSNOW=0.0
             ELSE
               FICE=0.
               FSNOW=SNCOV/PLAND
             ENDIF
!
! --- POTENTIAL TEMPERATURE AT Z2  (DEG. K)
!
             T2P = T2(IL) + Z2 * 0.0098
!***********************************************************************
!       FROM OCD MODEL: FOR CALCULATIONS OVER WATER
! COMPUTE VALUES OF CRITICAL PROFILE VARIABLES: L AND USTAR
!
!***********************************************************************
!**************************BEGIN FOR WATER******************************
             IF(I.EQ.13.OR.I.EQ.14)THEN
!
! ---    VAPOUR PRESSURE AT Z2
!
                E = RH(IL) * ES(T2(IL))
! ---    WATER VAPOUR MIXING RATIO AT  Z2
                Q = 0.622 * E/(PMB-E)
! ---    VIRTUAL POTENTIAL TEMPERATURE AT Z2 (DEG. K)
                T2PV = T2P * (1. + 0.61 * Q)
!
! --- ASSUME RH AT WATER SURFACE IS 100%
! ***    E = ES(TSW-273.15) !SAT. VAP PRESS AT SURFACE
! ---    SATURATED VAPOUR PRESSURE AT SURFACE
!
                TSW=TS(IL)
                E = ES(TSW)
!
! ---    SATURATED MIXING RATIO AT SURFACE
!
                QS = 0.622 * E / (PMB - E)
! ---    VIRTUAL POTENTIAL TEMPERATURE AT SURFACE (DEG. K)
                TSV = TSW * (1. + 0.61 * QS)
                Z0WATER = 1.0E-4
! ---    * SCALET  :  NOT REQUIRED IF  Z2 = 10M
                DTHV = (T2PV - TSV)
!
! --- CALCULATE DRAG COEFFICIENT CUN WITH NEUTRAL CONDITION ASSUMPTION
! ---       -- GARRATT (1977)
!
                CUN=7.5E-4+6.7E-5*U2
!
                EL=9999.
                IF(ABS(DTHV).GT.1.0E-6)EL=T2PV*CUN**1.5*U2**2/(5.096E-3  &
     &                                     *DTHV)
                IF(EL.GT.0..AND.EL.LT.5.0) EL = 5.0
                IF(EL.GT.-5.0.AND.EL.LT.0) EL =-5.0
                ZDL = Z2/EL
                IF(ZDL.LT.0.0) THEN
! --- WIND SPEED
                   X=(1.0 - 15.0*ZDL)**0.25
                   PSIU=2.*ALOG(0.5*(1.0+X))+ALOG(0.5*(1.0+X*X))-   &
     &                     2.0*ATAN(X) + 0.5*PI
! --- POT TEMP
                   Y=SQRT(1.-9.*ZDL)
                   PSIT=2.*0.74*ALOG((1+Y)/2.)
                ELSE
                   PSIU = -4.7*ZDL
                   PSIT = PSIU
                ENDIF
                Z0WATER = 0.000002 * U2**2.5
                USTAR = K*U2/(ALOG(Z2/Z0WATER) - PSIU)
                THSTAR = K*(T2P-TS(IL))/(0.74*ALOG(Z2/Z0WATER)-PSIT)
                ZZ0(IL)=Z0WATER
!
             ELSE
!
! --- COMPUTE USTAR AND L FOR LAND USE CATEGORIES OTHER THAN WATER.
! --- USE LOUIS METHOD. !PKK 7/16/85
! --- FIND BULK RICHARDSON NUMBER.
!
                RIB = G*Z2*(T2P - TS(IL))/(TS(IL)*U2**2)
!
! --- ENSURE THAT CONDITIONS OVER LAND ARE NEVER STABLE WHEN THERE IS
!     INCOMING SOLAR RADIATIOM
!
                IF (SRAD(IL).GT.0.0.AND.RIB.GT.0.0) RIB = 1.E-15
                DELTAT = T2P - TS(IL)
                IF(ABS(DELTAT).LT.1.E-10)DELTAT=SIGN(1.E-10,DELTAT)
                TBAR = 0.5*(T2P + TS(IL))
                RATIOZ = Z2/ZZ0(IL)
                LOGRATIO = ALOG(RATIOZ)
                ASQ = 0.16/(LOGRATIO**2)
                IF(RIB.LE.0.)THEN
                   AA = ASQ*9.4*SQRT(RATIOZ)
                   CM = 7.4*AA
                   CH = 5.3*AA
                   FM = 1. - (9.4*RIB/(1. + CM*SQRT(ABS(RIB))))
                   FH = 1. - (9.4*RIB/(1. + CH*SQRT(ABS(RIB))))
                ELSE
                   FM = 1./((1. + 4.7*RIB)**2)
                   FH = FM
                ENDIF
                USTARSQ = ASQ*U2**2*FM
                UTSTAR = ASQ*U2*DELTAT*FH/0.74
                USTAR = SQRT(USTARSQ)
                THSTAR = UTSTAR/USTAR
                EL = TBAR*USTARSQ/(0.4*9.8*THSTAR)
! ***
             ENDIF
             KUI=1./(K*USTAR)
             USI(IL,I)=USTAR
!
!---------------COMPUTE RA---------------
             Z=Z2
             ZL=Z/EL
             IF(ZL.GE.0.) THEN
                RA(IL,I)=KUI*(.74*ALOG(Z/ZZ0(IL))+4.7*ZL)
             ELSE
                RA(IL,I)=KUI*.74*(ALOG(Z/ZZ0(IL))-    &
     &                         2*ALOG((1+SQRT(1-9.*ZL))*0.5))
             ENDIF
                RA(IL,I)=AMAX1(RA(IL,I),0.99)
                RA(IL,I)=AMIN1(RA(IL,I),999.9)

!----------------COMPUTE RD AND RC--------------
!
! --- RD FOR GASES
!
             RD=5./USI(IL,I)*SC(J)**.666667
!
! --- RC FOR GASES
!
! --- FOR VEGETATION
!
             D=VI/SC(J)
!
! --- STOMATAL PORE RESISTANCE
!
             RP=P/(B*D)
!
! --- MESOPHYLL RESISTANCE
!
             RM=RM0(J)
!
! --- TOTAL STOMATAL RESISTANCE
!
             RST=RP+RM
!
! --- CUTICLE RESISTANCE
!
             RCUT=A0/AC(J)*RCUT0
      IF((ISEA.LE.2.OR.ISEA.EQ.5).AND.(J.EQ.1.OR.J.EQ.3))  &
     &           RCUT = 3000. * A0 / AC(J)
!
! --- FOLIAGE RESISTANCE INCLUDING THE EFFECT OF WETTED LEAVES
!
       RCF=1./(1./RST+(1.-PCWC)/RCUT+PCWC/RCW(J)/(LAI(I,ISEA)+0.01))
!
! --- GROUND RESISTANCE (SOIL OR SNOW)
!
             RCG=A0/AC(J)*RCG0
             IF (J.EQ.1.OR.J.EQ.3) THEN
                RCG = 800.
              IF (ISEA.NE.4.OR.ISEA.NE.3)   &
     &             RCG = 1000.* A0 / AC(J)
             ENDIF
             RSN=A0/AC(J)*RSN0
             IF (J.EQ.1.OR.J.EQ.3) THEN
                 IF (ISEA.EQ.4.OR.ISEA.EQ.3) RSN = 1000.
             ENDIF

!
! --- TOTAL CANOPY RESISTANCE INCLUDING RESISTANCE TO WATER SUFACES,
! ---  FOLIAGE, BARE GROUND, AND SNOW OR ICE
!
             IF ((ISEA.EQ.3.OR.ISEA.EQ.4).AND.(J.EQ.1.OR.J.EQ.3)) THEN

! ---   RC=1./(FSNOW/RSN+(1.-FSNOW)/RCG+(TS+20.-273.)/(100.*(1.+PCWC)))
!
                 TSCENT = TS(IL) - 273.16
                 TSCENT = AMAX1(-15.,TSCENT)
                 TSCENT = AMIN1(20.,TSCENT)
                 RC=1./(FSNOW/RSN+(1.-FSNOW)/RCG+(TSCENT+20.)/   &
     &                                             (10000.*(1.+PCWC)))
             ELSE
                 RC=1./(LAI(I,ISEA)/RCF+IW*(1.-FICE)/RGW(J)+   &
     &                  (1-IW)*(1.-FSNOW)/RCG+((1-IW)*FSNOW+FICE)/RSN)
             ENDIF
!--------------------COMPUTE VD FOR GASES------------------
! --- AVERAGE DEPOSITION VELOCITIES OVER LAND USE CATEGORIES.
!
             VDF = 1./(RA(IL,I)+RD+RC)
             VDG(IL,J) = VDG(IL,J) + VDF*F(IL,I)
! --- END OF GRID LOOP
700        CONTINUE
! --- END OF LAND USE CATEGORY LOOP
800     CONTINUE
! --- END OF GAS SPECIES LOOP
900   CONTINUE
      RETURN                                                             
      END
