      SUBROUTINE SFBC (TRAC,  DSHJ,   ILG,   LEV,  ILEV,  HI,   &
     &                    T,   NTR,    NN,   NTP, ISIZE,ZHLEV,  &
     &             AEROSIZE, RHOP0,RSFROW,PRESSG,    FX,  MAE,  &
     &               BLCROW,   PBC,   IL1,   IL2,  IAE1)
!-----------------------------------------------------------------------
!
!     HISTORY:
!
!     * MAR 11/199  - D. LAVOUE   DISTRIBUTE BLACK CARBON OF 5 TYPES
!     *               S.L. GONG   OVER THE INJECTION HEIGHT.
!     *                           ACCORDING TO:
!     *                                -----------------
!     *                                                   50.0%
!     *                                -----------------
!     *                                                   25.0%
!     *                                -----------------
!     *                                                   12.5%
!     *                                -----------------
!     *                                                   6.25%
!     *                                -----------------
!     *                                                    :
!     *                                -----------------
!     *
!     * FEB 02/1999 - S.L. GONG   NEW BC EMISSION INVENTORY [     ] 
!     *                           WAS ADDED WHICH CONTAINS 5 TYPES OF
!     *                           CONTRIBUTIONS AND AN INJECTION
!     *                           HEIGHT
!     *
!     * JAN 02/1998 - S.L. GONG   AEROSOL BLACK CARBON SURFACE FLUX
!                                 FOR COOK AND WILSON DATA SET
!
!     METHOD:
!     -------
!
!     ARGUMENTS:
!     ----------
!     SUBROUTINE CALLED FROM SFFLUX
!
!-----------------------------------------------------------------------

      implicit none

      REAL WW,TW,RAYON,ASQ,G,RGAS,RGOCP,  &
     &     RGOASQ,CPRES,RGASV,CPRESV
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES         
      COMMON /PARAMS/ RGASV,CPRESV                                      

      REAL DELT
      INTEGER IFDIFF,KSTART,KTOTAL,NEWRUN,NDAYS,NSECS,  &
     &        IDAY,LDAY,MDAY,INCD
      COMMON  /TIMESc/ DELT,IFDIFF,KSTART,KTOTAL,NEWRUN,     &
     &                NDAYS,NSECS,IDAY,LDAY,MDAY,INCD                   

      integer ILG, LEV, ILEV, NTR, NN, NTP, ISIZE,  &
     &         MAE, IL1, IL2, IAE1
      REAL TRAC   (ILG,LEV,NTR), DSHJ(ILG,ILEV), T(ILG,LEV)             
      REAL AEROSIZE(2,ISIZE),RHOP0(NTP),PRESSG(ILG)
      REAL RSFROW(ILG,NTR), BLCROW(ILG,5,2)
      REAL PBC(ISIZE), FX(ILEV,ILEV), ZHLEV(ILG,ILEV), HI(ILG)
! local variables
      integer I, IL, N, L, L1, NP, IDP
      real PTOT, TT, RWI, HII, TL
!
!     ASSUME THE INITIAL BC IS HYDROPHOBIC AND HAS A LOG-NORMAL
!     DISTRIBUTION WITH A MEAN DIAMETER OF 0.1 UM [BERNER ET AL. 1996]
!     HERE 1.33 = (2*3.1415926)^0.5*LN 1.7
!

      PTOT=0.0
      TT=1.E-3/(30.*24.*3600.)
      DO N=1, ISIZE
         RWI = (AEROSIZE(1,N)+AEROSIZE(2,N))*0.5
!        PBC(N) = 1./1.33 *EXP (-(LOG(RWI)-LOG(5.0E-8))**2/
         PBC(N) = EXP (-(LOG(RWI)-LOG(5.0E-8))**2/  &   ! Since the ratio is used, wzz
     &                          (2.0*(LOG(1.7))**2))
         PTOT=PTOT+PBC(N)
      END DO
! wzz, 2008.2.13
      DO N=1, ISIZE
         PBC(N) = PBC(N)/PTOT*TT
      END DO

!
!     *  INSERTION DU FLUX DE SURFACE
!
      CALL PUTZERO (FX, ILEV*ILEV)
      DO L=1+MAE,ILEV
         DO L1=L,ILEV-1
            IF( L1 .EQ. L) THEN
               FX(L1,L)=0.5
            ELSE
               FX(L1,L)=FX(L1-1,L)*0.5
            END IF
         END DO
         FX(ILEV,L)=FX(ILEV-1,L)
         IF(L.EQ.ILEV) FX(ILEV,L)=1.0
      END DO

!      write(0,*)'wzz: il1=', il1, il2
      DO 900 IDP=1,5
         CALL PUTZERO(HI,ILG)
         DO L=ILEV,2,-1
           DO IL=IL1,IL2
             HII=BLCROW(IL,IDP,2)+100.0          !Fossil Fuel Height
             IF (BLCROW(IL,IDP,1) .GT. 0.0 .AND.   &
     &           HII.GT.ZHLEV(IL,L) .AND. HII.LE.ZHLEV(IL,L-1)) THEN
               HI(IL)=REAL(L-1)
             END IF
             IF (BLCROW(IL,IDP,1) .GT. 0.0 .AND.  &
     &           HII.GT.0.0 .AND. HII.LE.ZHLEV(IL,ILEV))    THEN
               HI(IL)=REAL(ILEV)
             END IF
           END DO
         END DO

!        write(0,*)'wzz: HI=', HI(1),BLCROW(1,IDP,1),BLCROW(1,IDP,2), HII,
!    &         (ZHLEV(1,l),l=ilev,ilev-5,-1),IDP,ilev, mae
!
         DO L=1+MAE,ILEV
         DO 842 N = 1,ISIZE
            NP=ISIZE*(NN-1)+N+(IAE1-1)
            DO 842 I=IL1,IL2
               IF (HI(I) .GT. 0. .AND. L .GE. INT(HI(I))) THEN      
!                 TL=BLCROW(I,IDP,1)*PBC(N)/PTOT*FX(L,INT(HI(I)))*TT
                 TL=BLCROW(I,IDP,1)*PBC(N)*FX(L,INT(HI(I)))
                 RSFROW(I,NP)=RSFROW(I,NP)+TL
                 TRAC(I,L+1,NP) = TRAC(I,L+1,NP) + TL  &
     &                            /(DSHJ(I,L)*PRESSG(I)/G)*2.*DELT        
               END IF
  842    CONTINUE  
         END DO
  900 CONTINUE                                                        

      RETURN                                                          
      END
