      SUBROUTINE SFFLUX (TRAC,  FLAND,  DSHJ,    IL1,    IL2,  ILG, LEV, &
     &                   ILEV,    NTR, ICOMP,SUBNAME, T,FINTROW,VOLCROW, &
     &                ASO4ROW,PSO4ROW,BLCROW,DMSOROW,DMSLROW,H2SROW, GT, &
     &                ASO2ROW,PSO2ROW, ISIZE,  RHOP0,  ZHLEV, PXNEW,RGD, &
     &               AEROSIZE, SURFWD,RSFROW, PRESSG,   ICAM,   ROA, GC, &
     &                   IAE1,   IH2S,  IDMS,   ISO2,   ISO4, IREST,LUC, &
     &                   I_SS, OMCROW,SAND2ROW,CLAY2ROW,FSNOW,SREL,NATS, &
     &                 NSOIL,   MODE,   MAE,   NS,  Z0, SOILW,SS,SURFDG, &
     &                SAND6ROW, TBAR, trnch)
!     &                SAND6ROW, TBAR,   RAINFALL)
!------------------------------------------------------------------------------
!     PURPOSE:
!     --------
!     CALCULATE SURFACE FLUXES FOR AEROSOLS
!
!     HISTORY:
!     --------
!
!     * DEC. 22/2000 S.L. GONG   SOIL DUST EMISSION SCHEME IS ADDED
!     *                          GLOBAL 1x 1 SOIL TEXTURE DATA WAS
!     *                          USED FROM GCM12.
!     * MAR. 13/99 - S.L. GONG   GLOBAL OM SURFACE EMISSION [1x 1]
!     *                          WAS ADDED.
!
!     * FEB. 17/98 - S.L. GONG   GLOBAL BLACK CARBON SURFACE EMISSION [1x 1]
!     *                          WAS ADDED.
!     * JAN. 20/98 - S.L. GONG   GLOBAL DMS OCEAN CONCENTRATION DATA IS ADDED
!     *                          THE FLUX IS CALCULATED BY MASS TRANSFER
!     *                          EQUATION
!     * DEC. 20/96 - S.L. GONG   VECTORIZED THE FLUX CALCULATION FOR SEA-SALT
!     *                          AEROSOLS
!     * NOV. 21/94 - S.L. GONG   AEROSOL SURFACE FLUX COMPUTATION
!     *                          FOR SEA-SALT
!     *
!
!------------------------------------------------------------------------------
      implicit none

      REAL WW,TW,RAYON,ASQ,G,RGAS,RGOCP,  &
     &     RGOASQ,CPRES,RGASV,CPRESV
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES
      COMMON /PARAMS/ RGASV,CPRESV
!  wzz

      integer ILG, LEV, ILEV
      integer NTR, ICOMP
      integer IL1, IL2, I_SS, ICAM, ISIZE, NATS, NSOIL, MODE, MAE, NS
      integer IAE1, IH2S, IDMS, ISO2, ISO4, IREST, LUC
      integer trnch

      REAL T(ILG,LEV), ZHLEV(ILG,ILEV), GT(ILG), GC(ILG)
      REAL FLAND(ILG,LUC), TRAC(ILG,LEV,NTR), DSHJ(ILG,ILEV)
      REAL ASO4ROW(ILG),PSO4ROW(ILG),BLCROW(ILG,5,2),OMCROW(ILG,5,2)
      REAL ASO2ROW(ILG),PSO2ROW(ILG), VOLCROW(ILG,2)
      REAL RGD(ILG,ILEV,NTR), ROA(ILG,ILEV), FINTROW(ILG,ISIZE,3)
      REAL DMSOROW(ILG),DMSLROW(ILG),H2SROW(ILG)
      REAL AEROSIZE(2,ISIZE),SURFWD(ILG),RHOP0(ICOMP)
      REAL RSFROW(ILG,NTR),PRESSG(ILG), PXNEW(ILG,ILEV,NTR)
      REAL SAND2ROW(ILG,NATS),CLAY2ROW(ILG,2),SREL(ILG,NSOIL,NATS)
      REAL Z0(LUC,NS), SS(ILG,NSOIL),SURFDG(ILG),SOILW(ILG)
      REAL FSNOW(ILG),SAND6ROW(ILG,6), TBAR(ILG)
      CHARACTER*8 SUBNAME(ICOMP)
!      REAL RAINFALL(ILG)

      integer I, N, NN
!
!     * ZERO SURFACE FIELDS OF CAM COMPONENTS
!
      DO N = IAE1, ICAM
        DO I = IL1, IL2
          RSFROW(I,N) = 0.0
        END DO
      END DO
!
!     * SURFACE FLUX CALCULATION
!
      DO NN=1,ICOMP
!
!      * SULPHATE AEROSOLS
!-------------------------
          IF (SUBNAME(NN) .EQ. 'SULPHATE') THEN
               CALL SFSF (TRAC,   DSHJ,     ILG,    LEV,    ILEV,  MAE,  &
     &                     NTR,      NN,  ISIZE,AEROSIZE,VOLCROW,        &
     &                 ASO4ROW,PSO4ROW, ASO2ROW,PSO2ROW,  PRESSG,  ROA,  &
     &                 DMSOROW,DMSLROW,  H2SROW,  PXNEW,  RSFROW,  RGD,  &
     &                  SURFWD,  ZHLEV,   FLAND,    IL1,     IL2, IAE1,  &
     &                      GT,   IH2S,    IDMS,   ISO2,  LUC)
!    &                      GT,   IH2S,    IDMS,   ISO2,  LUC, GFEDROW,
!    &               OMCROW, BLCROW)
          END IF
!
!      * SEA-SALT AEROSOLS
!     if(trnch.eq.153) write(0,*)'wzz: TRAC 0=', TRAC(1,LEV,13), NN, SUBNAME
!-------------------------
          IF (SUBNAME(NN) .EQ. 'SEA-SALT') THEN
            CALL SFSS (TRAC,  DSHJ,  ILG,    LEV,  ILEV,  &
     &            PXNEW(1,1,1), PXNEW(1,1,2),PXNEW(1,1,3),PXNEW(1,1,4),  &
     &    PXNEW(1,1,5),  I_SS,  T,   NTR,   NN,  ICOMP, ISIZE, LUC,  &
     &                AEROSIZE,SURFWD,RHOP0, RSFROW,PRESSG,   GC,  &
     &                   FLAND,FINTROW, IL1,  IL2,    IAE1,IREST, trnch)
          END IF
!     if(trnch.eq.153) write(0,*)'wzz: TRAC 1=', TRAC(1,LEV,13), NN

!
!      * BLACK-CARBON AEROSOLS
!-------------------------
          IF (SUBNAME(NN) .EQ. 'BLCARBON') THEN
               CALL SFBC (TRAC,  DSHJ, ILG,  LEV,  ILEV,PXNEW(1,1,2),  &
     &                       T,   NTR,     NN,  ICOMP, ISIZE,ZHLEV,   &
     &                AEROSIZE,RHOP0,RSFROW,PRESSG,PXNEW(1,1,3),MAE,  &
     &                  BLCROW, PXNEW(1,1,1), IL1,    IL2,  IAE1)
          END IF
          
!
!      * ORGANIC-CARBON AEROSOLS
!-------------------------
          IF (SUBNAME(NN) .EQ. 'OMCARBON') THEN
               CALL SFOC (TRAC,  DSHJ, ILG,  LEV,  ILEV,PXNEW(1,1,2), &
     &                       T,   NTR,     NN,  ICOMP, ISIZE,ZHLEV, &
     &                AEROSIZE,RSFROW, PRESSG,PXNEW(1,1,3), MAE, &
     &                  OMCROW, PXNEW(1,1,1), IL1,    IL2,  IAE1)
          END IF


!
!     * SOLID DUSTS AEROSOLS
!--------------------------
!    
          IF (SUBNAME(NN) .EQ. 'SOILDUST') THEN
               CALL SFSD (TRAC,  DSHJ,   ILG,   LEV,  ILEV, &
     &                  SURFDG,   NTR,    NN, ICOMP, ISIZE, &
     &                AEROSIZE, RHOP0,RSFROW,PRESSG, FLAND, &
     &                  SURFWD,   ROA,   IL1,   IL2,  IAE1, &
     &         PXNEW(1,1,1),PXNEW(1,1,2),  SREL,PXNEW(1,1,3), &
     &     SAND2ROW,CLAY2ROW,SOILW,PXNEW(1,1,4),PXNEW(1,1,5), &
     &         PXNEW(1,1,7),PXNEW(1,1,8), &
     &                  NS,  Z0,   LUC, PXNEW(1,1,6),  NATS, &
     &         NSOIL, MODE,PXNEW(1,1,9),SS, PXNEW(1,1,10), &
     &     FSNOW,PXNEW(1,1,11),PXNEW(1,1,12),PXNEW(1,1,13), &
     &     SAND6ROW, TBAR, trnch)
!!     9     SAND6ROW, TBAR, RAINFALL)
          END IF

!     if(trnch.eq.153) write(0,*)'wzz: TRAC 2=', TRAC(1,LEV,13), NN

      END DO
!
      RETURN
      END
