      SUBROUTINE SFSF (TRAC,   DSHJ,    ILG,    LEV,    ILEV,   MAE,  &
     &                  NTR,     NN,  ISIZE,AEROSIZE, VOLCROW,        &
     &              ASO4ROW,PSO4ROW,ASO2ROW,PSO2ROW,  PRESSG,ROAROW,  &
     &              DMSOROW,DMSLROW, H2SROW,   FRAC,  RSFROW,   RGD,  &
     &               SURFWD,  ZHLEV,  FLAND,    IL1,     IL2,  IAE1,  &
     &                   GT,   IH2S,   IDMS,   ISO2,     LUC)
!------------------------------------------------------------------------------
!
!     HISTORY:
!     * JAN 30/1998 - S.L. GONG   ADDING DMS OCEAN SURFACE CONCENTRATION
!     *                           AND FLUX FORMULATION. REPLACING PREVIOUS
!     *                           FLUX INPUT FILE
!     *
!     * MAY 26/1997 - S.L. GONG   FIRST IMPLEMENTED
!     *                           OPTIMIZED BY VECTORIZATION
!
!     ----------
!     SUBROUTINE CALLED FROM SFFLUX
!
!------------------------------------------------------------------------------

      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES         !          SF
      COMMON /PARAMS/ RGASV,CPRESV                                      !          SF
      COMMON /TIMESc/ DELT,IFDIFF,KSTART,KTOTAL,NEWRUN,  &              !          SF
     &                NDAYS,NSECS,IDAY,LDAY,MDAY,INCD                   !          SF
      REAL TRAC   (ILG,LEV,NTR), DSHJ(ILG,ILEV), GT(ILG)
      REAL PRESSG(ILG),ZHLEV(ILG,ILEV),SURFWD(ILG)
      REAL RSFROW(ILG,NTR), AEROSIZE (2,ISIZE)
      REAL ASO4ROW(ILG),PSO4ROW(ILG),ASO2ROW(ILG),PSO2ROW(ILG)
      REAL DMSOROW(ILG),DMSLROW(ILG),H2SROW(ILG), VOLCROW(ILG,2)
      REAL RGD (ILG,ILEV,NTR), ROAROW(ILG,ILEV)
      REAL FRAC(ISIZE), FLAND(ILG,LUC)
      REAL HEIGHT
      DATA HEIGHT/100.0/
!
!     * SURFACE FLUX - UNIT KG M-2 S-1
!       FACTOR TT CONVERTS EMISSION UNIT FROM MT M-2 SEASON-1
!       TO KG M-2 S-1
!
      TT=1000.0/(365.*24.*3600./4.)
!
!     * SIZE DISTRIBUTION OF PRIMARY SULPHATE AEROSOLS
!       r=0.125 um, std dev=1.7
!
!     MMD 0.25 um
      AMEAN = LOG10(0.25)
      ASIGMA = 0.17
      TOTV=0.0
      DO N=1,ISIZE
          RWI = (AEROSIZE(1,N)+AEROSIZE(2,N))*0.5*1.E6
          ALOGDI = LOG10(2.*RWI)
!         MMD 0.25 um
!         AMEAN = LOG10(0.25)
!         ASIGMA = 0.17
          FRAC(N) = EXP (-(ALOGDI - AMEAN)**2./ASIGMA)               
          TOTV=TOTV+FRAC(N)
      END DO
      DO N=1,ISIZE
          FRAC(N)=FRAC(N)/TOTV
          IF (FRAC(N) .LT. 1.E-9)  FRAC(N)=0.0
      END DO
!        print *, 'FRAC=', FRAC

!
!     * FOR PRIMARY SULPHATE AEROSOLS
!
      DO N=1, ISIZE
         NF=ISIZE*(NN-1)+N+(IAE1-1)
         DO IL=IL1,IL2
            RSFROW(IL,NF) = ASO4ROW(IL)*FRAC(N)*TT
            TRAC(IL,ILEV+1,NF) = TRAC(IL,ILEV+1,NF) + RSFROW(IL,NF)  &
     &                      /(DSHJ(IL,ILEV)*PRESSG(IL)/G)*2.*DELT
            RSFROW(IL,NF) = RSFROW(IL,NF)+PSO4ROW(IL)*FRAC(N)*TT
         END DO
!
!      * HIGHER POINT SOURCES
!
         DO L=ILEV,2,-1
           DO IL=IL1,IL2
             IF(HEIGHT.GT.ZHLEV(IL,L).AND.HEIGHT.LE.ZHLEV(IL,L-1))THEN
                TRAC(IL,L,NF)=TRAC(IL,L,NF)+PSO4ROW(IL)*FRAC(N)*TT  &
     &                        /(DSHJ(IL,L-1)*PRESSG(IL)/G)*2.*DELT
             END IF
!            IF(HEIGHT.GT.0.0.AND.HEIGHT.LE.ZHLEV(IL,ILEV))THEN
!               TRAC(IL,LEV,NF)=TRAC(IL,LEV,NF)+PSO4ROW(IL)*FRAC(N)*TT  &
!    &                        /(DSHJ(IL,ILEV)*PRESSG(IL)/G)*2.*DELT
!            END IF
           END DO
         END DO
      END DO
      DO IL=IL1,IL2
             IF(HEIGHT.GT.0.0.AND.HEIGHT.LE.ZHLEV(IL,ILEV))THEN
                TRAC(IL,LEV,NF)=TRAC(IL,LEV,NF)+PSO4ROW(IL)*FRAC(N)*TT  &
     &                        /(DSHJ(IL,ILEV)*PRESSG(IL)/G)*2.*DELT
             END IF
      END DO
!
!     * FOR THE PRECUSORS DMS OF SULPHATE
!       FIRST USE WANNIKHOF MASS TRANSFER COEFFICIENT TO RELATE THE
!       DMS SURFACE FLUX WITH DMS OCEANIC CONCENTRATION
!       ZKL -- M S-1
!
      DO IL=IL1,IL2
         IF (FLAND (IL,14) .GT. 0.1) THEN
            GTC=GT(IL)-273.0
            SC=2674.0-147.12*GTC+3.726*GTC*GTC-   &
     &                           0.038*GTC*GTC*GTC
            SC=AMIN1(AMAX1(SC, 577.0), 2027.0)
            CONV=0.01/3600.
!
!     WANNINKOF SCHEME
!
            ZKL=0.31*SURFWD(IL)*SURFWD(IL)/SQRT(SC/660.0)*CONV
!
!     LISS AND MERLIVAT SCHEME
!
!            IF (SURFWD(IL) .LT. 3.6) THEN
!               ZKL=0.17*SURFWD(IL)*(SC/660.0)**(-2./3.)*CONV
!            END IF 
!            IF (SURFWD(IL) .GE. 3.6 .AND. SURFWD(IL) .LT. 13.) THEN
!               ZKL=(2.85*SURFWD(IL)-9.65)/SQRT(SC/660.0)*CONV
!            END IF            
!            IF (SURFWD(IL) .GE. 13.0 ) THEN
!               ZKL=(5.9*SURFWD(IL)-49.3)/SQRT(SC/660.0)*CONV
!            END IF
!
!     HENRY'S LAW CONSTANT FOR DMS
!
            if(GT(IL).lt.100.) GT(IL)=285.
!            print *,' GT(IL)=',GT(IL), IL
            H=EXP(-3547.0/GT(IL)+12.64)      !  [ATM L MOL-1]
!            print *, '0 H =', H
            H=H/(0.08205*GT(IL))             !  [DIMENSIONLESS]
!            print *, '1 H =', H
            RDMS=AMAX1(RGD(IL,ILEV,IDMS),0.0)
!            print *, 'DMS=',ZKL, DMSOROW(IL),RDMS,ROAROW(IL,ILEV),H
            RSFROW(IL,IDMS)=ZKL*(DMSOROW(IL)*62.128E-9-   &
     &                  RDMS*ROAROW(IL,ILEV)/H)
            RSFROW(IL,IDMS)= AMAX1(RSFROW(IL,IDMS),0.0)*FLAND(IL,14)
!          if(RSFROW(IL,IDMS).gt.0.and.il.eq.14) write(0,*)'wzz: DMS=', ZKL, 
!    &    DMSOROW(IL),RDMS,ROAROW(IL,ILEV),H, FLAND(IL,14), RSFROW(IL,IDMS)
         END IF
      END DO

      DO IL=IL1,IL2
         RSFROW(IL,IH2S)=H2SROW(IL)*TT*34.0/32.0
         RSFROW(IL,IDMS)=RSFROW(IL,IDMS)+DMSLROW(IL)*TT*62.128/32.0
         RSFROW(IL,ISO2)=(ASO2ROW(IL)+PSO2ROW(IL))*TT
         TRAC(IL,ILEV+1,IH2S)=TRAC(IL,ILEV+1,IH2S)+RSFROW(IL,IH2S)  &
     &                     /(DSHJ(IL,ILEV)*PRESSG(IL)/G)*2.*DELT
         TRAC(IL,ILEV+1,IDMS)=TRAC(IL,ILEV+1,IDMS)+RSFROW(IL,IDMS)  &
     &                     /(DSHJ(IL,ILEV)*PRESSG(IL)/G)*2.*DELT
         TRAC(IL,ILEV+1,ISO2)=TRAC(IL,ILEV+1,ISO2)+ASO2ROW(IL)*TT  &
     &                     /(DSHJ(IL,ILEV)*PRESSG(IL)/G)*2.*DELT
      END DO
!
!      * HIGHER POINT SOURCES
!
      DO L=ILEV,2+MAE,-1
         DO IL=IL1,IL2
           IF (HEIGHT.GT.ZHLEV(IL,L).AND.HEIGHT.LE.ZHLEV(IL,L-1)) THEN
               TRAC(IL,L,ISO2)=TRAC(IL,L,ISO2)+PSO2ROW(IL)*TT  &
     &                        /(DSHJ(IL,L-1)*PRESSG(IL)/G)*2.*DELT
           END IF
!          IF (HEIGHT.GT.0.0.AND.HEIGHT.LE.ZHLEV(IL,ILEV)) THEN
!              TRAC(IL,LEV,ISO2)=TRAC(IL,LEV,ISO2)+PSO2ROW(IL)*TT  &
!    &                        /(DSHJ(IL,ILEV)*PRESSG(IL)/G)*2.*DELT
!          END IF
         END DO
      END DO
      DO IL=IL1,IL2
           IF (HEIGHT.GT.0.0.AND.HEIGHT.LE.ZHLEV(IL,ILEV)) THEN
               TRAC(IL,LEV,ISO2)=TRAC(IL,LEV,ISO2)+PSO2ROW(IL)*TT  &
     &                        /(DSHJ(IL,ILEV)*PRESSG(IL)/G)*2.*DELT
           END IF
      END DO
!
!     * VOLCANIC EMISSIONS [THIS PART OF THE CODE WAS MODIFIED FROM LOHMANN'S
!       ORIGINAL VOLCANO CODE, WITH THANKS]
!
      JT=ISO2
      DO L=1+MAE,ILEV
         ZVOLCEMI=8.
         ZVOLCEMI1=ZVOLCEMI*0.36
         ZVOLCEMI2=ZVOLCEMI*0.36
         ZVOLCEMI3=ZVOLCEMI*0.28
         DO IL=IL1,IL2
           HTV=VOLCROW(IL,2)
           IF (HTV.GT.ZHLEV(IL,L).AND.HTV.LE.ZHLEV(IL,L-1)) THEN 
              ZDP1 =G/(DSHJ(IL,L)*PRESSG(IL))
              TRAC(IL,L+1,JT)=TRAC(IL,L+1,JT)+  &
     &                    ZVOLCEMI1*VOLCROW(IL,1)*ZDP1 *2.*DELT
           END IF
         END DO
         IF (L .EQ. ILEV) THEN
           DO IL=IL1,IL2
!
!    VOLCANIC BACKGROUND EMISSIONS
!
!      ZVOLCEMI  TOTAL VOLCANIC EMISSION SCALED TO 8 TG/YR IN KG/M2/S
!
            IF(VOLCROW(IL,2).GT.0.) THEN
!   wzz, ????
              ZDP13=G/(DSHJ(IL,13)*PRESSG(IL))
              ZDP14=G/(DSHJ(IL,14)*PRESSG(IL))
              ZDP18=G/(DSHJ(IL,25)*PRESSG(IL))
              ZDP19=G/(DSHJ(IL,26)*PRESSG(IL))

              RSFROW(IL,ISO2)=RSFROW(IL,ISO2)+VOLCROW(IL,1)*ZVOLCEMI
              TRAC(IL,27,JT)=TRAC(IL,27,JT)+   &
     &                0.5*ZVOLCEMI2*VOLCROW(IL,1)*ZDP19*2.*DELT
              TRAC(IL,26,JT)=TRAC(IL,26,JT)+   &
     &                0.5*ZVOLCEMI2*VOLCROW(IL,1)*ZDP18*2.*DELT
              TRAC(IL,15,JT)=TRAC(IL,15,JT)+   &
     &                0.5*ZVOLCEMI3*VOLCROW(IL,1)*ZDP14*2.*DELT
              TRAC(IL,14,JT)=TRAC(IL,14,JT)+   &
     &                0.5*ZVOLCEMI3*VOLCROW(IL,1)*ZDP13*2.*DELT
            END IF
           END DO
         END IF
      END DO

      END SUBROUTINE SFSF
