      SUBROUTINE CAS (  T,    COLEF,    RHOP,  ROAROW,  &                        !
     &             RHSIZE,   PRESSG,      QR,     MAE,  &                        !
     &                ILG,     ILEV,     LEV,   ISIZE,  &
     &                IL1,     IL2,    PDIFF,  &
     &              PDEPV,      SHJ)
!    &                IL1,     IL2,      VPR,   PDIFF,
!    &              PDEPV,      SHJ,     RRM,   ALPHA)                           !
!-----------------------------------------------------------------------------
!     PURPOSE:
!     --------
!     THIS IS A MODULE OF CALCULATING THE COLLECTION EFFICIENCY OF
!     THE AEROSOLS OF TYPE N BY COLLECTOR DROPLETS OF RADIUS RCOL.
!
!     HISTORY:
!     --------
!     *
!     * DEC 19/96 - S.L. GONG   VECTORIZED THE WHOLE PROGRAM AND ADD SOME
!     *                         WORKING SPACES.
!     *
!     * JUL  8/94 - S.L. GONG   FIRST VERSION
!
!     METHOD:
!     -------
!-----------------------------------------------------------------------------
!
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES                   !
      COMMON /PARAMS/ RGASV,CPRESV                                                !
      COMMON /NARCM /   AVNO,    RGASi,     AM,   BOLTZK,  PI,  A                 !
      COMMON /NARCM/ A0,RSN0,RCUT0,RCG0,PNON
      COMMON /HTCP  / T1S,T2S,AI,BI,AW,BW,SLP                                     !
      REAL T(ILG,LEV), RHOP(ILG,ILEV,ISIZE), ROAROW(ILG,ILEV)
      REAL QR(ILG,ILEV)
!     REAL QR(ILG,ILEV), VPR(ILG)                                                 !
      REAL RHSIZE(ILG,ILEV,ISIZE)
      REAL COLEF(ILG,ILEV,ISIZE), PRESSG(ILG)
      REAL PDEPV(ILG,ILEV,ISIZE), SHJ(ILG,ILEV)
      REAL PDIFF(ILG,ILEV,ISIZE)                                                  !
!
!     VECTORIZATION SPACES
!
!     REAL RRM(ILG), ALPHA(ILG)                                                   !		 
      REAL RRM, ALPHA, VPR                                              !		 
      DATA RHORAIN/1000.0/                                                        !
      DATA  AA1/1.249/, AA2/0.42/, AA3/0.87/                                      !
      AMUW = 1.002E-3                                                             !at 20 C [kg /m/sec]           !
!     T1S=273.15    !????? add by zhouch, no set before????????
!      print*,'in cas',is_x0(il1),ie_x0(il2)
      DO N=1,ISIZE
        DO L=1+MAE,ILEV                                                           !
          DO I=IL1,IL2                                                            !
!
!     * IN CASE OF PRECIPITATION
!
!           VPR(I)=0.0
            VPR=0.0
!           IF (QR(I,L) .GT. 1.E-15) THEN                                         !
            IF (QR(I,L) .GT. 1.E-20) THEN  ! keep consistent with that in BLCLD, wzz   !
               TL=T(I,L+1)                                                        !
               R=RHSIZE(I,L,N)                                                    !   
!
!     * AIR'S DYNAMIC VISCOSITY
!
               AMU=145.8*1.E-8*TL**1.5/(TL+110.4)                                 !
!
!     * MEAN MOLECULAR FREE PATH.
!       K.V. BEARD [1976], J ATM. SCI., 33
!
               AMFP=6.54E-8*(AMU/1.818E-5)*(1.013E5/(PRESSG(I)  &                !
     &              *SHJ(I,L)))*(TL/293.15)**0.5
!
!     * STOKES FRICTION AND DIFFUSION COEFFICIENTS.
!
               SCHM=AMU/PDIFF(I,L,N)/ROAROW(I,L)                                  !  !
!
!       COLLECTOR DROP SIZE
!
               IF (TL .GT. 273.15) THEN                                           !
!
!     * FOR RAIN
!
!     * MASS MEAN RAINDROP RADIUS RRM
!     * FACTOR 1.E-3 CONVERTS MM INTO M.
!
                 RRM=0.35*(QR(I,L)*3600.)**0.25*1.E-3                          !
                 PRII=2./9.*G/AMU                                                 !
                 PRIIV=PRII*(RHORAIN-ROAROW(I,L))                                 !
!
!     * CUNNINGHAM SLIP CORRECTION FACTOR
!       SETTLING VELOCITY
!
                 CFAC=1.+AMFP/RRM*(AA1+AA2*EXP(-AA3*RRM/AMFP))              !
                 VPR=PRIIV*RRM*RRM*CFAC                                  !
!                VPR(I)=PRIIV*RRM(I)**2*CFAC                                      !
               END IF                                                             !
               IF (TL.LE.T1S.AND.TL.GE.T1S-8.) THEN                               !
!
!     * NEEDLE SNOW SCAVENGING
!       DATA FROM SLINN(1984) IN ATMOSPHEIC SCIENCE AND POWER
!       PRODUCTION, ED. DARRYL RANDERSON.
!
!       FOR SNOW SCAVENGING, THE DNESITY OF SNOW IS SET AS
!       1/10 OF LIQUID WATER. THE FACTOR 1.0E-2 IN THE WETDEP
!       CALCULATION TAKES THIS INTO ACCOUNT PLUS THE UNIT CHANGE
!       INTO M S-1
!
                  VPR = 50.0E-2                                              !AVERAGE SETTLING VELOCIT!Y [
                  RRM = 10.E-6                                                !CHARACTERISTIC CAPTURE L!EHG
                  ALPHA = 1.0                                                !
               END IF                                                           !
               IF (TL.LT.T1S-8..AND.TL.GE.T1S-25.) THEN                         !
!
!     * STELLER SNOW SCAVENGING
!
                  VPR = 57.0E-2                                              !
                  RRM = 100.E-6                                              !
                  ALPHA = 0.5                                                !
               END IF                                                           !
               IF (TL .LT. T1S-25.) THEN                                        !
!
!     * GRAUPEL SCAVENGING
!
                 VPR = 180.0E-2                                              !
                 RRM = 1000.E-6                                              !
                 ALPHA = 2./3.                                               !
               ENDIF                                                            !
!
!     * REYNOLD NUMBER
!
               RE=RRM*VPR*ROAROW(I,L)/AMU                                 !
!
!     * STOKES NUMBER OF COLLECTED PARTICELS
!
               ST=PDEPV(I,L,N)/G*(VPR - PDEPV(I,L,N))  &
     &                                           /RRM                 !
!
!     * RATIO OF RADIUS OF COLLECTED PARTICEL AND COLLECTOR DROP
!
               RR = R/RRM
               VR = AMUW/AMU                                                    !
               SSTAR = (1.2+(1./12.)*LOG(1.+RE))/(1.+LOG(1.+RE))                !
               IF (ST .GT. SSTAR )                           THEN               !
!                 COLIMP = ((ST-SSTAR)/(ST-SSTAR+2./3.))**(3./2.)               !
                  COLIMP = ((ST-SSTAR)/(ST-SSTAR+2./3.))**1.5   &           !
     &                   *SQRT(1000.0/RHOP(I,L,N))                              !
               ELSE                                                             !
                  COLIMP = 0.0                                                  !
               END IF                                                           !
               IF (TL .GT. 273.15) THEN                                         !
!
!     * RAIN SCAVENGING EFFICIENCY
!
                 COLEF(I,L,N) = 4./(RE*SCHM)*(1.+0.4*SQRT(RE)*SCHM**  &
     &                      (1./3.)+0.16*SQRT(RE*SCHM)) + 4.*RR*(1./VR  &
     &                         + (1.+2.*SQRT(RE))*RR) + COLIMP                  !
               ELSE                                                             !
!
!     * SNOW SCAVENGING EFFICIENCY
!
                 COLEF(I,L,N) = (1./SCHM)**ALPHA +  &
     &                    (1.0-EXP(-(1+SQRT(RE))*RR*RR)) + COLIMP                !
!    &                    (1.0-EXP(-(1+SQRT(RE))*RR**2)) + COLIMP                !
               ENDIF                                                             !
!
!     * SET THE UPBOUND FOR COLLECTION EFFICIENCY
!
               COLEF(I,L,N) = AMAX1(0.0,AMIN1(1.00,COLEF(I,L,N)))
            END IF                                                               !
          END DO                                                                 !
        END DO
      END DO                                                                     !
!
      RETURN                                                                     !
      END                                                                        !
