      SUBROUTINE CLDCHEM (   IL1,      IL2,     ILEV,     ILG,    ISO2  &
     &                  ,   IHPO,       NN,      NTR,  NTRACA,   ISIZE  &
     &                  ,   IAE1,    RCRIT,    RCOEX,  CLSIZE,   CLDCV  &
     &                  ,  ZMLWC,  H2O2ROW,   ROAROW,      TH, HNO3ROW  &
     &                  , NH3ROW,    O3ROW,   CO2ROW,   RGRID,   RTHPO  &
     &                  ,  RTSO4,    RTSO2,   PRESSG,     SHJ, AERONUM  &
     &                  ,   GRAV,     DSHJ,    IDEEP,      JT,    MAXG  &
     &                  ,     JD,     EPS0,       MU,      EU,      DU  &
     &                  ,     MD,       MC,       ED,      DP,      DZ  &
     &                  ,     MB,      MSG,      ZFG, LENGATH, DSUBCLD  &
     &                  , ASPRVA,   AGAMMA,   ARESID,  ASPECI,  AGTSO2  &
     &                  , AGTSO4,    AGTO3,   AGTCO2,  AGTNH3, AGTHNO3  &
     &                  ,  AGTNA,   AGTHO2,   AOH2O2,   AEQCA,   AEQCB  &
     &                  ,  AEQCC,   AEQCAI,   AEQCBI,  AEQCHP,  AEQCHO  &
     &                  , AEQHNO,   AEQNH3,   AEQHCL,  AORHP1,  AORHP2  &
     &                  ,  AORHO,   ANTSO2,   ANTHO2,  ANTSO4,     AMH  &
     &                  ,HPOXROW,      MUR,      MDR,     DUR,     EUR  &
     &                  ,    EDR,    KCALC,        X,      XD,      XG  &
     &                  ,   DXDT,     XHAT,     DXDX,      XU,    FACX  &
     &                  ,     OU,    SUMDX,      XGA,     XUA,   AFRAC  &
     &                  , AFRACF,        Z,     XROW,   KOUNT,   PXNEW  &
     &                  ,     RU,      RUA,   ASRSO2,  ASRHPO,AEROSIZE  &
     &                  ,  THLEV,   RTBCLD,       ID,   GDREM, MAE, JLAT)
!***********************************************************************
!     Calculates incloud-oxidation and hydrogen peroxide production
!     and cumulus transport of tracers
!***********************************************************************
!
!---  method
!     This subroutine calculates the modifications of sulphur dioxide
!     and aerosol-sulphate concentrations due to incloud-oxidation.
!     It also calculates modifications of hydrogen peroxide concentratio
!     due to in-cloud oxidation and quasi-gas-phase production.
!     Sub-grid-scale stratiform and sub-grid-scale convective clouds
!     are considered.
!     Stratiform clouds: Initial S-species-concentrations are given by m
!                        grid point values; oxidation is calculated at e
!                        grid point for "closed-box"-boundary-values wit
!                        variable concentrations.
!     Convective clouds: Zhang & McFarlane scheme (1995) for S-species
!                        and hydrogen peroxide with stationary source/si
!                        terms during the transport-time-step and consta
!                        concentrations within updraft.
!
!---  HISTORY
!
!     JANUARY 20, 1998 KNUT VON SALZEN     NEW SUBROUTINE
!     JANUARY 27, 1998 KNUT VON SALZEN     SIMPLIFIED VERSION
!     FEBRUARY 2, 1998 KNUT VON SALZEN     MODIFIED FOR GCM
!     FEBUUARY 4, 1998 S.L.  GONG          VECTORIZATION OF WHOLE CODES
!                                          STRUCTUREAL CHANGES
!     APRIL 20, 1998   KNUT VON SALZEN     CORRECTED
!
!---  some important parameters
!
!     AERONUM REAL    INPUT - NUMBER-CONCENTRATION OF AEROSOL PARTICLES
!     AEQCA   REAL    WORK ARRAY
!     AEQCAI  REAL    WORK ARRAY
!     AEQCB   REAL    WORK ARRAY
!     AEQCBI  REAL    WORK ARRAY
!     AEQCC   REAL    WORK ARRAY
!     AEQCHNO REAL    WORK ARRAY
!     AEQCHO  REAL    WORK ARRAY
!     AEQCHP  REAL    WORK ARRAY
!     AEQHCL  REAL    WORK ARRAY
!     AEQNH3  REAL    WORK ARRAY
!     AFRAC   REAL    WORK ARRAY
!     AFRACF  REAL    WORK ARRAY
!     AGAMMA  REAL    WORK ARRAY
!     AGTCO2  REAL    WORK ARRAY
!     AGTHO2  REAL    WORK ARRAY
!     AGTHNO3 REAL    WORK ARRAY
!     AGTNA   REAL    WORK ARRAY
!     AGTNH3  REAL    WORK ARRAY
!     AGTO3   REAL    WORK ARRAY
!     AGTSO2  REAL    WORK ARRAY
!     AGTSO4  REAL    WORK ARRAY
!     AMH     REAL    WORK ARRAY
!     ANTHO2  REAL    WORK ARRAY
!     ANTSO2  REAL    WORK ARRAY
!     ANTSO4  REAL    WORK ARRAY
!     AOH2O2  REAL    WORK ARRAY
!     AORHO   REAL    WORK ARRAY
!     AORHP1  REAL    WORK ARRAY
!     AORHP2  REAL    WORK ARRAY
!     ARESID  REAL    WORK ARRAY
!     ASPECI  REAL    WORK ARRAY
!     ASPRVA  REAL    WORK ARRAY
!     CLDCV   REAL    INPUT - CLOUD COVER (1 -> STRATIFORM CLOUDS,
!                                          2 -> CONVECTIVE CLOUDS), 0 <=
!     CO2ROW  REAL    INPUT - CARBON DIOXIDE CONCENTRATION IN M**3/M**3
!     DP      REAL    INPUT
!     DZ      REAL    INPUT
!     DSHJ    REAL    INPUT
!     DU      REAL    INPUT
!     DUR     REAL    WORK ARRAY
!     DSUBCLD REAL    INPUT
!     DXDT    REAL    WORK ARRAY
!     DXDX    REAL    WORK ARRAY
!     EPS0    REAL    INPUT
!     ED      REAL    INPUT
!     EDR     REAL    WORK ARRAY
!     EU      REAL    INPUT
!     EUR     REAL    WORK ARRAY
!     FACX    REAL    WORK ARRAY
!     GRAV    REAL    INPUT - GRAVITATIONAL ACCELARATION
!     HNO3ROW REAL    INPUT - NITRIC ACID CONCENTRATION IN M**3/M**3
!     HPOXROW REAL    WORK ARRAY - BACKGROUND HYDROGEN PEROXIDE CONCENTR
!                                  IN KG-H2O2/KG-AIR
!     H2O2ROW REAL    INPUT - BACKGROUND HYDROGEN PEROXIDE CONCENTRATION
!                             IN M**3/M**3
!     IAE1    INTEGER INPUT - START INDEX FOR AEROSOL TRACER
!     IDEEP   INTEGER INPUT
!     ILEV    INTEGER INPUT - VERTICAL GRID COORDINATE
!     ILG     INTEGER INPUT - HORIZONTAL GRID COORDINATE
!     IL1     INTEGER INPUT - HORIZONTAL GRID COORDINATE
!     IL2     INTEGER INPUT - HORIZONTAL GRID COORDINATE
!     IHPO    INTEGER INPUT - TRACER INDEX FOR HYDROGEN PEROXIDE
!     ISO2    INTEGER INPUT - TRACER INDEX FOR SULPHUR DIOXIDE
!     ISIZE   INTEGER INPUT - NUMBER OF BINS
!     JD      INTEGER INPUT
!     JT      INTEGER INPUT
!     KCALC   LOGICAL WORK ARRAY
!     LENGATH INTEGER INPUT
!     MAXG    INTEGER INPUT
!     MB      REAL    INPUT
!     MC      REAL    INPUT
!     MD      REAL    INPUT
!     MSG     INTEGER INPUT
!     NH3ROW  REAL    INPUT - AMMONIA CONCENTRATION IN M**3/M**3
!     NN      INTEGER INPUT - AEROSOL TRACER TYPE INDEX FOR SULPHATE
!     NTR     INTEGER INPUT - TRACER INDEX
!     NTRACA  INTEGER INPUT - TRACER INDEX
!     MD      REAL    INPUT
!     MDR     REAL    WORK ARRAY
!     MU      REAL    INPUT
!     MUR     REAL    WORK ARRAY
!     OU      REAL    WORK ARRAY
!     O3ROW   REAL    INPUT - OZONE CONCENTRATION IN M**3/M**3
!     PRESSG  REAL    INPUT
!     RCOEX   REAL    INPUT - RADIUS OF AEROSOL-PARTICLES FOR PARTIALLY
!                             ACTIVATED BIN IN M
!     RCRIT   REAL    INPUT - BIN NUMBER (+ FRACTION UN-ACTIVATED) OF SM
!                             ACTIVATED BIN (1 -> STRATIFORM CLOUDS,
!                                            2 -> CONVECTIVE CLOUDS)
!     RGRID   REAL    INPUT - WORK SPACES
!     CLSIZE  REAL    INPUT - ACTIVATED RADIUS OF AEROSOL-PARTICLES IN M
!     ROAROW  REAL    INPUT - AIR DENSITY KG M-3
!     RTHPO   REAL    INPUT/OUTPUT - HYDROGEN PEROXIDE TENDENCY
!     RTICLD  REAL    INPUT/OUTPUT - TRACER TENDENCY
!     RTSO2   REAL    INPUT/OUTPUT - SULPHUR DIOXIDE TENDENCY
!     SHJ     REAL    INPUT
!     SUMDX   REAL    WORK ARRAY
!     TH      REAL    INPUT - TEMPERATURE IN K
!     X       REAL    WORK ARRAY
!     XD      REAL    WORK ARRAY
!     XG      REAL    WORK ARRAY
!     XGA     REAL    WORK ARRAY
!     XU      REAL    WORK ARRAY - UPDRAFT TRACER ARRAY IN KG-SPECIES/KG
!     XUA     REAL    WORK ARRAY
!     ZFG     REAL    INPUT
!     ZMLWC   REAL    INPUT - LIQ. CLOUD WATER CONTENT (1 -> STRATIFORM
!                                                       2 -> CONVECTIVE
!                             IN KG-H2O/KG-AIR
!-----------------------------------------------------------------------
!
      COMMON /TIMESc/ DELT
!
      LOGICAL KCALC(ILG,ILEV)
!
      integer JLAT
      INTEGER IDEEP(ILG),JT(ILG),MAXG(ILG),JD(ILG)
!
      REAL CLSIZE(ILG,ILEV,ISIZE),RCRIT(ILG,ILEV,2),XROW(ILG,ILEV+1,NTR) &
     &    ,TH(ILG,ILEV+1),H2O2ROW(ILG,ILEV),XU(ILG,ILEV,NTR) &
     &    ,HPOXROW(ILG,ILEV),O3ROW(ILG,ILEV),NH3ROW(ILG,ILEV) &
     &    ,HNO3ROW(ILG,ILEV),CO2ROW(ILG,ILEV),RCOEX(ILG,ILEV,2) &
     &    ,AERONUM(ILG,ILEV,ISIZE),CLDCV(ILG,ILEV,2),ZMLWC(ILG,ILEV,2) &
     &    ,AFRAC(ILG,ILEV,ISIZE,2),AFRACF(ILG,ILEV,2),AEROSIZE(2,ISIZE) &
     &    ,THLEV(ILG,ILEV),RTBCLD(ILG,ILEV,NTR),GDREM(ILG,NTR,3)
      REAL ROAROW(ILG,ILEV),AGAMMA(ILG,ILEV),AMASPE(3) &
     &    ,ARESID(ILG,ILEV),ASPECI(ILG,ILEV),AGTSO2(ILG,ILEV) &
     &    ,AGTSO4(ILG,ILEV),AGTO3(ILG,ILEV),AGTCO2(ILG,ILEV) &
     &    ,AGTNH3(ILG,ILEV),AGTHNO3(ILG,ILEV),AGTNA(ILG,ILEV) &
     &    ,AGTHO2(ILG,ILEV),AOH2O2(ILG,ILEV),AEQCA(ILG,ILEV) &
     &    ,AEQCB(ILG,ILEV),AEQCC(ILG,ILEV),AEQCAI(ILG,ILEV) &
     &    ,AEQCBI(ILG,ILEV),AEQCHP(ILG,ILEV),AEQCHO(ILG,ILEV) &
     &    ,AEQHNO(ILG,ILEV),AEQNH3(ILG,ILEV),AEQHCL(ILG,ILEV) &
     &    ,AORHP1(ILG,ILEV),AORHP2(ILG,ILEV),AORHO(ILG,ILEV) &
     &    ,ANTSO2(ILG,ILEV),ANTHO2(ILG,ILEV),ANTSO4(ILG,ILEV)
      REAL DSHJ(ILG,ILEV),EPS0(ILG),MU(ILG,ILEV),EU(ILG,ILEV)  &
     &    ,DU(ILG,ILEV),MD(ILG,ILEV),MC(ILG,ILEV),ED(ILG,ILEV)  &
     &    ,DP(ILG,ILEV),DZ(ILG,ILEV),MB(ILG),DSUBCLD(ILG)  &
     &    ,X(ILG,ILEV,NTR),XD(ILG,ILEV,NTR),XG(ILG,ILEV,NTR)  &
     &    ,DXDT(ILG,ILEV,NTR),XHAT(ILG,ILEV,NTR),DXDX(ILG,ILEV,NTR)  &
     &    ,FACX(ILG,ILEV,NTR),SUMDX(ILG),OU(ILG,ILEV,NTR),MUR(ILG,ILEV)  &
     &    ,MDR(ILG,ILEV),DUR(ILG,ILEV),EUR(ILG,ILEV),EDR(ILG,ILEV)  &
     &    ,ZFG(ILG,ILEV), PXNEW(ILG,ILEV,NTR)
      REAL RGRID(ILG,ILEV,ISIZE),RTSO4(ILG,ILEV,ISIZE),RTSO2(ILG,ILEV,2)  &
     &    ,RTHPO(ILG,ILEV),PRESSG(ILG),AMH(ILG,ILEV),SHJ(ILG,ILEV)  &
     &    ,XUA(ILG,ILEV,NTRACA),XGA(ILG,ILEV,NTRACA),Z(ILG,ILEV)  &
     &    ,ASPRVA(ILG,ILEV),ASRSO2(ILG,ILEV),ASRHPO(ILG,ILEV)
      REAL RU(ILG,ILEV,NTR),RUA(ILG,ILEV,NTRACA)
      DATA AMASPE / 96.0576E-03, 64.0588E-03, 34.0146E-03 /
!
      PARAMETER ( YTAU   = 129600.  &
     &           ,YCOM3L = 1.E+03  &
     &           ,YGASC  = 8.31441   &
     &           ,Y3LN10 = 6.90776 )
!
!-----------------------------------------------------------------------
!
!     (1) in-cloud production of sulphate
!
!-----------------------------------------------------------------------
!
!---  dimensions and initialization
      CALL OEQUIP (ILG,ILEV,IL1,IL2,AEQCA,AEQCB,AEQCC,AEQCAI,AEQCBI  &
     &            ,AEQCHP,AEQCHO,AEQHNO,AEQNH3,AEQHCL,AORHP1,AORHP2  &
     &            ,AORHO,TH(1,2),MAE)
!
!---  initial tendencies
      DO 90 JK=1+MAE,ILEV
      DO 90 IL=IL1,IL2
         RTHPO(IL,JK) = 0.
 90   CONTINUE
!
!---  initial concentrations
      DO 100 JK=1+MAE,ILEV
      DO 100 IL=IL1,IL2
!
!---     conversion from m**3/m**3 -> mol/l
         ROCO0 = (PRESSG(IL)*SHJ(IL,JK)) / (YGASC * TH(IL,JK+1))
         ROCON = ROCO0 / YCOM3L
         AGTHNO3(IL,JK) = HNO3ROW(IL,JK)*ROCON         ! nitric acid
         AGTNH3 (IL,JK) = NH3ROW(IL,JK) *ROCON         ! ammonia
         AGTO3  (IL,JK) = O3ROW(IL,JK)  *ROCON         ! ozone
         AGTCO2 (IL,JK) = CO2ROW(IL,JK) *ROCON         ! carbon dioxide
         AGTHO2(IL,JK)  = XROW(IL,JK+1,IHPO) * ROAROW(IL,JK)  &
     &                  / (AMASPE(3)*YCOM3L)           ! hydrogen peroxi
!
!---     conversion from m**3/m**3 -> kg/kg
         ROCON = ROCO0 * AMASPE(3) / ROAROW(IL,JK)
         HPOXROW(IL,JK) = H2O2ROW(IL,JK)*ROCON         ! hydrogen peroxi
!
!---     sodium concentration (for later applications)
         AGTNA(IL,JK) = 0.
 100  CONTINUE
!
!---  distribution of total produced sulphate on bins. AFRAC(I,L,N,NCT)
!---  is the volume of activated aerosol in bin N per total volume
!---  of activated aerosol for cloud type NCT
!
!     Note that only when the number density of a size bin is greater 
!       than 5 m-3, can it be serve as a activation bin to receive
!       cloud-produed sulphate.
!
      CALL PUTZERO(AFRACF,ILG*ILEV*2)
      CALL PUTZERO(AFRAC,ILG*ILEV*ISIZE*2)
      DO 150 NCT=1,2
      DO 150 N=1,ISIZE
         DO 160 L=1+MAE,ILEV
         DO 160 I=IL1,IL2
             IF (       (INT(RCRIT(I,L,NCT)) .GE. 1)  &
     &            .AND. (INT(RCRIT(I,L,NCT)) .LE. ISIZE)        )  THEN
                AERNO=AERONUM(I,L,N)*ROAROW(I,L)
                IF (N.EQ.INT(RCRIT(I,L,NCT)) .AND. AERNO .GT.1.E6) THEN
                   AFRAC(I,L,N,NCT) = AERONUM(I,L,N) * RCOEX(I,L,NCT)**3 &
     &                              * (1.-AMOD(RCRIT(I,L,NCT),1.))
                   AFRACF(I,L,NCT) = AFRAC(I,L,N,NCT)
                ELSE IF (N .GT. INT(RCRIT(I,L,NCT))   &
     &                                     .AND. AERNO .GT. 1.E6)  THEN
                   AFRAC(I,L,N,NCT) = AERONUM(I,L,N) * CLSIZE(I,L,N)**3
                   AFRACF(I,L,NCT) = AFRACF(I,L,NCT) + AFRAC(I,L,N,NCT)
                ELSE
                   AFRAC(I,L,N,NCT) = 0.
                END IF
                AFRACF(I,L,NCT) = MAX (AFRACF(I,L,NCT),1.E-33)
             END IF
 160      CONTINUE
 150  CONTINUE
      DO 170 NCT=1,2
      DO 170 N=1,ISIZE
      DO 170 L=1+MAE,ILEV
      DO 170 I=IL1,IL2
         IF (       (INT(RCRIT(I,L,NCT)) .GE. 1)  &
     &        .AND. (INT(RCRIT(I,L,NCT)) .LE. ISIZE)    ) THEN
              AFRAC(I,L,N,NCT) = AFRAC(I,L,N,NCT) / AFRACF(I,L,NCT)
         END IF
 170  CONTINUE
!
      
!     if(jlat.eq.185) then
!     write(0,*)'wzz: cldchem 0=', xrow(14,1+ilev,53), ISO2
!     endif

!---  call to perform stratiform cloud oxidation of sulphur dioxide
      CALL OXISTR(    IL1,      IL2,     ILEV,     ILG,  ISIZE &
     &            ,   NTR,     ISO2,     IHPO,      NN,   IAE1 &
     &            ,ROAROW,   AGAMMA,   ASPECI,  AGTSO2, AGTSO4 &
     &             ,AGTO3,   AGTCO2,   AGTNH3, AGTHNO3,  AGTNA &
     &            ,AGTHO2,    AEQCA,    AEQCB,   AEQCC, AEQCBI &
     &            ,AEQCHP,   AEQCHO,   AEQHNO,  AEQNH3, AEQHCL &
     &            ,AORHP1,   AORHP2,    AORHO,  ANTSO2, ANTHO2 &
     &            ,ANTSO4,     XROW,   AOH2O2,  ARESID,ZMLWC(1,1,1) &
     &      ,CLDCV(1,1,1),AFRAC(1,1,1,1), AMH,   KCALC, RTSO4 &
     &            ,RTHPO,RCRIT(1,1,1),AMASPE,ASPRVA &
     &            ,ASRSO2,   ASRHPO, MAE)
!     if(jlat.eq.185) then
!     write(0,*)'wzz: cldchem 1=', xrow(14,ilev+1,53), rtso4(14,ilev,5)
!     endif
!
!   UPDATE SO4
!
      DO N=1,ISIZE
         NF=ISIZE*(NN-1)+N+(IAE1-1)
         DO L=1+MAE,ILEV
            DO I=IL1,IL2
               XROW(I,L+1,NF)=XROW(I,L+1,NF)+2.*DELT*RTSO4(I,L,N)
            END DO
         END DO
      END DO
!
!   UPDATE SO2
! 
      DO L=1+MAE,ILEV
         DO I=IL1,IL2
            XROW(I,L+1,ISO2)=XROW(I,L+1,ISO2)+2.*DELT*ASPRVA(I,L)
            RTSO2(I,L,2) = ASPRVA(I,L)
         END DO
      END DO
         
!
!---  call tp perform convective cloud oxidation of sulphur dioxide
      CALL PUTZERO(RGRID,ILG*ILEV*ISIZE)
      CALL PUTZERO(ASPRVA,ILG*ILEV)
!1   zhouch  070902
!     if(jlat.eq.185) then
!     write(0,*)'wzz: cldchem 2=', xrow(14,ilev+1,53),rgrid(14,ilev,5)
!     endif
        CALL OXICON(   IL1,    IL2,    NTR,    ILEV,      ID &
     &          ,    ILG,   ISO2,   IHPO,      NN,   ISIZE &
     &          ,   IAE1,RCRIT(1,1,2),AGTO3,AGTCO2, AGTNH3 &
     &          ,AGTHNO3,  AGTNA,  AEQCB,  AEQCBI,  AEQNH3 &
     &          , AEQHNO, AEQHCL,  AEQCC,   AEQCA,  AEQCHO &
     &          , AEQCHP, AORHP1, AORHP2,   AORHO,  ROAROW &
     & ,AFRAC(1,1,1,2),CLDCV(1,1,2),ZMLWC(1,1,2),GRAV,DSHJ &
     &          ,  IDEEP,   EPS0,     JT,    MAXG,      JD &
     &          ,     DZ,     DP,     DU,      EU,      ED &
     &          ,     MU,     MD,     MC,      MB,     MSG &
     &          ,LENGATH,DSUBCLD,      X,      XU,      XD &
     &          ,     XG,   XHAT,   DXDT,    DXDX,    FACX &
     &          ,  SUMDX,     OU,    XGA,     XUA,  NTRACA &
     &          , AGTSO4, AGTSO2, AGTHO2,  AGAMMA,    XROW &
     &          ,    AMH,  KCALC,  RGRID,  ASPRVA,   RTHPO &
     &          , AMASPE,    MUR,    MDR,     DUR,     EUR &
     &          ,    EDR,   ZFG,PXNEW(1,1,1),PXNEW(1,1,2),PXNEW(1,1,3) &
     &          ,     RU,    RUA, TH(1,2),  THLEV,  RTBCLD, MAE, JLAT)
!
!     if(jlat.eq.185) then
!     write(0,*)'wzz: cldchem 3=', xrow(14,ilev+1,53),NN, IAE1,  &
!    &      rgrid(14,ilev,5),rtbcld(14,ilev,53),rtbcld(13,ilev,53), DELT
!     endif
!   UPDATE SO4
!
      DO N=1,ISIZE
         NF=ISIZE*(NN-1)+N+(IAE1-1)
         DO L=1+MAE,ILEV
            DO I=IL1,IL2
               XROW(I,L+1,NF)=XROW(I,L+1,NF)+2.*DELT*(RGRID(I,L,N)+  &
     &                                               RTBCLD(I,L,NF))             
!              GDREM(I,NF,2) =GDREM(I,NF,2)+RTBCLD(I,L,NF)*  
!    &                                        DSHJ(I,L)*PRESSG(I)/GRAV
               RTSO4(I,L,N)=RTSO4(I,L,N)+RGRID(I,L,N)
            END DO
         END DO
      END DO
!     if(jlat.eq.185) then
!     write(0,*)'wzz: cldchem 4=', xrow(14,ilev+1,53), ASPRVA(14,ilev), RTBCLD(14,ilev,53), IHPO
!     endif
!
!   UPDATE SO2
! 
      DO L=1+MAE,ILEV
         DO I=IL1,IL2
            XROW(I,L+1,ISO2)=XROW(I,L+1,ISO2)+2.*DELT*(ASPRVA(I,L)+  &
     &                                               RTBCLD(I,L,ISO2))             
!           GDREM(I,ISO2,2)=GDREM(I,ISO2,2)+RTBCLD(I,L,ISO2)*  
!    &                                        DSHJ(I,L)*PRESSG(I)/GRAV
            RTSO2(I,L,2) = RTSO2(I,L,2)+ASPRVA(I,L)
         END DO
      END DO
!     if(jlat.eq.185) then
!     write(0,*)'wzz: cldchem 5=', xrow(14,20,63)
!     endif
!
!-----------------------------------------------------------------------
!
!     (2) hydrogen peroxide production
!
!-----------------------------------------------------------------------
!
      DO L=1+MAE,ILEV
      DO I=IL1,IL2
         RTHPO(I,L) = RTHPO(I,L) - ( 1. / YTAU )  &
     &                  * ( XROW(I,L+1,IHPO) - HPOXROW(I,L) )
         XROW(I,L+1,IHPO)=XROW(I,L+1,IHPO)+RTHPO(I,L)*2.*DELT
          
      ENDDO
      ENDDO
!
      RETURN
      END
