      SUBROUTINE OXICON (     IL1,    IL2,  NTRAC,    ILEV,      ID  &
     &                   ,    ILG,   ISO2,   IHPO,      NN,   ISIZE  &
     &                   ,   IAE1,  RCRIT,  AGTO3,  AGTCO2,  AGTNH3  &
     &                   ,AGTHNO3,  AGTNA,  AEQCB,  AEQCBI,  AEQNH3  &
     &                   , AEQHNO, AEQHCL,  AEQCC,   AEQCA,  AEQCHO  &
     &                   , AEQCHP, AORHP1, AORHP2,   AORHO,  ROAROW  &
     &                   ,  AFRAC,  CLDCV,  ZMLWC,    GRAV,    DSHJ  &
     &                   ,  IDEEP,   EPS0,     JT,    MAXG,      JD  &
     &                   ,     DZ,     DP,     DU,      EU,      ED  &
     &                   ,     MU,     MD,     MC,      MB,     MSG  &
     &                   ,LENGATH,DSUBCLD,      X,      XU,      XD  &
     &                   ,     XG,   XHAT,   DXDT,    DXDX,    FACX  &
     &                   ,  SUMDX,     OU,    XGA,     XUA,  NTRACA  &
     &                   , AGTSO4, AGTSO2,  AGTHP,  AGAMMA,    XROW  &
     &                   ,    AMH,  KCALC,  RTSO4,   RTSO2,   RTHPO  &
     &                   , AMASPE,    MUR,    MDR,     DUR,     EUR  &
     &                   ,    EDR,    ZFG,   TMP1,    TMP2,    TMP3  &
     &                   ,     RU,    RUA,      T,     DZT,  RTBCLD,  &
     &                     MAE,   JLAT)
!***********************************************************************
!     Calculates transport and oxidation in convective clouds
!***********************************************************************
!
!---  History
!     APRIL 16, 1999    S.L. GONG      VECTORIZED THE NEW DIAGNOSE
!                                      PORTION OF THE ROUTINE AND TAKE
!                                      THE CONVECTIVE SCARVENGING 
!                                      AMMOUNT INTO TOTAL REMOVAL ARRAY
!                                      GDREM FOR TOTAL BUDGET.
!     MARCH 10, 1999    KNUT VON SALZEN     NEW DIAGNOSTICS
!     FEBRUARY 16, 1998 KNUT VON SALZEN     NEW SUBROUTINE
!     MARCH 12, 1998    KNUT VON SALZEN     ZHANG & MCFARLANE SCHEME
!
!-----------------------------------------------------------------------
!     I/O-parameters
!
!     AEQCA     REAL     INPUT
!     AEQCB     REAL     INPUT
!     AEQCC     REAL     INPUT
!     AEQCBI    REAL     INPUT
!     AEQCHO    REAL     INPUT
!     AEQCHP    REAL     INPUT
!     AEQHCL    REAL     INPUT
!     AEQHNO    REAL     INPUT
!     AEQNH3    REAL     INPUT
!     AFRAC     REAL     INPUT
!     AGAMMA    REAL     WORK ARRAY
!     AGTCO2    REAL     INPUT
!     AGTHNO3   REAL     INPUT
!     AGTHP     REAL     WORK ARRAY (UNITS: MOL HYDROGEN PEROXIDE
!                                          /CUBIC LITRE AIR)
!     AGTNA     REAL     INPUT
!     AGTNH3    REAL     INPUT
!     AGTO3     REAL     INPUT
!     AGTSO2    REAL     WORK ARRAY (UNITS: MOL SULPHUR SULPHUR DIOXIDE
!                                          /CUBIC LITRE AIR)
!     AGTSO4    REAL     WORK ARRAY (UNITS: MOL SULPHATE
!                                          /CUBIC LITRE AIR)
!     AORHO     REAL     INPUT
!     AORHP1    REAL     INPUT
!     AORHP2    REAL     INPUT
!     CLDCV     REAL     INPUT
!     DSHJ      REAL     INPUT-FROM->PHYMRCB
!     DXDT      REAL     WORK ARRAY
!     DXDX      REAL     WORK ARRAY
!     DP        REAL     INPUT-FROM->CONV4
!     DSUBCLD   REAL     INPUT-FROM->CONV4
!     DU        REAL     INPUT-FROM->CLDPRP4
!     DUR       REAL     WORK ARRAY
!     DZ        REAL     INPUT-FROM->CLDPRP4
!     EPS0      REAL     INPUT-FROM->CLDPRP4
!     ED        REAL     INPUT-FROM->CLDPRP4
!     EDR       REAL     WORK ARRAY
!     EU        REAL     INPUT-FROM->CLDPRP4
!     EUR       REAL     WORK ARRAY
!     FACX      REAL     WORK ARRAY
!     GRAV      REAL     INPUT
!     IAE1      INTEGER  INPUT
!     IDEEP     INTEGER  INPUT-FROM->CONV4
!     IL1       INTEGER  INPUT
!     IL2       INTEGER  INPUT
!     ILEV      INTEGER  INPUT
!     ILG       INTEGER  INPUT
!     IHPO      INTEGER  INPUT
!     ISO2      INTEGER  INPUT
!     ISIZE     INTEGER  INPUT
!     JD        INTEGER  INPUT-FROM->CLDPRP4
!     JT        INTEGER  INPUT-FROM->CLDPRP4
!     KCALC     LOGICAL  WORK ARRAY
!     LENGATH   INTEGER  INPUT-FROM->CONV4
!     MAXG      INTEGER  INPUT-FROM->CONV4
!     MB        REAL     INPUT-FROM->CONV4
!     MC        REAL     INPUT-FROM->CLDPRP4
!     MD        REAL     INPUT-FROM->CLDPRP4
!     MDR       REAL     WORK ARRAY
!     MU        REAL     INPUT-FROM->CLDPRP4
!     MUR       REAL     WORK ARRAY
!     MSG       INTEGER  INPUT-FROM->PHYMRCC
!     NN        INTEGER  INPUT
!     NTRAC     INTEGER  INPUT
!     RU        REAL     WORK ARRAY
!     RUA       REAL     WORK ARRAY
!     OU        REAL     WORK ARRAY
!     T         REAL     INPUT
!     RCRIT     REAL     INPUT
!     ROAROW    REAL     INPUT
!     SUMDX     REAL     WORK ARRAY
!     ZFG       REAL     INPUT-FROM->CONV4
!     ZMLWC     REAL     INPUT
!     X         REAL     WORK ARRAY
!     XG        REAL     WORK ARRAY
!     XGA       REAL     WORK ARRAY
!     XHAT      REAL     WORK ARRAY
!     XU        REAL     WORK ARRAY
!     XUA       REAL     WORK ARRAY
!     XD        REAL     WORK ARRAY
!     XROW      REAL     INPUT/OUTPUT-FROM->COMMON ICOM
!
!-----------------------------------------------------------------------
!
      implicit none
!
      integer IL1, IL2, ILG, ILEV
      integer NTRAC, NTRACA, ISIZE, NN
      integer ID, MSG, LENGATH
      integer MAE, IAE1
      integer ISO2, IHPO
      integer JLAT
!
      real DELT
      COMMON /TIMESc/ DELT
!
      integer IDEBUG,   IMM,    IMA,   INU,  ICOB,IDRYDEP,  &
     &        ITUBMIX, ICOAG,ICONVEC,IINCLD,IBLCLD
      COMMON /AEROSWTH/ IDEBUG,   IMM,    IMA,   INU,  ICOB,IDRYDEP,  &        
     &                 ITUBMIX, ICOAG,ICONVEC,IINCLD,IBLCLD     
      LOGICAL KCALC(ILG,ILEV)
!
      INTEGER IDEEP(ILG),JT(ILG),MAXG(ILG),JD(ILG)
      INTEGER ITERP,INH3,IHNO3,IO3,ICO2,ISO4
      PARAMETER (ITERP = 3)
      PARAMETER (INH3  = 1)
      PARAMETER (IHNO3 = INH3+1)
      PARAMETER (IO3   = IHNO3+1)
      PARAMETER (ICO2  = IO3+1)
      PARAMETER (ISO4  = ICO2+1)
!
      REAL XROW(ILG,ILEV+1,NTRAC),XG(ILG,ILEV,NTRAC) &                  !
     &    ,X(ILG,ILEV,NTRAC),XU(ILG,ILEV,NTRAC),XD(ILG,ILEV,NTRAC) &
     &    ,DXDT(ILG,ILEV,NTRAC), XHAT(ILG,ILEV,NTRAC)  & 
     &    ,DXDX(ILG,ILEV,NTRAC), FACX(ILG,ILEV,NTRAC)  &
     &    ,SUMDX(ILG),MB(ILG),DSUBCLD(ILG),EPS0(ILG)  &
     &    ,MU(ILG,ILEV),EU(ILG,ILEV),DU(ILG,ILEV),MD(ILG,ILEV)  &
     &    ,MC(ILG,ILEV),ED(ILG,ILEV),DP(ILG,ILEV),DZ(ILG,ILEV)  &
     &    ,DSHJ(ILG,ILEV),OU(ILG,ILEV,NTRAC),MUR(ILG,ILEV)  &
     &    ,MDR(ILG,ILEV),DUR(ILG,ILEV),EUR(ILG,ILEV)  &
     &    ,EDR(ILG,ILEV),ZFG(ILG,ILEV),RTBCLD(ILG,ILEV,NTRAC)
      REAL CLDCV(ILG,ILEV),AGTO3(ILG,ILEV),AMASPE(3)  &
     &    ,AGTCO2(ILG,ILEV),AGTNH3(ILG,ILEV),AGTHNO3(ILG,ILEV)  &
     &    ,ZMLWC(ILG,ILEV),RCRIT(ILG,ILEV),AGTSO4(ILG,ILEV)  &
     &    ,AGTSO2(ILG,ILEV),AGTHP(ILG,ILEV),ROAROW(ILG,ILEV)  &
     &    ,AGAMMA(ILG,ILEV),AGTNA(ILG,ILEV),AEQCB(ILG,ILEV)  &
     &    ,AEQCBI(ILG,ILEV),AEQNH3(ILG,ILEV),AEQHNO(ILG,ILEV)  &
     &    ,AEQHCL(ILG,ILEV),AEQCC(ILG,ILEV),AEQCA(ILG,ILEV)  &
     &    ,AEQCHO(ILG,ILEV),AEQCHP(ILG,ILEV),AORHP1(ILG,ILEV)  &
     &    ,AORHP2(ILG,ILEV),AORHO(ILG,ILEV),AFRAC(ILG,ILEV,ISIZE)  &
     &    ,RTSO4(ILG,ILEV,ISIZE),RTSO2(ILG,ILEV),RTHPO(ILG,ILEV)
      REAL RU(ILG,ILEV,NTRAC),RUA(ILG,ILEV,NTRACA),T(ILG,ILEV)
      REAL AMH(ILG,ILEV),XUA(ILG,ILEV,NTRACA),XGA(ILG,ILEV,NTRACA)
      REAL TMP1(ILG), TMP2(ILG),TMP3(ILG), DZT(ILG,ILEV)

      REAL AMSO3,AMNH4,AMNO3,AMSO4,AMNA,AMCL,AGTHCL,ADELTA,ATVAL  &
     &    ,AMO3,AFRAS,AFRAH,AFO3,AFH2O2,GRAV
      REAL YCOM3L,YEPS,YSMALL,YRHOW,RNU,BETA
      PARAMETER (YCOM3L = 1.E+03)
      PARAMETER (YEPS   = 1.E-33)
      PARAMETER (YSMALL = 1.E-03)
      PARAMETER (YRHOW  = 1.E+03)
      PARAMETER (RNU    = 1.)
      PARAMETER (BETA   = 1.)
      real TFREEZ,T2S,AI,BI,AW,BW,SLP
      COMMON/HTCP  / TFREEZ,T2S,AI,BI,AW,BW,SLP
!
      integer IL, L, K, J
      integer INDX, MASG
      integer N, NF, NFMIN, NFMAX
      real GO, FAC, C0
      real ACCO2, ACo3, ACNO3, ACHPO, ACNH4, ACSO2, ACSO4
      real AFOH, AFCO3, AFHSO, AFCL, AFNO3, AFNH4, AFAKT
      real ASRHNO, ASRNH3, ASRSO2, ASRHPO, ASRO3, ASRSO4, ASRCO2
      real ATENO3, ATENHP, ATERM1, ATERM2, ATAEQ, ATVO3
      real AXGP1, ADMUD, AMEAM, AXGM1, AXDIF, AXG, ABSB, ABSA
      real DDDT, DDDV
      real SO22, SO44, SORU
      real TTSO4, TTRU, TTSO2, TTOU, TTOV
!-----------------------------------------------------------------------
!
!     initial values and constants
!
!-----------------------------------------------------------------------
!
      if ( LENGATH .lt. 1 ) return      ! wzz, 2008.9.9
!---  minimum and maximum tracer index of sulphate aerosol
      NFMIN=ISIZE*(NN-1)+1+(IAE1-1)
      NFMAX=ISIZE*(NN-1)+ISIZE+(IAE1-1)
!
!---  scavenging ratios sulphate and carbon dioxide
      ASRSO4 = 1.
      ASRCO2 = 0.
!
!---  control parameter
      CALL PUTZERO (SUMDX,  ILG)

      CALL PUTZERO  (AGTSO2, ILG*ILEV)
      CALL PUTZERO  (AGTSO4, ILG*ILEV)
      CALL PUTZERO  (AGTHP , ILG*ILEV)
!
!---  assume no chemical sinks/sources
      CALL PUTZERO  (OU    , ILG*ILEV*NTRAC)
      CALL PUTZERO  (RU    , ILG*ILEV*NTRAC)
      CALL PUTZERO  (RUA   , ILG*ILEV*NTRACA)
      
      CALL PUTZERO  (MUR   , ILG*ILEV)
      CALL PUTZERO  (MDR   , ILG*ILEV)
      CALL PUTZERO  (EDR   , ILG*ILEV)
      CALL PUTZERO  (EUR   , ILG*ILEV)
      CALL PUTZERO  (DUR   , ILG*ILEV)
!
      MASG=MAX (MAE,MSG) 
!     write(*,*)'wzz: msg=', msg, mae, masg
!
!---  assume 10**(-pH) = 1
      DO 40 L=1+MASG,ILEV
      DO 40 IL=IL1,IL2
         AMH(IL,L) = 1.
 40   CONTINUE

      DO 60 L=MASG+1,ILEV
      DO 60 K=1,LENGATH
         MUR(K,L) = MU(K,L) / MAX(MB(K),YEPS)
         MDR(K,L) = MD(K,L) / MAX(MB(K),YEPS)
! wzz, 2008.10.9
!        EDR(K,L) = ED(K,L) / MAX(MB(K),YEPS)
!        EUR(K,L) = EU(K,L) / MAX(MB(K),YEPS)
 60   CONTINUE
      DO 62 L=MASG+1,ILEV-1
      DO 62 K=1,LENGATH
         DUR(K,L) = DU(K,L) * DP(K,L)  &
     &           / (MAX(MB(K),YEPS)*(ZFG(K,L)-ZFG(K,L+1)))
! wzz, 2008.10.9
         EDR(K,L) = ED(K,L) * DP(K,L)   &
     &           / (MAX(MB(K),YEPS)*(ZFG(K,L)-ZFG(K,L+1)))
         EUR(K,L) = EU(K,L) * DP(K,L)  &
     &           / (MAX(MB(K),YEPS)*(ZFG(K,L)-ZFG(K,L+1)))
 62   CONTINUE
      DO 63 K=1,LENGATH
         DUR(K,ILEV) = DU(K,ILEV) * DP(K,ILEV)  &
     &              / (MAX(MB(K),YEPS)*ZFG(K,ILEV))
! wzz, 2008.10.9
         EDR(K,ILEV) = ED(K,ILEV) * DP(K,ILEV)  &
     &              / (MAX(MB(K),YEPS)*ZFG(K,ILEV))
         EUR(K,ILEV) = EU(K,ILEV) * DP(K,ILEV)  &
     &              / (MAX(MB(K),YEPS)*ZFG(K,ILEV))
 63   CONTINUE
!
!---  copy XROW into the work array and change units from kg/kg
!---  to mol/kg
         DO N=1,NTRAC
            IF ( N .EQ. ISO2 ) THEN
               AFAKT = 1. / AMASPE(2)
            ELSE IF ( N .EQ. IHPO) THEN
               AFAKT = 1. / AMASPE(3)
            ELSE IF ( (N .GE. NFMIN) .OR. (N .LE. NFMAX) ) THEN
               AFAKT = 1. / AMASPE(1)
            ELSE
               AFAKT = 1.
            END IF
         DO 70 L=1+MASG,ILEV
!        DO 70 K=IL1,IL2
!           X(K,L,N) = AFAKT * XROW(K,L+1,N)
         DO 70 K=1,LENGATH
            XG(K,L,N) = AFAKT * XROW(IDEEP(K),L+1,N)
 70      CONTINUE
         ENDDO
!
!---     IDEEP(K) contains indices where convection occurs
!---     XG is "gathered" here
!        DO 90 N=1,NTRAC
!        DO 90 L=1+MASG,ILEV
!        DO 90 K=1,LENGATH
!           XG(K,L,N) = X(IDEEP(K),L,N)
!90      CONTINUE
!
!---     all chemical species which are not subject to
!---     other processes than cloud-chemical processes
!---     units: mol/kg
         DO 95 L=1+MASG,ILEV
         DO 95 K=1,LENGATH
            AFAKT = YCOM3L / ROAROW(IDEEP(K),L)
            XGA(K,L,INH3)  = AGTNH3(IDEEP(K),L)  * AFAKT
            XGA(K,L,IHNO3) = AGTHNO3(IDEEP(K),L) * AFAKT
            XGA(K,L,IO3)   = AGTO3(IDEEP(K),L)   * AFAKT
            XGA(K,L,ICO2)  = AGTCO2(IDEEP(K),L)  * AFAKT
            XGA(K,L,ISO4)  = 0.
 95      CONTINUE
!
!---     total activated sulphate
         DO 96 N=1,ISIZE
            NF=NFMIN+N-1
            DO 97 L=1+MASG,ILEV
            DO 97 K=1,LENGATH
               IF (N .EQ. INT(RCRIT(IDEEP(K),L)))  THEN
                  XGA(K,L,ISO4) = XG(K,L,NF)  &
     &                          *(1.-AMOD(RCRIT(IDEEP(K),L),1.))
               ELSE IF (N .GT. INT(RCRIT(IDEEP(K),L))) THEN
                  XGA(K,L,ISO4) = XGA(K,L,ISO4) + XG(K,L,NF)
               END IF
 97         CONTINUE
 96      CONTINUE
!
!---     compute XHAT from two different interpolation approaches
         DO 110 N=1,NTRAC
            IF (     (N .EQ. ISO2) .OR. (N .EQ. IHPO)  &
     &         .OR. ((N .GE. NFMIN) .AND. (N .LE. NFMAX)) ) THEN
               DO 115 L=MASG+2,ILEV
               DO 115 K=1,LENGATH
                  AXDIF = 0.
                  IF(XG(K,L-1,N) .GT. 0. .OR. XG(K,L,N) .GT. 0.)  &
     &               AXDIF = ABS((XG(K,L-1,N)-XG(K,L,N))  &
     &                     / MAX(XG(K,L-1,N),XG(K,L,N)))
                  IF (AXDIF .GT. YSMALL) THEN
                     AXGM1 = XG(K,L-1,N)
                     AXG   = XG(K,L,N)
                     IF ( AXGM1 .LE. 0. ) THEN
                        AXGM1 = YSMALL * AXG
                     ELSE IF ( AXG .LE. 0. ) THEN
                        AXG   = YSMALL * AXGM1
                     END IF
                     ABSA = ABS(AXGM1)
                     ABSB = ABS(AXG-AXGM1)
                     IF ( (ABSA.GT.YEPS) .AND. (ABSB.GT.YEPS) ) THEN
                        XHAT(K,L,N)= AXG * AXGM1 * LOG( AXG / AXGM1 )  &
     &                            / ( AXG - AXGM1 )
                     ELSE 
                        XHAT(K,L,N) = 0.
                     END IF
                  ELSE
                     XHAT(K,L,N)=0.5*(XG(K,L,N)+XG(K,L-1,N))
                  END IF
 115           CONTINUE
            END IF
 110     CONTINUE
!
!---     initial guess for updraft and downdraft concentrations
         DO 130 N=1,NTRAC
         DO 130 K=MASG+1,ILEV
         DO 130 IL=1,LENGATH
            XU(IL,K,N)=XG(IL,K,N)
            XD(IL,K,N)=XG(IL,K,N)
 130     CONTINUE
!        if(jlat.eq.249) then
!         write(0,*)'wzz: xd0=', xd(2,ilev,49), dz(2,ilev)
!        endif
         DO 135 N=1,NTRACA
         DO 135 K=MASG+1,ILEV
         DO 135 IL=1,LENGATH
            XUA(IL,K,N)=XGA(IL,K,N)
 135     CONTINUE
!     write(*,*)'wzz: step1'
!
!---     check for cumulus transport and activation
!---     restrict chemical reactions to grid cells with
!---     activation, sufficient cloud water and cloud cover
         DO 137 K=MASG+1,ILEV
         DO 137 IL=1,LENGATH
            KCALC(IL,K) = .FALSE.
            AGAMMA(IL,K) = 0.
 137     CONTINUE
         DO 140 J=ILEV,MASG+2,-1
         DO 140 IL=1,LENGATH
            IF(EPS0(IL).GT.0. .AND. (J.GE.JT(IL)  &
     &         .AND. J.LT.MAXG(IL)) .AND. MUR(IL,J).GT.0.  &
     &         .AND. MB(IL).GT.0.) THEN
               KCALC(IL,J) = .TRUE.
!       if(jlat.eq.185.and.ideep(il).eq.14.and.j.eq.20) then
!     write(0,*)'wzz: agamma', RCRIT(IDEEP(IL),J), ZMLWC(IDEEP(IL),J), 
!    &    ROAROW(IDEEP(IL),J-1),ROAROW(IDEEP(IL),J),YRHOW, IDEEP(IL), IL
!       endif
               IF (      (INT(RCRIT(IDEEP(IL),J)) .GE. 1)  &
     &             .AND. (INT(RCRIT(IDEEP(IL),J)) .LE. ISIZE)  &
!     2             .AND. (ZMLWC(IDEEP(IL),J) .GT. 1.E-04)
!     3             .AND. (CLDCV(IDEEP(IL),J) .GT. 1.E-04) ) THEN
!    2             .AND. (ZMLWC(IDEEP(IL),J) .GT. 1.E-09) ) THEN
     &             .AND. (ZMLWC(IDEEP(IL),J) .GT. 0.0   ) ) THEN    ! wzz, 2008.10.8
                  AGAMMA(IL,J) = ZMLWC(IDEEP(IL),J)  &
     &                         *.5*(ROAROW(IDEEP(IL),J-1)  &
     &                             +ROAROW(IDEEP(IL),J)) / YRHOW
                  AGAMMA(IL,J) = MAX (AGAMMA(IL,J),0.)
               END IF
!       if(jlat.eq.185.and.ideep(il).eq.14.and.j.eq.20) then
!     write(0,*)'wzz: agamma 1', AGAMMA(IL,J)
!       endif
            END IF
 140     CONTINUE
!
         DO 1000 J=ILEV,MASG+2,-1
!
!-----------------------------------------------------------------------
!
!     calculate updraft concentrations XU of reactive tracers
!     total sulphate, sulphur dioxide and hydrogen peroxide
!
!-----------------------------------------------------------------------
!
!---        sulphur dioxide
            DO 1150 IL=1,LENGATH
               IF ( KCALC(IL,J) ) THEN
                  AMEAM  = .5 * ( MUR(IL,J) + MUR(IL,J+1) )
                  ADMUD  = ( MUR(IL,J) - MUR(IL,J+1) ) / DZ(IL,J)
                  ATERM1 = EUR(IL,J) * XG(IL,J,ISO2)
                  ATERM2 = DUR(IL,J) + OU(IL,J+1,ISO2)  &
     &                   + RU(IL,J+1,ISO2) + ADMUD
                  IF (     (ABS(ATERM2) .GT. YEPS)  &
     &               .AND. (ABS(AMEAM)  .GT. YEPS)            ) THEN
                     XU(IL,J,ISO2) = XU(IL,J+1,ISO2) + ( ATERM1/ATERM2  &
     &                 - XU(IL,J+1,ISO2) ) * ( 1. - EXP(-DZ(IL,J)  &
     &                 * ATERM2 / AMEAM) )
                     AGTSO2(IL,J) = XU(IL,J+1,ISO2) + ( ATERM1/ATERM2  &
     &                 - XU(IL,J+1,ISO2) ) * ( 1. + AMEAM  &
     &                / (ATERM2 * DZ(IL,J)) * ( EXP(-DZ(IL,J)  &
     &                 * ATERM2 / AMEAM) - 1.) )
                  END IF
                  IF (      ABS(AMEAM)  .GT. YEPS  &
     &                .AND. ABS(ATERM2) .LE. YEPS              ) THEN
                     XU(IL,J,ISO2) = XU(IL,J+1,ISO2)  &
     &                             + ATERM1 * DZ(IL,J) / AMEAM
                     AGTSO2(IL,J) = XU(IL,J+1,ISO2)  &
     &                            + .5 * ATERM1 * DZ(IL,J) / AMEAM
                  END IF
                  IF (        ABS(ATERM2) .GT. YEPS  &
     &                .AND.   ABS(AMEAM)  .LE. YEPS            ) THEN
                     XU(IL,J,ISO2) = ATERM1 / ATERM2
                     AGTSO2(IL,J)  = .5 * ( XU(IL,J+1,ISO2)  &
     &                                    + XU(IL,J,ISO2)   )
                  END IF
                  XU(IL,J,ISO2) = MAX (XU(IL,J,ISO2), 0.)
               END IF
 1150       CONTINUE
!
!---        total sulphate
            DO 1120 IL=1,LENGATH
               IF ( KCALC(IL,J) ) THEN
                  AMEAM  = .5 * ( MUR(IL,J) + MUR(IL,J+1) )
                  ADMUD  = ( MUR(IL,J) - MUR(IL,J+1) ) / DZ(IL,J)
                  ATERM1 = EUR(IL,J) * XGA(IL,J,ISO4)  &
     &                   + OU(IL,J+1,ISO2)  &
     &                   * AGTSO2(IL,J)
                  ATERM2 = DUR(IL,J) + RUA(IL,J+1,ISO4) + ADMUD
                  IF (     (ABS(ATERM2) .GT. YEPS)  &
     &               .AND. (ABS(AMEAM)  .GT. YEPS)            ) THEN
                     XUA(IL,J,ISO4) = XUA(IL,J+1,ISO4)  &
     &                     + ( ATERM1/ATERM2 - XUA(IL,J+1,ISO4) )  &
     &                     * ( 1. - EXP(-DZ(IL,J) * ATERM2/AMEAM) )
                     AGTSO4(IL,J) = XUA(IL,J+1,ISO4) + ( ATERM1/ATERM2  &
     &                 - XUA(IL,J+1,ISO4) ) * ( 1. + AMEAM  &
     &                / (ATERM2 * DZ(IL,J)) * ( EXP(-DZ(IL,J)  &
     &                 * ATERM2 / AMEAM) - 1.) )
                  END IF
                  IF (      ABS(AMEAM)  .GT. YEPS  &
     &                .AND. ABS(ATERM2) .LE. YEPS              ) THEN
                     XUA(IL,J,ISO4) = XUA(IL,J+1,ISO4)  &
     &                              + ATERM1 * DZ(IL,J) / AMEAM
                     AGTSO4(IL,J) = XUA(IL,J+1,ISO4)  &
     &                            + .5 * ATERM1 * DZ(IL,J) / AMEAM
                  END IF
                  IF (        ABS(ATERM2) .GT. YEPS  &
     &                .AND.   ABS(AMEAM)  .LE. YEPS            ) THEN
                     XUA(IL,J,ISO4) = ATERM1 / ATERM2
                     AGTSO4(IL,J)  = .5 * ( XUA(IL,J+1,ISO4)  &
     &                                    + XUA(IL,J,ISO4)   )
                  END IF
                  XUA(IL,J,ISO4) = MAX (XUA(IL,J,ISO4), 0.)
               END IF
 1120       CONTINUE
!
!---        hydrogen peroxide
            DO 1160 IL=1,LENGATH
               IF ( KCALC(IL,J) ) THEN
                  AMEAM  = .5 * ( MUR(IL,J) + MUR(IL,J+1) )
                  ADMUD  = ( MUR(IL,J) - MUR(IL,J+1) ) / DZ(IL,J)
                  ATERM1 = EUR(IL,J) * XG(IL,J,IHPO)  &
     &                   - OU(IL,J+1,IHPO)  &
     &                   * AGTSO2(IL,J)
                  ATERM2 = DUR(IL,J) + RU(IL,J+1,IHPO) + ADMUD
                  IF (     (ABS(ATERM2) .GT. YEPS)  &
     &               .AND. (ABS(AMEAM)  .GT. YEPS)            ) THEN
                     XU(IL,J,IHPO) = XU(IL,J+1,IHPO) + ( ATERM1/ATERM2  &
     &                 - XU(IL,J+1,IHPO) ) * ( 1. - EXP(-DZ(IL,J)  &
     &                 * ATERM2 / AMEAM) )
                     AGTHP(IL,J) = XU(IL,J+1,IHPO) + ( ATERM1/ATERM2  &
     &                 - XU(IL,J+1,IHPO) ) * ( 1. + AMEAM  &
     &                / (ATERM2 * DZ(IL,J)) * ( EXP(-DZ(IL,J)  &
     &                 * ATERM2 / AMEAM) - 1.) )
                  END IF
                  IF (      ABS(AMEAM)  .GT. YEPS  &
     &                .AND. ABS(ATERM2) .LE. YEPS              ) THEN
                     XU(IL,J,IHPO) = XU(IL,J+1,IHPO)  &
     &                             + ATERM1 * DZ(IL,J) / AMEAM
                     AGTHP(IL,J) = XU(IL,J+1,IHPO)  &
     &                            + .5 * ATERM1 * DZ(IL,J) / AMEAM
                  END IF
                  IF (        ABS(ATERM2) .GT. YEPS  &
     &                .AND.   ABS(AMEAM)  .LE. YEPS            ) THEN
                     XU(IL,J,IHPO) = ATERM1 / ATERM2
                     AGTHP(IL,J)  = .5 * ( XU(IL,J+1,IHPO)  &
     &                                   + XU(IL,J,IHPO)   )
                  END IF
                  XU(IL,J,IHPO) = MAX (XU(IL,J,IHPO), 0.)
               END IF
 1160       CONTINUE
!
!-----------------------------------------------------------------------
!
!     compute updraft concentrations for additional tracers
!
!-----------------------------------------------------------------------
!
            DO 160 N=1,NTRACA-1
            DO 160 IL=1,LENGATH
               IF(KCALC(IL,J)) THEN
                  AMEAM  = .5 * ( MUR(IL,J) + MUR(IL,J+1) )
                  ADMUD  = ( MUR(IL,J) - MUR(IL,J+1) ) / DZ(IL,J)
                  ATERM1 = EUR(IL,J) * XGA(IL,J,N)
                  ATERM2 = DUR(IL,J) + RUA(IL,J+1,N) + ADMUD
                  IF (     (ABS(ATERM2) .GT. YEPS)  &
     &               .AND. (ABS(AMEAM)  .GT. YEPS)            ) THEN
                     XUA(IL,J,N) = XUA(IL,J+1,N) + ( ATERM1/ATERM2  &
     &                 - XUA(IL,J+1,N) ) * ( 1. - EXP(-DZ(IL,J)  &
     &                 * ATERM2 / AMEAM) )
                  END IF
                  IF (      ABS(AMEAM)  .GT. YEPS  &
     &                .AND. ABS(ATERM2) .LE. YEPS              ) THEN
                     XUA(IL,J,N) = XUA(IL,J+1,N)  &
     &                           + ATERM1 * DZ(IL,J) / AMEAM
                  END IF
                  IF (        ABS(ATERM2) .GT. YEPS  &
     &                .AND.   ABS(AMEAM)  .LE. YEPS            ) THEN
                     XUA(IL,J,N) = ATERM1 / ATERM2
                  END IF
               END IF
 160        CONTINUE
!
!-----------------------------------------------------------------------
!
!        determine chemical source/sink terms OU
!
!-----------------------------------------------------------------------
!
            DO 1170 IL=1,LENGATH
!       if(jlat.eq.185.and.ideep(il).eq.14.and.j.eq.20) then
!     write(0,*)'wzz: kcalc=', KCALC(IL,J), AGAMMA(IL,J), MUR(IL,J),MB(IL)
!       endif
               IF ( KCALC(IL,J) .AND. AGAMMA(IL,J) .GT. 0. ) THEN
!
!---              concentrations of relevant species
                  AFAKT = .5*(ROAROW(IDEEP(IL),J-1)+ROAROW(IDEEP(IL),J))  &
     &                  / YCOM3L
                  ACSO4 = XUA(IL,J,ISO4) * AFAKT
                  ACSO2 = XU(IL,J,ISO2) * AFAKT
                  ACHPO = XU(IL,J,IHPO) * AFAKT
                  ACNH4 = MAX(XUA(IL,J,INH3), 0.) * AFAKT
                  ACNO3 = MAX(XUA(IL,J,IHNO3), 0.)* AFAKT
                  ACCO2 = MAX(XUA(IL,J,ICO2), 0.) * AFAKT
                  ACO3  = MAX(XUA(IL,J,IO3), 0.)  * AFAKT
!
                  AMNH4 = ACNH4 / AGAMMA(IL,J)
                  AMNO3 = ACNO3 / AGAMMA(IL,J)
                  AMSO4 = ACSO4 / AGAMMA(IL,J)
                  AMSO3 = 0.
!
!---              ozone
                  ATVO3 = 1. / (1. + AGAMMA(IL,J)  &
     &                  *.5*(AEQCHO(IDEEP(IL),J-1)+AEQCHO(IDEEP(IL),J)))
                  ASRO3 = 1. - ATVO3
                  AMO3 = ACO3 * .5*(AEQCHO(IDEEP(IL),J-1)  &
     &                 + AEQCHO(IDEEP(IL),J)) * ATVO3
!
!---              for later applications...
                  AMNA  = AGTNA(IDEEP(IL),J) / AGAMMA(IL,J)
                  AMCL  = MAX(AMNA - 2.E-06,0.)
                  AGTHCL = AMCL * AGAMMA(IL,J)
!
!---              approach for pH from Tremblay with additional iteratio
                  ADELTA = AMNO3 + 2. * (AMSO4 + AMSO3) + AMCL  &
     &                   - AMNH4 - AMNA
                  AMH(IL,J) = 0.5*(ADELTA + SQRT (ADELTA**2  &
     &                      + 4.*(1.E-14 + .5*(AEQCB(IDEEP(IL),J-1)  &
     &                      + AEQCB(IDEEP(IL),J)) * ACSO2  &
     &                      + .5*(AEQCBI(IDEEP(IL),J-1)  &
     &                      + AEQCBI(IDEEP(IL),J)) * ACCO2)))
!
!---              restrict pH to avoid numerical problems
                  AMH(IL,J) = MAX (MIN(AMH(IL,J),1.E-01), 1.E-10)
!
!---              SO2-, NH3-, and NO3-equilibrium parameters
                  ATVAL=.5*(AEQCA(IDEEP(IL),J-1)+AEQCA(IDEEP(IL),J))  &
     &                 +.5*(AEQCB(IDEEP(IL),J-1)+AEQCB(IDEEP(IL),J))  &
     &                 / AMH(IL,J)  &
     &                 +.5*(AEQCC(IDEEP(IL),J-1)+AEQCC(IDEEP(IL),J))  &
     &                 / AMH(IL,J)**2
                  AFRAS = 1. / (1. + AGAMMA(IL,J) * ATVAL)
                  AFNH4 = (1.+ACNH4/(1./(.5*(AEQNH3(IDEEP(IL),J-1)  &
     &                  + AEQNH3(IDEEP(IL),J))) + AGAMMA(IL,J)  &
     &                  * AMH(IL,J)))**(-1)
                  AFNO3 = ACNO3 / ( 1./(.5*(AEQHNO(IDEEP(IL),J-1)  &
     &                  + AEQHNO(IDEEP(IL),J))) + AGAMMA(IL,J)  &
     &                  / AMH(IL,J) )
!
!---              subsequent iterations for pH-calculation
                  DO 1200 INDX = 1, ITERP
                     AFCL  = AGTHCL / ( 1./(.5*(AEQHCL(IDEEP(IL),J-1)  &
     &                     + AEQHCL(IDEEP(IL),J))) + AGAMMA(IL,J)  &
     &                     / AMH(IL,J) )
                     AFHSO = ACSO2 * AFRAS  &
     &                     *.5*(AEQCB(IDEEP(IL),J-1)+AEQCB(IDEEP(IL),J))
                     AFCO3 = ACCO2  &
     &                   *.5*(AEQCBI(IDEEP(IL),J-1)+AEQCBI(IDEEP(IL),J))
                     AFOH  = 1.E-14
                     AMSO3 = ( ACSO2 * AFRAS * .5  &
     &                     * (AEQCC(IDEEP(IL),J-1)+AEQCC(IDEEP(IL),J)) )  &
     &                     / AMH(IL,J)**2
                     ADELTA = AFNH4 * ( 2. * (AMSO4 + AMSO3) - AMNA )
                     AMH(IL,J) = 0.5*(ADELTA + SQRT (ADELTA**2  &
     &                         + 4. * AFNH4 * (AFOH + AFCO3 + AFHSO  &
     &                         + AFNO3 + AFCL)))
!       if(jlat.eq.185.and.ideep(il).eq.14.and.j.eq.20) then
!     write(0,*)'wzz: AMH  ', AMH(IL,J),INDX
!       endif
!
!---                 restrict pH to avoid numerical problems
                     AMH(IL,J) = MAX (MIN(AMH(IL,J),1.E-01), 1.E-10)
!
!---                 SO2- and H2O2-equilibrium parameters
                     ATVAL=.5*(AEQCA(IDEEP(IL),J-1)+AEQCA(IDEEP(IL),J))  &
     &                    +.5*(AEQCB(IDEEP(IL),J-1)+AEQCB(IDEEP(IL),J))  &
     &                    / AMH(IL,J)  &
     &                    +.5*(AEQCC(IDEEP(IL),J-1)+AEQCC(IDEEP(IL),J))  &
     &                    / AMH(IL,J)**2
                     AFRAS = 1. / (1. + AGAMMA(IL,J) * ATVAL)
                     AFNH4 = (1.+ACNH4/(1./(.5*(AEQNH3(IDEEP(IL),J-1)  &
     &                     + AEQNH3(IDEEP(IL),J))) + AGAMMA(IL,J)  &
     &                     * AMH(IL,J)))**(-1)
                     AFNO3 = ACNO3 / ( 1./(.5*(AEQHNO(IDEEP(IL),J-1)  &
     &                     + AEQHNO(IDEEP(IL),J))) + AGAMMA(IL,J)  &
     &                     / AMH(IL,J) )
!       if(jlat.eq.185.and.ideep(il).eq.14.and.j.eq.20) then
!     write(0,*)'wzz: AFRAS', AFRAS, AGAMMA(IL,J), ATVAL, AMH(IL,J),INDX
!       endif
 1200             CONTINUE
!
!---              scavenging ratios
                  ASRNH3 = AGAMMA(IL,J) * AMH(IL,J) * (1./AFNH4 - 1.)  &
     &                   / MAX (ACNH4,1.E-33)
                  ASRHNO = AGAMMA(IL,J) * AFNO3 / ( MAX (ACNO3,1.E-33)  &
     &                   * AMH(IL,J))
!
                  ASRSO2 = 1. - AFRAS
                  ATAEQ = .5*(AEQCHP(IDEEP(IL),J-1)+AEQCHP(IDEEP(IL),J))
                  AFRAH = 1. / (1. + AGAMMA(IL,J) * ATAEQ )
                  ASRHPO = 1. - AFRAH
!
!---              O3-oxidation rate parameter AFO3
                  AFO3 = (.5*(AORHP1(IDEEP(IL),J-1)+AORHP1(IDEEP(IL),J))  &
     &                  + .5*(AORHP2(IDEEP(IL),J-1)+AORHP2(IDEEP(IL),J))  &
     &                  / AMH(IL,J)) * AMO3 * AFRAS * ATVAL
!
!---              H2O2-oxidation rate parameter AFH2O2
                  AFH2O2 =(.5*(AORHO(IDEEP(IL),J-1)+AORHO(IDEEP(IL),J))  &
     &                   / (0.1+AMH(IL,J))) * AFRAS * AFRAH  &
     &                   *.5*(AEQCA(IDEEP(IL),J-1)+AEQCA(IDEEP(IL),J))  &
     &                   *.5*(AEQCHP(IDEEP(IL),J-1)+AEQCHP(IDEEP(IL),J))
!
!---              rate constants in 1/s
                  ATENO3 = AGAMMA(IL,J) * AFO3
                  ATENHP = AGAMMA(IL,J) * AFH2O2 * ACHPO
!
!---              chemical tendency parameters for sulphur dioxide
!---              and hydrogen peroxide in mbar/(s*m*MB)
                  AFAKT = CLDCV(IDEEP(IL),J)  &
     &                  * DP(IL,J)/(ZFG(IL,J)-ZFG(IL,J+1))/MB(IL)
                  OU(IL,J,ISO2) = ( ATENO3 + ATENHP ) * AFAKT
                  OU(IL,J,IHPO) = ATENHP * AFAKT
!
!-----------------------------------------------------------------------
!
!        determine scavenging term RU
!
!-----------------------------------------------------------------------
!                SHOULD T(IDEEP(IL)) be USED?
!
                  FAC=MAX(T(IL,MAXG(IL))-.5*(T(IL,J-1)+T(IL,J)),0.)  &
     &               /MAX(T(IL,MAXG(IL))-TFREEZ, 1.)
!                 C0= 2.E-03 * MIN(FAC**2, 1.) * MUR(IL,J)
                  C0= 2.E-03 * MIN(FAC*FAC, 1.) * MUR(IL,J)
                  ASRHPO = 0.
                  RU(IL,J,ISO2) = ASRSO2 * C0
!       if(jlat.eq.185.and.ideep(il).eq.14.and.j.eq.20) then
!     write(0,*)'wzz: step2', ru(il,j,ISO2), ASRSO2, C0, mur(il,j)
!       endif
                  RU(IL,J,IHPO) = ASRHPO * C0
                  RUA(IL,J,ISO4)  = ASRSO4 * C0
                  RUA(IL,J,INH3)  = ASRNH3 * C0
                  RUA(IL,J,IHNO3) = ASRHNO * C0
                  RUA(IL,J,ICO2)  = ASRCO2 * C0
                  RUA(IL,J,IO3)   = ASRO3  * C0
               END IF
 1170       CONTINUE
 1000    CONTINUE
!
!       if(jlat.eq.185) then
!     write(*,*)'wzz: step2', ru(11,20,ISO2), iterp
!       endif
!-----------------------------------------------------------------------
!
!     compute updraft concentrations XU of sulphate for each bin
!
!-----------------------------------------------------------------------
!
!---     the updraft concentrations are calculated from the
!---     precalculated source/sink terms of total sulphate
!---     to avoid numerical instabilities
         DO 156 N=1,ISIZE
            NF=NFMIN+N-1
            DO 158 J=ILEV,MASG+2,-1
            DO 158 IL=1,LENGATH
               IF ( KCALC(IL,J) ) THEN
                  AMEAM  = .5 * ( MUR(IL,J) + MUR(IL,J+1) )
                  ADMUD  = ( MUR(IL,J) - MUR(IL,J+1) ) / DZ(IL,J)
                  ATERM1 = EUR(IL,J) * XG(IL,J,NF) + OU(IL,J+1,ISO2)  &
     &                   * AGTSO2(IL,J) * AFRAC(IDEEP(IL),J,N)
                  IF (N .EQ. INT(RCRIT(IDEEP(IL),J)))  THEN
                     ATERM2 = DUR(IL,J) + ADMUD
                  END IF
                  IF (N .GT. INT(RCRIT(IDEEP(IL),J))) THEN
                     ATERM2 = DUR(IL,J) + RUA(IL,J+1,ISO4) + ADMUD
                  ELSE
                     ATERM2 = DUR(IL,J) + ADMUD
                  END IF
                  IF (     (ABS(ATERM2) .GT. YEPS)  &
     &               .AND. (ABS(AMEAM)  .GT. YEPS)            ) THEN
                     XU(IL,J,NF) = XU(IL,J+1,NF) + ( ATERM1/ATERM2  &
     &                    - XU(IL,J+1,NF) ) * ( 1. - EXP(-DZ(IL,J)  &
     &                    * ATERM2 / AMEAM) )
                  END IF
                  IF (      ABS(AMEAM)  .GT. YEPS  &
     &                .AND. ABS(ATERM2) .LE. YEPS              ) THEN
                     XU(IL,J,NF) = XU(IL,J+1,NF)  &
     &                           + ATERM1 * DZ(IL,J) / AMEAM
                  END IF
                  IF (        ABS(ATERM2) .GT. YEPS  &
     &                .AND.   ABS(AMEAM)  .LE. YEPS            ) THEN
                     XU(IL,J,NF) = ATERM1 / ATERM2
                  END IF
               END IF
 158        CONTINUE
 156     CONTINUE
!
!     write(*,*)'wzz: step2-1'
!-----------------------------------------------------------------------
!
!     compute downdraft concentrations XD of all prognostic tracers
!
!-----------------------------------------------------------------------
!        if(jlat.eq.249) then
!         write(0,*)'wzz: xd01=', xd(2,ilev,49),mdr(2,ilev),mdr(2,ilev-1)
!        endif
         DO 220 N=1,NTRAC
         IF ( (N .EQ. ISO2) .OR. (N .EQ. IHPO)  &
     &       .OR. ((N .GE. NFMIN) .AND. (N .LE. NFMAX)) ) THEN
!          if(n.eq.49.and.jlat.eq.249)write(0,*)'wzz: N=', N, NTRAC, ISO2, IHPO, yeps,
!    &        NFMIN, NFMAX, jd(2),maxg(2),masg
            DO 222 IL=1,LENGATH
               XD(IL,JD(IL),N)=XG(IL,JD(IL)-1,N)
  222       CONTINUE
            DO 225 J=MASG+2,ILEV
!           if(n.eq.49) then
!           write(*,*)'wzz: j=', j, jd(1), maxg(1),dz(i,j-1)
!           endif
            DO 225 IL=1,LENGATH
               IF((J.GT.JD(IL) .AND. J.LE.MAXG(IL))  &
     &           .AND. EPS0(IL).GT.0. .AND. JD(IL).LT.MAXG(IL)) THEN
                 AMEAM  = .5 * ( MDR(IL,J-1) + MDR(IL,J) )
                 ADMUD  = ( MDR(IL,J-1) - MDR(IL,J) ) / DZ(IL,J-1)
                 ATERM1 = EDR(IL,J-1) * XG(IL,J-1,N)
                 ATERM2 = ADMUD
!        if(jlat.eq.249.and.il.eq.2.and.n.eq.49) then
!         write(0,*)'wzz: aterm =', ATERM1, ATERM2, AMEAM, DZ(IL,J-1)
!    &            , j
!        endif
                 IF (     (ABS(ATERM2) .GT. YEPS)  &
     &              .AND. (ABS(AMEAM)  .GT. YEPS)            ) THEN
                    XD(IL,J,N) = XD(IL,J-1,N) - ( XD(IL,J-1,N)  &
     &                   - ATERM1/ATERM2 ) * ( 1. - EXP( DZ(IL,J-1)  &
     &                   * ATERM2/AMEAM) )
!        if(jlat.eq.249.and.il.eq.2.and.j.eq.ilev.and.n.eq.49) then
!        if(jlat.eq.249.and.il.eq.2.and.(XD(IL,J,N).ne.XD(IL,J,N)).and.n.eq.49) then
!         write(0,*)'wzz: xd02=', xd(il,j,n),XD(IL,J-1,N),ATERM1, ATERM2,
!    &            AMEAM, DZ(IL,J-1), EXP( DZ(IL,J-1)*ATERM2/AMEAM)
!    &            ,MDR(IL,J-1), MDR(IL,J), j
!        endif
                 END IF
                 IF (      ABS(AMEAM)  .GT. YEPS  &
     &                .AND. ABS(ATERM2) .LE. YEPS              ) THEN
                    XD(IL,J,N) = XD(IL,J-1,N)  &
     &                      - ATERM1 * DZ(IL,J-1) / AMEAM
!        if(jlat.eq.249.and.il.eq.2.and.j.eq.ilev.and.n.eq.49) then
!         write(0,*)'wzz: xd03=', xd(il,j,n),XD(IL,J-1,N),ATERM1, AMEAM,
!    &            DZ(IL,J-1)
!        endif
                 END IF
                 IF (        ABS(ATERM2) .GT. YEPS  &
     &                .AND.   ABS(AMEAM)  .LE. YEPS            ) THEN
                    XD(IL,J,N) = ATERM1 / ATERM2
!        if(jlat.eq.249.and.il.eq.2.and.j.eq.ilev.and.n.eq.49) then
!         write(0,*)'wzz: xd04=', xd(il,j,n),ATERM1, ATERM2
!        endif
                 END IF
                 XD(IL,J,N) = MAX(XD(IL,J,N), 0.)
               ENDIF
 225        CONTINUE
            END IF
 220     CONTINUE
!        if(jlat.eq.249) then
!         write(0,*)'wzz: xd 1=', xd(2,ilev,49)
!        endif
!
!-----------------------------------------------------------------------
!
!     compute tracer tendecies
!
!-----------------------------------------------------------------------
!
!     write(*,*)'wzz: step2-2'
!---  prepare arrays for the implicit time stepping/solver
!---  initialize the tendecy to zero on ALL levels
      CALL PUTZERO  (DXDT  , ILG*ILEV*NTRAC)
!
!---  calculate factor FACX for semi-implicit solver
         DO 260 N=1,NTRAC
            IF (     (N .EQ. ISO2) .OR. (N .EQ. IHPO)  &
     &         .OR. ((N .GE. NFMIN) .AND. (N .LE. NFMAX)) ) THEN
               DO 265 K=MASG+1,ILEV
               DO 265 IL=1,LENGATH
                  IF (K.GE.JT(IL) .AND. K.LT.ILEV) THEN
                     AXDIF = 0.
                     IF(XG(IL,K,N) .GT. 0. .AND. XG(IL,K+1,N) .GT. 0.)  &
     &                  AXDIF = ABS((XG(IL,K,N)-XG(IL,K+1,N))  &
     &                         / MAX(XG(IL,K,N),XG(IL,K+1,N)))
                     IF (AXDIF .GT. YSMALL) THEN
                        AXG   = XG(IL,K,N)
                        AXGP1 = XG(IL,K+1,N)
                        IF ( AXG .LE. 0. ) THEN
                           AXG   = YSMALL * AXGP1
                        END IF
                        IF ( AXGP1 .LE. 0. .AND. AXG .GT. 0.) THEN
                           AXGP1 = YSMALL * AXG
                        END IF
                        DXDX(IL,K,N) = AXGP1 / (AXG-AXGP1)  &
     &                                -LOG( AXG / AXGP1 )  &
     &                       * ( AXGP1 / (AXG-AXGP1) )**2
                     ELSE
                        DXDX(IL,K,N)=0.5
                     ENDIF
                     FACX(IL,K,N)=1./( 1. + MC(IL,K+1)  &
     &                           *2.*DELT/DP(IL,K)*RNU*DXDX(IL,K,N) )
                  ENDIF
 265           CONTINUE
            END IF
 260     CONTINUE
!
!     write(*,*)'wzz: step3'
!---  compute tendency for the tracers (DXDT)
!---  do this for top of the updraft; MB is the mass flux at cloud base
         DO 280 N=1,NTRAC
         IF ( (N .EQ. ISO2) .OR. (N .EQ. IHPO)  &
     &       .OR. ((N .GE. NFMIN) .AND. (N .LE. NFMAX)) ) THEN
            DO 285 J=MASG+1,ILEV-1
            DO 285 IL=1,LENGATH
               IF(J.EQ.JT(IL) .AND. MB(IL).GT.0.)       THEN
                  DXDT(IL,J,N)=FACX(IL,J,N)/DP(IL,J)  &
     &            * (MU(IL,J+1)*( XU(IL,J+1,N)-XHAT(IL,J+1,N) ) +  &
     &               MD(IL,J+1)*( XD(IL,J+1,N)-XHAT(IL,J+1,N) ))
               ENDIF
  285       CONTINUE
            END IF
  280    CONTINUE
!      if(jlat.eq.185) then
!        write(0,*)'wzz: dxdt00=', DXDT(11,19,63), XHAT(11,19,63),MASG,LENGATH
!      endif
!
!---  continue to compute the tendencies for the other layers
!---  between JT and MAXG (the launching level of the cloud base)
         DO 300 N=1,NTRAC
         IF ( (N .EQ. ISO2) .OR. (N .EQ. IHPO)  &
     &       .OR. ((N .GE. NFMIN) .AND. (N .LE. NFMAX)) ) THEN
            DO 305 J=MASG+1,ILEV
            DO 305 IL=1,LENGATH
               IF(J.GT.JT(IL) .AND. J.LT.MAXG(IL)  &
     &           .AND. MB(IL).GT.0.) THEN
                  XHAT(IL,J,N)=XHAT(IL,J,N)  &
     &              +DXDT(IL,J-1,N)*2.*DELT*RNU*DXDX(IL,J-1,N)
                  DXDT(IL,J,N)=FACX(IL,J,N)  &
     &           *((MC(IL,J)*(XHAT(IL,J,N)-XG(IL,J,N))  &
     &           +MC(IL,J+1)*(XG(IL,J,N)-XHAT(IL,J+1,N)))/DP(IL,J)  &
     &           +DU(IL,J)*(0.5*(XU(IL,J,N)+XU(IL,J+1,N))-XG(IL,J,N)) )
!      if(jlat.eq.185.and.N.eq.63.and.il.eq.11) then
!        write(0,*)'wzz: dxdt001=', DXDT(IL,J,N),FACX(IL,J,N),MC(IL,J),
!    &     XHAT(IL,J,N),XG(IL,J,N),MC(IL,J+1),XHAT(IL,J+1,N),DP(IL,J),
!    &      DU(IL,J), XU(IL,J,N),XU(IL,J+1,N),RNU,DXDT(IL,J-1,N),DXDX(IL,J-1,N)
!      endif
               ENDIF
  305       CONTINUE
            END IF
  300    CONTINUE
!      if(jlat.eq.185) then
!        write(0,*)'wzz: dxdt01=', DXDT(11,19,63),DXDT(12,19,63)
!      endif
!
!---  now compute the detrainment at cloud base
         DO 320 N=1,NTRAC
            IF ( (N .EQ. ISO2) .OR. (N .EQ. IHPO)  &
     &         .OR. ((N .GE. NFMIN) .AND. (N .LE. NFMAX)) ) THEN
            DO 325 J=MASG+1,ILEV
            DO 325 IL=1,LENGATH
               IF(J.EQ.MAXG(IL) .AND. MB(IL).GT.0.)           THEN
                  XHAT(IL,J,N)=XHAT(IL,J,N)  &
     &               +DXDT(IL,J-1,N)*2.*DELT*RNU*DXDX(IL,J-1,N)
                  DXDT(IL,J,N)=(1./DSUBCLD(IL))  &
     &               *( MU(IL,J)*( XHAT(IL,J,N)-XU(IL,J,N))  &
     &                 +MD(IL,J)*( XHAT(IL,J,N)-XD(IL,J,N)) )
!      if(jlat.eq.249.and.n.eq.49.and.il.eq.2.and.j.eq.ilev) then
!        write(0,*)'wzz: dxdt020=', DXDT(il,j,N),DSUBCLD(IL), MU(IL,J),
!    &    XHAT(IL,J,N), XU(IL,J,N), MD(IL,J), XHAT(IL,J,N), XD(IL,J,N)
!      endif
               ENDIF
 325        CONTINUE
            END IF
 320     CONTINUE
!      if(jlat.eq.185) then
!        write(0,*)'wzz: dxdt02=', DXDT(11,19,63)
!      endif
!
!-----------------------------------------------------------------------
!
!     calculate resulting large scale tendencies
!
!-----------------------------------------------------------------------
!
!---     hydrogen peroxide
         DO 345 L=MASG+1,ILEV
            DO K=1,LENGATH
               TMP2(K)=RTHPO(IDEEP(K),L)  &
     &                      +DXDT(K,L,IHPO)*AMASPE(3)
            END DO
            DO 345 K=1,LENGATH
!
!---           hydrogen peroxide tendency
               RTHPO(IDEEP(K),L) = TMP2(K)
 345     CONTINUE
!
!-----------------------------------------------------------------------
!
!     calculate in-cloud oxidation (OU) and washout rates (RU)
!     for diagnostics purposes
!
!-----------------------------------------------------------------------
!
!---  safe old tendencies for in-cloud production (in mol/m^3/s) 
!---  at lower thermodynamic layers
!
      DO 350 J=MASG+2,ILEV
      DO 350 IL=1,LENGATH
         IF (J .LT. ILEV) THEN
            AFAKT = MB(IL)*ROAROW(IDEEP(IL),J)/DP(IL,J)  &
     &                                      *(ZFG(IL,J)-ZFG(IL,J+1))
            OU(IL,J,ISO2) = - AFAKT * OU(IL,J+1,ISO2) * AGTSO2(IL,J) 
            OU(IL,J,IHPO) = - AFAKT * OU(IL,J+1,IHPO) * AGTHP (IL,J)
            RU(IL,J,ISO2) = - AFAKT * RU(IL,J+1,ISO2) * AGTSO2(IL,J) 
            RU(IL,J,IHPO) = - AFAKT * RU(IL,J+1,IHPO) * AGTHP (IL,J)
         ELSE
            OU(IL,J,ISO2) = 0.
            OU(IL,J,IHPO) = 0.
            RU(IL,J,ISO2) = 0.
            RU(IL,J,IHPO) = 0.
         END IF
 350  CONTINUE
!
!---  safe old tendencies for in-cloud washout (in mol/m^3/s) 
!---  at lower thermodynamic layers
!
      DO 365 N=1,ISIZE
         NF=NFMIN+N-1
         DO 360 J=MASG+2,ILEV
         DO 360 IL=1,LENGATH
            IF (J .LT. ILEV) THEN
               AFAKT = MB(IL)*ROAROW(IDEEP(IL),J)/DP(IL,J)  &
     &                            *(ZFG(IL,J)-ZFG(IL,J+1))
               RU(IL,J,NF) = - AFAKT * RUA(IL,J+1,ISO4)  &
     &                * AGTSO4(IL,J) * AFRAC(IDEEP(IL),J,N)
            ELSE
               RU(IL,J,NF) = 0.
            END IF
 360     CONTINUE
 365  CONTINUE
!
!---  correct diagnostics parameters
!
      CALL PUTZERO(TMP1, ILG)
      CALL PUTZERO(TMP2, ILG)
      CALL PUTZERO(TMP3, ILG)
      DO 385 J=MASG+1,ILEV
         DO 380 IL=1,LENGATH
            TMP1(IL) = TMP1(IL)+DZT(IDEEP(IL),J)*DXDT(IL,J,ISO2)  &
     &                      *ROAROW(IDEEP(IL),J)
            TMP2(IL) = TMP2(IL)+DZT(IDEEP(IL),J)*RU(IL,J,ISO2)
            TMP3(IL) = TMP3(IL)+DZT(IDEEP(IL),J)*OU(IL,J,ISO2)
 380     CONTINUE
 385  CONTINUE
      DO IL=1,LENGATH
         SUMDX(IL) = TMP1(IL) / MIN(TMP2(IL)+TMP3(IL),-YEPS) 
      END DO
      DO 390 J=MASG+2,ILEV-1
      DO 390 IL=1,LENGATH
         RU(IL,J,ISO2) = RU(IL,J,ISO2) * SUMDX(IL)
         OU(IL,J,ISO2) = OU(IL,J,ISO2) * SUMDX(IL)
         OU(IL,J,IHPO) = OU(IL,J,IHPO) * SUMDX(IL)
 390  CONTINUE

      CALL PUTZERO(TMP1, ILG)
      CALL PUTZERO(TMP2, ILG)
      CALL PUTZERO(TMP3, ILG)
      DO 400 J=MASG+1,ILEV
        DO 400 IL=1,LENGATH
            TMP1(IL) = TMP1(IL)+DZT(IDEEP(IL),J)*DXDT(IL,J,IHPO)  &
     &             *ROAROW(IDEEP(IL),J)
            TMP2(IL) = TMP2(IL)+DZT(IDEEP(IL),J)*RU(IL,J,IHPO)
            TMP3(IL) = TMP3(IL)+DZT(IDEEP(IL),J)*OU(IL,J,IHPO)
  400 CONTINUE

      DO 405 IL=1,LENGATH
          SUMDX(IL) = (TMP1(IL)-TMP3(IL)) / MIN(TMP2(IL),-YEPS) 
  405 CONTINUE
      DO 410 J=MASG+2,ILEV-1
      DO 410 IL=1,LENGATH
         RU(IL,J,IHPO) = RU(IL,J,IHPO) * SUMDX(IL)
 410  CONTINUE
      DO 450 N=1,ISIZE
         NF=NFMIN+N-1
         DO 455 J=MASG+2,ILEV-1
         DO 455 IL=1,LENGATH
            OU(IL,J,NF) = -OU(IL,J,ISO2) * AFRAC(IDEEP(IL),J,N)
  455    CONTINUE
  450 CONTINUE
      
      CALL PUTZERO(TMP1, ILG)
      CALL PUTZERO(TMP2, ILG)
      CALL PUTZERO(TMP3, ILG)
      DO 465 N=1,ISIZE
         NF=NFMIN+N-1
         DO 460 J=MASG+1,ILEV
            DO 460 IL=1,LENGATH
               TMP1(IL) = TMP1(IL)+DZT(IDEEP(IL),J)*DXDT(IL,J,NF) &
     &                     *ROAROW(IDEEP(IL),J)
               TMP2(IL) = TMP2(IL)+DZT(IDEEP(IL),J)*RU(IL,J,NF)
               TMP3(IL) = TMP3(IL)+DZT(IDEEP(IL),J)*OU(IL,J,NF)
  460    CONTINUE
  465 CONTINUE

      DO IL=1,LENGATH
         SUMDX(IL) = (TMP1(IL)-TMP3(IL))/MIN(TMP2(IL),-YEPS) 
      END DO
      DO N=1,ISIZE
         NF=NFMIN+N-1
         DO J=MASG+2,ILEV-1
            DO IL=1,LENGATH
               RU(IL,J,NF) = RU(IL,J,NF) * SUMDX(IL)
            END DO
         END DO
      END DO
!
!---  calculate final production and loss rates (in kg/kg/s)
      DO 500 J=MASG+1,ILEV
         DO IL=IL1,IL2
            TMP1(IL)=0.0
            TMP2(IL)=0.0
            TMP3(IL)=0.0
         END DO
         DO 510 IL=1,LENGATH
            TMP1(IDEEP(IL)) = RU(IL,J,ISO2) * AMASPE(2) &
     &                      / ROAROW(IDEEP(IL),J)
            TMP2(IDEEP(IL)) = OU(IL,J,ISO2) * AMASPE(2)&
     &                      / ROAROW(IDEEP(IL),J)
 510     CONTINUE
         DO 515 IL=IL1,IL2
            RU(IL,J,ISO2) = TMP1(IL)
            OU(IL,J,ISO2) = TMP2(IL)
 515     CONTINUE
         DO 517 IL=1,LENGATH
            TMP3(IDEEP(IL)) = DXDT(IL,J,ISO2) * AMASPE(2) &
     &                      - TMP1(IDEEP(IL)) - TMP2(IDEEP(IL))
 517     CONTINUE
         DO 518 IL=IL1,IL2
            DXDT(IL,J,ISO2) = TMP3(IL)
 518     CONTINUE
!
!     UPDATE THE SO2 TENDENCY
!
          DO 519 IL=IL1,IL2
!
!---           sulphur dioxide removal rate
              RTSO2(IL,J)=OU(IL,J,ISO2)+DXDT(IL,J,ISO2)
              RTBCLD(IL,J,ISO2)=RTBCLD(IL,J,ISO2)+  &
     &                                AMIN1(0.0,RU(IL,J,ISO2))
 519      CONTINUE
!      if(jlat.eq.185.and.j.eq.20) then
!        write(0,*)'wzz: dxdt1=', DXDT(14,J,63), ru(14,J,63),ou(14,J,63)
!    &        ,RTSO2(14,J), RTBCLD(14,J,ISO2)
!      endif
 500  CONTINUE
!
      DO 520 J=1+MASG,ILEV
         DO 525 IL=IL1,IL2
            TMP1(IL) = 0.
            TMP2(IL) = 0.
            TMP3(IL) = 0.
 525     CONTINUE
         DO 530 IL=1,LENGATH
            TMP1(IDEEP(IL)) = RU(IL,J,IHPO) * AMASPE(3)  &
     &                      / ROAROW(IDEEP(IL),J)
            TMP2(IDEEP(IL)) = OU(IL,J,IHPO) * AMASPE(3)  &
     &                      / ROAROW(IDEEP(IL),J)
 530     CONTINUE
         DO 535 IL=IL1,IL2
            RU(IL,J,IHPO) = TMP1(IL)
            OU(IL,J,IHPO) = TMP2(IL)
 535     CONTINUE
         DO 537 IL=1,LENGATH
            TMP3(IDEEP(IL)) = DXDT(IL,J,IHPO) * AMASPE(3)   &
     &                      - TMP1(IDEEP(IL)) - TMP2(IDEEP(IL))
 537     CONTINUE
         DO 538 IL=IL1,IL2
            DXDT(IL,J,IHPO) = TMP3(IL)
 538     CONTINUE
 520  CONTINUE
!      if(jlat.eq.249) then
!        write(0,*)'wzz: dxdt2=', DXDT(2,ilev,49)
!      endif
!
      DO 540 N=1,ISIZE
         NF=NFMIN+N-1
         DO 550 J=1+MASG,ILEV
            DO 555 IL=IL1,IL2
               TMP1(IL) = 0.
               TMP2(IL) = 0.
               TMP3(IL) = 0.
 555        CONTINUE
            DO 560 IL=1,LENGATH
               TMP1(IDEEP(IL)) = RU(IL,J,NF) * AMASPE(1) &
     &                         / ROAROW(IDEEP(IL),J)
               TMP2(IDEEP(IL)) = OU(IL,J,NF) * AMASPE(1)&
     &                         / ROAROW(IDEEP(IL),J)
 560        CONTINUE
            DO 565 IL=IL1,IL2
               RU(IL,J,NF) = TMP1(IL)
               OU(IL,J,NF) = TMP2(IL)
 565        CONTINUE
            DO 567 IL=1,LENGATH
               TMP3(IDEEP(IL)) = DXDT(IL,J,NF) * AMASPE(1) &
     &                         - TMP1(IDEEP(IL)) - TMP2(IDEEP(IL))
 567        CONTINUE
            DO 568 IL=IL1,IL2
               DXDT(IL,J,NF) = TMP3(IL)
 568        CONTINUE
 550     CONTINUE
 540  CONTINUE
!      if(jlat.eq.249) then
!        write(0,*)'wzz: dxdt3=', DXDT(2,ilev,49)
!      endif
!
!      UNDATE THE SULPHATE TENDENCY
!
      DO N=1,ISIZE
         NF=NFMIN+N-1
         DO L=MASG+1,ILEV
           DO IL=IL1,IL2
               RTSO4(IL,L,N)=OU(IL,L,NF)+DXDT(IL,L,NF)
               RTBCLD(IL,L,NF)=RTBCLD(IL,L,NF)+&
     &                                AMIN1(0.0,RU(IL,L,NF))
           END DO
         END DO
      END DO
!
!       DIAGNOSTIC WRITEOUTS
!
      IF (IDEBUG .GT. 1) THEN
         TTSO4=0.0
         TTRU=0.0
         TTSO2=0.0
         TTOV=0.0
         SORU=0.0
         DDDT=0.0
         DDDV=0.0
         TTOU=0.0
         DO N=1,ISIZE
         NF=NFMIN+N-1
            DO L=1+MASG,ILEV
              GO=DZT(ID,L)*ROAROW(ID,L)
              TTSO4=TTSO4+RTSO4(ID,L,N)*GO
              TTRU=TTRU+RU(ID,L,NF)*GO
              DDDV=DDDV+DXDT(ID,L,NF)*GO
              TTOV=TTOV+OU(ID,L,NF)*GO
             IF(N.EQ.1) THEN
                TTSO2 = TTSO2 + RTSO2(ID,L)*GO
                TTOU = TTOU + OU(ID,L,ISO2)*GO
                SORU = SORU + RU(ID,L,ISO2)*GO
                DDDT = DDDT+DXDT(ID,L,ISO2)*GO
             END IF
            END DO
         END DO
         SO22=TTOU+SORU
         SO44=TTRU-AMASPE(1)/AMASPE(2)*TTOU
                 WRITE (*,*) ' SO2 SO4 ++ ', TTSO2,TTSO4
                 WRITE (*,*) ' SO2 SO4    ', SO22,SO44
                 WRITE (*,*) ' DXDT       ', DDDT, DDDV
      END IF
!
      RETURN
      END
