!EOC
!------------------------------------------------------------------------------
!          Harvard University Atmospheric Chemistry Modeling Group            !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: read_jv_atms_dat
!
! !DESCRIPTION: Reads the default T and O3 profiles for FAST-J photolysis.
!  This replaces the obsolete rd\_prof.F routine, which read from the ASCII
!  file "jv\_atms.dat".
!\\
!\\
! !INTERFACE:
!
SUBROUTINE READ_JV_ATMS_DAT( am_I_Root )
!
! !USES:
!
  ! Modules for netCDF read
  USE m_netcdf_io_open
  USE m_netcdf_io_read
  USE m_netcdf_io_readattr
  USE m_netcdf_io_close

  ! GEOS-Chem modules
  USE DIRECTORY_MOD, ONLY : DATA_DIR_1x1    ! Data directory 
  USE JV_CMN_MOD,    ONLY : TREF            ! Default T profile [K]
  USE JV_CMN_MOD,    ONLY : OREF            ! Default O3 profile [ppm]
      
  IMPLICIT NONE

# include "netcdf.inc"

! !INPUT PARAMETERS:
!
  LOGICAL, INTENT(IN) :: am_I_Root          ! Is this the root CPU?
!
! !REMARKS:
!  This file was automatically generated by the Perl scripts in the
!  NcdfUtilities package (which ships w/ GEOS-Chem) and was subsequently
!  hand-edited.
!
! !REVISION HISTORY:
!  19 Apr 2012 - R. Yantosca - Initial version
!  30 Jul 2012 - R. Yantosca - Now accept am_I_Root as an argument when
!                              running with the traditional driver main.F
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
  ! netCDF file handle
  INTEGER            :: fId                 ! netCDF file ID

  ! Character strings
  CHARACTER(LEN=255) :: nc_dir              ! netCDF directory name
  CHARACTER(LEN=255) :: nc_file             ! netCDF file name
  CHARACTER(LEN=255) :: nc_path             ! netCDF path name
  CHARACTER(LEN=255) :: v_name              ! netCDF variable name 
  CHARACTER(LEN=255) :: a_name              ! netCDF attribute name
  CHARACTER(LEN=255) :: a_val               ! netCDF attribute value

  ! Arrays for netCDF start and count values
  INTEGER            :: st3d(3), ct3d(3)    ! For 3D arrays 

  !=========================================================================
  ! Open and read data from the netCDF file
  !=========================================================================

  ! Directory and file names
  nc_dir  = TRIM( DATA_DIR_1x1 ) // 'FastJ_201204/'      
  nc_file = 'fastj.jv_atms_dat.nc'  
  nc_path = TRIM( nc_dir ) // TRIM( nc_file )

  ! Open netCDF file
  CALL Ncop_Rd( fId, TRIM(nc_path) )
  
  ! Echo info to stdout
  IF ( am_I_Root ) THEN
     WRITE( 6, 100 ) REPEAT( '%', 79 )
     WRITE( 6, 110 ) TRIM(nc_file)
     WRITE( 6, 120 ) TRIM(nc_dir)
  ENDIF

  !----------------------------------------
  ! VARIABLE: T
  !----------------------------------------
  
  ! Variable name
  v_name = "T"
  
  ! Read T from file
  st3d   = (/  1,  1,  1 /)
  ct3d   = (/ 51, 18, 12 /)
  CALL NcRd( TREF, fId, TRIM(v_name), st3d, ct3d )
  
  ! Read the T:units attribute
  a_name = "units"
  CALL NcGet_Var_Attributes( fId,TRIM(v_name),TRIM(a_name),a_val )
  
  ! Echo info to stdout
  IF ( am_I_Root ) THEN 
     WRITE( 6, 130 ) TRIM(v_name), TRIM(a_val)
  ENDIF

  !----------------------------------------
  ! VARIABLE: O3
  !----------------------------------------
  
  ! Variable name
  v_name = "O3"
  
  ! Read O3 from file
  st3d   = (/  1,  1,  1 /)
  ct3d   = (/ 51, 18, 12 /)
  CALL NcRd( OREF, fId, TRIM(v_name), st3d, ct3d )
  
  ! Read the O3:units attribute
  a_name = "units"
  CALL NcGet_Var_Attributes( fId,TRIM(v_name),TRIM(a_name),a_val )
      
  ! Echo info to stdout
  IF ( am_I_Root ) THEN 
     WRITE( 6, 130 ) TRIM(v_name), TRIM(a_val)
  ENDIF

  !=================================================================
  ! Cleanup and quit
  !=================================================================

  ! Close netCDF file
  CALL NcCl( fId )

  ! Echo info to stdout
  IF ( am_I_Root ) THEN 
     WRITE( 6, 140 )
     WRITE( 6, 100 ) REPEAT( '%', 79 )
  ENDIF

  ! FORMAT statements
100 FORMAT( a                                              )
110 FORMAT( '%% Opening file  : ',         a               )
120 FORMAT( '%%  in directory : ',         a, / , '%%'     )
130 FORMAT( '%% Successfully read ',       a, ' [', a, ']' )
140 FORMAT( '%% Successfully closed file!'                 )
  
END SUBROUTINE READ_JV_ATMS_DAT
!EOC
