! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Linear Algebra Data and Routines File
! 
! Generated by KPP-2.2 symbolic chemistry Kinetics PreProcessor
!       (http://www.cs.vt.edu/~asandu/Software/KPP)
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian, Villanova University, USA
!        R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
! 
! File                 : gckpp_LinearAlgebra.f90
! Time                 : Tue Jul 24 13:27:00 2012
! Working directory    : /home/mpayer/KPP
! Equation file        : gckpp.kpp
! Output root filename : gckpp
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE gckpp_LinearAlgebra

  USE gckpp_Parameters
  USE gckpp_JacobianSP

  IMPLICIT NONE

CONTAINS


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! SPARSE_UTIL - SPARSE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecomp( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: IER
      REAL(kind=dp) :: JVS(LU_NONZERO), W(NVAR), a
      INTEGER  :: k, kk, j, jj

      a = 0. ! mz_rs_20050606
      IER = 0
      DO k=1,NVAR
        ! mz_rs_20050606: don't check if real value == 0
        ! IF ( JVS( LU_DIAG(k) ) .EQ. 0. ) THEN
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(a) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecomp


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplx( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization, complex
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: IER
      DOUBLE COMPLEX :: JVS(LU_NONZERO), W(NVAR), a
      REAL(kind=dp)  :: b = 0.0
      INTEGER        :: k, kk, j, jj

      IER = 0
      DO k=1,NVAR
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(b) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecompCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplxR( JVSR, JVSI, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!    Sparse LU factorization, complex
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: IER
      REAL(kind=dp) :: JVSR(LU_NONZERO), JVSI(LU_NONZERO) 
      REAL(kind=dp) :: WR(NVAR), WI(NVAR), ar, ai, den
      INTEGER       :: k, kk, j, jj

      IER = 0
      ar  = 0.0
      DO k=1,NVAR
        IF (  ( ABS(JVSR(LU_DIAG(k))) < TINY(ar) ) .AND. &
              ( ABS(JVSI(LU_DIAG(k))) < TINY(ar) ) )  THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              WR( LU_ICOL(kk) ) = JVSR(kk)
              WI( LU_ICOL(kk) ) = JVSI(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            den = JVSR(LU_DIAG(j))**2 + JVSI(LU_DIAG(j))**2
            ar = -(WR(j)*JVSR(LU_DIAG(j)) + WI(j)*JVSI(LU_DIAG(j)))/den
            ai = -(WI(j)*JVSR(LU_DIAG(j)) - WR(j)*JVSI(LU_DIAG(j)))/den
            WR(j) = -ar
            WI(j) = -ai
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               WR( LU_ICOL(jj) ) = WR( LU_ICOL(jj) ) + ar*JVSR(jj) - ai*JVSI(jj)
               WI( LU_ICOL(jj) ) = WI( LU_ICOL(jj) ) + ar*JVSI(jj) + ai*JVSR(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVSR(kk) = WR( LU_ICOL(kk) )
            JVSI(kk) = WI( LU_ICOL(kk) )
         END DO
      END DO

END SUBROUTINE KppDecompCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
	  X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
	END DO
      END DO

      DO i=NVAR, 1, -1
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
	  X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
	END DO
      END DO
      
END SUBROUTINE KppSolveTRIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveCmplx

! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), sumr, sumi, den

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             XR(i) = XR(i) - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
             XI(i) = XI(i) - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
         END DO  
      END DO

      DO i=NVAR,1,-1
        sumr = XR(i); sumi = XI(i)
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
            sumr = sumr - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
            sumi = sumi - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
        END DO
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (sumr*JVSR(LU_DIAG(i)) + sumi*JVSI(LU_DIAG(i)))/den
        XI(i) = (sumi*JVSR(LU_DIAG(i)) - sumr*JVSI(LU_DIAG(i)))/den
      END DO
      
END SUBROUTINE KppSolveCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
	  X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
	END DO
      END DO

      DO i=NVAR, 1, -1
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
	  X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
	END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve transpose subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), den

      DO i=1,NVAR
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (XR(i)*JVSR(LU_DIAG(i)) + XI(i)*JVSI(LU_DIAG(i)))/den
        XI(i) = (XI(i)*JVSR(LU_DIAG(i)) - XR(i)*JVSI(LU_DIAG(i)))/den
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
	  XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
	  XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
	END DO
      END DO

      DO i=NVAR, 1, -1
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
	  XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
	  XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
	END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplxR


!
! Next few commented subroutines perform sparse big linear algebra
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppDecompBig( JVS, IP, IER )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse LU factorization
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: IP3(3), IER, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), W(3,3,NVAR), a(3,3), E(3,3)
!      INTEGER  :: k, kk, j, jj
!
!      a = 0.0d0
!      IER = 0
!      DO k=1,NVAR
!        DO kk = LU_CROW(k), LU_CROW(k+1)-1
!              W( 1:3,1:3,LU_ICOL(kk) ) = JVS(1:3,1:3,kk)
!        END DO
!        DO kk = LU_CROW(k), LU_DIAG(k)-1
!            j = LU_ICOL(kk)
!            E(1:3,1:3) = JVS( 1:3,1:3,LU_DIAG(j) )
!            ! CALL DGETRF(3,3,E,3,IP3,IER) 
!            CALL FAC3(E,IP3,IER)
!            IF ( IER /= 0 )  RETURN
!            ! a = W(j) / JVS( LU_DIAG(j) )
!            a(1:3,1:3) = W( 1:3,1:3,j )
!            ! CALL DGETRS ('N',3,3,E,3,IP3,a,3,IER) 
!            CALL SOL3('N',E,IP3,a(1,1))
!            CALL SOL3('N',E,IP3,a(1,2))
!            CALL SOL3('N',E,IP3,a(1,3))
!            W(1:3,1:3,j) = a(1:3,1:3)
!            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
!               W( 1:3,1:3,LU_ICOL(jj) ) = W( 1:3,1:3,LU_ICOL(jj) ) &
!                        - MATMUL( a(1:3,1:3) , JVS(1:3,1:3,jj) )
!            END DO
!         END DO
!         DO kk = LU_CROW(k), LU_CROW(k+1)-1
!            JVS(1:3,1:3,kk) = W( 1:3,1:3,LU_ICOL(kk) )
!         END DO
!      END DO
!
!      DO k=1,NVAR
!         ! CALL WGEFA(JVS(1,1,LU_DIAG(k)),3,3,IP(1,k),IER)
!         ! CALL DGETRF(3,3,JVS(1,1,LU_DIAG(k)),3,IP(1,k),IER)
!         CALL FAC3(JVS(1,1,LU_DIAG(k)),IP(1,k),IER)
!         IF ( IER /= 0 )  RETURN
!      END DO 
!      
!END SUBROUTINE KppDecompBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBig( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse solve subroutine using indirect addressing
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: i, j, k, m, IP3(3), IP(3,NVAR), IER
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR), sum(3)
!
!      DO i=1,NVAR
!        DO j = LU_CROW(i), LU_DIAG(i)-1 
!          !X(1:3,i) = X(1:3,i) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!	       X(k,i) = X(k,i) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO  
!      END DO
!
!      DO i=NVAR,1,-1
!        sum(1:3) = X(1:3,i);
!        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
!          !sum(1:3) = sum(1:3) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!	       sum(k) = sum(k) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO
!        ! X(i) = sum/JVS(LU_DIAG(i));
!        ! CALL DGETRS ('N',3,1,JVS(1:3,1:3,LU_DIAG(i)),3,IP(1,i),sum,3,0) 
!        ! CALL WGESL('N',JVS(1,1,LU_DIAG(i)),3,3,IP(1,i),sum)
!        CALL SOL3('N',JVS(1,1,LU_DIAG(i)),IP(1,i),sum)
!        X(1:3,i) = sum(1:3)
!      END DO
!      
!END SUBROUTINE KppSolveBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBigTR( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Big sparse transpose solve using indirect addressing
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER       :: i, j, k, m, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR)
!
!      DO i=1,NVAR
!        ! X(i) = X(i)/JVS(LU_DIAG(i))
!        CALL SOL3('T',JVS(1,1,LU_DIAG(i)),IP(1,i),X(1,i))
!        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
!	  !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !    - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!	       X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!	END DO
!      END DO
!
!      DO i=NVAR, 1, -1
!        DO j=LU_CROW(i),LU_DIAG(i)-1
!	  !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !   - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!	       X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!	END DO
!      END DO
!      
!END SUBROUTINE KppSolveBigTR
!
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE FAC3(A,IPVT,INFO)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     FAC3 FACTORS THE MATRIX A (3,3) BY
!!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!!     LINPACK - LIKE 
!!
!!     Remove comments to perform pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!
!      REAL(kind=dp) :: A(3,3)
!      INTEGER       :: IPVT(3),INFO
!!      INTEGER       :: L
!!      REAL(kind=dp) :: t, dmax, da, TMP(3)
!      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0
!
!      info = 0
!!      t = TINY(da)
!!      
!!      da = ABS(A(1,1)); L = 1
!!      IF ( ABS(A(2,1))>da ) THEN
!!        da = ABS(A(2,1)); L = 2
!!        IF ( ABS(A(3,1))>da ) THEN
!!          L = 3
!!        END IF  
!!      END IF  
!!      IPVT(1)  = L
!!      IF (L /=1 ) THEN
!!         TMP(1:3) = A(L,1:3)
!!         A(L,1:3) = A(1,1:3)
!!         A(1,1:3) = TMP(1:3)
!!      END IF
!!      IF (ABS(A(1,1)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!
!      A(2,1) = A(2,1)/A(1,1)
!      A(2,2) = A(2,2) - A(2,1)*A(1,2)
!      A(2,3) = A(2,3) - A(2,1)*A(1,3)
!      A(3,1) = A(3,1)/A(1,1)
!      A(3,2) = A(3,2) - A(3,1)*A(1,2)
!      A(3,3) = A(3,3) - A(3,1)*A(1,3)
!      
!!      IPVT(2)  = 2
!!      IF (ABS(A(3,2))>ABS(A(2,2))) THEN
!!         IPVT(2)  = 3
!!         TMP(2:3) = A(3,2:3)
!!         A(3,2:3) = A(2,2:3)
!!         A(2,2:3) = TMP(2:3)
!!      END IF
!!      IF (ABS(A(2,2)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!      
!      A(3,2)   = A(3,2)/A(2,2)
!      A(3,3)   = A(3,3) - A(3,2)*A(2,3)
!      IPVT(3)  = 3
!      
!END SUBROUTINE FAC3
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE SOL3(Trans,A,IPVT,b)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     SOL3 solves the system 3x3
!!     A * x = b  or  trans(a) * x = b
!!     using the factors computed by WGEFA.
!!
!!     Trans      = 'N'   to solve  A*x = b ,
!!                = 'T'   to solve  transpose(A)*x = b
!!     LINPACK - LIKE 
!!
!!     Remove comments to use pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!      CHARACTER     :: Trans
!      REAL(kind=dp) :: a(3,3),b(3)
!      INTEGER       :: IPVT(3)
!!      INTEGER       :: L
!!      REAL(kind=dp) :: TMP
!      
!      SELECT CASE (Trans)
!
!      CASE ('n','N')  !  Solve  A * x = b
!
!!     Solve  L*y = b
!!         L = IPVT(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!         b(2) = b(2)-A(2,1)*b(1)
!         b(3) = b(3)-A(3,1)*b(1)
!         
!!         L = IPVT(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(3) = b(3)-A(3,2)*b(2)
!
!!     Solve  U*x = y
!         b(3) = b(3)/A(3,3)
!         b(2) = (b(2)-A(2,3)*b(3))/A(2,2)
!         b(1) = (b(1)-A(1,3)*b(3)-A(1,2)*b(2))/A(1,1)
!      
!      
!      CASE ('t','T')  !  Solve transpose(A) * x = b
!
!!      Solve transpose(U)*y = b
!         b(1) = b(1)/A(1,1)
!         b(2) = (b(2)-A(1,2)*b(1))/A(2,2)
!         b(3) = (b(3)-A(1,3)*b(1)-A(2,3)*b(2))/A(3,3)
!
!!      Solve transpose(L)*x = y
!         b(2) = b(2)-A(3,2)*b(3)
!!         L = ipvt(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(1) = b(1)-A(3,1)*b(3)-A(2,1)*b(2)
!!         L = ipvt(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!   
!      END SELECT
!
!END SUBROUTINE SOL3

! End of SPARSE_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolve - sparse back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolve ( JVS, X )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)

  X(78) = X(78)-JVS(233)*X(77)
  X(79) = X(79)-JVS(241)*X(67)
  X(92) = X(92)-JVS(301)*X(67)-JVS(302)*X(69)-JVS(303)*X(83)-JVS(304)*X(85)-JVS(305)*X(87)-JVS(306)*X(91)
  X(93) = X(93)-JVS(328)*X(91)
  X(94) = X(94)-JVS(334)*X(87)-JVS(335)*X(88)
  X(96) = X(96)-JVS(364)*X(64)
  X(97) = X(97)-JVS(375)*X(84)
  X(99) = X(99)-JVS(391)*X(60)-JVS(392)*X(63)-JVS(393)*X(66)-JVS(394)*X(69)-JVS(395)*X(83)-JVS(396)*X(84)-JVS(397)*X(97)
  X(100) = X(100)-JVS(420)*X(67)-JVS(421)*X(76)
  X(102) = X(102)-JVS(438)*X(85)
  X(103) = X(103)-JVS(446)*X(85)-JVS(447)*X(87)-JVS(448)*X(90)-JVS(449)*X(102)
  X(105) = X(105)-JVS(471)*X(89)-JVS(472)*X(104)
  X(106) = X(106)-JVS(481)*X(88)
  X(107) = X(107)-JVS(489)*X(81)-JVS(490)*X(93)
  X(108) = X(108)-JVS(500)*X(82)-JVS(501)*X(91)
  X(109) = X(109)-JVS(510)*X(86)-JVS(511)*X(98)
  X(110) = X(110)-JVS(520)*X(64)-JVS(521)*X(67)
  X(111) = X(111)-JVS(531)*X(80)-JVS(532)*X(93)
  X(112) = X(112)-JVS(542)*X(96)-JVS(543)*X(110)
  X(113) = X(113)-JVS(553)*X(87)-JVS(554)*X(105)
  X(114) = X(114)-JVS(563)*X(70)-JVS(564)*X(94)-JVS(565)*X(105)-JVS(566)*X(106)-JVS(567)*X(109)-JVS(568)*X(113)
  X(116) = X(116)-JVS(588)*X(71)-JVS(589)*X(76)-JVS(590)*X(81)-JVS(591)*X(82)-JVS(592)*X(86)-JVS(593)*X(88)-JVS(594)&
             &*X(95)-JVS(595)*X(98)-JVS(596)*X(100)-JVS(597)*X(101)-JVS(598)*X(104)-JVS(599)*X(106)-JVS(600)*X(107)-JVS(601)&
             &*X(108)-JVS(602)*X(109)-JVS(603)*X(110)-JVS(604)*X(111)-JVS(605)*X(113)-JVS(606)*X(115)
  X(117) = X(117)-JVS(622)*X(66)-JVS(623)*X(69)-JVS(624)*X(70)-JVS(625)*X(73)-JVS(626)*X(74)-JVS(627)*X(75)-JVS(628)&
             &*X(80)-JVS(629)*X(85)-JVS(630)*X(88)-JVS(631)*X(89)-JVS(632)*X(90)-JVS(633)*X(91)-JVS(634)*X(93)-JVS(635)&
             &*X(94)-JVS(636)*X(98)-JVS(637)*X(100)-JVS(638)*X(101)-JVS(639)*X(102)-JVS(640)*X(103)-JVS(641)*X(104)-JVS(642)&
             &*X(105)-JVS(643)*X(106)-JVS(644)*X(107)-JVS(645)*X(108)-JVS(646)*X(109)-JVS(647)*X(110)-JVS(648)*X(111)&
             &-JVS(649)*X(112)-JVS(650)*X(113)-JVS(651)*X(114)-JVS(652)*X(115)
  X(118) = X(118)-JVS(673)*X(87)-JVS(674)*X(88)-JVS(675)*X(98)-JVS(676)*X(102)-JVS(677)*X(103)-JVS(678)*X(105)-JVS(679)&
             &*X(106)-JVS(680)*X(109)-JVS(681)*X(111)-JVS(682)*X(112)-JVS(683)*X(113)-JVS(684)*X(115)
  X(119) = X(119)-JVS(702)*X(60)-JVS(703)*X(71)-JVS(704)*X(101)
  X(120) = X(120)-JVS(713)*X(101)-JVS(714)*X(112)-JVS(715)*X(115)-JVS(716)*X(119)
  X(121) = X(121)-JVS(726)*X(73)-JVS(727)*X(90)-JVS(728)*X(105)
  X(122) = X(122)-JVS(741)*X(89)-JVS(742)*X(91)-JVS(743)*X(104)-JVS(744)*X(108)
  X(123) = X(123)-JVS(754)*X(65)-JVS(755)*X(68)-JVS(756)*X(80)-JVS(757)*X(93)-JVS(758)*X(101)-JVS(759)*X(107)-JVS(760)&
             &*X(111)-JVS(761)*X(115)-JVS(762)*X(118)-JVS(763)*X(119)-JVS(764)*X(120)-JVS(765)*X(121)-JVS(766)*X(122)
  X(124) = X(124)-JVS(782)*X(89)-JVS(783)*X(91)-JVS(784)*X(104)
  X(125) = X(125)-JVS(793)*X(89)-JVS(794)*X(91)-JVS(795)*X(104)-JVS(796)*X(108)-JVS(797)*X(124)
  X(126) = X(126)-JVS(806)*X(68)-JVS(807)*X(83)-JVS(808)*X(116)-JVS(809)*X(119)-JVS(810)*X(120)-JVS(811)*X(122)-JVS(812)&
             &*X(124)-JVS(813)*X(125)
  X(127) = X(127)-JVS(825)*X(102)-JVS(826)*X(104)-JVS(827)*X(106)-JVS(828)*X(112)-JVS(829)*X(113)-JVS(830)*X(115)&
             &-JVS(831)*X(119)-JVS(832)*X(120)-JVS(833)*X(122)-JVS(834)*X(124)-JVS(835)*X(125)
  X(128) = X(128)-JVS(845)*X(61)-JVS(846)*X(73)-JVS(847)*X(74)-JVS(848)*X(88)-JVS(849)*X(96)-JVS(850)*X(98)-JVS(851)&
             &*X(100)-JVS(852)*X(101)-JVS(853)*X(102)-JVS(854)*X(103)-JVS(855)*X(104)-JVS(856)*X(105)-JVS(857)*X(106)&
             &-JVS(858)*X(107)-JVS(859)*X(108)-JVS(860)*X(109)-JVS(861)*X(110)-JVS(862)*X(111)-JVS(863)*X(112)-JVS(864)&
             &*X(113)-JVS(865)*X(114)-JVS(866)*X(115)-JVS(867)*X(118)-JVS(868)*X(119)-JVS(869)*X(120)-JVS(870)*X(121)&
             &-JVS(871)*X(122)-JVS(872)*X(123)-JVS(873)*X(124)-JVS(874)*X(125)-JVS(875)*X(126)-JVS(876)*X(127)
  X(129) = X(129)-JVS(887)*X(61)-JVS(888)*X(66)-JVS(889)*X(74)-JVS(890)*X(75)-JVS(891)*X(93)-JVS(892)*X(96)-JVS(893)&
             &*X(98)-JVS(894)*X(100)-JVS(895)*X(101)-JVS(896)*X(102)-JVS(897)*X(104)-JVS(898)*X(106)-JVS(899)*X(107)&
             &-JVS(900)*X(108)-JVS(901)*X(109)-JVS(902)*X(110)-JVS(903)*X(111)-JVS(904)*X(112)-JVS(905)*X(113)-JVS(906)&
             &*X(114)-JVS(907)*X(115)-JVS(908)*X(119)-JVS(909)*X(120)-JVS(910)*X(121)-JVS(911)*X(122)-JVS(912)*X(123)&
             &-JVS(913)*X(124)-JVS(914)*X(125)-JVS(915)*X(126)-JVS(916)*X(127)-JVS(917)*X(128)
  X(130) = X(130)-JVS(927)*X(58)-JVS(928)*X(59)-JVS(929)*X(64)-JVS(930)*X(65)-JVS(931)*X(66)-JVS(932)*X(68)-JVS(933)&
             &*X(69)-JVS(934)*X(70)-JVS(935)*X(71)-JVS(936)*X(72)-JVS(937)*X(75)-JVS(938)*X(76)-JVS(939)*X(80)-JVS(940)&
             &*X(81)-JVS(941)*X(82)-JVS(942)*X(85)-JVS(943)*X(86)-JVS(944)*X(87)-JVS(945)*X(88)-JVS(946)*X(89)-JVS(947)&
             &*X(90)-JVS(948)*X(91)-JVS(949)*X(92)-JVS(950)*X(93)-JVS(951)*X(94)-JVS(952)*X(95)-JVS(953)*X(96)-JVS(954)&
             &*X(97)-JVS(955)*X(98)-JVS(956)*X(100)-JVS(957)*X(101)-JVS(958)*X(102)-JVS(959)*X(103)-JVS(960)*X(104)-JVS(961)&
             &*X(105)-JVS(962)*X(106)-JVS(963)*X(107)-JVS(964)*X(108)-JVS(965)*X(109)-JVS(966)*X(110)-JVS(967)*X(111)&
             &-JVS(968)*X(112)-JVS(969)*X(113)-JVS(970)*X(114)-JVS(971)*X(115)-JVS(972)*X(116)-JVS(973)*X(117)-JVS(974)&
             &*X(118)-JVS(975)*X(119)-JVS(976)*X(120)-JVS(977)*X(121)-JVS(978)*X(122)-JVS(979)*X(123)-JVS(980)*X(124)&
             &-JVS(981)*X(125)-JVS(982)*X(126)-JVS(983)*X(127)-JVS(984)*X(128)-JVS(985)*X(129)
  X(131) = X(131)-JVS(994)*X(62)-JVS(995)*X(63)-JVS(996)*X(97)-JVS(997)*X(98)-JVS(998)*X(100)-JVS(999)*X(101)-JVS(1000)&
             &*X(102)-JVS(1001)*X(104)-JVS(1002)*X(106)-JVS(1003)*X(107)-JVS(1004)*X(108)-JVS(1005)*X(109)-JVS(1006)*X(110)&
             &-JVS(1007)*X(111)-JVS(1008)*X(112)-JVS(1009)*X(113)-JVS(1010)*X(114)-JVS(1011)*X(115)-JVS(1012)*X(119)&
             &-JVS(1013)*X(120)-JVS(1014)*X(121)-JVS(1015)*X(122)-JVS(1016)*X(124)-JVS(1017)*X(125)-JVS(1018)*X(126)&
             &-JVS(1019)*X(127)-JVS(1020)*X(128)-JVS(1021)*X(129)-JVS(1022)*X(130)
  X(132) = X(132)-JVS(1030)*X(56)-JVS(1031)*X(57)-JVS(1032)*X(61)-JVS(1033)*X(62)-JVS(1034)*X(63)-JVS(1035)*X(72)&
             &-JVS(1036)*X(81)-JVS(1037)*X(82)-JVS(1038)*X(84)-JVS(1039)*X(86)-JVS(1040)*X(90)-JVS(1041)*X(97)-JVS(1042)&
             &*X(98)-JVS(1043)*X(99)-JVS(1044)*X(100)-JVS(1045)*X(101)-JVS(1046)*X(102)-JVS(1047)*X(104)-JVS(1048)*X(106)&
             &-JVS(1049)*X(107)-JVS(1050)*X(108)-JVS(1051)*X(109)-JVS(1052)*X(110)-JVS(1053)*X(111)-JVS(1054)*X(112)&
             &-JVS(1055)*X(113)-JVS(1056)*X(114)-JVS(1057)*X(115)-JVS(1058)*X(116)-JVS(1059)*X(117)-JVS(1060)*X(118)&
             &-JVS(1061)*X(119)-JVS(1062)*X(120)-JVS(1063)*X(121)-JVS(1064)*X(122)-JVS(1065)*X(123)-JVS(1066)*X(124)&
             &-JVS(1067)*X(125)-JVS(1068)*X(126)-JVS(1069)*X(127)-JVS(1070)*X(128)-JVS(1071)*X(129)-JVS(1072)*X(130)&
             &-JVS(1073)*X(131)
  X(133) = X(133)-JVS(1080)*X(60)-JVS(1081)*X(61)-JVS(1082)*X(63)-JVS(1083)*X(66)-JVS(1084)*X(69)-JVS(1085)*X(72)&
             &-JVS(1086)*X(83)-JVS(1087)*X(84)-JVS(1088)*X(91)-JVS(1089)*X(93)-JVS(1090)*X(97)-JVS(1091)*X(99)-JVS(1092)&
             &*X(102)-JVS(1093)*X(104)-JVS(1094)*X(106)-JVS(1095)*X(108)-JVS(1096)*X(109)-JVS(1097)*X(113)-JVS(1098)*X(116)&
             &-JVS(1099)*X(117)-JVS(1100)*X(118)-JVS(1101)*X(119)-JVS(1102)*X(120)-JVS(1103)*X(121)-JVS(1104)*X(122)&
             &-JVS(1105)*X(123)-JVS(1106)*X(124)-JVS(1107)*X(125)-JVS(1108)*X(126)-JVS(1109)*X(127)-JVS(1110)*X(128)&
             &-JVS(1111)*X(129)-JVS(1112)*X(130)-JVS(1113)*X(131)-JVS(1114)*X(132)
  X(134) = X(134)-JVS(1120)*X(63)-JVS(1121)*X(90)-JVS(1122)*X(91)-JVS(1123)*X(93)-JVS(1124)*X(97)-JVS(1125)*X(105)&
             &-JVS(1126)*X(114)-JVS(1127)*X(121)-JVS(1128)*X(122)-JVS(1129)*X(124)-JVS(1130)*X(125)-JVS(1131)*X(128)&
             &-JVS(1132)*X(129)-JVS(1133)*X(130)-JVS(1134)*X(131)-JVS(1135)*X(132)-JVS(1136)*X(133)
  X(135) = X(135)-JVS(1141)*X(53)-JVS(1142)*X(54)-JVS(1143)*X(55)-JVS(1144)*X(58)-JVS(1145)*X(59)-JVS(1146)*X(60)&
             &-JVS(1147)*X(62)-JVS(1148)*X(64)-JVS(1149)*X(65)-JVS(1150)*X(66)-JVS(1151)*X(67)-JVS(1152)*X(68)-JVS(1153)&
             &*X(69)-JVS(1154)*X(70)-JVS(1155)*X(71)-JVS(1156)*X(72)-JVS(1157)*X(73)-JVS(1158)*X(74)-JVS(1159)*X(75)&
             &-JVS(1160)*X(76)-JVS(1161)*X(77)-JVS(1162)*X(78)-JVS(1163)*X(79)-JVS(1164)*X(80)-JVS(1165)*X(81)-JVS(1166)&
             &*X(82)-JVS(1167)*X(83)-JVS(1168)*X(84)-JVS(1169)*X(85)-JVS(1170)*X(86)-JVS(1171)*X(87)-JVS(1172)*X(88)&
             &-JVS(1173)*X(89)-JVS(1174)*X(90)-JVS(1175)*X(91)-JVS(1176)*X(92)-JVS(1177)*X(93)-JVS(1178)*X(94)-JVS(1179)&
             &*X(95)-JVS(1180)*X(96)-JVS(1181)*X(97)-JVS(1182)*X(98)-JVS(1183)*X(99)-JVS(1184)*X(100)-JVS(1185)*X(101)&
             &-JVS(1186)*X(102)-JVS(1187)*X(103)-JVS(1188)*X(104)-JVS(1189)*X(105)-JVS(1190)*X(106)-JVS(1191)*X(107)&
             &-JVS(1192)*X(108)-JVS(1193)*X(109)-JVS(1194)*X(110)-JVS(1195)*X(111)-JVS(1196)*X(112)-JVS(1197)*X(113)&
             &-JVS(1198)*X(114)-JVS(1199)*X(115)-JVS(1200)*X(116)-JVS(1201)*X(117)-JVS(1202)*X(118)-JVS(1203)*X(119)&
             &-JVS(1204)*X(120)-JVS(1205)*X(121)-JVS(1206)*X(122)-JVS(1207)*X(123)-JVS(1208)*X(124)-JVS(1209)*X(125)&
             &-JVS(1210)*X(126)-JVS(1211)*X(127)-JVS(1212)*X(128)-JVS(1213)*X(129)-JVS(1214)*X(130)-JVS(1215)*X(131)&
             &-JVS(1216)*X(132)-JVS(1217)*X(133)-JVS(1218)*X(134)
  X(136) = X(136)-JVS(1222)*X(57)-JVS(1223)*X(65)-JVS(1224)*X(116)-JVS(1225)*X(119)-JVS(1226)*X(120)-JVS(1227)*X(122)&
             &-JVS(1228)*X(124)-JVS(1229)*X(125)-JVS(1230)*X(127)-JVS(1231)*X(128)-JVS(1232)*X(129)-JVS(1233)*X(130)&
             &-JVS(1234)*X(131)-JVS(1235)*X(132)-JVS(1236)*X(133)-JVS(1237)*X(134)-JVS(1238)*X(135)
  X(137) = X(137)-JVS(1241)*X(53)-JVS(1242)*X(54)-JVS(1243)*X(55)-JVS(1244)*X(56)-JVS(1245)*X(67)-JVS(1246)*X(77)&
             &-JVS(1247)*X(78)-JVS(1248)*X(79)-JVS(1249)*X(83)-JVS(1250)*X(84)-JVS(1251)*X(96)-JVS(1252)*X(97)-JVS(1253)&
             &*X(110)-JVS(1254)*X(117)-JVS(1255)*X(119)-JVS(1256)*X(120)-JVS(1257)*X(121)-JVS(1258)*X(122)-JVS(1259)*X(123)&
             &-JVS(1260)*X(124)-JVS(1261)*X(125)-JVS(1262)*X(126)-JVS(1263)*X(127)-JVS(1264)*X(128)-JVS(1265)*X(129)&
             &-JVS(1266)*X(130)-JVS(1267)*X(131)-JVS(1268)*X(132)-JVS(1269)*X(133)-JVS(1270)*X(134)-JVS(1271)*X(135)&
             &-JVS(1272)*X(136)
  X(137) = X(137)/JVS(1273)
  X(136) = (X(136)-JVS(1240)*X(137))/(JVS(1239))
  X(135) = (X(135)-JVS(1220)*X(136)-JVS(1221)*X(137))/(JVS(1219))
  X(134) = (X(134)-JVS(1138)*X(135)-JVS(1139)*X(136)-JVS(1140)*X(137))/(JVS(1137))
  X(133) = (X(133)-JVS(1116)*X(134)-JVS(1117)*X(135)-JVS(1118)*X(136)-JVS(1119)*X(137))/(JVS(1115))
  X(132) = (X(132)-JVS(1075)*X(133)-JVS(1076)*X(134)-JVS(1077)*X(135)-JVS(1078)*X(136)-JVS(1079)*X(137))/(JVS(1074))
  X(131) = (X(131)-JVS(1024)*X(132)-JVS(1025)*X(133)-JVS(1026)*X(134)-JVS(1027)*X(135)-JVS(1028)*X(136)-JVS(1029)&
             &*X(137))/(JVS(1023))
  X(130) = (X(130)-JVS(987)*X(131)-JVS(988)*X(132)-JVS(989)*X(133)-JVS(990)*X(134)-JVS(991)*X(135)-JVS(992)*X(136)&
             &-JVS(993)*X(137))/(JVS(986))
  X(129) = (X(129)-JVS(919)*X(130)-JVS(920)*X(131)-JVS(921)*X(132)-JVS(922)*X(133)-JVS(923)*X(134)-JVS(924)*X(135)&
             &-JVS(925)*X(136)-JVS(926)*X(137))/(JVS(918))
  X(128) = (X(128)-JVS(878)*X(129)-JVS(879)*X(130)-JVS(880)*X(131)-JVS(881)*X(132)-JVS(882)*X(133)-JVS(883)*X(134)&
             &-JVS(884)*X(135)-JVS(885)*X(136)-JVS(886)*X(137))/(JVS(877))
  X(127) = (X(127)-JVS(837)*X(128)-JVS(838)*X(129)-JVS(839)*X(130)-JVS(840)*X(131)-JVS(841)*X(133)-JVS(842)*X(134)&
             &-JVS(843)*X(135)-JVS(844)*X(137))/(JVS(836))
  X(126) = (X(126)-JVS(815)*X(127)-JVS(816)*X(128)-JVS(817)*X(129)-JVS(818)*X(130)-JVS(819)*X(131)-JVS(820)*X(133)&
             &-JVS(821)*X(134)-JVS(822)*X(135)-JVS(823)*X(136)-JVS(824)*X(137))/(JVS(814))
  X(125) = (X(125)-JVS(799)*X(128)-JVS(800)*X(129)-JVS(801)*X(130)-JVS(802)*X(131)-JVS(803)*X(133)-JVS(804)*X(134)&
             &-JVS(805)*X(135))/(JVS(798))
  X(124) = (X(124)-JVS(786)*X(128)-JVS(787)*X(129)-JVS(788)*X(130)-JVS(789)*X(131)-JVS(790)*X(133)-JVS(791)*X(134)&
             &-JVS(792)*X(135))/(JVS(785))
  X(123) = (X(123)-JVS(768)*X(124)-JVS(769)*X(125)-JVS(770)*X(126)-JVS(771)*X(127)-JVS(772)*X(128)-JVS(773)*X(129)&
             &-JVS(774)*X(130)-JVS(775)*X(131)-JVS(776)*X(132)-JVS(777)*X(133)-JVS(778)*X(134)-JVS(779)*X(135)-JVS(780)&
             &*X(136)-JVS(781)*X(137))/(JVS(767))
  X(122) = (X(122)-JVS(746)*X(124)-JVS(747)*X(128)-JVS(748)*X(129)-JVS(749)*X(130)-JVS(750)*X(131)-JVS(751)*X(133)&
             &-JVS(752)*X(134)-JVS(753)*X(135))/(JVS(745))
  X(121) = (X(121)-JVS(730)*X(122)-JVS(731)*X(124)-JVS(732)*X(125)-JVS(733)*X(128)-JVS(734)*X(129)-JVS(735)*X(130)&
             &-JVS(736)*X(131)-JVS(737)*X(132)-JVS(738)*X(133)-JVS(739)*X(134)-JVS(740)*X(135))/(JVS(729))
  X(120) = (X(120)-JVS(718)*X(127)-JVS(719)*X(128)-JVS(720)*X(129)-JVS(721)*X(130)-JVS(722)*X(131)-JVS(723)*X(133)&
             &-JVS(724)*X(135)-JVS(725)*X(137))/(JVS(717))
  X(119) = (X(119)-JVS(706)*X(120)-JVS(707)*X(128)-JVS(708)*X(129)-JVS(709)*X(130)-JVS(710)*X(131)-JVS(711)*X(133)&
             &-JVS(712)*X(135))/(JVS(705))
  X(118) = (X(118)-JVS(686)*X(119)-JVS(687)*X(120)-JVS(688)*X(121)-JVS(689)*X(122)-JVS(690)*X(124)-JVS(691)*X(125)&
             &-JVS(692)*X(127)-JVS(693)*X(128)-JVS(694)*X(129)-JVS(695)*X(130)-JVS(696)*X(131)-JVS(697)*X(132)-JVS(698)&
             &*X(133)-JVS(699)*X(134)-JVS(700)*X(135)-JVS(701)*X(137))/(JVS(685))
  X(117) = (X(117)-JVS(654)*X(119)-JVS(655)*X(120)-JVS(656)*X(121)-JVS(657)*X(122)-JVS(658)*X(123)-JVS(659)*X(124)&
             &-JVS(660)*X(125)-JVS(661)*X(126)-JVS(662)*X(127)-JVS(663)*X(128)-JVS(664)*X(129)-JVS(665)*X(130)-JVS(666)&
             &*X(131)-JVS(667)*X(132)-JVS(668)*X(133)-JVS(669)*X(134)-JVS(670)*X(135)-JVS(671)*X(136)-JVS(672)*X(137))&
             &/(JVS(653))
  X(116) = (X(116)-JVS(608)*X(119)-JVS(609)*X(120)-JVS(610)*X(122)-JVS(611)*X(124)-JVS(612)*X(125)-JVS(613)*X(127)&
             &-JVS(614)*X(128)-JVS(615)*X(129)-JVS(616)*X(130)-JVS(617)*X(131)-JVS(618)*X(133)-JVS(619)*X(134)-JVS(620)&
             &*X(135)-JVS(621)*X(137))/(JVS(607))
  X(115) = (X(115)-JVS(581)*X(127)-JVS(582)*X(128)-JVS(583)*X(129)-JVS(584)*X(130)-JVS(585)*X(131)-JVS(586)*X(133)&
             &-JVS(587)*X(135))/(JVS(580))
  X(114) = (X(114)-JVS(570)*X(122)-JVS(571)*X(124)-JVS(572)*X(125)-JVS(573)*X(128)-JVS(574)*X(129)-JVS(575)*X(130)&
             &-JVS(576)*X(131)-JVS(577)*X(133)-JVS(578)*X(134)-JVS(579)*X(135))/(JVS(569))
  X(113) = (X(113)-JVS(556)*X(124)-JVS(557)*X(128)-JVS(558)*X(129)-JVS(559)*X(130)-JVS(560)*X(131)-JVS(561)*X(134)&
             &-JVS(562)*X(135))/(JVS(555))
  X(112) = (X(112)-JVS(545)*X(119)-JVS(546)*X(120)-JVS(547)*X(128)-JVS(548)*X(129)-JVS(549)*X(130)-JVS(550)*X(131)&
             &-JVS(551)*X(135)-JVS(552)*X(137))/(JVS(544))
  X(111) = (X(111)-JVS(534)*X(125)-JVS(535)*X(128)-JVS(536)*X(129)-JVS(537)*X(130)-JVS(538)*X(131)-JVS(539)*X(133)&
             &-JVS(540)*X(134)-JVS(541)*X(135))/(JVS(533))
  X(110) = (X(110)-JVS(523)*X(119)-JVS(524)*X(120)-JVS(525)*X(128)-JVS(526)*X(129)-JVS(527)*X(130)-JVS(528)*X(131)&
             &-JVS(529)*X(135)-JVS(530)*X(137))/(JVS(522))
  X(109) = (X(109)-JVS(513)*X(122)-JVS(514)*X(128)-JVS(515)*X(129)-JVS(516)*X(130)-JVS(517)*X(131)-JVS(518)*X(133)&
             &-JVS(519)*X(135))/(JVS(512))
  X(108) = (X(108)-JVS(503)*X(128)-JVS(504)*X(129)-JVS(505)*X(130)-JVS(506)*X(131)-JVS(507)*X(133)-JVS(508)*X(134)&
             &-JVS(509)*X(135))/(JVS(502))
  X(107) = (X(107)-JVS(492)*X(125)-JVS(493)*X(128)-JVS(494)*X(129)-JVS(495)*X(130)-JVS(496)*X(131)-JVS(497)*X(133)&
             &-JVS(498)*X(134)-JVS(499)*X(135))/(JVS(491))
  X(106) = (X(106)-JVS(483)*X(125)-JVS(484)*X(128)-JVS(485)*X(129)-JVS(486)*X(130)-JVS(487)*X(131)-JVS(488)*X(135))&
             &/(JVS(482))
  X(105) = (X(105)-JVS(474)*X(124)-JVS(475)*X(128)-JVS(476)*X(129)-JVS(477)*X(130)-JVS(478)*X(131)-JVS(479)*X(134)&
             &-JVS(480)*X(135))/(JVS(473))
  X(104) = (X(104)-JVS(466)*X(124)-JVS(467)*X(128)-JVS(468)*X(129)-JVS(469)*X(130)-JVS(470)*X(131))/(JVS(465))
  X(103) = (X(103)-JVS(451)*X(105)-JVS(452)*X(109)-JVS(453)*X(111)-JVS(454)*X(112)-JVS(455)*X(113)-JVS(456)*X(121)&
             &-JVS(457)*X(122)-JVS(458)*X(128)-JVS(459)*X(129)-JVS(460)*X(130)-JVS(461)*X(131)-JVS(462)*X(132)-JVS(463)&
             &*X(134)-JVS(464)*X(135))/(JVS(450))
  X(102) = (X(102)-JVS(440)*X(122)-JVS(441)*X(128)-JVS(442)*X(129)-JVS(443)*X(130)-JVS(444)*X(131)-JVS(445)*X(135))&
             &/(JVS(439))
  X(101) = (X(101)-JVS(432)*X(120)-JVS(433)*X(128)-JVS(434)*X(129)-JVS(435)*X(130)-JVS(436)*X(131)-JVS(437)*X(135))&
             &/(JVS(431))
  X(100) = (X(100)-JVS(423)*X(119)-JVS(424)*X(120)-JVS(425)*X(128)-JVS(426)*X(129)-JVS(427)*X(130)-JVS(428)*X(131)&
             &-JVS(429)*X(135)-JVS(430)*X(137))/(JVS(422))
  X(99) = (X(99)-JVS(399)*X(102)-JVS(400)*X(104)-JVS(401)*X(106)-JVS(402)*X(108)-JVS(403)*X(109)-JVS(404)*X(113)&
            &-JVS(405)*X(116)-JVS(406)*X(117)-JVS(407)*X(118)-JVS(408)*X(122)-JVS(409)*X(123)-JVS(410)*X(127)-JVS(411)&
            &*X(128)-JVS(412)*X(129)-JVS(413)*X(130)-JVS(414)*X(131)-JVS(415)*X(132)-JVS(416)*X(133)-JVS(417)*X(134)&
            &-JVS(418)*X(135)-JVS(419)*X(137))/(JVS(398))
  X(98) = (X(98)-JVS(385)*X(122)-JVS(386)*X(128)-JVS(387)*X(129)-JVS(388)*X(130)-JVS(389)*X(131)-JVS(390)*X(133))&
            &/(JVS(384))
  X(97) = (X(97)-JVS(377)*X(130)-JVS(378)*X(131)-JVS(379)*X(132)-JVS(380)*X(133)-JVS(381)*X(134)-JVS(382)*X(135)&
            &-JVS(383)*X(137))/(JVS(376))
  X(96) = (X(96)-JVS(366)*X(110)-JVS(367)*X(119)-JVS(368)*X(120)-JVS(369)*X(128)-JVS(370)*X(129)-JVS(371)*X(130)&
            &-JVS(372)*X(131)-JVS(373)*X(135)-JVS(374)*X(137))/(JVS(365))
  X(95) = (X(95)-JVS(348)*X(98)-JVS(349)*X(100)-JVS(350)*X(101)-JVS(351)*X(104)-JVS(352)*X(106)-JVS(353)*X(107)-JVS(354)&
            &*X(108)-JVS(355)*X(109)-JVS(356)*X(110)-JVS(357)*X(111)-JVS(358)*X(113)-JVS(359)*X(115)-JVS(360)*X(119)&
            &-JVS(361)*X(124)-JVS(362)*X(129)-JVS(363)*X(135))/(JVS(347))
  X(94) = (X(94)-JVS(337)*X(105)-JVS(338)*X(106)-JVS(339)*X(109)-JVS(340)*X(113)-JVS(341)*X(128)-JVS(342)*X(129)&
            &-JVS(343)*X(130)-JVS(344)*X(131)-JVS(345)*X(134)-JVS(346)*X(135))/(JVS(336))
  X(93) = (X(93)-JVS(330)*X(125)-JVS(331)*X(133)-JVS(332)*X(134)-JVS(333)*X(135))/(JVS(329))
  X(92) = (X(92)-JVS(308)*X(93)-JVS(309)*X(94)-JVS(310)*X(96)-JVS(311)*X(102)-JVS(312)*X(105)-JVS(313)*X(113)-JVS(314)&
            &*X(116)-JVS(315)*X(117)-JVS(316)*X(118)-JVS(317)*X(122)-JVS(318)*X(123)-JVS(319)*X(125)-JVS(320)*X(128)&
            &-JVS(321)*X(129)-JVS(322)*X(130)-JVS(323)*X(131)-JVS(324)*X(133)-JVS(325)*X(134)-JVS(326)*X(135)-JVS(327)&
            &*X(137))/(JVS(307))
  X(91) = (X(91)-JVS(298)*X(133)-JVS(299)*X(134)-JVS(300)*X(135))/(JVS(297))
  X(90) = (X(90)-JVS(293)*X(121)-JVS(294)*X(132)-JVS(295)*X(134)-JVS(296)*X(135))/(JVS(292))
  X(89) = (X(89)-JVS(288)*X(104)-JVS(289)*X(124)-JVS(290)*X(130)-JVS(291)*X(135))/(JVS(287))
  X(88) = (X(88)-JVS(284)*X(106)-JVS(285)*X(130)-JVS(286)*X(135))/(JVS(283))
  X(87) = (X(87)-JVS(280)*X(113)-JVS(281)*X(130)-JVS(282)*X(135))/(JVS(279))
  X(86) = (X(86)-JVS(275)*X(98)-JVS(276)*X(109)-JVS(277)*X(130)-JVS(278)*X(135))/(JVS(274))
  X(85) = (X(85)-JVS(271)*X(102)-JVS(272)*X(130)-JVS(273)*X(135))/(JVS(270))
  X(84) = (X(84)-JVS(267)*X(97)-JVS(268)*X(132)-JVS(269)*X(137))/(JVS(266))
  X(83) = (X(83)-JVS(263)*X(133)-JVS(264)*X(135)-JVS(265)*X(137))/(JVS(262))
  X(82) = (X(82)-JVS(259)*X(108)-JVS(260)*X(130)-JVS(261)*X(135))/(JVS(258))
  X(81) = (X(81)-JVS(255)*X(107)-JVS(256)*X(130)-JVS(257)*X(135))/(JVS(254))
  X(80) = (X(80)-JVS(251)*X(111)-JVS(252)*X(130)-JVS(253)*X(135))/(JVS(250))
  X(79) = (X(79)-JVS(243)*X(83)-JVS(244)*X(96)-JVS(245)*X(117)-JVS(246)*X(123)-JVS(247)*X(130)-JVS(248)*X(135)-JVS(249)&
            &*X(137))/(JVS(242))
  X(78) = (X(78)-JVS(235)*X(79)-JVS(236)*X(84)-JVS(237)*X(97)-JVS(238)*X(130)-JVS(239)*X(135)-JVS(240)*X(137))&
            &/(JVS(234))
  X(77) = (X(77)-JVS(228)*X(78)-JVS(229)*X(84)-JVS(230)*X(97)-JVS(231)*X(130)-JVS(232)*X(135))/(JVS(227))
  X(76) = (X(76)-JVS(224)*X(100)-JVS(225)*X(130)-JVS(226)*X(135))/(JVS(223))
  X(75) = (X(75)-JVS(220)*X(129)-JVS(221)*X(130)-JVS(222)*X(135))/(JVS(219))
  X(74) = (X(74)-JVS(216)*X(128)-JVS(217)*X(130)-JVS(218)*X(135))/(JVS(215))
  X(73) = (X(73)-JVS(212)*X(121)-JVS(213)*X(130)-JVS(214)*X(135))/(JVS(211))
  X(72) = (X(72)-JVS(208)*X(130)-JVS(209)*X(132)-JVS(210)*X(135))/(JVS(207))
  X(71) = (X(71)-JVS(204)*X(119)-JVS(205)*X(130)-JVS(206)*X(135))/(JVS(203))
  X(70) = (X(70)-JVS(200)*X(114)-JVS(201)*X(130)-JVS(202)*X(135))/(JVS(199))
  X(69) = (X(69)-JVS(197)*X(133)-JVS(198)*X(135))/(JVS(196))
  X(68) = (X(68)-JVS(193)*X(126)-JVS(194)*X(130)-JVS(195)*X(135))/(JVS(192))
  X(67) = (X(67)-JVS(190)*X(135)-JVS(191)*X(137))/(JVS(189))
  X(66) = (X(66)-JVS(187)*X(133)-JVS(188)*X(135))/(JVS(186))
  X(65) = (X(65)-JVS(183)*X(130)-JVS(184)*X(135)-JVS(185)*X(136))/(JVS(182))
  X(64) = (X(64)-JVS(179)*X(110)-JVS(180)*X(130)-JVS(181)*X(135))/(JVS(178))
  X(63) = (X(63)-JVS(176)*X(132)-JVS(177)*X(133))/(JVS(175))
  X(62) = (X(62)-JVS(172)*X(131)-JVS(173)*X(132)-JVS(174)*X(135))/(JVS(171))
  X(61) = (X(61)-JVS(169)*X(128)-JVS(170)*X(132))/(JVS(168))
  X(60) = (X(60)-JVS(166)*X(133)-JVS(167)*X(135))/(JVS(165))
  X(59) = (X(59)-JVS(162)*X(66)-JVS(163)*X(133)-JVS(164)*X(135))/(JVS(161))
  X(58) = (X(58)-JVS(159)*X(130)-JVS(160)*X(135))/(JVS(158))
  X(57) = (X(57)-JVS(156)*X(132)-JVS(157)*X(136))/(JVS(155))
  X(56) = (X(56)-JVS(153)*X(132)-JVS(154)*X(137))/(JVS(152))
  X(55) = (X(55)-JVS(151)*X(135))/(JVS(150))
  X(54) = (X(54)-JVS(149)*X(135))/(JVS(148))
  X(53) = (X(53)-JVS(147)*X(135))/(JVS(146))
  X(52) = (X(52)-JVS(144)*X(91)-JVS(145)*X(135))/(JVS(143))
  X(51) = (X(51)-JVS(127)*X(57)-JVS(128)*X(58)-JVS(129)*X(61)-JVS(130)*X(63)-JVS(131)*X(77)-JVS(132)*X(78)-JVS(133)&
            &*X(79)-JVS(134)*X(84)-JVS(135)*X(90)-JVS(136)*X(99)-JVS(137)*X(117)-JVS(138)*X(120)-JVS(139)*X(122)-JVS(140)&
            &*X(125)-JVS(141)*X(132)-JVS(142)*X(134))/(JVS(126))
  X(50) = (X(50)-JVS(119)*X(91)-JVS(120)*X(92)-JVS(121)*X(122)-JVS(122)*X(128)-JVS(123)*X(131)-JVS(124)*X(134)-JVS(125)&
            &*X(135))/(JVS(118))
  X(49) = (X(49)-JVS(117)*X(79))/(JVS(116))
  X(48) = (X(48)-JVS(115)*X(77))/(JVS(114))
  X(47) = (X(47)-JVS(113)*X(53))/(JVS(112))
  X(46) = (X(46)-JVS(111)*X(56))/(JVS(110))
  X(45) = (X(45)-JVS(109)*X(84))/(JVS(108))
  X(44) = (X(44)-JVS(107)*X(84))/(JVS(106))
  X(43) = (X(43)-JVS(105)*X(77))/(JVS(104))
  X(42) = (X(42)-JVS(103)*X(97))/(JVS(102))
  X(41) = (X(41)-JVS(101)*X(78))/(JVS(100))
  X(40) = (X(40)-JVS(99)*X(79))/(JVS(98))
  X(39) = (X(39)-JVS(97)*X(77))/(JVS(96))
  X(38) = (X(38)-JVS(95)*X(84))/(JVS(94))
  X(37) = (X(37)-JVS(92)*X(53)-JVS(93)*X(135))/(JVS(91))
  X(36) = (X(36)-JVS(89)*X(97)-JVS(90)*X(132))/(JVS(88))
  X(35) = (X(35)-JVS(86)*X(132)-JVS(87)*X(137))/(JVS(85))
  X(34) = (X(34)-JVS(83)*X(67)-JVS(84)*X(137))/(JVS(82))
  X(33) = (X(33)-JVS(80)*X(83)-JVS(81)*X(137))/(JVS(79))
  X(32) = (X(32)-JVS(77)*X(96)-JVS(78)*X(137))/(JVS(76))
  X(31) = (X(31)-JVS(74)*X(123)-JVS(75)*X(137))/(JVS(73))
  X(30) = (X(30)-JVS(71)*X(117)-JVS(72)*X(137))/(JVS(70))
  X(29) = (X(29)-JVS(68)*X(133)-JVS(69)*X(137))/(JVS(67))
  X(28) = (X(28)-JVS(65)*X(97)-JVS(66)*X(135))/(JVS(64))
  X(27) = (X(27)-JVS(62)*X(78)-JVS(63)*X(135))/(JVS(61))
  X(26) = (X(26)-JVS(59)*X(84)-JVS(60)*X(137))/(JVS(58))
  X(25) = (X(25)-JVS(56)*X(97)-JVS(57)*X(131))/(JVS(55))
  X(24) = (X(24)-JVS(54)*X(97))/(JVS(53))
  X(23) = (X(23)-JVS(52)*X(97))/(JVS(51))
  X(22) = (X(22)-JVS(49)*X(79)-JVS(50)*X(135))/(JVS(48))
  X(21) = (X(21)-JVS(46)*X(130)-JVS(47)*X(137))/(JVS(45))
  X(20) = (X(20)-JVS(43)*X(97)-JVS(44)*X(130))/(JVS(42))
  X(19) = (X(19)-JVS(40)*X(134)-JVS(41)*X(137))/(JVS(39))
  X(18) = (X(18)-JVS(38)*X(78))/(JVS(37))
  X(17) = (X(17)-JVS(36)*X(84))/(JVS(35))
  X(16) = (X(16)-JVS(34)*X(79))/(JVS(33))
  X(15) = (X(15)-JVS(32)*X(77))/(JVS(31))
  X(14) = (X(14)-JVS(29)*X(66)-JVS(30)*X(135))/(JVS(28))
  X(13) = (X(13)-JVS(26)*X(59)-JVS(27)*X(135))/(JVS(25))
  X(12) = (X(12)-JVS(24)*X(122))/(JVS(23))
  X(11) = (X(11)-JVS(22)*X(125))/(JVS(21))
  X(10) = (X(10)-JVS(20)*X(120))/(JVS(19))
  X(9) = (X(9)-JVS(18)*X(57))/(JVS(17))
  X(8) = (X(8)-JVS(16)*X(90))/(JVS(15))
  X(7) = (X(7)-JVS(14)*X(61))/(JVS(13))
  X(6) = (X(6)-JVS(12)*X(134))/(JVS(11))
  X(5) = (X(5)-JVS(10)*X(132))/(JVS(9))
  X(4) = (X(4)-JVS(8)*X(63))/(JVS(7))
  X(3) = (X(3)-JVS(6)*X(99))/(JVS(5))
  X(2) = (X(2)-JVS(4)*X(58))/(JVS(3))
  X(1) = (X(1)-JVS(2)*X(117))/(JVS(1))
      
END SUBROUTINE KppSolve

! End of KppSolve function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolveTR - sparse, transposed back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
!      XX        - Vector for output variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolveTR ( JVS, X, XX )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)
! XX - Vector for output variables
  REAL(kind=dp) :: XX(NVAR)

  XX(1) = X(1)/JVS(1)
  XX(2) = X(2)/JVS(3)
  XX(3) = X(3)/JVS(5)
  XX(4) = X(4)/JVS(7)
  XX(5) = X(5)/JVS(9)
  XX(6) = X(6)/JVS(11)
  XX(7) = X(7)/JVS(13)
  XX(8) = X(8)/JVS(15)
  XX(9) = X(9)/JVS(17)
  XX(10) = X(10)/JVS(19)
  XX(11) = X(11)/JVS(21)
  XX(12) = X(12)/JVS(23)
  XX(13) = X(13)/JVS(25)
  XX(14) = X(14)/JVS(28)
  XX(15) = X(15)/JVS(31)
  XX(16) = X(16)/JVS(33)
  XX(17) = X(17)/JVS(35)
  XX(18) = X(18)/JVS(37)
  XX(19) = X(19)/JVS(39)
  XX(20) = X(20)/JVS(42)
  XX(21) = X(21)/JVS(45)
  XX(22) = X(22)/JVS(48)
  XX(23) = X(23)/JVS(51)
  XX(24) = X(24)/JVS(53)
  XX(25) = X(25)/JVS(55)
  XX(26) = X(26)/JVS(58)
  XX(27) = X(27)/JVS(61)
  XX(28) = X(28)/JVS(64)
  XX(29) = X(29)/JVS(67)
  XX(30) = X(30)/JVS(70)
  XX(31) = X(31)/JVS(73)
  XX(32) = X(32)/JVS(76)
  XX(33) = X(33)/JVS(79)
  XX(34) = X(34)/JVS(82)
  XX(35) = X(35)/JVS(85)
  XX(36) = X(36)/JVS(88)
  XX(37) = X(37)/JVS(91)
  XX(38) = X(38)/JVS(94)
  XX(39) = X(39)/JVS(96)
  XX(40) = X(40)/JVS(98)
  XX(41) = X(41)/JVS(100)
  XX(42) = X(42)/JVS(102)
  XX(43) = X(43)/JVS(104)
  XX(44) = X(44)/JVS(106)
  XX(45) = X(45)/JVS(108)
  XX(46) = X(46)/JVS(110)
  XX(47) = X(47)/JVS(112)
  XX(48) = X(48)/JVS(114)
  XX(49) = X(49)/JVS(116)
  XX(50) = X(50)/JVS(118)
  XX(51) = X(51)/JVS(126)
  XX(52) = X(52)/JVS(143)
  XX(53) = (X(53)-JVS(92)*XX(37)-JVS(113)*XX(47))/(JVS(146))
  XX(54) = X(54)/JVS(148)
  XX(55) = X(55)/JVS(150)
  XX(56) = (X(56)-JVS(111)*XX(46))/(JVS(152))
  XX(57) = (X(57)-JVS(18)*XX(9)-JVS(127)*XX(51))/(JVS(155))
  XX(58) = (X(58)-JVS(4)*XX(2)-JVS(128)*XX(51))/(JVS(158))
  XX(59) = (X(59)-JVS(26)*XX(13))/(JVS(161))
  XX(60) = X(60)/JVS(165)
  XX(61) = (X(61)-JVS(14)*XX(7)-JVS(129)*XX(51))/(JVS(168))
  XX(62) = X(62)/JVS(171)
  XX(63) = (X(63)-JVS(8)*XX(4)-JVS(130)*XX(51))/(JVS(175))
  XX(64) = X(64)/JVS(178)
  XX(65) = X(65)/JVS(182)
  XX(66) = (X(66)-JVS(29)*XX(14)-JVS(162)*XX(59))/(JVS(186))
  XX(67) = (X(67)-JVS(83)*XX(34))/(JVS(189))
  XX(68) = X(68)/JVS(192)
  XX(69) = X(69)/JVS(196)
  XX(70) = X(70)/JVS(199)
  XX(71) = X(71)/JVS(203)
  XX(72) = X(72)/JVS(207)
  XX(73) = X(73)/JVS(211)
  XX(74) = X(74)/JVS(215)
  XX(75) = X(75)/JVS(219)
  XX(76) = X(76)/JVS(223)
  XX(77) = (X(77)-JVS(32)*XX(15)-JVS(97)*XX(39)-JVS(105)*XX(43)-JVS(115)*XX(48)-JVS(131)*XX(51))/(JVS(227))
  XX(78) = (X(78)-JVS(38)*XX(18)-JVS(62)*XX(27)-JVS(101)*XX(41)-JVS(132)*XX(51)-JVS(228)*XX(77))/(JVS(234))
  XX(79) = (X(79)-JVS(34)*XX(16)-JVS(49)*XX(22)-JVS(99)*XX(40)-JVS(117)*XX(49)-JVS(133)*XX(51)-JVS(235)*XX(78))&
             &/(JVS(242))
  XX(80) = X(80)/JVS(250)
  XX(81) = X(81)/JVS(254)
  XX(82) = X(82)/JVS(258)
  XX(83) = (X(83)-JVS(80)*XX(33)-JVS(243)*XX(79))/(JVS(262))
  XX(84) = (X(84)-JVS(36)*XX(17)-JVS(59)*XX(26)-JVS(95)*XX(38)-JVS(107)*XX(44)-JVS(109)*XX(45)-JVS(134)*XX(51)-JVS(229)&
             &*XX(77)-JVS(236)*XX(78))/(JVS(266))
  XX(85) = X(85)/JVS(270)
  XX(86) = X(86)/JVS(274)
  XX(87) = X(87)/JVS(279)
  XX(88) = X(88)/JVS(283)
  XX(89) = X(89)/JVS(287)
  XX(90) = (X(90)-JVS(16)*XX(8)-JVS(135)*XX(51))/(JVS(292))
  XX(91) = (X(91)-JVS(119)*XX(50)-JVS(144)*XX(52))/(JVS(297))
  XX(92) = (X(92)-JVS(120)*XX(50))/(JVS(307))
  XX(93) = (X(93)-JVS(308)*XX(92))/(JVS(329))
  XX(94) = (X(94)-JVS(309)*XX(92))/(JVS(336))
  XX(95) = X(95)/JVS(347)
  XX(96) = (X(96)-JVS(77)*XX(32)-JVS(244)*XX(79)-JVS(310)*XX(92))/(JVS(365))
  XX(97) = (X(97)-JVS(43)*XX(20)-JVS(52)*XX(23)-JVS(54)*XX(24)-JVS(56)*XX(25)-JVS(65)*XX(28)-JVS(89)*XX(36)-JVS(103)&
             &*XX(42)-JVS(230)*XX(77)-JVS(237)*XX(78)-JVS(267)*XX(84))/(JVS(376))
  XX(98) = (X(98)-JVS(275)*XX(86)-JVS(348)*XX(95))/(JVS(384))
  XX(99) = (X(99)-JVS(6)*XX(3)-JVS(136)*XX(51))/(JVS(398))
  XX(100) = (X(100)-JVS(224)*XX(76)-JVS(349)*XX(95))/(JVS(422))
  XX(101) = (X(101)-JVS(350)*XX(95))/(JVS(431))
  XX(102) = (X(102)-JVS(271)*XX(85)-JVS(311)*XX(92)-JVS(399)*XX(99))/(JVS(439))
  XX(103) = X(103)/JVS(450)
  XX(104) = (X(104)-JVS(288)*XX(89)-JVS(351)*XX(95)-JVS(400)*XX(99))/(JVS(465))
  XX(105) = (X(105)-JVS(312)*XX(92)-JVS(337)*XX(94)-JVS(451)*XX(103))/(JVS(473))
  XX(106) = (X(106)-JVS(284)*XX(88)-JVS(338)*XX(94)-JVS(352)*XX(95)-JVS(401)*XX(99))/(JVS(482))
  XX(107) = (X(107)-JVS(255)*XX(81)-JVS(353)*XX(95))/(JVS(491))
  XX(108) = (X(108)-JVS(259)*XX(82)-JVS(354)*XX(95)-JVS(402)*XX(99))/(JVS(502))
  XX(109) = (X(109)-JVS(276)*XX(86)-JVS(339)*XX(94)-JVS(355)*XX(95)-JVS(403)*XX(99)-JVS(452)*XX(103))/(JVS(512))
  XX(110) = (X(110)-JVS(179)*XX(64)-JVS(356)*XX(95)-JVS(366)*XX(96))/(JVS(522))
  XX(111) = (X(111)-JVS(251)*XX(80)-JVS(357)*XX(95)-JVS(453)*XX(103))/(JVS(533))
  XX(112) = (X(112)-JVS(454)*XX(103))/(JVS(544))
  XX(113) = (X(113)-JVS(280)*XX(87)-JVS(313)*XX(92)-JVS(340)*XX(94)-JVS(358)*XX(95)-JVS(404)*XX(99)-JVS(455)*XX(103))&
              &/(JVS(555))
  XX(114) = (X(114)-JVS(200)*XX(70))/(JVS(569))
  XX(115) = (X(115)-JVS(359)*XX(95))/(JVS(580))
  XX(116) = (X(116)-JVS(314)*XX(92)-JVS(405)*XX(99))/(JVS(607))
  XX(117) = (X(117)-JVS(2)*XX(1)-JVS(71)*XX(30)-JVS(137)*XX(51)-JVS(245)*XX(79)-JVS(315)*XX(92)-JVS(406)*XX(99))&
              &/(JVS(653))
  XX(118) = (X(118)-JVS(316)*XX(92)-JVS(407)*XX(99))/(JVS(685))
  XX(119) = (X(119)-JVS(204)*XX(71)-JVS(360)*XX(95)-JVS(367)*XX(96)-JVS(423)*XX(100)-JVS(523)*XX(110)-JVS(545)*XX(112)&
              &-JVS(608)*XX(116)-JVS(654)*XX(117)-JVS(686)*XX(118))/(JVS(705))
  XX(120) = (X(120)-JVS(20)*XX(10)-JVS(138)*XX(51)-JVS(368)*XX(96)-JVS(424)*XX(100)-JVS(432)*XX(101)-JVS(524)*XX(110)&
              &-JVS(546)*XX(112)-JVS(609)*XX(116)-JVS(655)*XX(117)-JVS(687)*XX(118)-JVS(706)*XX(119))/(JVS(717))
  XX(121) = (X(121)-JVS(212)*XX(73)-JVS(293)*XX(90)-JVS(456)*XX(103)-JVS(656)*XX(117)-JVS(688)*XX(118))/(JVS(729))
  XX(122) = (X(122)-JVS(24)*XX(12)-JVS(121)*XX(50)-JVS(139)*XX(51)-JVS(317)*XX(92)-JVS(385)*XX(98)-JVS(408)*XX(99)&
              &-JVS(440)*XX(102)-JVS(457)*XX(103)-JVS(513)*XX(109)-JVS(570)*XX(114)-JVS(610)*XX(116)-JVS(657)*XX(117)&
              &-JVS(689)*XX(118)-JVS(730)*XX(121))/(JVS(745))
  XX(123) = (X(123)-JVS(74)*XX(31)-JVS(246)*XX(79)-JVS(318)*XX(92)-JVS(409)*XX(99)-JVS(658)*XX(117))/(JVS(767))
  XX(124) = (X(124)-JVS(289)*XX(89)-JVS(361)*XX(95)-JVS(466)*XX(104)-JVS(474)*XX(105)-JVS(556)*XX(113)-JVS(571)*XX(114)&
              &-JVS(611)*XX(116)-JVS(659)*XX(117)-JVS(690)*XX(118)-JVS(731)*XX(121)-JVS(746)*XX(122)-JVS(768)*XX(123))&
              &/(JVS(785))
  XX(125) = (X(125)-JVS(22)*XX(11)-JVS(140)*XX(51)-JVS(319)*XX(92)-JVS(330)*XX(93)-JVS(483)*XX(106)-JVS(492)*XX(107)&
              &-JVS(534)*XX(111)-JVS(572)*XX(114)-JVS(612)*XX(116)-JVS(660)*XX(117)-JVS(691)*XX(118)-JVS(732)*XX(121)&
              &-JVS(769)*XX(123))/(JVS(798))
  XX(126) = (X(126)-JVS(193)*XX(68)-JVS(661)*XX(117)-JVS(770)*XX(123))/(JVS(814))
  XX(127) = (X(127)-JVS(410)*XX(99)-JVS(581)*XX(115)-JVS(613)*XX(116)-JVS(662)*XX(117)-JVS(692)*XX(118)-JVS(718)*XX(120)&
              &-JVS(771)*XX(123)-JVS(815)*XX(126))/(JVS(836))
  XX(128) = (X(128)-JVS(122)*XX(50)-JVS(169)*XX(61)-JVS(216)*XX(74)-JVS(320)*XX(92)-JVS(341)*XX(94)-JVS(369)*XX(96)&
              &-JVS(386)*XX(98)-JVS(411)*XX(99)-JVS(425)*XX(100)-JVS(433)*XX(101)-JVS(441)*XX(102)-JVS(458)*XX(103)-JVS(467)&
              &*XX(104)-JVS(475)*XX(105)-JVS(484)*XX(106)-JVS(493)*XX(107)-JVS(503)*XX(108)-JVS(514)*XX(109)-JVS(525)&
              &*XX(110)-JVS(535)*XX(111)-JVS(547)*XX(112)-JVS(557)*XX(113)-JVS(573)*XX(114)-JVS(582)*XX(115)-JVS(614)&
              &*XX(116)-JVS(663)*XX(117)-JVS(693)*XX(118)-JVS(707)*XX(119)-JVS(719)*XX(120)-JVS(733)*XX(121)-JVS(747)&
              &*XX(122)-JVS(772)*XX(123)-JVS(786)*XX(124)-JVS(799)*XX(125)-JVS(816)*XX(126)-JVS(837)*XX(127))/(JVS(877))
  XX(129) = (X(129)-JVS(220)*XX(75)-JVS(321)*XX(92)-JVS(342)*XX(94)-JVS(362)*XX(95)-JVS(370)*XX(96)-JVS(387)*XX(98)&
              &-JVS(412)*XX(99)-JVS(426)*XX(100)-JVS(434)*XX(101)-JVS(442)*XX(102)-JVS(459)*XX(103)-JVS(468)*XX(104)&
              &-JVS(476)*XX(105)-JVS(485)*XX(106)-JVS(494)*XX(107)-JVS(504)*XX(108)-JVS(515)*XX(109)-JVS(526)*XX(110)&
              &-JVS(536)*XX(111)-JVS(548)*XX(112)-JVS(558)*XX(113)-JVS(574)*XX(114)-JVS(583)*XX(115)-JVS(615)*XX(116)&
              &-JVS(664)*XX(117)-JVS(694)*XX(118)-JVS(708)*XX(119)-JVS(720)*XX(120)-JVS(734)*XX(121)-JVS(748)*XX(122)&
              &-JVS(773)*XX(123)-JVS(787)*XX(124)-JVS(800)*XX(125)-JVS(817)*XX(126)-JVS(838)*XX(127)-JVS(878)*XX(128))&
              &/(JVS(918))
  XX(130) = (X(130)-JVS(44)*XX(20)-JVS(46)*XX(21)-JVS(159)*XX(58)-JVS(180)*XX(64)-JVS(183)*XX(65)-JVS(194)*XX(68)&
              &-JVS(201)*XX(70)-JVS(205)*XX(71)-JVS(208)*XX(72)-JVS(213)*XX(73)-JVS(217)*XX(74)-JVS(221)*XX(75)-JVS(225)&
              &*XX(76)-JVS(231)*XX(77)-JVS(238)*XX(78)-JVS(247)*XX(79)-JVS(252)*XX(80)-JVS(256)*XX(81)-JVS(260)*XX(82)&
              &-JVS(272)*XX(85)-JVS(277)*XX(86)-JVS(281)*XX(87)-JVS(285)*XX(88)-JVS(290)*XX(89)-JVS(322)*XX(92)-JVS(343)&
              &*XX(94)-JVS(371)*XX(96)-JVS(377)*XX(97)-JVS(388)*XX(98)-JVS(413)*XX(99)-JVS(427)*XX(100)-JVS(435)*XX(101)&
              &-JVS(443)*XX(102)-JVS(460)*XX(103)-JVS(469)*XX(104)-JVS(477)*XX(105)-JVS(486)*XX(106)-JVS(495)*XX(107)&
              &-JVS(505)*XX(108)-JVS(516)*XX(109)-JVS(527)*XX(110)-JVS(537)*XX(111)-JVS(549)*XX(112)-JVS(559)*XX(113)&
              &-JVS(575)*XX(114)-JVS(584)*XX(115)-JVS(616)*XX(116)-JVS(665)*XX(117)-JVS(695)*XX(118)-JVS(709)*XX(119)&
              &-JVS(721)*XX(120)-JVS(735)*XX(121)-JVS(749)*XX(122)-JVS(774)*XX(123)-JVS(788)*XX(124)-JVS(801)*XX(125)&
              &-JVS(818)*XX(126)-JVS(839)*XX(127)-JVS(879)*XX(128)-JVS(919)*XX(129))/(JVS(986))
  XX(131) = (X(131)-JVS(57)*XX(25)-JVS(123)*XX(50)-JVS(172)*XX(62)-JVS(323)*XX(92)-JVS(344)*XX(94)-JVS(372)*XX(96)&
              &-JVS(378)*XX(97)-JVS(389)*XX(98)-JVS(414)*XX(99)-JVS(428)*XX(100)-JVS(436)*XX(101)-JVS(444)*XX(102)-JVS(461)&
              &*XX(103)-JVS(470)*XX(104)-JVS(478)*XX(105)-JVS(487)*XX(106)-JVS(496)*XX(107)-JVS(506)*XX(108)-JVS(517)&
              &*XX(109)-JVS(528)*XX(110)-JVS(538)*XX(111)-JVS(550)*XX(112)-JVS(560)*XX(113)-JVS(576)*XX(114)-JVS(585)&
              &*XX(115)-JVS(617)*XX(116)-JVS(666)*XX(117)-JVS(696)*XX(118)-JVS(710)*XX(119)-JVS(722)*XX(120)-JVS(736)&
              &*XX(121)-JVS(750)*XX(122)-JVS(775)*XX(123)-JVS(789)*XX(124)-JVS(802)*XX(125)-JVS(819)*XX(126)-JVS(840)&
              &*XX(127)-JVS(880)*XX(128)-JVS(920)*XX(129)-JVS(987)*XX(130))/(JVS(1023))
  XX(132) = (X(132)-JVS(10)*XX(5)-JVS(86)*XX(35)-JVS(90)*XX(36)-JVS(141)*XX(51)-JVS(153)*XX(56)-JVS(156)*XX(57)-JVS(170)&
              &*XX(61)-JVS(173)*XX(62)-JVS(176)*XX(63)-JVS(209)*XX(72)-JVS(268)*XX(84)-JVS(294)*XX(90)-JVS(379)*XX(97)&
              &-JVS(415)*XX(99)-JVS(462)*XX(103)-JVS(667)*XX(117)-JVS(697)*XX(118)-JVS(737)*XX(121)-JVS(776)*XX(123)&
              &-JVS(881)*XX(128)-JVS(921)*XX(129)-JVS(988)*XX(130)-JVS(1024)*XX(131))/(JVS(1074))
  XX(133) = (X(133)-JVS(68)*XX(29)-JVS(163)*XX(59)-JVS(166)*XX(60)-JVS(177)*XX(63)-JVS(187)*XX(66)-JVS(197)*XX(69)&
              &-JVS(263)*XX(83)-JVS(298)*XX(91)-JVS(324)*XX(92)-JVS(331)*XX(93)-JVS(380)*XX(97)-JVS(390)*XX(98)-JVS(416)&
              &*XX(99)-JVS(497)*XX(107)-JVS(507)*XX(108)-JVS(518)*XX(109)-JVS(539)*XX(111)-JVS(577)*XX(114)-JVS(586)*XX(115)&
              &-JVS(618)*XX(116)-JVS(668)*XX(117)-JVS(698)*XX(118)-JVS(711)*XX(119)-JVS(723)*XX(120)-JVS(738)*XX(121)&
              &-JVS(751)*XX(122)-JVS(777)*XX(123)-JVS(790)*XX(124)-JVS(803)*XX(125)-JVS(820)*XX(126)-JVS(841)*XX(127)&
              &-JVS(882)*XX(128)-JVS(922)*XX(129)-JVS(989)*XX(130)-JVS(1025)*XX(131)-JVS(1075)*XX(132))/(JVS(1115))
  XX(134) = (X(134)-JVS(12)*XX(6)-JVS(40)*XX(19)-JVS(124)*XX(50)-JVS(142)*XX(51)-JVS(295)*XX(90)-JVS(299)*XX(91)&
              &-JVS(325)*XX(92)-JVS(332)*XX(93)-JVS(345)*XX(94)-JVS(381)*XX(97)-JVS(417)*XX(99)-JVS(463)*XX(103)-JVS(479)&
              &*XX(105)-JVS(498)*XX(107)-JVS(508)*XX(108)-JVS(540)*XX(111)-JVS(561)*XX(113)-JVS(578)*XX(114)-JVS(619)&
              &*XX(116)-JVS(669)*XX(117)-JVS(699)*XX(118)-JVS(739)*XX(121)-JVS(752)*XX(122)-JVS(778)*XX(123)-JVS(791)&
              &*XX(124)-JVS(804)*XX(125)-JVS(821)*XX(126)-JVS(842)*XX(127)-JVS(883)*XX(128)-JVS(923)*XX(129)-JVS(990)&
              &*XX(130)-JVS(1026)*XX(131)-JVS(1076)*XX(132)-JVS(1116)*XX(133))/(JVS(1137))
  XX(135) = (X(135)-JVS(27)*XX(13)-JVS(30)*XX(14)-JVS(50)*XX(22)-JVS(63)*XX(27)-JVS(66)*XX(28)-JVS(93)*XX(37)-JVS(125)&
              &*XX(50)-JVS(145)*XX(52)-JVS(147)*XX(53)-JVS(149)*XX(54)-JVS(151)*XX(55)-JVS(160)*XX(58)-JVS(164)*XX(59)&
              &-JVS(167)*XX(60)-JVS(174)*XX(62)-JVS(181)*XX(64)-JVS(184)*XX(65)-JVS(188)*XX(66)-JVS(190)*XX(67)-JVS(195)&
              &*XX(68)-JVS(198)*XX(69)-JVS(202)*XX(70)-JVS(206)*XX(71)-JVS(210)*XX(72)-JVS(214)*XX(73)-JVS(218)*XX(74)&
              &-JVS(222)*XX(75)-JVS(226)*XX(76)-JVS(232)*XX(77)-JVS(239)*XX(78)-JVS(248)*XX(79)-JVS(253)*XX(80)-JVS(257)&
              &*XX(81)-JVS(261)*XX(82)-JVS(264)*XX(83)-JVS(273)*XX(85)-JVS(278)*XX(86)-JVS(282)*XX(87)-JVS(286)*XX(88)&
              &-JVS(291)*XX(89)-JVS(296)*XX(90)-JVS(300)*XX(91)-JVS(326)*XX(92)-JVS(333)*XX(93)-JVS(346)*XX(94)-JVS(363)&
              &*XX(95)-JVS(373)*XX(96)-JVS(382)*XX(97)-JVS(418)*XX(99)-JVS(429)*XX(100)-JVS(437)*XX(101)-JVS(445)*XX(102)&
              &-JVS(464)*XX(103)-JVS(480)*XX(105)-JVS(488)*XX(106)-JVS(499)*XX(107)-JVS(509)*XX(108)-JVS(519)*XX(109)&
              &-JVS(529)*XX(110)-JVS(541)*XX(111)-JVS(551)*XX(112)-JVS(562)*XX(113)-JVS(579)*XX(114)-JVS(587)*XX(115)&
              &-JVS(620)*XX(116)-JVS(670)*XX(117)-JVS(700)*XX(118)-JVS(712)*XX(119)-JVS(724)*XX(120)-JVS(740)*XX(121)&
              &-JVS(753)*XX(122)-JVS(779)*XX(123)-JVS(792)*XX(124)-JVS(805)*XX(125)-JVS(822)*XX(126)-JVS(843)*XX(127)&
              &-JVS(884)*XX(128)-JVS(924)*XX(129)-JVS(991)*XX(130)-JVS(1027)*XX(131)-JVS(1077)*XX(132)-JVS(1117)*XX(133)&
              &-JVS(1138)*XX(134))/(JVS(1219))
  XX(136) = (X(136)-JVS(157)*XX(57)-JVS(185)*XX(65)-JVS(671)*XX(117)-JVS(780)*XX(123)-JVS(823)*XX(126)-JVS(885)*XX(128)&
              &-JVS(925)*XX(129)-JVS(992)*XX(130)-JVS(1028)*XX(131)-JVS(1078)*XX(132)-JVS(1118)*XX(133)-JVS(1139)*XX(134)&
              &-JVS(1220)*XX(135))/(JVS(1239))
  XX(137) = (X(137)-JVS(41)*XX(19)-JVS(47)*XX(21)-JVS(60)*XX(26)-JVS(69)*XX(29)-JVS(72)*XX(30)-JVS(75)*XX(31)-JVS(78)&
              &*XX(32)-JVS(81)*XX(33)-JVS(84)*XX(34)-JVS(87)*XX(35)-JVS(154)*XX(56)-JVS(191)*XX(67)-JVS(240)*XX(78)-JVS(249)&
              &*XX(79)-JVS(265)*XX(83)-JVS(269)*XX(84)-JVS(327)*XX(92)-JVS(374)*XX(96)-JVS(383)*XX(97)-JVS(419)*XX(99)&
              &-JVS(430)*XX(100)-JVS(530)*XX(110)-JVS(552)*XX(112)-JVS(621)*XX(116)-JVS(672)*XX(117)-JVS(701)*XX(118)&
              &-JVS(725)*XX(120)-JVS(781)*XX(123)-JVS(824)*XX(126)-JVS(844)*XX(127)-JVS(886)*XX(128)-JVS(926)*XX(129)&
              &-JVS(993)*XX(130)-JVS(1029)*XX(131)-JVS(1079)*XX(132)-JVS(1119)*XX(133)-JVS(1140)*XX(134)-JVS(1221)*XX(135)&
              &-JVS(1240)*XX(136))/(JVS(1273))
  XX(137) = XX(137)
  XX(136) = XX(136)-JVS(1272)*XX(137)
  XX(135) = XX(135)-JVS(1238)*XX(136)-JVS(1271)*XX(137)
  XX(134) = XX(134)-JVS(1218)*XX(135)-JVS(1237)*XX(136)-JVS(1270)*XX(137)
  XX(133) = XX(133)-JVS(1136)*XX(134)-JVS(1217)*XX(135)-JVS(1236)*XX(136)-JVS(1269)*XX(137)
  XX(132) = XX(132)-JVS(1114)*XX(133)-JVS(1135)*XX(134)-JVS(1216)*XX(135)-JVS(1235)*XX(136)-JVS(1268)*XX(137)
  XX(131) = XX(131)-JVS(1073)*XX(132)-JVS(1113)*XX(133)-JVS(1134)*XX(134)-JVS(1215)*XX(135)-JVS(1234)*XX(136)-JVS(1267)&
              &*XX(137)
  XX(130) = XX(130)-JVS(1022)*XX(131)-JVS(1072)*XX(132)-JVS(1112)*XX(133)-JVS(1133)*XX(134)-JVS(1214)*XX(135)-JVS(1233)&
              &*XX(136)-JVS(1266)*XX(137)
  XX(129) = XX(129)-JVS(985)*XX(130)-JVS(1021)*XX(131)-JVS(1071)*XX(132)-JVS(1111)*XX(133)-JVS(1132)*XX(134)-JVS(1213)&
              &*XX(135)-JVS(1232)*XX(136)-JVS(1265)*XX(137)
  XX(128) = XX(128)-JVS(917)*XX(129)-JVS(984)*XX(130)-JVS(1020)*XX(131)-JVS(1070)*XX(132)-JVS(1110)*XX(133)-JVS(1131)&
              &*XX(134)-JVS(1212)*XX(135)-JVS(1231)*XX(136)-JVS(1264)*XX(137)
  XX(127) = XX(127)-JVS(876)*XX(128)-JVS(916)*XX(129)-JVS(983)*XX(130)-JVS(1019)*XX(131)-JVS(1069)*XX(132)-JVS(1109)&
              &*XX(133)-JVS(1211)*XX(135)-JVS(1230)*XX(136)-JVS(1263)*XX(137)
  XX(126) = XX(126)-JVS(875)*XX(128)-JVS(915)*XX(129)-JVS(982)*XX(130)-JVS(1018)*XX(131)-JVS(1068)*XX(132)-JVS(1108)&
              &*XX(133)-JVS(1210)*XX(135)-JVS(1262)*XX(137)
  XX(125) = XX(125)-JVS(813)*XX(126)-JVS(835)*XX(127)-JVS(874)*XX(128)-JVS(914)*XX(129)-JVS(981)*XX(130)-JVS(1017)&
              &*XX(131)-JVS(1067)*XX(132)-JVS(1107)*XX(133)-JVS(1130)*XX(134)-JVS(1209)*XX(135)-JVS(1229)*XX(136)-JVS(1261)&
              &*XX(137)
  XX(124) = XX(124)-JVS(797)*XX(125)-JVS(812)*XX(126)-JVS(834)*XX(127)-JVS(873)*XX(128)-JVS(913)*XX(129)-JVS(980)&
              &*XX(130)-JVS(1016)*XX(131)-JVS(1066)*XX(132)-JVS(1106)*XX(133)-JVS(1129)*XX(134)-JVS(1208)*XX(135)-JVS(1228)&
              &*XX(136)-JVS(1260)*XX(137)
  XX(123) = XX(123)-JVS(872)*XX(128)-JVS(912)*XX(129)-JVS(979)*XX(130)-JVS(1065)*XX(132)-JVS(1105)*XX(133)-JVS(1207)&
              &*XX(135)-JVS(1259)*XX(137)
  XX(122) = XX(122)-JVS(766)*XX(123)-JVS(811)*XX(126)-JVS(833)*XX(127)-JVS(871)*XX(128)-JVS(911)*XX(129)-JVS(978)&
              &*XX(130)-JVS(1015)*XX(131)-JVS(1064)*XX(132)-JVS(1104)*XX(133)-JVS(1128)*XX(134)-JVS(1206)*XX(135)-JVS(1227)&
              &*XX(136)-JVS(1258)*XX(137)
  XX(121) = XX(121)-JVS(765)*XX(123)-JVS(870)*XX(128)-JVS(910)*XX(129)-JVS(977)*XX(130)-JVS(1014)*XX(131)-JVS(1063)&
              &*XX(132)-JVS(1103)*XX(133)-JVS(1127)*XX(134)-JVS(1205)*XX(135)-JVS(1257)*XX(137)
  XX(120) = XX(120)-JVS(764)*XX(123)-JVS(810)*XX(126)-JVS(832)*XX(127)-JVS(869)*XX(128)-JVS(909)*XX(129)-JVS(976)&
              &*XX(130)-JVS(1013)*XX(131)-JVS(1062)*XX(132)-JVS(1102)*XX(133)-JVS(1204)*XX(135)-JVS(1226)*XX(136)-JVS(1256)&
              &*XX(137)
  XX(119) = XX(119)-JVS(716)*XX(120)-JVS(763)*XX(123)-JVS(809)*XX(126)-JVS(831)*XX(127)-JVS(868)*XX(128)-JVS(908)&
              &*XX(129)-JVS(975)*XX(130)-JVS(1012)*XX(131)-JVS(1061)*XX(132)-JVS(1101)*XX(133)-JVS(1203)*XX(135)-JVS(1225)&
              &*XX(136)-JVS(1255)*XX(137)
  XX(118) = XX(118)-JVS(762)*XX(123)-JVS(867)*XX(128)-JVS(974)*XX(130)-JVS(1060)*XX(132)-JVS(1100)*XX(133)-JVS(1202)&
              &*XX(135)
  XX(117) = XX(117)-JVS(973)*XX(130)-JVS(1059)*XX(132)-JVS(1099)*XX(133)-JVS(1201)*XX(135)-JVS(1254)*XX(137)
  XX(116) = XX(116)-JVS(808)*XX(126)-JVS(972)*XX(130)-JVS(1058)*XX(132)-JVS(1098)*XX(133)-JVS(1200)*XX(135)-JVS(1224)&
              &*XX(136)
  XX(115) = XX(115)-JVS(606)*XX(116)-JVS(652)*XX(117)-JVS(684)*XX(118)-JVS(715)*XX(120)-JVS(761)*XX(123)-JVS(830)&
              &*XX(127)-JVS(866)*XX(128)-JVS(907)*XX(129)-JVS(971)*XX(130)-JVS(1011)*XX(131)-JVS(1057)*XX(132)-JVS(1199)&
              &*XX(135)
  XX(114) = XX(114)-JVS(651)*XX(117)-JVS(865)*XX(128)-JVS(906)*XX(129)-JVS(970)*XX(130)-JVS(1010)*XX(131)-JVS(1056)&
              &*XX(132)-JVS(1126)*XX(134)-JVS(1198)*XX(135)
  XX(113) = XX(113)-JVS(568)*XX(114)-JVS(605)*XX(116)-JVS(650)*XX(117)-JVS(683)*XX(118)-JVS(829)*XX(127)-JVS(864)&
              &*XX(128)-JVS(905)*XX(129)-JVS(969)*XX(130)-JVS(1009)*XX(131)-JVS(1055)*XX(132)-JVS(1097)*XX(133)-JVS(1197)&
              &*XX(135)
  XX(112) = XX(112)-JVS(649)*XX(117)-JVS(682)*XX(118)-JVS(714)*XX(120)-JVS(828)*XX(127)-JVS(863)*XX(128)-JVS(904)&
              &*XX(129)-JVS(968)*XX(130)-JVS(1008)*XX(131)-JVS(1054)*XX(132)-JVS(1196)*XX(135)
  XX(111) = XX(111)-JVS(604)*XX(116)-JVS(648)*XX(117)-JVS(681)*XX(118)-JVS(760)*XX(123)-JVS(862)*XX(128)-JVS(903)&
              &*XX(129)-JVS(967)*XX(130)-JVS(1007)*XX(131)-JVS(1053)*XX(132)-JVS(1195)*XX(135)
  XX(110) = XX(110)-JVS(543)*XX(112)-JVS(603)*XX(116)-JVS(647)*XX(117)-JVS(861)*XX(128)-JVS(902)*XX(129)-JVS(966)&
              &*XX(130)-JVS(1006)*XX(131)-JVS(1052)*XX(132)-JVS(1194)*XX(135)-JVS(1253)*XX(137)
  XX(109) = XX(109)-JVS(567)*XX(114)-JVS(602)*XX(116)-JVS(646)*XX(117)-JVS(680)*XX(118)-JVS(860)*XX(128)-JVS(901)&
              &*XX(129)-JVS(965)*XX(130)-JVS(1005)*XX(131)-JVS(1051)*XX(132)-JVS(1096)*XX(133)-JVS(1193)*XX(135)
  XX(108) = XX(108)-JVS(601)*XX(116)-JVS(645)*XX(117)-JVS(744)*XX(122)-JVS(796)*XX(125)-JVS(859)*XX(128)-JVS(900)&
              &*XX(129)-JVS(964)*XX(130)-JVS(1004)*XX(131)-JVS(1050)*XX(132)-JVS(1095)*XX(133)-JVS(1192)*XX(135)
  XX(107) = XX(107)-JVS(600)*XX(116)-JVS(644)*XX(117)-JVS(759)*XX(123)-JVS(858)*XX(128)-JVS(899)*XX(129)-JVS(963)&
              &*XX(130)-JVS(1003)*XX(131)-JVS(1049)*XX(132)-JVS(1191)*XX(135)
  XX(106) = XX(106)-JVS(566)*XX(114)-JVS(599)*XX(116)-JVS(643)*XX(117)-JVS(679)*XX(118)-JVS(827)*XX(127)-JVS(857)&
              &*XX(128)-JVS(898)*XX(129)-JVS(962)*XX(130)-JVS(1002)*XX(131)-JVS(1048)*XX(132)-JVS(1094)*XX(133)-JVS(1190)&
              &*XX(135)
  XX(105) = XX(105)-JVS(554)*XX(113)-JVS(565)*XX(114)-JVS(642)*XX(117)-JVS(678)*XX(118)-JVS(728)*XX(121)-JVS(856)&
              &*XX(128)-JVS(961)*XX(130)-JVS(1125)*XX(134)-JVS(1189)*XX(135)
  XX(104) = XX(104)-JVS(472)*XX(105)-JVS(598)*XX(116)-JVS(641)*XX(117)-JVS(743)*XX(122)-JVS(784)*XX(124)-JVS(795)&
              &*XX(125)-JVS(826)*XX(127)-JVS(855)*XX(128)-JVS(897)*XX(129)-JVS(960)*XX(130)-JVS(1001)*XX(131)-JVS(1047)&
              &*XX(132)-JVS(1093)*XX(133)-JVS(1188)*XX(135)
  XX(103) = XX(103)-JVS(640)*XX(117)-JVS(677)*XX(118)-JVS(854)*XX(128)-JVS(959)*XX(130)-JVS(1187)*XX(135)
  XX(102) = XX(102)-JVS(449)*XX(103)-JVS(639)*XX(117)-JVS(676)*XX(118)-JVS(825)*XX(127)-JVS(853)*XX(128)-JVS(896)&
              &*XX(129)-JVS(958)*XX(130)-JVS(1000)*XX(131)-JVS(1046)*XX(132)-JVS(1092)*XX(133)-JVS(1186)*XX(135)
  XX(101) = XX(101)-JVS(597)*XX(116)-JVS(638)*XX(117)-JVS(704)*XX(119)-JVS(713)*XX(120)-JVS(758)*XX(123)-JVS(852)&
              &*XX(128)-JVS(895)*XX(129)-JVS(957)*XX(130)-JVS(999)*XX(131)-JVS(1045)*XX(132)-JVS(1185)*XX(135)
  XX(100) = XX(100)-JVS(596)*XX(116)-JVS(637)*XX(117)-JVS(851)*XX(128)-JVS(894)*XX(129)-JVS(956)*XX(130)-JVS(998)&
              &*XX(131)-JVS(1044)*XX(132)-JVS(1184)*XX(135)
  XX(99) = XX(99)-JVS(1043)*XX(132)-JVS(1091)*XX(133)-JVS(1183)*XX(135)
  XX(98) = XX(98)-JVS(511)*XX(109)-JVS(595)*XX(116)-JVS(636)*XX(117)-JVS(675)*XX(118)-JVS(850)*XX(128)-JVS(893)*XX(129)&
             &-JVS(955)*XX(130)-JVS(997)*XX(131)-JVS(1042)*XX(132)-JVS(1182)*XX(135)
  XX(97) = XX(97)-JVS(397)*XX(99)-JVS(954)*XX(130)-JVS(996)*XX(131)-JVS(1041)*XX(132)-JVS(1090)*XX(133)-JVS(1124)&
             &*XX(134)-JVS(1181)*XX(135)-JVS(1252)*XX(137)
  XX(96) = XX(96)-JVS(542)*XX(112)-JVS(849)*XX(128)-JVS(892)*XX(129)-JVS(953)*XX(130)-JVS(1180)*XX(135)-JVS(1251)&
             &*XX(137)
  XX(95) = XX(95)-JVS(594)*XX(116)-JVS(952)*XX(130)-JVS(1179)*XX(135)
  XX(94) = XX(94)-JVS(564)*XX(114)-JVS(635)*XX(117)-JVS(951)*XX(130)-JVS(1178)*XX(135)
  XX(93) = XX(93)-JVS(490)*XX(107)-JVS(532)*XX(111)-JVS(634)*XX(117)-JVS(757)*XX(123)-JVS(891)*XX(129)-JVS(950)*XX(130)&
             &-JVS(1089)*XX(133)-JVS(1123)*XX(134)-JVS(1177)*XX(135)
  XX(92) = XX(92)-JVS(949)*XX(130)-JVS(1176)*XX(135)
  XX(91) = XX(91)-JVS(306)*XX(92)-JVS(328)*XX(93)-JVS(501)*XX(108)-JVS(633)*XX(117)-JVS(742)*XX(122)-JVS(783)*XX(124)&
             &-JVS(794)*XX(125)-JVS(948)*XX(130)-JVS(1088)*XX(133)-JVS(1122)*XX(134)-JVS(1175)*XX(135)
  XX(90) = XX(90)-JVS(448)*XX(103)-JVS(632)*XX(117)-JVS(727)*XX(121)-JVS(947)*XX(130)-JVS(1040)*XX(132)-JVS(1121)&
             &*XX(134)-JVS(1174)*XX(135)
  XX(89) = XX(89)-JVS(471)*XX(105)-JVS(631)*XX(117)-JVS(741)*XX(122)-JVS(782)*XX(124)-JVS(793)*XX(125)-JVS(946)*XX(130)&
             &-JVS(1173)*XX(135)
  XX(88) = XX(88)-JVS(335)*XX(94)-JVS(481)*XX(106)-JVS(593)*XX(116)-JVS(630)*XX(117)-JVS(674)*XX(118)-JVS(848)*XX(128)&
             &-JVS(945)*XX(130)-JVS(1172)*XX(135)
  XX(87) = XX(87)-JVS(305)*XX(92)-JVS(334)*XX(94)-JVS(447)*XX(103)-JVS(553)*XX(113)-JVS(673)*XX(118)-JVS(944)*XX(130)&
             &-JVS(1171)*XX(135)
  XX(86) = XX(86)-JVS(510)*XX(109)-JVS(592)*XX(116)-JVS(943)*XX(130)-JVS(1039)*XX(132)-JVS(1170)*XX(135)
  XX(85) = XX(85)-JVS(304)*XX(92)-JVS(438)*XX(102)-JVS(446)*XX(103)-JVS(629)*XX(117)-JVS(942)*XX(130)-JVS(1169)*XX(135)
  XX(84) = XX(84)-JVS(375)*XX(97)-JVS(396)*XX(99)-JVS(1038)*XX(132)-JVS(1087)*XX(133)-JVS(1168)*XX(135)-JVS(1250)&
             &*XX(137)
  XX(83) = XX(83)-JVS(303)*XX(92)-JVS(395)*XX(99)-JVS(807)*XX(126)-JVS(1086)*XX(133)-JVS(1167)*XX(135)-JVS(1249)*XX(137)
  XX(82) = XX(82)-JVS(500)*XX(108)-JVS(591)*XX(116)-JVS(941)*XX(130)-JVS(1037)*XX(132)-JVS(1166)*XX(135)
  XX(81) = XX(81)-JVS(489)*XX(107)-JVS(590)*XX(116)-JVS(940)*XX(130)-JVS(1036)*XX(132)-JVS(1165)*XX(135)
  XX(80) = XX(80)-JVS(531)*XX(111)-JVS(628)*XX(117)-JVS(756)*XX(123)-JVS(939)*XX(130)-JVS(1164)*XX(135)
  XX(79) = XX(79)-JVS(1163)*XX(135)-JVS(1248)*XX(137)
  XX(78) = XX(78)-JVS(1162)*XX(135)-JVS(1247)*XX(137)
  XX(77) = XX(77)-JVS(233)*XX(78)-JVS(1161)*XX(135)-JVS(1246)*XX(137)
  XX(76) = XX(76)-JVS(421)*XX(100)-JVS(589)*XX(116)-JVS(938)*XX(130)-JVS(1160)*XX(135)
  XX(75) = XX(75)-JVS(627)*XX(117)-JVS(890)*XX(129)-JVS(937)*XX(130)-JVS(1159)*XX(135)
  XX(74) = XX(74)-JVS(626)*XX(117)-JVS(847)*XX(128)-JVS(889)*XX(129)-JVS(1158)*XX(135)
  XX(73) = XX(73)-JVS(625)*XX(117)-JVS(726)*XX(121)-JVS(846)*XX(128)-JVS(1157)*XX(135)
  XX(72) = XX(72)-JVS(936)*XX(130)-JVS(1035)*XX(132)-JVS(1085)*XX(133)-JVS(1156)*XX(135)
  XX(71) = XX(71)-JVS(588)*XX(116)-JVS(703)*XX(119)-JVS(935)*XX(130)-JVS(1155)*XX(135)
  XX(70) = XX(70)-JVS(563)*XX(114)-JVS(624)*XX(117)-JVS(934)*XX(130)-JVS(1154)*XX(135)
  XX(69) = XX(69)-JVS(302)*XX(92)-JVS(394)*XX(99)-JVS(623)*XX(117)-JVS(933)*XX(130)-JVS(1084)*XX(133)-JVS(1153)*XX(135)
  XX(68) = XX(68)-JVS(755)*XX(123)-JVS(806)*XX(126)-JVS(932)*XX(130)-JVS(1152)*XX(135)
  XX(67) = XX(67)-JVS(241)*XX(79)-JVS(301)*XX(92)-JVS(420)*XX(100)-JVS(521)*XX(110)-JVS(1151)*XX(135)-JVS(1245)*XX(137)
  XX(66) = XX(66)-JVS(393)*XX(99)-JVS(622)*XX(117)-JVS(888)*XX(129)-JVS(931)*XX(130)-JVS(1083)*XX(133)-JVS(1150)*XX(135)
  XX(65) = XX(65)-JVS(754)*XX(123)-JVS(930)*XX(130)-JVS(1149)*XX(135)-JVS(1223)*XX(136)
  XX(64) = XX(64)-JVS(364)*XX(96)-JVS(520)*XX(110)-JVS(929)*XX(130)-JVS(1148)*XX(135)
  XX(63) = XX(63)-JVS(392)*XX(99)-JVS(995)*XX(131)-JVS(1034)*XX(132)-JVS(1082)*XX(133)-JVS(1120)*XX(134)
  XX(62) = XX(62)-JVS(994)*XX(131)-JVS(1033)*XX(132)-JVS(1147)*XX(135)
  XX(61) = XX(61)-JVS(845)*XX(128)-JVS(887)*XX(129)-JVS(1032)*XX(132)-JVS(1081)*XX(133)
  XX(60) = XX(60)-JVS(391)*XX(99)-JVS(702)*XX(119)-JVS(1080)*XX(133)-JVS(1146)*XX(135)
  XX(59) = XX(59)-JVS(928)*XX(130)-JVS(1145)*XX(135)
  XX(58) = XX(58)-JVS(927)*XX(130)-JVS(1144)*XX(135)
  XX(57) = XX(57)-JVS(1031)*XX(132)-JVS(1222)*XX(136)
  XX(56) = XX(56)-JVS(1030)*XX(132)-JVS(1244)*XX(137)
  XX(55) = XX(55)-JVS(1143)*XX(135)-JVS(1243)*XX(137)
  XX(54) = XX(54)-JVS(1142)*XX(135)-JVS(1242)*XX(137)
  XX(53) = XX(53)-JVS(1141)*XX(135)-JVS(1241)*XX(137)
  XX(52) = XX(52)
  XX(51) = XX(51)
  XX(50) = XX(50)
  XX(49) = XX(49)
  XX(48) = XX(48)
  XX(47) = XX(47)
  XX(46) = XX(46)
  XX(45) = XX(45)
  XX(44) = XX(44)
  XX(43) = XX(43)
  XX(42) = XX(42)
  XX(41) = XX(41)
  XX(40) = XX(40)
  XX(39) = XX(39)
  XX(38) = XX(38)
  XX(37) = XX(37)
  XX(36) = XX(36)
  XX(35) = XX(35)
  XX(34) = XX(34)
  XX(33) = XX(33)
  XX(32) = XX(32)
  XX(31) = XX(31)
  XX(30) = XX(30)
  XX(29) = XX(29)
  XX(28) = XX(28)
  XX(27) = XX(27)
  XX(26) = XX(26)
  XX(25) = XX(25)
  XX(24) = XX(24)
  XX(23) = XX(23)
  XX(22) = XX(22)
  XX(21) = XX(21)
  XX(20) = XX(20)
  XX(19) = XX(19)
  XX(18) = XX(18)
  XX(17) = XX(17)
  XX(16) = XX(16)
  XX(15) = XX(15)
  XX(14) = XX(14)
  XX(13) = XX(13)
  XX(12) = XX(12)
  XX(11) = XX(11)
  XX(10) = XX(10)
  XX(9) = XX(9)
  XX(8) = XX(8)
  XX(7) = XX(7)
  XX(6) = XX(6)
  XX(5) = XX(5)
  XX(4) = XX(4)
  XX(3) = XX(3)
  XX(2) = XX(2)
  XX(1) = XX(1)
      
END SUBROUTINE KppSolveTR

! End of KppSolveTR function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! BLAS_UTIL - BLAS-LIKE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!--------------------------------------------------------------
!
! BLAS/LAPACK-like subroutines used by the integration algorithms
! It is recommended to replace them by calls to the optimized
!      BLAS/LAPACK library for your machine
!
!  (C) Adrian Sandu, Aug. 2004
!      Virginia Polytechnic Institute and State University
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE WCOPY(N,X,incX,Y,incY)
!--------------------------------------------------------------
!     copies a vector, x, to a vector, y:  y <- x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL  SCOPY(N,X,1,Y,1)   or   CALL  DCOPY(N,X,1,Y,1)
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N)

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = X(i)
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i) = X(i)
        Y(i + 1) = X(i + 1)
        Y(i + 2) = X(i + 2)
        Y(i + 3) = X(i + 3)
        Y(i + 4) = X(i + 4)
        Y(i + 5) = X(i + 5)
        Y(i + 6) = X(i + 6)
        Y(i + 7) = X(i + 7)
      END DO

      END SUBROUTINE WCOPY


!--------------------------------------------------------------
      SUBROUTINE WAXPY(N,Alpha,X,incX,Y,incY)
!--------------------------------------------------------------
!     constant times a vector plus a vector: y <- y + Alpha*x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SAXPY(N,Alpha,X,1,Y,1) or  CALL DAXPY(N,Alpha,X,1,Y,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N),Alpha
      REAL(kind=dp), PARAMETER :: ZERO = 0.0_dp

      IF (Alpha .EQ. ZERO) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,4)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = Y(i) + Alpha*X(i)
        END DO
        IF( N .LT. 4 ) RETURN
      END IF
      MP1 = M + 1
      DO i = MP1,N,4
        Y(i) = Y(i) + Alpha*X(i)
        Y(i + 1) = Y(i + 1) + Alpha*X(i + 1)
        Y(i + 2) = Y(i + 2) + Alpha*X(i + 2)
        Y(i + 3) = Y(i + 3) + Alpha*X(i + 3)
      END DO
      
      END SUBROUTINE WAXPY



!--------------------------------------------------------------
      SUBROUTINE WSCAL(N,Alpha,X,incX)
!--------------------------------------------------------------
!     constant times a vector: x(1:N) <- Alpha*x(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SSCAL(N,Alpha,X,1) or  CALL DSCAL(N,Alpha,X,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,M,MP1,N
      REAL(kind=dp)  :: X(N),Alpha
      REAL(kind=dp), PARAMETER  :: ZERO=0.0_dp, ONE=1.0_dp

      IF (Alpha .EQ. ONE) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,5)
      IF( M .NE. 0 ) THEN
        IF (Alpha .EQ. (-ONE)) THEN
          DO i = 1,M
            X(i) = -X(i)
          END DO
        ELSEIF (Alpha .EQ. ZERO) THEN
          DO i = 1,M
            X(i) = ZERO
          END DO
        ELSE
          DO i = 1,M
            X(i) = Alpha*X(i)
          END DO
        END IF
        IF( N .LT. 5 ) RETURN
      END IF
      MP1 = M + 1
      IF (Alpha .EQ. (-ONE)) THEN
        DO i = MP1,N,5
          X(i)     = -X(i)
          X(i + 1) = -X(i + 1)
          X(i + 2) = -X(i + 2)
          X(i + 3) = -X(i + 3)
          X(i + 4) = -X(i + 4)
        END DO
      ELSEIF (Alpha .EQ. ZERO) THEN
        DO i = MP1,N,5
          X(i)     = ZERO
          X(i + 1) = ZERO
          X(i + 2) = ZERO
          X(i + 3) = ZERO
          X(i + 4) = ZERO
        END DO
      ELSE
        DO i = MP1,N,5
          X(i)     = Alpha*X(i)
          X(i + 1) = Alpha*X(i + 1)
          X(i + 2) = Alpha*X(i + 2)
          X(i + 3) = Alpha*X(i + 3)
          X(i + 4) = Alpha*X(i + 4)
        END DO
      END IF

      END SUBROUTINE WSCAL

!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WLAMCH( C )
!--------------------------------------------------------------
!     returns epsilon machine
!     after LAPACK
!     replace this by the function from the optimized LAPACK implementation:
!          CALL SLAMCH('E') or CALL DLAMCH('E')
!--------------------------------------------------------------
!      USE gckpp_Precision

      CHARACTER ::  C
      INTEGER    :: i
      REAL(kind=dp), SAVE  ::  Eps
      REAL(kind=dp)  ::  Suma
      REAL(kind=dp), PARAMETER  ::  ONE=1.0_dp, HALF=0.5_dp
      LOGICAL, SAVE   ::  First=.TRUE.
      
!$OMP THREADPRIVATE( Eps, First )

      IF (First) THEN
        First = .FALSE.
        Eps = HALF**(16)
        DO i = 17, 80
          Eps = Eps*HALF
          CALL WLAMCH_ADD(ONE,Eps,Suma)
          IF (Suma.LE.ONE) GOTO 10
        END DO
        PRINT*,'ERROR IN WLAMCH. EPS < ',Eps
        RETURN
10      Eps = Eps*2
        i = i-1      
      END IF

      WLAMCH = Eps

      END FUNCTION WLAMCH
     
      SUBROUTINE WLAMCH_ADD( A, B, Suma )
!      USE gckpp_Precision
      
      REAL(kind=dp) A, B, Suma
      Suma = A + B

      END SUBROUTINE WLAMCH_ADD
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE SET2ZERO(N,Y)
!--------------------------------------------------------------
!     copies zeros into the vector y:  y <- 0
!     after BLAS
!--------------------------------------------------------------
      
      INTEGER ::  i,M,MP1,N
      REAL(kind=dp) ::  Y(N)
      REAL(kind=dp), PARAMETER :: ZERO = 0.0d0

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = ZERO
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i)     = ZERO
        Y(i + 1) = ZERO
        Y(i + 2) = ZERO
        Y(i + 3) = ZERO
        Y(i + 4) = ZERO
        Y(i + 5) = ZERO
        Y(i + 6) = ZERO
        Y(i + 7) = ZERO
      END DO

      END SUBROUTINE SET2ZERO


!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WDOT (N, DX, incX, DY, incY) 
!--------------------------------------------------------------
!     dot produce: wdot = x(1:N)*y(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SDOT(N,X,1,Y,1) or  CALL DDOT(N,X,1,Y,1)
!--------------------------------------------------------------
!      USE messy_mecca_kpp_Precision
!--------------------------------------------------------------
      IMPLICIT NONE
      INTEGER :: N, incX, incY
      REAL(kind=dp) :: DX(N), DY(N) 

      INTEGER :: i, IX, IY, M, MP1, NS
                                 
      WDOT = 0.0D0 
      IF (N .LE. 0) RETURN 
      IF (incX .EQ. incY) IF (incX-1) 5,20,60 
!                                                                       
!     Code for unequal or nonpositive increments.                       
!                                                                       
    5 IX = 1 
      IY = 1 
      IF (incX .LT. 0) IX = (-N+1)*incX + 1 
      IF (incY .LT. 0) IY = (-N+1)*incY + 1 
      DO i = 1,N 
        WDOT = WDOT + DX(IX)*DY(IY) 
        IX = IX + incX 
        IY = IY + incY 
      END DO 
      RETURN 
!                                                                       
!     Code for both increments equal to 1.                              
!                                                                       
!     Clean-up loop so remaining vector length is a multiple of 5.      
!                                                                       
   20 M = MOD(N,5) 
      IF (M .EQ. 0) GO TO 40 
      DO i = 1,M 
         WDOT = WDOT + DX(i)*DY(i) 
      END DO 
      IF (N .LT. 5) RETURN 
   40 MP1 = M + 1 
      DO i = MP1,N,5 
          WDOT = WDOT + DX(i)*DY(i) + DX(i+1)*DY(i+1) + DX(i+2)*DY(i+2) +  &
                   DX(i+3)*DY(i+3) + DX(i+4)*DY(i+4)                   
      END DO 
      RETURN 
!                                                                       
!     Code for equal, positive, non-unit increments.                    
!                                                                       
   60 NS = N*incX 
      DO i = 1,NS,incX 
        WDOT = WDOT + DX(i)*DY(i) 
      END DO 

      END FUNCTION WDOT                                          


!--------------------------------------------------------------
      SUBROUTINE WADD(N,X,Y,Z)
!--------------------------------------------------------------
!     adds two vectors: z <- x + y
!     BLAS - like
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER :: i, M, MP1, N
      REAL(kind=dp) :: X(N),Y(N),Z(N)

      IF (N.LE.0) RETURN

      M = MOD(N,5)
      IF( M /= 0 ) THEN
         DO i = 1,M
            Z(i) = X(i) + Y(i)
         END DO
         IF( N < 5 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,5
         Z(i)     = X(i)     + Y(i)
         Z(i + 1) = X(i + 1) + Y(i + 1)
         Z(i + 2) = X(i + 2) + Y(i + 2)
         Z(i + 3) = X(i + 3) + Y(i + 3)
         Z(i + 4) = X(i + 4) + Y(i + 4)
      END DO

      END SUBROUTINE WADD
      
      
      
!--------------------------------------------------------------
      SUBROUTINE WGEFA(N,A,Ipvt,info)
!--------------------------------------------------------------
!     WGEFA FACTORS THE MATRIX A (N,N) BY
!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!     LINPACK - LIKE 
!--------------------------------------------------------------
!
      INTEGER       :: N,Ipvt(N),info
      REAL(kind=dp) :: A(N,N)
      REAL(kind=dp) :: t, dmax, da
      INTEGER       :: j,k,l
      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0

      info = 0

size: IF (n > 1) THEN
      
col:  DO k = 1, n-1

!        find l = pivot index
!        l = idamax(n-k+1,A(k,k),1) + k - 1
         l = k; dmax = abs(A(k,k))
         DO j = k+1,n
            da = ABS(A(j,k))
            IF (da > dmax) THEN
              l = j; dmax = da
            END IF
         END DO
         Ipvt(k) = l

!        zero pivot implies this column already triangularized
         IF (ABS(A(l,k)) < TINY(ZERO)) THEN
            info = k
            return
         ELSE   
            IF (l /= k) THEN
               t = A(l,k); A(l,k) = A(k,k); A(k,k) = t
            END IF
            t = -ONE/A(k,k)
            CALL WSCAL(n-k,t,A(k+1,k),1)
            DO j = k+1, n
               t = A(l,j)
               IF (l /= k) THEN
                  A(l,j) = A(k,j); A(k,j) = t
               END IF
               CALL WAXPY(n-k,t,A(k+1,k),1,A(k+1,j),1)
            END DO         
         END IF
         
       END DO col
       
      END IF size
      
      Ipvt(N) = N
      IF (ABS(A(N,N)) == ZERO) info = N
      
      END SUBROUTINE WGEFA


!--------------------------------------------------------------
      SUBROUTINE WGESL(Trans,N,A,Ipvt,b)
!--------------------------------------------------------------
!     WGESL solves the system
!     a * x = b  or  trans(a) * x = b
!     using the factors computed by WGEFA.
!
!     Trans      = 'N'   to solve  A*x = b ,
!                = 'T'   to solve  transpose(A)*x = b
!     LINPACK - LIKE 
!--------------------------------------------------------------

      INTEGER       :: N,Ipvt(N)
      CHARACTER     :: trans
      REAL(kind=dp) :: A(N,N),b(N)
      REAL(kind=dp) :: t
      INTEGER       :: k,kb,l

      
      SELECT CASE (Trans)

      CASE ('n','N')  !  Solve  A * x = b

!        first solve  L*y = b
         IF (n >= 2) THEN
          DO k = 1, n-1
            l = Ipvt(k)
            t = b(l)
            IF (l /= k) THEN
               b(l) = b(k)
               b(k) = t
            END IF
            CALL WAXPY(n-k,t,a(k+1,k),1,b(k+1),1)
          END DO
         END IF
!        now solve  U*x = y
         DO kb = 1, n
            k = n + 1 - kb
            b(k) = b(k)/a(k,k)
            t = -b(k)
            CALL WAXPY(k-1,t,a(1,k),1,b(1),1)
         END DO
      
      CASE ('t','T')  !  Solve transpose(A) * x = b

!        first solve  trans(U)*y = b
         DO k = 1, n
            t = WDOT(k-1,a(1,k),1,b(1),1)
            b(k) = (b(k) - t)/a(k,k)
         END DO
!        now solve trans(L)*x = y
         IF (n >= 2) THEN
         DO kb = 1, n-1
            k = n - kb
            b(k) = b(k) + WDOT(n-k,a(k+1,k),1,b(k+1),1)
            l = Ipvt(k)
            IF (l /= k) THEN
               t = b(l); b(l) = b(k); b(k) = t
            END IF
         END DO
         END IF
   
      END SELECT

      END SUBROUTINE WGESL
! End of BLAS_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



END MODULE gckpp_LinearAlgebra

