module gc_chem_utils

  use gc_type2_mod,  only : chemstate
  use physics_types, only : physics_state
  use shr_kind_mod, only: r8 => shr_kind_r8

  implicit none
  private
  public :: bcc_pass_to_gc, gc_pass_to_bcc, get_pbllev

  contains

! AT THE MOMENT, THE ROUTINES BELOW WRITE SOME OF THE 
! VALUES NECESSARY TO RUN GEOS-Chem DIRECTLY TO THE
! GEOS-Chem ARRAYS ASSOCIATED VIA MODULES
! THIS WILL NEED TO BE CHANGED.

  subroutine bcc_pass_to_gc(gc_state)

    USE DAO_MOD, ONLY : OPTDEP

    implicit none

    type(chemstate),     intent(out):: gc_state

      ! Pass list:
      ! Specific humidity
      ! PBL Height (PBLH)
  end subroutine bcc_pass_to_gc

  subroutine gc_pass_to_bcc(gc_state)
    
    implicit none
    
!    type(physics_state), intent(out) :: bcc_state
    type(chemstate)    , intent(in)  :: gc_state
  end subroutine gc_pass_to_bcc

  SUBROUTINE get_pbllev(pblht,zmid,ncol,pbllev)
    USE ppgrid

    integer,  intent(in)   :: ncol ! Columns per chunk.                                                                                    

    ! Calculate pbllev_cam across ncol                                                                                                     
    real(r8), intent(in)   :: pblht(pcols)
    real(r8), intent(in)   :: zmid(pcols, pver)
    real(r8), intent(out)  :: pbllev(pcols)
    integer                :: i, z

    pbllev = 0

    DO i=1, ncol
       DO z=1, pver-1
          IF (pblht(i) <= zmid(i,pver-(z))) THEN
             IF (pblht(i) > zmid(i,pver-(z-1))) THEN
                pbllev(i) = z
             ENDIF
          ENDIF
       ENDDO
    ENDDO
  END SUBROUTINE get_pbllev

end module gc_chem_utils
