module gc_chemdr

  use shr_kind_mod, only: r8 => shr_kind_r8
  USE CONSTITUENTS
      
  implicit none

  private

  public :: do_gc_chem, molar_mass

  real(r8), save       :: molar_mass(ppcnst)      

  contains

    subroutine do_gc_chem(mmr, spechum , t_in      ,                      &
                    pmid_in     , pdel_in    , pint_in,                   &
                    albedo_in   , cltot_in, landfrac, pblht         ,     &
                    precl_in , precc_in, ptrop_in  , trefht         ,     &
                    sst_in   , u_10m   , v_10m     , ustar_in       ,     &
                    z0_in    , cld     , mc        , du2            ,     &
                    cicewp   , cliqwp  , relhum    , shflx          ,     &
                    rho      , coslat  , tau       ,                      &
                    sols     , solsd   , fsds      , uv_albedo      ,     &
                    dz       , tend_ciw, tend_clw  , tend_w         ,     &
                    moistq_in, ts_in   , gwettop_in,          &
                    dlat, dlon, lat, lon, ncol, zmid, lchnk)

      use ppgrid,          only: pcols, pver
      use pmgrid,          only: masterproc
      use physics_types,   only: physics_state, physics_tend, physics_ptend
      use gc_type_mod,     only: gc_met_local
      use gc_type2_mod,    only: chemstate, EXT_STRATOH, EXT_SJVALUE,     &
                                 EXT_COPROD, EXT_COLOSS, GET_TRACER_ID
      use gc_chem_utils
      use chemistry_mod,   only: do_chemistry
      use chemistry,       only: ncnst, ixchm
      use dao_mod,         only: airqnt, sphu
      use gc_interface_mod,only: gc_met, gc_state
      use PBL_MIX_MOD,     only: PBL_TOP_L, PBL_TOP_m, INIT_PBL_MIX

! TEMPORARY USE
      USE GRID_MOD,     ONLY : AREA_M2, YEDGE, XEDGE, YMID, XMID
      USE CMN_SIZE_MOD, ONLY : DJSIZE, DISIZE, LLPAR, IIPAR, JJPAR
      ! 3-D
      USE DAO_MOD,      ONLY : TO3, OPTD, CLDFRC, AIRVOL, BXHEIGHT, &
                               CLDF, CMFMC, DQIDTMST, DQLDTMST,     &
                               DQVDTMST, DTRAIN, MOISTQ, OPTDEP,    &
                               DELP, RH, SPHU, T, TAUCLI, TAUCLW
      ! 2-D
      USE DAO_MOD,      ONLY : ALBD, CLDFRC, GWETTOP, HFLUX, LWI,   &
                               PARDR, PARDF, PRECON, PREACC,        &
                               RADSWG, TSKIN, SUNCOS, TROPP, TS,    &
                               U10M, V10M, Z0, USTAR, AD
      USE CMN_DEP_MOD,  ONLY : FRCLND
      USE UVALBEDO_MOD, ONLY : UVALBEDO
      USE COMODE_LOOP_MOD
      USE COMODE_MOD,   ONLY : AIRDENS
      USE TRACER_MOD,   ONLY : TCVV, TRACER_NAME, N_TRACERS
      USE TROPOPAUSE_MOD, ONLY: LMIN, LMAX ! THIS WILL BE AN ISSUE B/C OF INVERTED PRESSURE LEVELS
      
      USE PRESSURE_MOD, ONLY : EXTERNAL_PEDGE

      use time_manager, only: get_nstep

      implicit none
      
#     include "define.h"

      !----------------------------------------
      ! Link data fields from BCC to GEOS-Chem
      ! These are calculated online
      !----------------------------------------
      ! Tracer mass mixing-ratio array
      real(r8), intent(inout) :: mmr(ncol,pver,ncnst)
      ! Albedo
      real(r8), intent(in) :: albedo_in(pcols)  ! albedo
      ! Column-integrated cloud fraction
      real(r8), intent(in) :: cltot_in(pcols)
      ! specific humidity
      real(r8), intent(in) :: spechum(pcols,pver)
      ! Land fraction
      real(r8), intent(in) :: landfrac(pcols)
      ! Land-Water-Ice Flags
      
      ! PBL Height (m)
      real(r8), intent(in) :: pblht(pcols)
      ! Convective Precip
      real(r8), intent(in) :: precc_in(pcols)
      ! LS Precip
      real(r8), intent(in) :: precl_in(pcols)
      ! AN Precip
!!!!      real(r8), intent(in) :: ptrop(:) ! Is this correct?
      ! Tropopause pressure
      real(r8), intent(in) :: ptrop_in(pcols)
      ! Surface Temperature (2-meter air temperature??)
      real(r8), intent(in) :: trefht(pcols)
      ! Radiation @ Ground (net, long or shortwave?)
      real(r8), intent(in) :: fsds(pcols)
      ! Sea-surface Temperature
      real(r8), intent(in) :: sst_in(pcols)
      ! 10-meter meridional wind-speed
      real(r8), intent(in) :: u_10m(pcols)
      ! 10-meter zonal wind-speed
      real(r8), intent(in) :: v_10m(pcols)
      ! Friction velocity (u*)
      real(r8), intent(in) :: ustar_in(pcols)
      ! Roughness height
      real(r8), intent(in) :: z0_in(pcols)
      ! Cloud Fraction
      real(r8), intent(in) :: cld(pcols,pver)
      ! Convective (cloud) Mass Flux
      real(r8), intent(in) :: mc(pcols,pver)
      ! Detrainment Flux (convective: shallow & deep?)
      real(r8), intent(in) :: du2(pcols,pver)
      ! Visible (Ice) Optical Depth
      real(r8), intent(in) :: cicewp(pcols,pver)
      ! Visible (Liquid Water) Optical Depth
      real(r8), intent(in) :: cliqwp(pcols,pver)
      ! Relative Humidity
      real(r8), intent(in) :: relhum(pcols,pver)
      ! Sensible Heat Flux
      real(r8), intent(in) :: shflx(pcols)
      ! Air Temperature
      real(r8), intent(in) :: t_in(pcols,pver)
      ! Air density
      real(r8), intent(in) :: rho(pcols,pver)
      ! Cosine of Solar zenith-angle
      real(r8), intent(in) :: coslat(pcols)
      ! Moist (q) tendency
      real(r8), intent(in) :: moistq_in(pcols,pver)
      ! Model layer pressure-thickness (pdel)
      real(r8), intent(in) :: pdel_in(pcols,pver)
      ! Air pressure (gridbox edges)
      real(r8), intent(in) :: pint_in(pcols,pver+1)
      ! Mid-level perssures
      real(r8), intent(in) :: pmid_in(pcols,pver)
      ! Total (column integrated) overhead ozone column
!      real(r8), intent(in) :: 
      ! Grid-box area
!      real(r8), intent(in) :: area_m2(:,:)
      ! Total optical depth
      real(r8), intent(in) :: tau(pcols,pver)
      ! Direct PAR (this may not be the correct value)
      real(r8), intent(in) :: sols(pcols)
      ! Diffuse PAR
      real(r8), intent(in) :: solsd(pcols)
      ! UV Albedo
      real(r8), intent(in) :: uv_albedo(pcols)
      real(r8), intent(in) :: dz(pcols,pver)
      real(r8), intent(in) :: tend_ciw(pcols,pver)
      real(r8), intent(in) :: tend_clw(pcols,pver)
      real(r8), intent(in) :: tend_w(pcols,pver)
      real(r8), intent(in) :: ts_in(pcols)
      real(r8), intent(in) :: lat(pcols), lon(pcols)
      real(r8), intent(in) :: gwettop_in(pcols)

      real(r8), intent(in) :: zmid(pcols,pver)
      real(r8), intent(in) :: dlat, dlon

      real(r8)             :: tmp_mmr(ncol,pver,ncnst)
      
      real(r8) :: mmr2mcc(pcols,pver)
      real(r8) :: mcc2mmr(pcols,pver)

      integer,  intent(in) :: ncol, lchnk ! ID & Number of columns in chunk
      INTEGER               :: ELAPSED_TODAY, HOUR, MINUTE, IDX, i

      LOGICAL               :: LPRINT_DIAG = .FALSE.

      ! TEMPORARY ARRAYS - DECLARED AND INITIALIZED HERE
      ! TO FACILITATE TEST RUNS. THEIR VALUES WILL NEED TO 
      ! BE PROPERLY DEALT WITH!


      !-------------------------------------------------
      ! The following fields need to be read from file.
      ! The will need to be distributed with a
      ! "scatter_field_to_chunk" routine call.
      !-------------------------------------------------
      ! IREG - Number of Olsen Land Types
      ! ILAND (1-IREG) Olsen Land type index
      ! IUSE  (1-IREG) Fraction of Olsen land type in box
      ! LAI (1-IREG) Leaf-area index for land type
      ! Stratospheric OH
      ! Stratospheric PAN J-values
      ! Stratospheric H2O2 J-values
      ! Stratosephric Acetone J-values
      ! Stratospheric Ketones J-values
      ! Strat. Acetaldehyde J-values
      ! Strat. Aldehyde(>C3) J-values
      ! Strat. Methylvinylketone J-values
      ! Strat. Methacrolein J-values
      ! Strat. Alkylnitrate(>C3) J-values
      ! Strat. HCHO J-values
      ! Strat. N2O5 J-values
      ! Strat. HNO4 J-values
      ! Strat. CH3OOH J-values
      ! Strat. CO production rates
      ! Strat. CO loss rates

! --- UNIT CONVERSION FACTORS ----
! mmr -> mcl/cm3
   mmr2mcc(:ncol,:) = 6.022d23*0.1203d0*(pmid_in(:ncol,:)/t_in(:ncol,:)) &
                      *1.d-6 * 28.98d0
! mcl//cm3 --> mmr
   mcc2mmr(:ncol,:) = (1.d0/6.022d23)*8.314d0*(t_in(:ncol,:)/pmid_in(:ncol,:)) &
                      *1.d6 * 0.03451d0

!      call bcc_pass_to_gc(gc_met, gc_state)

      DISIZE = dlon*(360.d0/(2.*3.1416)) ! Radians to degrees
      DJSIZE = dlat*(360.d0/(2.*3.1416)) ! Radians to degrees

      ! The following are short-term fixes
!      YEDGE(1:pcols) = LAT
!      YEDGE(pcols+1) = LAT(1)
!      XEDGE(1)       = LON(1)
!      XEDGE(2)       = LON(2)

!      YMID(:pcols)   = LAT
!      XMID(1)        = LON(1)

!      RRATE = 0.d0
!      TRATE = 0.d0


!X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X
!X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X
!X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X
      !THIS IS BAD!!!
      !BUT WE DON"T WANT GEOS-CHEM DOING CHEMISTRY
      ! OVER 1:PCOLS, WHEN ONLY 1:NCOL ARE DEFINED
      ! THIS MAY BE CRASHING THE HISTROY WRITE ROUTINES
      JJPAR = ncol
!X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X
!X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X
!X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X

      ! Arbitrary values passed to SCHEM (Stratpspheric chemistry)
      ! They are generally read from files. This should be done.
      EXT_STRATOH = 1.e-13
      EXT_SJVALUE = 1.e-13
      EXT_COPROD  = 1.e-15
      EXT_COLOSS  = 0.9e-15

      LMIN = 25

      LMAX = 2

      ! Set 2-D Vars
      TO3 = 1.e12  ! Total column O3 burden. Simply a guess. Should be in dobson units
      ALBD(1,:)    = albedo_in   ! Visible surface albedo [unitless]
!      AREA_M2(:)   = AREA_M2     ! Grid box surface area [cm2]
      CLDFRC(1,:)  = cltot_in    ! Column cloud fraction [unitless]
      FRCLND(1,:)  = landfrac    ! Olson land fraction [unitless]
      GWETTOP(1,:) = gwettop_in  ! Top soil moisture [unitless]
      HFLUX(1,:)   = shflx       ! Sensible heat flux [W/m2]
      LWI          = 1           ! Land/water indices [unitless]
      PARDR(1,:)   = sols        ! Direct  photsyn active rad [W/m2]
      PARDF(1,:)   = solsd       ! Diffuse photsyn active rad [W/m2]
      PBL_TOP_m(1,:)=pblht       ! PBL height [m]
      PRECON(1,:)  = precc_in    ! Conv  precip @ ground [kg/m2/s]
      PREACC(1,:)  = precl_in+precc_in    ! Total precip @ ground [kg/m2/s]
      RADSWG(1,:)  = shflx       ! Solar radiation @ ground [W/m2]
      TSKIN(1,:)   = TS_in       ! Sea surface temperature [K]
      SUNCOS(1.:)  = COSLAT      ! Cosine of solar zenith angle
      TROPP(1,:)   = PTROP_in    ! Tropopause pressure [hPa]
      TS(1,:)      = TREFHT      ! Surface temperature [K]
      U10M(1,:)    = u_10m       ! E/W wind speed @ 10m height [m/s]
      USTAR(1,:)   = USTAR_in    ! Friction velocity [m/s]
      UVALBEDO(1,:) = uv_albedo  ! UV surface albedo [unitless]
      V10M(1,:)    = v_10m       ! N/S wind speed @ 10m height [m/s]
      Z0(1,:)      = z0_in       ! Roughness height

! SET 3-D Vars      
      AD = 1.e0        ! Air mass [kg]
      AIRDENS = reshape(rho,(/pver*pcols/)) ! Air density [kg/m3]

      ! Taper up the OH concentration for stiffness' sake.
      
      i = get_nstep()

!      if (i .lt. 72) then
!         call cnst_get_ind('OH',idx)
!         mmr(:ncol,:,idx) = mmr(:ncol,:,idx)/(7200-(100*i))
!      endif


      DO i=1,N_TRACERS
         call cnst_get_ind(trim(TRACER_NAME(i)),idx)
         idx = idx-ixchm+1
         if (idx .gt. 0) then
            GC_STATE%TRACERS(1,:ncol,:,i) = mmr(:ncol,pver:1:-1,idx)
         endif
         idx = -1
      ENDDO

!      call cnst_get_ind('OH',idx)
!      idx = idx-ixchm+1
!      write(*,*) 'PRE TEST OH: ', sum(mmr(:ncol,:,idx))
!      write(*,*) 'PRE TEST OH: ', sum(GC_STATE%TRACERS(:,:ncol,:,get_tracer_id('OH')))

!XXXXXXXXXXXXXXXXXXX
      AIRVOL   = 100.d0 ! Grid box volume [m3]
      BXHEIGHT(1,:,:) = dz(:,pver:1:-1)  ! Grid box height [m]
!XXXXXXXXXXXXXXXXXXX

      CLDF(:pver,1,:)    = reshape(cld, (/pver,pcols/))! 3-D cloud fraction [unitless]
      CMFMC(1,:,:pver)   = mc(:,pver:1:-1)     ! Cloud mass flux [kg/m2/s]

!XXXXXXXXXXXXXXXXXXX
      DQIDTMST = 0.d0 ! Ice tendency, mst proc [kg/kg/s]
      DQLDTMST = 0.d0 ! H2O tendency, mst proc [kg/kg/s]
      DQVDTMST = 0.d0 ! Vapor tendency, mst proc [kg/kg/s]
!XXXXXXXXXXXXXXXXXXX

      DTRAIN(1,:,:pver)  = du2(:,pver:1:-1)   ! Detrainment flux [kg/m2/s]

!XXXXXXXXXXXXXXXXXXX
      MOISTQ(:pver,1,:)  = reshape(moistq_in,(/pver,pcols/)) ! Tendency in sp. humidity [kg/kg/s]
      OPTD(:pver,1,:)    = reshape(tau      ,(/pver,pcols/)) ! Visible optical depth [unitless]
!XXXXXXXXXXXXXXXXXXX

      GC_MET%PEDGE(1,:,:pver)   = pint_in(:ncol,pver:1:-1)    ! Pressure @ level edges [Pa]
      EXTERNAL_PEDGE(1,:,:pver+1)   = pint_in(:ncol,pver:1:-1)    ! Pressure @ level edges [Pa]
      GC_MET%PMID(1,:,:pver)    = pmid_in(:ncol,pver:1:-1)      ! Pressure @ level centers [Pa]
      DELP(pver:1:-1,1,:)    = reshape(pdel_in  ,(/pver,pcols/))      !
      RH(1,:,:pver)      = relhum(:ncol,pver:1:-1)        ! Relative humidity [unitless]
      SPHU(1,:,:pver)    = spechum(:ncol,pver:1:-1)      ! Specific humidity [kg/kg]
      T(1,:,:pver)       = t_in(:ncol,pver:1:-1)         ! Temperature [K]
      TAUCLI(1,:,:pver)  = cicewp(:ncol,pver:1:-1)  ! Opt depth of ice clouds [unitless]
      TAUCLW(1,:,:pver)  = cliqwp(:ncol,pver:1:-1)  ! Opt depth of H2O clouds [unitless]

      ! Calculate PBL Level index

      call get_pbllev(pblht,zmid,ncol,PBL_TOP_L(1,1:pcols))

      SPHU(1,:,:) = SPECHUM(:ncol,pver:1:-1)
      
      tmp_mmr = mmr
      
      !Call Chemistry
      call do_chemistry(masterproc,  ncol, 1, pver, GC_STATE, GC_MET)

      !Reinterface with GCM: From GEOS-Chem to BCC
!      call gc_pass_to_bcc(gc_state) ! Stub as of Sept-12 2012. MSL


      DO i=1,N_TRACERS
         call cnst_get_ind(trim(TRACER_NAME(i)),idx)
         idx = idx-ixchm+1
         if (idx .gt. 0) then
            mmr(:ncol,:,idx) = GC_STATE%TRACERS(1,:ncol,pver:1:-1,i)
         endif
         idx = -1
      ENDDO

      IF ( LPRINT_DIAG ) THEN
         call cnst_get_ind('OH',idx)
         idx=idx-ixchm+1
         write(*,*) 'OH SUM : ', sum(mmr(:ncol,:,idx)), sum(mmr(:ncol,:,idx))-sum(tmp_mmr(:ncol,:,idx)) 
         call cnst_get_ind('CO2',idx)
         idx=idx-ixchm+1
         write(*,*) 'CO2 SUM : ', sum(mmr(:ncol,:,idx)), sum(mmr(:ncol,:,idx))-sum(tmp_mmr(:ncol,:,idx))
         call cnst_get_ind('CO',idx)
         idx=idx-ixchm+1
         write(*,*) 'CO SUM : ', sum(mmr(:ncol,:,idx)), sum(mmr(:ncol,:,idx))-sum(tmp_mmr(:ncol,:,idx))
         call cnst_get_ind('CH4',idx)
         idx=idx-ixchm+1
         write(*,*) 'CH4 SUM : ', sum(mmr(:ncol,:,idx)), sum(mmr(:ncol,:,idx))-sum(tmp_mmr(:ncol,:,idx))
         call cnst_get_ind('CH2O',idx)
         idx=idx-ixchm+1
         write(*,*) 'CH2O SUM : ', sum(mmr(:ncol,:,idx)), sum(mmr(:ncol,:,idx))-sum(tmp_mmr(:ncol,:,idx))

!         call cnst_get_ind('OH',idx)
!         idx=idx-ixchm+1
!         write(*,*) 'OH SUM : ',  (mmr(1,26,idx)), (mmr(1,26,idx))-(tmp_mmr(1,26,idx)) 
!         call cnst_get_ind('CO2',idx)
!         idx=idx-ixchm+1
!         write(*,*) 'CO2 SUM : ', (mmr(1,26,idx)), (mmr(1,26,idx))-(tmp_mmr(1,26,idx)) 
!         call cnst_get_ind('CO',idx)
!         idx=idx-ixchm+1
!         write(*,*) 'CO SUM : ',  (mmr(1,26,idx)), (mmr(1,26,idx))-(tmp_mmr(1,26,idx)) 
!         call cnst_get_ind('CH4',idx)
!         idx=idx-ixchm+1
!         write(*,*) 'CH4 SUM : ',   (mmr(1,26,idx)), (mmr(1,26,idx))-(tmp_mmr(1,26,idx)) 
!         call cnst_get_ind('CH2O',idx)
!         idx=idx-ixchm+1
!         write(*,*) 'CH2O SUM : ',(mmr(1,26,idx)), (mmr(1,26,idx))-(tmp_mmr(1,26,idx)) 
         
!         call cnst_get_ind('CO2',idx)
!         idx=idx-ixchm+1
!         write(*,*) 'CO2 SUM : ', sum(mmr(:ncol,:,idx)), sum(mmr(:ncol,:,idx))-sum(tmp_mmr(:ncol,:,idx))
!         write(*,*) 'PPCNST, NCNST : ', ppcnst, ncnst, ixchm
      ENDIF

      return
    end subroutine do_gc_chem

end module gc_chemdr
