
      module gc_emis_edgar
!---------------------------------------------------------------
! 	... Interpolation surface emissions
!---------------------------------------------------------------

      use pmgrid,         only: plon, plat, masterproc
      use gc_emis_utils,  only: gc_read_emis

      implicit none

      private
      public  :: gc_edgar_emis_inti, coembdy

      character(len=256), public  :: bndedgarco      ! full pathname for EDGAR CO

      save

      real, allocatable, dimension(:,:,:) ::  coembdy

      real    :: factor
      integer :: co_ndx
      logical :: co_em

      contains

      subroutine gc_edgar_emis_inti( plonl, platl, pplon )
!-----------------------------------------------------------------------
! 	... Read the primary emission flux datasets for GEOS-Chem
!       Adopted from the BCC-AGCM MOZART2 surface emissions routines.
!       MSL 2012.02.08 
!-----------------------------------------------------------------------

      use mo_chem_utls,     only : get_spc_ndx, has_srfems
      use mo_chem_mods,     only : adv_mass

!add by zf 2008.03.13
      use ppgrid,         only:  pcols, begchunk, endchunk   !2008.08.11
      use phys_grid,      only:  scatter_field_to_chunk      !2008.08.11
      use ioFileMod,      only: getfil
      use abortutils,     only: endrun
!end zf
      implicit none

!-----------------------------------------------------------------------
! 	... dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) :: plonl
      integer, intent(in) :: platl
      integer, intent(in) :: pplon

!-----------------------------------------------------------------------
! 	... Local variables
!-----------------------------------------------------------------------
      real, allocatable, dimension(:,:,:) :: coem

      real, parameter :: amufac = 1.65979e-23         ! 1.e4* kg / amu
      integer :: astat
      integer :: i, j, ncid                              ! Indices
      real    :: mw
      character(len=10) :: species
      character(len=256) :: locfn    ! netcdf local filename to open   add by zf 2008.03.13

!-----------------------------------------------------------------------
! 	... Setup the indicies
!-----------------------------------------------------------------------
!      co_ndx       = get_spc_ndx( 'CO' )
       co_ndx = 1

!      co_em       = has_srfems( 'CO' )
      co_em = .true.

      if (masterproc) then
        write(*,*) 'gc_edgar_emis_inti : diagnostic'
        write(*,'(10i5)') co_ndx
        write(*,*)  co_em
      endif

!-----------------------------------------------------------------------
! 	... allocate working arrays
!-----------------------------------------------------------------------
      allocate( coem(plonl,platl,12), stat=astat )
      if( astat/= 0 ) then
	 write(*,*) 'gc_edgar_emis_inti: failed to allocate wrking arrays; error = ',astat
	 call endrun
      end if

!-----------------------------------------------------------------------
! 	... Diagnostics setup
!-----------------------------------------------------------------------

      factor = 1.e-6 ! Kg -> Tg

      if (masterproc) then

!-----------------------------------------------------------------------
! 	... CO
!-----------------------------------------------------------------------
      if( co_em ) then
         species  = 'CO'
!mozart2         mw       = adv_mass(co_ndx)               ! g / mole
         mw = 0.0e0
         call getfil (bndedgarco, locfn)
         call wrap_open(locfn,0,ncid)
         call gc_read_emis( plonl, platl, pplon, species, mw, 'Kg/yr', ncid, coem )
      end if

!-----------------------------------------------------------------------

      endif

      allocate( coembdy(pcols,begchunk:endchunk,12), stat=astat )
      if( astat/= 0 ) then
	 write(*,*) 'gc_edgar_emis_inti: failed to allocate wrking arrays; error = ',astat
	 call endrun
      end if

      call scatter_field_to_chunk(1,1,12,plonl,coem,coembdy)   ! coembdy will be used in srf_emis 

      end subroutine gc_edgar_emis_inti

!---------------------------------------------------
!      subroutine srf_emis( lchnk, phir, calday, sflx, oro, &
!                           local_angle, polar_night, polar_day, sunon, sunoff, plonl )
!      real, intent(inout) ::   sflx(plonl,pcnstm1)   ! surface emissions ( kg/m^2/s )
!
!      srf_emiss will be called by subroutine sflxdr in gc_surface.F90
!---------------------------------------------------

      end module gc_emis_edgar
