
      module gc_initrun_mod

      implicit none
      private
      public gc_initrun, gc_getopts, gc_init_dimensions

      save 

      contains

      subroutine gc_initrun(local_met, chem_state, pcols, pver)

      use shr_kind_mod,       only : r8 => shr_kind_r8
      use pmgrid,             only : masterproc, plon, plat, plev, plevp
      use gc_chemini
!      use ppgrid,             only : pcols, pver
      use CMN_SIZE_MOD,       only : IIPAR, JJPAR, LLTROP
      USE GC_ENVIRONMENT_MOD, ONLY : ALLOCATE_ALL, INIT_ALL
      USE GRID_MOD,           ONLY : AREA_M2
      USE TIME_MANAGER,       ONLY : GET_STEP_SIZE
      USE GC_TYPE_MOD,        ONLY : GC_MET_LOCAL
      USE GC_TYPE2_MOD,       ONLY : CHEMSTATE
      USE gc_gridarea
      USE dao_mod,            ONLY : INIT_DAO
      USE COMODE_MOD,         ONLY : INIT_COMODE
      USE PRESSURE_MOD,       ONLY : INIT_PRESSURE
      USE GCKPP_COMODE_MOD,   ONLY : INIT_GCKPP_COMODE
      USE LOGICAL_MOD,        ONLY : LKPP, LPRT, LEMIS, LCHEM
      USE TRACER_MOD,         ONLY : ITS_A_FULLCHEM_SIM, ITS_AN_AEROSOL_SIM, &
                                     ITS_A_CO2CH4CO_SIM
      use PBL_MIX_MOD,        ONLY : INIT_PBL_MIX
      use TOMS_MOD,           ONLY : INIT_TOMS

      USE COMODE_LOOP_MOD

      USE ABORTUTILS,         ONLY : ENDRUN

      implicit none

      integer             :: dtime ! Physics time step
      integer             :: dt_factor = 2 ! 2 x model time step?
      integer             :: k, RC
      integer, intent(in) :: pcols, pver
      TYPE(CHEMSTATE), INTENT(out)    :: CHEM_STATE
      TYPE(GC_MET_LOCAL), INTENT(out) :: LOCAL_MET

!---------------------------------------------------------------------

!      call gc_init_dimensions(pcols,pver)

      RC = 0

      call ndxx_setup

      dtime = dt_factor * get_step_size()

      call gc_init_timeinterface( dtime )

      call allocate_all

      call allocate_interface(pcols, pver)

!      call gc_getopts

      call gc_setenv

      call gc_calcarea(AREA_M2)

      call init_all(LOCAL_MET, CHEM_STATE)

      LOCAL_MET%AREA_M2 = AREA_M2
 
      call gc_cheminit( plon, plat )

      ! GEOS-Chem initialization routines
      CALL INIT_DAO

      ! Initialize the new hybrid pressure module.  Define Ap and Bp.
      CALL INIT_PRESSURE(masterproc)
      IF ( LPRT ) CALL ENDRUN( '### MAIN: a INIT_PRESSURE' )

      CALL INIT_PBL_MIX

      CALL INIT_TOMS

      ! Initialize allocatable SMVGEAR/KPP arrays
      IF ( LEMIS .or. LCHEM ) THEN
         IF ( ITS_A_FULLCHEM_SIM() ) CALL INIT_COMODE(masterproc)
         IF ( ITS_AN_AEROSOL_SIM() ) CALL INIT_COMODE(masterproc)
         IF ( ITS_A_CO2CH4CO_SIM() ) CALL INIT_COMODE(masterproc)
         IF ( LKPP )  CALL INIT_GCKPP_COMODE( masterproc, IIPAR, JJPAR, &
              LLTROP, ITLOOP, NMTRATE, IGAS, RC )
         IF ( RC == 1 ) &
              CALL ENDRUN( "Alloc error: INIT_GCKPP_COMODE" )
!         IF ( LPRT ) CALL DEBUG_MSG( '### MAIN: a INIT_COMODE' )
      ENDIF

      if (masterproc) write(*,*) 'gc_inirun: finished GC_CHEMINI'

!---------------------------------------------------------------------
!     	... Initialize deposition velocities
!--------------------------------------------------------------------- 

      end subroutine gc_initrun

      subroutine gc_getopts

        use pmgrid,      ONLY : masterproc
        use mpishorthand
        USE INPUT_MOD,   ONLY : READ_INPUT_FILE

!        IF (MASTERPROC) THEN
           CALL READ_INPUT_FILE(masterproc)
!        ENDIF

      end subroutine gc_getopts

      subroutine gc_init_timeinterface(dt)

        use shr_kind_mod, only: r8 => shr_kind_r8
        use julday_mod,   only : julday, caldate
        use time_manager, only : get_curr_date
        use time_mod
        use pmgrid,      ONLY : masterproc

        implicit none

        integer             :: NYMDb, NHMSb, Y, M, D, TOD
        integer, intent(in) :: dt
        integer             :: dtm
        real(r8)            :: frac_day

        ! Set GEOS-Chem timesteps from CAM timesteps
        dtm = dt/60 ! Timestep from Sec -> Min
        CALL SET_TIMESTEPS( masterproc, dtm, dtm, dtm, dtm, dtm, dtm)

        ! Set NYMDb & NHMSb

        CALL GET_CURR_DATE( Y, M, D, TOD )

        IF ( TOD .gt. 0 ) THEN
           FRAC_DAY = DBLE(D) + DBLE(TOD)/DBLE(86400)
        ELSE
           FRAC_DAY = DBLE(D)
        ENDIF

        CALL CALDATE( JULDAY( Y, M, FRAC_DAY ), NYMDb, NHMSb )

        ! Set BEGIN & CURRENT time values from CAM values

        CALL SET_BEGIN_TIME( NYMDb, NHMSb )

        CALL SET_CURRENT_TIME()

        ! Initialize and set timestep counts using
        ! GEOS-Chem's initialize (see GEOS-Chem's main.F)
        CALL INITIALIZE( 2, masterproc )
        CALL INITIALIZE( 3, masterproc )

      end subroutine gc_init_timeinterface

      subroutine gc_init_dimensions(pcols,pver)

        use CMN_SIZE_MOD, only: IIPAR,JJPAR,IGLOB,JGLOB, &
             LGLOB, LLPAR

        integer, intent(in) :: pcols,pver
        ! Set model dimensions

        IGLOB = 1
        JGLOB = pcols
        LGLOB = pver

        IIPAR = 1
        JJPAR = pcols
        LLPAR = pver

      end subroutine gc_init_dimensions

      subroutine allocate_interface(pcols, pver)

        USE CMN_SIZE_MOD, ONLY : IIPAR, JJPAR
        USE ERROR_MOD,    ONLY : ALLOC_ERR
        USE GRID_MOD,     ONLY : AREA_M2, INIT_GRID
        USE UVALBEDO_MOD, ONLY : UVALBEDO
        USE DAO_MOD,      ONLY : TO3
        
        IMPLICIT NONE

        integer, intent(in) :: pcols, pver
        integer             :: AS


!        CALL INIT_GRID(1,pcols,1,1,1,pver)
!        ALLOCATE( AREA_M2( JJPAR ), STAT=AS )
!        IF ( AS /= 0 ) CALL ALLOC_ERR( 'AREA_M2_G' )
!        AREA_M2 = 0d0
        
!        ALLOCATE( AREA_CM2( JJPAR ), STAT=AS )
!        IF ( AS /= 0 ) CALL ALLOC_ERR( 'AREA_CM2_G' )
!        AREA_CM2 = 0d0
         ! Allocate UVALBEDO
         ALLOCATE( UVALBEDO( IIPAR, JJPAR ), STAT=AS )
         IF ( AS /= 0 ) CALL ALLOC_ERR( 'UVALBEDO' )

         ! Zero UVALBEDO
         UVALBEDO(:,:) = 0d0

!         ALLOCATE( TO3( IIPAR, JJPAR ), STAT=AS )
!         IF ( AS /= 0 ) CALL ALLOC_ERR( 'TO3 in SETENV' )

      end  subroutine allocate_interface

      subroutine gc_setenv
      end subroutine gc_setenv

    end module gc_initrun_mod
