   module iceMixed
     use shr_kind_mod,  only:  r8 => shr_kind_r8
     use iceBNact,      only : CNT,PDA08
     implicit none
     private
     save
     public MixMeyer,MixCooper,MixCNT,MixPDA08    ! ! Public interfaces
   contains
   
     subroutine MixCooper(T,Nice)
       implicit none
       real(r8), intent(in)  :: T  ! SCI unit  k Pa  m/s
       real(r8), intent(out) :: Nice   !#/m3
       if (T.lt.268.15_r8) then
          Nice=0.005_r8*exp(0.304_r8*(273.15_r8-T))*1000._r8
       end if
       Nice=min(Nice,208.9e3_r8)
     end subroutine MixCooper
     
     subroutine MixMeyer(T,Nice)
       implicit none
       real(r8), intent(in)  :: T  ! SCI unit  k Pa  m/s
       real(r8), intent(out) :: Nice  ! #/m3
       real(r8) esl,esi,deles
       esl = polysvp(T,0)     ! over water in mixed clouds
       esi = polysvp(T,1)     ! over ice
       deles = (esl - esi)
       Nice=1.e3_r8*exp(12.96_r8*deles/esi - 0.639_r8) 
       Nice=min(Nice,208.9e3_r8) !!! sxj add maxmun value,  same as cooper
     end subroutine MixMeyer
     
     subroutine MixCNT(T,Ndust,Nbc,Nice)
       implicit none
       real(r8), intent(in)  :: T  ! SCI unit  k Pa  m/s
       real(r8), intent(in)  :: Ndust,Nbc!
       real(r8), intent(out) :: Nice  ! #/m3
       real(r8) esl,esi,Si
       esl = polysvp(T,0)     ! over water in mixed clouds
       esi = polysvp(T,1)     ! over ice
       Si = (esl - esi)/esi
       call CNT(Si,T,Ndust,Nbc,Nice)
     end subroutine MixCNT
     
     subroutine MixPDA08(T,Ndust,Nbc,Nice)
       implicit none
       real(r8), intent(in)  :: T  ! SCI unit  k Pa  m/s
       real(r8), intent(in)  :: Ndust,Nbc!
       real(r8), intent(out) :: Nice  ! #/m3
       real(r8) esl,esi,Si
       esl = polysvp(T,0)     ! over water in mixed clouds
       esi = polysvp(T,1)     ! over ice
       Si = (esl - esi)/esi
       call PDA08(Si,T,Ndust,Nbc,Nice)
     end subroutine MixPDA08
       
    
     function polysvp (T,type)
          implicit none
          real(r8) dum
          real(r8) T,polysvp
          integer type
          if (type.eq.1) then
             polysvp = 10._r8**(-9.09718_r8*(273.16_r8/t-1._r8)-3.56654_r8* &
             log10(273.16_r8/t)+0.876793_r8*(1._r8-t/273.16_r8)+ &
             log10(6.1071_r8))*100._r8
          end if
          if (type.eq.0) then
             polysvp = 10._r8**(-7.90298_r8*(373.16_r8/t-1._r8)+ &
                 5.02808_r8*log10(373.16_r8/t)- &
                 1.3816e-7_r8*(10._r8**(11.344_r8*(1._r8-t/373.16_r8))-1._r8)+ &
                 8.1328e-3_r8*(10._r8**(-3.49149_r8*(373.16_r8/t-1._r8))-1._r8)+ &
                 log10(1013.246_r8))*100._r8
          end if
     end function polysvp
       

   endmodule iceMixed

    


 
   

  
