module nenesact
      use shr_kind_mod,  only:  r8 => shr_kind_r8
      !useprescribed_aerosols,      only: naer_all, n_aer=>naer   !! naer_all=13   naer=11 (not include the last two)
      implicit none

      private


      !integer, parameter :: NSMX=n_aer, Npgauss=10
      integer, parameter :: NSMX=5, Npgauss=10

      real(r8), parameter :: AMW    = 18e-3_r8                 ! Water molecular weight
      real(r8), parameter :: AMA    = 29e-3_r8                 ! Air molecular weight
      real(r8), parameter :: GRAV   = 9.81e0_r8                ! g constant
      real(r8), parameter :: RGAS   = 8.31e0_r8                ! Universal gas constant
!     real(r8), parameter :: ACCOM  = 0.042_r8                 ! Default accommodation coef
      real(r8), parameter :: ACCOM  = 1.0_r8                   ! Default accommodation coef
      real(r8), parameter :: MAXIT  = 100._r8                  ! Max iterations for solution
      real(r8), parameter :: EPS    = 1e-6_r8                  ! Convergence criterion
      real(r8), parameter :: PI     = 3.1415927_r8             ! Some constants
      real(r8), parameter :: ZERO   = 0._r8
      real(r8), parameter :: GREAT  = 1e30_r8
      real(r8), parameter :: SQ2PI  = 2.5066282746_r8
      logical, parameter :: CCNSPST = .FALSE.           ! Internal consistency check


      public    :: NSMX,Npgauss  !sxj add NSMX Npgauss
      public    :: nenes 
      

contains

subroutine nenes(PPARC, TPARC, WPARC, SIGW, atpi, adpgi, asigi, adensi, adenii, aamfsi, aamsi, avhfi, anact, asmax, aa, mmmode)
!subroutine nenes(PPARC, TPARC, WPARC, SIGW, TPI, DPGI, SIGI, DENSI, DENII, AMFSI, AMSI, VHFI, anact, asmax)

      implicit none

        real(r8)  PPARC, TPARC, WPARC, SIGW   ! pressure, temperature, updraft velocity, velocity dispersion
        real(r8),dimension(1:NSMX):: atpi, adpgi, asigi, adensi, adenii, aamfsi, aamsi, avhfi
        integer  mmmode
        real(r8),dimension(1:NSMX):: anact,aa
        real(r8),dimension(1:NSMX):: TPI, DPGI, SIGI, DENSI, DENII, AMFSI, AMSI, VHFI
        real(r8),dimension(1:NSMX)::  NACT
	real(r8) SMAX,asmax

!xujun OpenMP
       integer :: i,NMD

       real(r8) :: TEMP, PRES,      &
                   DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,    &
                   AKOH, SSPLT, ALFA, BET1, BET2
       real(r8), dimension(1:NSMX) :: SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS
       real(r8), dimension(1:Npgauss) :: XGS, WGS

!       print *, "Entering Nenes scheme!!!"

!       if(atpi.le.1.e-39) return

	do i=1,NSMX
        TPI(i) = atpi(i)
        DPGI(i) = adpgi(i)
        SIGI(i) = asigi(i)
        DENSI(i) = adensi(i)
        DENII(i) =  adenii(i)
        AMFSI(i) = aamfsi(i)
        AMSI(i) = aamsi(i)
        VHFI(i) = avhfi(i)
	end do

!       print *, "TPI= ", TPI(1), "DPGI= ",DPGI(1), "SIGI= ",SIGI(1), "DENSI= ", DENSI(1),  &
!                "DENII= ", DENII(1), "AMFSI= ", AMFSI(1), "AMSI= ",  &
!                AMSI(1), "VHFI= ", VHFI(1), "PPARC=",PPARC, "TPARC= ", &
!                TPARC, "WPARC= ",WPARC, "SIGW= ", SIGW

!       TPI(1) = 6.4E+09
!       DPGI(1) = 1.5E-08
!       SIGI(1) = 1.7 
!       DENSI(1) = 1.76E+03
!       DENII(1) = 1.76E+03 
!       AMFSI(1) = 0.5
!       AMSI(1) = 1.32E-01

!       print *, "TPI= ", TPI(1), "DPGI= ",DPGI(1), "SIGI= ",SIGI(1), "DENSI= ", DENSI(1), "DENII= ", DENII(1), "AMFSI= ", AMFSI(1), "AMSI= ", AMSI(1), "VHFI= ", VHFI(1)

        call GAULEG (XGS, WGS, Npgauss)
        call CCNSPEC (TPI, DPGI, SIGI, AMFSI, VHFI, AMSI, DENSI, DENII, TPARC, PPARC, NSMX, &
                      TEMP, PRES, DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,    &
                      AKOH, SSPLT, ALFA, BET1, BET2,     &
                      SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS, NMD)
        call PDFACTIV (WPARC, SIGW, TPARC, PPARC, NACT, SMAX,aa,       &
                       TEMP, PRES, DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,    &
                       AKOH, SSPLT, ALFA, BET1, BET2,     &
                       SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS, NMD,XGS, WGS)

	do i=1,NSMX
           anact(i) = NACT(i)
	end do

        asmax = SMAX
!       print *, "mass activation fraction in sub=",aa

        return
end subroutine nenes

!C=======================================================================
!C
!C *** SUBROUTINE CCNSPEC
!C *** THIS SUBROUTINE CALCULATES THE CCN SPECTRUM OF THE AEROSOL USING
!C     THE APPROPRIATE FORM OF KOHLER THEORY     
!C
!C  Developed by
!C  Athanasios Nenes
!C  School of Earth and Atmospheric Sciences
!C  School of Chemical and Biomolecular Engineering
!C  Georgia Institute of Technology
!C  Atlanta, Georgia, 30332-0340
!C  e-mail: thanos.nenes@che.gatech.edu
!C  URL: http:\\nenes.eas.gatech.edu
!C
!C=======================================================================
!C
      SUBROUTINE CCNSPEC (TPI,DPGI,SIGI,AMFSI,VHFI,AMSI,DENSI,DENII,   &
                          TPARC,PPARC,NMODES,    &
                          TEMP,PRES,DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,  &
                          AKOH, SSPLT, ALFA, BET1, BET2,            &
                          SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS, NMD)
 
!     INCLUDE 'parametr.inc'

      implicit none

      integer  ::      NMODES
      real(r8) ::          DPGI(NMODES), VHFI(NMODES), AMSI(NMODES),    &
                       DENSI(NMODES),SIGI(NMODES), TPI(NMODES),     &
                       AMFSI(NMODES),DENII(NMODES),                 &
                       TPARC, PPARC
!xujun OpenMP
     integer  ::      I,K,NMD
     real(r8) ::      AMFI, DENP, VLFS, PAR1, PAR2 

     real(r8) ::      TEMP, PRES,     &
                      DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,  &
                      AKOH, SSPLT, ALFA, BET1, BET2
     real(r8), dimension(1:NSMX) :: SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS

      NMD  = NMODES                ! Save aerosol params in COMMON
      DO I=1,NMD
         DPG(I) = DPGI(I)
         VHF(I) = VHFI(I)
         AMS(I) = AMSI(I)
         DENS(I)= DENSI(I)
         SIG(I) = SIGI(I)
         TP(I)  = TPI(I)
         AMFS(I)= AMFSI(I)
         DENI(I)= DENII(I)
      ENDDO
 
      TEMP = TPARC                 ! Save parcel props in COMMON
      PRES = PPARC
 
      CALL PROPS(TEMP,PRES,DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT)                   ! Calculate thermophysical properties
 
! *** Calculate critical properties
!
      AKOH = 4._r8*AMW*SURT/RGAS/TEMP/DENW   ! Curvature param
      DO K=1,NMD
         AMFI = MAX(1.0_r8-AMFS(K),0.0_r8)                 ! Insoluble Mass.Frac.
         DENP = AMFS(K)*DENS(K) + AMFI*DENI(K)       ! Particle Density
         VLFS = AMFS(K)/DENS(K)/(AMFS(K)/DENS(K)+AMFI/DENI(K)) ! Vol.Fr.Salt
!print *, "VLFS= ",AMFS(K)/DENS(K)+AMFI/DENI(K)
         PAR1 = 4._r8*DENW*AMS(K)/27._r8/VHF(K)/DENP/AMW/DPG(K)**3._r8
         PAR1 = PAR1/VLFS                    ! Adjust for vol.frac. of salt
!print *, "PAR1= ",PAR1
         PAR2 = SQRT(PAR1*AKOH**3._r8)
!print *,"PAR2= ",SQRT(PAR1*AKOH**3)
         SG(K)= EXP(PAR2) - 1._r8              ! Sc of Dpg
!print *,"SG(K)= ",K,SG(K)
      ENDDO
      
      RETURN
END SUBROUTINE CCNSPEC


!C=======================================================================
!C
!C *** SUBROUTINE PDFACTIV
!C *** THIS SUBROUTINE CALCULATES THE CCN ACTIVATION FRACTION ACCORDING 
!C     TO THE Nenes and Seinfeld (2003) PARAMETERIZATION, WITH
!C     MODIFICATION FOR NON-CONTUNUUM EFFECTS AS PROPOSED BY Fountoukis
!C     and Nenes (2005). THIS ROUTINE CALCULATES FOR A PDF OF
!C     UPDRAFT VELOCITIES.
!C
!C  Developed by
!C  Athanasios Nenes
!C  School of Earth and Atmospheric Sciences
!C  School of Chemical and Biomolecular Engineering
!C  Georgia Institute of Technology
!C  Atlanta, Georgia, 30332-0340
!C  e-mail: thanos.nenes@che.gatech.edu
!C  URL: http:\\nenes.eas.gatech.edu
!C
!C=======================================================================
!C
      SUBROUTINE PDFACTIV (WPARC,SIGW,TPARC,PPARC,NACT,SMAX,aaa,     &
                           TEMP,PRES,DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,   &
                           AKOH, SSPLT, ALFA, BET1, BET2,      &
                           SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS,NMD,XGS, WGS)
 
!     8NCLUDE 'parametr.inc'

      implicit none

      real(r8) :: TPART, NACT(NSMX), NACTI(NSMX),aaa(NSMX)
      REAL(r8) :: PDF

      real(r8) :: WPARC,SIGW,TPARC,PPARC,SMAX
      real(r8) :: WPI,SMAXI
!xujun OpenMP
      integer :: i, ISEC,NMD
      real(r8) :: DPNMX,PLIMT, PROBI, WHI, WLO, SCAL

      real(r8) :: TEMP,PRES,           &
                  DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,     &
                  AKOH, SSPLT, ALFA, BET1, BET2
       real(r8), dimension(1:NSMX) :: SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS
      real(r8), dimension(1:Npgauss) :: WPDBG, PDDBG, NADBG, SMDBG
       real(r8), dimension(1:Npgauss) :: XGS, WGS

! *** Case where updraft is very small
 
      IF (WPARC.LE.1e-6_r8) THEN
         SMAX  = 0._r8
        
	do i=1,NSMX
	 NACT(i)  = 0._r8
	end do

         ISEC  = 1
         DPNMX = GREAT
         RETURN
      ENDIF
 
! *** Single updraft case
 
      IF (SIGW.LT.1e-10_r8) THEN
         CALL ACTIVATE (WPARC,NACT,SMAX,aaa,     &
                        TEMP,PRES,DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,   &
                        AKOH, SSPLT, ALFA, BET1, BET2,     &
                        SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS,NMD)
         WPDBG(1) = WParc                      ! Save debug info
         PDDBG(1) = 1.0_r8
         NADBG(1) = NACT(1)
         SMDBG(1) = SMAX
 
! *** PDF of updrafts
 
      ELSE           
         NACT  = ZERO
         SMAX  = ZERO
         PLIMT = 1e-3     ! Probability of High Updraft limit
         PROBI = SQRT(-2.0_r8*LOG(PLIMT*SIGW*SQ2PI))
         WHI   = WPARC + SIGW*PROBI             ! Upper updrft limit
         WLO   = 0.05_r8  ! WPARC - SIGW*PROBI     ! Low updrft limit
         SCAL  = 0.5_r8*(WHI-WLO)                  ! Scaling for updrafts
         DO I=1,Npgauss                 
            WPI  = WLO + SCAL*(1.0_r8-XGS(i))      ! Updraft
            CALL ACTIVATE (WPI,NACTI,SMAXI,aaa,     &
                           TEMP,PRES,DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,    &
                           AKOH, SSPLT, ALFA, BET1, BET2,     &
                           SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS,NMD)     ! # of drops
            PDF  = (1.0/SQ2PI/SIGW)*EXP(-0.5*((WPI-WPARC)/SIGW)**2.0)  ! Probability of updraft
            NACT = NACT + WGS(i)*(PDF*NACTI)    ! Integral for drops
            SMAX = SMAX + WGS(i)*(PDF*SMAXI)    ! Integral for Smax
            WPDBG(I) = WPI                      ! Save debug info
            PDDBG(I) = PDF
            NADBG(I) = NACTI(I)
            SMDBG(I) = SMAXI
            IF (PDF.LT.PLIMT) GOTO 100
         ENDDO
 100     NACT = NACT*SCAL                       ! Scale Integrals
         SMAX = SMAX*SCAL
      ENDIF
 
      RETURN
 
END SUBROUTINE PDFACTIV


!=======================================================================
!
! *** SUBROUTINE ACTIVATE
! *** THIS SUBROUTINE CALCULATES THE CCN ACTIVATION FRACTION ACCORDING 
!     TO THE Nenes and Seinfeld (2003) PARAMETERIZATION, WITH
!     MODIFICATION FOR NON-CONTUNUUM EFFECTS AS PROPOSED BY Fountoukis
!     and Nenes (2005).
!
!  Developed by
!  Athanasios Nenes
!  School of Earth and Atmospheric Sciences
!  School of Chemical and Biomolecular Engineering
!  Georgia Institute of Technology
!  Atlanta, Georgia, 30332-0340
!  e-mail: thanos.nenes@che.gatech.edu
!  URL: http:\\nenes.eas.gatech.edu
!
!=======================================================================
!
      SUBROUTINE ACTIVATE (WPARC,NDRPL,SMAX,aaaa,     &
                           TEMP,PRES,DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,   &
                           AKOH, SSPLT, ALFA, BET1, BET2,      &
                           SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS, NMD)
!     INCLUDE 'parametr.inc'
!     DOUBLE PRECISION NDRPL

      implicit none

      real(r8) :: NDRPL(NSMX),aaaa(NSMX)

      real(r8) :: WPARC,SMAX

!xujun OpenMP
      real(r8) :: BETA, CF1, CF2, X1, SINTEG1, SINTEG2, Y1, X2, Y2, X3, Y3
      integer  :: I,NMD

      real(r8) :: WPARCEL,TEMP,PRES,NITER,                &
                  DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT,   &
                  AKOH, SSPLT, ALFA, BET1, BET2
      real(r8), dimension(1:NSMX) :: SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS
 
! *** Setup common block variables
 
      WPARCEL = WPARC
 
! *** Setup constants
 
      ALFA = GRAV*AMW*DHV/CPAIR/RGAS/TEMP/TEMP - GRAV*AMA/RGAS/TEMP
      BET1 = PRES*AMA/PSAT/AMW + AMW*DHV*DHV/CPAIR/RGAS/TEMP/TEMP
      BET2 = RGAS*TEMP*DENW/PSAT/DV/AMW/4d0 +      &             
             DHV*DENW/4._r8/AKA/TEMP*(DHV*AMW/RGAS/TEMP - 1._r8)
      BETA = 0.5*PI*BET1*DENW/BET2/ALFA/WPARC/DAIR            
      CF1  = 0.5*(((1d0/BET2)/(ALFA*WPARC))**0.5)
      CF2  = AKOH/3._r8
 
! *** INITIAL VALUES FOR BISECTION **************************************
 
      X1   = 1.0e-5_r8   ! Min cloud supersaturation -> 0

      CALL SINTEGRAL (X1,NDRPL,SINTEG1,SINTEG2,aaaa,     &
                      WPARCEL,AKOH, SSPLT, ALFA, BET1, BET2,     &
                      SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS,NMD)
      Y1   = (SINTEG1*CF1+SINTEG2*CF2)*BETA*X1 - 1d0
 
      X2   = 1._r8      ! MAX cloud supersaturation = 100%
      CALL SINTEGRAL (X2,NDRPL,SINTEG1,SINTEG2,aaaa,     &
                      WPARCEL,AKOH, SSPLT, ALFA, BET1, BET2,     &
                      SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS,NMD)
      Y2   = (SINTEG1*CF1+SINTEG2*CF2)*BETA*X2 - 1._r8
 
! *** PERFORM BISECTION *************************************************
 
20    DO 30 I=1,MAXIT
         X3   = 0.5*(X1+X2)
 
         CALL SINTEGRAL (X3,NDRPL,SINTEG1,SINTEG2,aaaa,   &
                         WPARCEL,AKOH, SSPLT, ALFA, BET1, BET2,    &
                         SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS,NMD)
         Y3 = (SINTEG1*CF1+SINTEG2*CF2)*BETA*X3 - 1._r8
 
         IF (SIGN(1._r8,Y1)*SIGN(1._r8,Y3) .LE. ZERO) THEN  ! (Y1*Y3 .LE. ZERO)
            Y2    = Y3
            X2    = X3
         ELSE
            Y1    = Y3
            X1    = X3
         ENDIF
 
         IF (ABS(X2-X1) .LE. EPS*X1) GOTO 40
         NITER = I
 
30    CONTINUE
 
! *** CONVERGED ; RETURN ************************************************
 
40    X3   = 0.5*(X1+X2)
 
      CALL SINTEGRAL (X3,NDRPL,SINTEG1,SINTEG2,aaaa,       &
                      WPARCEL,AKOH, SSPLT, ALFA, BET1, BET2,      &
                      SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS,NMD)
      Y3   = (SINTEG1*CF1+SINTEG2*CF2)*BETA*X3 - 1._r8
      SMAX = X3
 
      RETURN
 
END SUBROUTINE ACTIVATE


!=======================================================================
!
! *** SUBROUTINE SINTEGRAL
! *** THIS SUBROUTINE CALCULATES THE CONDENSATION INTEGRALS, ACCORDING
!     TO THE POPULATION SPLITTING ALGORITHM OF Nenes and Seinfeld (2003)
!     Modal formulation according to Fountoukis and Nenes (2005)
!
!  Developed by
!  Athanasios Nenes
!  School of Earth and Atmospheric Sciences
!  School of Chemical and Biomolecular Engineering
!  Georgia Institute of Technology
!  Atlanta, Georgia, 30332-0340
!  e-mail: thanos.nenes@che.gatech.edu
!  URL: http:\\nenes.eas.gatech.edu
!
!=======================================================================
 
      SUBROUTINE SINTEGRAL (SPAR, SUMMA, SUM, SUMMAT,amassfra,     &
                            WPARCEL,AKOH, SSPLT, ALFA, BET1, BET2,  &
                            SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS,NMD)
!     INCLUDE 'parametr.inc'

      implicit none

      real(r8) ::          SUMMA(NSMX)
	real(r8) :: amassfra(NSMX),franum(NSMX)
      real(r8) ::          SUM, SUMMAT,  Nd(NSMX),    &
                       INTEG1(NSMX),INTEG2(NSMX)
      REAL(r8) ::             ERF   !  sxj  if define ERF , fortran function ERF will be used here.
      REAL(r8) ::          ORISM1, ORISM2, ORISM3, ORISM4, ORISM5, ORISM6
      real(r8) ::          DLGSG, DLGSP

!xujun OpenMP
      real(r8) :: SPAR, DESCR, RATIO, SSPLT2, SSPLT1, SQTWO,EKTH
      integer  :: i, j, NMD

      real(r8) :: WPARCEL,AKOH, SSPLT, ALFA, BET1, BET2
      real(r8), dimension(1:NSMX) :: SG, TP, DPG, SIG, VHF, AMS, DENS, DENI, AMFS
      logical :: CRIT2
 
! *** Here is where the criterion with the descriminant is put. When it
!     is < 0, then set CRIT2 = .TRUE. Otherwise, set the two values of
!     SSPLT and continue.
!
      DESCR  = 1._r8 - (16._r8/9._r8)*ALFA*WPARCEL*BET2*(AKOH/SPAR**2d0)**2d0
 
      IF (DESCR.LE.0._r8) THEN
         CRIT2  = .TRUE.             ! SSPLT1,SSPLT2 do not exist
         RATIO  = (2.0e7_r8/3.0_r8)*AKOH*SPAR**(-0.3824_r8)
         IF (RATIO.GT.1.0_r8) RATIO = 1.0_r8
         SSPLT2 = SPAR*RATIO
      ELSE
         CRIT2  = .FALSE.
         SSPLT1 = 0.5_r8*(1._r8-SQRT(DESCR)) ! min root of both
         SSPLT2 = 0.5_r8*(1._r8+SQRT(DESCR)) ! max root of both
         SSPLT1 = SQRT(SSPLT1)*SPAR       ! Multiply ratios with Smax
         SSPLT2 = SQRT(SSPLT2)*SPAR
      ENDIF
 
      SSPLT = SSPLT2  ! Store Ssplit in COMMON
 
! *** Calculate integrals
 
      SUM       = 0._r8
      SUMMAT    = 0._r8

	do i=1,NSMX
      SUMMA(i)     = 0._r8
 	end do

      SQTWO     = SQRT(2._r8)
 
      DO 999 j = 1, NMD
 
      !++sxj DLOG changed to LOG
      !DLGSG     = DLOG(SIG(j))
      !DLGSP     = DLOG(SG(j)/SPAR)
      !ORISM1    = 2._r8*DLOG(SG(j)/SSPLT2)/(3._r8*SQTWO*DLGSG )
	  DLGSG     = LOG(SIG(j))
      DLGSP     = LOG(SG(j)/SPAR)
      ORISM1    = 2._r8*LOG(SG(j)/SSPLT2)/(3._r8*SQTWO*DLGSG ) 
      !--sxj
      ORISM2    = ORISM1 - 3._r8*DLGSG/(2._r8*SQTWO)
      ORISM3    = 2._r8*DLGSP/(3._r8*SQTWO*DLGSG)-3._r8*DLGSG/(2._r8*SQTWO)
      ORISM4    = ORISM1 + 3._r8*DLGSG/SQTWO
      ORISM5    = 2._r8*DLGSP/(3._r8*SQTWO*DLGSG)
      EKTH      = EXP(9._r8/2._r8*DLGSG*DLGSG)
      INTEG1(j) = TP(j)*SPAR*((1._r8-erf(ORISM1)) -                     &
                  0.5_r8*((SG(j)/SPAR)**2._r8)*EKTH*(1._r8-erf(ORISM4)))
      INTEG2(j) = (EXP(9._r8/8._r8*DLGSG*DLGSG)*TP(j)/SG(j))*              &
                  (erf(ORISM2) - erf(ORISM3))
 
! *** Calculate number of Drops
 
      Nd(j)     = (TP(j)/2.0_r8)*(1._r8-erf(ORISM5))
      !ORISM6    = ORISM5-(3.0_r8*SQTWO/2.0_r8)*dlog(2._r8)
      ORISM6    = ORISM5-(3.0_r8*SQTWO/2.0_r8)*log(2._r8) ! sxj DLOG changed to LOG
      amassfra(j) = (1.0_r8/2.0_r8)*(1.0_r8-erf(ORISM6))

      franum(j)     = (1._r8/2._r8)*(1._r8-erf(ORISM5))
!     print *, "number activated fraction= ", franum
!     print *, "mass activated fraction= ", amassfra
 
      SUM       = SUM    + INTEG1(j)
      SUMMAT    = SUMMAT + INTEG2(j)
!     SUMMA     = SUMMA  + Nd(j)
      SUMMA(j)     = Nd(j)
 999  Continue
 
      RETURN
END SUBROUTINE SINTEGRAL

!=======================================================================
!
! *** SUBROUTINE PROPS
! *** THIS SUBROUTINE CALCULATES THE THERMOPHYSICAL PROPERTIES
!
!  Developed by
!  Athanasios Nenes
!  School of Earth and Atmospheric Sciences
!  School of Chemical and Biomolecular Engineering
!  Georgia Institute of Technology
!  Atlanta, Georgia, 30332-0340
!  e-mail: thanos.nenes@che.gatech.edu
!  URL: http:\\nenes.eas.gatech.edu
!
!=======================================================================
 
      SUBROUTINE PROPS(TEMP,PRES,DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT) 
!     INCLUDE 'parametr.inc'

      implicit none

      !REAL(r8) ::  VPRES, SFT               ! sxj can not define VPRES, SFT here 

!xujun OpenMP
      real(r8) :: PRESA, COEF
      real(r8) :: TEMP, PRES,                &
                  DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT
      real(r8) :: DBIG, DLOW

      DENW  = 1.e3_r8                           ! Water density
      DHV   = 2.25e6_r8                        ! Water enthalpy of vaporization
      CPAIR = 1.0061e3_r8                      ! Air Cp
      PRESA = PRES/1.013e5_r8                  ! Pressure (Pa)
      DAIR  = PRES*AMA/RGAS/TEMP            ! Air density
 
      AKA   = (4.39_r8+0.071_r8*TEMP)*1e-3_r8        ! Air thermal conductivity
 
      DV    = (0.211_r8/PRESA)*(TEMP/273._r8)**1.94_r8
      DV    = DV*1.e-4_r8                       ! Water vapor diffusivity in air
      DBIG  = 5.0e-6_r8
      DLOW  = 0.207683_r8*((ACCOM)**(-0.33048_r8))
      DLOW  = DLOW*1.e-6_r8
 
! Dv average
 
      COEF  = ((2._r8*PI*AMW/(RGAS*TEMP))**0.5_r8)
 
      DV    = (DV/(DBIG-DLOW))*((DBIG-DLOW)-(2._r8*DV/ACCOM)*COEF*         &
 !             (DLOG((DBIG+(2._r8*DV/ACCOM)*COEF)/(DLOW+(2._r8*DV/ACCOM)*    &
               (LOG((DBIG+(2._r8*DV/ACCOM)*COEF)/(DLOW+(2._r8*DV/ACCOM)*    &   ! sxj DLOG=> LOG
              COEF))))                      ! Non-continuum effects
 
      PSAT  = VPRES(TEMP)*(1.e5_r8/1.e3_r8) ! Saturation vapor pressure
!print *, "SNGL(TEMP)= ",SNGL(TEMP) 
      SURT  = SFT(TEMP)       ! Surface Tension for water (J m-2)
!print *, DENW, CPAIR, DHV, AKA, DV, PSAT, DAIR, SURT, TEMP 
      RETURN
 
END SUBROUTINE PROPS


!=======================================================================
!
! *** FUNCTION VPRES
! *** THIS FUNCTION CALCULATES SATURATED WATER VAPOUR PRESSURE AS A 
!     FUNCTION OF TEMPERATURE. VALID FOR TEMPERATURES BETWEEN -50 AND 
!     50 C.
!
! ======================== ARGUMENTS / USAGE ===========================
!
!  INPUT:
!     [T] 
!     REAL variable.
!     Ambient temperature expressed in Kelvin. 
!
!  OUTPUT:
!     [VPRES] 
!     REAL variable.  
!     Saturated vapor pressure expressed in mbar.
!
!=======================================================================
!
!     REAL FUNCTION VPRES (T)
      FUNCTION VPRES (T)

      implicit none

      real(r8) VPRES
      REAL(r8) :: A(0:6), T

!xujun OpenMP
      real(r8) :: TTEMP
      integer  :: I

      DATA A/6.107799610E+0_r8, 4.436518521E-1_r8, 1.428945805E-2_r8,   &
             2.650648471E-4_r8, 3.031240396E-6_r8, 2.034080948E-8_r8,   &
             6.136820929E-11_r8/
 
! Calculate polynomial (without exponentiation).
!print *, "T= ",T 
      TTEMP = T-273._r8
      VPRES = A(6)*TTEMP
      DO I=5,1,-1
         VPRES = (VPRES + A(I))*TTEMP
      ENDDO
      VPRES = VPRES + A(0)
 
! End of FUNCTION VPRES
 
      RETURN
END FUNCTION VPRES



!=======================================================================
!
! *** FUNCTION SFT
! *** THIS FUNCTION CALCULATES WATER SURFACE TENSION AS A 
!     FUNCTION OF TEMPERATURE. VALID FOR TEMPERATURES BETWEEN -40 AND 
!     40 C.
!
! ======================== ARGUMENTS / USAGE ===========================
!
!  INPUT:
!     [T] 
!     REAL variable.
!     Ambient temperature expressed in Kelvin.
!
!  OUTPUT:
!     [SFT] 
!     REAL variable.  
!     Surface Tension expressed in J m-2.
!
!=======================================================================
!
      FUNCTION SFT (T)

      implicit none
      
      real(r8) :: SFT

      REAL(r8) :: T

!xujun OpenMP
      real(r8) :: TPARS
 
      TPARS = T-273._r8
      SFT   = 0.0761_r8-1.55e-4_r8*TPARS  
!print *, "T= ", T, "TPARS= ", TPARS,"SFT= ", SFT 
      RETURN
END FUNCTION SFT


! ***********************************************************************
!
      SUBROUTINE GAULEG (X,W,N)
!
! Calculation of points and weights for N point GAUSS integration
! ***********************************************************************
!      DIMENSION X(N), W(N)
!      PARAMETER (EPS=1.E-6)
!      PARAMETER (X1=-1.0, X2=1.0)

      implicit none

!xujun OpenMP
      integer :: N,I,J,M
      real(r8), dimension(1:N) :: X,W 
      real(r8), parameter :: EPS = 1.E-6_r8
      real(r8), parameter :: X1 = -1._r8
      real(r8), parameter :: X2 = 1._r8
      real(r8) :: XM, XL, Z, P1, P2, P3, PP, Z1

! Calculation
 
      M=(N+1)/2._r8
      XM=0.5_r8*(X2+X1)
      XL=0.5_r8*(X2-X1)
      DO 12 I=1,M
        Z=COS(3.141592654_r8*(I-.25_r8)/(N+.5_r8))
1       CONTINUE
          P1=1._r8
          P2=0._r8
          DO 11 J=1,N
            P3=P2
            P2=P1
            P1=((2._r8*J-1._r8)*Z*P2-(J-1._r8)*P3)/J
11        CONTINUE
          PP=N*(Z*P1-P2)/(Z*Z-1._r8)
          Z1=Z
          Z=Z1-P1/PP
        IF(ABS(Z-Z1).GT.EPS)GO TO 1
        X(I)=XM-XL*Z
        X(N+1-I)=XM+XL*Z
        W(I)=2._r8*XL/((1._r8-Z*Z)*PP*PP)
        W(N+1-I)=W(I)
12    CONTINUE
      RETURN
END SUBROUTINE GAULEG



!=======================================================================
!
! *** REAL FUNCTION ERF
! *** THIS SUBROUTINE CALCULATES THE ERROR FUNCTION
!
! *** OBTAINED FROM NUMERICAL RECIPIES v.2.0
!
!=======================================================================
!
      REAL FUNCTION ERF(X)

      implicit none

!xujun OpenMP
      real(r8) :: X

      IF(X.LT.0.)THEN
        ERF=-GAMMP(.5_r8,X**2)
      ELSE
        ERF=GAMMP(.5_r8,X**2)
      ENDIF
      RETURN
END FUNCTION ERF

!
!=======================================================================
!
      FUNCTION GAMMLN(XX)
!
!=======================================================================
!
!      REAL*8 COF(6),STP,HALF,ONE,FPF,X,TMP,SER

      implicit none

      REAL(r8) ::  COF(6),STP,HALF,ONE,FPF,X,TMP,SER

!xujun OpenMP
      real(r8) :: XX, GAMMLN
      integer  :: J

      DATA COF,STP/76.18009173_r8,-86.50532033_r8,24.01409822_r8,     &
          -1.231739516_r8,.120858003e-2_r8,-.536382e-5_r8,2.50662827465_r8/
      DATA HALF,ONE,FPF/0.5_r8,1._r8,5.5_r8/
      X=XX-ONE
      TMP=X+FPF
      TMP=(X+HALF)*LOG(TMP)-TMP
      SER=ONE
      DO 11 J=1,6
        X=X+ONE
        SER=SER+COF(J)/X
11    CONTINUE
      GAMMLN=TMP+LOG(STP*SER)
      RETURN
END FUNCTION GAMMLN


!
!=======================================================================
!
      FUNCTION GAMMP(A,X)
!
!=======================================================================
!
      implicit none

!xujun OpenMP
      real(r8) :: A, X, GAMMP, GAMSER, GLN, GAMMCF

      IF(X.LT.0._r8.OR.A.LE.0._r8)PAUSE
      IF(X.LT.A+1._r8)THEN
        CALL GSER(GAMSER,A,X,GLN)
        GAMMP=GAMSER
      ELSE
        CALL GCF(GAMMCF,A,X,GLN)
        GAMMP=1._r8-GAMMCF
      ENDIF
      RETURN
END FUNCTION GAMMP


!
!=======================================================================
!
      SUBROUTINE GCF(GAMMCF,A,X,GLN)
!
!=======================================================================
!
!     PARAMETER (ITMAX=100,EPS=3.E-7)

      implicit none

!xujun OpenMP
      integer, parameter :: ITMAX=100
      real(r8), parameter :: EPS=3.E-7_r8
      real(r8) :: GAMMCF, A, X, GLN, GOLD, A0, A1, B0, B1, FAC, AN, ANA, ANF, G
      integer :: N

      GLN=GAMMLN(A)
      GOLD=0._r8
      A0=1._r8
      A1=X
      B0=0._r8
      B1=1._r8
      FAC=1._r8
      DO 11 N=1,ITMAX
        AN=FLOAT(N)
        ANA=AN-A
        A0=(A1+A0*ANA)*FAC
        B0=(B1+B0*ANA)*FAC
        ANF=AN*FAC
        A1=X*A0+ANF*A1
        B1=X*B0+ANF*B1
        IF(A1.NE.0._r8)THEN
          FAC=1._r8/A1
          G=B1*FAC
          IF(ABS((G-GOLD)/G).LT.EPS)GO TO 1
          GOLD=G
        ENDIF
11    CONTINUE
      PAUSE 'A too large, ITMAX too small'
1     GAMMCF=EXP(-X+A*LOG(X)-GLN)*G
      RETURN
END SUBROUTINE GCF


!
!=======================================================================
!
      SUBROUTINE GSER(GAMSER,A,X,GLN)
!
!=======================================================================
!
!      PARAMETER (ITMAX=100,EPS=3.E-7)

      implicit none

!xujun OpenMP
      integer, parameter :: ITMAX=100
      real(r8), parameter :: EPS=3.E-7_r8
      real(r8) :: GAMSER, A, X, GLN, AP, SUM, DEL
      integer :: N

      GLN=GAMMLN(A)
      IF(X.LE.0._r8)THEN
        IF(X.LT.0._r8)PAUSE
        GAMSER=0._r8
        RETURN
      ENDIF
      AP=A
      SUM=1._r8/A
      DEL=SUM
      DO 11 N=1,ITMAX
        AP=AP+1._r8
        DEL=DEL*X/AP
        SUM=SUM+DEL
        IF(ABS(DEL).LT.ABS(SUM)*EPS)GO TO 1
11    CONTINUE
      PAUSE 'A too large, ITMAX too small'
1     GAMSER=SUM*EXP(-X+A*LOG(X)-GLN)
      RETURN
END SUBROUTINE GSER

 end module nenesact
