module bcc_aero_em3_prod

      implicit none

      private
      public :: aero_em3_prod
      
      contains
      subroutine aero_em3_prod ( em3data, nem3, m, ncol, delt, vmr,  &
                                 em3_IPCC6,    nem3_IPCC6,           &
                                 emis_cb1_vmr_d3, emis_oc1_vmr_d3,   &
                                 emis_oc2_vmr_d3, emis_nh3_vmr_d3,   &
                                 emis_so2_vmr_d3, emis_cb2_vmr_d3, emis_so4_vmr_d3)

!----------------------------------------------------------------------------------
!  Authors:  Wu Tongwen   
!------------------------------------------------------------------------------------
      use ppgrid,       only : pver
      use mo_chem_utls, only : get_spc_ndx
      use shr_kind_mod, only : r8 => shr_kind_r8
      use ppgrid,       only:  pcols
      use mo_grid,      only : pcnstm1
      use prescribed_em_d3,    only : idx3_NO2, idx3_SO2, idx3_CB2, idx3_SO4
      use prescribed_emis_3d,  only : idxCB1_IPCC6, idxCB2_IPCC6 , idxOC1_IPCC6, idxOC2_IPCC6, idxSO2_IPCC6, idxNH3_IPCC6

      implicit none
!--------------------------------------------------------------------
! ... Dummy args
!--------------------------------------------------------------------
      integer, intent(in) :: nem3, nem3_IPCC6
      integer, intent(in) :: ncol
      real,    intent(in) :: delt
      real,    intent(in) :: m(pcols, pver)  ! invariant densities (molecules/cm^3)
      real,    intent(inout) :: vmr(pcols,pver,pcnstm1)
      real,    intent(in) :: em3data(pcols, pver, nem3)   ! mol/cm3/s
      real,    intent(in) :: em3_IPCC6(pcols, pver, nem3_IPCC6)   ! mol/cm3/s
      real,    intent(out) ::  emis_so2_vmr_d3(pcols,pver)
      real,    intent(out) ::  emis_cb2_vmr_d3(pcols,pver)
      real,    intent(out) ::  emis_so4_vmr_d3(pcols,pver)

      real,    intent(out) ::  emis_cb1_vmr_d3(pcols,pver)
      real,    intent(out) ::  emis_oc1_vmr_d3(pcols,pver)
      real,    intent(out) ::  emis_oc2_vmr_d3(pcols,pver)
      real,    intent(out) ::  emis_nh3_vmr_d3(pcols,pver)

!-------------------------------------------------------
!       ... local variables
!-------------------------------------------------------
      integer  :: so2_ndx, so4_ndx, cb2_ndx
      integer  :: oc1_ndx, oc2_ndx, cb1_ndx, nh3_ndx

      integer  :: k, i

!===================================================================================     

     so2_ndx = get_spc_ndx( 'SO2' )
     so4_ndx = get_spc_ndx( 'SO4' )
     cb2_ndx = get_spc_ndx( 'CB2' )

     cb1_ndx = get_spc_ndx( 'CB1' )
     oc1_ndx = get_spc_ndx( 'OC1' )
     oc2_ndx = get_spc_ndx( 'OC2' )
     nh3_ndx = get_spc_ndx( 'NH3' )

     emis_so2_vmr_d3(:,:) = 0.0
     emis_cb2_vmr_d3(:,:) = 0.0
     emis_so4_vmr_d3(:,:) = 0.0
     emis_cb1_vmr_d3(:,:) = 0.0
     emis_oc1_vmr_d3(:,:) = 0.0
     emis_oc2_vmr_d3(:,:) = 0.0
     emis_nh3_vmr_d3(:,:) = 0.0

!--------------------------------------------
      do k=1, pver
         if( idxCB1_IPCC6 > 0 ) then
             vmr(:ncol,k,cb1_ndx) = vmr(:ncol,k,cb1_ndx) + em3_IPCC6(:ncol,k,idxCB1_IPCC6) /m(:ncol,k) *delt
             emis_cb1_vmr_d3(:ncol,k) = em3_IPCC6(:ncol,k,idxCB1_IPCC6) /m(:ncol,k) *delt
         endif
         if( idxCB2_IPCC6 > 0 ) then
             vmr(:ncol,k,cb2_ndx) = vmr(:ncol,k,cb2_ndx) + em3_IPCC6(:ncol,k,idxCB2_IPCC6) /m(:ncol,k) *delt
             emis_cb2_vmr_d3(:ncol,k) = em3_IPCC6(:ncol,k,idxCB2_IPCC6) /m(:ncol,k) *delt
         endif
         if( idxOC1_IPCC6 > 0 ) then
             vmr(:ncol,k,oc1_ndx) = vmr(:ncol,k,oc1_ndx) + em3_IPCC6(:ncol,k,idxOC1_IPCC6) /m(:ncol,k) *delt
             emis_oc1_vmr_d3(:ncol,k) = em3_IPCC6(:ncol,k,idxOC1_IPCC6) /m(:ncol,k) *delt
         endif
         if( idxOC2_IPCC6 > 0 ) then
             vmr(:ncol,k,oc2_ndx) = vmr(:ncol,k,oc2_ndx) + em3_IPCC6(:ncol,k,idxOC2_IPCC6) /m(:ncol,k) *delt
             emis_oc2_vmr_d3(:ncol,k) = em3_IPCC6(:ncol,k,idxOC2_IPCC6) /m(:ncol,k) *delt
         endif
         if( idxSO2_IPCC6 > 0 ) then
             vmr(:ncol,k,so2_ndx) = vmr(:ncol,k,so2_ndx) + em3_IPCC6(:ncol,k,idxSO2_IPCC6) /m(:ncol,k) *delt
             emis_so2_vmr_d3(:ncol,k) = em3_IPCC6(:ncol,k,idxSO2_IPCC6) /m(:ncol,k) *delt
         endif
         if( idxNH3_IPCC6 > 0 ) then
             vmr(:ncol,k,nh3_ndx) = vmr(:ncol,k,nh3_ndx) + em3_IPCC6(:ncol,k,idxNH3_IPCC6) /m(:ncol,k) *delt
             emis_nh3_vmr_d3(:ncol,k) = em3_IPCC6(:ncol,k,idxNH3_IPCC6) /m(:ncol,k) *delt
         endif

         if( idx3_SO2 > 0 ) then
             vmr(:ncol,k,so2_ndx) = vmr(:ncol,k,so2_ndx) + em3data(:ncol,k,idx3_SO2) /m(:ncol,k) *delt
             emis_so2_vmr_d3(:ncol,k) = em3data(:ncol,k,idx3_SO2) /m(:ncol,k) *delt
         endif
         if( idx3_CB2 > 0 ) then
             vmr(:ncol,k,cb2_ndx) = vmr(:ncol,k,cb2_ndx) + em3data(:ncol,k,idx3_CB2) /m(:ncol,k) *delt
             emis_cb2_vmr_d3(:ncol,k) = em3data(:ncol,k,idx3_CB2) /m(:ncol,k) *delt
         endif
         if( idx3_SO4 > 0 ) then
             vmr(:ncol,k,so4_ndx) = vmr(:ncol,k,so4_ndx) + em3data(:ncol,k,idx3_SO4) /m(:ncol,k) *delt
             emis_so4_vmr_d3(:ncol,k) = em3data(:ncol,k,idx3_SO4) /m(:ncol,k) *delt
         endif
      enddo
      end subroutine aero_em3_prod
  end module bcc_aero_em3_prod

