#include <misc.h>
#include <params.h>

module bcc_nature_emis_drydep

    use shr_kind_mod,         only : r8 => shr_kind_r8

    implicit none

  private
  save

  public :: nature_emis_drydep

contains

   subroutine nature_emis_drydep (lchnk, ncol, sflx, q, pdel, &
                                  pmid, t, vlc_trb_SO4, vlc_trb_NH4, vlc_trb_CB2, vlc_trb_SO2 ) 


    use shr_kind_mod,         only : r8 => shr_kind_r8
    use ppgrid,               only : pcols, pver
    use constituents,         only : ppcnst
    use physconst,            only : gravit, rair, zvir
    use constituents,         only : cnst_get_ind

    !-----------------------------------------------------------------------
    implicit none
    !-----------------------------------------------------------------------
    !
    ! Arguments:
    !
      real(r8),intent(in) :: vlc_trb_SO4(pcols)       ! dry deposite velcity (m/s)
      real(r8),intent(in) :: vlc_trb_NH4(pcols)       ! dry deposite velcity (m/s)
      real(r8),intent(in) :: vlc_trb_CB2(pcols)       ! dry deposite velcity (m/s)
      real(r8),intent(in) :: vlc_trb_SO2(pcols)       ! dry deposite velcity (m/s)
      real, intent(in)    ::  pmid(pcols,pver)            ! midpoint pressure in surface layer (Pa)
      real                ::  tv(pcols,pver)           ! virtual temperature in surface layer (K)
      real, intent(in)    ::  t(pcols,pver)          

    integer,  intent(in) :: ncol, lchnk
    real(r8), intent(inout) :: sflx(pcols, ppcnst) 
    real(r8), intent(in) :: q(pcols,pver, ppcnst)        ! temperature (K)
    real(r8), intent(in) :: pdel(pcols,pver)             ! pressure difference across layers
    !
    ! Local variables
    !
    integer :: m,n,           &
                  cb1_ndx,    &
                  cb2_ndx,    &
                  oc1_ndx,    &
                  oc2_ndx,    &
                  so2_ndx,    &
                  so4_ndx,    &
				  co2_ndx

    real(r8) :: dflx(pcols,ppcnst)   ! flux due to dry deposition (kg/m^2/sec)
    real(r8) :: wrk(pcols) 
!-----------------------------------------------------------------------
!       ... Note the factor 1.e-2 in the wrk array calculchnkion is
!           to transform the incoming dep vel from cm/s to m/s
!-----------------------------------------------------------------------
    tv(:,pver) = t(:,pver) * (1._r8 + zvir * q(:,pver,1))
    wrk(:) =  1.e-2 * pmid(:,pver) / (rair * tv(:,pver))

    dflx(:,:) = 0.0 
!--------------------------------------------------------
!       ... Set cb1 dvel   wtw add
!--------------------------------------------------------
       call cnst_get_ind('CB1', cb1_ndx)
       dflx(:,cb1_ndx)   = wrk(:) * vlc_trb_CB2(:) * q(:,pver,cb1_ndx)
!
!  this is considered in other_aerosols_drydep
!
!--------------------------------------------------------
!       ... Set cb2 dvel   wtw add
!--------------------------------------------------------
       call cnst_get_ind('CB2', cb2_ndx)
       dflx(:,cb2_ndx)   = wrk(:) * vlc_trb_CB2(:) * q(:,pver,cb2_ndx)
!
!  this is considered in other_aerosols_drydep
!
!--------------------------------------------------------
!       ... Set oc1 dvel   wtw add
!--------------------------------------------------------
       call cnst_get_ind('OC1', oc1_ndx)
       dflx(:,oc1_ndx)   = wrk(:) * vlc_trb_CB2(:) * q(:,pver,oc1_ndx)
!
!  this is considered in other_aerosols_drydep
!
!--------------------------------------------------------
!       ... Set oc2 dvel   wtw add
!--------------------------------------------------------
       call cnst_get_ind('OC2', oc2_ndx)
       dflx(:,oc2_ndx)   = wrk(:) * vlc_trb_CB2(:) * q(:,pver,oc2_ndx)
!
!  this is considered in other_aerosols_drydep
!
!--------------------------------------------------------
!       ... Set so4 dvel   wtw add
!--------------------------------------------------------
       call cnst_get_ind('SO4', so4_ndx)
       dflx(:,so4_ndx)   = wrk(:) * vlc_trb_SO4(:) * q(:,pver,so4_ndx)
!
!  this is considered in other_aerosols_drydep
!
!--------------------------------------------------------
!       ... Set so2 dvel   wtw add
!--------------------------------------------------------
       call cnst_get_ind('SO2', so2_ndx)
       dflx(:,so2_ndx)   = wrk(:) * vlc_trb_SO2(:) * q(:,pver,so2_ndx)

!-------------------------------------------------------------------------------

    do m = 1,ppcnst
        sflx(:ncol, m) = sflx(:ncol,m) - dflx(:ncol,m)
    end do  
    return
    end subroutine nature_emis_drydep

end module bcc_nature_emis_drydep

