#include <misc.h>
#include <params.h>

module bcc_photo
  !----------------------------------------------------------------------
  !	... photolysis interp table and related arrays
  !----------------------------------------------------------------------

  use shr_kind_mod,     only : r8 => shr_kind_r8
  use ppgrid,           only : pcols, pver, pcols, pverp, begchunk, endchunk
  use abortutils,       only : endrun
  use mo_const_mozart,  only : pi,r2d,d2r
  use bcc_chem_mods,    only : get_rxt_ndx_bcc, get_inv_ndx
  use mo_chem_utls,     only : get_spc_ndx
  use physconst,        only : boltz  
  use pmgrid,           only : masterproc
  use pmgrid,           only : plev, plevp, plon, plat
#ifdef SPMD
  use mpishorthand,  only : mpicom, mpir8, mpiint
#endif

  implicit none

#include <comhyb.h>

!  private

  public :: photo_inti
  public :: table_photo
  public :: photo_timestep_init

  save

  real(r8), parameter :: kg2g = 1.e3_r8
  integer, parameter  :: pverm = pver - 1

  integer ::  jno_ndx
  integer ::  jonitr_ndx
  integer ::  jho2no2_ndx
  integer ::  jch3cho_a_ndx, jch3cho_b_ndx, jch3cho_c_ndx
  integer ::  jo2_a_ndx, jo2_b_ndx
  integer ::  ox_ndx, o3_ndx, o3_inv_ndx, o3rad_ndx
  integer ::  n2_ndx, no_ndx, o2_ndx, o_ndx
  integer, allocatable :: lng_indexer(:)
  integer, allocatable :: sht_indexer(:)
  integer, allocatable :: euv_indexer(:)

  integer              :: ki
  integer              :: last
  integer              :: next
  integer              :: nlev
  real(r8)                 :: delp
  real(r8)                 :: dels
  real(r8), allocatable    :: days(:)
  real(r8), allocatable    :: levs(:)
  real(r8), allocatable    :: o2_exo_coldens(:,:,:,:)
  real(r8), allocatable    :: o3_exo_coldens(:,:,:,:)
  logical              :: o_is_inv
  logical              :: o2_is_inv
  logical              :: n2_is_inv
  logical              :: o3_is_inv
  logical              :: no_is_inv
  logical              :: has_o2_col
  logical              :: has_o3_col
  logical              :: has_fixed_press
  logical  :: waccm
  real(r8) :: max_zen_angle       ! degrees

  integer :: jo1d_ndx, jo3p_ndx, jno2_ndx, jn2o5_ndx
  integer :: jhno3_ndx, jno3_ndx, jpan_ndx, jmpan_ndx

  integer :: jo1da_ndx, jo3pa_ndx, jno2a_ndx, jn2o5a_ndx, jn2o5b_ndx
  integer :: jhno3a_ndx, jno3a_ndx, jpana_ndx, jmpana_ndx, jho2no2a_ndx 
  integer :: jonitra_ndx

  integer :: ncid

contains

  subroutine photo_inti( solar_data_file, &
                         xs_coef_file, xs_short_file, xs_long_file, rsf_file, &
                         euvacdat_file, photon_file, electron_file, &
                         exo_coldens_file) 

    !----------------------------------------------------------------------
    !	... initialize photolysis module
    !----------------------------------------------------------------------
    use phys_grid,      only : scatter_field_to_chunk
    use bcc_chem_mods,  only : phtcnt
    use bcc_chem_mods,  only : ncol_abs => nabscol
    use bcc_chem_mods,  only : rxt_tag_lst, pht_alias_lst, pht_alias_mult
    use commap,         only : clat, clon
    use ioFileMod,      only : getfil
    use bcc_jlong,      only : jlong_init
    use bcc_jshort,     only : jshort_init
    use bcc_jeuv,       only : jeuv_init
    use phys_grid,      only : get_ncols_p, get_rlat_all_p    
    use mo_CALENDAR,    only : CALDAYR

    implicit none

    !----------------------------------------------------------------------
    !	... dummy arguments
    !----------------------------------------------------------------------
    character(len=*), intent(in) :: solar_data_file
    character(len=*), intent(in) :: xs_long_file, rsf_file
    character(len=*), intent(in) :: exo_coldens_file

    character(len=*), intent(in) :: xs_coef_file
    character(len=*), intent(in) :: xs_short_file
    character(len=*), intent(in) :: euvacdat_file
    character(len=*), intent(in) :: photon_file
    character(len=*), intent(in) :: electron_file

    logical   :: xactive_prates

    !----------------------------------------------------------------------
    !	... local variables
    !----------------------------------------------------------------------
    real(r8), parameter   :: hPa2Pa = 100._r8
    integer           :: vid 
!---------
    integer           :: k, n, i, j, l
    integer           :: dimid
    integer           :: nlat
    integer           :: ntimes
    integer           :: astat
    integer           :: gndx
    integer           :: ndx
    integer           :: spc_ndx
    integer           :: ierr
    integer           :: c, ncols
    integer, allocatable :: dates(:)
    real(r8)              :: pinterp
    real(r8), allocatable :: lats(:)
    real(r8), allocatable :: coldens(:,:,:)
	real(r8), allocatable :: coldens_temp(:,:,:,:)
    character(len=256)    :: locfn
    character(len=256)    :: filespec
    real(r8), parameter :: trop_thrshld = 1._r8 ! Pa
    real(r8) :: to_lats(pcols)

!------------------------------------------------
    waccm          = .true.
!----
! wtw:  xactive_prates ==
!       If TRUE then use the FTUV method to calculate the photolysis reactions rates,
!       otherwise use the look up table method.
!       Default: FALSE
!------
	xactive_prates = .false.
!------
    if ( waccm ) then
       max_zen_angle = 94._r8   ! ?? degrees
    else
       max_zen_angle = 88.85_r8 ! ?? degrees
    endif

    ! jeuv_1,,, jeuv_25 --> need euv calculations --> must be waccm 
    ! how to determine if shrt calc is needed ?? -- use top level pressure => waccm = true ? false

    !----------------------------------------------------------------------
    !	... allocate indexers
    !----------------------------------------------------------------------
    allocate( lng_indexer(phtcnt),stat=astat )
    allocate( sht_indexer(phtcnt),stat=astat )
    allocate( euv_indexer(phtcnt),stat=astat )

    lng_indexer(:) = 0
    sht_indexer(:) = 0
    euv_indexer(:) = 0

    jno_ndx     = get_rxt_ndx_bcc( 'jno' )
    jo2_a_ndx   = get_rxt_ndx_bcc( 'jo2_a' )
    jo2_b_ndx   = get_rxt_ndx_bcc( 'jo2_b' )

    jo1da_ndx   = get_rxt_ndx_bcc( 'jo1da' )
    jo3pa_ndx   = get_rxt_ndx_bcc( 'jo3pa' )
    jno2a_ndx   = get_rxt_ndx_bcc( 'jno2a' )
    jn2o5a_ndx  = get_rxt_ndx_bcc( 'jn2o5a' )
    jn2o5b_ndx  = get_rxt_ndx_bcc( 'jn2o5b' )
    jhno3a_ndx  = get_rxt_ndx_bcc( 'jhno3a' )
    jno3a_ndx   = get_rxt_ndx_bcc( 'jno3a' )
    jpana_ndx   = get_rxt_ndx_bcc( 'jpana' )
    jmpana_ndx  = get_rxt_ndx_bcc( 'jmpana' )
    jho2no2a_ndx= get_rxt_ndx_bcc( 'jho2no2a' )
    jonitra_ndx = get_rxt_ndx_bcc( 'jonitra' )

    jo1d_ndx    = get_rxt_ndx_bcc( 'jo1d' )
    jo3p_ndx    = get_rxt_ndx_bcc( 'jo3p' )
    jno2_ndx    = get_rxt_ndx_bcc( 'jno2' )
    jn2o5_ndx   = get_rxt_ndx_bcc( 'jn2o5' )
    jn2o5_ndx   = get_rxt_ndx_bcc( 'jn2o5' )
    jhno3_ndx   = get_rxt_ndx_bcc( 'jhno3' )
    jno3_ndx    = get_rxt_ndx_bcc( 'jno3' )
    jpan_ndx    = get_rxt_ndx_bcc( 'jpan' )
    jmpan_ndx   = get_rxt_ndx_bcc( 'jmpan' )
    jho2no2_ndx = get_rxt_ndx_bcc( 'jho2no2' )
    jonitr_ndx  = get_rxt_ndx_bcc( 'jonitr' )

    !----------------------------------------------------------------------
    !	... call module initializers
    !----------------------------------------------------------------------
!
! solar_data_file= atm/cam/solar/spectral_irradiance_Lean_1610-2009_ann_c100405.nc
! xs_long_file = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/temp_prs_GT200nm_jpl06_c080930.nc
! rsf_file     = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/RSF_GT200nm_v3.0_c080811.nc
!
       call jlong_init( solar_data_file, xs_long_file, rsf_file, lng_indexer )

       if (waccm) then
!
! euvacdat_file = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/euvac.dat
! photon_file = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/photon.dat
! electron_file = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/electron.dat 
!
          call jeuv_init ( euvacdat_file, photon_file, electron_file, euv_indexer )
!
! xs_coef_file = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/effxstex.txt
! xs_short_file = /home/rddixon/cesm1_0/inputdata/atm/waccm/phot/xs_short_jpl06_c080930.nc
!
          call jshort_init( xs_coef_file, xs_short_file, sht_indexer )

       endif
       jho2no2_ndx = get_rxt_ndx_bcc( 'jho2no2_b' )

    !----------------------------------------------------------------------
    !        ... check that each photorate is in short or long datasets
    !----------------------------------------------------------------------
    if( any( (abs(sht_indexer(:)) + abs(lng_indexer(:)) + abs(euv_indexer(:))) == 0 ) ) then
       write(*,*) ' '
       write(*,*) 'photo_inti: the following photorate(s) are not in'
       write(*,*) '            either the short or long datasets'
       write(*,*) ' '
       do ndx = 1,phtcnt
          if( abs(sht_indexer(ndx)) + abs(lng_indexer(ndx)) == 0 ) then
             write(*,*) '           ',trim( rxt_tag_lst(ndx) )
          end if
       end do
       call endrun
    end if

    !----------------------------------------------------------------------
    !        ... output any aliased photorates
    !----------------------------------------------------------------------
    if( masterproc ) then
       if( any( pht_alias_lst(:,1) /= ' ' ) ) then
          write(*,*) ' '
          write(*,*) 'photo_inti: the following short photorate(s) are aliased'
          write(*,*) ' '
          do ndx = 1,phtcnt
             if( pht_alias_lst(ndx,1) /= ' ' ) then
                if( pht_alias_mult(ndx,1) == 1. ) then
                   write(*,*) '           ',trim(rxt_tag_lst(ndx)),' -> ',trim(pht_alias_lst(ndx,1))
                else
                   write(*,*) '           ',trim(rxt_tag_lst(ndx)),' -> ',pht_alias_mult(ndx,1),'*',trim(pht_alias_lst(ndx,1))
                end if
             end if
          end do
       end if
       if( any( pht_alias_lst(:,2) /= ' ' ) ) then
          write(*,*) ' '
          write(*,*) 'photo_inti: the following long photorate(s) are aliased'
          write(*,*) ' '
          do ndx = 1,phtcnt
             if( pht_alias_lst(ndx,2) /= ' ' ) then
                if( pht_alias_mult(ndx,2) == 1. ) then
                   write(*,*) '           ',trim(rxt_tag_lst(ndx)),' -> ',trim(pht_alias_lst(ndx,2))
                else
                   write(*,*) '           ',trim(rxt_tag_lst(ndx)),' -> ',pht_alias_mult(ndx,2),'*',trim(pht_alias_lst(ndx,2))
                end if
             end if
          end do
       end if

       write(*,*) ' '
       write(*,*) '*********************************************'
       write(*,*) 'photo_inti: euv_indexer'
       write(*,'(10i6)') euv_indexer(:)
       write(*,*) 'photo_inti: sht_indexer'
       write(*,'(10i6)') sht_indexer(:)
       write(*,*) 'photo_inti: lng_indexer'
       write(*,'(10i6)') lng_indexer(:)
       write(*,*) '*********************************************'
       write(*,*) ' '
    endif

    ox_ndx     = get_spc_ndx( 'OX' )
    if( ox_ndx < 1 ) then
       ox_ndx  = get_spc_ndx( 'O3' )
    end if
    o3_ndx     = get_spc_ndx( 'O3' )
    o3rad_ndx  = get_spc_ndx( 'O3RAD' )
    o3_inv_ndx = get_inv_ndx( 'O3' )

    n2_ndx     = get_inv_ndx( 'N2' )
    n2_is_inv  = n2_ndx > 0
    if( .not. n2_is_inv ) then
       n2_ndx = get_spc_ndx( 'N2' )
    end if
    o2_ndx     = get_inv_ndx( 'O2' )
    o2_is_inv  = o2_ndx > 0
    if( .not. o2_is_inv ) then
       o2_ndx = get_spc_ndx( 'O2' )
    end if
    no_ndx     = get_spc_ndx( 'NO' )
    no_is_inv  = no_ndx < 1
    if( no_is_inv ) then
       no_ndx = get_inv_ndx( 'NO' )
    end if
    o3_is_inv  = o3_ndx < 1

    o_ndx     = get_spc_ndx( 'O' )
    o_is_inv  = o_ndx < 1
    if( o_is_inv ) then
       o_ndx = get_inv_ndx( 'O' )
    end if

    !----------------------------------------------------------------------
    !	... check for o2, o3 absorber columns
    !----------------------------------------------------------------------
    if( ncol_abs > 0 ) then
       spc_ndx = ox_ndx
       if( spc_ndx < 1 ) then
          spc_ndx = o3_ndx
       end if
       if( spc_ndx > 0 ) then
          has_o3_col = .true.
       else
          has_o3_col = .false.
       end if
       if( ncol_abs > 1 ) then
          if( o2_ndx > 1 ) then
             has_o2_col = .true.
          else
             has_o2_col = .false.
          end if
       else
          has_o2_col = .false.
       end if
    else
       has_o2_col = .false.
       has_o3_col = .false.
    end if

    if ( len_trim(exo_coldens_file) == 0 ) then
       has_o2_col = .false.
       has_o3_col = .false.
    endif

    if( has_o2_col .or. has_o3_col ) then
       !-----------------------------------------------------------------------
       !	... open exo coldens file
       !-----------------------------------------------------------------------
    if (masterproc) then

       call getfil( exo_coldens_file, locfn)
       call wrap_open(locfn,0,ncid)

       !-----------------------------------------------------------------------
       !       ... get grid dimensions from file
       !-----------------------------------------------------------------------
       !       ... timing
       !-----------------------------------------------------------------------
       call wrap_inq_dimid (ncid, 'month' , dimid)
       call wrap_inq_dimlen(ncid, dimid, ntimes)

       if( ntimes /= 12 ) then
          call endrun('photo_inti: exo coldens is not annual period')
       end if
  
   end if   !masterproc

#if (defined SPMD )
    call mpibcast(ntimes, 1, mpiint, 0, mpicom )
#endif

       allocate( dates(ntimes),days(ntimes),stat=astat )
       if( astat /= 0 ) then
          write(*,*) 'photo_inti: dates,days allocation error = ',astat
          call endrun
       end if
       dates(:) = (/ 116, 214, 316, 415,  516,  615, &
            716, 816, 915, 1016, 1115, 1216 /)
       !-----------------------------------------------------------------------
       !	... initialize the monthly day of year times
       !-----------------------------------------------------------------------
       do n = 1,ntimes
          days(n) = caldayr( dates(n), 0 )
       end do
       deallocate( dates )

   if (masterproc) then
       !-----------------------------------------------------------------------
       !       ... latitudes
       !-----------------------------------------------------------------------
       call wrap_inq_dimid (ncid, 'lat' , dimid)
       call wrap_inq_dimlen(ncid, dimid , nlat)
   end if   !masterproc

#if (defined SPMD )
    call mpibcast(nlat, 1, mpiint, 0, mpicom )
#endif
!       allocate( lats(nlat), stat=astat )
!       if( astat /= 0 .or. nlat /= plat ) then
!          write(*,*) 'photo_inti: lats allocation error = ',astat
!          call endrun
!       end if
!       call wrap_inq_varid(ncid, 'lat'    , vid)
!       call wrap_get_var_realx(ncid, vid, lats )
!       lats(:nlat) = lats(:nlat) * d2r
       !-----------------------------------------------------------------------
       !       ... levels
       !-----------------------------------------------------------------------
   if (masterproc) then
       call wrap_inq_dimid (ncid, 'lev' , dimid)
       call wrap_inq_dimlen(ncid, dimid , nlev )
 
   end if   !masterproc

#if (defined SPMD )
    call mpibcast(nlev, 1, mpiint, 0, mpicom )
#endif
       allocate( levs(nlev), stat=astat )
       if( astat /= 0 ) then
          write(*,*) 'photo_inti: levs allocation error = ',astat
          call endrun
       end if

    if (masterproc) then
       call wrap_inq_varid(ncid, 'lev'   , vid)
       call wrap_get_var_realx(ncid, vid, levs )
       levs(:nlev) = levs(:nlev) * hPa2Pa
     end if

#if (defined SPMD )
    call mpibcast(levs, nlev, mpir8, 0, mpicom )
#endif
       !-----------------------------------------------------------------------
       !       ... set up regridding
       !-----------------------------------------------------------------------

       allocate( coldens(nlat,nlev,ntimes),stat=astat )
       if( astat /= 0 ) then
          write(*,*) 'photo_inti: coldens allocation error = ',astat
          call endrun
       end if

       if( has_o2_col ) then
          allocate( o2_exo_coldens(pcols,nlev,begchunk:endchunk,ntimes),stat=astat )
          if( astat /= 0 ) then
             write(*,*) 'photo_inti: o2_exo_coldens allocation error = ',astat
             call endrun
          end if
	  allocate( coldens_temp(plon, nlev, plat, ntimes) )

         if (masterproc) then
             call wrap_inq_varid(ncid, 'O2_column_density'   , vid)
             call wrap_get_var_realx(ncid, vid, coldens )
             
			 do i = 1, plon
			 do j = 1, plat
			 do k = 1, nlev
			 do l = 1, ntimes
			     coldens_temp(i,k,j,l) = coldens(j,k,l)
		         end do
			 end do
			 end do
			 end do

          end if   !masterproc

          do l = 1, ntimes
             call scatter_field_to_chunk(1,nlev,1,plon,coldens_temp(:,:,:,l),o2_exo_coldens(:,:,:,l))
          enddo
	  deallocate(coldens_temp)

       end if

       if( has_o3_col ) then
          allocate( o3_exo_coldens(pcols,nlev,begchunk:endchunk,ntimes),stat=astat )
          if( astat /= 0 ) then
             write(*,*) 'photo_inti: o3_exo_coldens allocation error = ',astat
             call endrun
          end if

	  allocate( coldens_temp(plon, nlev, plat, ntimes))

          if (masterproc) then

             call wrap_inq_varid( ncid, 'O3_column_density'   , vid)
             call wrap_get_var_realx( ncid, vid, coldens )
            
			 do i = 1, plon
			 do j = 1, plat
			 do k = 1, nlev
			 do l = 1, ntimes
			     coldens_temp(i,k,j,l) = coldens(j,k,l)
		     end do
			 end do
			 end do
			 end do

          end if   !masterproc

          do l = 1, ntimes
             call scatter_field_to_chunk(1,nlev,1,plon,coldens_temp(:,:,:,l),o3_exo_coldens(:,:,:,l))
          enddo
	  deallocate(coldens_temp)
       endif

       if (masterproc) then
          call wrap_close (ncid)
       end if   !masterproc

       deallocate( coldens,stat=astat )
       if( astat /= 0 ) then
          write(*,*) 'photo_inti: failed to deallocate coldens; error = ',astat
          call endrun
       end if

       has_fixed_press = hybi(1) == 0._r8
       !-----------------------------------------------------------------------
       !	... setup the pressure interpolation
       !-----------------------------------------------------------------------
       if( has_fixed_press ) then
          pinterp = ps0 * hyai(1)
          if( pinterp <= levs(1) ) then
             ki   = 1
             delp = 0._r8
          else
             do ki = 2,nlev
                if( pinterp <= levs(ki) ) then
                   delp = log( pinterp/levs(ki-1) )/log( levs(ki)/levs(ki-1) )
                   exit
                end if
             end do
          end if
          write(*,*) '-----------------------------------'
          write(*,*) 'photo_inti: diagnostics'
          write(*,*) 'ki, delp = ',ki,delp
          write(*,*) 'pinterp,levs(ki-1:ki) = ',pinterp,levs(ki-1:ki)
          write(*,*) '-----------------------------------'
       end if

    end if
  end subroutine photo_inti

  subroutine table_photo( photos, pmid, pdel, temper, zmid, zint, &
                          col_dens, zen_angle, srf_alb, lwc, clouds, &
                          esfact, vmr, invariants, ncol, lchnk )
!-----------------------------------------------------------------
!   	... table photorates for wavelengths > 200nm
!-----------------------------------------------------------------

    use bcc_chem_mods,   only : ncol_abs => nabscol, phtcnt
    use mo_grid,         only : pcnstm1
    use mo_chem_mods,    only : nfs, indexm
    use bcc_chem_mods,   only : pht_alias_mult
    use bcc_jshort,      only : nsht => nj, jshort
    use bcc_jlong,       only : nlng => numj, jlong
    use bcc_jeuv,        only : neuv, jeuv

    implicit none

!-----------------------------------------------------------------
!   	... dummy arguments
!-----------------------------------------------------------------
    integer,  intent(in)    :: lchnk
    integer,  intent(in)    :: ncol
    real(r8), intent(in)    :: esfact                       ! earth sun distance factor
    real(r8), intent(in)    :: vmr(pcols,pver,max(1,pcnstm1)) ! vmr
    real(r8), intent(in)    :: col_dens(pcols,pver,ncol_abs) ! column densities (molecules/cm^2)
    real(r8), intent(in)    :: zen_angle(pcols)              ! solar zenith angle (radians)
    real(r8), intent(in)    :: srf_alb(pcols)               ! surface albedo
    real(r8), intent(in)    :: lwc(pcols,pver)               ! liquid water content (kg/kg)
    real(r8), intent(in)    :: clouds(pcols,pver)            ! cloud fraction
    real(r8), intent(in)    :: pmid(pcols,pver)             ! midpoint pressure (Pa)
    real(r8), intent(in)    :: pdel(pcols,pver)             ! pressure delta about midpoint (Pa)
    real(r8), intent(in)    :: temper(pcols,pver)           ! midpoint temperature (K)
    real(r8), intent(in)    :: zmid(pcols,pver)              ! midpoint height (km)
    real(r8), intent(in)    :: zint(pcols,pver)              ! interface height (km)
    real(r8), intent(in)    :: invariants(pcols,pver,max(1,nfs)) ! invariant densities (molecules/cm^3)
    real(r8), intent(out)   :: photos(pcols,pver,phtcnt)     ! photodissociation rates (1/s)

!-----------------------------------------------------------------
!    	... local variables
!-----------------------------------------------------------------
    real(r8), parameter :: Pa2mb         = 1.e-2_r8       ! pascals to mb

    integer ::  i, k, m, n                    ! indicies
    integer ::  astat
    real(r8) ::  sza
    real(r8) ::  alias_factor
    real(r8) ::  zarg(pver)
    real(r8) ::  fac1(pver)                ! work space for j(no) calc
    real(r8) ::  fac2(pver)                ! work space for j(no) calc
    real(r8) ::  colo3(pver)               ! vertical o3 column density
    real(r8) ::  parg(pver)                ! vertical pressure array (hPa)
    real(r8) ::  tline(pver)               ! vertical temperature array
    real(r8) ::  cld_line(pver)            ! vertical cloud array
    real(r8) ::  lwc_line(pver)            ! vertical lwc array
    real(r8) ::  eff_alb(pver)             ! effective albedo from cloud modifications
    real(r8) ::  cld_mult(pver)            ! clould multiplier
    real(r8) ::  tmp(pcols,pver)            ! wrk array
    real(r8), allocatable ::  lng_prates(:,:) ! photorates matrix (1/s)
    real(r8), allocatable ::  sht_prates(:,:) ! photorates matrix (1/s)
    real(r8), allocatable ::  euv_prates(:,:) ! photorates matrix (1/s)
    
    real(r8) :: o_den(pver)               ! o density (molecules/cm^3)
    real(r8) :: o2_den(pver)              ! o2 density (molecules/cm^3)
    real(r8) :: o3_den(pver)              ! o3 density (molecules/cm^3)
    real(r8) :: no_den(pver)              ! no density (molecules/cm^3)
    real(r8) :: n2_den(pver)              ! n2 density (molecules/cm^3)
    real(r8) :: jno_sht(pver)             ! no short photorate
    real(r8) :: jo2_sht(pver,2)        ! o2 short photorate

!-----------------------------------------------------------------
!	... allocate short, long temp arrays
!-----------------------------------------------------------------
    if ( waccm ) then
       if (nsht>0) then
          allocate( sht_prates(pver,nsht),stat=astat )
          if( astat /= 0 ) then
             write(*,*) 'photo: Failed to allocate sht_prates; error = ',astat
             call endrun
          end if
       endif
       if (neuv>0) then
          allocate( euv_prates(pver,neuv),stat=astat )
          if( astat /= 0 ) then
             write(*,*) 'photo: Failed to allocate euv_prates; error = ',astat
             call endrun
          end if
       endif
    endif

    if (nlng>0) then
       allocate( lng_prates(nlng,pver),stat=astat )
       if( astat /= 0 ) then
          write(*,*) 'photo: Failed to allocate lng_prates; error = ',astat
          call endrun
       end if
    endif

!-----------------------------------------------------------------
!	... zero all photorates
!-----------------------------------------------------------------
    do m = 1,max(1,phtcnt)
       do k = 1,pver
          photos(:,k,m) = 0._r8
       end do
    end do

    col_loop : do i = 1,ncol
       sza = zen_angle(i)*r2d
       daylight : if( sza >= 0._r8 .and. sza < max_zen_angle ) then
          parg(:)     = Pa2mb*pmid(i,:)
          colo3(:)    = col_dens(i,:,1)
          fac1(:)     = pdel(i,:)
          tline(:)    = temper(i,:)
          lwc_line(:) = lwc(i,:)
          cld_line(:) = clouds(i,:)



          if ( waccm ) then
             zarg(:)   = zmid(i,:)

             if ( o_is_inv ) then
                o_den(:) = invariants(i,:,o_ndx)
             else
                o_den(:) = vmr(i,:,o_ndx) * invariants(i,:,indexm)
             endif
             if ( o2_is_inv ) then
                o2_den(:) = invariants(i,:,o2_ndx)
             else
                o2_den(:) = vmr(i,:,o2_ndx) * invariants(i,:,indexm)
             endif
             if ( o3_is_inv ) then
                o3_den(:) = invariants(i,:,o3_ndx)
             else
                o3_den(:) = vmr(i,:,o3_ndx) * invariants(i,:,indexm)
             endif
             if ( n2_is_inv ) then
                n2_den(:) = invariants(i,:,n2_ndx)
             else
                n2_den(:) = vmr(i,:,n2_ndx) * invariants(i,:,indexm)
             endif
             if ( no_is_inv ) then
                no_den(:) = invariants(i,:,no_ndx)
             else
                no_den(:) = vmr(i,:,no_ndx) * invariants(i,:,indexm)
             endif

             !-----------------------------------------------------------------
             !	... short wave length component
             !-----------------------------------------------------------------
             call jshort( pver, sza, n2_den, o2_den, o3_den, &
                          no_den, tline, zarg, jo2_sht, jno_sht, sht_prates )  
             do m = 1,phtcnt
                if( sht_indexer(m) > 0 ) then
                   alias_factor = pht_alias_mult(m,1)
                   if( alias_factor == 1. ) then
                      photos(i,pver:1:-1,m) = sht_prates(:,sht_indexer(m))
                   else
                      photos(i,pver:1:-1,m) = alias_factor * sht_prates(:,sht_indexer(m))
                   end if
                end if
             end do
             photos(i,pver:1:-1,jno_ndx)   = jno_sht(:)
             photos(i,pver:1:-1,jo2_a_ndx) = jo2_sht(:,2)
             photos(i,pver:1:-1,jo2_b_ndx) = jo2_sht(:,1)

             !-----------------------------------------------------------------
             !	... euv photorates do not include cloud effects ??
             !-----------------------------------------------------------------
             call jeuv( pver, sza, o_den, o2_den, n2_den,  zarg, euv_prates )
             do m = 1,phtcnt
                if( euv_indexer(m) > 0 ) then
                   photos(i,:,m) = esfact * euv_prates(:,euv_indexer(m))
                endif
             enddo


          endif
          !-----------------------------------------------------------------
          !     ... compute eff_alb and cld_mult -- needs to be before jlong
          !-----------------------------------------------------------------
          call cloud_mod( zen_angle(i), cld_line, lwc_line, fac1, srf_alb(i), &
                          eff_alb, cld_mult )
          cld_mult(:) = esfact * cld_mult(:)

          !-----------------------------------------------------------------
          !	... long wave length component
          !-----------------------------------------------------------------
          call jlong( pver, sza, eff_alb, parg, tline, colo3, lng_prates )          
          do m = 1,phtcnt
             if( lng_indexer(m) > 0 ) then
                alias_factor = pht_alias_mult(m,2)
                if( alias_factor == 1. ) then
                   photos(i,:,m) = (photos(i,:,m) + lng_prates(lng_indexer(m),:))*cld_mult(:)
                else
                   photos(i,:,m) = (photos(i,:,m) + alias_factor * lng_prates(lng_indexer(m),:))*cld_mult(:)
                end if
             end if
          end do

          !-----------------------------------------------------------------
          !	... calculate j(no) from formula
          !-----------------------------------------------------------------
          if( jno_ndx > 0 ) then
             if( has_o2_col .and. has_o3_col ) then
                fac1(:) = 1.e-8_r8 * (abs(col_dens(i,:,2)/cos(zen_angle(i))))**.38_r8
                fac2(:) = 5.e-19_r8 * abs(col_dens(i,:,1)/cos(zen_angle(i)))
                photos(i,:,jno_ndx) = photos(i,:,jno_ndx) + 4.5e-6_r8 * exp( -(fac1(:) + fac2(:)) )
             end if
          end if
          !-----------------------------------------------------------------
          ! 	... add near IR correction to ho2no2
          !-----------------------------------------------------------------
          if( jho2no2_ndx > 0 ) then
             photos(i,:,jho2no2_ndx) = photos(i,:,jho2no2_ndx) + 1.e-5_r8*cld_mult(:)
          endif

       end if daylight
    end do col_loop

    if ( allocated(lng_prates) ) deallocate( lng_prates )
    if ( allocated(sht_prates) ) deallocate( sht_prates )
    if ( allocated(euv_prates) ) deallocate( euv_prates )

    call set_xnox_photo( photos, ncol  )

  end subroutine table_photo


  subroutine cloud_mod( zen_angle, clouds, lwc, delp, srf_alb, &
                        eff_alb, cld_mult )
    !-----------------------------------------------------------------------
    ! 	... cloud alteration factors for photorates and albedo
    !-----------------------------------------------------------------------

    implicit none

    !-----------------------------------------------------------------------
    ! 	... dummy arguments
    !-----------------------------------------------------------------------
    real(r8), intent(in)    ::  zen_angle         ! zenith angle
    real(r8), intent(in)    ::  srf_alb           ! surface albedo
    real(r8), intent(in)    ::  clouds(pver)       ! cloud fraction
    real(r8), intent(in)    ::  lwc(pver)          ! liquid water content (mass mr)
    real(r8), intent(in)    ::  delp(pver)         ! del press about midpoint in pascals
    real(r8), intent(out)   ::  eff_alb(pver)      ! effective albedo
    real(r8), intent(out)   ::  cld_mult(pver)     ! photolysis mult factor

    !-----------------------------------------------------------------------
    ! 	... local variables
    !-----------------------------------------------------------------------
    integer :: k
    real(r8)    :: coschi
    real(r8)    :: del_lwp(pver)
    real(r8)    :: del_tau(pver)
    real(r8)    :: above_tau(pver)
    real(r8)    :: below_tau(pver)
    real(r8)    :: above_cld(pver)
    real(r8)    :: below_cld(pver)
    real(r8)    :: above_tra(pver)
    real(r8)    :: below_tra(pver)
    real(r8)    :: fac1(pver)
    real(r8)    :: fac2(pver)

    real(r8), parameter :: rgrav = 1._r8/9.80616_r8

    !---------------------------------------------------------
    !	... modify lwc for cloud fraction and form
    !	    liquid water path for each layer
    !---------------------------------------------------------
    where( clouds(:) /= 0._r8 )
       del_lwp(:) = rgrav * lwc(:) * delp(:) * 1.e3_r8 / clouds(:)
    elsewhere
       del_lwp(:) = 0._r8
    endwhere
    !---------------------------------------------------------
    !    	... form tau for each model layer
    !---------------------------------------------------------
    where( clouds(:) /= 0._r8 )
       del_tau(:) = del_lwp(:) *.155_r8 * clouds(:)**1.5_r8
    elsewhere
       del_tau(:) = 0._r8
    end where
    !---------------------------------------------------------
    !    	... form integrated tau from top down
    !---------------------------------------------------------
    above_tau(1) = 0._r8
    do k = 1,pverm
       above_tau(k+1) = del_tau(k) + above_tau(k)
    end do
    !---------------------------------------------------------
    !    	... form integrated tau from bottom up
    !---------------------------------------------------------
    below_tau(pver) = 0._r8
    do k = pverm,1,-1
       below_tau(k) = del_tau(k+1) + below_tau(k+1)
    end do
    !---------------------------------------------------------
    !	... form vertically averaged cloud cover above and below
    !---------------------------------------------------------
    above_cld(1) = 0._r8
    do k = 1,pverm
       above_cld(k+1) = clouds(k) * del_tau(k) + above_cld(k)
    end do
    do k = 2,pver
       if( above_tau(k) /= 0._r8 ) then
          above_cld(k) = above_cld(k) / above_tau(k)
       else
          above_cld(k) = above_cld(k-1)
       end if
    end do
    below_cld(pver) = 0._r8
    do k = pverm,1,-1
       below_cld(k) = clouds(k+1) * del_tau(k+1) + below_cld(k+1)
    end do
    do k = pverm,1,-1
       if( below_tau(k) /= 0._r8 ) then
          below_cld(k) = below_cld(k) / below_tau(k)
       else
          below_cld(k) = below_cld(k+1)
       end if
    end do
    !---------------------------------------------------------
    !	... modify above_tau and below_tau via jfm
    !---------------------------------------------------------
    where( above_cld(2:pver) /= 0._r8 )
       above_tau(2:pver) = above_tau(2:pver) / above_cld(2:pver)
    end where
    where( below_cld(:pverm) /= 0._r8 )
       below_tau(:pverm) = below_tau(:pverm) / below_cld(:pverm)
    end where
    where( above_tau(2:pver) < 5._r8 )
       above_cld(2:pver) = 0._r8
    end where
    where( below_tau(:pverm) < 5._r8 )
       below_cld(:pverm) = 0._r8
    end where
    !---------------------------------------------------------
    !	... form transmission factors
    !---------------------------------------------------------
    above_tra(:) = 11.905_r8 / (9.524_r8 + above_tau(:))
    below_tra(:) = 11.905_r8 / (9.524_r8 + below_tau(:))
    !---------------------------------------------------------
    !	... form effective albedo
    !---------------------------------------------------------
    where( below_cld(:) /= 0._r8 )
       eff_alb(:) = srf_alb + below_cld(:) * (1._r8 - below_tra(:)) &
                  * (1._r8 - srf_alb)
    elsewhere
       eff_alb(:) = srf_alb
    end where
    coschi = max( cos( zen_angle ),.5_r8 )
    where( del_lwp(:)*.155_r8 < 5._r8 )
       fac1(:) = 0._r8
    elsewhere
       fac1(:) = 1.4_r8 * coschi - 1._r8
    end where
    fac2(:)     = min( 0._r8,1.6_r8*coschi*above_tra(:) - 1._r8 )
    cld_mult(:) = 1._r8 + fac1(:) * clouds(:) + fac2(:) * above_cld(:)
    cld_mult(:) = max( .05_r8,cld_mult(:) )

  end subroutine cloud_mod

  subroutine photo_timestep_init( calday )
    use time_manager,    only : is_end_curr_day
    use bcc_euvac,       only : euvac_set_etf
    use bcc_solar_parms, only : get_solar_parms

    !-----------------------------------------------------------------------------
    !	... setup the time interpolation
    !-----------------------------------------------------------------------------

    implicit none

    !-----------------------------------------------------------------------------
    !	... dummy arguments
    !-----------------------------------------------------------------------------
    real(r8), intent(in)    ::  calday                                   ! day of year at end of present time step

    !-----------------------------------------------------------------------------
    !	... local variables
    !-----------------------------------------------------------------------------
    integer :: m
    real(r8) :: f107
    real(r8) :: f107a

    if (waccm) then
       if( is_end_curr_day() ) then
          call get_solar_parms( f107_s = f107, f107a_s = f107a )
          call euvac_set_etf( f107, f107a )
       end if
    endif

    if( has_o2_col .or. has_o3_col ) then
       if( calday < days(1) ) then
          next = 1
          last = 12
          dels = (365._r8 + calday - days(12)) / (365._r8 + days(1) - days(12))
       else if( calday >= days(12) ) then
          next = 1
          last = 12
          dels = (calday - days(12)) / (365._r8 + days(1) - days(12))
       else
          do m = 11,1,-1
             if( calday >= days(m) ) then
                exit
             end if
          end do
          last = m
          next = m + 1
          dels = (calday - days(m)) / (days(m+1) - days(m))
       end if
#ifdef DEBUG
       write(*,*) '-----------------------------------'
       write(*,*) 'photo_timestep_init: diagnostics'
       write(*,*) 'calday, last, next, dels = ',calday,last,next,dels
       write(*,*) '-----------------------------------'
#endif
    end if

  end subroutine photo_timestep_init

  !--------------------------------------------------------------------------
  !--------------------------------------------------------------------------
  subroutine set_xnox_photo( photos, ncol )
    use bcc_chem_mods,    only : ncol_abs => nabscol, phtcnt 
    use mo_chem_mods,     only : nfs
    use mo_grid,          only : pcnstm1
    implicit none

    real(r8), intent(inout) :: photos(pcols,pver,phtcnt)     ! photodissociation rates (1/s)
    integer, intent(in)     :: ncol

    if( jno2a_ndx > 0 .and. jno2_ndx > 0 ) then
       photos(:,:,jno2a_ndx) = photos(:,:,jno2_ndx)
    end if
    if( jn2o5a_ndx > 0 .and. jn2o5_ndx > 0 ) then
       photos(:,:,jn2o5a_ndx) = photos(:,:,jn2o5_ndx)
    end if
    if( jn2o5b_ndx > 0 .and. jn2o5_ndx > 0 ) then
       photos(:,:,jn2o5b_ndx) = photos(:,:,jn2o5_ndx)
    end if
    if( jhno3a_ndx > 0 .and. jhno3_ndx > 0 ) then
       photos(:,:,jhno3a_ndx) = photos(:,:,jhno3_ndx)
    end if

    if( jno3a_ndx > 0 .and. jno3_ndx > 0 ) then
       photos(:,:,jno3a_ndx) = photos(:,:,jno3_ndx)
    end if
    if( jho2no2a_ndx > 0 .and. jho2no2_ndx > 0 ) then
       photos(:,:,jho2no2a_ndx) = photos(:,:,jho2no2_ndx)
    end if
    if( jmpana_ndx > 0 .and. jmpan_ndx > 0 ) then
       photos(:,:,jmpana_ndx) = photos(:,:,jmpan_ndx)
    end if
    if( jpana_ndx > 0 .and. jpan_ndx > 0 ) then
       photos(:,:,jpana_ndx) = photos(:,:,jpan_ndx)
    end if
    if( jonitra_ndx > 0 .and. jonitr_ndx > 0 ) then
       photos(:,:,jonitra_ndx) = photos(:,:,jonitr_ndx)
    end if
    if( jo1da_ndx > 0 .and. jo1d_ndx > 0 ) then
       photos(:,:,jo1da_ndx) = photos(:,:,jo1d_ndx)
    end if
    if( jo3pa_ndx > 0 .and. jo3p_ndx > 0 ) then
       photos(:,:,jo3pa_ndx) = photos(:,:,jo3p_ndx)
    end if

  endsubroutine set_xnox_photo

end module bcc_photo
