#include <misc.h>
#include <params.h>

module bcc_timestep_init

      implicit none
      private

      public :: timestep_init

      save
    
contains

  subroutine timestep_init (phys_state)

    use shr_kind_mod,     only : r8 => shr_kind_r8
    use ppgrid,           only : begchunk, endchunk
    use physics_types,    only : physics_state
    use time_manager,     only : get_curr_calday

    use bcc_jshort,       only : jshort_timestep_init
    use bcc_jlong,        only : jlong_timestep_init
    use bcc_photo,        only : photo_timestep_init
    use bcc_solar_data,   only : solar_data_advance
    use bcc_solar_parms,  only : solar_parms_timestep_init

    implicit none

    type(physics_state), intent(inout) :: phys_state(begchunk:endchunk)                 
!
! Local variables
!
    real(r8) :: calday

    !-----------------------------------------------------------------------
    ! get current calendar day of year
    !-----------------------------------------------------------------------
    calday = get_curr_calday( )

    !---------------------
     call solar_data_advance()

    !-----------------------------------------------------------------------
    ! Set solar parameters
    !-----------------------------------------------------------------------
    call solar_parms_timestep_init

    !-----------------------------------------------------------------------------
    !	... setup the time interpolation for mo_photo
    !-----------------------------------------------------------------------------
    call photo_timestep_init( calday )

    !-----------------------------------------------------------------------
    ! Set jlong etf
    !-----------------------------------------------------------------------
    call jlong_timestep_init

    !-----------------------------------------------------------------------
    ! Set jshort etf
    !-----------------------------------------------------------------------
    call jshort_timestep_init

  end subroutine timestep_init

end module bcc_timestep_init
