      module mo_chem_mods
!--------------------------------------------------------------
! ... basic chemistry array parameters
!--------------------------------------------------------------
      use mo_grid,       only : pcnstm1
      implicit none

      save

      integer, parameter :: hetcnt = 26, & ! number of heterogeneous processes
                            phtcnt = 33, & ! number of photo processes
                            rxntot = 168, & ! number of total reactions
                            gascnt = 135, & ! number of gas phase reactions
                            nfs = 4, & ! number of "fixed" species
                            relcnt = 0, & ! number of relationship species
                            grpcnt = 3, & ! number of group members
                            imp_nzcnt = 587, & ! number of non-zero implicit matrix entries
                            rod_nzcnt = 0, & ! number of non-zero rodas matrix entries
                            extcnt = 3, & ! number of species with external forcing
                            clscnt1 = 8, & ! number of species in explicit class
                            clscnt2 = 0, & ! number of species in hov class
                            clscnt3 = 0, & ! number of species in ebi class
                            clscnt4 = 55, & ! number of species in implicit class
                            clscnt5 = 0, & ! number of species in rodas class
                            indexm = 1, & ! index of total atm density in invariant array
                            ncol_abs = 2, & ! number of column densities
                            indexh2o = 4, & ! index of water vapor density
                            clsze = 4 ! loop length for implicit chemistry

      integer :: ngrp = 0
      integer :: drydep_cnt = 0
      integer :: srfems_cnt = 0
      integer :: rxt_alias_cnt = 0
      integer, allocatable :: grp_mem_cnt(:)
      integer, allocatable :: rxt_alias_map(:)

      real :: adv_mass(pcnstm1)
      real :: nadv_mass(grpcnt)

      character(len=16), allocatable :: rxt_alias_lst(:)
      character(len=8), allocatable :: drydep_lst(:)
      character(len=8), allocatable :: srfems_lst(:)
      character(len=8), allocatable :: grp_lst(:)
      character(len=8) :: het_lst(max(1,hetcnt))
      character(len=8) :: extfrc_lst(max(1,extcnt))
      character(len=8) :: inv_lst(max(1,nfs))

      type solver_class
      integer :: clscnt
      integer :: lin_rxt_cnt
      integer :: nln_rxt_cnt
      integer :: indprd_cnt
      integer :: iter_max
      integer :: cls_rxt_cnt(4)
      integer, pointer :: permute(:)
      integer, pointer :: diag_map(:)
      integer, pointer :: clsmap(:)
      end type solver_class

      type(solver_class) :: explicit, implicit, rodas

      contains

      subroutine chem_mods_inti
!--------------------------------------------------------------
! ... intialize the class derived type
!--------------------------------------------------------------
      use abortutils,     only: endrun   !add by zf 2008.05.28
      implicit none

      integer :: astat

      explicit%clscnt = 8
      explicit%indprd_cnt = 28
      implicit%clscnt = 55
      implicit%lin_rxt_cnt = 71
      implicit%nln_rxt_cnt = 113
      implicit%indprd_cnt = 3
      implicit%iter_max = 11
      rodas%clscnt = 0
      rodas%lin_rxt_cnt = 0
      rodas%nln_rxt_cnt = 0
      rodas%indprd_cnt = 0

      if( explicit%clscnt > 0 ) then
        allocate( explicit%clsmap(explicit%clscnt),stat=astat )
        if( astat /= 0 ) then
          write(*,*) 'chem_mods_inti: failed to allocate explicit%clsmap ; error = ',astat
          call endrun
        end if
        explicit%clsmap(:) = 0
      end if

      if( implicit%clscnt > 0 ) then

        allocate( implicit%permute(implicit%clscnt),stat=astat )
        if( astat /= 0 ) then
          write(*,*) 'chem_mods_inti: failed to allocate implicit%permute ; error = ',astat
          call endrun
        end if
        implicit%permute(:) = 0

        allocate( implicit%diag_map(implicit%clscnt),stat=astat )
        if( astat /= 0 ) then
          write(*,*) 'chem_mods_inti: failed to allocate implicit%diag_map ; error = ',astat
          call endrun
        end if
        implicit%diag_map(:) = 0

        allocate( implicit%clsmap(implicit%clscnt),stat=astat )
        if( astat /= 0 ) then
          write(*,*) 'chem_mods_inti: failed to allocate implicit%clsmap ; error = ',astat
          call endrun
        end if
        implicit%clsmap(:) = 0

      end if

      if( rodas%clscnt > 0 ) then

        allocate( rodas%permute(rodas%clscnt),stat=astat )
        if( astat /= 0 ) then
          write(*,*) 'chem_mods_inti: failed to allocate rodas%permute ; error = ',astat
          call endrun
        end if
        rodas%permute(:) = 0

        allocate( rodas%diag_map(rodas%clscnt),stat=astat )
        if( astat /= 0 ) then
          write(*,*) 'chem_mods_inti: failed to allocate rodas%diag_map ; error = ',astat
          call endrun
        end if
        rodas%diag_map(:) = 0

        allocate( rodas%clsmap(rodas%clscnt),stat=astat )
        if( astat /= 0 ) then
          write(*,*) 'chem_mods_inti: failed to allocate rodas%clsmap ; error = ',astat
          call endrun
        end if
        rodas%clsmap(:) = 0

      end if

      end subroutine chem_mods_inti
      end module mo_chem_mods
