      module mo_chemdr

      implicit none

      private
      public :: chemdr

      save

      real :: esfact = 1.           ! earth sun distance factor

      contains

      subroutine chemdr( ztodt   , mmr    , ps     , pmid   , pdel  ,         &
                         tsurf  , zma    , zi     , phis  ,         &
                         cldfr  , cldtop , cmfdqr , nrain ,         &
                         nevapr , cwat   , tfld   , sh    ,         &
                         lchnk  , ncol   , plonl  , nas   ,         &
                         em3data, nem3   , aerosul    ,&
                         em3_IPCC6       , nem3_IPCC6,              & 
                         asdir , troplev , prod_no )

!-----------------------------------------------------------------------
!     ... Chem_solver advances the volumetric mixing ratio
!         forward one time step via a combination of explicit,
!         ebi, hov, fully implicit, and/or rodas algorithms.
!-----------------------------------------------------------------------

      use time_manager,  only : get_curr_calday, is_perpetual, get_perp_date, get_curr_date
      use ppgrid,        only : pver, pverp
      use physconst,     only: gravit
      use mo_tracname,   only : tracnam
      use mo_grid,       only : pcnstm1, plnplv 
      use mo_chem_mods,  only : indexm, adv_mass, nadv_mass, phtcnt, gascnt, rxntot, clscnt1,   &
                                clscnt4, clscnt5,ncol_abs, grpcnt, nfs, extcnt, hetcnt
      use mo_photo,      only : set_ub_col, setcol, diurnal_geom, sundis, photo
      use mo_chem_utls,  only : inti_mr_xform, mmr2vmr, adjh2o, get_spc_ndx, get_grp_mem_ndx,vmr2mmr
      use mo_setinv,     only : setinv
      use mo_setrxt,     only : setrxt
      use mo_sulf,       only : sulf_interp
      use mo_usrrxt,     only : usrrxt
      use mo_adjrxt,     only : adjrxt
      use mo_srfalb,     only : srfalb
      use mo_phtadj,     only : phtadj
      use mo_sethet,     only : sethet
      use mo_setext,     only : setext
      use mo_grp_ratios, only : set_grp_ratios
      use mo_rxt_mod,    only : rxt_mod
      use mo_exp_sol,    only : exp_sol
      use mo_imp_sol,    only : imp_sol
      use mo_ub_vals,    only : set_ub_vals
      use mo_make_grp_vmr,only : mak_grp_vmr
      use constituents,  only : cnst_name
      use history,       only : outfld  
      use phys_grid,     only : get_rlat_all_p, get_rlon_all_p
      use chemistry,     only : ixchm

      use mo_exp_prod_loss_others, only : exp_prod_loss_others
      use time_manager,  only : get_nstep
      use bcc_photo,     only : table_photo
      use bcc_chem_mods, only : phtcnt_bcc => phtcnt
      use bcc_setsox,           only : setsox
      use bcc_aerosols_intr,    only : do_cam_sulfchem 
      use bcc_aero_em3_prod,    only : aero_em3_prod
      use prescribed_ch4,       only : getch4   !zftest 20160722

      implicit none

!-----------------------------------------------------------------------
!        ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) ::  plonl
      integer, intent(in) ::  lchnk
      integer, intent(in) ::  ncol 
      real              ::    qh2o(plonl, pver)       ! specific humidity( kg/kg )
      real, intent(in)  ::    ps(plonl)               ! surface press ( pascals )
      real, intent(in)  ::    tsurf(plonl)            ! surface temperature
      real, intent(in)  ::    phis(plonl)             ! surf geopot
      real, intent(in)  ::    cldtop(plonl)           ! cloud top level ( 1 ... pver )
      real, intent(in)  ::    pmid(plonl,pver)        ! midpoint press ( pascals )
      real, intent(in)  ::    pdel(plonl,pver)        ! delta press across midpoints
      real, intent(in)  ::    zma(plonl,pver)         ! abs geopot height at midpoints ( m )
      real, intent(in)  ::    cldfr(plonl,pver)       ! cloud fraction
      real, intent(in)  ::    cmfdqr(plonl,pver)      ! dq/dt for convective rainout
      real, intent(in)  ::    nrain(plonl,pver)       ! release of strt precip ( 1/s )
      real, intent(in)  ::    nevapr(plonl,pver)      ! evap precip ( 1/s )
      real, intent(in)  ::    cwat(plonl,pver)        ! total cloud water (kg/kg)
      real, intent(in)  ::    tfld(plonl,pver)        ! midpoint temperature
      real, intent(in)  ::    sh(plonl,pver)          ! specific humidity ( kg/kg )
      real, intent(in)  ::    zi(plonl,pverp)         ! abs geopot height at interfaces ( m )
      real, intent(in)  ::    prod_no(plonl,pver)

      real, intent(inout) ::  mmr(plonl,pver,pcnstm1) ! xported species ( mmr )
      integer , intent(in) :: nem3, nem3_IPCC6
      real, intent(in)  ::    em3data(plonl,pver, nem3) 
      real, intent(in)  ::    em3_IPCC6(plonl,pver, nem3_IPCC6)
      
      integer,intent(in) :: troplev(plonl) 
!----
!  wtw 2012.10
!
      real,    intent(in) :: aerosul(plonl,pver)   ! mmr

      real                ::  reaction_rates_bcc(plonl,pver,phtcnt_bcc)
!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer  ::  ncdate, yr, mon, day, ncsec
      integer  ::  i, k, m, n, hndx, file
      integer  ::  ox_ndx, o3_ndx

      real     ::  ztodt                  ! timestep in seconds
      real     ::  sunon(plonl)    
      real     ::  sunoff(plonl)    
      real     ::  caldayn               ! day of year at end of time step
      real     ::  invariants(plonl,pver,max(1,nfs))
      real     ::  group_ratios(plonl,pver,max(1,grpcnt))
      real     ::  group_vmr(plonl,pver,max(1,grpcnt))
      real     ::  col_dens(plonl,pver,max(1,ncol_abs))    ! column densities (molecules/cm^2)
      real     ::  col_delta(plonl,0:pver,max(1,ncol_abs)) ! layer column densities (molecules/cm^2)
      real     ::  het_rates(plonl,pver,max(1,hetcnt))
      real     ::  extfrc(plonl,pver,max(1,extcnt))
      real     ::  vmr(plonl,pver,pcnstm1)                 ! xported species ( vmr )
      real     ::  reaction_rates(plonl,pver,rxntot)
      real     ::  nas(plonl,pver,max(1,grpcnt))           ! non-advected species( mmr )
      real     ::  h2ovmr(plonl,pver)    ! water vapor volume mixing ratio
      real     ::  mbar(plonl,pver)      ! mean wet atmospheric mass ( amu )
      real     ::  zmid(plonl,pver)      ! midpoint geopotential in km
      real     ::  sulfate(plonl,pver)   ! sulfate aerosols
      real     ::  zint(plonl,pverp)     ! interface geopotential in km
      real     ::  zen_angle(plonl)
      real     ::  loc_angle(plonl)
      real     ::  albs(plonl)
      real     ::  clat(plonl)           ! current latitudes(radians)
      real     ::  clon(plonl)           ! current longitudes(radians)
      logical  ::  polar_night(plonl), &
                   polar_day(plonl)

      real     ::  het_rates_so2(plonl,pver)
      real     ::  het_rates_oc2(plonl,pver)
      real     ::  het_rates_cb2(plonl,pver)
      real     ::  het_rates_so4(plonl,pver)
      integer  :: nstep
      integer  :: co2_ndx, ch4_ndx
      integer  :: dms_ndx, so2_ndx, so4_ndx, sslt01_ndx, sslt02_ndx, sslt03_ndx, sslt04_ndx, & 
                  dst01_ndx, dst02_ndx, dst03_ndx, dst04_ndx, oc1_ndx, oc2_ndx, cb1_ndx, cb2_ndx
      real     :: asdir(plonl)                    ! albedo: shortwave, direct
      real     :: o3vmr(plonl,pver)   ! CMIP5 O3 vmr
      real     :: ch4(plonl,pver)     ! CESM CH4 vmr  zftest 20160722

      real :: sflux(plonl)
      real :: wetdep_so2_vmr(plonl,pver)
      real :: chem_dms_vmr(plonl,pver)
      real :: chem_so2_vmr(plonl,pver)
      real :: chem_so4_vmr(plonl,pver)
      real :: chem_cb1_vmr(plonl,pver)
      real :: chem_cb2_vmr(plonl,pver)
      real :: chem_oc1_vmr(plonl,pver)
      real :: chem_oc2_vmr(plonl,pver)
      real :: emis_so2_dms(plonl,pver)
      real :: aqu_gas_so2(plonl,pver)
      real :: aqu_gas_so4(plonl,pver)
      real :: emis_so2_vmr_d3(plonl,pver)
      real :: emis_cb2_vmr_d3(plonl,pver)
      real :: emis_so4_vmr_d3(plonl,pver)
      real ::  emis_cb1_vmr_d3(plonl,pver)
      real ::  emis_oc1_vmr_d3(plonl,pver)
      real ::  emis_oc2_vmr_d3(plonl,pver)
      real ::  emis_nh3_vmr_d3(plonl,pver)

!----------------------------------------------------------------------------

      plnplv = plonl*pver
      caldayn = get_curr_calday()

      if ( is_perpetual() ) then
        call get_perp_date(yr, mon, day, ncsec)
      else
        call get_curr_date(yr, mon, day, ncsec)
      end if
      ncdate = yr*10000 + mon*100 + day

      if( phtcnt /= 0 ) then
!-----------------------------------------------------------------------
!        ... Calculate parameters for diurnal geometry
!-----------------------------------------------------------------------
        call get_rlat_all_p(lchnk, ncol, clat)    !add by zf 2008.07.02
        call get_rlon_all_p(lchnk, ncol, clon)    !add by zf 2008.08.07

        do i = 1, plonl
          call diurnal_geom( clon(i), clat(i), caldayn, polar_night(i), polar_day(i), &
                             sunon(i), sunoff(i), loc_angle(i),  zen_angle(i) )
        enddo

      end if

!-----------------------------------------------------------------------
!        ... Initialize xform between mass and volume mixing ratios
!-----------------------------------------------------------------------
      call inti_mr_xform( sh, mbar, plonl )

!--------------------------------------------
!  initial CO2
!------------------------------------
      nstep = get_nstep()
      co2_ndx =  get_spc_ndx( 'CO2' )
      ox_ndx  =  get_spc_ndx( 'OX' )

      dms_ndx =  get_spc_ndx( 'DMS' )
      so2_ndx =  get_spc_ndx( 'SO2' )
      so4_ndx =  get_spc_ndx( 'SO4' )
      sslt01_ndx =  get_spc_ndx( 'SSLT01' )
      sslt02_ndx =  get_spc_ndx( 'SSLT02' )
      sslt03_ndx =  get_spc_ndx( 'SSLT03' )
      sslt04_ndx =  get_spc_ndx( 'SSLT04' )
      dst01_ndx =  get_spc_ndx( 'DST01' )
      dst02_ndx =  get_spc_ndx( 'DST02' )
      dst03_ndx =  get_spc_ndx( 'DST03' )
      dst04_ndx =  get_spc_ndx( 'DST04' )
      oc1_ndx =  get_spc_ndx( 'OC1' )
      oc2_ndx =  get_spc_ndx( 'OC2' )
      cb1_ndx =  get_spc_ndx( 'CB1' )
      cb2_ndx =  get_spc_ndx( 'CB2' )

!-----------------------------------------------------------------------
!        ... Xform from mmr to vmr
!-----------------------------------------------------------------------
      call mmr2vmr( vmr, mmr, mbar, plonl )

   !  if( nstep < 10*365*72 ) then
   !      ch4_ndx =  get_spc_ndx( 'CH4' )
   !      call getch4( lchnk, plonl, ch4)    !zftest 20160722
   !      vmr(:,:,ch4_ndx) = ch4(:,:)*(1. - nstep/(10.*365.*72.))   + vmr(:,:,ch4_ndx)*(nstep/(10.*365.*72.))  !test2 
   !      vmr(:,:,co2_ndx) = 285.e-6
   !   endif
!-----------------------------------------------------------------------
!        ... Xform water vapor from mmr to vmr and adjust in stratosphere
!-----------------------------------------------------------------------
      call adjh2o( h2ovmr, sh, mbar, vmr, plonl )
!-----------------------------------------------------------------------
!        ... Xform geopotential height from m to km
!            and pressure from hPa to mb
!-----------------------------------------------------------------------
      do k = 1,pver
         zmid(:,k) = 1.e-3 * zma(:,k)
         zint(:,k) = 1.e-3 * zi(:,k)
      end do
      zint(:,pverp) = 1.e-3 * zi(:,pverp)

      if( nfs > 0 ) then
!-----------------------------------------------------------------------
!        ... Set the "invariants"
!-----------------------------------------------------------------------
         call setinv( invariants, tfld, h2ovmr, pmid, plonl )
      end if

      if( ncol_abs > 0 .and. phtcnt > 0 ) then
!-----------------------------------------------------------------------
!        ... Xform family ox assuming that all ox is o3
!-----------------------------------------------------------------------
         ox_ndx = get_spc_ndx( 'OX' )
         if( ox_ndx > 0 ) then
            o3_ndx = get_grp_mem_ndx( 'O3' )
            if( o3_ndx > 0 ) then
               vmr(:,:,ox_ndx) = mbar(:,:) * mmr(:,:,ox_ndx) / nadv_mass(o3_ndx)

     !if( nstep < 10*365*72 ) then
     !    call radozn(lchnk   ,plonl    ,pmid    ,o3vmr   )
     !    vmr(:,:,ox_ndx)  = o3vmr(:,:)*(1. - nstep/(10.*365.*72.)) + vmr(:,:,ox_ndx)*(nstep/(10.*365.*72.))  !zftest 20160804
     !end if

            end if
         end if

!-----------------------------------------------------------------------
!        ... Set the column densities at the upper boundary
!-----------------------------------------------------------------------
         call set_ub_col( col_delta, vmr, pdel, plonl )
      end if

      if( gascnt > 0 ) then
!-----------------------------------------------------------------------
!       ...  Set rates for "tabular" and user specified reactions
!-----------------------------------------------------------------------
         call setrxt( reaction_rates, tfld, invariants(1,1,indexm), plonl )

!         call sulf_interp( pmid, caldayn, sulfate, plonl, lchnk )
!---
! wtw  2012.10
!
         sulfate(:,1:pver) = aerosul(:,1:pver) * mbar(:,1:pver) / adv_mass(so4_ndx)
                              ! mmr -> vmr 
!---
         call usrrxt( reaction_rates, tfld, invariants, h2ovmr, ps, &
                      pmid, invariants(1,1,indexm), sulfate, plonl, ncol )

         call adjrxt( reaction_rates, invariants, invariants(1,1,indexm), plnplv )
       end if

      if( phtcnt > 0 ) then
!-----------------------------------------------------------------------
!        ... Compute the photolysis rates at time = t(n+1)
!-----------------------------------------------------------------------
           esfact = sundis( ncdate )
            if( ncol_abs > 0 ) then
!-----------------------------------------------------------------------
!       ... Set the column densities
!-----------------------------------------------------------------------
               call setcol( col_delta, col_dens, vmr, pdel, plonl )
            end if
!-----------------------------------------------------------------------
!       ... Calculate the surface albedo
!-----------------------------------------------------------------------
            call srfalb( albs, caldayn, tsurf, plonl, lchnk, ncol)
!-----------------------------------------------------------------------
!       ... Calculate the photodissociation rates
!-----------------------------------------------------------------------
            call photo( reaction_rates, pmid, pdel, tfld, zmid, &
                        col_dens, zen_angle, albs, cwat, cldfr, &
                        esfact, plonl )
         do i = 1, plonl
           if( polar_night(i) ) then
             do m = 1, phtcnt
               reaction_rates(i,:,m) = 0. 
             end do
           end if
         end do
!-----------------------------------------------------------------------
!       ... Adjust the photodissociation rates
!-----------------------------------------------------------------------
         call phtadj( reaction_rates, invariants, invariants(1,1,indexm), plnplv )
      end if

      if( hetcnt > 0 ) then
!-----------------------------------------------------------------------
!        ... Compute the heterogeneous rates at time = t(n+1)
!-----------------------------------------------------------------------
         call sethet( het_rates, pmid, zmid, phis, &
                      tfld, cmfdqr, nrain, nevapr, ztodt, &
                      invariants(1,1,indexm), vmr, plonl ,lchnk, &
                      het_rates_so2, het_rates_oc2, het_rates_cb2, het_rates_so4)
      end if

!-----------------------------------------------------------------------
!       ...  History output for time averaged wet removal rates
!-----------------------------------------------------------------------
      call outfld('H2O2WD  ',het_rates(1,1,1), plonl, lchnk)
      call outfld('HNO3WD  ',het_rates(1,1,2), plonl, lchnk)
      call outfld('CH2OWD  ',het_rates(1,1,3), plonl, lchnk)
      call outfld('CH3OOWD ',het_rates(1,1,4), plonl, lchnk)
      call outfld('POOHWD  ',het_rates(1,1,5), plonl, lchnk)
      call outfld('CH3COWD ',het_rates(1,1,6), plonl, lchnk)
      call outfld('HO2N2WD ',het_rates(1,1,7), plonl, lchnk)
      call outfld('ONITWD  ',het_rates(1,1,8), plonl, lchnk)
      call outfld('MVKWD   ',het_rates(1,1,9), plonl, lchnk)
      call outfld('MACRWD  ',het_rates(1,1,10), plonl, lchnk)
      call outfld('C2H5OWD ',het_rates(1,1,11), plonl, lchnk)
      call outfld('C3H7HWD ',het_rates(1,1,12), plonl, lchnk)
      call outfld('ROOHWD  ',het_rates(1,1,13), plonl, lchnk)
      call outfld('CH3CnWD ',het_rates(1,1,14), plonl, lchnk)
      call outfld('PbWD    ',het_rates(1,1,15), plonl, lchnk)
      call outfld('MACRHWD ',het_rates(1,1,16), plonl, lchnk)
      call outfld('XOOHWD  ',het_rates(1,1,17), plonl, lchnk)
      call outfld('ONITRWD ',het_rates(1,1,18), plonl, lchnk)
      call outfld('ISOPHWD ',het_rates(1,1,19), plonl, lchnk)
      call outfld('GLYADWD ',het_rates(1,1,22), plonl, lchnk)
      call outfld('HYACWD  ',het_rates(1,1,23), plonl, lchnk)
      call outfld('CH3OHWD ',het_rates(1,1,20), plonl, lchnk)
      call outfld('C2H5HWD ',het_rates(1,1,21), plonl, lchnk)
      call outfld('HYDRDWD ',het_rates(1,1,24), plonl, lchnk)

      if( extcnt > 0 ) then
!-----------------------------------------------------------------------
!        ... Compute the extraneous frcing at time = t(n+1)
!-----------------------------------------------------------------------
         call setext( extfrc, zint, cldtop, plonl, lchnk  ,&
                      em3_IPCC6,    nem3_IPCC6,            &
                      em3data, nem3 , prod_no)
         
         call outfld('NOEXTFR ', extfrc(:,:,1), plonl, lchnk)
         call outfld('CH4EXTFR', extfrc(:,:,3), plonl, lchnk)
         call outfld('COEXTFR ', extfrc(:,:,2), plonl, lchnk)

         do m = 1,max(1,extcnt)
            do k = 1,pver
               extfrc(:,k,m) = extfrc(:,k,m) / invariants(:,k,indexm)
            end do
         end do
      end if
      if( grpcnt > 0 ) then
!-----------------------------------------------------------------------
!        ... Set the group ratios
!-----------------------------------------------------------------------
         call set_grp_ratios( group_ratios, reaction_rates, vmr, mmr, nas, &
                              mbar, invariants, plonl )
!-----------------------------------------------------------------------
!       ... Modify the reaction rate of any reaction
!           with group member or proportional reactant(s)
!-----------------------------------------------------------------------
         call rxt_mod( reaction_rates, het_rates, group_ratios, plnplv )
      end if

!=======================================================================
!        ... Call the class solution algorithms
!=======================================================================
      if( clscnt1 > 0 .and. rxntot > 0 ) then
!-----------------------------------------------------------------------
!       ... Solve for "explicit" species
!-----------------------------------------------------------------------
         call exp_sol( vmr, reaction_rates, het_rates, extfrc, &
                       ztodt, invariants(1,1,indexm), plonl, plnplv )
      end if

      if( clscnt4 > 0 .and. rxntot > 0 ) then
!-----------------------------------------------------------------------
!       ... Solve for "Implicit" species
!-----------------------------------------------------------------------
         call imp_sol( vmr, reaction_rates, het_rates, extfrc, &
                       ztodt, invariants(1,1,indexm), plonl, plnplv )
      end if

!-----------------------------------------------------------------------
!         ... Check for negative values and reset to zero
!-----------------------------------------------------------------------
!zf      call negtrc( lat, 'After chemistry ', vmr, plonl )
!-----------------------------------------------------------------------
!         ... Set values near upper boundary
!-----------------------------------------------------------------------

      call radozn(lchnk   ,ncol    ,pmid    ,o3vmr   )
      call set_ub_vals( ztodt, lchnk, vmr, pmid, zmid, &
                        tfld, caldayn, plonl , troplev, o3vmr )

!-----------------------------------------------------------------------
!         ... Output time averaged "wet" non-advected volume mixing ratios
!-----------------------------------------------------------------------
      call mak_grp_vmr( vmr, group_ratios(1,1,1), group_vmr(1,1,1), plonl )

      call outfld('O3      ' ,group_vmr(:,:,1), plonl, lchnk)
      call outfld('O1D     ' ,group_vmr(:,:,2), plonl, lchnk)
      call outfld('O       ' ,group_vmr(:,:,3), plonl, lchnk)

!-----------------------------------------------------
!    Calculate CO2 and other aerosols 
!---------------------------------------------------
      call table_photo( reaction_rates_bcc, pmid, pdel, tfld, zmid, zint, &
                       col_dens, zen_angle, asdir, cwat, cldfr, &
                       esfact, vmr, invariants, ncol, lchnk )

!--
! 3 D emiss (SO2, CB2, SO4)

      call aero_em3_prod ( em3data, nem3, invariants(1,1,indexm), plonl, ztodt, vmr ,&
                           em3_IPCC6,    nem3_IPCC6,                                 &
                           emis_cb1_vmr_d3, emis_oc1_vmr_d3,                         &
                           emis_oc2_vmr_d3, emis_nh3_vmr_d3,                         &
                           emis_so2_vmr_d3, emis_cb2_vmr_d3, emis_so4_vmr_d3)

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + emis_so2_vmr_d3(i,k)/ztodt  * adv_mass(so2_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo 
      call outfld('EM3D_SO2 ' ,sflux , plonl, lchnk)
         
      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + emis_cb2_vmr_d3(i,k)/ztodt  * adv_mass(cb2_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo 
      call outfld('EM3D_CB2 ' ,sflux , plonl, lchnk)
         
      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + emis_so4_vmr_d3(i,k)/ztodt  * adv_mass(so4_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo
      call outfld('EM3D_SO4 ' ,sflux , plonl, lchnk)

!----------------------------------
      call exp_prod_loss_others ( reaction_rates_bcc, tfld, invariants(1,1,indexm), plonl, ztodt, vmr ,&
                                  het_rates_so2, wetdep_so2_vmr,        &
                                  chem_dms_vmr, chem_so2_vmr, chem_so4_vmr, chem_cb1_vmr, chem_cb2_vmr,     &
                                  chem_oc1_vmr, chem_oc2_vmr, emis_so2_dms)

!-------------------------

      sflux(:) = 0.0   
      do k=1, pver
         do i=1, plonl 
            sflux(i) = sflux(i) + wetdep_so2_vmr(i,k)/ztodt  * adv_mass(so2_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo
      call outfld('WET_SO2 ' ,sflux , plonl, lchnk)

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
             sflux(i) = sflux(i) + emis_so2_dms(i,k)/ztodt  * adv_mass(so2_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s

         enddo
      enddo
      call outfld('DMS_SO2 ' ,sflux, plonl, lchnk)

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + chem_DMS_vmr(i,k) /ztodt * adv_mass(dms_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo
      call outfld('CHEM_DMS' ,sflux, plonl, lchnk)

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + chem_so2_vmr(i,k)/ztodt * adv_mass(so2_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo
      call outfld('CHEM_SO2' ,sflux, plonl, lchnk)

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + chem_so4_vmr(i,k) /ztodt * adv_mass(so4_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s 
         enddo
      enddo 
      call outfld('CHEM_SO4' ,sflux, plonl, lchnk)

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + chem_cb1_vmr(i,k) /ztodt * adv_mass(cb1_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo
      call outfld('CHEM_CB1' ,sflux, plonl, lchnk)

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + chem_cb2_vmr(i,k) /ztodt * adv_mass(cb2_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo
      call outfld('CHEM_CB2' ,sflux, plonl, lchnk)

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + chem_oc1_vmr(i,k) /ztodt * adv_mass(oc1_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo
      call outfld('CHEM_OC1' ,sflux, plonl, lchnk)

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + chem_oc2_vmr(i,k) /ztodt * adv_mass(oc2_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo
      call outfld('CHEM_OC2' ,sflux, plonl, lchnk)
!-------------------------------------------------------------------
! ....  aerosol packages
!------------------------------------------------------------------------

      if( .not. do_cam_sulfchem ) then  ! .false.
          qh2o(:,:) = sh(:,:)

          call setsox( plonl,  ncol,   &
                       pmid,   &
                       ztodt,   &
                       tfld,   &
                       qh2o,   &
                       cwat,   &
                       invariants(1,1,indexm), &
                       vmr, invariants,        &
                       aqu_gas_so2, aqu_gas_so4 )
      endif

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + aqu_gas_so2(i,k) /ztodt * adv_mass(so2_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo
      call outfld('AQU_SO2 ', sflux, plonl, lchnk )   

      sflux(:) = 0.0
      do k=1, pver
         do i=1, plonl
            sflux(i) = sflux(i) + aqu_gas_so4(i,k) /ztodt * adv_mass(so4_ndx)/mbar(i,k) &
                              *pdel(i,k)/gravit   ! kg/m2/s
         enddo
      enddo  
      call outfld('AQU_SO4 ', sflux, plonl, lchnk )   
!-----------------------------------------------------------------------
!         ... Output time averaged "wet" advected volume mixing ratios
!-----------------------------------------------------------------------
      do m=1, pcnstm1
        call outfld(cnst_name(m+ixchm-1), vmr(:,:,m), plonl, lchnk )
      end do

!-----------------------------------------------------------------------
!         ... Xform from vmr to mmr
!-----------------------------------------------------------------------
      call vmr2mmr( vmr, mmr, nas, group_ratios, mbar, plonl )

      end subroutine chemdr

      end module mo_chemdr
