module mo_exp_prod_loss_others

      implicit none

      private
      public :: exp_prod_loss_others
      
      contains
      subroutine exp_prod_loss_others ( rxt, temp, m, ncol, delt, vmr, &
                                        het_rates_so2, wetdep_so2_vmr, &
                                        chem_dms_vmr, chem_so2_vmr, chem_so4_vmr,   &
                                        chem_cb1_vmr, chem_cb2_vmr,     &
                                        chem_oc1_vmr, chem_oc2_vmr, emis_so2_dms )

!----------------------------------------------------------------------------------
!  Authors:  Wu Tongwen   
!------------------------------------------------------------------------------------
      use mo_jpl,       only : jpl
      use pmgrid,       only : plev
      use mo_chem_utls, only : get_spc_ndx
      use mo_grid,      only : pcnstm1
      use bcc_chem_mods,only : phtcnt
      use shr_kind_mod, only : r8 => shr_kind_r8

      implicit none
!--------------------------------------------------------------------
! ... Dummy args
!--------------------------------------------------------------------
      integer, intent(in) :: ncol
      real,    intent(in) :: delt
      real,    intent(in) :: temp(ncol, plev)
      real,    intent(in) :: m(ncol, plev)
      real,    intent(in) :: het_rates_so2(ncol, plev)
      real,    intent(inout) :: vmr(ncol,plev,pcnstm1)
      real  :: vmr_co2(ncol,plev)
      real  :: vmr_nh3(ncol,plev)
      real  :: vmr_so2(ncol,plev)
      real  :: vmr_so4(ncol,plev)
      real  :: vmr_cb1(ncol,plev)
      real  :: vmr_cb2(ncol,plev)
      real  :: vmr_oc1(ncol,plev)
      real  :: vmr_oc2(ncol,plev)
      real  :: vmr_dms(ncol,plev)

      real,   intent(out) :: wetdep_so2_vmr(ncol,plev)
      real,   intent(out) :: chem_dms_vmr(ncol,plev)
      real,   intent(out) :: chem_so2_vmr(ncol,plev)
      real,   intent(out) :: chem_so4_vmr(ncol,plev)
      real,   intent(out) :: chem_cb1_vmr(ncol,plev)
      real,   intent(out) :: chem_cb2_vmr(ncol,plev)
      real,   intent(out) :: chem_oc1_vmr(ncol,plev)
      real,   intent(out) :: chem_oc2_vmr(ncol,plev)
      real,   intent(out) :: emis_so2_dms(ncol,plev)

      real,    intent(in) :: rxt(ncol,plev,phtcnt)  
!-------------------------------------------------------
!       ... local variables
!-------------------------------------------------------
      integer  :: co2_ndx, ch4_ndx, oh_ndx, co_ndx, no3_ndx, dms_ndx
      integer  :: so2_ndx, so4_ndx, cb1_ndx, cb2_ndx, oc1_ndx, oc2_ndx
      integer  :: nh3_ndx

      integer  :: k, i
      real     :: t0

      real, dimension(ncol,plev) ::                    &
               tend_co2, tend_so2, tend_so4, tend_dms,  &
               tend_cb1, tend_cb2, tend_oc1, tend_oc2,  &
               tend_nh3

      real, dimension(ncol,plev) :: &
                itemp,               &
                rxt46,  rxt48,  rxt137, rxt138, rxt165, rxt167,  rxt164, rxt166, &
		rxt185, rxt184, rxt190, rxt189,                                  &
		loss_co2, prod_co2, loss_dms, prod_dms, loss_so4, prod_so4,      &
		loss_so2, prod_so2, loss_cb1, prod_cb1, loss_cb2, prod_cb2,      &
		loss_oc1, prod_oc1, loss_oc2, prod_oc2, loss_nh3, prod_nh3

     real, dimension(ncol) :: ko,     kinf,   term1,  term2,  fc,     exp_fac

     real :: AA(ncol), BB(ncol)

!===================================================================================     

     co2_ndx = get_spc_ndx( 'CO2' )
     ch4_ndx = get_spc_ndx( 'CH4' )
     oh_ndx  = get_spc_ndx( 'OH' )
     co_ndx  = get_spc_ndx( 'CO' )
     no3_ndx = get_spc_ndx( 'NO3' )
     dms_ndx = get_spc_ndx( 'DMS' )
     so2_ndx = get_spc_ndx( 'SO2' )
     so4_ndx = get_spc_ndx( 'SO4' )
     cb1_ndx = get_spc_ndx( 'CB1' )
     cb2_ndx = get_spc_ndx( 'CB2' )
     oc1_ndx = get_spc_ndx( 'OC1' )
     oc2_ndx = get_spc_ndx( 'OC2' )
     nh3_ndx = get_spc_ndx( 'NH3' )

!--------------------------------------------
      do k=1, plev
            itemp(:,k) = 1._r8 / temp(:,k)

!----------------------------------------
!
!    d(CO2)/dt = .44*j48*CH4  + r64*M*CO*OH  + r65*CO*OH                                                                 
!                 - j46*CO2 
!
!-------------------
!???            rxt46(:,k) = 1.1e-10_r8
!???        rxt48(:,k) = 2.2e-11_r8 * exp(120._r8 * itemp(:,k))
!???            rxt48(:,k) = 0.0_r8

            rxt46(:,k) = rxt(:,k,46)
            rxt48(:,k) = rxt(:,k,48)

            ko(:) = 5.90e-33_r8 * itemp(:,k)**1.4_r8
            kinf(:) = 1.10e-12_r8 * itemp(:,k)**(-1.3_r8)

            call jpl( rxt137(:,k), m(:,k), 0.6_r8, ko(:), kinf(:), ncol )
 	    rxt137(:,k) = rxt137(:,k) * m(:,k) * m(:,k)

        !-----------------------------------------------------------------
        ! 	... co + oh --> co2 + h (second branch JPL06; pg2.2; 2.10) WACCM
        !-----------------------------------------------------------------

           t0     = 300._r8                ! K
           kinf(:)  = 2.1e+09_r8 * (temp(:,k)/ t0)**(6.1_r8)
           ko  (:)  = 1.5e-13_r8 * (temp(:,k)/ t0)**(0.6_r8)

           term1(:) = ko(:) / ( (kinf(:) / m(:,k)) )
           term2(:) = ko(:) / (1._r8 + term1(:))

           term1(:) = log10( term1(:) )
           term1(:) = 1.0_r8 / (1.0_r8 + term1(:)*term1(:))

           rxt138(:,k) = term2(:) * (0.6_r8)**term1(:)         ! usr_CO_OH_b_ndx = 138
           rxt138(:,k) = rxt138(:,k) * m(:,k)

           loss_co2(:,k) = rxt46(:,k) * vmr(:,k,co2_ndx)
           prod_co2(:,k) = (rxt137(:,k) * vmr(:,k,oh_ndx) + rxt138(:,k)*vmr(:,k,oh_ndx)) * &
                            vmr(:,k,co_ndx)  +.440_r8*rxt48(:,k) * vmr(:,k,ch4_ndx)
           tend_co2(:,k) = prod_co2(:,k) - loss_co2(:,k)
           vmr_co2(:,k) = vmr(:,k,co2_ndx) + tend_co2(:,k) * delt

!--------------------------------------------------------
!  d(DMS)/dt = -r165*OH*DMS  - r166*OH*DMS  - r167*NO3*DMS     
!  d(SO2)/dt =  r165*DMS*OH  + .5*r166*DMS*OH  + r167*DMS*NO3    
!                 - r164*OH*SO2  
!  d(SO4)/dt =  r185*SO2*OH        ! SO4 in gas       
!-------------------------------------------
           rxt165(:,k) = 9.60E-12*exp(   -234.* itemp(:,k)) 
           rxt165(:,k) = rxt165(:,k) * m(:,k)

           rxt167(:,k) = 1.90E-13*exp( 520.* itemp(:,k)) 
           rxt167(:,k) = rxt167(:,k) * m(:,k)

!-----------------------------------------------------------------
!       ... SO2 + OH  --> SO4  (REFERENCE?? - not Liao)
!-----------------------------------------------------------------
           fc(:) = 3.0e-31_r8 *(300._r8*itemp(:,k))**3.3_r8
           ko(:) = fc(:)*m(:,k)/(1._r8 + fc(:)*m(:,k)/1.5e-12_r8) 

           rxt164(:,k) = ko(:)*.6_r8**(1._r8 + (log10(fc(:)*m(:,k)/1.5e-12_r8))**2._r8)**(-1._r8)
           rxt164(:,k) = rxt164(:,k) * m(:,k)              ! usr_SO2_OH_ndx

!-----------------------------------------------------------------
!       ... DMS + OH  --> .5 * SO2
!-----------------------------------------------------------------

          call comp_exp( exp_fac(:), 7460._r8*itemp(:,k), ncol )
          ko(:) = 1._r8 + 5.5e-31_r8 * exp_fac(:) * m(:,k) * 0.21_r8
          call comp_exp( exp_fac(:), 7810._r8*itemp(:,k), ncol )

          rxt166(:,k) = 1.7e-42_r8 * exp_fac(:) * m(:,k) * 0.21_r8 / ko(:)
          rxt166(:,k) = rxt166(:,k) * m(:,k)

!-----------------------------------------------------------------
!       ... SO2 + OH  --> SO4  (REFERENCE?? - not Liao)
!-----------------------------------------------------------------
          fc(:) = 3.0e-31_r8 *(300._r8*itemp(:,k))**3.3_r8
          ko(:) = fc(:)*m(:,k)/(1._r8 + fc(:)*m(:,k)/1.5e-12_r8) 
          rxt185(:,k) = ko(:)*.6_r8**(1._r8 + (log10(fc(:)*m(:,k)/1.5e-12_r8))**2._r8)**(-1._r8)
          rxt185(:,k) = rxt185(:,k) * m(:,k)
!---------
          prod_DMS(:,k) = 0.0_r8
	  loss_DMS(:,k) = rxt165(:,k) * vmr(:,k,oh_ndx) * vmr(:,k,dms_ndx) + &
                          rxt166(:,k) * vmr(:,k,oh_ndx) * vmr(:,k,dms_ndx)  &
	                + rxt167(:,k) *vmr(:,k,no3_ndx) * vmr(:,k,dms_ndx) 
          tend_DMS(:,k) = prod_DMS(:,k)- loss_DMS(:,k)
          vmr_dms(:,k) = vmr(:,k,dms_ndx) + tend_DMS(:,k) * delt
!----------
          prod_SO2(:,k) = rxt165(:,k) * vmr(:,k,dms_ndx)*vmr(:,k,oh_ndx) + &
                         .5_r8 * rxt166(:,k)*vmr(:,k,dms_ndx)*vmr(:,k,oh_ndx)  &
		         +rxt167(:,k) * vmr(:,k,dms_ndx)*vmr(:,k,no3_ndx) 
	  loss_so2(:,k) = rxt164(:,k)*vmr(:,k,oh_ndx)*vmr(:,k,so2_ndx) 

          tend_SO2(:,k) = prod_SO2(:,k)- loss_SO2(:,k)
          vmr_so2(:,k) = vmr(:,k,SO2_ndx) + tend_SO2(:,k) * delt

!----------
          prod_SO4(:,k) = rxt185(:,k) * vmr(:,k,so2_ndx) * vmr(:,k,oh_ndx)
          loss_SO4(:,k) = 0._r8
          tend_SO4(:,k) = prod_SO4(:,k)- loss_SO4(:,k)
          vmr_so4(:,k) = vmr(:,k,SO4_ndx) + tend_SO4(:,k) * delt
!
!-------------------------------------------------
!  d(CB1)/dt =  - r184*CB1                                                                                             
!  d(CB2)/dt =   r184*CB1                                                                                              
!  d(OC1)/dt =  - r190*OC1                                                                                             
!  d(OC2)/dt =  r190*OC1 
!  d(NH3)/dt = -r189*OH*NH3
!---------------------------------------

          rxt189(:,k) = 1.70e-12_r8 * exp( -710._r8 * itemp(:,k) )  
                          !   rate(:,:,259) in pp_trop_strat_bam_v1
          rxt189(:,k) =  rxt189(:,k) * m(:,k) 

          prod_NH3(:,k) = 0._r8
          loss_NH3(:,k) = rxt189(:,k) * vmr(:,k,oh_ndx) * vmr(:,k,nh3_ndx)
          tend_NH3(:,k) = prod_NH3(:,k)- loss_NH3(:,k)
          vmr_nh3(:,k) = vmr(:,k,NH3_ndx) + tend_NH3(:,k) * delt

!----------

          rxt184(:,k) = 7.10E-06    ! rate(:,:,254) in pp_trop_strat_bam_v1
          rxt190(:,k) = 7.10E-06 

          prod_CB1(:,k) = 0._r8
	  loss_CB1(:,k) = rxt184(:,k) * vmr(:,k,cb1_ndx)
          tend_CB1(:,k) = prod_CB1(:,k)- loss_CB1(:,k)
          vmr_cb1(:,k) = vmr(:,k,CB1_ndx) + tend_CB1(:,k) * delt

!----------

	  prod_CB2(:,k) = rxt184(:,k) * vmr(:,k,cb1_ndx)
          loss_CB2(:,k) = 0._r8
          tend_CB2(:,k) = prod_CB2(:,k)- loss_CB2(:,k)
          vmr_cb2(:,k) = vmr(:,k,CB2_ndx) + tend_CB2(:,k) * delt

!----------
           
          prod_OC1(:,k) = 0._r8
          loss_OC1(:,k) = rxt190(:,k) * vmr(:,k,oc1_ndx)
          tend_OC1(:,k) = prod_OC1(:,k)- loss_OC1(:,k)
          vmr_oc1(:,k) = vmr(:,k,OC1_ndx) + tend_OC1(:,k) * delt

!----------

	  prod_OC2(:,k) = rxt190(:,k) * vmr(:,k,oc1_ndx)
          loss_OC2(:,k) = 0._r8
          tend_OC2(:,k) = prod_OC2(:,k)- loss_OC2(:,k)
          vmr_oc2(:,k) = vmr(:,k,OC2_ndx) + tend_OC2(:,k) * delt

!--------------------2015-05-29---------------------------------------

          emis_so2_dms(:,k) =  prod_SO2(:,k) * delt
          chem_so2_vmr(:,k) =  vmr_so2(:,k) - emis_so2_dms(:,k) - vmr(:,k,so2_ndx)

          chem_dms_vmr(:,k) = vmr_dms(:,k) -  vmr(:,k,dms_ndx)
          chem_so4_vmr(:,k) = vmr_so4(:,k) -  vmr(:,k,so4_ndx)
          chem_cb1_vmr(:,k) = vmr_cb1(:,k) -  vmr(:,k,cb1_ndx)
          chem_cb2_vmr(:,k) = vmr_cb2(:,k) -  vmr(:,k,cb2_ndx)
          chem_oc1_vmr(:,k) = vmr_oc1(:,k) -  vmr(:,k,oc1_ndx)
          chem_oc2_vmr(:,k) = vmr_oc2(:,k) -  vmr(:,k,oc2_ndx)
!--
! SO2 wet deposition
!
          wetdep_so2_vmr(:,k) = - het_rates_so2(:,k)*vmr(:,k,so2_ndx) * delt
          vmr(:,k,so2_ndx) = max( vmr_so2(:,k) + wetdep_so2_vmr(:,k), 0._r8)
!--
          vmr(:,k,co2_ndx) = max( vmr_co2(:,k), 0._r8)
          vmr(:,k,dms_ndx) = max( vmr_dms(:,k), 0._r8)
          vmr(:,k,so4_ndx) = max( vmr_so4(:,k), 0._r8)
          vmr(:,k,nh3_ndx) = max( vmr_nh3(:,k), 0._r8)
          vmr(:,k,cb1_ndx) = max( vmr_cb1(:,k), 0._r8)
          vmr(:,k,cb2_ndx) = max( vmr_cb2(:,k), 0._r8)
          vmr(:,k,oc1_ndx) = max( vmr_oc1(:,k), 0._r8)
          vmr(:,k,oc2_ndx) = max( vmr_oc2(:,k), 0._r8)

!--------------------2015-05-29---------------------------------------
      enddo

      end subroutine exp_prod_loss_others
        

      subroutine semi_imp(delt, AA, BB, y, y_1, ncol)
      implicit none
      integer, intent(in) :: ncol
      real, intent(in)    :: AA(ncol), BB(ncol), y(ncol)
      real, intent(in)    :: delt
      real, intent(out)   :: y_1(ncol)
      integer i

!-----------------------------------------------
      do i=1, ncol
         if( abs(1./delt -AA(i)/2.0) > 1.e-28 ) then 
             y_1(i) = ((1./delt +AA(i)/2.0)*y(i) + BB(i))/ &
		           (1./delt -AA(i)/2.0) 
             if( y_1(i) < 0.0) y_1(i) = 0.0
         else
             y_1(i) = y(i) + (AA(i)*y(i) + BB(i))* delt 
         endif  
         if(  y_1(i) < 0.0 ) y_1(i) = 0.0
      enddo

!--------------------------------------
      return
      end  subroutine semi_imp

  subroutine comp_exp( x, y, n )
    use shr_kind_mod, only: r8 => shr_kind_r8
    implicit none

    real(r8), intent(out) :: x(n)
    real(r8), intent(in)  :: y(n)
    integer,  intent(in)  :: n
    integer :: i

!wtw#ifdef IBM
!wtw    call vexp( x, y, n )
!wtw#else
    do i=1, n
       x(i) = exp( y(i) )
    enddo
!wtw#endif

  end subroutine comp_exp


end module mo_exp_prod_loss_others
