
      module mo_exp_sol

      implicit none

      save

      private
      public :: exp_slv_inti, exp_sol

      integer ::  o3s_ndx, o3inert_ndx
      integer ::  oh_ndx, ho2_ndx, c2h4_ndx, c3h6_ndx, isop_ndx, &
                  mvk_ndx, macr_ndx, c10h16_ndx, no2_ndx, n2o5_ndx, &
                  no3_ndx, ox_ndx
      integer ::  jo1d_ndx, ox_l1_ndx, o1d_n2_ndx, o1d_o2_ndx, ox_l2_ndx, &
                  ox_l3_ndx, ox_l4_ndx, ox_l5_ndx, ox_l6_ndx, ox_l7_ndx, &
                  ox_l8_ndx, ox_l9_ndx, usr4_ndx, usr16_ndx, usr17_ndx
      logical ::  o3s_loss
      logical ::  class_hist_prod = .false.
      logical ::  class_hist_loss = .false.

      contains

      subroutine exp_slv_inti
!-----------------------------------------------------------------------
!       ... initialize the explicit solver
!-----------------------------------------------------------------------

      use mo_chem_utls, only : get_spc_ndx, get_rxt_ndx

      implicit none

!-----------------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------------
      integer :: m, file, timetype
      integer :: il, iu

      o3s_ndx     = get_spc_ndx( 'O3S' )
      o3inert_ndx = get_spc_ndx( 'O3INERT' )
      ox_ndx = get_spc_ndx( 'OX' )
      oh_ndx = get_spc_ndx( 'OH' )
      ho2_ndx = get_spc_ndx( 'HO2' )
      c2h4_ndx = get_spc_ndx( 'C2H4' )
      c3h6_ndx = get_spc_ndx( 'C3H6' )
      isop_ndx = get_spc_ndx( 'ISOP' )
      mvk_ndx = get_spc_ndx( 'MVK' )
      macr_ndx = get_spc_ndx( 'MACR' )
      c10h16_ndx = get_spc_ndx( 'C10H16' )
      no2_ndx = get_spc_ndx( 'NO2' )
      n2o5_ndx = get_spc_ndx( 'N2O5' )
      no3_ndx = get_spc_ndx( 'NO3' )

      jo1d_ndx = get_rxt_ndx( 'jo1d' )
      ox_l1_ndx = get_rxt_ndx( 'ox_l1' )
      ox_l2_ndx = get_rxt_ndx( 'ox_l2' )
      ox_l3_ndx = get_rxt_ndx( 'ox_l3' )
      ox_l4_ndx = get_rxt_ndx( 'ox_l4' )
      ox_l5_ndx = get_rxt_ndx( 'ox_l5' )
      ox_l6_ndx = get_rxt_ndx( 'ox_l6' )
      ox_l7_ndx = get_rxt_ndx( 'ox_l7' )
      ox_l8_ndx = get_rxt_ndx( 'ox_l8' )
      ox_l9_ndx = get_rxt_ndx( 'ox_l9' )
      o1d_n2_ndx = get_rxt_ndx( 'o1d_n2' )
      o1d_o2_ndx = get_rxt_ndx( 'o1d_o2' )
      usr4_ndx = get_rxt_ndx( 'usr4' )
      usr16_ndx = get_rxt_ndx( 'usr16' )
      usr17_ndx = get_rxt_ndx( 'usr17' )

      end subroutine exp_slv_inti

      subroutine exp_sol( base_sol, reaction_rates, het_rates, extfrc, &
                          delt, hnm, plonl, plnplv )
!-----------------------------------------------------------------------
!       ... exp_sol advances the volumetric mixing ratio
!           forward one time step via the fully explicit
!           euler scheme
!           note : this code has o3inert and o3s as the last
!                  two class members;  neither has production
!                  or loss - some dimensionality below has been
!                  altered to acount for this
!-----------------------------------------------------------------------

      use mo_chem_mods,     only : clscnt1, explicit, extcnt, hetcnt, rxntot
      use mo_indprd,        only : indprd
      use mo_exp_prod_loss, only : exp_prod_loss
      use mo_grid,          only : plev, pcnstm1

      implicit none
!-----------------------------------------------------------------------
!       ... dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) ::  plonl            ! lon tile dim
      integer, intent(in) ::  plnplv           ! plonl*plev
      real, intent(in)    ::  delt             ! time step in seconds
      real, intent(in)    ::  reaction_rates(plnplv,max(1,rxntot))
      real, intent(in)    ::  hnm(plnplv), &
                              het_rates(plnplv,max(1,hetcnt)), &
                              extfrc(plnplv,max(1,extcnt))
      real, intent(inout) ::  base_sol(plnplv,pcnstm1)

!-----------------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------------
      integer  ::  k, l, m, n, class, cls_ndx, file
      real     ::  timer
      real, dimension(plnplv,max(1,clscnt1)) :: &
                   prod, &
                   loss, &
                   ind_prd
      real, dimension(plnplv) :: &
                   wrk
      character(len=32) :: &
                   fldname
      if( explicit%indprd_cnt /= 0 .or. extcnt > 0 ) then
!-----------------------------------------------------------------------
!        ... put "independent" production in the forcing
!-----------------------------------------------------------------------
         call indprd( 1, ind_prd, base_sol, extfrc, reaction_rates )
      else
         do m = 1,max(1,clscnt1)
            ind_prd(:,m) = 0.
         end do
      end if
!-----------------------------------------------------------------------
!       ... form f(y)
!-----------------------------------------------------------------------
      call exp_prod_loss( prod, loss, base_sol, reaction_rates, het_rates )

!-----------------------------------------------------------------------
!       ... solve for the mixing ratio at t(n+1)
!-----------------------------------------------------------------------
      do m = 1,clscnt1
         l = explicit%clsmap(m)
         if( l /= o3s_ndx .and. l /= o3inert_ndx ) then
            base_sol(:,l) = base_sol(:,l) + delt * (prod(:,m) + ind_prd(:,m) - loss(:,m))
         else if( l == o3s_ndx ) then
!-----------------------------------------------------------------------
!       ... special code for o3s
! nb: the coefficients for o3s loss from rxn with isop, mvk, macr, and c10h16
!     are unity. for the ox loss rate (in imp_sol) they are adjusted (downward)
!     to account for the regeneration of ox by these rxns. but here, we
!     consider this regenerated ox to be "tropospheric."  -- lwh 2/01
!     also include o3s loss from no2+oh, n2o5+aerosol, no3+aerosol
!-----------------------------------------------------------------------
            do k = 1,plnplv
               loss(k,m) = &
                  reaction_rates(k,jo1d_ndx)*reaction_rates(k,ox_l1_ndx) &
                  /(reaction_rates(k,o1d_n2_ndx) + reaction_rates(k,o1d_o2_ndx) &
                    + reaction_rates(k,ox_l1_ndx)) &
                + reaction_rates(k,ox_l2_ndx)*base_sol(k,oh_ndx) &
                + reaction_rates(k,ox_l3_ndx)*base_sol(k,ho2_ndx) &
                + reaction_rates(k,ox_l6_ndx)*base_sol(k,c2h4_ndx) &
                + reaction_rates(k,ox_l4_ndx)*base_sol(k,c3h6_ndx) &
                + reaction_rates(k,ox_l5_ndx)*base_sol(k,isop_ndx) &
                + reaction_rates(k,ox_l7_ndx)*base_sol(k,mvk_ndx) &
                + reaction_rates(k,ox_l8_ndx)*base_sol(k,macr_ndx) &
                + reaction_rates(k,ox_l9_ndx)*base_sol(k,c10h16_ndx) &
                + ((reaction_rates(k,usr4_ndx)*base_sol(k,no2_ndx)*base_sol(k,oh_ndx) &
                   + 3.*reaction_rates(k,usr16_ndx)*base_sol(k,n2o5_ndx) &
                   + 2.*reaction_rates(k,usr17_ndx)*base_sol(k,no3_ndx)) &
                   / max( base_sol(k,ox_ndx), 1.e-20 ))
               base_sol(k,l) = base_sol(k,l)*exp( -delt*loss(k,m) )
               loss(k,m)     = loss(k,m)*base_sol(k,l)
            end do
         end if
      end do

!-----------------------------------------------------------------------
      end subroutine exp_sol

      end module mo_exp_sol
