      module mo_imp_sol

      use mo_chem_mods, only : clscnt4

      implicit none

      private
      public :: imp_slv_inti, imp_sol

      save

      integer, parameter ::  inst = 1, avrg = 2
      real, parameter    ::  rel_err      = 1.e-3
      real, parameter    ::  high_rel_err = 1.e-4
!-----------------------------------------------------------------------
!           newton-raphson iteration limits
!-----------------------------------------------------------------------
      integer, parameter :: cut_limit    = 5

      integer :: ox_ndx
      integer :: oh_ndx, ho2_ndx, ch3o2_ndx, po2_ndx, ch3co3_ndx, &
                 c2h5o2_ndx, isopo2_ndx, macro2_ndx, mco3_ndx, c3h7o2_ndx, &
                 ro2_ndx, xo2_ndx, no_ndx, no2_ndx, no3_ndx, n2o5_ndx, &
                 c2h4_ndx, c3h6_ndx, isop_ndx, mvk_ndx, c10h16_ndx
      integer :: ox_p1_ndx, ox_p2_ndx, ox_p3_ndx, ox_p4_ndx, ox_p5_ndx, &
                 ox_p6_ndx, ox_p7_ndx, ox_p8_ndx, ox_p9_ndx, ox_p10_ndx, &
                 ox_p11_ndx
      integer :: ox_l1_ndx, ox_l2_ndx, ox_l3_ndx, ox_l4_ndx, ox_l5_ndx, &
                 ox_l6_ndx, ox_l7_ndx, ox_l8_ndx, ox_l9_ndx, usr4_ndx, &
                 usr16_ndx, usr17_ndx
      logical :: do_ox_pl = .true.

      type hst_pl
         integer  ::  cnt(2)
         logical  ::  do_hst(2)
      end type hst_pl

      real                      ::   small
      real                      ::   epsilon(max(1,clscnt4))
      type(hst_pl), allocatable ::   imp_hst_prod(:)
      type(hst_pl), allocatable ::   imp_hst_loss(:)
      logical, allocatable      ::   factor(:)

      contains

      subroutine imp_slv_inti
!-----------------------------------------------------------------------
!       ... initialize the implict solver
!-----------------------------------------------------------------------

      use mo_chem_mods,  only : clscnt4, implicit
      use mo_grid,       only : pcnstm1
      use mo_chem_utls,  only : get_spc_ndx, get_rxt_ndx

      use abortutils,    only: endrun    !add by zf 2008.06.19

      implicit none

!-----------------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------------
      integer :: m, astat, file, timetype
      integer :: il, iu
      integer :: wrk(21)
      real    :: eps(pcnstm1)

      allocate( factor(implicit%iter_max),stat=astat )
      if( astat /= 0 ) then
         write(*,*) 'imp_slv_inti: failed to allocate factor array; error = ',astat
         call endrun
      end if
      factor(:) = .true.
      eps(:)    = rel_err
      ox_ndx = get_spc_ndx( 'OX' )
      if( ox_ndx > 0 ) then
         eps(ox_ndx) = high_rel_err
      else
         m = get_spc_ndx( 'O3' )
         if( m > 0 ) then
            eps(m) = high_rel_err
         end if
      end if
      m = get_spc_ndx( 'NO' )
      if( m > 0 ) then
         eps(m) = high_rel_err
      end if
      m = get_spc_ndx( 'NO2' )
      if( m > 0 ) then
         eps(m) = high_rel_err
      end if
      m = get_spc_ndx( 'NO3' )
      if( m > 0 ) then
         eps(m) = high_rel_err
      end if
      m = get_spc_ndx( 'HNO3' )
      if( m > 0 ) then
         eps(m) = high_rel_err
      end if
      m = get_spc_ndx( 'HO2NO2' )
      if( m > 0 ) then
         eps(m) = high_rel_err
      end if
      m = get_spc_ndx( 'N2O5' )
      if( m > 0 ) then
         eps(m) = high_rel_err
      end if
      m = get_spc_ndx( 'OH' )
      if( m > 0 ) then
         eps(m) = high_rel_err
      end if
      m = get_spc_ndx( 'HO2' )
      if( m > 0 ) then
         eps(m) = high_rel_err
      end if

      do m = 1,max(1,clscnt4)
         epsilon(m) = eps(implicit%clsmap(m))
      end do

      if( ox_ndx > 0 ) then
         ox_p1_ndx = get_rxt_ndx( 'ox_p1' )
         ox_p2_ndx = get_rxt_ndx( 'ox_p2' )
         ox_p3_ndx = get_rxt_ndx( 'ox_p3' )
         ox_p4_ndx = get_rxt_ndx( 'ox_p4' )
         ox_p5_ndx = get_rxt_ndx( 'ox_p5' )
         ox_p6_ndx = get_rxt_ndx( 'ox_p6' )
         ox_p7_ndx = get_rxt_ndx( 'ox_p7' )
         ox_p8_ndx = get_rxt_ndx( 'ox_p8' )
         ox_p9_ndx = get_rxt_ndx( 'ox_p9' )
         ox_p10_ndx = get_rxt_ndx( 'ox_p10' )
         ox_p11_ndx = get_rxt_ndx( 'ox_p11' )
         wrk(1:11) = (/ ox_p1_ndx, ox_p2_ndx, ox_p3_ndx, ox_p4_ndx, ox_p5_ndx, &
                        ox_p6_ndx, ox_p7_ndx, ox_p8_ndx, ox_p9_ndx, ox_p10_ndx, ox_p11_ndx /)
         if( any( wrk(1:11) < 1 ) ) then
            do_ox_pl = .false.
         end if
         if( do_ox_pl ) then
            ox_l1_ndx = get_rxt_ndx( 'ox_l1' )
            ox_l2_ndx = get_rxt_ndx( 'ox_l2' )
            ox_l3_ndx = get_rxt_ndx( 'ox_l3' )
            ox_l4_ndx = get_rxt_ndx( 'ox_l4' )
            ox_l5_ndx = get_rxt_ndx( 'ox_l5' )
            ox_l6_ndx = get_rxt_ndx( 'ox_l6' )
            ox_l7_ndx = get_rxt_ndx( 'ox_l7' )
            ox_l8_ndx = get_rxt_ndx( 'ox_l8' )
            ox_l9_ndx = get_rxt_ndx( 'ox_l9' )
            usr4_ndx = get_rxt_ndx( 'usr4' )
            usr16_ndx = get_rxt_ndx( 'usr16' )
            usr17_ndx = get_rxt_ndx( 'usr17' )
            wrk(1:12) = (/ ox_l1_ndx, ox_l2_ndx, ox_l3_ndx, ox_l4_ndx, ox_l5_ndx, &
                           ox_l6_ndx, ox_l7_ndx, ox_l8_ndx, ox_l9_ndx, usr4_ndx, &
                           usr16_ndx, usr17_ndx /)
            if( any( wrk(1:12) < 1 ) ) then
               do_ox_pl = .false.
            end if
         end if

         if( do_ox_pl ) then
            oh_ndx = get_spc_ndx( 'OH' )
            ho2_ndx = get_spc_ndx( 'HO2' )
            ch3o2_ndx = get_spc_ndx( 'CH3O2' )
            po2_ndx = get_spc_ndx( 'PO2' )
            ch3co3_ndx = get_spc_ndx( 'CH3CO3' )
            c2h5o2_ndx = get_spc_ndx( 'C2H5O2' )
            macro2_ndx = get_spc_ndx( 'MACRO2' )
            mco3_ndx = get_spc_ndx( 'MCO3' )
            c3h7o2_ndx = get_spc_ndx( 'C3H7O2' )
            ro2_ndx = get_spc_ndx( 'RO2' )
            xo2_ndx = get_spc_ndx( 'XO2' )
            no_ndx = get_spc_ndx( 'NO' )
            no2_ndx = get_spc_ndx( 'NO2' )
            no3_ndx = get_spc_ndx( 'NO3' )
            n2o5_ndx = get_spc_ndx( 'N2O5' )
            c2h4_ndx = get_spc_ndx( 'C2H4' )
            c3h6_ndx = get_spc_ndx( 'C3H6' )
            isop_ndx = get_spc_ndx( 'ISOP' )
            isopo2_ndx = get_spc_ndx( 'ISOPO2' )
            mvk_ndx = get_spc_ndx( 'MVK' )
            c10h16_ndx = get_spc_ndx( 'C10H16' )
            wrk(1:21) = (/ oh_ndx, ho2_ndx, ch3o2_ndx, po2_ndx, ch3co3_ndx, &
                           c2h5o2_ndx, macro2_ndx, mco3_ndx, c3h7o2_ndx, ro2_ndx, &
                           xo2_ndx, no_ndx, no2_ndx, no3_ndx, n2o5_ndx, &
                           c2h4_ndx, c3h6_ndx, isop_ndx, isopo2_ndx, mvk_ndx, c10h16_ndx /)
            if( any( wrk(1:21) < 1 ) ) then
               do_ox_pl = .false.
            end if
         end if
      else
         do_ox_pl = .false.
      end if

      small = 1.e6 * tiny( small )

      end subroutine imp_slv_inti

      subroutine imp_sol( base_sol, reaction_rates, het_rates, extfrc, &
                          delt, hnm, plonl, plnplv )
!-----------------------------------------------------------------------
!       ... imp_sol advances the volumetric mixing ratio
!           forward one time step via the fully implicit
!           euler scheme
!-----------------------------------------------------------------------

      use mo_chem_mods,      only : clscnt4, imp_nzcnt, clsze, rxntot, hetcnt, extcnt, implicit
      use mo_tracname,       only : tracnam
!zf      use mo_control,        only : pdiags
      use mo_grid,           only : plev, pcnstm1
      use mo_indprd,         only : indprd
      use mo_imp_prod_loss,  only : imp_prod_loss
      use mo_imp_lin_matrix, only : imp_linmat
      use mo_imp_nln_matrix, only : imp_nlnmat
      use mo_imp_factor,     only : imp_lu_fac
      use mo_imp_solve,      only : imp_lu_slv

      implicit none

!-----------------------------------------------------------------------
!       ... dummy args
!-----------------------------------------------------------------------
      integer, intent(in) ::   plonl                     ! longitude tile index
      integer, intent(in) ::   plnplv                    ! plonl*plev
      real, intent(in)    ::   delt                      ! time step (seconds)
      real, intent(in)    ::   reaction_rates(plnplv,max(1,rxntot)), &
                               het_rates(plnplv,max(1,hetcnt)), &
                               extfrc(plnplv,max(1,extcnt)), &
                               hnm(plnplv)
      real, intent(inout) ::   base_sol(plnplv,pcnstm1)

!-----------------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------------
      type hst_buff
         real, pointer, dimension(:,:) :: buff
      end type hst_buff

      integer ::   nr_iter, &
                   lev, &
                   ofl, ofu, &
                   i, isec, &
                   j, &
                   k, l, &
                   m, &
                   n, &
                   class, cls_ndx, file, &
                   cut_cnt, stp_con_cnt
      integer ::   timetype, hndx
      integer ::   astat
      integer ::   clscnt
      real :: interval_done, dt, dti
      real :: max_delta(max(1,clscnt4))
      real, dimension(clsze,max(1,imp_nzcnt)) :: &
                   sys_jac, &
                   lin_jac
      real, dimension(clsze,max(1,clscnt4)) :: &
                   solution, &
                   forcing, &
                   iter_invariant, &
                   prod, &
                   loss
      real :: lrxt(clsze,max(1,rxntot))
      real :: lsol(clsze,max(1,pcnstm1))
      real :: lhet(clsze,max(1,hetcnt))
      real, dimension(clsze) :: &
                   wrk
      real, dimension(plnplv) :: &
                   wrk_buff
      real, dimension(plnplv,max(1,clscnt4)) :: &
                   ind_prd
      real    ::   timer
      logical ::   convergence
      logical ::   dump_buff, fill_buff
      logical ::   frc_mask(clsze), iter_conv(clsze)
      logical ::   converged(max(1,clscnt4))
      character(len=32) :: fldname
      type(hst_buff), allocatable :: prod_buff(:), loss_buff(:)
!-----------------------------------------------------------------------
      max_delta= 0.0
      sys_jac = 0.0
      lin_jac = 0.0
      solution = 0.0
      forcing = 0.0
      iter_invariant = 0.0
      prod = 0.0
      loss = 0.0
      lrxt = 0.0
      lsol = 0.0
      lhet = 0.0
      wrk = 0.0
      wrk_buff = 0.0
      ind_prd = 0.0

!----------------------------------------------------
      clscnt = max(1,clscnt4)
      if( implicit%indprd_cnt > 0 .or. extcnt > 0 ) then
!-----------------------------------------------------------------------
!        ... class independent forcing
!-----------------------------------------------------------------------
         call indprd( 4, ind_prd, base_sol, extfrc, reaction_rates )
      else
         do m = 1,clscnt
            ind_prd(:,m) = 0.
         end do
      end if
level_loop : &
      do lev = 1,plev
lon_tile_loop : &
         do isec = 1,plonl/clsze
            ofl = (lev - 1)*plonl + (isec - 1)*clsze + 1
            ofu = ofl + clsze - 1
!-----------------------------------------------------------------------
!        ... transfer from base to local work arrays
!-----------------------------------------------------------------------
            do m = 1,max(1,rxntot)
               lrxt(:,m) = reaction_rates(ofl:ofu,m)
            end do
            if( hetcnt > 0 ) then
               do m = 1,max(1,hetcnt)
                  lhet(:,m) = het_rates(ofl:ofu,m)
               end do
            end if
!-----------------------------------------------------------------------
!        ... time step loop
!-----------------------------------------------------------------------
            dt            = delt
            cut_cnt       = 0
            stp_con_cnt   = 0
            interval_done = 0.
time_step_loop : &
            do
               dti = 1. / dt
!-----------------------------------------------------------------------
!        ... transfer from base to local work arrays
!-----------------------------------------------------------------------
               do m = 1,pcnstm1
                  lsol(:,m) = base_sol(ofl:ofu,m)
               end do
!-----------------------------------------------------------------------
!        ... transfer from base to class array
!-----------------------------------------------------------------------
               do k = 1,clscnt
                  j = implicit%clsmap(k)
                  m = implicit%permute(k)
                  solution(:,m) = lsol(:,j)
               end do
!-----------------------------------------------------------------------
!        ... set the iteration invariant part of the function f(y)
!        ... if there is "independent" production put it in the forcing
!-----------------------------------------------------------------------
               if( implicit%indprd_cnt > 0 .or. extcnt > 0 ) then
                  do m = 1,clscnt
                     iter_invariant(:,m) = dti * solution(:,m) + ind_prd(ofl:ofu,m)
                  end do
               else
                  do m = 1,clscnt
                     iter_invariant(:,m) = dti * solution(:,m)
                  end do
               end if
!-----------------------------------------------------------------------
!        ... the linear component
!-----------------------------------------------------------------------
               if( implicit%lin_rxt_cnt > 0 ) then
                  call imp_linmat( lin_jac, lsol, lrxt, lhet )
               end if

!=======================================================================
!        the newton-raphson iteration for f(y) = 0
!=======================================================================
iter_loop : &
               do nr_iter = 1,implicit%iter_max
!-----------------------------------------------------------------------
!        ... the non-linear component
!-----------------------------------------------------------------------
                  if( factor(nr_iter) ) then
                     call imp_nlnmat( sys_jac, lsol, lrxt, lin_jac, dti )
!-----------------------------------------------------------------------
!         ... factor the "system" matrix
!-----------------------------------------------------------------------
                     call imp_lu_fac( sys_jac )
                  end if
!-----------------------------------------------------------------------
!       ... form f(y)
!-----------------------------------------------------------------------
                  call imp_prod_loss( prod, loss, lsol, lrxt, lhet )
                  do m = 1,clscnt
                     forcing(:,m) = solution(:,m)*dti - (iter_invariant(:,m) + prod(:,m) - loss(:,m))
                  end do
!-----------------------------------------------------------------------
!         ... solve for the mixing ratio at t(n+1)
!-----------------------------------------------------------------------
                  call imp_lu_slv( sys_jac, forcing )
                  do m = 1,clscnt
                     solution(:,m) = solution(:,m) + forcing(:,m)
                  end do
!-----------------------------------------------------------------------
!       ... convergence measures
!-----------------------------------------------------------------------
                  if( nr_iter > 1 ) then
                     do k = 1,clscnt
                        m = implicit%permute(k)
                        where( abs(solution(:,m)) > 1.e-40 )
                           wrk(:) = abs( forcing(:,m)/solution(:,m) )
                        elsewhere
                           wrk(:) = 0.
                        endwhere
                        max_delta(k) = maxval( wrk(:) )
                     end do
                  end if
!-----------------------------------------------------------------------
!       ... limit iterate
!-----------------------------------------------------------------------
                  do m = 1,clscnt
                     where( solution(:,m) < 0. )
                        solution(:,m) = 0.
                     endwhere
                  end do
!-----------------------------------------------------------------------
!       ... transfer latest solution back to work array
!-----------------------------------------------------------------------
                  do k = 1,clscnt
                     j = implicit%clsmap(k)
                     m = implicit%permute(k)
                     lsol(:,j) = solution(:,m)
                  end do
!-----------------------------------------------------------------------
!       ... check for convergence
!-----------------------------------------------------------------------
                  if( nr_iter > 1 ) then
                     do k = 1,clscnt
                        m = implicit%permute(k)
                        frc_mask(:) = abs( forcing(:,m) ) > small
                        where( frc_mask(:) )
                           iter_conv(:) =  abs(forcing(:,m)) <= epsilon(k)*abs(solution(:,m))
                        elsewhere
                           iter_conv(:) =  .true.
                        endwhere
                        converged(k) =  all( iter_conv(:) )
                     end do
                     convergence = all( converged(:clscnt4) )
                     if( convergence ) then
                        exit
                     end if
                  end if
               end do iter_loop

!-----------------------------------------------------------------------
!       ... check for newton-raphson convergence
!-----------------------------------------------------------------------
               if( .not. convergence ) then
!-----------------------------------------------------------------------
!       ... non-convergence
!-----------------------------------------------------------------------
!zf                  if( pdiags%imp_slv ) then
!zf                     write(*,'('' imp_sol: time step '',1p,e21.13,'' failed to converge'')') dt
!zf                  end if
                  stp_con_cnt = 0
                  if( cut_cnt < cut_limit ) then
                     cut_cnt = cut_cnt + 1
                     dt = .5 * dt
                     cycle
                  else
!zf                     write(*,'('' imp_sol: failed to converge @ (isec,ip,lat,lev,dt) = '',4i5,1p,e21.13)') isec,ip,lat,lev,dt
                     do m = 1,clscnt4
                        if( .not. converged(m) ) then
!zftest                           write(*,'(1x,a8,1x,1pe10.3)') tracnam(implicit%clsmap(m)), max_delta(m)
                        end if
                     end do
                  end if
               end if
!-----------------------------------------------------------------------
!       ... check for interval done
!-----------------------------------------------------------------------
               interval_done = interval_done + dt
               if( abs( delt - interval_done ) <= .0001 ) then
                  exit
               else
!-----------------------------------------------------------------------
!       ... transfer latest solution back to base array
!-----------------------------------------------------------------------
                  if( convergence ) then
                     stp_con_cnt = stp_con_cnt + 1
                  end if
                  do m = 1,pcnstm1
                     base_sol(ofl:ofu,m) = lsol(:,m)
                  end do
                  if( stp_con_cnt >= 2 ) then
                     dt = 2.*dt
                     stp_con_cnt = 0
                  end if
                  dt = min( dt,delt-interval_done )
!zf                  if( pdiags%imp_slv ) then
!zf                     write(*,'('' imp_sol: new time step '',1p,e21.13)') dt
!zf                  end if
               end if
            end do time_step_loop
!-----------------------------------------------------------------------
!       ... transfer latest solution back to base array
!-----------------------------------------------------------------------
            do k = 1,clscnt
               j = implicit%clsmap(k)
               m = implicit%permute(k)
               base_sol(ofl:ofu,j) = solution(:,m)
            end do

         end do lon_tile_loop
      end do level_loop

      end subroutine imp_sol

      end module mo_imp_sol
