
#include <misc.h>
#include <params.h>

#ifdef MOZART2

      module mo_inirun

      implicit none

      save 

      contains

      subroutine inirun

      use shr_kind_mod, only: r8 => shr_kind_r8
      use pmgrid,          only : masterproc
      use mo_grid,         only : plon, plat, plev, plevp

      use mo_const_mozart, only : mo_constants_inti, ktop
      use mo_chemini,      only : chemini
      use mo_hook,         only : moz_hook_inti
      use mo_drydep,       only : dvel_inti
      use mo_surface,      only : inisflx

      implicit none

#include <comhyb.h>

      real, parameter :: p_limit = 10.e2     ! findsp pressure limit (Pa)
      real, parameter :: rair   = 287.04 
      real, parameter :: gravit = 9.80616
      real, parameter :: cpair  = 1004.64
      real, parameter :: latvap = 2.5104e06
      real, parameter :: rhoh2o = 1.e3

      logical   ::         diconvccm
      logical   ::         arconvccm

      real      ::         ref_pmid(plev)
      integer   ::         k

!---------------------------------------------------------------------
 
      arconvccm = .false.
      diconvccm = .true.      

      call mo_constants_inti

!---------------------------------------------------------------------
!       ... Diagnostics
!---------------------------------------------------------------------
!      pdiags%negtrc  = negtrc_diagprnt
!      pdiags%imp_slv = imp_slv_diagprnt
!      pdiags%adv     = adv_diagprnt
!---------------------------------------------------------------------
!     	... Initialize surface flux calculation.
!---------------------------------------------------------------------
      call inisflx( rair, gravit )
      write(*,*) 'inirun: finished INISFLX'
!---------------------------------------------------------------------
!       ... Initialize Hacks convective mass flux adjustment parameterization.
!---------------------------------------------------------------------
!zftest      if( arconvccm .or. diconvccm ) then
!zftest         call mfinti( hypi, rair, cpair, gravit, latvap, rhoh2o )
!zftest         write(*,*) 'inirun: finished MFINTI'
!zftest      end if
!---------------------------------------------------------------------
!       ... Initialize tropospheric chemistry calculation
!---------------------------------------------------------------------
      call chemini( plon, plat )
      if (masterproc) write(*,*) 'inirun: finished CHEMINI'
!---------------------------------------------------------------------
!     	... Initialize deposition velocities
!--------------------------------------------------------------------- 
      call dvel_inti( plat, plon )
!---------------------------------------------------------------------
!       ... Find level where etamids are all > 10hPa
!---------------------------------------------------------------------
      call moz_hook_inti(1.2)

      do k = 1, plev
        ref_pmid(k) = (hyam(k) + hybm(k))*1.e5
      end do

      ktop = 0
      if( ref_pmid(1) < p_limit ) then
         do k = 1,plev
            if( ref_pmid(k) < p_limit ) then
               ktop = k
            end if
         end do
      end if
      end subroutine inirun

      end module mo_inirun
#endif

