      module mo_photo
!----------------------------------------------------------------------
!       ... photolysis interp table and related arrays
!----------------------------------------------------------------------

      implicit none

      private
      public :: prate_inti, photo, set_ub_col, setcol, diurnal_geom, sundis

      character(len=256) , public  ::  bndprate

      save

      integer, parameter :: jdim     = 19
      integer, parameter :: altdim   = 18
      integer, parameter :: zangdim  = 8
      integer, parameter :: o3ratdim = 7
      integer, parameter :: albdim   = 4
      integer, parameter :: t500dim  = 3
      integer, parameter :: t200dim  = 2
      integer, parameter :: tabdim   = jdim*altdim*zangdim*o3ratdim*albdim*t500dim*t200dim

      integer ::  offset(7)
      integer ::  indexer(jdim)
      integer ::  jno_ndx, jpooh_ndx, jc2h5ooh_ndx, jc3h7ooh_ndx, jrooh_ndx, &
                  jch3co3h_ndx, jmpan_ndx, jmacr_a_ndx, jmacr_b_ndx, jonitr_ndx, &
                  jxooh_ndx, jisopooh_ndx, jglyald_ndx, jhyac_ndx, jch3ooh_ndx, &
                  jh2o2_ndx, jpan_ndx, jch3cho_ndx
      integer ::  ox_ndx, o3_ndx
      real    ::  ajl(jdim,altdim,zangdim,o3ratdim,albdim,t500dim,t200dim) = 0.
      real    ::  vo3(0:50)
      real    ::  delvo3(0:49)
      real    ::  delz(altdim-1)
      real    ::  delang(zangdim-1)
      real    ::  delv(o3ratdim-1)
      real    ::  delalb(albdim-1)
      real    ::  delt500(t500dim-1)
      real    ::  delt200(t200dim-1)

      real :: zz(altdim) = &
             (/ 0., 1., 2., 3., 5., 7., 9., 12., 15., 18., 21., 24., &
                27., 30., 35., 40., 45., 50. /)
      real :: vsec(zangdim) = (/ 1., 1.3, 1.6, 2., 3., 6., 12., 50. /)
      real :: xv3(o3ratdim) = (/ .5, .75, 1., 1.25, 1.5, 2., 5. /)
      real :: albev(albdim) = (/ .05, .2, .5, 1. /)
      real :: t500(t500dim) = (/ 228., 248., 268. /)
      real :: t200(t200dim) = (/ 205., 225. /)

      private :: cloud_mod, t_int, photo_interp

      contains

      subroutine prate_inti
!----------------------------------------------------------------------
!     ... read in the photorate tables and arrays
!         results are "returned" via the common block photo_tables
!         this is for the new, expanded chemistry (11/21/94)
!----------------------------------------------------------------------

      use mo_chem_utls,  only : get_spc_ndx, get_rxt_ndx
      use units,         only : getunit                     !add by zf 2008.06.23
      use abortutils,    only : endrun                      !add by zf 2008.06.23

      implicit none


!----------------------------------------------------------------------
!        ... dummy args
!----------------------------------------------------------------------

!----------------------------------------------------------------------
!        ... local variables
!----------------------------------------------------------------------
      integer    :: it500, it200, izen, ialb, idob
      integer    :: ios, ofl, k
      integer    :: retval, unit
      logical    :: cosb
      real       :: temp(tabdim)

!----------------------------------------------------------------------
!        ... all compute nodes open file
!----------------------------------------------------------------------

      unit = getunit ()
      open( unit   = unit, &
            file   = bndprate, &
            status = 'old', &
            form   = 'formatted', &
            recl   = 4500, &
            iostat = ios )
      if( ios /= 0 ) then
!----------------------------------------------------------------------
!       ... open error exit
!----------------------------------------------------------------------
         write(6,'('' prate_inti : error ('',i5,'') opening file '',a)') &
            ios, bndprate 
         call endrun
      end if

!----------------------------------------------------------------------
!        ... readin the reference o3 column and photorate table
!----------------------------------------------------------------------
      read(unit,*,iostat=ios) vo3
      if( ios /= 0 ) then
         write(*,*) ' prate_inti: failed to read o3 column'
         call endrun
      end if

      do it500 = 1,t500dim
         do it200 = 1,t200dim
            do izen = 1,zangdim
               do ialb = 1,albdim
                  do idob = 1,o3ratdim
                     read(unit,*,iostat=ios) ajl(:,:,izen,idob,ialb,it500,it200)
                     if( ios /= 0 ) then
                        write(*,*) ' prate_inti: failed to read photo table; error = ',ios
                        call endrun
                     end if
                  end do
               end do
            end do
         end do
      end do
      close( unit )

!----------------------------------------------------------------------
!       ... set module variables
!----------------------------------------------------------------------
      delz(:altdim-1)     = 1. / (zz(2:altdim) - zz(:altdim-1))
      delvo3(0:49)        = vo3(1:50) - vo3(0:49)
      delang(:zangdim-1)  = 1. / (vsec(2:zangdim) - vsec(:zangdim-1))
      delv(:o3ratdim-1)   = 1. / (xv3(2:o3ratdim) - xv3(:o3ratdim-1))
      delalb(:albdim-1)   = 1. / (albev(2:albdim) - albev(:albdim-1))
      delt500(:t500dim-1) = 1. / (t500(2:t500dim) - t500(:t500dim-1))
      delt200(:t200dim-1) = 1. / (t200(2:t200dim) - t200(:t200dim-1))

      offset(1) = jdim
      offset(2) = offset(1)*altdim
      offset(3) = offset(2)*zangdim
      offset(4) = offset(3)*o3ratdim
      offset(5) = offset(4)*albdim
      offset(6) = offset(5)*t500dim
      offset(7) = sum( offset(1:6) )

!-----------------------------------------------------------------
!       ... setup mapping array, indexer, from table to model
!-----------------------------------------------------------------
      indexer(1)  = get_rxt_ndx( 'jo2' )
      indexer(2)  = get_rxt_ndx( 'jo1d' )
      indexer(3)  = get_rxt_ndx( 'jo3p' )
      indexer(4)  = get_rxt_ndx( 'jn2o' )
      indexer(5)  = get_rxt_ndx( 'jno2' )
      indexer(6)  = get_rxt_ndx( 'jn2o5' )
      indexer(7)  = get_rxt_ndx( 'jhno3' )
      indexer(8)  = get_rxt_ndx( 'jno3' )
      indexer(9)  = get_rxt_ndx( 'jho2no2' )
      indexer(10) = get_rxt_ndx( 'jch3ooh' )
      indexer(11) = get_rxt_ndx( 'jch2o_a' )
      indexer(12) = get_rxt_ndx( 'jch2o_b' )
      indexer(13) = get_rxt_ndx( 'jh2o2' )
      indexer(14) = get_rxt_ndx( 'jch3cho' )
      indexer(15) = get_rxt_ndx( 'jpan' )
      indexer(16) = get_rxt_ndx( 'jmacr_a' )
      indexer(17) = get_rxt_ndx( 'jmvk' )
      indexer(18) = get_rxt_ndx( 'jacet' )
      indexer(19) = get_rxt_ndx( 'jmgly' )

      jno_ndx      = get_rxt_ndx( 'jno' )
      jpooh_ndx    = get_rxt_ndx( 'jpooh' )
      jc2h5ooh_ndx = get_rxt_ndx( 'jc2h5ooh' )
      jc3h7ooh_ndx = get_rxt_ndx( 'jc3h7ooh' )
      jrooh_ndx    = get_rxt_ndx( 'jrooh' )
      jch3co3h_ndx = get_rxt_ndx( 'jch3co3h' )
      jmpan_ndx    = get_rxt_ndx( 'jmpan' )
      jmacr_a_ndx  = get_rxt_ndx( 'jmacr_a' )
      jmacr_b_ndx  = get_rxt_ndx( 'jmacr_b' )
      jonitr_ndx   = get_rxt_ndx( 'jonitr' )
      jxooh_ndx    = get_rxt_ndx( 'jxooh' )
      jisopooh_ndx = get_rxt_ndx( 'jisopooh' )
      jglyald_ndx  = get_rxt_ndx( 'jglyald' )
      jhyac_ndx    = get_rxt_ndx( 'jhyac' )
      jch3ooh_ndx  = get_rxt_ndx( 'jch3ooh' )
      jh2o2_ndx    = get_rxt_ndx( 'jh2o2' )
      jpan_ndx     = get_rxt_ndx( 'jpan' )
      jch3cho_ndx  = get_rxt_ndx( 'jch3cho' )

      ox_ndx = get_spc_ndx( 'OX' )
      o3_ndx = get_spc_ndx( 'O3' )

      end subroutine prate_inti

      subroutine photo( photos, pmid, pdel, temper, zmid, &
                        col_dens, zen_angle, srf_alb, lwc, clouds, &
                        esfact, plonl )

      use mo_grid,      only : plev
      use mo_chem_mods, only : ncol_abs, phtcnt
      use mo_rxt_id

      implicit none

!-----------------------------------------------------------------
!       ... dummy arguments
!-----------------------------------------------------------------
      integer, intent(in) :: plonl
      real, intent(in) ::   esfact                           ! earth sun distance factor
      real, intent(in) ::   col_dens(:,:,:), &               ! column densities
                            zen_angle(plonl), &              ! solar zenith angle
                            srf_alb(plonl), &                ! surface albedo
                            lwc(plonl,plev), &               ! liquid water content (mass mr)
                            clouds(plonl,plev), &            ! cloud fraction
                            pmid(plonl,plev), &              ! midpoint pressure in pascals
                            pdel(plonl,plev), &              ! del pressure about midpoint in pascals
                            zmid(plonl,plev), &              ! midpoint height
                            temper(plonl,plev)               ! midpoint temperature
      real, intent(out) ::  photos(:,:,:)                    ! photodissociation rates

!-----------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------
      integer  ::  i, k, m                 ! indicies
      logical  ::  zagtz(plonl)             ! zenith angle > 0 flag array
      real    ::   secant
      real    ::   t500, t200             ! 500 & 200 mb temperatures
      real, dimension(plev) :: &
                   fac1, &                ! work space for j(no) calc
                   fac2, &                ! work space for j(no) calc
                   colo3, &               ! vertical o3 column density
                   zarg, &                ! vertical height array
                   pline, &               ! vertical pressure array
                   tline, &               ! vertical temperature array
                   cld_line, &            ! vertical cloud array
                   lwc_line, &            ! vertical lwc array
                   eff_alb, &             ! effective albedo from cloud modifications
                   cld_mult               ! clould multiplier
      real, dimension(plonl,plev) :: &
                   tmp, &                        ! wrk array
                   tmp_jch3ooh, &                ! wrk array
                   tmp_jpan, &                   ! wrk array
                   tmp_jh2o2, &                  ! wrk array
                   tmp_jch3cho, &                ! wrk array
                   tmp_jmacr_a                   ! wrk array
      real    ::   prates(jdim,plev)      ! photorates matrix

!-----------------------------------------------------------------
!       ... zero all photorates
!-----------------------------------------------------------------
      do m = 1,max(1,phtcnt)
         do k = 1,plev
            photos(:,k,m)    = 0.
            tmp_jch3ooh(:,k) = 0.
            tmp_jpan(:,k)    = 0.
            tmp_jh2o2(:,k)   = 0.
            tmp_jch3cho(:,k) = 0.
            tmp_jmacr_a(:,k) = 0.
         end do
      end do
      zagtz(:) = zen_angle(:) > 0.

      do i = 1,plonl
         if( zagtz(i) ) then
            secant = 1. / cos(zen_angle(i))
            if( secant <= 50. ) then
               zarg(:)     = zmid(i,:)
               colo3(:)    = col_dens(i,:,1)
               pline(:)    = pmid(i,:)
               fac1(:)     = pdel(i,:)
               tline(:)    = temper(i,:)
               lwc_line(:) = lwc(i,:)
               cld_line(:) = clouds(i,:)
               call cloud_mod( zen_angle(i), cld_line, lwc_line, fac1, srf_alb(i), &
                               eff_alb, cld_mult )
               call t_int( pline, tline, t500, t200 )
               call photo_interp( zarg, secant, colo3, eff_alb, t500, &
                                  t200, prates )
               do m = 1,jdim
                  if( indexer(m) > 0 ) then
                     photos(i,:,indexer(m)) = esfact *prates(m,:) * cld_mult(:)
                  else
                     select case( m )
                        case( 10 )
                           tmp_jch3ooh(i,:) = esfact *prates(m,:) * cld_mult(:)
                        case( 13 )
                           tmp_jh2o2(i,:) = esfact *prates(m,:) * cld_mult(:)
                        case( 14 )
                           tmp_jch3cho(i,:) = esfact *prates(m,:) * cld_mult(:)
                        case( 15 )
                           tmp_jpan(i,:) = esfact *prates(m,:) * cld_mult(:)
                        case( 16 )
                           tmp_jmacr_a(i,:) = esfact *prates(m,:) * cld_mult(:)
                     end select
                  end if
               end do
               if( jno_ndx > 0 ) then
!-----------------------------------------------------------------
!       ... calculate j(no) from formula
!-----------------------------------------------------------------
                  fac1(:) = 1.e-8  * (col_dens(i,:,2)/cos(zen_angle(i)))**.38
                  fac2(:) = 5.e-19 * col_dens(i,:,1) / cos(zen_angle(i))
                  photos(i,:,jno_ndx) = 4.5e-6 * esfact * exp( -(fac1(:) + fac2(:)) ) * cld_mult(:)
               end if
            end if
         end if
      end do

!-----------------------------------------------------------------
!       ... set j(pooh) from j(ch3ooh)
!               j(c2h5ooh) from j(ch3ooh)
!               j(c3h7ooh) from j(ch3ooh)
!               j(rooh) from j(ch3ooh)
!               j(ch3coooh) = .28 * j(h2o2)
!               j(mpan) from j(pan)
!               j(macr_a) and j(macr_b) = 1/2 * j(macr_total)
!               j(onitr) from j(ch3cho)
!               j(xooh) from j(ch3ooh)
!               j(isopooh) from j(ch3ooh)
!               j(glyald) = 3 * j(ch3cho)
!               j(hyac) from j(ch3cho)
!-----------------------------------------------------------------
      if( jch3ooh_ndx > 0 ) then
         tmp(:,:) = photos(:,:,jch3ooh_ndx)
      else
         tmp(:,:) = tmp_jch3ooh(:,:)
      end if
      if( jpooh_ndx > 0 ) then
         photos(:,:,jpooh_ndx)    = tmp(:,:)
      end if
      if( jc2h5ooh_ndx > 0 ) then
         photos(:,:,jc2h5ooh_ndx) = tmp(:,:)
      end if
      if( jc3h7ooh_ndx > 0 ) then
         photos(:,:,jc3h7ooh_ndx) = tmp(:,:)
      end if
      if( jrooh_ndx > 0 ) then
         photos(:,:,jrooh_ndx)    = tmp(:,:)
      end if
      if( jxooh_ndx > 0 ) then
        photos(:,:,jxooh_ndx)    = tmp(:,:)
      end if
      if( jisopooh_ndx > 0 ) then
         photos(:,:,jisopooh_ndx) = tmp(:,:)
      end if
      if( jch3co3h_ndx > 0 ) then
         if( jh2o2_ndx > 0 ) then
            photos(:,:,jch3co3h_ndx) = .28 * photos(:,:,jh2o2_ndx)
         else
            photos(:,:,jch3co3h_ndx) = .28 * tmp_jh2o2(:,:)
         end if
      end if
      if( jmpan_ndx > 0 ) then
         if( jpan_ndx > 0 ) then
            photos(:,:,jmpan_ndx)    = photos(:,:,jpan_ndx)
         else
            photos(:,:,jmpan_ndx)    = tmp_jpan(:,:)
         end if
      end if
      if( jmacr_a_ndx > 0 ) then
         photos(:,:,jmacr_a_ndx)  = .5 * photos(:,:,jmacr_a_ndx)
      end if
      if( jmacr_b_ndx > 0 ) then
         if( jmacr_a_ndx > 0 ) then
            photos(:,:,jmacr_b_ndx)  = photos(:,:,jmacr_a_ndx)
         else
            photos(:,:,jmacr_b_ndx)  = .5 * tmp_jmacr_a(:,:)
         end if
      end if
      if( jonitr_ndx > 0 ) then
         if( jch3cho_ndx > 0 ) then
            photos(:,:,jonitr_ndx)   = photos(:,:,jch3cho_ndx)
         else
            photos(:,:,jonitr_ndx)   = tmp_jch3cho(:,:)
         end if
      end if
      if( jglyald_ndx > 0 ) then
         if( jch3cho_ndx > 0 ) then
            photos(:,:,jglyald_ndx)  = 3. * photos(:,:,jch3cho_ndx)
         else
            photos(:,:,jglyald_ndx)   = 3. *tmp_jch3cho(:,:)
         end if
      end if
      if( jhyac_ndx > 0 ) then
         if( jch3cho_ndx > 0 ) then
            photos(:,:,jhyac_ndx)    = photos(:,:,jch3cho_ndx)
         else
            photos(:,:,jhyac_ndx)    = tmp_jch3cho(:,:)
         end if
      end if

      end subroutine photo

      subroutine cloud_mod( zen_angle, clouds, lwc, delp, srf_alb, &
                            eff_alb, cld_mult )
!-----------------------------------------------------------------------
!       ... cloud alteration factors for photorates and albedo
!-----------------------------------------------------------------------

      use mo_grid,         only : plev, plevm
      use mo_const_mozart, only : rgrav

      implicit none

!-----------------------------------------------------------------------
!       ... dummy arguments
!-----------------------------------------------------------------------
      real, intent(in)    ::  zen_angle         ! zenith angle
      real, intent(in)    ::  srf_alb           ! surface albedo
      real, intent(in)    ::  clouds(plev)       ! cloud fraction
      real, intent(in)    ::  lwc(plev)          ! liquid water content (mass mr)
      real, intent(in)    ::  delp(plev)         ! del press about midpoint in pascals
      real, intent(out)   ::  eff_alb(plev)      ! effective albedo
      real, intent(out)   ::  cld_mult(plev)     ! photolysis mult factor

!-----------------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------------
      integer :: k
      real    :: coschi
      real    :: del_lwp(plev)
      real    :: del_tau(plev)
      real    :: above_tau(plev)
      real    :: below_tau(plev)
      real    :: above_cld(plev)
      real    :: below_cld(plev)
      real    :: above_tra(plev)
      real    :: below_tra(plev)
      real    :: fac1(plev)
      real    :: fac2(plev)

!---------------------------------------------------------
!       ... modify lwc for cloud fraction and form
!           liquid water path for each layer
!---------------------------------------------------------
      where( clouds(:) /= 0. )
         del_lwp(:) = rgrav * lwc(:) * delp(:) * 1.e3 / clouds(:)
      elsewhere
         del_lwp(:) = 0.
      endwhere
!---------------------------------------------------------
!       ... form tau for each model layer
!---------------------------------------------------------
      where( clouds(:) /= 0. )
         del_tau(:) = del_lwp(:) *.155 * clouds(:)**1.5
      elsewhere
         del_tau(:) = 0.
      end where
!---------------------------------------------------------
!       ... form integrated tau from top down
!---------------------------------------------------------
      above_tau(1) = 0.
      do k = 1,plevm
         above_tau(k+1) = del_tau(k) + above_tau(k)
      end do
!---------------------------------------------------------
!       ... form integrated tau from bottom up
!---------------------------------------------------------
      below_tau(plev) = 0.
      do k = plevm,1,-1
         below_tau(k) = del_tau(k+1) + below_tau(k+1)
      end do
!---------------------------------------------------------
!       ... form vertically averaged cloud cover above and below
!---------------------------------------------------------
      above_cld(1) = 0.
      do k = 1,plevm
         above_cld(k+1) = clouds(k) * del_tau(k) + above_cld(k)
      end do
      do k = 2,plev
         if( above_tau(k) /= 0. ) then
            above_cld(k) = above_cld(k) / above_tau(k)
         else
            above_cld(k) = above_cld(k-1)
         end if
      end do
      below_cld(plev) = 0.
      do k = plevm,1,-1
         below_cld(k) = clouds(k+1) * del_tau(k+1) + below_cld(k+1)
      end do
      do k = plevm,1,-1
         if( below_tau(k) /= 0. ) then
            below_cld(k) = below_cld(k) / below_tau(k)
         else
            below_cld(k) = below_cld(k+1)
         end if
      end do
!---------------------------------------------------------
!       ... modify above_tau and below_tau via jfm
!---------------------------------------------------------
      where( above_cld(2:plev) /= 0. )
         above_tau(2:plev) = above_tau(2:plev) / above_cld(2:plev)
      end where
      where( below_cld(:plevm) /= 0. )
         below_tau(:plevm) = below_tau(:plevm) / below_cld(:plevm)
      end where
      where( above_tau(2:plev) < 5. )
            above_cld(2:plev) = 0.
      end where
      where( below_tau(:plevm) < 5. )
         below_cld(:plevm) = 0.
      end where
!---------------------------------------------------------
!       ... form transmission factors
!---------------------------------------------------------
      above_tra(:) = 11.905 / (9.524 + above_tau(:))
      below_tra(:) = 11.905 / (9.524 + below_tau(:))
!---------------------------------------------------------
!       ... form effective albedo
!---------------------------------------------------------
      where( below_cld(:) /= 0. )
         eff_alb(:) = srf_alb + below_cld(:) * (1. - below_tra(:)) &
                                             * (1. - srf_alb)
      elsewhere
         eff_alb(:) = srf_alb
      end where
      coschi = max( cos( zen_angle ),.5 )
      where( del_lwp(:)*.155 < 5. )
         fac1(:) = 0.
      elsewhere
         fac1(:) = 1.4 * coschi - 1.
      end where
      fac2(:)     = min( 0.,1.6*coschi*above_tra(:) - 1. )
      cld_mult(:) = 1. + fac1(:) * clouds(:) + fac2(:) * above_cld(:)
      cld_mult(:) = max( .05,cld_mult(:) )

      end subroutine cloud_mod

      subroutine t_int( p, t, t500, t200 )
!----------------------------------------------------------------
!       ... interpolate for temperature on 500 and 200mb surfaces
!----------------------------------------------------------------

      use mo_grid, only : plev, plevm, plevp

      implicit none

!----------------------------------------------------------------
!       ... dummy args
!----------------------------------------------------------------
      real, intent(in)  ::  p(plev)           ! pressure in pascals
      real, intent(in)  ::  t(plev)           ! temperature on grid
      real, intent(out) ::  t500, t200        ! temp at 500 and 200mb

!----------------------------------------------------------------
!       ... local variables
!----------------------------------------------------------------
      integer :: k, k1
      real    :: delp

      if( p(plev) < 500.e2 ) then
         t500 = t(plev)
         k1 = plevp
      else
         do k = plevm,1,-1
            if( p(k) < 500.e2 ) then
               k1 = k
               exit
            end if
         end do
         delp = log( 500.e2/p(k) ) / log( p(k+1)/p(k) )
         t500 = t(k) + delp * (t(k+1) - t(k))
      end if
      do k = k1-1,1,-1
         if( p(k) < 200.e2 ) then
            exit
         end if
      end do
      delp = log( 200.e2/p(k) ) / log( p(k+1)/p(k) )
      t200 = t(k) + delp * (t(k+1) - t(k))

      end subroutine t_int

      subroutine photo_interp( zin, sin, vin, albin, t500in, &
                               t200in, ajout )
!----------------------------------------------------------------------
!       ... loglinear interpolation for the photodissociation rates
!           note: this subroutine computes photorates for a vertical
!                 column at a given longitude and latitude
!           this routine uses a six parameter table via a taylor
!           series expansion.
!           stacy walters, sep 30, 1996.  changed code to strictly limit
!           the 200mb and 500mb temperature interpolation to the table
!           endpoints; i.e. no extrapolation beyond the table is allowed.
!----------------------------------------------------------------------

      use mo_grid, only : plev

      implicit none

!----------------------------------------------------------------------
!        ... dummy arguments
!----------------------------------------------------------------------
      real, intent(in)  ::   zin(plev), &           ! geo height of midpoints
                             sin, &                 ! secant solar zenith angle
                             vin(plev), &           ! o3 column density
                             albin(plev), &         ! surface albedo
                             t500in, &              ! temp on 500mb surface
                             t200in                 ! temp on 200mb surface
      real, intent(out) ::   ajout(jdim,plev)       ! photodissociation rates

!----------------------------------------------------------------------
!        ... local variables
!----------------------------------------------------------------------
      integer, parameter ::  it200 = 1, it200p1 = 2
      integer  ::  iz, is, iv, ial, nn, it500
      integer  ::  izp1, isp1, ivp1, ialp1, it500p1
      integer  ::  i, k
      integer  ::  izl
      integer, dimension(plev) :: altind, ratind, albind
      real     ::  wght0
      real     ::  v3std
      real     ::  dels(6)
      real, dimension(plev) :: v3rat

!----------------------------------------------------------------------
!        ... find the zenith angle index ( same for all levels )
!----------------------------------------------------------------------
      do is = 1,zangdim
         if( vsec(is) > sin ) then
            exit
         end if
      end do
      is       = max( min( is,zangdim ) - 1,1 )
      isp1     = is + 1
      dels(2)  = min( 1.,max( 0.,(sin - vsec(is)) * delang(is) ) )

!----------------------------------------------------------------------
!        ... find the 500mb temp index ( same for all levels )
!----------------------------------------------------------------------
      do it500 = 1,t500dim
         if( t500(it500) > t500in ) then
            exit
         end if
      end do
      it500    = max( min( it500,t500dim ) - 1,1 )
      it500p1  = it500 + 1
      dels(5)  = min( 1.,max( 0.,(t500in - t500(it500)) * delt500(it500) ) )

!----------------------------------------------------------------------
!        ... find the 200mb temp index ( same for all levels )
!----------------------------------------------------------------------
      dels(6)  = min( 1.,max( 0.,(t200in - t200(it200)) * delt200(it200) ))

      izl = 1
      do k = plev,1,-1
!----------------------------------------------------------------------
!        ... find albedo indicies
!----------------------------------------------------------------------
         do ial = 1,albdim
            if( albev(ial) > albin(k) ) then
               exit
            end if
         end do
         albind(k) = max( min( ial,albdim ) - 1,1 )
!----------------------------------------------------------------------
!        ... find level indicies
!----------------------------------------------------------------------
         do iz = izl,altdim
            if( zz(iz) > zin(k) ) then
               izl = iz
               exit
            end if
         end do
         altind(k) = max( min( iz,altdim ) - 1,1 )
!----------------------------------------------------------------------
!        ... find "o3 ratio" indicies
!----------------------------------------------------------------------
         i        = max( min( 49,int( zin(k) ) ),0 )
         v3std    = vo3(i) + (zin(k) - real(i)) * delvo3(i)
         v3rat(k) = vin(k) / v3std
         do iv = 1,o3ratdim
            if( xv3(iv) > v3rat(k) ) then
               exit
            end if
         end do
         ratind(k) = max( min( iv,o3ratdim ) - 1,1 )
      end do
vert_loop : &
      do k = 1,plev
         iz    = altind(k)
         izp1  = iz + 1
         iv    = ratind(k)
         ivp1  = iv + 1
         ial   = albind(k)
         ialp1 = ial + 1
!----------------------------------------------------------------------
!        ... interval deltas and primary weight
!----------------------------------------------------------------------
         dels(1)  = min( 1.,max( 0.,(zin(k) - zz(iz)) * delz(iz) ) )
         dels(3)  = min( 1.,max( 0.,(v3rat(k) - xv3(iv)) * delv(iv) ) )
         dels(4)  = min( 1.,max( 0.,(albin(k) - albev(ial)) * delalb(ial) ) )
         wght0    = 1. - sum( dels )
rate_loop : &
         do nn = 1,jdim
            ajout(nn,k) = exp( wght0     * ajl(nn,iz,is,iv,ial,it500,it200) &
                               + dels(1) * ajl(nn,izp1,is,iv,ial,it500,it200) &
                               + dels(2) * ajl(nn,iz,isp1,iv,ial,it500,it200) &
                               + dels(3) * ajl(nn,iz,is,ivp1,ial,it500,it200) &
                               + dels(4) * ajl(nn,iz,is,iv,ialp1,it500,it200) &
                               + dels(5) * ajl(nn,iz,is,iv,ial,it500p1,it200) &
                               + dels(6) * ajl(nn,iz,is,iv,ial,it500,it200p1) )
         end do rate_loop
      end do vert_loop

      end subroutine photo_interp

      subroutine set_ub_col( col_delta, vmr, pdel, plonl )
!---------------------------------------------------------------
!        ... set the column densities at the upper boundary
!---------------------------------------------------------------

      use mo_chem_mods, only : ncol_abs
      use mo_grid,      only : pcnstm1
      use ppgrid,       only : pver

      implicit none

!---------------------------------------------------------------
!        ... dummy args
!---------------------------------------------------------------
      integer, intent(in) ::  plonl
      real, intent(out)   ::  col_delta(:,0:,:)                        ! /cm**2
      real, intent(in)    ::  vmr(plonl,pver,pcnstm1), &               ! xported species vmr
                              pdel(plonl,pver)

!---------------------------------------------------------------
!        note: xfactor = 10.*r/(k*g) in cgs units.
!              the factor 10. is to convert pdel
!              from pascals to dyne/cm**2.
!---------------------------------------------------------------
      real, parameter :: xfactor = 2.8704e21/(9.80616*1.38044)
      integer :: k, spc_ndx

!---------------------------------------------------------------
!        ... assign column density at the upper boundary
!            the first column is o3 and the second is o2.
!            add 10 du o3 column above top of model.
!---------------------------------------------------------------
      col_delta = 0.0  ! wtw

      spc_ndx = ox_ndx
      if( spc_ndx < 1 ) then
         spc_ndx = o3_ndx
      end if
      if( spc_ndx > 0 ) then
         col_delta(:,0,1) = 2.687e16*10.
         do k = 1,pver
            col_delta(:,k,1) = xfactor * pdel(:,k) * vmr(:,k,spc_ndx)
         end do
      end if
      col_delta(:,0,2) = 2.8e22
      do k = 1,pver
         col_delta(:,k,2) = xfactor * pdel(:,k) * .21
      end do

      end subroutine set_ub_col

      subroutine setcol( col_delta, col_dens, vmr, pdel, plonl )
!---------------------------------------------------------------
!       ... set the column densities
!---------------------------------------------------------------

      use mo_chem_mods, only : ncol_abs
      use mo_grid,      only : pcnstm1
      use ppgrid,       only : pver

      implicit none

!---------------------------------------------------------------
!       ... dummy arguments
!---------------------------------------------------------------
      integer, intent(in) :: plonl
      real, intent(in)  ::   vmr(plonl,pver,pcnstm1)           ! xported species vmr
      real, intent(in)  ::   pdel(plonl,pver)                  ! delta about midpoints
      real, intent(in)  ::   col_delta(:,0:,:)                 ! layer column densities (molecules/cm^2)
      real, intent(out) ::   col_dens(:,:,:)                    ! column densities ( /cm**2 )

!---------------------------------------------------------------
!        the local variables
!---------------------------------------------------------------
      integer  ::   i, k, km1      ! long, alt indicies
      integer  ::   spc_ndx

!---------------------------------------------------------------
!        note: xfactor = 10.*r/(k*g) in cgs units.
!              the factor 10. is to convert pdel
!              from pascals to dyne/cm**2.
!---------------------------------------------------------------
      real, parameter :: xfactor = 2.8704e21/(9.80616*1.38044)

!---------------------------------------------------------------
!       ... compute column densities down to the
!           current eta index in the calling routine.
!           the first column is o3 and the second is o2.
!---------------------------------------------------------------
      col_dens = 0.0      

      spc_ndx = ox_ndx
      if( spc_ndx < 1 ) then
         spc_ndx = o3_ndx
      end if
      if( spc_ndx > 0 ) then
         col_dens(:,1,1) = col_delta(:,0,1) + .5 * col_delta(:,1,1)
         do k = 2,pver
            km1 = k - 1
            col_dens(:,k,1) = col_dens(:,km1,1) + .5 * (col_delta(:,km1,1) + col_delta(:,k,1))
         end do
      end if
      col_dens(:,1,2) = col_delta(:,0,2) + .5 * col_delta(:,1,2)
      do k = 2,pver
         km1 = k - 1
         col_dens(:,k,2) = col_dens(:,km1,2) + .5 * (col_delta(:,km1,2) + col_delta(:,k,2))
      end do

      end subroutine setcol

      subroutine diurnal_geom( longitude, latitude, time_of_year, polar_night, polar_day, &
                               sunon, sunoff, loc_angle, zen_angle )
!------------------------------------------------------------------
!       ... diurnal geometry factors
!------------------------------------------------------------------

      use mo_const_mozart, only : pi, twopi, pid2, dayspy, d2r

      implicit none

!------------------------------------------------------------------
!       ... dummy arguments
!------------------------------------------------------------------
      real,    intent(in)  ::     latitude           ! latitude 
      real,    intent(in)  ::     longitude          ! latitude 
      real, intent(in)     ::     time_of_year       ! time of year
      real, intent(out)    ::     sunon           ! sunrise angle in radians
      real, intent(out)    ::     sunoff          ! sunset angle in radians
      real, intent(out)    ::     zen_angle       ! solar zenith angle
      real, intent(out)    ::     loc_angle       ! "local" time angle
      logical, intent(out) ::     polar_day       ! continuous daylight flag
      logical, intent(out) ::     polar_night     ! continuous night flag

!------------------------------------------------------------------
!        ... local variables
!------------------------------------------------------------------
      real    ::  dec_max
      real    ::  declination
      real    ::  doy_loc            ! day of year
      real    ::  tod                ! time of day
      real    ::  sin_dec, cos_dec   ! sin, cos declination
      real    ::  cosphi             ! cos latitude
      real    ::  sinphi             ! sin latitude

      dec_max     = 23.45 * d2r
      sinphi      = sin( latitude )
      cosphi      = cos( latitude )
      polar_day   = .false.
      polar_night = .false.
!------------------------------------------------------------------
!        note: this formula assumes a 365 day year !
!------------------------------------------------------------------
      doy_loc     = aint( time_of_year )
      declination = dec_max * cos((doy_loc - 172.)*twopi/dayspy)
!------------------------------------------------------------------
!        determine if in polar day or night
!        if not in polar day or night then
!        calculate terminator longitudes
!------------------------------------------------------------------
      if( abs(latitude) >= (pid2 - abs(declination)) ) then
         if( sign(1.,declination) == sign(1.,latitude) ) then
            polar_day = .true.
            sunoff    = 2.*twopi
            sunon     = -twopi
         else
            polar_night  = .true.
            zen_angle = -1.0
            return
         end if
      else
         sunoff = acos( -tan(declination)*tan(latitude) )
         sunon  = twopi - sunoff
      end if

      sin_dec = sin( declination )
      cos_dec = cos( declination )
!------------------------------------------------------------------
!       ... compute base for zenith angle
!------------------------------------------------------------------
      tod = (time_of_year - doy_loc) + .5
!-------------------------------------------------------------------
!        note: longitude 0 (greenwich) at 0:00 hrs
!              maps to local angle = pi
!-------------------------------------------------------------------
      loc_angle = (tod + longitude/2.8125*180./pi/128.)*twopi
      loc_angle = mod( loc_angle,twopi )

      if( polar_day ) then
         zen_angle = acos( sinphi*sin_dec + cosphi*cos_dec*cos(loc_angle) )
      else
         if( loc_angle <= sunoff .or. loc_angle >= sunon ) then
            zen_angle = acos( sinphi*sin_dec + cosphi*cos_dec*cos(loc_angle) )
         else
            zen_angle = -1.
         endif
      end if

      end subroutine diurnal_geom

      real function sundis( idate )
!-----------------------------------------------------------------------------
!=  purpose:                                                                 =*
!=  calculate earth-sun distance variation for a given date.  based on       =*
!=  fourier coefficients originally from:  spencer, j.w., 1971, fourier      =*
!=  series representation of the position of the sun, search, 2:172          =*
!-----------------------------------------------------------------------------*
!=  parameters:                                                              =*
!=  idate  - integer, specification of the date, from yymmdd              (i)=*
!=  esrm2  - real, variation of the earth-sun distance                    (o)=*
!=           esrm2 = (average e/s dist)^2 / (e/s dist on day idate)^2        =*
!-----------------------------------------------------------------------------*
!=  edit history:                                                            =*
!=  01/95  changed computation of trig function values                       =*
!-----------------------------------------------------------------------------*
!= this program is free software;  you can redistribute it and/or modify     =*
!= it under the terms of the gnu general public license as published by the  =*
!= free software foundation;  either version 2 of the license, or (at your   =*
!= option) any later version.                                                =*
!= the tuv package is distributed in the hope that it will be useful, but    =*
!= without any warranty;  without even the implied warranty of merchantibi-  =*
!= lity or fitness for a particular purpose.  see the gnu general public     =*
!= license for more details.                                                 =*
!= to obtain a copy of the gnu general public license, write to:             =*
!= free software foundation, inc., 675 mass ave, cambridge, ma 02139, usa.   =*
!-----------------------------------------------------------------------------*
!= to contact the authors, please mail to:                                   =*
!= sasha madronich, ncar/acd, p.o.box 3000, boulder, co, 80307-3000, usa  or =*
!= send email to:  sasha@ucar.edu                                            =*
!-----------------------------------------------------------------------------*
!= copyright (c) 1994,95,96  university corporation for atmospheric research =*
!-----------------------------------------------------------------------------

      use mo_const_mozart, only : pi
      use abortutils,      only : endrun                      !add by zf 2008.06.23

      implicit none

!-----------------------------------------------------------------------------
!       ... dummy arguments
!-----------------------------------------------------------------------------
      integer, intent(in) :: idate

!-----------------------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------------------
      integer :: iyear, imonth, iday, mday, month, jday
      integer, save :: imn(12) = (/ 31,28,31,30,31,30,31,31,30,31,30,31 /)
      real    :: dayn, thet0
      real    :: sinth, costh, sin2th, cos2th

!-----------------------------------------------------------------------------
!       ... parse date to find day number (julian day)
!-----------------------------------------------------------------------------
      iyear  = int( idate/10000 )
      imonth = int( (idate - 10000*iyear)/100 )
      iday   = idate - (10000*iyear + 100*imonth)

      if( imonth > 12 ) then
         write(*,*) 'month in date exceeds 12'
         write(*,*) 'date = ', idate
         write(*,*) 'month = ', imonth
         call endrun
      end if

      if( mod(iyear,4) == 0 ) then
         imn(2) = 29
      else
         imn(2) = 28
      end if

      if( iday > imn(imonth) ) then
         write(*,*) 'day in date exceeds days in month'
         write(*,*) 'date = ', idate
         write(*,*) 'day = ', iday
         call endrun()
      end if

      mday = 0
      do month = 1,imonth-1
         mday = mday + imn(month)
      end do
      jday = mday + iday
      dayn = real(jday - 1) + .5

!-----------------------------------------------------------------------------
!       ... define angular day number and compute esrm2:
!-----------------------------------------------------------------------------
      thet0 = 2.*pi*dayn/365.

!-----------------------------------------------------------------------------
!       ... calculate sin(2*thet0), cos(2*thet0)
!-----------------------------------------------------------------------------
      sinth   = sin( thet0 )
      costh   = cos( thet0 )
      sin2th  = 2.*sinth*costh
      cos2th  = costh*costh - sinth*sinth
      sundis  = 1.000110 + .034221*costh  +  .001280*sinth + .000719*cos2th +  .000077*sin2th

      end function sundis

      end module mo_photo
