      module mo_setext

      private
      public :: setext_inti, setext

      save

      integer :: co_ndx, no_ndx, ch4_ndx

      contains

      subroutine setext_inti
!--------------------------------------------------------
!       ... Initialize the external forcing module
!--------------------------------------------------------

      use mo_chem_utls, only : get_extfrc_ndx
      use pmgrid,       only : masterproc    !add by zf 2008.06.20

      implicit none

      co_ndx = get_extfrc_ndx( 'CO' )
      no_ndx = get_extfrc_ndx( 'NO' )
      ch4_ndx = get_extfrc_ndx( 'CH4' )

      if (masterproc) then     !add by zf 2008.06.20
      write(*,*) ' '
      write(*,*) 'setext_inti: diagnostics '
      write(*,'(10i5)') co_ndx, no_ndx, ch4_ndx
      endif

      end subroutine setext_inti

      subroutine setext( extfrc, zint, cldtop, plonl, lchnk , &
                         em3_IPCC6,    nem3_IPCC6,            &      
                         em3data, nem3 ,prod_no)               ! wtw 2012.10
!--------------------------------------------------------
!     ... for this latitude slice:
!         - form the nox production from lighting
!         - form the nox production from airplanes
!         - form the co production from airplanes
!         - form the ch4 production from airplanes
!--------------------------------------------------------

      use mo_grid,     only : plev, plevp
      use mo_airplane, only : pnobdy, pcobdy, pch4bdy, air_altitude
      use mo_chem_mods,only : extcnt
      use prescribed_em_d3 , only: idx3_NO2, &
                                   idx3_SO2, idx3_CB2, idx3_SO4
      use prescribed_emis_3d, only: idxCO_IPCC6, idxNO_IPCC6

      implicit none

!--------------------------------------------------------
!     ... dummy arguments
!--------------------------------------------------------
      integer, intent(in) ::   plonl                              ! lon tile dim
      integer, intent(in) ::   lchnk                              ! incoming longitude tile index
      real, intent(in)    ::   prod_no(plonl,plev) 
      real, intent(in)    ::   zint(plonl,plevp)                  ! interface geopot height ( km )
      real, intent(in)    ::   cldtop(plonl)                      ! cloud top index
      real, intent(out)   ::   extfrc(plonl,plev,max(1,extcnt))   ! the "extraneous" forcing
!---
! wtw 2012.10
!
      integer, intent(in) ::  nem3
      real, intent(in)    ::  em3data(plonl,plev,nem3)            ! kg(NO).m-3.s-1 --> mol.cm-3.s-1
      integer, intent(in) ::  nem3_IPCC6
      real, intent(in)    ::  em3_IPCC6(plonl,plev,nem3_IPCC6)    ! mol.cm-3.s-1
      integer   ksource
!--------------------------------------------------------
!     ... local variables
!--------------------------------------------------------
      integer :: i, k, k1, kk, cldind
      real    :: frac
      real    :: zlev_top, zlev_bot, zdel, zdeli

!--------------------------------------------------------
!     ... set nox production from lighting
!         note: from ground to cloud top production is c shaped
!--------------------------------------------------------
      extfrc(:,:,:) = 0.
      if( co_ndx > 0 .or. no_ndx > 0 .or. ch4_ndx > 0 ) then
         if( no_ndx > 0 ) then
            do i = 1,plonl
               cldind = nint( cldtop(i) )
               if( cldind < plev .and. cldind > 0  ) then
              !   extfrc(i,cldind:plev,no_ndx) = prod_no(i,cldind:plev,lchnk)
                  extfrc(i,cldind:plev,no_ndx) = prod_no(i,cldind:plev)
               end if
            end do
         end if
!---------------------------------------------------------------------
!     ... add the airplane emissions; must do vertical interpolation
!---------------------------------------------------------------------
       ksource = 1      ! 1 == use the IPCC AR6 airplane emission NO data
                        ! others == use other data
       if (ksource .eq. 1) then
          if( no_ndx > 0 ) extfrc(:, :, no_ndx) = em3_IPCC6(:, :, idxNO_IPCC6)
          if( co_ndx > 0 ) extfrc(:, :, co_ndx) = em3_IPCC6(:, :, idxCO_IPCC6)
       else

        do k = 1,plev
           do i = 1,plonl
               zlev_top = zint(i,k)                 ! altitude at top of model level (km)
               zlev_bot = zint(i,k+1)               ! altitude at bottom of model level (km)
               zdel = (zlev_top - zlev_bot) * 1.e5  ! model level thickness (cm)

               zdeli = 1./zdel
               if( zlev_bot <= maxval(air_altitude) .and. zlev_top >= minval(air_altitude) ) then
                  do kk = 1,size(air_altitude)-1
                     if( zlev_bot <= air_altitude(kk+1) .and. zlev_top >= air_altitude(kk) ) then
                        frac = (min( zlev_top, air_altitude(kk+1) ) - max( zlev_bot, air_altitude(kk) )) &
                               /(air_altitude(kk+1) - air_altitude(kk))
                        if( no_ndx > 0 ) then
                            if( idx3_NO2 > 0 ) then
                                extfrc(i,k,no_ndx) = extfrc(i,k,no_ndx) + em3data(i,k,idx3_NO2) * (1/1000)*(1./30.0) * 6.02e+23
                                         ! use  the IPCC AR5 airplane emission NO data           
                                         ! from kg(NO).m-3.s-1 --> mol.cm-3.s-1

                            else
                                extfrc(i,k,no_ndx) = extfrc(i,k,no_ndx) + frac * pnobdy(i,kk,lchnk) * zdeli
                            endif
                        end if
                        if( co_ndx > 0 ) then
                           extfrc(i,k,co_ndx) = extfrc(i,k,co_ndx) + frac * pcobdy(i,kk,lchnk) * zdeli
                        end if
                        if( ch4_ndx > 0 ) then
                           extfrc(i,k,ch4_ndx) = extfrc(i,k,ch4_ndx) + frac * pch4bdy(i,kk,lchnk) * zdeli
                        end if
                     end if
                  end do
               end if
           end do
        end do
       end if    ! ksource = 1
!-------------------
      end if

      end subroutine setext

      end module mo_setext
