
      module mo_setinv

      contains

      subroutine setinv( invariants, tfld, h2ovmr, pmid, plonl )
!-----------------------------------------------------------------
!        ... set the invariant densities (molecules/cm**3)
!-----------------------------------------------------------------
      
      use pmgrid      , only : plev
      use mo_chem_mods, only : nfs

      implicit none

!-----------------------------------------------------------------
!        ... dummy arguments
!-----------------------------------------------------------------
      integer, intent(in) ::    plonl
      real, intent(in)  ::      tfld(plonl,plev)           ! temperature
      real, intent(in)  ::      h2ovmr(plonl,plev)         ! water vapor vmr
      real, intent(in)  ::      pmid(plonl,plev)           ! pressure
      real, intent(out) ::      invariants(plonl,plev,nfs) ! invariant array
      
!-----------------------------------------------------------------
!        .. local variables
!-----------------------------------------------------------------
      real, parameter ::  boltz = 1.38044e-16      ! erg/k

      integer :: k

!-----------------------------------------------------------------
!        note: invariants are in cgs density units.
!              the pmid array is in pascals and must be
!	       mutiplied by 10. to yield dynes/cm**2.
!-----------------------------------------------------------------

!-----------------------------------------------------------------
!	... set m, n2, o2, and h2o densities
!-----------------------------------------------------------------
      invariants = 0.0
      do k = 1,plev
         invariants(:,k,1) = 10. * pmid(:,k) / (boltz*tfld(:,k))
         invariants(:,k,2) = .79 * invariants(:,k,1)
         invariants(:,k,3) = .21 * invariants(:,k,1)
         invariants(:,k,4) = h2ovmr(:,k) * invariants(:,k,1)
      end do

      end subroutine setinv

      end module mo_setinv
