#include <misc.h>
#include <params.h>

      module mo_srf_emis
!---------------------------------------------------------------
! 	... Interpolation surface emissions
!---------------------------------------------------------------

      use pmgrid,         only: plon, plat, masterproc

      implicit none

      private
      public  :: srf_emis_inti, nppbdy, srf_emis, baseflux, landflux

      character(len=256), public  :: bndacet    ! full pathname for Mozart
      character(len=256), public  :: bndc2h4    ! full pathname for Mozart
      character(len=256), public  :: bndc3h6    ! full pathname for Mozart
      character(len=256), public  :: bndc2h6    ! full pathname for Mozart
      character(len=256), public  :: bndc3h8    ! full pathname for Mozart
      character(len=256), public  :: bndch2o    ! full pathname for Mozart
      character(len=256), public  :: bndch3oh   ! full pathname for Mozart
      character(len=256), public  :: bndch4     ! full pathname for Mozart
      character(len=256), public  :: bndco      ! full pathname for Mozart
      character(len=256), public  :: bndh2      ! full pathname for Mozart
      character(len=256), public  :: bndisoprene! full pathname for Mozart
      character(len=256), public  :: bndn2o     ! full pathname for Mozart
      character(len=256), public  :: bndnmv     ! full pathname for Mozart
      character(len=256), public  :: bndnox     ! full pathname for Mozart
      character(len=256), public  :: bndnpp     ! full pathname for Mozart
      character(len=256), public  :: bndterpense! full pathname for Mozart

      save

      real, allocatable, dimension(:,:,:) :: &
               noxembdy, ch4embdy, ch2oembdy, coembdy, c3h6embdy, &
               isoembdy, terpembdy, c2h4embdy, c2h6embdy, c4h10embdy, &
               n2oembdy, c3h8embdy, acetembdy, h2embdy, ch3ohembdy, &
               nppbdy

      real ::  baseflux = 0., &
               landflux = 0.

      real    :: sf(plat)
      real    :: factor
      integer :: no_ndx, ch4_ndx, co_ndx, c3h6_ndx, isop_ndx, c10h16_ndx, &
                 c2h4_ndx, c2h6_ndx, c3h8_ndx, ch3coch3_ndx, c4h10_ndx, &
		 n2o_ndx, h2_ndx, ch3oh_ndx, Rn_ndx, ch2o_ndx

      integer :: dms_ndx, so2_ndx, cb1_ndx, cb2_ndx, oc1_ndx, oc2_ndx
      integer :: c2h5oh_ndx, ch3cho_ndx,  hcn_ndx, nh3_ndx, ch3cocho_ndx  
 
      logical :: no_em, ch4_em, co_em, c3h6_em, isop_em, c10h16_em, &
                 c2h4_em, c2h6_em, c3h8_em, ch3coch3_em, c4h10_em, &
		 n2o_em, h2_em, ch3oh_em, Rn_em, ch2o_em

      contains

      subroutine srf_emis_inti( plonl, platl, pplon )
!-----------------------------------------------------------------------
! 	... Read the primary emission flux datasets 
!-----------------------------------------------------------------------

      use mo_chem_utls,     only : get_spc_ndx, has_srfems
      use mo_sim_chm,       only : latwts    !zf test
      use mo_chem_mods,     only : adv_mass
      use mo_const_mozart,  only : d2r, pi, rearth

!add by zf 2008.03.13
      use ppgrid,         only:  pcols, begchunk, endchunk   !2008.08.11
      use phys_grid,      only:  scatter_field_to_chunk      !2008.08.11
      use ioFileMod,      only: getfil
      use abortutils,     only: endrun
!end zf
      implicit none

!-----------------------------------------------------------------------
! 	... dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) :: plonl
      integer, intent(in) :: platl
      integer, intent(in) :: pplon

!-----------------------------------------------------------------------
! 	... Local variables
!-----------------------------------------------------------------------
      real, allocatable, dimension(:,:,:) :: &
               noxem, ch4em, ch2oem, coem, c3h6em, &
               isoem, terpem, c2h4em, c2h6em, c4h10em, &
               n2oem, c3h8em, acetem, h2em, ch3ohem, &
               npp

      real, parameter :: amufac = 1.65979e-23         ! 1.e4* kg / amu
      integer :: astat
      integer :: i, j, ncid                              ! Indices
      real    :: mw
      real    :: seq
      character(len=10) :: species
      character(len=256) :: locfn    ! netcdf local filename to open   add by zf 2008.03.13

!-----------------------------------------------------------------------
! 	... Setup the indicies
!-----------------------------------------------------------------------
      no_ndx       = get_spc_ndx( 'NO' )
      ch4_ndx      = get_spc_ndx( 'CH4' )
      ch2o_ndx     = get_spc_ndx( 'CH2O' )
      co_ndx       = get_spc_ndx( 'CO' )
      c3h6_ndx     = get_spc_ndx( 'C3H6' )
      isop_ndx     = get_spc_ndx( 'ISOP' )
      c10h16_ndx   = get_spc_ndx( 'C10H16' )
      c2h4_ndx     = get_spc_ndx( 'C2H4' )
      c2h6_ndx     = get_spc_ndx( 'C2H6' )
      c3h8_ndx     = get_spc_ndx( 'C3H8' )
      ch3coch3_ndx = get_spc_ndx( 'CH3COCH3' )
      c4h10_ndx    = get_spc_ndx( 'C4H10' )
      n2o_ndx      = get_spc_ndx( 'N2O' )
      h2_ndx       = get_spc_ndx( 'H2' )
      ch3oh_ndx    = get_spc_ndx( 'CH3OH' )
      Rn_ndx       = get_spc_ndx( 'Rn' )

      dms_ndx       = get_spc_ndx( 'DMS' )
      so2_ndx       = get_spc_ndx( 'SO2' )
      cb1_ndx       = get_spc_ndx( 'CB1' )
      cb2_ndx       = get_spc_ndx( 'CB2' )
      oc1_ndx       = get_spc_ndx( 'OC1' )
      oc2_ndx       = get_spc_ndx( 'OC2' )
      c2h5oh_ndx    = get_spc_ndx( 'C2H5OH' )
      ch3cho_ndx    = get_spc_ndx( 'CH3CHO' )
      hcn_ndx       = get_spc_ndx( 'HCN' )
      nh3_ndx       = get_spc_ndx( 'NH3' )
      ch3cocho_ndx  = get_spc_ndx( 'CH3COCHO' )

      no_em       = has_srfems( 'NO' )
      ch4_em      = has_srfems( 'CH4' )
      ch2o_em     = has_srfems( 'CH2O' )
      co_em       = has_srfems( 'CO' )
      c3h6_em     = has_srfems( 'C3H6' )
      isop_em     = has_srfems( 'ISOP' )
      c10h16_em   = has_srfems( 'C10H16' )
      c2h4_em     = has_srfems( 'C2H4' )
      c2h6_em     = has_srfems( 'C2H6' )
      c3h8_em     = has_srfems( 'C3H8' )
      ch3coch3_em = has_srfems( 'CH3COCH3' )
      c4h10_em    = has_srfems( 'C4H10' )
      n2o_em      = has_srfems( 'N2O' )
      h2_em       = has_srfems( 'H2' )
      ch3oh_em    = has_srfems( 'CH3OH' )
      Rn_em       = has_srfems( 'Rn' )

      if (masterproc) then
      write(*,*) 'srf_emis_inti : diagnostic'
      write(*,'(10i5)') no_ndx, ch4_ndx, co_ndx, c3h6_ndx, isop_ndx, c10h16_ndx, &
                 c2h4_ndx, c2h6_ndx, c3h8_ndx, ch3coch3_ndx, c4h10_ndx, &
		 n2o_ndx, h2_ndx, ch3oh_ndx, Rn_ndx, ch2o_ndx
      write(*,*)  no_em, ch4_em, co_em, c3h6_em, isop_em, c10h16_em, &
                 c2h4_em, c2h6_em, c3h8_em, ch3coch3_em, c4h10_em, &
		 n2o_em, h2_em, ch3oh_em, Rn_em, ch2o_em
      endif

!-----------------------------------------------------------------------
! 	... allocate working arrays
!-----------------------------------------------------------------------
      allocate( noxem(plonl,platl,12), ch4em(plonl,platl,12), &
                ch2oem(plonl,platl,12), coem(plonl,platl,12), &
                c3h6em(plonl,platl,12), isoem(plonl,platl,12), &
                terpem(plonl,platl,12), c2h4em(plonl,platl,12), &
                c2h6em(plonl,platl,12), c4h10em(plonl,platl,12), &
                n2oem(plonl,platl,12), c3h8em(plonl,platl,12), &
                acetem(plonl,platl,12), h2em(plonl,platl,12), &
                ch3ohem(plonl,platl,12), npp(plonl,platl,12), &
		stat=astat )
      if( astat/= 0 ) then
	 write(*,*) 'srf_emis_inti: failed to allocate wrking arrays; error = ',astat
	 call endrun
      end if

!-----------------------------------------------------------------------
! 	... Diagnostics setup
!-----------------------------------------------------------------------
      seq = 2.*pi*1.e4*rearth**2/REAL(plon)
      sf(:plat) = seq*latwts(:plat)
      factor = 86400. * 365. &   ! sec / year
             / 6.022e23 &        ! molec / mole 
             * 1.e-12            ! Tg / g

      if (masterproc) then
!-----------------------------------------------------------------------
! 	... NO
!-----------------------------------------------------------------------
      if( no_em ) then
         species  = 'NO'
         mw       = 14.00674                      ! g N / mole
         call getfil (bndnox, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'TgN/y', ncid, noxem )
         noxem(:,:,:)   = amufac * adv_mass(no_ndx) * noxem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... CO
!-----------------------------------------------------------------------
      if( co_em ) then
         species  = 'CO'
         mw       = adv_mass(co_ndx)               ! g / mole
         call getfil (bndco, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'Tg/y', ncid, coem )
         coem(:,:,:) = amufac * adv_mass(co_ndx) * coem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... CH4
!-----------------------------------------------------------------------
      if( ch4_em ) then
         species  = 'CH4'
         mw       = adv_mass(ch4_ndx)               ! g / mole
         call getfil (bndch4, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'Tg/y', ncid, ch4em )
         ch4em(:,:,:)   = amufac * adv_mass(ch4_ndx) * ch4em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... C2H6
!-----------------------------------------------------------------------
      if( c2h6_em ) then
         species  = 'C2H6'
         mw       = 2.*12.011                       ! g C / mole
         call getfil (bndc2h6, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'TgC/y', ncid, c2h6em )
         c2h6em(:,:,:) = amufac * adv_mass(c2h6_ndx) * c2h6em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... C3H8
!-----------------------------------------------------------------------
      if( c3h8_em ) then
         species  = 'C3H8'
         mw       = 3.*12.011                       ! g C / mole
         call getfil (bndc3h8, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'TgC/y', ncid, c3h8em )
         c3h8em(:,:,:) = amufac * adv_mass(c3h8_ndx) * c3h8em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... C2H4
!-----------------------------------------------------------------------
      if( c2h4_em ) then
         species  = 'C2H4'
         mw       = 2.*12.011                       ! g C / mole
         call getfil (bndc2h4, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'TgC/y', ncid, c2h4em )
         c2h4em(:,:,:) = amufac * adv_mass(c2h4_ndx) * c2h4em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... C3H6
!-----------------------------------------------------------------------
      if( c3h6_em ) then
         species  = 'C3H6'
         mw       = 3.*12.011                       ! g C / mole
         call getfil (bndc3h6, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'TgC/y', ncid, c3h6em )
         c3h6em(:,:,:) = amufac * adv_mass(c3h6_ndx) * c3h6em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... ACET
!-----------------------------------------------------------------------
      if( ch3coch3_em ) then
         species  = 'ACET'
         mw       = adv_mass(ch3coch3_ndx)           ! g / mole
         call getfil (bndacet, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'Tg/y', ncid, acetem )
         acetem(:,:,:) = amufac * adv_mass(ch3coch3_ndx) * acetem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... CH2O
!-----------------------------------------------------------------------
      if( ch2o_em ) then
         species  = 'CH2O'
         mw       = adv_mass(ch2o_ndx)               ! g / mole
         call getfil (bndch2o, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'Tg/y', ncid, ch2oem )
         ch2oem(:,:,:)  = amufac * adv_mass(ch2o_ndx) * ch2oem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... C4H10
!-----------------------------------------------------------------------
      if( c4h10_em ) then
         species  = 'C4H10'
         mw       = 4.*12.011                        ! g C / mole
         call getfil (bndnmv, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'TgC/y', ncid, c4h10em )
         c4h10em(:,:,:) = amufac * adv_mass(c4h10_ndx) * c4h10em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... ISO
!-----------------------------------------------------------------------
      if( isop_em ) then
         species  = 'ISOP'
         mw       = 5.*12.011                      ! g C / mole
         call getfil (bndisoprene, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'TgC/y', ncid, isoem )
         isoem(:,:,:) = amufac * adv_mass(isop_ndx) * isoem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... TERP
!-----------------------------------------------------------------------
      if( c10h16_em ) then
         species  = 'TERP'
         mw       = 10.*12.011                     ! g C / mole
         call getfil (bndterpense, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'TgC/y', ncid, terpem )
         terpem(:,:,:) = amufac * adv_mass(c10h16_ndx) * terpem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... N2O
!-----------------------------------------------------------------------
      if( n2o_em ) then
         species  = 'N2O'
         mw       = adv_mass(n2o_ndx)               ! g / mole
         call getfil (bndn2o , locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'Tg/y', ncid, n2oem )
         n2oem(:,:,:) = amufac * adv_mass(n2o_ndx) * n2oem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... H2
!-----------------------------------------------------------------------
      if( h2_em ) then
         species  = 'H2'
         mw       = adv_mass(h2_ndx)               ! g / mole
         call getfil (bndh2, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'Tg/y', ncid, h2em )
         h2em(:,:,:) = amufac * adv_mass(h2_ndx) * h2em(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... CH3OH
!-----------------------------------------------------------------------
      if( ch3oh_em ) then
         species  = 'CH3OH'
         mw       = adv_mass(ch3oh_ndx)               ! g / mole
         call getfil (bndch3oh, locfn)
         call wrap_open(locfn,0,ncid)
         call read_emis( plonl, platl, pplon, species, mw, 'Tg/y', ncid, ch3ohem )
         ch3ohem(:,:,:) = amufac * adv_mass(ch3oh_ndx) * ch3ohem(:,:,:)
      end if

!-----------------------------------------------------------------------
! 	... NPP
!-----------------------------------------------------------------------
      species  = 'NPP'
      mw       = 1.               ! g / mole
      call getfil (bndnpp, locfn)
      call wrap_open(locfn,0,ncid)
      call read_emis( plonl, platl, pplon, species, mw, '', ncid, npp )

      endif

      allocate( noxembdy(pcols,begchunk:endchunk,12), ch4embdy(pcols,begchunk:endchunk,12), &
                ch2oembdy(pcols,begchunk:endchunk,12), coembdy(pcols,begchunk:endchunk,12), &
                c3h6embdy(pcols,begchunk:endchunk,12), isoembdy(pcols,begchunk:endchunk,12), &
                terpembdy(pcols,begchunk:endchunk,12), c2h4embdy(pcols,begchunk:endchunk,12), &
                c2h6embdy(pcols,begchunk:endchunk,12), c4h10embdy(pcols,begchunk:endchunk,12), &
                n2oembdy(pcols,begchunk:endchunk,12), c3h8embdy(pcols,begchunk:endchunk,12), &
                acetembdy(pcols,begchunk:endchunk,12), h2embdy(pcols,begchunk:endchunk,12), &
                ch3ohembdy(pcols,begchunk:endchunk,12), nppbdy(pcols,begchunk:endchunk,12), &
		stat=astat )
      if( astat/= 0 ) then
	 write(*,*) 'srf_emis_inti: failed to allocate wrking arrays; error = ',astat
	 call endrun
      end if

      call scatter_field_to_chunk(1,1,12,plonl,noxem,noxembdy)
      call scatter_field_to_chunk(1,1,12,plonl,ch4em,ch4embdy)
      call scatter_field_to_chunk(1,1,12,plonl,ch2oem,ch2oembdy)
      call scatter_field_to_chunk(1,1,12,plonl,coem,coembdy)
      call scatter_field_to_chunk(1,1,12,plonl,c3h6em,c3h6embdy)
      call scatter_field_to_chunk(1,1,12,plonl,isoem,isoembdy)
      call scatter_field_to_chunk(1,1,12,plonl,terpem,terpembdy)
      call scatter_field_to_chunk(1,1,12,plonl,c2h4em,c2h4embdy)
      call scatter_field_to_chunk(1,1,12,plonl,c2h6em,c2h6embdy)
      call scatter_field_to_chunk(1,1,12,plonl,c4h10em,c4h10embdy)
      call scatter_field_to_chunk(1,1,12,plonl,n2oem,n2oembdy)
      call scatter_field_to_chunk(1,1,12,plonl,c3h8em,c3h8embdy)
      call scatter_field_to_chunk(1,1,12,plonl,acetem,acetembdy)
      call scatter_field_to_chunk(1,1,12,plonl,h2em,h2embdy)
      call scatter_field_to_chunk(1,1,12,plonl,ch3ohem,ch3ohembdy)
      call scatter_field_to_chunk(1,1,12,plonl,npp,nppbdy)

      end subroutine srf_emis_inti

      subroutine read_emis( plonl, platl, pplon, species, mw, units, ncid, emis )
!-----------------------------------------------------------------------
!       ... Read surface emissions from NetCDF file
!-----------------------------------------------------------------------

      use mo_netcdf

      use mo_const_mozart,  only : phi, lam, d2r

      use abortutils,     only: endrun
      use error_messages, only: handle_ncerr

      implicit none
!-----------------------------------------------------------------------
!       ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) :: plonl, platl, pplon
      integer, intent(in) :: ncid
      real, intent(in)    :: mw
      real, dimension(plonl,platl,12), intent(out) :: emis
      character(len=*), intent(in) :: species, units

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer :: j, jl, ju, m               ! Indices
      integer :: jlim(2)
      integer :: ierr
      integer :: vid, nlat, nlon, nmonth, nvars, ndims, attlen
      integer :: dimid_lat, dimid_lon, dimid_month
      integer, allocatable :: dimid(:)
      integer :: gndx
      real, allocatable :: lat(:), lon(:)
      real :: total, total_wrk, scale_factor
      real :: wrk(plonl,platl,12)
      character(len=NF_MAX_NAME) :: varname
      character(len=100) :: var_longname

!-----------------------------------------------------------------------
!       ... Get grid dimensions from file
!-----------------------------------------------------------------------
      call handle_ncerr( NF_INQ_DIMID( ncid, 'lat', dimid_lat ), &
                         'read_emis: Failed to find dimension lat for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lat, nlat ), &
                         'read_emis: Failed to get length of dimension lat for species ' // &
                         trim(species) )
      allocate( lat(nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'read_emis: lat allocation error = ',ierr
         call endrun
      end if
      call handle_ncerr( NF_INQ_VARID( ncid, 'lat', vid ), &
                         'read_emis: Failed to find variable lat for species ' // trim(species) )
      call handle_ncerr( NF_GET_VAR_DOUBLE( ncid, vid, lat ), &
                         'read_emis: Failed to read variable lat for species ' // trim(species) )
      lat(:nlat) = lat(:nlat) * d2r
 
      call handle_ncerr( NF_INQ_DIMID( ncid, 'lon', dimid_lon ), &
                         'read_emis: Failed to find dimension lon for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lon, nlon ), &
                         'read_emis: Failed to find dimension lon for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lon, nlon ), &
                         'read_emis: Failed to get length of dimension lon for species ' // &
                         trim(species) )
      allocate( lon(nlon), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'read_emis: lon allocation error = ',ierr
         call endrun
      end if
      call handle_ncerr( NF_INQ_VARID( ncid, 'lon', vid ), &
                         'read_emis: Failed to find variable lon for species ' // trim(species) )
      call handle_ncerr( NF_GET_VAR_DOUBLE( ncid, vid, lon ), &
                         'read_emis: Failed to read variable lon for species ' // trim(species) )
      lon(:nlon) = lon(:nlon) * d2r
 
      call handle_ncerr( NF_INQ_DIMID( ncid, 'month', dimid_month ), &
                         'read_emis: Failed to find dimension month for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_month, nmonth ), &
                         'read_emis: Failed to get length of dimension month for species ' // &
                         trim(species) )
      if( nmonth /= 12 ) then
         write(*,*) 'read_emis: Error, number of months = ',nmonth,' for species ' // trim(species)
         call endrun
      end if

      call handle_ncerr( NF_INQ_NVARS( ncid, nvars ), &
                         'read_emis: Failed to get number of variables for species ' // &
                         trim(species) )

!-----------------------------------------------------------------------
!       ... Set total emissions to zero
!-----------------------------------------------------------------------
      emis(:,:,:) = 0.
!-----------------------------------------------------------------------
!       ... Loop over file variables
!-----------------------------------------------------------------------
      write(*,*) 'Annual average ',trim(species),' emissions ',trim(units)
      do vid = 1,nvars
         ierr = NF_INQ_VARNAME( ncid, vid, varname )
         if( ierr /= 0 ) then
            write(*,*) 'read_emis: Failed to get name of variable # ',vid, &
                '; species=' // trim(species)
            call endrun
         end if
         call handle_ncerr( NF_INQ_VARNDIMS( ncid, vid, ndims ), &
                            'read_emis: Failed to get number of dimensions for ' // &
                            'variable ' // trim(varname) // ', species=' // trim(species) )
         if( ndims < 3 ) then
            cycle
         elseif( ndims > 3 ) then
            write(*,*) 'read_emis: Skipping variable ', trim(varname),', ndims = ',ndims, &
                       ' , species=',trim(species)
            cycle
         end if
!-----------------------------------------------------------------------
!       ... Check order of dimensions. Must be (lon, lat, month).
!-----------------------------------------------------------------------
         allocate( dimid( ndims ), stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'read_emis: dimid allocation error = ',ierr
            call endrun
         end if
         call handle_ncerr( NF_INQ_VARDIMID( ncid, vid, dimid ), &
                            'read_emis: Failed to get dimension IDs for variable ' // &
                            trim(varname)  // ', species=' // trim(species) )
         if( dimid(1) /= dimid_lon .or. dimid(2) /= dimid_lat .or. dimid(3) /= dimid_month ) then
            write(*,*) 'read_emis: Dimensions in wrong order for variable ',trim(varname)
            write(*,*) '...      Expecting (lon, lat, month)'
            call endrun
         end if
         deallocate( dimid, stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'read_emis: Failed to deallocate dimid, ierr = ',ierr
            call endrun
         end if
!-----------------------------------------------------------------------
!       ... Check for long_name of variable
!-----------------------------------------------------------------------
         var_longname = ' '
         ierr = NF_GET_ATT_TEXT( ncid, vid, 'long_name', var_longname )
         if( ierr /= 0 ) then
            var_longname = trim( varname )
         end if
!-----------------------------------------------------------------------
!       ... Read data from this variable
!-----------------------------------------------------------------------
         call handle_ncerr( NF_GET_VARA_DOUBLE( ncid, vid, &
                                                (/ 1, 1, 1/), &                 ! start
                                                (/ nlon, nlat, 12 /), &  ! count
                                                wrk), &
                            'read_emis: Failed to read variable ' // trim( varname ) // &
                            ', species=' // trim(species) )
!-----------------------------------------------------------------------
!       ... Check for scale_factor
!-----------------------------------------------------------------------
         ierr = NF_GET_ATT_DOUBLE( ncid, vid, 'scale_factor', scale_factor )
         if( ierr /= NF_NOERR  ) then
            scale_factor = 1.
         end if

         total = 0.
         do m = 1,12
           do j = 1,plat
             total = total + sf(j) * SUM( wrk( :plon, j, m ) )
           enddo
           emis (:,:,m) = emis(:,:,m) + wrk(:,:,m)
         enddo
!-----------------------------------------------------------------------
!       ... Get global emission from this source
!-----------------------------------------------------------------------
         total = total * factor * mw / 12. ! convert from molec/s to Tg/y, divide by 12 (months)
 
            write(*,'(2a10,1x,a50,1x,f10.3)') trim(species), trim(varname), trim(var_longname), total
      end do  ! (vid)
!-----------------------------------------------------------------------
!       ... Close NetCDF file
!-----------------------------------------------------------------------
      call handle_ncerr( NF_CLOSE( ncid ), &
                         'read_emis: Failed to close NetCDF file, species=' // trim(species) )

!-----------------------------------------------------------------------
!       ... Get total global emission for this species
!-----------------------------------------------------------------------
      total = 0.
      do j = 1,platl
         total = total + sf(j) * SUM( emis( :plonl, j, :12 ) )
      end do
      total = total * factor * mw / 12. ! convert from molec/s to Tg/y, divide by 12 (months)
 
      write(*,'(2a10,1x,a50,1x,f10.3)') trim(species), 'TOTAL', 'TOTAL', total

      end subroutine read_emis

      subroutine srf_emis( lchnk, phir, calday, sflx, oro, &
                           local_angle, polar_night, polar_day, sunon, sunoff, plonl, &
                           flux_ISOP      , flux_ACET      ,  flux_C3H6,              &
                           flux_C2H4      , flux_OC2       ,  flux_C10H16,            &
                           flux_CO2       , flux_N2O       ,  flux_DMS,               &
                           nem2_anthro    , em2data_anthro ,                          &
                           nem2, em2data )
!--------------------------------------------------------
!       ... Form the surface fluxes for this
!           latitude slice
!--------------------------------------------------------

      use mo_const_mozart, only : pi, d2r, lat60, lat70
      use mo_grid,         only : pcnstm1
      use mo_calendar,     only : caldayr

      use mo_chem_mods,    only : adv_mass

#if (defined MOZART2)
      use prescribed_em_d2, only:  &
                      idxCH3COCH3, idxC2H4 ,  idxC2H5OH, idxC2H6,    idxC3H6  , &
                      idxC3H8,     idxCH2O ,  idxCH3CHO, idxCH3OH ,  idxN2O   , &
                      idxCO,       idxH2   ,  idxISOP  , idxNO     , idxC4H10 , &
                      idxCH4 ,                                                  &
                      idxDMS ,     idxSO2  ,  idxCB1   , idxCB2     , idxOC1  , &
                      idxOC2 ,     idxNH3  ,  idxC10H16, idxCH3COCHO, idxCH3COCHO

      use prescribed_em_d2_anthro, only:  &
                      idxCH3COCHO_anthro, idxC2H4_anthro, idxC2H5OH_anthro, idxC2H6_anthro,      &
                      idxC3H6_anthro,     idxC3H8_anthro, idxCH2O_anthro,   idxCH3CHO_anthro,    &
                      idxCH3OH_anthro,    idxN2O_anthro,  idxCO_anthro,     idxNO_anthro,        &
                      idxC4H10_anthro,    idxCH4_anthro,  idxSO2_anthro,    idxCB1_anthro,       &
                      idxCB2_anthro,      idxOC1_anthro,  idxOC2_anthro,    idxNH3_anthro,       &
                      idxC10H16_anthro,   idxH2_anthro,   idxISOP_anthro,   idxCH3COCH3_anthro,  &
                      idxDMS_anthro
#endif
      implicit none

!--------------------------------------------------------
!       ... Dummy arguments
!--------------------------------------------------------
      integer, intent(in) ::   plonl, lchnk
      integer, intent(in) ::   oro(plonl)         ! incoming land surface type
      real, intent(in)    ::   sunon(plonl)      ! sunrise angle in radians
      real, intent(in)    ::   sunoff(plonl)     ! sundown angle in radians
      real, intent(in)    ::   local_angle(plonl)  ! local "time" angle
      real, intent(out)   ::   sflx(plonl,pcnstm1)   ! surface emissions ( kg/m^2/s )
      real, intent(in)    ::   calday            ! time of year in days
      real, intent(in)    ::   phir(plonl)       ! latitude                
      logical, intent(in) ::   polar_night(plonl)! polar night flag
      logical, intent(in) ::   polar_day(plonl)  ! polar day flag

 
      real,intent(in)    :: flux_ISOP(plonl)
      real,intent(in)    :: flux_ACET(plonl)
      real,intent(in)    :: flux_C3H6(plonl)
      real,intent(in)    :: flux_C2H4(plonl)
      real,intent(in)    :: flux_OC2(plonl)
      real,intent(in)    :: flux_C10H16(plonl)

      real,intent(in)    :: flux_CO2(plonl)
      real,intent(in)    :: flux_N2O(plonl)
      real,intent(in)    :: flux_DMS(plonl)
!---
! wtw 2012.10
!
      integer, intent(in) :: nem2
      integer, intent(in) :: nem2_anthro
      real, intent(in)    :: em2data(plonl,nem2)
      real, intent(in)    :: em2data_anthro(plonl,nem2_anthro)
      real, parameter     :: amufac = 1.65979e-23         ! 1.e4* kg / amu
!--------------------------------------------------------
!       ... Local variables
!--------------------------------------------------------
      integer, parameter :: land = 1, ocean = 0
      integer  ::  surf_type
      integer  ::  i, m, last, next
      integer  ::  dates(12) = (/ 116, 214, 316, 415,  516,  615, &
                                  716, 816, 915, 1016, 1115, 1216 /)
      real     ::  factor, dels
      real     ::  dayfrac            ! fration of day in light
      real     ::  iso_off            ! time iso flux turns off
      real     ::  iso_on             ! time iso flux turns on
      real, save ::  days(12)
      logical, save  ::  entered = .false.

!--------------------------------------------------------
!       ... Initialize the monthly day of year times
!--------------------------------------------------------
      if( .not. entered ) then
         do m = 1,12
            days(m) = caldayr( dates(m), 0 )
         end do
         entered = .true.
      end if

!--------------------------------------------------------
!       ... Setup the time interpolation
!--------------------------------------------------------
      if( calday < days(1) ) then
         next = 1
         last = 12
         dels = (365. + calday - days(12)) / (365. + days(1) - days(12))
      else if( calday >= days(12) ) then
         next = 1
         last = 12
         dels = (calday - days(12)) / (365. + days(1) - days(12))
      else
         do m = 11,1,-1
            if( calday >= days(m) ) then
               exit
            end if
         end do
         last = m
         next = m + 1
         dels = (calday - days(m)) / (days(m+1) - days(m))
      end if
      dels = max( min( 1.,dels ),0. )

#ifdef MOZART2

!--------------------------------------------------------
!       ... Set non-zero emissions
!--------------------------------------------------------
      if( no_em ) then
         if( idxNO < 1 .and. idxNO_anthro < 1) then
             sflx(:,no_ndx) = noxembdy(:,lchnk ,last) &
                            + dels * (noxembdy(:,lchnk ,next) - noxembdy(:,lchnk,last))
         else
             if (idxNO > 0 .and. idxNO_anthro > 0) then
                sflx(:,no_ndx) = em2data(:,idxNO) + em2data_anthro(:, idxNO_anthro)        ! zf   kg m-2 s-1
             else if ( idxNO > 0 ) then
!zf            sflx(:,no_ndx) = amufac * adv_mass(no_ndx) * em2data(:,idxNO)               ! wtw
               sflx(:,no_ndx) = em2data(:,idxNO)                                           ! zf   kg m-2 s-1
             else
               sflx(:,no_ndx) = em2data_anthro(:,idxNO_anthro)                             ! zf   kg m-2 s-1
             endif
         endif
      end if

      if( ch4_em ) then
         if( idxCH4 < 1 .and. idxCH4_anthro < 1) then
             sflx(:,ch4_ndx) = ch4embdy(:,lchnk,last) &
                             + dels * (ch4embdy(:,lchnk,next) - ch4embdy(:,lchnk,last))
         else
             if (idxNO > 0 .and. idxNO_anthro > 0) then
                sflx(:,ch4_ndx) = em2data(:,idxCH4) + em2data_anthro(:,idxCH4_anthro)            ! zf   kg m-2 s-1
             else if  ( idxNO > 0 ) then
!zf             sflx(:,ch4_ndx) = amufac * adv_mass(ch4_ndx) * em2data(:,idxCH4)   ! wtw
                sflx(:,ch4_ndx) = em2data(:,idxCH4)                                ! zf   kg m-2 s-1
             else
                sflx(:,ch4_ndx) = em2data_anthro(:,idxCH4_anthro)                                ! zf   kg m-2 s-1
             endif
         endif
      end if

      if( ch2o_em ) then
         if( idxCH2O < 1 .and. idxCH2O_anthro < 1) then
             sflx(:,ch2o_ndx) = ch2oembdy(:,lchnk,last) &
                           + dels * (ch2oembdy(:,lchnk,next) - ch2oembdy(:,lchnk,last))
         else
             if (idxCH2O > 0 .and. idxCH2O_anthro > 0) then
               sflx(:,ch2o_ndx) = em2data(:, idxCH2O) + em2data_anthro(:,idxCH2O_anthro)              ! zf   kg m-2 s-1
             else if (idxCH2O > 0 ) then
!zf            sflx(:,ch2o_ndx) = amufac * adv_mass(ch2o_ndx) * em2data(:,idxCH2O)   ! wtw
               sflx(:,ch2o_ndx) = em2data(:,idxCH2O)                                 ! zf   kg m-2 s-1
             else
               sflx(:,ch2o_ndx) = em2data_anthro(:,idxCH2O_anthro)                                 ! zf   kg m-2 s-1
             endif
         endif
      end if

      if( co_em ) then
         if( idxCO < 1 .and. idxCO_anthro < 1) then
            sflx(:,co_ndx) = coembdy(:,lchnk,last) &
                           + dels * (coembdy(:,lchnk,next) - coembdy(:,lchnk,last))
         else
            if (idxCO > 0 .and. idxCO_anthro > 0 ) then
              sflx(:,co_ndx) = em2data(:,idxCO) + em2data_anthro(:, idxCO_anthro)  ! zf   kg m-2 s-1
            else if (idxCO > 0 ) then
!zf           sflx(:,co_ndx) = amufac * adv_mass(co_ndx) * em2data(:,idxCO)   ! wtw
              sflx(:,co_ndx) = em2data(:,idxCO)                               ! zf   kg m-2 s-1
            else
              sflx(:,co_ndx) = em2data_anthro(:, idxCO_anthro)  ! zf   kg m-2 s-1
            endif
         endif
      end if

      if( c3h6_em ) then
         if( idxC3H6 < 1 .and. idxC3H6_anthro < 1) then
            sflx(:,c3h6_ndx) = c3h6embdy(:,lchnk,last) &
                             + dels * (c3h6embdy(:,lchnk,next) - c3h6embdy(:,lchnk,last))
         else
            if (idxC3H6 > 0 .and. idxC3H6_anthro > 0 ) then
              sflx(:,c3h6_ndx) = em2data(:,idxC3H6) + em2data_anthro(:, idxC3H6_anthro) + flux_C3H6(:) * adv_mass(c3h6_ndx) / 12.
            else if (idxC3H6 > 0 ) then
!zf           sflx(:,c3h6_ndx) = amufac * adv_mass(c3h6_ndx) * em2data(:,idxC3H6)   ! wtw
              sflx(:,c3h6_ndx) = em2data(:,idxC3H6) + flux_C3H6(:) * adv_mass(c3h6_ndx) / 12.                 ! zf   kg m-2 s-1 
                                                     !zf  + flux_C3H6(:) * adv_mass(c3h6_ndx) / 12.
            else 
              sflx(:,c3h6_ndx) = em2data_anthro(:,idxC3H6_anthro)+ flux_C3H6(:) * adv_mass(c3h6_ndx) / 12.       ! zf   kg m-2 s-1 
            endif
         endif
      end if

      if( isop_em ) then
         if( idxISOP < 1 .and. idxISOP_anthro < 1) then
            sflx(:,isop_ndx) = isoembdy(:,lchnk,last) &
                             + dels * (isoembdy(:,lchnk,next) - isoembdy(:,lchnk,last))
         else
            if ( idxISOP > 0 .and. idxISOP_anthro > 0 )then
              sflx(:,isop_ndx) = em2data(:, idxISOP) + em2data_anthro(:,idxISOP_anthro) + flux_ISOP(:)*adv_mass(isop_ndx)/12.
            else if ( idxISOP > 0 )then
!zf           sflx(:,isop_ndx) = amufac * adv_mass(isop_ndx) * em2data(:,idxISOP)   ! wtw
              sflx(:,isop_ndx) = em2data(:,idxISOP) + flux_ISOP(:)*adv_mass(isop_ndx)/12.  !zf kg m-2 s-1
                                                     !zf   + flux_ISOP(:)*adv_mass(isop_ndx)/12.
            else
              sflx(:,isop_ndx) = em2data_anthro(:,idxISOP_anthro) + flux_ISOP(:)*adv_mass(isop_ndx)/12. !zf kg m-2 s-1
            endif
         endif
      end if

      if( c10h16_em ) then
         if( idxC10H16 < 1 .and. idxC10H16_anthro < 1) then
             sflx(:,c10h16_ndx) = terpembdy(:,lchnk,last) &
                                + dels * (terpembdy(:,lchnk,next) - terpembdy(:,lchnk,last))
         else
            if ( idxC10H16 > 0 .and. idxC10H16_anthro > 0 ) then
              sflx(:,c10h16_ndx) = em2data(:,idxC10H16) + em2data_anthro(:,idxC10H16_anthro)   &
                                 + flux_C10H16 * adv_mass(c10h16_ndx) / 12.
            else if ( idxC10H16 > 0 ) then
!zf           sflx(:,c10h16_ndx) = amufac * adv_mass(c10h16_ndx) * em2data(:,idxC10H16)   ! wtw
              sflx(:,c10h16_ndx) = em2data(:,idxC10H16) + flux_C10H16 * adv_mass(c10h16_ndx) / 12.
            else
              sflx(:,c10h16_ndx) = em2data_anthro(:,idxC10H16_anthro) + flux_C10H16 * adv_mass(c10h16_ndx) / 12.
            endif
                                                          !zf   + flux_C10H16 * adv_mass(c10h16_ndx) / 12. 
         endif
      end if

      if( c2h4_em ) then
         if( idxC2H4 < 1 .and. idxC2H4_anthro < 1) then
             sflx(:,c2h4_ndx) = c2h4embdy(:,lchnk,last) &
                              + dels * (c2h4embdy(:,lchnk,next) - c2h4embdy(:,lchnk,last))
         else
            if ( idxC2H4 > 0 .and. idxC2H4_anthro > 0 ) then
               sflx(:,c2h4_ndx) = em2data(:,idxC2H4) + em2data_anthro(:,idxC2H4_anthro) + flux_C2H4(:) * adv_mass(c2h4_ndx) / 12.
            else if ( idxC2H4 > 0 ) then
!zf            sflx(:,c2h4_ndx) = amufac * adv_mass(c2h4_ndx) * em2data(:,idxC2H4)   ! wtw
               sflx(:,c2h4_ndx) = em2data(:,idxC2H4) + flux_C2H4(:) * adv_mass(c2h4_ndx) / 12. 
                                                      !zf   + flux_C2H4(:) * adv_mass(c2h4_ndx) / 12.
            else
               sflx(:,c2h4_ndx) = em2data_anthro(:,idxC2H4_anthro) + flux_C2H4(:) * adv_mass(c2h4_ndx) / 12.
            endif
         endif
      end if

      if( c2h6_em ) then
         if( idxC2H6 < 1 .and. idxC2H6_anthro < 1 ) then
              sflx(:,c2h6_ndx) = c2h6embdy(:,lchnk,last) &
                               + dels * (c2h6embdy(:,lchnk,next) - c2h6embdy(:,lchnk,last))
         else
             if ( idxC2H4 > 0 .and. idxC2H4_anthro > 0 ) then
                sflx(:,c2h6_ndx) = em2data(:,idxC2H6) + em2data_anthro(:,idxC2H6_anthro)               ! zf   kg m-2 s-1
             else if ( idxC2H4 > 0 ) then
!zf             sflx(:,c2h6_ndx) = amufac * adv_mass(c2h6_ndx) * em2data(:,idxC2H6)   ! wtw
                sflx(:,c2h6_ndx) = em2data(:,idxC2H6)                                 ! zf   kg m-2 s-1
             else
                sflx(:,c2h6_ndx) = em2data_anthro(:,idxC2H6_anthro)                                 ! zf   kg m-2 s-1
             endif
         endif
      end if

      if( c3h8_em ) then
         if( idxC3H8 < 1 .and. idxC3H8 < 1 ) then
              sflx(:,c3h8_ndx) = c3h8embdy(:,lchnk,last) &
                               + dels * (c3h8embdy(:,lchnk,next) - c3h8embdy(:,lchnk,last))
         else
             if ( idxC3H8 > 0 .and. idxC3H8_anthro > 0 ) then
                sflx(:,c3h8_ndx) = em2data(:,idxC3H8) + em2data_anthro(:,idxC3H8_anthro)                  ! zf   kg m-2 s-1
             else if ( idxC3H8 > 0 ) then
!zf             sflx(:,c3h8_ndx) = amufac * adv_mass(c3h8_ndx) * em2data(:,idxC3H8)   ! wtw
                sflx(:,c3h8_ndx) = em2data(:,idxC3H8)                                 ! zf   kg m-2 s-1
             else
                sflx(:,c3h8_ndx) = em2data_anthro(:,idxC3H8_anthro)                                 ! zf   kg m-2 s-1
             endif
         endif
      end if

      if( ch3coch3_em ) then
         if( idxCH3COCH3 < 1 .and. idxCH3COCH3_anthro < 1) then
              sflx(:,ch3coch3_ndx) = acetembdy(:,lchnk,last) &
                                   + dels * (acetembdy(:,lchnk,next) - acetembdy(:,lchnk,last))
         else
            if (idxCH3COCH3 > 0 .and. idxCH3COCH3_anthro > 0 ) then
               sflx(:,ch3coch3_ndx) = em2data(:,idxCH3COCH3) + em2data_anthro(:,idxCH3COCH3_anthro)     &
                                    + flux_ACET(:) * adv_mass(ch3coch3_ndx) / 12.                 
            else if (idxCH3COCH3 > 0 ) then
!zf            sflx(:,ch3coch3_ndx) = amufac * adv_mass(ch3coch3_ndx) * em2data(:,idxCH3COCH3)   ! wtw
               sflx(:,ch3coch3_ndx) = em2data(:,idxCH3COCH3) + flux_ACET(:) * adv_mass(ch3coch3_ndx) / 12.
                                                               !zf   + flux_ACET(:) * adv_mass(ch3coch3_ndx) / 12.
            else
               sflx(:,ch3coch3_ndx) = em2data_anthro(:,idxCH3COCH3_anthro) + flux_ACET(:) * adv_mass(ch3coch3_ndx) / 12.
            endif
         endif 
      end if

      if( c4h10_em ) then
         if( idxC4H10 < 1 .and. idxC4H10_anthro < 1) then
              sflx(:,c4h10_ndx) = c4h10embdy(:,lchnk,last) &
                                + dels * (c4h10embdy(:,lchnk,next) - c4h10embdy(:,lchnk,last))
         else
            if ( idxC4H10 > 0 .and. idxC4H10_anthro > 0 ) then
               sflx(:,c4h10_ndx) = em2data(:,idxC4H10) + em2data_anthro(:,idxC4H10_anthro)      ! zf   kg m-2 s-1
            else if ( idxC4H10 > 0 ) then
!zf            sflx(:,c4h10_ndx) = amufac * adv_mass(c4h10_ndx) * em2data(:,idxC4H10)   ! wtw
               sflx(:,c4h10_ndx) = em2data(:,idxC4H10)                                  ! zf   kg m-2 s-1
            else
               sflx(:,c4h10_ndx) = em2data_anthro(:,idxC4H10_anthro)                    ! zf   kg m-2 s-1
            endif
         endif 
      end if

      if( n2o_em ) then
         if( idxN2O < 1 .and. idxN2O_anthro < 1) then
              sflx(:,n2o_ndx) = n2oembdy(:,lchnk,last) &
                              + dels * (n2oembdy(:,lchnk,next) - n2oembdy(:,lchnk,last))
         else
             if ( idxN2O > 0 .and. idxN2O_anthro > 0 ) then
                sflx(:,n2o_ndx) = em2data(:,idxN2O) + em2data_anthro(:,idxN2O_anthro)           ! zf   kg m-2 s-1
             else if ( idxN2O > 0 ) then
!zf             sflx(:,n2o_ndx) = amufac * adv_mass(n2o_ndx) * em2data(:,idxN2O)   ! wtw
                sflx(:,n2o_ndx) = em2data(:,idxN2O)                                ! zf   kg m-2 s-1
             else
                sflx(:,n2o_ndx) = em2data_anthro(:,idxN2O_anthro)                                ! zf   kg m-2 s-1
             endif
#ifdef HAMOCC
     ! flux_N2O from HAMOCC is not used.  The prescribed data is used in prescribed_em_d2.F90.   
     !        sflx(:,n2o_ndx) = sflx(:,n2o_ndx) + flux_N2O(:)
#endif
         endif
      end if

      if( h2_em ) then
         if( idxH2 < 1 .and. idxH2_anthro < 1) then
              sflx(:,h2_ndx) = h2embdy(:,lchnk,last) &
                             + dels * (h2embdy(:,lchnk,next) - h2embdy(:,lchnk,last))
         else
            if ( idxH2 > 0 .and. idxH2_anthro > 0 ) then
               sflx(:,h2_ndx) = em2data(:,idxH2) + em2data_anthro(:,idxH2_anthro)          ! zf   kg m-2 s-1
            else if ( idxH2 > 0 ) then
!zf            sflx(:,h2_ndx) = amufac * adv_mass(h2_ndx) * em2data(:,idxH2)   ! wtw
               sflx(:,h2_ndx) = em2data(:,idxH2)                               ! zf   kg m-2 s-1
            else
               sflx(:,h2_ndx) = em2data_anthro(:,idxH2_anthro)                               ! zf   kg m-2 s-1
            endif
         endif
      end if

      if( ch3oh_em ) then
         if( idxCH3OH < 1 .and. idxCH3OH_anthro < 1 ) then
              sflx(:,ch3oh_ndx) = ch3ohembdy(:,lchnk,last) &
                                + dels * (ch3ohembdy(:,lchnk,next) - ch3ohembdy(:,lchnk,last))
         else
            if ( idxCH3OH > 0 .and. idxCH3OH_anthro > 0 ) then 
               sflx(:,ch3oh_ndx) = em2data(:,idxCH3OH) + em2data_anthro(:,idxCH3OH_anthro)      ! zf   kg m-2 s-1
            else if ( idxCH3OH > 0 ) then 
!zf            sflx(:,ch3oh_ndx) = amufac * adv_mass(ch3oh_ndx) * em2data(:,idxCH3OH)   ! wtw
               sflx(:,ch3oh_ndx) = em2data(:,idxCH3OH)                                  ! zf   kg m-2 s-1
            else
               sflx(:,ch3oh_ndx) = em2data_anthro(:,idxCH3OH_anthro)                                  ! zf   kg m-2 s-1
            endif
         endif
      end if

!--------
      if( idxC2H5OH > 0 .and. idxC2H5OH_anthro > 0 ) then
         sflx(:,c2h5oh_ndx) = em2data(:,idxC2H5OH) + em2data_anthro(:,idxC2H5OH_anthro)         ! zf   kg m-2 s-1
      else if( idxC2H5OH > 0 ) then
!zf      sflx(:,c2h5oh_ndx) = amufac * adv_mass(c2h5oh_ndx) * em2data(:,idxC2H5OH)   ! wtw
         sflx(:,c2h5oh_ndx) = em2data(:,idxC2H5OH)                                   ! zf   kg m-2 s-1
      else
         sflx(:,c2h5oh_ndx) = em2data_anthro(:,idxC2H5OH_anthro)                                   ! zf   kg m-2 s-1
      end if

      if( idxCH3CHO > 0 .and. idxCH3CHO_anthro > 0 ) then
         sflx(:,ch3cho_ndx) = em2data(:,idxCH3CHO) + em2data_anthro(:,idxCH3CHO_anthro)         ! zf   kg m-2 s-1
      else if( idxCH3CHO > 0 ) then
!zf      sflx(:,ch3cho_ndx) = amufac * adv_mass(ch3cho_ndx) * em2data(:,idxCH3CHO)   ! wtw
         sflx(:,ch3cho_ndx) = em2data(:,idxCH3CHO)                                   ! zf   kg m-2 s-1
      else
         sflx(:,ch3cho_ndx) = em2data_anthro(:,idxCH3CHO_anthro)                                   ! zf   kg m-2 s-1
      end if

      if( idxDMS > 0 .and. idxDMS_anthro > 0) then
         sflx(:,dms_ndx) = em2data(:,idxDMS) + em2data_anthro(:, idxDMS_anthro)
      else if ( idxDMS > 0 ) then
!zf      sflx(:,dms_ndx) = amufac * adv_mass(dms_ndx) * em2data(:,idxDMS)   ! wtw
         sflx(:,dms_ndx) = em2data(:,idxDMS)                                ! zf   kg m-2 s-1
      else
         sflx(:,dms_ndx) = em2data_anthro(:,idxDMS_anthro)                  ! zf   kg m-2 s-1
      end if
#ifdef HAMOCC
     ! flux_DMS from HAMOCC is not used.  The prescribed data is used in prescribed_em_d2.F90.
     !        sflx(:,dms_ndx) = sflx(:,dms_ndx) + flux_DMS(:)
#endif

      if( idxSO2 > 0 .and. idxSO2_anthro > 0 ) then
         sflx(:,so2_ndx) = em2data(:,idxSO2) + em2data_anthro(:,idxSO2_anthro)    ! zf   kg m-2 s-1
      else if( idxSO2 > 0 ) then
!zf      sflx(:,so2_ndx) = amufac * adv_mass(so2_ndx) * em2data(:,idxSO2)   ! wtw
         sflx(:,so2_ndx) = em2data(:,idxSO2)                                ! zf   kg m-2 s-1
      else
         sflx(:,so2_ndx) = em2data_anthro(:,idxSO2_anthro)                  ! zf   kg m-2 s-1
      end if

      if( idxCB1 > 0 .and. idxCB1_anthro > 0 ) then
         sflx(:,cb1_ndx) = em2data(:,idxCB1) + em2data_anthro(:,idxCB1_anthro)                                ! zf   kg m-2 s-1
      else if( idxCB1 > 0 ) then
!zf      sflx(:,cb1_ndx) = amufac * adv_mass(cb1_ndx) * em2data(:,idxCB1)   ! wtw
         sflx(:,cb1_ndx) = em2data(:,idxCB1)                                ! zf   kg m-2 s-1
      else
         sflx(:,cb1_ndx) = em2data_anthro(:,idxCB1_anthro)                                ! zf   kg m-2 s-1
      end if

      if( idxCB2 > 0 .and. idxCB2_anthro > 0 ) then
         sflx(:,cb2_ndx) = em2data(:,idxCB2) + em2data_anthro(:,idxCB2_anthro)                                ! zf   kg m-2 s-1
      else if( idxCB2 > 0 ) then
!zf      sflx(:,cb2_ndx) = amufac * adv_mass(cb2_ndx) * em2data(:,idxCB2)   ! wtw
         sflx(:,cb2_ndx) = em2data(:,idxCB2)                                ! zf   kg m-2 s-1
      else
         sflx(:,cb2_ndx) = em2data_anthro(:,idxCB2_anthro)                                ! zf   kg m-2 s-1
      end if

      if( idxOC1 > 0 .and. idxOC1_anthro > 0) then
         sflx(:,oc1_ndx) = em2data(:,idxOC1) + em2data_anthro(:,idxOC1_anthro)                ! zf   kg m-2 s-1
      else if( idxOC1 > 0 ) then
!zf      sflx(:,oc1_ndx) = amufac * adv_mass(oc1_ndx) * em2data(:,idxOC1)   ! wtw
         sflx(:,oc1_ndx) = em2data(:,idxOC1)                    ! zf   kg m-2 s-1
      else
         sflx(:,oc1_ndx) = em2data_anthro(:,idxOC1_anthro)               ! zf   kg m-2 s-1
      end if   

      if( idxOC2 > 0 .and. idxOC2_anthro > 0 ) then
         sflx(:,oc2_ndx) = em2data(:,idxOC2) + em2data_anthro(:,idxOC2_anthro) + flux_OC2(:)                               ! zf   kg m-2 s-1
      else if( idxOC2 > 0 ) then
!zf      sflx(:,oc2_ndx) = amufac * adv_mass(oc2_ndx) * em2data(:,idxOC2)   ! wtw
         sflx(:,oc2_ndx) = em2data(:,idxOC2) + flux_OC2(:)                             ! zf   kg m-2 s-1
      else
         sflx(:,oc2_ndx) = em2data_anthro(:,idxOC2_anthro) + flux_OC2(:)                               ! zf   kg m-2 s-1
      end if

      if( idxNH3 > 0 .and. idxNH3_anthro > 0) then
         sflx(:,nh3_ndx) = em2data(:,idxNH3) + em2data_anthro(:,idxNH3_anthro)                                ! zf   kg m-2 s-1
      else if( idxNH3 > 0 ) then
!zf      sflx(:,nh3_ndx) = amufac * adv_mass(nh3_ndx) * em2data(:,idxNH3)   ! wtw
         sflx(:,nh3_ndx) = em2data(:,idxNH3)                                ! zf   kg m-2 s-1
      else
         sflx(:,nh3_ndx) = em2data_anthro(:,idxNH3_anthro)                                ! zf   kg m-2 s-1
      end if
#endif

      if( idxCH3COCHO > 0 .and. idxCH3COCHO_anthro > 0) then
         sflx(:,ch3cocho_ndx) = em2data(:,idxCH3COCHO) + em2data_anthro(:, idxCH3COCHO_anthro)
      else if ( idxCH3COCHO > 0 ) then
         sflx(:,ch3cocho_ndx) = em2data(:,idxCH3COCHO)                                ! zf   kg m-2 s-1
      else
         sflx(:,ch3cocho_ndx) = em2data_anthro(:,idxCH3COCHO_anthro)                  ! zf   kg m-2 s-1
      end if

!--------------------------------------------------------
!       ... Adjust alpha-pinene for diurnal variation
!--------------------------------------------------------
      if( c10h16_em ) then
       do i = 1, plonl
         if( .not. polar_night(i) .and. .not. polar_day(i) ) then
            dayfrac = sunoff(i) / pi
            sflx(i,c10h16_ndx) = sflx(i,c10h16_ndx) / (.7 + .3*dayfrac)
            if( local_angle(i) >= sunoff(i) .and. local_angle (i) <= sunon(i) ) then
               sflx(i,c10h16_ndx) = sflx(i,c10h16_ndx) * .7
            end if
         end if
       enddo
      end if

!--------------------------------------------------------
!       ... Adjust isoprene for diurnal variation
!--------------------------------------------------------
      if( isop_em ) then
        do i = 1,plonl
         if( .not. polar_night(i) ) then
            if( polar_day(i) ) then
               iso_off = .8 * pi
               iso_on  = 1.2 * pi
            else
               iso_off = .8 * sunoff(i)
               iso_on  = 2. * pi - iso_off
            end if
               if( local_angle(i) >= iso_off .and. local_angle(i) <= iso_on ) then
                  sflx(i,isop_ndx) = 0.
               else
                  factor = local_angle(i) - iso_on
                  if( factor <= 0. ) then
                     factor = factor + 2.*pi
                  end if
                  factor = factor / (2.*iso_off + 1.e-6)
                  sflx(i,isop_ndx) = sflx(i,isop_ndx) * 2. / iso_off * pi * (sin(pi*factor))**2
               end if
         else
            sflx(i,isop_ndx) = 0.
         end if
        end do
      end if
!--------------------------------------------------------
!       ... Set Radon flux
!       ... If |lat| > 70N or < 60S then zero the flux
!--------------------------------------------------------
      if( Rn_em ) then
        do i = 1,plonl
         if( phir(i) >= lat70 .or. phir(i) <= -lat60 ) then
            sflx(i,Rn_ndx) = baseflux
         else
               surf_type = oro(i)
               if( surf_type == land ) then
                  if( phir(i) >= lat60 ) then
                     sflx(i,Rn_ndx) = .5*landflux
                  else
                     sflx(i,Rn_ndx) = landflux
                  end if
               else if( surf_type == ocean ) then
                  sflx(i,Rn_ndx)  = baseflux
               else
                  sflx(i,Rn_ndx) = baseflux
               end if
         end if
        end do
      end if

      end subroutine srf_emis

      end module mo_srf_emis

