
      module mo_srfalb

      contains

      subroutine srfalb( albs, calday, tsurf, plonl, lchnk, ncol )
!-----------------------------------------------------------------------
! 	... compute surface albedo
!-----------------------------------------------------------------------

      use mo_surf,         only : desertbdy, oceanbdy, seaicebdy
      use mo_const_mozart, only : phi, d2r
      use mo_calendar,     only : caldayr
      use phys_grid,       only : get_rlat_all_p     !add by zf 2008.07.07

      implicit none

!-----------------------------------------------------------------------
! 	... dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) ::  lchnk, ncol
      integer, intent(in) ::  plonl        ! lon tile dim
      real, intent(in)    ::  calday       ! time of year
      real, intent(in)    ::  tsurf(plonl)  ! surface temperature
      real, intent(out)   ::  albs(plonl)  ! srf alb

!-----------------------------------------------------------------------
! 	... local variables
!-----------------------------------------------------------------------
      integer :: i, m, last, next            ! longitude index
      integer  ::  dates(12) = (/ 116, 214, 316, 415,  516,  615, 716, 816, 915, 1016, 1115, 1216 /)
      real :: dels
      real :: polar_limit
      real :: days(12)
      real, dimension(plonl) :: sea_ice, alb0, open_ocean, temp_fac 
      real :: clat(plonl)                   ! current latitudes(radians)

!--------------------------------------------------------
!	... initialize the monthly day of year times
!--------------------------------------------------------
	 do m = 1,12
	    days(m) = caldayr( dates(m), 0 )
	 end do
	 polar_limit = 59. * d2r

         albs = 0.0
!--------------------------------------------------------
!	... setup the time interpolation
!--------------------------------------------------------
      if( calday < days(1) ) then
	 next = 1
	 last = 12
	 dels = (365. + calday - days(12)) / (365. + days(1) - days(12))
      else if( calday >= days(12) ) then
	 next = 1
	 last = 12
	 dels = (calday - days(12)) / (365. + days(1) - days(12))
      else
         do m = 11,1,-1
	    if( calday >= days(m) ) then
	       exit
	    end if
         end do
	 last = m
	 next = m + 1
	 dels = (calday - days(m)) / (days(m+1) - days(m))
      end if
      dels = max( min( 1.,dels ),0. )

!--------------------------------------------------------
!	... interpolate the seaice
!--------------------------------------------------------
      sea_ice(:) = seaicebdy(:,lchnk,last) &
                 + dels * (seaicebdy(:,lchnk,next) - seaicebdy(:,lchnk,last))
      open_ocean(:) = max( 0.,oceanbdy(:,lchnk) - sea_ice(:) )
      temp_fac(:) = min( 1., max( 0., (tsurf(:) - 268.) / 5. ) )

      call get_rlat_all_p(lchnk, ncol, clat)

      do i = 1, plonl
      if( abs( clat(i) ) >= polar_limit ) then
         albs(i) = .05 + .45 * desertbdy(i,lchnk) + .15 * sea_ice(i)
      else
         albs(i) = .05 + .1 * desertbdy(i,lchnk) + .15 * sea_ice(i)
      end if
      alb0(i) = .5  - .45 * open_ocean(i)
      albs(i) = albs(i) + (alb0(i) - albs(i)) * (1. - temp_fac(i))
      end do

      end subroutine srfalb

      end module mo_srfalb
