
      module mo_usrrxt

      private
      public :: usrrxt_inti, usrrxt

      save

      integer :: usr1_ndx, usr2_ndx, usr3_ndx, usr5_ndx, usr6_ndx, usr7_ndx, &
                 usr8_ndx, usr9_ndx, usr11_ndx, usr12_ndx, usr14_ndx, usr15_ndx, &
                 usr16_ndx, usr17_ndx, usr21_ndx, usr22_ndx

      contains

      subroutine usrrxt_inti
!-----------------------------------------------------------------
!        ... Intialize the user reaction constants module
!-----------------------------------------------------------------

      use mo_chem_utls, only : get_rxt_ndx
      use pmgrid,       only : masterproc    !add by zf 2008.06.20

      implicit none

      usr1_ndx = get_rxt_ndx( 'usr1' )
      usr2_ndx = get_rxt_ndx( 'usr2' )
      usr3_ndx = get_rxt_ndx( 'usr3' )
      usr5_ndx = get_rxt_ndx( 'usr5' )
      usr6_ndx = get_rxt_ndx( 'usr6' )
      usr7_ndx = get_rxt_ndx( 'usr7' )
      usr8_ndx = get_rxt_ndx( 'usr8' )
      usr9_ndx = get_rxt_ndx( 'usr9' )
      usr11_ndx = get_rxt_ndx( 'usr11' )
      usr12_ndx = get_rxt_ndx( 'usr12' )
      usr14_ndx = get_rxt_ndx( 'usr14' )
      usr15_ndx = get_rxt_ndx( 'usr15' )
      usr16_ndx = get_rxt_ndx( 'usr16' )
      usr17_ndx = get_rxt_ndx( 'usr17' )
      usr21_ndx = get_rxt_ndx( 'usr21' )
      usr22_ndx = get_rxt_ndx( 'usr22' )

      if (masterproc) then     !add by zf 2008.06.20
      write(*,*) ' '
      write(*,*) 'usrrxt_inti: diagnostics '
      write(*,'(10i5)') usr1_ndx, usr2_ndx, usr3_ndx, usr5_ndx, usr6_ndx, usr7_ndx, &
                 usr8_ndx, usr9_ndx, usr11_ndx, usr12_ndx, usr14_ndx, usr15_ndx, &
                 usr16_ndx, usr17_ndx, usr21_ndx, usr22_ndx
      endif

      end subroutine usrrxt_inti

      subroutine usrrxt( rxt, temp, invariants, h2ovmr,  ps, &
                         pmid, m, sulfate, plonl, ncol )
!-----------------------------------------------------------------
!        ... set the user specified reaction rates
!-----------------------------------------------------------------

      use ppgrid,          only : pver
      use wv_saturation,   only : aqsat

      use mo_chem_mods,    only : nfs, rxntot, indexh2o
      use mo_const_mozart, only : pi
      use mo_rxt_id
      use mo_setrxt,only : vexp

      implicit none

!-----------------------------------------------------------------
!        ... dummy arguments
!-----------------------------------------------------------------
      integer, intent(in) :: plonl
      integer, intent(in) :: ncol                   ! number of atmospheric columns
      real, intent(in) ::   temp(plonl,pver), &         ! temperature
                            m(plonl,pver), &            ! total atm density
                            sulfate(plonl,pver), &      ! sulfate aerosol vmr
                            h2ovmr(plonl,pver), &       ! water vapor vmr
                            pmid(plonl,pver), &         ! midpoint pressure in pa
                            ps(plonl), &                ! surface pressure in pa
                            invariants(plonl,pver,nfs)  ! invariants density
      real, intent(inout) ::  rxt(plonl,pver,rxntot)    ! gas phase rates

!-----------------------------------------------------------------
!        ... local variables
!-----------------------------------------------------------------
      real, parameter :: boltz = 1.38044e-16            ! erg / k
      real, parameter :: avo   = 6.023e23               ! molecules/mole
!-----------------------------------------------------------------
!       ... density of sulfate aerosol
!-----------------------------------------------------------------
      real, parameter :: gam1 = 0.04                    ! n2o5+sul ->2hno3
      real, parameter :: wso4 = 98.
      real, parameter :: den  = 1.15                    ! each molecule of so4(aer) density g/cm3
!-------------------------------------------------
!       ... volume of sulfate particles
!           assuming mean rm
!           continient 0.05um  0.07um  0.09um
!           ocean      0.09um  0.25um  0.37um
!                      0.16um                  blake jgr,7195, 1995
!-------------------------------------------------
      real, parameter :: rm1  = 0.16*1.e-4                   ! mean radii in cm
      real, parameter :: fare = 4.*3.14*rm1*rm1              ! each mean particle(r=0.1u) area   cm2/cm3

      integer  ::  i, k
      real     ::  amas
      real, dimension(plonl) :: &
                   tp, &                    ! 300/t
                   tinv, &                  ! 1/t
                   ko, &
                   kinf, &
                   fc, &
                   relhum, &                ! relative humidity
                   satq, &                  ! saturation specific humidity
                   satv, &                  ! saturation vapor pressure
                   xr, &                    ! factor to increase particle radii depending on rel hum
                   sur, &                   ! sulfate particle surface area (cm^2/cm^3)
                   exp_fac                  ! vector exponential

      amas = 4.*pi*rm1**3*den/3.            ! each mean particle(r=0.1u) mass (g)
!-----------------------------------------------------------------
!       ... o + o2 + m --> o3 + m
!-----------------------------------------------------------------
      do k = 1,pver
         tinv(:)           = 1. / temp(:,k)
         tp(:)             = 300. * tinv(:)
         if( usr1_ndx > 0 ) then
            rxt(:,k,usr1_ndx) = 6.e-34 * tp(:)**2.4
         end if
!-----------------------------------------------------------------
!       ... n2o5 + m --> no2 + no3 + m
!-----------------------------------------------------------------
         if( usr3_ndx > 0 ) then
            if( usr2_ndx > 0 ) then
               call vexp( exp_fac, -10991.*tinv, plonl )
               rxt(:,k,usr3_ndx) = rxt(:,k,usr2_ndx) * 3.333e26 * exp_fac(:)
            else
               rxt(:,k,usr3_ndx) = 0.
            end if
         end if

!-----------------------------------------------------------------
!       set rates for:
!       ... hno3 + oh --> no3 + h2o
!           ho2no2 + m --> ho2 + no2 + m
!           co + oh --> co2 + ho2
!-----------------------------------------------------------------
         if( usr5_ndx > 0 ) then
            call vexp( exp_fac, 1335.*tinv, plonl )
            ko(:) = m(:,k) * 6.5e-34 * exp_fac(:)
            call vexp( exp_fac, 2199.*tinv, plonl )
            ko(:) = ko(:) / (1. + ko(:)/(2.7e-17*exp_fac(:)))
            call vexp( exp_fac, 460.*tinv, plonl )
            rxt(:,k,usr5_ndx) = ko(:) + 2.4e-14*exp_fac(:)
         end if
         if( usr7_ndx > 0 ) then
            if( usr6_ndx > 0 ) then
               call vexp( exp_fac, -10900.*tinv, plonl )
               rxt(:,k,usr7_ndx) = rxt(:,k,usr6_ndx) * exp_fac(:) / 2.1e-27
            else
               rxt(:,k,usr7_ndx) = 0.
            end if
         end if
         if( usr8_ndx > 0 ) then
            rxt(:,k,usr8_ndx) = 1.5e-13 * (1. + 6.e-7*boltz*m(:,k)*temp(:,k))
         end if

!-----------------------------------------------------------------
!       ... ho2 + ho2 --> h2o2
!       note: this rate involves the water vapor number density
!-----------------------------------------------------------------
         if( usr9_ndx > 0 ) then
            if( indexh2o > 0 ) then
               call vexp( exp_fac, 600.*tinv, plonl )
               ko(:)   = 2.3e-13 * exp_fac(:)
               call vexp( exp_fac, 1000.*tinv, plonl )
               kinf(:) = 1.7e-33 * m(:,k) * exp_fac(:)
               call vexp( exp_fac, 2200.*tinv, plonl )
               fc(:)   = 1. + 1.4e-21 * invariants(:,k,indexh2o) * exp_fac(:)
               rxt(:,k,usr9_ndx) = (ko(:) + kinf(:)) * fc(:)
            else
               rxt(:,k,usr9_ndx) = 0.
            end if
         end if

!-----------------------------------------------------------------
!       ... mco3 + no2 -> mpan
!-----------------------------------------------------------------
         if( usr14_ndx > 0 ) then
            rxt(:,k,usr14_ndx) = 1.1e-11 * tp(:) / m(:,k)
         end if

!-----------------------------------------------------------------
!       ... pan + m --> ch3co3 + no2 + m
!-----------------------------------------------------------------
         call vexp( exp_fac, -14000.*tinv, plonl )
         if( usr12_ndx > 0 ) then
            if( usr11_ndx > 0 ) then
               rxt(:,k,usr12_ndx) = rxt(:,k,usr11_ndx) * 1.111e28 * exp_fac(:)
            else
               rxt(:,k,usr12_ndx) = 0.
            end if
         end if

!-----------------------------------------------------------------
!       ... mpan + m --> mco3 + no2 + m
!-----------------------------------------------------------------
         if( usr15_ndx > 0 ) then
            if( usr14_ndx > 0 ) then
               rxt(:,k,usr15_ndx) = rxt(:,k,usr14_ndx) * 1.111e28 * exp_fac(:)
            else
               rxt(:,k,usr15_ndx) = 0.
            end if
         end if

!-----------------------------------------------------------------
!       ... xooh + oh -> h2o + oh
!-----------------------------------------------------------------
         if( usr21_ndx > 0 ) then
            call vexp( exp_fac, 253.*tinv, plonl )
            rxt(:,k,usr21_ndx) = temp(:,k)**2 * 7.69e-17 * exp_fac(:)
         end if

!-----------------------------------------------------------------
!       ... ch3coch3 + oh -> ro2 + h2o
!-----------------------------------------------------------------
         if( usr22_ndx > 0 ) then
            call vexp( exp_fac, -1320.*tinv, plonl )
            call vexp( xr, 423.*tinv, plonl )
            rxt(:,k,usr22_ndx) = 8.8e-12 * exp_fac(:) + 1.7e-14 * xr(:)
         end if
         if( usr16_ndx > 0 .or. usr17_ndx > 0 ) then
!-----------------------------------------------------------------
!       ... n2o5 --> 2*hno3
!           no3 --> hno3
!-----------------------------------------------------------------
!       ... first compute the relative humidity
!-----------------------------------------------------------------
            call aqsat( temp(1,k), pmid(1,k), satv, satq, plonl, &
                        ncol, 1, 1, 1 )
            relhum(:) = .622 * h2ovmr(:,k) / satq(:)
            relhum(:) = max( 0.,min( 1.,relhum(:) ) )
!-------------------------------------------------------------------------
!       ... estimate humidity effect on aerosols (from shettle and fenn, 1979)
!           xr is a factor of the increase aerosol radii with hum (hum=0., factor=1)
!-------------------------------------------------------------------------
            xr(:)     = .999151 + relhum(:)*(1.90445 + relhum(:)*(-6.35204 + relhum(:)*5.32061))
!-------------------------------------------------------------------------
!       ... estimate sulfate particles surface area (cm2/cm3) in each grid
!-------------------------------------------------------------------------
            sur(:)    = sulfate(:,k)*m(:,k)/avo*wso4 &              ! xform mixing ratio to g/cm3
                        / amas &                                    ! xform g/cm3 to num particels/cm3
                        * fare &                                    ! xform num particels/cm3 to cm2/cm3
                        * xr(:)*xr(:)                               ! humidity factor
!-----------------------------------------------------------------
!       ... compute the "aerosol" reaction rates
!-----------------------------------------------------------------
!             k = gam * a * velo/4
!
!       where velo = sqrt[ 8*bk*t/pi/(w/av) ]
!             bk = 1.381e-16
!             av = 6.02e23
!             w  = 108 (n2o5)  ho2(33)  ch2o (30)  nh3(15)
!
!       so that velo = 1.40e3*sqrt(t)  (n2o5)   gama=0.1
!       so that velo = 2.53e3*sqrt(t)  (ho2)    gama>0.2
!       so that velo = 2.65e3*sqrt(t)  (ch2o)   gama>0.022
!       so that velo = 3.75e3*sqrt(t)  (nh3)    gama=0.4
!--------------------------------------------------------
            xr(:) = .25 * gam1 * sur(:) * 1.40e3 * sqrt( temp(:,k) )
            if( usr16_ndx > 0 ) then
               rxt(:,k,usr16_ndx) = xr(:)
            end if
            if( usr17_ndx > 0 ) then
               rxt(:,k,usr17_ndx) = xr(:)
            end if
         end if
      end do

      end subroutine usrrxt

      end module mo_usrrxt
