#include <misc.h> 
#include <params.h>
#if ( defined SCAM )
#include <max.h>
#endif

module prescribed_ch4

   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid, only: pcols, pver, pverp, begchunk, endchunk
   use pmgrid, only: plon, plat, plev, plevp, masterproc
   use phys_grid, only: get_ncols_p, scatter_field_to_chunk
   use time_manager, only: get_curr_calday, get_curr_date
   use infnan, only: inf, bigint
   use abortutils, only: endrun
!---------------------------zf 2010.11.08------------------------------
   use rgrid,          only: nlon
   use commap,         only: latdeg, londeg
   use pmgrid,         only: plond
!-----------------------------------------------------------------------
   implicit none

   integer, public :: rampyear_prescribed_ch4       = bigint ! this is the only acceptable value

  private
  save

  real(r8) :: cdaym = inf          ! calendar day of prv month
  real(r8) :: cdayp = inf          ! calendar day of next month

  public ch4ini
  public ch4int
  public getch4

!--------------------zf 2010.11.10------------------
   integer,private   :: lonch4siz     ! size of longitude dimension on ch4 dataset
   integer,private   :: latch4siz     ! size of latitude dimension on ch4 dataset
   integer,private   :: levch4siz     ! size of latitude dimension on ch4 dataset

   real(r8),private, allocatable :: ch4lon(:)
   real(r8),private, allocatable :: ch4lat(:)
!---------------------------------------------------

   real(r8),private,allocatable,dimension(:,:,:,:) ::  ch4i
                      ! input ch4 mixing ratios (pcols,pver,begchunk:endchunk,2)
   real(r8),private,allocatable,dimension(:,:,:)   ::  ch4
                      ! time interpolated ch4 bio mixing ratios (pcols,pver,begchunk:endchunk)

   real(r8), private :: cdaych4m        ! calendar day for prv. month ch4 values read in
   real(r8), private :: cdaych4p        ! calendar day for nxt. month ch4 values read in

   real(r8),private :: bndry_m          ! calendar day for the lower time-boundary of inputdata
   real(r8),private :: bndry_p          ! calendar day for the upper time-boundary of inputdata
   logical, private :: outofbndry       ! whether current model time is out of the time-boundary

   integer,private,parameter :: nfile = 17    ! number of input data files
   integer,private,parameter :: yr_ch4(nfile) = (/ &
                      1845, 1855, 1865, 1875, 1885, 1895, 1905, &
                      1915, 1925, 1935, 1945, 1955, &
                      1965, 1975, 1985, 1995, 2005  &
                                                  /)

   integer :: nmsul = 1             ! index to prv month in array. init to 1 and toggle between 1 and 2
   integer :: npsul = 2             ! index to nxt month in array. init to 2 and toggle between 1 and 2

   character*256,private :: file_ch4(nfile)

   integer, private,parameter :: nsample = 12   ! number of samples in each input data file
   real(r8),private           :: sample_ch4(nsample) =      (/&
                1296000.0,  3888000.0,  6393600.0,  9072000.0, &
               11664000.0, 14342400.0, 16934400.0, 19612800.0, &  
               22291200.0, 24883200.0, 27561600.0, 30153600.0 /)
! Integer data
!
   integer, private :: sp_m, sp_p
   integer, private :: yrm,  yrp
   integer, private :: yr_am, yr_bm, yr_ap, yr_bp, indxbp
!
   integer, private :: ncid_ch4_ap, ncid_ch4_bp  ! ch4 dataset id

   character*80,  private ::  ch4path
   character*80,  private ::  ch4data   
   character*256, private ::  ch4datafile_bp ! full pathname for ch4 dataset

!------------------zf 2010.11.15---------------------------------------------------
   interface

   subroutine bilin (arrin, xin, yin, nlondin, nlonin, &
                     nlevdin, nlev, nlatin, arrout, xout, &
                     yout, nlondout, nlonout, nlevdout, nlatout, fillvalue)

   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun

   integer, intent(in) :: nlondin                        ! longitude dimension of input grid
   integer, intent(in) :: nlonin                         ! number of real longitudes (input)
   integer, intent(in) :: nlevdin                        ! vertical dimension of input grid
   integer, intent(in) :: nlev                           ! number of vertical levels
   integer, intent(in) :: nlatin                         ! number of input latitudes
   integer, intent(in) :: nlatout                        ! number of output latitudes
   integer, intent(in) :: nlondout                       ! longitude dimension of output grid
   integer, intent(in) :: nlonout(nlatout)               ! number of output longitudes per lat
   integer, intent(in) :: nlevdout                       ! vertical dimension of output grid

   real(r8), intent(in) :: arrin(nlondin,nlevdin,nlatin) ! input array of values to interpolate
   real(r8), intent(in) :: xin(nlondin)                  ! input x mesh
   real(r8), intent(in) :: yin(nlatin)                   ! input y mesh
   real(r8), intent(in) :: xout(nlondout,nlatout)        ! output x mesh
   real(r8), intent(in) :: yout(nlatout)                 ! output y mesh
   real(r8), optional, intent(in) :: fillvalue           ! undef. value

   real(r8), intent(out) :: arrout(nlondout,nlevdout,nlatout) ! interpolated array

   end subroutine bilin

   end interface
!----------------------------------------------------------------------------------

contains

subroutine ch4ini
!----------------------------------------------------------------------- 
! 
! Purpose: Do initial read of time-variant ch4 dataset, containing
!          ch4 mixing ratios as a function of time.  It is currently
!          required that the ch4 dataset have the *SAME* horizontal
!          and vertical resolution as the model. Therefore, ONLY a time
!          interpolation of the dataset is currently performed.
! 
!-----------------------------------------------------------------------
   use ioFileMod,      only: getfil
   use error_messages, only: alloc_err, handle_ncerr
   use time_manager,   only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use dycore,         only: get_resolution
   use mo_tracinp,     only : bndic

   implicit none

   integer :: ncid_ch4_am=-1, ncid_ch4_bm  ! ch4 dataset id
   integer :: ch4_id_ap,   ch4_id_bp
   integer :: ch4_id_am,   ch4_id_bm 
 
   integer :: timesiz     ! size of time dimension on sulfate dataset

   character*256 ::  ch4datafile_am ! full pathname for sulfate dataset
   character*256 ::  ch4datafile_bm ! full pathname for sulfate dataset
   character*256 ::  ch4datafile_ap ! full pathname for sulfate dataset

   character*80, parameter ::  p1t42   ='CH4_'                  ! part of the name of sulfate data file
   character*80, parameter ::  p2L26='_clim_c090605.nc'   ! part of the name of CH4 data file
   character*80, parameter ::  p2L46='_clim_c20150527.nc'   ! part of the name of CH4 data file

!-----------------------------------------------------------------------
   include 'netcdf.inc'
!
! Local workspace
!
   character(len=256) :: locfn_am, locfn_bm, locfn_ap, locfn_bp     ! local filenames

   integer londimid              ! netcdf id for longitude dimension
   integer latdimid              ! netcdf id for latitude dimension
   integer levdimid              ! netcdf id for level dimension
   integer lonid                 ! netcdf id for longitude variable
   integer latid                 ! netcdf id for latitude variable
   integer timeid                ! netcdf id for time variable
   integer cnt4(4)               ! array of counts for each dimension
   integer strt4(4)              ! array of starting indices
   integer i, k, lat, n          ! longitude, level, latitude, time indices
   integer istat                 ! error return
   integer dimids(nf_max_var_dims) ! netcdf variable shape

   integer yr, mon, day, ncsec ! components of a date
   real(r8) calday                   ! current calendar day
   real(r8) caldayloc                ! calendar day (includes yr if no cycling)
   real(r8) fact1, fact2, deltat

   character*4 yrtmp1, yrtmp2
   character*9 yrtmp

!---------------zf 2010.11.11------------------------
   real(r8) xch4i_am(plond,pver,plat)  ! input sulfate bio mixing ratios
   real(r8) xch4i_bm(plond,pver,plat)  ! input sulfate ant mixing ratios
   real(r8) xch4i_ap(plond,pver,plat)  ! input sulfate bio mixing ratios
   real(r8) xch4i_bp(plond,pver,plat)  ! input sulfate ant mixing ratios
!-----------------------------------------------------

   real(r8) xch4i(plon,pver,plat,2)   ! input ch4 mixing ratios

   real(r8), allocatable :: xch4i_tempa(:,:,:) 
   real(r8), allocatable :: xch4i_tempb(:,:,:)
!
!-----------------------------------------------------------------------
!
! Allocate space for data.
!
  allocate( ch4i(pcols,pver,begchunk:endchunk,2), stat=istat )
  call alloc_err( istat, 'ch4ini', 'ch4i', pcols*pver*(endchunk-begchunk+1)*2 )
  allocate( ch4(pcols,pver,begchunk:endchunk), stat=istat )
  call alloc_err( istat, 'aeroini', 'aero',  pcols*pver*(endchunk-begchunk+1) )
!
! unit shift from second to day
!
   do n=1,nsample
      sample_ch4(n) = sample_ch4(n)/86400.0+1.0
   enddo
!
! the names of rawdata files
!
      do i = len_trim(bndic), 1, -1
         if(bndic(i:i).eq.'/') exit
      enddo
      ch4path = bndic(1:i)
      ch4data = trim(ch4path)//'resolution-data/'

   yrtmp = '1950-1959'
   do n=1,nfile
      write(yrtmp1,'(i4.4)') yr_ch4(n)-5
      write(yrtmp2,'(i4.4)') yr_ch4(n)+4
      yrtmp = yrtmp1//'-'//yrtmp2
!zf      if( get_resolution() == 'T106' .or. get_resolution() == 'T266' .or. get_resolution() == 'T255') then
         if( pver ==  46 ) then
             file_ch4(n) = trim(p1t42)//yrtmp//trim(p2L46)
         else ! pver == 26
             file_ch4(n) = trim(p1t42)//yrtmp//trim(p2L26)
         endif
!zf      elseif(  get_resolution() == 'T42' ) then
!         file_ch4(n) = trim(p1t42)//yrtmp//trim(p2)
!zf      endif
   enddo

! set the time_boundaries of input dataset
!
   bndry_m = yr_ch4(1    )*365.0 + sample_ch4(1) 
   bndry_p = yr_ch4(nfile)*365.0 + sample_ch4(nsample) 
  
!!   if (masterproc) then
!!       write(*,*) 'bndry_m=',bndry_m
!!       write(*,*) 'bndry_p=',bndry_p
!!   endif
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
!  if (masterproc) then

     calday = get_curr_calday()
     call get_curr_date(yr, mon, day, ncsec)
     if ( rampyear_prescribed_ch4 .ne. bigint ) then
        yr = rampyear_prescribed_ch4     
     endif
     caldayloc = calday + yr*365.

!----------------------------------------------------------------
! locate the model time among the samples of ch4 data file      
!---------------------------------------------------------------
    if ( caldayloc.lt.bndry_m ) then

        outofbndry = .true.
        cdaych4p  = bndry_m
        cdayp      = sample_ch4(1)    !???
        sp_p   = 1
        yrp    = yr_ch4(1)
        yr_ap  = yr_ch4(1)
        yr_bp  = yr_ch4(2)
        indxbp = 2
        ch4datafile_ap = trim(ch4data)//file_ch4(1)
        ch4datafile_bp = trim(ch4data)//file_ch4(2)

    elseif ( caldayloc.ge.bndry_p ) then

        outofbndry = .true.
        sp_p   = nsample 
        yrp    = yr_ch4(nfile)
        yr_ap  = yr_ch4(nfile)
        yr_bp  = yr_ch4(nfile)
        ch4datafile_ap = trim(ch4data)//file_ch4(nfile)

    else
        outofbndry = .false.
!
! month
!
       if ( calday.lt.sample_ch4(1) ) then
          yrm  = yr-1
          yrp  = yr
          sp_m = nsample 
          sp_p = 1
          goto 100
       elseif (calday.ge.sample_ch4(nsample) ) then
          yrm = yr
          yrp = yr+1
          sp_m = nsample
          sp_p = 1 
          goto 100
       else 
         do n=1,nsample-1
            if ( calday.ge.sample_ch4(n) .and. calday.lt.sample_ch4(n+1) ) then
               yrm  = yr
               yrp  = yr
               sp_m = n
               sp_p = n+1
               goto 100
            endif
         enddo
       endif
       write(*,*) 'CH4INI: Failed to find seconds bracketing model_time =',calday
       call endrun
100    continue
       if ( rampyear_prescribed_ch4 .ne. bigint ) then
          yrm = rampyear_prescribed_ch4  
          yrp = rampyear_prescribed_ch4
       endif
       cdaych4m = yrm*365.0+sample_ch4(sp_m)
       cdaych4p = yrp*365.0+sample_ch4(sp_p)
       cdaym = sample_ch4(sp_m)
       cdayp = sample_ch4(sp_p)
!
! year ( both yrm & yrp )
!
       do n=1,nfile-1
          yr_am = yr_ch4(n)
          yr_bm = yr_ch4(n+1)
          if ( yrm.ge.yr_am .and. yrm.lt.yr_bm ) goto 200
       enddo
       write(*,*) 'CH4INI: Failed to find years bracketing year  yrm=',yrm
       call endrun
200    continue
       ch4datafile_am = trim(ch4data)//trim(file_ch4(n))
       ch4datafile_bm = trim(ch4data)//trim(file_ch4(n+1))
 
       do n=1,nfile-1
          yr_ap = yr_ch4(n)
          yr_bp = yr_ch4(n+1)
          if ( yrp.ge.yr_ap .and. yrp.lt.yr_bp ) goto 300
       enddo
       write(*,*) 'CH4INI: Failed to find years bracketing year  yrp=',yrp
       call endrun
300    indxbp = n+1 
       ch4datafile_ap = trim(ch4data)//trim(file_ch4(n))
       ch4datafile_bp = trim(ch4data)//trim(file_ch4(n+1))

    endif

   if (masterproc) then
!--------------------------------------------------------------------------
! open the right data files and read in raw data
!--------------------------------------------------------------------------
    if ( outofbndry ) then

      call getfil(ch4datafile_ap, locfn_ap)
      call wrap_open(locfn_ap, 0, ncid_ch4_ap)

      write(6,*) 'Now open CH4 file (locfn_ap):', locfn_ap
!
! Obtain dimension id's
!
      call wrap_inq_dimid( ncid_ch4_ap, 'lon', londimid)
      call wrap_inq_dimid( ncid_ch4_ap, 'lat', latdimid)
      call wrap_inq_dimid( ncid_ch4_ap, 'lev', levdimid)
      call wrap_inq_dimid( ncid_ch4_ap, 'date',timeid  )
!
! Obtain size of dimensions.
! Check that horizontal and vertical dimensions are same as model's
! (check one file only and assume that all the other files are of the same setting)
!
       call wrap_inq_dimlen( ncid_ch4_ap, londimid, lonch4siz   ) !zf 2010.11.10

       call wrap_inq_dimlen( ncid_ch4_ap, latdimid, latch4siz   )  !zf 2010.11.10

!-----------------zf 2010.11.10------------
      allocate( ch4lon(lonch4siz), stat=istat )  
      allocate( ch4lat(latch4siz), stat=istat )  

      call wrap_inq_varid( ncid_ch4_am, 'lon', lonid   )
      call wrap_inq_varid( ncid_ch4_am, 'lat', latid   )
      call wrap_get_var_realx( ncid_ch4_am, lonid, ch4lon )
      call wrap_get_var_realx( ncid_ch4_am, latid, ch4lat )
!---------------------------------------------------------------
      call wrap_inq_dimlen( ncid_ch4_ap, levdimid, levch4siz   )
!zf      if (levch4siz > pver) then
!zf         write(6,*)'CH4INI: levch4siz=',levch4siz,' must le ',pver
!zf         call endrun
!zf      end if

      call wrap_inq_dimlen( ncid_ch4_ap, timeid, timesiz   )
      if (timesiz /= nsample) then
         write(6,*)'CH4INI: timesiz=',timesiz,' must = ', nsample 
         call endrun
      end if
!
! wzz, 2009.9.28
!     call wrap_inq_varid( ncid_ch4_ap, 'lev' , aero_id_ap )
!     call wrap_get_var_realx (ncid_ch4_ap,aero_id_ap,m_bi)
!
! the variable
!

      call wrap_inq_varid( ncid_ch4_ap, 'CH4' , ch4_id_ap )

      call wrap_inq_vardimid (ncid_ch4_ap, ch4_id_ap, dimids)
      if (dimids(1) /= londimid .and. dimids(3) /= levdimid .and. dimids(2) /= latdimid) then
         write(6,*)'CH4INI: Data must be ordered lon, lat, lev, time'
         call endrun
      end if
!
!---------------------------------------------------------------
! if not out of boundary
!---------------------------------------------------------------
    else   
  
      call getfil(ch4datafile_am, locfn_am)
      call wrap_open(locfn_am, 0, ncid_ch4_am)
      write(6,*) 'Now open CH4 file (locfn_am):', locfn_am

      call getfil(ch4datafile_bm, locfn_bm)
      call wrap_open(locfn_bm, 0, ncid_ch4_bm)
      write(6,*) 'Now open CH4 file (locfn_bm):', locfn_bm

      call getfil(ch4datafile_ap, locfn_ap)
      call wrap_open(locfn_ap, 0, ncid_ch4_ap)
      write(6,*) 'Now open CH4 file (locfn_ap):', locfn_ap

      call getfil(ch4datafile_bp, locfn_bp)
      call wrap_open(locfn_bp, 0, ncid_ch4_bp)
      write(6,*) 'Now open CH4 file (locfn_bp):', locfn_bp
!
      call wrap_inq_dimid( ncid_ch4_am, 'lon', londimid)
      call wrap_inq_dimid( ncid_ch4_am, 'lat', latdimid)
      call wrap_inq_dimid( ncid_ch4_am, 'lev', levdimid)
      call wrap_inq_dimid( ncid_ch4_am, 'time',timeid  )

      call wrap_inq_dimlen( ncid_ch4_am, londimid, lonch4siz   )
      call wrap_inq_dimlen( ncid_ch4_am, latdimid, latch4siz   )
      call wrap_inq_dimlen( ncid_ch4_am, levdimid, levch4siz   )
!zf      if (levch4siz >  pver) then
!zf         write(6,*)'CH4INI: levch4siz=',levch4siz,' must le ',pver
!zf         call endrun
!zf      end if

      call wrap_inq_dimlen( ncid_ch4_am, timeid, timesiz   )
      if (timesiz /= nsample) then
         write(6,*)'CH4INI: timesiz=',timesiz,' must = ', nsample 
         call endrun
      end if
!------------------zf 2010.11.08-----------------------------------
      allocate( ch4lon(lonch4siz), stat=istat )   
      allocate( ch4lat(latch4siz), stat=istat )

      call wrap_inq_varid( ncid_ch4_am, 'lon', lonid   )
      call wrap_inq_varid( ncid_ch4_am, 'lat', latid   )
      call wrap_get_var_realx( ncid_ch4_am, lonid, ch4lon )
      call wrap_get_var_realx( ncid_ch4_am, latid, ch4lat )
!--------------------------------------------------------------------
!
! wzz, 2009.9.28
!     call wrap_inq_varid( ncid_ch4_ap, 'lev' , aero_id_ap )
!     call wrap_get_var_realx (ncid_ch4_ap,aero_id_ap,m_bi)
!
! Obtain ch4 mixing ratio id

      call wrap_inq_varid( ncid_ch4_am, 'CH4' , ch4_id_am )

      call wrap_inq_vardimid (ncid_ch4_am, ch4_id_am, dimids)
      if (dimids(1) /= londimid .and. dimids(3) /= levdimid .and. dimids(2) /= latdimid) then
         write(6,*)'CH4INI: Data must be ordered lon, lat, lev, time'
         call endrun
      end if
    endif    ! end if (.not.outofbndry)
!------------------------------------------------------------------------------
! master does the work above
!------------------------------------------------------------------------------
   endif
!
   allocate( xch4i_tempa(lonch4siz,latch4siz,levch4siz), stat=istat )   !zf2010.11.08
   allocate( xch4i_tempb(lonch4siz,levch4siz,latch4siz), stat=istat )   !zf2010.11.08

   if (masterproc) then
!--------------------------------------------------------------------------
! open the right data files and read in raw data
!--------------------------------------------------------------------------
    
    if ( outofbndry ) then
! Set up hyperslab corners
!
      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
      cnt4(1)  = lonch4siz  !zf
      cnt4(2)  = latch4siz  !zf
      cnt4(3)  = levch4siz
      cnt4(4)  = 1
!
! need to read in only 1 sample of rawdata
!
      strt4(4) = sp_p 
      xch4i_ap = 0.0
      xch4i_tempa = 0.0   !zf
      xch4i_tempb = 0.0   !zf

      call wrap_inq_varid( ncid_ch4_ap, 'CH4', ch4_id_ap )
      call wrap_get_vara_realx (ncid_ch4_ap,ch4_id_ap,strt4,cnt4, &
           xch4i_tempa(:,:,1:levch4siz))  !zf

!-------------zf 2010.11.10-----------------
      do i = 1, lonch4siz
      do n = 1, latch4siz
      do k = 1, levch4siz
        xch4i_tempb(i, k, n) = xch4i_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonch4siz .eq. plon .and. latch4siz .eq. plat) then
      xch4i_ap = xch4i_tempb
     else
      
      call bilin(xch4i_tempb,ch4lon,ch4lat,lonch4siz,lonch4siz,levch4siz,levch4siz,   &
                 latch4siz,xch4i_ap,londeg,latdeg,plond,nlon,levch4siz,plat)
     end if
!-----------------------------------------------

      do n    = 1,2
       do lat = 1,plat
        do k  = 1,pver
         do i = 1,plon
           xch4i(i,k,lat,n)= xch4i_ap(i,k,lat)
         enddo
        enddo
       enddo
      enddo

     else
! Set up hyperslab corners
!
      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
      cnt4(1)  = lonch4siz  !zf
      cnt4(2)  = latch4siz   !zf
      cnt4(3)  = levch4siz
      cnt4(4)  = 1
!
! read in the 4 sample of rawdata
!
      strt4(4) = sp_m 
      xch4i_am = 0.0

      call wrap_inq_varid( ncid_ch4_am, 'CH4', ch4_id_am )
      call wrap_inq_varid( ncid_ch4_bm, 'CH4', ch4_id_bm )
      call wrap_inq_varid( ncid_ch4_ap, 'CH4', ch4_id_ap )
      call wrap_inq_varid( ncid_ch4_bp, 'CH4', ch4_id_bp )

      xch4i_tempa = 0.0   !zf
      xch4i_tempb = 0.0   !zf

      call wrap_get_vara_realx (ncid_ch4_am,ch4_id_am,strt4,cnt4, &
           xch4i_tempa(:,:,1:levch4siz))  !zf

!-------------zf 2010.11.10-----------------
      do i = 1, lonch4siz
      do n = 1, latch4siz
      do k = 1, levch4siz
        xch4i_tempb(i, k, n) = xch4i_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonch4siz .eq. plon .and. latch4siz .eq. plat) then
      xch4i_am = xch4i_tempb
     else
      call bilin(xch4i_tempb,ch4lon,ch4lat,lonch4siz,lonch4siz,levch4siz,levch4siz,   &
                 latch4siz,xch4i_am,londeg,latdeg,plond,nlon,levch4siz,plat)
     endif
!-----------------------------------------------

      xch4i_bm = 0.0
      xch4i_tempa = 0.0   !zf
      xch4i_tempb = 0.0   !zf
      call wrap_get_vara_realx (ncid_ch4_bm,ch4_id_bm,strt4,cnt4, &
           xch4i_tempa(:,:,1:levch4siz))  !zf
!zf           xch4i_bm(:,:,1:levch4siz))

!-------------zf 2010.11.10-----------------
      do i = 1, lonch4siz
      do n = 1, latch4siz
      do k = 1, levch4siz
        xch4i_tempb(i, k, n) = xch4i_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonch4siz .eq. plon .and. latch4siz .eq. plat) then
      xch4i_bm = xch4i_tempb
     else
      call bilin(xch4i_tempb,ch4lon,ch4lat,lonch4siz,lonch4siz,levch4siz,levch4siz,   &
                 latch4siz,xch4i_bm,londeg,latdeg,plond,nlon,levch4siz,plat)
     endif
!-----------------------------------------------

!
      strt4(4) = sp_p 
      xch4i_ap = 0.0
      xch4i_tempa = 0.0   !zf
      xch4i_tempb = 0.0   !zf
      call wrap_get_vara_realx (ncid_ch4_ap,ch4_id_ap,strt4,cnt4, &
           xch4i_tempa(:,:,1:levch4siz))  !zf
!zf           xch4i_ap(:,:,1:levch4siz))

!-------------zf 2010.11.10-----------------
      do i = 1, lonch4siz
      do n = 1, latch4siz
      do k = 1, levch4siz
        xch4i_tempb(i, k, n) = xch4i_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonch4siz .eq. plon .and. latch4siz .eq. plat) then
      xch4i_ap = xch4i_tempb
     else
      call bilin(xch4i_tempb,ch4lon,ch4lat,lonch4siz,lonch4siz,levch4siz,levch4siz,   &
                 latch4siz,xch4i_ap,londeg,latdeg,plond,nlon,levch4siz,plat)
     endif
!-----------------------------------------------

      xch4i_bp = 0.0
      xch4i_tempa = 0.0   !zf
      xch4i_tempb = 0.0   !zf
      call wrap_get_vara_realx (ncid_ch4_bp,ch4_id_bp,strt4,cnt4, &
           xch4i_tempa(:,:,1:levch4siz))  !zf
!zf           xch4i_bp(:,:,1:levch4siz))

!-------------zf 2010.11.10-----------------
      do i = 1, lonch4siz
      do n = 1, latch4siz
      do k = 1, levch4siz
        xch4i_tempb(i, k, n) = xch4i_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonch4siz .eq. plon .and. latch4siz .eq. plat) then
      xch4i_bp = xch4i_tempb
     else
      call bilin(xch4i_tempb,ch4lon,ch4lat,lonch4siz,lonch4siz,levch4siz,levch4siz,   &
                 latch4siz,xch4i_bp,londeg,latdeg,plond,nlon,levch4siz,plat)
     endif
!-----------------------------------------------


 
! Normal interpolation between consecutive time slices.
!
      write(6,*) "CH4 : yr_bm=",yr_bm,"yr_am=",yr_am, "yrm=",yrm
      deltat = dble( yr_bm - yr_am )
      fact1  = dble( yr_bm - yrm  )/deltat
      fact2  = dble( yrm   - yr_am)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'CH4INI--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
       do k  = 1,pver
        do i = 1,plon
           xch4i(i,k,lat,nmsul)= fact1*xch4i_am(i,k,lat)+fact2*xch4i_bm(i,k,lat)
        enddo
       enddo
      enddo
!
      deltat = dble( yr_bp - yr_ap )
      fact1  = dble( yr_bp - yrp  )/deltat
      fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'CH4INI--2: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
       do k  = 1,pver
        do i = 1,plon
           xch4i(i,k,lat,npsul)= fact1*xch4i_ap(i,k,lat)+fact2*xch4i_bp(i,k,lat)
        enddo
       enddo
      enddo

    endif    ! end if (.not.outofbndry)
!------------------------------------------------------------------------------
! master does the work above
!------------------------------------------------------------------------------
   endif
!------------------------------------------------------------------------------


   call scatter_field_to_chunk(1,pver,2,plon,xch4i,ch4i(:,:,:,:))

!zf   if (outofbndry) then
!zf      call scatter_field_to_chunk(1,pver,1,plon,xch4i_ap,ch4(:,:,:))  !??
!zf   endif

   deallocate (xch4i_tempa)   !zf2010.11.08
   deallocate (xch4i_tempb)   !zf2010.11.08

   return
end subroutine ch4ini

!###############################################################################
!
subroutine ch4int
!----------------------------------------------------------------------- 
!
! Purpose: Interpolate ch4 mixing ratios to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
!-----------------------------------------------------------------------
   use ioFileMod,    only: getfil
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use timeinterp,   only: getfactors

   implicit none

!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
!
! Local workspace
!
   integer cnt4(4)            ! array of counts for each dimension
   integer strt4(4)           ! array of starting indices
   integer i, k, lchnk,lat    ! column, level, chunk, latitude indices
   integer ncol               ! number of columns in current chunk
   integer ntmp               ! temporary
   integer yr, mon, day       ! components of a date
   integer ncsec              ! current time of day [seconds]

   character(len=256)  locfn_bp     ! local filename

   real(r8) calday            ! current calendar day
   real(r8) caldayloc         ! calendar day (includes yr if no cycling)
   real(r8) deltat            ! time (days) between interpolating ch4 data
   real(r8) fact1, fact2      ! time interpolation factors

   real(r8) xch4i   (plon,pver,plat)  ! input ch4 bio mixing ratios
!--------------------------------------zf---------------------------------
   real(r8) xch4i_ap(plond,pver,plat)  ! input ch4 bio mixing ratios
   real(r8) xch4i_bp(plond,pver,plat)  ! input ch4 ant mixing ratios
   real(r8), allocatable :: xch4i_tempa(:,:,:)   
   real(r8), allocatable :: xch4i_tempb(:,:,:) 
   integer istat                        !  error return
!--------------------------------------------------------------------------

   integer :: sul_id_ap, sul_id_bp
!-----------------------------------------------------------------------
!
! Use year information only if a multiyear dataset
!
   calday = get_curr_calday()
   call get_curr_date(yr, mon, day, ncsec)
   if ( rampyear_prescribed_ch4 .ne. bigint ) then
      yr = rampyear_prescribed_ch4
   endif
   caldayloc = calday + yr*365.

   if ((caldayloc.lt.bndry_m).or.(caldayloc.ge.bndry_p)) then
        outofbndry = .true.
   else
        outofbndry = .false.
   endif
   allocate( xch4i_tempa(lonch4siz,latch4siz,pver), stat=istat )   !zf2010.11.08
   allocate( xch4i_tempb(lonch4siz,pver,latch4siz), stat=istat )   !zf2010.11.08
!-------------------------------------------------------------------------------
!  there is no need to do anything when current model time is
!   out of the time-boundary of input data files
!-------------------------------------------------------------------------------
 if (.not.outofbndry) then
!
! Initialize hyperslab corners
!
   if (masterproc) then

      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
      cnt4(1)  = lonch4siz  !zf
      cnt4(2)  = latch4siz  !zf
      cnt4(3)  = pver       ! wzz, 2010.4.26
      cnt4(4)  = 1

   endif
!
! If model time is past current forward ch4 timeslice, read in the next
! timeslice for time interpolation. 
!
   if ( (caldayloc .gt. cdaych4p .and. rampyear_prescribed_ch4 == bigint) .or. &
        (caldayloc .gt. cdaych4p .and. .not. (sp_p == 1 .and. caldayloc > cdaych4m))) then
!zf   if ( (caldayloc .gt. cdaych4p )) then
      sp_m = sp_p
      sp_p = sp_p+1

      if ( sp_p.gt.nsample ) then     ! a new year
          sp_p = 1
          yrp  = yrp+1
          if ( rampyear_prescribed_ch4 .ne. bigint ) then
             yrp = rampyear_prescribed_ch4
          endif

          if (masterproc) then

            if ( yrp.gt.yr_bp ) then    ! need to locate the new year
             indxbp = indxbp+1
             yr_ap = yr_bp
             yr_bp = yr_ch4(indxbp)

             ncid_ch4_ap = ncid_ch4_bp

             ch4datafile_bp = trim(ch4data)//trim(file_ch4(indxbp))
             call getfil(ch4datafile_bp, locfn_bp)
             call wrap_open(locfn_bp, 0, ncid_ch4_bp)
             write(6,*) 'Now open CH4 file (locfn_bp):', locfn_bp 

            endif

          endif

      endif
      
      cdaych4m = cdaych4p
      cdaych4p = yrp*365.0+sample_ch4(sp_p)

      cdaym = cdayp
      cdayp = sample_ch4(sp_p)

      ntmp = nmsul
      nmsul   = npsul
      npsul   = ntmp

      if (masterproc) then
       write(*,*)'CH4: new month= ', sp_p, npsul, indxbp, yr_ap,  yr_bp
        strt4(4) = sp_p 
        xch4i_ap = 0.0
        xch4i_tempa = 0.0  !zf
        xch4i_tempb = 0.0  !zf
        call wrap_inq_varid( ncid_ch4_ap, 'CH4' , sul_id_ap )
        call wrap_get_vara_realx (ncid_ch4_ap,sul_id_ap,strt4,cnt4, &
             xch4i_tempa(:,:,1:pver))  !zf

!-------------zf 2010.11.10-----------------
      do i = 1, lonch4siz
      do lat = 1, latch4siz
      do k = 1, levch4siz
        xch4i_tempb(i, k, lat) = xch4i_tempa(i, lat, k)
      enddo
      enddo
      enddo
       
     if(lonch4siz .eq. plon .and. latch4siz .eq. plat) then
      xch4i_ap = xch4i_tempb
     else
      call bilin(xch4i_tempb,ch4lon,ch4lat,lonch4siz,lonch4siz,levch4siz,levch4siz,  &
                 latch4siz,xch4i_ap,londeg,latdeg,plond,nlon,levch4siz,plat)
     endif
!-----------------------------------------------

!
        xch4i_bp = 0.0
        xch4i_tempa = 0.0  !zf
        xch4i_tempb = 0.0  !zf
        call wrap_inq_varid( ncid_ch4_bp, 'CH4', sul_id_bp )
        call wrap_get_vara_realx (ncid_ch4_bp,sul_id_bp,strt4,cnt4, &
             xch4i_tempa(:,:,1:pver))  !zf

!-------------zf 2010.11.10-----------------
      do i = 1, lonch4siz
      do lat = 1, latch4siz
      do k = 1, levch4siz
        xch4i_tempb(i, k, lat) = xch4i_tempa(i, lat, k)
      enddo
      enddo
      enddo
       
     if(lonch4siz .eq. plon .and. latch4siz .eq. plat) then
      xch4i_bp = xch4i_tempb
     else
      call bilin(xch4i_tempb,ch4lon,ch4lat,lonch4siz,lonch4siz,levch4siz,levch4siz,  &
                 latch4siz,xch4i_bp,londeg,latdeg,plond,nlon,levch4siz,plat)
     endif
!-----------------------------------------------

!
! Normal interpolation between consecutive time slices.
!
        write(6,*) "CH4 : yr_bp,yr_ap,yrp" , yr_bp,yr_ap,yrp
        deltat = dble( yr_bp - yr_ap )
        fact1  = dble( yr_bp - yrp  )/deltat
        fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'CH4INT--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
        do lat = 1,plat
         do k  = 1,pver
          do i = 1,plon
             xch4i(i,k,lat)= fact1*xch4i_ap(i,k,lat)+fact2*xch4i_bp(i,k,lat)
          enddo
         enddo
        enddo
!
! master does the work above
!
      endif  

      call scatter_field_to_chunk(1,pver,1,plon,xch4i,ch4i(1,1,begchunk,npsul))

   end if
!
! Determine time interpolation factor.   
!
   if ( rampyear_prescribed_ch4 .ne. bigint ) then   ! wzz, 2009.10.20
      call getfactors (.true., sp_p, sample_ch4(sp_m), sample_ch4(sp_p), calday, &
                       fact1, fact2, 'ch4int:')
   else
      call getfactors (.false., sp_p, cdaych4m, cdaych4p, caldayloc, &
                       fact1, fact2, 'ch4int:')
   endif
!
! Time interpolation.
!
   do lchnk=begchunk,endchunk
      ncol = get_ncols_p(lchnk)
      do k=1,pver
         do i=1,ncol
            ch4(i,k,lchnk) = ch4i(i,k,lchnk,nmsul)*fact1 +  &
                              ch4i(i,k,lchnk,npsul)*fact2
         end do
      end do
   end do
!

!----------------------------------------------------------------------------
 endif
!----------------------------------------------------------------------------

   deallocate (xch4i_tempa)   !zf2010.11.08
   deallocate (xch4i_tempb)   !zf2010.11.08

   return
end subroutine ch4int

subroutine getch4( lchnk, ncol, ch4_ub )

!-----------------------------------------------------------------------
!
! Purpose: Return slice of time interpolated CH4.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
!
! Arguments
!
  integer , intent(in)   :: lchnk            ! chunk identifier
  integer , intent(in)   :: ncol             ! number of atmospheric columns
  real(r8), intent(out)  :: ch4_ub(pcols,pver)  ! biogenic sulfate
!
! Local variables.
!
   integer :: i, k, n                 ! longitude, level indices
!-----------------------------------------------------------------------
   do k = 1, pver
      do i = 1, ncol
         ch4_ub(i,k) = ch4(i,k,lchnk)
      end do
   enddo

   return
end subroutine getch4
!###############################################################################
end module prescribed_ch4
