#include <misc.h>
#include <params.h>

module chem_timestep_init

      implicit none
      private

      public :: timestep_init

      save

contains

  subroutine timestep_init
    use shr_kind_mod,      only : r8 => shr_kind_r8
    use time_manager,      only : get_curr_calday
    use mo_srf_emissions,  only : set_srf_emissions_time
    use mo_strato_aerosol, only : aero_timestep_init
    use mo_sulf,           only : set_sulf_time
    use mo_drydep,         only : interpdvel
    use mo_solar_parms,    only : solar_parms_timestep_init
    use mo_waccm_photo,    only : photo_timestep_init
    use mo_jshort,         only : jshort_timestep_init
    use mo_jlong,          only : jlong_timestep_init
    use mo_aurora,         only : aurora_timestep_init

    implicit none

!-----------------------------------------------------------------------
! Local variables
!-----------------------------------------------------------------------
    real(r8) :: calday

!-----------------------------------------------------------------------
! get current calendar day of year
!-----------------------------------------------------------------------
    calday = get_curr_calday( )

!-----------------------------------------------------------------------
! Set emissions timing factors
!-----------------------------------------------------------------------
    call set_srf_emissions_time( calday )

!-----------------------------------------------------------------------
! Set dry deposition time interpolation
!-----------------------------------------------------------------------
    call interpdvel( calday )

!-----------------------------------------------------------------------
! Check sad data time window
!-----------------------------------------------------------------------
    call aero_timestep_init

!-----------------------------------------------------------------------
! Set sulf timing factors
!-----------------------------------------------------------------------
    call set_sulf_time( calday )

!-----------------------------------------------------------------------
! Set solar parameters
!-----------------------------------------------------------------------
    call solar_parms_timestep_init

!-----------------------------------------------------------------------
! Set source etf
!-----------------------------------------------------------------------
    call photo_timestep_init

!-----------------------------------------------------------------------
! Set jshort etf
!-----------------------------------------------------------------------
    call jshort_timestep_init
!-----------------------------------------------------------------------
! Set jlong etf
!-----------------------------------------------------------------------
    call jlong_timestep_init

!-----------------------------------------------------------------------
! Set up aurora
!-----------------------------------------------------------------------
    call aurora_timestep_init

  end subroutine timestep_init

end module chem_timestep_init
