
      module mo_chemini

      implicit none
      private

      public :: chemini

      contains

      subroutine chemini

!-----------------------------------------------------------------------
! 	... chemistry module intialization
!-----------------------------------------------------------------------
      use shr_kind_mod,      only : r8 => shr_kind_r8
      use pmgrid,            only : plev, plevp
      use mo_airplane,       only : airpl_src
      use mo_srf_emissions,  only : srf_emissions_inti
      use mo_sad,            only : sad_inti
      use mo_sulf,           only : sulf_inti
      use mo_strato_aerosol, only : aero_init
      use mo_waccm_photo,    only : waccm_prate_inti
      use mo_lightning,      only : lightning_inti
      use mo_drydep,         only : dvel_inti
      use mo_aurora,         only : aurora_inti
      use woods,             only : woods_init, woods_set_etf
      use neckel,            only : neckel_init, neckel_scale_etf
      use euvac,             only : euvac_init, euvac_set_etf
      use mo_heatnirco2,     only : heatnirco2_init
      use mo_solar_parms,    only : solar_parms_init, get_solar_parms
      use mo_imp_sol,        only : imp_slv_inti
      use pmgrid,            only : iam, plon, plat
      use abortutils,        only : endrun
      use iondrag,           only : iondrag_init
      use mo_surf,           only : surf_inti

      use mo_tracinp,         only: bndic
      use upper_bc,           only : ubc_init

#include <comhyb.h>

      character(len=256)    :: solar_parms_file
      character(len=256)    :: woods_etf_file
      character(len=256)    :: neckel_file
      character(len=256)    :: euvac_file
      character(len=256)    :: euvacdat_file
      character(len=256)    :: photon_file
      character(len=256)    :: electron_file
      character(len=256)    :: airpl_emis_file
      character(len=256)    :: nox_emis_file
      character(len=256)    :: co_emis_file
      character(len=256)    :: ch2o_emis_file
      character(len=256)    :: sad_scenario
      integer               :: sad_fixed_date
      character(len=256)    :: sad_file
      character(len=256)    :: sulf_file
      character(len=256)    :: depvel_file
      character(len=256)    :: xs_coef_file
      character(len=256)    :: xs_short_file
      character(len=256)    :: xs_long_file
      character(len=256)    :: rsf_file
      character(len=256)    :: bndsurf 

      character(len=256)    :: efield_lflux_file
      character(len=256)    :: efield_hflux_file
      character(len=256)    :: efield_wei96_file

      character(len=256) :: tgcm_ubc_file
      character(len=256) :: snoe_ubc_file
!-----------------------------------------------------------------------
! 	... local variables
!-----------------------------------------------------------------------
      integer :: i

      real(r8)          ::   f107
      real(r8)          ::   f107a
      character(len=256)::   locfn
      character(len=256) emisspath

      character(len=20)  time_type    !  'SERIAL', or 'CYCLICAL'
      integer            cycle_yr     !  for time_type = 'CYCLICAL'

    !-----------------------------------------------------------------------
    ! initial the addfile 
	!-----------------------------------------------------------
    call chem_addfld_ini

	!---------------------------------------------------------

    do i = len_trim(bndic), 1, -1
       if(bndic(i:i).eq.'/') exit
    enddo
    emisspath = bndic(1:i)

!-----------------------------------------------------------------------
! 	... read time-independent airplane emissions
!-----------------------------------------------------------------------
!      airpl_emis_file = trim(emisspath)//'emissions.aircraft.T42LR.nc.t42.nc'
      airpl_emis_file = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/cam/chem/trop_mozart/emis/'//&
                   'emissions.aircraft.T42LR.nc'

      call airpl_src(airpl_emis_file)
      write(*,*) 'chemini: after airpl_src on node ',iam

!-----------------------------------------------------------------------
! 	... read time-dependent surface flux dataset
!-----------------------------------------------------------------------
      nox_emis_file = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/cam2/chem/trop_mozart/emis'// &
	                   '/emis.nox.nc'
      co_emis_file  = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/cam/chem/trop_mozart/emis/'// &
	                   'emis.co.nc'
      ch2o_emis_file= '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/cam/chem/trop_mozart/emis/'// &
                       'emis.ch2o.nc'

      call srf_emissions_inti(nox_emis_file, co_emis_file, ch2o_emis_file)
      write(*,*) 'chemini: after srf_emissions on node ',iam

!-----------------------------------------------------------------------
! 	... Read time-dependent surface variables dataset
!-----------------------------------------------------------------------
!     bndsurf = '/home/zhangf/gfs2/BCC_AGCM3/inputdata/chem_data/surfdata.t42.nc'
      bndsurf = '/cma/g4/wutw/INIDATA/chem_data/surfdata.t42.nc'
      call surf_inti( bndsurf )

!-----------------------------------------------------------------------
! 	... read time-dependent sulfate dataset
!-----------------------------------------------------------------------
!     sad_file = '/gfs2/zhangf/BCCCHEM/data/WACCM3.0/data/atm/waccm/sulf/'// &
!                'SAD_GS2D_1950-2005_L66_t42L26.nc'
!
!!    time_type = 'SERIAL'         !  'SERIAL', or 'CYCLICAL'
!!    cycle_yr  = 0
!
!     time_type = 'CYCLICAL'
!     if( time_type .eq. 'CYCLICAL' ) then
!        cycle_yr  =  1971             !  only for time_type = 'CYCLICAL'
!     endif
!
!     call aero_init (sad_file, time_type, cycle_yr )   

      sad_file = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/sulf'// &
                '/SAD_SAGE_4x5_t42L26.nc'

      sad_fixed_date = 19710101
      sad_scenario   = 'FIXED'                ! 'FIXED' or 'RAMPED'   
      call aero_init ( sad_scenario, sad_fixed_date, sad_file)

!-----------------------------------------------------------------------

      sulf_file = '/cma/g4/wutw/INIDATA/chem_data/sulfate.M1.nc'

      call sulf_inti(sulf_file)
      write(*,*) 'chemini: after sulf_inti on node ',iam

!-----------------------------------------------------------------------
!	... initialize the sad module
!-----------------------------------------------------------------------
      call sad_inti
      write(*,*) 'chemini: after sad_inti on node ',iam

!-----------------------------------------------------------------------
!	... initialize the lightning module
!-----------------------------------------------------------------------
      call lightning_inti
      write(*,*) 'chemini: after lightning_inti'

!-----------------------------------------------------------------------
!	... initialize the dry deposition module
!-----------------------------------------------------------------------
      depvel_file = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/cam/chem/trop_mozart/dvel'// &
                    '/depvel_monthly.nc'

      call dvel_inti(depvel_file)
      write(*,*) 'chemini: after dvel_inti'

!-----------------------------------------------------------------------
! 	... initialize the solar parameters module
!-----------------------------------------------------------------------
      solar_parms_file = trim(emisspath)//'proxy_solar_Solomon_Richmond_1845-2008_daily_noleap_c110526.nc'

      call solar_parms_init (solar_parms_file)
      call get_solar_parms( f107_s = f107, f107a_s = f107a )

      write(*,*) 'chemini: f107,f107a = ',f107,f107a
!-----------------------------------------------------------------------
! 	... initialize the woods etf module
!-----------------------------------------------------------------------
      woods_etf_file = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/phot/woods.nc'
      call woods_init (woods_etf_file)
      call woods_set_etf( f107, f107a )
!-----------------------------------------------------------------------
! 	... initialize the neckel etf module
!-----------------------------------------------------------------------
      neckel_file = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/phot/neckel.flx'
      call neckel_init (neckel_file)
      call neckel_scale_etf( f107, f107a )
!-----------------------------------------------------------------------
! 	... initialize the euvac etf module
!-----------------------------------------------------------------------
      euvac_file = trim(emisspath)//'euvac_v1.2.nc' 
      call euvac_init (euvac_file)
      call euvac_set_etf( f107, f107a )

!-----------------------------------------------------------------------
! 	... initialize the co2 nir heating module
!-----------------------------------------------------------------------
      call heatnirco2_init

!-----------------------------------------------------------------------
! 	... initialize photorate module
!-----------------------------------------------------------------------
       xs_long_file   = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/phot'//&
                        '/temp_prs_GT200nm.nc'
       xs_coef_file   = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/phot'// &
                        '/effxstex.txt'
       xs_short_file  = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/phot'// &
                        '/xs_short.nc'
       rsf_file       = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/phot'// &
                        '/RSF_GT200nm.v050316.nc'
       euvacdat_file  = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/phot'// &
                        '/euvac.dat'
       photon_file    = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/phot'// &
                        '/photon.dat'
       electron_file  = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/phot'// &
                        '/electron.dat'

      call waccm_prate_inti(xs_coef_file, xs_short_file, xs_long_file, rsf_file, &
                            euvacdat_file, photon_file, electron_file)
      write(*,*) 'chemini: after waccm_prate_inti'

!-----------------------------------------------------------------------
!	... initialize ion production
!-----------------------------------------------------------------------
      call aurora_inti
      write(*,*) 'chemini: after aurora_inti'

!-----------------------------------------------------------------------
!	... initialize the implicit solver
!-----------------------------------------------------------------------
      call imp_slv_inti

      write(*,*) 'chemini: finished on node ',iam

!--------wtw--------------------------------

      efield_lflux_file = trim(emisspath)//'coeff_lflux.dat'
      efield_hflux_file = trim(emisspath)//'coeff_hflux.dat'
      efield_wei96_file = trim(emisspath)//'wei96.cofcnts'

      call iondrag_init( hypm , efield_lflux_file, efield_hflux_file, efield_wei96_file)

!-----------------------------------
      tgcm_ubc_file = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/ub/tgcm_ubc.nc'
      snoe_ubc_file = '/cma/g4/wutw/BCCCHEM/data/WACCM3.0/data/atm/waccm/ub/snoe_eof.nc'

      call ubc_init(tgcm_ubc_file, snoe_ubc_file)

      end subroutine chemini

!==================================================
  subroutine chem_addfld_ini
!----------------------------------------------------------------------- 
! 
! Purpose: initialize parameterized greenhouse gas chemistry
!          (declare history variables)
! 
! Method: 
! <Describe the algorithm(s) used in the routine.> 
! <Also include any applicable external references.> 
! 
! Author: NCAR CMS
! 
!-----------------------------------------------------------------------
    use history,      only : addfld, add_default, phys_decomp
    use chem_mods,    only : gas_pcnst, imozart
    use m_spc_id,     only : id_h2o
    use ppgrid,               only : pver
    use constituents, only : cnst_name
    use constituents, only : cnst_get_ind
    use phys_buffer,  only : pbuf_get_fld_idx
    use mo_chm_diags, only : chm_diags_inti

!-----------------------------------------------------------------------
! Local variables
!-----------------------------------------------------------------------
    integer :: m, n                                ! tracer indicies
    character(len=3)  :: num
    character(len=8)  :: spc_name
    character(len=32) :: attr
    character(len=8)  :: srcnam(gas_pcnst)
!-----------------------------------------------------------------------
! Set names of chemistry variable tendencies and declare them as history variables
!-----------------------------------------------------------------------
    do m = 1,gas_pcnst
       write(num,'(i3)') 100+m
       srcnam(m) = 'CHMTND' // num(2:3)
       call addfld( srcnam(m), 'kg/kg/s ', pver, 'A', trim(cnst_name(imozart-1+m))//' source/sink', phys_decomp )
    end do

    call addfld( 'JEUV-1', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-2', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-3', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-4', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-5', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-6', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-7', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-8', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-9', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-10', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-11', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-12', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-13', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-14', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-15', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-16', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-17', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-18', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-19', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-20', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-21', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-22', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-23', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-24', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'JEUV-25', '/s ', pver, 'I', 'euv photo-ionization', phys_decomp )
    call addfld( 'CPAIR', 'J/K/kg', pver, 'I', 'specific heat cap air', phys_decomp )
    call addfld( 'JO2_A', '/s ', pver, 'I', 'o2 photolysis', phys_decomp )
    call addfld( 'JO2_B', '/s ', pver, 'I', 'o2 photolysis', phys_decomp )
    call addfld( 'JO3_A', '/s ', pver, 'I', 'ozone photolysis', phys_decomp )
    call addfld( 'JO3_B', '/s ', pver, 'I', 'ozone photolysis', phys_decomp )
    call addfld( 'JH2O', '/s ', pver, 'I', 'water vapor photolysis', phys_decomp )
    call addfld( 'JNO', '/s ', pver, 'I', 'no photolysis', phys_decomp )
    call addfld( 'JNO3', '/s ', pver, 'I', 'no3 photolysis(no3+hv->no+o2)', phys_decomp )
    call addfld( 'JCL2O2', '/s ', pver, 'I', 'cl2o2 photolysis', phys_decomp )
    call addfld( 'QDSAD', '/s ', pver, 'I', 'water vapor sad delta', phys_decomp )
    call addfld( 'SAD_SULF', 'cm2/cm3 ', pver, 'I', 'sulfate aerosol SAD', phys_decomp )
    call addfld( 'SAD_SNAT', '/cm ', pver, 'I', 'small nat sad', phys_decomp )
    call addfld( 'SAD_LNAT', 'cm2/cm3 ', pver, 'I', 'large-mode NAT aerosol SAD', phys_decomp )
    call addfld( 'SAD_ICE', 'cm2/cm3 ', pver, 'I', 'water-ice aerosol SAD', phys_decomp )
    call addfld( 'RAD_SULF', 'cm ', pver, 'I', 'sad sulfate', phys_decomp )
    call addfld( 'RAD_SNAT', 'cm ', pver, 'I', 'small nat radius', phys_decomp )
    call addfld( 'RAD_LNAT', 'cm ', pver, 'I', 'large nat radius', phys_decomp )
    call addfld( 'RAD_ICE', 'cm ', pver, 'I', 'sad ice', phys_decomp )
    call addfld( 'VEL_NAT1', 'cm/s', pver, 'I', 'small nat settling velocity', phys_decomp )
    call addfld( 'VEL_NAT2', 'cm/s', pver, 'I', 'large nat settling velocity', phys_decomp )
    call addfld( 'HNO3_CD1', 'mol/mol', pver, 'I', 'STS condensed HNO3', phys_decomp )
    call addfld( 'HNO3_CD2', 'mol/mol', pver, 'I', 'condensed hno3 concentration', phys_decomp )
    call addfld( 'HNO3_CD3', 'mol/mol', pver, 'I', 'large-mode NAT condensed HNO3', phys_decomp )
    call addfld( 'QDSETT', '/s ', pver, 'I', 'water vapor settling delta', phys_decomp )
    call addfld( 'QDCHEM', '/s ', pver, 'I', 'water vapor chemistry delta', phys_decomp )
    call addfld( 'HET1', '/s ', pver, 'I', 'N2O5 + H2O on sulfate aerosol', phys_decomp )
    call addfld( 'HET2', '/s ', pver, 'I', 'CLONO2 + H2O on sulfate aerosol', phys_decomp )
    call addfld( 'HET3', '/s ', pver, 'I', 'BRONO2 + H2O on sulfate aerosol', phys_decomp )
    call addfld( 'HET4', '/s ', pver, 'I', 'CLONO2 + HCL on sulfate aerosol', phys_decomp )
    call addfld( 'HET5', '/s ', pver, 'I', 'HOCL + HCL on sulfate aerosol', phys_decomp )
    call addfld( 'HET6', '/s ', pver, 'I', 'HOBR + HCL on sulfate aerosol', phys_decomp )
    call addfld( 'HET7', '/s ', pver, 'I', 'N2O5 + H2O on NAD aerosol', phys_decomp )
    call addfld( 'HET8', '/s ', pver, 'I', 'CLONO2 + H2O on NAD aerosol', phys_decomp )
    call addfld( 'HET9', '/s ', pver, 'I', 'CLONO2 + HCL on NAD aerosol', phys_decomp )
    call addfld( 'HET10', '/s ', pver, 'I', 'HOCL + HCL on NAD aerosol', phys_decomp )
    call addfld( 'HET11', '/s ', pver, 'I', 'BRONO2 + HCL on NAD aerosol', phys_decomp )
    call addfld( 'HET12', '/s ', pver, 'I', 'N2O5 + H2O on ice aerosol', phys_decomp )
    call addfld( 'HET13', '/s ', pver, 'I', 'CLONO2 + H2O on ice aerosol', phys_decomp )
    call addfld( 'HET14', '/s ', pver, 'I', 'BRONO2 + H2O on ice aerosol', phys_decomp )
    call addfld( 'HET15', '/s ', pver, 'I', 'CLONO2 + HCL on ice aerosol', phys_decomp )
    call addfld( 'HET16', '/s ', pver, 'I', 'HOCL + HCL on ice aerosol', phys_decomp )
    call addfld( 'HET17', '/s ', pver, 'I', 'HOBR + HCL on ice aerosol', phys_decomp )
    call addfld( 'HNO3_GAS', 'mol/mol', pver, 'I', 'gas-phase hno3', phys_decomp )
    call addfld( 'P_Op', '/s ', pver, 'I', 'production o+', phys_decomp )
    call addfld( 'P_O2p', '/s ', pver, 'I', 'production o2+', phys_decomp )
    call addfld( 'P_N2p', '/s ', pver, 'I', 'production n2+', phys_decomp )
    call addfld( 'P_Np', '/s ', pver, 'I', 'production n+', phys_decomp )
    call addfld( 'P_IONS', '/s ', pver, 'I', 'total ion production', phys_decomp )
    call addfld( 'QSUM', '/s ', pver, 'I', 'total ion production', phys_decomp )
    call addfld( 'QRS_AUR', 'K/s', pver, 'I', 'total auroral heating rate', phys_decomp )
    call addfld( 'QRS_CO2NIR', 'K/s', pver, 'I', 'co2 nir heating rate', phys_decomp )
    call addfld( 'QTHERMAL', 'K/s', pver, 'I', 'non-euv photolysis heating rate', phys_decomp )
    call addfld( 'NO_Aircraft', 'molec/cm3/s', pver, 'I', 'aircraft NO emissions', phys_decomp )
    call addfld( 'NO_Lightning', 'molec/cm3/s', pver, 'I', 'lightning NO emissions', phys_decomp )
!-----------------------------------------------------------------------
! Set names of cp and airglow heating history variables
!-----------------------------------------------------------------------
    call addfld( 'AG1', 'K/s ', pver, 'I', 'O2_1S -> O2 + 762nm airglow loss', phys_decomp )
    call addfld( 'AG2', 'K/s ', pver, 'I', 'O2_1D -> O2 + 1.27 micron airglow loss', phys_decomp )
    call addfld( 'AGTOT', 'K/s ', pver, 'I', 'airglow total loss', phys_decomp )
    call addfld( 'CPH1', 'K/s ', pver, 'I', 'O + O3 -> 2*O2 + 93.46 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH2', 'K/s ', pver, 'I', 'O + O + M -> O2 + M + 119.12 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH3', 'K/s ', pver, 'I', 'O + OH -> H + O2 + 16.35 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH4', 'K/s ', pver, 'I', 'O + HO2 -> OH + O2 + 53.97 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH5', 'K/s ', pver, 'I', 'H + O2 + M -> HO2 + M + 48.8 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH6', 'K/s ', pver, 'I', 'O + O2 + M -> O3 + M + 25.66 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH7', 'K/s ', pver, 'I', 'H + O3 -> OH + O2 + 46.27 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH8', 'K/s ', pver, 'I', 'HO2 + NO -> NO2 + OH + 8.06 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH9', 'K/s ', pver, 'I', 'HO2 + O3 -> 2*O2 + OH + 28.31 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH10', 'K/s ', pver, 'I', 'HO2 + HO2 -> H2O2 + O2 + 39.08 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH11', 'K/s ', pver, 'I', 'OH + O3 -> HO2 + O2 + 39.49 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH12', 'K/s ', pver, 'I', 'NO + O3 -> NO2 + O2 + 47.55 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH13', 'K/s ', pver, 'I', 'NO2 + O -> NO + O2 + 45.91 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH14', 'K/s ', pver, 'I', 'OH + HO2 -> H2O + O2 + 69.99 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH15', 'K/s ', pver, 'I', 'H + HO2 -> H2 + O2 + 55.4 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH16', 'K/s ', pver, 'I', 'O1D + O2 -> O + O2_1S + 7.63 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH17', 'K/s ', pver, 'I', 'O1D + N2 -> O + N2 + 45.14 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH18', 'K/s ', pver, 'I', 'O2_1S + O -> O2_1D + O + 14.97 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH19', 'K/s ', pver, 'I', 'O2_1S + O2 -> O2_1D + O2 + 14.97 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH20', 'K/s ', pver, 'I', 'O2_1S + N2 -> O2_1D + N2 + 14.97 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH21', 'K/s ', pver, 'I', 'O2_1S + O3 -> O2_1D + O3 + 14.97 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH22', 'K/s ', pver, 'I', 'O2_1D + O -> O2 + O + 22.54 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH23', 'K/s ', pver, 'I', 'O2_1D + O2 -> 2*O2 + 22.54 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH24', 'K/s ', pver, 'I', 'O2_1D + N2 -> O2 + N2 + 22.54 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH25', 'K/s ', pver, 'I', 'N2D + O2 -> NO + O1D + 42.43 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH26', 'K/s ', pver, 'I', 'N2D + O -> N + O + 54.88 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH27', 'K/s ', pver, 'I', 'N + O2 -> NO + O + 32.28 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH28', 'K/s ', pver, 'I', 'N + NO -> N2 + O + 61.8 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH29', 'K/s ', pver, 'I', 'O1D + O2 -> O + O2 + 45.14 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH30', 'K/s ', pver, 'I', 'Op + O2 -> O2p + O + 35.878 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH31', 'K/s ', pver, 'I', 'Op + N2 -> NOp + N + 25.105 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH32', 'K/s ', pver, 'I', 'N2p + O -> NOp + N2D + 16.140 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH33', 'K/s ', pver, 'I', 'O2p + N -> NOp + O + 97.073 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH34', 'K/s ', pver, 'I', 'O2p + NO -> NOp + O2 + 64.862 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH35', 'K/s ', pver, 'I', 'Np + O2 -> O2p + N + 57.322 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH36', 'K/s ', pver, 'I', 'Np + O2 -> NOp + O + 154.46 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH37', 'K/s ', pver, 'I', 'Np + O -> Op + N + 22.597 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH38', 'K/s ', pver, 'I', 'N2p + O2 -> O2p + N2 + 81.164 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH39', 'K/s ', pver, 'I', 'NOp + e -> 19.691 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH40', 'K/s ', pver, 'I', 'O2p + e -> 121.65 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'CPH41', 'K/s ', pver, 'I', 'N2p + e -> 84.807 kcal/mol chem pot heating rate', phys_decomp )
    call addfld( 'QCP', 'K/s ', pver, 'I', 'chem pot heating rate', phys_decomp )

    call chm_diags_inti
    end subroutine chem_addfld_ini

end module mo_chemini
