
      module mo_chm_diags

      use shr_kind_mod,      only : r8 => shr_kind_r8

      private
      public :: chm_diags_inti
      public :: chm_diags

      contains

      subroutine chm_diags_inti
!--------------------------------------------------------------------
!	... initialize utility routine
!--------------------------------------------------------------------

      use ppgrid,       only : pver
      use history,      only : addfld, phys_decomp

      call addfld( 'NOX', 'mol/mol', pver, 'I', 'nox volume mixing ratio', phys_decomp )
      call addfld( 'NOY', 'mol/mol', pver, 'I', 'noy volume mixing ratio', phys_decomp )
      call addfld( 'BROX', 'mol/mol', pver, 'I', 'brox volume mixing ratio', phys_decomp )
      call addfld( 'BROY', 'mol/mol', pver, 'I', 'broy volume mixing ratio', phys_decomp )
      call addfld( 'CLOX', 'mol/mol', pver, 'I', 'clox volume mixing ratio', phys_decomp )
      call addfld( 'CLOY', 'mol/mol', pver, 'I', 'cloy volume mixing ratio', phys_decomp )
      call addfld( 'TCLY', 'mol/mol', pver, 'I', 'total cl volume mixing ratio', phys_decomp )
      call addfld( 'TOTH', 'mol/mol', pver, 'I', 'total h volume mixing ratio', phys_decomp )
      call addfld( 'PION_EUV', '/cm^3/s', pver, 'I', 'total euv ionization rate', phys_decomp )

      call addfld( 'PEUV1', '/cm^3/s', pver, 'I', '(j1+j2+j3)*o', phys_decomp )
      call addfld( 'PEUV1e', '/cm^3/s', pver, 'I', '(j14+j15+j16)*o', phys_decomp )
      call addfld( 'PEUV2', '/cm^3/s', pver, 'I', 'j4*n', phys_decomp )
      call addfld( 'PEUV3', '/cm^3/s', pver, 'I', '(j5+j7+j8+j9)*o2', phys_decomp )
      call addfld( 'PEUV3e', '/cm^3/s', pver, 'I', '(j17+j19+j20+j21)*o2', phys_decomp )
      call addfld( 'PEUV4', '/cm^3/s', pver, 'I', '(j10+j11)*n2', phys_decomp )
      call addfld( 'PEUV4e', '/cm^3/s', pver, 'I', '(j22+j23)*n2', phys_decomp )
      call addfld( 'PEUVN2D', '/cm^3/s', pver, 'I', '(j11+j13)*n2', phys_decomp )
      call addfld( 'PEUVN2De', '/cm^3/s', pver, 'I', '(j23+j25)*n2', phys_decomp )
      call addfld( 'PJNO', '/cm^3/s', pver, 'I', 'jno*no', phys_decomp )
      call addfld( 'PJNO_I', '/cm^3/s', pver, 'I', 'jno_i*no', phys_decomp )

      end subroutine chm_diags_inti

      subroutine chm_diags( lchnk, ncol, vmr, rxt_rates, invariants )
!--------------------------------------------------------------------
!	... utility routine to output chemistry diagnostic variables
!--------------------------------------------------------------------

      use chem_mods,    only : rxntot, nfs, gas_pcnst, indexm
      use ppgrid,       only : pver
      use history,      only : outfld
      use m_spc_id
      use m_rxt_id

      implicit none

!--------------------------------------------------------------------
!	... dummy arguments
!--------------------------------------------------------------------
      integer, intent(in)   :: lchnk
      integer, intent(in)   :: ncol
      real(r8), intent(in)  :: vmr(ncol,pver,gas_pcnst)
      real(r8), intent(in)  :: rxt_rates(ncol,pver,rxntot)
      real(r8), intent(in)  :: invariants(ncol,pver,max(1,nfs))

!--------------------------------------------------------------------
!	... local variables
!--------------------------------------------------------------------
      integer     :: k, n
      real(r8)    :: wrk(ncol,pver)
      real(r8)    :: tmp(ncol,pver)
      real(r8)    :: m(ncol,pver)
      real(r8)    :: un2(ncol)

!--------------------------------------------------------------------
!	... "diagnostic" groups
!--------------------------------------------------------------------
      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_n) + vmr(:,k,id_no) + vmr(:,k,id_no2)
      end do
      call outfld( 'NOX', wrk(:,:), ncol, lchnk )

      do k = 1,pver
         wrk(:,k) = wrk(:,k) + vmr(:,k,id_no3) + 2.*vmr(:,k,id_n2o5) + vmr(:,k,id_hno3) &
		             + vmr(:,k,id_ho2no2) + vmr(:,k,id_clono2) + vmr(:,k,id_brono2)
      end do
      call outfld( 'NOY', wrk, ncol, lchnk )

      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_cl) + vmr(:,k,id_clo) + vmr(:,k,id_hocl) &
                    + 2._r8*(vmr(:,k,id_cl2) + vmr(:,k,id_cl2o2)) + vmr(:,k,id_oclo)
      end do
      call outfld( 'CLOX', wrk, ncol, lchnk )

      do k = 1,pver
	 wrk(:,k) = wrk(:,k) + vmr(:,k,id_hcl) + vmr(:,k,id_clono2) + vmr(:,k,id_brcl)
      end do
      call outfld( 'CLOY', wrk, ncol, lchnk )

      do k = 1,pver
	 wrk(:,k) = wrk(:,k) + 4._r8*vmr(:,k,id_ccl4) &
			     + 3._r8*(vmr(:,k,id_cfc11) + vmr(:,k,id_cfc113) + vmr(:,k,id_ch3ccl3)) &
			     + 2._r8*vmr(:,k,id_cfc12) + vmr(:,k,id_ch3cl) + vmr(:,k,id_hcfc22) + vmr(:,k,id_cf2clbr)
      end do
      call outfld( 'TCLY', wrk, ncol, lchnk )

      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_br) + vmr(:,k,id_bro)
      end do
      call outfld( 'BROX', wrk, ncol, lchnk )

      do k = 1,pver
	 wrk(:,k) = wrk(:,k) + vmr(:,k,id_hbr) + vmr(:,k,id_brono2) + vmr(:,k,id_brcl) + vmr(:,k,id_hobr)
      end do
      call outfld( 'BROY', wrk, ncol, lchnk )

      do k = 1,pver
	 wrk(:,k) = 2._r8*vmr(:,k,id_ch4) + vmr(:,k,id_h2o) + vmr(:,k,id_h2)
      end do
      call outfld( 'TOTH', wrk, ncol, lchnk )

!--------------------------------------------------------------------
!	... euv ion production
!--------------------------------------------------------------------
      do k = 1,pver
	 un2(:)   = 1._r8 - (vmr(:,k,id_o) + vmr(:,k,id_o2) + vmr(:,k,id_h))
	 wrk(:,k) = vmr(:,k,id_o)*(rxt_rates(:,k,rid_jeuv_1) + rxt_rates(:,k,rid_jeuv_2) &
				   + rxt_rates(:,k,rid_jeuv_3) + rxt_rates(:,k,rid_jeuv_14) &
				   + rxt_rates(:,k,rid_jeuv_15) + rxt_rates(:,k,rid_jeuv_16)) &
	          + vmr(:,k,id_n)*rxt_rates(:,k,rid_jeuv_4) &
	          + vmr(:,k,id_o2)*(rxt_rates(:,k,rid_jeuv_5) + rxt_rates(:,k,rid_jeuv_7) &
				    + rxt_rates(:,k,rid_jeuv_8) + rxt_rates(:,k,rid_jeuv_9) &
				    + rxt_rates(:,k,rid_jeuv_17) + rxt_rates(:,k,rid_jeuv_19) &
				    + rxt_rates(:,k,rid_jeuv_20) + rxt_rates(:,k,rid_jeuv_21)) &
	          + un2(:)*(rxt_rates(:,k,rid_jeuv_6) + rxt_rates(:,k,rid_jeuv_10) &
			    + rxt_rates(:,k,rid_jeuv_11) + rxt_rates(:,k,rid_jeuv_18) &
			    + rxt_rates(:,k,rid_jeuv_22) + rxt_rates(:,k,rid_jeuv_23))
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PION_EUV', wrk, ncol, lchnk )

!--------------------------------------------------------------------
!	... marsh diagnostics
!--------------------------------------------------------------------
      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_o)*(rxt_rates(:,k,rid_jeuv_1) + rxt_rates(:,k,rid_jeuv_2) &
				   + rxt_rates(:,k,rid_jeuv_3))
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PEUV1', wrk, ncol, lchnk )
      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_o)*(rxt_rates(:,k,rid_jeuv_14) + rxt_rates(:,k,rid_jeuv_15) &
				   + rxt_rates(:,k,rid_jeuv_16))
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PEUV1e', wrk, ncol, lchnk )
      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_n)*rxt_rates(:,k,rid_jeuv_4)
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PEUV2', wrk, ncol, lchnk )
      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_o2)*(rxt_rates(:,k,rid_jeuv_5) + rxt_rates(:,k,rid_jeuv_7) &
				   + rxt_rates(:,k,rid_jeuv_8) + rxt_rates(:,k,rid_jeuv_9))
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PEUV3', wrk, ncol, lchnk )
      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_o2)*(rxt_rates(:,k,rid_jeuv_17) + rxt_rates(:,k,rid_jeuv_19) &
				   + rxt_rates(:,k,rid_jeuv_20) + rxt_rates(:,k,rid_jeuv_21))
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PEUV3e', wrk, ncol, lchnk )
      do k = 1,pver
	 un2(:)   = 1._r8 - (vmr(:,k,id_o) + vmr(:,k,id_o2) + vmr(:,k,id_h))
	 wrk(:,k) = un2(:)*(rxt_rates(:,k,rid_jeuv_6) + rxt_rates(:,k,rid_jeuv_10) + rxt_rates(:,k,rid_jeuv_11))
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PEUV4', wrk, ncol, lchnk )
      do k = 1,pver
	 un2(:)   = 1._r8 - (vmr(:,k,id_o) + vmr(:,k,id_o2) + vmr(:,k,id_h))
	 wrk(:,k) = un2(:)*(rxt_rates(:,k,rid_jeuv_18) + rxt_rates(:,k,rid_jeuv_22) + rxt_rates(:,k,rid_jeuv_23))
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PEUV4e', wrk, ncol, lchnk )
      do k = 1,pver
	 un2(:)   = 1._r8 - (vmr(:,k,id_o) + vmr(:,k,id_o2) + vmr(:,k,id_h))
	 wrk(:,k) = un2(:)*(rxt_rates(:,k,rid_jeuv_11) + rxt_rates(:,k,rid_jeuv_13))
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PEUVN2D', wrk, ncol, lchnk )
      do k = 1,pver
	 un2(:)   = 1._r8 - (vmr(:,k,id_o) + vmr(:,k,id_o2) + vmr(:,k,id_h))
	 wrk(:,k) = un2(:)*(rxt_rates(:,k,rid_jeuv_23) + rxt_rates(:,k,rid_jeuv_25))
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PEUVN2De', wrk, ncol, lchnk )
      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_no)*rxt_rates(:,k,rid_jno_i)
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PJNO_I', wrk, ncol, lchnk )
      do k = 1,pver
	 wrk(:,k) = vmr(:,k,id_no)*rxt_rates(:,k,rid_jno)
	 wrk(:,k) = wrk(:,k) * invariants(:,k,indexm)
      end do
      call outfld( 'PJNO', wrk, ncol, lchnk )

      end subroutine chm_diags

      end module mo_chm_diags
