
      module mo_exp_sol

      private
      public :: exp_sol

      contains

      subroutine exp_sol( base_sol, reaction_rates, het_rates, extfrc, delt, ncol )
!-----------------------------------------------------------------------
!      	... Exp_sol advances the volumetric mixing ratio
!           forward one time step via the fully explicit
!           Euler scheme
!-----------------------------------------------------------------------

      use chem_mods,     only : clscnt1, extcnt, gas_pcnst, clsmap, rxntot, hetcnt
      use ppgrid,        only : pver
      use mo_prod_loss,  only : exp_prod_loss
      use mo_indprd,     only : indprd
      use shr_kind_mod,  only : r8 => shr_kind_r8

      implicit none
!-----------------------------------------------------------------------
!     	... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in)  ::  ncol                                ! columns in chunck
      real(r8), intent(in) ::  delt                                ! time step (s)
      real(r8), intent(in) ::  het_rates(ncol,pver,max(1,hetcnt))  ! het rates (1/cm^3/s)
      real(r8), intent(in) ::  reaction_rates(ncol,pver,rxntot), & ! rxt rates (1/cm^3/s)
                               extfrc(ncol,pver,extcnt)            ! "external insitu forcing" (1/cm^3/s)
      real(r8), intent(inout) ::  base_sol(ncol,pver,gas_pcnst)    ! working mixing ratios (vmr)

!-----------------------------------------------------------------------
!     	... Local variables
!-----------------------------------------------------------------------
      integer  ::  k, l, m
      real(r8), dimension(ncol,pver,clscnt1) :: &
                   prod, &
                   loss, &
                   ind_prd

!-----------------------------------------------------------------------      
!        ... Put "independent" production in the forcing
!-----------------------------------------------------------------------      
      call indprd( 1, ind_prd, clscnt1, base_sol, extfrc, &
		   reaction_rates, ncol )

!-----------------------------------------------------------------------      
!      	... Form F(y)
!-----------------------------------------------------------------------      
      call exp_prod_loss( prod, loss, base_sol, reaction_rates, het_rates )

!-----------------------------------------------------------------------      
!    	... Solve for the mixing ratio at t(n+1)
!-----------------------------------------------------------------------      
      do m = 1,clscnt1
         l = clsmap(m,1)
	 do k = 1,pver
            base_sol(:,k,l)  = base_sol(:,k,l) + delt * (prod(:,k,m) + ind_prd(:,k,m) - loss(:,k,m))
	 end do
      end do

      end subroutine exp_sol

      end module mo_exp_sol
