
      module mo_gas_phase_chemdr

      use constituents,  only : cnst_name, cnst_get_ind, ppcnst
      use constituents,  only : sflxnam, cnst_mw

      private
      public :: gas_phase_chemdr

      save

      contains

      subroutine gas_phase_chemdr( lchnk, ncol, imozart, q, qtend, &
                                   cflx, phis, zm, zi, calday, &
                                   tfld, pmid, pdel, cldfr, cldw, &
                                   cmfdqr, nevapr, prain,  cldtop, pbuf, delt )
!-----------------------------------------------------------------------
!     ... Chem_solver advances the volumetric mixing ratio
!         forward one time step via a combination of explicit,
!         ebi, hov, fully implicit, and/or rodas algorithms.
!-----------------------------------------------------------------------

      use shr_kind_mod,      only : r8 => shr_kind_r8
      use chem_mods,         only : nabscol, nfs, gas_pcnst, extcnt, rxntot, phtcnt, indexm, hetcnt, adv_mass
      use ppgrid,            only : pcols, pver
      use pmgrid,            only : masterproc
      use comsrf,            only : srfflx_state2d
      use physconst,         only : rga
      use mo_waccm_photo,    only : set_ub_col, setcol, waccm_photo
      use mo_exp_sol,        only : exp_sol
      use mo_imp_sol,        only : imp_sol
      use mo_setrxt,         only : setrxt
      use mo_adjrxt,         only : adjrxt
      use mo_phtadj,         only : phtadj
      use mo_usrrxt,         only : usrrxt
      use mo_setinv,         only : setinv
      use mo_negtrc,         only : negtrc
      use mo_sad,            only : sad_strat_calc
      use mo_sulf,           only : sulf_interp
      use mo_strato_rates,   only : ratecon_sfstrat
      use mo_srf_emissions,  only : set_srf_emissions
      use mo_strato_aerosol, only : aero_set
      use mo_aero_settling,  only : strat_aer_settling
      use mo_lightning,      only : prod_no
      use mo_setext,         only : setext
      use mo_wetdep,         only : wetdep
      use mo_drydep,         only : mkdvel, drydep
      use mo_mass_xforms,    only : mmr2vmr, vmr2mmr, h2o_to_vmr, mmr2vmri
      use phys_grid,         only : get_rlat_all_p, get_rlon_all_p, &
                                    get_lat_all_p, get_lon_all_p
      use mo_mean_mass,      only : set_mean_mass
      use history,           only : outfld
      use phys_buffer,       only : pbuf_size_max, pbuf_fld, pbuf_old_tim_idx
      use mo_chm_diags,      only : chm_diags
      use shr_orb_mod,       only : shr_orb_decl
      use charge_neutrality, only : charge_balance
      use m_spc_id
      use m_rxt_id
      use chem_mods,     only : map2chm
      use time_manager,  only: get_nstep

      implicit none

!-----------------------------------------------------------------------
!        ... dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in)  ::  lchnk                             ! chunk index
      integer, intent(in)  ::  ncol                              ! number columns in chunk
      integer, intent(in)  ::  imozart                           ! gas phase start index in q
      real(r8), intent(in) ::  delt                              ! timestep (s)
      real(r8), intent(in) ::  calday                            ! day of year
      real(r8), intent(in) ::  phis(pcols)                       ! surface geopotential
      real(r8), intent(in) ::  tfld(pcols,pver)                  ! midpoint temperature (K)
      real(r8), intent(in) ::  cldw(pcols,pver)                  ! cloud water (kg/kg)
      real(r8), intent(in) ::  pmid(pcols,pver)                  ! midpoint pressures (Pa)
      real(r8), intent(in) ::  pdel(pcols,pver)                  ! pressure delta about midpoints (Pa)
      real(r8), intent(in) ::  zm(pcols,pver)                    ! midpoint geopotential height above the surface (m)
      real(r8), intent(in) ::  zi(pcols,pver+1)                  ! interface geopotential height above the surface (m)
      real(r8), intent(in) ::  q(pcols,pver,ppcnst)              ! species concentrations (kg/kg)
      real(r8), intent(inout) ::  qtend(pcols,pver,ppcnst)       ! species tendencies (kg/kg/s)
      real(r8), intent(inout) :: cflx(pcols,gas_pcnst)           ! constituent surface flux (kg/m^2/s)

      real(r8)     ::  cldfr(pcols,pver)                              ! cloud fraction
      real(r8)     ::  cmfdqr(pcols,pver)
      real(r8)     ::  nevapr(pcols,pver)
      real(r8)     ::  prain(pcols,pver)
      real(r8)     ::  cldtop(pcols)                                  ! level index for cloud top

      type(pbuf_fld), intent(in) :: pbuf(pbuf_size_max)

!-----------------------------------------------------------------------
!     	... local variables
!-----------------------------------------------------------------------
      real(r8), parameter :: m2km  = 1.e-3_r8
      real(r8), parameter :: Pa2mb = 1.e-2_r8

      integer      ::  i, k, m, n
      integer      ::  id, kd, nstep
      integer      ::  ioff
      integer      ::  latndx(pcols)                                 ! chunk lat indicies
      integer      ::  lonndx(pcols)                                 ! chunk lon indicies
      real(r8)     ::  delt_inverse
      real(r8)     ::  esfact
      real(r8)     ::  delta
      real(r8)     ::  invariants(ncol,pver,nfs)
      real(r8)     ::  col_dens(ncol,pver,nabscol)                   ! column densities (molecules/cm^2)
      real(r8)     ::  col_delta(ncol,0:pver,nabscol)                ! layer column densities (molecules/cm^2)
      real(r8)     ::  extfrc(ncol,pver,extcnt)                      ! insitu chemical prod/loss (1/s)
      real(r8)     ::  vmr(ncol,pver,gas_pcnst)                      ! chem working concentrations (mol/mol)
      real(r8)     ::  mmr(pcols,pver,gas_pcnst)                     ! chem working concentrations (kg/kg)
      real(r8)     ::  reaction_rates(ncol,pver,rxntot)              ! reaction rates
      real(r8)     ::  depvel(ncol,gas_pcnst)                        ! dry deposition velocity (cm/s)
      real(r8)     ::  het_rates(ncol,pver,max(1,hetcnt))            ! washout rate (1/s)
      real(r8)     ::  cldice(pcols,pver)                            ! cloud water "ice" (kg/kg)
      real(r8)     ::  h2ovmr(ncol,pver)                             ! water vapor concentration (mol/mol)
      real(r8)     ::  o2mmr(ncol,pver)                              ! o2 concentration (kg/kg)
      real(r8)     ::  ommr(ncol,pver)                               ! o concentration (kg/kg)
      real(r8)     ::  mbar(ncol,pver)                               ! mean wet atmospheric mass ( amu )
      real(r8)     ::  zmid(ncol,pver)                               ! midpoint geopotential (km)
      real(r8)     ::  hno3_gas(ncol,pver)                           ! hno3 gas phase concentration (mol/mol)
      real(r8)     ::  hno3_cond(ncol,pver,3)                        ! hno3 condensed phase concentration (mol/mol)
      real(r8)     ::  h2o_gas(ncol,pver)                            ! h2o gas phase concentration (mol/mol)
      real(r8)     ::  h2o_cond(ncol,pver)                           ! h2o condensed phase concentration (mol/mol)
      real(r8)     ::  sad(ncol,pver)                                ! SAGEII surf area dens ( cm^2/cm^3 )
      real(r8)     ::  sulfate(ncol,pver)                            ! trop sulfate aerosols
      real(r8)     ::  pmb(ncol,pver)                                ! pressure at midpoints ( hPa )
      real(r8)     ::  cwat(ncol,pver)                               ! cloud water mass mixing ratio (kg/kg)
      real(r8)     ::  settl_rad(ncol,pver)
      real(r8)     ::  wrk(ncol,pver)
      real(r8)     ::  radius_strat(ncol,pver,4)                     ! radius of sulfate, nat, & ice ( cm )
      real(r8)     ::  sad_strat(ncol,pver,4)                        ! surf area density of sulfate, nat, & ice ( cm^2/cm^3 )
      real(r8)     ::  zint(ncol,pver+1)                             ! interface geopotential (km)
      real(r8)     ::  zen_angle(ncol)                               ! solar zenith angles (radians)
      real(r8)     ::  zsurf(ncol)                                   ! surface height (m)
      real(r8)     ::  rlats(ncol)                                   ! chunk latitudes (radians)
      real(r8)     ::  rlons(ncol)                                   ! chunk longitudes (radians)
      character(len=8) :: varname

      real(r8)     :: vmr_sol(ncol,pver,gas_pcnst)
!----------------------------------------------------------------------------
#include <comsol.h>

      delt_inverse = 1._r8 / delt
      nstep  = get_nstep()

!-----------------------------------------------------------------------      
!        ... get chunck latitudes and longitudes
!-----------------------------------------------------------------------      
      call get_lat_all_p( lchnk, ncol, latndx )
      call get_lon_all_p( lchnk, ncol, lonndx )
      call get_rlat_all_p( lchnk, ncol, rlats )
      call get_rlon_all_p( lchnk, ncol, rlons )
!-----------------------------------------------------------------------      
!        ... calculate cosine of zenith angle then cast back to angle
!-----------------------------------------------------------------------      
      call zenith( calday, rlats, rlons, zen_angle, ncol )
      zen_angle(:) = acos( zen_angle(:) )

!-----------------------------------------------------------------------      
!        ... map incoming concentrations to working array
!-----------------------------------------------------------------------      
      do m = 1,ppcnst
         n = map2chm(m)
         if( n > 0 ) then
            do k = 1,pver
               mmr(:ncol,k,n) = q(:ncol,k,m)
            end do
         end if
      end do

!-----------------------------------------------------------------------      
!        ... set atmospheric mean mass
!-----------------------------------------------------------------------      
      call set_mean_mass( ncol, mmr, mbar )
!-----------------------------------------------------------------------      
!        ... xform from mass to volume mixing ratios
!-----------------------------------------------------------------------      
      call mmr2vmr( mmr, vmr, mbar, ncol )

!-----------------------------------------------------------------------      
!        ... store water vapor in wrk variable
!-----------------------------------------------------------------------      
      do k = 1,pver
         h2ovmr(:ncol,k) = vmr(:ncol,k,id_h2o)
      end do
!-----------------------------------------------------------------------      
!        ... xform geopotential height from m to km 
!            and pressure from Pa to mb
!-----------------------------------------------------------------------      
      zsurf(:ncol) = rga * phis(:ncol)
      do k = 1,pver
         zmid(:ncol,k) = m2km * (zm(:ncol,k) + zsurf(:ncol))
         zint(:ncol,k) = m2km * (zi(:ncol,k) + zsurf(:ncol))
         pmb(:ncol,k)  = Pa2mb * pmid(:ncol,k)
      end do
      zint(:ncol,pver+1) = m2km * (zi(:ncol,pver+1) + zsurf(:ncol))

!-----------------------------------------------------------------------      
!        ... force ion/electron balance
!-----------------------------------------------------------------------      
      call charge_balance( ncol, vmr(:,:,:), imozart )

!-----------------------------------------------------------------------      
!        ... set the "invariants"
!-----------------------------------------------------------------------      
      call setinv( lchnk, calday, invariants, tfld, pmid, &
                   vmr, lonndx, latndx, ncol )

!-----------------------------------------------------------------------      
!        ... interpolate SAGEII data for surface area
!-----------------------------------------------------------------------      
!      call aero_set( pmid, sad, ncol, lchnk )

      call aero_set( lchnk, ncol, sad )

!-----------------------------------------------------------------------      
!        ... initialize condensed and gas phases; all hno3 to gas
!-----------------------------------------------------------------------      
      call cnst_get_ind( 'CLDICE', m )
      do k = 1,pver
         hno3_gas(:,k)   = vmr(:,k,id_hno3)
         h2o_gas(:,k)    = h2ovmr(:,k)
         wrk(:,k)        = h2ovmr(:,k)
         cldice(:ncol,k) = q(:ncol,k,m)
      end do
      do m = 1,3
         do k = 1,pver
            hno3_cond(:,k,m) = 0._r8
         end do
      end do
      call mmr2vmri( cldice, h2o_cond, mbar, cnst_mw(m), ncol )

!-----------------------------------------------------------------------      
!        ... call SAD routine
!-----------------------------------------------------------------------      
      call sad_strat_calc( lchnk, invariants(1,1,indexm), pmb, tfld, hno3_gas, &
                           hno3_cond, h2o_gas, h2o_cond, sad, radius_strat, &
                           sad_strat, latndx, lonndx, ncol )
      do k = 1,pver
         vmr(:,k,id_hno3) = hno3_gas(:,k)
         h2ovmr(:,k)      = h2o_gas(:,k)
         vmr(:,k,id_h2o)  = h2o_gas(:,k)
         wrk(:,k)         = (h2ovmr(:,k) - wrk(:,k))*delt_inverse
      end do

      do k = 1,pver
         sad_strat(:,k,2)  = sad_strat(:,k,2) + sad_strat(:,k,3)
      end do

!-----------------------------------------------------------------------      
!        ... call aerosol reaction rates
!-----------------------------------------------------------------------      
      call ratecon_sfstrat( invariants(1,1,indexm), pmid, tfld, &
                            radius_strat, sad_strat, sad_strat(1,1,2), &
                            sad_strat(1,1,4), h2ovmr, vmr, reaction_rates, ncol )

!-----------------------------------------------------------------------      
!        ... Set the column densities at the upper boundary
!-----------------------------------------------------------------------      
      call set_ub_col( col_delta, vmr, pdel, ncol )

!-----------------------------------------------------------------------      
!       ...  Set rates for "tabular" and user specified reactions
!-----------------------------------------------------------------------      
      call setrxt( reaction_rates, tfld, invariants(1,1,indexm), ncol )

      call sulf_interp( lonndx, latndx, pmid, sulfate, ncol )

      call usrrxt( reaction_rates, tfld, tfld, tfld, invariants, &
                   h2ovmr, pmid, invariants(:,:,indexm), ncol, sulfate )

      call adjrxt( reaction_rates, invariants, invariants(1,1,indexm), ncol )

!-----------------------------------------------------------------------
!        ... Compute the photolysis rates at time = t(n+1)
!-----------------------------------------------------------------------      
!-----------------------------------------------------------------------      
!     	... Set the column densities
!-----------------------------------------------------------------------      
      call setcol( col_delta, col_dens, ncol )

!-----------------------------------------------------------------------      
!     	... Calculate the photodissociation rates
!-----------------------------------------------------------------------      
      do k = 1,pver
         cwat(:,k)  = cldw(:ncol,k)
      end do
!-----------------------------------------------------------------------      
!     	... set the earth-sun distance factor
!-----------------------------------------------------------------------      
      call shr_orb_decl( calday, eccen, mvelpp, lambm0, obliqr  , &
                         delta, esfact )
      call t_startf( 'photo' )
      call waccm_photo( reaction_rates, pmid, pdel, tfld, zmid, &
                        col_dens, zen_angle, srfflx_state2d(lchnk)%asdir, cwat, cldfr, &
                        esfact, vmr, invariants, ncol, lchnk )
      call t_stopf( 'photo' )

      do k = 1,pver
         wrk(:,k) = reaction_rates(:,k,rid_jeuv_7) + reaction_rates(:,k,rid_jeuv_8) &
                  + reaction_rates(:,k,rid_jeuv_9) + reaction_rates(:,k,rid_jeuv_19) &
                  + reaction_rates(:,k,rid_jeuv_20) + reaction_rates(:,k,rid_jeuv_21) &
                  + 2._r8*(reaction_rates(:,k,rid_jeuv_12) + reaction_rates(:,k,rid_jeuv_24))
      end do

!-----------------------------------------------------------------------      
!     	... Adjust the photodissociation rates
!-----------------------------------------------------------------------      
      call phtadj( reaction_rates, invariants, invariants(:,:,indexm), ncol )

!-----------------------------------------------------------------------
!        ... Compute the extraneous frcing at time = t(n+1)
!-----------------------------------------------------------------------      
      do k = 1,pver
         o2mmr(:ncol,k) = mmr(:ncol,k,id_o2)
         ommr(:ncol,k)  = mmr(:ncol,k,id_o)
      end do
      call setext( lonndx, latndx, extfrc, zint, cldtop, &
                   zmid, lchnk, tfld, o2mmr, ommr, &
                   pmid, mbar, rlats, calday, ncol, rlons )

      do m = 1,extcnt
         do k = 1,pver
            extfrc(:ncol,k,m) = extfrc(:ncol,k,m) / invariants(:ncol,k,indexm)
         end do
      end do

!-----------------------------------------------------------------------
!        ... Form the washout rates
!-----------------------------------------------------------------------      

      call wetdep( het_rates, pmid, zmid, phis, tfld, &
                   cmfdqr, prain, nevapr, delt, invariants(1,1,indexm), &
                   vmr, ncol )

!=======================================================================
!        ... Call the class solution algorithms
!=======================================================================

!-----------------------------------------------------------------------
!	... Solve for "Explicit" species
!-----------------------------------------------------------------------
      do m=1, gas_pcnst
         write(311,*) 'nstep, m', nstep, m
         write(311,*) vmr(:ncol,:pver,m)
      enddo

      call exp_sol( vmr, reaction_rates, het_rates, extfrc, delt, ncol )

!-----------------------------------------------------------------------
!	... Solve for "Implicit" species
!-----------------------------------------------------------------------
      wrk(:,:) = vmr(:,:,id_h2o)

!      do m=1, gas_pcnst
!         write(211,*) 'nstep, m', nstep, m
!         write(211,*) vmr(:ncol,:pver,m) 
!      enddo

!      call imp_sol( vmr, vmr_sol, reaction_rates, het_rates, extfrc, delt, &
!                    ncol, lchnk )
!      vmr(:,:,:) = vmr_sol(:,:,:)

!-----------------------------------------------------------------------
!         ... Set values near upper boundary
!-----------------------------------------------------------------------

      wrk(:,:) = (vmr(:,:,id_h2o) - wrk(:,:))*delt_inverse


!-----------------------------------------------------------------------     
!         ... aerosol settling
!             first settle hno3(2) using radius ice
!             secnd settle hno3(3) using radius large nat
!-----------------------------------------------------------------------      

      wrk(:,:) = vmr(:,:,id_h2o)
#ifdef ALT_SETTL
      where( h2o_cond(:,:) > 0._r8 )
         settl_rad(:,:) = radius_strat(:,:,4)
      elsewhere
         settl_rad(:,:) = 0._r8
      endwhere
      call strat_aer_settling( invariants(1,1,indexm), pmid, delt, zmid, tfld, &
                               hno3_cond(1,1,2), settl_rad, ncol, lchnk, 1 )

      where( h2o_cond(:,:) == 0._r8 )
         settl_rad(:,:) = radius_strat(:,:,3)
      elsewhere
         settl_rad(:,:) = 0._r8
      endwhere
      call strat_aer_settling( invariants(1,1,indexm), pmid, delt, zmid, tfld, &
                               hno3_cond(1,1,3), settl_rad, ncol, lchnk, 2 )
#else
      call strat_aer_settling( invariants(1,1,indexm), pmid, delt, zmid, tfld, &
                               hno3_cond(1,1,3), radius_strat(1,1,3), ncol, lchnk, 2 )
#endif
!-----------------------------------------------------------------------      
!	... reform total hno3 = gas + all condensed
!-----------------------------------------------------------------------      
      do k = 1,pver
         vmr(:,k,id_hno3) = vmr(:,k,id_hno3) + hno3_cond(:,k,1) &
                          + hno3_cond(:,k,2) + hno3_cond(:,k,3)
      end do

!      call outfld( 'HNO3_CD1', hno3_cond, ncol ,lchnk )
!      call outfld( 'HNO3_CD2', hno3_cond(1,1,2), ncol ,lchnk )
!      call outfld( 'HNO3_CD3', hno3_cond(1,1,3), ncol ,lchnk )
!      wrk(:,:) = (vmr(:,:,id_h2o) - wrk(:,:))*delt_inverse
!      call outfld( 'QDSETT', wrk(:,:), ncol, lchnk )

!-----------------------------------------------------------------------      
!         ... Check for negative values and reset to zero
!-----------------------------------------------------------------------      
      call negtrc( 'After chemistry ', vmr, ncol )

!-----------------------------------------------------------------------      
!         ... output species concentrations( mol/mol )
!-----------------------------------------------------------------------      

      call chm_diags( lchnk, ncol, vmr, reaction_rates, invariants )

!-----------------------------------------------------------------------      
!         ... xform from volume to mass mixing ratio
!-----------------------------------------------------------------------      

      call vmr2mmr( vmr, mmr, mbar, ncol )

!-----------------------------------------------------------------------      
!         ... form the tendencies
!-----------------------------------------------------------------------      
      do m = 1,ppcnst
         n = map2chm(m)
         if( n > 0 ) then
            do k = 1,pver
               qtend(:ncol,k,m) = (mmr(:ncol,k,n) - q(:ncol,k,m))*delt_inverse
            end do
         end if
      end do

!-----------------------------------------------------------------------      
!	... set surface emissions
!-----------------------------------------------------------------------      
      call set_srf_emissions( latndx, lonndx, cflx, ncol )
      call cnst_get_ind( 'CO', m, abort=.false. )

      call cnst_get_ind( 'NO', m, abort=.false. )

      call mkdvel( latndx, lonndx, srfflx_state2d(lchnk)%ts, zen_angle, depvel, &
                   ncol, lchnk )
      call drydep( depvel, mmr, pmid(1,pver), srfflx_state2d(lchnk)%ts, cflx, ncol )
 
      end subroutine gas_phase_chemdr

      end module mo_gas_phase_chemdr
