
#include <misc.h>

      module mo_lightning

      use shr_kind_mod,  only : r8 => shr_kind_r8
      use shr_const_mod, only : pi => shr_const_pi
      use ppgrid,        only : begchunk, endchunk, pcols, pver
      use dycore,        only : get_resolution
      use abortutils,    only : endrun

      implicit none

      private
      public  :: lightning_inti, lightning_no_prod, prod_no, factor

      save

      real(r8) :: csrf, lat25
!----------------------------------------------------------------------
!	Set global lightning NOx scaling factor
!----------------------------------------------------------------------
      real(r8) :: factor = .4_r8                                    ! user-controlled scaling factor to achieve arbitrary NO prod.
      real(r8) :: vdist(16,3)                                       ! vertical distribution of lightning
      real(r8), allocatable :: prod_no(:,:,:)
      real(r8), allocatable :: prod_no_col(:,:)
      real(r8), allocatable :: flash_freq(:,:)

      contains

      subroutine lightning_inti
!----------------------------------------------------------------------
!       ... Initialize the chemistry "hook" routine
!----------------------------------------------------------------------

      use dynconst, only : rearth
      use pmgrid,   only : plon, plat

      implicit none

!----------------------------------------------------------------------
!       ... Local variables
!----------------------------------------------------------------------
      integer  :: astat
      real(r8) :: twopi

!wtw      select case( get_resolution() )
!wtw         case( '1.9x2.5' )
!wtw            factor = .38_r8
!wtw         case( '2x2.5' )
!wtw            factor = .4_r8
!wtw         case( '4x5' )
!wtw            factor = 1.6_r8
!wtw         case( '10x15' )
!wtw            factor = 11._r8
!wtw         case default
!wtw            write(*,*) 'lightning_inti: unknown resolution'
!wtw            call endrun
!wtw      end select
      
         select case( get_resolution() )
         case( 'T42' )
            factor = .6_r8
         case( 'T106' )
            factor = .1_r8
         case default
            write(*,*) 'lightning_inti: unknown resolution'
            call endrun
         end select

      write(*,*) 'lightning_inti: Lightning no production scaling factor = ',factor

      twopi = 2._r8 * pi
      lat25 = twopi * 25._r8 / 360._r8
      csrf  = twopi*rearth*rearth/real(plon)                           ! rearth in m
      allocate( prod_no(pcols,pver,begchunk:endchunk),stat=astat )
      if( astat /= 0 ) then
	 write(*,*) 'lightning_inti : failed to allocate prod_no ; error = ',astat
	 call endrun
      end if
      allocate( prod_no_col(pcols,begchunk:endchunk),stat=astat )
      if( astat /= 0 ) then
	 write(*,*) 'lightning_inti : failed to allocate prod_no_col ; error = ',astat
	 call endrun
      end if
      allocate( flash_freq(pcols,begchunk:endchunk),stat=astat )
      if( astat /= 0 ) then
	 write(*,*) 'lightning_inti : failed to allocate flash_freq ; error = ',astat
	 call endrun
      end if
!----------------------------------------------------------------------
!       ... vdist(kk,itype) = % of lightning NOx between (kk-1) and (kk)
!           km for profile itype
!----------------------------------------------------------------------
      vdist(:,1) = (/ 20.1_r8, 2.3_r8, 0.8_r8, 1.5_r8, 3.4_r8, 5.3_r8, 3.6_r8, 3.8_r8, &       ! midlat cont
                       5.4_r8, 6.6_r8, 8.3_r8, 9.6_r8,12.8_r8,10.0_r8, 6.2_r8, 0.3_r8 /)
      vdist(:,2) = (/  5.8_r8, 2.9_r8, 2.6_r8, 2.4_r8, 2.2_r8, 2.1_r8, 2.3_r8, 6.1_r8, &       ! trop marine
                      16.5_r8,14.1_r8,13.7_r8,12.8_r8,12.5_r8, 2.8_r8, 0.9_r8, 0.3_r8 /)
      vdist(:,3) = (/  8.2_r8, 1.9_r8, 2.1_r8, 1.6_r8, 1.1_r8, 1.6_r8, 3.0_r8, 5.8_r8, &       ! trop cont
                       7.6_r8, 9.6_r8,10.5_r8,12.3_r8,11.8_r8,12.5_r8, 8.1_r8, 2.3_r8 /)

      end subroutine lightning_inti

      subroutine lightning_no_prod( state, pbuf ,cldtop, cldbot )
!----------------------------------------------------------------------
!    	... Calculate lightning NOx source
!----------------------------------------------------------------------

      use physconst,     only : rga
      use comsrf,        only : landfrac, ocnfrac
      use phys_grid,     only : get_rlat_all_p, get_lat_all_p
      use physics_types, only : physics_state
      use commap,        only : latwts =>  w
      use pmgrid,        only : masterproc
      use time_manager,  only : is_end_curr_day
      use phys_buffer,   only : pbuf_size_max, pbuf_fld, pbuf_get_fld_idx
#ifdef SPMD
      use mpishorthand,  only : mpicom
#endif

      implicit none

!----------------------------------------------------------------------
!	... Dummy args
!----------------------------------------------------------------------
      type(physics_state), intent(in) :: state(begchunk:endchunk) ! physics state
      type(pbuf_fld), intent(in)      :: pbuf(pbuf_size_max)      ! physics buffer

!----------------------------------------------------------------------
!	... Local variables
!----------------------------------------------------------------------
      integer, parameter     :: land = 1, ocean = 0
      real(r8), parameter    :: secpyr = 365._r8 * 8.64e4_r8
      real(r8), parameter    :: m2km = 1.e-3_r8, km2cm = 1.e5_r8

      integer  :: c, i, istat, &
                  cldtind, &                              ! level index for cloud top
                  cldbind, &                              ! level index for cloud base > 273K
                  file, &                                 ! history file index
                  surf_type
      integer  :: lchnk                                   ! chunk index
      integer  :: ncol                                    ! columns per chunk
      integer  :: k, kk, zlow_ind, zhigh_ind, itype
      integer  :: im1, ipm1, ipp1, lev_avrg
      integer  :: cldtop_ndx, cldbot_ndx
      integer  :: latndx(pcols,begchunk:endchunk)
      real(r8) :: cldtop(pcols)                           ! cloud top level index
      real(r8) :: cldbot(pcols)                           ! cloud bottom level index
      real(r8) :: glob_flashfreq, &                       ! global flash frequency [s-1]
                  glob_noprod, &                          ! global rate of NO production [as TgN/yr]
                  frac_sum, &                             ! fraction sum for mpi
                  wrk, wrk1                               ! work variable
      real(r8) :: zlow, zhigh, zlow_scal, zhigh_scal, fraction
      real(r8) :: sflux1, sflux2, sumoc, sumln
      real(r8) :: flash_freq_land, flash_freq_ocn
      real(r8) :: total_flux
      real(r8), dimension(pcols,begchunk:endchunk) :: &
                 dchgzone, &                              ! depth of discharge zone [km]
                 cldhgt, &                                ! cloud top height [km]
                 cgic, &                                  ! Cloud-Ground/Intracloud discharge ratio
                 flash_energy, &                          ! Energy of flashes per second
                 glob_prod_no_col                         ! Global NO production rate for diagnostics
      real(r8) :: zmid(pcols,pver)                        ! geopot height above surface at midpoints (m)
      real(r8) :: zint(pcols,pver+1,begchunk:endchunk)    ! geopot height above surface at interfaces (m)
      real(r8) :: zsurf(pcols)                            ! geopot height above surface at interfaces (m)
      real(r8) :: rlats(pcols,begchunk:endchunk)          ! column latitudes in chunks
!----------------------------------------------------------------------
! 	... Parameters to determine CG/IC ratio [Price and Rind, 1993]
!----------------------------------------------------------------------
      real(r8), parameter  :: ca = .021_r8, cb = -.648_r8, cc = 7.49_r8, cd = -36.54_r8, ce = 64.09_r8

#ifdef SPMD
#include <mpif.h>
#endif

!----------------------------------------------------------------------
!	Lightning NO production : Initialize ...
!----------------------------------------------------------------------
      flash_freq(:,:)       = 0._r8
      prod_no(:,:,:)        = 0._r8
      prod_no_col(:,:)      = 0._r8
      cldhgt(:,:)           = 0._r8
      dchgzone(:,:)         = 0._r8
      cgic(:,:)             = 0._r8
      flash_energy(:,:)     = 0._r8
      glob_prod_no_col(:,:) = 0._r8

!      cldtop_ndx = pbuf_get_fld_idx( 'CLDTOP' )
!      cldbot_ndx = pbuf_get_fld_idx( 'CLDBOT' )

!--------------------------------------------------------------------------------
!	... Estimate flash frequency and resulting NO emissions
!           [Price, Penner, Prather, 1997 (JGR)]
!    Lightning only occurs in convective clouds with a discharge zone, i.e.
!    an altitude range where liquid water, ice crystals, and graupel coexist.
!    We test this by examining the temperature at the cloud base.
!    It is assumed that only one thunderstorm occurs per grid box, and its
!    flash frequency is determined by the maximum cloud top height (not the
!    depth of the discharge zone). This is somewhat speculative but yields
!    reasonable results.
!
!       The CG/IC ratio is determined by an empirical formula from Price and
!    Rind [1993]. The average energy of a CG flash is estimated as 6.7e9 J,
!    and the average energy of a IC flash is assumed to be 1/10 of that value.
!       The NO production rate is assumed proportional to the discharge energy
!    with 1e17 N atoms per J. The total number of N atoms is then distributed
!    over the complete column of grid boxes.
!--------------------------------------------------------------------------------
Chunk_loop : &
      do c = begchunk,endchunk
	 ncol  = state(c)%ncol
	 lchnk = state(c)%lchnk
!	 cldtop(:ncol) = pbuf(cldtop_ndx)%fld_ptr(1,1:ncol,1,lchnk,1)
!	 cldbot(:ncol) = pbuf(cldbot_ndx)%fld_ptr(1,1:ncol,1,lchnk,1)
         zsurf(:ncol) = state(c)%phis(:ncol)*rga
	 call get_rlat_all_p( c, ncol, rlats(1,c) )
	 call get_lat_all_p( c, ncol, latndx(1,c) )
	 do k = 1,pver
	    zmid(:ncol,k)   = state(c)%zm(:ncol,k) + zsurf(:ncol)
	    zint(:ncol,k,c) = state(c)%zi(:ncol,k) + zsurf(:ncol)
	 end do
	 zint(:ncol,pver+1,c) = state(c)%zi(:ncol,pver+1) + zsurf(:ncol)
Column_loop : &
         do i = 1,ncol
!--------------------------------------------------------------------------------
! 	... Find cloud top and bottom level above 273K
!--------------------------------------------------------------------------------
	    cldtind = nint( cldtop(i) )
            cldbind = nint( cldbot(i) )
            do
               if( cldbind <= cldtind .or. state(c)%t(i,cldbind) < 273._r8 ) then
		  exit
	       end if
               cldbind = cldbind - 1
            end do
	    if( cldtind < pver .and. cldtind > 0 .and. cldtind < cldbind ) then
!--------------------------------------------------------------------------------
!       ... Compute cloud top height and depth of charging zone
!--------------------------------------------------------------------------------
	       cldhgt(i,c)   = m2km*max( 0._r8,zint(i,cldtind,c) )
               dchgzone(i,c) = cldhgt(i,c) - m2km*zmid(i,cldbind)
!--------------------------------------------------------------------------------
!       ... Compute flash frequency for given cloud top height
!           (flashes storm^-1 min^-1)
!--------------------------------------------------------------------------------
	       flash_freq_land = 3.44e-5_r8 * cldhgt(i,c)**4.9_r8 
	       flash_freq_ocn  = 6.40e-4_r8 * cldhgt(i,c)**1.7_r8
	       flash_freq(i,c) = landfrac(i,c)*flash_freq_land + ocnfrac(i,c)*flash_freq_ocn
!--------------------------------------------------------------------------------
!       ... Compute CG/IC ratio
!           cgic = proportion of CG flashes (=PG from PPP paper)
!--------------------------------------------------------------------------------
               cgic(i,c) = 1._r8/((((ca*dchgzone(i,c) + cb)*dchgzone(i,c) + cc)*dchgzone(i,c) + cd)*dchgzone(i,c) + ce)
               if( dchgzone(i,c) < 5.5_r8 ) then
		  cgic(i,c) = 0._r8
               else if( dchgzone(i,c) > 14._r8 ) then
		  cgic(i,c) = .02_r8
	       end if
!--------------------------------------------------------------------------------
!       ... Compute flash energy (CG*6.7e9 + IC*6.7e8)
!           and convert to total energy per second
!--------------------------------------------------------------------------------
               flash_energy(i,c) = cgic(i,c)*6.7e9_r8 + (1._r8 - cgic(i,c))*6.7e8_r8
               flash_energy(i,c) = flash_energy(i,c)*flash_freq(i,c)/60._r8

!--------------------------------------------------------------------------------
! 	... Compute number of N atoms produced per second
!           and convert to N atoms per second per cm2 and apply fudge factor
!--------------------------------------------------------------------------------
               prod_no_col(i,c) = 1.e17_r8*flash_energy(i,c)/(1.e4_r8*csrf*latwts(latndx(i,c))) * factor
!--------------------------------------------------------------------------------
! 	... Compute global NO production rate in TgN/yr:
!           TgN per second: * 14.00674 * 1.65979e-24 * 1.e-12
!             NB: 1.65979e-24 = 1/AVO
!           TgN per year: * secpyr
!--------------------------------------------------------------------------------
               glob_prod_no_col(i,c) = 1.e17_r8*flash_energy(i,c) &
                                               * 14.00674_r8 * 1.65979e-24_r8 * 1.e-12_r8 * secpyr * factor
	    end if
         end do Column_loop
      end do Chunk_loop

!--------------------------------------------------------------------------------
! 	... Accumulate global total, convert to flashes per second
!--------------------------------------------------------------------------------
      glob_flashfreq = sum( flash_freq(:,:) )/60._r8

!--------------------------------------------------------------------------------
! 	... Accumulate global NO production rate
!--------------------------------------------------------------------------------
      glob_noprod = sum( glob_prod_no_col(:,:) )

!--------------------------------------------------------------------------------
!	... Gather,scatter global sum
!--------------------------------------------------------------------------------
#ifdef SPMD
      call mpi_allreduce( glob_flashfreq, wrk, 1, mpi_double_precision, mpi_sum, mpicom, istat )
      if( istat /= MPI_SUCCESS ) then
         write(*,*) 'wetdep: MPI_ALLREDUCE for flashfreq failed; error = ',istat
	 call endrun
      end if
      glob_flashfreq = wrk
      call mpi_allreduce( glob_noprod, wrk, 1, mpi_double_precision, mpi_sum, mpicom, istat )
      if( istat /= MPI_SUCCESS ) then
         write(*,*) 'MOZ_HOOK: MPI_ALLREDUCE for noprod failed; error = ',istat
	 call endrun
      end if
      glob_noprod = wrk
#endif

      if( masterproc ) then
         if( is_end_curr_day() ) then
            write(*,'(''Global flash freq (/s), lightning NOx (TgN/y) = '',1p2g15.7)') &
                      glob_flashfreq, glob_noprod
         end if
      end if

      if( glob_noprod > 0._r8 ) then
!--------------------------------------------------------------------------------
!	... Distribute production up to cloud top [Pickering et al., 1998 (JGR)]
!--------------------------------------------------------------------------------
         do c = begchunk,endchunk
	    ncol  = state(c)%ncol
	    lchnk = state(c)%lchnk
!	    cldtop(:ncol) = pbuf(cldtop_ndx)%fld_ptr(1,1:ncol,1,lchnk,1)
	    do i = 1,ncol
	       cldtind = nint( cldtop(i) )
 	       if( prod_no_col(i,c) > 0._r8 ) then
	          if( cldhgt(i,c) > 0._r8 ) then
                     if( abs( rlats(i,c) ) > lat25 ) then
                        itype = 1                                                    ! midlatitude continental
                     else if( nint( landfrac(i,c) ) == land ) then
                        itype = 3                                                    ! tropical continental
                     else
                        itype = 2                                                    ! topical marine
                     end if
                     frac_sum = 0._r8
                     do k = cldtind,pver
                        zlow       = zint(i,k+1,c) * m2km                            ! lower interface height (km)
                        zlow_scal  = zlow * 16._r8/cldhgt(i,c)                       ! scale to 16 km convection height
                        zlow_ind   = max( 1,INT(zlow_scal)+1 )                       ! lowest vdist index to include in layer
                        zhigh      = zint(i,k,c) * m2km                              ! upper interface height (km)
                        zhigh_scal = zhigh * 16._r8/cldhgt(i,c)                      ! height (km) scaled to 16km convection height
                        zhigh_ind  = max( 1,MIN( 16,INT(zhigh_scal)+1 ) )            ! highest vdist index to include in layer
                        do kk = zlow_ind,zhigh_ind
			   wrk  = kk
			   wrk1 = kk - 1
                           fraction = min( zhigh_scal,wrk ) &                         ! fraction of vdist in this model layer
                                      - max( zlow_scal,wrk1 )
                           fraction = max( 0._r8, min( 1._r8,fraction ) )
                           frac_sum = frac_sum + fraction*vdist(kk,itype)
                           prod_no(i,k,c) = prod_no(i,k,c) &                         ! sum the fraction of column NOx in layer k
                                            + fraction*vdist(kk,itype)*.01_r8
                        end do
			prod_no(i,k,c) = prod_no_col(i,c) * prod_no(i,k,c) &         ! multiply fraction by column amount
                                         / (km2cm*(zhigh - zlow))                    ! and convert to atom N cm^-3 s^-1
                     end do
	          end if
	       end if
            end do
         end do
      end if

!--------------------------------------------------------------------------------
!       ... Output lightning no production to history file
!--------------------------------------------------------------------------------
!     do file = 1,sim_file_cnt
!        do ip = 1,pplon
!           do j = 1,platl
!              call OUTFLD( 'LNO_PROD', glob_prod_no_col(:,j,ip), plonl, ip, j, file )
!              call OUTFLD( 'FLASHFRQ', flash_freq(:,j,ip), plonl, ip, j, file )
!              call OUTFLD( 'CLDHGT', cldhgt(:,j,ip), plonl, ip, j, file )
!              call OUTFLD( 'DCHGZONE', dchgzone(:,j,ip), plonl, ip, j, file )
!              call OUTFLD( 'CGIC', cgic(:,j,ip), plonl, ip, j, file )
!           end do
!        end do
!     end do

      end subroutine lightning_no_prod

      end module mo_lightning
