
      module mo_read_chm_sim

      private
      public :: read_chm_sim

      contains

      subroutine read_chm_sim( sim_data_flsp )
!--------------------------------------------------------
!    	... Initialize chemistry modules
!--------------------------------------------------------

      use mo_tracname, only : solsym
      use chem_mods,   only : clscnt, cls_rxt_cnt, clsmap, &
			      adv_mass, permute, diag_map, gas_pcnst
      use units, only : getunit, freeunit

      use abortutils, only: endrun

      implicit none


!--------------------------------------------------------
!    	... Dummy args
!--------------------------------------------------------
      character(len=*), intent(in) :: sim_data_flsp

!--------------------------------------------------------
!    	... Local variables
!--------------------------------------------------------
      integer  ::  i, k, ios, astat, funit

      funit = getunit()
!--------------------------------------------------------
!    	... Open chem input unit
!--------------------------------------------------------
      open( unit = funit, &
            file = trim( sim_data_flsp ), &
            status = 'old', &
            recl   = 2048, &
            iostat = ios )
      if( ios /= 0 ) then
         write(*,*) ' read_chm_sim: Failed to open file ',trim( sim_data_flsp )
         write(*,*) ' Error code = ',ios
         call endrun
      end if

!--------------------------------------------------------
!        ... Read map info from data file
!--------------------------------------------------------
      read(funit,'(5i4)',iostat=ios) clscnt
      if( ios /= 0 ) then
         write(*,*) ' read_chm_sim: Failed to read clscnt; error = ',ios
         call endrun
      end if
      read(funit,'(4i4)',iostat=ios) cls_rxt_cnt
      if( ios /= 0 ) then
         write(*,*) ' read_chm_sim: Failed to read cls_rxt_cnt; error = ',ios
         call endrun
      end if
      do k = 1,5
	 if( clscnt(k) /= 0 ) then
	    read(funit,'(20i4)',iostat=ios) clsmap(:clscnt(k),k)
            if( ios /= 0 ) then
               write(*,*) ' read_chm_sim: Failed to read clsmap; error = ',ios
               call endrun
            end if
         end if
      end do
      read(funit,*,iostat=ios) adv_mass(:gas_pcnst)
      if( ios /= 0 ) then
         write(*,*) ' read_chm_sim: Failed to read adv_mass; error = ',ios
         call endrun
      end if
      read(funit,'(10a8)',iostat=ios) solsym(:gas_pcnst)
      if( ios /= 0 ) then
         write(*,*) ' read_chm_sim: Failed to read solsym; error = ',ios
         call endrun
      end if
      do i = 2,5
	 if( clscnt(i) /= 0 ) then
	    read(funit,'(20i4)',iostat=ios) permute(:clscnt(i),i)
            if( ios /= 0 ) then
               write(*,*) ' read_chm_sim: Failed to read permute; error = ',ios
               call endrun
            end if
	    if( i > 3 ) then
	       read(funit,'(20i4)',iostat=ios) diag_map(:clscnt(i))
               if( ios /= 0 ) then
                  write(*,*) ' read_chm_sim: Failed to read diag_map; error = ',ios
                  call endrun
               end if
	       exit
	    end if
	 end if
      end do

      close( funit )
      call freeunit( funit )

      end subroutine read_chm_sim

      end module mo_read_chm_sim
