
      module mo_setinv

      private
      public :: setinv

      contains

      subroutine setinv( lchnk, calday, invariants, tfld, pmid, &
                         vmr, lonndx, latndx, ncol )
!-----------------------------------------------------------------
!        ... Set the invariant densities (molecules/cm**3)
!-----------------------------------------------------------------
      
      use shr_kind_mod,  only : r8 => shr_kind_r8
      use ppgrid,        only : pcols, pver
      use m_spc_id,      only : id_o2, id_o, id_h
      use chem_mods,     only : nfs, gas_pcnst

      implicit none

!-----------------------------------------------------------------
!        ... Dummy arguments
!-----------------------------------------------------------------
      integer, intent(in)   ::      lchnk                      ! chunk id
      integer, intent(in)   ::      ncol                       ! chunk column count
      integer, intent(in)   ::      lonndx(pcols)              ! chunk longitude indicies
      integer, intent(in)   ::      latndx(pcols)              ! chunk latitude  indicies
      real(r8), intent(in)  ::      calday                     ! calendar day of year
      real(r8), intent(in)  ::      tfld(pcols,pver)           ! temperature (K)
      real(r8), intent(in)  ::      pmid(pcols,pver)           ! pressure (Pa)
      real(r8), intent(in)  ::      vmr(ncol,pver,gas_pcnst)   ! vmr
      real(r8), intent(out) ::      invariants(ncol,pver,nfs)  ! invariant array (1/cm^3)

!-----------------------------------------------------------------
!        .. Local variables
!-----------------------------------------------------------------
      real(r8), parameter ::  boltz   = 1.38044e-16_r8         ! erg/K
      real(r8), parameter ::  Pa_xfac = 10._r8                 ! Pascals to dyne/cm^2

      integer  :: k
      real(r8) :: sum1(ncol)
      real(r8) :: n2d_tmp(ncol,pver)

!-----------------------------------------------------------------
!        NOTE: Invariants are in cgs density units.
!              The pmid array is in pascals and must be
!	       mutiplied by 10. to yield dynes/cm**2.
!-----------------------------------------------------------------
!-----------------------------------------------------------------
!	... Set M, N2
!-----------------------------------------------------------------
      do k = 1,pver
         invariants(:ncol,k,1) = Pa_xfac * pmid(:ncol,k) / (boltz*tfld(:ncol,k))
	 sum1(:)               = (vmr(:,k,id_o) + vmr(:,k,id_o2) + vmr(:,k,id_h))
         invariants(:,k,2)     = (1._r8 - sum1(:)) * invariants(:,k,1)
      end do

      end subroutine setinv

      end module mo_setinv
