#include <misc.h>
#include <params.h>
 module mo_tracinp

!-----------------------------------------------------------------------
! 	... Tracer initial conditions
!-----------------------------------------------------------------------
      use chem_mods  ,     only : gas_pcnst
      use pmgrid,          only : plon, plat, plev
      use mo_netcdf
      use error_messages,  only : handle_ncerr

      implicit none

      save

      public  :: initrac

      real , public                :: ic_vmr_int(plon,plat,plev,gas_pcnst)   
      character(len=256) , public  :: bndic

      integer :: ncid       ! netcdf file id

      CONTAINS

      subroutine initrac
!-----------------------------------------------------------------------
! 	... Initialize /tracinp/, and tracer arrays.   The tracer fields
!           must come directly from a time sample that exists on the input
!           file (though not necessarily the first time sample).
!-----------------------------------------------------------------------


!add by zf 2008.06.13
      use pmgrid,         only: masterproc
      use ioFileMod,      only: getfil
      use abortutils,     only: endrun
!end zf

      implicit none

!-----------------------------------------------------------------------
! 	... Local variables
!-----------------------------------------------------------------------
      integer :: astat,dimid, varid,i, j, k, m

      character(len=256) :: locfn    ! netcdf local filename to open   add by zf

!-----------------------------------------------------------------------
!     	... Open netcdf ic file
!-----------------------------------------------------------------------

    if (masterproc) then
      call getfil (bndic, locfn)
      call wrap_open(locfn,0,ncid)

!-----------------------------------------------------------------------
!   	... Allocate memory for ic and ic surface pressure
!-----------------------------------------------------------------------
      call GETTRAC( ic_vmr_int )

!--------------------------------------------------------------------------
!  	... Close file
!--------------------------------------------------------------------------
      call HANDLE_NCERR( NF_CLOSE( ncid ), 'INITRAC: Failed to close file ' // bndic  )

      end if          !  masterproc

      end subroutine INITRAC

!---------------------------------------------------
      subroutine GETTRAC( ic_vmr )
!-----------------------------------------------------------------------
! 	... Read constituent data from netcdf intial condition file
!-----------------------------------------------------------------------

      use netcdf
      use chem_mods,     only : gas_pcnst
      use mo_tracname,   only : solsym
      use pmgrid,        only : masterproc     !add by zf 2008.06.13
 
      implicit none

!-----------------------------------------------------------------------
! 	... Dummy arguments
!-----------------------------------------------------------------------
      real, intent(out) :: ic_vmr(plon,plat,plev,gas_pcnst)     ! ic initial concentrations

!-----------------------------------------------------------------------
! 	... Local variables
!-----------------------------------------------------------------------
      integer          ::  i,j, k, m              ! indicies
      integer          ::  varid                ! variable id
      integer          ::  ncret                ! netcdf return code
      integer          ::  start3(4), count3(4)
      character(len=8) ::  fldname

    if (masterproc) then
      start3(:) = (/ 1,1,1,1 /)
      count3(:) = (/ plon,plat,plev,1/)

!-----------------------------------------------------------------------
! 	... Get transported variables
!-----------------------------------------------------------------------
      do m = 1, gas_pcnst
         fldname = solsym (m)

         ic_vmr(:,:,:,m) = 1.e-38

         ncret = NF_INQ_VARID( ncid, TRIM( fldname ), varid )
         if( ncret /= NF_NOERR ) then
            write(*,*) 'GETTRAC: Failed to get variable id for ' // TRIM( fldname ) &
                          // ', will initialize with 10^-38'
            ic_vmr(:,:,:,m) = 1.e-38
         else
            call HANDLE_NCERR( NF_GET_VARA_DOUBLE( ncid, varid, start3, count3, ic_vmr(1,1,1,m) ), &
                            'GETTRAC: Failed to get variable ' // TRIM( fldname ) )
            write(*,*) 'GETTRAC: Loaded variable ' // TRIM( fldname ) // '.'
!--------------
            do i=1, plon
               do j=1, plat
                    ic_vmr(i,j,plev,m) = ic_vmr(i,j,plev-1,m) 
               enddo
            enddo
!-------------------         
         end if
         
   

      end do

      end if          !  masterproc

      end subroutine GETTRAC

end module mo_tracinp
