
      module mo_usrrxt

      use shr_kind_mod,  only : r8 => shr_kind_r8

      private
      public :: usrrxt, usrrxt_hrates

      real(r8), parameter :: t0     = 300._r8                ! K
      real(r8), parameter :: trlim2 = 17._r8/3._r8           ! K
      real(r8), parameter :: trlim3 = 15._r8/3._r8           ! K

      contains

      subroutine usrrxt( rxt, tempn, tempi, tempe, invariants, &
			 h2ovmr, pmid, m, ncol, sulfate )
!-----------------------------------------------------------------
!       ... set the user specified reaction rates
!-----------------------------------------------------------------

      use m_rxt_id
      use chem_mods,     only : nfs, rxntot, indexm
      use ppgrid,        only : pver, pcols
      use wv_saturation, only : aqsat
      use shr_const_mod, only : pi => shr_const_pi

      implicit none

!-----------------------------------------------------------------
!       ... dummy arguments
!-----------------------------------------------------------------
      integer, intent(in)  :: ncol                         ! number columns in chunk
      real(r8), intent(in) :: tempn(pcols,pver), &         ! neutral temperature (K)
                              tempi(pcols,pver), &         ! ion temperature (K)
                              tempe(pcols,pver), &         ! electron temperature (K)
                              m(ncol,pver), &              ! total atm density (1/cm^3)
                              h2ovmr(ncol,pver), &         ! water vapor (vmr)
                              pmid(pcols,pver), &          ! midpoint pressure (Pa)
                              invariants(ncol,pver,nfs)    ! invariants density (1/cm^3)
      real(r8), optional, intent(in) :: &
                              sulfate(ncol,pver)           ! trop sulfate aerosol
      real(r8), intent(inout) ::  rxt(ncol,pver,rxntot)    ! gas phase rates
      
!-----------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------
      real(r8), parameter :: boltz  = 1.38044e-16_r8         ! erg / K
      real(r8), parameter :: avo    = 6.023e23_r8            ! molecules/mole
!-----------------------------------------------------------------
!	... density of sulfate aerosol
!-----------------------------------------------------------------
      real(r8), parameter :: gam1 = 0.04_r8                 ! N2O5+SUL ->2HNO3
      real(r8), parameter :: wso4 = 98._r8
      real(r8), parameter :: den  = 1.15_r8                 ! each molecule of SO4(aer) density g/cm3
!-------------------------------------------------
! 	... volume of sulfate particles
!           assuming mean rm 
!           continient 0.05um  0.07um  0.09um
!           ocean      0.09um  0.25um  0.37um
!                      0.16um                  Blake JGR,7195, 1995
!-------------------------------------------------
      real(r8), parameter :: rm1  = 0.16_r8*1.e-4_r8             ! mean radii in cm

      integer  ::  k
      real(r8) ::  amas, fare
      real(r8), dimension(ncol) :: &
                   tp, &
                   tinv, &
                   ko, &
                   kinf, &
                   fc, &
                   xr, &                    ! factor to increase particle radii depending on rel hum
                   sur                      ! sulfate particle surface area (cm^2/cm^3)
      real(r8), dimension(pcols) :: &
                   relhum, &                ! relative humidity
                   satq, &                  ! saturation specific humidity
                   satv                     ! saturation vapor pressure

      fare = 4._r8*pi*rm1*rm1               ! each mean particle(r=0.1u) area   cm2/cm3
!-----------------------------------------------------------------
!	... o + o2 + m --> o3 + m
!-----------------------------------------------------------------
level_loop1 : &
      do k = 1,pver
         tinv(:ncol)       = 1._r8 / tempn(:ncol,k)
         tp(:)             = t0 * tinv(:)
         rxt(:,k,rid_usr1) = 6.e-34_r8 * tp(:)**2.4_r8

!-----------------------------------------------------------------
!	... o + o + m -> o2 + m
!-----------------------------------------------------------------
         rxt(:ncol,k,rid_usr2) = 4.23e-28_r8 * tempn(:ncol,k)**(-2)

!-----------------------------------------------------------------
!	... n2o5 + m --> no2 + no3 + m
!-----------------------------------------------------------------
         rxt(:,k,rid_usr3a) = rxt(:,k,rid_usr3) * 3.333e26_r8 * exp( -10991._r8*tinv(:) )

!-----------------------------------------------------------------
!	set rates for:
! 	... hno3 + oh --> no3 + h2o
!           ho2no2 + m --> ho2 + no2 + m
!           co + oh --> co2 + ho2
!-----------------------------------------------------------------
         ko(:) = m(:,k) * 6.5e-34_r8 * exp( 1335._r8*tinv(:) )
         ko(:) = ko(:) / (1._r8 + ko(:)/(2.7e-17_r8*exp( 2199._r8*tinv(:) )))
         rxt(:,k,rid_usr4)  = ko(:) + 2.4e-14_r8*exp( 460._r8*tinv(:) )
         rxt(:,k,rid_usr5a) = rxt(:,k,rid_usr5) * exp( -10900._r8*tinv(:) ) / 2.1e-27_r8
         rxt(:ncol,k,rid_usr6)  = 1.5e-13_r8 * (1._r8 + 6.e-7_r8*boltz*m(:ncol,k)*tempn(:ncol,k))

!-----------------------------------------------------------------
!	... ho2 + ho2 --> h2o2
!	Note: this rate involves the water vapor number density
!-----------------------------------------------------------------
         ko(:)   = 2.3e-13_r8 * exp( 600._r8*tinv(:) )
         kinf(:) = 1.7e-33_r8 * m(:,k) * exp( 1000._r8*tinv(:) )
         fc(:)   = 1._r8 + 1.4e-21_r8 * m(:,k) * h2ovmr(:,k) * exp( 2200._r8*tinv(:) )
         rxt(:,k,rid_usr7) = (ko(:) + kinf(:)) * fc(:)

!-----------------------------------------------------------------
! 	... cl2o2 + m -> 2*clo + m
!-----------------------------------------------------------------
         ko(:)            = 1.3e-27_r8 * exp( 8744._r8* tinv(:) )
         rxt(:,k,rid_usr8a) = rxt(:,k,rid_usr8)/ko(:)
      end do level_loop1

!-----------------------------------------------------------------
! 	... the ionic rates
!-----------------------------------------------------------------
level_loop2 : &
      do k = 1,pver
         tp(:ncol)         = (2._r8*tempi(:ncol,k) + tempn(:ncol,k)) / ( 3._r8 * t0 )
	 tp(:)             = max( min( tp(:),20._r8 ),1._r8 )
         rxt(:,k,rid_ion1) = 2.82e-11_r8 + tp(:)*(-7.74e-12_r8 + tp(:)*(1.073e-12_r8  &
					 + tp(:)*(-5.17e-14_r8 + 9.65e-16_r8*tp(:))))
         tp(:ncol)         = (.6363_r8*tempi(:ncol,k) + .3637_r8*tempn(:ncol,k)) / t0
	 tp(:)             = max( min( tp(:),trlim2 ),1._r8 )
         rxt(:,k,rid_ion2) = 1.533e-12_r8 + tp(:)*(-5.92e-13_r8 + tp(:)*8.6e-14_r8)
         tp(:ncol)         = 2._r8 * t0 /(tempi(:ncol,k) + tempn(:ncol,k))
	 where( tp(:ncol) < trlim3 )
            rxt(:,k,rid_ion3)  = 1.4e-10_r8 * tp(:)**.44_r8
            rxt(:,k,rid_ion11) = 1.e-11_r8 * tp(:)**.23_r8
	 elsewhere
            rxt(:,k,rid_ion3)  = 5.2e-11_r8 / tp(:)**.2_r8
            rxt(:,k,rid_ion11) = 3.6e-12_r8 / tp(:)**.41_r8
	 endwhere
         tp(:ncol)          = t0 / tempe(:ncol,k)
         rxt(:,k,rid_elec1) = 4.e-7_r8 * tp(:)**.85_r8
         rxt(:,k,rid_elec3) = 1.8e-7_r8 * tp(:)**.39_r8
	 where( tp(:ncol) < 4._r8 )
            rxt(:,k,rid_elec2) = 2.7e-7_r8 * tp(:)**.7_r8
	 elsewhere
            rxt(:,k,rid_elec2) = 1.6e-7_r8 * tp(:)**.55_r8
	 endwhere
      end do level_loop2

!-----------------------------------------------------------------
!	... tropospheric "aerosol" rate constants
!-----------------------------------------------------------------
      amas = 4._r8*pi*rm1**3*den/3._r8            ! each mean particle(r=0.1u) mass (g)
      if( present( sulfate ) ) then
         do k = 1,pver
!-----------------------------------------------------------------
!	... compute the relative humidity
!-----------------------------------------------------------------
            call aqsat( tempn(1,k), pmid(1,k), satv, satq, pcols, &
                        ncol, 1, 1, 1 )
            relhum(:ncol) = .622_r8 * h2ovmr(:ncol,k) / satq(:ncol)
            relhum(:ncol) = max( 0.,min( 1.,relhum(:ncol) ) )
!-------------------------------------------------------------------------
! 	... estimate humidity effect on aerosols (from Shettle and Fenn, 1979)
!           xr is a factor of the increase aerosol radii with hum (hum=0., factor=1)
!-------------------------------------------------------------------------
            xr(:)     = .999151_r8 + relhum(:ncol)*(1.90445_r8 + relhum(:ncol)*(-6.35204_r8 + relhum(:ncol)*5.32061_r8))
!-------------------------------------------------------------------------
! 	... estimate sulfate particles surface area (cm2/cm3) in each grid
!-------------------------------------------------------------------------
            sur(:)    = sulfate(:,k)*m(:,k)/avo*wso4 &              ! xform mixing ratio to g/cm3
                        / amas &                                    ! xform g/cm3 to num particels/cm3
                        * fare &                                    ! xform num particels/cm3 to cm2/cm3
                        * xr(:)*xr(:)                               ! humidity factor
!-----------------------------------------------------------------
!	... compute the "aerosol" reaction rates
!-----------------------------------------------------------------
!             k = gam * A * velo/4
!
!       where velo = sqrt[ 8*bk*T/pi/(w/av) ]
!             bk = 1.381e-16
!             av = 6.02e23
!             w  = 108 (n2o5)  HO2(33)  CH2O (30)  NH3(15)  
!
!       so that velo = 1.40e3*sqrt(T)  (n2o5)   gama=0.1
!       so that velo = 2.53e3*sqrt(T)  (HO2)    gama>0.2
!       so that velo = 2.65e3*sqrt(T)  (CH2O)   gama>0.022
!       so that velo = 3.75e3*sqrt(T)  (NH3)    gama=0.4
!--------------------------------------------------------
!-----------------------------------------------------------------
!	... use this n2o5 -> 2*hno3 only in tropopause
!-----------------------------------------------------------------
	    rxt(:,k,rid_het1) = rxt(:,k,rid_het1) &
                                +.25_r8 * gam1 * sur(:) * 1.40e3_r8 * sqrt( tempn(:ncol,k) )
         end do
      end if

      end subroutine usrrxt

      subroutine usrrxt_hrates( rxt, tempn, tempi, tempe, invariants, &
				h2ovmr, pmid, m, ncol, kbot )
!-----------------------------------------------------------------
!        ... set the user specified reaction rates for heating
!-----------------------------------------------------------------

      use shr_kind_mod,  only : r8 => shr_kind_r8
      use m_rxt_id
      use chem_mods,     only : nfs, rxntot, indexm
      use ppgrid,        only : pver, pcols

      implicit none

!-----------------------------------------------------------------
!        ... dummy arguments
!-----------------------------------------------------------------
      integer, intent(in)     :: ncol                         ! number columns in chunk
      integer, intent(in)     :: kbot                         ! heating levels
      real(r8), intent(in)    :: tempn(pcols,pver)            ! neutral temperature (K)
      real(r8), intent(in)    :: tempi(pcols,pver)            ! ion temperature (K)
      real(r8), intent(in)    :: tempe(pcols,pver)            ! electron temperature (K)
      real(r8), intent(in)    :: m(ncol,pver)                 ! total atm density (1/cm^3)
      real(r8), intent(in)    :: h2ovmr(ncol,pver)            ! water vapor (vmr)
      real(r8), intent(in)    :: pmid(pcols,pver)             ! midpoint pressure (Pa)
      real(r8), intent(in)    :: invariants(ncol,pver,nfs)    ! invariants density (1/cm^3)
      real(r8), intent(inout) :: rxt(ncol,pver,rxntot)        ! gas phase rates
      
!-----------------------------------------------------------------
!        ... local variables
!-----------------------------------------------------------------
      real(r8), parameter :: boltz = 1.38044e-16_r8         ! erg / K
      real(r8), parameter :: avo   = 6.023e23_r8            ! molecules/mole

      integer  ::  k
      real(r8), dimension(ncol) :: &
                   tp, &
                   tinv, &
                   ko, &
                   kinf, &
                   fc, &
                   xr                       ! factor to increase particle radii depending on rel hum

!-----------------------------------------------------------------
!	... o + o2 + m --> o3 + m
!-----------------------------------------------------------------
      do k = 1,kbot
         tinv(:ncol)       = 1._r8 / tempn(:ncol,k)
         tp(:)             = 300._r8 * tinv(:)
         rxt(:,k,rid_usr1) = 6.e-34_r8 * tp(:)**2.4_r8

!-----------------------------------------------------------------
!	... o + o + m -> o2 + m
!-----------------------------------------------------------------
         rxt(:ncol,k,rid_usr2) = 4.23e-28_r8 * tempn(:ncol,k)**(-2)

!-----------------------------------------------------------------
!	... ho2 + ho2 --> h2o2
!	Note: this rate involves the water vapor number density
!-----------------------------------------------------------------
         ko(:)   = 2.3e-13_r8 * exp( 600._r8*tinv(:) )
         kinf(:) = 1.7e-33_r8 * m(:,k) * exp( 1000._r8*tinv(:) )
         fc(:)   = 1._r8 + 1.4e-21_r8 * m(:,k) * h2ovmr(:,k) * exp( 2200._r8*tinv(:) )
         rxt(:,k,rid_usr7) = (ko(:) + kinf(:)) * fc(:)
      end do

!-----------------------------------------------------------------
! 	... the ionic rates
!-----------------------------------------------------------------
level_loop2 : &
      do k = 1,kbot
         tp(:ncol)         = (2._r8*tempi(:ncol,k) + tempn(:ncol,k)) / ( 3._r8 * t0 )
	 tp(:)             = max( min( tp(:),20._r8 ),1._r8 )
         rxt(:,k,rid_ion1) = 2.82e-11_r8 + tp(:)*(-7.74e-12_r8 + tp(:)*(1.073e-12_r8  &
					 + tp(:)*(-5.17e-14_r8 + 9.65e-16_r8*tp(:))))
         tp(:ncol)         = (.6363_r8*tempi(:ncol,k) + .3637_r8*tempn(:ncol,k)) / t0
	 tp(:)             = max( min( tp(:),trlim2 ),1._r8 )
         rxt(:,k,rid_ion2) = 1.533e-12_r8 + tp(:)*(-5.92e-13_r8 + tp(:)*8.6e-14_r8)
         tp(:ncol)         = 2._r8 * t0 /(tempi(:ncol,k) + tempn(:ncol,k))
	 where( tp(:ncol) < trlim3 )
            rxt(:,k,rid_ion3)  = 1.4e-10_r8 * tp(:)**.44_r8
	 elsewhere
            rxt(:,k,rid_ion3)  = 5.2e-11_r8 / tp(:)**.2_r8
	 endwhere
         tp(:ncol)          = t0 / tempe(:ncol,k)
         rxt(:,k,rid_elec1) = 4.e-7_r8 * tp(:)**.85_r8
         rxt(:,k,rid_elec3) = 1.8e-7_r8 * tp(:)**.39_r8
	 where( tp(:ncol) < 4._r8 )
            rxt(:,k,rid_elec2) = 2.7e-7_r8 * tp(:)**.7_r8
	 elsewhere
            rxt(:,k,rid_elec2) = 1.6e-7_r8 * tp(:)**.55_r8
	 endwhere
      end do level_loop2

      end subroutine usrrxt_hrates

      end module mo_usrrxt
