
      module mo_waccm_photo
!----------------------------------------------------------------------
!	... Photolysis interp table and related arrays for stratosphere
!----------------------------------------------------------------------

      use shr_kind_mod,  only : r8 => shr_kind_r8
      use shr_const_mod, only : pi => shr_const_pi
      use ppgrid,        only : pcols, pver
      use abortutils,    only : endrun

      implicit none

      private
      public :: waccm_prate_inti
      public :: waccm_photo
      public :: set_ub_col
      public :: setcol
      public :: sundis
      public :: photo_timestep_init
      public :: r2d

      save

      real(r8), parameter :: max_zen_angle = 94._r8

      integer  :: pverm
      integer, allocatable :: sht_indexer(:), lng_indexer(:)
      real(r8) :: r2d

      contains

      subroutine waccm_prate_inti(xs_coef_file, xs_short_file, xs_long_file, rsf_file, &
           euvacdat_file, photon_file, electron_file)
!----------------------------------------------------------------------
!     ... Read in the stratospheric photorate tables and arrays
!----------------------------------------------------------------------

      use m_rxt_id
      use mo_jshort, only : jshort_init, nsht => nj
      use mo_jlong,  only : jlong_init, nlng => numj
      use mo_jeuv,   only : jeuv_init

      implicit none

!----------------------------------------------------------------------
!        ... dummy args
!----------------------------------------------------------------------
      character(len=*), intent(in) :: &
         xs_coef_file,  &
         xs_short_file, &
         xs_long_file,  &
         rsf_file
      character(len=*), intent(in) :: euvacdat_file
      character(len=*), intent(in) :: photon_file
      character(len=*), intent(in) :: electron_file

!----------------------------------------------------------------------
!        ... local variables
!----------------------------------------------------------------------
      integer :: i, astat

      r2d   = 180._r8/pi
      pverm = pver - 1
!----------------------------------------------------------------------
!        ... initialize photolysis modules
!----------------------------------------------------------------------
      call jshort_init(xs_coef_file, xs_short_file)
      call jlong_init(xs_long_file, rsf_file)
      call jeuv_init (euvacdat_file, photon_file, electron_file)
      allocate( sht_indexer(nsht),stat=astat )
      if( astat /= 0 ) then
         write(*,*) 'waccm_prate_inti: Failed to allocate sht_indexer; error = ',astat
	 call endrun
      end if
      sht_indexer(:) = &
	   (/ rid_jo3_a,     rid_jo3_b,     rid_jn2o,    rid_jhno3,  rid_jho2no2_a, rid_jho2no2_b, &
	      rid_jh2o_a,    rid_jh2o_b,    rid_jh2o_c,  rid_jh2o2,  rid_jcl2o2,    rid_jhcl, &
	      rid_jclono2_a, rid_jclono2_b, rid_jch3cl,  rid_jccl4,  rid_jch3ccl3,  rid_jcfcl3, &
	      rid_jcf2cl2,   rid_jcfc113,   rid_jhcfc22, rid_jch3br, rid_jcf3br,    rid_jcf2clbr, &
	      rid_jco2,      rid_jch4_a,    rid_jch4_b,  rid_jno_i /)
      allocate( lng_indexer(nlng),stat=astat )
      if( astat /= 0 ) then
	 write(*,*) 'waccm_prate_inti: Failed to allocate lng_indexer; error = ',astat
	 call endrun
      end if

      end subroutine waccm_prate_inti

      subroutine photo_timestep_init
!----------------------------------------------------------------------
!        ... check for etf setup
!----------------------------------------------------------------------
      
      use time_manager,   only : is_end_curr_day
      use mo_solar_parms, only : get_solar_parms
      use woods,          only : woods_set_etf
      use neckel,         only : neckel_scale_etf
      use euvac,          only : euvac_set_etf

      implicit none

!----------------------------------------------------------------------
!        ... local variables
!----------------------------------------------------------------------
      real(r8) :: f107
      real(r8) :: f107a

      if( is_end_curr_day() ) then
         call get_solar_parms( f107_s = f107, f107a_s = f107a )
         call woods_set_etf( f107, f107a )
         call neckel_scale_etf( f107, f107a )
         call euvac_set_etf( f107, f107a )
      end if

      end subroutine photo_timestep_init

      subroutine waccm_photo( photos, pmid, pdel, temper, zmid, &
                              col_dens, zen_angle, srf_alb, lwc, clouds, &
                              esfact, vmr, invariants, ncol, lchnk )
!-----------------------------------------------------------------
!   	... waccm photorates
!-----------------------------------------------------------------

      use m_spc_id
      use m_rxt_id
      use chem_mods,  only : nabscol, phtcnt, nfs, indexm, gas_pcnst
      use mo_jshort,  only : nsht => nj, jshort
      use mo_jlong,   only : nlng => numj, jlong
      use mo_jeuv,    only : neuv, jeuv
      use history,    only : outfld

      implicit none

!-----------------------------------------------------------------
!   	... dummy arguments
!-----------------------------------------------------------------
      integer, intent(in)     ::   ncol
      integer, intent(in)     ::   lchnk
      real(r8), intent(in)    ::   esfact                                  ! earth-sun distance factor
      real(r8), intent(in)    ::   vmr(ncol,pver,max(1,gas_pcnst))         ! vmr
      real(r8), intent(in)    ::   invariants(ncol,pver,max(1,nfs))        ! invariant densities (molecules/cm^3)
      real(r8), intent(in)    ::   col_dens(ncol,pver,max(1,nabscol))      ! column densities (molecules/cm^2)
      real(r8), intent(in)    ::   zen_angle(ncol)                         ! solar zenith angle (radians)
      real(r8), intent(in)    ::   srf_alb(pcols)                          ! surface albedo
      real(r8), intent(in)    ::   lwc(ncol,pver)                          ! liquid water content (kg/kg)
      real(r8), intent(in)    ::   clouds(ncol,pver)                       ! cloud fraction
      real(r8), intent(in)    ::   pmid(pcols,pver)                        ! midpoint pressure (Pa)
      real(r8), intent(in)    ::   pdel(pcols,pver)                        ! del pressure about midpoint (Pa)
      real(r8), intent(in)    ::   zmid(ncol,pver)                         ! midpoint height (km)
      real(r8), intent(in)    ::   temper(pcols,pver)                      ! midpoint temperature (K)
      real(r8), intent(out)   ::   photos(ncol,pver,phtcnt)                ! photodissociation rates (1/s)

!-----------------------------------------------------------------
!    	... local variables
!-----------------------------------------------------------------
      real(r8), parameter :: Pa2hPa = 1.e-2_r8
      integer  ::  i, &                    ! indicies
                   k, &
                   m, &
                   n, &
                   astat

      real(r8) ::  sza                    ! solar zenith angle (degrees)
      real(r8) ::  jo2_sht(pver,2)        ! o2 short photorate
      real(r8), dimension(pver) :: &
                   fac1, &                ! work array
                   colo3, &               ! vertical o3 column density
                   zarg, &                ! vertical height array
                   parg, &                ! vertical pressure array (hPa)
                   tline, &               ! vertical temperature array
                   cld_line, &            ! vertical cloud array
                   lwc_line, &            ! vertical lwc array
                   eff_alb, &             ! effective albedo from cloud modifications
                   o_den, &               ! o density (molecules/cm^3)
                   o2_den, &              ! o2 density (molecules/cm^3)
                   o3_den, &              ! o3 density (molecules/cm^3)
                   no_den, &              ! no density (molecules/cm^3)
                   n2_den, &              ! n2 density (molecules/cm^3)
                   jno_sht, &             ! no short photorate
                   cld_mult               ! clould multiplier
      real(r8), allocatable :: &
                   sht_prates(:,:), &     ! short photorates
                   lng_prates(:,:), &     ! long  photorates
                   euv_prates(:,:)        ! euv   photorates

!-----------------------------------------------------------------
!	... allocate short, long, euv temp arrays
!-----------------------------------------------------------------
      allocate( sht_prates(pver,nsht),stat=astat )
      if( astat /= 0 ) then
         write(*,*) 'photo: Failed to allocate sht_prates; error = ',astat
         call endrun
      end if
      allocate( lng_prates(nlng,pver),stat=astat )
      if( astat /= 0 ) then
         write(*,*) 'photo: Failed to allocate lng_prates; error = ',astat
         call endrun
      end if
      allocate( euv_prates(pver,neuv),stat=astat )
      if( astat /= 0 ) then
         write(*,*) 'photo: Failed to allocate euv_prates; error = ',astat
         call endrun
      end if

!-----------------------------------------------------------------
!	... zero all photorates
!-----------------------------------------------------------------
      do m = 1,phtcnt
         do k = 1,pver
            photos(:,k,m) = 0._r8
         end do
      end do

      do i = 1,ncol
         sza = zen_angle(i)*r2d
         if( sza < max_zen_angle ) then
            zarg(:)     = zmid(i,:)
            parg(:)     = Pa2hPa*pmid(i,:)
            colo3(:)    = col_dens(i,:,1)
            fac1(:)     = pdel(i,:)
            tline(:)    = temper(i,:)
            o_den(:)    = vmr(i,:,id_o) * invariants(i,:,indexm)
            o2_den(:)   = vmr(i,:,id_o2) * invariants(i,:,indexm)
            o3_den(:)   = vmr(i,:,id_o3) * invariants(i,:,indexm)
            no_den(:)   = vmr(i,:,id_no) * invariants(i,:,indexm)
            n2_den(:)   = invariants(i,:,2)
            lwc_line(:) = lwc(i,:)
            cld_line(:) = clouds(i,:)
            call cloud_mod( zen_angle(i), cld_line, lwc_line, fac1, srf_alb(i), &
                            eff_alb, cld_mult )
            cld_mult(:) = esfact * cld_mult(:)
            call jshort( pver, sza, n2_den, o2_den, o3_den, &
                         no_den, tline, zarg, jo2_sht, jno_sht, sht_prates )
            call jlong( pver, sza, eff_alb, parg, tline, &
                        colo3, lng_prates )
            call jeuv( pver, sza, o_den, o2_den, n2_den, &
                       zarg, euv_prates )
            do m = 1,nsht
               if( sht_indexer(m) /= 0 ) then
                  photos(i,pver:1:-1,sht_indexer(m)) = sht_prates(:,m)
               end if
            end do
            photos(i,pver:1:-1,rid_jno)   = jno_sht(:)
            photos(i,pver:1:-1,rid_jo2_a) = jo2_sht(:,2)
            photos(i,pver:1:-1,rid_jo2_b) = jo2_sht(:,1)
            do m = 1,rid_jno_i-1
               photos(i,:,m) = (photos(i,:,m) + lng_prates(m,:))*cld_mult(:)
            end do
            do m = rid_jno_i+1,rid_jch4_b
               photos(i,:,m) = (photos(i,:,m) + lng_prates(m-1,:))*cld_mult(:)
            end do
!           do m = 1,rid_jch4_b
!              photos(i,:,m) = (photos(i,:,m) + lng_prates(m,:))*cld_mult(:)
!           end do
            do m = rid_jch4_b+1,phtcnt
               photos(i,pver:1:-1,m) = esfact * euv_prates(:,m-rid_jch4_b)
            end do
#ifdef PHOTO_DEBUG
            write(*,*) ' '
            write(*,*) '-------------------------------------------------'
            write(*,*) 'waccm_photo: euv_prates @ i,lchnk = ',i,lchnk
            write(*,'(1p,5g15.7)') photos(i,:,rid_jch4_b+1)
            write(*,*) '-------------------------------------------------'
            write(*,*) ' '
#endif
	 end if
      end do

      deallocate( sht_prates, lng_prates, euv_prates )

      end subroutine waccm_photo

      subroutine cloud_mod( zen_angle, clouds, lwc, delp, srf_alb, &
                            eff_alb, cld_mult )
!-----------------------------------------------------------------------
! 	... Cloud alteration factors for photorates and albedo
!-----------------------------------------------------------------------

      implicit none

      real(r8), parameter :: gi = 1._r8/9.80616_r8

!-----------------------------------------------------------------------
! 	... Dummy arguments
!-----------------------------------------------------------------------
      real(r8), intent(in)    ::  zen_angle         ! zenith angle
      real(r8), intent(in)    ::  srf_alb           ! surface albedo
      real(r8), intent(in)    ::  clouds(pver)       ! cloud fraction
      real(r8), intent(in)    ::  lwc(pver)          ! liquid water content (mass mr)
      real(r8), intent(in)    ::  delp(pver)         ! del press about midpoint in pascals
      real(r8), intent(out)   ::  eff_alb(pver)      ! effective albedo
      real(r8), intent(out)   ::  cld_mult(pver)     ! photolysis mult factor

!-----------------------------------------------------------------------
! 	... Local variables
!-----------------------------------------------------------------------
      integer :: k
      real(r8)    :: coschi
      real(r8)    :: del_lwp(pver)
      real(r8)    :: del_tau(pver)
      real(r8)    :: above_tau(pver)
      real(r8)    :: below_tau(pver)
      real(r8)    :: above_cld(pver)
      real(r8)    :: below_cld(pver)
      real(r8)    :: above_tra(pver)
      real(r8)    :: below_tra(pver)
      real(r8)    :: fac1(pver)
      real(r8)    :: fac2(pver)

!---------------------------------------------------------
!	... Modify lwc for cloud fraction and form
!	    liquid water path for each layer
!---------------------------------------------------------
      where( clouds(:) /= 0._r8 )
         del_lwp(:) = gi * lwc(:) * delp(:) * 1.e3_r8 / clouds(:)
      elsewhere
	 del_lwp(:) = 0._r8
      endwhere
!---------------------------------------------------------
!    	... Form tau for each model layer
!---------------------------------------------------------
      where( clouds(:) /= 0._r8 )
	 del_tau(:) = del_lwp(:) *.155_r8 * clouds(:)**1.5
      elsewhere
	 del_tau(:) = 0._r8
      end where
!---------------------------------------------------------
!    	... Form integrated tau from top down
!---------------------------------------------------------
      above_tau(1) = 0._r8
      do k = 1,pverm
	 above_tau(k+1) = del_tau(k) + above_tau(k)
      end do
!---------------------------------------------------------
!    	... Form integrated tau from bottom up
!---------------------------------------------------------
      below_tau(pver) = 0._r8
      do k = pverm,1,-1
	 below_tau(k) = del_tau(k+1) + below_tau(k+1)
      end do
!---------------------------------------------------------
!	... Form vertically averaged cloud cover above and below
!---------------------------------------------------------
      above_cld(1) = 0._r8
      do k = 1,pverm
	 above_cld(k+1) = clouds(k) * del_tau(k) + above_cld(k)
      end do
      do k = 2,pver
	 if( above_tau(k) /= 0._r8 ) then
	    above_cld(k) = above_cld(k) / above_tau(k)
	 else
	    above_cld(k) = above_cld(k-1)
	 end if
      end do
      below_cld(pver) = 0._r8
      do k = pverm,1,-1
	 below_cld(k) = clouds(k+1) * del_tau(k+1) + below_cld(k+1)
      end do
      do k = pverm,1,-1
	 if( below_tau(k) /= 0._r8 ) then
	    below_cld(k) = below_cld(k) / below_tau(k)
	 else
	    below_cld(k) = below_cld(k+1)
	 end if
      end do
!---------------------------------------------------------
!	... Modify above_tau and below_tau via jfm
!---------------------------------------------------------
      where( above_cld(2:pver) /= 0._r8 )
	 above_tau(2:pver) = above_tau(2:pver) / above_cld(2:pver)
      end where
      where( below_cld(:pverm) /= 0._r8 )
         below_tau(:pverm) = below_tau(:pverm) / below_cld(:pverm)
      end where
      where( above_tau(2:pver) < 5._r8 )
	    above_cld(2:pver) = 0._r8
      end where
      where( below_tau(:pverm) < 5._r8 )
	 below_cld(:pverm) = 0._r8
      end where
!---------------------------------------------------------
!	... Form transmission factors
!---------------------------------------------------------
      above_tra(:) = 11.905_r8 / (9.524_r8 + above_tau(:))
      below_tra(:) = 11.905_r8 / (9.524_r8 + below_tau(:))
!---------------------------------------------------------
!	... Form effective albedo
!---------------------------------------------------------
      where( below_cld(:) /= 0._r8 )
	 eff_alb(:) = srf_alb + below_cld(:) * (1._r8 - below_tra(:)) &
                                             * (1._r8 - srf_alb)
      elsewhere
	 eff_alb(:) = srf_alb
      end where
      coschi = max( cos( zen_angle ),.5_r8 )
      where( del_lwp(:)*.155_r8 < 5._r8 )
	 fac1(:) = 0._r8
      elsewhere
	 fac1(:) = 1.4_r8 * coschi - 1._r8
      end where
      fac2(:)     = min( 0._r8,1.6_r8*coschi*above_tra(:) - 1._r8 )
      cld_mult(:) = 1._r8 + fac1(:) * clouds(:) + fac2(:) * above_cld(:)
      cld_mult(:) = max( .05_r8,cld_mult(:) )

      end subroutine cloud_mod

      subroutine set_ub_col( col_delta, vmr, pdel, ncol )
!---------------------------------------------------------------
!        ... Set the column densities at the upper boundary
!---------------------------------------------------------------

      use m_spc_id,  only : id_o2, id_o3
      use chem_mods, only : nabscol, gas_pcnst

      implicit none

!---------------------------------------------------------------
!        ... Dummy args
!---------------------------------------------------------------
      integer, intent(in)     ::  ncol
      real(r8), intent(out)   ::  col_delta(ncol,0:pver,max(1,nabscol))     ! /cm**2
      real(r8), intent(in)    ::  vmr(ncol,pver,gas_pcnst), &               ! xported species vmr
                                  pdel(pcols,pver)                          ! pressure delta about midpoints (Pa)

!---------------------------------------------------------------
!        NOTE: xfactor = 10.*R/(K*g) in cgs units.
!              The factor 10. is to convert pdel
!              from pascals to dyne/cm**2.
!---------------------------------------------------------------
      real(r8), parameter :: xfactor = 2.8704e21_r8/(9.80616_r8*1.38044_r8)
      integer :: k

!---------------------------------------------------------------
!        ... Assign column density at the upper boundary
!            The first column is O3 and the second is O2.
!---------------------------------------------------------------
      col_delta(:,0,1) = 0._r8
      col_delta(:,0,2) = 0._r8
      do k = 1,pver
         col_delta(:ncol,k,1) = xfactor * pdel(:ncol,k) * vmr(:ncol,k,id_o3)
         col_delta(:ncol,k,2) = xfactor * pdel(:ncol,k) * vmr(:ncol,k,id_o2)
      end do

      end subroutine set_ub_col

      subroutine setcol( col_delta, col_dens, ncol )
!---------------------------------------------------------------
!     	... Set the column densities
!---------------------------------------------------------------

      use chem_mods, only : nabscol, gas_pcnst

      implicit none

!---------------------------------------------------------------
!     	... Dummy arguments
!---------------------------------------------------------------
      integer, intent(in)   ::  ncol
      real(r8), intent(in)  ::  col_delta(ncol,0:pver,max(1,nabscol))         ! layer column densities (molecules/cm^2)
      real(r8), intent(out) ::  col_dens(ncol,pver,max(1,nabscol))            ! column densities ( /cm**2 )

!---------------------------------------------------------------
!        The local variables
!---------------------------------------------------------------
      integer  ::   i, k, km1      ! long, alt indicies

!---------------------------------------------------------------
!        NOTE: xfactor = 10.*R/(K*g) in cgs units.
!              The factor 10. is to convert pdel
!              from pascals to dyne/cm**2.
!---------------------------------------------------------------
      real(r8), parameter :: xfactor = 2.8704e21_r8/(9.80616_r8*1.38044_r8)

!---------------------------------------------------------------
!   	... Compute column densities down to the
!           current eta index in the calling routine.
!           The first column is O3 and the second is O2.
!---------------------------------------------------------------
      col_dens(:,1,1) = col_delta(:,0,1) + .5_r8 * col_delta(:,1,1)
      col_dens(:,1,2) = col_delta(:,0,2) + .5_r8 * col_delta(:,1,2)
      do k = 2,pver
	 km1 = k - 1
	 col_dens(:,k,1) = col_dens(:,km1,1) + .5_r8 * (col_delta(:,km1,1) + col_delta(:,k,1))
	 col_dens(:,k,2) = col_dens(:,km1,2) + .5_r8 * (col_delta(:,km1,2) + col_delta(:,k,2))
      end do

      end subroutine setcol

      function sundis( idate )
!-----------------------------------------------------------------------------
!   PURPOSE:
!   Calculate Earth-Sun distance variation for a given date.  Based on
!   Fourier coefficients originally from:  Spencer, J.W., 1971, Fourier
!   series representation of the position of the sun, Search, 2:172
!-----------------------------------------------------------------------------
!   PARAMETERS:
!   IDATE  - INTEGER, specification of the date, from YYMMDD              (I)
!   ESRM2  - REAL, variation of the Earth-sun distance                    (O)
!            ESRM2 = (average e/s dist)^2 / (e/s dist on day IDATE)^2
!-----------------------------------------------------------------------------
!   EDIT HISTORY:
!   01/95  Changed computation of trig function values
!-----------------------------------------------------------------------------
!  This program is free software;  you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by the
!  Free Software Foundation;  either version 2 of the license, or (at your
!  option) any later version.
!  The TUV package is distributed in the hope that it will be useful, but
!  WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHANTIBI-
!  LITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
!  License for more details.
!  To obtain a copy of the GNU General Public License, write to:
!  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!-----------------------------------------------------------------------------
!  To contact the authors, please mail to:
!  Sasha Madronich, NCAR/ACD, P.O.Box 3000, Boulder, CO, 80307-3000, USA  or
!  send email to:  sasha@ucar.edu
!-----------------------------------------------------------------------------
!  Copyright (C) 1994,95,96  University Corporation for Atmospheric Research
!-----------------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------------
!	... Dummy arguments
!-----------------------------------------------------------------------------
      integer, intent(in) :: idate

!-----------------------------------------------------------------------------
!	... Function declarations
!-----------------------------------------------------------------------------
      real(r8) :: sundis

!-----------------------------------------------------------------------------
!	... Local variables
!-----------------------------------------------------------------------------
      integer :: iyear, imonth, iday, mday, month, jday
      integer, save :: imn(12) = (/ 31,28,31,30,31,30,31,31,30,31,30,31 /)
      real(r8)    :: dayn, thet0
      real(r8)    :: sinth, costh, sin2th, cos2th

!-----------------------------------------------------------------------------
! 	... Parse date to find day number (Julian day)
!-----------------------------------------------------------------------------
      iyear  = int( idate/10000 )
      imonth = int( (idate - 10000*iyear)/100 )
      iday   = idate - (10000*iyear + 100*imonth)

      if( imonth > 12 ) then
         write(*,*) 'Month in date exceeds 12'
         write(*,*) 'date = ', idate
         write(*,*) 'month = ', imonth
         call endrun
      end if

      if( mod(iyear,4) == 0 ) then
         imn(2) = 29
      else
         imn(2) = 28
      end if

      if( iday > imn(imonth) ) then
         write(*,*) 'Day in date exceeds days in month'
         write(*,*) 'date = ', idate
         write(*,*) 'day = ', iday
         call endrun
      end if

      mday = 0
      do month = 1,imonth-1
         mday = mday + imn(month)	  	   
      end do
      jday = mday + iday
      dayn = jday - 1 + .5_r8

!-----------------------------------------------------------------------------
! 	... Define angular day number and compute esrm2:
!-----------------------------------------------------------------------------
      thet0 = 2._r8*pi*dayn/365._r8

!-----------------------------------------------------------------------------
! 	... Calculate SIN(2*thet0), COS(2*thet0) 
!-----------------------------------------------------------------------------
      sinth   = sin( thet0 )
      costh   = cos( thet0 )
      sin2th  = 2._r8*sinth*costh
      cos2th  = costh*costh - sinth*sinth
      sundis  = 1.000110_r8 + .034221_r8*costh  +  .001280_r8*sinth + .000719_r8*cos2th +  .000077_r8*sin2th

      end function sundis

      end module mo_waccm_photo
