!===============================================================================
! CVS $Id: advance.F90,v 1.6 2001/08/01 22:56:45 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/datm5/advance.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

      SUBROUTINE advance_atmdata()

      use calendar_mod  ! calendar routines
      use shr_timer_mod ! shared code: timing routines
      use control_mod 
      use buf_mod       ! model   data   
      use atmdata_mod 

      implicit none

      !----- local  ------
      integer   i,j,n        ! generic loop indicies
      integer   k1,k2        ! raw data time-slice indicies
      real      f1,f2        ! raw data fraction
      real      f            ! factor used wrt sw-down components
      integer   timer_n      ! timer number
      logical   first_call   ! flags 1st envocation of this routine
      data      first_call /.true./
      save

!-------------------------------------------------------------------------------
! PURPOSE:
!   advance the model state to the current date
!   
! METHOD:
!   fill atm output state & flux field arrays appropriately
!   (these arrays will subsequently be sent to the coupler)
!-------------------------------------------------------------------------------

      if (first_call) call shr_timer_get(timer_n,'advance')
      call shr_timer_start(timer_n)

      !-----------------------------------------------------------------
      ! get raw data time slices that bracket current model date & time 
      !-----------------------------------------------------------------
      call data_in_atm()

      !-----------------------------------------------------------------
      ! determine time interpolation factors
      !-----------------------------------------------------------------
      write(6,9001) 'advancing to date = ',cdate,sec

      !--- want r_sec(k1) < sec < r_sec(k2) ---

      k1 = r_sec(1)
      k2 = r_sec(2) 
      if (r_sec(1) .gt. sec) k1 = r_sec(1) - 86400 
      if (r_sec(2) .lt. sec) k2 = r_sec(2) + 86400
      f1 = real(k2-sec)/real(k2-k1)
      f2 = 1.0 - f1
!      write(6,'(2(a4,f5.3),2(a4,i6),3(a10,i6))')'f1=',f1,'f2=',f2,'k1=',k1,'k2=',k2,'r_sec(1)=',r_sec(1),'r_sec(2)=',r_sec(2),'sec=',sec

      !-----------------------------------------------------------------
      ! time interpolate raw data into model data fields
      !-----------------------------------------------------------------
     
      DO j=1,ny_a
      DO i=1,nx_a
         n = (j-1)*nx_a + i
         z    (n) = 30.0 
         u    (n) = f1*r_u    (i,j,1) + f2*r_u    (i,j,2)
         v    (n) = f1*r_v    (i,j,1) + f2*r_v    (i,j,2)
         t    (n) = f1*r_t    (i,j,1) + f2*r_t    (i,j,2)
         ptem (n) = f1*r_ptem (i,j,1) + f2*r_ptem (i,j,2)
         pres (n) = f1*r_pres (i,j,1) + f2*r_pres (i,j,2)
         pslv (n) = f1*r_pslv (i,j,1) + f2*r_pslv (i,j,2)
         shum (n) = f1*r_shum (i,j,1) + f2*r_shum (i,j,2)
         dens (n) = f1*r_dens (i,j,1) + f2*r_dens (i,j,2)
         precc(n) = f1*r_precc(i,j,1) + f2*r_precc(i,j,2)
         precl(n) = f1*r_precl(i,j,1) + f2*r_precl(i,j,2)
        !snowc(n) = f1*r_snowc(i,j,1) + f2*r_snowc(i,j,2)
        !snowl(n) = f1*r_snowl(i,j,1) + f2*r_snowl(i,j,2)
         lwdn (n) = f1*r_lwdn (i,j,1) + f2*r_lwdn (i,j,2)
         swnet(n) = f1*r_swnet(i,j,1) + f2*r_swnet(i,j,2)
         swvdr(n) = f1*r_swvdr(i,j,1) + f2*r_swvdr(i,j,2)
         swndr(n) = f1*r_swndr(i,j,1) + f2*r_swndr(i,j,2)
         swvdf(n) = f1*r_swvdf(i,j,1) + f2*r_swvdf(i,j,2)
         swndf(n) = f1*r_swndf(i,j,1) + f2*r_swndf(i,j,2)
         co2  (n) = 355.e-06  
         if ( t(n)<275.65) then 
          snowc(n) = precc(n)
          snowl(n) = precl(n)
          precc(n) = 0.
          precl(n) = 0.
         else 
          snowc(n) = 0.
          snowl(n) = 0.
          precc(n) = precc(n)
          precl(n) = precl(n)
         endif
      END DO
      END DO
    
      !write(6,*)'u    (n)=',u    (30731)
      !write(6,*)'t    (n)=',t    (30731)
      !write(6,*)'ptem (n)=',ptem (30731)
      !write(6,*)'pres (n)=',pres (30731)
      !write(6,*)'pslv (n)=',pslv (30731)
      !write(6,*)'shum (n)=',shum (30731)
      !write(6,*)'dens (n)=',dens (30731)
      !write(6,*)'precc(n)=',precc(30731)
      !write(6,*)'snowc(n)=',snowc(30731)
      !write(6,*)'lwdn (n)=',lwdn (30731)
      !write(6,*)'swnet(n)=',swnet(30731)
      !write(6,*)'swvdr(n)=',swvdr(30731)

      !-----------------------------------------------------------------
      ! albedo-feedback is on => alter short-wave wrt surface albedo
      !-----------------------------------------------------------------
        if (first_call .and. rest_type == 'initial') then
          write(6,9000) 'NOTE: albedo feedback is ON'
          !--- cannot apply FB on 1st call because albedos are not set ---
        else
          !--- compute swnet consisten with (1-albedo)*swdn ---
          !--- note: need albedos from cpl before doing this ---
          do n=1,n_a
            f =     ( 1.0 - Ss_a(n,kSs_avsdr))*swvdr(n)
            f = f + ( 1.0 - Ss_a(n,kSs_anidr))*swndr(n)
            f = f + ( 1.0 - Ss_a(n,kSs_avsdf))*swvdf(n)
            f = f + ( 1.0 - Ss_a(n,kSs_anidf))*swndf(n)
            swnet(n) = f
          enddo
        end if

      call shr_timer_stop(timer_n)

      first_call = .false.
      RETURN
 9000 format('(advance) ',a,a)
 9001 format('(advance) ',a,i8.8,i6,'sec')
      END

!=======================================================================
