module atmdata_mod

   use control_mod      ! control data
   use buf_mod          ! model   data
!  use map_mod  

   implicit none
    
      !----- timing info -----
      integer r_kold           ! = 1 or 2: k index of older data slice
      integer r_cdate(2)       ! calendar dates for raw data time slices
      integer r_sec  (2)       ! elapsed secs   for raw data time slices

      !----- states -----
      REAL   ,ALLOCATABLE ::  z    (:)      ! bottom level height       ~ m
      REAL   ,ALLOCATABLE ::  u    (:)      ! wind, zonal               ~ m/s
      REAL   ,ALLOCATABLE ::  v    (:)      ! wind, meridional          ~ m/s
      REAL   ,ALLOCATABLE ::  t    (:)      ! temperature               ~ Kelvin
      REAL   ,ALLOCATABLE ::  ptem (:)      ! potential temperature     ~ Kelvin
      REAL   ,ALLOCATABLE ::  pres (:)      ! pressure, bottom layer    ~ Pa
      REAL   ,ALLOCATABLE ::  pslv (:)      ! pressure, sea level       ~ Pa
      REAL   ,ALLOCATABLE ::  shum (:)      ! specific humidity         ~ kg/kg
      REAL   ,ALLOCATABLE ::  dens (:)      ! density                   ~ kg/m^3
      
      !----- fluxes -----    
      REAL   ,ALLOCATABLE ::  precc(:)      ! precip, liquid, convective  ~ kg/s/m^2
      REAL   ,ALLOCATABLE ::  precl(:)      ! precip, liquid, large-scale ~ kg/s/m^2
      REAL   ,ALLOCATABLE ::  snowc(:)      ! precip, frozen, convective  ~ kg/s/m^2
      REAL   ,ALLOCATABLE ::  snowl(:)      ! precip, frozen, large-scale ~ kg/s/m^2
      REAL   ,ALLOCATABLE ::  lwdn (:)      ! longwave, downward        ~    W/m^2
      REAL   ,ALLOCATABLE ::  swnet(:)      ! shortwave, net            ~    W/m^2
      REAL   ,ALLOCATABLE ::  swvdr(:)      ! shortwave, down, vis-dir  ~    W/m^2
      REAL   ,ALLOCATABLE ::  swndr(:)      ! shortwave, down, nir-dir  ~    W/m^2
      REAL   ,ALLOCATABLE ::  swvdf(:)      ! shortwave, down, vis-diff ~    W/m^2
      REAL   ,ALLOCATABLE ::  swndf(:)      ! shortwave, down, nir-diff ~    W/m^2
      REAL   ,ALLOCATABLE ::  co2  (:)      ! shortwave, down, nir-diff ~    W/m^2

      !----- states -----
      REAL   ,ALLOCATABLE :: r_z    (:,:,:)    ! bottom level height       ~ m
      REAL   ,ALLOCATABLE :: r_u    (:,:,:)    ! wind, zonal               ~ m/s
      REAL   ,ALLOCATABLE :: r_v    (:,:,:)    ! wind, meridional          ~ m/s
      REAL   ,ALLOCATABLE :: r_t    (:,:,:)    ! temperature               ~ Kelvin
      REAL   ,ALLOCATABLE :: r_ptem (:,:,:)    ! potential temperature     ~ Kelvin
      REAL   ,ALLOCATABLE :: r_pres (:,:,:)    ! pressure, bottom layer    ~ Pa
      REAL   ,ALLOCATABLE :: r_pslv (:,:,:)    ! pressure, sea level       ~ Pa
      REAL   ,ALLOCATABLE :: r_shum (:,:,:)    ! specific humidity         ~ kg/kg
      REAL   ,ALLOCATABLE :: r_dens (:,:,:)    ! density                   ~ kg/m^3

      !----- fluxes -----
      REAL   ,ALLOCATABLE :: r_precc(:,:,:)    ! precip, liquid, convective  ~ kg/s/m^2
      REAL   ,ALLOCATABLE :: r_precl(:,:,:)    ! precip, liquid, large-scale ~ kg/s/m^2
      REAL   ,ALLOCATABLE :: r_snowc(:,:,:)    ! precip, frozen, convective  ~ kg/s/m^2
      REAL   ,ALLOCATABLE :: r_snowl(:,:,:)    ! precip, frozen, large-scale ~ kg/s/m^2
      REAL   ,ALLOCATABLE :: r_lwdn (:,:,:)    ! longwave, downward        ~    W/m^2
      REAL   ,ALLOCATABLE :: r_swnet(:,:,:)    ! shortwave, net            ~    W/m^2
      REAL   ,ALLOCATABLE :: r_swdn (:,:,:)    ! shortwave, downward       ~    W/m^2
      REAL   ,ALLOCATABLE :: r_swvdr(:,:,:)    ! shortwave, down, vis-dir  ~    W/m^2
      REAL   ,ALLOCATABLE :: r_swndr(:,:,:)    ! shortwave, down, nir-dir  ~    W/m^2
      REAL   ,ALLOCATABLE :: r_swvdf(:,:,:)    ! shortwave, down, vis-diff ~    W/m^2
      REAL   ,ALLOCATABLE :: r_swndf(:,:,:)    ! shortwave, down, nir-diff ~    W/m^2

   CONTAINS
   
SUBROUTINE atmdata_alloc()

      IMPLICIT none

      allocate(   z    (n_a) )      ! bottom level height       ~ m
      allocate(   u    (n_a) )      ! wind, zonal               ~ m/s
      allocate(   v    (n_a) )      ! wind, meridional          ~ m/s
      allocate(   t    (n_a) )      ! temperature               ~ Kelvin
      allocate(   ptem (n_a) )      ! potential temperature     ~ Kelvin
      allocate(   pres (n_a) )      ! pressure, bottom layer    ~ Pa
      allocate(   pslv (n_a) )      ! pressure, sea level       ~ Pa
      allocate(   shum (n_a) )      ! specific humidity         ~ kg/kg
      allocate(   dens (n_a) )      ! density                   ~ kg/m^3

      !----- fluxes -----    
      allocate(   precc(n_a) )      ! precip, liquid, convective  ~ kg/s/m^2
      allocate(   precl(n_a) )      ! precip, liquid, large-scale ~ kg/s/m^2
      allocate(   snowc(n_a) )      ! precip, frozen, convective  ~ kg/s/m^2
      allocate(   snowl(n_a) )      ! precip, frozen, large-scale ~ kg/s/m^2
      allocate(   lwdn (n_a) )      ! longwave, downward        ~    W/m^2
      allocate(   swnet(n_a) )      ! shortwave, net            ~    W/m^2
      allocate(   swvdr(n_a) )      ! shortwave, down, vis-dir  ~    W/m^2
      allocate(   swndr(n_a) )      ! shortwave, down, nir-dir  ~    W/m^2
      allocate(   swvdf(n_a) )      ! shortwave, down, vis-diff ~    W/m^2
      allocate(   swndf(n_a) )      ! shortwave, down, nir-diff ~    W/m^2i
      allocate(   co2  (n_a) )      ! shortwave, down, nir-diff ~    W/m^2i

      !----- states -----
       allocate( r_z    (nx_a,ny_a,2)  )    ! bottom level height       ~ m
       allocate( r_u    (nx_a,ny_a,2)  )    ! wind, zonal               ~ m/s
       allocate( r_v    (nx_a,ny_a,2)  )    ! wind, meridional          ~ m/s
       allocate( r_t    (nx_a,ny_a,2)  )    ! temperature               ~ Kelvin
       allocate( r_ptem (nx_a,ny_a,2)  )    ! potential temperature     ~ Kelvin
       allocate( r_pres (nx_a,ny_a,2)  )    ! pressure, bottom layer    ~ Pa
       allocate( r_pslv (nx_a,ny_a,2)  )    ! pressure, sea level       ~ Pa
       allocate( r_shum (nx_a,ny_a,2)  )    ! specific humidity         ~ kg/kg
       allocate( r_dens (nx_a,ny_a,2)  )    ! density                   ~ kg/m^3
     
      !----- fluxes -----
       allocate( r_precc(nx_a,ny_a,2)  )    ! precip, liquid, convective  ~ kg/s/m^2
       allocate( r_precl(nx_a,ny_a,2)  )    ! precip, liquid, large-scale ~ kg/s/m^2
       allocate( r_snowc(nx_a,ny_a,2)  )    ! precip, frozen, convective  ~ kg/s/m^2
       allocate( r_snowl(nx_a,ny_a,2)  )    ! precip, frozen, large-scale ~ kg/s/m^2
       allocate( r_lwdn (nx_a,ny_a,2)  )    ! longwave, downward        ~    W/m^2
       allocate( r_swnet(nx_a,ny_a,2)  )    ! shortwave, net            ~    W/m^2
       allocate( r_swdn (nx_a,ny_a,2)  )    ! shortwave, downward       ~    W/m^2
       allocate( r_swvdr(nx_a,ny_a,2)  )    ! shortwave, down, vis-dir  ~    W/m^2
       allocate( r_swndr(nx_a,ny_a,2)  )    ! shortwave, down, nir-dir  ~    W/m^2
       allocate( r_swvdf(nx_a,ny_a,2)  )    ! shortwave, down, vis-diff ~    W/m^2
       allocate( r_swndf(nx_a,ny_a,2)  )    ! shortwave, down, nir-diff ~    W/m^2i

      z      = 0. 
      u      = 0. 
      v      = 0. 
      t      = 0. 
      ptem   = 0. 
      pres   = 0. 
      pslv   = 0. 
      shum   = 0. 
      dens   = 0. 
      precc  = 0. 
      precl  = 0. 
      snowc  = 0. 
      snowl  = 0. 
      lwdn   = 0. 
      precl  = 0. 
      swnet  = 0.
      swvdr  = 0.
      swndr  = 0.   
      swvdf  = 0.  
      swndf  = 0.

      r_z      = 0.
      r_u      = 0.
      r_v      = 0.
      r_t      = 0.
      r_ptem   = 0.
      r_pres   = 0.
      r_pslv   = 0.
      r_shum   = 0.
      r_dens   = 0.
      r_precc  = 0.
      r_precl  = 0.
      r_snowc  = 0.
      r_snowl  = 0.
      r_lwdn   = 0.
      r_precl  = 0.
      r_swnet  = 0.
      r_swvdr  = 0.
      r_swndr  = 0.
      r_swvdf  = 0.
      r_swndf  = 0.

END SUBROUTINE atmdata_alloc

SUBROUTINE replace_atmdata(Sa,Faxa,Faia,Fala,Faoa)
 
       IMPLICIT none 
       !--- input ---
       real         :: Sa  (n_a,nSa  ) ! atm state
       real         :: Faxa(n_a,nFaxa) ! atm/srf fluxes from atm
       real         :: Faia(n_a,nFaia) ! atm/ice fluxes from atm
       real         :: Fala(n_a,nFala) ! atm/lnd fluxes from atm
       real         :: Faoa(n_a,nFaoa) ! atm/ocn fluxes from atm 
       !-----local ----
       integer      :: n

      !write(6,*)'relace data begin'

      DO n=1,n_a 

       Sa  (n,kSa_z      ) = z    (n)
       Sa  (n,kSa_u      ) = u    (n)
       Sa  (n,kSa_v      ) = v    (n)
       Sa  (n,kSa_tbot   ) = t    (n)
       Sa  (n,kSa_ptem   ) = ptem (n)
       Sa  (n,kSa_pbot   ) = pres (n)
       Sa  (n,kSa_pslv   ) = pslv (n)
       Sa  (n,kSa_shum   ) = shum (n)
       Sa  (n,kSa_dens   ) = dens (n)
       Sa  (n,kSa_p_surf ) = pres (n)

      !--- output fluxes: atm/lnd ---
       Fala(n,kFala_lwdn )   = lwdn (n) 
!      if( abs(domain_a%yc(n)).le.40.0 )then
!       Fala(n,kFala_rainc)   = precc(n)    !not used the NCEP precp             
!       Fala(n,kFala_rainl)   = precl(n)    !not used the NCEP precp 
!       Fala(n,kFala_snowc)   = snowc(n)    !not used the NCEP precp 
!       Fala(n,kFala_snowl)   = snowl(n)    !not used the NCEP precp  
       Fala(n,kFala_swndr)   = swndr(n) 
       Fala(n,kFala_swvdr)   = swvdr(n) 
       Fala(n,kFala_swndf)   = swndf(n) 
       Fala(n,kFala_swvdf)   = swvdf(n) 
       if( carbon_flux ) Fala(n,kFala_co2) = co2(n) 

       !--- output fluxes: atm/ice ---
       Faia(n,kFaia_lwdn )  = lwdn (n)
       Faia(n,kFaia_rain )  = precc(n) + precl(n) 
       Faia(n,kFaia_snow )  = snowc(n) + snowl(n) 
       Faia(n,kFaia_swndr)  = swndr(n) 
       Faia(n,kFaia_swvdr)  = swvdr(n) 
       Faia(n,kFaia_swndf)  = swndf(n) 
       Faia(n,kFaia_swvdf)  = swvdf(n) 

       !--- output fluxes: atm/ocn ---
       Faoa(n,kFaoa_lwdn ) = lwdn (n) 
       Faoa(n,kFaoa_rain ) = precc(n) + precl(n) 
       Faoa(n,kFaoa_snow ) = snowc(n) + snowl(n)  
       Faoa(n,kFaoa_swndr) = swndr(n) 
       Faoa(n,kFaoa_swvdr) = swvdr(n)  
       Faoa(n,kFaoa_swndf) = swndf(n) 
       Faoa(n,kFaoa_swvdf) = swvdf(n)  
       if( carbon_flux ) Faoa(n,kFaoa_co2) =  co2(n) 

      !--- output fluxes: atm/srf ---
       Faxa(n,kFaxa_swnet) = swnet(n)

     enddo

     !write(6,*)'replace data after'

END SUBROUTINE replace_atmdata

end module atmdata_mod
