!===============================================================================
! CVS $Id: bitcheck.F90,v 1.1 2001/02/02 00:25:38 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/bitcheck.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

SUBROUTINE bitcheck()

   use buf_mod       ! model data buffers
   use control_mod   ! control flags
#ifdef MPI_P
!   use mpmd_mod
   use cpl_mod,only:cpl_tid
   use shr_kind_mod, only: SHR_KIND_IN, SHR_KIND_R8
#endif

   implicit none
#ifdef MPI_P
   include 'mpif.h'
#endif

   !----- local -----
   integer :: i,n        ! generic loop indicies
   real    :: mn,av,mx   ! mininum,average,maximum
   real    :: T          ! temperature
   logical :: do_check   ! flag wrt doing the check or not

   logical :: first_call = .true. ! flags one-time setup
#ifdef MPI_P
   REAL (SHR_KIND_R8) :: min_redu1(4),min_redu2(4)
   REAL (SHR_KIND_R8) :: ave_redu1(4),ave_redu2(4)
   REAL (SHR_KIND_R8) :: max_redu1(4),max_redu2(4)
   integer n_redu1(4),n_redu2(4)
#endif

   save

   !----- formats -----
   character(len=*),parameter :: F00 = &
   &    "('(bitcheck) key: date,model,case,min,avg,max (surface T), Num_Grids')"
   character(len=*),parameter :: F01 = &
   &    "('(bitcheck) ',i8.8,i6,'s ',a,1x,a,3es26.19,i10)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   print some quantities out to full precision for testing
!   exact restarts, variations between machines, etc.
!-------------------------------------------------------------------------------

   do_check = .false.

   if (info_bcheck == 1) then
     if (sec==0 ) then
       if (day==1 .or. day==2 .or. day==3 .or. (eday-eday0)<3 ) do_check=.true.
     end if
   else if (info_bcheck == 2) then
     if ( sec==0 )  do_check=.true.
   else if (info_bcheck > 2) then
     do_check=.true.
   end if

   if ( .not. do_check ) return

   if ( first_call ) write(6,F00)
   first_call = .false.

   !--- atm surface temperature ----
   mx = -1.0e30
   mn =  1.0e30
   av = 0.0
   n  = 0
#ifdef MPI_P
   do i=ist_a,iend_a
!   do i=1,nx_a
      T = Sa_a(i,kSa_tbot)
      mx = max(mx,T)
      mn = min(mn,T)
      av = av+T
      n  = n+1
   end do
      max_redu1(1) = mx
      min_redu1(1) = mn
      ave_redu1(1) = av
      n_redu1  (1) = n
#else
   do i=1,nx_a
      T = Sa_a(i,kSa_tbot)
      mx = max(mx,T)
      mn = min(mn,T)
      av = av+T
      n  = n+1
   end do
   av = av/float(n)
   write(6,F01) cdate,sec,'atm',trim(case_name),mn,av,mx,n
#endif

   !--- ice surface temperature ----
   mx = -1.0e30
   mn =  1.0e30
   av = 0.0
   n  = 0
#ifdef MPI_P
   do i=ist_i,iend_i
      if (mask_i(i)/=0 .and. Si_i(i,kSi_ifrac)>0.0) then
         T = Si_i(i,kSi_t)
         mx = max(mx,T)
         mn = min(mn,T)
         av = av+T
         n  = n+1
      end if
   end do
      max_redu1(2) = mx
      min_redu1(2) = mn
      ave_redu1(2) = av
      n_redu1  (2) = n
#else
   do i=1,n_i
      if (mask_i(i)/=0 .and. Si_i(i,kSi_ifrac)>0.0) then
         T = Si_i(i,kSi_t)
         mx = max(mx,T)
         mn = min(mn,T)
         av = av+T
         n  = n+1
      end if
   end do
   av = av/float(n)
   write(6,F01) cdate,sec,'ice',trim(case_name),mn,av,mx,n
#endif

   !--- lnd surface temperature ----
   mx = -1.0e30
   mn =  1.0e30
   av = 0.0
   n  = 0
#ifdef MPI_P
!xjx   write(30,*)"xjx..bitcheck..Sl_l=",Sl_l
   do i=ist_l,iend_l
      if (mask_l(i)/=0) then
         T = Sl_l(i,kSl_t)
         mx = max(mx,T)
         mn = min(mn,T)
         av = av+T
         n  = n+1
      end if
   end do
      max_redu1(3) = mx
      min_redu1(3) = mn
      ave_redu1(3) = av
      n_redu1  (3) = n
#else
   do i=1,n_l
      if (mask_l(i)/=0) then
         T = Sl_l(i,kSl_t)
         mx = max(mx,T)
         mn = min(mn,T)
         av = av+T
         n  = n+1
      end if
   end do
   av = av/float(n)
   write(6,F01) cdate,sec,'lnd',trim(case_name),mn,av,mx,n
#endif

   !--- ocn surface temperature ----
   mx = -1.0e30
   mn =  1.0e30
   av = 0.0
   n  = 0
#ifdef MPI_P
!xjx   write(31,*)"xjx..bitcheck..So_o=",So_o

   do i=ist_o,iend_o
      if (mask_o(i) .ne. 0) then
         T = So_o(i,kSo_t)
         mx = max(mx,T)
         mn = min(mn,T)
         av = av+T
         n  = n+1
      end if
   end do
      max_redu1(4) = mx
      min_redu1(4) = mn
      ave_redu1(4) = av
      n_redu1(4)   = n
#else
   do i=1,n_o
      if (mask_o(i) .ne. 0) then
         T = So_o(i,kSo_t)
         mx = max(mx,T)
         mn = min(mn,T)
         av = av+T
         n  = n+1
      end if
   end do
   av = av/float(n)
   write(6,F01) cdate,sec,'ocn',trim(case_name),mn,av,mx,n
#endif


#ifdef MPI_P
       call mpi_reduce(min_redu1(1),min_redu2(1),4,MPI_REAL8, &
                   &MPI_MIN,0,MPI_COMM_CPL,ierr)

       call mpi_reduce(ave_redu1(1),ave_redu2(1),4,MPI_real8, &
                   &MPI_SUM,0,MPI_COMM_CPL,ierr)

       call mpi_reduce(max_redu1(1),max_redu2(1),4,MPI_real8, &
                   &MPI_MAX,0,MPI_COMM_CPL,ierr)

       call mpi_reduce(n_redu1(1),n_redu2(1),4,MPI_INTEGER, &
                   &MPI_SUM,0,MPI_COMM_CPL,ierr)

   if(master_task) then
      ave_redu2(1) = ave_redu2(1)/float(n_redu2(1))
      write(6,F01) cdate,sec,'atm',trim(case_name), &
                & min_redu2(1),ave_redu2(1),max_redu2(1),n_redu2(1)  
!
      ave_redu2(2) = ave_redu2(2)/float(n_redu2(2))
      write(6,F01) cdate,sec,'ice',trim(case_name), &
                & min_redu2(2),ave_redu2(2),max_redu2(2),n_redu2(2)  
!
      ave_redu2(3) = ave_redu2(3)/float(n_redu2(3))
      write(6,F01) cdate,sec,'lnd',trim(case_name), &
                & min_redu2(3),ave_redu2(3),max_redu2(3),n_redu2(3)  
!
      ave_redu2(4) = ave_redu2(4)/float(n_redu2(4))
      write(6,F01) cdate,sec,'ocn',trim(case_name), &
                & min_redu2(4),ave_redu2(4),max_redu2(4),n_redu2(4)  
   endif
#endif
  
RETURN
END

!===============================================================================
