#ifdef MPI_P

MODULE comm_mod
       USE buf_mod        ! model   data
       USE mpmd_mod
       USE spmd_mod
       USE cpl_mod

     IMPLICIT none

  CONTAINS
!
!----iblock-----number of cpus(cpl) 
!----offset2----offset in id,one big_block may have many arrays(a(:,:)+b(:,:)+c(:,:))  
!----offset4----offset between arrays varibal
!----offset1----offset in RHS 
!----offset3----offset in one arrays
!----------------------------------------------
!    |       |       |////////////////////////|
!    |   offset4     |////////////////////////| A(:,nSa=3)
!    |       |       |////////////////////////|
!    o ----------------------------------------
!    f       |       |\\\\\\\\\\\\\\\\\\\\\\\\|
!    f               |\\\\\\\\\\\\\\\\\\\\\\\\|
!    s   offset4     |\\\\\\\\\\\\\\\\\\\\\\\\| B(:,nSa=5)
!    e               |\\\\\\\\\\\\\\\\\\\\\\\\|
!    t       |       |\\\\\\\\\\\\\\\\\\\\\\\\|
!    2 ------------ --------------------------
!    |       |       |************************|
!    |               |************************| C(:,nSa=4)
!    |   offset4     |************************|
!    |               |************************|
!==============================================
!    |       |       |////////////////////////|
!    |   offset4     |////////////////////////| A
!    |       |       |////////////////////////|
!    o ----------------------------------------
!    f       |       |\\\\\\\\\\\\\\\\\\\\\\\\|
!    f               |\\\\\\\\\\\\\\\\\\\\\\\\|
!    s   offset4     |\\\\\\\\\\\\\\\\\\\\\\\\| B
!    e               |\\\\\\\\\\\\\\\\\\\\\\\\|
!    t       |       |\\\\\\\\\\\\\\\\\\\\\\\\|
!    2 ------------ --------------------------
!    |       |       |************************|
!    |               |************************| C
!    |   offset4     |************************|
!    |               |************************|
!------------|---------------------------------
!                       .
!                       .
!                       .
!
!===============================================================================
  SUBROUTINE comm1(Ss_a,Faxx_a)
  
    implicit none

    include "mpif.h"
    real  ,intent(inout) ::Ss_a  (n_a,nSs)
    real  ,intent(inout) ::Faxx_a(n_a,nFaxx)
    integer i,j,k,n,ierr,iv,sendcnt,recvcnt
    integer offset1,offset2,offset3,offset4
    integer iblock,block_len
#if 1
    real (SHR_KIND_R8), allocatable :: redu3(:),redu4(:)

      block_len = n_a/CPL_SIZE
      allocate(redu3(block_len*(nSs+nFaxx)))
      allocate(redu4(n_a*(nSs+nFaxx)))
      redu3=0.
      redu4=0.
      iv = 1
      do i = 1, nSs
          do n = ist_a,iend_a
          redu3(iv) = Ss_a(n,i)
          iv = iv+1
          enddo
      enddo
!
      do i = 1, nFaxx
          do n = ist_a,iend_a
          redu3(iv) = Faxx_a(n,i)
          iv = iv+1
          enddo
      enddo
      sendcnt = iv-1
      recvcnt = iv-1
      call MPI_allgather(redu3,sendcnt,MPI_REAL8,&
           &  redu4,recvcnt,MPI_REAL8,MPI_COMM_CPL,ierr)
!
      offset4 = 0
      do i = 1, nSs
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_len*(nSs+nFaxx)*(iblock-1)
         do j = 1,block_len
         Ss_a(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
      offset4 = block_len *nSs
      do i = 1, nFaxx
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_len*(nSs+nFaxx)*(iblock-1)
         do j = 1,block_len
         Faxx_a(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
#else
      real (SHR_KIND_R8), allocatable :: redu3(:,:),redu4(:,:)
      allocate(redu3(n_a,nSs+nFaxx))
      allocate(redu4(n_a,nSs+nFaxx))
      redu3=0.
      redu4=0.
      do i = 1, nSs
          do n = ist_a,iend_a
          redu3(n,i) = Ss_a(n,i)
          enddo
      enddo
!
      do i = 1, nFaxx
          do n = ist_a,iend_a
          redu3(n,i+nSs) = Faxx_a(n,i)
          enddo
      enddo

      call MPI_ALLREDUCE(redu3, redu4, n_a*(nSs+nFaxx),&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)

      do i = 1, nSs
          Ss_a(:,i) = redu4(:,i)
      enddo
      do i = 1, nFaxx
          Faxx_a(:,i) = redu4(:,i+nSs)
      enddo
#endif   
      deallocate(redu3)
      deallocate(redu4)

  END SUBROUTINE comm1
!===============================================================================

  SUBROUTINE comm2(aFoxx_o)
    implicit none

    include "mpif.h"
    real  ,intent(inout) ::aFoxx_o(n_o,nFoxx)
    integer i,j,k,ierr,iv,sendcnt,recvcnt
    integer maxlen,rank,mpes
    integer offset1,offset2,offset3,offset4
    integer iblock,block_len

#if 1
    real (SHR_KIND_R8), allocatable :: redu3(:),redu4(:)

      block_len = n_o/CPL_SIZE
      allocate(redu3(block_len*nFoxx))
      allocate(redu4(n_o*nFoxx))
      redu3=0.
      redu4=0.
      iv = 1
      do i = 1, nFoxx
      do j = ist_o,iend_o
         redu3(iv) = aFoxx_o(j,i)
         iv = iv + 1
      enddo
      enddo
      sendcnt = iv-1
      recvcnt = iv-1
      call MPI_allgather(redu3,sendcnt,MPI_REAL8,&
           &  redu4,recvcnt,MPI_REAL8,MPI_COMM_CPL,ierr)
      offset4 = 0
      do i = 1, nFoxx
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_len*nFoxx*(iblock-1)
         do j = 1,block_len
         aFoxx_o(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
      
#else
    real (SHR_KIND_R8),allocatable:: redu3(:,:),redu4(:,:)
      allocate(redu3(n_o,nFoxx))
      allocate(redu4(n_o,nFoxx))
      redu3=0.
      redu4=0.
      do i = 1, nFoxx
      do j = ist_o,iend_o
         redu3(j,i) = aFoxx_o(j,i)
      enddo
      enddo

      call MPI_ALLREDUCE(redu3, redu4, n_o*nFoxx,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)

      do i = 1, nFoxx
         aFoxx_o(:,i) = redu4(:,i)
      enddo
#endif
      deallocate(redu3)
      deallocate(redu4)


  END SUBROUTINE comm2

!===============================================================================
  SUBROUTINE comm3(Sa_l,Fala_l)
    implicit none

    include "mpif.h"
    real  ,intent(inout) ::Sa_l  (n_l,nSa  )
    real  ,intent(inout) ::Fala_l(n_l,nFala)
    integer i,j,k,n,ierr,iv,sendcnt,recvcnt
    integer offset1,offset2,offset3,offset4
    integer iblock,block_len

#if 1
    real (SHR_KIND_R8), allocatable :: redu3(:),redu4(:)
      block_len = n_l/CPL_SIZE
      allocate(redu3(block_len*(nFala+nSa)))
      allocate(redu4(n_l*(nFala+nSa)))
      redu3=0.
      redu4=0.
      iv = 1
      do i = 1, nSa
          do n = ist_l,iend_l
          redu3(iv) = Sa_l(n,i)
          iv = iv+1
          enddo
      enddo
!
      do i = 1, nFala
          do n = ist_l,iend_l
          redu3(iv) = Fala_l(n,i)
          iv = iv+1
          enddo
      enddo
      sendcnt = iv-1
      recvcnt = iv-1
      call MPI_allgather(redu3,sendcnt,MPI_REAL8,&
           &  redu4,recvcnt,MPI_REAL8,MPI_COMM_CPL,ierr)
      offset4 = 0
      do i = 1, nSa
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_len*(nFala+nSa)*(iblock-1)
         do j = 1,block_len
         Sa_l(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
      offset4 = block_len *nSa
      do i = 1, nFala
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_len*(nFala+nSa)*(iblock-1)
         do j = 1,block_len
         Fala_l(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
      deallocate(redu3)
      deallocate(redu4)
!
#else
    real (SHR_KIND_R8), allocatable :: redu3(:,:),redu4(:,:)
      allocate(redu3(n_l,nFala+nSa))
      allocate(redu4(n_l,nFala+nSa))
      redu3=0.
      redu4=0.
      do i = 1, nSa
          do j = ist_l,iend_l
          redu3(j,i) = Sa_l(j,i)
          enddo
      enddo
      do i = 1, nFala
          do j = ist_l,iend_l
          redu3(j,i+nSa) = Fala_l(j,i)
          enddo
      enddo

      call MPI_ALLREDUCE(redu3, redu4, n_l*(nFala+nSa),&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)

      do i = 1, nSa
          Sa_l(:,i) = redu4(:,i)
      enddo
      do i = 1, nFala
          Fala_l(:,i) = redu4(:,i+nSa)
      enddo
      deallocate(redu3)
      deallocate(redu4)
#endif

  END SUBROUTINE comm3
!===============================================================================

  SUBROUTINE comm4(Sa_i,So_i,Faia_i,Fioo_i,roff_o)
    implicit none

    include "mpif.h"
    real  ,intent(inout) :: Sa_i  (n_i,nSa)
    real  ,intent(inout) :: So_i  (n_i,nSo)
    real  ,intent(inout) :: Faia_i(n_i,nFaia)
    real  ,intent(inout) :: Fioo_i(n_i,nFioo)
    real  ,intent(inout) :: roff_o(n_o ,1)
    integer i,j,k,ierr,iv,sendcnt,recvcnt
    integer offset1,offset2,offset3,offset4
    integer iblock,block_len,block_strid

#if 1
    real (SHR_KIND_R8), allocatable :: redu3(:),redu4(:)
      j = nSa+nSo+nFaia+nFioo+1
!      k = n_i*(nSa+nSo+nFaia+nFioo) + n_i *1
      block_len = n_i/CPL_SIZE
      allocate(redu3(block_len*j))
      allocate(redu4(n_i*j))
      redu3=0.
      redu4=0.
      iv = 1
      do i = 1, nSa
          do j = ist_i,iend_i
          redu3(iv) = Sa_i(j,i)
          iv = iv+1
          enddo
      enddo
!
      do i = 1, nSo
          do j = ist_i,iend_i
          redu3(iv) = So_i(j,i)
          iv = iv+1
          enddo
      enddo
!
      do i = 1, nFaia
          do j = ist_i,iend_i
          redu3(iv) = Faia_i(j,i)
          iv = iv+1
          enddo
      enddo
!
      do i = 1, nFioo
          do j = ist_i,iend_i
          redu3(iv) = Fioo_i(j,i)
          iv = iv+1
          enddo
      enddo

      do i = 1, 1
          do j = ist_i,iend_i
          redu3(iv) = roff_o(j,i)
          iv = iv+1
          enddo
      enddo

      sendcnt = iv-1
      recvcnt = iv-1
      call MPI_allgather(redu3,sendcnt,MPI_REAL8,&
           &  redu4,recvcnt,MPI_REAL8,MPI_COMM_CPL,ierr)

!!      k = n_i*(nSa+nSo+nFaia+nFioo) + n_i *1
      block_strid = block_len*(nSa+nSo+nFaia+nFioo+1)
     
      offset4 = 0
      do i = 1, nSa
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         Sa_i(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
      offset4 = block_len *nSa
      do i = 1, nSo
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         So_i(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
      offset4 = block_len *(nSa+nSo)
      do i = 1, nFaia
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         Faia_i(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
      offset4 = block_len *(nSa+nSo+nFaia)
      do i = 1, nFioo
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         Fioo_i(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
      offset4 = block_len *(nSa+nSo+nFaia+nFioo)
      do i = 1, 1
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         roff_o(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
#else
    real (SHR_KIND_R8), allocatable :: redu3(:,:),redu4(:,:)
      allocate(redu3(n_i,j))
      allocate(redu4(n_i,j))
!----------add roff_o for msg_ices,the size n_o=n_i -----------
      redu3=0.
      redu4=0.
      do i = 1, nSa
          do j = ist_i,iend_i
          redu3(j,i) = Sa_i(j,i)
          enddo
      enddo
!
      k = nSa
      do i = 1, nSo
          do j = ist_i,iend_i
          redu3(j,i+k) = So_i(j,i)
          enddo
      enddo
!
      k = nSa + nSo
      do i = 1, nFaia
          do j = ist_i,iend_i
          redu3(j,i+k) = Faia_i(j,i)
          enddo
      enddo
!
      k = nSa + nSo + nFaia
      do i = 1, nFioo
          do j = ist_i,iend_i
          redu3(j,i+k) = Fioo_i(j,i)
          enddo
      enddo

      k = nSa+nSo+nFaia+nFioo
      do i = 1, 1
          do j = ist_i,iend_i
          redu3(j,i+k) = roff_o(j,i)
          enddo
      enddo

      k = n_i*(nSa+nSo+nFaia+nFioo) + n_i *1

      call MPI_ALLREDUCE(redu3, redu4, k,&
           & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
!
      do i = 1, nSa
          Sa_i(:,i) = redu4(:,i)
      enddo

      k = nSa
      do i = 1, nSo
          So_i(:,i) = redu4(:,i+k)
      enddo

      k = nSa + nSo
      do i = 1, nFaia
          Faia_i(:,i) = redu4(:,i+k)
      enddo
!
      k = nSa + nSo + nFaia
      do i = 1, nFioo
          Fioo_i(:,i) = redu4(:,i+k)
      enddo

      k = nSa+nSo+nFaia+nFioo
      do i = 1, 1
          roff_o(:,i) = redu4(:,i+k)
      enddo
#endif
      deallocate(redu3)
      deallocate(redu4)

  END SUBROUTINE comm4

!===============================================================================
  SUBROUTINE comm5(Sa_o,So_o,Faoc_o)
    implicit none

    include "mpif.h"
    real  ,intent(inout) :: Sa_o  (n_o,nSa)
    real  ,intent(inout) :: So_o  (n_o,nSo)
    real  ,intent(inout) :: Faoc_o(n_o,nFaoc)
    integer i,j,k,ierr,iv,sendcnt,recvcnt
    integer offset1,offset2,offset3,offset4
    integer iblock,block_len,block_strid
#if 1
    real (SHR_KIND_R8), allocatable :: redu3(:),redu4(:)
      block_len = n_o/CPL_SIZE
      allocate(redu3(block_len*(nSa+nSo+nFaoc)))
      allocate(redu4(n_o*(nSa+nSo+nFaoc)))
      redu3=0.
      redu4=0.
      iv = 1
      do i = 1, nSa
          do j = ist_o,iend_o
          redu3(iv) = Sa_o(j,i)
          iv = iv+1
          enddo
      enddo
      do i = 1, nSo
          do j = ist_o,iend_o
          redu3(iv) = So_o(j,i)
          iv = iv+1
          enddo
      enddo
      do i = 1, nFaoc
          do j = ist_o,iend_o
          redu3(iv) = Faoc_o(j,i)
          iv = iv+1
          enddo
      enddo
      sendcnt = iv-1
      recvcnt = iv-1
      call MPI_allgather(redu3,sendcnt,MPI_REAL8,&
           &  redu4,recvcnt,MPI_REAL8,MPI_COMM_CPL,ierr)

      block_strid = block_len*(nSa+nSo+nFaoc)
     
      offset4 = 0
      do i = 1, nSa
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         Sa_o(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
      offset4 = block_len*nSa
      do i = 1, nSo
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         So_o(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
      offset4 = block_len*(nSa+nSo)
      do i = 1, nFaoc
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         Faoc_o(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
#else
    real (SHR_KIND_R8), allocatable :: redu3(:,:),redu4(:,:)
      allocate(redu3(n_o,nSa+nSo+nFaoc))
      allocate(redu4(n_o,nSa+nSo+nFaoc))
      redu3=0.
      redu4=0.
      do i = 1, nSa
          do j = ist_o,iend_o
          redu3(j,i) = Sa_o(j,i)
          enddo
      enddo
      do i = 1, nSo
          do j = ist_o,iend_o
          redu3(j,i+nSa) = So_o(j,i)
          enddo
      enddo
      do i = 1, nFaoc
          do j = ist_o,iend_o
          redu3(j,i+nSa+nSo) = Faoc_o(j,i)
          enddo
      enddo
      call MPI_ALLREDUCE(redu3, redu4, n_o*(nSa+nSo+nFaoc),&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)

      do i = 1, nSa
          Sa_o(:,i) = redu4(:,i)
      enddo
      do i = 1, nSo
          So_o(:,i) = redu4(:,i+nSa)
      enddo
      do i = 1, nFaoc
          Faoc_o(:,i) = redu4(:,i+nSa+nSo)
      enddo
#endif
       deallocate(redu3)
       deallocate(redu4)

  END SUBROUTINE comm5

!===============================================================================
  SUBROUTINE comm7(Foxx_o,Faoc_o)
    implicit none

    include "mpif.h"
    real  ,intent(inout) :: Foxx_o(n_o,nFoxx)
    real  ,intent(inout) :: Faoc_o(n_o,nFaoc)
    integer i,j,k,ierr,iv,sendcnt,recvcnt
    integer offset1,offset2,offset3,offset4
    integer iblock,block_len,block_strid
    real (SHR_KIND_R8), allocatable :: redu3(:),redu4(:)
  
      block_len = n_o/CPL_SIZE
      allocate(redu3(block_len*(nFoxx+nFaoc)))
      allocate(redu4(n_o*(nFoxx+nFaoc)))
      redu3=0.
      redu4=0.
      iv = 1
      do i = 1, nFoxx
          do j = ist_o,iend_o
          redu3(iv) = Foxx_o(j,i)
          iv = iv+1
          enddo
      enddo
      do i = 1, nFaoc
          do j = ist_o,iend_o
          redu3(iv) = Faoc_o(j,i)
          iv = iv+1
          enddo
      enddo
      sendcnt = iv-1
      recvcnt = iv-1
      call MPI_allgather(redu3,sendcnt,MPI_REAL8,&
           &  redu4,recvcnt,MPI_REAL8,MPI_COMM_CPL,ierr)
       block_strid = block_len*(nFoxx+nFaoc)

      offset4 = 0
      do i = 1, nFoxx
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         Foxx_o(offset1+j,i) = redu4(offset2+offset4+offset3+j)
         enddo
         enddo
      enddo
!
      offset4 = block_len * nFoxx
      do i = 1, nFaoc
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         Faoc_o(offset1+j,i) = redu4(offset2+offset4+offset3+j)
         enddo
         enddo
      enddo
!
       deallocate(redu3)
       deallocate(redu4)
      
  END SUBROUTINE comm7



!===============================================================================
  SUBROUTINE comm6(Ss_a,Faxx_a)
    implicit none

    include "mpif.h"
    real  ,intent(inout) :: Ss_a  (n_a,nSs)
    real  ,intent(inout) :: Faxx_a(n_a,nFaxx)
    integer i,j,k,ierr,iv,sendcnt,recvcnt
    integer offset1,offset2,offset3,offset4
    integer iblock,block_len,block_strid

#if 1
    real (SHR_KIND_R8), allocatable :: redu3(:),redu4(:)
      block_len = n_a/CPL_SIZE
      allocate(redu3(block_len*(nSs+nFaxx)))
      allocate(redu4(n_a*(nSs+nFaxx)))
      redu3=0.
      redu4=0.
      iv = 1
      do i = 1, nSs
          do j = ist_a,iend_a
          redu3(iv) = Ss_a(j,i)
          iv = iv+1
          enddo
      enddo
      do i = 1, nFaxx
          do j = ist_a,iend_a
          redu3(iv) = Faxx_a(j,i)
          iv = iv+1
          enddo
      enddo
      sendcnt = iv-1
      recvcnt = iv-1
      call MPI_allgather(redu3,sendcnt,MPI_REAL8,&
           &  redu4,recvcnt,MPI_REAL8,MPI_COMM_CPL,ierr)

      block_strid = block_len*(nSs+nFaxx)
     
      offset4 = 0
      do i = 1, nSs
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         Ss_a(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
      offset4 = block_len * nSs
      do i = 1, nFaxx
         offset3 = block_len*(i-1)
         do iblock=1,CPL_SIZE
         offset1 = block_len*(iblock-1)
         offset2 = block_strid*(iblock-1)
         do j = 1,block_len
         Faxx_a(offset1+j,i) = redu4(offset2+offset4+offset3+j) 
         enddo
         enddo
      enddo
!
#else
    real (SHR_KIND_R8), allocatable :: redu3(:,:),redu4(:,:)
      allocate(redu3(n_a,nSs+nFaxx))
      allocate(redu4(n_a,nSs+nFaxx))
      redu3=0.
      redu4=0.
      do i = 1, nSs
          do j = ist_a,iend_a
          redu3(j,i) = Ss_a(j,i)
          enddo
      enddo
      do i = 1, nFaxx
          do j = ist_a,iend_a
          redu3(j,i+nSs) = Faxx_a(j,i)
          enddo
      enddo

      call MPI_ALLREDUCE(redu3, redu4, n_a*(nSs+nFaxx),&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)

      do i = 1, nSs
          Ss_a(:,i) = redu4(:,i)
      enddo
      do i = 1, nFaxx
          Faxx_a(:,i) = redu4(:,i+nSs)
      enddo
#endif
      deallocate(redu3)
      deallocate(redu4)

  END SUBROUTINE comm6

!===============================================================================

END MODULE comm_mod
#endif
