!===============================================================================
! CVS $Id: control_mod.F90,v 1.12 2002/03/07 23:35:39 tcraig Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/control_mod.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

MODULE control_mod

   USE calendar_mod  ! access to calendar routines
   USE kind_mod      ! access to kind declarations
#ifdef MPI_P
   use mpmd_mod , only: master_task
#endif

!-------------------------------------------------------------------------------
! Note: for integers logicals, 0 <=> false
!-------------------------------------------------------------------------------

   integer,parameter :: tmpunit = 9 ! unit # reserved for temporary usage

   !----- case ID info ----------------------------------------------
   character(CLONG)  :: case_name   ! case name
   character(CLONG)  :: case_desc   ! case description

   !----- restart file selection ------------------------------------
   character(len=32) :: rest_type   ! run type: initial, cont, regen, branch
   character(CLONG)  :: rest_bfile  ! IC data file for branch runs
   character(CLONG)  :: rest_pfile  ! restart pointer file path & name
   integer           :: rest_date   ! start date for initial or branch runs
   character(CLONG)  :: rest_str    ! text string used to describe the run
   character(len=32) :: rest_freq   ! restart file frequency
   integer           :: rest_n      ! restart file interval    (nday option)
   integer           :: rest_odate  ! restart file offset date (nday option)

   !----- mapping matrix data files ---------------------------------
   character(CLONG)  :: map_a2of_fn ! map data file: atm  -> ocn fluxes
   character(CLONG)  :: map_a2lf_fn ! map data file: atm  -> lnd fluxes
   character(CLONG)  :: map_a2os_fn ! map data file: atm  -> ocn states
   character(CLONG)  :: map_o2af_fn ! map data file: ocn  -> atm fluxes
   character(CLONG)  :: map_r2o_fn  ! map data file: roff -> ocn runoff

   !----- stopping date selection -----------------------------------
   character(len=16) :: stop_option ! specify when to stop models
   integer           :: stop_date   ! date on which to stop the models
   integer           :: stop_n      ! number of days to advance the models

   !----- history file selection ------------------------------------
   character(len=16) :: hist_freq   ! specify when to create hist data
   character(len=16) :: hist_bundle ! specify bundling of hist data
   integer           :: hist_n      ! history file interval (wrt nday, nstep)
   integer           :: hist_odate  ! restart file offset date (wrt nday)
   integer           :: hist_tavg   ! T => request time-avg history data

   !----- mass store mswrite options --------------------------------
   character(CLONG)  :: mss_dir     ! MSS archive directory name
   character(len=16) :: mss_pass    ! MSS password
   character(len=64) :: mss_opts    ! MSS mswrite options
   integer           :: mss_rtpd    ! MSS retention period
   integer           :: mss_rmlf    ! T => remove local files after mswrite

   !-------information for the dataatm by fyj--------------------------
   character(len=64) :: data_dir    ! MSS mswrite options
   character(len=64) :: forc_dir    ! MSS mswrite options
   character(len=64) :: data_fname  ! MSS mswrite options

   !----- dbugging & timing info selection --------------------------
   integer           :: info_dbug   ! dbug level: > 0 => print extra info
   integer           :: info_time   ! T => print extra timing info
   integer           :: info_bcheck ! bitcheck level: > 0 => restart test

   !----- physics diagnostic info selection -------------------------
   character(len=16) :: diag_freq   ! specify when to create diagnostic data
   integer           :: diag_n      ! diag data interval (wrt nday )
   integer           :: diag_odate  ! diag data offset date (wrt nday)

   !----- flux related options --------------------------------------
   integer           :: flx_albav   ! T => NO diurnal cycle in albedos
   character(len=16) :: flx_epbal   ! flag useage of E,P,R adjustment factor
   integer           :: flx_epfac   ! E,P,R adjustment factor received from ocn

   !----- orbital related parameters --------------------------------
   real              :: orb_eccen   ! Eccentricity of earths orbit (unitless)
   real              :: orb_obliqr  ! Earth's obliquity in radians
   real              :: orb_lambm0  ! Mean longitude of perihelion at the
                                    ! vernal equinox (radians)
   real              :: orb_mvelpp  ! moving vernal equinox longitude
                                    ! of perihelion plus pi (radians)

   !----- initialization procedure control flags --------------------
   integer           :: init_solar  ! T => do solar flux initialization 
   integer           :: init_tavg   ! T => do daily avg flux initialization
   integer           :: init_maskl  ! T => do land domain mask initialization
   integer           :: init_rica   ! T => use restart IC's from atm
   integer           :: init_rici   ! T => use restart IC's from ice
   integer           :: init_ricl   ! T => use restart IC's from lnd
   integer           :: init_rico   ! T => use restart IC's from ocn

   !----- rest, stop, hist, diag control flags ----------------------
   integer           ::  stop_now   ! T => stop model now
   integer           ::  stop_eod   ! T => stop model at end of day
   integer           ::  rest_now   ! T => create restart data now
   integer           ::  rest_eod   ! T => create restart data at end of day
   integer           ::  hist_now   ! T => create history data now
   integer           ::  hist_eod   ! T => create history data at end of day
   integer           ::  hist_save  ! T => archive history data now
   integer           ::  diag_now   ! T => create diagnostic data now
   integer           ::  diag_eod   ! T => create diagnostic data at eod

   !----- timestep & model date info --------------------------------
   integer           ::  eday       ! elapsed days, 0 <=> 1 Jan, year 0, 0s
   integer           ::  eday0      ! value of eday  on startup
   integer           ::  edayf      ! value of eday  at end of run
   integer           ::  year       ! cpl year
   integer           ::  month      ! cpl month
   integer           ::  day        ! cpl day
   integer           ::  sec        ! cpl elapsed seconds during model day
   integer           ::  cdate      ! cpl year+month+date, coded: yyyymmdd
   integer           ::  ncpl_a     ! number of atm communications per day
   integer           ::  ncpl_i     ! number of ice communications per day
   integer           ::  ncpl_l     ! number of lnd communications per day
   integer           ::  ncpl_o     ! number of ocn communications per day
   integer           ::  ashift     ! shift albedo calc by ashift seconds
   integer           ::  atm_begin  ! start time for dataatm
   integer           ::  atm_end    ! end time for dataatm

   !----- global error code -----------------------------------------
   integer           ::  ierr       ! 0 <=> no errors or warnings
!--------------------wangln 2007-05-28 Add the Bilinear Correcting-----------
   logical           ::  bilinear   ! Y/N using the Bilinear Correcting
   character(len=3)  ::  ocn_type   ! using different Ocn 2007-06-20
   character(len=4)  ::  atm_type   ! using different Atm 2007-06-20
   integer           ::  albedo_type ! albedo option: MOM1=1, MOM2=2, MOM3=3, MOM4=4, CPL=5   
   logical           ::  land_correct   ! Y/N using the Wutw Correcting
   logical           ::  climate_runoff  ! Y/N using the climate Runoff for MOM
   logical           ::  carbon_flux     ! Y/N using the Carbon flux for MOM/SIS
   logical           ::  data_atm        ! Y/N using the dataatm 
!--------------------wangln 2007-05-28 Add the Bilinear Correcting-----------

CONTAINS

!
!===============================================================================

SUBROUTINE control()

   IMPLICIT none

   !----- formats -----
   character(len=*),parameter :: F00 ="('(control) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   Set control flags for stopping, creating restart & history data,
!   and doing stdout diagnostics.
!   These flags are used by the coupler, but are also sent to component 
!   models to facilitate the coordination of data sets
!
! Note:
!   o when integer values are used to implement boolean logic, 
!     zero <=> false, non-zero <=> true
!-------------------------------------------------------------------------------

   call control_stop()
   call control_rest()
   call control_hist()
   call control_diag()

   !-----------------------------------------------------------------
   ! print out the stop, rest, & hist flags??
   !-----------------------------------------------------------------
#ifdef MPI_P
     if(master_task) then
#endif
   if ( info_dbug .eq. 1 .and. sec.eq.0) then
     if (stop_now .ne. 0) write(6,F00) 'stop_now = true'
     if (stop_eod .ne. 0) write(6,F00) 'stop_eod = true'
     if (rest_now .ne. 0) write(6,F00) 'rest_now = true'
     if (rest_eod .ne. 0) write(6,F00) 'rest_eod = true'
     if (hist_now .ne. 0) write(6,F00) 'hist_now = true'
     if (hist_eod .ne. 0) write(6,F00) 'hist_eod = true'
     if (diag_now .ne. 0) write(6,F00) 'diag_now = true'
     if (diag_eod .ne. 0) write(6,F00) 'diag_eod = true'
   else if ((info_dbug.eq.2 .and. sec.eq.0) .or. info_dbug.ge.3) then
     if (stop_now .ne. 0) write(6,F00) 'stop_now = true'
     if (stop_now .eq. 0) write(6,F00) 'stop_now = false'
     if (stop_eod .ne. 0) write(6,F00) 'stop_eod = true'
     if (stop_eod .eq. 0) write(6,F00) 'stop_eod = false'
     if (rest_now .ne. 0) write(6,F00) 'rest_now = true'
     if (rest_now .eq. 0) write(6,F00) 'rest_now = false'
     if (rest_eod .ne. 0) write(6,F00) 'rest_eod = true'
     if (rest_eod .eq. 0) write(6,F00) 'rest_eod = false'
     if (hist_now .ne. 0) write(6,F00) 'hist_now = true'
     if (hist_now .eq. 0) write(6,F00) 'hist_now = false'
     if (hist_eod .ne. 0) write(6,F00) 'hist_eod = true'
     if (hist_eod .eq. 0) write(6,F00) 'hist_eod = false'
     if (diag_now .ne. 0) write(6,F00) 'diag_now = true'
     if (diag_now .eq. 0) write(6,F00) 'diag_now = false'
     if (diag_eod .ne. 0) write(6,F00) 'diag_eod = true'
     if (diag_eod .eq. 0) write(6,F00) 'diag_eod = false'
   end if
#ifdef MPI_P
     endif
#endif
!xjx   print *,"xjx.......control..info_dbug,sec=",info_dbug,sec
!xjx   print *,"xjx.......control..hist_save,diag_now=",hist_save,diag_now

END SUBROUTINE control

!===============================================================================

SUBROUTINE control_diag()

   IMPLICIT none

   !--- local ---
   integer :: true  = 1             ! 1  => true
   integer :: false = 0             ! 0 <=> false
   integer :: diag_oeday            ! eday wrt diag_odate
   integer :: first_call = 1        ! flags 1st time initializations
   integer :: n                     ! time step within current day

   save

   !--- formats ---
   character(len=*),parameter :: F00 ="('(control_diag) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   Set control flags wrt the creation of runtime diagnostics
!
! ASSUMPTIONS:
!   this routine is called once at every time step
!
! Note:
!   o when integer values are used to implement boolean logic, 
!     zero <=> false, non-zero <=> true
!-------------------------------------------------------------------------------

   if ( diag_freq(1:5).eq.'never' ) then
     diag_now = false
     diag_eod = false 
     RETURN
   end if

   !-----------------------------------------------------------------
   ! do one-time calculations
   !-----------------------------------------------------------------
   if ( first_call .eq. true ) then

     !--- compute offset eday from offset date ---
     if (diag_freq(1:4).eq.'nday') then
       if (diag_odate .le. 0) then
          !--- every n days relative to start of run ---
          diag_oeday=eday0  
       else if (diag_odate .eq. 1) then
          !--- every n days relative to 1st of month ---
          diag_oeday=-1
       else if ( valid_date(diag_odate) ) then
          !--- every n days relative to given date ---
          call date2eday(diag_odate,diag_oeday)
       else 
          write(6,F00) 'ERROR: bad diag_odate=',diag_odate
          STOP 
       end if
     end if

     first_call = false
   end if

   diag_eod = false !--- no advanced warning for diagnostics

   !-----------------------------------------------------------------
   ! at least one sample per day
   !-----------------------------------------------------------------
   if ( diag_freq(1:5) .eq. 'nstep' ) then
     if ( sec .eq. 0 ) n = 0
     if ( sec .ne. 0 ) n = n+1
     diag_now = false
     if ( mod(n,diag_n) .eq. 0 ) diag_now = true         

   !-----------------------------------------------------------------
   ! four samples per day (at most)
   !-----------------------------------------------------------------
   else if ( mod(sec,21600) .ne. 0 ) then
     diag_now = false
   else
     diag_now = false
     if      ( diag_freq(1:5).eq.'daily'  ) then
       diag_now = true
     else if ( diag_freq(1:4).eq.'week'   ) then
       if (day .eq. 1) diag_now = true
       if (day .eq. 8) diag_now = true
       if (day .eq.15) diag_now = true
       if (day .eq.22) diag_now = true
     else if ( diag_freq(1:6).eq.'biweek' ) then
       if (day .eq. 1) diag_now = true
       if (day .eq.15) diag_now = true
     else if ( diag_freq(1:5).eq.'month'  ) then
       if (day .eq. 1) diag_now = true
     else if ( diag_freq(1:5).eq.'quart'  ) then
       if (month .eq. 1 .and. day .eq.1 ) diag_now = true
       if (month .eq. 4 .and. day .eq.1 ) diag_now = true
       if (month .eq. 7 .and. day .eq.1 ) diag_now = true
       if (month .eq.10 .and. day .eq.1 ) diag_now = true
     else if ( diag_freq(1:4).eq.'year'   ) then
       if (month .eq. 1 .and. day .eq.1 ) diag_now = true
     else if ( diag_freq(1:4).eq.'nday'   ) then
       if ( diag_oeday.lt.0) then
         !--- every n days relative to 1st day of month ---
         if ( mod(day - 1         ,diag_n).eq.0) diag_now = true
       else 
         !--- every n days relative to some offset date ---
         if (mod(eday - diag_oeday,diag_n).eq.0) diag_now = true
       end if
     else
       write(6,F00) 'ERROR: bad diag_freq=',diag_freq
       STOP
     end if
     !--- diag_now > 1 => also print running means ---
     if (month.eq.12 .and. day.eq.31 .and. sec.eq.0) diag_now = 2
     if (       stop_eod.ne.0        .and. sec.eq.0) diag_now = 2
   end if

   !--- special option for debugging ---
   if ( info_dbug.gt.3 .and. cdate.ge.info_dbug) diag_now = true

END SUBROUTINE control_diag

!
!===============================================================================

SUBROUTINE control_hist()

   IMPLICIT none

   !--- local ---
   integer :: true  = 1             ! 1  => true
   integer :: false = 0             ! 0 <=> false
   integer :: first_call = 1        ! flags 1st time initializations
   integer :: year2,month2,day2     ! tomorrow's year,month,day
   integer :: hist_oeday            ! eday wrt hist_odate
   integer :: n                     ! time step within current day

   save

   !--- formats ---
   character(len=*),parameter :: F00 ="('(control_hist) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   Set control flags wrt the creation & archiving of history files
!
! ASSUMPTIONS:
!   this routine is called once at every time step
!
! Note:
!   o when integer values are used to implement boolean logic, 
!     zero <=> false, non-zero <=> true
!-------------------------------------------------------------------------------

   !-----------------------------------------------------------------
   ! do one-time calculations
   !-----------------------------------------------------------------
   if ( first_call .eq. true ) then

     !--- compute offset eday from offset date ---
     if (hist_freq(1:4).eq.'nday') then
       if (hist_odate .le. 0) then
          !--- every n days relative to start of run ---
          hist_oeday=eday0  
       else if (hist_odate .eq. 1) then
          !--- every n days relative to 1st of month ---
          hist_oeday=-1
       else if ( valid_date(hist_odate) ) then
          !--- every n days relative to given date ---
          call date2eday(hist_odate,hist_oeday)
       else if ( hist_freq(1:5).ne.'never'   ) then
          write(6,F00) 'ERROR: bad hist_odate=',hist_odate
          STOP 
       end if
     end if

     first_call = false
   end if

   !-----------------------------------------------------------------
   ! at least one sample per day
   !-----------------------------------------------------------------
   if ( hist_freq(1:5) .eq. 'nstep' ) then
     if ( sec .eq. 0 ) n = 0
     if ( sec .ne. 0 ) n = n+1
     hist_now = false         
     hist_eod = true         
     if ( mod(n,hist_n).eq.0 ) hist_now = true         

   !-----------------------------------------------------------------
   ! at most one sample per day, and only if seconds = 0
   !-----------------------------------------------------------------
   else if ( sec .ne. 0 ) then
     hist_now = false
   else
     call eday2ymd (eday+1,year2,month2,day2) 
     hist_now = false         
     hist_eod = false         
     if      ( hist_freq(1:5).eq.'daily'  ) then
       hist_now = true
       hist_eod = true
     else if ( hist_freq(1:4).eq.'week'   ) then
       if (day .eq. 1) hist_now = true
       if (day .eq. 8) hist_now = true
       if (day .eq.15) hist_now = true
       if (day .eq.22) hist_now = true
       if (day2.eq. 1) hist_eod = true
       if (day2.eq. 8) hist_eod = true
       if (day2.eq.15) hist_eod = true
       if (day2.eq.22) hist_eod = true
     else if ( hist_freq(1:6).eq.'biweek' ) then
       if (day .eq. 1) hist_now = true
       if (day .eq.15) hist_now = true
       if (day2.eq. 1) hist_eod = true
       if (day2.eq.15) hist_eod = true
     else if ( hist_freq(1:5).eq.'month'  ) then
       if (day .eq. 1) hist_now = true
       if (day2.eq. 1) hist_eod = true
     else if ( hist_freq(1:5).eq.'quart'  ) then
       if (month .eq. 1 .and. day .eq.1 ) hist_now = true
       if (month .eq. 4 .and. day .eq.1 ) hist_now = true
       if (month .eq. 7 .and. day .eq.1 ) hist_now = true
       if (month .eq.10 .and. day .eq.1 ) hist_now = true
       if (month2.eq. 1 .and. day2.eq.1 ) hist_eod = true
       if (month2.eq. 4 .and. day2.eq.1 ) hist_eod = true
       if (month2.eq. 7 .and. day2.eq.1 ) hist_eod = true
       if (month2.eq.10 .and. day2.eq.1 ) hist_eod = true
     else if ( hist_freq(1:4).eq.'year'   ) then
       if (month .eq. 1 .and. day .eq.1 ) hist_now = true
       if (month2.eq. 1 .and. day2.eq.1 ) hist_eod = true
     else if ( hist_freq(1:4).eq.'nday'   ) then
       if ( hist_oeday.lt.0) then
         !--- every n days relative to 1st day of month ---
         if ( mod(day -1,hist_n).eq.0) hist_now = true
         if ( mod(day2-1,hist_n).eq.0) hist_eod = true
       else 
         !--- every n days relative to some offset date ---
         if (mod(eday  -hist_oeday,hist_n).eq.0) hist_now = true
         if (mod(eday+1-hist_oeday,hist_n).eq.0) hist_eod = true
       end if
     else if ( hist_freq(1:5).ne.'never'   ) then
       write(6,F00) 'ERROR: bad hist_freq=',hist_freq
       STOP
     end if
   end if

   !--- don't save data on the final time step ---
   if ( stop_now .ne. 0 ) hist_now = false

   !--- special option for debugging ---
   if ( info_dbug.ge.3) then
      if (cdate .ge. info_dbug) hist_now = true
      hist_eod = true
   end if

   !-----------------------------------------------------------
   ! archive history data now?
   !-----------------------------------------------------------
   hist_save = false
   if ( stop_now .ne. 0 ) then
     hist_save = true
   else if ( hist_bundle(1:4).eq."dail" ) then
     if (    sec.eq.0)  hist_save = true
   else if ( hist_bundle(1:4).eq."mont" ) then
     if (day+sec.eq.1) hist_save = true
   else if ( hist_bundle(1:4).eq."quar" ) then
     if (month.eq. 1 .and. day+sec.eq.1) hist_save = true
     if (month.eq. 4 .and. day+sec.eq.1) hist_save = true
     if (month.eq. 7 .and. day+sec.eq.1) hist_save = true
     if (month.eq.10 .and. day+sec.eq.1) hist_save = true
   else if ( hist_bundle(1:4).eq."year" ) then
     if (month+day+sec.eq.2) hist_save = true
   else 
     !--- default: no bundling at all => always save ---
     hist_save = true
   end if

END SUBROUTINE control_hist

!
!===============================================================================

   SUBROUTINE control_rest()

   IMPLICIT none

   !--- local ---
   integer :: true  = 1             ! 1  => true
   integer :: false = 0             ! 0 <=> false
   integer :: first_call = 1        ! flags 1st time initializations
   integer :: year2,month2,day2     ! tomorrow's year,month,day
   integer :: rest_oeday            ! eday wrt rest_odate

   save

   !--- formats ---
   character(len=*),parameter :: F00 ="('(control_rest) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   Set control flags wrt restart file creation
!
! Note:
!   o when integer values are used to implement boolean logic, 
!     zero <=> false, non-zero <=> true
!-------------------------------------------------------------------------------

   !-----------------------------------------------------------------
   ! do one-time calculations
   !-----------------------------------------------------------------
   if ( first_call .eq. true ) then

     !-------------------------------------------------------
     ! compute restart file offset timestep from offset date
     !-------------------------------------------------------
     if (rest_freq(1:4).eq.'nday') then
       if (rest_odate .le. 0) then
          rest_oeday=eday0
       else if (valid_date(rest_odate)) then
          call date2eday(rest_odate,rest_oeday)
       else
          write(6,*) '(control) ERROR: bad rest_odate=',rest_odate
          stop 'rest_odate'
       end if
     end if

     first_call = false
     rest_now   = false         
     rest_eod   = false         
   end if

   !-----------------------------------------------------------------
   ! restart data is only created when sec=0
   !-----------------------------------------------------------------
   if ( sec .ne. 0 ) then
     rest_now  = false         

   !-----------------------------------------------------------------
   ! it's a new day (sec=0), re-do the calcuations
   !-----------------------------------------------------------------
   else
     rest_now  = false         
     rest_eod  = false         

     !-------------------------------------------------------
     ! what is tomorrow's year, month, & day??
     !-------------------------------------------------------
     call eday2ymd (eday+1,year2,month2,day2) 

     !-------------------------------------------------------
     ! make restart data now or at the end of this day??
     !-------------------------------------------------------
     if      (rest_freq(1:5) .eq. 'month' ) then
       if (                   day .eq.1 ) rest_now = true
       if (                   day2.eq.1 ) rest_eod = true
     else if (rest_freq(1:5) .eq. 'quart' ) then
       if (month .eq. 1 .and. day .eq.1 ) rest_now = true
       if (month .eq. 4 .and. day .eq.1 ) rest_now = true
       if (month .eq. 7 .and. day .eq.1 ) rest_now = true
       if (month .eq.10 .and. day .eq.1 ) rest_now = true
       if (month2.eq. 1 .and. day2.eq.1 ) rest_eod = true
       if (month2.eq. 4 .and. day2.eq.1 ) rest_eod = true
       if (month2.eq. 7 .and. day2.eq.1 ) rest_eod = true
       if (month2.eq.10 .and. day2.eq.1 ) rest_eod = true
     else if (rest_freq(1:4) .eq. 'half'  ) then
       if (month .eq. 1 .and. day .eq.1 ) rest_now = true
       if (month .eq. 7 .and. day .eq.1 ) rest_now = true
       if (month2.eq. 1 .and. day2.eq.1 ) rest_eod = true
       if (month2.eq. 7 .and. day2.eq.1 ) rest_eod = true
     else if (rest_freq(1:4) .eq. 'year'  ) then
       if (month .eq. 1 .and. day .eq.1 ) rest_now = true
       if (month2.eq. 1 .and. day2.eq.1 ) rest_eod = true
     else if (rest_freq(1:4) .eq. 'nday'  ) then
       if (mod(eday  -rest_oeday,rest_n).eq.0) rest_now =true
       if (mod(eday+1-rest_oeday,rest_n).eq.0) rest_eod =true
     else
       write(6,F00) 'ERROR: bad rest_freq=',rest_freq
       STOP
     end if
     if (eday .eq. eday0) rest_now  = false ! run is just starting

   end if

   END SUBROUTINE control_rest

!
!===============================================================================

SUBROUTINE control_stop()

   IMPLICIT none

   !--- local ---
   integer :: true  = 1             ! 1  => true
   integer :: false = 0             ! 0 <=> false
   integer :: first_call = 1        ! flags 1st time initializations
   integer :: year2,month2,day2     ! tomorrow's year,month,day

   save

   character(len=*),parameter :: F00 = "('(control_stop) ',2a)"
   character(len=*),parameter :: F01 = "('(control_stop) will advance ', &
   &                                    i4,' days, stop on ',i8.8)"
   character(len=*),parameter :: F02 = "('(control_stop) ',a,i8.8)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   Set control flags wrt when the integration stops
!
! Note:
!   o when integer values are used to implement boolean logic, 
!     zero <=> false, non-zero <=> true
!-------------------------------------------------------------------------------

   !-----------------------------------------------------------------
   ! do one-time calculations
   !-----------------------------------------------------------------
   if ( first_call .eq. true ) then

     !-------------------------------------------------------
     ! compute edayf - value of eday when integration stops
     !-------------------------------------------------------
     IF (stop_option(1:4).eq.'date') THEN
        !-- stop on given date --
        if (.not. valid_date(stop_date) ) then
          write(6,F02) 'invalid stop_date = ',stop_date
          STOP 'control'
        end if
        call date2eday(stop_date,edayf)
     ELSE IF (stop_option(1:4).eq.'nday') THEN
        !-- stop after n days --
        edayf=eday0+stop_n
        call eday2date(edayf,stop_date)
     ELSE IF (stop_option(1:6).eq.'nmonth') THEN
        !-- stop on day 1, n months from this month --
        month2 =     mod(month+stop_n-1,12)+1
        year2  = year + (month+stop_n-1)/12
        call ymd2eday(year2, month2, 1, edayf)
        call eday2date(edayf,stop_date)
     ELSE IF (stop_option(1:8).eq.'newmonth') THEN
        !-- stop on day 1 of next month -- 
        if (month.eq.12) call ymd2eday(year+1,      1,1,edayf)
        if (month.ne.12) call ymd2eday(year  ,month+1,1,edayf)
        call eday2date(edayf,stop_date)
     ELSE IF (stop_option(1:8).eq.'halfyear') THEN
        !-- stop on next Jul or Jan 1 -- 
        if (month > 6)  call ymd2eday(year+1,1,1,edayf)
        if (month < 7)  call ymd2eday(year  ,7,1,edayf)
        call eday2date(edayf,stop_date)
     ELSE IF (stop_option(1:7).eq.'newyear') THEN
        !-- stop on January 1 of next year -- 
        call ymd2eday(year+1,1,1,edayf)
        call eday2date(edayf,stop_date)
     ELSE IF (stop_option(1:9).eq.'newdecade') THEN
        !-- stop on January 1 of next decade --
        call ymd2eday(((year/10)+1)*10,1,1,edayf)
        call eday2date(edayf,stop_date)
     ELSE 
        write(6,F00) 'invalid stop_option = ',stop_option
        STOP 'control'
     END IF
     write(6,F01) edayf-eday0,stop_date

     stop_now   = true         
     stop_eod   = false         
     first_call = false
   end if

   if ( sec .ne. 0 ) then
     stop_now = false         
   else
     stop_now = false         
     stop_eod = false         
     call eday2ymd (eday+1,year2,month2,day2) 
     if (  eday    .ge. edayf)  stop_now  = true         
     if ( (eday+1) .eq. edayf)  stop_eod  = true
   end if

END SUBROUTINE control_stop

!===============================================================================

END MODULE control_mod
