module cpl_mod    


   use mpmd_mod
   use shr_kind_mod, only: SHR_KIND_IN, SHR_KIND_R8       ! defines real & integer kinds

   implicit none

   integer(SHR_KIND_IN), public :: cpl_tid  !SHR_MSG_TID_CPL   ! cpl model task id
   integer(SHR_KIND_IN), public :: atm_tid  !SHR_MSG_TID_ATM   ! atm model task id
   integer(SHR_KIND_IN), public :: ocn_tid  !SHR_MSG_TID_OCN   ! ocn model task id
   integer(SHR_KIND_IN), public :: ice_tid  !SHR_MSG_TID_ICE   ! ice model task id
   integer(SHR_KIND_IN), public :: lnd_tid  !SHR_MSG_TID_LND   ! lnd model task id
 
!!   integer            :: coupled_freq 
!!   integer            :: ocean_step 
!!   integer            :: ocean_days, ocean_seconds
!!   logical            :: first_call = .true.
    
!!   integer, parameter :: nsfields     = 22    ! number of fields sent  to   coupler
!!   integer, parameter :: nrfields     = 25    ! number of fields recvd from coupler 
!!   integer, parameter :: nibuff       = 100   ! size of integer control buffe
!!   integer, parameter :: msgtype_c2ii = 31    ! message type for cpl->ice init 
!!   integer, parameter :: msgtype_i2ci = 40    ! message type for ice->cpl init 
!!   integer, parameter :: msgtype_c2i  = 30    ! message type for cpl->ice 
!!   integer, parameter :: msgtype_i2c  = 41     ! message type for ice->cpl 
   
   

!!   real(kind=8)                     :: tlast_coupled
!!   real(kind=8)                     :: wln_radian
    
!!   real(kind=8), public, allocatable :: sendfields( :,:,: ) 
!!   real(kind=8), public, allocatable :: recvfields( :,:,: ) 
!!   integer, public  :: ibuff( nibuff )
      
!!   logical, public       :: cpl_stop_now           ! flag id for stop_now flag
!!   logical, public       :: sis_csim               ! flag id for Sea Ice model type : True is SIS , False is CSIM  
!!   logical, public       :: ex_ice_ocn             ! flag id From SIS to MOM exchange 
!!   logical, public       :: ex_ocn_ice             ! flag id MOM to SIS  exchange 
!!   real(kind=8), public  :: dt_evp                 ! Time parameter for Dhdx & dhdy
!!   real(kind=8), public  :: slow_freq              ! slow process frequency
   

!xjx   integer            :: cpl_task !  task id for coupler
!!   integer, private   :: cpl_ts                ! flag id for coupled_ts flag           
!!   integer, private   :: cpl_write_restart     ! flag id for write restart           
!!   integer, private   :: cpl_write_history     ! flag id for write history           
!!   integer, private   :: cpl_write_tavg        ! flag id for write tavg           
!!   integer, private   :: cpl_diag_global       ! flag id for computing diagnostics           
!!   integer, private   :: cpl_diag_transp       ! flag id for computing diagnostics

!!   real(kind=8), private, allocatable   :: xc_i(:,:)    ! longitude of grid center
!!   real(kind=8), private, allocatable   :: yc_i(:,:)    ! latitude of grid center
!!   real(kind=8), private, allocatable   :: xv_i(:,:,:)  ! longitude of grid corner 
!!   real(kind=8), private, allocatable   :: yv_i(:,:,:)  ! latitude of grid corner 
!!   real(kind=8), private, allocatable   :: area_i(:,:)  ! area of grid 
!!   integer     , private, allocatable   :: mask_i(:,:)  ! mask of grid center

!!   real(kind=8), public, allocatable   :: global_sin(:,:)    ! Sin of the grid angle
!!   real(kind=8), public, allocatable   :: global_cos(:,:)    ! Cos of the grid angle 


contains
!======================================================================================================
  subroutine setup_mpi

   implicit none
   include 'mpif.h'   
! MPI Fortran include file
  

   integer    :: ierr

   call mpi_init ( ierr)
   if( ierr == MPI_SUCCESS ) write( *,*) ' MPI_INIT is OK !'

   call mpi_coupled
   write(6,*) ' Coupled Initialied is OK! '

   !-----set up the CPL Flag

   master_task = .true.
   
   write(6,*) ' Setup_mpi is Ok!'

 end subroutine setup_mpi  
!-----------------------------------------------------------------------------------------------------
  subroutine mpi_coupled

    implicit none
    include 'mpif.h'   
! MPI Fortran include file


    logical          :: mpi_running
    character(len=3) :: cmodel   ! model name temporary
    integer          :: range(3) ! array for creating groups for each coupled model

    integer       :: MPI_GROUP_WORLD        ! group id for MPI_COMM_WORLD
    integer       :: MPI_GROUP_CPL          ! group of processors assigned to cpl
    integer       :: MPI_GROUP_ATM          ! group of processors assigned to atm
    integer       :: MPI_GROUP_OCN          ! group of processors assigned to ocn
    integer       :: MPI_GROUP_ICE          ! group of processors assigned to ice
    integer       :: MPI_GROUP_LND           ! group of processors assigned to lnd

    integer       :: n                           ! dummy loop counter
    integer       :: ierr                        ! error flag for MPI comms
    integer       :: nprocs_cpl                  ! total processor count
    integer       :: my_task_coupled             ! rank of process in coupled domain
    integer       :: cpl_rank_min, cpl_rank_max  ! processor range for each 
    integer       :: ocn_rank_min, ocn_rank_max  !  component model
    integer       :: atm_rank_min, atm_rank_max
    integer       :: ice_rank_min, ice_rank_max
    integer       :: lnd_rank_min, lnd_rank_max
!xjx    integer       :: atm_tid          ! task id for the process in each component
!xjx    integer       :: ice_tid          !   which communicates w coupler - these are
!xjx    integer       :: lnd_tid          !   defined as the lowest rank process
!xjx    integer       :: ocn_tid          !   in the processor group
    integer       :: MPI_COMM_ATM     ! MPI communicator to be used for
    integer       :: MPI_COMM_LND      !   these must be defined by all models even though only used internally by each model
    integer       :: MPI_COMM_ICE     !   messages internal to the component model
    integer       :: MPI_COMM_OCN      !   these must be defined by all models even though only used internally by each model
    
  
       !call mpi_initialized(mpi_running, ierr)
       
       !if( .not. mpi_running )  call mpi_init( ierr)

       call MPI_COMM_RANK(MPI_COMM_WORLD, my_task_coupled, ierr)
       write(6,*) 'establish intra-model communicators, my mpi_comm_world rank is ',my_task_coupled

       call MPI_COMM_SIZE(MPI_COMM_WORLD, nprocs_cpl, ierr)
       
       atm_rank_min = nprocs_cpl
       atm_rank_max = 0
       ocn_rank_min = nprocs_cpl
       ocn_rank_max = 0
       ice_rank_min = nprocs_cpl
       ice_rank_max = 0
       lnd_rank_min = nprocs_cpl
       lnd_rank_max = 0
       cpl_rank_min = nprocs_cpl
       cpl_rank_max = 0

       !*** each processor broadcasts its model to all the processors in the coupled domain

       do n=0,nprocs_cpl-1
        if (n == my_task_coupled) then
          cmodel = 'cpl'
        else
          cmodel = 'unk'
        endif

        call MPI_BCAST(cmodel, 3, MPI_CHARACTER, n, MPI_COMM_WORLD, ierr)

         select case(cmodel)
        case ('atm')
          atm_rank_min = min(atm_rank_min, n)
          atm_rank_max = max(atm_rank_max, n)
        case ('ocn')
          ocn_rank_min = min(ocn_rank_min, n)
          ocn_rank_max = max(ocn_rank_max, n)
        case ('ice')
          ice_rank_min = min(ice_rank_min, n)
          ice_rank_max = max(ice_rank_max, n)
        case ('lnd')
          lnd_rank_min = min(lnd_rank_min, n)
          lnd_rank_max = max(lnd_rank_max, n)
        case ('cpl')
          cpl_rank_min = min(cpl_rank_min, n)
          cpl_rank_max = max(cpl_rank_max, n)
        case ('dud')
        case default
          write(6,*) 'Unknown model ' ,cmodel,' in coupled model domain'
          write(6,*) 'Model must be atm, cpl, ice, lnd, ocn'
          stop
        end select

      end do
       

      !     assume the first processor in each domain is the task that will communicate coupled model messages

      atm_tid  = atm_rank_min
      cpl_tid  = cpl_rank_min
      ice_tid  = ice_rank_min
      lnd_tid  = lnd_rank_min
      ocn_tid  = ocn_rank_min

      !     create subroup and communicators for each models internal
      !     communciations, note that MPI_COMM_CREATE must be called by
      !     all processes in MPI_COMM_WORLD so this must be done by all
      !     models consistently and in the same order.

      call MPI_COMM_GROUP(MPI_COMM_WORLD, MPI_GROUP_WORLD, ierr)
 !     write(6,*) 'MPI_COMM_GROUP: MPI_COMM_WORLD, MPI_GROUP_WORLD = ', MPI_COMM_WORLD, MPI_GROUP_WORLD

      range(3) = 1
      range(1) = atm_rank_min
      range(2) = atm_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_ATM, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_ATM= ', MPI_GROUP_WORLD, MPI_GROUP_ATM

      range(1) = ocn_rank_min
      range(2) = ocn_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_OCN, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_OCN= ', MPI_GROUP_WORLD, MPI_GROUP_OCN

      range(1) = ice_rank_min
      range(2) = ice_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_ICE, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_ICE= ', MPI_GROUP_WORLD, MPI_GROUP_ICE

      range(1) = lnd_rank_min
      range(2) = lnd_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_LND, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_LND= ', MPI_GROUP_WORLD, MPI_GROUP_LND

      range(1) = cpl_rank_min
      range(2) = cpl_rank_max 
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_CPL, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_CPL= ', MPI_GROUP_WORLD, MPI_GROUP_CPL

      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_ATM, MPI_COMM_ATM, ierr)
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_OCN, MPI_COMM_OCN, ierr)
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_ICE, MPI_COMM_ICE, ierr)
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_LND, MPI_COMM_LND, ierr)
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_CPL, MPI_COMM_CPL, ierr)

      MPI_COMM_CPL_GROUP = MPI_GROUP_CPL

       print *,"MPI_GROUP_CPL,MPI_COMM_CPL=",MPI_GROUP_CPL,MPI_COMM_CPL

      !-----set up the Ocn Flag

      !master_task = .true.
      
  end subroutine mpi_coupled
!-----------------------------------------------------------------------------------------------------
!======================================================================================================
end module cpl_mod


