!===============================================================================
! CVS $Id: data_in_nc.F90,v 1.6 2002/06/19 15:23:08 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/datm5/data_in_nc.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

subroutine data_in_nc(dyear,dmonth,dday,dsec,k)

   use shr_sys_mod  ! shared code: system calls
   use shr_file_mod ! shared code: file archive
   use const_mod    ! physical constants
   use calendar_mod ! calendar routines
   use kind_mod     ! access to kind declarations
   use control_mod 
   use atmdata_mod 

   implicit none

   !----- global ------
#include <netcdf.inc>

   !----- arguments ------
   integer,intent(in) :: dyear    ! (in) data year  to read
   integer,intent(in) :: dmonth   ! (in) data month to read
   integer,intent(in) :: dday     ! (in) data day   to read
   integer,intent(in) :: dsec     ! (in) data sec   to read
   integer,intent(in) :: k        ! (in) raw data k index to be replaced

   !----- local ------
   character(CLONG)      :: mss_fn  ! MSS file name
   character(CLONG)      :: loc_fn  ! local file name
   character,allocatable :: str(:)  ! variable length char string
   character(CLONG)      :: attstr  ! netCDF attribute name string

   integer :: i,j,n                 ! generic indicies
   integer :: y,m,d                 ! year,month,day
   integer :: fid                   ! nc file ID
   integer :: vid                   ! nc variable ID
   integer :: did                   ! nc dimension ID
   integer :: beg(3)                ! nc hyperslab begin vector
   integer :: cnt(3)                ! nc hyperslab count vector
   integer :: ddate                 ! time coordinate (date) from data file
   integer :: rcode                 ! return code
   integer :: last_dyear = -999     ! year of last nc data read in
   logical :: first_call = .true.   ! flags 1st envocation of this routine
   integer :: nr,ntime  

   !--- used to derive fields missing from CAM2 history files ---
   real    :: RAIN  (nx_a,ny_a)         ! net large-scale precip (snow+rain)
   real    :: FSDS  (nx_a,ny_a)         ! net large-scale precip (snow+rain)
   real    :: PBOT  (nx_a,ny_a)         ! bottom level pressure
   real    :: PS    (nx_a,ny_a)         ! surface pressure
   integer :: lev                   ! bottom-most level of 3d data
   real    :: hyam                  ! "hybrid A coefficient at layer midpoint"
   real    :: hybm                  ! "hybrid B coefficient at layer midpoint"
   real    :: P0                    ! reference pressure

   !--- formats ---
   character(len=*),parameter :: F00 = "('(data_in_nc) ',4a)"
   character(len=*),parameter :: F01 = "('(data_in_nc) ',a,i4.4)"
   character(len=*),parameter :: F02 = "('(data_in_nc) ',a,i4.4,2('-',i2.2))"
   character(len=*),parameter :: F03 = "('(data_in_nc) ',a,2i5)"
   character(len=*),parameter :: F04 = "('(data_in_nc) o ',2a,1024(a1))"
   character(len=*),parameter :: F05 = "('(data_in_nc) return code',i4,5a)"
   character(len=*),parameter :: F10 = "('(data_in_nc) ',a,e12.3)"

   save    

!-------------------------------------------------------------------------------
! PURPOSE:
!   acquire a data file for a given year, and read data for a given month & day
!
! ASSUMPTIONS
!  o data file is from CCSM2.0/CAM2.0 history files (wrt missing fields and
!    deriving missing data)
!  o new data overwrites data in location data(:,:,k)
!  o each file has 365 daily mean fields
!  o replacing "yyyy" with the desired year data_fname yields a valid file name
!-------------------------------------------------------------------------------
   !----------------------------------------------------------------------------
   ! acquire & open a new data file
   !----------------------------------------------------------------------------
       if( first_call ) then
       i = len_trim(forc_dir)
       if (forc_dir(i:i) /= '/') forc_dir(i+1:i+1) = '/'

       write(6,*)'forc_dir=',trim(forc_dir)

       rcode = nf_open(trim(forc_dir) // 'basic.nc', NF_NOWRITE,fid)
       write(6,*) 'Fid =',  fid
        if (rcode /= NF_NOERR) then
         write(6,F00) 'ERROR: reading netCDF file, ',nf_strerror(rcode)
         call shr_sys_flush(6)
         call shr_sys_abort('datm: ERROR on nf_open')
        end if
       rcode = nf_inq_dimid (fid, 'lev', did)  ! size of dimension
       rcode = nf_inq_dimlen(fid,  did , lev)
       beg(1) = lev
       cnt(1) = 1
       rcode = nf_inq_varid(fid,'hyam'       ,vid)
       rcode = nf_get_vara_double(fid,vid,beg,cnt,hyam)
       rcode = nf_inq_varid(fid, 'hybm'       ,vid)
       rcode = nf_get_vara_double(fid,vid,beg,cnt,hybm)
       rcode = nf_inq_varid(fid,'P0'         ,vid)
       rcode = nf_get_var_double(fid,vid,P0)

       write(6,F03) "o lev    = ",lev
       write(6,F10) "o hyam   = ",hyam
       write(6,F10) "o hybm   = ",hybm
       write(6,F10) "o rdair  = ",rdair
       write(6,F10) "o cpdair = ",cpdair
       
       rcode = nf_close( fid )
      endif

   if (dyear /= last_dyear) then

     !----- close & remove an old file -----
      if (.not. first_call) then
           rcode = nf_close(fid)
      end if

      last_dyear = dyear

      write(6,F01) 'acquire file for data year = ',dyear

      !----- acquire & open a new file -----
      i = index(data_fname,'yyyy')
      if (i<1) then
         write(6,F00) 'ERROR: invalid data_fname = ',trim(data_fname)
         call shr_sys_flush(6)
         call shr_sys_abort('datm: invalid data_fname')
      end if
      loc_fn = data_fname
      write(loc_fn(i:i+3),'(i4.4)') dyear
      write(6,*)'loc_fn=',loc_fn

     i = len_trim(forc_dir)
     if (forc_dir(i:i) /= '/') forc_dir(i+1:i+1) = '/'
     write(6,*)'forc_dir=',trim(forc_dir)
     
     mss_fn = trim(forc_dir) // trim(loc_fn)
     call shr_file_get (rcode,loc_fn,mss_fn)
     write(6,F04) rcode,' on acquire ',trim(loc_fn),' ',trim(mss_fn)

     write(6,F00) 'open netCDF file ',loc_fn
     rcode = nf_open(loc_fn,NF_NOWRITE,fid)
       if (rcode /= NF_NOERR) then
           write(6,F00) 'ERROR: reading netCDF file, ',nf_strerror(rcode)
           call shr_sys_flush(6)
           call shr_sys_abort('datm: ERROR on nf_open')
       end if
    endif
   
   !----- compute time index for desired data sample -----
   call ymd2eday(0,dmonth,dday,n) ! each file has 4*365 data time samples
   ntime = n*4 + INT((dsec+10800)/21600.0)  
   write(6,*)'ntime=',ntime 

   !----- get the data -----
   beg(1) = 1
   beg(2) = 1
   beg(3) = ntime   ! day of year => 1460 times worth of data in data file
   cnt(1) = nx_a
   cnt(2) = ny_a
   cnt(3) = 1

   rcode = nf_inq_varid(fid,'UBOT'       ,vid)
   rcode = nf_get_vara_double(fid,vid,beg,cnt,r_u    (1,1,k))
  !write(6,*)'r_u    (1,1,k)=',r_u    (50,50,k)
   rcode = nf_inq_varid(fid,'VBOT'       ,vid)
   rcode = nf_get_vara_double(fid,vid,beg,cnt,r_v    (1,1,k))
   rcode = nf_inq_varid(fid,'TBOT'       ,vid)
   rcode = nf_get_vara_double(fid,vid,beg,cnt,r_t    (1,1,k))
  !write(6,*)'r_t    (1,1,k)=',r_t    (50,50,k)
   rcode = nf_inq_varid(fid,'SLP'        ,vid)
   rcode = nf_get_vara_double(fid,vid,beg,cnt,r_pslv (1,1,k))
  !write(6,*)'r_pslv (1,1,k)=',r_pslv (50,50,k)
   rcode = nf_inq_varid(fid,'QBOT'       ,vid)
   rcode = nf_get_vara_double(fid,vid,beg,cnt,r_shum (1,1,k))
   rcode = nf_inq_varid(fid,'FSDS'       ,vid)
   rcode = nf_get_vara_double(fid,vid,beg,cnt,FSDS(1,1))
  !write(6,*)'FSDS(1,1)=',FSDS(50,50)
      r_swndr(:,:,k) = FSDS(:,:)*0.35
      r_swvdr(:,:,k) = FSDS(:,:)*0.35
      r_swndf(:,:,k) = FSDS(:,:)*0.15
      r_swvdf(:,:,k) = FSDS(:,:)*0.15
      r_swnet(:,:,k) = FSDS(:,:)
   rcode = nf_inq_varid(fid,'RAIN'     ,vid)
   rcode = nf_get_vara_double(fid,vid,beg,cnt,RAIN(1,1))
      r_precc(:,:,k) = max(RAIN(:,:)*0.1,0.0)
      r_precl(:,:,k) = max(RAIN(:,:)*0.9,0.0)
   rcode = nf_inq_varid(fid,'FLDS'     ,vid)
   rcode = nf_get_vara_double(fid,vid,beg,cnt,r_lwdn(1,1,k))

   !----------------------------------------------------------------------------
   ! derive data for missing fields (and read in related data as necessary)
   !----------------------------------------------------------------------------
   rcode = nf_inq_varid(fid,'PSRF'         ,vid)
   rcode = nf_get_vara_double(fid,vid,beg,cnt,PS     (1,1))

   PBOT(:,:) = PS(:,:)*hybm + P0*hyam

   r_ptem (:,:,k) = r_t(:,:,k)*(PS(:,:)/PBOT(:,:))**(rdair/cpdair)
   r_pres (:,:,k) = PBOT
   r_dens (:,:,k) = PBOT/( rdair*r_t(:,:,k))

   !write(6,*)'r_dens (:,:,k)',r_dens (60,60,k),'k=',k
   !write(6,*)'r_ptem (:,:,k)',r_ptem (60,60,k),'k=',k
   !write(6,*)'r_pres (:,:,k)',r_pres (60,60,k),'k=',k
   !write(6,*)'r_lwdn (:,:,k)',r_lwdn (60,60,k),'k=',k

   !-------------------------------------------------------------------------
   ! convert CAM history file units to CCSM cpl/component convention
   !-------------------------------------------------------------------------
   if (first_call) then
      write(6,F00) 'Assuming all data file data has correct units except for'
      write(6,F00) '    precip, which will be converted from m/s to kg/s/m^2'
   end if

   first_call = .false.

RETURN
END

!===============================================================================
