!===============================================================================
! CVS $Id: flux_albedo.F90,v 1.6 2001/12/07 22:13:35 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/flux_albedo.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================


SUBROUTINE flux_albo( So )

   USE buf_mod
   USE control_mod
   USE shr_orb_mod
   USE shr_timer_mod ! timing routines
   use const_mod     ! provides physical constants

   implicit none

   real,parameter :: albdif     = 0.06 ! 60 deg ref albedo, diffuse
   real,parameter :: albdir     = 0.07 ! 60 deg ref albedo, direct 

   !----- arguments -----
   real    :: So(n_o,nSo)         ! ocn state buffer
 
   !----- local -----
   integer :: n                   ! loop index
   real    :: rlat                ! gridcell latitude in radians
   real    :: rlon                ! gridcell longitude in radians
   real    :: delta               ! Solar declination angle in radians
   real    :: eccf                ! Earth-sun distance factor
   real    :: cosz                ! Cosine of solar zenith angle
   real    :: calday              ! calendar day including fraction, at 0e
   real    :: calendar(12)        ! days of year past at beginning of each month
   logical :: first_call = .true. ! flags 1st envocation of this routine
   integer :: timer_n             ! system-clock-timer number

!--------wangln 2007-07-02 For the MOM 1st Albedo Scheme---------------
   real    :: trans, zen, dz, dt, dzdt
   integer :: iw , jw  
!--------wangln 2007-07-02 For the MOM 1st Albedo Scheme---------------

    real    :: const_alb           = 0.10

!   data used for option 1

    real                 :: albedo_data(21,20)
    real                 :: trn(21)
    real                 :: za(20)
    real                 :: dza(19)

 
!    data used for option 2

    real     :: alb2( n_o )
    !real,    allocatable :: xx(:)
    !integer, allocatable :: j1(:)

!==================================================================================

      data  albedo_data (1:21,1:7)                                     &
             / .061,.062,.072,.087,.115,.163,.235,.318,.395,.472,.542, &
     .604,.655,.693,.719,.732,.730,.681,.581,.453,.425,.061,.062,.070, &
     .083,.108,.145,.198,.263,.336,.415,.487,.547,.595,.631,.656,.670, &
     .652,.602,.494,.398,.370,.061,.061,.068,.079,.098,.130,.174,.228, &
     .290,.357,.424,.498,.556,.588,.603,.592,.556,.488,.393,.342,.325, &
     .061,.061,.065,.073,.086,.110,.150,.192,.248,.306,.360,.407,.444, &
     .469,.480,.474,.444,.386,.333,.301,.290,.061,.061,.065,.070,.082, &
     .101,.131,.168,.208,.252,.295,.331,.358,.375,.385,.377,.356,.320, &
     .288,.266,.255,.061,.061,.063,.068,.077,.092,.114,.143,.176,.210, &
     .242,.272,.288,.296,.300,.291,.273,.252,.237,.266,.220,.061,.061, &
     .062,.066,.072,.084,.103,.127,.151,.176,.198,.219,.236,.245,.250, &
     .246,.235,.222,.211,.205,.200 /

      data  albedo_data (1:21,8:14)                                    &
             / .061,.061,.061,.065,.071,.079,.094,.113,.134,.154,.173, &
     .185,.190,.193,.193,.190,.188,.185,.182,.180,.178,.061,.061,.061, &
     .064,.067,.072,.083,.099,.117,.135,.150,.160,.164,.165,.164,.162, &
     .160,.159,.158,.157,.157,.061,.061,.061,.062,.065,.068,.074,.084, &
     .097,.111,.121,.127,.130,.131,.131,.130,.129,.127,.126,.125,.122, &
     .061,.061,.061,.061,.062,.064,.070,.076,.085,.094,.101,.105,.107, &
     .106,.103,.100,.097,.096,.095,.095,.095,.061,.061,.061,.060,.061, &
     .062,.065,.070,.075,.081,.086,.089,.090,.088,.084,.080,.077,.075, &
     .074,.074,.074,.061,.061,.060,.060,.060,.061,.063,.065,.068,.072, &
     .076,.077,.076,.074,.071,.067,.064,.062,.061,.061,.061,.061,.061, &
     .060,.060,.060,.060,.061,.062,.065,.068,.069,.069,.068,.065,.061, &
     .058,.055,.054,.053,.052,.052 /

      data  albedo_data (1:21,15:20)                                   &
             / .061,.061,.060,.060,.060,.060,.060,.060,.062,.065,.065, &
     .063,.060,.057,.054,.050,.047,.046,.045,.044,.044,.061,.061,.060, &
     .060,.060,.059,.059,.059,.059,.059,.058,.055,.051,.047,.043,.039, &
     .035,.033,.032,.031,.031,.061,.061,.060,.060,.060,.059,.059,.058, &
     .057,.056,.054,.051,.047,.043,.039,.036,.033,.030,.028,.027,.026, &
     .061,.061,.060,.060,.060,.059,.059,.058,.057,.055,.052,.049,.045, &
     .040,.036,.032,.029,.027,.026,.025,.025,.061,.061,.060,.060,.060, &
     .059,.059,.058,.056,.053,.050,.046,.042,.038,.034,.031,.028,.026, &
     .025,.025,.025,.061,.061,.060,.060,.059,.058,.058,.057,.055,.053, &
     .050,.046,.042,.038,.034,.030,.028,.029,.025,.025,.025 /

      data  trn (1:21)                                             &
             /.00,.05,.10,.15,.20,.25,.30,.35,.40,.45,.50,.55,.60, &
              .65,.70,.75,.80,.85,.90,.95,1.00/

      data  za (1:20)                                               &
             / 90.,88.,86.,84.,82.,80.,78.,76.,74.,70.,66.,62.,58., &
               54.,50.,40.,30.,20.,10., 0. /

      data  dza (1:19) /8*2.0,6*4.0,5*10.0/


      data calendar /0,31,59,90,120,151,181,212,243,273,304,334/

   SAVE

   !----- formats -----
   character(len=*),parameter :: F00 = "('(flux_albo) ',3a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   if flux_albav/=0 (ie. "on" or "true")
!     Compute four effective daily avg surface albedos for all 
!     combinations of visible/near-infrared and direct/diffuse radiation
!     without accounting for zenith angle (ie. a "daily average" albedo)
!   else
!     Compute four surface albedos for all combinations of visible/
!     near-infrared and direct/diffuse radiation, accounting for 
!     instantaneous zenith angle
!
! NOTE:
!   o upon input, albedos are assumed to be a 60 degree reference albedo
!   o albedos are computed by taking the 60 deg reference albedo
!     and then adjusting this value based on zenith angle
!   o Albedos are independent of spectral interval and other physical
!     factors such as surface wind speed.
!
!   For more details see Briegleb, Bruce P., 1992: Delta-Eddington
!   Approximation for Solar Radiation in the NCAR Community Climate 
!   Model, Journal of Geophysical Research, Vol 97, D7, pp7603-7612.
!
! CODE HISTORY:
!   Original version:           CCM1
!   Standardized:               J. Rosinski, June 1992
!   Rewritten for land only:    J. Rosinski, May  1994
!   Rewritten for ocean only:   B. Kauffman, July 1994
!  
!-------------------------------------------------------------------------------

   !if (first_call) call shr_timer_get(timer_n,'flux_albo')
   !call shr_timer_start(timer_n)

!---------------wangln 2007-07-02 Add the 4 types MOM albedo Scheme---------------------------
    So( :, kSo_albedo ) = 0.0 
 if ( albedo_type == 1 ) then    ! MOM 1st Albedo Scheme 

    trans = 0.537
    calday = calendar(month) + day + (sec+ashift)/86400.0  !--- julian day ---
    call shr_orb_decl(calday,orb_eccen,orb_mvelpp,orb_lambm0,orb_obliqr,delta,eccf) !--- solar declination ---

#ifdef MPI_P
    do n = IST_o,IEND_o
#else
    do n = 1, n_o
#endif
        zen = 0.0
        if( xc_o(n) >= 0. ) then  
           rlon = DEG2RAD*xc_o(n)
        else
           rlon = DEG2RAD* (360. + xc_o(n) )
        endif 
        rlat = DEG2RAD*yc_o(n)
        cosz = shr_orb_cosz( calday, rlat, rlon, delta )
        cosz = max( 0.0, cosz )
        zen  = acos( cosz ) * RAD2DEG 
        iw   = 20. * trans + 1
        if( zen >= 74.0 )  then
          jw = 0.5 * ( 90- zen ) + 1.    
        else if( zen <  50.0 )  then
          jw = 0.1 * ( 50.- zen ) + 15.0    
        else if( zen <  74.0 .and. zen >= 50.0  )  then
          jw = 0.25 * ( 74- zen ) + 9.0    
        endif

        dz = - ( zen -za(jw) ) / dza( jw ) 
        dt = 20. * ( trans - trn(iw) )
        dzdt = dz * dt 
        So( n, kSo_albedo ) = albedo_data( iw,   jw   ) * ( 1. - dz - dt + dzdt ) + &  
                              albedo_data( iw+1, jw   ) * ( dt - dzdt ) +           &
                              albedo_data( iw,   jw+1 ) * ( dz - dzdt ) +           &
                              albedo_data( iw+1, jw+1 ) * dzdt
        if( cosz >=0 ) then
            So( n, kSo_anidr) = So( n, kSo_albedo )
            So( n, kSo_avsdr) = So( n, kSo_albedo )
            So(n,kSo_anidf) = albdif
            So(n,kSo_avsdf) = albdif
        else
            So(n,kSo_anidr) = 1.0
            So(n,kSo_avsdr) = 1.0
            So(n,kSo_anidf) = 1.0
            So(n,kSo_avsdf) = 1.0
        endif
    enddo 
        
    
 else if( first_call .and. ( albedo_type == 2)   ) then ! MOM 2nd Albedo Scheme 

     call  ocean_albedo_init( alb2 )
     
#ifdef MPI_P
     do n = IST_o,IEND_o
#else
     do n = 1, n_o
#endif
        So( n,  kSo_albedo ) = alb2( n )
        So( n, kSo_anidr) = So( n, kSo_albedo )
        So( n, kSo_avsdr) = So( n, kSo_albedo )
        So( n, kSo_anidf) = albdif
        So( n, kSo_avsdf) = albdif
     enddo
     first_call = .false.

 else if ( albedo_type == 3 ) then   ! MOM 3rd Albedo Scheme 
     
     calday = calendar(month) + day + (sec+ashift)/86400.0  !--- julian day ---
     call shr_orb_decl(calday,orb_eccen,orb_mvelpp,orb_lambm0,orb_obliqr,delta,eccf) !--- solar declination ---

#ifdef MPI_P
     do n = IST_o,IEND_o
#else
     do n = 1, n_o
#endif
        rlat = DEG2RAD*yc_o(n)
        if( xc_o(n) >=0 ) then  
           rlon = DEG2RAD*xc_o(n)
        else
           rlon = DEG2RAD* (360 + xc_o(n) )
        endif 
        cosz = shr_orb_cosz( calday, rlat, rlon, delta )
        cosz = max( 0.0, cosz )
        if (cosz  /=  0.0) then !--- sun hit --
            So( n, kSo_albedo ) = 0.037 / ( 1.1 * (cosz**1.4) + 0.15 )
        else
            So( n, kSo_albedo ) = 0.24666666   ! 0.037 / 0.15
        endif
        if( cosz >=0 ) then
            So( n, kSo_anidr) = So( n, kSo_albedo )
            So( n, kSo_avsdr) = So( n, kSo_albedo )
            So(n,kSo_anidf) = albdif
            So(n,kSo_avsdf) = albdif
        else
            So(n,kSo_anidr) = 1.0
            So(n,kSo_avsdr) = 1.0
            So(n,kSo_anidf) = 1.0
            So(n,kSo_avsdf) = 1.0
        endif
     enddo 

 else if ( albedo_type == 4 ) then   ! MOM 4th Albedo Scheme

     !So( :, kSo_albedo ) = const_alb
#ifdef MPI_P
     do n = IST_o,IEND_o
#else
     do n = 1, n_o
#endif
        So( n, kSo_albedo ) = const_alb
     enddo

 else if  ( albedo_type == 5 ) then   ! CPL  Albedo Scheme 
!---------------wangln 2007-07-02 Add the 4 types MOM albedo Scheme---------------------------
   IF (flx_albav /= 0) THEN
     !--------------------------------------------------------------------
     !if (first_call) write(6,F00) 'ocn albedo is zenith angle independant'
     !--------------------------------------------------------------------
#ifdef MPI_P
     do n=IST_o,IEND_o
#else
     do n=1,n_o
#endif
        So(n,kSo_anidr) = albdif
        So(n,kSo_avsdr) = albdif
        So(n,kSo_anidf) = albdif
        So(n,kSo_avsdf) = albdif
     end do
   ELSE
     !--------------------------------------------------------------------
     !if (first_call) write(6,F00) 'ocn albedo is zenith angle dependant'
     !--------------------------------------------------------------------

     !--- julian day ---
     calday = calendar(month) + day + (sec+ashift)/86400.0

     !--- solar declination ---
     call shr_orb_decl(calday,orb_eccen,orb_mvelpp,orb_lambm0,orb_obliqr,delta,eccf)

!$OMP PARALLEL DO PRIVATE(n,rlat,rlon,cosz)
#ifdef MPI_P
     do n=IST_o,IEND_o
#else
     do n=1,n_o
#endif
         rlat = DEG2RAD*yc_o(n)
     !!    if( xc_o(n) >=0 ) then  
     !!       rlon = DEG2RAD*xc_o(n)
     !!    else
     !!       rlon = DEG2RAD* (360 + xc_o(n) )
     !!    endif 
         rlon = DEG2RAD*xc_o(n)
         cosz = shr_orb_cosz( calday, rlat, rlon, delta )
         if (cosz  >  0.0) then !--- sun hit --
           So(n,kSo_anidr) = (.026/(cosz**1.7 + 0.065)) +   &
           &                 (.150*(cosz      - 0.10 )  *   &
           &                       (cosz      - 0.50 )  *   &
           &                       (cosz      - 1.00 )  )
           So(n,kSo_avsdr) = So(n,kSo_anidr)
           So(n,kSo_anidf) = albdif
           So(n,kSo_avsdf) = albdif
         else !--- dark side of earth ---
         !! So(n,kSo_anidr) = 1.0
         !!  So(n,kSo_avsdr) = 1.0
         !!  So(n,kSo_anidf) = 1.0
         !!  So(n,kSo_avsdf) = 1.0
           So(n,kSo_anidr) = 0.3925
           So(n,kSo_avsdr) = 0.3925
           So(n,kSo_anidf) = albdif
           So(n,kSo_avsdf) = albdif
         end if
     end do
!$OMP  END PARALLEL DO
   END IF


 ENDIF   ! ENDIF of the Albedo_type 
   !call shr_timer_stop(timer_n)
   !first_call=.false.

 END SUBROUTINE flux_albo

!===============================================================================

SUBROUTINE flux_albi( Si )

   USE buf_mod
   USE control_mod
   USE shr_orb_mod
   USE shr_timer_mod ! timing routines
   use const_mod     ! provides physical constants

   implicit none

   !----- arguments -----
   real    :: Si(n_i,nSi)         ! ice state buffer
 
   !----- local -----
   integer :: n                   ! loop index
   real    :: rlat                ! gridcell latitude in radians
   real    :: rlon                ! gridcell longitude in radians
   real    :: delta               ! Solar declination angle in radians
   real    :: eccf                ! Earth-Sun distance factor
   real    :: calday              ! calendar day including fraction, at 0e
   real    :: cosz                ! Cosine of solar zenith angle
   real    :: calendar(12)        ! days of year past at beginning of each month
   integer :: timer_n             ! system-clock-timer number
   logical :: first_call = .true.  

   data calendar /0,31,59,90,120,151,181,212,243,273,304,334/

   SAVE

   character(len=*),parameter :: F00 = "('(flux_albi) ',3a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   if flux_albav/=0 (ie. "on" or "true")
!     Impose a zenith angle dependance on ice model albedos.
!     Currently this only involves setting albedos to zero
!     on the dark side of the earth.
!   else
!     do not alter ice albedos 
! NOTE:
!   o upon input, albedos are assumed to be a 60 degree reference albedo
!-------------------------------------------------------------------------------

 if (first_call) call shr_timer_get(timer_n,'flux_albi')
     call shr_timer_start(timer_n)

 IF (flx_albav /= 0) THEN
     if (first_call) write(6,F00) 'ice albedo is zenith angle independant'
 ELSE
     if (first_call) write(6,F00) 'ice albedo is zenith angle dependant'
     !--- julian day ---
     calday = calendar(month) + day + (sec+ashift)/86400.0

     !--- solar declination ---
     call shr_orb_decl(calday,orb_eccen,orb_mvelpp,orb_lambm0,orb_obliqr,delta,eccf)
   
!$OMP PARALLEL DO PRIVATE(n,rlat,rlon,cosz)
#ifdef MPI_P
     do n=IST_i,IEND_i
#else
     do n=1,n_i
#endif
        rlat = DEG2RAD*yc_o(n)
        rlon = DEG2RAD*xc_o(n)
        cosz = shr_orb_cosz( calday, rlat, rlon, delta )
        if (cosz < 0.0) then !--- dark side of earth ---
          Si(n,kSi_anidr) = 1.0
          Si(n,kSi_avsdr) = 1.0
          Si(n,kSi_anidf) = 1.0
          Si(n,kSi_avsdf) = 1.0
       end if
     end do
!$OMP  END PARALLEL DO
 ENDIF

 call shr_timer_stop(timer_n)
 first_call=.false. 

  END SUBROUTINE flux_albi

!===============================================================================

  SUBROUTINE ocean_albedo_init( alb2 )

   USE buf_mod
   USE control_mod

   implicit none 
  

    real     :: alb2( n_o )
    real     :: xx(n_o)
    integer  :: j1(n_o)

    real                  :: rad2deg
    integer               :: i, j, k
    real   :: albedo_mcm( 19 ) = (/ 0.026, 0.161, 0.110, 0.097, 0.089, 0.076, 0.068, 0.063, 0.060, 0.060,&
                                    0.060, 0.063, 0.068, 0.076, 0.089, 0.097, 0.110, 0.161, 0.206 /)


       rad2deg = 90./asin(1.0)
       
#ifdef MPI_P
       do k=IST_o,IEND_o
#else
       do k=1, n_o
#endif
             xx( k ) = ( yc_o(k) + 90 ) / 10.0
             j1( k ) = int( xx(k) )
             xx( k ) = xx( k ) - j1( k )
       enddo 

#ifdef MPI_P
       do k=IST_o,IEND_o
#else
       do k = 1, n_o
#endif
          alb2(k) = albedo_mcm( j1(k)+1 ) + xx(k) * ( albedo_mcm( j1(k)+2 ) - albedo_mcm( j1(k)+1 ) ) 
       enddo 


End SUBROUTINE ocean_albedo_init
