!===============================================================================
! CVS $Id: flux_ao.F90,v 1.4 2001/12/07 22:59:16 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/flux_ao.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

SUBROUTINE flux_ao(Sa,So,Faoc)

   USE buf_mod

   implicit none

   !--- arguments ---
   real    :: Sa  (nx_o,ny_o,nSa  ) ! atm state
   real    :: So  (nx_o,ny_o,nSo  ) ! ocn state
   real    :: Faoc(nx_o,ny_o,nFaoc) ! atm/ocn fluxes computed by cpl

   !--- local ---
   integer :: j,n0                  ! loop index

!-------------------------------------------------------------------------------
! PURPOSE:
!   compute certain atm/ocn fluxes (on the ocn grid)
!
! INPUT:
!   Sa  : atm state variables 
!   So  : ocn state variables 
!
! OUTPUT:
!   Faoc: atm/ocn fluxes
!
! NOTE: 
!   net heat flux = net solar + lw up + lw down + sensible + latent + Q
!
! ASSUMPTIONS:
!   all fields are on ocn grid
!
!-------------------------------------------------------------------------------

!$OMP PARALLEL DO PRIVATE(j,n0)

   DO j=1,ny_o
      n0 = 1 + (j-1)*nx_o  ! wrt data(n) vs data(i,j)
      call srfflx_ao(nx_o,                &
      &           Sa  (:,j,kSa_z      ),  &
      &           Sa  (:,j,kSa_u      ),  &
      &           Sa  (:,j,kSa_v      ),  &
      &           Sa  (:,j,kSa_ptem   ),  &
      &           Sa  (:,j,kSa_shum   ),  &
      &           Sa  (:,j,kSa_dens   ),  &
      &           Sa  (:,j,kSa_tbot   ),  &
      &           So  (:,j,kSo_t      ),  &
      &           mask_o(n0:n0+nx_o-1) ,  &
      &           Faoc(:,j,kFaoc_sen  ),  &
      &           Faoc(:,j,kFaoc_lat  ),  &
      &           Faoc(:,j,kFaoc_lwup ),  &
      &           Faoc(:,j,kFaoc_evap ),  &
      &           Faoc(:,j,kFaoc_taux ),  &
      &           Faoc(:,j,kFaoc_tauy ),  &

!----------wangln 20050608 add a new variable of Qref---------------------------------
      !&           Faoc(:,j,kFaoc_tref ))
      &           Faoc(:,j,kFaoc_tref ),  &
      &           Faoc(:,j,kFaoc_qref ))
!----------wangln 20050608 add a new variable of Qref---------------------------------


   END DO

END

!===============================================================================

SUBROUTINE srfflx_ao(imax  ,zbot  ,ubot  ,vbot  ,thbot ,   & 
           &         qbot  ,rbot  ,tbot  ,ts    ,mask  ,   &
           &         sen   ,lat   ,lwup  ,evap  ,taux  ,   &
!----------wangln 20050608 add a new variable of Qref---------------------------------
          ! &         tauy  ,tref  )
!----
           &         tauy  ,tref,qref  )
!----------wangln 20050608 add a new variable of Qref---------------------------------


   use const_mod     ! provides physical constants

   implicit none

   !--- input arguments --------------------------------
   integer,intent(in) ::       imax  ! array dimensions
   real,intent(in)    :: zbot (imax) ! atm level height      (m)
   real,intent(in)    :: ubot (imax) ! atm u wind            (m/s)
   real,intent(in)    :: vbot (imax) ! atm v wind            (m/s)
   real,intent(in)    :: thbot(imax) ! atm potential T       (K)
   real,intent(in)    :: qbot (imax) ! atm specific humidity (kg/kg)
   real,intent(in)    :: rbot (imax) ! atm air density       (kg/m^3)
   real,intent(in)    :: tbot (imax) ! atm T                 (K) 
   real,intent(in)    :: ts   (imax) ! surface temperature   (K)
   integer,intent(in) :: mask (imax) ! domain mask 0 <=> point is not in domain

   !--- output arguments -------------------------------
   real,intent(out)   :: sen  (imax) ! sensible         heat flux  (W/m^2)
   real,intent(out)   :: lat  (imax) ! latent           heat flux  (W/m^2)
   real,intent(out)   :: lwup (imax) ! long-wave upward heat flux  (W/m^2)
   real,intent(out)   :: evap (imax) ! evaporative water flux ((kg/s)/m^2)
   real,intent(out)   :: taux (imax) ! x surface stress (N)
   real,intent(out)   :: tauy (imax) ! y surface stress (N)
   real,intent(out)   :: tref (imax) ! 2m reference height temperature (K)

!----------wangln 20050608 add a new variable of Qref---------------------------------
   real,intent(out)   :: qref (imax) ! 2m reference height special humidity 
!----------wangln 20050608 add a new variable of Qref---------------------------------
 
   !--- local constants --------------------------------
   real,parameter :: umin  =  1.0    ! minimum wind speed (m/s)
   real,parameter :: zref  = 10.0    ! reference height (m)
   real,parameter :: ztref =  2.0    ! reference height for air T (m)

   !--- local variables --------------------------------
   integer :: i      ! vector index
   real    :: vmag   ! surface wind magnitude   (m/s)
   real    :: thvbot ! virtual temperature      (K)
   real    :: ssq    ! sea surface humidity     (kg/kg)
   real    :: delt   ! potential T difference   (K)
   real    :: delq   ! humidity difference      (kg/kg)
   real    :: stable ! stability factor
   real    :: rdn    ! sqrt of neutral exchange coefficient (momentum) 
   real    :: rhn    ! sqrt of neutral exchange coefficient (heat)     
   real    :: ren    ! sqrt of neutral exchange coefficient (water)    
   real    :: rd     ! sqrt of exchange coefficient (momentum)         
   real    :: rh     ! sqrt of exchange coefficient (heat)             
   real    :: re     ! sqrt of exchange coefficient (water)            
   real    :: ustar  ! ustar             
   real    :: qstar  ! qstar             
   real    :: tstar  ! tstar             
   real    :: hol    ! H (at zbot) over L
   real    :: xsq    ! temporary variable                         
   real    :: xqq    ! temporary variable                         
   real    :: psimh  ! stability function at zbot (momentum)      
   real    :: psixh  ! stability function at zbot (heat and water)
   real    :: alz    ! ln(zbot/z10)
   real    :: u10n   ! 10m neutral wind 
   real    :: tau    ! stress at zbot
   real    :: cp     ! specific heat of moist air
   real    :: bn     ! exchange coef funct for interpolation
   real    :: bh     ! exchange coef funct for interpolation
   real    :: fac    ! interpolation factor
   real    :: ln0    ! log factor for interpolation
   real    :: ln3    ! log factor for interpolation

!----------
   real    :: u10m, xkar, bf, beta, ug, zoq10, visa, rd10m
   real    :: zi, zo10, zot10, gravit, cpair, rair, vmag0

   !--- local functions --------------------------------
   real    :: Tk     ! dummy arg ~ temperature (K)
   real    :: qsat   ! the saturation humididty of air (kg/m^3)
   real    :: Umps   ! wind velocity (m/s)
   real    :: cdn    ! function: neutral drag coeff at 10m
   real    :: psimhu ! unstable part of psimh
   real    :: psixhu ! unstable part of psimx
   real    :: xd     ! dummy argument  
   real    :: coef   ! wtw add
   real    :: pbot   ! wtw add
 
   qsat(Tk)   = 640380.0 / exp(5107.4/Tk)
   cdn(Umps)  = 0.0027 / Umps + 0.000142 + 0.0000764 * Umps
   psimhu(xd) = log((1.0+xd*(2.0+xd))*(1.0+xd*xd)/8.0) - 2.0*atan(xd) + 1.571
   psixhu(xd) = 2.0 * log((1.0 + xd*xd)/2.0)
 
!-------------------------------------------------------------------------------
! PURPOSE:
!   computes certain atm/ocn surface fluxes
!
! NOTES: 
!   o all fluxes are positive downward
!   o net heat flux = net sw + lw up + lw down + sen + lat
!   o here, tstar = <WT>/U*, and qstar = <WQ>/U*.
!   o wind speeds should all be above a minimum speed (eg. 1.0 m/s)
! 
! ASSUMPTIONS:
!   o Neutral 10m drag coeff: cdn = .0027/U10 + .000142 + .0000764 U10
!   o Neutral 10m stanton number: ctn = .0327 sqrt(cdn), unstable
!                                 ctn = .0180 sqrt(cdn), stable
!   o Neutral 10m dalton number:  cen = .0346 sqrt(cdn)
!   o The saturation humidity of air at T(K): qsat(T)  (kg/m^3)
!-------------------------------------------------------------------------------
 
  !*********************set constants**********************
   xkar   = 0.4
   rair   = 287.0423    !J/Kg/K
   cpair  = 1004.64     !J/Kg/K
   gravit = 9.80616    !m/s2
   beta   = 1.2
   zi     = 600

!   umin   = 1.0
!   zref   = 10.0
!   cpvir =0.80164     !J/Kg/K
!   zvir  =0.607793     !
!   stebol = 5.67e-08
!---------------------------------------------

   DO i=1,imax
     if (mask(i) /= 0) then
    
!
!---------------------------------------------------------------
! Set up necessary variables
!---------------------------------------------------------------
!
!input        rbot(i)= pmidm1(i) / (rair*tbot(i))
!
        vmag   = max(umin, sqrt(ubot(i)**2+vbot(i)**2))
        thvbot = thbot(i) * (1.0 + zvir * qbot(i)) ! virtual temp (K)
!--------------------------------
! wtw
!
       pbot = rbot(i)*rair*tbot(i)             ! rair=287.04
       call humidity(ts(i), pbot, ssq)           ! sea ssq returned in Pa      
       ssq= .62197*(ssq/(pbot - 0.378*ssq))      ! convert from Pa to spec. humidity  kg/kg 
       ssq = 0.98 * ssq                               ! reduced for salinity Kraus 1972 p. 46

       coef = sqrt(ubot(i)**2+vbot(i)**2)
       if(coef.le.5) coef=5.0
       coef = exp((5.0-coef)/40.0)                    ! considered the sea wave influence   /23 in BCC_CSM (test2), 1/40 in SCM

       delt   = (thbot(i) - ts(i))*coef
       delq   = (qbot(i) - ssq)*coef  
       visa   = 1.326e-5*(1+6.542e-3*(thbot(i)-273.16)+8.301e-6*(thbot(i)-273.16)**2 &
	           -4.84e-9*(thbot(i)-273.16)**3)
!------
!      ssq    = 0.98 * qsat(ts(i)) / rbot(i)      ! sea surf hum (kg/kg)
!      delt   = thbot(i) - ts(i)                  ! pot temp diff (K)
!      delq   = qbot(i) - ssq                     ! spec hum dif (kg/kg)
!--------------------------------------------------------------------------
        alz    = log(zbot(i)/zref) 
        cp     = cpair*(1.0 + cpvir*ssq) 
   
        !------------------------------------------------------------
        ! first estimate of Z/L and ustar, tstar and qstar
        !------------------------------------------------------------
   
        !--- neutral coefficients, z/L = 0.0 ---
        stable = 0.5 + sign(0.5 , delt)
        rdn    = sqrt(cdn(vmag))
        rhn    = (1.0-stable) * 0.0327 + stable * 0.018 
        ren    = 0.0346 
   
        !--- ustar, tstar, qstar ---
        ustar = rdn * vmag
        tstar = rhn * delt  
        qstar = ren * delq  
   
        !--- compute stability & evaluate all stability functions ---
        hol  = karman*g*zbot(i)*(tstar/thvbot+qstar/(1.0/zvir+qbot(i)))/ustar**2
        hol  = sign( min(abs(hol),10.0), hol )
        stable = 0.5 + sign(0.5 , hol)
        xsq    = max(sqrt(abs(1.0 - 16.0*hol)) , 1.0)
        xqq    = sqrt(xsq)
        psimh  = -5.0*hol*stable + (1.0-stable)*psimhu(xqq)
        psixh  = -5.0*hol*stable + (1.0-stable)*psixhu(xqq)
!
! Update ustar, tstar, qstar using updated, shifted coeffs 
!
        zo10 = 0.013*ustar**2/gravit + 0.11*visa/ustar
        zot10 = 0.30*visa/ustar
        zoq10 = 0.40*visa/ustar
        rh = xkar/(log(zbot(i)/zo10)-psimh) * xkar/(log(zbot(i)/zot10)-psixh)        
        re = xkar/(log(zbot(i)/zo10)-psimh) * xkar/(log(zbot(i)/zoq10)-psixh)        
        rh = sqrt(rh)
        re = sqrt(re)
!---
!        rd10m = xkar/(log(10.0/zo10)-psimh) 
!        u10m =  vmag*log(10.0/zo10)/log(zbot(i)/zo10)   
!        bf=-gravit/tbot(i)*ustar*(tstar+zvir*tbot(i)*qstar)
!        if (bf>0) then
!	       ug=beta*(bf*zi)**0.333
!        else
!           ug=0.2
!        end if
!        u10m =sqrt(u10m**2+ug**2)
!        ustar = rd10m * u10m 
!---
        rd = xkar/(log(zbot(i)/zo10)-psimh)    
        ustar = rd * vmag
        tstar = rh * delt 
        qstar = re * delq 
   
        !------------------------------------------------------------
        ! iterate to converge on Z/L, ustar, tstar and qstar
        !------------------------------------------------------------
    
        !--- compute stability & evaluate all stability functions ---
        hol  = karman*g*zbot(i)*(tstar/thvbot+qstar/(1.0/zvir+qbot(i)))/ustar**2
        hol  = sign( min(abs(hol),10.0), hol )
        stable = 0.5 + sign(0.5 , hol)
        xsq    = max(sqrt(abs(1.0 - 16.0*hol)) , 1.0)
        xqq    = sqrt(xsq)
        psimh  = -5.0*hol*stable + (1.0-stable)*psimhu(xqq)
        psixh  = -5.0*hol*stable + (1.0-stable)*psixhu(xqq)
!
! Update ustar, tstar, qstar using updated, shifted coeffs 
!
        zo10 = 0.013*ustar**2/gravit + 0.11*visa/ustar
        zot10 = 0.30*visa/ustar
        zoq10 = 0.40*visa/ustar
        rh = xkar/(log(zbot(i)/zo10)-psimh) * xkar/(log(zbot(i)/zot10)-psixh)        
        re = xkar/(log(zbot(i)/zo10)-psimh) * xkar/(log(zbot(i)/zoq10)-psixh)        
        rh = sqrt(rh)
        re = sqrt(re)
!---
!        rd10m = xkar/(log(10.0/zo10)-psimh) 
!        u10m =  vmag*log(10.0/zo10)/log(zbot(i)/zo10)   
!        bf=-gravit/tbot(i)*ustar*(tstar+zvir*tbot(i)*qstar)
!        if (bf>0) then
!	       ug=beta*(bf*zi)**0.333
!        else
!           ug=0.2
!        end if
!        u10m =sqrt(u10m**2+ug**2)
!        ustar = rd10m * u10m 
!---
        rd = xkar/(log(zbot(i)/zo10)-psimh)    
        ustar = rd * vmag
        tstar = rh * delt 
        qstar = re * delq 

        !------------------------------------------------------------
        ! compute the fluxes
        !------------------------------------------------------------
    
        tau = rbot(i) * ustar * ustar 
       
        !--- momentum flux ---
        taux(i) = tau * ubot(i) / vmag 
        tauy(i) = tau * vbot(i) / vmag 
        
        !--- heat flux ---
        sen (i) =      cp * tau * tstar / ustar 
        lat (i) =  latvap * tau * qstar / ustar
        lwup(i) = -stebol * ts(i)**4 
      
        !--- water flux ---
        evap(i) = lat(i)/latvap 
    
        !------------------------------------------------------------
        ! compute 2m reference height temperature
        !------------------------------------------------------------
    
        !--- Compute function of exchange coefficients. ---
        !--- Assume that   cn = rdn*rdn, cm=rd*rd  and  ch=rh*rd
        !--- and therefore 1/sqrt(cn(i)) = 1/rdn and sqrt(cm(i))/ch(i) = 1/rh 
!       bn = karman/rdn
!       bh = karman/rh
        bn = xkar/log(zbot(i)/zo10)
        bh = xkar/log(zbot(i)/zot10)
    
        !--- interpolation factor for stable and unstable cases ---
        ln0 = log(1.0 + (ztref/zbot(i))*(exp(bn) - 1.0))
        ln3 = log(1.0 + (ztref/zbot(i))*(exp(bn - bh) - 1.0))
        fac = (ln0-ztref/zbot(i)*(bn-bh))/bh*stable+(ln0-ln3)/bh*(1.0-stable)
        fac = min(max(fac,0.0),1.0)
    
        !--- actual interpolation ---
        tref(i) = ts(i) + (tbot(i) - ts(i))*fac
    
!----------wangln 20050608 add a new variable of Qref---------------------------------
!       qref(i)= qbot(i)-fac*delq 
        qref(i)= qbot(i)

!----------wangln 20050608 add a new variable of Qref---------------------------------

     else
        sen  (i) = spval  ! sensible         heat flux  (W/m^2)
        lat  (i) = spval  ! latent           heat flux  (W/m^2)
        lwup (i) = spval  ! long-wave upward heat flux  (W/m^2)
        evap (i) = spval  ! evaporative water flux ((kg/s)/m^2)
        taux (i) = spval  ! x surface stress (N)
        tauy (i) = spval  ! y surface stress (N)
        tref (i) = spval  ! 2m reference height temperature (K)
     endif
   ENDDO 

END

subroutine humidity(T, P,Qsat)                                 
!
! Tetens' formula for saturation vp Buck(1981) JAM 20, 1527-1532 
!     
  real  ::  T      !  K
  real  ::  P      !  Pa
  real  ::  Qsat   ! saturated vapor of water ( Pa)
!-----------------------     
  Qsat = (1.0007+3.46e-8*P)*6.1121*exp(17.502*(T-273.16)/(240.97+(T-273.16)))*100.0
!
END

