!===============================================================================
! CVS $Id: flux_epbal.F90,v 1.6 2001/10/08 22:07:21 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/flux_epbal.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

SUBROUTINE flux_epbal(Faia,Faii,Faoa,Faoc,Flol,fa,fi)

   USE buf_mod       ! model data fields
   USE control_mod   ! control flags
   USE map_mod       ! map data, eg. cell areas & interface fractions
   USE shr_timer_mod ! timing routines

#ifdef MPI_P
!    use mpmd_mod, only: MPI_COMM_CPL,master_task
    use shr_kind_mod, only: SHR_KIND_IN, SHR_KIND_R8
#endif
   IMPLICIT none
#ifdef MPI_P
   include "mpif.h"
#endif

   !----- local -----
   real,intent(inout) :: Faia(n_o,nFaia) ! atm/ice flux computed by atm (precip)
   real,intent(in   ) :: Faii(n_o,nFaii) ! atm/ice flux computed by ice (evap)
   real,intent(inout) :: Faoa(n_o,nFaoa) ! atm/ocn flux computed by atm (precip)
   real,intent(in   ) :: Faoc(n_o,nFaoc) ! atm/ocn flux computed by cpl (evap)
   real,intent(inout) :: Flol(n_o,1    ) ! lnd/ocn flux computed by lnd (runoff)
   real,intent(in   ) :: fi(n_o)         ! frac of ice over ocn cell
   real,intent(in   ) :: fa(n_o)         ! frac of atm over ocn cell

   !----- local -----
   integer :: i,j,n               ! generic loop indicies
   real    :: area                ! area of entire ocn domain
   real    :: da                  ! area of one ocn grid cell = dth*dph
   real    :: dai                 ! area of ocn grid covered by ice
   real    :: dao                 ! area of ocn grid covered by atm
   real    :: prec                ! water ~ precipitation
   real    :: evap                ! water ~ evaporation
   real    :: roff                ! water ~ runoff
   real    :: factor              ! prec adjustment factor: evap/prec
   logical :: first_call = .true. ! flags 1st invocation of routine
   integer :: timer_n             ! system-clock-timer number
#ifdef MPI_P
   real (SHR_KIND_R8) :: redu1(10),redu2(10)
#endif

   save  

   !----- formats -----
   character(len=*),parameter :: F00 = "('(flux_epbal) ',4a)"
   character(len=*),parameter :: F01 = "('(flux_epbal) ',a,3e11.3,a,f6.3)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   adjust precip (an atm output flux) and runoff (a lnd output) sent to
!   ice & ocn by a scalar factor f, st  P'+R'+E = f(P+R)+E=0.
!   Why?  This will insure a net zero fresh-water flux into ocn+ice.
!   This could be used to compensate for fresh-water flux imbalances,
!   eg. due to the lack of river runoff from the lnd model.
!
! ASSUMPTIONS:
!   o ice & ocn have identical domains & grids
!-------------------------------------------------------------------------------
   
   if (flx_epbal(1:3) == 'off') return

   if (first_call) then
     write(6,F00) "apply adjustment factor to prec & runoff over ice & ocn"
     call shr_timer_get(timer_n,'flux_epbal')
     first_call = .false.
   end if

   call shr_timer_start(timer_n)

   !----------------------------------------------------------------------------
   ! compute global integrals
   !----------------------------------------------------------------------------
   area = 0.0
   prec = 0.0
   evap = 0.0
   roff = 0.0
#ifdef MPI_P
   do n=ist_o,iend_o
#else
   do n=1,n_o
#endif
      if (domain_o%mask(n) /= 0) then
         da   = domain_o%area(n)
         dai  = da*fi(n)
         dao  = da*fa(n)
         area = area + dai + dao
         
         prec  = prec + dai *(Faia(n,kFaia_rain) + Faia(n,kFaia_snow))
         prec  = prec + dao *(Faoa(n,kFaoa_rain) + Faoa(n,kFaoa_snow))
         evap  = evap + dai * Faii(n,kFaii_evap)
         evap  = evap + dao * Faoc(n,kFaoc_evap)
         roff  = roff + da  * Flol(n,1         )
      end if
   end do
#ifdef MPI_P
   redu1(1) = prec
   redu1(2) = evap
   redu1(3) = roff
   redu1(4) = area
   call MPI_ALLREDUCE(redu1,redu2, 4,  &
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
   prec = redu2(1)
   evap = redu2(2)
   roff = redu2(3)
   area = redu2(4)
#endif
   prec = prec/area
   evap = evap/area
   roff = roff/area

   !----------------------------------------------------------------------------
   ! compute factor
   !----------------------------------------------------------------------------
   if (flx_epbal(1:3) == 'ocn') then
     !---------------------------------------------------------------
     ! use factor supplied by ocn, NOTE: may not cause E+f(P+R)=0
     !---------------------------------------------------------------
     if (first_call) write(6,F01) 'use adjustment factor provided by ocn'

     factor = float(flx_epfac)*1.0e-6
     if (factor .le. 0.0) then
        write(6,F01) 'WARNING: factor from ocn = ',factor
        write(6,F01) 'WARNING: resetting factor to 1.0'
        factor = 1.0
     end if

   else if (flx_epbal(1:4) == 'inst') then
      !--------------------------------------------------------------
      ! compute factor st f(P+R)+E=0 at every timestep
      !--------------------------------------------------------------
      if (first_call) write(6,F00) "choosing factor st P'+E+R'=0"

      if ( (prec+roff) > 0.0) then
        factor=-evap/(prec+roff)
      else
        factor=1.0
        write(6,F01) 'WARNING: avg  P,R,(P+R) = ',prec,roff,prec+roff
        write(6,F01) 'WARNING: setting factor = 1.0'
      end if

   else
      !--------------------------------------------------------------
      ! invalid option
      !--------------------------------------------------------------
      write(6,F00) 'ERROR: unknown epbal option: ',flx_epbal
      stop

   end if

   !----------------------------------------------------------------------------
   ! apply factor
   !----------------------------------------------------------------------------
#ifdef MPI_P
   do n=ist_o,iend_o
#else
   do n=1,n_o
#endif
     if (domain_o%mask(n) /= 0) then
       Faia(n,kFaia_rain ) = Faia(n,kFaia_rain )*factor
       Faia(n,kFaia_snow ) = Faia(n,kFaia_snow )*factor
       Faoa(n,kFaoa_rain ) = Faoa(n,kFaoa_rain )*factor
       Faoa(n,kFaoa_snow ) = Faoa(n,kFaoa_snow )*factor
       Flol(n,1          ) = Flol(n,1          )*factor
     end if
   end do

   !----------------------------------------------------------------------------
   ! document factor
   !----------------------------------------------------------------------------
   if ( factor < .75  .or.  1.25 < factor ) then
      write(6,F01) '<E> <P> <R> =',evap,prec,roff,'  f =',factor
      write(6,F01) 'WARNING: erroneous adjustment factor?'
   else if ( day+sec .eq. 1 ) then
      write(6,F01) '<E> <P> <R> =',evap,prec,roff,'  f =',factor
   else if ( info_dbug > 1 .and. sec == 0 ) then
      write(6,F01) '<E> <P> <R> =',evap,prec,roff,'  f =',factor
   end if

   call shr_timer_stop(timer_n)

END

!===============================================================================
