!===============================================================================
! CVS $Id: histavg_mod.F90,v 1.16 2002/07/02 22:08:45 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/histavg_mod.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

MODULE histavg_mod

   USE kind_mod      ! access to kind declarations
   USE calendar_mod  ! access to calendar routines
   USE shr_sys_mod   ! shared code: system calls
   USE shr_file_mod  ! shared code: file archive
   USE shr_timer_mod ! timing routines
   USE const_mod     ! constants module

#include <netcdf.inc>              

   real,parameter :: spv      = spval  ! special value
   integer        :: nsamples = 0      ! number of samples taken 

   !-----------------------------------------------------------------
   ! output state buffers 
   !-----------------------------------------------------------------
   real, pointer :: a_Sa_a(:,:)    ! atm states on atm grid
   real, pointer :: a_Si_i(:,:)    ! ice states on ice grid
   real, pointer :: a_Sl_l(:,:)    ! lnd states on lnd grid   
   real, pointer :: a_So_o(:,:)    ! ocn states on ocn grid   

   !-----------------------------------------------------------------
   ! merged input state buffers  
   !-----------------------------------------------------------------
   real, pointer :: a_Ss_a(:,:)    ! merged surface states, atm grid

   !-----------------------------------------------------------------
   ! merged input flux buffers 
   !-----------------------------------------------------------------
   real, pointer :: a_Faxx_a(:,:)  ! merged atm input flux
   real, pointer :: a_Foxx_o(:,:)  ! merged ocn input flux

   !-----------------------------------------------------------------
   ! component flux buffers 
   !-----------------------------------------------------------------
   real, pointer :: a_Faxa_a(:,:)  ! a/x flux, from atm, atm grid

   real, pointer :: a_Faia_a(:,:)  ! a/i flux, from atm, atm grid
   real, pointer :: a_Faii_i(:,:)  ! a/i flux, from ice, ice grid

   real, pointer :: a_Fala_a(:,:)  ! a/l flux, from atm, atm grid
   real, pointer :: a_Fall_l(:,:)  ! a/l flux, from lnd, lnd grid

   real, pointer :: a_Faoa_a(:,:)  ! a/o flux, from atm, atm grid
   real, pointer :: a_Faoc_o(:,:)  ! a/o flux, from cpl, ocn grid

   real, pointer :: a_Fioi_i(:,:)  ! i/o flux, from ice, ice grid
   real, pointer :: a_Fioo_o(:,:)  ! i/o flux, from ocn, ocn grid

   real, pointer :: a_roff_rc(:,:) ! lnd runoff vector

CONTAINS

!===============================================================================

SUBROUTINE histavg ()

   USE control_mod          ! control flags
   USE buf_mod              ! model data buffers
   USE map_mod              ! mapping domain info

   IMPLICIT none

   !----- local -----
   character(CLONG) :: loc_fn    = 'null' ! name of current  history file
   character(CLONG) :: loc_fn_old= 'null' ! name of previous history file
   character(CLONG) :: rem_fn             ! remote file name (on archive)
   character(CLONG) :: str                ! generic string variable

   integer :: i,j         ! generic indicies
   integer :: rcode       ! mswrite error code
   integer :: n           ! number of writes into a file
   real    :: time        ! time in elapsed days

   logical,parameter:: tavg=.true. ! flags time mean data

   integer :: fid         ! nc file     ID
   integer :: vid         ! nc variable ID
   integer :: start(2)    ! vector of starting indicies
   integer :: count(2)    ! vector of edge lengths

   real    :: eday_first  ! time of 1st  t-avg sample
   real    :: eday_last   ! time of last t-avg sample
   integer :: edaym       ! eday  in middle of t interval
   integer :: yearm       ! year  in middle of t interval
   integer :: monthm      ! month in middle of t interval
   integer :: daym        ! day   in middle of t interval
   integer :: secm        ! sec   in middle of t interval

   logical :: sum_zero    ! flag ~ initialize (zero-out) summation?
   logical :: sum_add     ! flag ~ add to summation?
   logical :: sum_avg     ! flag ~ divide summation to form t-avg?
   logical :: sum_start   ! flag ~ begin doing sum zero/add/avg?
   logical :: file_new    ! flag ~ open a new file?
   logical :: file_cat    ! flag ~ concatinate to an existing file?
   logical :: file_arch   ! flag ~ file_arch file? (eg. to MSS)
   logical :: first_call = .true.  ! do 1st call setup stuff


   real,allocatable  :: a_roff_r(:) ! temporary, expanded runoff data

   integer :: timer_n     ! system-clock-timer number

   SAVE         

   !----- formats -----
   character(len=*),parameter :: F00 = "('(histavg) ',4a)"
   character(len=*),parameter :: F01 = "('(histavg) adding t = ', &
   &                             i4.4,2('-',i2.2),i6,'s',' to ',a)"
   character(len=*),parameter :: F02 = "('(histavg) code ', i3,4(1x,a) )"
   character(len=*),parameter :: F06 = "('(histavg) ',a,i4,a,a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   if hist_tavg = TRUE, form & save t-avg data
!
! NOTE:
! o all files contain *monthly* averages UNLESS the simulation doesn't
!   start and end on the 1st day of a month, in which case the 1st and
!   last file may contain a partial-month average.  Always consult the
!   time_bound array to see exactly which time interval applies.
! o the six local flags are hard-wired to give one time average per file
! o history file file_arch directory is same as for restart files
!-------------------------------------------------------------------------------

   if ( hist_tavg == 0 ) return

   !-----------------------------------------------------------------
   ! allocate memory for summation arrays
   !-----------------------------------------------------------------
   if (first_call) then
      allocate(a_roff_r(n_r))
      do i=1,n_r
        a_roff_r(i) = spv
        if (domain_r%mask(i) /= 0) a_roff_r(i)=0.0
      end do
   
     call shr_timer_get(timer_n,'histavg')

     !--- allocate memory for time-avg data ---
     allocate(   a_Sa_a(n_a,nSa)  ) ! atm states on atm grid
     allocate(   a_Si_i(n_i,nSi)  ) ! ice states on ice grid
     allocate(   a_Sl_l(n_l,nSl)  ) ! lnd states on lnd grid   
     allocate(   a_So_o(n_o,nSo)  ) ! ocn states on ocn grid   

     allocate(   a_Ss_a(n_a,nSs)  ) ! merged surface states, atm grid

     allocate( a_Faxx_a(n_a,nFaxx)) ! merged atm input flux
     allocate( a_Foxx_o(n_o,nFoxx)) ! merged ocn input flux

     allocate( a_Faxa_a(n_a,nFaxa)) ! a/x flux, from atm, atm grid
     allocate( a_Faia_a(n_a,nFaia)) ! a/i flux, from atm, atm grid
     allocate( a_Faii_i(n_i,nFaii)) ! a/i flux, from ice, ice grid
     allocate( a_Fala_a(n_a,nFala)) ! a/l flux, from atm, atm grid
     allocate( a_Fall_l(n_l,nFall)) ! a/l flux, from lnd, lnd grid
     allocate( a_Faoa_a(n_a,nFaoa)) ! a/o flux, from atm, atm grid
     allocate( a_Faoc_o(n_o,nFaoc)) ! a/o flux, from cpl, ocn grid
     allocate( a_Fioi_i(n_i,nFioi)) ! i/o flux, from ice, ice grid
     allocate( a_Fioo_o(n_o,nFioo)) ! i/o flux, from ocn, ocn grid
     allocate( a_roff_rc(n_rc,1  )) ! lnd runoff vector
   end if

   call shr_timer_start(timer_n)

   !-----------------------------------------------------------------
   ! set summation and file handling control flags
   !-----------------------------------------------------------------

   !--- don't start anything until you hit a 1st day of month ---
   if (first_call    ) sum_start = .false.  ! begin time averaging?
   if ((day+sec) == 1) sum_start = .true.
   
   !--- set control flag values ---
   sum_zero  = .false.  ! initialize summation?
   sum_add   = .false.  ! add to summation?
   sum_avg   = .false.  ! divide summation to form t-avg?
   file_cat  = .false.  ! concatinate avg to an existing file?
   file_arch = .false.  ! archive exisiting file?
   file_new  = .false.  ! open new file?

   if (sum_start) then
     if ((day+sec) == 1                   ) sum_zero = .true.
     if ( .true.                          ) sum_add  = .true.  
     if ((day+sec) == 1 .and. nsamples > 0) sum_avg  = .true.
     if ( sum_zero ) file_new  = .true.
     if ( sum_avg  ) file_cat  = .true.
     if ( file_new ) file_arch = .true.
   end if

   !-----------------------------------------------------------------
   ! form time average
   !-----------------------------------------------------------------
   if ( sum_avg ) call histavg_avg ('avg')

   !-----------------------------------------------------------------
   ! add data to an existing file
   !-----------------------------------------------------------------
   if ( file_cat ) then

     !--- open the nc file ---
#ifdef MPI_P
     if(master_task) then
#endif
     rcode = nf_open(loc_fn,NF_WRITE,fid)
     if (rcode /= NF_NOERR) then 
       write(6,F00) nf_strerror(rcode)
       write(6,F06) 'rcode=',rcode,' opening ',loc_fn
     else
       edaym = nint((eday_first + eday_last)/2.0)
       secm  = 0
       call eday2ymd(edaym,yearm,monthm,daym)
       write(6,F01)yearm,monthm,daym,secm,loc_fn(1:len_trim(loc_fn))

       !--- put time & time_bound data ---
       n=n+1
       time = (eday_first+eday_last)/2.0 ! middle of t-avg interval
       start(1)=n
       count(1)=1
       rcode = nf_inq_varid(fid,'time',vid)
       if (rcode /= NF_NOERR) write(6,F00) nf_strerror(rcode)
       rcode = nf_put_vara_double(fid, vid, start, count, time)
       if (rcode /= NF_NOERR) write(6,F00) nf_strerror(rcode)
       start(1)=1
       start(2)=n
       count(1)=1
       count(2)=1
       rcode = nf_inq_varid(fid,'time_bound',vid)
       if (rcode /= NF_NOERR) write(6,F00) nf_strerror(rcode)
       rcode = nf_put_vara_double(fid, vid, start, count, eday_first)
       if (rcode /= NF_NOERR) write(6,F00) nf_strerror(rcode)
       start(1)=2
       rcode = nf_put_vara_double(fid, vid, start, count, eday_last )
       if (rcode /= NF_NOERR) write(6,F00) nf_strerror(rcode)

       !--- define 2D time-variant field variariables? ---
       if (n == 1) then
         rcode = nf_redef(fid)
  
         !--- define time-variant data fields ---
         call nc_Sa   ('define',fid,a_Sa_a   ,n,tavg,spv,rcode)
         call nc_Si   ('define',fid,a_Si_i   ,n,tavg,spv,rcode)
         call nc_Sl   ('define',fid,a_Sl_l   ,n,tavg,spv,rcode)
         call nc_So   ('define',fid,a_So_o   ,n,tavg,spv,rcode)
         call nc_Ss   ('define',fid,a_Ss_a   ,n,tavg,spv,rcode)
  
         call nc_Faxx ('define',fid,a_Faxx_a ,n,tavg,spv,rcode)
         call nc_Foxx ('define',fid,a_Foxx_o ,n,tavg,spv,rcode)
  
         call nc_Faxa ('define',fid,a_Faxa_a ,n,tavg,spv,rcode)
         call nc_Faia ('define',fid,a_Faia_a ,n,tavg,spv,rcode)
         call nc_Faii ('define',fid,a_Faii_i ,n,tavg,spv,rcode)
         call nc_Fala ('define',fid,a_Fala_a ,n,tavg,spv,rcode)
         call nc_Faoa ('define',fid,a_Faoa_a ,n,tavg,spv,rcode)
         call nc_Fall ('define',fid,a_Fall_l ,n,tavg,spv,rcode)
         call nc_Fioi ('define',fid,a_Fioi_i ,n,tavg,spv,rcode)
         call nc_Fioo ('define',fid,a_Fioo_o ,n,tavg,spv,rcode)
         call nc_roff ('define',fid,a_roff_r ,n,tavg,spv,rcode)
  
         call nc_Faoc ('define',fid,a_Faoc_o ,n,tavg,spv,rcode)
  
         rcode = nf_enddef(fid)
       end if

       !--- put data ---
       call nc_Sa   ('put',fid,a_Sa_a   ,n,tavg,spv,rcode)
       call nc_Si   ('put',fid,a_Si_i   ,n,tavg,spv,rcode)
       call nc_Sl   ('put',fid,a_Sl_l   ,n,tavg,spv,rcode)
       call nc_So   ('put',fid,a_So_o   ,n,tavg,spv,rcode)
       call nc_Ss   ('put',fid,a_Ss_a   ,n,tavg,spv,rcode)

       call nc_Faxx ('put',fid,a_Faxx_a ,n,tavg,spv,rcode)
       call nc_Foxx ('put',fid,a_Foxx_o ,n,tavg,spv,rcode)

       call nc_Faxa ('put',fid,a_Faxa_a ,n,tavg,spv,rcode)
       call nc_Faia ('put',fid,a_Faia_a ,n,tavg,spv,rcode)
       call nc_Faii ('put',fid,a_Faii_i ,n,tavg,spv,rcode)
       call nc_Fala ('put',fid,a_Fala_a ,n,tavg,spv,rcode)
       call nc_Faoa ('put',fid,a_Faoa_a ,n,tavg,spv,rcode)
       call nc_Fall ('put',fid,a_Fall_l ,n,tavg,spv,rcode)
       call nc_Fioi ('put',fid,a_Fioi_i ,n,tavg,spv,rcode)
       call nc_Fioo ('put',fid,a_Fioo_o ,n,tavg,spv,rcode)
       do i=1,n_rc
          a_roff_r(domain_r%iunc(i)) = a_roff_rc(i,1)
       end do
       call nc_roff ('put',fid,a_roff_r ,n,tavg,spv,rcode)

       call nc_Faoc ('put',fid,a_Faoc_o ,n,tavg,spv,rcode)

       rcode = nf_close(fid)
     end if
#ifdef MPI_P
     endif
#endif
   end if

   !-----------------------------------------------------------------
   ! archive existing history file (eg. copy/archive to MSS)
   !-----------------------------------------------------------------
   if ( file_arch .and. loc_fn(1:4) /= "null") then
     rem_fn = trim(mss_dir) // 'hist/' // trim(loc_fn) // ' '

!!==================wangln 2007-07-27:: to avoid the Remote Copy==============
     !!call shr_file_put (rcode, loc_fn,rem_fn,mss_pass,mss_rtpd)
     !!write(6,F02) rcode,'archiving',trim(loc_fn),'to',trim(rem_fn)
!!==================wangln 2007-07-27:: to avoid the Remote Copy==============

     !--- remove older history file ---
     if (mss_rmlf /= 0 .and. loc_fn_old(1:4) /= "null") then
       call shr_sys_system('rm '//loc_fn_old,rcode)
     end if
     loc_fn_old = loc_fn
     loc_fn     = "null"
   end if

   !-----------------------------------------------------------------
   ! create a new nc file, define & put time-invariant data
   !-----------------------------------------------------------------
   if ( file_new ) then
     !--- create a new file ---
     !oc_fn='123456789+1234567 '
     loc_fn='yyyy-mm.nc '
     write(loc_fn( 1: 4),'(i4.4)') year
     write(loc_fn( 6: 7),'(i2.2)') month
     loc_fn = trim(case_name) // '.cpl5.ha.' // trim(loc_fn) // ' '

     call nc_header(loc_fn,case_desc,rcode)
     if (rcode /= 0) loc_fn = "null"
     n=0 ! number of writes into this file
   end if

   !-----------------------------------------------------------------
   ! initialize time-avg summation
   !-----------------------------------------------------------------
   if ( sum_zero ) then
     call histavg_avg ('zero')
     eday_first  = float(eday) + sec/86400.0
   end if

   !-----------------------------------------------------------------
   ! add to summation time-avg summation
   !-----------------------------------------------------------------
   if ( sum_add ) then
     call histavg_avg ('sum')
     eday_last  = float(eday) + sec/86400.0
   end if

   first_call=.false.
   call shr_timer_stop(timer_n)
   
END SUBROUTINE histavg

!
!===============================================================================

SUBROUTINE histavg_avg (cFlag)

   USE buf_mod              ! model data buffers

   implicit none

   !----- arguments -----
   character(len=*) :: cFlag   ! initialize, add to sum, or divide??

   !----- local -----
   integer          :: n,k     ! character string index
   real             :: fact    ! 1/nsamples
   real             :: ifact   ! 1/(ice fraction)
   real             :: ifrac   ! ice fraction
   real             :: a_ifrac ! ice fraction, time-avg
   logical          :: invalid ! flags invalid data

   !----- formats -----
   character(len=*),parameter :: F00 = "('(histavg_avg) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   form a time-average, a_F  (found in histavg_mod),
!   of an array            F, (found in buffer_mod)
!
! ASSUMPTIONS:
!   there are no undefined array elements
!-------------------------------------------------------------------------------
   
   !----------------------------------------------------------------------------
   ! set summation to zero
   !----------------------------------------------------------------------------
   IF (cFlag .eq. 'zero') THEN
      nsamples = 0

      !---------- fields on atm grid ----------
!$OMP PARALLEL DO PRIVATE(n,k)
      do n=1,n_a
         do k=1,nSa
            a_Sa_a(n,k)   = 0.0
         end do
         do k=1,nSs
            a_Ss_a(n,k)   = 0.0
         end do
         do k=1,nFaxx
            a_Faxx_a(n,k) = 0.0
         end do
         do k=1,nFaxa
            a_Faxa_a(n,k) = 0.0
         end do
         do k=1,nFala
            a_Fala_a(n,k) = 0.0
         end do
         do k=1,nFaoa
            a_Faoa_a(n,k) = 0.0
         end do
         do k=1,nFaia
            a_Faia_a(n,k) = 0.0
         end do
      end do
!$omp  end parallel do
      !---------- fields on ice grid ----------
!$OMP PARALLEL DO PRIVATE(n,k)
      do n=1,n_i
         do k=1,nSi
            a_Si_i(n,k)   = 0.0
         end do
         do k=1,nFioi
            a_Fioi_i(n,k) = 0.0
         end do
         do k=1,nFaii
            a_Faii_i(n,k) = 0.0
         end do
      end do
!$omp  end parallel do
      !---------- fields on lnd grid ----------
!$OMP PARALLEL DO PRIVATE(n,k)
      do n=1,n_l
         do k=1,nSl
            a_Sl_l(n,k)   = 0.0
         end do
         do k=1,nFall
            a_Fall_l(n,k) = 0.0
         end do
      end do
!$omp  end parallel do
      !---------- fields on ocn grid ----------
!$OMP PARALLEL DO PRIVATE(n,k)
      do n=1,n_o
         do k=1,nSo
            a_So_o(n,k)   = 0.0
         end do
         do k=1,nFoxx
            a_Foxx_o(n,k) = 0.0
         end do
         do k=1,nFaoc
            a_Faoc_o(n,k) = 0.0
         end do
         do k=1,nFioo
            a_Fioo_o(n,k) = 0.0
         end do
      end do
!$omp  end parallel do
      !---------- other: runoff vector ----------
      a_roff_rc = 0.0

   !----------------------------------------------------------------------------
   ! add to summation 
   !----------------------------------------------------------------------------
   ELSE IF (cFlag .eq. 'sum') THEN
      nsamples = nsamples + 1

      !---------- fields on atm grid ----------
!$OMP PARALLEL DO PRIVATE(n,k)
      do n=1,n_a
         do k=1,nSa
            a_Sa_a(n,k)   = a_Sa_a(n,k)   + Sa_a(n,k)
         end do
         do k=1,nSs
            a_Ss_a(n,k)   = a_Ss_a(n,k)   + Ss_a(n,k)
         end do
         do k=1,nFaxx
            a_Faxx_a(n,k) = a_Faxx_a(n,k) + Faxx_a(n,k)
         end do
         do k=1,nFaxa
            a_Faxa_a(n,k) = a_Faxa_a(n,k) + Faxa_a(n,k)
         end do
         do k=1,nFala
            a_Fala_a(n,k) = a_Fala_a(n,k) + Fala_a(n,k)
         end do
         do k=1,nFaoa
            a_Faoa_a(n,k) = a_Faoa_a(n,k) + Faoa_a(n,k)
         end do
         do k=1,nFaia
            a_Faia_a(n,k) = a_Faia_a(n,k) + Faia_a(n,k)
         end do
      end do
!$omp  end parallel do
      !---------- fields on ice grid ----------
!!!----!$OMP PARALLEL DO PRIVATE(n,k)
!$OMP PARALLEL DO PRIVATE(n,k,ifrac,a_ifrac)
      do n=1,n_i
           ifrac =   Si_i(n,kSi_ifrac)
         a_ifrac = a_Si_i(n,kSi_ifrac) + ifrac
         do k=1,nSi
            a_Si_i(n,k)   = a_Si_i(n,k)   + ifrac*Si_i(n,k)
         end do
         a_Si_i(n,kSi_ifrac) = a_ifrac ! correct/restore ifrac
         do k=1,nFioi
            a_Fioi_i(n,k) = a_Fioi_i(n,k) + ifrac*Fioi_i(n,k)
         end do
         do k=1,nFaii
            a_Faii_i(n,k) = a_Faii_i(n,k) + ifrac*Faii_i(n,k)
         end do
      end do
!$omp  end parallel do
      !---------- fields on lnd grid ----------
!$OMP PARALLEL DO PRIVATE(n,k)
      do n=1,n_l
         do k=1,nSl
            a_Sl_l(n,k)   = a_Sl_l(n,k)   + Sl_l(n,k)
         end do
         do k=1,nFall
            a_Fall_l(n,k) = a_Fall_l(n,k) + Fall_l(n,k)
         end do
      end do
!$omp  end parallel do
      !---------- fields on ocn grid ----------
!$OMP PARALLEL DO PRIVATE(n,k)
      do n=1,n_o
         do k=1,nSo
            a_So_o(n,k)   = a_So_o(n,k)   + So_o(n,k)
         end do
         do k=1,nFoxx
            a_Foxx_o(n,k) = a_Foxx_o(n,k) + Foxx_o(n,k)
         end do
         do k=1,nFaoc
            a_Faoc_o(n,k) = a_Faoc_o(n,k) + Faoc_o(n,k)
         end do
         do k=1,nFioo
            a_Fioo_o(n,k) = a_Fioo_o(n,k) + Fioo_o(n,k)
         end do
      end do
!$omp  end parallel do
      !---------- other: runoff vector ----------
      a_roff_rc = a_roff_rc + roff_rc

   !----------------------------------------------------------------------------
   ! divide to form average
   !----------------------------------------------------------------------------
   ELSE IF (cFlag .eq. 'avg') THEN
      fact = 1.0/float(nsamples)

      !---------- fields on atm grid ----------
!$OMP PARALLEL DO PRIVATE(n,k)
      do n=1,n_a
         do k=1,nSa
            a_Sa_a(n,k)   = a_Sa_a(n,k)  *fact
         end do
         do k=1,nSs
            a_Ss_a(n,k)   = a_Ss_a(n,k)  *fact
         end do
         do k=1,nFaxx
            a_Faxx_a(n,k) = a_Faxx_a(n,k)*fact
         end do
         do k=1,nFaxa
            a_Faxa_a(n,k) = a_Faxa_a(n,k)*fact
         end do
         do k=1,nFala
            a_Fala_a(n,k) = a_Fala_a(n,k)*fact
         end do
         do k=1,nFaoa
            a_Faoa_a(n,k) = a_Faoa_a(n,k)*fact
         end do
         do k=1,nFaia
            a_Faia_a(n,k) = a_Faia_a(n,k)*fact
         end do
      end do
!$omp  end parallel do
      !---------- fields on ice grid ----------
!$OMP PARALLEL DO PRIVATE(n,k,ifact,a_ifrac,invalid)
      do n=1,n_i
         a_ifrac = a_Si_i(n,kSi_ifrac)
         if (a_ifrac.gt.1.0e-6) ifact = 1.0/a_ifrac
         if (a_ifrac.le.1.0e-6) ifact = 0.0
         invalid = .false.
         if (mask_i(n).eq.0 .or. ifact.le.0.0) invalid = .true.
         do k=1,nSi
            a_Si_i(n,k)   = a_Si_i(n,k)  *ifact
            if (invalid) a_Si_i(n,k) = spv
         end do
         if (mask_i(n).ne.0) a_Si_i(n,kSi_ifrac) = a_ifrac*fact
         do k=1,nFioi
            a_Fioi_i(n,k) = a_Fioi_i(n,k)*ifact
            if (invalid) a_Fioi_i(n,k) = spv
         end do
         do k=1,nFaii
            a_Faii_i(n,k) = a_Faii_i(n,k)*ifact
            if (invalid) a_Faii_i(n,k) = spv
         end do
      end do
!$omp  end parallel do
      !---------- fields on lnd grid ----------
!$OMP PARALLEL DO PRIVATE(n,k,invalid)
      do n=1,n_l
         invalid = .false.
         if (mask_l(n).eq.0) invalid = .true.
         do k=1,nSl
            a_Sl_l(n,k)   = a_Sl_l(n,k)  *fact
            if (invalid)    a_Sl_l(n,k)   = spv
         end do
         do k=1,nFall
            a_Fall_l(n,k) = a_Fall_l(n,k)*fact
            if (invalid)    a_Fall_l(n,k) = spv
         end do
      end do
!$omp  end parallel do
      !---------- fields on ocn grid ----------
!$OMP PARALLEL DO PRIVATE(n,k,invalid)
      do n=1,n_o
         invalid = .false.
         if (mask_o(n).eq.0) invalid = .true.
         do k=1,nSo
            a_So_o(n,k)   = a_So_o(n,k)  *fact
            if (invalid)    a_So_o(n,k) = spv
         end do
         do k=1,nFoxx
            a_Foxx_o(n,k) = a_Foxx_o(n,k)*fact
            if (invalid)    a_Foxx_o(n,k) = spv
         end do
         do k=1,nFaoc
            a_Faoc_o(n,k) = a_Faoc_o(n,k)*fact
            if (invalid)    a_Faoc_o(n,k) = spv
         end do
         do k=1,nFioo
            a_Fioo_o(n,k) = a_Fioo_o(n,k)*fact
            if (invalid)    a_Fioo_o(n,k) = spv
         end do
      end do
!$omp  end parallel do
      !---------- other: runoff vector ----------
      a_roff_rc = a_roff_rc*fact

   !----------------------------------------------------------------------------
   ELSE
      write(6,F00) 'ERROR: invalid cFlag=',cFlag
      STOP 'cFlag'
   END IF

END SUBROUTINE histavg_avg

!===============================================================================

END MODULE histavg_mod
