!===============================================================================
! CVS $Id: history.F90,v 1.14 2002/01/23 21:23:11 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/history.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

   SUBROUTINE history ()

   USE kind_mod      ! access to kind declarations
   USE control_mod   ! control flags
   USE buf_mod       ! model data buffers
   USE map_mod       ! map domain info
   USE shr_sys_mod   ! shared code: system calls
   USE shr_file_mod  ! shared code: file archive
   use const_mod     ! provides physical constants

   implicit none


#include <netcdf.inc>

   !----- local -----
   character(CLONG)  :: loc_fn     = 'null' ! local  file name
   character(CLONG)  :: loc_fn_old = 'null' ! name of previous history file
   character(CLONG)  :: rem_fn              ! remote file name (on archive)
   character(CLONG)  :: str                 ! generic string variable
   integer           :: i,j                 ! character string index
   integer           :: rcode               ! mswrite error code
   integer           :: n                   ! number of writes into a file
   real              :: time                ! time in elapsed days
   integer           :: fid                 ! nc file     ID
   integer           :: vid                 ! nc variable ID
   integer           :: start(2)            ! vector of starting indicies
   integer           :: count(2)            ! vector of edge lengths
   real,allocatable  :: roff_r(:)           ! temporary, expanded runoff data
   logical,parameter :: tavg = .false.      ! flags time mean data
   logical           :: first_call = .true. ! flags one-time set-up stuff

   SAVE

   !----- formats -----
   character(len=*),parameter :: F00 = "('(history) ',4a)"
   character(len=*),parameter :: F01 = &
   &               "('(history) adding t = ',i4.4,2('-',i2.2),i6,'s',' to ',a)"
   character(len=*),parameter :: F02 = "('(history) code ',i4, 4(1x,a) )"
   character(len=*),parameter :: F06 = "('(history) ',a,i4,a,a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   if hist_now  /= 0 (ie. true), then put more data to history file
!   if hist_save /= 0 (ie. true), then archive file & open a new file
!
! NOTE:
!   history file archive directory is same as for restart files
!-------------------------------------------------------------------------------

   if (first_call) then
      allocate(roff_r(n_r))
      do i=1,n_r
        roff_r(i) = spval
        if (domain_r%mask(i) /= 0) roff_r(i)=0.0
      end do
   end if

   !----------------------------------------------------------------------------
   ! archive existing history file
   !----------------------------------------------------------------------------
   if ( hist_save.ne.0 .and. loc_fn(1:4).ne."null" ) then


     rem_fn = trim(mss_dir) // 'hist/' // trim(loc_fn) // ' '

!!==================wangln 2007-07-27:: to avoid the Remote Copy==============
     !!call shr_file_put (rcode,loc_fn,rem_fn,mss_pass,mss_rtpd)
     !!write(6,F02) rcode,'archiving ',trim(loc_fn),'to',trim(rem_fn)
!!==================wangln 2007-07-27:: to avoid the Remote Copy==============

     !--- remove older history file ---
     if (mss_rmlf /= 0 .and. loc_fn_old(1:4) /= "null" ) then
       call shr_sys_system('rm '//loc_fn_old,rcode)
     end if
     loc_fn_old = loc_fn
     loc_fn     = "null"
   end if

   !----------------------------------------------------------------------------
   ! create a new nc file, define & put time-invariant data
   !----------------------------------------------------------------------------
   if ( hist_now.ne.0 .and. loc_fn(1:4).eq."null" ) then
     !--- create a new file ---
     !oc_fn='123456789+123456.nc '
     loc_fn='yyyy-mm-dd-sssss.nc '
     write(loc_fn( 1: 4),'(i4.4)') year
     write(loc_fn( 6: 7),'(i2.2)') month
     write(loc_fn( 9:10),'(i2.2)') day
     write(loc_fn(12:16),'(i5.5)') sec
     loc_fn = trim(case_name) // '.cpl5.hi.' // trim(loc_fn) // ' '
#ifdef MPI_P
     
!xjx     print *,"cpl_rank,cpl_size,master_task=",cpl_rank,cpl_size,master_task
!xjx     write(6, *),"cpl_rank,cpl_size,master_task=",cpl_rank,cpl_size,master_task

!     if(cpl_rank == cpl_size-1) then
     if(master_task) then
#endif
     call nc_header(loc_fn,case_desc,rcode)
     if (rcode.ne.0) loc_fn = "null"
#ifdef MPI_P
     endif
#endif
     n=0 ! number of writes into this file
   end if

   !----------------------------------------------------------------------------
   ! put data to an existing file
   !----------------------------------------------------------------------------

   if ( hist_now.ne.0 .and. loc_fn(1:4).ne."null" ) then

     !--- open the nc file ---
#ifdef MPI_P
     if(master_task) then
!     if(cpl_rank == cpl_size-1) then
#endif
     rcode = nf_open(loc_fn,NF_WRITE,fid)
     if (rcode.ne.NF_NOERR) write(6,F00) nf_strerror(rcode)
     if (rcode.ne.0) then
       write(6,F06) 'rcode=',rcode,' opening ',loc_fn
       return
     else
       write(6,F01) year,month,day,sec,loc_fn(1:len_trim(loc_fn))
     end if
#ifdef MPI_P
     endif
#endif

     !--- put time & time_bound data ---
     n=n+1
     time = float(eday) + sec/86400.0 ! elapsed days
     start(1)=n
     count(1)=1


#ifdef MPI_P
     if(master_task) then
!     if(cpl_rank == cpl_size-1) then
#endif
     rcode = nf_inq_varid(fid,'time',vid)
     if (rcode.ne.NF_NOERR) write(6,F00) nf_strerror(rcode)

     rcode = nf_put_vara_double(fid, vid, start, count, time)
     if (rcode.ne.NF_NOERR) write(6,F00) nf_strerror(rcode)
#ifdef MPI_P
    endif
#endif

     start(1)=1
     start(2)=n
     count(1)=1
     count(2)=1

#ifdef MPI_P
     if(master_task) then
!     if(cpl_rank == cpl_size-1) then
#endif
     rcode = nf_inq_varid(fid,'time_bound',vid)
     if (rcode.ne.NF_NOERR ) write(6,F00) nf_strerror(rcode)

     rcode = nf_put_vara_double(fid, vid, start, count, time)
     if (rcode.ne.NF_NOERR ) write(6,F00) nf_strerror(rcode)
     start(1)=2

     rcode = nf_put_vara_double(fid, vid, start, count, time)
     if (rcode.ne.NF_NOERR ) write(6,F00) nf_strerror(rcode)
#ifdef MPI_P
   endif
#endif

     !--- define 2D time-variant field variariables ---
     if (n.eq.1) then


#ifdef MPI_P
     if(master_task) then
!     if(cpl_rank == cpl_size-1) then
#endif
       rcode = nf_redef(fid)
       !--- define time-variant data fields ---
       call nc_Sa   ('define',fid,Sa_a   ,n,tavg,spval,rcode)
       call nc_Si   ('define',fid,Si_i   ,n,tavg,spval,rcode)
       call nc_Sl   ('define',fid,Sl_l   ,n,tavg,spval,rcode)
       call nc_So   ('define',fid,So_o   ,n,tavg,spval,rcode)
       call nc_Ss   ('define',fid,Ss_a   ,n,tavg,spval,rcode)

       call nc_Faxx ('define',fid,Faxx_a ,n,tavg,spval,rcode)
       call nc_Foxx ('define',fid,Foxx_o ,n,tavg,spval,rcode)
       call nc_aFoxx('define',fid,aFoxx_o,n,tavg,spval,rcode)

       call nc_Faxa ('define',fid,Faxa_a ,n,tavg,spval,rcode)
       call nc_Faia ('define',fid,Faia_a ,n,tavg,spval,rcode)
       call nc_Faii ('define',fid,Faii_i ,n,tavg,spval,rcode)
       call nc_Fala ('define',fid,Fala_a ,n,tavg,spval,rcode)
       call nc_Faoa ('define',fid,Faoa_a ,n,tavg,spval,rcode)
       call nc_Fall ('define',fid,Fall_l ,n,tavg,spval,rcode)
       call nc_Fioi ('define',fid,Fioi_i ,n,tavg,spval,rcode)
       call nc_Fioo ('define',fid,Fioo_o ,n,tavg,spval,rcode)
       call nc_roff ('define',fid,roff_r ,n,tavg,spval,rcode)
       call nc_Faoc ('define',fid,Faoc_o ,n,tavg,spval,rcode)
       rcode = nf_enddef(fid)
#ifdef MPI_P
     endif
#endif

     end if

#ifdef MPI_P
     if(master_task) then
!     if(cpl_rank == cpl_size-1) then
#endif
     !--- put data ---
     call nc_Sa   ('put',fid,Sa_a   ,n,tavg,spval,rcode)
     call nc_Si   ('put',fid,Si_i   ,n,tavg,spval,rcode)
     call nc_Sl   ('put',fid,Sl_l   ,n,tavg,spval,rcode)
     call nc_So   ('put',fid,So_o   ,n,tavg,spval,rcode)
     call nc_Ss   ('put',fid,Ss_a   ,n,tavg,spval,rcode)


     call nc_Faxx ('put',fid,Faxx_a ,n,tavg,spval,rcode)
     call nc_Foxx ('put',fid,Foxx_o ,n,tavg,spval,rcode)
     call nc_aFoxx('put',fid,aFoxx_o,n,tavg,spval,rcode)

     call nc_Faxa ('put',fid,Faxa_a ,n,tavg,spval,rcode)
     call nc_Faia ('put',fid,Faia_a ,n,tavg,spval,rcode)
     call nc_Faii ('put',fid,Faii_i ,n,tavg,spval,rcode)
     call nc_Fala ('put',fid,Fala_a ,n,tavg,spval,rcode)
     call nc_Faoa ('put',fid,Faoa_a ,n,tavg,spval,rcode)


     call nc_Fall ('put',fid,Fall_l ,n,tavg,spval,rcode)
     call nc_Fioi ('put',fid,Fioi_i ,n,tavg,spval,rcode)
     call nc_Fioo ('put',fid,Fioo_o ,n,tavg,spval,rcode)

     do i=1,n_rc
        roff_r(domain_r%iunc(i)) = roff_rc(i,1)
     end do
     call nc_roff ('put',fid,roff_r ,n,tavg,spval,rcode)
     call nc_Faoc ('put',fid,Faoc_o ,n,tavg,spval,rcode)
     rcode = nf_close(fid)
#ifdef MPI_P
     endif
#endif

   end if

   first_call = .false.

END

!===============================================================================
