!=======================================================================
! CVS $Id: initial.F90,v 1.10 2002/02/20 21:56:48 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/initial.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!=======================================================================

SUBROUTINE initial ()

   USE control_mod        ! control flags
   USE buf_mod            ! buffer (model field) dimensions
   USE shr_orb_mod        ! orbital constants & routines
   use shr_sys_mod        ! shared system routines

   IMPLICIT none

   !--- local ---
   integer :: orb_year    ! year (AD) wrt orbital parameters
   real    :: orb_obliq   ! obliquity of orbit (degrees)
   real    :: orb_mvelp   ! moving vernel equinox of orbit (degrees)
   logical :: orb_log     ! flag to print-out orbital info
   integer :: i           ! generic loop index
   integer :: rcode       ! return code

   integer,parameter :: true  = 1         ! non-zero => true
   integer,parameter :: false = 0         ! zero    <=> false
   integer,parameter :: unset = -999999   ! flags unset input namelist values

   NAMELIST / inparm /  &
   &     case_name   ,  &
   &     case_desc   ,  &
   &     rest_type   ,  &
   &     rest_bfile  ,  &
   &     rest_pfile  ,  &
   &     rest_date   ,  &
   &     rest_str    ,  &
   &     rest_freq   ,  &
   &     rest_n      ,  &
   &     rest_odate  ,  &
   &     map_a2of_fn ,  &
   &     map_a2lf_fn ,  &
   &     map_a2os_fn ,  &
   &     map_o2af_fn ,  &
   &     map_r2o_fn  ,  &
   &     stop_option ,  &
   &     stop_n      ,  &
   &     stop_date   ,  &
   &     hist_freq   ,  &
   &     hist_bundle ,  &
   &     hist_n      ,  &
   &     hist_odate  ,  &
   &     hist_tavg   ,  &
   &     diag_freq   ,  &
   &     diag_n      ,  &
   &     info_dbug   ,  &
   &     info_time   ,  &
   &     info_bcheck ,  &
   &     mss_dir     ,  &
   &     mss_opts    ,  &
   &     mss_pass    ,  &
   &     mss_rtpd    ,  &
   &     mss_rmlf    ,  &
   &     data_dir    ,  &    ! for dataatm
   &     forc_dir    ,  &    ! for dataatm
   &     data_fname  ,  &    ! for dataatm
   &     flx_albav   ,  &
   &     flx_epbal   ,  &
   &     init_tavg   ,  &
   &     orb_year    ,  &
   &     orb_eccen   ,  &
   &     orb_mvelp   ,  &
   &     orb_obliq   ,  &
!-------------------wangln 2007-05-28 Add the Bilinear Correcting------
   &     bilinear    ,  &
   &     ocn_type    ,  &  ! using different Ocn Model : POP, MOM, IAP(LiCOM) and must using the Capital 2007-06-20
   &     atm_type    ,  &  ! using different Atm Model : CAM3/BCC, CAM2 and must using the Capital 2007-06-20
   &     albedo_type ,  &  ! albedo option
   &     land_correct,  &  ! Wutw Land Correction scheme
   &     data_atm,      &  ! for dataatm  
   &     atm_begin,     &  ! for dataatm 
   &     atm_end,       &  ! for dataatm  
   &     climate_runoff,&  ! Climate Runoff data
   &     carbon_flux,   &  ! crabon flux
!-------------------wangln 2007-05-28 Add the Bilinear Correcting------
   &     nx_a,ny_a   ,  &
   &     nx_i,ny_i   ,  &
   &     nx_l,ny_l   ,  &
   &     nx_o,ny_o   ,  &
   &     nx_r,ny_r
 
   !----- formats ---
   character(len=*),parameter :: F00 = "('(initial) ',2a)"
   character(len=*),parameter :: F01 = "('(initial) ',a,i7)"
   character(len=*),parameter :: F11 = "('(initial) ',a,i4)"
   character(len=*),parameter :: F12 = "('(initial) ',a,f11.6)"
   character(len=*),parameter :: F05 = "(80('-'))"

!-------------------------------------------------------------------------------
! PURPOSE:
!   o set default input parameters
!   o read cpl parameter (namelist) file
!-------------------------------------------------------------------------------
   
   ierr   = 0  ! assume no errors yet :)

   !----------------------------------------------------------------------------
   ! input parmarameter defaults
   !----------------------------------------------------------------------------
   case_name   = 'null'
   case_desc   = 'null'
   rest_type   = 'initial'
   rest_bfile  = 'null'
   rest_pfile  = 'null'
   rest_date   = unset
   rest_str    = 'null'
   rest_freq   = 'monthly'
   rest_n      = 10
   rest_odate  = 0
   map_a2of_fn = 'unknown'
   map_a2lf_fn = 'unknown'
   map_a2os_fn = 'unknown'
   map_o2af_fn = 'unknown'
   map_r2o_fn  = 'unknown'
   stop_option = 'newyear'
   stop_n      = 0
   stop_date   = 010101
   hist_freq   = 'never'
   hist_bundle = 'null'
   hist_n      = 1   
   hist_odate  = 1
   hist_tavg   = 1
   diag_freq   = 'null'
   diag_n      = 10
   diag_odate  = 1
   info_dbug   = 1
   info_time   = 0
   info_bcheck = 1
   mss_dir     = 'null'
   mss_opts    = 'nowait,nomail '
   mss_pass    = ' '   
   mss_rtpd    = 365
   mss_rmlf    = 0
   data_dir    = 'null'      ! for dataatm
   forc_dir    = 'null'      ! for dataatm
   data_fname  = 'null'      ! for dataatm
   flx_albav   = 0
   flx_epbal   = 'off'
   flx_epfac   = 0
   init_tavg   = unset
   orb_year    = unset
   orb_eccen   = unset
   orb_mvelp   = unset
   orb_obliq   = unset
!-------------------wangln 2007-05-28 Add the Bilinear Correcting------
    bilinear   = .false. 
    ocn_type   = 'POP'
    atm_type   = 'CAM2'
    albedo_type = 5
    land_correct   = .false.
    data_atm       = .false.      ! for dataatm
    atm_begin      = -1           ! for dataatm 
    atm_end        = -1           ! for dataatm
    climate_runoff = .false.
    carbon_flux    = .false.
!-------------------wangln 2007-05-28 Add the Bilinear Correcting------
   nx_a        = unset
   ny_a        = unset
   nx_i        = unset
   ny_i        = unset
   nx_l        = unset
   ny_l        = unset
   nx_o        = unset
   ny_o        = unset
   nx_r        = unset
   ny_r        = unset

   !-----------------------------------------------
   ! read input namelist
   !-----------------------------------------------
!   write(6, *) "-------------in initial...master_task=",master_task

   write(6,F00) 'reading input namelist.'
   read (5,inparm,iostat=rcode)
   if (rcode > 0) then
       write(6,F01) 'ERROR: reading input namelist, iostat=',rcode
       call shr_sys_abort("cpl5 namelist read error")
   end if

   !----------------------------------------------------------------------------
   ! check & fix input parms as neccessary
   !----------------------------------------------------------------------------

   !--- MS path name must have trailing slash ---
   i = index(mss_dir,' ')-1
   if (mss_dir(i:i) .ne. '/') mss_dir(i+1:i+2)='/ '

   !----------------------------------------------------------------------------
   ! re-set defaults based on input parms
   !----------------------------------------------------------------------------

   !----- restart options --------------------------------
   if ( rest_date.eq.unset) then
     if (rest_type(1:4) .eq. 'bran') rest_date=-1
     if (rest_type(1:4) .eq. 'cont') rest_date=-1
     if (rest_type(1:4) .eq. 'rege') rest_date=-1
     if (rest_type(1:4) .eq. 'init') rest_date= 101
   end if
   if ( rest_pfile(1:4).eq.'null') then
     if ( case_name(1:4).eq.'null') then
       rest_pfile = 'rpointer.cpl'
     else
       i = index(case_name,' ')-1
       rest_pfile = '$HOME/cpl.'//case_name(1:i)//'.rpointer'
     endif
   else if ( rest_pfile(1:11).eq.'./rpointer ') then
     rest_pfile = 'rpointer.cpl'
   endif

   !----- diagnostic options -----------------------------
   if ( diag_freq.eq.'null' ) diag_freq = 'never'

   !----- initial tavg procedures ------------------------
   if ( init_tavg .eq. unset) then
     if (rest_type(1:4) .eq. 'init') init_tavg=true
     if (rest_type(1:4) .ne. 'init') init_tavg=false
   end if
   
!!!=============wangln 2008-02-16 For MOM Ahead======================== 
   !!if( ocn_type(1:3) == 'MOM' ) init_tavg=false
!!!=============wangln 2008-02-16 For MOM Ahead======================== 

   !----------------------------------------------------------------------------
   ! print values of all input parms
   !----------------------------------------------------------------------------
   write(6,F00) 'parameter values:'
   write(6,inparm)

   !----------------------------------------------------------------------------
   ! orbital year and/or parameter selection 
   !----------------------------------------------------------------------------
   if ( orb_year .ne. unset ) then
     write(6,F01) 'orbit based on orb_year = ',orb_year
     orb_eccen = SHR_ORB_UNDEF_REAL 
     orb_obliq = SHR_ORB_UNDEF_REAL 
     orb_mvelp = SHR_ORB_UNDEF_REAL
   else if ( orb_eccen .ne. unset &
   &   .and. orb_obliq .ne. unset &
   &   .and. orb_mvelp .ne. unset ) then
     write(6,F12) 'orbit based on orb_eccen = ',orb_eccen
     write(6,F12) 'orbit based on orb_obliq = ',orb_obliq
     write(6,F12) 'orbit based on orb_mvelp = ',orb_mvelp
     orb_year  = SHR_ORB_UNDEF_INT
   else
     write(6,F00) 'orbit ERROR: must set input parm orb_year or'
     write(6,F00) 'all three of orb_eccen ,orb_obliq, & orb_mvelp.'
     STOP 'initial'
   endif
#ifdef MPI_P
     if(master_task) then
       orb_log = .true.
     else
       orb_log = .false.
     endif
#else
   orb_log = .true.
#endif
   call shr_orb_params( orb_year  , orb_eccen , orb_obliq , orb_mvelp, &
   &                    orb_obliqr, orb_lambm0, orb_mvelpp, orb_log  )

   !----------------------------------------------------------------------------
   ! need array dimensions for all component models
   !----------------------------------------------------------------------------
   if (nx_a .eq. unset) ierr=777
   if (ny_a .eq. unset) ierr=777
   if (nx_i .eq. unset) ierr=777
   if (ny_i .eq. unset) ierr=777
   if (nx_l .eq. unset) ierr=777
   if (ny_l .eq. unset) ierr=777
   if (nx_o .eq. unset) ierr=777
   if (ny_o .eq. unset) ierr=777
   !! (nx_r .eq. unset) ierr=777
   !! (ny_r .eq. unset) ierr=777
   if (nx_r .eq. unset) nx_r = 720 ! assume 1/2 x 1/2 degree runoff grid
   if (ny_r .eq. unset) ny_r = 360 ! assume 1/2 x 1/2 degree runoff grid
   if (ierr .eq. 777) then
      write(6,F00) 'ERROR: must input dimensions for all components'
      STOP
   else
      n_a = nx_a*ny_a
      n_i = nx_i*ny_i
      n_l = nx_l*ny_l
      n_o = nx_o*ny_o
      n_r = nx_r*ny_r
      n_rc = n_r      ! init assumed to have null compression
   endif

   write(6,F05)

END

!===============================================================================
