#include <misc.h>

PROGRAM main 

   USE map_mod        ! mapping data
   USE buf_mod        ! model   data
   USE control_mod    ! control data
   USE calendar_mod   ! calendar routines
   USE histavg_mod    ! time-avg history data file routines
   USE shr_msg_mod    ! shared code: msg passing
   USE shr_sys_mod    ! shared code: system calls
   USE msg_atm_mod    ! msg passing module ~ atm specific
   USE msg_ice_mod    ! msg passing module ~ ice specific
   USE msg_lnd_mod    ! msg passing module ~ lnd specific
   USE msg_ocn_mod    ! msg passing module ~ ocn specific
   USE diag_mod       ! diagnostics module
   USE atmdata_mod    ! for dataatm fangyj
#ifdef MPI_P
   use mpmd_mod
   use spmd_mod
   use cpl_mod   
   use comm_mod
#endif

!---------------------wangln 2007-06-26----------------------------- 
   !USE Albedo_mod       ! diagnostics module
!---------------------wangln 2007-06-26----------------------------- 

   IMPLICIT none
!   include 'mpif.h'

   !----- local -----
   integer           :: nsFoxx       ! number of Foxx   tavg samples
   real   ,pointer   :: sFoxx_o(:,:) ! array to forming Foxx   tavg's
   real   ,pointer   :: junk1 (:)    ! temporary/dummy/discard arrays
   real   ,pointer   :: junk2 (:)    ! for unwanted model IC states
   real   ,pointer   :: junk3 (:)
   real   ,pointer   :: junk4 (:)
   real   ,pointer   :: junk5 (:)
   integer           :: n            ! inner time-stepping loop index
   character(len= 8) :: dstr         ! wall clock date string yyyymmdd
   character(len=10) :: tstr         ! wall clock time string hhmmss.sss

   integer    :: t00,t01,t02,t03,t04,t05,t06,t07,t08,t09 
   integer    :: t10,t11,t12,t13,t14,t15,t16,t17,t18,t19
   integer    :: t20,t21,t22,t23,t24,t25,t26,t27,t28,t29
   integer    :: s00,s01,s02,s03,s04,s05,s06,s07,s08,s09 
   integer    :: s10,s11,s12,s13,s14,s15,s16,s17,s18,s19 
   integer    :: s20,s21,s22,s23,s24,s25,s26,s27,s28,s29 
#ifdef MPI_P
   integer    :: s30,s31,s32,s33,s34,s35,s36,s37,s38
#endif

   !----- i/o formats -----
   character(len=*), parameter :: F00 = "('(main) ',16a)"
   character(len=*), parameter :: F01 = "('(main) ',a,i4)"
   character(len=*), parameter :: F02 = "(16a)"
   character(len=*), parameter :: F06 = "(80('='))"
   character(len=*), parameter :: F07 = "(80('=')/)"
   character(len=*), parameter :: F08 = "(80('-'))"
   character(len=*), parameter :: F09 = "(80('-')/)"
   character(len=*), parameter :: F10 = &
   &   "('(main) wall clock:',1x,a4,'-',a2,'-',a2,1x,a2,':',a2,':',a2)"
   character(len=*), parameter :: F12 = &
   &   "(      'date & time:',1x,a4,'-',a2,'-',a2,1x,a2,':',a2,':',a2)"
   character(len=*), parameter :: F99 = &
   & "('NCAR CSM Flux Coupler, (C) 2001 UCAR, All Rights Reserved')"

#ifdef MPI_P
   integer iv,sendcnt,recvcnt
   integer i,j,k,maxlen,rank,mpes
   real (SHR_KIND_R8), allocatable :: redu1(:),redu2(:)
   real (SHR_KIND_R8), allocatable :: redu3(:,:),redu4(:,:)
#endif 
!-------------------------------------------------------------------------------
! PURPOSE:
!   This is the CSM Flux Coupler component which coordinates the 
!   flux calculations & time integration of all component models: 
!   o cpl - this flux coupler
!   o atm - the atmosphere model
!   o ice - the sea ice model
!   o lnd - the land surface model
!   o ocn - the global ocn model
!-------------------------------------------------------------------------------

#ifdef MPI_P
    write(6,*) "11111111111111111111111111111"
    call shr_sys_flush(6)

    call mpi_init( ierr)
    if( ierr == MPI_SUCCESS ) write( *,*) ' MPI_INIT is OK !'

    print *,"122222222221111111111111111111111111111"
    call shr_sys_flush(6)

    call mpi_coupled
    write(6,*) ' Coupled Initialied is OK! '
    call shr_sys_flush(6)

    print *,"-----------------Coupled Initialied is OK!"
    call shr_sys_flush(6)

    !-----set up the CPL Flag
 
    master_task = .true.
 
    write(6,*) ' Setup_mpi is Ok!'
    call shr_sys_flush(6)
#endif

   call shr_msg_stdio('cpl') ! redirect stdin/stdout
                             ! include( call shr_msg_chdir(model)/call shr_msg_dirio(model)  )

   print *,"aaa22222222222222222222222222"

#ifdef MPI_P
!   call spmd_init(MPI_COMM_CPL)
    call mpi_comm_size( MPI_COMM_CPL, mpes, ierr )
    call mpi_comm_rank( MPI_COMM_CPL, rank, ierr )

    if( rank==0 ) then
       master_task = .true.
    else
       master_task = .false.
    endif
#endif

   print *,"bbb22222222222222222222222222"

   call date_and_time(dstr,tstr)

   write(6,F06)
   write(6,F06)
   write(6,F00) 'Executing CSM Flux Coupler'
   write(6,F12) dstr(1:4),dstr(5:6),dstr(7:8) ,tstr(1:2),tstr(3:4),tstr(5:6)
   write(6,F00) 'CVS tag $Name: ccsm2_0_beta58 $'
   write(6,F06)

   call shr_timer_init()
   call shr_timer_get(t00,'t00 - startup overhead')
   call shr_timer_get(t01,'t01')
   call shr_timer_get(t02,'t02')
   call shr_timer_get(t03,'t03')
   call shr_timer_get(t04,'t04')
   call shr_timer_get(t05,'t05')
   call shr_timer_get(t06,'t06')
   call shr_timer_get(t07,'t07')
   call shr_timer_get(t08,'t08')
   call shr_timer_get(t09,'t09')
   call shr_timer_get(t10,'t10')
   call shr_timer_get(t11,'t11')
   call shr_timer_get(t12,'t12')
   call shr_timer_get(t13,'t13')
   call shr_timer_get(t14,'t14')
   call shr_timer_get(t15,'t15')
   call shr_timer_get(t16,'t16')
   call shr_timer_get(t17,'t17')
   call shr_timer_get(t18,'t18')
   call shr_timer_get(t19,'t19')
   call shr_timer_get(t20,'t20')
   call shr_timer_get(t21,'t21')
   call shr_timer_get(t22,'t22')
   call shr_timer_get(t23,'t23')
   call shr_timer_get(t24,'t24')
   call shr_timer_get(t25,'t25')
   call shr_timer_get(t26,'t26')
   call shr_timer_get(t27,'t27')
   call shr_timer_get(t28,'t28')
   call shr_timer_get(s01,'s01')
   call shr_timer_get(s02,'s02')
   call shr_timer_get(s03,'s03')
   call shr_timer_get(s11,'s11')
   call shr_timer_get(s12,'s12')
   call shr_timer_get(s13,'s13')
   call shr_timer_get(s21,'s21')
   call shr_timer_get(s22,'s22')
   call shr_timer_get(s23,'s23')

#ifdef MPI_P
   call shr_timer_get(s30,'s30')
   call shr_timer_get(s31,'s31')
   call shr_timer_get(s32,'s32')
   call shr_timer_get(s33,'s33')
   call shr_timer_get(s34,'s34')
   call shr_timer_get(s35,'s35')
#endif
   call shr_timer_start(t00)

   !----------------------------------------------------------------------------
   ! initialize
   !----------------------------------------------------------------------------
#if (!defined MPI_P)
   call msg_cplc ()             ! connect to msg passing environment
   write(6,*) 'Main call msg_cplc ()';  call shr_sys_flush(6) 
#endif

   call initial ()              ! initialize & input namelist & related

   print *,"333333333333333333333332"
   write(6,*) 'Main call initial ()';  call shr_sys_flush(6) 

   call buf_alloc ()            ! allocate model msg buffers

   call atmdata_alloc()         ! dataatm  

   allocate(sFoxx_o(n_o,nFoxx)) ! alloc sFoxx    (wrt ocn daily-avgs)
   sFoxx_o = 0.

   write(6,*) 'Main call bufalloc ()';  call shr_sys_flush(6) 
   write(6,*) 'Main allocate(sFoxx_o(n_o,nFoxx))';  call shr_sys_flush(6) 

   call restart ('read')        ! get restart data (input or fabricate)
   write(6,*) 'Main restart';  call shr_sys_flush(6) 

   call control()               ! set control flags: stop, rest, hist, etc
   write(6,*) 'Main control';  call shr_sys_flush(6) 

   call map_init ()             ! input/initialize mapping data
   write(6,*) 'Main map_init ()';  call shr_sys_flush(6) 


   !----------------------------------------------------------------------------
   ! exchange time invariant msgs with component models
   !----------------------------------------------------------------------------
   call msg_atmi ()
   print *,"4444444444444444444444444444....end msg_atmi"
   call msg_icei ()
   print *,"4444444444444444444444444444....end msg_icei"
   call msg_ocni ()
   print *,"4444444444444444444444444444....end msg_ocni"
   call msg_lndi ()
   print *,"4444444444444444444444444444....end msg_lndi"
  

   !----------------------------------------------------------------------------
   ! verify communication intervals are acceptable
   !----------------------------------------------------------------------------
   write(6,F01) 'number of atm communications/day = ',ncpl_a
   write(6,F01) 'number of lnd communications/day = ',ncpl_l
   write(6,F01) 'number of ice communications/day = ',ncpl_i
   write(6,F01) 'number of ocn communications/day = ',ncpl_o
   if (ncpl_a /= ncpl_l) then
      write(6,F02) 'ERROR: communication interval ncpl_a /= ncpl_l'
      call shr_sys_abort("(main) comm interval mis-match")
   else if (ncpl_a /= ncpl_i) then
      write(6,F02) 'ERROR: communication interval ncpl_a /= ncpl_i '
      call shr_sys_abort("(main) comm interval mis-match")
   else if (ncpl_a < ncpl_o) then
      write(6,F02) 'ERROR: communication interval ncpl_a < ncpl_o '
      call shr_sys_abort("(main) comm interval mis-match")
   else if (mod(ncpl_a,ncpl_o) /= 0) then
      write(6,F02) 'ERROR: communication interval mod(ncpl_a,ncpl_o) /= 0 '
      call shr_sys_abort("(main) comm interval mis-match")
   end if
   call shr_sys_flush(6)

   print *, "5555555555555555555555555555"
   call shr_sys_flush(6)

   !----------------------------------------------------------------------------
   ! optional initialization: cpl tells lnd what domain spec to use
   !----------------------------------------------------------------------------
   if ( init_maskl.ne.0 ) then
      write(6,F00) 'send lnd domain spec to lnd (optional, requested by lnd)'
      xc_l   = xc_a            ! coordinates & cell area same as for atm
      yc_l   = yc_a
      xv_l   = xv_a
      yv_l   = yv_a
      area_l = area_a 
      mask_l = domain_l%mask   ! lnd mask as determined by cpl (wrt map data)
     

      allocate(junk1(n_i))
      junk1   = 0.0            ! use bogus ice fraction = zero to ...
      call map_frac(junk1)     ! ... compute all surface fractions (esp. fl_l)
      deallocate(junk1)
      write(6,*) ' ==========Now begin the MSG_LNDM================='
      call msg_lndm()          ! send this domain spec to lnd
   print *, "end msg_lndm...666666666666666666666666666666"
   end if

   print *, "666666666666666666666666666666"
   call shr_sys_flush(6)

   !----------------------------------------------------------------------------
   ! receive IC data from models (& decide whether to use it or not)
   !----------------------------------------------------------------------------
   if (rest_type(1:4) .eq. 'init') then
      !--- initial run => use model's IC data ---
      write(6,F00) 'using IC data from models'
      call msg_icer ( Si_i , Fioi_i , Faii_i)
      call msg_ocnr ( So_o , Fioo_o )
      call msg_lndr ( Sl_l , Fall_l , roff_rc)
      call msg_atmr ( Sa_a , Faxa_a , Faia_a , Fala_a , Faoa_a)

      if(data_atm .and. cdate >= atm_begin .and. cdate < atm_end) then   ! dataatm fangyj 
       call advance_atmdata()                                            ! dataatm fangyj
       call replace_atmdata( Sa_a , Faxa_a , Faia_a , Fala_a , Faoa_a)   ! dataatm fangyj
      endif                                                              ! dataatm fangyj
   else
      !--- continue run => use model or cpl IC data? ---
      if (init_rica .ne. 0) then
         write(6,F00) 'using atm IC data provided by atm model'
         call shr_sys_flush(6)
         call msg_atmr (Sa_a , Faxa_a , Faia_a , Fala_a , Faoa_a)
      else           
         write(6,F00) 'using atm IC data from cpl restart file'
         call shr_sys_flush(6)
         allocate(junk1(n_a*nSa  ),junk2(n_a*nFaxa),junk3(n_a*nFaia))
         allocate(junk4(n_a*nFala),junk5(n_a*nFaoa)                 )
!--------xjx added 20080510
         junk1 = 0.0
         junk2 = 0.0
         junk3 = 0.0
         junk4 = 0.0
         junk5 = 0.0
!--------xjx end
         call msg_atmr (junk1,junk2,junk3,junk4,junk5) 
         deallocate    (junk1,junk2,junk3,junk4,junk5)
      end if
      if (init_rici .ne. 0) then
         write(6,F00) 'using ice IC data provided by ice model'
         call msg_icer ( Si_i , Fioi_i , Faii_i)
         call flux_albi( Si_i)
         call map( map_i2af, Faii_i, Faii_a ,fi_i,fi_a)
      else           
         write(6,F00) 'using ice IC data from cpl restart file'
         allocate   (junk1(n_i*nSi),junk2(n_i*nFioi),junk3(n_i*nFaii))
!--------xjx added 20080510
         junk1 = 0.0
         junk2 = 0.0
         junk3 = 0.0
!--------xjx end
         call msg_icer (junk1,junk2,junk3)
         deallocate    (junk1,junk2,junk3)
      end if
      if (init_ricl .ne. 0) then
         write(6,F00) 'using lnd IC data provided by lnd model'
         call msg_lndr ( Sl_l ,  Fall_l ,  roff_rc)
      else           
         write(6,F00) 'using lnd IC data from cpl restart file'
         allocate   (junk1(n_l*nSl),junk2(n_l*nFall),junk3(n_rc))
         call msg_lndr (junk1,junk2,junk3)
         deallocate    (junk1,junk2,junk3)
      end if
      if (init_rico .ne. 0) then
         write(6,F00) 'using ocn IC data provided by ocn model'
         call msg_ocnr ( So_o ,  Fioo_o )
         call flux_albo( So_o)   !combute computer
      else           
         write(6,F00) 'using ocn IC data from cpl restart file'
         allocate   (junk1(n_o*nSo),junk2(n_o*nFioo))
         call msg_ocnr (junk1,junk2)
         deallocate    (junk1,junk2)
      end if
   end if

   print *, "77777777777777777777777"
   call shr_sys_flush(6)

   !----------------------------------------------------------------------------
   ! consistency check of model domains & mapping data
   !----------------------------------------------------------------------------
#ifdef MPI_P
#if 1
   allocate(redu1(n_i/CPL_SIZE))
   allocate(redu2(n_i))
   redu1=0.
   redu2=0.
     iv = 1
     do j=ist_i,iend_i
       redu1(iv) = Si_i(j,kSi_ifrac)
       iv = iv+1
     enddo
     call MPI_allgather(redu1,iv-1,MPI_REAL8,&
           &  redu2,iv-1,MPI_REAL8,MPI_COMM_CPL,ierr)
#else
   allocate(redu1(n_i))
   allocate(redu2(n_i))
   redu1=0.
   redu2=0.
     do j=ist_i,iend_i
       redu1(j) = Si_i(j,kSi_ifrac)
     enddo
     call MPI_ALLREDUCE(redu1,redu2,n_i,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
#endif
   Si_i(:,kSi_ifrac) = redu2(:)
   deallocate(redu1)
   deallocate(redu2)
#endif
   call map_frac (Si_i(:,kSi_ifrac))  ! compute surface fractions
   call map_check()
   call shr_sys_flush(6)

   !----------------------------------------------------------------------------
   ! optional initialization: recompute atm solar radiation wrt surface albedo
   !----------------------------------------------------------------------------
   if (init_solar.ne.0 .and. stop_now.eq.0) then
      write(6,F00) 're-initialize atm solar (optional, requested by atm)'
      call shr_sys_flush(6)

      !--- prepare atm input *states* & send to atm model ---
      call map( map_i2as , Si_i, Si_a ,fi_i,fi_a)
      call map( map_o2as , So_o, So_a ,fa_o,fo_a)
      call map( map_l2as , Sl_l, Sl_a )
      call mrgs_atm (  Si_a ,  Sl_a ,  So_a ,  Ss_a)

#ifdef MPI_P
      call comm1(Ss_a,Faxx_a)
#endif
      call msg_atms (Faxx_a ,  Ss_a )  ! -> only Ss_a is valid <-

      !--- receive atm outputs ---
      call msg_atmr (Sa_a ,Faxa_a, Faia_a ,Fala_a ,Faoa_a)

      if(data_atm .and. cdate >= atm_begin .and. cdate < atm_end) then   ! dataatm fangyj
       call advance_atmdata()                                            ! dataatm fangyj
       call replace_atmdata( Sa_a , Faxa_a , Faia_a , Fala_a , Faoa_a)   ! dataatm fangyj
      endif                                                              ! dataatm fangyj
   end if

   !----------------------------------------------------------------------------
   ! check time coordination of IC data
   !----------------------------------------------------------------------------
   if (init_tavg.eq.0) then
      !--- in this case, all model should be coordinated in time ---
      write(6,F08)
      write(6,F01) ' Start of time coordinated integration'
      write(6,F08)
      call tcheck(1)  ! 1 => enforce time coordination
   else
      !--- in this case, ocn is initially ahead of other models ---
      write(6,F08)
      write(6,F01) ' Integrate to form initial tavg ocn input flux'
      write(6,F08)
      call tcheck(0)  ! 0 <=> don't enforce time coordination
   end if
   call shr_sys_flush(6)

   print *, "88888888888888888888888888"
   !----------------------------------------------------------------------------
   ! recreate data needed for 1st iteration of main event loop
   !----------------------------------------------------------------------------

   !--- need So_i, Fioo_i for 1st ice-send ---
   call map( map_o2is,   So_o,   So_i )
   call map( map_o2if, Fioo_o, Fioo_i )

   !--- need Fioi_o for 1st ocn mrg ---
   call map( map_i2of, Fioi_i, Fioi_o )

   !--- need Ss_a for 1st diagnostic call ---
   call map( map_i2as, Si_i, Si_a ,fi_i,fi_a)
   call map( map_l2as, Sl_l, Sl_a )
   call map( map_o2as, So_o, So_a ,fa_o,fo_a)
   call mrgs_atm ( Si_a, Sl_a, So_a, Ss_a)

   !--- need Faxx_a for 1st diagnostic call ---
   call map( map_l2af, Fall_l, Fall_a )
   call map( map_i2af, Faii_i, Faii_a ,fi_i, fi_a)
   call map( map_o2af, Faoc_o, Faoc_a ,fa_o, fo_a)
   call mrgf_atm (Faii_a, Fall_a ,Faoc_a ,Faxx_a)   !xjx???

   call shr_timer_stop (t00)
   call shr_timer_print(t00)
   call shr_timer_free (t00)
   call shr_timer_get  (t00,'t00 - main integration loop')
   call shr_timer_start(t00)


   print *, "99999999999999999999999999999999"
      call shr_sys_flush(6)

#ifdef MPI_P
!   call MPI_BARRIER(MPI_COMM_CPL,ierr)
#endif
   !----------------------------------------------------------------------------
   ! start of model integration
   !----------------------------------------------------------------------------
   DO WHILE ( stop_now .eq. 0 )

      DO n=0,ncpl_a-1  ! advance one day
         
!      write(6,*)"....loop......step,ncpl_a,ncpl_o,init_tavg=",n,ncpl_a,ncpl_o,init_tavg

      if( n== 0 ) write(6,*) '---- Now Open the New Cycle--------------------------' 
      call shr_sys_flush(6)

         call shr_timer_start(t01)

         !--- record wall clock time ---
         if (day.eq.1 .and. sec.eq.0) then
            call date_and_time(dstr,tstr)
            write(6,F10) dstr(1:4),dstr(5:6),dstr(7:8) &
            &           ,tstr(1:2),tstr(3:4),tstr(5:6)
         end if
         call shr_timer_stop (t01) ; call shr_timer_start(t02)

         !--- full precision result test ---
         call bitcheck()

         call shr_timer_stop (t02) ; call shr_timer_start(t03)

         !--- send msg to ocean ---
         write(6,*) '--fangyj ncpl_a ncpl_l ncpl_o,mod(n,ncpl_a/ncpl_o)= ', n, ncpl_a,ncpl_l, ncpl_o, mod(n,ncpl_a/ncpl_o)
         if (mod(n,ncpl_a/ncpl_o) == 0 ) then
            call shr_timer_start(s11)
            if (init_tavg == 0) then
#ifdef MPI_P
               call comm2(aFoxx_o)
#endif
               call msg_ocns (aFoxx_o) 
            endif
            call shr_timer_stop (s11) ; call shr_timer_start(s12)

            !--- re-initialize ocn tavg data ---
            !!if( n /= 0 ) then
#ifdef MPI_P
             call t_avg('zero',n_o,nFoxx,Foxx_o(1,1),sFoxx_o(1,1),nsFoxx  )
#else
             call t_avg('zero',n_o*nFoxx,Foxx_o(1,1),sFoxx_o(1,1),nsFoxx  )
#endif
            !!endif
            call shr_timer_stop (s12)
         endif
         call shr_timer_stop (t03) ; call shr_timer_start(t04)

         !--- map atm states & fluxes to lnd ---

!--------------------wangln 2007-06-12 Add the Bilinear Correct-------------------
         call map( map_a2ls,   Sa_a,   Sa_l )
         call map( map_a2lf, Fala_a, Fala_l )
         !!call map( map_a2ls,   Sa_a,   Sa_l, remap_method=bilinear )
         !!call map( map_a2lf, Fala_a, Fala_l, remap_method=bilinear )
!--------------------wangln 2007-06-12 Add the Bilinear Correct-------------------

         call shr_timer_stop (t04) ; call shr_timer_start(t05)

!lnds    !--- send data to lnd model ---
#ifdef MPI_P
         call comm3(Sa_l,Fala_l)
#endif
         call msg_lnds (Fala_l , Sa_l   )
         call shr_timer_stop (t05) ; call shr_timer_start(t06)

         !--- compute and map ice inputs ----

!--------------------wangln 2007-06-12 Add the Bilinear Correct-------------------
         !call map( map_a2is,   Sa_a,   Sa_i ) ; !call map_npfix(Sa_a,Sa_i)
         !call map( map_a2if, Faia_a, Faia_i )
         call map( map_a2is,   Sa_a,   Sa_i, remap_method=bilinear ) 
         call map( map_a2if, Faia_a, Faia_i, remap_method=bilinear )
!--------------------wangln 2007-06-12 Add the Bilinear Correct-------------------

         call shr_timer_stop (t06) ; call shr_timer_start(t07)

         !--- force zero net water flux into ocn+ice ? ----
         if (flx_epbal(1:3) /= 'off') then
             call map( map_rc2o,  roff_rc, roff_o )

!--------------------wangln 2007-06-12 Add the Bilinear Correct-------------------
             !call map( map_a2of,   Faoa_a, Faoa_o )
             !call map( map_a2os,     Sa_a,   Sa_o ) ; !call map_npfix(Sa_a,Sa_o)
             call map( map_a2of,   Faoa_a, Faoa_o, remap_method=bilinear )
             call map( map_a2os,     Sa_a,   Sa_o, remap_method=bilinear ) 
!--------------------wangln 2007-06-12 Add the Bilinear Correct-------------------

!!xjx-----------in this case,not call here--------------
             call flux_epbal(Faia_i,Faii_i,Faoa_o,Faoc_o,roff_o,fa_o,fi_o)
         endif
         call shr_timer_stop (t07) ; call shr_timer_start(t08)

!ices    !--- send msg to ice (tavg data) ---

#ifdef MPI_P
         call shr_timer_start(s30)
         call comm4(Sa_i,So_i,Faia_i,Fioo_i,roff_o)
         call shr_timer_stop (s30)
#endif
         call msg_ices (Sa_i, So_i, Faia_i, Fioo_i)
         call shr_timer_stop (t08) ; call shr_timer_start(t09)

         !--- map atm & lnd output fluxes to ocn ---
         if (flx_epbal(1:3) == 'off') then
            call map( map_rc2o,  roff_rc, roff_o )

!--------------------wangln 2007-06-12 Add the Bilinear Correct-------------------
             !call map( map_a2of,   Faoa_a, Faoa_o )
             !call map( map_a2os,     Sa_a,   Sa_o ) ; !call map_npfix(Sa_a,Sa_o)
             call map( map_a2of,   Faoa_a, Faoa_o, remap_method=bilinear )
             call map( map_a2os,     Sa_a,   Sa_o, remap_method=bilinear ) 
!--------------------wangln 2007-06-12 Add the Bilinear Correct-------------------

         endif

         call flux_solar(Faoa_o, So_o, Faoc_o )
         call shr_timer_stop (t09) ; call shr_timer_start(t10)

!ocns    !--- merge/"send" ocn inputs ---

#ifdef HAMOCC
         call mrgf_ocn ( Faoa_o, Faoc_o, Fioi_o, roff_o, Sa_o, fi_o, Si_i, Foxx_o)
#else
         call mrgf_ocn ( Faoa_o, Faoc_o, Fioi_o, roff_o, Sa_o, fi_o, Foxx_o)
#endif

         call shr_timer_stop (t10) ; call shr_timer_start(t11)


         !--- form tavg partial sum of ocn inputs ---
#ifdef MPI_P
         call t_avg('sum',n_o,nFoxx,Foxx_o(1,1),sFoxx_o(1,1),nsFoxx  )
#else
         call t_avg('sum',n_o*nFoxx,Foxx_o(1,1),sFoxx_o(1,1),nsFoxx  )
#endif
         call shr_timer_stop (t11) ; call shr_timer_start(t12)

         !--- do diagnostics ---
         if (init_tavg == 0 .or. n /= 0) then
            call diagnos ()
         endif
         call shr_timer_stop (t12) ; call shr_timer_start(t13)

!ocnr    !--- "recv" new ocn data ---
         call flux_albo(So_o) ! compute ocn albedos
         call shr_timer_stop (t13) ; call shr_timer_start(t14)

         !--- compute cpl's atm/ocn fluxes ---
         if( ocn_type(1:3) == 'POP' ) then
             call flux_ao( Sa_o, So_o, Faoc_o )
         else if( ocn_type(1:3) == 'MOM' ) then
#ifdef MPI_P
         call shr_timer_start(s31)
         call comm5(Sa_o,So_o,Faoc_o)
         call shr_timer_stop (s31)
#endif

         call flux_ao_mom( Sa_o, So_o, Faoc_o ) 
!wtw         call flux_ao( Sa_o, So_o, Faoc_o )
!----
         endif 
         call shr_timer_stop (t14) ; call shr_timer_start(t15)

!icer    !--- recv new ice data ---

         call msg_icer (Si_i ,Fioi_i ,Faii_i)

         call shr_timer_stop (t15) ; call shr_timer_start(t16)
         call flux_albi(Si_i) ! modify ice albedos?

#ifdef MPI_P
#if 1
       allocate(redu1(n_i/CPL_SIZE))
       allocate(redu2(n_i))
       redu1=0.
       redu2=0.
       iv = 1
       do j=ist_i,iend_i
         redu1(iv) = Si_i(j,kSi_ifrac)
         iv = iv+1
       enddo
       call MPI_allgather(redu1,iv-1,MPI_REAL8,&
           &  redu2,iv-1,MPI_REAL8,MPI_COMM_CPL,ierr)
#else
       allocate(redu1(n_i))
       allocate(redu2(n_i))
       redu1=0.
       redu2=0.
       do j=ist_i,iend_i
         redu1(j) = Si_i(j,kSi_ifrac)
       enddo
       call MPI_ALLREDUCE(redu1,redu2,n_i,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
          Si_i(:,kSi_ifrac) = redu2(:)
#endif
       deallocate(redu1)
       deallocate(redu2)
#endif
         !--- update surface fracs wrt new ice frac ---
         call map_frac (Si_i(:,kSi_ifrac))
         call shr_timer_stop (t16) ; call shr_timer_start(t17)

         !--- map states/fluxes to atm & ocn grid ---
         call map( map_i2af, Faii_i, Faii_a ,fi_i, fi_a)
         call map( map_i2as,   Si_i,   Si_a ,fi_i, fi_a)
         call map( map_i2of, Fioi_i, Fioi_o )
         call shr_timer_stop (t17) ; call shr_timer_start(t18)

         !--- map ocn states/fluxes to atm ---
         call map( map_o2af, Faoc_o, Faoc_a, fa_o, fo_a)
         call map( map_o2as,   So_o,   So_a, fa_o, fo_a)
         call shr_timer_stop (t18) ; call shr_timer_start(t19)

!lndr    !--- recv new lnd data ---
         call msg_lndr (Sl_l, Fall_l, roff_rc )
         call shr_timer_stop (t19) ; call shr_timer_start(t20)

         !--- verify net solar calcs are coordinated ---
         call diag_solar ()
         call shr_timer_stop (t20) ; call shr_timer_start(t21)


         !--- map lnd states & fluxes to atm ---
         call map( map_l2af, Fall_l, Fall_a )
         call map( map_l2as,   Sl_l,   Sl_a )
         call shr_timer_stop (t21) ; call shr_timer_start(t22)

         !--- merge atm states & fluxes ---
         call mrgf_atm (Faii_a, Fall_a, Faoc_a, Faxx_a)   !xjx????
         call mrgs_atm (  Si_a,   Sl_a,   So_a,   Ss_a)   !xjx???/
         call shr_timer_stop (t22) ; call shr_timer_start(t23)
       
!atms    !--- send data to atm ---
#ifdef MPI_P
         call comm1(Ss_a,Faxx_a)
#endif
         call msg_atms (Faxx_a, Ss_a )
         call shr_timer_stop (t23) ; call shr_timer_start(t24)

         !--- add data to history file ---
#ifdef MPI_P
        call shr_timer_start(s32)
        call comm7(Foxx_o,Faoc_o)
        call shr_timer_stop (s32) ;
        call shr_timer_start(s33)
#endif
!! wangln 2009-01-24        call history ()

#ifdef MPI_P
        call shr_timer_stop (s33) ;  call shr_timer_start(s34) 
        if(master_task) call histavg ()
        call shr_timer_stop (s34)  
#else
        call histavg ()
#endif

         call shr_timer_stop (t24) ; call shr_timer_start(t25)

         if (mod(n+1,ncpl_a/ncpl_o) == 0 ) then
            call shr_timer_start(s21)
            !--- form tavg of ocn inputs ---
#ifdef MPI_P
            call t_avg('avg',n_o,nFoxx,sFoxx_o(1,1),aFoxx_o(1,1),nsFoxx)
#else
            call t_avg('avg',n_o*nFoxx,sFoxx_o(1,1),aFoxx_o(1,1),nsFoxx)
#endif
            call shr_timer_stop (s21) ; call shr_timer_start(s22)
 
            !--- recv msg from ocn ---

            if (init_tavg == 0) then
               call msg_ocnr(So_o, Fioo_o)
            endif
            call shr_timer_stop (s22) ; call shr_timer_start(s23)
           
            !--- map ocn to ice ---
            call map( map_o2is,   So_o,   So_i )
            call map( map_o2if, Fioo_o, Fioo_i )

            call shr_timer_stop (s23) 

            !--- start normal interaction with ocn ---
            if (init_tavg /= 0) then
               write(6,F08)
               write(6,F01) ' Start of time coordinated integration'
               write(6,F08)
               init_tavg=0   ! done forming initial tavg ocn forcing
            end if
         end if
         call shr_timer_stop (t25) ; call shr_timer_start(t26)
 
!atmr    !--- recv msg from atm ---
         call msg_atmr (Sa_a, Faxa_a, Faia_a, Fala_a, Faoa_a)
         if(data_atm .and. cdate >= atm_begin .and. cdate < atm_end) then    ! dataatm fangyj
           call advance_atmdata()                                            ! dataatm fangyj
           call replace_atmdata( Sa_a , Faxa_a , Faia_a , Fala_a , Faoa_a)   ! dataatm fangyj
         endif                                                               ! dataatm fangyj
         call shr_timer_stop (t26) ; call shr_timer_start(t27)

         !--- advanced model time ---
         sec = sec + 86400/ncpl_a
         if (sec.ge.86400) then
            sec = 0       ! it's a new day!
            eday = eday + 1  
            call eday2date(eday,cdate)
            call eday2ymd (eday,year,month,day)
         end if
         call control() ! reset stop, rest, hist control flags

         call shr_timer_stop (t27)

      END DO

#ifdef MPI_P
      if(master_task) &
     & write(6,*) "end one day............................"
#endif
      print *

      call tcheck(1)   ! 1 => enforce time coordination

      !--- make a cpl restart file ---
      call shr_timer_start(t28)
      call restart ('write')
      call shr_timer_stop (t28) 

      call shr_sys_flush(6)
   END DO

   !--- wall-clock time-stamp ---
   write(6,F10) dstr(1:4),dstr(5:6),dstr(7:8) ,tstr(1:2),tstr(3:4),tstr(5:6)

   !----------------------------------------------------------------------------
   ! send final msg to models, dis-connect msg-passing, shutdown
   !----------------------------------------------------------------------------
   call shr_timer_stop (t00)
   call shr_timer_print_all()
   call shr_sys_flush(6)
#ifdef MPI_P
   call comm7(Foxx_o,Faoc_o)
#endif
   call history  ()  ! last chance to form/save/archive data 

#ifdef MPI_P
   if(master_task) call histavg ()
#else
   call histavg ()
#endif
   call msg_atmf ()
   call msg_lndf ()
   call msg_icef ()
   call msg_ocnf ()
   call msg_cpld ()

 999 CONTINUE


   call date_and_time(dstr,tstr)
   write(6,F06)
   write(6,F00) 'Terminating CSM coupler'
   write(6,F12) dstr(1:4),dstr(5:6),dstr(7:8) ,tstr(1:2),tstr(3:4),tstr(5:6)
   write(6,F07)
   call shr_sys_flush(6)
STOP
END

!===============================================================================
