!===============================================================================
! CVS $Id: map_mod.F90,v 1.17 2002/01/12 00:23:38 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/map_mod.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

MODULE map_mod

   use kind_mod         ! access to kind declarations
   use buf_mod          ! model field data buffers
   use shr_timer_mod    ! timing routines
   use shr_sys_mod      ! system calls
   use control_mod      ! mapping data file names

   implicit none

   !----------------------------------------------------------------------------
   ! 2d domain: a 1d vector that describes equivalent 2d f90 ordered array data
   ! 
   ! notes:
   ! o all arrays have size (n), except xv & yv which are (4,n) and iunc(nc)
   ! 
   ! notes on domain compression:
   ! o a domain has only one associated compression scheme
   ! o key,nc,iunc,icom describe the associated compression scheme
   ! o nc is the number of non-zero elements in the key
   ! o nc is the number of elements in a compressed data vector
   ! o key(m) = 0 <=> element m has been removed in compressed vector
   ! o if key(m) /= 0 , this is called "null compression" (ie. not compressed)
   ! o n == nc                           <=> null compression scheme
   ! o iunc(:) = icom(:) = {1,2,3,...,n} <=> null compression scheme
   ! o iunc(:) & icom(:) must both be strictly increasing
   !----------------------------------------------------------------------------
   TYPE domain   
     integer         :: n         ! n = ni*nj ~ total number of grid points
     integer         :: ni        ! number of 2d array i indicies
     integer         :: nj        ! number of 2d array j indicies
     integer         :: nv        ! number of verticies per grid cell ~ always 4
     real   ,pointer :: xc  (:)   ! x-coords of centers   ~ degrees east
     real   ,pointer :: yc  (:)   ! y-coords of centers   ~ degrees north
     real   ,pointer :: xv  (:,:) ! x-coords of verticies ~ degrees east, (nv,n)
     real   ,pointer :: yv  (:,:) ! y-coords of verticies ~ degrees north (nv,n)
     integer,pointer :: mask(:)   ! mask: 0 <=> out-of-domain (invalid data)
     real   ,pointer :: area(:)   ! area of grid cell ~ radians squared
     integer,pointer :: key (:)   ! key: 0 <=> data omitted in compressed vector
     integer         :: nc        ! number of grid points in compressed vector
     integer,pointer :: iunc(:)   ! uncomp index for given compressed index
     integer,pointer :: icom(:)   ! compressed index for given uncomp index
   END TYPE domain   

   !----------------------------------------------------------------------------
   ! mapping matrix data (f90 ordering, but with zeros removed)
   !----------------------------------------------------------------------------
   TYPE sparse_matrix
     integer         :: n         ! number of non-zero matrix elements
     real   ,pointer :: s  (:)    ! the non-zero matrix elements
     integer,pointer :: row(:)    ! matrix row corresponding to each element
     integer,pointer :: col(:)    ! matrix col corresponding to each element
     integer,pointer :: sn1(:)    ! sn1(i) = least n st row(n)=i
     integer         :: n_a       ! size of input vector
     integer         :: n_b       ! size of output vector
     integer,pointer :: mask_a(:) ! mask of source vector
     integer,pointer :: mask_b(:) ! mask of destination vector

!----------------wangln 2007-05-27 Add the Bilinear parameter---------------------
     real   ,pointer :: dx  (:)    ! the X-coordinate bilinear cofficient ( = Coff. * 1/( cossin( lat) *dlon) )
     real   ,pointer :: dy  (:)    ! the Y-coordinate bilinear cofficient ( = Coff. * 1/ dlat )
     integer,pointer :: x1(:)      ! The West  X-coordinate grid No. for Side1
     integer,pointer :: x2(:)      ! The East  X-coordinate grid No. for Side1
     integer,pointer :: y1(:)      ! The South Y-coordinate grid No. for Side1
     integer,pointer :: y2(:)      ! The North Y-coordinate grid No. for Side1
!----------------wangln 2007-05-27 Add the Bilinear parameter---------------------

   END TYPE sparse_matrix

   !----------------------------------------------------------------------------
   ! domain specifications found in mapping matrix data file
   !----------------------------------------------------------------------------
   type(domain) :: domain_a    ! atm domain
   type(domain) :: domain_i    ! ice domain
   type(domain) :: domain_l    ! lnd domain
   type(domain) :: domain_o    ! ocn domain
   type(domain) :: domain_r    ! runoff domain

   !----------------------------------------------------------------------------
   ! mapping matrix data (actual data or pointer to data)
   !----------------------------------------------------------------------------
   type(sparse_matrix),target  :: map_a2of 
   type(sparse_matrix),target  :: map_a2os
   type(sparse_matrix),target  :: map_o2af 
   type(sparse_matrix),POINTER :: map_o2as  ! will point to map_o2af
   type(sparse_matrix),POINTER :: map_a2if
   type(sparse_matrix),POINTER :: map_a2is  ! assume ocn & ice are on same grid
   type(sparse_matrix),POINTER :: map_i2af  ! and point to a2o & o2a maps
   type(sparse_matrix),POINTER :: map_i2as  ! instead of duplicating data 
   type(sparse_matrix),target  :: map_a2lf 
   type(sparse_matrix),target  :: map_a2ls
   type(sparse_matrix),target  :: map_l2af 
   type(sparse_matrix),target  :: map_l2as 
   type(sparse_matrix),target  :: map_o2if 
   type(sparse_matrix),target  :: map_o2is 
   type(sparse_matrix),target  :: map_i2of 
   type(sparse_matrix),target  :: map_i2os 
   type(sparse_matrix),target  :: map_r2o   ! original, uncompressed map
   type(sparse_matrix),target  :: map_rc2o  ! compressed map

   !----------------------------------------------------------------------------
   ! interface fractions (fraction of atm,ice,lnd,ocn on various grids)
   !----------------------------------------------------------------------------
   real,allocatable,target :: fa_a(:),fl_a(:),fi_a(:),fo_a(:) ! on atm grid
   real,allocatable,target :: fa_i(:),fl_i(:),fi_i(:),fo_i(:) ! on ice grid
   real,allocatable,target :: fa_l(:),fl_l(:),fi_l(:),fo_l(:) ! on lnd grid
   real,allocatable,target :: fa_o(:),fl_o(:),fi_o(:),fo_o(:) ! on ocn grid

   SAVE

!===============================================================================
CONTAINS 
!===============================================================================

SUBROUTINE map_init()

   !--- modules ---

   implicit none

   !--- includes ---
#include <netcdf.inc>
#ifdef MPI_P
   include "mpif.h"
#endif

   !--- arguments ---

   !--- local ---
   real   ,pointer :: f_o(:,:)  ! frac of land on ocn grid, cast into 2d array
   real   ,pointer :: f_a(:,:)  ! frac of land on atm grid, cast into 2d array

   !--- formats ---
   character(len=*),parameter :: F00 = "('(map_init) ',a60)"

#ifdef MPI_P
   real (SHR_KIND_R8), allocatable :: redu1(:),redu2(:)
   integer j,iv
#endif
!-------------------------------------------------------------------------------
! PURPOSE:
! o allocate memory for mapping data (sparse matrix data) 
! o initialize mapping data (input data from a netCDF file)
! o allocate surface fraction vectors (values are not initialized here)
! 
! NOTES:
!   this routine is hard-coded for these assumptions:
!   1) ice & ocn have identical domains
!   2) atm & lnd have identical domains, except for the lnd mask
!   3) only these types of maps are needed: a2of, a2os, o2af, r2o with
!      all other maps being either pointers to these maps or the identity map.
!      It shouldn't be difficult to relax these assumptions if & when necessary.
!-------------------------------------------------------------------------------

   write(6,F00) 'allocate and initialize mapping data...'
   !----------------------------------------------------------------------------
   write(6,F00) '* initialize domain info'
   !----------------------------------------------------------------------------
   call map_domRead(domain_o, map_o2af_fn)   ! get domain from map data file
   call map_domRead(domain_a, map_a2of_fn)  
   call map_domDup (domain_o, domain_i)      ! assume ocn & ice have same domain
   call map_domDup (domain_a, domain_l)      ! will need to fix lnd mask
   call map_domRead(domain_r, map_r2o_fn )   ! map for roff -> ocn runoff 

!!=================wangln 2007-08-15 Read the X/Y Coordination data=================
   if ( ocn_type(1:3) == 'MOM' )  call Read_XY(map_a2of_fn, map_o2af_fn)
!!=================wangln 2007-08-15 Read the X/Y Coordination data=================

   !----------------------------------------------------------------------------
   write(6,F00) '* initialize mapping data'
   !----------------------------------------------------------------------------
   if( ocn_type(1:3) == 'POP' ) then
       call map_mapRead(map_a2of, map_a2of_fn)   ! map for atm -> ocn flux  fields
       call map_mapRead(map_a2os, map_a2os_fn)   ! map for atm -> ocn state fields
       call map_mapRead(map_o2af, map_o2af_fn)   ! map for ocn -> atm flux  fields
   else if ( ocn_type(1:3) == 'MOM' ) then
       call map_Read(map_a2of, map_a2of_fn, 9 )
       call map_Read(map_o2af, map_o2af_fn, 3 )
       call map_Read(map_a2lf, map_a2lf_fn, 9 )
       call map_Read(map_a2ls, map_a2lf_fn, 9 )
       call map_Read(map_a2os, map_a2of_fn, 9 )
   endif
!==================================================================================
 ! 9 means reading 9 parameter
 ! 3 means reading 3 parameter
!==================================================================================

   map_o2as => map_o2af                      ! map for ocn -> atm state fields
   map_a2if => map_a2of          ! assume ocn & ice are on same grid, thus can
   map_a2is => map_a2os          ! use pointers instead of duplicating data
   map_i2af => map_o2af 
   map_i2as => map_o2af

!------------wangln 2007-05-27 Add the Reading the Atm -> Lnd Flux Fields
   if( ocn_type(1:3) == 'POP' ) then
      call map_mapMakeID(map_a2lf)  ! assumes atm & lnd are on same grid
      call map_mapMakeID(map_a2ls)
   endif
!------------wangln 2007-05-27 Add the Reading the Atm -> Lnd Flux Fields


   call map_mapMakeID(map_a2lf)  ! assumes atm & lnd are on same grid
   call map_mapMakeID(map_a2ls)
   call map_mapMakeID(map_l2af)
   call map_mapMakeID(map_l2as)

   call map_mapMakeID(map_o2if)  ! assumes ocn & ice are on same grid
   call map_mapMakeID(map_o2is)
   call map_mapMakeID(map_i2of)
   call map_mapMakeID(map_i2os)

   !--- map for roff -> ocn runoff (note: don't have compression key yet) ---
   call map_mapRead(map_r2o,map_r2o_fn)   ! original, uncompressed map data
   call map_mapDup (map_r2o,map_rc2o)     ! a copy that later may be compressed

   !----------------------------------------------------------------------------
   write(6,F00) '* compute required lnd mask based on ocn mask'
   !----------------------------------------------------------------------------
   allocate(f_o(domain_o%n,1)) ! used to cast 1d fo_o array into 2d array
   allocate(f_a(domain_a%n,1)) ! used to cast 1d fo_a array into 2d array
   f_o = 1.0                   ! constant field of 1 on ocn grid ...
!--------xjx added 
   f_a = 0.
!--------xjx added 
   call map(map_o2af,f_o,f_a)  ! ...mapped to atm, gives frac of ocn on atm grid

#ifdef MPI_P
#if 1
     allocate(redu1(n_a/CPL_SIZE))
     allocate(redu2(n_a))
     redu1=0.
     redu2=0.
     iv = 1
     do j=ist_a,iend_a
       redu1(iv) = f_a(j,1)
       iv = iv+1
     enddo
     call MPI_allgather(redu1,iv-1,MPI_REAL8,&
           &  redu2,iv-1,MPI_REAL8,MPI_COMM_CPL,ierr)
#else
     allocate(redu1(n_a))
     allocate(redu2(n_a))
     redu1=0.
     redu2=0.
     do j=ist_a,iend_a
       redu1(j) = f_a(j,1)
     enddo
     call MPI_ALLREDUCE(redu1,redu2,n_a,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
#endif
     f_a(:,1) = redu2(:)
     deallocate(redu1)
     deallocate(redu2)
#endif

   !!------------------------ wangln 2008-03-19------------------
   if( ocn_type(1:3) == 'POP' ) then
       where ( f_a(:,1) < 0.999 )  ! not all ocn => some lnd (note HARDCODED thresh)
          domain_l%mask = 1         ! note: ASSUMES atm & lnd have same grid 
       elsewhere                  
          domain_l%mask = 0
       end where
   else if( ocn_type(1:3) == 'MOM' ) then
       where ( f_a(:,1) < 0.9999 )  ! not all ocn => some lnd (note HARDCODED thresh)
          domain_l%mask = 1         ! note: ASSUMES atm & lnd have same grid 
       elsewhere                  
          domain_l%mask = 0
       end where
   endif 
   !!------------------------ wangln 2008-03-19------------------

   deallocate(f_o)
   deallocate(f_a)

   !----------------------------------------------------------------------------
   write(6,F00) '* allocate memory for interface fractions'
   !----------------------------------------------------------------------------
   allocate(fa_a(domain_a%n),fl_a(domain_a%n),fi_a(domain_a%n),fo_a(domain_a%n))
   allocate(fa_i(domain_i%n),fl_i(domain_i%n),fi_i(domain_i%n),fo_i(domain_i%n))
   allocate(fa_l(domain_l%n),fl_l(domain_l%n),fi_l(domain_l%n),fo_l(domain_l%n))
   allocate(fa_o(domain_o%n),fl_o(domain_o%n),fi_o(domain_o%n),fo_o(domain_o%n))
   print *,"map_init...1-size=",size(fa_a),size(fl_a),size(fi_a),size(fo_a)
   print *,"map_init...2-size=",size(fa_i),size(fl_i),size(fi_i),size(fo_i)
   print *,"map_init...3-size=",size(fa_l),size(fl_l),size(fi_l),size(fo_l)
   print *,"map_init...4-size=",size(fa_o),size(fl_o),size(fi_o),size(fo_o)
   print *
!--------xjx added ---20081009
   fa_a = 0.
   fl_a = 0.
   fi_a = 0.
   fo_a = 0.
   fa_i = 0.
   fl_i = 0.
   fi_i = 0.
   fo_i = 0.
   fa_l = 0.
   fl_l = 0.
   fi_l = 0.
   fo_l = 0.
   fa_o = 0.
   fl_o = 0.
   fi_o = 0.
   fo_o = 0.
!--------xjx added ---20081009

END SUBROUTINE map_init

!===============================================================================

!------------wangln 2008-03-18-------------------------------------
SUBROUTINE Read_XY( filename1, filename2)
  
   implicit none
#include <netcdf.inc>

   character(len=100), intent(in) :: filename1 
   character(len=100), intent(in) :: filename2 
   
   integer               :: rcode   ! netCDF routine return code
   integer               :: fid     ! netCDF file      ID
   integer               :: vid     ! netCDF variable  ID
   integer               :: did     ! netCDF dimension ID

   rcode = nf_open( trim( filename1), NF_NOWRITE, fid)
   rcode = nf_inq_varid( fid, 'ni_a', vid)
   rcode = nf_get_var_double( fid, vid,  ni_a) 
   rcode = nf_inq_varid( fid, 'nj_a', vid)
   rcode = nf_get_var_double( fid, vid,  nj_a) 
   rcode = nf_close( fid )
   ni_l = ni_a;   nj_l = nj_a
   
   rcode = nf_open( trim( filename2), NF_NOWRITE, fid)
   rcode = nf_inq_varid( fid, 'ni_a', vid)
   rcode = nf_get_var_double( fid, vid,  ni_o) 
   rcode = nf_inq_varid( fid, 'nj_a', vid)
   rcode = nf_get_var_double( fid, vid,  nj_o) 
   rcode = nf_close( fid )
   ni_i = ni_o;   nj_i = nj_o
   
end SUBROUTINE Read_XY
!------------wangln 2008-03-18-------------------------------------


SUBROUTINE map_domRead(dom, filename)

   !--- modules ---

   implicit none

   !--- includes ---
#include <netcdf.inc>

   !--- arguments ---
   type(domain), intent(out) :: dom       ! domain info to be read in
   character(*), intent(in)  :: filename  ! name of mapping data file

   !--- local ---
   integer         :: i,m       ! generic indicies
   integer         :: n,ni,nj   ! domain dimensions ~ n = ni*nj 
   integer         :: nv        ! number of cell verticies

   real   ,pointer :: xc  (:)   ! x-coords, center
   real   ,pointer :: yc  (:)   ! y-coords, center
   real   ,pointer :: xv  (:,:) ! x-coords, verticies
   real   ,pointer :: yv  (:,:) ! y-coords, verticies
   integer,pointer :: mask(:)   ! domain mask
   real   ,pointer :: area(:)   ! area of grid cell

   character,allocatable :: str(:)  ! variable length char string
   character(CLONG)      :: attstr  ! netCDF attribute name string
   integer               :: rcode   ! netCDF routine return code
   integer               :: fid     ! netCDF file      ID
   integer               :: vid     ! netCDF variable  ID
   integer               :: did     ! netCDF dimension ID

   !--- formats ---
   character(len=*),parameter :: F00 = "('(map_domRead) ',3a)"
   character(len=*),parameter :: F02 = "('(map_domRead) ',a11,a3,60(a1))"

!-------------------------------------------------------------------------------
! PURPOSE:
! o reads domain information from remapping data file (netCDF file)
! 
! NOTE:
! o domain info is initialized, *including* setting key(n)=1 for all n,
!   ie. key indicates no compression scheme.  Adding a non-null compression key
!       and updating domain%,nc,iunc,icom must be done elsewhere.
!-------------------------------------------------------------------------------

   !--- document global attributes -----------------
   write(6,F00) 'data file',' = ',trim(filename)
   rcode = nf_open(filename,NF_NOWRITE,fid)
   if ( ocn_type(1:3) == 'POP' ) then
     do i=1,5
       if (i == 1) attstr = 'title'      
       if (i == 2) attstr = 'map_method'
       if (i == 3) attstr = 'domain_a'
       if (i == 4) attstr = 'conventions'
       if (i == 5) attstr = 'history'
       m = len_trim(attstr)
       rcode = nf_inq_attlen  (fid, NF_GLOBAL, attstr(1:m), n)
       allocate(str(n))
       rcode = nf_get_att_text(fid, NF_GLOBAL, attstr(1:m), str)
       write(6,F02) trim(attstr),' = ',str(1:n)
       deallocate(str)
     end do
   endif

   !--- get domain info (from "source" grid, *_a) ---
   rcode = nf_inq_dimid (fid, 'n_a' , did)
   rcode = nf_inq_dimlen(fid, did   , n  )
   rcode = nf_inq_dimid (fid, 'ni_a', did)
   rcode = nf_inq_dimlen(fid, did   , ni )
   rcode = nf_inq_dimid (fid, 'nj_a', did)
   rcode = nf_inq_dimlen(fid, did   , nj )
   rcode = nf_inq_dimid (fid, 'nv_a', did)
   rcode = nf_inq_dimlen(fid, did   , nv )

   dom%n  = n   ! total number of grid points
   dom%ni = ni  ! for 2D array, i-dimension
   dom%nj = nj  ! for 2D array, j-dimension
   dom%nv = nv  ! number of cell verticies
   dom%nc = n   ! num of points in compressed vector (n=nc <=> null compression)

   allocate(dom%xc  (   n)) ! x-coordinates of center
   allocate(dom%yc  (   n)) ! y-coordinates of center
   allocate(dom%xv  (nv,n)) ! x-coordinates of verticies
   allocate(dom%yv  (nv,n)) ! y-coordinates of verticies
   allocate(dom%mask(   n)) ! domain mask
   allocate(dom%area(   n)) ! grid cell area
   allocate(dom%key (   n)) ! compression key 
   allocate(dom%icom(   n)) ! index into compressed   from uncompressed vector
   allocate(dom%iunc(   n)) ! index into uncompressed from compressed   vector

   xc   => dom%xc
   yc   => dom%yc
   xv   => dom%xv
   yv   => dom%yv
   mask => dom%mask
   area => dom%area

   rcode = nf_inq_varid     (fid,'xc_a'  ,vid)
   rcode = nf_get_var_double(fid,vid     ,xc )
   rcode = nf_inq_varid     (fid,'yc_a'  ,vid)
   rcode = nf_get_var_double(fid,vid     ,yc )
   rcode = nf_inq_varid     (fid,'xv_a'  ,vid)
   rcode = nf_get_var_double(fid,vid     ,xv )
   rcode = nf_inq_varid     (fid,'yv_a'  ,vid)
   rcode = nf_get_var_double(fid,vid     ,yv )
   rcode = nf_inq_varid     (fid,'mask_a',vid )
   rcode = nf_get_var_int   (fid,vid     ,mask)
   rcode = nf_inq_varid     (fid,'area_a',vid )
   rcode = nf_get_var_double(fid,vid     ,area)

   rcode = nf_close(fid)

   !--- key and re-indexing indicate null/nothing-done compression scheme ---
   dom%key(:) = 1
   do i=1,n
      dom%iunc(i) = i
      dom%icom(i) = i
   enddo

END SUBROUTINE map_domRead

!===============================================================================

SUBROUTINE map_domDup(dom_in, dom_out)

   !--- modules ---

   implicit none

   !--- includes ---

   !--- arguments ---
   type(domain), intent(in ) :: dom_in  ! source
   type(domain), intent(out) :: dom_out ! destination

   !--- local ---
   integer         :: n         ! domain size ~ n = ni*nj 
   integer         :: nv        ! number of cell verticies
   integer         :: nc        ! number of compress grid points

   !--- formats ---
   character(len=*),parameter :: F01 = "('(map_domDup) ',4a)"

!-------------------------------------------------------------------------------
! PURPOSE:
! o duplicate (copy) domain info
!-------------------------------------------------------------------------------

   n  = dom_in%n                  ! total number of grid points
   nv = dom_in%nv                 ! number of cell verticies
   nc = dom_in%nc                 ! number of compress grid points

   allocate(dom_out%xc  (   n))   ! x-coordinates of center
   allocate(dom_out%yc  (   n))   ! y-coordinates of center
   allocate(dom_out%xv  (nv,n))   ! x-coordinates of verticies
   allocate(dom_out%yv  (nv,n))   ! y-coordinates of verticies
   allocate(dom_out%mask(   n))   ! domain mask
   allocate(dom_out%area(   n))   ! grid cell area
   allocate(dom_out%key (  nc))   ! grid key
   allocate(dom_out%iunc(  nc))   ! grid uncompress index
   allocate(dom_out%icom(  nc))   ! grid compress index

   dom_out%n    = dom_in%n 
   dom_out%ni   = dom_in%ni
   dom_out%nj   = dom_in%nj
   dom_out%nv   = dom_in%nv
   dom_out%xc(:)   = dom_in%xc(:)
   dom_out%yc(:)   = dom_in%yc(:)
   dom_out%xv(:,:) = dom_in%xv(:,:)
   dom_out%yv(:,:) = dom_in%yv(:,:)
   dom_out%mask(:) = dom_in%mask(:)
   dom_out%area(:) = dom_in%area(:)
   dom_out%key(:)  = dom_in%key(:)
   dom_out%nc   = dom_in%nc
   dom_out%iunc(:) = dom_in%iunc(:)
   dom_out%icom(:) = dom_in%icom(:)

END SUBROUTINE map_domDup

!===============================================================================

SUBROUTINE map_domChgKey(dom, key)

   !--- modules ---

   implicit none

   !--- includes ---

   !--- arguments ---
   type(domain), intent(inout) :: dom     ! domain whose key gets modified
   integer     , intent(in)    :: key(:)  ! key to apply

   !--- local ---
   integer         :: i,k       ! generic indicies

   !--- formats ---
   character(len=*),parameter :: F00 = "('(map_domChgKey) ',4a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!    give domain a new compression key and update index maps accordingly
!-------------------------------------------------------------------------------

   !----------------------------------------------------------------------------
   write(6,F00) '* change compression key and re-compute indexing'
   !----------------------------------------------------------------------------

   if (size(key) /= dom%n) then
     write(6,*) 'ERROR: wrong key size: dom%n,size(key)',dom%n,size(key)
     stop 'key size'
   end if

   dom%key = key  ! new key

   !--- count size of associated compressed vector ---
   k=0
   do i=1,dom%n
     if (dom%key(i) /= 0) k=k+1
   enddo
   dom%nc = k    ! corresponding size of compressed data

   !--- allocate right amount of space for comp->uncompr index map ---
   deallocate(dom%iunc)  
   allocate(dom%iunc(dom%nc))
!xjx added 20080510
    dom%iunc(1:dom%nc) = 0.0
!xjx end

   !--- compute indicies: compressed to/from uncompressed vectors ---
   k=0
   do i=1,dom%n
     if (dom%key(i) /= 0) then
       k=k+1
       dom%icom(i) = k  ! k (compressed)  <=   i (uncompressed)
       dom%iunc(k) = i  ! k (compressed)   =>  i (uncompressed)
     else
       dom%icom(i) = 0  ! no corresponding index in compressed vector
     end if
   enddo

END SUBROUTINE map_domChgKey

!===============================================================================

SUBROUTINE map_domPad(dom, mask)

   !--- modules ---

   implicit none

   !--- includes ---

   !--- arguments ---
   type(domain), intent(inout) :: dom     ! domain whose key gets modified
   integer     , intent(in)    :: mask(:) ! key to apply

   !--- local ---
   type(domain)    :: dom_old   ! old un-padded domain
   integer         :: i         ! generic indicies
   integer         :: cnt       ! counter
   integer         :: nsiz      ! size(mask)
   integer         :: nv        ! number of vertices in domain

   !--- formats ---
   character(len=*),parameter :: F00 = "('(map_domPad) ',4a)"
   character(len=*),parameter :: F01 = "('(map_domPad) ',a,4i8)"

!-------------------------------------------------------------------------------
! PURPOSE:
!    pad domain based on mask
!    assume no compression based on key has occurred yet
!-------------------------------------------------------------------------------

   nv = dom%nv
   nsiz = size(mask)
   if (nsiz /= dom%n) then
     !--- test that mask can pad dom, must have same number of valid elements --
     cnt=0
     do i=1,nsiz
       if (mask(i) /= 0) cnt=cnt+1
     enddo
     if (dom%n /= cnt) then
       write(6,*) 'ERROR: wrong mask size: dom%n,size(mask)',dom%n,nsiz
       stop 'map_domPad: mask size'
     endif

     !--- duplicate the domain ---
     print *,"=============  call map_domDup =============="
     call map_domDup(dom,dom_old)

     !--- reallocate dom for new padded size ---
     deallocate(dom%xc  )
     deallocate(dom%yc  )
     deallocate(dom%xv  )
     deallocate(dom%yv  )
     deallocate(dom%mask)
     deallocate(dom%area)
     deallocate(dom%key )
     deallocate(dom%icom)
     deallocate(dom%iunc)
     allocate(dom%xc  (   nsiz)) ! x-coordinates of center
     allocate(dom%yc  (   nsiz)) ! y-coordinates of center
     allocate(dom%xv  (nv,nsiz)) ! x-coordinates of verticies
     allocate(dom%yv  (nv,nsiz)) ! y-coordinates of verticies
     allocate(dom%mask(   nsiz)) ! domain mask
     allocate(dom%area(   nsiz)) ! grid cell area
     allocate(dom%key (   nsiz)) ! compression key 
     allocate(dom%icom(   nsiz)) ! index into compressed
     allocate(dom%iunc(   nsiz)) ! index into uncompressed

     !--- pad ---
     dom%n  = nsiz        ! total number of grid points
     dom%ni = nsiz        ! for 2D array, i-dimension
     dom%nj = 1           ! for 2D array, j-dimension
     dom%nv = dom_old%nv  ! number of cell verticies
     dom%nc = nsiz        ! num of points in compressed vector

     cnt=0
     do i=1,nsiz
       if (mask(i) /= 0) then
         cnt=cnt+1
         dom%xc(i)   = dom_old%xc(cnt)
         dom%yc(i)   = dom_old%yc(cnt)
         dom%xv(:,i) = dom_old%xv(:,cnt)
         dom%yv(:,i) = dom_old%yv(:,cnt)
         dom%mask(i) = dom_old%mask(cnt)
         dom%area(i) = dom_old%area(cnt)
         dom%key(i)  = 1
         dom%icom(i) = i
         dom%iunc(i) = i
       else
         dom%xc(i)   = 0.
         dom%yc(i)   = 0.
         dom%xv(:,i) = 0.
         dom%yv(:,i) = 0.
         dom%mask(i) = 0
         dom%area(i) = 0.
         dom%key(i)  = 1
         dom%icom(i) = i
         dom%iunc(i) = i
       endif
     enddo

     deallocate(dom_old%xc  )
     deallocate(dom_old%yc  )
     deallocate(dom_old%xv  )
     deallocate(dom_old%yv  )
     deallocate(dom_old%mask)
     deallocate(dom_old%area)
     deallocate(dom_old%key )
     deallocate(dom_old%icom)
     deallocate(dom_old%iunc)

     write(6,F01) 'map_domPad executed: dom%n,size(mask)',dom%n,nsiz
   else
     write(6,F01) 'map_domPad not needed: dom%n,size(mask)',dom%n,nsiz
   endif

END SUBROUTINE map_domPad
!===============================================================================

SUBROUTINE map_Read(map, filename, nums_parameter)

   !--- modules ---

   implicit none

   !--- includes ---
#include <netcdf.inc>

   !--- arguments ---
   type(sparse_matrix),intent(out) :: map        ! mapping data to be read in
   character(*)       ,intent(in)  :: filename   ! name of mapping data file
   integer            ,intent(in)  :: nums_parameter       ! nus of the Cofficient

   !--- local ---
   integer         :: i,j,k,m   ! generic indicies
   integer         :: n,ni,nj   ! domain dimensions ~ n = ni*nj 
   integer         :: nv        ! number of cell verticies
   integer         :: ns        ! num of non-zero elements in map matrix
   integer         :: na,nb     ! size of input & output vectors

   real   ,pointer :: s   (:)   ! the non-zero matrix elements
   integer,pointer :: row (:)   ! the row    of the matrix elements
   integer,pointer :: col (:)   ! the column of the matrix elements
   integer,pointer :: sn1 (:)   ! the column of the matrix elements
   integer,pointer :: mask_a(:) ! source domain mask
   integer,pointer :: mask_b(:) ! destination domain mask
   real   ,pointer :: frac(:)   ! frac of dest intersecting src
!----------------wangln 2007-05-27 Add the Bilinear parameter---------------------
   real   ,pointer :: dx  (:)    ! the X-coordinate bilinear cofficient ( = Coff. * 1/( cossin( lat) *dlon) )
   real   ,pointer :: dy  (:)    ! the Y-coordinate bilinear cofficient ( = Coff. * 1/ dlat )
   integer,pointer :: x1(:)      ! The West  X-coordinate grid No. for Side1
   integer,pointer :: x2(:)      ! The East  X-coordinate grid No. for Side1
   integer,pointer :: y1(:)      ! The South Y-coordinate grid No. for Side1
   integer,pointer :: y2(:)      ! The North Y-coordinate grid No. for Side1
!----------------wangln 2007-05-27 Add the Bilinear parameter---------------------


   character,allocatable :: str(:)  ! variable length char string
   character(CLONG)      :: attstr  ! netCDF attribute name string
   integer               :: rcode   ! netCDF routine return code
   integer               :: fid     ! netCDF file      ID
   integer               :: vid     ! netCDF variable  ID
   integer               :: did     ! netCDF dimension ID

   !--- formats ---
   character(len=*),parameter :: F00 = "('(map_Read) ',3a)"
   character(len=*),parameter :: F01 = "('(map_Read) ',a20,a3,60(a1))"

!-------------------------------------------------------------------------------
! PURPOSE:
! o allocate memory for mapping data (sparse matrix data) 
! o initialize mapping data (input data from a netCDF file)
!-------------------------------------------------------------------------------

   !--- document global attributes -----------------
   write(6,F00) 'data file',' = ',trim(filename)
   rcode = nf_open(filename,NF_NOWRITE,fid)
   if ( ocn_type(1:3) == 'POP' ) then
     do i=1,6
       if (i == 1) attstr = 'title'      
       if (i == 2) attstr = 'map_method'
       if (i == 3) attstr = 'domain_a'
       if (i == 4) attstr = 'domain_b'
       if (i == 5) attstr = 'conventions'
       if (i == 6) attstr = 'history'
       m = len_trim(attstr)
       rcode = nf_inq_attlen  (fid, NF_GLOBAL, attstr(1:m), n)
       allocate(str(n))
       rcode = nf_get_att_text(fid, NF_GLOBAL, attstr(1:m), str)
       write(6,F01) trim(attstr),' = ',str(1:n)
       deallocate(str)
     end do
   endif

   !--- allocate memory & get matrix data ----------
   rcode = nf_inq_dimid (fid, 'n_s', did)  ! size of sparse matrix
   rcode = nf_inq_dimlen(fid, did  , ns)
   rcode = nf_inq_dimid (fid, 'n_a', did)  ! size of  input vector
   rcode = nf_inq_dimlen(fid, did  , na)
   rcode = nf_inq_dimid (fid, 'n_b', did)  ! size of output vector
   rcode = nf_inq_dimlen(fid, did  , nb)

   map%n   = ns
   map%n_a = na
   map%n_b = nb
   allocate(map%s  (ns))
   allocate(map%row(ns))
   allocate(map%col(ns))
   allocate(map%sn1(nb+1))
   allocate(map%mask_b(nb))
   allocate(map%mask_a(na))
   s   => map%s
   row => map%row
   col => map%col
   sn1 => map%sn1
   mask_a => map%mask_a
   mask_b => map%mask_b
!------------------------------------wangln 2007-05-27----------------------
   if( nums_parameter == 9 ) then
      allocate(map%dx  (ns))
      allocate(map%dy  (ns))
      allocate(map%x1  (ns))
      allocate(map%x2  (ns))
      allocate(map%y1  (ns))
      allocate(map%y2  (ns))

      dx => map%dx
      dy => map%dy
      x1 => map%x1
      x2 => map%x2
      y1 => map%y1
      y2 => map%y2
   endif
!------------------------------------wangln 2007-05-27----------------------


   rcode = nf_inq_varid     (fid,'S'  ,vid)
   rcode = nf_get_var_double(fid,vid  ,s  )
   rcode = nf_inq_varid     (fid,'row',vid)
   rcode = nf_get_var_int   (fid,vid  ,row)
   rcode = nf_inq_varid     (fid,'col',vid)
   rcode = nf_get_var_int   (fid,vid  ,col)
   rcode = nf_inq_varid     (fid,'mask_a',vid)
   rcode = nf_get_var_int   (fid,vid  ,mask_a)
   rcode = nf_inq_varid     (fid,'mask_b',vid)
   rcode = nf_get_var_int   (fid,vid  ,mask_b)

!----------------------wangln 2008-03-18--------------------------------------
   if( nums_parameter == 9 ) then
   rcode = nf_inq_varid     (fid,'Dx'  ,vid)
   rcode = nf_get_var_double(fid,vid  ,dx  )
   rcode = nf_inq_varid     (fid,'Dy'  ,vid)
   rcode = nf_get_var_double(fid,vid  ,dy  )
   rcode = nf_inq_varid     (fid,'X1',vid)
   rcode = nf_get_var_int   (fid,vid  ,x1)
   rcode = nf_inq_varid     (fid,'X2',vid)
   rcode = nf_get_var_int   (fid,vid  ,x2)
   rcode = nf_inq_varid     (fid,'Y1',vid)
   rcode = nf_get_var_int   (fid,vid  ,Y1)
   rcode = nf_inq_varid     (fid,'Y2',vid)
   rcode = nf_get_var_int   (fid,vid  ,Y2)
   endif
!----------------------wangln 2008-03-18--------------------------------------

   rcode = nf_close(fid)

   !--- check row ordering ---
   do i=1,ns-1
     if (row(i+1).lt.row(i)) then
       write(6,F00) 'ERROR: row unsorted ',trim(filename)
       STOP
     endif
   enddo

   !--- compute sn1 for this map, sn1(i) = least n st row(n)=i ---
   call map_mapSn1(map)

END SUBROUTINE map_Read

!===============================================================================

SUBROUTINE map_mapRead(map, filename)

   !--- modules ---

   implicit none

   !--- includes ---
#include <netcdf.inc>

   !--- arguments ---
   type(sparse_matrix),intent(out) :: map        ! mapping data to be read in
   character(*)       ,intent(in)  :: filename   ! name of mapping data file

   !--- local ---
   integer         :: i,j,k,m   ! generic indicies
   integer         :: n,ni,nj   ! domain dimensions ~ n = ni*nj 
   integer         :: nv        ! number of cell verticies
   integer         :: ns        ! num of non-zero elements in map matrix
   integer         :: na,nb     ! size of input & output vectors

   real   ,pointer :: s   (:)   ! the non-zero matrix elements
   integer,pointer :: row (:)   ! the row    of the matrix elements
   integer,pointer :: col (:)   ! the column of the matrix elements
   integer,pointer :: sn1 (:)   ! the column of the matrix elements
   integer,pointer :: mask_a(:) ! source domain mask
   integer,pointer :: mask_b(:) ! destination domain mask
   real   ,pointer :: frac(:)   ! frac of dest intersecting src

   character,allocatable :: str(:)  ! variable length char string
   character(CLONG)      :: attstr  ! netCDF attribute name string
   integer               :: rcode   ! netCDF routine return code
   integer               :: fid     ! netCDF file      ID
   integer               :: vid     ! netCDF variable  ID
   integer               :: did     ! netCDF dimension ID

   !--- formats ---
   character(len=*),parameter :: F00 = "('(map_mapRead) ',3a)"
   character(len=*),parameter :: F01 = "('(map_mapRead) ',a20,a3,60(a1))"

!-------------------------------------------------------------------------------
! PURPOSE:
! o allocate memory for mapping data (sparse matrix data) 
! o initialize mapping data (input data from a netCDF file)
!-------------------------------------------------------------------------------

   !--- document global attributes -----------------
   write(6,F00) 'data file',' = ',trim(filename)
   rcode = nf_open(filename,NF_NOWRITE,fid)
   do i=1,6
     if (i == 1) attstr = 'title'      
     if (i == 2) attstr = 'map_method'
     if (i == 3) attstr = 'domain_a'
     if (i == 4) attstr = 'domain_b'
     if (i == 5) attstr = 'conventions'
     if (i == 6) attstr = 'history'
     m = len_trim(attstr)
     rcode = nf_inq_attlen  (fid, NF_GLOBAL, attstr(1:m), n)
     allocate(str(n))
     rcode = nf_get_att_text(fid, NF_GLOBAL, attstr(1:m), str)
     write(6,F01) trim(attstr),' = ',str(1:n)
     deallocate(str)
   end do

   !--- allocate memory & get matrix data ----------
   rcode = nf_inq_dimid (fid, 'n_s', did)  ! size of sparse matrix
   rcode = nf_inq_dimlen(fid, did  , ns)
   rcode = nf_inq_dimid (fid, 'n_a', did)  ! size of  input vector
   rcode = nf_inq_dimlen(fid, did  , na)
   rcode = nf_inq_dimid (fid, 'n_b', did)  ! size of output vector
   rcode = nf_inq_dimlen(fid, did  , nb)

   write(6,*) 'map_Readmap ns=', ns
   map%n   = ns
   map%n_a = na
   map%n_b = nb
   allocate(map%s  (ns))
   allocate(map%row(ns))
   allocate(map%col(ns))
   allocate(map%sn1(nb+1))
   allocate(map%mask_b(nb))
   allocate(map%mask_a(na))
   s   => map%s
   row => map%row
   col => map%col
   sn1 => map%sn1
   mask_a => map%mask_a
   mask_b => map%mask_b

   rcode = nf_inq_varid     (fid,'S'  ,vid)
   rcode = nf_get_var_double(fid,vid  ,s  )
   rcode = nf_inq_varid     (fid,'row',vid)
   rcode = nf_get_var_int   (fid,vid  ,row)
   rcode = nf_inq_varid     (fid,'col',vid)
   rcode = nf_get_var_int   (fid,vid  ,col)
   rcode = nf_inq_varid     (fid,'mask_a',vid)
   rcode = nf_get_var_int   (fid,vid  ,mask_a)
   rcode = nf_inq_varid     (fid,'mask_b',vid)
   rcode = nf_get_var_int   (fid,vid  ,mask_b)

   rcode = nf_close(fid)

   !--- check row ordering ---
   do i=1,ns-1
     if (row(i+1).lt.row(i)) then
       write(6,F00) 'ERROR: row unsorted ',trim(filename)
       STOP
     endif
   enddo

   !--- compute sn1 for this map, sn1(i) = least n st row(n)=i ---
   call map_mapSn1(map)

END SUBROUTINE map_mapRead

!===============================================================================

SUBROUTINE map_mapMakeID(map)

   implicit none

   !--- arguments ---
   type(sparse_matrix),intent(out) :: map

!-------------------------------------------------------------------------------
! PURPOSE:
! o initialize mapping data as an identity map
!-------------------------------------------------------------------------------

   map%n = 0

END SUBROUTINE map_mapMakeID

!===============================================================================

SUBROUTINE map_mapDup(map_in,map_out)

   implicit none

   !--- arguments ---
   type(sparse_matrix),intent( in)   :: map_in
   type(sparse_matrix),intent(inout) :: map_out

   !--- local ---
   integer :: rcode ! return code

!-------------------------------------------------------------------------------
! PURPOSE:
! o create a duplicate copy a sparse_matrix (cannot deallocate existing data)
!
! NOTES
! o the return code allows the deallocate to fail without stopping program 
!   exectution.  This would happen, eg, if the pointer has never been 
!   allocated to begin with (dealloc doesn't like undefied pointers). 
!   F90 has no way (?) of testing an undefined pointer (eg. never allocated).
!   See Fortran 90/95 Explained, Melcaf, 1998, p 124, 175
!-------------------------------------------------------------------------------

   deallocate(map_out%s     ,STAT=rcode) 
   deallocate(map_out%row   ,STAT=rcode) 
   deallocate(map_out%col   ,STAT=rcode) 
   deallocate(map_out%sn1   ,STAT=rcode) 
   deallocate(map_out%mask_a,STAT=rcode) 
   deallocate(map_out%mask_b,STAT=rcode) 

   allocate(map_out%s     (map_in%n  ) )  
   allocate(map_out%row   (map_in%n  ) )  
   allocate(map_out%col   (map_in%n  ) )  
   allocate(map_out%sn1   (map_in%n_b+1) )  
   allocate(map_out%mask_a(map_in%n_a) )
   allocate(map_out%mask_b(map_in%n_b) )  

   map_out%n      = map_in%n
   map_out%n_a    = map_in%n_a
   map_out%n_b    = map_in%n_b
   map_out%s      = map_in%s
   map_out%row    = map_in%row
   map_out%col    = map_in%col
   map_out%sn1    = map_in%sn1
   map_out%mask_a = map_in%mask_a
   map_out%mask_b = map_in%mask_b

END SUBROUTINE map_mapDup

!===============================================================================

SUBROUTINE map_mapCompress(map_in, map_out, key, flag )

   !--- modules ---

   implicit none

   !--- arguments ---
   type(sparse_matrix),intent(in)    :: map_in  ! mapping data to be compressed
   type(sparse_matrix),intent(inout) :: map_out ! mapping data to be compressed
   integer            ,intent(in)    :: key(:)  ! compression key
   character(len=*)   ,intent(in)    :: flag    ! compress wrt "src" or "dest" ?

   !--- local ---
   integer         :: i,j,k,n   ! generic indicies
   integer         :: n2        ! counts new (reduced) size of compressed matrix

   integer,allocatable::jnew(:) ! maps uncompressed j-index to compressed j
   integer,allocatable::inew(:) ! maps uncompressed i-index to compressed j

   !--- formats ---
   character(len=*),parameter :: F01 = "('(map_mapCompress) ',3a)"

!-------------------------------------------------------------------------------
! PURPOSE:
! o set compressed map matrix according to original map plus compression key
!
! NOTE:
! o compute new matrix size, adjust s, row, & col values, and recompute sn1
! o key could apply to either the input vector or the output vector
!-------------------------------------------------------------------------------

   if ( map_in%n    > size(map_out%s) .or. &
   &    map_in%n_a /= map_out%n_a     .or. &
   &    map_in%n_b /= map_out%n_b     ) then
      write(6,*) 'ERROR: incompatible original & compressed map data'
      stop 'domain'
   end if

   if ( flag(1:3) == "src" ) then
     !--------------------------------------------------------------------------
     ! o remove columns of matrix where key(j)=0 
     ! o re-index matrix col values so it works with a compressed input vector 
     !--------------------------------------------------------------------------

     if (size(key) /= map_in%n_a) then
       write(6,*)'ERROR: wrong key size: map%n_a,size(key)',map_in%n_a,size(key)
       stop 'key size'
     end if

     !--- compute re-indexing ---
     allocate(jnew(map_in%n_a))
     k=0
     do j=1,map_in%n_a
       if (key(j) /= 0) then
         k=k+1
         jnew(j) = k ! j in uncompressed vect corresponds to jnew in compr vect
       end if
     enddo

     !--- remove matrix columns and re-index columns ---
     n2 = 0
     do n=1,map_in%n
        j = map_in%col(n) 
        if (key(j) /= 0 ) then
          n2 = n2+1
          map_out%s  (n2) = map_in%s  (n)
          map_out%row(n2) = map_in%row(n)
          map_out%col(n2) = jnew(map_in%col(n))
        end if
     end do
     map_out%s  (n2+1:map_out%n) = 0. ! invalid data
     map_out%row(n2+1:map_out%n) = 0  ! invalid data
     map_out%col(n2+1:map_out%n) = 0  ! invalid data
     map_out%n = n2                   ! new, smaller size of matrix
     call map_mapSn1(map_out)         ! recompute sn1
     deallocate(jnew)

   else if ( flag(1:4) == "dest" ) then
     !--------------------------------------------------------------------------
     ! o remove rows of matrix where key(i)=0 
     ! o re-index matrix row values so it works with a compressed input vector 
     !--------------------------------------------------------------------------

     if (size(key) /= map_in%n_b) then
       write(6,*)'ERROR: wrong key size: map%n_a,size(key)',map_in%n_a,size(key)
       stop 'key size'
     end if

     n2 = 0
     do n=1,map_in%n
       i = map_in%row(n) 
       if (key(i) /= 0 ) then
         n2 = n2+1
         map_out%s  (n2) = map_in%s  (n)
         map_out%row(n2) = map_in%row(n)
         map_out%col(n2) = map_in%col(n)
       end if
     end do
     map_out%s  (n2+1:map_out%n) = 0. ! invalid data
     map_out%row(n2+1:map_out%n) = 0  ! invalid data
     map_out%col(n2+1:map_out%n) = 0  ! invalid data
     map_out%n = n2                   ! new, smaller size of matrix
     call map_mapSn1(map_out)         ! recompute sn1

   else 
     write(6,*) "ERROR: map_mapCompress: unrecognized flag = ",trim(flag)
   end if

END SUBROUTINE map_mapCompress

!===============================================================================

SUBROUTINE map_mapPad(map, mask, flag)

   !--- modules ---

   implicit none

   !--- arguments ---
   type(sparse_matrix),intent(inout) :: map     ! mapping data to be padded
   integer            ,intent(in)    :: mask(:) ! mask
   character(len=*)   ,intent(in)    :: flag    ! compress wrt "src" or "dest" ?

   !--- local ---
   integer              :: i,j,k,n     ! generic indicies
   integer              :: cnt         ! counter
   integer              :: nsiz        ! size(mask)
   integer              :: nmat        ! number of non-zero matrix elements
   integer,allocatable  :: newindex(:) ! new indices

   !--- formats ---
   character(len=*),parameter :: F00 = "('(map_mapPad) ',4a)"
   character(len=*),parameter :: F01 = "('(map_mapPad) ',a,2i8)"

!-------------------------------------------------------------------------------
! PURPOSE:
!    reindex weights file based on mask
!-------------------------------------------------------------------------------

   nsiz = size(mask)
   nmat = map%n
   cnt=0
   do i=1,nsiz
     if (mask(i) /= 0) cnt=cnt+1
   enddo
   if (flag(1:3) == 'src') then
     if (map%n_a /= nsiz) then
       if (map%n_a /= cnt) then
         write(6,*) 'ERROR: wrong mask size: map%n_a,size(mask)',map%n_a,nsiz
         stop 'map_mapPad: mask size'
       endif
       allocate(newindex(map%n_a))
!xjx added 20080510
     newindex = 0
!xjx end

       cnt=0
       do i=1,nsiz
         if (mask(i) /= 0) then
           cnt=cnt+1
           newindex(cnt) = i
         endif
       enddo
       map%n_a = nsiz
       deallocate(map%mask_a)
       allocate(map%mask_a(map%n_a))
       map%mask_a(:) = mask(:)
       do i=1,map%n
         map%col(i) = newindex(map%col(i))
       enddo
       write(6,F01) 'map_mapPad executed: map%n_a,size(mask)',map%n_a,nsiz
     else
       write(6,F01) 'map_mapPad not needed: map%n_a,size(mask)',map%n_a,nsiz
     endif
   elseif (flag(1:4) == 'dest') then
     if (map%n_b /= nsiz) then
       if (map%n_b /= cnt) then
         write(6,*) 'ERROR: wrong mask size: map%n_b,size(mask)',map%n_b,nsiz
         stop 'map_mapPad: mask size'
       endif
       allocate(newindex(map%n_b))
!xjx added 20080510
     newindex = 0
!xjx end
       cnt=0
       do i=1,nsiz
         if (mask(i) /= 0) then
           cnt=cnt+1
           newindex(cnt) = i
         endif
       enddo
       map%n_b = nsiz
       deallocate(map%mask_b)
       allocate(map%mask_b(map%n_b))
       map%mask_b(:) = mask(:)
       do i=1,map%n
         map%row(i) = newindex(map%row(i))
       enddo
       write(6,F01) 'map_mapPad executed: map%n_b,size(mask)',map%n_b,nsiz
     else
       write(6,F01) 'map_mapPad not needed: map%n_b,size(mask)',map%n_b,nsiz
     endif
   else
     write(6,F00) "ERROR: map_mapPad: unrecognized flag = ",trim(flag)
   endif

END SUBROUTINE map_mapPad

!===============================================================================

SUBROUTINE map_mapSn1(map)

   implicit none

   !--- arguments ---
   type(sparse_matrix),intent(inout) :: map

   !--- local ---
   integer :: i,n

!-------------------------------------------------------------------------------
! PURPOSE:
! o compute sn1(i) = least n st row(n)=i 
!-------------------------------------------------------------------------------

   i=0 ! last row for which sn1(i) was calculated
   do n=1,map%n
     if (map%row(n) /= i) then
       map%sn1(i+1:map%row(n)) = n
       i=map%row(n)
     endif
   enddo
   map%sn1(i+1:map%n_b+1)=map%n+1

END SUBROUTINE map_mapSn1

!===============================================================================

SUBROUTINE map_frac(ifrac_i)

   implicit none

#ifdef MPI_P
   include "mpif.h"
#endif

   !--- arguments ---
   real            :: ifrac_i(:) ! ice fract on ice grid (an ice output state)

   !--- local ---
   real,allocatable :: t_o(:,:) ! used to cast 1d ocn grid array as 2d
   real,allocatable :: t_a(:,:) ! used to cast 1d atm grid array as 2d

   integer         :: n          ! generic loop index
   logical         :: first_call = .true.
   real ,parameter :: eps = 1.0e-9 ! epsilon
   real ,parameter :: eps_mom = 1.0e-10 ! epsilon

   !--- formats ---
   character(len=*),parameter :: F01 = "('(map_frac) ',a,50(f6.2 ))"
   character(len=*),parameter :: F02 = "('(map_frac) ',a,50(i3,3x))"

#ifdef MPI_P
   real (SHR_KIND_R8), allocatable :: redu1(:),redu2(:)
   integer j,iv
#endif
   SAVE

!-------------------------------------------------------------------------------
! PURPOSE:
!   based on frac_i, compute all other interface fractions
!-------------------------------------------------------------------------------

   IF ( first_call) THEN
     !--------------------------------------------------------
     write(6,F01) 'initializing interface fractions'
     !--------------------------------------------------------

     !--- fractions on ice grid ---
     where (domain_i%mask /= 0) 
       fa_i=1.0        ! always true (atm is global)
       fl_i=0.0        ! always true (no ice/lnd overlap)
       fo_i=1.0        ! always true (assumes ice & ocn have same domain)
       fi_i=ifrac_i    ! an ice output state
     elsewhere
       fa_i = 1.0
       fl_i = 1.0
       fi_i = 0.0
       fo_i = 0.0
     end where

     !--- fractions on ocn grid ---
     where (domain_o%mask /= 0) 
       fo_o=1.0        ! always true (assumes no fractional ocn cells)
       fl_o=0.0        ! always true (no ocn/lnd overlap)
       fi_o=fi_i       ! always true (assumes ice & ocn have same domain)
       fa_o=1.0-fi_o   ! not ice => atm
     elsewhere
       fa_o = 1.0
       fl_o = 1.0
       fi_o = 0.0
       fo_o = 0.0
     end where

     !--- fractions on atm grid ---
     fa_a = 1.0                    ! always true (atm is global)
     fi_a = 0.0                    ! initialize before use
     fl_a = 0.0                    ! initialize before use
     allocate(t_o(size(fi_o),1))   ! used to cast 1d array into 2d array
     allocate(t_a(size(fi_a),1))   ! used to cast 1d array into 2d array
     print *,"size(t_o)=",size(t_o),size(fi_o)
     print *,"size(t_a)=",size(t_a),size(fi_a)

#ifdef MPI_P

     t_o(:,1) = fi_o ; call map(map_i2af,t_o,t_a) ; 
#if 1
     allocate(redu1(n_a/CPL_SIZE))
     allocate(redu2(n_a))
     redu1=0.
     redu2=0.
     iv = 1
     do j=ist_a,iend_a
       redu1(iv) = t_a(j,1)
       iv = iv+1
     enddo
     call MPI_allgather(redu1,iv-1,MPI_REAL8,&
           &  redu2,iv-1,MPI_REAL8,MPI_COMM_CPL,ierr)
#else
     allocate(redu1(n_a))
     allocate(redu2(n_a))
     redu1=0.
     redu2=0.
     do j=ist_a,iend_a
       redu1(j) = t_a(j,1)
     enddo
     call MPI_ALLREDUCE(redu1,redu2,n_a,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
#endif
     t_a(:,1) = redu2(:)
     fi_a = t_a(:,1)
!-----
!-----
     t_o(:,1) = fo_o ; call map(map_o2af,t_o,t_a) ; 

     redu1=0.
     redu2=0.
#if 1
     iv = 1
     do j=ist_a,iend_a
       redu1(iv) = t_a(j,1)
       iv = iv+1
     enddo
     call MPI_allgather(redu1,iv-1,MPI_REAL8,&
           &  redu2,iv-1,MPI_REAL8,MPI_COMM_CPL,ierr)
#else
     do j=ist_a,iend_a
       redu1(j) = t_a(j,1)
     enddo
     call MPI_ALLREDUCE(redu1,redu2,n_a,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
#endif
     t_a(:,1) = redu2(:)
     fo_a = t_a(:,1)
     deallocate(redu1)
     deallocate(redu2)
#else
     t_o(:,1) = fi_o ; call map(map_i2af,t_o,t_a) ; fi_a = t_a(:,1)
     t_o(:,1) = fo_o ; call map(map_o2af,t_o,t_a) ; fo_a = t_a(:,1)
#endif
     do n=1,size(fl_a)
        fl_a(n) = 1.0 - fi_a(n) - fo_a(n)    ! not (ice or ocn) => lnd

!!-----------------wangln 2008-08-12 according with the former in MOM type--------------
        if( ocn_type(1:3) == 'POP' ) then
            if ( fl_a(n) > 1.0-eps) fl_a(n)=1.0  ! let's just call it "one"
            if ( fl_a(n) < 0.001  ) fl_a(n)=0.0  ! mininmum non-zero land frac
        else if ( ocn_type(1:3) == 'MOM' ) then
            if ( fl_a(n) > 1.0-eps) fl_a(n)=1.0  ! let's just call it "one"
            if ( fl_a(n) < 0.001  ) fl_a(n)=0.0  ! mininmum non-zero land frac
        endif
!!-----------------wangln 2008-08-12 according with the former in MOM type--------------

        !--- should we be concerned about this "lost" area? ---
!------------------wangln 2008-09-10 test the lw error------------------------------------
       !if ( fl_a(n) > 0.0  .and.  fl_a(n) < 0.001 ) then
       !  fl_a(n) = 0.0 
       !  fo_a(n) = fo_a(n)/(fo_a(n)+fi_a(n)) ! adjust ice & ocn fractions??
       !  fi_a(n) = fi_a(n)/(fo_a(n)+fi_a(n))
       !end if
!------------------wangln 2008-09-10 test the lw error------------------------------------
     end do

     !--- fractions on lnd grid ---
     where (domain_l%mask /= 0) 
       fa_l=1.0     ! always true
       fl_l=fl_a    ! always true (assumes atm & lnd have same grid)
       fi_l=0.0     ! always true (no ice/lnd overlap)
       fo_l=0.0     ! always true (no ocn/lnd overlap)
     elsewhere
       fa_l = 1.0
       fl_l = 0.0
       fi_l = fi_a
       fo_l = 1.0
     end where

   ELSE
     !--------------------------------------------------------
     ! update interface fractions
     !--------------------------------------------------------

     !--- update fractions on ice grid ---
     where (domain_i%mask /= 0) 
       fi_i=ifrac_i                ! an ice output state
     end where
  
     !--- update fractions on ocn grid ---
     where (domain_o%mask /= 0) 
       fi_o=    fi_i               ! assumes ice & ocn have same grid
       fa_o=max(0.0,1.0-fi_o)      ! not ice => atm
     end where

     !--- update fractions on atm grid ---
#ifdef MPI_P
     t_o(:,1) = fi_o ; call map(map_o2af,t_o,t_a) ; 
#if 1
     allocate(redu1(n_a/CPL_SIZE))
     allocate(redu2(n_a))
     redu1=0.
     redu2=0.
     iv = 1
     do j=ist_a,iend_a
       redu1(iv) = t_a(j,1)
       iv = iv+1
     enddo
     call MPI_allgather(redu1,iv-1,MPI_REAL8,&
           &  redu2,iv-1,MPI_REAL8,MPI_COMM_CPL,ierr)
#else
     allocate(redu1(n_a))
     allocate(redu2(n_a))
     redu1=0.
     redu2=0.
     do j=ist_a,iend_a
       redu1(j) = t_a(j,1)
     enddo
     call MPI_ALLREDUCE(redu1,redu2,n_a,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
#endif
     t_a(:,1) = redu2(:)
     fi_a = t_a(:,1)
     deallocate(redu1)
     deallocate(redu2)
#else
     t_o(:,1) = fi_o ; call map(map_o2af,t_o,t_a) ; fi_a = t_a(:,1)
#endif
     do n=1,size(fo_a)
       fo_a(n) = max(0.0,1.0-fi_a(n)-fl_a(n)) ! not (ice or lnd) => ocn
     end do

   END IF
  
   first_call = .false.

END SUBROUTINE map_frac

!===============================================================================

!----------------------wangln 2007-05-27--------------------------------
!!SUBROUTINE map(smat,src,dst  ,  fs,fd)
!!mapmap---xujinxiu
SUBROUTINE map(smat,src,dst  ,  fs,fd, remap_method)

   implicit none

   !--- arguments ---
   type(sparse_matrix),intent(in ) :: smat   ! mapping method selection
   real            ,intent(in ) :: src(:,:) ! multiple data fields on src grid
   real            ,intent(out) :: dst(:,:) ! multiple data fields on dst grid
   real   ,optional,intent(in ) :: fs (:)   ! fraction applied to src grid
   real   ,optional,intent(in ) :: fd (:)   ! fs mapped to dst grid

!----------------------wangln 2007-05-27--------------------------------
   logical,optional,intent(in ) :: remap_method ! if Y/N use the bilinear
   logical                      :: bilinear_correct = .false.
!----------------------wangln 2007-05-27--------------------------------


   !--- local ---
   integer            :: n,k     ! generic loop indicies
   integer,save       :: timer_n ! system-clock-timer number
   logical,save       :: first_call = .true.

   !--- formats ---
   character(len=*),parameter :: F00 = "('(map) ',a,50f6.2)"
   character(len=*),parameter :: F01 = "('(map) ',a,50i6  )"
   character(len=*),parameter :: F02 = "('(map) ',a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   given input data and a desired mapping type,
!   gather the approprate matrix data and do the mapping
!-------------------------------------------------------------------------------
!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++
    if( present( remap_method ) ) bilinear_correct = remap_method
    !if( bilinear_correct ) write(6,*) '++++++++++++++++++ Now The Remap Method  Is TRUE'
!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++


   if (size(src,DIM=2) /= size(dst,DIM=2)) then
!     call shr_abort('<map> num of flds')
     write(6,F02) '<map> num of flds'
     call shr_sys_flush(6)
     stop
   endif

   if (first_call) call shr_timer_get(timer_n,'map')
   first_call=.false.
   call shr_timer_start(timer_n)

   if (.not. PRESENT(fs))  then  
     !---------------------------------------------------
     ! straight map - no input fraction, no normalization
     !---------------------------------------------------
     if (smat%n == 0) then  !--- identity map
       if (size(src,DIM=1) /= size(dst,DIM=1)) then
!         call shr_abort('<map> fld size ')
         write(6,F02) '<map> fld size'
         call shr_sys_flush(6)
         stop
       endif
       dst = src
     else
!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++
       !call map_matmult(src,dst,smat)
       call map_matmult(src,dst,smat, correcting=bilinear_correct)
!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++
     end if
   else  
     !---------------------------------------------------
     ! map with input fraction and normalized result
     !---------------------------------------------------
     if (smat%n == 0) then  !--- identity map
       if (size(src,DIM=1) /= size(dst,DIM=1)) then
!         call shr_abort('<map> fld size ')
         write(6,F02) '<map> fld size'
         call shr_sys_flush(6)
         stop
       endif
       dst = src
       do n=1,size(src,DIM=1)
         if (fs(n) == 0.0) dst(n,:) = 0.0
       end do
     else
       if (.not. PRESENT(fd))  then
!         call shr_abort('<map> fs but no fd')
         write(6,F02) '<map> fs but no fd'
         call shr_sys_flush(6)
         stop
       endif
       call map_matmult(src,dst,smat,fa=fs,fb=fd)
     end if
   endif

   call shr_timer_stop(timer_n)

END SUBROUTINE map

!===============================================================================
!!SUBROUTINE map_matmult(a,b,smat,fa,fb)
SUBROUTINE map_matmult(a,b,smat,fa,fb, correcting)

   implicit none

   !--- arguments ---
   real               ,intent(in ) :: a   (:,:) ! RHS data vectors
   real               ,intent(out) :: b   (:,:) ! LHS data vectors
   type(sparse_matrix),intent(in ) :: smat      ! sparse matrix data type
   real      ,optional,intent(in ) :: fa  (:)   ! fraction applied to RHS vector
   real      ,optional,intent(in ) :: fb  (:)   ! fb=S*fa

!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++
   logical   ,optional,intent(in ) :: correcting

!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++


   !--- local ---
   integer    :: i                 ! index wrt b(i,m)
   integer    :: j                 ! index wrt a(j,m) 
   integer    :: n                 ! index wrt S(n) 
   integer    :: m                 ! index wrt b(i,m),a(j,m)
   integer    :: nf                ! number of fields to map: size of b(i,:)
   integer    :: nb                ! length of field  vector: size of b(:,m)
   real       :: sum               ! register variable for b(i,m)
   real       :: coef              ! sum coefficient
   real       :: fbr               ! 1.0/fb
   logical    :: normalize         ! true => apply fa(:) & divide by fb(:)
   integer    :: timer_m = 0       ! system-clock-timer number
   integer    :: timer_n = 0       ! system-clock-timer number

#ifdef  _OPENMP
   integer  :: omp_get_max_threads !  $OMP function call
#endif

   !--- formats ---
   character(len=*),parameter :: F00 = "('<map_matmult> ',a,50(f6.2 ))"
   character(len=*),parameter :: F01 = "('<map_matmult> ',a,50(i3,3x))"
   character(len=*),parameter :: F02 = "('<map_matmult> ',    f7.3 )"
   character(len=*),parameter :: F03 = "('<map_matmult> ',a,i5,a,i2)"

!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++
   integer          :: x1, x2, y1, y2
   real             :: dx, dy
   real             :: dxx, dyy
!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++
#ifdef MPI_P
    INTEGER ::   IST_b,IEND_b,IPT_b,MM,rcode
#endif


!-------------------------------------------------------------------------------
! PURPOSE:
!   does a sparse-matrix-format multiply: b = S*a 
!   with optional RHS fraction & normalization b=(S*fa*a)/(S*fa)
! ASSUMPTION:
!   either fa & fb are both present, or are both ommited
!-------------------------------------------------------------------------------

!  if (   (PRESENT(fb) .and. .not. PRESENT(fa)) &
!  & .or. (PRESENT(fa) .and. .not. PRESENT(fb)) ) then
!    write(6,F00) 'ERROR: presence of fa <=> presence of fb' 
!    stop
!  end if

   if (timer_m == 0) then
#ifdef  _OPENMP
     n = omp_get_max_threads()
     write(6,F03) 'FYI: this routine is threaded, omp_get_max_threads() = ',n
#else
     write(6,F03) 'FYI: this routine is not threaded'
#endif
   end if

   if (timer_m == 0) call shr_timer_get(timer_m,'map_matmult w/  fraction')
   if (timer_n == 0) call shr_timer_get(timer_n,'map_matmult w/o fraction')

   normalize = .false.
   if (PRESENT(fb)) normalize = .true.
   nb = size(b,DIM=1)
   nf = size(b,DIM=2)
!   print *,"---------map_matmult---nb,nf,CPL_SIZE=",nb,nf,CPL_SIZE
!   print *

#ifdef MPI_P
!call mpi_group_size(SHR_MSG_COMM_CPL, CPL_SIZE, rcode)
!call mpi_group_rank(SHR_MSG_COMM_CPL, CPL_RANK, rcode)
    
    MM = MOD(nb, CPL_SIZE)
    IF(CPL_RANK < MM) then
      IEND_b = nb / CPL_SIZE + 1
      IPT_b  = CPL_RANK * IEND_b
    ELSE
      IEND_b = nb / CPL_SIZE
      IPT_b  = CPL_RANK * IEND_b + MM
    ENDIF
    IST_b = IPT_b + 1
    IEND_b = IST_b + IEND_b - 1
#endif
  
!    print *,"xjx,,,in map_mod...IST_b, IEND_b=",IST_b, IEND_b
   !-----------------------------------------------------------------
   ! matrix multiply 
   !-----------------------------------------------------------------
   if ( normalize ) then  !--- apply fraction vector & normalize ---

     call shr_timer_start(timer_m)
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i,j,n,m,fbr ) reduction(+:sum)
     do i = IST_b, IEND_b
#else
!$OMP PARALLEL DO PRIVATE(i,j,n,m,sum,fbr )
     do i = 1,nb
#endif
       fbr  = 1.0
       if ( fb(i) > 0 ) fbr = 1.0/fb(i)
       do m=1,nf
         sum  = 0.0
         coef = 0.0                 ! wtw 2009.01
         do n=smat%sn1(i),smat%sn1(i+1)-1
           j=smat%col(n)
           sum = sum + smat%S(n)*fa(j)*a(j,m)
           coef =coef + smat%S(n)*fa(j)   ! wtw 2009.01
         end do
         b(i,m) = sum*fbr 
         if( coef > 0.01 ) b(i,m) = sum / coef  ! wtw 2009.01
       end do
     end do
!$OMP  END PARALLEL DO
     call shr_timer_stop(timer_m)
!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++

   else                   !--- don't apply fraction vector or normalize ---
!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++
     if( correcting ) then
        !write(6,*) '+++++++++++map_matmult Correcting==TRUE'
        call shr_timer_start(timer_n)
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i,j,n,m,dx,dy,x1,x2,y1,y2,dxx,dyy ) reduction(+:sum)
       do i = IST_b, IEND_b
#else
!$OMP PARALLEL DO PRIVATE(i,j,n,m,sum,fbr )
       do i=1,nb
#endif
          do m=1,nf
             sum  = 0.0
             do n=smat%sn1(i),smat%sn1(i+1)-1
                j=smat%col(n)
                dx = smat%dx(n)
                dy = smat%dy(n)
                x1 = smat%x1(n)
                x2 = smat%x2(n)
                y1 = smat%y1(n)
                y2 = smat%y2(n)
                dxx = ( a(x1,m)-a(x2,m) ) * dx
                dyy = ( a(y1,m)-a(y2,m) ) * dy
!---------------------------------------wangln 2007-05-30------------------------
                sum = sum + smat%S(n) * ( a(j,m) + dxx + dyy )
!---------------------------------------wangln 2007-05-30------------------------
             end do
             b(i,m) = sum
          end do
        end do
!$OMP  END PARALLEL DO
!xjx        call shr_timer_stop(timer_m)
        call shr_timer_stop(timer_n)
!++++++++++++++++wangln 2007-05-27 Decide Y/N using the Bilinear Correcting+++++++++++++
     else  ! if correcting=.false.
        call shr_timer_start(timer_n)
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i,j,n,m) reduction(+:sum)
     do i = IST_b, IEND_b
#else
!$OMP PARALLEL DO PRIVATE(i,j,n,m,sum)
        do i=1,nb
#endif
           do m=1,nf
              sum = 0.0
             do n=smat%sn1(i),smat%sn1(i+1)-1
                j=smat%col(n)
                sum = sum + smat%S(n)*a(j,m)
             end do
             b(i,m) = sum
          end do
       end do
!$OMP  END PARALLEL DO
       call shr_timer_stop(timer_n)

     endif ! ENDIF with the correcting

  end if


END SUBROUTINE map_matmult

!===============================================================================
END MODULE map_mod
!===============================================================================
