#include <misc.h>

!===============================================================================
! CVS $Id: mrgf_atm.F90,v 1.1 2000/01/03 22:56:20 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/mrgf_atm.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

SUBROUTINE mrgf_atm(Faii,Fall,Faoc,Faxx)

   USE buf_mod
   USE map_mod
!-----------wangln 2007-06-23 For MME-----------------------------
   USE control_mod, only : atm_type
!-----------wangln 2007-06-23 For MME-----------------------------

   implicit none

   !----- arguments -----
   real,intent(in)  :: Faii(n_a,nFaii) ! atm/ice fluxes computed by ice
   real,intent(in)  :: Fall(n_a,nFall) ! atm/lnd fluxes computed by atm
   real,intent(in)  :: Faoc(n_a,nFaoc) ! atm/ocn fluxes computed by cpl
   real,intent(out) :: Faxx(n_a,nFaxx) ! merged atm/surface fluxes together

   !----- local -----
   integer          :: ki,kl,ko,kx     ! indicies to buffer fields
#ifdef MPI_P
   integer i
#endif

!-------------------------------------------------------------------------------
! PURPOSE:
!   gather various flux component fields (all on the atm grid) and
!   merge these atm/surface component fluxes (via geographical weights),
!   forming the complete set of atm input fluxes
!-------------------------------------------------------------------------------

   !--- merge taux -------------------
   ki=kFaii_taux
   kl=kFall_taux
   ko=kFaoc_taux
   kx=kFaxx_taux
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
   Faxx(i,kx) = fi_a(i)*Faii(i,ki) + fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif
    
   !--- merge tauy -------------------
   ki=kFaii_tauy
   kl=kFall_tauy
   ko=kFaoc_tauy
   kx=kFaxx_tauy
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
   Faxx(i,kx) = fi_a(i)*Faii(i,ki) + fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif

   !--- merge latent heat flux -------
   ki=kFaii_lat
   kl=kFall_lat
   ko=kFaoc_lat
   kx=kFaxx_lat
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
   Faxx(i,kx) = fi_a(i)*Faii(i,ki) + fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif

   !--- merge sensible heat flux -----
   ki=kFaii_sen
   kl=kFall_sen
   ko=kFaoc_sen
   kx=kFaxx_sen
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
   Faxx(i,kx) = fi_a(i)*Faii(i,ki) + fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif

   !--- merge longwave upward flux ---
   ki=kFaii_lwup
   kl=kFall_lwup
   ko=kFaoc_lwup
   kx=kFaxx_lwup
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
   Faxx(i,kx) = fi_a(i)*Faii(i,ki) + fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif

   !--- merge evaporative flux -------
   ki=kFaii_evap
   kl=kFall_evap
   ko=kFaoc_evap
   kx=kFaxx_evap
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
   Faxx(i,kx) = fi_a(i)*Faii(i,ki) + fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif

   !--- merge 2m reference temp-------
   ki=kFaii_tref
   kl=kFall_tref
   ko=kFaoc_tref
   kx=kFaxx_tref
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
   Faxx(i,kx) = fi_a(i)*Faii(i,ki) + fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif

!-----------wangln 2007-06-23 For MME-----------------------------
   if( atm_type(1:4) == 'CAM3' .or. atm_type(1:3) == 'BCC') then
      ki=kFaii_qref
      kl=kFall_qref
      ko=kFaoc_qref
      kx=kFaxx_qref
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
   Faxx(i,kx) = fi_a(i)*Faii(i,ki) + fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif
   endif
!-----------wangln 2007-06-23 For MME-----------------------------
   if( (atm_type(1:4)=='CAM3' .or. atm_type(1:3)=='BCC') .and. carbon_flux ) then
      kl=kFall_carbon
      ko=kFaoc_carbon
      kx=kFaxx_carbon
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) =  fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) =  fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif
   endif

#ifdef MOZART2
!-------------------------------------------
! wtw for MOZART2
!
   !--- merge mss_dst01 ---
   kl=kFall_mss_dst01
   kx=kFaxx_mss_dst01
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge mss_dst02 ---
   kl=kFall_mss_dst02
   kx=kFaxx_mss_dst02
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else 
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge mss_dst03 ---
   kl=kFall_mss_dst03
   kx=kFaxx_mss_dst03
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else 
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge mss_dst04 ---
   kl=kFall_mss_dst04
   kx=kFaxx_mss_dst04
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else 
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge vlc_dst01 ---
   kl=kFall_vlc_dst01
   kx=kFaxx_vlc_dst01
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else 
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge vlc_dst02 ---
   kl=kFall_vlc_dst02
   kx=kFaxx_vlc_dst02
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge vlc_dst03 ---
   kl=kFall_vlc_dst03
   kx=kFaxx_vlc_dst03
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge vlc_dst04 ---
   kl=kFall_vlc_dst04
   kx=kFaxx_vlc_dst04
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge lnd_ram1 ---
   kl=kFall_lnd_ram1
   kx=kFaxx_lnd_ram1
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge lnd_fv ---
   kl=kFall_lnd_fv
   kx=kFaxx_lnd_fv
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge vlc_SO4 ---
   kl=kFall_vlc_SO4
   kx=kFaxx_vlc_SO4
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge vlc_NH4 ---
   kl=kFall_vlc_NH4
   kx=kFaxx_vlc_NH4
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge vlc_CB2 ---
   kl=kFall_vlc_CB2
   kx=kFaxx_vlc_CB2
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge vlc_SO2 ---
   kl=kFall_vlc_SO2
   kx=kFaxx_vlc_SO2
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

!-----------------------------------------------------------------

   !--- merge flux_ISOP -
   kl=kFall_flux_ISOP
   kx=kFaxx_flux_ISOP
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge flux_ACET--
   kl=kFall_flux_ACET
   kx=kFaxx_flux_ACET
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge flux_C3H6--
   kl=kFall_flux_C3H6
   kx=kFaxx_flux_C3H6
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge flux_c2H4--
   kl=kFall_flux_C2H4
   kx=kFaxx_flux_C2H4
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge flux_OC1---
   kl=kFall_flux_OC1
   kx=kFaxx_flux_OC1
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

   !--- merge flux_C10H16
   kl=kFall_flux_C10H16
   kx=kFaxx_flux_C10H16
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) = 0.0 + fl_a(i)*Fall(i,kl) + 0.0
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) = 0.0 + fl_a(i)*Fall(:,kl) + 0.0
#endif

!---------------------------------------------------------------
#endif  ! end define MOZART2

!-----------------------------------------------------------------
#ifdef HAMOCC

   !--- merge flux_co2
      ko=kFaoc_co2flux
      kx=kFaxx_flux_co2
      kl=kFall_carbon
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) =   fl_a(i)*Fall(i,kl) + fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) =   fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)
#endif

   !--- merge flux_n2o
      ko=kFaoc_n2oflux
      kx=kFaxx_flux_n2o
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) =   fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) =   fo_a*Faoc(:,ko)
#endif

   !--- merge flux_dms
      ko=kFaoc_dmsflux
      kx=kFaxx_flux_dms
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i=ist_a, iend_a
      Faxx(i,kx) =   fo_a(i)*Faoc(i,ko)
   enddo
!$omp  end parallel do
#else
      Faxx(:,kx) =   fo_a*Faoc(:,ko)
#endif

#endif   ! end ifdef HAMOCC 

END

!===============================================================================
