#include <misc.h>

#ifdef HAMOCC
  SUBROUTINE mrgf_ocn(Faoa,Faoc,Fioi,roff, Sa, fi, Si, Foxx)
#else
  SUBROUTINE mrgf_ocn(Faoa,Faoc,Fioi,roff, Sa, fi, Foxx)
#endif
   USE buf_mod   ! contains field buffers & buffer indicies
   USE map_mod   ! contains cell fractions
!----------------wangln 2007-11-29 add lat 7 sen for mom/sis--------
   USE control_mod, only : ocn_type
   USE const_mod,   only : latvap
   USE msg_ocn_mod, only : angle_sin, angle_cos
!----------------wangln 2007-11-29 add lat 7 sen for mom/sis--------

   implicit none

   !----- arguments -----
   real,intent(in)  :: Faoa(n_o,nFaoa)  ! (in) atm/ocn flux, computed by atm
   real,intent(in)  :: Faoc(n_o,nFaoc)  ! (in) atm/ocn flux, computed by cpl
   real,intent(in)  :: Fioi(n_o,nFioi)  ! (in) ice/ocn flux, computed by ice
   real,intent(in)  :: roff(n_o,1)      ! (in) lnd/ocn flux, computed by lnd
   real,intent(in)  :: Sa  (n_o,nSa)    ! (in) atm state
   real,intent(in)  :: fi  (n_o)        ! (in) fraction of ice
   real,intent(out) :: Foxx(n_o,nFoxx)  ! (out) merged ocn/x flux: sent to ocn

#ifdef HAMOCC
   real,intent(in)  :: Si  (n_i,nSi)    ! (in) ice state !wufh add HAMOCC 2015-03-13
#endif
   !----- local variables -----
   integer :: n                         ! counter
   real    :: sfa_o, sfi_o              ! scalars of array values

!-------------------------------------------------------------------------------
! PURPOSE:
!   gather various flux component fields (all on the ocn grid),
!   and then merge these ocn/comp component fluxes 
!   forming the complete set of ocn input fluxes
! NOTE:
!   The presence of Sa and fi is somewhat of a hack, sending such data was not
!   in the original requirements/design of cpl5.   Adding them to the list of
!   time avg'd input to the ocn was most easily accomplished by adding them to
!   the list of ocn-input fluxes.
!-------------------------------------------------------------------------------

 if( ocn_type(1:3) == 'POP' ) then
!$OMP PARALLEL DO PRIVATE(n,sfa_o,sfi_o)
#ifdef MPI_P
   do n=IST_o,IEND_o 
#else
   do n=1,n_o
#endif
   sfa_o = fa_o(n)
   sfi_o = fi_o(n)
   !--- merge tau ---------------------------------------------------
   Foxx(n,kFoxx_taux ) = sfa_o*Faoc(n,kFaoc_taux ) + sfi_o*Fioi(n,kFioi_taux)
   Foxx(n,kFoxx_tauy ) = sfa_o*Faoc(n,kFaoc_tauy ) + sfi_o*Fioi(n,kFioi_tauy)

   !--- merge heat flux ---------------------------------------------
   Foxx(n,kFoxx_swnet) = sfa_o*Faoc(n,kFaoc_swnet) + sfi_o*Fioi(n,kFioi_swnet)
   Foxx(n,kFoxx_lat  ) = sfa_o*Faoc(n,kFaoc_lat )
   Foxx(n,kFoxx_sen  ) = sfa_o*Faoc(n,kFaoc_sen )
       
   Foxx(n,kFoxx_lwup ) = sfa_o*Faoc(n,kFaoc_lwup)
   Foxx(n,kFoxx_lwdn ) = sfa_o*Faoa(n,kFaoa_lwdn)
   Foxx(n,kFoxx_melth) = sfi_o*Fioi(n,kFioi_melth)

   !--- merge water flux --------------------------------------------
   Foxx(n,kFoxx_rain ) = sfa_o*Faoa(n,kFaoa_rain)
   Foxx(n,kFoxx_snow ) = sfa_o*Faoa(n,kFaoa_snow)
   Foxx(n,kFoxx_evap ) = sfa_o*Faoc(n,kFaoc_evap)
   Foxx(n,kFoxx_meltw) = sfi_o*Fioi(n,kFioi_meltw)
   Foxx(n,kFoxx_roff ) =      roff(n,1)

   !--- merge salt flux --------------------------------------------
   Foxx(n,kFoxx_salt ) = sfi_o*Fioi(n,kFioi_salt)

   !--- merge input states -----------------------------------------
   Foxx(n,kFoxx_pslv ) = Sa(n,kSa_pslv)
   Foxx(n,kFoxx_ifrac) = fi(n)

   enddo
!$omp  end parallel do
 else if( ocn_type(1:3) == 'MOM' ) then
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(n,sfa_o,sfi_o)
   do n=IST_o,IEND_o 
#else
!$OMP PARALLEL DO PRIVATE(n,sfa_o,sfi_o)
   do n=1,n_o
#endif
      if( mask_o(n) /= 0 ) then

        sfa_o = fa_o(n)
        sfi_o = fi_o(n)
   !--- merge tau ---------------------------------------------------
        Foxx(n,kFoxx_taux ) =  sfa_o*Faoc(n,kFaoc_taux ) + sfi_o * Fioi(n,kFioi_taux)   ! downward (+) 
        Foxx(n,kFoxx_tauy ) =  sfa_o*Faoc(n,kFaoc_tauy ) + sfi_o * Fioi(n,kFioi_tauy)   ! downward (+) 

   !--- merge heat flux ---------------------------------------------
        Foxx(n,kFoxx_swnet) =  sfa_o*Faoc(n,kFaoc_swnet) + sfi_o * Fioi(n,kFioi_swnet)

        Foxx(n,kFoxx_lat  ) =  sfa_o*Faoc(n,kFaoc_lat )  
!!xjx--20081225        Foxx(n,kFoxx_evap ) = -sfa_o*Faoc(n,kFaoc_evap)  
        Foxx(n,kFoxx_evap ) = -sfa_o*Faoc(n,kFaoc_evap)  + sfi_o * Fioi(n,kFioi_evap )   ! upward (+)
        Foxx(n,kFoxx_sen  ) = -sfa_o*Faoc(n,kFaoc_sen )  + sfi_o * Fioi(n,kFioi_sen )    ! upward (+)
       
        Foxx(n,kFoxx_lwup ) = sfa_o*Faoc(n,kFaoc_lwup)
        Foxx(n,kFoxx_lwdn ) = sfa_o*Faoa(n,kFaoa_lwdn)
        Foxx(n,kFoxx_melth) = sfi_o*Fioi(n,kFioi_melth)

   !--- merge water flux --------------------------------------------
        Foxx(n,kFoxx_rain ) = sfa_o*Faoa(n,kFaoa_rain) + sfi_o *  Fioi(n,kFioi_meltw) 

!xjx-20081225        Foxx(n,kFoxx_rain ) = sfa_o*Faoa(n,kFaoa_rain) + sfi_o * ( Fioi(n,kFioi_meltw) - Fioi(n,kFioi_evap ) )

        Foxx(n,kFoxx_snow ) = sfa_o*Faoa(n,kFaoa_snow)
        Foxx(n,kFoxx_roff ) =      roff(n,1)

   !--- merge salt flux --------------------------------------------
        Foxx(n,kFoxx_salt ) = sfi_o*Fioi(n,kFioi_salt)

   !--- merge input states -----------------------------------------
!wangln       Foxx(n,kFoxx_pslv ) = Sa(n,kSa_pslv) + Fioi(n,kFioi_psurf)

        Foxx(n,kFoxx_pslv ) = Sa(n,kSa_pslv) 

        !!if( carbon_flux ) Foxx(n,kFoxx_co2 ) = sfa_o*Faoa(n,kFaoa_co2)
        if( carbon_flux ) Foxx(n,kFoxx_co2 ) = Faoa(n,kFaoa_co2)

!---------------
! wtw -- 20101101
!
        Foxx(n,kFoxx_dEdt ) = sfa_o*Faoc(n,kFaoc_dEdt)
!----------------

#ifdef HAMOCC
        Foxx(n,kFoxx_ubot) = Sa(n,kSa_u)
        Foxx(n,kFoxx_vbot) = Sa(n,kSa_v)
        Foxx(n,kFoxx_ifrac) = fi(n)
        Foxx(n,kFoxx_sictho) = Si(n,kSi_sictho)
        Foxx(n,kFoxx_sicsno) = Si(n,kSi_sicsno)
        Foxx(n,kFoxx_n2o) = Faoa(n,kFaoa_n2o) ! 3.e-7
        Foxx(n,kFoxx_dms) = Faoa(n,kFaoa_dms) ! 0.1e-9
#endif


     endif
  enddo
!$omp  end parallel do
 endif

END

!===============================================================================
