!===============================================================================
! CVS $Id: mrgs_atm.F90,v 1.1 2000/01/03 22:56:20 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/mrgs_atm.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

SUBROUTINE mrgs_atm(Si,Sl,So, Ss)

   USE buf_mod
   USE map_mod

!-----------wangln 2007-06-23 For MME-----------------------------
   USE control_mod, only : atm_type
!-----------wangln 2007-06-23 For MME-----------------------------

   IMPLICIT none

   !----- arguments----
   real,intent(in)  :: Si(n_a,nSi)   ! ice state
   real,intent(in)  :: Sl(n_a,nSl)   ! lnd state
   real,intent(in)  :: So(n_a,nSo)   ! ocn state
   real,intent(out) :: Ss(n_a,nSs)   ! merged surface states

   !----- local -----
   integer          :: ki,kl,ko,kx   ! indicies to buffer fields
#ifdef MPI_P
   integer i
#endif

!-------------------------------------------------------------------------------
! PURPOSE:
!   merge surface states, forming the complete set of atm input states
!   (eg. surface temperature &  surface albedo)
!-------------------------------------------------------------------------------
   
   !--- 3-way merge of T ----------------------
   ki=kSi_t
   kl=kSl_t
   ko=kSo_t
   kx=kSs_t
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
   Ss(i,kx) = fi_a(i)*Si(i,ki) + fl_a(i)*Sl(i,kl) + fo_a(i)*So(i,ko)
   enddo
!$omp  end parallel do
#else
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)
#endif

!   print *,"mrgs_atm.F90-----size(fi_a,fl_a,fo_a)=",size(fi_a),size(fl_a),size(fo_a)

   !--- 3-way merge of visible/direct albedo ---
   ki=kSi_avsdr
   kl=kSl_avsdr
   ko=kSo_avsdr
   kx=kSs_avsdr
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
   Ss(i,kx) = fi_a(i)*Si(i,ki) + fl_a(i)*Sl(i,kl) + fo_a(i)*So(i,ko)
   enddo
!$omp  end parallel do
#else
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)
#endif

   !--- 3-way merge of near-infrared/direct albedo ---
   ki=kSi_anidr
   kl=kSl_anidr
   ko=kSo_anidr
   kx=kSs_anidr
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
   Ss(i,kx) = fi_a(i)*Si(i,ki) + fl_a(i)*Sl(i,kl) + fo_a(i)*So(i,ko)
   enddo
!$omp  end parallel do
#else
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)
#endif

   !--- 3-way merge of visible/diffuse albedo ---
   ki=kSi_avsdf
   kl=kSl_avsdf
   ko=kSo_avsdf
   kx=kSs_avsdf
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
   Ss(i,kx) = fi_a(i)*Si(i,ki) + fl_a(i)*Sl(i,kl) + fo_a(i)*So(i,ko)
   enddo
!$omp  end parallel do
#else
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)
#endif

   !--- 3-way merge of near-infrared/diffuse albedo ---
   ki=kSi_anidf
   kl=kSl_anidf
   ko=kSo_anidf
   kx=kSs_anidf
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
   Ss(i,kx) = fi_a(i)*Si(i,ki) + fl_a(i)*Sl(i,kl) + fo_a(i)*So(i,ko)
   enddo
!$omp  end parallel do
#else
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)
#endif

   !--- snow depth, no merge, set equal to land value-------------
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
   Ss(i,kSs_snowh) = fl_a(i)*Sl(i,kSl_snowh)
   enddo
!$omp  end parallel do
#else
   Ss(:,kSs_snowh) = fl_a*Sl(:,kSl_snowh)
#endif

!-----------wangln 2007-06-23 For MME-----------------------------
   if( atm_type(1:4) == 'CAM3' .or. atm_type(1:3) == 'BCC') then
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
      Ss(i,kSst_t) = So(i,kSo_t) 
   enddo
!$omp  end parallel do
#else
   Ss(:,kSst_t) = So(:,kSo_t) 
#endif
    !!   ko=kSo_t
    !!   kx=kSst_t
    !!   Ss(:,kx)=-9999999.9
    !!   Ss(:,kx) =So(:,ko)
   endif
!-----------wangln 2007-06-23 For MME-----------------------------

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
   if( atm_type(1:4) == 'CAM3' .or. atm_type(1:3) == 'BCC') then
   !--- 3-way merge of  10m U wind speed ---
   ki=kSi_u10m
   kl=kSl_u10m
   ko=kSo_u10m
   kx=kSs_u10m
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
      Ss(i,kx) = fi_a(i)*Si(i,ki) + fl_a(i)*Sl(i,kl) + fo_a(i)*So(i,ko)
   enddo
!$omp  end parallel do
#else
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)
#endif
   !--- 3-way merge of  10m V wind speed ---
   ki=kSi_v10m
   kl=kSl_v10m
   ko=kSo_v10m
   kx=kSs_v10m
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
      Ss(i,kx) = fi_a(i)*Si(i,ki) + fl_a(i)*Sl(i,kl) + fo_a(i)*So(i,ko)
   enddo
!$omp  end parallel do
#else
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)
#endif
   !------ get Ocn, Ice & lnd compentent model surface temperatrure
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i)
   do i = ist_a, iend_a
      Ss(i,kSs_tocn) = So(i, kSo_t) 
      Ss(i,kSs_tice) = Si(i, kSi_t) 
      Ss(i,kSs_tlnd) = Sl(i, kSl_t) 
   enddo
!$omp  end parallel do
#else
   Ss(:,kSs_tocn) = So(:, kSo_t)   
   Ss(:,kSs_tice) = Si(:, kSi_t)   
   Ss(:,kSs_tlnd) = Sl(:, kSl_t)   
#endif
   endif

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

END

!===============================================================================
