#include <misc.h>

!===============================================================================
! CVS $Id: msg_atm_mod.F90,v 1.15 2001/12/07 22:13:36 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/msg_atm_mod.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

MODULE msg_atm_mod

   use control_mod      ! control data
   use buf_mod          ! model   data
   use const_mod        ! physical constants
   use map_mod          ! map data
   use shr_msg_mod      ! msg constants, data, routines
   use shr_timer_mod    ! timing routines
#ifdef MPI_P
   use cpl_mod,  only: atm_tid
#endif

   implicit none

   private  ! resticted access
   public  :: msg_atmi,msg_atmr,msg_atms,msg_atmf

   !--- module variables ---

!--------------------wangln 2007-06-23 For MME--------------
   !integer,parameter   :: nsflds=15        ! number of fields sent
   !integer,parameter   :: nrflds=19        ! number of fields recv'd
   integer   :: nsflds        ! number of fields sent
   integer   :: nrflds        ! number of fields recv'd
!--------------------wangln 2007-06-23 For MME--------------

   real   ,allocatable :: buffs(:,:)       ! msg buffer: send
   real   ,allocatable :: buffr(:,:)       ! msg buffer: recv
   real   ,allocatable :: area_fact_c2a(:) ! cpl->model area correction factor
   real   ,allocatable :: area_fact_a2c(:) ! model->cpl area correction factor

   !--- aggregate timing info ---
   integer      :: timer_send    ! shr_timer timer number for data send
   integer      :: timer_recv    ! shr_timer timer number for data recv

   !--- detailed timing info ---
!  integer(8)   :: irtc_w        ! rtc tics when waiting for msg
!  integer(8)   :: irtc_r        ! rtc tics when msg received
!  integer(8)   :: irtc_u        ! rtc tics when msg unpacked
!  integer(8)   :: irtc_p        ! rtc tics when packing msg
!  integer(8)   :: irtc_s        ! rtc tics when sending msg 

   !--- formats ---
   character(len=*),parameter :: F98 = "('comm_diag ',a3,1x,a4,1x,i3,es26.19,1x,a)"
   character(len=*),parameter :: F99 = "('[mp timing]  irtc = ',i20,' ',a)"

CONTAINS

!===============================================================================

SUBROUTINE msg_atmi ()

   implicit none

#ifdef MPI_P
   integer  SHR_MSG_TID_ATM 
#endif

   !--- local ---
   integer :: i,j,n            ! generic loop indicies
   real    :: minf,maxf        ! min/max cell area correction factors

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_atmi) ',a,i4)"
   character(len=*),parameter :: F02 = "('(msg_atmi) ',a,2f12.9)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   send/recv initial msg to/from atm model
! INPUT:
!   o flags contained in control.h
! OUTPUT:
!   o atm info-buffer data         : ibuff_a
!   o atm spatial coordinate arrays: xv_a,xc_a,yv_a,yc_a
! MESSAGES:
!   o data sent  : cpl info-buffer flags
!   o data recv'd: atm info-buffer flags, grids
!-------------------------------------------------------------------------------

!--------------------wangln 2007-06-23 For MME--------------
   if( atm_type(1:4) == 'CAM2') then
       nsflds = 15
!!       nrflds = 19
       nrflds = 20       !! for MOM & POP

   else if( atm_type(1:4) == 'CAM3' .or. atm_type(1:3) == 'BCC') then
       if( carbon_flux ) then

!--------------
#ifdef MOZART2
#ifdef HAMOCC
           nsflds = 23 + 14 + 6 + 3     ! wufh add co2flux, n2oflux, dmsflux for atm 2017-01-19
           nrflds = 21 + 2              ! wufh add n2o, dms from atm to HAMOCC  2017-01-19
#else
           nsflds = 23 + 14 + 6
           nrflds = 21
#endif
!-------------------------
#else

#ifdef HAMOCC
           nsflds = 23 + 3     ! wufh add co2flux, n2oflux, dmsflux for atm 2017-01-19
           nrflds = 21 + 2              ! wufh add n2o, dms from atm to HAMOCC  2017-01-19
#else
           nsflds = 23
           nrflds = 21
#endif

#endif
!-----------------------------------------------
       else
           nsflds = 17
           nrflds = 20 
       endif
   endif 
       
!--------------------wangln 2007-06-23 For MME--------------

   !--- allocate timer numbers ---
   call shr_timer_get(timer_send,'msg: cpl  -> atm')
   call shr_timer_get(timer_recv,'msg: cpl <-  atm')

   !--- allocate send & recv buffers ---
   allocate(buffr(n_a,nrflds))
   allocate(buffs(n_a,nsflds))

   allocate(area_fact_c2a(n_a))  ! area factment factor
   allocate(area_fact_a2c(n_a))  ! area factment factor
!------xjx added 20080510
   buffr = 0
   buffs = 0
   area_fact_c2a = 0
   area_fact_a2c = 0
!------xjx end

   !-----------------------------------------------------------------
   ! send initial msg
   !-----------------------------------------------------------------
   write(6,F01) 'send atm msg (initial)'

   !--- send integer info-buffer ---
   do i=1,nibuff
     ibuff_c(i) = 0
   end do
   ibuff_c( 1) = ierr              ! error code, 0 => no error
   ibuff_c( 2) = stop_eod          ! flag: stop at end of day
   ibuff_c( 3) = stop_now          ! flag: stop now
   ibuff_c( 4) = cdate             ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec               ! seconds
   ibuff_c(11) = info_time         ! flag: mp timing
   ibuff_c(12) = info_dbug         ! dbug diagnostic level
   ibuff_c(40) = SHR_MSG_A_MAJ_V04 ! cpl/atm interface: major version #
   ibuff_c(41) = SHR_MSG_A_MIN_V00 ! cpl/atm interface: minor version #
   ibuff_c(42) = 0                 ! size of character data to send
   if( carbon_flux ) ibuff_c(77) = 1 ! open the carbon flux switch
#ifdef MPI_P
   SHR_MSG_TID_ATM  = atm_tid
   if(master_task) &
   & call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_C2AI)
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_C2AI)
#endif

   !--- send floating point info-buffer ---
   do i=1,nrbuff
     rbuff_c(i) = spval
   end do
   rbuff_c(1)  = spval      ! Special value
   rbuff_c(2)  = orb_eccen  ! eccentricity
   rbuff_c(3)  = orb_obliqr ! obliquity
   rbuff_c(4)  = orb_lambm0 ! longitude of perihelion at v-equinox
   rbuff_c(5)  = orb_mvelpp ! moving vernal equinox + pi
   write(6,F01) 'send atm floating pt msg (initial)'
#ifdef MPI_P
   if(master_task) &
   &call shr_msg_send_r(rbuff_c, nrbuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_C2AI)
#else
   call shr_msg_send_r(rbuff_c, nrbuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_C2AI)
#endif

   !-----------------------------------------------------------------
   ! receive initial msg
   !-----------------------------------------------------------------
#ifdef MPI_P
   if(master_task) then
   write(6,F01) 'recv atm msg (initial)'
   call shr_msg_recv_i(ibuff_a, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(  xc_a , n_a       , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(  yc_a , n_a       , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(  xv_a , n_a*4     , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(  yv_a , n_a*4     , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(area_a , n_a       , SHR_MSG_TID_atm, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_i(mask_a , n_a       , SHR_MSG_TID_atm, SHR_MSG_TAG_A2CI)
   endif
   call MPI_BCAST(ibuff_a,size(ibuff_a),MPI_integer,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(xc_a,   size(xc_a),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yc_a,   size(yc_a),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(xv_a,   size(xv_a),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yv_a,   size(yv_a),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(area_a,size(area_a),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(mask_a,size(mask_a),  MPI_integer,0,MPI_COMM_CPL,ierr)
#else
   write(6,F01) 'recv atm msg (initial)'
   call shr_msg_recv_i(ibuff_a, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(  xc_a , n_a       , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(  yc_a , n_a       , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(  xv_a , n_a*4     , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(  yv_a , n_a*4     , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_r(area_a , n_a       , SHR_MSG_TID_atm, SHR_MSG_TAG_A2CI)
   call shr_msg_recv_i(mask_a , n_a       , SHR_MSG_TID_atm, SHR_MSG_TAG_A2CI)
#endif

   !--- extract info-buffer values ---
   if (ibuff_a(7) .ne. nx_a) stop 'nx'
   if (ibuff_a(8) .ne. ny_a) stop 'ny'
   ncpl_a     = ibuff_a( 9) ! number of msg pairs per day 
   ashift     = ibuff_a(32) ! shift albedo ahead by ashift seconds
   init_solar = ibuff_a(34) ! T => do optional atm initialization
   init_rica  = ibuff_a(35) ! T => use model's restart IC data

!  --- tcmods, needs more testing, using mask to pad out the map
   call map_domPad(domain_a,mask_a)
!  call map_domDup(domain_a,domain_l)  ! doesn't work, (domain_l !== domain_a)
   call map_domPad(domain_l,mask_a)
   call map_mapPad(map_a2of,mask_a,'src')
   call map_mapPad(map_a2os,mask_a,'src')
   call map_mapPad(map_o2af,mask_a,'dest')
   !--- already set, no changes needed ---
!   map_o2as => map_o2af
!   map_a2if => map_a2of
!   map_a2is => map_a2os
!   map_i2af => map_o2af 
!   map_i2as => map_o2af
!   call map_mapMakeID(map_a2lf)
!   call map_mapMakeID(map_a2ls)
!   call map_mapMakeID(map_l2af)
!   call map_mapMakeID(map_l2as)
!   --- tcmode

   !-----------------------------------------------------------------
   ! compute factors that account for inconsistent cell area values 
   !-----------------------------------------------------------------
   area_fact_c2a = 1.0
   area_fact_a2c = 1.0
   minf =  1.0e30
   maxf = -1.0e30
   do n=1,n_a
     if (mask_a(n) /= 0) then
       area_fact_c2a(n) = domain_a%area(n)/area_a(n)
       area_fact_a2c(n) = area_a(n)/domain_a%area(n)
       minf = min(minf,area_fact_c2a(n))
       maxf = max(maxf,area_fact_c2a(n))
     end if
   end do
   write(6,F02) 'min c2a area factor:',minf
   write(6,F02) 'max c2a area factor:',maxf
   write(6,F02) 'min a2c area factor:',min(minf,area_fact_a2c(n))
   write(6,F02) 'max a2c area factor:',max(maxf,area_fact_a2c(n))

END SUBROUTINE msg_atmi

!===============================================================================

SUBROUTINE msg_atmr (Sa,Faxa,Faia,Fala,Faoa)

   implicit none

#ifdef MPI_P
   integer  SHR_MSG_TID_ATM 
#endif
   !--- input ---
   real         :: Sa  (n_a,nSa  ) ! atm state
   real         :: Faxa(n_a,nFaxa) ! atm/srf fluxes from atm
   real         :: Faia(n_a,nFaia) ! atm/ice fluxes from atm
   real         :: Fala(n_a,nFala) ! atm/lnd fluxes from atm
   real         :: Faoa(n_a,nFaoa) ! atm/ocn fluxes from atm

   !--- local ---
   integer      :: n             ! generic loop indicies
   integer      :: m             ! generic loop index
   real         :: f             ! factor wrt area correction
   real         :: gsum          ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg 

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_atmr) ',a,i4)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   receive msg from atm model 
! INPUT:
!   o none
! OUTPUT:
!   o ibuff_a, Sa_a, Faia_a, Fala_a, Faoa_a
! MESSAGES:
!   o data recv'd: atm info-buffer, state variables, output fluxes
!-------------------------------------------------------------------------------
   if (info_dbug.ge.3 ) write(6,F01) 'recv atm msg'

   !--- receive msg ---
   irtc_w = shr_sys_irtc()
   call shr_timer_start(timer_recv)
#ifdef MPI_P
   SHR_MSG_TID_ATM  = atm_tid
   if(master_task) &
   &call shr_msg_recv_i(ibuff_a, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2C)
   call MPI_BCAST(ibuff_a,size(ibuff_a),  MPI_integer,0,MPI_COMM_CPL,ierr)
#else
   call shr_msg_recv_i(ibuff_a, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_A2C)
#endif

   irtc_r = shr_sys_irtc()

#ifdef MPI_P
   if(master_task) &
   &call shr_msg_recv_r( buffr ,size(buffr), SHR_MSG_TID_ATM, SHR_MSG_TAG_A2C)
   call MPI_BCAST(buffr,size(buffr),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
#else
   call shr_msg_recv_r( buffr ,size(buffr), SHR_MSG_TID_ATM, SHR_MSG_TAG_A2C)
#endif

   
!#ifdef MPI_P
!   DO n=IST_a,IEND_a
!#else
!$OMP PARALLEL DO PRIVATE(n,f)
   DO n=1,n_a
!#endif

!wtw      f = area_fact_a2c(n) ! wrt flux conservation & varying cell area calcs

      f = 1.0

      !--- atm states ---
      Sa  (n,kSa_z      ) = buffr(n, 1)   
      Sa  (n,kSa_u      ) = buffr(n, 2)   
      Sa  (n,kSa_v      ) = buffr(n, 3)   
      Sa  (n,kSa_tbot   ) = buffr(n, 4)   
      Sa  (n,kSa_ptem   ) = buffr(n, 5)   
      Sa  (n,kSa_pbot   ) = buffr(n, 6)   
      Sa  (n,kSa_pslv   ) = buffr(n, 7)   
      Sa  (n,kSa_shum   ) = buffr(n, 8)   
      Sa  (n,kSa_dens   ) = buffr(n, 9)   
      Sa  (n,kSa_p_surf ) = buffr(n, 20)   
      Sa  (n,kSa_precc  ) = buffr(n, 12)

      !--- output fluxes: atm/lnd ---
      Fala(n,kFala_lwdn ) = buffr(n,11) * f
      Fala(n,kFala_rainc) = buffr(n,12) * f
      Fala(n,kFala_rainl) = buffr(n,13) * f
      Fala(n,kFala_snowc) = buffr(n,14) * f
      Fala(n,kFala_snowl) = buffr(n,15) * f
      Fala(n,kFala_swndr) = buffr(n,16) * f
      Fala(n,kFala_swvdr) = buffr(n,17) * f
      Fala(n,kFala_swndf) = buffr(n,18) * f
      Fala(n,kFala_swvdf) = buffr(n,19) * f
      if( carbon_flux ) Fala(n,kFala_co2) = buffr(n,21) * f

      !--- output fluxes: atm/ice ---
      Faia(n,kFaia_lwdn ) = buffr(n,11) * f
      Faia(n,kFaia_rain ) =(buffr(n,12) + buffr(n,13)) * f
      Faia(n,kFaia_snow ) =(buffr(n,14) + buffr(n,15)) * f
      Faia(n,kFaia_swndr) = buffr(n,16) * f
      Faia(n,kFaia_swvdr) = buffr(n,17) * f
      Faia(n,kFaia_swndf) = buffr(n,18) * f
      Faia(n,kFaia_swvdf) = buffr(n,19) * f

      !--- output fluxes: atm/ocn ---
      Faoa(n,kFaoa_lwdn ) = buffr(n,11) * f 
      Faoa(n,kFaoa_rain ) =(buffr(n,12) + buffr(n,13)) * f
      Faoa(n,kFaoa_snow ) =(buffr(n,14) + buffr(n,15)) * f
      Faoa(n,kFaoa_swndr) = buffr(n,16) * f
      Faoa(n,kFaoa_swvdr) = buffr(n,17) * f
      Faoa(n,kFaoa_swndf) = buffr(n,18) * f
      Faoa(n,kFaoa_swvdf) = buffr(n,19) * f
      if( carbon_flux ) Faoa(n,kFaoa_co2) = buffr(n,21) * f

#ifdef HAMOCC
      Faoa(n,kFaoa_n2o) = buffr(n,22) * f
      Faoa(n,kFaoa_dms) = buffr(n,23) * f
#endif
      !--- output fluxes: atm/srf ---
      Faxa(n,kFaxa_swnet) = buffr(n,10) * f

   END DO
!$omp  end parallel do

   if (info_dbug >= 2) then
      do m=1,nrflds
         gsum = 0.
         do n=1,n_a
            if (mask_a(n) /= 0) gsum = gsum + buffr(n,m)*area_a(n)*rEarth*rEarth
         enddo
         write(6,F98)'atm','send',m,gsum,'in_cpl'
      enddo
   endif

   irtc_u = shr_sys_irtc()
   call shr_timer_stop(timer_recv)

   !--- extract info-buffer values ---
   ashift=ibuff_a(32)  ! shift albedo ahead by ashift seconds

   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_w,'a->c waiting'
      write(6,F99) irtc_r,'a->c received'
      write(6,F99) irtc_u,'a->c unpacked'
   end if

END SUBROUTINE msg_atmr

!===============================================================================

SUBROUTINE msg_atms (Faxx,Ss)

#ifdef MPI_P
   integer  SHR_MSG_TID_ATM 
   real gsum0
#endif
   !--- arguments ---
   real,intent(in) :: Faxx(n_a,nFaxx)  ! atm input fluxes
   real,intent(in) :: Ss  (n_a,nSs  )  ! merged surface states

   !--- local ---
   integer      :: n                ! generic loop index
   integer      :: m                ! generic loop index
   real         :: f                ! factor wrt area correction
   real         :: gsum             ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg 
 
   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_atms) ',a,i4)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   send msg to atm model 
! INPUT:
!   o Faxx_a, Ss_a & various flags contained in control.h
!   o fraction of ice & ocn for each atm grid cell (merge weights
!     found in map.h)
! OUTPUT:
!   o none
! MESSAGES:
!   o data sent: ibuff_d, Faxx_a, Ss, fraction of ice and ocn
!-------------------------------------------------------------------------------
   
   if (info_dbug.ge.3) write(6,F01) 'send atm msg'

   !--- fill info-buffer array ---
   do n=1,nibuff
     ibuff_c(n) = 0
   end do
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = stop_now  ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now

   !--- pack msg buffer ---
   irtc_p = shr_sys_irtc()
   call shr_timer_start(timer_send)

!$OMP PARALLEL DO PRIVATE(n,f)
   DO n=1,n_a

!fangyj      f = area_fact_c2a(n) ! wrt flux conservation & varying cell area calcs

      f = 1.0

      !--- atm/srf fluxes ---
      buffs(n, 1) = Faxx(n,kFaxx_taux ) * f
      buffs(n, 2) = Faxx(n,kFaxx_tauy ) * f
      buffs(n, 3) = Faxx(n,kFaxx_lat  ) * f
      buffs(n, 4) = Faxx(n,kFaxx_sen  ) * f
      buffs(n, 5) = Faxx(n,kFaxx_lwup ) * f
      buffs(n, 6) = Faxx(n,kFaxx_evap ) * f

      !--- merged srf states ---
      buffs(n, 7) = Ss  (n,kSs_avsdr  ) 
      buffs(n, 8) = Ss  (n,kSs_anidr  ) 
      buffs(n, 9) = Ss  (n,kSs_avsdf  ) 
      buffs(n,10) = Ss  (n,kSs_anidf  ) 
      buffs(n,11) = Ss  (n,kSs_t      ) 
      buffs(n,12) = Ss  (n,kSs_snowh  ) 

      !--- fraction of ice & ocn ---
      buffs(n,13) = fi_a(n) 
      buffs(n,14) = fo_a(n) 
      !!buffs(n,14) = 1. - fi_a(n) - fl_a(n) 


      !--- merged 2m reference temperature ---
      buffs(n,15) = Faxx (n,kFaxx_tref )

!--------------------wangln 2007-06-23 For MME--------------
      if( atm_type(1:4) == 'CAM3' .or. atm_type(1:3) == 'BCC') then 
         buffs(n,16) = Faxx (n,kFaxx_qref )
         buffs(n,17) = Ss (n,kSst_t )
         if( carbon_flux ) buffs(n,18) = Faxx (n,kFaxx_carbon )

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
         if( carbon_flux ) then
            buffs(n, 19) = Ss  (n,kSs_u10m )
            buffs(n, 20) = Ss  (n,kSs_v10m )
            buffs(n, 21) = Ss  (n,kSs_tocn )
            buffs(n, 22) = Ss  (n,kSs_tice )
            buffs(n, 23) = Ss  (n,kSs_tlnd )
         endif

#ifdef MOZART2
!---
! wtw added 14 variables for MOZART2
         buffs(n,24) = Faxx (n,kFaxx_mss_dst01 )
         buffs(n,25) = Faxx (n,kFaxx_mss_dst02 )
         buffs(n,26) = Faxx (n,kFaxx_mss_dst03 )
         buffs(n,27) = Faxx (n,kFaxx_mss_dst04 )
         buffs(n,28) = Faxx (n,kFaxx_vlc_dst01 )
         buffs(n,29) = Faxx (n,kFaxx_vlc_dst02 )
         buffs(n,30) = Faxx (n,kFaxx_vlc_dst03 )
         buffs(n,31) = Faxx (n,kFaxx_vlc_dst04 )
         buffs(n,32) = Faxx (n,kFaxx_lnd_ram1 )
         buffs(n,33) = Faxx (n,kFaxx_lnd_fv )
         buffs(n,34) = Faxx (n,kFaxx_vlc_SO4 )
         buffs(n,35) = Faxx (n,kFaxx_vlc_NH4 )
         buffs(n,36) = Faxx (n,kFaxx_vlc_CB2 )
         buffs(n,37) = Faxx (n,kFaxx_vlc_SO2 )

         buffs(n,38) = Faxx (n,kFaxx_flux_ISOP )
         buffs(n,39) = Faxx (n,kFaxx_flux_ACET )
         buffs(n,40) = Faxx (n,kFaxx_flux_C3H6 )
         buffs(n,41) = Faxx (n,kFaxx_flux_C2H4 )
         buffs(n,42) = Faxx (n,kFaxx_flux_OC1 )
         buffs(n,43) = Faxx (n,kFaxx_flux_C10H16 )
#ifdef HAMOCC
        buffs(n,44) = Faxx (n,kFaxx_flux_co2 )  ! wufh from HAMOCC CO2 flux (upward +)
        buffs(n,45) = Faxx (n,kFaxx_flux_n2o )  ! wufh from HAMOCC N2O flux (upward +)
        buffs(n,46) = Faxx (n,kFaxx_flux_dms )  ! wufh from HAMOCC DMS flux (upward +)
#endif
!------------------
#else

#ifdef HAMOCC
        buffs(n,24) = Faxx (n,kFaxx_flux_co2 )  ! wufh from HAMOCC CO2 flux (upward +)
        buffs(n,25) = Faxx (n,kFaxx_flux_n2o )  ! wufh from HAMOCC N2O flux (upward +)
        buffs(n,26) = Faxx (n,kFaxx_flux_dms )  ! wufh from HAMOCC DMS flux (upward +)
#endif
!----
#endif 
      endif
!--------------------wangln 2007-06-23 For MME--------------
   end do
!$omp  end parallel do
   !!write(77) fi_a
   !!write(77) fo_a
   !!close(77)

   if (info_dbug >= 2) then
      do m=1,nsflds
         gsum = 0.
         do n=1,n_a
            if (mask_a(n) /= 0) gsum = gsum + buffs(n,m)*area_a(n)*rEarth*rEarth
         enddo
         write(6,F98)'atm','recv',m,gsum,'in_cpl'
      enddo
   endif

   !--- send msg ---
   irtc_s = shr_sys_irtc()
#ifdef MPI_P
   SHR_MSG_TID_ATM  = atm_tid
   if(master_task) then
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_C2A)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_ATM, SHR_MSG_TAG_C2A)
   endif
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_C2A)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_ATM, SHR_MSG_TAG_C2A)
#endif
   call shr_timer_stop(timer_send)

   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_p,'c->a packing'
      write(6,F99) irtc_s,'c->a sending'
   end if

END SUBROUTINE msg_atms

!=======================================================================

SUBROUTINE msg_atmf ()

   implicit none

#ifdef MPI_P
   integer  SHR_MSG_TID_ATM 
#endif
   !--- local ---
   integer      :: n         ! loop index

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_atmf) ',a,i4)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   send final msg to atm model 
! INPUT:
!   various flags contained in control.h
! OUTPUT:
!   none
! MESSAGES:
!   data sent: coupler info-buffer
!-------------------------------------------------------------------------------
   
   write(6,F01) 'send atm msg (final)'

   !--- set info-buffer values ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = 1         ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now

   !--- pack msg buffer (with special-values) ---
   buffs=spval

#ifdef MPI_P
   SHR_MSG_TID_ATM  = atm_tid
   !--- send msg ---
   if(master_task) then
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_C2A)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_ATM, SHR_MSG_TAG_C2A)
   endif
#else
   !--- send msg ---
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ATM, SHR_MSG_TAG_C2A)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_ATM, SHR_MSG_TAG_C2A)
#endif

END SUBROUTINE msg_atmf

!===============================================================================

END MODULE msg_atm_mod
