#include <misc.h>

MODULE msg_ice_mod

   use control_mod      ! control data
   use buf_mod          ! model   data
   use const_mod        ! physical constants
   use map_mod          ! map data
   use shr_msg_mod      ! msg constants, data, routines
   use shr_timer_mod    ! timing routines
#ifdef MPI_P
   use cpl_mod, only:ice_tid
!   use mpmd_mod, only: master_task,MPI_COMM_CPL
#endif

   implicit none

   private  ! resticted access
   public  :: msg_icei,msg_icer,msg_ices,msg_icef

!-----------------wangln 2007-06-24 For MME---------------------------
   !integer,parameter   :: nsflds=21        ! number of fields sent
   !integer,parameter   :: nrflds=21        ! number of fields recv'd
   integer   :: nsflds        ! number of fields sent
   integer   :: nrflds        ! number of fields recv'd
!-----------------wangln 2007-06-24 For MME---------------------------

   real   ,allocatable :: buffs(:,:)       ! msg buffer: send
   real   ,allocatable :: buffr(:,:)       ! msg buffer: recv
   integer,allocatable :: msg_mask_i(:)    ! mask for ice message packing
   real   ,allocatable :: area_fact_c2i(:) ! cpl->model area correction factor
   real   ,allocatable :: area_fact_i2c(:) ! model->cpl area correction factor

   !--- aggregate timing info ---
   integer      :: timer_send    ! shr_timer timer number for data send
   integer      :: timer_recv    ! shr_timer timer number for data recv
   integer      :: t1,t2,t3,t4,t5,t6,t7,t8,t9,t10

   !--- detailed timing info ---
!  integer(8)   :: irtc_w        ! rtc tics when waiting for msg
!  integer(8)   :: irtc_r        ! rtc tics when msg received
!  integer(8)   :: irtc_u        ! rtc tics when msg unpacked
!  integer(8)   :: irtc_p        ! rtc tics when packing msg
!  integer(8)   :: irtc_s        ! rtc tics when sending msg 

   !--- formats ---
   character(len=*),parameter :: F98 = "('comm_diag ',a3,1x,a4,1x,i3,es26.19,1x,a)"
   character(len=*),parameter :: F99 = "('[mp timing]  irtc = ',i20,' ',a)"

   save

CONTAINS

!===============================================================================

SUBROUTINE msg_icei ()

   implicit none

   !--- local ---
   integer :: n             ! generic loop indecies
   real    :: minf,maxf     ! min/max cell area correction factors

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_icei) ',a,i4)"
   character(len=*),parameter :: F02 = "('(msg_icei) ',a,2f12.9)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   send/recv initial msg to/from ice 
! INPUT:
!   o flags contained in control.h
! OUTPUT:
!   o ice info-buffer data         : ibuff_i
!   o ice spatial coordinate arrays: xv_i,xc_i,yv_i,yc_i
!   o ice model domain mask        : mask_i
! MESSAGES:
!   o data  sent : cpl info-buffer flags
!   o data recv'd: ice info-buffer flags, grids, domain mask
!-------------------------------------------------------------------------------
#ifdef MPI_P
   integer  SHR_MSG_TID_ICE
#endif
 

   if( ocn_type(1:3) == 'MOM' ) then
       nsflds = 25        ! number of fields recv'd

#ifdef HAMOCC
       nrflds = 27        ! number of fields recv'd !wufh add HAMOCC sictho, sicsno
#else
       nrflds = 25        ! number of fields recv'd
#endif

   else if ( ocn_type(1:3) == 'POP' ) then
       nsflds = 21        ! number of fields sent
       if( atm_type(1:4) == 'CAM2' ) then 
           nrflds = 21        ! number of fields recv'd
       else if ( atm_type(1:4) == 'CAM3' .or. atm_type(1:3) == 'BCC') then
           nrflds = 22        ! number of fields recv'd
       endif
   endif
              

   !--- allocate timer numbers ---
   call shr_timer_get(timer_send,'msg: cpl  -> ice')
   call shr_timer_get(timer_recv,'msg: cpl <-  ice')
   call shr_timer_get(t1 ,'msg: cpl  -> ice t1, pack rbuf')
   call shr_timer_get(t2 ,'msg: cpl  -> ice t2, send ibuf')
   call shr_timer_get(t3 ,'msg: cpl  -> ice t3, send rbuf')
   call shr_timer_get(t4 ,'msg: cpl  -> ice t4')
   call shr_timer_get(t5 ,'msg: cpl  -> ice t5')
   call shr_timer_get(t6 ,'msg: cpl <-  ice t6, recv ibuf')
   call shr_timer_get(t7 ,'msg: cpl <-  ice t7, recv rbuf')
   call shr_timer_get(t8 ,'msg: cpl <-  ice t8, upck rbuf')
   call shr_timer_get(t9 ,'msg: cpl <-  ice t9')
   call shr_timer_get(t10,'msg: cpl <-  ice t10')


   !--- allocate local buffers/arrays ---
!!=======================wangln 2007-08-24=======================
   if( ocn_type(1:3) == 'MOM' ) then
       allocate(buffr(n_i, nrflds))   ! recv buffer
   else if ( ocn_type(1:3) == 'POP' ) then
       allocate(buffr(nrflds, n_i))   ! recv buffer
   endif
!!=======================wangln 2007-08-24=======================

   allocate(buffs(n_i,nsflds))   ! send buffer
   allocate(msg_mask_i   (n_i))  ! mask for ice messages and packing
   allocate(area_fact_c2i(n_i))  ! area factment factor
   allocate(area_fact_i2c(n_i))  ! area factment factor
!xjx added 20080510
        buffr = 0.
        buffs = 0.
        msg_mask_i = 0
        area_fact_c2i = 0.
        area_fact_i2c = 0.
!xjx end

   !-----------------------------------------------------------------
   ! send initial msg
   !-----------------------------------------------------------------
   ibuff_c     = 0
   ibuff_c( 1) = ierr              ! error code, 0 => no error
   ibuff_c( 2) = stop_eod          ! flag: stop at end of day
   ibuff_c( 3) = stop_now          ! flag: stop now
   ibuff_c( 4) = cdate             ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec               ! seconds
   ibuff_c(11) = info_time         ! flag: mp timing
   ibuff_c(12) = info_dbug         ! dbug diagnostic level
   ibuff_c(40) = SHR_MSG_I_MAJ_V04 ! cpl/ice interface: major version #
   ibuff_c(41) = SHR_MSG_I_MIN_V00 ! cpl/ice interface: minor version #

   write(6,F01) 'send ice msg (initial)'
#ifdef MPI_P
   SHR_MSG_TID_ICE  = ice_tid
   if(master_task) &
   &call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_C2II)
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_C2II)
#endif
   
   !-----------------------------------------------------------------
   ! receive initial msg
   !-----------------------------------------------------------------
#ifdef MPI_P
   if(master_task) then
   write(6,F01) 'recv ice msg (initial)'
   call shr_msg_recv_i(ibuff_i, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(  xc_i ,    n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(  yc_i ,    n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(  xv_i ,  4*n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(  yv_i ,  4*n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(area_i ,    n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_i(mask_i ,    n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   endif
   call MPI_BCAST(ibuff_i,size(ibuff_i),MPI_integer,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(xc_i,   size(xc_i),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yc_i,   size(yc_i),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(xv_i,   size(xv_i),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yv_i,   size(yv_i),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(area_i,size(area_i),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(mask_i,size(mask_i),  MPI_integer,0,MPI_COMM_CPL,ierr)
#else
   write(6,F01) 'recv ice msg (initial)'
   call shr_msg_recv_i(ibuff_i, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(  xc_i ,    n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(  yc_i ,    n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(  xv_i ,  4*n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(  yv_i ,  4*n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_r(area_i ,    n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
   call shr_msg_recv_i(mask_i ,    n_i    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2CI)
#endif

  
   !--- extract info-buffer values ---
   if (ibuff_i(7) .ne. nx_i) stop 'nx'
   if (ibuff_i(8) .ne. ny_i) stop 'ny'
   ncpl_i    = ibuff_i( 9) ! number of msg pairs per day 
   init_rici = ibuff_i(35) ! T => use model's restart IC data

   !-----------------------------------------------------------------
   ! compute factors that account for inconsistent cell area values 
   !-----------------------------------------------------------------
   area_fact_c2i = 1.0
   area_fact_i2c = 1.0
   minf =  1.0e30
   maxf = -1.0e30
   do n=1,n_i
     if (mask_i(n) /= 0) then
       !!write(6,*) ' Ice & CPL Area=', area_i(n), domain_i%area(n)
       area_fact_c2i(n) = domain_i%area(n)/area_i(n)
       area_fact_i2c(n) = area_i(n)/domain_i%area(n)
       minf = min(minf,area_fact_c2i(n))
       maxf = max(maxf,area_fact_c2i(n))
     end if
   end do
   write(6,F02) 'min c2i area factor:',minf
   write(6,F02) 'max c2i area factor:',maxf
   !!open(93, file='Frac.dat')

END SUBROUTINE msg_icei

!===============================================================================
SUBROUTINE msg_icer (Si,Fioi,Faii)

   implicit none

   !--- output ---
   real         :: Si  (n_i,nSi  ) ! ice state
   real         :: Fioi(n_i,nFioi) ! ice/ocn fluxes from ice
   real         :: Faii(n_i,nFaii) ! ice/atm fluxes from ice

   !--- local variables ---
   integer      :: n             ! loop index
   integer      :: m             ! generic loop index
   integer      :: k,ki,ni       ! array indexes
   integer      :: nrecv         ! number of elements to recv
   integer      :: trecv         ! total number of words to recv
   real         :: f             ! factor wrt area correction
   real         :: gsum(nrflds)  ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg

   logical      :: ice_mask(n_i)  

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_icer) ',a,i4)"

#ifdef MPI_P
   integer  SHR_MSG_TID_ICE
#endif
!-------------------------------------------------------------------------------
! PURPOSE:
!   receive msg from ice model 
! INPUT:
!   o none
! OUTPUT:
!   o ibuff_i, Si_i, Fioi_i
! MESSAGES:
!   o data recv'd: ice info-buffer, state variables, output fluxes
!-------------------------------------------------------------------------------
   
   if (info_dbug.ge.3) write(6,F01) 'recv ice msg'

   !--- receive msg ---
   irtc_w = shr_sys_irtc()
   call shr_timer_start(timer_recv)
   call shr_timer_start(t6)

#ifdef MPI_P
   SHR_MSG_TID_ICE  = ice_tid
   if(master_task) &
   &call shr_msg_recv_i(ibuff_i, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2C)
   call MPI_BCAST(ibuff_i,size(ibuff_i),  MPI_integer,0,MPI_COMM_CPL,ierr)
#else
   call shr_msg_recv_i(ibuff_i, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2C)
#endif
   call shr_timer_stop(t6)

   call shr_timer_start(t7)
   nrecv = ibuff_i(10)   ! size of compressed data, one field
   trecv = nrecv*nrflds  ! size of compressed data, *all* fields

#ifdef MPI_P
   if(master_task) &
   &call shr_msg_recv_r(buffr  ,trecv      , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2C)
    call MPI_BCAST(buffr,size(buffr),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
#else
   call shr_msg_recv_r(buffr  ,trecv      , SHR_MSG_TID_ICE, SHR_MSG_TAG_I2C)
#endif
   
   call shr_timer_stop(t7)
   irtc_r = shr_sys_irtc()

   call shr_timer_start(t8)

   call shr_timer_start(t10)
   Si(:,kSi_ifrac)=0.  ! ice frac is zero wherever data is not sent
   msg_mask_i(:) = 0   ! mask     is zero wherever data is not sent
   call shr_timer_stop(t10)

!!!$OMP PARALLEL DO PRIVATE(n,k,ki,ni)
 
 if( ocn_type(1:3) == 'POP' ) then
!$OMP PARALLEL DO PRIVATE(n,k,ki,ni)
   do n=1,nrecv
      ni =             nint( buffr(21,n) ) ! index in uncompressed vector
      msg_mask_i(ni) = 1
      Si  (ni,kSi_ifrac)   = buffr( 1,n)
      Si  (ni,kSi_t    )   = buffr( 2,n)
      Si  (ni,kSi_avsdr)   = buffr( 3,n)
      Si  (ni,kSi_anidr)   = buffr( 4,n)
      Si  (ni,kSi_avsdf)   = buffr( 5,n)
      Si  (ni,kSi_anidf)   = buffr( 6,n)
      Faii(ni,kFaii_taux)  = buffr( 7,n)*area_fact_i2c(ni)
      Faii(ni,kFaii_tauy)  = buffr( 8,n)*area_fact_i2c(ni)
      Faii(ni,kFaii_lat)   = buffr( 9,n)*area_fact_i2c(ni)
      Faii(ni,kFaii_sen)   = buffr(10,n)*area_fact_i2c(ni)
      Faii(ni,kFaii_lwup)  = buffr(11,n)*area_fact_i2c(ni)
      Faii(ni,kFaii_evap)  = buffr(12,n)*area_fact_i2c(ni)
      Faii(ni,kFaii_tref)  = buffr(13,n)*area_fact_i2c(ni)
      Fioi(ni,kFioi_swnet) = buffr(14,n)*area_fact_i2c(ni)
      Fioi(ni,kFioi_melth) = buffr(15,n)*area_fact_i2c(ni)
      Fioi(ni,kFioi_meltw) = buffr(16,n)*area_fact_i2c(ni)
      Fioi(ni,kFioi_salt)  = buffr(17,n)*area_fact_i2c(ni)
      Fioi(ni,kFioi_taux)  = buffr(18,n)*area_fact_i2c(ni)
      Fioi(ni,kFioi_tauy)  = buffr(19,n)*area_fact_i2c(ni)
      Faii(ni,kFaii_swnet) = buffr(20,n)*area_fact_i2c(ni)

!--------------wangln 2007-06-24 For MME ----------------------
!xjx--20081225      if( atm_type(1:4) == 'CAM3' .or. atm_type(1:3) == 'BCC' ) then
          Faii(ni,kFaii_qref) = buffr(22,n)*area_fact_i2c(ni)
!xjx--20081225      endif
!--------------wangln 2007-06-24 For MME ----------------------

   enddo
 else if( ocn_type(1:3) == 'MOM' ) then
 ice_mask = .false.
!#ifdef MPI_P
!    do n = IST_i, IEND_i
!#else
!$OMP PARALLEL DO PRIVATE(n,f)
    do n = 1, n_i
!#endif
      f = area_fact_i2c( n )

      Si  (n,kSi_ifrac)     = buffr(n, 1) 
      Si  (n,kSi_t    )     = buffr(n, 2)
      if( Si  (n,kSi_ifrac) > 0.0 ) then
         if(Si(n,kSi_t) > 274) write(6,*) 'T>274, & T IFrac=', Si(n,kSi_ifrac), Si(n,kSi_t) 
         if(Si(n,kSi_t) < 100) write(6,*) 'T<100, & T IFrac=', Si(n,kSi_ifrac), Si(n,kSi_t) 
      endif
         
      Si  (n,kSi_avsdr)     = buffr(n, 3)
      Si  (n,kSi_anidr)     = buffr(n, 4)
      Si  (n,kSi_avsdf)     = buffr(n, 5)
      Si  (n,kSi_anidf)     = buffr(n, 6)
      Faii(n,kFaii_taux)    = buffr(n, 7) * f
      Faii(n,kFaii_tauy)    = buffr(n, 8) * f
      Faii(n,kFaii_lat)     = buffr(n, 9) * f 
      Faii(n,kFaii_sen)     = buffr(n,10) * f  
      Faii(n,kFaii_lwup)    = buffr(n,11) * f 
      Faii(n,kFaii_evap)    = buffr(n,12) * f 
      Faii(n,kFaii_swnet )  = buffr(n,13) * f 
      Faii(n,kFaii_tref  )  = buffr(n,14)  
      Faii(n,kFaii_qref  )  = buffr(n,15) 
      Fioi(n,kFioi_meltw)   = buffr(n,16) * f 
      Fioi(n,kFioi_evap)    = buffr(n,17) * f 
      Fioi(n,kFioi_sen)     = buffr(n,18) * f 
      Fioi(n,kFioi_swnet)   = buffr(n,19) * f 
      Fioi(n,kFioi_salt)    = buffr(n,20) * f
      Fioi(n,kFioi_taux)    = buffr(n,21) * f
      Fioi(n,kFioi_tauy)    = buffr(n,22) * f
      Fioi(n,kFioi_psurf)   = buffr(n,23) * f

      Si  (n,kSi_u10m)     = buffr(n, 24)
      Si  (n,kSi_v10m)     = buffr(n, 25)

#ifdef HAMOCC
      Si  (n,kSi_sictho)  = buffr(n, 26)
      Si  (n,kSi_sicsno)  = buffr(n, 27)
#endif

    enddo
!$omp  end parallel do

 endif
  

   call shr_timer_stop(t8)
   irtc_u = shr_sys_irtc()
   call shr_timer_stop(timer_recv)

   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_w,'i->c waiting'
      write(6,F99) irtc_r,'i->c received'
      write(6,F99) irtc_u,'i->c unpacked'
   end if

   !------------------------------------------------------------------
   ! global sum test: compare with equivalent calc in ice model 
   !------------------------------------------------------------------

 if( ocn_type(1:3) == 'POP' ) then  
   if (info_dbug >= 2) then
      gsum = 0.
      do n=1,nrecv
         ni = nint(buffr(21,n))
         do m=1,nrflds
            if (mask_i(ni) /= 0) then
               gsum(m) = gsum(m) + buffr(m,n)*area_i(ni)*rEarth*rEarth
            end if
         enddo
      enddo
      do m=1,nrflds
        write(6,F98)'ice','send',m,gsum(m),'in_cpl'
      enddo
   endif
 endif  ! ENDIF OCN_TYPE


END SUBROUTINE msg_icer

!===============================================================================

SUBROUTINE msg_ices (Sa, So, Faia, Fioo)

   implicit none

   !--- input ---
   real         :: Sa  (n_i,nSa  ) ! atm state variables
   real         :: So  (n_i,nSo  ) ! ocn state variables
   real         :: Faia(n_i,nFaia) ! atm/ice fluxes
   real         :: Fioo(n_i,nFioo) ! ocn/ice fluxes

   !--- local ---
   integer      :: m,n           ! generic loop index
   integer      :: ns,nn,nc      ! compression indicies
   integer,save :: neq = 0       ! equitorial index
   integer      :: k,ki          ! array indexes
   real         :: f             ! factor wrt area correction
   real         :: gsum          ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_ices) ',a,i4)"

#ifdef MPI_P
   integer  SHR_MSG_TID_ICE
#endif
!-------------------------------------------------------------------------------
! PURPOSE:
!   send msg to ice model 
! INPUT:
!   o Sa     atm state variables
!   o So     ocn state variables
!   o Faia_i fluxes at a/i interface
!   o Fioo_i fluxes at o/i interface
! OUTPUT:
!   o none
! MESSAGES:
!   o data sent: coupler info-buffer, ice input fluxes
!-------------------------------------------------------------------------------
   
   if (info_dbug.ge.3) write(6,F01) 'send ice msg'
   call shr_timer_start(timer_send)
    
   !--------------------------------------------------------------
   ! compression: compute indicies
   !--------------------------------------------------------------
   if (neq < 1) then
      do n=1,n_i
         if (yc_i(n) >= 0.0) then
            neq = n
            exit
         endif
      end do
   end if

   do nn = neq,n_i
      if (fi_i(nn)         > 0.0) exit 
      if (Fioo(nn,kFioo_q) > 0.0) exit
      if (msg_mask_i(nn)  /= 0  ) exit
   end do

   do ns = neq,1,-1
      if (fi_i(ns)         > 0.0) exit 
      if (Fioo(ns,kFioo_q) > 0.0) exit
      if (msg_mask_i(ns)  /= 0  ) exit
   end do

   ns = min(ns+5*nx_i,neq)
   nn = max(nn-5*nx_i,neq+1)

   nc=0
   do n=1,n_i
      if (n<=ns .or. n>=nn) nc=nc+1 ! index in compressed grid
   end do

!-------------wangln 2007-06-24-------------------------
   !deallocate(buffs)
   !allocate(buffs(nc,nsflds))   ! send buffer
!-------------wangln 2007-06-24-------------------------

   !--------------------------------------------------------------
   ! pack and send the data
   !--------------------------------------------------------------

   !--- fill info-buffer array ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = stop_now  ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(13) = nc        ! number of points excluding (ns+1:nn-1)
   ibuff_c(14) = ns        ! data removed from (ns+1:nn-1)
   ibuff_c(15) = nn        ! data removed from (ns+1:nn-1)
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now
   ibuff_c(32) = ashift    ! albedo time shift

   !--- pack compressed msg buffer ---
   irtc_p = shr_sys_irtc()
   call shr_timer_start(t1)


!---------------------wangln 2007-06-20 For SIS--------------------
 if( ocn_type(1:3) == 'POP' ) then

!-------------wangln 2007-06-24-------------------------
   deallocate(buffs)
   allocate(buffs(nc,nsflds))   ! send buffer
!xjx added 20080510
    buffs = 0.
!xjx end
!-------------wangln 2007-06-24-------------------------
!!$OMP PARALLEL DO PRIVATE(n,k,ki)
   do k=1,nsflds
      if (k ==  1) ki=kSo_t
      if (k ==  2) ki=kSo_s
      if (k ==  3) ki=kSo_u
      if (k ==  4) ki=kSo_v
      if (k ==  5) ki=kSa_z
      if (k ==  6) ki=kSa_u
      if (k ==  7) ki=kSa_v
      if (k ==  8) ki=kSa_ptem
      if (k ==  9) ki=kSa_tbot
      if (k == 10) ki=kSa_shum
      if (k == 11) ki=kSa_dens
      if (k == 12) ki=kFioo_dhdx
      if (k == 13) ki=kFioo_dhdy
      if (k == 14) ki=kFioo_q
      if (k == 15) ki=kFaia_swndr
      if (k == 16) ki=kFaia_swvdr
      if (k == 17) ki=kFaia_swndf
      if (k == 18) ki=kFaia_swvdf
      if (k == 19) ki=kFaia_lwdn
      if (k == 20) ki=kFaia_rain
      if (k == 21) ki=kFaia_snow
      if (k >= 15) then
            buffs(   1:ns,k) = Faia( 1:ns ,ki)*area_fact_c2i( 1:ns )
            buffs(ns+1:nc,k) = Faia(nn:n_i,ki)*area_fact_c2i(nn:n_i)
      elseif (k >= 14) then
            buffs(   1:ns,k) = Fioo( 1:ns ,ki)*area_fact_c2i( 1:ns )
            buffs(ns+1:nc,k) = Fioo(nn:n_i,ki)*area_fact_c2i(nn:n_i)
      elseif (k >= 12) then
            buffs(   1:ns,k) = Fioo( 1:ns ,ki)
            buffs(ns+1:nc,k) = Fioo(nn:n_i,ki)
      elseif (k >= 5) then
            buffs(   1:ns,k) = Sa( 1:ns ,ki)
            buffs(ns+1:nc,k) = Sa(nn:n_i,ki)
      else
            buffs(   1:ns,k) = So( 1:ns ,ki)
            buffs(ns+1:nc,k) = So(nn:n_i,ki)
      endif
   enddo

!---------------------wangln 2007-06-20 For SIS--------------------   
  else if( ocn_type(1:3) == 'MOM' ) then
!$OMP PARALLEL DO PRIVATE(n,f)
    do n = 1, n_i

       f = area_fact_c2i(n)

  !---------------For ocean_ice_boundary-----------------------------
       buffs( n, 1 ) =   So( n, kSo_t )
       buffs( n, 2 ) =   So( n, kSo_s )
       buffs( n, 3 ) =   So( n, kSo_u )
       buffs( n, 4 ) =   So( n, kSo_v )
       buffs( n, 5 ) = Fioo( n, KFioo_dhdx ) !For MOM dhdx=dhdy=sea_level
       buffs( n, 6 ) = Fioo( n, KFioo_q )   

  !--------------For atmos_ice_boundary---------------------------------
       buffs( n, 7 ) = Sa( n, kSa_z )   
       buffs( n, 8 ) = Sa( n, kSa_u )   
       buffs( n, 9 ) = Sa( n, kSa_v )   
       buffs( n,10 ) = Sa( n, kSa_ptem )   
       buffs( n,11 ) = Sa( n, kSa_tbot )   
       buffs( n,12 ) = Sa( n, kSa_shum )   
       buffs( n,13 ) = Sa( n, kSa_dens )   
       buffs( n,14 ) = Sa( n, kSa_pbot )   
       buffs( n,15 ) = Faia( n, kFaia_swndr)  * f 
       buffs( n,16 ) = Faia( n, kFaia_swvdr)  * f 
       buffs( n,17 ) = Faia( n, kFaia_swndf)  * f 
       buffs( n,18 ) = Faia( n, kFaia_swvdf)  * f 
       buffs( n,19 ) = Faia( n, kFaia_rain )  * f 
       buffs( n,20 ) = Faia( n, kFaia_snow )  * f 
       buffs( n,21 ) = Faia( n, kFaia_lwdn )  * f 
       buffs( n,22 ) = Sa( n, kSa_pslv   )   
        
  !--------------For land_ice_boundary---------------------------------
       buffs( n,23 ) = roff_o( n, 1 ) * f
       buffs( n,24 ) = 0.0
       buffs( n,25 ) = Sa( n, kSa_p_surf )
          
    enddo 
!$omp  end parallel do
            
   
  endif   ! ENDIF the OCN type

   call shr_timer_stop(t1)

   !--- send msg ---
   irtc_s = shr_sys_irtc()
   call shr_timer_start(t2)
#ifdef MPI_P
   SHR_MSG_TID_ICE  = ice_tid
   if(master_task) &
   &call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_C2I)
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_C2I)
#endif
   call shr_timer_stop(t2)
   call shr_timer_start(t3)
#ifdef MPI_P
   if(master_task) &
   &call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_ICE, SHR_MSG_TAG_C2I)
#else
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_ICE, SHR_MSG_TAG_C2I)
#endif
   call shr_timer_stop(t3)
   call shr_timer_stop(timer_send)
  
   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_p,'c->i packing'
      write(6,F99) irtc_s,'c->i sending'
   end if

   !------------------------------------------------------------------
   ! global sum test: compare with equivalent calc in ice model 
   !------------------------------------------------------------------
 
if( ocn_type(1:3) == 'POP' ) then
   if (info_dbug >= 2) then
      do m=1,nsflds
          gsum = 0.
          do n=1,ns !--- southern hemisphere ---
             if (mask_i(n) /= 0) then
                gsum = gsum + buffs(n,m)*area_i(n        )*rEarth*rEarth
             endif
          enddo
          do n=ns+1,nc !--- northern hemisphere ---
             if (mask_i(n+nn-ns-1) /= 0) then
                gsum = gsum + buffs(n,m)*area_i(n+nn-ns-1)*rEarth*rEarth
             endif
          enddo
          write(6,F98)'ice','recv',m,gsum,'in_cpl'
      enddo
   endif
 endif ! ENDIF OCN_TYPE

END SUBROUTINE msg_ices

!===============================================================================

SUBROUTINE msg_icef ( )

   implicit none

   !--- local ---
   integer      :: nc,ns,nn  ! compression info

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_icef) ',a,i4)"

#ifdef MPI_P
   integer  SHR_MSG_TID_ICE
#endif
!-------------------------------------------------------------------------------
! PURPOSE:
!   send final msg to ice model 
! INPUT:
!   o various flags contained in control.h 
! OUTPUT:
!   o none
! MESSAGES:
!   o data sent: coupler info-buffer
!-------------------------------------------------------------------------------
   
   write(6,F01) 'send ice msg (final)'

   !--- compression: compute indicies (no meaningful data is sent) ---
   nc=100
   ns =50
   nn =n_i-50+1
   deallocate(buffs)
   allocate(buffs(nc,nsflds))
!xjx added 20080510
    buffs = 0.
!xjx end
   
   !--- fill info-buffer array ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = 1         ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(13) = nc        ! number of points excluding (ns+1:nn-1)
   ibuff_c(14) = ns        ! data removed from (ns+1:nn-1)
   ibuff_c(15) = nn        ! data removed from (ns+1:nn-1)
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now
   ibuff_c(32) = ashift    ! albedo time shift

   !--- pack msg buffer (with special-values) ---
   buffs = spval

#ifdef MPI_P
   SHR_MSG_TID_ICE  = ice_tid
   !--- send msg ---
   if(master_task) then
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_C2I)
   call shr_msg_send_r(buffs  ,size(buffs), SHR_MSG_TID_ICE, SHR_MSG_TAG_C2I)
   endif
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_ICE, SHR_MSG_TAG_C2I)
   call shr_msg_send_r(buffs  ,size(buffs), SHR_MSG_TID_ICE, SHR_MSG_TAG_C2I)
#endif
   
END SUBROUTINE msg_icef

!===============================================================================

END MODULE msg_ice_mod
