#include <misc.h>

!===============================================================================
! CVS $Id: msg_lnd_mod.F90,v 1.18 2001/12/07 22:13:36 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/msg_lnd_mod.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

MODULE msg_lnd_mod

   use control_mod      ! control data
   use buf_mod          ! model   data
   use const_mod        ! physical constants
   use map_mod          ! map data
   use shr_msg_mod      ! msg constants, data, routines
   use shr_sys_mod      ! system calls
   use shr_timer_mod    ! timing routines
#ifdef MPI_P
   use cpl_mod, only:lnd_tid
!   use mpmd_mod, only: master_task,MPI_COMM_CPL
#endif

   implicit none

   private  ! resticted access
   public  :: msg_lndi,msg_lndr,msg_lnds,msg_lndf,msg_lndm

!----------------------------wangln 2007-06-24 For MME-------------------
   !integer,parameter   :: nsflds=16         ! number of fields sent
   !integer,parameter   :: nrflds=14         ! number of fields recv'd
   integer   :: nsflds         ! number of fields sent
   integer   :: nrflds         ! number of fields recv'd
!----------------------------wangln 2007-06-24 For MME-------------------

   real   ,allocatable :: buffs(:,:)        ! msg buffer: send
   real   ,allocatable :: buffr(:,:)        ! msg buffer: recv
   real   ,allocatable :: area_fact_c2l(:)  ! cpl->model area correction factor
   real   ,allocatable :: area_fact_l2c(:)  ! model->cpl area correction factor
   real   ,allocatable :: area_fact_rc2c(:) ! model->cpl area correction factor

   !--- aggregate timing info ---
   integer      :: timer_send    ! shr_timer timer number for data send
   integer      :: timer_recv    ! shr_timer timer number for data recv

   !--- detailed timing info ---
!  integer(8)   :: irtc_w        ! rtc tics when waiting for msg
!  integer(8)   :: irtc_r        ! rtc tics when msg received
!  integer(8)   :: irtc_u        ! rtc tics when msg unpacked
!  integer(8)   :: irtc_p        ! rtc tics when packing msg
!  integer(8)   :: irtc_s        ! rtc tics when sending msg 

   !--- formats ---
   character(len=*),parameter :: F98 = "('comm_diag ',a3,1x,a4,1x,i3,es26.19,1x,a)"
   character(len=*),parameter :: F99 = "('[mp timing]  irtc = ',i20,' ',a)"

   save

CONTAINS

!===============================================================================

SUBROUTINE msg_lndi ()

   implicit none

   !--- local ---
   integer             :: i,j,n,m   ! generic loop indicies
   real                :: minf,maxf ! min/max cell area correction factors
   integer,allocatable :: row_rc(:) ! describes runoff compression key
   integer,allocatable :: col_rc(:) ! describes runoff compression key
   integer,allocatable :: key_r (:) ! compression key wrt row_rc,col_rc
   integer             :: n_rc2     ! lnd wants n_rc changed to this value

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_lndi) ',a,2i10)"
   character(len=*),parameter :: F02 = "('(msg_lndi) ',a,2f12.9)"

#ifdef MPI_P
   integer  SHR_MSG_TID_LND
#endif
!-------------------------------------------------------------------------------
! PURPOSE:
!   send/recv initial msg to/from lnd
! INPUT:
!   o flags contained in control.h
! OUTPUT:
!   o lnd info-buffer data: ibuff_l
!   o lnd and runoff data domain specs
! MESSAGES:
!   o data sent  : cpl info-buffer flags
!   o data recv'd: lnd info-buffer flags, lnd and runoff domain spec
!-------------------------------------------------------------------------------


!----------------------------wangln 2007-06-24 For MME-------------------
   if( atm_type(1:4) == 'CAM2' ) then
       nsflds=16         
       nrflds=14         
   else if( atm_type(1:4) == 'CAM3' .or. atm_type(1:3) == 'BCC') then
     if( carbon_flux ) then

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

#ifdef MOZART2
       nrflds = 18 + 14 +6   ! added 14 variables related to MOZART2 
#else
       nrflds = 18
#endif 

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
       nsflds = 17
     else
       nrflds = 15
       nsflds = 16
     endif
   endif
!----------------------------wangln 2007-06-24 For MME-------------------

   !--- allocate timer numbers ---
   call shr_timer_get(timer_send,'msg: cpl  -> lnd')
   call shr_timer_get(timer_recv,'msg: cpl <-  lnd')

   !--- allocate send & recv buffers ---
   allocate(buffr(n_l,nrflds))
   allocate(buffs(n_l,nsflds))
   allocate(area_fact_c2l (n_l )) ! area correction factor
   allocate(area_fact_l2c (n_l )) ! area correction factor
   allocate(area_fact_rc2c(n_rc)) ! area correction factor
!----xjx added 20080510
    buffr = 0.
    buffs = 0.
    area_fact_c2l = 0.
    area_fact_l2c = 0.
    area_fact_rc2c = 0.
!----xjx end

   !----------------------------------------------------------------------------
   ! send initial msg
   !----------------------------------------------------------------------------
   write(6,F01) 'send lnd msg (initial)'
   call shr_sys_flush(6)

   !--- send integer info-buffer ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr              ! error code, 0 => no error
   ibuff_c( 2) = stop_eod          ! flag: stop at end of day
   ibuff_c( 3) = stop_now          ! flag: stop now
   ibuff_c( 4) = cdate             ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec               ! seconds
   ibuff_c(11) = info_time         ! flag: mp timing
   ibuff_c(12) = info_dbug         ! dbug diagnostic level
   ibuff_c(40) = SHR_MSG_L_MAJ_V04 ! cpl/lnd interface: major version #
   ibuff_c(41) = SHR_MSG_L_MIN_V00 ! cpl/lnd interface: minor version #
   ibuff_c(42) = 0                 ! size of character data to send
   if( carbon_flux ) ibuff_c(77) = 1 ! open the carbon flux switch
#ifdef MPI_P
   SHR_MSG_TID_LND  = lnd_tid
   if(master_task) &
   &call shr_msg_send_i(ibuff_c,   nibuff  , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
#else
   call shr_msg_send_i(ibuff_c,   nibuff  , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
#endif

   !--- send floating point info-buffer ---
   rbuff_c     = spval
   rbuff_c(1)  = spval      ! Special value
   rbuff_c(2)  = orb_eccen  ! eccentricity
   rbuff_c(3)  = orb_obliqr ! obliquity
   rbuff_c(4)  = orb_lambm0 ! longitude of perihelion at v-equinox
   rbuff_c(5)  = orb_mvelpp ! moving vernal equinox + pi

   write(6,F01) 'send lnd floating pt msg (initial)'
   call shr_sys_flush(6)
#ifdef MPI_P
   if(master_task) &
   &call shr_msg_send_r(rbuff_c, nrbuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
#else
   call shr_msg_send_r(rbuff_c, nrbuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
#endif

   !----------------------------------------------------------------------------
   ! receive initial msg
   !----------------------------------------------------------------------------
   write(6,F01) 'recv lnd msg (initial)'
   call shr_sys_flush(6)
#ifdef MPI_P
   if(master_task) &
   &call shr_msg_recv_i(ibuff_l, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call MPI_BCAST(ibuff_l,size(ibuff_l),MPI_integer,0,MPI_COMM_CPL,ierr)
#else
   call shr_msg_recv_i(ibuff_l, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
#endif

   !--- extract info-buffer values ---
   if (ibuff_l(7) .ne. nx_l) call shr_sys_abort("(msg_lndi) nx_l = ?")
   if (ibuff_l(8) .ne. ny_l) call shr_sys_abort("(msg_lndi) ny_l = ?")
   ncpl_l    = ibuff_l( 9) ! number of msg pairs per day
   init_maskl= ibuff_l(34) ! T => requests cpl to send lnd domain info to lnd
   init_ricl = ibuff_l(35) ! T => use model's restart IC data
   n_rc2     = ibuff_l(36) ! size of compressed vector of runoff data
   if (ibuff_l(37) .ne. nx_r) call shr_sys_abort("(msg_lndi) nx_r = ?")
   if (ibuff_l(38) .ne. ny_r) call shr_sys_abort("(msg_lndi) ny_r = ?")


#ifdef MPI_P
   if(master_task) then
#endif
   !--- recv domain data (lnd) ---
   call shr_msg_recv_r(  xc_l , n_l       , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_r(  yc_l , n_l       , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_r(  xv_l , n_l*4     , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_r(  yv_l , n_l*4     , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_r(area_l , n_l       , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_i(mask_l , n_l       , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)

#ifdef MPI_P
   endif
   call MPI_BCAST(xc_l,   size(xc_l),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yc_l,   size(yc_l),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(xv_l,   size(xv_l),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yv_l,   size(yv_l),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(area_l,size(area_l),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(mask_l,size(mask_l),  MPI_integer,0,MPI_COMM_CPL,ierr)
#endif

#ifdef MPI_P
   if(master_task) then
#endif
   !--- recv domain data (runoff, uncompressed) ---
   call shr_msg_recv_r(  xc_r , n_r       , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_r(  yc_r , n_r       , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_r(  xv_r , n_r*4     , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_r(  yv_r , n_r*4     , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_r(area_r , n_r       , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   call shr_msg_recv_i(mask_r , n_r       , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
#ifdef MPI_P
   endif 
   call MPI_BCAST(xc_r,   size(xc_r),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yc_r,   size(yc_r),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(xv_r,   size(xv_r),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yv_r,   size(yv_r),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(area_r,size(area_r),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(mask_r,size(mask_r),  MPI_integer,0,MPI_COMM_CPL,ierr)
#endif

   if (n_rc2 > 0) then
     !--- recv runoff vector compression info ---
     allocate (key_r(n_r),row_rc (n_rc2),col_rc(n_rc2))
!----xjx added 20080510
     key_r = 0
!----xjx end
#ifdef MPI_P
   if(master_task) then
     call shr_msg_recv_i( row_rc, n_rc2   , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
     call shr_msg_recv_i( col_rc, n_rc2   , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   endif
   call MPI_BCAST(row_rc,size(row_rc),MPI_integer,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(col_rc,size(row_rc),MPI_integer,0,MPI_COMM_CPL,ierr)
#else
     call shr_msg_recv_i( row_rc, n_rc2   , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
     call shr_msg_recv_i( col_rc, n_rc2   , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
#endif
   endif

   !----------------------------------------------------------------------------
   ! resize compressed runoff vector arrays (if necessary) 
   !----------------------------------------------------------------------------

   !--- create new key from row & col ---
   if (n_rc2 > 0) then
      key_r(:)=0
      do n=1,n_rc2
         i = row_rc(n)
         j = col_rc(n)
         m = (j-1)*nx_r + i ! data_rc(n) corresponds to data_r(m)
         key_r(m)=1
      end do
   endif

   write(6,F01) 'FYI: original compressed runoff, n_r, n_rc = ',n_r,n_rc

   if (n_rc2 == 0) then
      write(6,F01) 'FYI: no new runoff compression info, *not* resizing'
      call shr_sys_flush(6)
   else if (n_rc2 == n_rc) then
      write(6,F01) 'FYI: size of compressed runoff data has not changed'
      write(6,F01) 'FYI: *not* resizing compressed runoff'
      write(6,F01) 'FYI: using compression key supplied by component'
      call shr_sys_flush(6)
   !! do n=1,n_r
   !!    if ( key_r(n) /= domain_r%key(n) ) then
   !!       write(6,F01) 'ERROR: new key inconsistent with old key'
   !!       write(6,F01) 'ERROR: key must not change'
   !!       call shr_sys_abort('(msg_lndi) inconsistent compression keys')
   !!    end if
   !! end do
      call map_domChgKey(domain_r,key_r)
      call map_mapCompress(map_r2o,map_rc2o,key_r,"src")
   else if ( 0 < n_rc2  .and.  n_rc2 <= n_rc  ) then
      write(6,F01) 'FYI: size of compressed runoff data has changed'
      write(6,F01) 'FYI: existing compressed runoff data will be lost'
      call shr_sys_flush(6)
      !--- resize/re-initialize compressed runoff array ---
      n_rc = n_rc2
      deallocate(roff_rc        )
      allocate  (roff_rc(n_rc,1))
      roff_rc = 0.0
      !--- reset compression scheme ---
      call map_domChgKey(domain_r,key_r)
      call map_mapCompress(map_r2o,map_rc2o,key_r,"src")
   else 
      write(6,F01) 'ERROR: invalid compressed runoff data vector size = ',n_rc2
      call shr_sys_flush(6)
      call shr_sys_abort('(msg_lndi) invalid compressed runoff data size')
   endif
   write(6,F01) 'FYI: new compressed runoff, n_r, n_rc = ',n_r,n_rc
   call shr_sys_flush(6)

   !--- dealloc temporary arrays needed to resize compressed runoff ---
   if (n_rc2 > 0) deallocate  (key_r,row_rc,col_rc)

   !----------------------------------------------------------------------------
   ! compute factors that account for inconsistent cell area values 
   !----------------------------------------------------------------------------
   area_fact_c2l = 1.0
   area_fact_l2c = 1.0
   minf =  1.0e30
   maxf = -1.0e30
   do n=1,n_l
     if (mask_l(n) /= 0) then
       area_fact_c2l(n) = domain_l%area(n)/area_l(n)
       area_fact_l2c(n) = area_l(n)/domain_l%area(n)
       minf = min(minf,area_fact_c2l(n))
       maxf = max(maxf,area_fact_c2l(n))
     end if
   end do
   write(6,F02) 'min c2l area factor:',minf
   write(6,F02) 'max c2l area factor:',maxf

   if (size(area_fact_rc2c) /= size(roff_rc)) then
      deallocate(area_fact_rc2c      )
      allocate  (area_fact_rc2c(n_rc)) 
   endif
   area_fact_rc2c = 1.0
   minf =  1.0e30
   maxf = -1.0e30
   do n=1,n_rc
      m = domain_r%iunc(n)
      area_fact_rc2c(n) = area_r(m)/domain_r%area(m) ! areas are uncompressed
      minf = min(minf,area_fact_rc2c(n))             ! factors are compressed
      maxf = max(maxf,area_fact_rc2c(n))
   end do
   write(6,F02) 'min r2c area factor:',minf
   write(6,F02) 'max r2c area factor:',maxf
      call shr_sys_flush(6)


END SUBROUTINE msg_lndi

!===============================================================================

SUBROUTINE msg_lndm ()

   implicit none

   !--- local ---
   integer             :: i,j,n,m   ! generic loop indicies
   real                :: minf,maxf ! min/max cell area correction factors
   integer             :: n_rc2     ! lnd wants n_rc changed to this value
   integer,allocatable :: row_rc(:) ! describes runoff compression key
   integer,allocatable :: col_rc(:) ! describes runoff compression key
   integer,allocatable :: key_r (:) ! compression key wrt row_rc,col_rc

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_lndm) ',a,2i10)"
   character(len=*),parameter :: F02 = "('(msg_lndm) ',a,2f12.9)"
#ifdef MPI_P
   integer  SHR_MSG_TID_LND
#endif

!-------------------------------------------------------------------------------
! PURPOSE:
!   1) send domain info to lnd model
!   2) recv new runoff domain compression info from lnd model (optional)
! NOTE:
!   this is a special, and optional, msg specifically created for the NCAR/LSM 
!   (others could use it too).  It tells the lnd what it's domain should be.  
!   The underlying asumption here is that the lnd component is capable of 
!   adapting to *any* domain that might be passed to it by this routine.
!   It is also assumed that the lnd is on the same grid as the atm.
!
!   After being told what its domain should be, the lnd then MAY send to the cpl
!   new/updated runoff domain compression info.  This allows the lnd to adapt
!   its runoff domain wrt the lnd domain info just sent to the lnd model.
! INPUT:
!   o coordinate arrays, domain mask, lnd fraction, and atm domain mask
!   o various flags contained in control_mod.F90
! OUTPUT:
!   o updated runoff domain info
! MESSAGES:
!   o data sent: cpl info-buffer & complete domain info
!   o data recv'd: runoff domain info
!-------------------------------------------------------------------------------
   
   !----------------------------------------------------------------------------
   write(6,F01) 'send lnd msg (lnd domain)'
      call shr_sys_flush(6)
   !----------------------------------------------------------------------------

   !--- fill info-buffer array ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = stop_now  ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c( 7) = nx_l      ! lnd buffer, as a 2d array, is nx_l*ny_l cells
   ibuff_c( 8) = ny_l      ! 
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level

#ifdef MPI_P
   SHR_MSG_TID_LND  = lnd_tid
   if(master_task) then
#endif
   !--- send domain info msg ---
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
   call shr_msg_send_r(  xc_l , n_l       , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
   call shr_msg_send_r(  yc_l , n_l       , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
   call shr_msg_send_r(  xv_l , n_l*4     , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
   call shr_msg_send_r(  yv_l , n_l*4     , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
   call shr_msg_send_r(area_l , n_l       , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
   call shr_msg_send_r(  fl_l , n_l       , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
   call shr_msg_send_i(mask_l , n_l       , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
   call shr_msg_send_i(mask_a , n_a       , SHR_MSG_TID_LND, SHR_MSG_TAG_C2LI)
#ifdef MPI_P
   endif

   if(master_task) then
   write(6,F01) 'recv lnd msg (new runoff compression key)'
      call shr_sys_flush(6)
   call shr_msg_recv_i(ibuff_l, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   endif
   call MPI_BCAST(ibuff_l,size(ibuff_l),MPI_integer,0,MPI_COMM_CPL,ierr)
#else
   !----------------------------------------------------------------------------
   write(6,F01) 'recv lnd msg (new runoff compression key)'
      call shr_sys_flush(6)
   !----------------------------------------------------------------------------
   call shr_msg_recv_i(ibuff_l, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
#endif

   !--- extract info-buffer values ---
   n_rc2     = ibuff_l(36) ! size of compressed vector of runoff data
   if (ibuff_l(37) .ne. nx_r) call shr_sys_abort("(msg_lndm) nx_r = ?")
   if (ibuff_l(38) .ne. ny_r) call shr_sys_abort("(msg_lndm) ny_r = ?")

   if (n_rc2 > 0) then
     !--- recv runoff vector compression info ---
     allocate (key_r(n_r),row_rc (n_rc2),col_rc(n_rc2))
!xjx added 20080510
     key_r = 0
!xjx end
#ifdef MPI_P
   if(master_task) then
     call shr_msg_recv_i( row_rc, n_rc2   , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
     call shr_msg_recv_i( col_rc, n_rc2   , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
   endif
     call MPI_BCAST(row_rc,size(row_rc),MPI_integer,0,MPI_COMM_CPL,ierr)
     call MPI_BCAST(col_rc,size(row_rc),MPI_integer,0,MPI_COMM_CPL,ierr)
#else
     call shr_msg_recv_i( row_rc, n_rc2   , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
     call shr_msg_recv_i( col_rc, n_rc2   , SHR_MSG_TID_LND, SHR_MSG_TAG_L2CI)
#endif
   endif

   !----------------------------------------------------------------------------
   ! resize compressed runoff vector arrays (if necessary) 
   !----------------------------------------------------------------------------

   !--- create new key from row & col ---
   if (n_rc2 > 0) then
      key_r(:)=0
      do n=1,n_rc2
         i = row_rc(n)
         j = col_rc(n)
         m = (j-1)*nx_r + i ! data_rc(n) corresponds to data_r(m)
         key_r(m)=1
      end do
   endif

   write(6,F01) 'FYI: original compressed runoff, n_r, n_rc = ',n_r,n_rc
      call shr_sys_flush(6)
   if (n_rc2 == 0) then
      write(6,F01) 'FYI: no new runoff compression info, *not* resizing'
   else if (n_rc2 == n_rc) then
      write(6,F01) 'FYI: size of compressed runoff data has not changed'
      write(6,F01) 'FYI: *not* resizing compressed runoff'
      write(6,F01) 'FYI: using compression key supplied by component'
      call shr_sys_flush(6)
   !! do n=1,n_r
   !!    if ( key_r(n) /= domain_r%key(n) ) then
   !!       write(6,F01) 'ERROR: new key inconsistent with old key'
   !!       write(6,F01) 'ERROR: key must not change'
   !!       call shr_sys_abort('(msg_lndm) inconsistent compression keys')
   !!    end if
   !! end do
      call map_domChgKey(domain_r,key_r)
      call map_mapCompress(map_r2o,map_rc2o,key_r,"src")
   else if ( 0 < n_rc2  .and.  n_rc2 <= n_rc  ) then
      write(6,F01) 'FYI: size of compressed runoff data has changed'
      write(6,F01) 'FYI: existing compressed runoff data will be lost'
      call shr_sys_flush(6)
      !--- resize/re-initialize compressed runoff array ---
      n_rc = n_rc2
      deallocate(roff_rc        )
      allocate  (roff_rc(n_rc,1))
      roff_rc = 0.0
      !--- reset compression scheme ---
      call map_domChgKey(domain_r,key_r)
      call map_mapCompress(map_r2o,map_rc2o,key_r,"src")
   else 
      write(6,F01) 'ERROR: invalid compressed runoff data vector size = ',n_rc2
      call shr_sys_flush(6)
      call shr_sys_abort('(msg_lndm) invalid compressed runoff data size')
   endif
   write(6,F01) 'FYI: new compressed runoff, n_r, n_rc = ',n_r,n_rc
      call shr_sys_flush(6)

   !--- dealloc temporary arrays needed to resize compressed runoff ---
   if (n_rc2 > 0) deallocate  (key_r,row_rc,col_rc)

   !----------------------------------------------------------------------------
   ! compute factors that account for inconsistent cell area values 
   !----------------------------------------------------------------------------
   area_fact_c2l = 1.0
   area_fact_l2c = 1.0
   minf =  1.0e30
   maxf = -1.0e30
   do n=1,n_l
     if (mask_l(n) /= 0) then
       area_fact_c2l(n) = domain_l%area(n)/area_l(n)
       area_fact_l2c(n) = area_l(n)/domain_l%area(n)
       minf = min(minf,area_fact_c2l(n))
       maxf = max(maxf,area_fact_c2l(n))
     end if
   end do
   write(6,F02) 'min c2l area factor:',minf
   write(6,F02) 'max c2l area factor:',maxf

   if (size(area_fact_rc2c) /= size(roff_rc)) then
      deallocate(area_fact_rc2c      )
      allocate  (area_fact_rc2c(n_rc)) 
!xjx added 20080510
     area_fact_rc2c = 0
!xjx end
   endif
   area_fact_rc2c = 1.0
   minf =  1.0e30
   maxf = -1.0e30
   do n=1,n_rc
      m = domain_r%iunc(n)
      area_fact_rc2c(n) = area_r(m)/domain_r%area(m) ! areas are uncompressed
      minf = min(minf,area_fact_rc2c(n))             ! factors are compressed
      maxf = max(maxf,area_fact_rc2c(n))
   end do
   write(6,F02) 'min r2c area factor:',minf
   write(6,F02) 'max r2c area factor:',maxf
      call shr_sys_flush(6)

END SUBROUTINE msg_lndm

!===============================================================================

SUBROUTINE msg_lndr (Sl,Fall,roff)

   implicit none

   !--- arguments ---
   real         :: Sl  (n_l,nSl  )  ! lnd state
   real         :: Fall(n_l,nFall)  ! atm/lnd fluxes from lnd
   real         :: roff(n_rc,1)     ! incoming runoff data

   !--- local ---
   integer      :: n             ! generic loop indicies
   integer      :: m             ! generic loop index
   real         :: f             ! factor wrt area correction
   real         :: gsum          ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_lndr) ',a,i4)"

#ifdef MPI_P
   integer  SHR_MSG_TID_LND
#endif
!-------------------------------------------------------------------------------
! PURPOSE:
!   receive msg from lnd model
! INPUT:
!   o none
! OUTPUT:
!   o ibuff_l, Sl_l, Fall_l
! MESSAGES:
!   o data received: lnd info-buffer, state variables, output fluxes
!-------------------------------------------------------------------------------
   
   if (info_dbug.ge.3) write(6,F01) 'recv lnd msg'

   !--- receive msg ---
   irtc_w = shr_sys_irtc()
   call shr_timer_start(timer_recv)
#ifdef MPI_P
   SHR_MSG_TID_LND  = lnd_tid
   if(master_task) &
   &call shr_msg_recv_i(ibuff_l, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_L2C)
   call MPI_BCAST(ibuff_l,size(ibuff_l),MPI_integer,0,MPI_COMM_CPL,ierr)
#else
   call shr_msg_recv_i(ibuff_l, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_L2C)
#endif

!! !--- check size of incoming compressed runoff data ---
!! n = ibuff_l(36) 
!! if (n /= n_rc) then
!!    write(6,F01) 'ERROR: invalid compressed runoff data vector size = ',n
!!    call shr_sys_abort('(msg_lndr) invalid compressed runoff data size')
!! end if

   irtc_r = shr_sys_irtc()
#ifdef MPI_P
   if(master_task) then
   call shr_msg_recv_r(buffr  ,size(buffr), SHR_MSG_TID_LND, SHR_MSG_TAG_L2C)
   call shr_msg_recv_r(roff   , n_rc      , SHR_MSG_TID_LND, SHR_MSG_TAG_L2C)
   endif
   call MPI_BCAST(buffr,size(buffr),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(roff,size(roff),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
#else
   call shr_msg_recv_r(buffr  ,size(buffr), SHR_MSG_TID_LND, SHR_MSG_TAG_L2C)
   call shr_msg_recv_r(roff   , n_rc      , SHR_MSG_TID_LND, SHR_MSG_TAG_L2C)
#endif


!#ifdef MPI_P
!   DO n=IST_l,IEND_l
!#else
!$OMP PARALLEL DO PRIVATE(n,f)
   DO n=1,n_l
!#endif
      f = area_fact_l2c(n) ! wrt flux conservation & varying cell area calcs

      !--- lnd states ---
      Sl  (n,kSl_t      ) = buffr(n, 1)   
      Sl  (n,kSl_avsdr  ) = buffr(n, 2)   
      Sl  (n,kSl_anidr  ) = buffr(n, 3)   
      Sl  (n,kSl_avsdf  ) = buffr(n, 4)   
      Sl  (n,kSl_anidf  ) = buffr(n, 5)   
      Sl  (n,kSl_snowh  ) = buffr(n, 6)   

      !--- atm/lnd fluxes ---
      Fall(n,kFall_taux ) = buffr(n, 7) * f
      Fall(n,kFall_tauy ) = buffr(n, 8) * f
      Fall(n,kFall_lat  ) = buffr(n, 9) * f
      Fall(n,kFall_sen  ) = buffr(n,10) * f
      Fall(n,kFall_lwup ) = buffr(n,11) * f
      Fall(n,kFall_evap ) = buffr(n,12) * f
      Fall(n,kFall_swnet) = buffr(n,14) * f

      !--- 2m reference temperature -----
      Fall(n,kFall_tref ) = buffr(n,13)  

!----------------------------wangln 2007-06-24 For MME-------------------
      if( atm_type(1:4) == 'CAM3' .or. atm_type(1:3) == 'BCC')  Fall(n,kFall_qref ) = buffr(n,15)  ! 2m reference special humiduty 
      if( carbon_flux ) Fall(n,kFall_carbon ) = buffr(n,16)  ! lnd carbon flux

      if( carbon_flux ) then
         Sl  (n,kSl_u10m  ) = buffr(n, 17)
         Sl  (n,kSl_v10m  ) = buffr(n, 18)
      endif

#ifdef MOZART2
!-----
! added 14 variables related to MOZART2  
      Fall(n,kFall_mss_dst01 ) = buffr(n, 19) 
      Fall(n,kFall_mss_dst02 ) = buffr(n, 20) 
      Fall(n,kFall_mss_dst03 ) = buffr(n, 21) 
      Fall(n,kFall_mss_dst04 ) = buffr(n, 22) 
      Fall(n,kFall_vlc_dst01 ) = buffr(n, 23) 
      Fall(n,kFall_vlc_dst02 ) = buffr(n, 24) 
      Fall(n,kFall_vlc_dst03 ) = buffr(n, 25) 
      Fall(n,kFall_vlc_dst04 ) = buffr(n, 26) 
      Fall(n,kFall_lnd_ram1 ) = buffr(n, 27) 
      Fall(n,kFall_lnd_fv ) = buffr(n, 28) 
      Fall(n,kFall_vlc_SO4 ) = buffr(n, 29) 
      Fall(n,kFall_vlc_NH4 ) = buffr(n, 30) 
      Fall(n,kFall_vlc_CB2 ) = buffr(n, 31) 
      Fall(n,kFall_vlc_SO2 ) = buffr(n, 32) 
     
      Fall(n,kFall_flux_ISOP ) = buffr(n, 33)
      Fall(n,kFall_flux_ACET ) = buffr(n, 34)
      Fall(n,kFall_flux_C3H6 ) = buffr(n, 35)
      Fall(n,kFall_flux_C2H4 ) = buffr(n, 36)
      Fall(n,kFall_flux_OC1 )  = buffr(n, 37)
      Fall(n,kFall_flux_C10H16 ) = buffr(n, 38) 
!----------------
#endif

   end do
!$omp  end parallel do

   if (info_dbug >= 2) then
      do m=1,nrflds
         gsum = 0.
         do n=1,n_l
            if (mask_l(n) /= 0) gsum = gsum + buffr(n,m)*area_l(n)*rEarth*rEarth
         enddo
         write(6,F98)'lnd','send',m,gsum,'in_cpl'
      enddo
      gsum = 0.
      do n=1,n_rc
         gsum = gsum + roff(n,1)
      enddo
      write(6,F98)'lnd','send',nrflds+1,gsum,'in_cpl - sum(roff)'
      call shr_sys_flush(6)
   endif

   irtc_u = shr_sys_irtc()
   call shr_timer_stop(timer_recv)

   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_w,'l->c waiting'
      write(6,F99) irtc_r,'l->c received'
      write(6,F99) irtc_u,'l->c unpacked'
      call shr_sys_flush(6)
   end if

!  - tcmods, set runoff to zero
!   roff = 0.
!  - tcmode
 
END SUBROUTINE msg_lndr

!=======================================================================

SUBROUTINE msg_lnds (Fala,Sa)

   implicit none

   !--- arguments ---
   real         :: Fala(n_l,nFala)  ! atm/lnd fluxes on the land grid
   real         ::   Sa(n_l,nSa  )  ! atm state on the land grid

   !--- local ---
   integer      :: n                ! generic loop indicies
   integer      :: m                ! generic loop index
   real         :: f                ! factor wrt area correction
   real         :: gsum             ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_lnds) ',a,i4)"
#ifdef MPI_P
   integer  SHR_MSG_TID_LND
#endif

!-------------------------------------------------------------------------------
! PURPOSE:
!   send msg to lnd model
! INPUT:
!   o Fala_l, Sa_l
!   o various flags contained in control.h
! OUTPUT:
!   o none
! MESSAGES:
!   o data sent: cpl info-buffer, Fala_l, Sa_l
!-----------------------------------------------------------------------
   
   if (info_dbug.ge.3) write(6,F01) 'send lnd msg'
      call shr_sys_flush(6)

   !--- fill info-buffer array ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = stop_now  ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now
   ibuff_c(32) = ashift    ! albedo time shift

   !--- pack msg buffer ---
   irtc_p = shr_sys_irtc()
   call shr_timer_start(timer_send)

!$OMP PARALLEL DO PRIVATE(n,f)
   DO n=1,n_l

!fangyj      f = area_fact_c2l(n) ! wrt flux conservation & varying cell area calcs

      f = 1.0
 
     !--- atm state on land grid ---
      buffs(n, 1) = Sa  (n,kSa_z      ) 
      buffs(n, 2) = Sa  (n,kSa_u      ) 
      buffs(n, 3) = Sa  (n,kSa_v      ) 
      buffs(n, 4) = Sa  (n,kSa_ptem   ) 
      buffs(n, 5) = Sa  (n,kSa_shum   ) 
      buffs(n, 6) = Sa  (n,kSa_pbot   ) 
      buffs(n, 7) = Sa  (n,kSa_tbot   ) 

      !--- atm/lnd fluxes ---
      buffs(n, 8) = Fala(n,kFala_lwdn ) * f
      buffs(n, 9) = Fala(n,kFala_rainc) * f
      buffs(n,10) = Fala(n,kFala_rainl) * f
      buffs(n,11) = Fala(n,kFala_snowc) * f
      buffs(n,12) = Fala(n,kFala_snowl) * f
      buffs(n,13) = Fala(n,kFala_swndr) * f
      buffs(n,14) = Fala(n,kFala_swvdr) * f
      buffs(n,15) = Fala(n,kFala_swndf) * f
      buffs(n,16) = Fala(n,kFala_swvdf) * f
      if( carbon_flux ) buffs(n,17) = Fala(n,kFala_co2) * f
   end do
!$omp  end parallel do

   if (info_dbug >= 2) then
      do m=1,nsflds
         gsum = 0.
         do n=1,n_l
            if (mask_l(n) /= 0) gsum = gsum + buffs(n,m)*area_l(n)*rEarth*rEarth
         enddo
         write(6,F98)'lnd','recv',m,gsum,'in_cpl'
      call shr_sys_flush(6)
      enddo
   endif

   !--- send msg ---
   irtc_s = shr_sys_irtc()
#ifdef MPI_P
   SHR_MSG_TID_LND  = lnd_tid
   if(master_task) then
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_C2L)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_LND, SHR_MSG_TAG_C2L)
   endif
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_C2L)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_LND, SHR_MSG_TAG_C2L)
#endif
   call shr_timer_stop(timer_send)
   
   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_p,'c->l packing'
      write(6,F99) irtc_s,'c->l sending'
      call shr_sys_flush(6)
   end if

END SUBROUTINE msg_lnds

!=======================================================================

SUBROUTINE msg_lndf ()

   implicit none

   !--- local ---
   integer      :: n,k          ! loop indicies

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_lndf) ',a,i4)"
#ifdef MPI_P
   integer  SHR_MSG_TID_LND
#endif

!-----------------------------------------------------------------------
! PURPOSE:
!   send final msg to lnd model
! INPUT:
!   various flags contained in control.h
! OUTPUT:
!   none
! MESSAGES:
!   data sent: coupler info-buffer
!-----------------------------------------------------------------------
   
   write(6,F01) 'send lnd msg (final)'
      call shr_sys_flush(6)

   !--- set info-buffer values ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = 1         ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now
   ibuff_c(32) = ashift    ! albedo time shift


   !--- pack msg buffer (with special-values) ---
   buffs = spval

   !--- pack & send msg ---
#ifdef MPI_P
   SHR_MSG_TID_LND  = lnd_tid
   if(master_task) then
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_C2L)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_LND, SHR_MSG_TAG_C2L)
   endif
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_LND, SHR_MSG_TAG_C2L)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_LND, SHR_MSG_TAG_C2L)
#endif
   
END SUBROUTINE msg_lndf

!=======================================================================

END MODULE msg_lnd_mod
