#include <misc.h>

MODULE msg_ocn_mod

   use control_mod      ! control data
   use buf_mod          ! model   data
   use const_mod        ! physical constants
   use map_mod          ! map data
   use shr_msg_mod      ! msg constants, data, routines
   use shr_timer_mod    ! timing routines
#ifdef MPI_P
   use cpl_mod, only:ocn_tid
!   use mpmd_mod, only: master_task,MPI_COMM_CPL
#endif

   implicit none

   private  ! resticted access
   public  :: msg_ocni,msg_ocnr,msg_ocns,msg_ocnf
   public  :: angle_sin, angle_cos

!------------wangln 2007-06-24 For MME -------------------------------------
   !integer,parameter   :: nsflds=14        ! number of fields sent
   !integer,parameter   :: nrflds= 7        ! number of fields recv'd
   integer   :: nsflds        ! number of fields sent
   integer   :: nrflds        ! number of fields recv'd
!------------wangln 2007-06-24 For MME -------------------------------------

   real   ,allocatable :: buffs(:,:)       ! msg buffer: send
   real   ,allocatable :: buffr(:,:)       ! msg buffer: recv
   real   ,allocatable :: area_fact_c2o(:) ! cpl->ocn area correction factor
   real   ,allocatable :: area_fact_o2c(:) ! ocn->cpl area correction factor
   real   ,allocatable :: angle_o(:)     !  MOM T angle For Transfer
   real   ,allocatable :: angle_sin(:)     !  MOM T-angle  SIN For Transfer
   real   ,allocatable :: angle_cos(:)     !  MOM T-angle  COS For Transfer


   !--- aggregate timing info ---
   integer      :: timer_send    ! shr_timer timer number for data send
   integer      :: timer_recv    ! shr_timer timer number for data recv

   !--- detailed timing info ---
!  integer(8)   :: irtc_w        ! rtc tics when waiting for msg
!  integer(8)   :: irtc_r        ! rtc tics when msg received
!  integer(8)   :: irtc_u        ! rtc tics when msg unpacked
!  integer(8)   :: irtc_p        ! rtc tics when packing msg
!  integer(8)   :: irtc_s        ! rtc tics when sending msg 

   !--- formats ---
   character(len=*),parameter :: F98 = "('comm_diag ',a3,1x,a4,1x,i3,es26.19,1x,a)"
   character(len=*),parameter :: F99 = "('[mp timing]  irtc = ',i20,' ',a)"
!-----------------foe Climate Runoff data 2008-06-03 wangln-------------------
   real   ,allocatable :: runoff(:)
!-----------------foe Climate Runoff data 2008-06-03 wangln-------------------

   save

CONTAINS

!===============================================================================

SUBROUTINE msg_ocni ()

   implicit none

   !--- local ---
   integer :: n             ! generic loop index
   real    :: minf,maxf     ! min/max cell area correction factors

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_ocni) ',a,i4)"
   character(len=*),parameter :: F02 = "('(msg_ocni) ',a,2f12.9)"
#ifdef MPI_P
   integer  SHR_MSG_TID_OCN
#endif

!-------------------------------------------------------------------------------
! PURPOSE:
!   send/recv initial msg to/from ocn 
! INPUT:
!   o flags contained in control.h
! OUTPUT:
!   o ocn info-buffer data         : ibuff_o
!   o ocn spatial coordinate arrays: xv_o,xc_o,yv_o,yc_o
!   o ocn model domain mask        : mask_o
! MESSAGES:
!   o data  sent : cpl info-buffer flags
!   o data recv'd: ocn info-buffer flags, grids, domain mask
!-------------------------------------------------------------------------------
 

!------------wangln 2007-06-24 For MME -------------------------------------
   if( ocn_type(1:3) == 'POP' ) then 
       nsflds = 14       ! number of fields sent
       nrflds =  7       ! number of fields recv'd
   else if( ocn_type(1:3) == 'MOM' ) then
       if( carbon_flux ) then
#ifdef HAMOCC
           nsflds = 21       ! number of fields sent
           nrflds = 10       ! number of fields recv'd
#else
           nsflds = 14       ! number of fields sent
           nrflds =  7       ! number of fields recv'd
#endif
       else
           nsflds = 12       ! number of fields sent
           nrflds =  6       ! number of fields recv'd
       endif
   endif
!------------wangln 2007-06-24 For MME -------------------------------------

   !--- allocate timer numbers ---
   call shr_timer_get(timer_send,'msg: cpl  -> ocn')
   call shr_timer_get(timer_recv,'msg: cpl <-  ocn')
   
   !--- allocate local buffers/arrays ---
   allocate(buffr(n_o,nrflds))   ! recv buffer
   allocate(buffs(n_o,nsflds))   ! send buffer
   allocate(area_fact_c2o(n_o))  ! area factment factor
   allocate(area_fact_o2c(n_o))  ! area factment factor
!xjx-- added 20080510
   buffr = 0.
   buffs = 0.
   area_fact_c2o = 0.
   area_fact_o2c = 0.
!xjx-- end
!-----------------for Climate Runoff data 2008-06-03 wangln-------------------
   if( ocn_type(1:3) == 'MOM' .and. climate_runoff ) then
     allocate( runoff( n_o ) ) !  get runoff data
     runoff = 0.
     open(77, file='climate_runoff.bin', form='unformatted' )
          read(77)  runoff
     close(77)
   endif
   if( ocn_type(1:3) == 'MOM' ) then
      allocate( angle_o(n_o) )  ! MOM T-angle
      allocate( angle_sin(n_o) )  ! MOM T-angle SIN
      allocate( angle_cos(n_o) )  ! MOM T-angle COS
      angle_o = 0.
      angle_sin = 0.
      angle_cos = 0.
   endif
!-----------------for Climate Runoff data 2008-06-03 wangln-------------------


   !-----------------------------------------------------------------
   ! send initial msg
   !-----------------------------------------------------------------
   ibuff_c     = 0
   ibuff_c( 1) = ierr              ! error code, 0 => no error
   ibuff_c( 2) = stop_eod          ! flag: stop at end of day
   ibuff_c( 3) = stop_now          ! flag: stop now
   ibuff_c( 4) = cdate             ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec               ! seconds
   ibuff_c(11) = info_time         ! flag: mp timing
   ibuff_c(12) = info_dbug         ! dbug diagnostic level
   ibuff_c(40) = SHR_MSG_O_MAJ_V04 ! cpl/ocn interface: major version #
   ibuff_c(41) = SHR_MSG_O_MIN_V00 ! cpl/ocn interface: minor version #
   if( carbon_flux ) ibuff_c(77) = 1  ! open carbon flux switch

#ifdef HAMOCC
   ibuff_c(66) = 1  ! wufh add open HAMOCC switch
#endif

   !ibuff_c( 100 ) = 1  ! For SIS 
   ibuff_c( 100 ) = 0  ! For CSIM 
   if( ocn_type(1:3) == 'MOM' ) ibuff_c( 100 ) = 1  ! wangln 2007-06-24
!----------------------wangln 2007-01-24 Add the CSIM/SIS Logical Variable-----------------
   

   write(6,F01) 'send ocn msg (initial)'
#ifdef MPI_P
   SHR_MSG_TID_OCN  = ocn_tid
   if(master_task) &
   &call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_C2OI)
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_C2OI)
#endif
   
   !-----------------------------------------------------------------
   ! receive initial msg
   !-----------------------------------------------------------------

   write(6,F01) 'recv ocn msg (initial)'
#ifdef MPI_P
   if(master_task) then
   call shr_msg_recv_i(ibuff_o, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  xc_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  yc_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  xv_o , n_o*4     , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  yv_o , n_o*4     , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(area_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_i(mask_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   if( ocn_type(1:3) == 'MOM' ) then
        call shr_msg_recv_r(angle_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   endif
   endif

   call MPI_BCAST(ibuff_o,size(ibuff_o),MPI_integer,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(xc_o,   size(xc_o),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yc_o,   size(yc_o),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(xv_o,   size(xv_o),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(yv_o,   size(yv_o),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(area_o,size(area_o),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
   call MPI_BCAST(mask_o,size(mask_o),  MPI_integer,0,MPI_COMM_CPL,ierr)
   if( ocn_type(1:3) == 'MOM' ) then
   call MPI_BCAST(angle_o,size(angle_o),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
        angle_sin = sin ( angle_o )
        angle_cos = cos ( angle_o )
        deallocate( angle_o )
   endif
#else
   call shr_msg_recv_i(ibuff_o, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  xc_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  yc_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  xv_o , n_o*4     , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  yv_o , n_o*4     , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(area_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_i(mask_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   if( ocn_type(1:3) == 'MOM' ) then
        call shr_msg_recv_r(angle_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
        angle_sin = sin ( angle_o )
        angle_cos = cos ( angle_o )
        deallocate( angle_o )
   endif
#endif
  
   !--- extract info-buffer values ---
   if (ibuff_o(7) .ne. nx_o) stop 'nx'
   if (ibuff_o(8) .ne. ny_o) stop 'ny'
   ncpl_o    = ibuff_o( 9) ! number of msg pairs per day
   flx_epfac = ibuff_o(31) ! E~P adjustment factor
   init_rico = ibuff_o(35) ! T => use model's restart IC data

   !-----------------------------------------------------------------
   ! compute factors that account for inconsistent cell area values 
   !-----------------------------------------------------------------
   area_fact_c2o = 1.0
   area_fact_o2c = 1.0
   minf =  1.0e30
   maxf = -1.0e30
   do n=1,n_o
     if (mask_o(n) /= 0) then
       area_fact_c2o(n) = domain_o%area(n)/area_o(n)
       area_fact_o2c(n) = area_o(n)/domain_o%area(n)
       minf = min(minf,area_fact_c2o(n))
       maxf = max(maxf,area_fact_c2o(n))
     end if
   end do
   write(6,F02) 'min c2o area factor:',minf
   write(6,F02) 'max c2o area factor:',maxf

   !-----------------------------------------------------------------
   ! KLUDGE: scrip & ocn xc & yc should be the same but aren't
   !-----------------------------------------------------------------
   write(6,F01) 'WARNING: replacing scrip xc,yc with values provided by ocn'
   domain_o%xc = xc_o
   domain_o%yc = yc_o

END SUBROUTINE msg_ocni

!===============================================================================

SUBROUTINE msg_ocnr (So,Fioo)

   implicit none

   !--- arguments ---
   real         :: So  (n_o,nSo  )  ! ocn state
   real         :: Fioo(n_o,nFioo)  ! ice/ocn fluxes from ocn

   !--- local ---
   integer      :: n                ! generic loop index
   integer      :: m                ! generic loop index
   real         :: f                ! factor wrt area correction
   real         :: gsum             ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg


   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_ocnr) ',a,i4)"
#ifdef MPI_P
   integer  SHR_MSG_TID_OCN
#endif

!-------------------------------------------------------------------------------
! PURPOSE:
!   receive msg from ocn model 
! INPUT:
!   o none
! OUTPUT:
!   o ibuff_o, So_o, Fioo_o
! MESSAGES:
!   o data received: ocn info-buffer, state variables, output fluxes
!-------------------------------------------------------------------------------
   
   if (info_dbug.ge.3) write(6,F01) 'recv ocn msg'

   irtc_w = shr_sys_irtc()
   call shr_timer_start(timer_recv)
#ifdef MPI_P
   SHR_MSG_TID_OCN  = ocn_tid
   if(master_task) &
   &call shr_msg_recv_i(ibuff_o, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2C)
   call MPI_BCAST(ibuff_o,size(ibuff_o),MPI_integer,0,MPI_COMM_CPL,ierr)
#else
   call shr_msg_recv_i(ibuff_o, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2C)
#endif

   irtc_r = shr_sys_irtc()

#ifdef MPI_P
   if(master_task) &
   &call shr_msg_recv_r( buffr ,size(buffr), SHR_MSG_TID_OCN, SHR_MSG_TAG_O2C)
   call MPI_BCAST(buffr,size(buffr),MPI_real8,0,MPI_COMM_CPL,ierr)
#else
   call shr_msg_recv_r( buffr ,size(buffr), SHR_MSG_TID_OCN, SHR_MSG_TAG_O2C)
#endif

   
!!!!$OMP PARALLEL DO PRIVATE(n,f)

!------------wangln 2007-06-24 For MME -------------------------------------
   if( ocn_type(1:3) == 'POP' ) then

!#ifdef MPI_P
!   DO n=IST_o,IEND_o
!#else
!$OMP PARALLEL DO PRIVATE(n,f)
   DO n=1,n_o
!#endif
         f = area_fact_o2c(n) ! wrt flux conservation & varying cell area calcs

         !--- ocn states ---
         So (n,kSo_t      ) = buffr(n,1)
         So (n,kSo_s      ) = buffr(n,2)
         So (n,kSo_u      ) = buffr(n,3)
         So (n,kSo_v      ) = buffr(n,4)

         !--- output fluxes: ice/ocn ---
         Fioo(n,kFioo_dhdx) = buffr(n,5)
         Fioo(n,kFioo_dhdy) = buffr(n,6)
         Fioo(n,kFioo_q   ) = buffr(n,7) * f
      end do

   else if( ocn_type(1:3) == 'MOM' ) then

!#ifdef MPI_P
!   DO n=IST_o,IEND_o
!#else
!$OMP PARALLEL DO PRIVATE(n,f)
   DO n=1,n_o
!#endif
         f = area_fact_o2c(n) ! wrt flux conservation & varying cell area calcs

         !--- ocn states ---
         So (n,kSo_t      ) = buffr(n,1)
         So (n,kSo_s      ) = buffr(n,2)
         So (n,kSo_u      ) = buffr(n,3)
         So (n,kSo_v      ) = buffr(n,4)

         !--- output fluxes: ice/ocn ---
         Fioo(n,kFioo_dhdx) = buffr(n,5) ! For MOM This is the Sea_level
         Fioo(n,kFioo_q   ) = buffr(n,6) * f
         if( carbon_flux )  So(n, kSo_carbon ) =  buffr(n,7) * f

#ifdef HAMOCC
         So(n, kSo_co2flux ) =  buffr(n,8) * f !units: g C m-2 s-1, upward (+)
         So(n, kSo_n2oflux ) =  buffr(n,9) * f !units: kg N2O m-2 s-1, upward (+)
         So(n, kSo_dmsflux ) =  buffr(n,10) * f !units: kg DMS m-2 s-1, upward (+)
#endif
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

         if( mask_o(n) /=0 .and. So(n,kSo_t) >=310 ) write(6,*) 'MOM T>310=',n, So(n,kSo_t) 
         if( mask_o(n) /=0 .and. So(n,kSo_t) <=260 ) write(6,*) 'MOM T<260=',n, So(n,kSo_t) 

      end do
!$omp  end parallel do

   endif
!------------wangln 2007-06-24 For MME -------------------------------------

   if (info_dbug >= 2) then
      do m=1,nrflds
         gsum = 0.
         do n=1,n_o
            if (mask_o(n) /= 0) gsum = gsum + buffr(n,m)*area_o(n)*rEarth*rEarth
         enddo
         write(6,F98)'ocn','send',m,gsum,'in_cpl'
      enddo
   endif
   

   irtc_u = shr_sys_irtc()
   call shr_timer_stop(timer_recv)

   !--- extract info-buffer values ---
   flx_epfac = ibuff_o(31) ! E~P factment factor

   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_w,'o->c waiting'
      write(6,F99) irtc_r,'o->c received'
      write(6,F99) irtc_u,'o->c unpacked'
   end if

END SUBROUTINE msg_ocnr

!===============================================================================

SUBROUTINE msg_ocns (Foxx)

   implicit none

   !--- input ---
   real         :: Foxx(n_o,nFoxx)  ! ocn input fluxes

   !--- local ---
   integer      :: n                ! generic loop index
   integer      :: m                ! generic loop index
   real         :: f                ! factor wrt area correction
   real         :: gsum             ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_ocns) ',a,i4)"
#ifdef MPI_P
   integer  SHR_MSG_TID_OCN
#endif

!-------------------------------------------------------------------------------
! PURPOSE:
!   send msg to ocn model 
! INPUT:
!   o Foxx_o & various flags contained in control.h 
! OUTPUT:
!   o none
! MESSAGES:
!   o data sent: coupler info-buffer, ocn input fluxes
!-------------------------------------------------------------------------------
   
   if (info_dbug.ge.3) write(6,F01) 'send ocn msg'

   !--- fill info-buffer array ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = stop_now  ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now
   ibuff_c(32) = ashift    ! albedo time shift

   !--- pack msg buffer ---
   irtc_p = shr_sys_irtc()
   call shr_timer_start(timer_send)

!!!$OMP PARALLEL DO PRIVATE(n,f)

!------------wangln 2007-06-24 For MME -------------------------------------
   if( ocn_type(1:3) == 'POP' ) then

!$OMP PARALLEL DO PRIVATE(n,f)
   DO n=1,n_o
         f = area_fact_c2o(n) ! wrt flux conservation & varying cell area calcs

         !--- state variables ---
         buffs(n, 1) = Foxx(n,kFoxx_ifrac )   ! ice fraction
         buffs(n, 2) = Foxx(n,kFoxx_pslv  )   ! sea level pressure

         !--- merged ocn/[atm|ice] fluxes ---
         buffs(n, 3) = Foxx(n,kFoxx_taux ) * f
         buffs(n, 4) = Foxx(n,kFoxx_tauy ) * f
         buffs(n, 5) = Foxx(n,kFoxx_swnet) * f
         buffs(n, 6) = Foxx(n,kFoxx_sen  ) * f
         buffs(n, 7) = Foxx(n,kFoxx_lwup ) * f
         buffs(n, 8) = Foxx(n,kFoxx_lwdn ) * f
         buffs(n, 9) = Foxx(n,kFoxx_melth) * f
         buffs(n,10) = Foxx(n,kFoxx_salt ) * f
         buffs(n,11) =(Foxx(n,kFoxx_rain ) + Foxx(n,kFoxx_snow)) * f
         buffs(n,12) = Foxx(n,kFoxx_evap ) * f
         buffs(n,13) = Foxx(n,kFoxx_meltw) * f
         buffs(n,14) = Foxx(n,kFoxx_roff ) * f
      end do

   else if( ocn_type(1:3) == 'MOM' ) then
 
!$OMP PARALLEL DO PRIVATE(n,f)
      do n=1,n_o
         f = area_fact_c2o(n) ! wrt flux conservation & varying cell area calcs
  
         buffs(n, 1) = Foxx(n,kFoxx_taux ) * f 
         buffs(n, 2) = Foxx(n,kFoxx_tauy ) * f
         buffs(n, 3) = Foxx(n,kFoxx_sen  ) * f
         buffs(n, 4) = Foxx(n,kFoxx_evap ) * f
         buffs(n, 5) = Foxx(n,kFoxx_salt ) * f
         buffs(n, 6) = ( Foxx(n,kFoxx_lwdn ) + Foxx(n,kFoxx_lwup ) ) * f
         buffs(n, 7) = Foxx(n,kFoxx_swnet) * f
         buffs(n, 8) = Foxx(n,kFoxx_rain ) * f
!xjx---20081225         buffs(n, 9) = Foxx(n,kFoxx_snow ) * f
         buffs(n, 9) = 0.0
         buffs(n,10) = Foxx(n,kFoxx_roff ) * f
         buffs(n,11) = Foxx(n,kFoxx_calving) * f ! Calving from Ice
         buffs(n,12) = Foxx(n,kFoxx_pslv ) * f
         if( carbon_flux ) buffs(n,13) = Foxx(n,kFoxx_co2 ) * f
!-wutw --20101101
         buffs(n,14) = Foxx(n,kFoxx_dEdt ) * f

#ifdef HAMOCC
         buffs(n, 15) = Foxx(n,kFoxx_ubot )    ! zonal wind speed
         buffs(n, 16) = Foxx(n,kFoxx_vbot )    ! meridional wind speed
         buffs(n, 17) = Foxx(n,kFoxx_ifrac )   ! ice fraction
         buffs(n, 18) = Foxx(n,kFoxx_sictho )   ! ice fraction
         buffs(n, 19) = Foxx(n,kFoxx_sicsno )   ! ice fraction
         buffs(n, 20) = Foxx(n,kFoxx_n2o ) * f   ! n2o, units: kmol N2 m-3
         buffs(n, 21) = Foxx(n,kFoxx_dms ) * f  ! dms, units: kmol S m-3
#endif
!-------------         
      end do
!$omp  end parallel do
   endif
   if( ocn_type(1:3) == 'MOM' .and. climate_runoff ) then
       do n = 1, n_o
          buffs(n,10) = runoff( n ) * area_fact_c2o(n)
       enddo
   endif

!------------wangln 2007-06-24 For MME -------------------------------------
      

   if (info_dbug >= 2) then
      do m=1,nsflds
         gsum = 0.
         do n=1,n_o
            if (mask_o(n) /= 0) gsum = gsum + buffs(n,m)*area_o(n)*rEarth*rEarth
         enddo
         write(6,F98)'ocn','recv',m,gsum,'in_cpl'
      enddo
   endif

   !--- send msg ---
   irtc_s = shr_sys_irtc()
#ifdef MPI_P
   SHR_MSG_TID_OCN  = ocn_tid
   if(master_task) then
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   endif
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
#endif
   call shr_timer_stop(timer_send)



   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_p,'c->o packing'
      write(6,F99) irtc_s,'c->o sending'
   end if

END SUBROUTINE msg_ocns

!===============================================================================

SUBROUTINE msg_ocnf ()

   implicit none

   !--- local ---
   integer      :: n         ! generic loop index

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_ocnf) ',a,i4)"
#ifdef MPI_P
   integer  SHR_MSG_TID_OCN
#endif

!-------------------------------------------------------------------------------
! PURPOSE:
!   send final msg to ocn model
! INPUT:
!   o various flags contained in control.h 
! OUTPUT:
!   o none
! MESSAGES:
!   o data sent: coupler info-buffer
!-------------------------------------------------------------------------------
   
   write(6,F01) 'send ocn msg (final)'

   !--- fill info-buffer array ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = 1         ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now
   ibuff_c(32) = ashift    ! albedo time shift

   !--- pack msg buffer (with special-values) ---
   buffs = spval

   !--- send msg ---
#ifdef MPI_P
   SHR_MSG_TID_OCN  = ocn_tid
   if(master_task) then
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   endif
#else
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
#endif
   
END SUBROUTINE msg_ocnf

!===============================================================================

END MODULE msg_ocn_mod
