#include <misc.h>

SUBROUTINE nc_Sa(op,fid,F,n,tavg,spv,rcode)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_a,ny_a,*)  ! (in) Sa on atm grid
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: rcode           ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Sa) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Sa_a buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     rcode = nf_inq_dimid(fid,'ni_a'  ,vdims(1))
     rcode = nf_inq_dimid(fid,'nj_a'  ,vdims(2))
     rcode = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Sa_a_z'
     lname = 'bottom level height'
     units = 'm'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sa_a_u'
     lname = 'u velocity'
     units = 'm/s'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sa_a_v'
     lname = 'v velocity'
     units = 'm/s'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sa_a_tbot'
     lname = 'temperature'
     units = 'kelvin'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sa_a_ptem'
     lname = 'potential temperature'
     units = 'kelvin'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sa_a_pbot'
     lname = 'pressure'
     units = 'Pa'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sa_a_pslv'
     lname = 'sea level pressure'
     units = 'Pa'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sa_a_shum'
     lname = 'specific humidity'
     units = 'kg/kg'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sa_a_dens'
     lname = 'air density'
     units = 'kg/m^3'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
!--------------------wangln 2007-11-21 add the P_surf------------------------
     name  = 'Sa_a_p_surf'
     lname = 'Surface pressure'
     units = 'Pa'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
!--------------------wangln 2007-11-21 add the P_surf------------------------
   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_a
     cnt(2) = ny_a
     cnt(3) = 1
     call nc_pvar(fid,'Sa_a_z'    ,beg,cnt,F(:,:,kSa_z   ),rcode)
     call nc_pvar(fid,'Sa_a_u'    ,beg,cnt,F(:,:,kSa_u   ),rcode)
     call nc_pvar(fid,'Sa_a_v'    ,beg,cnt,F(:,:,kSa_v   ),rcode)
     call nc_pvar(fid,'Sa_a_tbot' ,beg,cnt,F(:,:,kSa_tbot),rcode)
     call nc_pvar(fid,'Sa_a_ptem' ,beg,cnt,F(:,:,kSa_ptem),rcode)
     call nc_pvar(fid,'Sa_a_pbot' ,beg,cnt,F(:,:,kSa_pbot),rcode)
     call nc_pvar(fid,'Sa_a_pslv' ,beg,cnt,F(:,:,kSa_pslv),rcode)
     call nc_pvar(fid,'Sa_a_shum' ,beg,cnt,F(:,:,kSa_shum),rcode)
     call nc_pvar(fid,'Sa_a_dens' ,beg,cnt,F(:,:,kSa_dens),rcode)
!--------------------wangln 2007-11-21 add the P_surf------------------------
     call nc_pvar(fid,'Sa_a_p_surf' ,beg,cnt,F(:,:,kSa_p_surf),rcode)
!--------------------wangln 2007-11-21 add the P_surf------------------------
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Si(op,fid,F,n,tavg,spv,rcode)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_i,ny_i,*)  ! (in) Si on ice grid
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: rcode           ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Si) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Si_i buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     rcode = nf_inq_dimid(fid,'ni_i'  ,vdims(1))
     rcode = nf_inq_dimid(fid,'nj_i'  ,vdims(2))
     rcode = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Si_i_t'
     lname = 'temperature'
     units = 'K'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Si_i_ifrac'
     lname = 'ice fraction'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Si_i_avsdr'
     lname = 'albedo: visible/direct'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Si_i_anidr'
     lname = 'albedo: near-infrared/direct'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Si_i_avsdf'
     lname = 'albedo: visible/diffuse'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Si_i_anidf'
     lname = 'albedo: near-infrared/diffuse'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_i
     cnt(2) = ny_i
     cnt(3) = 1
     call nc_pvar(fid,'Si_i_t'    ,beg,cnt,F(1,1,kSi_t    ),rcode)
     call nc_pvar(fid,'Si_i_ifrac',beg,cnt,F(1,1,kSi_ifrac),rcode)
     call nc_pvar(fid,'Si_i_avsdr',beg,cnt,F(1,1,kSi_avsdr),rcode)
     call nc_pvar(fid,'Si_i_anidr',beg,cnt,F(1,1,kSi_anidr),rcode)
     call nc_pvar(fid,'Si_i_avsdf',beg,cnt,F(1,1,kSi_avsdf),rcode)
     call nc_pvar(fid,'Si_i_anidf',beg,cnt,F(1,1,kSi_anidf),rcode)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Sl(op,fid,F,n,tavg,spv,rcode)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_l,ny_l,*)  ! (in) Sl on lnd grid
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: rcode           ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Sl) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Sl_l buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     rcode = nf_inq_dimid(fid,'ni_l'  ,vdims(1))
     rcode = nf_inq_dimid(fid,'nj_l'  ,vdims(2))
     rcode = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Sl_l_t'
     lname = 'temperature'
     units = 'K'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sl_l_snowh'
     lname = 'snow depth'
     units = 'm'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sl_l_avsdr'
     lname = 'albedo: visible/direct'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sl_l_anidr'
     lname = 'albedo: near-infrared/direct'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sl_l_avsdf'
     lname = 'albedo: visible/diffuse'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Sl_l_anidf'
     lname = 'albedo: near-infrared/diffuse'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_l
     cnt(2) = ny_l
     cnt(3) = 1
     call nc_pvar(fid,'Sl_l_t'    ,beg,cnt,F(1,1,kSl_t    ),rcode)
     call nc_pvar(fid,'Sl_l_snowh',beg,cnt,F(1,1,kSl_snowh),rcode)
     call nc_pvar(fid,'Sl_l_avsdr',beg,cnt,F(1,1,kSl_avsdr),rcode)
     call nc_pvar(fid,'Sl_l_anidr',beg,cnt,F(1,1,kSl_anidr),rcode)
     call nc_pvar(fid,'Sl_l_avsdf',beg,cnt,F(1,1,kSl_avsdf),rcode)
     call nc_pvar(fid,'Sl_l_anidf',beg,cnt,F(1,1,kSl_anidf),rcode)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_So(op,fid,F,n,tavg,spv,rcode)

   USE buf_mod                          ! model data buffers
   USE control_mod, only : carbon_flux

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_o,ny_o,*)  ! (in) So on ocn grid
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: rcode           ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_So) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add So_o buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     rcode = nf_inq_dimid(fid,'ni_o'  ,vdims(1))
     rcode = nf_inq_dimid(fid,'nj_o'  ,vdims(2))
     rcode = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'So_o_u'
     lname = 'u velocity'
     units = 'm/s'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'So_o_v'
     lname = 'v velocity'
     units = 'm/s'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'So_o_t'
     lname = 'temperature'
     units = 'K'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'So_o_s'
     lname = 'salinity'
     units = 'g/kg'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'So_o_avsdr'
     lname = 'albedo: visible/direct'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'So_o_anidr'
     lname = 'albedo: near-infrared/direct'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'So_o_avsdf'
     lname = 'albedo: visible/diffuse'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'So_o_anidf'
     lname = 'albedo: near-infrared/diffuse'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
   
     if( carbon_flux ) then
     name  = 'So_o_carbon'
     lname = 'carbon flux'
     units = 'kg/m2s'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     endif

#ifdef HAMOCC
     name  = 'So_o_co2flux'
     lname = 'co2 flux from HAMOCC'
     units = 'kg CO2 m-2 s-1'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'So_o_n2oflux'
     lname = 'n2o flux from HAMOCC'
     units = 'kg N2O m-2 s-1'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'So_o_dmsflux'
     lname = 'dms flux from HAMOCC'
     units = 'kg DMS m-2 s-1'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
#endif

   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_o
     cnt(2) = ny_o
     cnt(3) = 1
     call nc_pvar(fid,'So_o_u'    ,beg,cnt,F(1,1,kSo_u    ),rcode)
     call nc_pvar(fid,'So_o_v'    ,beg,cnt,F(1,1,kSo_v    ),rcode)
     call nc_pvar(fid,'So_o_t'    ,beg,cnt,F(1,1,kSo_t    ),rcode)
     call nc_pvar(fid,'So_o_s'    ,beg,cnt,F(1,1,kSo_s    ),rcode)
     call nc_pvar(fid,'So_o_avsdr',beg,cnt,F(1,1,kSo_avsdr),rcode)
     call nc_pvar(fid,'So_o_anidr',beg,cnt,F(1,1,kSo_anidr),rcode)
     call nc_pvar(fid,'So_o_avsdf',beg,cnt,F(1,1,kSo_avsdf),rcode)
     call nc_pvar(fid,'So_o_anidf',beg,cnt,F(1,1,kSo_anidf),rcode)
     if( carbon_flux ) call nc_pvar(fid,'So_o_carbon',beg,cnt,F(1,1,kSo_carbon),rcode)

#ifdef HAMOCC
     call nc_pvar(fid,'So_o_co2flux',beg,cnt,F(1,1,kSo_co2flux),rcode)
     call nc_pvar(fid,'So_o_n2oflux',beg,cnt,F(1,1,kSo_n2oflux),rcode)
     call nc_pvar(fid,'So_o_dmsflux',beg,cnt,F(1,1,kSo_dmsflux),rcode)
#endif

   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Ss(op,fid,F,n,tavg,spv,rcode)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_a,ny_a,*)  ! (in) Ss on atm grid
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: rcode           ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Ss) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Ss_a buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     rcode = nf_inq_dimid(fid,'ni_a'  ,vdims(1))
     rcode = nf_inq_dimid(fid,'nj_a'  ,vdims(2))
     rcode = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Ss_a_t'
     lname = 'temperature'
     units = 'K'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Ss_a_snowh'
     lname = 'snow depth'
     units = 'm'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Ss_a_avsdr'
     lname = 'albedo: visible/direct'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Ss_a_anidr'
     lname = 'albedo: near-infrared/direct'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Ss_a_avsdf'
     lname = 'albedo: visible/diffuse'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Ss_a_anidf'
     lname = 'albedo: near-infrared/diffuse'
     units = 'unitless'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_a
     cnt(2) = ny_a
     cnt(3) = 1
     call nc_pvar(fid,'Ss_a_t'    ,beg,cnt,F(1,1,kSs_t    ),rcode)
     call nc_pvar(fid,'Ss_a_snowh',beg,cnt,F(1,1,kSs_snowh),rcode)
     call nc_pvar(fid,'Ss_a_avsdr',beg,cnt,F(1,1,kSs_avsdr),rcode)
     call nc_pvar(fid,'Ss_a_anidr',beg,cnt,F(1,1,kSs_anidr),rcode)
     call nc_pvar(fid,'Ss_a_avsdf',beg,cnt,F(1,1,kSs_avsdf),rcode)
     call nc_pvar(fid,'Ss_a_anidf',beg,cnt,F(1,1,kSs_anidf),rcode)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================
