!===============================================================================
! CVS $Id: nc_fields3.F90,v 1.13 2002/01/05 00:07:19 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/nc_fields3.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================
! o These routines can define and add time-variant data to a netCDF file
! o fid must be the file id of an existing netCDF file 
!   1) be open
!   2) have all dimensions defined
!   3) if op="define" then file must be in define-mode
!      if op="put"    then file must NOT be in define-mode
! o data may be instantanious or time-averaged, global attributes 
!   of the time dimension should specify which is the case.
!===============================================================================

SUBROUTINE nc_Faxa(op,fid,F,n,tavg,spv,code)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_a,ny_a,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: code            ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Faia) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Faxa_a buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     code = nf_inq_dimid(fid,'ni_a'  ,vdims(1))
     code = nf_inq_dimid(fid,'nj_a'  ,vdims(2))
     code = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Faxa_a_swnet'
     lname = 'heat flux: long wave down'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)

   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_a
     cnt(2) = ny_a
     cnt(3) = 1
     call nc_pvar(fid,'Faxa_a_swnet',beg,cnt,F(1,1,kFaxa_swnet),code)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Faia(op,fid,F,n,tavg,spv,code)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_a,ny_a,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: code            ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Faia) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Faia_a buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     code = nf_inq_dimid(fid,'ni_a'  ,vdims(1))
     code = nf_inq_dimid(fid,'nj_a'  ,vdims(2))
     code = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Faia_a_rain'
     lname = 'water flux: rain'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faia_a_snow'
     lname = 'water flux: snow'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faia_a_lwdn'
     lname = 'heat flux: long wave down'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faia_a_swndr'
     lname = 'heat flux: nir direct solar rad'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faia_a_swvdr'
     lname = 'heat flux: vis direct short wave'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faia_a_swndf'
     lname = 'heat flux: nir diffuse solar rad'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faia_a_swvdf'
     lname = 'heat flux: vis diffuse solar rad'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)

   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_a
     cnt(2) = ny_a
     cnt(3) = 1
     call nc_pvar(fid,'Faia_a_rain' ,beg,cnt,F(1,1,kFaia_rain ),code)
     call nc_pvar(fid,'Faia_a_snow' ,beg,cnt,F(1,1,kFaia_snow ),code)
     call nc_pvar(fid,'Faia_a_lwdn' ,beg,cnt,F(1,1,kFaia_lwdn ),code)
     call nc_pvar(fid,'Faia_a_swndr',beg,cnt,F(1,1,kFaia_swndr),code)
     call nc_pvar(fid,'Faia_a_swvdr',beg,cnt,F(1,1,kFaia_swvdr),code)
     call nc_pvar(fid,'Faia_a_swndf',beg,cnt,F(1,1,kFaia_swndf),code)
     call nc_pvar(fid,'Faia_a_swvdf',beg,cnt,F(1,1,kFaia_swvdf),code)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Faii(op,fid,F,n,tavg,spv,code)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_i,ny_i,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: code            ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Faii) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Faii_i buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     code = nf_inq_dimid(fid,'ni_i'  ,vdims(1))
     code = nf_inq_dimid(fid,'nj_i'  ,vdims(2))
     code = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Faii_i_taux'
     lname = 'momentum flux: zonal wind stress'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faii_i_tauy'
     lname = 'momentum flux: meridional wind stress'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faii_i_lat'
     lname = 'heat flux: latent'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faii_i_sen'
     lname = 'heat flux: sensible'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faii_i_lwup'
     lname = 'heat flux: long wave upward'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faii_i_evap'
     lname = 'water flux: evaporation'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faii_i_tref'
     lname = '2m reference temperature'
     units = 'Kelvin'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faii_i_swnet'
     lname = 'heat flux: shortwave absorbed'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_i
     cnt(2) = ny_i
     cnt(3) = 1
     call nc_pvar(fid,'Faii_i_taux' ,beg,cnt,F(1,1,kFaii_taux ),code)
     call nc_pvar(fid,'Faii_i_tauy' ,beg,cnt,F(1,1,kFaii_tauy ),code)
     call nc_pvar(fid,'Faii_i_lat ' ,beg,cnt,F(1,1,kFaii_lat  ),code)
     call nc_pvar(fid,'Faii_i_sen ' ,beg,cnt,F(1,1,kFaii_sen  ),code)
     call nc_pvar(fid,'Faii_i_lwup' ,beg,cnt,F(1,1,kFaii_lwup ),code)
     call nc_pvar(fid,'Faii_i_evap' ,beg,cnt,F(1,1,kFaii_evap ),code)
     call nc_pvar(fid,'Faii_i_tref' ,beg,cnt,F(1,1,kFaii_tref ),code)
     call nc_pvar(fid,'Faii_i_swnet',beg,cnt,F(1,1,kFaii_swnet),code)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Fala(op,fid,F,n,tavg,spv,code)

   USE buf_mod                          ! model data buffers
   USE control_mod, only : carbon_flux

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_a,ny_a,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: code            ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Fala) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Fala_a buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     code = nf_inq_dimid(fid,'ni_a'  ,vdims(1))
     code = nf_inq_dimid(fid,'nj_a'  ,vdims(2))
     code = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Fala_a_lwdn'
     lname = 'heat flux: long wave down'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)

     name  = 'Fala_a_rainc'
     lname = 'water flux: precip, liquid, convective'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fala_a_rainl'
     lname = 'water flux: precip, liquid, large-scale'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)

     name  = 'Fala_a_snowc'
     lname = 'water flux: precip, frozen, convective'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fala_a_snowl'
     lname = 'water flux: precip, frozen, large-scale'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)

     name  = 'Fala_a_swndr'
     lname = 'heat flux: downward short wave, near-infrared, direct'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fala_a_swvdr'
     lname = 'heat flux: downward short wave, visible, direct'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fala_a_swndf'
     lname = 'heat flux: downward short wave, near-infrared, diffuse'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fala_a_swvdf'
     lname = 'heat flux: downward short wave, visible, diffuse'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
   
     if( carbon_flux ) then
         name  = 'Fala_a_co2'
         lname = ' Co2 concentration in Atm for lnd'
         units = 'ppm'
         call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     endif

   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_a
     cnt(2) = ny_a
     cnt(3) = 1
     call nc_pvar(fid,'Fala_a_lwdn' ,beg,cnt,F(1,1,kFala_lwdn ),code)
     call nc_pvar(fid,'Fala_a_rainc',beg,cnt,F(1,1,kFala_rainc),code)
     call nc_pvar(fid,'Fala_a_rainl',beg,cnt,F(1,1,kFala_rainl),code)
     call nc_pvar(fid,'Fala_a_snowc',beg,cnt,F(1,1,kFala_snowc),code)
     call nc_pvar(fid,'Fala_a_snowl',beg,cnt,F(1,1,kFala_snowl),code)
     call nc_pvar(fid,'Fala_a_swndr',beg,cnt,F(1,1,kFala_swndr),code)
     call nc_pvar(fid,'Fala_a_swvdr',beg,cnt,F(1,1,kFala_swvdr),code)
     call nc_pvar(fid,'Fala_a_swndf',beg,cnt,F(1,1,kFala_swndf),code)
     call nc_pvar(fid,'Fala_a_swvdf',beg,cnt,F(1,1,kFala_swvdf),code)
     if( carbon_flux )call nc_pvar(fid,'Fala_a_co2',beg,cnt,F(1,1,kFala_co2),code)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Faoa(op,fid,F,n,tavg,spv,code)

   USE buf_mod                          ! model data buffers
   USE control_mod, only : carbon_flux

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_a,ny_a,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: code            ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Faoa) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Faoa_a buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     code = nf_inq_dimid(fid,'ni_a'  ,vdims(1))
     code = nf_inq_dimid(fid,'nj_a'  ,vdims(2))
     code = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Faoa_a_swndr'
     lname = 'heat flux: downward short wave, near-infrared, direct'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faoa_a_swvdr'
     lname = 'heat flux: downward short wave, visible, direct'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faoa_a_swndf'
     lname = 'heat flux: downward short wave, near-infrared, diffuse'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faoa_a_swvdf'
     lname = 'heat flux: downward short wave, visible, diffuse'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faoa_a_lwdn'
     lname = 'heat flux: long wave down'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faoa_a_rain'
     lname = 'water flux: rain'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faoa_a_snow'
     lname = 'water flux: snow'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     if( carbon_flux ) then
         name  = 'Faoa_a_co2'
         lname = 'co2 concentration in atm for Ocn'
         units = 'ppm'
         call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     endif

   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_a
     cnt(2) = ny_a
     cnt(3) = 1
     call nc_pvar(fid,'Faoa_a_swndr',beg,cnt,F(1,1,kFaoa_swndr),code)
     call nc_pvar(fid,'Faoa_a_swvdr',beg,cnt,F(1,1,kFaoa_swvdr),code)
     call nc_pvar(fid,'Faoa_a_swndf',beg,cnt,F(1,1,kFaoa_swndf),code)
     call nc_pvar(fid,'Faoa_a_swvdf',beg,cnt,F(1,1,kFaoa_swvdf),code)
     call nc_pvar(fid,'Faoa_a_lwdn' ,beg,cnt,F(1,1,kFaoa_lwdn ),code)
     call nc_pvar(fid,'Faoa_a_rain' ,beg,cnt,F(1,1,kFaoa_rain ),code)
     call nc_pvar(fid,'Faoa_a_snow' ,beg,cnt,F(1,1,kFaoa_snow ),code)
     if( carbon_flux ) call nc_pvar(fid,'Faoa_a_co2' ,beg,cnt,F(1,1,kFaoa_co2 ),code)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Fall(op,fid,F,n,tavg,spv,code)

   USE buf_mod                          ! model data buffers
   USE control_mod, only : carbon_flux

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_l,ny_l,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: code            ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Fall) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Fall_l buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     code = nf_inq_dimid(fid,'ni_l'  ,vdims(1))
     code = nf_inq_dimid(fid,'nj_l'  ,vdims(2))
     code = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Fall_l_taux'
     lname = 'surface stress: zonal'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fall_l_tauy'
     lname = 'surface stress: meridional'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fall_l_lat'
     lname = 'heat flux: latent'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fall_l_sen'
     lname = 'heat flux: sensible'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fall_l_lwup'
     lname = 'heat flux: longwave up'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fall_l_evap'
     lname = 'water flux: evaporation'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fall_l_tref'
     lname = 'diagnostic: 2m reference air temperature'
     units = 'K'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Fall_l_swnet'
     lname = 'heat flux: shortwave absorbed'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     if( carbon_flux ) then
         name  = 'Fall_l_carbon'
         lname = ' carbon flux in lnd '
         units = 'kg/m^2s'
         call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     endif

   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_l
     cnt(2) = ny_l
     cnt(3) = 1
     call nc_pvar(fid,'Fall_l_taux' ,beg,cnt,F(1,1,kFall_taux ),code)
     call nc_pvar(fid,'Fall_l_tauy' ,beg,cnt,F(1,1,kFall_tauy ),code)
     call nc_pvar(fid,'Fall_l_lat'  ,beg,cnt,F(1,1,kFall_lat  ),code)
     call nc_pvar(fid,'Fall_l_sen'  ,beg,cnt,F(1,1,kFall_sen  ),code)
     call nc_pvar(fid,'Fall_l_lwup' ,beg,cnt,F(1,1,kFall_lwup ),code)
     call nc_pvar(fid,'Fall_l_evap' ,beg,cnt,F(1,1,kFall_evap ),code)
     call nc_pvar(fid,'Fall_l_tref' ,beg,cnt,F(1,1,kFall_tref ),code)
     call nc_pvar(fid,'Fall_l_swnet',beg,cnt,F(1,1,kFall_swnet),code)
     if( carbon_flux ) call nc_pvar(fid,'Fall_l_carbon',beg,cnt,F(1,1,kFall_carbon),code)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Fioi(op,fid,F,n,tavg,spv,rcode)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_i,ny_i,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: rcode           ! (out) return rcode

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Fioi) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Fioi_i buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     rcode = nf_inq_dimid(fid,'ni_i'  ,vdims(1))
     rcode = nf_inq_dimid(fid,'nj_i'  ,vdims(2))
     rcode = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Fioi_i_swnet'
     lname = 'heat flux: shortwave penetration'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioi_i_melth'
     lname = 'heat flux: ocean heat used to melt ice'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioi_i_meltw'
     lname = 'water flux: water from melting ice'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioi_i_salt'
     lname = 'salt flux: salt from melting ice'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioi_i_taux'
     lname = 'momentum flux: zonal stress at ice/ocn interface'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioi_i_tauy'
     lname = 'momentum flux: meridional stress at ice/ocn interface'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioi_i_sen'
     lname = 'heat flux: ice heat used to ocn'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioi_i_evap'
     lname = 'heat flux: ice heat used to ocn'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioi_i_psurf'
     lname = ' sureflux: ice pressure used to ocn'
     units = 'hPa/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_i
     cnt(2) = ny_i
     cnt(3) = 1
     call nc_pvar(fid,'Fioi_i_swnet',beg,cnt,F(1,1,kFioi_swnet),rcode)
     call nc_pvar(fid,'Fioi_i_melth',beg,cnt,F(1,1,kFioi_melth),rcode)
     call nc_pvar(fid,'Fioi_i_meltw',beg,cnt,F(1,1,kFioi_meltw),rcode)
     call nc_pvar(fid,'Fioi_i_salt ',beg,cnt,F(1,1,kFioi_salt ),rcode)
     call nc_pvar(fid,'Fioi_i_taux ',beg,cnt,F(1,1,kFioi_taux ),rcode)
     call nc_pvar(fid,'Fioi_i_tauy ',beg,cnt,F(1,1,kFioi_tauy ),rcode)
     call nc_pvar(fid,'Fioi_i_sen  ',beg,cnt,F(1,1,kFioi_sen  ),rcode)
     call nc_pvar(fid,'Fioi_i_evap ',beg,cnt,F(1,1,kFioi_evap ),rcode)
     call nc_pvar(fid,'Fioi_i_psurf',beg,cnt,F(1,1,kFioi_psurf),rcode)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Fioo(op,fid,F,n,tavg,spv,rcode)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_o,ny_o,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: rcode           ! (out) return rcode

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Fioo) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Fioo_o buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     rcode = nf_inq_dimid(fid,'ni_o'  ,vdims(1))
     rcode = nf_inq_dimid(fid,'nj_o'  ,vdims(2))
     rcode = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Fioo_o_dhdx'
     lname = 'ocean surface slope: zonal'
     units = 'm/m'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioo_o_dhdy'
     lname = 'ocean surface slope: meridional'
     units = 'm/m'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Fioo_o_q'
     lname = 'q<0 = heat of fusion, q>0 = melting potential'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_o
     cnt(2) = ny_o
     cnt(3) = 1
     call nc_pvar(fid,'Fioo_o_dhdx',beg,cnt,F(1,1,kFioo_dhdx ),rcode)
     call nc_pvar(fid,'Fioo_o_dhdy',beg,cnt,F(1,1,kFioo_dhdy ),rcode)
     call nc_pvar(fid,'Fioo_o_q'   ,beg,cnt,F(1,1,kFioo_q    ),rcode)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_roff(op,fid,F,n,tavg,spv,rcode)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(*)            ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: rcode           ! (out) return rcode

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_roff) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add runoff_r buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     rcode = nf_inq_dimid(fid,'ni_r'  ,vdims(1))
     rcode = nf_inq_dimid(fid,'nj_r'  ,vdims(2))
     rcode = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'runoff_r'
     lname = 'runoff from land into ocean'
     units = 'kg/s/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_r
     cnt(2) = ny_r
     cnt(3) = 1
     call nc_pvar(fid,'runoff_r',beg,cnt,F,rcode)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================
