#include <misc.h>

!===============================================================================

SUBROUTINE nc_Faoc(op,fid,F,n,tavg,spv,rcode)

   USE buf_mod                          ! model data buffers
   USE control_mod, only : carbon_flux

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_o,ny_o,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: rcode           ! (out) return rcode

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Faoc) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Faoc_o buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     rcode = nf_inq_dimid(fid,'ni_o'  ,vdims(1))
     rcode = nf_inq_dimid(fid,'nj_o'  ,vdims(2))
     rcode = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Faoc_o_taux'
     lname = 'surface stress: zonal'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Faoc_o_tauy'
     lname = 'surface stress: meridional'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Faoc_o_lat'
     lname = 'heat flux: latent'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Faoc_o_sen'
     lname = 'heat flux: sensible'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Faoc_o_lwup'
     lname = 'heat flux: longwave up'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Faoc_o_swnet'
     lname = 'heat flux: sw absorbed'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Faoc_o_evap'
     lname = 'water flux: evaporation'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     name  = 'Faoc_o_tref'
     lname = 'diagnostic: 2m reference air temperature'
     units = 'K'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     if( carbon_flux ) then
         name  = 'Faoc_o_carbon'
         lname = ' carbon flux in ocn'
         units = 'kg/m^2s'
         call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
     endif

#ifdef HAMOCC
        name  = 'Faoc_o_co2flux'
        lname = ' co2 flux from HAMOCC'
        units = 'kg CO2 m-2 s-1'
        call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
        name  = 'Faoc_o_n2oflux'
        lname = ' n2o flux from HAMOCC'
        units = 'kg N2O m-2 s-1'
        call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
        name  = 'Faoc_o_dmsflux'
        lname = ' dms flux from HAMOCC'
        units = 'kg DMS m-2 s-1'
        call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,rcode)
#endif

   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_o
     cnt(2) = ny_o
     cnt(3) = 1
     call nc_pvar(fid,'Faoc_o_taux',beg,cnt,F(1,1,kFaoc_taux ),rcode)
     call nc_pvar(fid,'Faoc_o_tauy',beg,cnt,F(1,1,kFaoc_tauy ),rcode)
     call nc_pvar(fid,'Faoc_o_lat' ,beg,cnt,F(1,1,kFaoc_lat  ),rcode)
     call nc_pvar(fid,'Faoc_o_sen' ,beg,cnt,F(1,1,kFaoc_sen  ),rcode)
     call nc_pvar(fid,'Faoc_o_lwup',beg,cnt,F(1,1,kFaoc_lwup ),rcode)
     call nc_pvar(fid,'Faoc_o_swnet',beg,cnt,F(1,1,kFaoc_swnet ),rcode)
     call nc_pvar(fid,'Faoc_o_evap',beg,cnt,F(1,1,kFaoc_evap ),rcode)
     call nc_pvar(fid,'Faoc_o_tref',beg,cnt,F(1,1,kFaoc_tref ),rcode)
     if( carbon_flux ) call nc_pvar(fid,'Faoc_o_carbon',beg,cnt,F(1,1,kFaoc_carbon ),rcode)
#ifdef HAMOCC
         call nc_pvar(fid,'Faoc_o_co2flux',beg,cnt,F(1,1,kFaoc_co2flux ),rcode)
         call nc_pvar(fid,'Faoc_o_n2oflux',beg,cnt,F(1,1,kFaoc_n2oflux ),rcode)
         call nc_pvar(fid,'Faoc_o_dmsflux',beg,cnt,F(1,1,kFaoc_dmsflux ),rcode)
#endif

   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================
