!===============================================================================
! CVS $Id: nc_header.F90,v 1.8 2002/01/09 00:31:26 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/nc_header.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

SUBROUTINE nc_header(fn,title,rcode)

   USE buf_mod                   ! model data buffers
   USE map_mod                   ! mapping data (eg. fl_l)

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len=*)  :: fn       ! (in) name of nc file
   character(len=*)  :: title    ! (in) file title/ID string
   integer           :: rcode    ! (out) error code

   !----- local -----
   integer           :: strlen   ! string length
   character(len=80) :: str      ! restart file ID string
   character(len= 8) :: datestr  ! current date
   character(len=10) :: timestr  ! current time
   integer           :: start(3) ! vector of starting indicies
   integer           :: count(3) ! vector of edge lengths
   integer           :: vdid(3)  ! vector of dimension IDs
   integer           :: vid      ! variable ID
   integer           :: did      ! dimension ID
   integer           :: fid      ! file ID

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_header) ',4a)"
   character(len=*),parameter :: F03 = "('Data created ',a,1x,a)"
   character(len=*),parameter :: F05 = "('(nc_header) code ', &
   &                                   i4, ' creating netCDF file ',a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   create a new netCDF file and add header (time invariant) data
!-------------------------------------------------------------------------------

   !----------------------------------------------------------------------------
   ! create new file 
   !----------------------------------------------------------------------------
   strlen=len_trim(Fn)
   rcode = nf_create(Fn(1:strlen),NF_CLOBBER,fid)
   write(6,F05) rcode,Fn(1:strlen)

   !----------------------------------------------------------------------------
   ! add global attributes
   !----------------------------------------------------------------------------
   str   = title
   strlen=len_trim(str)
   rcode = nf_put_att_text(fid,NF_GLOBAL,'title'      ,strlen,str)
 
   call date_and_time(datestr,timestr)
   str = 'File created: '                                            &
   &     //datestr(1:4)//'-'//datestr(5:6)//'-'//datestr(7:8)//' '   &
   &     //timestr(1:2)//':'//timestr(3:4)//':'//timestr(5:6)
   strlen=len_trim(str)
   rcode = nf_put_att_text(fid,NF_GLOBAL,'history'    ,strlen,str)
 
   str   ='CF-1.0'
   strlen=len_trim(str)
   rcode = nf_put_att_text(fid,NF_GLOBAL,'Conventions',strlen,str)
 
   str   ='NCAR/CSM Flux Coupler (CVS Tag $Name: ccsm2_0_beta58 $)'
   strlen=len_trim(str)
   rcode = nf_put_att_text(fid,NF_GLOBAL,'source'     ,strlen,str)

   !----------------------------------------------------------------------------
   !  define dimension data
   !----------------------------------------------------------------------------
   rcode = nf_def_dim(fid,  'n_a',  n_a, did) ! atm domain dimensions
   rcode = nf_def_dim(fid, 'ni_a', nx_a, did)  
   rcode = nf_def_dim(fid, 'nj_a', ny_a, did) 
   rcode = nf_def_dim(fid,  'n_i',  n_i, did) ! ice domain dimensions
   rcode = nf_def_dim(fid, 'ni_i', nx_i, did)
   rcode = nf_def_dim(fid, 'nj_i', ny_i, did)
   rcode = nf_def_dim(fid,  'n_l',  n_l, did) ! lnd domain dimensions
   rcode = nf_def_dim(fid, 'ni_l', nx_l, did)
   rcode = nf_def_dim(fid, 'nj_l', ny_l, did)
   rcode = nf_def_dim(fid,  'n_o',  n_o, did) ! ocn domain dimensions
   rcode = nf_def_dim(fid, 'ni_o', nx_o, did)
   rcode = nf_def_dim(fid, 'nj_o', ny_o, did)
   rcode = nf_def_dim(fid, 'nv'  ,   4 , did)
   rcode = nf_def_dim(fid, 'ni_r', nx_r, did) ! roff domain dimensions
   rcode = nf_def_dim(fid, 'nj_r', ny_r, did)
   rcode = nf_def_dim(fid, 'nv'  ,   4 , did) ! 4 verticies per cell

   !----------------------------------------------------------------------------
   !  define domain data
   !----------------------------------------------------------------------------
   
   !--- atm ---------------------------------------
   rcode = nf_inq_dimid(fid,'ni_a',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_a',vdid(2))

!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_def_var  (fid,'ni_a', NF_FLOAT , 1, vdid(1), vid)
   str   = 'longitude of grid cell center (atm)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_E'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'ni_a'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_def_var  (fid,'nj_a', NF_FLOAT , 1, vdid(2), vid)
   str   = 'latitude of grid cell center (atm)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_N'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'nj_a'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_def_var  (fid,'xc_a',NF_FLOAT ,2,vdid,vid)
   str   = 'longitude of grid cell center (atm)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees east'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'xv_a'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_def_var  (fid,'yc_a',NF_FLOAT ,2,vdid,vid)
   str   = 'latitude of grid cell center (atm)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees north'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'yv_a'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'nv'  ,vdid(1))
   rcode = nf_inq_dimid(fid,'ni_a',vdid(2))
   rcode = nf_inq_dimid(fid,'nj_a',vdid(3))

   rcode = nf_def_var  (fid,'xv_a',NF_FLOAT ,3,vdid,vid)
   str   = 'longitude of grid cell verticies (atm)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees east'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_def_var  (fid,'yv_a',NF_FLOAT ,3,vdid,vid)
   str   = 'latitude of grid cell verticies (atm)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees north'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'ni_a',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_a',vdid(2))

   rcode = nf_def_var  (fid,'mask_a',NF_INT ,2,vdid,vid)
   str   = 'domain mask (atm)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)

   rcode = nf_def_var  (fid,'area_a',NF_FLOAT  ,2,vdid,vid)
   str   = 'grid cell area (atm)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'radians^2'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   !--- ice ---------------------------------------
   rcode = nf_inq_dimid(fid,'ni_i',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_i',vdid(2))


!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_def_var  (fid,'ni_i', NF_FLOAT , 1, vdid(1), vid)
   str   = 'longitude of grid cell center (ice)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_E'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'ni_i'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_def_var  (fid,'nj_i', NF_FLOAT , 1, vdid(2), vid)
   str   = 'latitude of grid cell center (ice)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_N'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'nj_i'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_def_var  (fid,'xc_i',NF_FLOAT ,2,vdid,vid)
   str   = 'longitude of grid cell center (ice)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees east'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'xv_i'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_def_var  (fid,'yc_i',NF_FLOAT ,2,vdid,vid)
   str   = 'latitude of grid cell center (ice)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees north'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'yv_i'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'nv'  ,vdid(1))
   rcode = nf_inq_dimid(fid,'ni_i',vdid(2))
   rcode = nf_inq_dimid(fid,'nj_i',vdid(3))

   rcode = nf_def_var  (fid,'xv_i',NF_FLOAT ,3,vdid,vid)
   str   = 'longitude of grid cell verticies (ice)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees east'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_def_var  (fid,'yv_i',NF_FLOAT ,3,vdid,vid)
   str   = 'latitude of grid cell verticies (ice)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees north'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'ni_i',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_i',vdid(2))

   rcode = nf_def_var  (fid,'mask_i',NF_INT ,2,vdid,vid)
   str   = 'domain mask (ice)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)

   rcode = nf_def_var  (fid,'area_i',NF_FLOAT  ,2,vdid,vid)
   str   = 'grid cell area (ice)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'radians^2'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   !--- lnd ---------------------------------------
   rcode = nf_inq_dimid(fid,'ni_l',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_l',vdid(2))


!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_def_var  (fid,'ni_l', NF_FLOAT , 1, vdid(1), vid)
   str   = 'longitude of grid cell center (lnd)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_E'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'ni_i'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_def_var  (fid,'nj_l', NF_FLOAT , 1, vdid(2), vid)
   str   = 'latitude of grid cell center (lnd)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_N'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'nj_l'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_def_var  (fid,'xc_l',NF_FLOAT ,2,vdid,vid)
   str   = 'longitude of grid cell center (lnd)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees east'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'xv_l'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_def_var  (fid,'yc_l',NF_FLOAT ,2,vdid,vid)
   str   = 'latitude of grid cell center (lnd)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees north'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'yv_l'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'nv'  ,vdid(1))
   rcode = nf_inq_dimid(fid,'ni_l',vdid(2))
   rcode = nf_inq_dimid(fid,'nj_l',vdid(3))

   rcode = nf_def_var  (fid,'xv_l',NF_FLOAT ,3,vdid,vid)
   str   = 'longitude of grid cell verticies (lnd)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees east'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_def_var  (fid,'yv_l',NF_FLOAT ,3,vdid,vid)
   str   = 'latitude of grid cell verticies (lnd)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees north'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'ni_l',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_l',vdid(2))

   rcode = nf_def_var  (fid,'mask_l',NF_INT ,2,vdid,vid)
   str   = 'domain mask (lnd)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)

   rcode = nf_def_var  (fid,'area_l',NF_FLOAT  ,2,vdid,vid)
   str   = 'grid cell area (lnd)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'radians^2'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_def_var  (fid,'fl_l',NF_FLOAT  ,2,vdid,vid)
   str   = 'fraction of grid cell that is active (lnd)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)

   !--- ocn ---------------------------------------
   rcode = nf_inq_dimid(fid,'ni_o',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_o',vdid(2))


!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_def_var  (fid,'ni_o', NF_FLOAT , 1, vdid(1), vid)
   str   = 'longitude of grid cell center (ocn)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_E'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'ni_o'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_def_var  (fid,'nj_o', NF_FLOAT , 1, vdid(2), vid)
   str   = 'latitude of grid cell center (ocn)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_N'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'nj_o'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_def_var  (fid,'xc_o',NF_FLOAT ,2,vdid,vid)
   str   = 'longitude of grid cell center (ocn)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees east'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'xv_o'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_def_var  (fid,'yc_o',NF_FLOAT ,2,vdid,vid)
   str   = 'latitude of grid cell center (ocn)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees north'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'yv_o'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'nv'  ,vdid(1))
   rcode = nf_inq_dimid(fid,'ni_o',vdid(2))
   rcode = nf_inq_dimid(fid,'nj_o',vdid(3))

   rcode = nf_def_var  (fid,'xv_o',NF_FLOAT ,3,vdid,vid)
   str   = 'longitude of grid cell verticies (ocn)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees east'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_def_var  (fid,'yv_o',NF_FLOAT ,3,vdid,vid)
   str   = 'latitude of grid cell verticies (ocn)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees north'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'ni_o',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_o',vdid(2))

   rcode = nf_def_var  (fid,'mask_o',NF_INT ,2,vdid,vid)
   str   = 'domain mask (ocn)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)

   rcode = nf_def_var  (fid,'area_o',NF_FLOAT  ,2,vdid,vid)
   str   = 'grid cell area (ocn)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'radians^2'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   !--- roff --------------------------------------
   rcode = nf_inq_dimid(fid,'ni_r',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_r',vdid(2))


!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_def_var  (fid,'ni_r', NF_FLOAT , 1, vdid(1), vid)
   str   = 'longitude of grid cell center (runoff)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_E'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'ni_r'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)

   rcode = nf_def_var  (fid,'nj_r', NF_FLOAT , 1, vdid(2), vid)
   str   = 'latitude of grid cell center (runoff)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees_N'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)
   str   = 'nj_r'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_def_var  (fid,'xc_r',NF_FLOAT ,2,vdid,vid)
   str   = 'longitude of grid cell center (roff)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees east'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_def_var  (fid,'yc_r',NF_FLOAT ,2,vdid,vid)
   str   = 'latitude of grid cell center (roff)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'degrees north'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'nv'  ,vdid(1))
   rcode = nf_inq_dimid(fid,'ni_r',vdid(2))
   rcode = nf_inq_dimid(fid,'nj_r',vdid(3))

!  rcode = nf_def_var  (fid,'xv_r',NF_FLOAT ,3,vdid,vid)
!  str   = 'longitude of grid cell verticies (roff)'
!  rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
!  str   = 'degrees east'
!  rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

!  rcode = nf_def_var  (fid,'yv_r',NF_FLOAT ,3,vdid,vid)
!  str   = 'latitude of grid cell verticies (roff)'
!  rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
!  str   = 'degrees north'
!  rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   rcode = nf_inq_dimid(fid,'ni_r',vdid(1))
   rcode = nf_inq_dimid(fid,'nj_r',vdid(2))

   rcode = nf_def_var  (fid,'mask_r',NF_INT ,2,vdid,vid)
   str   = 'domain mask (roff)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)

   rcode = nf_def_var  (fid,'area_r',NF_FLOAT  ,2,vdid,vid)
   str   = 'grid cell area (roff)'
   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(str),str)
   str   = 'radians^2'
   rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(str),str)

   !----------------------------------------------------------------------------
   !  put domain data
   !----------------------------------------------------------------------------

   rcode = nf_enddef(fid) ! end "define" mode

   !--- atm ---------------------------------------
!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_inq_varid     (fid,  'ni_a', vid)
   rcode = nf_put_var_double(fid,  vid   , ni_a)
   rcode = nf_inq_varid     (fid,  'nj_a', vid)
   rcode = nf_put_var_double(fid,  vid   , nj_a)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_inq_varid     (fid,  'xc_a', vid)
   rcode = nf_put_var_double(fid,  vid   , xc_a)
   rcode = nf_inq_varid     (fid,  'yc_a', vid)
   rcode = nf_put_var_double(fid,  vid   , yc_a)
   rcode = nf_inq_varid     (fid,  'xv_a', vid)
   rcode = nf_put_var_double(fid,  vid   , xv_a)
   rcode = nf_inq_varid     (fid,  'yv_a', vid)
   rcode = nf_put_var_double(fid,  vid   , yv_a)
   rcode = nf_inq_varid     (fid,'mask_a', vid)
   rcode = nf_put_var_int   (fid,  vid   , mask_a)
   rcode = nf_inq_varid     (fid,'area_a', vid)
   rcode = nf_put_var_double(fid,  vid   , domain_a%area)

   !--- ice ---------------------------------------
!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_inq_varid     (fid,  'ni_i', vid)
   rcode = nf_put_var_double(fid,  vid   , ni_i)
   rcode = nf_inq_varid     (fid,  'nj_i', vid)
   rcode = nf_put_var_double(fid,  vid   , nj_i)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_inq_varid     (fid,  'xc_i', vid)
   rcode = nf_put_var_double(fid,  vid   , xc_i)
   rcode = nf_inq_varid     (fid,  'yc_i', vid)
   rcode = nf_put_var_double(fid,  vid   , yc_i)
   rcode = nf_inq_varid     (fid,  'xv_i', vid)
   rcode = nf_put_var_double(fid,  vid   , xv_i)
   rcode = nf_inq_varid     (fid,  'yv_i', vid)
   rcode = nf_put_var_double(fid,  vid   , yv_i)
   rcode = nf_inq_varid     (fid,'mask_i', vid)
   rcode = nf_put_var_int   (fid,  vid   , mask_i)
   rcode = nf_inq_varid     (fid,'area_i', vid)
   rcode = nf_put_var_double(fid,  vid   , domain_i%area)

   !--- lnd ---------------------------------------
!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_inq_varid     (fid,  'ni_l', vid)
   rcode = nf_put_var_double(fid,  vid   , ni_l)
   rcode = nf_inq_varid     (fid,  'nj_l', vid)
   rcode = nf_put_var_double(fid,  vid   , nj_l)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_inq_varid     (fid,  'xc_l', vid)
   rcode = nf_put_var_double(fid,  vid   , xc_l)
   rcode = nf_inq_varid     (fid,  'yc_l', vid)
   rcode = nf_put_var_double(fid,  vid   , yc_l)
   rcode = nf_inq_varid     (fid,  'xv_l', vid)
   rcode = nf_put_var_double(fid,  vid   , xv_l)
   rcode = nf_inq_varid     (fid,  'yv_l', vid)
   rcode = nf_put_var_double(fid,  vid   , yv_l)
   rcode = nf_inq_varid     (fid,'mask_l', vid)
   rcode = nf_put_var_int   (fid,  vid   , mask_l)
   rcode = nf_inq_varid     (fid,'area_l', vid)
   rcode = nf_put_var_double(fid,  vid   , domain_l%area)
   rcode = nf_inq_varid     (fid,  'fl_l', vid)
   rcode = nf_put_var_double(fid,  vid   , fl_l)

   !--- ocn ---------------------------------------
!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_inq_varid     (fid,  'ni_o', vid)
   rcode = nf_put_var_double(fid,  vid   , ni_o)
   rcode = nf_inq_varid     (fid,  'nj_o', vid)
   rcode = nf_put_var_double(fid,  vid   , nj_o)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_inq_varid     (fid,  'xc_o', vid)
   rcode = nf_put_var_double(fid,  vid   , xc_o)
   rcode = nf_inq_varid     (fid,  'yc_o', vid)
   rcode = nf_put_var_double(fid,  vid   , yc_o)
   rcode = nf_inq_varid     (fid,  'xv_o', vid)
   rcode = nf_put_var_double(fid,  vid   , xv_o)
   rcode = nf_inq_varid     (fid,  'yv_o', vid)
   rcode = nf_put_var_double(fid,  vid   , yv_o)
   rcode = nf_inq_varid     (fid,'mask_o', vid)
   rcode = nf_put_var_int   (fid,  vid   , mask_o)
   rcode = nf_inq_varid     (fid,'area_o', vid)
   rcode = nf_put_var_double(fid,  vid   , domain_o%area)

   !--- roff --------------------------------------
!!=============wangln 2007-08-15 For NC File  ==================================
   rcode = nf_inq_varid     (fid,  'ni_r', vid)
   rcode = nf_put_var_double(fid,  vid   , ni_r)
   rcode = nf_inq_varid     (fid,  'nj_r', vid)
   rcode = nf_put_var_double(fid,  vid   , nj_r)
!!=============wangln 2007-08-15 For NC File  ==================================

   rcode = nf_inq_varid     (fid,  'xc_r', vid)
   rcode = nf_put_var_double(fid,  vid   , xc_r)
   rcode = nf_inq_varid     (fid,  'yc_r', vid)
   rcode = nf_put_var_double(fid,  vid   , yc_r)
!  rcode = nf_inq_varid     (fid,  'xv_r', vid)
!  rcode = nf_put_var_double(fid,  vid   , xv_r)
!  rcode = nf_inq_varid     (fid,  'yv_r', vid)
!  rcode = nf_put_var_double(fid,  vid   , yv_r)
   rcode = nf_inq_varid     (fid,'mask_r', vid)
   rcode = nf_put_var_int   (fid,  vid   , mask_r)
   rcode = nf_inq_varid     (fid,'area_r', vid)
   rcode = nf_put_var_double(fid,  vid   , domain_r%area)

   rcode = nf_redef(fid) ! re-enter "define" mode

   !----------------------------------------------------------------------------
   ! special handling of time coords & related
   !----------------------------------------------------------------------------
   rcode = nf_def_dim(fid,'time',NF_UNLIMITED,did)
   rcode = nf_def_var(fid,'time',NF_REAL,1,did,vid)
   str   = 'time'
   rcode = nf_put_att_text(fid,vid,'long_name',len_trim(str),str)
   str   = 'days since 0000-01-01 00:00:00'
   rcode = nf_put_att_text(fid,vid,'units'    ,len_trim(str),str)
   str   = 'noleap'
   rcode = nf_put_att_text(fid,vid,'calendar' ,len_trim(str),str)
   str   = 'time_bound'
   rcode = nf_put_att_text(fid,vid,"bounds"   ,len_trim(str),str)
 
   rcode = nf_def_dim(fid,'d2',2,did) ! upper & lower time bounds
 
   rcode = nf_inq_dimid(fid,'d2'  ,vdid(1))
   rcode = nf_inq_dimid(fid,'time',vdid(2))
   rcode = nf_def_var(fid,'time_bound',NF_REAL,2,vdid,vid)
   str   = 'time average interval boundaries'
   rcode = nf_put_att_text(fid,vid,'long_name',len_trim(str),str)
   str   = 'days since 0000-01-01 00:00:00'
   rcode = nf_put_att_text(fid,vid,'units'    ,len_trim(str),str)
   str   = 'noleap'
   rcode = nf_put_att_text(fid,vid,'calendar' ,len_trim(str),str)

   !----------------------------------------------------------------------------
   ! close the file
   !----------------------------------------------------------------------------
   rcode = nf_close(fid)

END

!===============================================================================
