!===============================================================================
! CVS $Id: nc_misc.F90,v 1.3 2002/01/09 00:31:27 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/nc_misc.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================
! lower-level, generic netCDF jackets
!===============================================================================

SUBROUTINE nc_dvar(fid,name,ndim,did,lname,units,tavg,spv,rcode)
   
   implicit none

   !--- global ---
#include <netcdf.inc>

   !--- arguments ---
   integer          ,intent(in)  :: fid       ! file id
   character(len=*) ,intent(in)  :: name      ! variable name
   integer          ,intent(in)  :: ndim      ! number of dimension
   integer          ,intent(in)  :: did(ndim) ! vector of dimension ids
   character(len=*) ,intent(in)  :: lname     ! variable long name
   character(len=*) ,intent(in)  :: units     ! variable units
   logical          ,intent(in)  :: tavg      ! flags time-avg attribute
   real             ,intent(in)  :: spv       ! missing value, special value
   integer          ,intent(out) :: rcode     ! error code

   !--- local ---
   integer                       :: vid       ! variable id
   character(len=16)             :: str       ! text string

   !--- local ---
   character(len=*),parameter :: F00 = "('(nc_dvar) ',a,1x,a)"

!-------------------------------------------------------------------------------
!  define a new data array & associated attributes
!-------------------------------------------------------------------------------

   rcode = nf_def_var(fid,name,NF_FLOAT,ndim,did,vid)
   if (rcode.ne.NF_NOERR) write(6,F00) name, nf_strerror(rcode)

   rcode = nf_put_att_text(fid,vid,"long_name",len_trim(lname),lname)
   if (rcode.ne.NF_NOERR) write(6,F00) name, nf_strerror(rcode)

   if ( len_trim(units) > 0 .and. units(1:8) /= "unitless" ) then
     rcode = nf_put_att_text(fid,vid,"units"    ,len_trim(units),units)
     if (rcode.ne.NF_NOERR) write(6,F00) name, nf_strerror(rcode)
   end if

   if (tavg) then
     str = "time: mean"
     rcode = nf_put_att_text(fid,vid,"cell_method",len_trim(str),str)
     if (rcode.ne.NF_NOERR) write(6,F00) name, nf_strerror(rcode)
   end if

   rcode = nf_put_att_double(fid,vid,"missing_value",NF_FLOAT,1,spv)
   if (rcode.ne.NF_NOERR) write(6,F00) name, nf_strerror(rcode)

   rcode = nf_put_att_double(fid,vid,"_FillValue"   ,NF_FLOAT,1,spv)
   if (rcode.ne.NF_NOERR) write(6,F00) name, nf_strerror(rcode)

END

!===============================================================================

SUBROUTINE nc_pvar(fid,name,start,count,data,rcode)

   implicit none

   !--- global ---
#include <netcdf.inc>

   !--- arguments ---
   integer         ,intent(in)  :: fid        ! file id
   character(len=*),intent(in)  :: name       ! the variable name
   integer         ,intent(in)  :: start(*)   ! starting indicies
   integer         ,intent(in)  :: count(*)   ! number of data elements
   real            ,intent(in)  :: data (*)   ! the data
   integer         ,intent(out) :: rcode      ! error code

   !--- local ---
   integer          :: vid        ! variable ID

   !--- local ---
   character(len=*),parameter :: F00 = "('(nc_pvar) ',a,1x,a)"

!-------------------------------------------------------------------------------
!  put 3d (x,y,t) data array into a netCDF file
!-------------------------------------------------------------------------------

   rcode = nf_inq_varid(fid,name,vid)
   if (rcode.ne.NF_NOERR) write(6,F00) name, nf_strerror(rcode)

   rcode = nf_put_vara_double(fid,vid,start,count,data)
   if (rcode.ne.NF_NOERR) write(6,F00) name, nf_strerror(rcode)

END

!===============================================================================
