!===============================================================================
! CVS $Id: restart.F90,v 1.10 2002/01/11 00:25:15 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/restart.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

   SUBROUTINE restart (cOption)

   USE kind_mod      ! access to kind declarations
   USE buf_mod       ! 2d data arrays/buffers
   USE control_mod   ! control flags
   USE calendar_mod  ! calendar routines
   USE diag_mod      ! diagnostic routines & data
   USE shr_sys_mod   ! shared code: system calls
   USE shr_file_mod  ! shared code: file archive
#ifdef MPI_P
!   use mpmd_mod, only: master_task,MPI_COMM_CPL
#endif

   implicit none

#ifdef MPI_P
   include "mpif.h"
#endif

   !---- arguments -----
   character(len=*)  :: cOption     ! read or write a restart file?

   !----- local -----
   character(CLONG)  :: loc_fn     = "null" ! current  local file name
   character(CLONG)  :: loc_fn_old = "null" ! previous local file name
   character(CLONG)  :: rem_fn              ! remote file name
   character(CLONG)  :: str         ! restart file ID string
   character(len=8 ) :: datestr     ! Current date
   character(len=10) :: timestr     ! Current time
   integer           :: i,j         ! character string indicies
   integer           :: LstIndex    ! last index funct (see intrinsic index)
   integer           :: rcode       ! subroutine return code
   logical :: file_exists = .false. ! a local restart file exists

   SAVE

   !----- formats -----
   character(len=*),parameter :: F00 = "('(restart) ',2a)"
   character(len=*),parameter :: F01 = "('(restart) ',a,1x,i8.8)"
   character(len=*),parameter :: F02 = "('(restart) code ',i3,4(1x,a))"
   character(len=*),parameter :: F05 = "(8a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   IF (cOption='read')
!     Acquire cpl restart/IC data (read a data file xor fabricate)
!     Restart data is the model output and daily avg fluxes
!     computed during the previous day of integration.
!   IF (cOption='write')
!     AND  the flag rest_now is non-zero (ie. is "true"), 
!     THEN create restart & pointer files & send them to the mass store
!
!  DETAILS 
!    wrt cOption='read':
!    IF (rest_type = 'initial' )
!       IC's are fabricated by this routine
!       initial date is specified by input parameter "rest_date"
!    IF (rest_type = 'continue' )
!       IC's are in the data file pointed to by the pointer file
!       initial date is specified by the IC data file
!    IF (rest_type = 'branch' )
!       IC's are in the file specified by "rest_bfile"
!       if   rest_date > 0, initial date is specified by "rest_date"
!       else initial date is specified by the IC data file
!    IF (rest_type = 'regeneration' )
!       IC's are in the file specified by "rest_bfile"
!       initial date is specified by the IC data file
!
!-------------------------------------------------------------------------------
   
   IF (cOption .eq. 'read') THEN
      !-------------------------------------------------------------------------
      ! read a restart file
      !-------------------------------------------------------------------------

      !----- get IC file name -----
      IF      (rest_type(1:4) .eq. 'init') THEN
         write(6,F00) 'this is an initial run...'
         write(6,F00) 'IC data fabricated internally'
         rem_fn='<invalid>'
      ELSE IF (rest_type(1:4) .eq. 'cont') THEN
         write(6,F00) 'this is a continuation run...'
         write(6,F00) 'IC data file specified by pointer file'
         if (rest_pfile(1:9).ne.'rpointer ') then
            call shr_file_get (rcode,"rpointer.cpl",rest_pfile,clobber=.true.)
          write(6,F02) rcode,'retrieving',"rpointer.cpl",'from',trim(rest_pfile)
         end if
         open (tmpunit,file='rpointer.cpl',form='formatted')
         read (tmpunit,'(a)') rem_fn
         close(tmpunit)
      ELSE IF (rest_type(1:4) .eq. 'bran') THEN
         write(6,F00) 'this is a branch run...'
         write(6,F00) 'IC data file specified by input parameter'
         rem_fn=rest_bfile
      ELSE IF (rest_type(1:4) .eq. 'rege') THEN
         write(6,F00) 'this is a regeneration run...'
         write(6,F00) 'IC data file specified by input parameter'
         rem_fn=rest_bfile
      ELSE
         write(6,*) 'invalid rest_type = ',rest_type
         STOP 'restart'
      END IF

      !----- acquire IC data -----
      IF (rest_type(1:4) .eq. 'init') THEN
         write(6,F00) 'internally fabricating IC data.'
         Sa_a    = 0.0
         Faxa_a  = 0.0
         Faia_a  = 0.0
         Fala_a  = 0.0
         Faoa_a  = 0.0
         Sl_l    = 0.0
         Fall_l  = 0.0
         roff_rc = 0.0
         Si_i    = 0.0
         Fioi_i  = 0.0
         Faii_i  = 0.0
         So_o    = 0.0
         Fioo_o  = 0.0
         Faoc_o  = 0.0
         aFoxx_o = 0.0
         str   = 'internally fabricated data'
         cdate = 10101
      ELSE ! continue, branch, or regeneration
         write(6,F00) 'acquire IC file   : ',rem_fn
         i      = LstIndex(rem_fn,'/')+1
         j      = len_trim(rem_fn)
         loc_fn = rem_fn(i:j) // ' '
         call shr_file_get(rcode, loc_fn , rem_fn)
         loc_fn_old = loc_fn
         write(6,F02) rcode,'retrieving',trim(loc_fn),'from',trim(rem_fn)
         if (rcode.ne.0) write(6,F00) 'NOTE return code on file retrieval'

         file_exists = .true. 
#ifdef MPI_P
       if(master_task) then
#endif
         open (tmpunit,file=loc_fn,form='unformatted')
         read (tmpunit) str 
         read (tmpunit) cdate
         read (tmpunit)  Sa_a     !--- from atm
         read (tmpunit)  Faxa_a
         read (tmpunit)  Faia_a
         read (tmpunit)  Fala_a
         read (tmpunit)  Faoa_a
         read (tmpunit)  Sl_l     !--- from lnd
         read (tmpunit)  Fall_l
         read (tmpunit)  n_rc ! size of roff_rc
         deallocate(roff_rc) ; allocate(roff_rc(n_rc,1))  
         read (tmpunit)  roff_rc
         read (tmpunit)  Si_i     !--- from ice
         read (tmpunit)  Fioi_i
         read (tmpunit)  Faii_i
         read (tmpunit)  So_o     !--- from ocn
         read (tmpunit)  Fioo_o
         read (tmpunit)  Faoc_o   !--- from cpl
         read (tmpunit) aFoxx_o
         read (tmpunit) diag_eday0,diag_eday1,diag_ns
         read (tmpunit) diag_datai,diag_datas,diag_dataa
         close(tmpunit)
         write(6,F00) 'IC file ID string : ',str
         write(6,F01) 'IC file date      : ',cdate
#ifdef MPI_P
       endif
       call MPI_BCAST(str,      CLONG,  MPI_character,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(cdate,    1,              MPI_integer,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Sa_a,     size(Sa_a),     MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Faxa_a,   size(Faxa_a),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Faia_a,   size(Faia_a),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Fala_a,   size(Fala_a),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Faoa_a,   size(Faoa_a),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Sl_l,     size(Sl_l),     MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Fall_l,   size(Fall_l),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(n_rc,     1,              MPI_integer,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(roff_rc,  size(roff_rc),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Si_i,     size(Si_i),     MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Fioi_i,   size(Fioi_i),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Faii_i,   size(Faii_i),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(So_o,     size(So_o),     MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Fioo_o,   size(Fioo_o),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(Faoc_o,   size(Faoc_o),   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(aFoxx_o,  size(aFoxx_o),  MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(diag_eday0,   1,   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(diag_eday1,   1,   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(diag_ns,      1,   MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(diag_datai,   size(diag_datai),MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(diag_datas,   size(diag_datas),MPI_REAL8,0,MPI_COMM_CPL,ierr)
       call MPI_BCAST(diag_dataa,   size(diag_dataa),MPI_REAL8,0,MPI_COMM_CPL,ierr)
#endif
      END IF

      !----- reset IC data date?? -----
      IF      (rest_type(1:4) .eq. 'init') THEN
        !--- optional date selection ---
        IF (rest_date.gt.0) cdate=rest_date 
      ELSE IF (rest_type(1:4) .eq. 'cont') THEN
        ! do not reset date -- use date from IC file
      ELSE IF (rest_type(1:4) .eq. 'bran') THEN
        !--- optional date selection ---
        IF (rest_date.gt.0) cdate=rest_date
      ELSE IF (rest_type(1:4) .eq. 'rege') THEN
        ! do not reset date -- use date from IC file
      END IF
      call date2ymd (cdate,year,month,day)
      call date2eday(cdate,eday)
      eday0=eday
      sec=0
      write(6,F01) 'using initial date: ',cdate


   ELSE IF (cOption .eq. 'write') THEN
      !-------------------------------------------------------------------------
      !  write a restart file
      !-------------------------------------------------------------------------
      IF (rest_now .ne. 0) THEN
         !--- create a new file name ---
         !oc_fn='123456789+1234567'
         loc_fn='yyyy-mm-dd-sssss '
         write(loc_fn( 1: 4),'(i4.4)') year
         write(loc_fn( 6: 7),'(i2.2)') month
         write(loc_fn( 9:10),'(i2.2)') day
         write(loc_fn(12:16),'(i5.5)') sec
         loc_fn = trim(case_name) // '.cpl5.r.' // loc_fn(1:16)
         !--- create a new file ---
         call date_and_time(datestr,timestr)
         str = '(' //datestr(1:4)//'-'//datestr(5:6)//'-'//datestr(7:8)//' ' &
         &         //timestr(1:2)//':'//timestr(3:4)//':'//timestr(5:6)//')' &
         &         //' '//case_desc
#ifdef MPI_P
       if(master_task) then
#endif
         write(6,F00) 'create restart file : ',loc_fn
         write(6,F00) 'restart file ID str : ',str(1:len_trim(str))
         open (tmpunit,file=loc_fn,form='unformatted')
         write(tmpunit) str 
         write(tmpunit) cdate
         write(tmpunit)  Sa_a     !--- from atm
         write(tmpunit)  Faxa_a
         write(tmpunit)  Faia_a
         write(tmpunit)  Fala_a
         write(tmpunit)  Faoa_a
         write(tmpunit)  Sl_l     !--- from lnd
         write(tmpunit)  Fall_l
         write(tmpunit)     n_rc
         write(tmpunit)  roff_rc
         write(tmpunit)  Si_i     !--- from ice
         write(tmpunit)  Fioi_i
         write(tmpunit)  Faii_i
         write(tmpunit)  So_o     !--- from ocn
         write(tmpunit)  Fioo_o
         write(tmpunit)  Faoc_o   !--- from cpl
         write(tmpunit) aFoxx_o
         write(tmpunit) diag_eday0,diag_eday1,diag_ns
         write(tmpunit) diag_datai,diag_datas,diag_dataa
         close(tmpunit)
#ifdef MPI_P
       endif
#endif
         file_exists = .true. 

         !--- archive new restart data ---
!xjx---20090902         rem_fn = trim(mss_dir) // 'rest/' // trim(loc_fn) //' '
         rem_fn = trim(mss_dir) // trim(loc_fn) //' '
!!==================wangln 2007-07-27:: to avoid the Remote Copy==============
         !!call shr_file_put ( rcode, loc_fn, rem_fn, mss_pass, mss_rtpd)
         !!write(6,F02) rcode,'archiving',trim(loc_fn),'to',trim(rem_fn)
!!==================wangln 2007-07-27:: to avoid the Remote Copy==============

         !--- update pointer file ---
!         IF (rcode == 0) THEN
#ifdef MPI_P
       if(master_task) then
#endif
           open (tmpunit,file='rpointer.cpl',form='formatted')
           write(tmpunit,'(2a)') trim(rem_fn)
           write(tmpunit,'(2a)')'restart pointer for case ',trim(case_name)
           write(tmpunit,F05   ) trim(str)
           close(tmpunit)
           if (rest_pfile(1:9).ne.'rpointer ') then
             call shr_file_put (rcode,"rpointer.cpl",rest_pfile,mss_pass,mss_rtpd)
             write(6,F02) rcode,'archiving',"rpointer.cpl",'to',trim(rest_pfile)
           end if
#ifdef MPI_P
       endif
#endif
!         ELSE
!           write(6,F00) 'ERROR on file archive?'
!         END IF
   
         !--- remove older history file ---
         if (mss_rmlf /= 0 .and. loc_fn_old(1:4) /= "null" ) then
           call shr_sys_system('rm '//loc_fn_old,rcode)
         end if
         loc_fn_old = loc_fn

      END IF

   ELSE
      !-------------------------------------------------------------------------
      !  there aren't any other choices
      !-------------------------------------------------------------------------
      write(6,*) 'ERROR: invalid cOption = ',cOption

   END IF

END SUBROUTINE

!===============================================================================
