!===============================================================================
! CVS $Id: t_avg.F90,v 1.4 2001/10/12 17:33:13 tcraig Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/t_avg.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

#ifdef MPI_P
SUBROUTINE t_avg (cFlag,nx,nnFoxx,A,B,ns)
#else
SUBROUTINE t_avg (cFlag,nx,A,B,ns)
#endif

   USE shr_timer_mod  ! timing routines
#ifdef MPI_P
   USE buf_mod, only: IST_o,IEND_o,IPT_o
#endif

   IMPLICIT none

   !----- arguments -----
   character(len=*) :: cFlag          ! operation request flag
#ifdef MPI_P
   integer          :: nx,nnFoxx,j                  ! size of arrays
   real             :: A(nx,nnFoxx),B(nx,nnFoxx)    ! data arrays
#else
   integer          :: nx             ! size of arrays
   real             :: A(nx),B(nx)    ! data arrays
#endif
   integer          :: ns             ! number of samples in summation

   !----- local -----
   real             :: fact           ! = 1/ns
   integer          :: i              ! generic loop index
   integer,save     :: timer_n = 0    ! timer number

!-------------------------------------------------------------------------------
! PURPOSE:
!   form the average, B, of a series of input arrays A
!
!   cFlag='zero' => B=0   , ns=0
!   cFlag='sum'  => B=B+A , ns=ns+1
!   cFlag='avg'  => B=A/ns
!
! ASSUMPTIONS:
!   there are no undefined array elements
!-------------------------------------------------------------------------------
   
   if (timer_n < 1) call shr_timer_get(timer_n,'time averaging/accumulation')
   call shr_timer_start(timer_n)

!  print *,"-------------t_avg------------"

   SELECT CASE (cFlag)
      CASE ( 'zero' ) 
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i,j)
         do j=1,nnFoxx
         do i=IST_o,IEND_o
           B(i,j)=0.0
         enddo
         enddo
!$omp  end parallel do
#else
         do i=1,nx
           B(i)=0.0
         enddo
#endif
         ns=0

      CASE ( 'sum' ) 
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i,j) 
         do j=1,nnFoxx
         do i=IST_o,IEND_o
           B(i,j)=B(i,j)+A(i,j)
         enddo
         enddo
!$omp  end parallel do
#else
         do i=1,nx
           B(i)=B(i)+A(i)
         enddo
#endif
         ns=ns+1

      CASE ( 'avg' ) 
         IF (ns .le. 0) THEN
           write(6,*) '(t_avg) ERROR: ns = ',ns
           STOP 't_avg'
         ELSE IF (ns == 1) THEN
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i,j)
           do j=1,nnFoxx
           do i=IST_o,IEND_o
             B(i,j)=A(i,j)
           enddo
           enddo
!$omp  end parallel do
#else
           do i=1,nx
             B(i)=A(i)
           enddo
#endif
         ELSE
           fact=1.0/float(ns)
#ifdef MPI_P
!$OMP PARALLEL DO PRIVATE(i,j)
           do j=1,nnFoxx
           do i=IST_o,IEND_o
             B(i,j)=A(i,j)*fact
           enddo
           enddo
!$omp  end parallel do
#else
           do i=1,nx
             B(i)=A(i)*fact
           enddo
#endif
         END IF
      CASE DEFAULT
         write(6,*) '(t_avg) ERROR: invalid cFlag=',cFlag
         STOP 'cFlag'
   END SELECT

   call shr_timer_stop (timer_n)

END

!===============================================================================
