!!                                                                   !!
!!                   GNU General Public License                      !!
!!                                                                   !!
!! This file is part of the Flexible Modeling System (FMS).          !!
!!                                                                   !!
!! FMS is free software; you can redistribute it and/or modify       !!
!! it and are expected to follow the terms of the GNU General Public !!
!! License as published by the Free Software Foundation.             !!
!!                                                                   !!
!! FMS is distributed in the hope that it will be useful,            !!
!! but WITHOUT ANY WARRANTY; without even the implied warranty of    !!
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     !!
!! GNU General Public License for more details.                      !!
!!                                                                   !!
!! You should have received a copy of the GNU General Public License !!
!! along with FMS; if not, write to:                                 !!
!!          Free Software Foundation, Inc.                           !!
!!          59 Temple Place, Suite 330                               !!
!!          Boston, MA  02111-1307  USA                              !!
!! or see:                                                           !!
!!          http://www.gnu.org/licenses/gpl.txt                      !!
!!                                                                   !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
! ice_type_mod - maintains the sea ice data, reads/writes restarts, reads the  !
!                namelist and initializes diagnostics. - Mike Winton           !
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
module ice_type_mod

  use mpp_mod,          only: mpp_pe, mpp_root_pe, mpp_sum, mpp_clock_id, CLOCK_COMPONENT
  use mpp_domains_mod,  only: domain2D, mpp_update_domains
  use fms_mod,          only: file_exist, open_namelist_file, check_nml_error
  use fms_mod,          only: read_data, write_data, close_file
  use fms_mod,          only: stdlog, error_mesg, FATAL, WARNING, clock_flag_default
  use diag_manager_mod, only: diag_axis_init, register_diag_field
  use diag_manager_mod, only: register_static_field, send_data
  use time_manager_mod, only: time_type, get_time
  use constants_mod,    only: Tfreeze
  use ice_grid_mod,     only: set_ice_grid, t_to_uv, ice_grid_end
  use ice_grid_mod,     only: Domain, isc, iec, jsc, jec, isd, ied, jsd, jed, im, jm, km
  use ice_grid_mod,     only: geo_lon, geo_lat, cell_area, sin_rot, cos_rot, wett, xb1d, yb1d
  use ice_thm_mod,      only: ice_thm_param
  use ice_dyn_mod,      only: ice_dyn_param
  use mpp_mod,          only: stdout

 !================================= liuxw 20200616 ==============================
  use time_manager_mod, only: get_date
  use mpp_mod,          only: mpp_error, mpp_sync, mpp_broadcast
  use mpp_io_mod,       only: mpp_open, mpp_close, MPP_RDONLY
  use mpmd_mod              ! use of mpi parameters in MPI_BCAST function 
!================================= liuxw 20200616 ==============================

  !--------------wangln 2008-02-09 Get the Ice initial Value---------------------
  !!use ice_spec_mod,     only: get_sea_surface
  !--------------wangln 2008-02-09 Get the Ice initial Value---------------------

  implicit none
  private

public :: ice_data_type, ice_model_init, ice_model_end, kmelt, mom_rough_ice, &
          heat_rough_ice, atmos_winds, hlim, slab_ice, spec_ice, verbose,     &
          ice_bulk_salin, do_ice_restore, do_ice_limit, max_ice_limit,        &
          ice_restore_timescale, do_init, h2o, heat,                          &
          cm2_bugs, conservation_check,                                       &
          sic_force_tscale, sic_force_s, sic_force_e                          ! for sic obs  restore

public  :: id_cn, id_hi, id_hs, id_t1, id_t2, id_ts
public  :: id_mi, id_sh, id_lh, id_sw, id_lw, id_snofl, id_rain, id_runoff,    &
           id_calving, id_evap, id_saltf, id_tmelt, id_bmelt, id_bheat, id_e2m,&
! wutw 2009-10-23 add id_albbar ==>
!          id_frazil, id_alb, id_xprt, id_lsrc, id_lsnk, id_bsnk, id_strna,    &
           id_frazil, id_alb, id_albbar, id_xprt, id_lsrc, id_lsnk, id_bsnk, id_strna,    &
! wutw 2009-10-23 add id_albbar <==
           id_sigi, id_sigii, id_stren, id_ui, id_vi, id_fax, id_fay, id_fix,  &
! wutw 2009-10-23 add id_snowmelt ==>
!          id_fiy, id_fcx, id_fcy, id_fwx, id_fwy, id_swdn, id_lwdn, id_sn2ic, &
! chum 2010-03-29 add id_swup
!           id_fiy, id_fcx, id_fcy, id_fwx, id_fwy, id_swdn, id_lwdn, id_sn2ic, id_snowmelt, &
           id_fiy, id_fcx, id_fcy, id_fwx, id_fwy, id_swdn, id_swup, id_lwdn, id_sn2ic, id_snowmelt, &
! chum 2010-03-29 add id_swup
! wutw 2009-10-23 add id_snowmelt <==
           id_slp, id_ext, id_sst, id_sss, id_ssh, id_uo, id_vo, id_ta, id_obi,&
           id_qfres, id_qflim, id_ix_trans, id_iy_trans

public  :: iceClock

  !---- assimil module by scnie----------------

      public :: init_assimil, init_cycle_assimil, get_cycle_assimil

      integer :: &
        nx_block, &
        ny_block, &
        my_task, &
        master_task0, &
        nu_diag

      real, parameter, public :: &
        c1   = 1.0, &
        c360 = 360.0, &
        puny = 1.0e-11, &
        p5   = 0.5, &
        spval_dbl = 1.0e30

      logical :: &
         assimil = .false.      ! assimilate ice state or not

      character (len=128), public :: &
         assimil_type = 'initial',     & ! assimilation type: initial or cycle
         assimil_dataset = 'none',     & ! observed data type used for assimilation
         assimil_dataform = 'none',    & ! observed data format used for assimilation
         assimil_dir = './INPUT',      & ! directory of observed data file
         assimil_file = './INPUT/ice.assimil.nc' ! observed data file used for assimilation

      integer, public :: &
         adate_start = 20000101,    & ! first date of assimilation cycle
         adate_end   = 20000201,    & ! end date of assimilation cycle
         afreq      = 1,       & ! interval of cycle assimilation (days)
         ayear                   ! current year in assimilation cycle

      real, public :: &
         aobsvar = 0.0001! observation error variances (constant)

      real, dimension(:), allocatable :: &
         aice_obs_g,           & ! ice concentration from observational data
         mask_obs_g,           & ! mask of observation
         lon_obs_g,            & ! longitude of observation
         lat_obs_g,            & ! latitude of observation
         aice_obs                ! ice concentration from observational data (block)

      real, dimension (:,:), allocatable :: &
         aice_inc,             & ! ice concentration increments
         tmask_dbl               ! mask used to gather tmask (since tmask is
                                 ! logical variable

  !---- end assimil module----------------

  !---- id for diagnositics -------------------
  integer :: id_xb, id_xt, id_yb, id_yt, id_ct, id_xv, id_yv
  integer :: id_cn, id_hi, id_hs, id_t1, id_t2, id_ts
  integer :: id_mi, id_sh, id_lh, id_sw, id_lw, id_snofl, id_rain, id_runoff
  integer :: id_calving, id_evap, id_saltf, id_tmelt, id_bmelt, id_bheat, id_e2m
! wutw 2009-10-23 add id_albbar ==>
! integer :: id_frazil, id_alb, id_xprt, id_lsrc, id_lsnk, id_bsnk, id_strna
  integer :: id_frazil, id_alb, id_albbar, id_xprt, id_lsrc, id_lsnk, id_bsnk, id_strna
! wutw 2009-10-23 add id_albbar <==
  integer :: id_sigi, id_sigii, id_stren, id_ui, id_vi, id_fax, id_fay, id_fix
! wutw 2009-10-23 add id_snowmelt ==>
! integer :: id_fiy, id_fcx, id_fcy, id_fwx, id_fwy, id_swdn, id_lwdn, id_sn2ic
! chum 2010-03-29 add id_swup
!  integer :: id_fiy, id_fcx, id_fcy, id_fwx, id_fwy, id_swdn, id_lwdn, id_sn2ic, id_snowmelt
  integer :: id_fiy, id_fcx, id_fcy, id_fwx, id_fwy, id_swdn, id_swup, id_lwdn, id_sn2ic, id_snowmelt
! chum 2010-03-29 add id_swup
! wutw 2009-10-23 add id_snowmelt <==
  integer :: id_slp, id_ext, id_sst, id_sss, id_ssh, id_uo, id_vo, id_ta, id_obi
  integer :: id_qfres, id_qflim, id_ix_trans, id_iy_trans
  integer :: id_sw_vis, id_sw_dir, id_sw_dif, id_sw_vis_dir, id_sw_vis_dif

  !--- namelist interface --------------
  real    :: mom_rough_ice  = 1.0e-4     ! momentum same, cd10=(von_k/ln(10/z0))^2
  real    :: heat_rough_ice = 1.0e-4     ! heat roughness length
  real    :: kmelt          = 6e-5*4e6   ! ocean/ice heat flux constant
  real    :: alb_sno        = 0.85       ! snow albedo (less if melting)
  real    :: alb_ice        = 0.5826     ! ice albedo (less if melting)
  real    :: pen_ice        = 0.3        ! part unreflected solar penetrates ice
  real    :: opt_dep_ice    = 0.67       ! ice optical depth
  real    :: t_range_melt   = 1.0        ! melt albedos scaled in over T range
  real    :: ice_bulk_salin = 0.0        ! ice bulk salinity (for ocean salt flux)
  real    :: p0             = 2.75e4     ! ice strength parameter
  real    :: c0             = 20.0       ! another ice strength parameter
  real    :: cdw            = 3.24e-3    ! water/ice drag coefficient
  real    :: wd_turn        = 25.0       ! water/ice drag turning angle
  integer :: nsteps_dyn     = 432        ! dynamics steps per slow timestep
  integer :: nsteps_adv     = 8          ! advection steps per slow timestep
  integer :: num_part       = 6          ! number of ice grid partitions
                                         ! partition 1 is open water
                                         ! partitions 2 to num_part-1 are
                                         !   thickness limited ice categories
                                         ! partition num_part is unlimited ice
  logical :: atmos_winds = .true.        ! wind stress from atmosphere model over t points and has wrong sign
  logical :: slab_ice    = .false.       ! do old-style GFDL slab ice?
  logical :: spec_ice    = .false.       ! old-style GFDL slab ice with SST, ice thickness and conc. from data
  logical :: do_ice_restore  = .false.   ! restore sea-ice toward climatology
  logical :: do_ice_limit    = .false.   ! limit sea ice to max_ice_limit
  real    :: max_ice_limit   = 4.0       ! maximum sea ice height(m),
                                         ! if do_ice_limit is true
                                         ! TK: default chosen based on observed
                                         !     ice thickness data used by climate
                                         !     group, which range up to 7.31 m
  real    :: ice_restore_timescale = 5.0 ! time scale for restoring ice (days)
  logical :: conservation_check = .true. ! check for heat and h2o conservation
  logical :: cm2_bugs           = .false.! keep cm2 bugs for reproducibility        
  logical :: verbose            = .false.! control printing message, will slow model down when turn true
  integer :: layout(2)          = (/0, 0/)

! define and initialize variables of sic restoring
  integer :: sic_force_tscale     = -1         ! for sic obs restore
  integer :: sic_force_s          = -1         ! for sic obs restore
  integer :: sic_force_e          = -1         ! for sic obs restore

  namelist /ice_model_nml/ mom_rough_ice, heat_rough_ice, p0, c0, cdw, wd_turn,  &
                           kmelt, alb_sno, alb_ice, pen_ice, opt_dep_ice,        &
                           nsteps_dyn, nsteps_adv, num_part, atmos_winds,        &
                           slab_ice, spec_ice, ice_bulk_salin, layout,           &
                           do_ice_restore, do_ice_limit, max_ice_limit,          &
                           ice_restore_timescale, conservation_check,            &
                           t_range_melt, cm2_bugs, verbose,                      &
                           sic_force_tscale, sic_force_s, sic_force_e,           & ! for sic obs restore
  !---- assimil module by scnie----------------
                           assimil, assimil_type, assimil_dataset, assimil_dataform, &
                           assimil_dir, adate_start, adate_end, afreq, aobsvar
  !---- end assimil module----------------

 !================================= liuxw 20200616 ==============================
  integer, save          :: read_ana       = 0  
  logical, save          :: firstana       = .false.
  integer, save          :: dtts           = 0              ! dt_ice 
 !integer, parameter     :: dtts           = 1800           ! dt_ice 
  real,parameter         :: ocnmiss        = -1.0e3
  integer,parameter      :: waitstep_limit = 720           ! each step is about 5s 
  character(len=8), save :: id_anafile     = 'none'
  integer, save          :: inidate,id_currwddate,id_nextwddate,numwd
  integer, allocatable, dimension(:),save   :: wddates
  real, allocatable, dimension(:,:), save   :: ana_ci 
  real, allocatable, dimension(:,:), save   :: ana_hi 
  real, allocatable, dimension(:,:), save   :: ana_ui 
  real, allocatable, dimension(:,:), save   :: ana_vi 
 !public :: ida_use,ida_start, ida_stop, increment_adjust, cn_adj_pre, enthalpy_adj, cn_adj_post, ana_ci, ana_hi
  public :: ida_use,ida_start, ida_stop, cn_adj_pre, enthalpy_adj, cn_adj_post, hi_adj_post, ana_ci, ana_hi
  public ida_analysis
  public ida_operate
  private ida_init
  private ida_update
  public  cat_ice

  logical   :: ida_use            = .false.
  integer   :: ida_start          = 0
  integer   :: ida_stop           = 0
  integer   :: ida_window         = 1                  !! days of assimilation window
 !logical   :: increment_adjust   = .false.
  logical   :: cn_adj_pre         = .false.
  logical   :: cn_adj_post        = .false.
  logical   :: hi_adj_post        = .false.
  logical   :: enthalpy_adj       = .false.
  character(len=100) :: ida_wkdir = 'none'
 !namelist /ice_data_ass_nml/ ida_use, ida_start, ida_stop, ida_window, increment_adjust, cn_adj_pre, cn_adj_newice, enthalpy_adj, ida_wkdir
  namelist /ice_data_ass_nml/ ida_use, ida_start, ida_stop, ida_window, cn_adj_pre, enthalpy_adj, cn_adj_post, hi_adj_post, ida_wkdir

  integer,private,dimension(12)  :: daynum
  DATA daynum  / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /          ! fixed
 !================================= liuxw 20200616 ==============================


  logical :: do_init = .false.
  real    :: hlim(8) = (/ 0.0, 0.1, 0.3, 0.7, 1.1, 1.5, 2.0, 2.5 /) ! thickness limits 1...num_part-1
  real    :: h2o(4), heat(4) ! for conservation analysis
                             ! 1 - initial ice h2o/heat content
                             ! 2 - h2o/heat flux down at top of ice
                             ! 3 - h2o/heat flux down at bottom of ice
                             ! 4 - final ice h2o/heat content

  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
  ! This structure contains the ice model data (some used by calling routines);  !
  ! the third index is partition (1 is open water; 2 is ice cover)               !
  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
  type ice_data_type
     type(domain2D)                     :: Domain
     type (time_type)                   :: Time_Init, Time
     type (time_type)                   :: Time_step_fast, Time_step_slow
     integer                            :: avg_kount
     logical, pointer, dimension(:,:)   :: mask                =>NULL() ! where ice can be
     logical, pointer, dimension(:,:,:) :: ice_mask            =>NULL() ! where ice actually is
     real,    pointer, dimension(:,:,:) :: part_size           =>NULL()
     real,    pointer, dimension(:,:,:) :: part_size_uv        =>NULL()
     real,    pointer, dimension(:,:,:) :: albedo              =>NULL()
! wutw 2009-10-23 add albedo_bareice ==>
     real,    pointer, dimension(:,:,:) :: albedo_bareice      =>NULL()
! wutw 2009-10-23 add albedo_bareice <==
     real,    pointer, dimension(:,:,:) :: albedo_vis_dir      =>NULL()
     real,    pointer, dimension(:,:,:) :: albedo_nir_dir      =>NULL()
     real,    pointer, dimension(:,:,:) :: albedo_vis_dif      =>NULL()
     real,    pointer, dimension(:,:,:) :: albedo_nir_dif      =>NULL()
     real,    pointer, dimension(:,:,:) :: rough_mom           =>NULL()
     real,    pointer, dimension(:,:,:) :: rough_heat          =>NULL()
     real,    pointer, dimension(:,:,:) :: rough_moist         =>NULL()
     real,    pointer, dimension(:,:,:) :: t_surf              =>NULL()
     real,    pointer, dimension(:,:,:) :: u_surf              =>NULL()
     real,    pointer, dimension(:,:,:) :: v_surf              =>NULL()
     real,    pointer, dimension(:,:)   :: sea_lev             =>NULL()
     real,    pointer, dimension(:,:)   :: s_surf              =>NULL()
     real,    pointer, dimension(:,:)   :: u_ocn               =>NULL()
     real,    pointer, dimension(:,:)   :: v_ocn               =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_u_top          =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_v_top          =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_t_top          =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_q_top          =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_sw_top         =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_lw_top         =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_sw_vis_top     =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_sw_dir_top     =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_sw_dif_top     =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_sw_vis_dir_top =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_sw_vis_dif_top =>NULL()
     real,    pointer, dimension(:,:,:) :: flux_lh_top         =>NULL()
     real,    pointer, dimension(:,:,:) :: lprec_top           =>NULL()
     real,    pointer, dimension(:,:,:) :: fprec_top           =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_u              =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_v              =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_t              =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_q              =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_sw             =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_lw             =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_sw_vis         =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_sw_dir         =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_sw_dif         =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_sw_vis_dir     =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_sw_vis_dif     =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_lh             =>NULL()
     real,    pointer, dimension(:,:  ) :: lprec               =>NULL()
     real,    pointer, dimension(:,:  ) :: fprec               =>NULL()
     real,    pointer, dimension(:,:  ) :: p_surf              =>NULL()
     real,    pointer, dimension(:,:  ) :: runoff              =>NULL()
     real,    pointer, dimension(:,:  ) :: calving             =>NULL()
     real,    pointer, dimension(:,:  ) :: flux_salt           =>NULL()
     real,    pointer, dimension(:,:)   :: lwdn                =>NULL()
     real,    pointer, dimension(:,:  ) :: swdn                =>NULL() ! downward long/shortwave
! chum 2010-03-29 add id_swup
     real,    pointer, dimension(:,:  ) :: swup                =>NULL() ! upward long/shortwave
! chum 2010-03-29 add id_swup
     real,    pointer, dimension(:,:,:) :: pen                 =>NULL()
     real,    pointer, dimension(:,:,:) :: trn                 =>NULL() ! ice optical parameters
     real,    pointer, dimension(:,:,:) :: tmelt               =>NULL()
     real,    pointer, dimension(:,:,:) :: bmelt               =>NULL()
     real,    pointer, dimension(:,:,:) :: h_snow              =>NULL()
     real,    pointer, dimension(:,:,:) :: h_ice               =>NULL()
     real,    pointer, dimension(:,:,:) :: t_ice1              =>NULL()
     real,    pointer, dimension(:,:,:) :: t_ice2              =>NULL()
     real,    pointer, dimension(:,:)   :: u_ice               =>NULL()
     real,    pointer, dimension(:,:)   :: v_ice               =>NULL()
     real,    pointer, dimension(:,:)   :: sig11               =>NULL()
     real,    pointer, dimension(:,:)   :: sig22               =>NULL()
     real,    pointer, dimension(:,:)   :: sig12               =>NULL()
     real,    pointer, dimension(:,:)   :: frazil              =>NULL()
     real,    pointer, dimension(:,:)   :: bheat               =>NULL()
     real,    pointer, dimension(:,:)   :: qflx_lim_ice        =>NULL()
     real,    pointer, dimension(:,:)   :: qflx_res_ice        =>NULL()
!---------------------wangln 2007-11-20-------------------------------------
     real,    pointer, dimension(:,:)   :: atmos_tsurf       =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_tref        =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_qref        =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_latent      =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_sen         =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_evap        =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_lwup        =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_swnet       =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_taux        =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_tauy        =>NULL()
     real,    pointer, dimension(:,:,:) :: atmos_albedo      =>NULL()
     !add by fangyj 20220222
     real,    pointer, dimension(:,:,:) :: atmos_snowdp      =>NULL() 
     !end by fangyj 20220222 
     real,    pointer, dimension(:,:,:) :: previous_t_surf   =>NULL()
     real,    pointer, dimension(:,:,:) :: dedt_surf         =>NULL()
     real,    pointer, dimension(:,:,:) :: dhdt_surf         =>NULL()
     real,    pointer, dimension(:,:,:) :: drdt_surf         =>NULL()
     real,    pointer, dimension(:,:,:) :: ocn_swnet         =>NULL()
     real,    pointer, dimension(:,:,:) :: ocn_q             =>NULL()
     real,    pointer, dimension(:,:,:) :: ocn_t             =>NULL()
     real,    pointer, dimension(:,:)   :: ocn_taux          =>NULL()
     real,    pointer, dimension(:,:)   :: ocn_tauy          =>NULL()
     real,    pointer, dimension(:,:,:) :: meltw             =>NULL()
     real,    pointer, dimension(:,:,:) :: ifrac             =>NULL()
     real,    pointer, dimension(:,:)   :: ocn_p             =>NULL()
!---------------------wangln 2007-11-20-------------------------------------

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
     real,    pointer, dimension(:,:,:)   :: u_10m             =>NULL()
     real,    pointer, dimension(:,:,:)   :: v_10m             =>NULL()
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

  end type ice_data_type

  integer :: iceClock

  contains

  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
  ! ice_model_init - initializes ice model data, parameters and diagnostics      !
  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
  subroutine ice_model_init (Ice, Time_Init, Time, Time_step_fast, Time_step_slow )

    type (ice_data_type), intent(inout) :: Ice
    type (time_type)    , intent(in)    :: Time_Init      ! starting time of model integration
    type (time_type)    , intent(in)    :: Time           ! current time
    type (time_type)    , intent(in)    :: Time_step_fast ! time step for the ice_model_fast
    type (time_type)    , intent(in)    :: Time_step_slow ! time step for the ice_model_slow

    integer           :: io, ierr, nlon, nlat, npart, unit, log_unit, k
    integer           :: sc, dy, i, j 
    real              :: dt_slow
    character(len=22) :: restart = 'INPUT/ice_model.res.nc'
    !
    ! read namelist and write to logfile
    !
    unit = open_namelist_file()
    read  (unit, ice_model_nml,iostat=io)
    write (stdlog(), ice_model_nml)
    ierr = check_nml_error(io, 'ice_model_nml')
    call close_file(unit)

    if (spec_ice) then
       slab_ice = .true.
       nsteps_dyn = 0
       nsteps_adv = 0
    end if
    if (slab_ice) num_part = 2 ! open water and ice ... but never in same place
    if (num_part>size(hlim(:))+1) &
         call error_mesg ('ice_model_init', 'not enough thickness limits', FATAL)

    call get_time(Time_step_slow, sc, dy); dt_slow=864e2*dy+sc
    
    write(stdout(),*) 'Dt_Slow=', dt_slow

    call set_ice_grid(Ice%domain, dt_slow, nsteps_dyn, nsteps_adv, num_part, layout )

    call ice_dyn_param(p0, c0, cdw, wd_turn, slab_ice)
    call ice_thm_param(alb_sno, alb_ice, pen_ice, opt_dep_ice, slab_ice, t_range_melt)

    allocate ( Ice % mask     (isc:iec, jsc:jec)       , &
         Ice % ice_mask       (isc:iec, jsc:jec, km)   , &
         Ice % t_surf         (isc:iec, jsc:jec, km)   , &
         Ice % s_surf         (isc:iec, jsc:jec)       , &
         Ice % sea_lev        (isd:ied, jsd:jed)       , &
         Ice % part_size      (isd:ied, jsd:jed, km)   , &
         Ice % part_size_uv   (isc:iec, jsc:jec, km)   , &
         Ice % u_surf         (isc:iec, jsc:jec, km)   , &
         Ice % v_surf         (isc:iec, jsc:jec, km)   , &
         Ice % u_ocn          (isd:ied, jsd:jed)       , &
         Ice % v_ocn          (isd:ied, jsd:jed)       , &
         Ice % rough_mom      (isc:iec, jsc:jec, km)   , &
         Ice % rough_heat     (isc:iec, jsc:jec, km)   , &
         Ice % rough_moist    (isc:iec, jsc:jec, km)   , &
         Ice % albedo         (isc:iec, jsc:jec, km)   , &                
! wutw 2009-10-23 add Ice % albedo_bareice ==>
         Ice % albedo_bareice (isc:iec, jsc:jec, km)   , &                
! wutw 2009-10-23 add Ice % albedo_bareice <==
         Ice % albedo_vis_dir (isc:iec, jsc:jec, km)   , &
         Ice % albedo_nir_dir (isc:iec, jsc:jec, km)   , &
         Ice % albedo_vis_dif (isc:iec, jsc:jec, km)   , &
         Ice % albedo_nir_dif (isc:iec, jsc:jec, km)   )

    allocate ( Ice % flux_u_top   (isd:ied, jsd:jed, km) ,       &
         Ice % flux_v_top         (isd:ied, jsd:jed, km) ,       &
         Ice % flux_t_top         (isc:iec, jsc:jec, km) ,       &
         Ice % flux_q_top         (isc:iec, jsc:jec, km) ,       &
         Ice % flux_sw_top        (isc:iec, jsc:jec, km) ,       &
         Ice % flux_sw_vis_top    (isc:iec, jsc:jec, km) ,       &
         Ice % flux_sw_dir_top    (isc:iec, jsc:jec, km) ,       &
         Ice % flux_sw_dif_top    (isc:iec, jsc:jec, km) ,       &
         Ice % flux_sw_vis_dir_top(isc:iec, jsc:jec, km) ,       &
         Ice % flux_sw_vis_dif_top(isc:iec, jsc:jec, km) ,       &
         Ice % flux_lw_top        (isc:iec, jsc:jec, km) ,       &
         Ice % flux_lh_top        (isc:iec, jsc:jec, km) ,       &
         Ice % lprec_top          (isc:iec, jsc:jec, km) ,       &
!---------------------wangln 2007-11-20----------------------------------------------------------------
         !Ice % fprec_top          (isc:iec, jsc:jec, km)   )
         Ice % fprec_top          (isc:iec, jsc:jec, km) ,       &   
         Ice % atmos_tsurf        (isc:iec, jsc:jec ) ,          &
         Ice % atmos_tref         (isc:iec, jsc:jec, km) ,       &
         Ice % atmos_qref         (isc:iec, jsc:jec, km) ,       &
         Ice % atmos_latent       (isc:iec, jsc:jec, km) ,       &
         Ice % atmos_sen          (isc:iec, jsc:jec, km) ,       &
         Ice % atmos_evap         (isc:iec, jsc:jec, km) ,       &
         Ice % atmos_lwup         (isc:iec, jsc:jec, km) ,       & 
         Ice % atmos_swnet        (isc:iec, jsc:jec, km) ,       & 
         Ice % atmos_taux         (isc:iec, jsc:jec, km) ,       & 
         Ice % atmos_tauy         (isc:iec, jsc:jec, km) ,       & 
         Ice % atmos_albedo       (isc:iec, jsc:jec, 1:4) ,       & 
         !add by fangyj 20220222 
         Ice % atmos_snowdp       (isc:iec, jsc:jec, km) ,       &
         !end by fangyj 20220222
         Ice % ocn_swnet          (isc:iec, jsc:jec, km) ,       & 
         Ice % ocn_q              (isc:iec, jsc:jec, km) ,       & 
         Ice % ocn_t              (isc:iec, jsc:jec, km) ,       & 
         Ice % ocn_p              (isc:iec, jsc:jec    ) ,       & 
         Ice % ocn_taux           (isc:iec, jsc:jec    ) ,       & 
         Ice % ocn_tauy           (isc:iec, jsc:jec    ) ,       & 
         Ice % meltw              (isc:iec, jsc:jec, km) ,       & 
         Ice % ifrac              (isc:iec, jsc:jec, km) ,       & 
         Ice % previous_t_surf    (isc:iec, jsc:jec, km) ,       & 
         Ice % dedt_surf          (isc:iec, jsc:jec, km) ,       & 
         Ice % drdt_surf          (isc:iec, jsc:jec, km) ,       & 

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
         Ice % u_10m              (isc:iec, jsc:jec, km) ,       &
         Ice % v_10m              (isc:iec, jsc:jec, km) ,       &
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

         Ice % dhdt_surf          (isc:iec, jsc:jec, km) ) 
!---------------------wangln 2007-11-20----------------------------------------------------------------


    allocate ( Ice % flux_u    (isc:iec, jsc:jec ) ,       &
         Ice % flux_v          (isc:iec, jsc:jec ) ,       &
         Ice % flux_t          (isc:iec, jsc:jec ) ,       &
         Ice % flux_q          (isc:iec, jsc:jec ) ,       &
         Ice % flux_sw         (isc:iec, jsc:jec ) ,       &
         Ice % flux_sw_vis     (isc:iec, jsc:jec) ,        &
         Ice % flux_sw_dir     (isc:iec, jsc:jec) ,        &
         Ice % flux_sw_dif     (isc:iec, jsc:jec) ,        &
         Ice % flux_sw_vis_dir (isc:iec, jsc:jec) ,        &
         Ice % flux_sw_vis_dif (isc:iec, jsc:jec) ,        &
         Ice % flux_lw         (isc:iec, jsc:jec ) ,       &
         Ice % flux_lh         (isc:iec, jsc:jec ) ,       &
         Ice % lprec           (isc:iec, jsc:jec ) ,       &
         Ice % fprec           (isc:iec, jsc:jec ) ,       &
         Ice % p_surf          (isc:iec, jsc:jec ) ,       &
         Ice % runoff          (isc:iec, jsc:jec ) ,       &
         Ice % calving         (isc:iec, jsc:jec ) ,       &
         Ice % flux_salt       (isc:iec, jsc:jec ) ,       &
         Ice % lwdn            (isc:iec, jsc:jec ) ,       &
         Ice % swdn            (isc:iec, jsc:jec ) ,       &
         Ice % swup            (isc:iec, jsc:jec )         )
! chum 2010-03-29 add id_swup
    allocate ( Ice % frazil (isc:iec, jsc:jec), Ice % bheat  (isc:iec, jsc:jec), &
               Ice % u_ice  (isd:ied, jsd:jed), Ice % v_ice  (isd:ied, jsd:jed), &
               Ice % sig11  (isd:ied, jsd:jed), Ice % sig22  (isd:ied, jsd:jed), &
               Ice % sig12  (isd:ied, jsd:jed)                               )
    allocate ( Ice % tmelt  (isc:iec, jsc:jec, 2:km), Ice % bmelt  (isc:iec, jsc:jec, 2:km) , &
               Ice % pen    (isc:iec, jsc:jec, 2:km), Ice % trn    (isc:iec, jsc:jec, 2:km) , &
               Ice % h_snow (isd:ied, jsd:jed, 2:km), Ice % h_ice  (isd:ied, jsd:jed, 2:km) , &
               Ice % t_ice1 (isd:ied, jsd:jed, 2:km), Ice % t_ice2 (isd:ied, jsd:jed, 2:km)   )
    allocate ( Ice % qflx_lim_ice  (isc:iec, jsc:jec) , Ice % qflx_res_ice  (isc:iec, jsc:jec)   )

    Ice % flux_sw_vis     =0.
    Ice % flux_sw_dir     =0.
    Ice % flux_sw_dif     =0.
    Ice % flux_sw_vis_dir =0.
    Ice % flux_sw_vis_dif =0.
    Ice % flux_lh         =0. 
    Ice % lwdn            =0.
    Ice % swdn            =0.
! chum 2010-03-29 add id_swup
    Ice % swup            =0.
! chum 2010-03-29 add id_swup
    Ice % flux_u_top      =0. 
    Ice % flux_v_top      =0.
    Ice % sea_lev         =0.
    Ice % part_size       =0.
    Ice % u_ocn           =0.
    Ice % v_ocn           =0.
    Ice % u_ice           =0.
    Ice % v_ice           =0.
    Ice % sig11           =0.
    Ice % sig12           =0.
    Ice % sig22           =0.
    Ice % h_snow          =0.
    Ice % h_ice           =0.
    Ice % t_ice1          =0.
    Ice % t_ice2          =0.
!---------------------wangln 2007-11-20----------------------------------------------------------------
    Ice % atmos_tsurf           =0.
    Ice % atmos_tref            =0.
    Ice % atmos_qref            =0.
    Ice % previous_t_surf = Tfreeze-5.0
    Ice % atmos_swnet     =0.
    Ice % atmos_latent    =0.
    Ice % atmos_sen       =0.
    Ice % atmos_lwup      =0.
    Ice % atmos_evap      =0.
    Ice % atmos_taux      =0.
    Ice % atmos_tauy      =0.
    Ice % atmos_albedo    =0.
    !add by fangyj 20220222
    Ice % atmos_snowdp    =0.
    Ice % dhdt_surf       =0.
    Ice % drdt_surf       =0.
    Ice % dedt_surf       =0.
    Ice % ocn_swnet       =0. 
    Ice % ocn_q           =0. 
    Ice % ocn_t           =0. 
    Ice % ocn_p           =0. 
    Ice % ocn_taux        =0. 
    Ice % ocn_tauy        =0. 
    Ice % meltw           =0. 
    Ice % ifrac           =0. 
    Ice % pen             =0. 
    Ice % albedo_bareice    = 0.06 !! wutw 2009-10-23 add
!---------------------wangln 2007-11-20----------------------------------------------------------------

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
    Ice % u_10m             =0. 
    Ice % v_10m             =0. 
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------


    do j = jsc, jec
       do i = isc, iec
          Ice % mask(i,j) = wett(i,j)
       enddo
    enddo

    Ice % Time           = Time
    Ice % Time_Init      = Time_Init
    Ice % Time_step_fast = Time_step_fast
    Ice % Time_step_slow = Time_step_slow

    Ice % avg_kount      = 0
    !
    ! read restart
    !
    !! Need to create new restart version including the needed albedos
    !  that have been added ??

    if (file_exist(restart)) then
       !Balaji: netCDF restart via fms_io
!-------------------------------wangln 2007-11-05------------------------------
        write(stdout(),*) '---------SIS Read Restart Data-----------------------'
       call read_data( restart, 'part_size',   Ice%part_size,        domain )
       call read_data( restart, 'albedo',      Ice%albedo,           domain )
       call read_data( restart, 'rough_mom',   Ice%rough_mom,        domain )
       call read_data( restart, 'rough_heat',  Ice%rough_heat,       domain )
       call read_data( restart, 'rough_moist', Ice%rough_moist,      domain )
       call read_data( restart, 't_surf',      Ice%t_surf,           domain )
       call read_data( restart, 'h_snow',      Ice%h_snow(:,:,2:km), domain )
       call read_data( restart, 'h_ice',       Ice%h_ice (:,:,2:km), domain )
       call read_data( restart, 't_ice1',      Ice%t_ice1(:,:,2:km), domain )
       call read_data( restart, 't_ice2',      Ice%t_ice2(:,:,2:km), domain )
       call read_data( restart, 'u_ice',       Ice%u_ice,            domain )
       call read_data( restart, 'v_ice',       Ice%v_ice,            domain )
       call read_data( restart, 'sig11',       Ice%sig11,            domain )
       call read_data( restart, 'sig22',       Ice%sig22,            domain )
       call read_data( restart, 'sig12',       Ice%sig12,            domain )
       call read_data( restart, 'flux_u',      Ice%flux_u,           domain )
       call read_data( restart, 'flux_v',      Ice%flux_v,           domain )
       call read_data( restart, 'flux_t',      Ice%flux_t,           domain )
       call read_data( restart, 'flux_q',      Ice%flux_q,           domain )
       call read_data( restart, 'flux_salt',   Ice%flux_salt,        domain )
       call read_data( restart, 'flux_sw',     Ice%flux_sw,          domain )
       call read_data( restart, 'flux_lw',     Ice%flux_lw,          domain )
       call read_data( restart, 'lprec',       Ice%lprec,            domain )
       call read_data( restart, 'fprec',       Ice%fprec,            domain )
       call read_data( restart, 'runoff',      Ice%runoff,           domain )
       call read_data( restart, 'calving',     Ice%calving,          domain )
       call read_data( restart, 'p_surf',      Ice%p_surf,           domain )

  !---- assimil module by scnie----------------
       call init_assimil(Ice)         ! process of initial assimilation
       if (trim(assimil_type) == 'cycle') then
          call init_cycle_assimil(Ice)   ! initialization of cycle assimilation
       endif
  !---- end assimil module----------------

       !--- update to data domain
       call mpp_update_domains(Ice%part_size, Domain)
       call mpp_update_domains(Ice%h_snow(:,:,2:km), Domain )
       call mpp_update_domains(Ice%h_ice (:,:,2:km), Domain )
       call mpp_update_domains(Ice%t_ice1(:,:,2:km), Domain )
       call mpp_update_domains(Ice%t_ice2(:,:,2:km), Domain )
       call mpp_update_domains(Ice%u_ice, Domain )
       call mpp_update_domains(Ice%v_ice, Domain )
       call mpp_update_domains(Ice%sig11, Domain )
       call mpp_update_domains(Ice%sig22, Domain )
       call mpp_update_domains(Ice%sig12, Domain )
       
       write(stdout(),*) '-----Max Ice area=', maxval( Ice%part_size(:,:,2:km) ) 
       write(stdout(),*) '-----Max Ice area=', minval( Ice%part_size(:,:,2:km) ) 

    else ! no restart => no ice
       Ice % part_size    = 0.0
       !   where (Ice%mask) Ice % part_size (:,:,1) = 1.0  - flux_exchange won't allow
       Ice % part_size (:,:,1) = 1.0
       Ice % albedo            = 0.06 !! wutw 2009-10-19 0.0
       Ice % rough_mom         = mom_rough_ice
       Ice % rough_heat        = heat_rough_ice
       Ice % rough_moist       = heat_rough_ice
       Ice % t_surf            = Tfreeze-5.0
       Ice % h_snow            = 0.0
       Ice % h_ice             = 0.0
       Ice % t_ice1            = -5.0
       Ice % t_ice2            = -5.0
       Ice % u_ice             = 0.0
       Ice % v_ice             = 0.0
       Ice % sig11             = 0.0
       Ice % sig22             = 0.0
       Ice % sig12             = 0.0
       Ice % flux_u            = 0.0 
       Ice % flux_v            = 0.0
       Ice % flux_t            = 0.0 
       Ice % flux_q            = 0.0
       Ice % flux_sw           = 0.0 
       Ice % flux_lw           = 0.0
       Ice % flux_salt         = 0.0 
       Ice % lprec             = 0.0
       Ice % fprec             = 0.0
       Ice % p_surf            = 0.0
       Ice % runoff            = 0.0
       Ice % calving           = 0.0
       Ice % frazil            = 0.0
       do_init = .true. ! done in ice_model
    !!call get_sea_surface(Ice%Time, Ice%t_surf(:,:,1), Ice%part_size(isc:iec,jsc:jec,1:2), Ice%h_ice (isc:iec,jsc:jec,2))
    end if

  !! INITIALIZE the dir, dif and vis, nir albedoes appropriately.
!!! THIS NEEDS TO BE DEFINED PROPERLY -- ARE THEY NEEDED ON RESTART ??
!!!  FOR NOW, simply set all values to that of Ice%albedo, In general,
!! the ability to read from restart file and initialize when none
!! present should be incorporated above.

! wutw 2009-10-19
!!! Ice%albedo_vis_dir = 0.0         
!!! Ice%albedo_nir_dir = 0.0       
!!! Ice%albedo_vis_dif = 0.0       
!!! Ice%albedo_nir_dif = 0.0       
    Ice%albedo_vis_dir = 0.06        
    Ice%albedo_nir_dir = 0.06      
    Ice%albedo_vis_dif = 0.06      
    Ice%albedo_nir_dif = 0.06      

    Ice % tmelt       = 0.0
    Ice % bmelt       = 0.0

    Ice % qflx_lim_ice = 0.0
    Ice % qflx_res_ice = 0.0

    Ice%part_size_uv(:,:,1) = 1.0
    do k=2,km
       call t_to_uv(Ice%part_size(:,:,k),Ice%part_size_uv(:,:,k))
       Ice%part_size_uv (:,:,1) = Ice%part_size_uv(:,:,1)-Ice%part_size_uv (:,:,k)
    end do

 !================================= liuxw 20200616 ==============================
    call ida_init(Time_Init)         ! Ice%Time_Init, Ice%Time
 !================================= liuxw 20200616 ==============================

    call ice_diagnostics_init(Ice)
    !Balaji
    iceClock = mpp_clock_id( 'Ice', flags=clock_flag_default, grain=CLOCK_COMPONENT )
 
  end subroutine ice_model_init


  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
  ! ice_model_end - writes the restart file                                      !
  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
  subroutine ice_model_end (Ice)
    type (ice_data_type), intent(inout) :: Ice
    integer           :: k

    integer           :: unit
    character(len=22) :: restart='RESTART/ice_model.res'

    if (conservation_check) then
       do k=1,4
          call mpp_sum(h2o(k))
          call mpp_sum(heat(k))
       end do
       if (mpp_pe()==mpp_root_pe()) then
          !!print *
          !!print '(a10,5a13)',   'ICE MODEL ','   AT START  ', &
          write(stdout(),*) '------------------------------------------------------------------------------------'
          write(stdout(),'(a10,5a13)')   'ICE MODEL ','   AT START  ', &
               ' TOP FLUX DN.', &
               ' BOT FLUX DN.', &
               '   AT END    ', &
               '   ERROR     '
          write(stdout(),'(a10,5es13.5)') 'WATER     ', h2o , h2o (4)-(h2o (1)+h2o (2)-h2o (3))
          write(stdout(),'(a10,5es13.5)') 'HEAT      ', heat, heat(4)-(heat(1)+heat(2)-heat(3))
          write(stdout(),*) '------------------------------------------------------------------------------------'
          !!print '(a10,5es13.5)','WATER     ', h2o , h2o (4)-(h2o (1)+h2o (2)-h2o (3))
          !!print '(a10,5es13.5)','HEAT      ', heat, heat(4)-(heat(1)+heat(2)-heat(3))
          !!print *
       end if
    end if
    !Balaji: netCDF restart via fms_io
    call write_data( restart, 'part_size',   Ice%part_size,        domain )
    call write_data( restart, 'albedo',      Ice%albedo,           domain )
    call write_data( restart, 'rough_mom',   Ice%rough_mom,        domain )
    call write_data( restart, 'rough_heat',  Ice%rough_heat,       domain )
    call write_data( restart, 'rough_moist', Ice%rough_moist,      domain )
    call write_data( restart, 't_surf',      Ice%t_surf,           domain )
    call write_data( restart, 'h_snow',      Ice%h_snow(:,:,2:km), domain )
    call write_data( restart, 'h_ice',       Ice%h_ice (:,:,2:km), domain )
    call write_data( restart, 't_ice1',      Ice%t_ice1(:,:,2:km), domain )
    call write_data( restart, 't_ice2',      Ice%t_ice2(:,:,2:km), domain )
    call write_data( restart, 'u_ice',       Ice%u_ice,            domain )
    call write_data( restart, 'v_ice',       Ice%v_ice,            domain )
    call write_data( restart, 'sig11',       Ice%sig11,            domain )
    call write_data( restart, 'sig22',       Ice%sig22,            domain )
    call write_data( restart, 'sig12',       Ice%sig12,            domain )
    call write_data( restart, 'flux_u',      Ice%flux_u,           domain )
    call write_data( restart, 'flux_v',      Ice%flux_v,           domain )
    call write_data( restart, 'flux_t',      Ice%flux_t,           domain )
    call write_data( restart, 'flux_q',      Ice%flux_q,           domain )
    call write_data( restart, 'flux_salt',   Ice%flux_salt,        domain )
    call write_data( restart, 'flux_sw',     Ice%flux_sw,          domain )
    call write_data( restart, 'flux_lw',     Ice%flux_lw,          domain )
    call write_data( restart, 'lprec',       Ice%lprec,            domain )
    call write_data( restart, 'fprec',       Ice%fprec,            domain )
    call write_data( restart, 'runoff',      Ice%runoff,           domain )
    call write_data( restart, 'calving',     Ice%calving,          domain )
    call write_data( restart, 'p_surf',      Ice%p_surf,           domain )

    !--- release memory ------------------------------------------------

!!=====================wangln 2007-07-29 : To Output Restart File on Frequency Time===============
    !!call ice_grid_end()

    !!deallocate(Ice % mask, Ice % ice_mask, Ice % t_surf, Ice % s_surf, Ice % sea_lev )
    !!deallocate(Ice % part_size, Ice % part_size_uv, Ice % u_surf, Ice % v_surf )
    !!deallocate(Ice % u_ocn, Ice % v_ocn ,  Ice % rough_mom, Ice % rough_heat )
    !!deallocate(Ice % rough_moist, Ice % albedo, Ice % flux_u_top, Ice % flux_v_top )
    !!deallocate(Ice % flux_t_top, Ice % flux_q_top, Ice % flux_sw_top, Ice % flux_lw_top )
    !!deallocate(Ice % flux_lh_top, Ice % lprec_top, Ice % fprec_top, Ice % flux_u )
    !!deallocate(Ice % flux_v, Ice % flux_t, Ice % flux_q, Ice % flux_sw, Ice % flux_lw )
    !!deallocate(Ice % flux_lh, Ice % lprec, Ice % fprec, Ice % p_surf, Ice % runoff ) 
    !!deallocate(Ice % calving)
    !!deallocate(Ice % flux_salt)
    !!deallocate( Ice % lwdn)
    !!deallocate( Ice % swdn)
    !!deallocate( Ice % frazil )
    !!deallocate(Ice % bheat, Ice % u_ice, Ice % v_ice, Ice % sig11, Ice % sig22 )
    !!deallocate(Ice % sig12, Ice % tmelt, Ice % bmelt, Ice % pen, Ice % trn )
    !!deallocate(Ice % h_snow, Ice % h_ice, Ice % t_ice1, Ice % t_ice2  )
    !!deallocate(Ice % qflx_lim_ice, Ice % qflx_res_ice )
!!=====================wangln 2007-07-29 : To Output Restart File on Frequency Time===============

  end subroutine ice_model_end

  !#######################################################################

  subroutine ice_diagnostics_init(Ice)
    type (ice_data_type), intent(in) :: Ice

!!=================wangln 2007-08-21 According with the CPL=====================
    !!real, parameter       :: missing = -1e34
    real, parameter       :: missing = -1e30
!!=================wangln 2007-08-21 According with the CPL=====================

    integer, dimension(2) :: axt, axv, axtv, axvt
    integer, dimension(3) :: axt2
    integer               :: id_geo_lon, id_geo_lat, id_sin_rot, id_cos_rot, id_cell_area
    logical               :: sent

    !
    ! diagnostics MUST use a domain without halos otherwise same as the
    ! regular domain:  Domain (see ice_grid.f90)
    !
    id_xv = diag_axis_init('xv', xb1d(2:im+1), 'degrees_E', 'X','longitude', set_name='ice', Domain2=Domain )
    id_yv = diag_axis_init('yv', yb1d(2:jm+1), 'degrees_N', 'Y','latitude',  set_name='ice', Domain2=Domain )
    id_xb = diag_axis_init('xb', xb1d, 'degrees_E', 'X', 'longitude', set_name='ice', Domain2=Domain )
    id_yb = diag_axis_init('yb', yb1d, 'degrees_N', 'Y', 'latitude', set_name='ice', Domain2=Domain )
    id_xt = diag_axis_init('xt', (xb1d(1:im)+xb1d(2:im+1))/2, 'degrees_E', 'X', &
            'longitude',set_name='ice',edges=id_xb,Domain2=Domain)
    id_yt = diag_axis_init('yt', (yb1d(1:jm)+yb1d(2:jm+1))/2, 'degrees_N', 'Y', &
            'latitude',set_name='ice', edges=id_yb,Domain2=Domain)
    id_ct = diag_axis_init('ct', hlim(1:num_part-1), 'meters','Z', 'thickness')
    axv  = (/ id_xv, id_yv       /)
    axt  = (/ id_xt, id_yt       /)
    axt2 = (/ id_xt, id_yt, id_ct/)
    axtv = (/ id_xt, id_yv /); ! for north faces of t-cells
    axvt = (/ id_xv, id_yt /); ! for east  faces of t-cells

    ! note: set all require=.true. when diag_manager writes these
    !       fields only to "ice_model"
    id_sin_rot   = register_static_field('ice_model', 'SINROT', axt,              &
                   '-SINROT,COSROT points north', 'none', require=.false.)
    id_cos_rot   = register_static_field('ice_model', 'COSROT', axt,              &
                   'COSROT,SINROT points east','none', require=.false.)
    id_geo_lon   = register_static_field('ice_model', 'GEOLON', axt, 'longitude', &
                   'degrees', require=.false.)
    id_geo_lat   = register_static_field('ice_model', 'GEOLAT', axt, 'latitude',  &
                   'degrees', require=.false.)
    id_cell_area = register_static_field('ice_model', 'CELL_AREA', axt,           &
                   'cell area', 'sphere',require=.false.)
    id_ext       = register_diag_field('ice_model', 'MOI', axt, Ice%Time,   &
                   'ice modeled', '0 or 1', missing_value=missing)
    if (id_ext > 0 ) then
       call error_mesg ('ice_model_init', &
            'Diagnostic MOI has been renamed EXT.  Change your diag_table.', WARNING)
    else
       id_ext = register_diag_field('ice_model', 'EXT', axt, Ice%Time, &
                'ice modeled', '0 or 1', missing_value=missing)
    end if
    id_mi       = register_diag_field('ice_model', 'MI', axt, Ice%Time,                  &
                 'ice mass', 'kg/m^2', missing_value=missing)
    id_cn       = register_diag_field('ice_model', 'CN', axt2, Ice%Time,                 &
                 'ice concentration', '0-1', missing_value=missing)
    id_hs       = register_diag_field('ice_model', 'HS', axt, Ice%Time,                  &
                 'snow thickness', 'm-snow', missing_value=missing)
    id_hi       = register_diag_field('ice_model', 'HI', axt, Ice%Time,                  &
                 'ice thickness', 'm-ice', missing_value=missing)
    id_t1       = register_diag_field('ice_model', 'T1', axt, Ice%Time,                  &
                 'upper ice layer temperature', 'C',  missing_value=missing)
    id_t2       = register_diag_field('ice_model', 'T2', axt, Ice%Time,                  &
                 'lower ice layer temperature', 'C',  missing_value=missing)
    id_ts       = register_diag_field('ice_model', 'TS', axt, Ice%Time,                  &
                 'surface temperature', 'C', missing_value=missing)
    id_sh       = register_diag_field('ice_model','SH' ,axt, Ice%Time,                   &
                 'sensible heat flux', 'W/m^2',  missing_value=missing)
    id_lh       = register_diag_field('ice_model','LH' ,axt, Ice%Time,                   &
                 'latent heat flux', 'W/m^2', missing_value=missing)
    id_sw       = register_diag_field('ice_model','SW' ,axt, Ice%Time,                   &
                 'short wave heat flux', 'W/m^2', missing_value=missing)
    id_lw       = register_diag_field('ice_model','LW' ,axt, Ice%Time,                   &
                 'long wave heat flux over ice', 'W/m^2', missing_value=missing)
    id_snofl    = register_diag_field('ice_model','SNOWFL' ,axt, Ice%Time,               &
                 'rate of snow fall', 'kg/(m^2*s)', missing_value=missing)
    id_rain     = register_diag_field('ice_model','RAIN' ,axt, Ice%Time,                 &
                 'rate of rain fall', 'kg/(m^2*s)', missing_value=missing)
    id_runoff   = register_diag_field('ice_model','RUNOFF' ,axt, Ice%Time,               &
                 'liquid runoff', 'kg/(m^2*s)', missing_value=missing)
    id_calving  = register_diag_field('ice_model','CALVING',axt, Ice%Time,               &
                 'frozen runoff', 'kg/(m^2*s)', missing_value=missing)
    id_evap     = register_diag_field('ice_model','EVAP',axt, Ice%Time,                  &
                 'evaporation', 'kg/(m^2*s)', missing_value=missing)
    id_saltf    = register_diag_field('ice_model','SALTF' ,axt, Ice%Time,                &
                 'ice to ocean salt flux', 'kg/(m^2*s)', missing_value=missing)
    id_sn2ic    = register_diag_field('ice_model','SN2IC'  ,axt,Ice%Time,                &
                 'rate of snow to ice conversion', 'kg/(m^2*s)', missing_value=missing)
! wutw 2009-10-23 add SNOWMELT output units??==>
    id_snowmelt = register_diag_field('ice_model','SNOWMELT'  ,axt,Ice%Time,             &
                 'SNOW MELT', 'kg/(m^2*s)', missing_value=missing)
! wutw 2009-10-23 add SNOWMELT output <==
    id_tmelt    = register_diag_field('ice_model','TMELT'  ,axt, Ice%Time,               &
                 'upper surface melting energy flux', 'W/m^2', missing_value=missing)
    id_bmelt    = register_diag_field('ice_model','BMELT'  ,axt, Ice%Time,               &
                 'bottom surface melting energy flux', 'W/m^2', missing_value=missing)
    id_bheat    = register_diag_field('ice_model','BHEAT'  ,axt, Ice%Time,               &
                 'ocean to ice heat flux', 'W/m^2', missing_value=missing)
    id_e2m      = register_diag_field('ice_model','E2MELT' ,axt, Ice%Time,               &
                 'heat needed to melt ice', 'J/m^2', missing_value=missing)
    id_frazil   = register_diag_field('ice_model','FRAZIL' ,axt, Ice%Time,               &
                 'energy flux of frazil formation', 'W/m^2', missing_value=missing)
    id_alb      = register_diag_field('ice_model','ALB',axt, Ice%Time,                   &
                 'surface albedo','0-1', missing_value=missing )
! wutw 2009-10-23 add register id_albbar ==>
    id_albbar   = register_diag_field('ice_model','ALBBARE',axt, Ice%Time,               &
                 'bare ice surface albedo','0-1', missing_value=missing )
! wutw 2009-10-23 add register id_albbar <==
    id_xprt     = register_diag_field('ice_model','XPRT',axt, Ice%Time,                  &
                 'frozen water transport convergence', 'kg/(m^2*yr)', missing_value=missing)
    id_lsrc     = register_diag_field('ice_model','LSRC', axt, Ice%Time,                 &
                 'frozen water local source', 'kg/(m^2*yr)', missing_value=missing)
    id_lsnk     = register_diag_field('ice_model','LSNK',axt, Ice%Time,                  &
                 'frozen water local sink', 'kg/(m^2*yr)', missing_value=missing)
    id_bsnk     = register_diag_field('ice_model','BSNK',axt, Ice%Time,                  &
                 'frozen water local bottom sink', 'kg/(m^2*yr)', missing_value=missing)
    id_qfres    = register_diag_field('ice_model', 'QFLX_RESTORE_ICE', axt, Ice%Time,    &
                 'Ice Restoring heat flux', 'W/m^2', missing_value=missing)
    id_qflim    = register_diag_field('ice_model', 'QFLX_LIMIT_ICE', axt, Ice%Time,      &
                 'Ice Limit heat flux', 'W/m^2', missing_value=missing)
    id_strna    = register_diag_field('ice_model','STRAIN_ANGLE', axt,Ice%Time,          &
                 'strain angle', 'none', missing_value=missing)
    id_sigi     = register_diag_field('ice_model','SIGI' ,axt, Ice%Time,                 &
                 'first stress invariant', 'none', missing_value=missing)
    id_sigii    = register_diag_field('ice_model','SIGII' ,axt, Ice%Time,                &
                 'second stress invariant', 'none', missing_value=missing)
    id_stren    = register_diag_field('ice_model','STRENGTH' ,axt, Ice%Time,             &
                 'ice strength', 'Pa*m', missing_value=missing)
    id_ui       = register_diag_field('ice_model', 'UI', axv, Ice%Time,                  &
                 'ice velocity - x component', 'm/s', missing_value=missing)
    id_vi       = register_diag_field('ice_model', 'VI', axv, Ice%Time,                  &
                 'ice velocity - y component', 'm/s', missing_value=missing)
    id_ix_trans = register_diag_field('ice_model', 'IX_TRANS', axvt, Ice%Time,           &
                 'x-direction ice transport', 'kg/s', missing_value=missing)
    id_iy_trans = register_diag_field('ice_model', 'IY_TRANS', axtv, Ice%Time,           &
                 'y-direction ice transport', 'kg/s', missing_value=missing)
    id_fax      = register_diag_field('ice_model', 'FA_X', axv, Ice%Time,                &
                 'air stress on ice - x component', 'Pa', missing_value=missing)
    id_fay      = register_diag_field('ice_model', 'FA_Y', axv, Ice%Time,                &
                 'air stress on ice - y component', 'Pa', missing_value=missing)
    id_fix      = register_diag_field('ice_model', 'FI_X', axv, Ice%Time,                &
                 'ice internal stress - x component', 'Pa', missing_value=missing)
    id_fiy      = register_diag_field('ice_model', 'FI_Y', axv, Ice%Time,                &
                 'ice internal stress - y component', 'Pa', missing_value=missing)
    id_fcx      = register_diag_field('ice_model', 'FC_X', axv, Ice%Time,                &
                 'coriolis force - x component', 'Pa', missing_value=missing)
    id_fcy      = register_diag_field('ice_model', 'FC_Y', axv, Ice%Time,                &
                 'coriolis force - y component', 'Pa', missing_value=missing)
    id_fwx      = register_diag_field('ice_model', 'FW_X', axv, Ice%Time,                &
                 'water stress on ice - x component', 'Pa', missing_value=missing)
    id_fwy      = register_diag_field('ice_model', 'FW_Y', axv, Ice%Time,                &
                 'water stress on ice - y component', 'Pa', missing_value=missing)
    id_uo       = register_diag_field('ice_model', 'UO', axv, Ice%Time,                  &
                 'surface current - x component', 'm/s', missing_value=missing)
    id_vo       = register_diag_field('ice_model', 'VO', axv, Ice%Time,                  &
                 'surface current - y component', 'm/s', missing_value=missing)
    !
    ! diagnostics for quantities produced outside the ice model
    !
    id_swdn  = register_diag_field('ice_model','SWDN' ,axt, Ice%Time,       &
               'downward shortwave flux', 'W/m^2', missing_value=missing)
! chum 2010-03-29 add id_swup
    id_swup  = register_diag_field('ice_model','SWUP' ,axt, Ice%Time,       &
               'upward shortwave flux', 'W/m^2', missing_value=missing)
! chum 2010-03-29 add id_swup
    id_lwdn  = register_diag_field('ice_model','LWDN' ,axt, Ice%Time,       &
               'downward longwave flux', 'W/m^2', missing_value=missing)
    id_ta    = register_diag_field('ice_model', 'TA', axt, Ice%Time,        &
               'surface air temperature', 'C', missing_value=missing)
    id_slp   = register_diag_field('ice_model', 'SLP', axt, Ice%Time,       &
               'sea level pressure', 'Pa', missing_value=missing)
    id_sst   = register_diag_field('ice_model', 'SST', axt, Ice%Time,       &
               'sea surface temperature', 'deg-C', missing_value=missing)
    id_sss   = register_diag_field('ice_model', 'SSS', axt, Ice%Time,       &
               'sea surface salinity', 'psu', missing_value=missing)
    id_ssh   = register_diag_field('ice_model', 'SSH', axt, Ice%Time,       &
               'sea surface height', 'm', missing_value=missing)
    id_obi   = register_diag_field('ice_model', 'OBI', axt, Ice%Time,       &
         'ice observed', '0 or 1', missing_value=missing)

    if (id_sin_rot>0)   sent=send_data(id_sin_rot, sin_rot(isc:iec,jsc:jec), Ice%Time);
    if (id_cos_rot>0)   sent=send_data(id_cos_rot, cos_rot(isc:iec,jsc:jec), Ice%Time);
    if (id_geo_lon>0)   sent=send_data(id_geo_lon, geo_lon, Ice%Time);
    if (id_geo_lat>0)   sent=send_data(id_geo_lat, geo_lat, Ice%Time);
    if (id_cell_area>0) sent=send_data(id_cell_area, cell_area, Ice%Time);

  end subroutine ice_diagnostics_init



 !================================= liuxw 20200616 ==============================
 !#######################################################################
! <SUBROUTINE NAME="ocean_data_assimilation">
  !------------------------------------------------------
  subroutine ida_init(initime)

    type(time_type), intent(in)  :: initime
    integer :: ioun, io_status, ierr
    integer :: date(6)
    integer :: tmpdate,i

    ioun = open_namelist_file()
    read  (ioun, ice_data_ass_nml,iostat=io_status)
    write (stdout(),'(/)')
    write (stdout(), ice_data_ass_nml)
    write (stdlog(), ice_data_ass_nml)
    ierr = check_nml_error(io_status,'ice_data_ass_nml')
    call close_file (ioun)

    call get_date(initime, date(1), date(2), date(3), date(4), date(5), date(6))
    inidate = date(1)*10000+date(2)*100+date(3)
    write(stdout(),*) '--ida inidate info--', inidate

    if(ida_use)then

    write (stdout(),'(a,1x,3i10)')'ida initialization: inidate,ida_start,ida_stop = ',inidate,ida_start,ida_stop
    if(ida_start>ida_stop)then
      call mpp_error(FATAL,'==>Error in ida_start or ida_stop value')
    endif

    tmpdate = ida_start
    call datelength(ida_start,ida_stop,ida_window,numwd)
    allocate(wddates(numwd))
    wddates(1) = ida_start
    if(numwd>1)then
      do i = 2,numwd
      call objdate(tmpdate,ida_window)
      wddates(i)=tmpdate
      enddo
    endif

    id_currwddate = 0
    id_nextwddate = 0
    if(inidate<wddates(1))then
      id_nextwddate = 1
    elseif(inidate==wddates(1))then
      id_currwddate = 1
      id_nextwddate = 2
    elseif(inidate>=wddates(numwd))then
      id_currwddate = numwd
    else
      do i=2,numwd
      if(wddates(i-1)<=inidate.and.inidate<wddates(i))then
        id_currwddate=i-1
        id_nextwddate=i
        exit
      endif
      enddo
    endif
    write (stdout(),'(a,1x,i10,3i5)')'(ida_init) inidate,numwindow,id_currwddate,id_nextwddate = ',inidate,numwd,id_currwddate,id_nextwddate
    if(id_currwddate/=0)write (stdout(),'(a,1x,2i10)')'(ida_init) inidate,currwddate = ',inidate,wddates(id_currwddate)
    if(id_nextwddate/=0)write (stdout(),'(a,1x,2i10)')'(ida_init) inidate,nextwddate = ',inidate,wddates(id_nextwddate)
    if(id_currwddate==0.and.id_nextwddate==0)call mpp_error(FATAL,'==>(ida_init) unknown position of inidate among windowdates')
    if(id_nextwddate/=0)firstana=.true.

    allocate(ana_ci(isd:ied,jsd:jed))
    allocate(ana_hi(isd:ied,jsd:jed))
    allocate(ana_ui(isd:ied,jsd:jed))
    allocate(ana_vi(isd:ied,jsd:jed))
   !ana_ci   = 0.0 
   !ana_hi   = 0.0 
   !ana_ui   = 0.0 
   !ana_vi   = 0.0 
    ana_ci   = ocnmiss
    ana_hi   = ocnmiss
    ana_ui   = ocnmiss
    ana_vi   = ocnmiss

    endif

  end subroutine ida_init

  !------------------------------------------------------
  subroutine ida_operate(Ice,timeice,dtime)

    type (ice_data_type), intent(in) :: Ice
    type(time_type), intent(in)      :: timeice
    integer,intent(in)               :: dtime

    integer :: yr, mon, day, hr, minite, sec
    integer :: nowdate, currsecs
    logical :: useanafile

    dtts = dtime
    call get_date(timeice, yr, mon, day, hr, minite, sec)
    write (stdout(),'(a,1x,7i5)')'(ida_operate) time ',yr,mon,day,hr,minite,sec,dtts
    nowdate = yr*10000+mon*100+day
    if( ida_use .and. nowdate >= ida_start .and. nowdate <= ida_stop )then
      currsecs = hr*3600 + minite*60 + sec
      write (stdout(),'(a,1x,3i5)')'(ida_operate) numwindow,id_currdate,id_nextdate = ',numwd,id_currwddate,id_nextwddate
      if(id_currwddate>=1.and.id_currwddate<=numwd)then
        useanafile = ( nowdate==wddates(id_currwddate) ) .or. ( nowdate>wddates(id_currwddate).and.nowdate==inidate )
        if( useanafile .and. currsecs==dtts )then
          read_ana=1
          call ida_update(Ice)
        ! if( nowdate==wddates(id_currwddate) )call ida_update(Ice)
        else
          read_ana=0
        endif
      else
        write (stdout(),'(a,1x,6i5)')'(ida_operate) in ida range, but do not need ida update at ',yr,mon,day,hr,minite,sec
      endif
    endif

  end subroutine ida_operate

  !------------------------------------------------------
  subroutine ida_analysis(date)

  integer,intent(in) :: date(6)
  integer   :: nowdate
  character(len=8) :: cdate
  integer   :: unit = 101
  integer   :: sleepstep
  logical   :: have_file, ida_out

  nowdate = date(1)*10000+date(2)*100+date(3)

  if( ida_use .and. nowdate >= ida_start .and. nowdate <= ida_stop )then    ! nowdate >= ida_start will make analysis on starting date

   if(id_nextwddate>=1.and.id_nextwddate<=numwd)then

    if( nowdate==wddates(id_nextwddate) )then

      if(firstana)then
        write(stdout(),*) '(ida_analysis) Waiting for IDA startup ...'
        sleepstep = 0
        if (mpp_pe() == mpp_root_pe()) then
          do while(ida_use)
            inquire( file=trim(ida_wkdir)// '/exec/NOUT-000', exist=ida_out )
            if(ida_out)then
              write(stdout(),*) '(ida_analysis) IDA Moduel is activated!'
              exit
            else
              call system('/usr/bin/sleep 5')
              sleepstep = sleepstep + 1
              write(stdout(),*) '(ida_analysis) startup waiting step ', sleepstep
              if(sleepstep >= 10*waitstep_limit)stop 'Stop: long time to wait for IDA startup! Stop due to impatient waiting!'
            endif
          enddo
        endif
        firstana = .false.
      endif

      call mpp_sync()
      write(stdout(),*) '(ida_analysis) Assimilation day: ',nowdate
      write(stdout(),*) '(ida_analysis) Writing assimilation flag file ...'
      write(cdate,'(i8.8)')nowdate
      if (mpp_pe() == mpp_root_pe()) then
        call mpp_open( unit, trim(ida_wkdir)//'/exec/ida_'//cdate, nohdrs=.TRUE. )           ! 20210316  assimilation -> ida
        write( unit, * )date(1),date(2),date(3),date(4),date(5),date(6)
        call mpp_close(unit)
      endif
      sleepstep = 0
      write(stdout(),*) '(ida_analysis) Waiting for analysis ...'
      if (mpp_pe() == mpp_root_pe()) then
        do while(ida_use)
        inquire( file=trim(ida_wkdir)// '/exec/ida_'//cdate, exist=have_file )               ! 20210316  assimilation -> ida
        if(have_file)then
          call system('/usr/bin/sleep 5')
          sleepstep = sleepstep + 1
          write(stdout(),*) '(ida_analysis) integration waiting step ', sleepstep
          inquire( file=trim(ida_wkdir)// '/exec/NOUT-000', exist=ida_out )
          if(sleepstep >= waitstep_limit)call mpp_error(FATAL,'==>long time to wait for analysis! IDA module may be terminated')
        else
          write(stdout(),*) '(ida_analysis) logfile is deleted! Analysis data is prepared', cdate
          exit
        endif
        enddo
      endif
      call mpp_sync()

      id_currwddate = id_nextwddate
      id_nextwddate = id_nextwddate + 1

    else

      write(stdout(),*) '(ida_analysis) model date is not on the ida window open date'

    endif

   else

    write (stdout(),'(a,1x,2i10,3i5)')'(ida_analysis) out-of-range id_nextwddate: nowdate,ida_stop,numwindow,id_currwddate,id_nextwddate = ',  &
                                       nowdate,ida_stop,numwd,id_currwddate,id_nextwddate

   endif

  else

    write(stdout(),*) '(ida_analysis) ida_use option is not opened, or current date is beyond the valid ida range'

  if(nowdate > ida_stop)then
     if( allocated(ana_ci) .and. allocated(ana_hi) )then
     deallocate(ana_ci)
     deallocate(ana_hi)
     deallocate(ana_ui)
     deallocate(ana_vi)
     endif
     if(allocated(wddates))deallocate(wddates)
  endif

  endif

  return

  end subroutine ida_analysis
  !------------------------------------------------------
  subroutine ida_update(Ice)

  type (ice_data_type), intent(in) :: Ice

  integer :: ii, jj, kk
  integer :: yr, mon, day, hr, minite, sec
  character(len=100) :: FileName
  character(len=4) :: yyyy
  character(len=2) :: mm
  character(len=2) :: dd
  character(len=8) :: cdate
  real    :: ana_coeff
  real    :: sic_bg,sit_bg,sic_ana,sit_ana 
  logical :: alive, allmissing,isumask
  logical :: isnan
  
  !--------------------------------------------------------------
  real, allocatable, dimension(:,:)    :: anaci_glob               ! or real(kind=8)
  real, allocatable, dimension(:,:)    :: anahi_glob
  real, allocatable, dimension(:,:)    :: anaui_glob
  real, allocatable, dimension(:,:)    :: anavi_glob
  integer, dimension(3)                :: start3,count3
  integer, dimension(4)                :: start4,count4
  integer                              :: rcode,file_id,var_id,ierr
  include 'mpif.h'
  include 'netcdf.inc'
  !--------------------------------------------------------------

  call get_date(Ice%Time, yr, mon, day, hr, minite, sec)
  write(yyyy,'(i4.4)')yr
  write(mm,'(i2.2)')mon
  write(dd,'(i2.2)')day

  write(cdate,'(i8.8)')wddates(id_currwddate)
  write (stdout(),'(3(a,1x))')'(ida_update) modeldate,currwddate = ',yyyy//mm//dd,cdate

  if(read_ana==1)then
    FileName = trim(ida_wkdir)//'/anal/anal_'//cdate//'_mean.nc'
    inquire( file=trim(FileName), exist=alive)
    if(alive)then
        write(stdout(),*)'read global analysis field from ', trim(FileName)
        allocate(anaci_glob(im,jm))
        allocate(anahi_glob(im,jm))
        allocate(anaui_glob(im,jm))
        allocate(anavi_glob(im,jm))
        if(mpp_pe() == mpp_root_pe()) then
          start3(1)=1
          count3(1)=im
          start3(2)=1
          count3(2)=jm
          start3(3)=1
          count3(3)=1
          rcode = nf_open(trim(FileName),NF_OPEN,file_id)
          if(rcode/=0)call mpp_error(FATAL,'==>ida_update: failure to open ana data')
          rcode = nf_inq_varid( file_id, 'ci', var_id )
          if(rcode/=0)call mpp_error(FATAL,'==>ida_update: failure to find ci variable in ana data')
          rcode = nf_get_vara_double( file_id, var_id, start3, count3, anaci_glob )
          if(rcode/=0) call mpp_error(FATAL,'==>ida_update: failure to read ci variable in ana data')
          rcode = nf_inq_varid( file_id, 'hi', var_id )
          if(rcode/=0)call mpp_error(FATAL,'==>ida_update: failure to find hi variable in ana data')
          rcode = nf_get_vara_double( file_id, var_id, start3, count3, anahi_glob )
          if(rcode/=0) call mpp_error(FATAL,'==>ida_update: failure to read hi variable in ana data')
          rcode = nf_close(file_id)
        endif
        call MPI_BCAST(anaci_glob,im*jm,MPI_REAL8,0,MPI_COMM_ICE,ierr)
        call MPI_BCAST(anahi_glob,im*jm,MPI_REAL8,0,MPI_COMM_ICE,ierr)
        call MPI_BCAST(anaui_glob,im*jm,MPI_REAL8,0,MPI_COMM_ICE,ierr)
        call MPI_BCAST(anavi_glob,im*jm,MPI_REAL8,0,MPI_COMM_ICE,ierr)
        ana_ci(isd:ied,jsd:jed)=anaci_glob(isd:ied,jsd:jed)
        ana_hi(isd:ied,jsd:jed)=anahi_glob(isd:ied,jsd:jed)
        ana_ui(isd:ied,jsd:jed)=anaui_glob(isd:ied,jsd:jed)
        ana_vi(isd:ied,jsd:jed)=anavi_glob(isd:ied,jsd:jed)
        !write(stdout(),*)'check points ',mpp_root_pe(),mpp_pe(),isc,jsc,ana_ci(isc,jsc), ana_hi(isc,jsc),ana_ui(isc,jsc),ana_vi(isc,jsc)
        deallocate(anaci_glob)
        deallocate(anahi_glob)
        deallocate(anaui_glob)
        deallocate(anavi_glob)

        id_anafile = yyyy//mm//dd
    else
      call mpp_error(FATAL,'==>Error in ida_update: failure to find analysis data')
    endif
  endif

  allmissing = .false.
  ana_coeff = 1.0
  if( all(ana_ci<ocnmiss+1) .and. all(ana_hi<ocnmiss+1) .and. all(ana_ui<ocnmiss+1) .and. all(ana_vi<ocnmiss+1) )then
     ana_coeff = 0.
     allmissing = .true.
     write(stdout(),100)'(ida_update) all invalid values for analysis ',yr,mon,day,hr,minite,sec,'read_ana = ',read_ana,ana_coeff,id_anafile
  endif
  100 format(a,1x,i5,5i3,2x,a11,i2,f8.4,2x,a)


  isnan = .false.
  if(.not.allmissing)then
  write(stdout(),100)'(ida_update) update analyses ',yr,mon,day,hr,minite,sec,'read_ana = ',read_ana,ana_coeff,id_anafile
  do ii=isc,iec
  do jj=jsc,jec
    if(Ice%mask(ii,jj))then
      if(ana_coeff==0..or.ana_ci(ii,jj)<ocnmiss+1.or.ana_hi(ii,jj)<ocnmiss+1)   &
         call mpp_error(FATAL,'==>Error in ice_type_mod: wrong ana_ci or ana_hi points')
      if(abs(ana_ci(ii,jj))<1.0E-5)ana_ci(ii,jj)=0.
      if(abs(ana_hi(ii,jj))<1.0E-5)ana_hi(ii,jj)=0.

      sic_bg=sum(Ice%part_size(ii,jj,2:km))
      sit_bg=0.
      do kk=2,km
        sit_bg=sit_bg+Ice%part_size(ii,jj,kk)*Ice%h_ice(ii,jj,kk)
      enddo
      if(sic_bg>=1.0E-5)then
        sit_bg=sit_bg/sic_bg
      else
        sic_bg=0.
        sit_bg=0.
      endif

      ana_ci(ii,jj)=sic_bg+ana_ci(ii,jj)
      ana_hi(ii,jj)=sit_bg+ana_hi(ii,jj)
      if(ana_ci(ii,jj)>1.0)ana_ci(ii,jj)=1.0
      if(ana_ci(ii,jj)<0.0)ana_ci(ii,jj)=0.0
      if(ana_hi(ii,jj)<0.0)ana_hi(ii,jj)=0.0

      if(ana_ci(ii,jj)==0.0.and.ana_hi(ii,jj)>0.0)then
         ana_hi(ii,jj)=0.0  
      endif
      if(ana_ci(ii,jj)>0.0.and.ana_hi(ii,jj)==0.0)then
         ana_ci(ii,jj)=0.0          
      endif

    endif

  enddo
  enddo
  endif

  if(isnan)then
    call mpp_sync()
    write(stdout(),*)'Nan points are found'
    call mpp_error(FATAL,'==>Error in ocean_model_mod: NanQ points')
  endif

  end subroutine ida_update
  !------------------------------------------------------
!---------------------------------------------------------------------------------------
subroutine remap_ice(sic, sit, ai, hi, hs, t1, t2, ts)

    implicit none

    ! interface
    real,                intent(in   )  ::  sic
    real,                intent(in   )  ::  sit
    real,  dimension(5), intent(inout)  ::  ai
    real,  dimension(5), intent(inout)  ::  hi
    real,  dimension(5), intent(inout)  ::  hs
    real,  dimension(5), intent(inout)  ::  t1
    real,  dimension(5), intent(inout)  ::  t2
    real,  dimension(5), intent(inout)  ::  ts

    ! local
    real, dimension(5)            :: rc
    real, dimension(5)            :: rt
    real, dimension(5)            :: vi
    real, dimension(5)            :: ai_old
    real, dimension(5)            :: hi_old
    real, dimension(5)            :: hs_old
    real, dimension(5)            :: t1_old
    real, dimension(5)            :: t2_old
    real, dimension(5)            :: ts_old
    real                          :: dT1
    real                          :: dT2
    real, parameter               :: Tf   = 271.36
    real, parameter               :: T0   = 273.16
    real, dimension(5), parameter :: hdef = (/0.0, 0.1, 0.3, 0.7, 1.1/)

    integer :: i
    integer :: j
    integer :: k

    ai_old = ai
    hi_old = hi
    hs_old = hs
    t1_old = t1
    t2_old = t2
    ts_old = ts

    ! ratio
    rc     = ai_old / sum(ai_old)
    rt     = ai_old * hi_old / sum(ai_old * hi_old)

    ! new ice
    ai = rc * sic
    vi = sit * sum(ai) * rt

    ! new hi
    do i = 1, 5
        if (ai(i) < 1.0E-4) then
            ai(i) = ai_old(i)
            hi(i) = hi_old(i)
        else
            hi(i) = vi(i) / ai(i)
        end if
    end do

    !-------------------------------------
    ! redistribution
    !-------------------------------------
    ! lower bound
    do i = 5, 2, -1
        if ( hi(i) > 0.0 .and. hi(i) < hdef(i) ) then
            if ( (ai(i) + ai(i-1)) < 1.0E-6 ) then
                hi(i-1) = hi(i)
                hs(i-1) = hs(i)
                ai(i-1) = ai(i)
            else
                hi(i-1) = (ai(i) * hi(i) + ai(i-1) * hi(i-1)) / (ai(i-1) + ai(i))
                hs(i-1) = (ai(i) * hs(i) + ai(i-1) * hs(i-1)) / (ai(i-1) + ai(i))
                ai(i-1) = ai(i-1) + ai(i)
            end if
            hi(i) = 0.0
            hs(i) = 0.0
            ai(i) = 0.0
        end if
    end do

    ! upper bound
    do i = 1, 4
        if ( hi(i) > 0.0 .and. hi(i) > hdef(i) ) then
            if ( (ai(i) + ai(i+1)) < 1.0E-6 ) then
                hi(i+1) = hi(i)
                hs(i+1) = hs(i)
                ai(i+1) = ai(i)
            else
                hi(i+1) = (ai(i) * hi(i) + ai(i+1) * hi(i+1)) / (ai(i+1) + ai(i))
                hs(i+1) = (ai(i) * hs(i) + ai(i+1) * hs(i+1)) / (ai(i+1) + ai(i))
                ai(i+1) = ai(i+1) + ai(i)
            end if
            hi(i) = 0.0
            hs(i) = 0.0
            ai(i) = 0.0
        end if
    end do

    !-------------------------------------
    ! change T1, T2
    !-------------------------------------
    do i = 1, 5
        if ( hi_old(i) > 0.0 ) then
            if( hi(i) < 1.0E-06 ) then
                hi(i) = 0.0
                t1(i) = 0.0 
                t2(i) = 0.0
                ts(i) = Tf
            else
                if ( hi_old(i) > 0.01 .and. hi(i) > 0.01 ) then
                    dT2   = ((Tf - T0) - t2_old(i)) / (0.25 * hi_old(i))
                    t2(i) =  (Tf - T0) - dT2 * (0.25 * hi(i))
                    if ( t2(i) > 0.0 )  t2(i) = t2_old(i)
                    dT1   = (t2_old(i) - t1_old(i)) / (0.5 * hi_old(i))
                    t1(i) = t2(i) - dT1 * (0.5 * hi(i))
                    if ( t1(i) > 0.0 )  t1(i) = t1_old(i)
                end if
            end if
        else
            if( hi(i) < 1.0E-06 ) then
                hi(i) = 0.0
                t1(i) = 0.0
                t2(i) = 0.0
                ts(i) = Tf
            else
                k = 0
                do j = 1, 4
                    if ((i-j) > 0 .and. hi_old(i-j) > 0.01) then
                        k = i - j
                        exit
                    end if
                    if ((i+j) < 6 .and. hi_old(i+j) > 0.01) then
                        k = i + j
                        exit
                    end if
                end do

                if (k > 0) then
                    dT2   = ((Tf - T0) - t2_old(k)) / (0.25 * hi_old(k))
                    t2(i) =  (Tf - T0) - dT2 * (0.25 * hi(i))
                    if ( t2(i) > 0.0 )  t2(i) = t2_old(k)

                    dT1   = (t2_old(k) - t1_old(k)) / (0.5 * hi_old(k))
                    t1(i) = t2(i) - dT1 * (0.5 * hi(i))
                    if ( t1(i) > 0.0 )  t1(i) = t1_old(k)
                    ts(i) = ts_old(k)
                else
                    ai = ai_old
                    hi = hi_old
                    hs = hs_old
                    t1 = t1_old
                    t2 = t2_old
                    ts = ts_old
                    return
                end if
            end if
        end if
    end do

end subroutine remap_ice
!---------------------------------------------------------------------------------------
! target ice in which category
subroutine cat_ice(sit, n)
    implicit none
    real,     intent(in)   ::  sit
    integer,  intent(out)  ::  n
    real, dimension(5), parameter :: hdef = (/0.1, 0.3, 0.7, 1.1, 100.0/)
    integer :: i

    do i = 1, 5
        if (sit <= hdef(i)) then
            n = i
            return
        end if
    end do

end subroutine  cat_ice
  
!------------------------------------------------------
!purpose: compute the time length
  subroutine datelength(sdat,edat,interval,nt)

     integer,intent(in)     :: sdat,edat,interval
     integer,intent(out)    :: nt
     integer                :: y1,y2,m1,m2,d1,d2,i

     if(sdat>edat)call mpp_error(FATAL,'==>Error in ocean_model_mod: Wrong sdate or edate! sdate must <= edate') 
     y1=int(sdat/10000.)
     y2=int(edat/10000.)
     m1=mod(int(sdat/100.),100)
     m2=mod(int(edat/100.),100)
     d1=mod(sdat,100)
     d2=mod(edat,100)
     nt=0
     if(y1<y2)then
       nt=(y2-y1-1)*365+1-d1-(daynum(m2)-d2)
       do i=m1,12
         nt=nt+daynum(i)
       enddo
       do i=1,m2
         nt=nt+daynum(i)
       enddo
     elseif(y1==y2)then
       nt=1-d1-(daynum(m2)-d2)
       do i=m1,m2
         nt=nt+daynum(i)
       enddo
     endif
     nt=CEILING(real(nt)/real(interval))
     if(nt<=0)call mpp_error(FATAL,'==>Error in ocean_model_mod: Wrong day length')

  end subroutine datelength
  !------------------------------------------------------
  !purpose:: compute the objective date
  subroutine objdate(dat,nd)

     integer,intent(inout)   :: dat
     integer,intent(in)      :: nd
     integer                 :: y1,m1,d1,i

     if(nd<1)call mpp_error(FATAL,'==>Error in ocean_model_mod: Wrong day increment')
     do i=1,nd
       dat=dat+1
       y1=int(dat/10000.)
       m1=mod(int(dat/100.),100)
       d1=mod(dat,100)
       if(d1>daynum(m1))then
         m1=m1+1
         d1=1
       endif
       if(m1>12)then
         y1=y1+1
         m1=1
       endif
       dat=y1*10000+m1*100+d1
     enddo

  end subroutine objdate
 !#######################################################################
 !================================= liuxw 20200616 ==============================


!=======================================================================
!---- assimil module by scnie----------------
!=======================================================================
!
! This program will execute assimilation process or not at the initial
!   stage. If assimil is true, then assimilation is executed.

      subroutine init_assimil(Ice)

      use time_manager_mod, only: operator(-), get_date, set_date

      type (ice_data_type), intent(inout) :: Ice
      integer :: date_model, day_init
      integer :: second, minute, hour, day, month, year
      type(time_type) :: Time_tmp

      nx_block = iec - isc + 1
      ny_block = jec - jsc + 1
      my_task = mpp_pe()
      master_task0 = mpp_root_pe()
      nu_diag = stdout()

      if (trim(assimil_dataset) == 'nsidc-0051') then
         write(assimil_file,'(a)') trim(assimil_dir)//'/'//'nt_2000_v1.1_n.nc'
      elseif (trim(assimil_dataset) == 'oisst') then
         write(assimil_file,'(a)') trim(assimil_dir)//'/'//'icec.day.mean.2000.v2.nc'
      end if

      call get_date(Ice%Time_Init, year, month, day, hour, minute, second)
      ayear = year ! current assimilation year
      date_model = year*10000+month*100+day

      Time_tmp = set_date(year, month, day)-set_date(year, 1, 1)
      call get_time(Time_tmp, second, day)
      day_init = day

      if (assimil) then

         call file_year(assimil_file, ayear)

         if (date_model < adate_end .and. date_model >= adate_start ) then

            call read_assimil_obs( day_init + 1 )
            call assimil_get_increment(Ice)
            call assimil_update_state(Ice)

         endif

         if (my_task == master_task0) then
            write(nu_diag, *) ' init_assimil end'
         endif

      endif

      end subroutine init_assimil

!=======================================================================
!
! Determine the current and final year of the assimilation cycle based on
!   namelist input; initialize the ice assimilation data filenames.

      subroutine init_cycle_assimil(Ice)

      use time_manager_mod, only: get_date

      type (ice_data_type), intent(in) :: Ice

      integer :: second, minute, hour, day, month, year

      call get_date(Ice%Time, year, month, day, hour, minute, second)
      ayear = year ! current assimilation year

      if (assimil) then

         if (trim(assimil_dataset) /= 'default' .and. &
                             my_task == master_task0) then
            write (nu_diag,*) ' Initial assimilation date = ', adate_start
            write (nu_diag,*) ' Final assimilation date = ', adate_end
         endif

      endif

      end subroutine init_cycle_assimil

!=======================================================================
!
! Get ice assimilation data and execute cycle assimilation

      subroutine get_cycle_assimil(Ice)

      use time_manager_mod, only: operator(-), get_date, set_date

      type (ice_data_type), intent(inout) :: Ice

      integer :: second, minute, hour, day, month, year
      integer :: ndy, dy_init, nsec, date_model, day_tmp
      type(time_type) :: Time_tmp

      call get_date(Ice%Time, year, month, day, hour, minute, second)
      ayear = year ! current assimilation year
      date_model = year*10000+month*100+day

      Time_tmp = set_date(year, month, day)-set_date(year, 1, 1)
      call get_time(Time_tmp, second, day)
      day_tmp = day

      call get_time(Ice%Time, second, day)
      ndy = day
      nsec = second

      call get_time(Ice%Time_Init, second, day)
      dy_init = day

      if (assimil) then

         call file_year(assimil_file, ayear)

         if (date_model < adate_end .and. date_model >= adate_start ) then

            if (mod(ndy-dy_init, afreq) == 0 .and. nsec == 0) then  ! Conduct assimilation every n steps
               call read_assimil_obs( day_tmp + 1 )
               call assimil_get_increment(Ice)
               call assimil_update_state(Ice)
            endif

         endif

      endif

      end subroutine get_cycle_assimil

!=======================================================================
!
! Read the observational data used for assimilation

      subroutine read_assimil_obs(recnum)

      integer, optional, intent(in) :: &
         recnum

      integer :: &
         rec_num

      if (present(recnum)) then
         rec_num = recnum
      else
         rec_num = 1
      endif

      if (trim(assimil_dataset) == 'nsidc-0051') then

         allocate(aice_obs_g(304*448+316*332))
         allocate(mask_obs_g(304*448+316*332))
         allocate(lon_obs_g(304*448+316*332))
         allocate(lat_obs_g(304*448+316*332))

         aice_obs_g(:) = 0.0
         mask_obs_g(:) = 0.0
         lon_obs_g(:) = 0.0
         lat_obs_g(:) = 0.0


         if (trim(assimil_dataform) == 'nc') then
            call read_obs_nsidc_nc(aice_obs_g, mask_obs_g, lon_obs_g, lat_obs_g, rec_num)
         else
            if (my_task == master_task0) then
               call error_mesg ( 'read_assimil_obs', &
                  'assmil_dataform is not supported: ' &
                  //trim(assimil_dataform), FATAL )
            endif
         endif

      elseif (trim(assimil_dataset) == 'oisst') then

         allocate(aice_obs_g(1440*720))
         allocate(mask_obs_g(1440*720))
         allocate(lon_obs_g(1440*720))
         allocate(lat_obs_g(1440*720))

         aice_obs_g(:) = 0.0
         mask_obs_g(:) = 0.0
         lon_obs_g(:) = 0.0
         lat_obs_g(:) = 0.0

         if (trim(assimil_dataform) == 'nc') then
            call read_obs_oisst_nc(aice_obs_g, mask_obs_g, lon_obs_g, lat_obs_g, rec_num)
         else
            if (my_task == master_task0) then
               call error_mesg ( 'read_assimil_obs', &
                  'assmil_dataform is not supported: ' &
                  //trim(assimil_dataform), FATAL )
            endif
         endif 

      else
         allocate(aice_obs_g(1))
         allocate(mask_obs_g(1))
         allocate(lon_obs_g(1))
         allocate(lat_obs_g(1))
         if (my_task == master_task0) then
           call error_mesg ( 'read_assimil_obs', &
              'assmil_dataset is not supported: ' &
              //trim(assimil_dataset), FATAL )
         endif
      endif

      end subroutine read_assimil_obs

!=======================================================================
!
! Calculate increments of ice state variables

      subroutine assimil_get_increment(Ice)

      use ice_grid_mod, only: geo_lon, geo_lat

      type (ice_data_type), intent(in) :: Ice

      integer ::   &
         i,         &
         nxo,       &
         nyo

      integer ::   &
         nxyo

      real, dimension(nx_block,ny_block) ::   &
         aice_bkg,     &
         mask_bkg,     &
         lon_bkg,      &
         lat_bkg

      real, dimension(nx_block*ny_block) ::   &
         aice_bkg_1d,     &
         mask_bkg_1d,     &
         lon_bkg_1d,      &
         lat_bkg_1d

      real, dimension(:, :), allocatable ::   &
         aice_obs_2d,     &
         mask_obs_2d,     &
         lon_obs_2d,      &
         lat_obs_2d

      allocate(aice_obs(nx_block*ny_block))
      allocate(aice_inc(nx_block,ny_block))
      allocate(tmask_dbl(nx_block,ny_block))

      aice_obs = 0.0
      aice_inc(:,:) = 0.0

      nxyo = nx_block*ny_block

      where (Ice%mask(isc:iec,jsc:jec))
         tmask_dbl = c1
      elsewhere
         tmask_dbl = 0.0
      endwhere

      lon_bkg = geo_lon
      lat_bkg = geo_lat
      mask_bkg = tmask_dbl
      aice_bkg = sum(Ice%part_size(isc:iec,jsc:jec,2:km), 3)

      where (lon_bkg <= 0.0) lon_bkg = lon_bkg + c360
      where (lon_bkg > c360) lon_bkg = lon_bkg - c360

      lon_bkg_1d = reshape(lon_bkg, (/nxyo/))
      lat_bkg_1d = reshape(lat_bkg, (/nxyo/))
      mask_bkg_1d = reshape(mask_bkg, (/nxyo/))
      aice_bkg_1d = reshape(aice_bkg, (/nxyo/))


      if (trim(assimil_dataset) == 'nsidc-0051' .and.   &
         trim(assimil_dataform) == 'nc') then

         nxo = 304
         nyo = 448
         allocate(aice_obs_2d(nxo,nyo))
         allocate(mask_obs_2d(nxo,nyo))
         allocate(lon_obs_2d(nxo,nyo))
         allocate(lat_obs_2d(nxo,nyo))
         aice_obs_2d = reshape(aice_obs_g(1:304*448), (/nxo, nyo/))
         mask_obs_2d = reshape(mask_obs_g(1:304*448), (/nxo, nyo/))
         lon_obs_2d = reshape(lon_obs_g(1:304*448), (/nxo, nyo/))
         lat_obs_2d = reshape(lat_obs_g(1:304*448), (/nxo, nyo/))
         call obs_interpto_model(lon_bkg_1d,  lat_bkg_1d,    &
                                 mask_bkg_1d, aice_obs,      &
                                 nxo,         nyo,           &
                                 lon_obs_2d,  lat_obs_2d,    &
                                 mask_obs_2d, aice_obs_2d)
         deallocate(aice_obs_2d)
         deallocate(mask_obs_2d)
         deallocate(lon_obs_2d)
         deallocate(lat_obs_2d)

         nxo = 316
         nyo = 332
         allocate(aice_obs_2d(nxo,nyo))
         allocate(mask_obs_2d(nxo,nyo))
         allocate(lon_obs_2d(nxo,nyo))
         allocate(lat_obs_2d(nxo,nyo))
         aice_obs_2d = reshape(aice_obs_g(304*448+1:304*448+316*332), (/nxo, nyo/))
         mask_obs_2d = reshape(mask_obs_g(304*448+1:304*448+316*332), (/nxo, nyo/))
         lon_obs_2d = reshape(lon_obs_g(304*448+1:304*448+316*332), (/nxo, nyo/))
         lat_obs_2d = reshape(lat_obs_g(304*448+1:304*448+316*332), (/nxo, nyo/))
         call obs_interpto_model(lon_bkg_1d,  lat_bkg_1d,    &
                                 mask_bkg_1d, aice_obs,      &
                                 nxo,         nyo,           &
                                 lon_obs_2d,  lat_obs_2d,    &
                                 mask_obs_2d, aice_obs_2d)
         deallocate(aice_obs_2d)
         deallocate(mask_obs_2d)
         deallocate(lon_obs_2d)
         deallocate(lat_obs_2d)

         call model_assimilto_obs(nx_block*ny_block, lon_bkg(:,:),     &
                                  lat_bkg(:,:),      mask_bkg(:,:),    &
                                  aice_bkg(:,:),     lon_bkg_1d(:),    &
                                  lat_bkg_1d(:),     mask_bkg_1d(:),   &
                                  aice_obs(:),       aice_bkg_1d(:),   &
                                  aice_inc)

      elseif (trim(assimil_dataset) == 'oisst' .and.   &
         trim(assimil_dataform) == 'nc') then

         nxo = 1440
         nyo = 720
         allocate(aice_obs_2d(nxo,nyo))
         allocate(mask_obs_2d(nxo,nyo))
         allocate(lon_obs_2d(nxo,nyo))
         allocate(lat_obs_2d(nxo,nyo))
         aice_obs_2d = reshape(aice_obs_g, (/nxo, nyo/))
         mask_obs_2d = reshape(mask_obs_g, (/nxo, nyo/))
         lon_obs_2d = reshape(lon_obs_g, (/nxo, nyo/))
         lat_obs_2d = reshape(lat_obs_g, (/nxo, nyo/))
         call obs_interpto_model(lon_bkg_1d,  lat_bkg_1d,    &
                                 mask_bkg_1d, aice_obs,      &
                                 nxo,         nyo,           &
                                 lon_obs_2d,  lat_obs_2d,    &
                                 mask_obs_2d, aice_obs_2d)
         deallocate(aice_obs_2d)
         deallocate(mask_obs_2d)
         deallocate(lon_obs_2d)
         deallocate(lat_obs_2d)

         call model_assimilto_obs(nx_block*ny_block, lon_bkg(:,:),     &
                                  lat_bkg(:,:),      mask_bkg(:,:),    &
                                  aice_bkg(:,:),     lon_bkg_1d(:),    &
                                  lat_bkg_1d(:),     mask_bkg_1d(:),   &
                                  aice_obs(:),       aice_bkg_1d(:),   &
                                  aice_inc)

      endif

      deallocate(aice_obs_g)
      deallocate(mask_obs_g)
      deallocate(lon_obs_g)
      deallocate(lat_obs_g)

      deallocate(aice_obs)
      deallocate(tmask_dbl)

      end subroutine assimil_get_increment

!=======================================================================
!
! Update ice state variables with the increments calculated in assimilation

      subroutine assimil_update_state(Ice)

      use mpp_domains_mod, only: mpp_update_domains

      type (ice_data_type), intent(inout) :: Ice

      integer :: &
         i, j, ij, i2, j2, & ! horizontal indices
         ilo,ihi,jlo,jhi, & ! beginning and end of physical domain
         n, k               ! thickness category index

      integer :: &
         nelevate        ! number of elevations of increments to higher 
                         ! category (diagnostic)

      integer :: &
         icells          ! number of ocean cells

      integer, dimension(nx_block*ny_block) :: &
         indxi, indxj    ! indirect indices for cells with aicen > puny

      real :: & 
         hi_da_new    , & ! specified ice thickness for new ice created by DA
         hicen        , & ! ice thickness
         hsnon        , & ! snow thickness
         daice            ! change in ice concentration (for first category)

      real, dimension(nx_block,ny_block) :: &
         aice

      !------------------------------------------------------------------
      ! Set thickness for new ice
      ! (Currently using value of 0.5m, which was value thin ice was 
      !  incremented toward in LIM ice model).
      !-----------------------------------------------------------------

      hi_da_new =  0.50! if ncat>1, this has to be less than
                                    ! the 1st category thickness limit

      aice(:,:) = sum(Ice%part_size(isc:iec,jsc:jec,2:km), 3)
      !----------------------------------------------------------------
      ! Update category state variables
      !----------------------------------------------------------------

      ilo = isc
      ihi = iec
      jlo = jsc
      jhi = jec

      !----------------------------------------------------------------
      ! Find ocean points where data assimilation abs(increment) > puny
      ! (Note, aice_inc is the increments of ice concentration due
      !  to data assimilation)
      !----------------------------------------------------------------

      icells = 0
      do j = jlo, jhi
      do i = ilo, ihi

         i2 = i - isc + 1
         j2 = j - jsc + 1

         if (Ice%mask(i,j) .and. abs(aice_inc(i2,j2)) > puny) then
            icells = icells + 1
            indxi(icells) = i
            indxj(icells) = j
         endif

      enddo               ! i
      enddo               ! j

      if (icells > 0) then
         
         n = 2                ! only ever add increment to 1st category
           do ij = 1, icells
              i = indxi(ij)
              j = indxj(ij)

              i2 = i - isc + 1
              j2 = j - jsc + 1

              !---------------------------------------------------
              ! Apply concentration increment and associated
              ! volume change
              !---------------------------------------------------

              if (Ice%part_size(i,j,n) > puny) then
                 ! if decreasing concentration, subtract ice volume at 
                 ! current thickness
                 hicen             = Ice%h_ice(i,j,n)

                 ! if increasing concentration, add ice volume at hi_da_new 
                 ! thickness
                 if ( aice_inc(i2,j2) > puny) hicen = hi_da_new

                 ! whether in/decreasing concentration, add/subtract snow 
                 ! volume at current thickness

                 daice             = & 
                   min( ( c1 - aice(i2,j2) ), ( aice_inc(i2,j2) ) )
                 Ice%part_size(i,j,n) = Ice%part_size(i,j,n) + daice  
                 Ice%h_ice(i,j,n) = Ice%h_ice(i,j,n) + &
                   (hicen - Ice%h_ice(i,j,n))*daice/Ice%part_size(i,j,n)

              !---------------------------------------------------
              ! Create new ice points with specified thickness
              !---------------------------------------------------

              else 
                 Ice%part_size(i,j,n) = & 
                   min( ( c1 - aice(i2,j2) ), ( aice_inc(i2,j2) ) )
                 ! note aicen/vicen < 0.0 taken care below
                 Ice%h_ice(i,j,n) = Ice%part_size(i,j,n) * hi_da_new
                 Ice%h_snow(i,j,n) = 0.0

              endif
              Ice%t_ice1(i,j,n) = -5.0
              Ice%t_ice2(i,j,n) = -5.0

           enddo ! ij

         do n = 2,km
           nelevate=0
           do ij = 1,icells
              i = indxi(ij)
              j = indxj(ij)
            
              i2 = i - isc + 1
              j2 = j - jsc + 1

              !----------------------------------------------------
              ! Check is aicen < puny
              !  - remove from next category if necessary
              !  - otherwise just remove it
              ! Ignoring conservation issues here
              !----------------------------------------------------

              if (Ice%part_size(i,j,n) < puny) then
                 if ( n < km ) then
                  if (Ice%part_size(i,j,n) < -1.0*puny ) then
                   nelevate=nelevate+1
                  endif
                   ! take concentration from next category -- constant thickness
                   if ( Ice%part_size(i,j,n+1) > puny ) then 
                     hicen = Ice%h_ice(i,j,n+1)
                     hsnon = Ice%h_snow(i,j,n+1)
                   else
                     hicen = 0.0
                     hsnon = 0.0
                   endif  ! aicen(n+1) > puny
                   Ice%part_size(i,j,n+1) = Ice%part_size(i,j,n+1) + Ice%part_size(i,j,n)
                   Ice%h_ice(i,j,n+1) = hicen
                   Ice%h_snow(i,j,n+1) = hsnon
                 endif ! n < km
                 Ice%part_size(i,j,n) = 0.0
                 Ice%h_ice(i,j,n) = 0.0
                 Ice%h_snow(i,j,n) = 0.0
              endif ! aicen(n) < puny
              Ice%t_ice1(i,j,n) = -5.0
              Ice%t_ice2(i,j,n) = -5.0

           enddo ! ij
!           write(nu_diag,*) 'Elevated ', nelevate, ' incs to category ', n+1
         enddo   ! n
      endif      ! icells 

      Ice%part_size(:,:,1) = 1.0
      do k=2,km
         Ice%part_size (:,:,1) = Ice%part_size(:,:,1)-Ice%part_size (:,:,k)
      end do

      deallocate(aice_inc)

      end subroutine assimil_update_state

!=======================================================================
!
! Read nsidc-0051 ice concentration data

      subroutine read_obs_nsidc_nc(work, mask, lon, lat, recnum)

      use netcdf

      integer :: &
         ncid,            & ! netcdf file id for nsidc data
         varid,           & ! variable id
         iostatus,        & ! status output from netcdf routines
         i

      integer, parameter :: &
         nnx = 304,       &
         nny = 448,       &
         snx = 316,       &
         sny = 332

      character (len=128) :: &
         data_file

      real, dimension(nnx*nny+snx*sny), intent(out) :: &
         work,            &
         mask,            &
         lon,             &
         lat

      real, dimension(:,:), allocatable :: &
         work_g,          &
         lon_g,           &
         lat_g

      integer, intent(in) :: &
         recnum

      if (trim(assimil_type) == 'initial') &
         write(nu_diag, *) 'read_obs_nsidc_nc:'

      ! Read data in northern hemisphere
      i = index(assimil_file,'.nc')
      write(data_file, '(a)') assimil_file(1:i-2)//'n.nc'
      write(nu_diag, '(a, a, i5)') trim(data_file), ': recnum = ', recnum

      allocate(work_g(nnx, nny))
      allocate(lon_g(nnx, nny))
      allocate(lat_g(nnx, nny))

      iostatus = nf90_open(data_file, NF90_NOWRITE, ncid)
      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_nsidc_nc', &
            'Cannot open '//trim(data_file), FATAL )
      endif

      !-------------------------------------------------------------
      ! Read longitude and latitude
      !-------------------------------------------------------------

      iostatus = nf90_inq_varid(ncid, 'lon', varid)

      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_nsidc_nc', &
            'Cannot find variable lon', FATAL )
      endif

      iostatus = nf90_get_var(ncid, varid, lon_g, &
         start=(/1, 1/), &
         count=(/nnx, nny/) )

      iostatus = nf90_inq_varid(ncid, 'lat', varid)

      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_nsidc_nc', &
            'Cannot find variable lat', FATAL )
      endif

      iostatus = nf90_get_var(ncid, varid, lat_g, &
         start=(/1, 1/), &
         count=(/nnx, nny/) )

      !-------------------------------------------------------------
      ! Read ice concentration
      !-------------------------------------------------------------

      iostatus = nf90_inq_varid(ncid, 'sic', varid)

      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_nsidc_nc', & 
            'Cannot find variable sic', FATAL )
      endif

      iostatus = nf90_get_var(ncid, varid, work_g, &
         start=(/1, 1, recnum/), &
         count=(/nnx, nny, 1/) )

      !-------------------------------------------------------------

      if (trim(assimil_type) == 'initial') &
         write(nu_diag, *) ' north: min and max = ', minval(work_g), &
             maxval(work_g, mask=work_g/=1.0e30_4)

      do i = 1, nny
         work((i-1)*nnx+1:i*nnx) = work_g(:, i)
         lon((i-1)*nnx+1:i*nnx) = lon_g(:, i)
         lat((i-1)*nnx+1:i*nnx) = lat_g(:, i)
      enddo

      deallocate(work_g)
      deallocate(lon_g)
      deallocate(lat_g)

      iostatus = nf90_close(ncid)

      ! Read data in southern hemisphere
      i = index(assimil_file,'.nc')
      write(data_file, '(a)') assimil_file(1:i-2)//'s.nc'
      write(nu_diag, '(a, a, i5)') trim(data_file), ': recnum = ', recnum

      allocate(work_g(snx, sny))
      allocate(lon_g(snx, sny))
      allocate(lat_g(snx, sny))

      iostatus = nf90_open(data_file, NF90_NOWRITE, ncid)
      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_nsidc_nc', &
            'Cannot open '//trim(data_file), FATAL )
      endif

      !-------------------------------------------------------------
      ! Read longitude and latitude
      !-------------------------------------------------------------

      iostatus = nf90_inq_varid(ncid, 'lon', varid)

      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_nsidc_nc', &
            'Cannot find variable lon', FATAL )
      endif

      iostatus = nf90_get_var(ncid, varid, lon_g, &
         start=(/1, 1/), &
         count=(/snx, sny/) )

      iostatus = nf90_inq_varid(ncid, 'lat', varid)

      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_nsidc_nc', &
            'Cannot find variable lat', FATAL )
      endif

      iostatus = nf90_get_var(ncid, varid, lat_g, &
         start=(/1, 1/), &
         count=(/snx, sny/) )

      !-------------------------------------------------------------
      ! Read ice concentration
      !-------------------------------------------------------------

      iostatus = nf90_inq_varid(ncid, 'sic', varid)

      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_nsidc_nc', & 
            'Cannot find variable sic', FATAL )
      endif

      iostatus = nf90_get_var(ncid, varid, work_g, &
         start=(/1, 1, recnum/), &
         count=(/snx, sny, 1/) )

      !-------------------------------------------------------------

      if (trim(assimil_type) == 'initial') &
         write(nu_diag, *) ' south: min and max = ', minval(work_g), &
             maxval(work_g, mask=work_g/=1.0e30_4)

      do i = 1, sny
         work((i-1)*snx+nnx*nny+1:i*snx+nnx*nny) = work_g(:, i)
         lon((i-1)*snx+nnx*nny+1:i*snx+nnx*nny) = lon_g(:, i)
         lat((i-1)*snx+nnx*nny+1:i*snx+nnx*nny) = lat_g(:, i)
      enddo

      deallocate(work_g)
      deallocate(lon_g)
      deallocate(lat_g)

      iostatus = nf90_close(ncid)

      where (lon <= 0.0)
         lon = lon + c360
      endwhere

      where((work < 1.0e30) .and. (work > -p5))
         mask = c1
      elsewhere
         mask = 0.0
      endwhere

      end subroutine read_obs_nsidc_nc

!=======================================================================
!
! Read OISST ice concentration data

      subroutine read_obs_oisst_nc(work, mask, lon, lat, recnum)

      use netcdf

      integer :: &
         ncid,            & ! netcdf file id for oisst data
         varid,           & ! variable id
         iostatus,        & ! status output from netcdf routines
         i

      integer, parameter :: &
         gnx = 1440,       &
         gny = 720

      character (len=128) :: &
         data_file

      real, dimension(gnx*gny), intent(out) :: &
         work,            &
         mask,            &
         lon,             &
         lat

      real, dimension(:,:), allocatable :: &
         work_g,          &
         lon_g,           &
         lat_g

      integer, intent(in) :: &
         recnum

      if (trim(assimil_type) == 'initial') &
         write(nu_diag, *) 'read_obs_oisst_nc:'

      ! Read data in global
      write(data_file, '(a)') assimil_file
      write(nu_diag, '(a, a, i5)') trim(data_file), ': recnum = ', recnum

      allocate(work_g(gnx, gny))
      allocate(lon_g(gnx, gny))
      allocate(lat_g(gnx, gny))

      iostatus = nf90_open(data_file, NF90_NOWRITE, ncid)
      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_oisst_nc', &
            'Cannot open '//trim(data_file), FATAL )
      endif

      !-------------------------------------------------------------
      ! Read longitude and latitude
      !-------------------------------------------------------------

      iostatus = nf90_inq_varid(ncid, 'lon', varid)

      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_oisst_nc', &
            'Cannot find variable lon', FATAL )
      endif

      iostatus = nf90_get_var(ncid, varid, lon_g(:,1), &
         start=(/1/), &
         count=(/gnx/) )

      do i = 2, gny
         lon_g(:,i) = lon_g(:,1)
      enddo

      iostatus = nf90_inq_varid(ncid, 'lat', varid)

      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_oisst_nc', &
            'Cannot find variable lat', FATAL )
      endif

      iostatus = nf90_get_var(ncid, varid, lat_g(1,:), &
         start=(/1/), &
         count=(/gny/) )

      do i = 2, gnx
         lat_g(i,:) = lat_g(1,:)
      enddo

      !-------------------------------------------------------------
      ! Read ice concentration
      !-------------------------------------------------------------

      iostatus = nf90_inq_varid(ncid, 'icec', varid)

      if (iostatus /= nf90_noerr) then
        call error_mesg ( 'read_obs_oisst_nc', & 
            'Cannot find variable icec', FATAL )
      endif

      iostatus = nf90_get_var(ncid, varid, work_g, &
         start=(/1, 1, recnum/), &
         count=(/gnx, gny, 1/) )

      !-------------------------------------------------------------

      if (trim(assimil_type) == 'initial') &
         write(nu_diag, *) ' north: min and max = ', minval(work_g), &
             maxval(work_g, mask=work_g/=-9.96921e36_4)

      do i = 1, gny
         work((i-1)*gnx+1:i*gnx) = work_g(:, i)
         lon((i-1)*gnx+1:i*gnx) = lon_g(:, i)
         lat((i-1)*gnx+1:i*gnx) = lat_g(:, i)
      enddo

      deallocate(work_g)
      deallocate(lon_g)
      deallocate(lat_g)

      iostatus = nf90_close(ncid)

      where (lon <= 0.0)
         lon = lon + c360
      endwhere

      where((work < -p5))
         work = 0.0
      endwhere
      mask = c1

      end subroutine read_obs_oisst_nc

!=======================================================================

      subroutine model_interpto_obs(nxyo, xo, yo, mo, worko,   &
                                    xi, yi, mi, worki)

      use netcdf
      use ice_assimil_oi, only: drcm2points

      integer, intent(in) ::   &
         nxyo

      integer, parameter ::   &
         ngrd = 1

      integer ::   &
         nxi, &
         nyi

      integer ::   &
         nxyo2

      integer ::   &
         i,         &
         ncid,      &
         varid,     &
         iostatus,  &
         xdimid, ydimid

      integer ::    &
         opt,       &
         ncrit,     &
         kval,      &
         ier

      real :: &
         xmsg

      real, dimension(:, :), intent(in) ::   &
         xi,        &
         yi,        &
         mi,        &
         worki

      real, dimension(:, :, :), allocatable ::   &
         fi

      real, dimension(nxyo), intent(in) ::   &
         xo,      &
         yo,      &
         mo

      real, dimension(nxyo), intent(inout) ::   &
         worko

      real, dimension(nxyo) ::   &
         xo2,     &
         yo2

      real, dimension(nxyo,ngrd) ::   &
         fo,      &
         fo2

      integer, dimension(nxyo) ::   &
         xyo2i

      character (len=128) ::    &
         filename

      integer ::   &
         count_int,      &
         count_rate_int, &
         count_max_int

      real ::   &
         stime,      &
         etime

      nxi = im
      nyi = jm

      allocate(fi(nxi, nyi, ngrd))

      call system_clock(count_int, count_rate_int, count_max_int)
      stime = c1*count_int/count_rate_int

      if (trim(assimil_type) == 'initial') then
         if (my_task == master_task0) &
            write(nu_diag, *) 'model_interpto_obs:'
      endif

      xmsg = spval_dbl
      opt = 2
      ncrit = 3
      kval = 1

      fi(:,:,1) = worki
      fo(:,1) = worko(:)

      nxyo2 = 0
      do i = 1, nxyo
         if (mo(i) > p5) then
            nxyo2 = nxyo2 + 1
            xo2(nxyo2) = xo(i)
            yo2(nxyo2) = yo(i)
            xyo2i(nxyo2) = i
         endif
      enddo

      where (mi < p5)
         fi(:,:,1) = spval_dbl
      endwhere
      where (mo < p5)
         fo(:,1) = spval_dbl
      endwhere

      call drcm2points(ngrd, nyi, nxi, yi, xi, fi, nxyo2,   &
                       yo2(1:nxyo2), xo2(1:nxyo2), fo2(1:nxyo2,1),   &
                       xmsg, opt, ncrit, kval, ier)

      do i = 1, nxyo2
         if (fo2(i,1) < spval_dbl) &
            fo(xyo2i(i),1) = fo2(i,1)
      enddo

      if (trim(assimil_type) == 'initial') then
         if (my_task == master_task0) then
            write(nu_diag, *) ' fi: min and max = ', minval(fi), &
                maxval(fi, mask=fi/=spval_dbl)
            write(nu_diag, *) ' fo: min and max = ', minval(fo), &
                maxval(fo, mask=fo/=spval_dbl)
            write(nu_diag, *) ' mi: count = ', sum(mi)
            write(nu_diag, *) ' mo: count = ', sum(mo)
         endif
      endif

      worko(:) = fo(:,1)

      call system_clock(count_int, count_rate_int, count_max_int)
      etime = c1*count_int/count_rate_int

      if (trim(assimil_type) == 'initial') then
         if (my_task == master_task0) &
            write(nu_diag, *) ' Time used (s): ', etime - stime
      endif

      end subroutine model_interpto_obs

!=======================================================================

      subroutine obs_interpto_model(xo, yo, mo, worko,   &
                                    nxi, nyi, xi, yi, mi, worki)

      use netcdf
      use ice_assimil_oi, only: drcm2points

      integer ::   &
         nxyo

      integer, intent(in) ::   &
         nxi            ,   &
         nyi

      integer, parameter ::   &
         ngrd = 1

      integer ::   &
         nxyo2

      integer ::   &
         i,         &
         ncid,      &
         varid,     &
         iostatus,  &
         xdimid, ydimid

      integer ::    &
         opt,       &
         ncrit,     &
         kval,      &
         ier

      real :: &
         xmsg

      real, dimension(nxi, nyi), intent(in) ::   &
         xi,        &
         yi,        &
         mi,        &
         worki

      real, dimension(nxi, nyi, ngrd) ::   &
         fi

      real, dimension(:), intent(in) ::   &
         xo,      &
         yo,      &
         mo

      real, dimension(:), intent(inout) ::   &
         worko

      real, dimension(:), allocatable ::   &
         xo2,     &
         yo2

      real, dimension(:,:), allocatable ::   &
         fo,      &
         fo2

      integer, dimension(:), allocatable ::   &
         xyo2i

      character (len=128) ::    &
         filename

      integer ::   &
         count_int,      &
         count_rate_int, &
         count_max_int

      real ::   &
         stime,      &
         etime

      nxyo = nx_block*ny_block

      allocate(xo2(nxyo))
      allocate(yo2(nxyo))
      allocate(fo(nxyo,ngrd))
      allocate(fo2(nxyo,ngrd))
      allocate(xyo2i(nxyo))

      call system_clock(count_int, count_rate_int, count_max_int)
      stime = c1*count_int/count_rate_int

      if (trim(assimil_type) == 'initial') then
         if (my_task == master_task0) &
            write(nu_diag, *) 'obs_interpto_model:'
      endif

      xmsg = spval_dbl
      opt = 2
      ncrit = 3
      kval = 1

      fi(:,:,1) = worki
      fo(:,1) = worko(:)

      nxyo2 = 0
      do i = 1, nxyo
         if (mo(i) > p5) then
            nxyo2 = nxyo2 + 1
            xo2(nxyo2) = xo(i)
            yo2(nxyo2) = yo(i)
            xyo2i(nxyo2) = i
         endif
      enddo

      where (mi < p5)
         fi(:,:,1) = spval_dbl
      endwhere
      where (mo < p5)
         fo(:,1) = spval_dbl
      endwhere

      call drcm2points(ngrd, nyi, nxi, yi, xi, fi, nxyo2,   &
                       yo2(1:nxyo2), xo2(1:nxyo2), fo2(1:nxyo2,1),   &
                       xmsg, opt, ncrit, kval, ier)

      do i = 1, nxyo2
         if (fo2(i,1) < spval_dbl) &
            fo(xyo2i(i),1) = fo2(i,1)
      enddo

      if (trim(assimil_type) == 'initial') then
         if (my_task == master_task0) then
            write(nu_diag, *) ' fi: min and max = ', minval(fi), &
                maxval(fi, mask=fi/=spval_dbl)
            write(nu_diag, *) ' fo: min and max = ', minval(fo), &
                maxval(fo, mask=fo/=spval_dbl)
            write(nu_diag, *) ' mi: count = ', sum(mi)
            write(nu_diag, *) ' mo: count = ', sum(mo)
         endif
      endif

      worko(:) = fo(:,1)

      call system_clock(count_int, count_rate_int, count_max_int)
      etime = c1*count_int/count_rate_int

      if (trim(assimil_type) == 'initial') then
         if (my_task == master_task0) &
            write(nu_diag, *) ' Time used (s): ', etime - stime
      endif

      end subroutine obs_interpto_model

!=======================================================================

      subroutine model_assimilto_obs(nxyo, xb, yb, mb, fb,   &
                                     xo, yo, mo, fo, fbi, work)

      use netcdf
      use ice_assimil_oi, only: optiminterp

      integer, intent(in) ::   &
         nxyo

      integer, parameter ::   &
         m = 30

      integer ::   &
         nxb, &
         nyb

      integer ::    &
         i,         &
         ncid,      &
         varid,     &
         iostatus,  &
         xdimid, ydimid

      character (len=128) ::    &
         filename

      real, dimension(:, :), intent(in) ::   &
         xb,        &
         yb,        &
         mb

      real, dimension(:, :), intent(inout) ::   &
         fb

      real, dimension(:, :), allocatable, intent(out) ::   &
         work

      real, dimension(:, :), allocatable ::   &
         xyb

      real, dimension(nxyo), intent(in) ::   &
         xo,        &
         yo,        &
         mo

      real, dimension(nxyo), intent(inout) ::   &
         fo,       &
         fbi

      real, dimension(2, nxyo) ::   &
         xyo

      real, dimension(1, nxyo) ::   &
         df

      real, dimension(:, :), allocatable ::   &
         dfa

      real, dimension(nxyo) ::   &
         ovar

      real, dimension(:), allocatable ::   &
         avar

      real, dimension(2) ::   &
         param

      integer ::   &
         count_int,      &
         count_rate_int, &
         count_max_int

      real ::   &
         stime,      &
         etime

      nxb = nx_block
      nyb = ny_block

      allocate(work(nxb, nyb))
      allocate(xyb(2, nxb*nyb))
      allocate(dfa(1, nxb*nyb))
      allocate(avar(nxb*nyb))

      call system_clock(count_int, count_rate_int, count_max_int)
      stime = c1*count_int/count_rate_int

      if (trim(assimil_type) == 'initial') then
         if (my_task == master_task0) &
            write(nu_diag, *) 'model_assimilto_obs:'
      endif

      where (mb < p5)
         fb = 0.0
      endwhere

      where (mo < p5)
         fo = 0.0
      endwhere

      where (mo < p5)
         fbi = 0.0
      endwhere

      xyo(1,:) = xo
      xyo(2,:) = yo
      df(1,:) = fo - fbi
      ovar(:) = aobsvar
      param(:) = 1/10.0
      xyb(1,:) = reshape(xb, (/nxb*nyb/))
      xyb(2,:) = reshape(yb, (/nxb*nyb/))

      call optiminterp(xyo, df, ovar, param, m, xyb, dfa, avar)

      work = reshape(dfa(1,:), (/nxb,nyb/))

      if (trim(assimil_type) == 'initial') then
         if (my_task == master_task0) then
            write(nu_diag, *) ' xb: min/max: ', minval(xb), maxval(xb)
            write(nu_diag, *) ' yb: min/max: ', minval(yb), maxval(yb)
            write(nu_diag, *) ' xo: min/max: ', minval(xo), maxval(xo)
            write(nu_diag, *) ' yo: min/max: ', minval(yo), maxval(yo)
            write(nu_diag, *) ' fb: min/max: ', minval(fb), maxval(fb)
            write(nu_diag, *) ' fbi: min/max: ', minval(fbi), maxval(fbi)
            write(nu_diag, *) ' fo: min/max: ', minval(fo), maxval(fo)
            write(nu_diag, *) ' df: min/max: ', minval(df), maxval(df)
            write(nu_diag, *) ' dfa: min/max: ', minval(dfa), maxval(dfa)
            write(nu_diag, *) ' avar: min/max: ', minval(avar), maxval(avar)
         endif
      endif

      call system_clock(count_int, count_rate_int, count_max_int)
      etime = c1*count_int/count_rate_int

      if (trim(assimil_type) == 'initial') then
         if (my_task == master_task0) &
            write(nu_diag, *) ' Time used (s): ', etime - stime
      endif

      end subroutine model_assimilto_obs

!=======================================================================

      subroutine file_year (data_file, yr)

! Construct the correct name of the atmospheric data file
! to be read, given the year and assuming the naming convention
! that filenames end with 'yyyy.dat' or 'yyyy.r' or 'yyyy.nc'.

      character (len=128), intent(inout) ::  data_file

      integer, intent(in) :: yr

      character (len=128) :: tmpname

      integer :: i

      tmpname = data_file
      i = index(data_file,'.nc')
      if (i > 0) then
         if (trim(assimil_dataset) == 'nsidc-0051') then
            write(data_file,'(a,i4.4,a,a)') tmpname(1:i-12), yr, tmpname(i-7:i-1), '.nc'
         elseif (trim(assimil_dataset) == 'oisst') then
            write(data_file,'(a,i4.4,a,a)') tmpname(1:i-8), yr, tmpname(i-3:i-1), '.nc'
         else
            write(data_file,'(a,i4.4,a)') tmpname(1:i-5), yr, '.nc'
         end if
      else
         i = index(data_file,'.dat')
         write(data_file,'(a,i4.4,a)') tmpname(1:i-5), yr, '.dat'
      end if

      end subroutine file_year

  !---- end assimil module----------------

end module ice_type_mod
