 Subroutine atmos_ice_flux_cpl( Ice, Atmos, isc, iec, jsc, jec)

 use ice_model_mod, only : atmos_ice_boundary_type, ice_data_type
 use const_mod
 use ice_grid_mod,     only : km, sin_rot, cos_rot
 use fms_mod,          only : stdout, mpp_pe, mpp_root_pe
 !!use cpl_mod,          only : first_call
 use sat_vapor_pres_mod             

 type (atmos_ice_boundary_type), intent(inout)   :: Atmos
 type (ice_data_type),           intent(inout)   :: ice 
 integer,                        intent(in)      :: isc, iec, jsc, jec   

 !xjx-20090514 implicit none

 real(kind=8) :: zbot, ubot, vbot, thbot, qbot, rbot, tbot, ts
 real(kind=8) :: albedo, swndr, swvdr, swndf, swvdf, swnet   
 real(kind=8) :: dhdt, dedt, drdt 

 !--- local constants --------------------------------
 real(kind=8),parameter :: emissity = 1.0 ! emissivity of snow and ice
 real(kind=8),parameter :: umin  =  1.0    ! minimum wind speed (m/s)
 real(kind=8),parameter :: zref  = 10.0    ! reference height (m)
 real(kind=8),parameter :: ztref =  2.0    ! reference height for air T (m)
 real(kind=8),parameter :: wzsice = 0.0005 ! ice surface roughness 
 real(kind=8),parameter :: TTT = 5897.8    ! for qsat over ice
 real(kind=8),parameter :: QQQ = 11637800.0    ! for qsat over ice
 real(kind=8),parameter :: d622 = 0.622    ! rdgas / rvgas
 real(kind=8),parameter :: d378 = 0.378    ! 1 - rdgas / rvgas
 logical                :: first_call
!----------------------wangln 2007-06-24-------------------------------

!--- local variables --------------------------------
 integer         :: i, j, k, n      ! vector index
 real(kind=8)    :: vmag   ! surface wind magnitude   (m/s)
 real(kind=8)    :: thvbot ! virtual temperature      (K)
 real(kind=8)    :: ssq    ! sea surface humidity     (kg/kg)
 real(kind=8)    :: dssqdt ! derivative of ssq wrt Ts (kg/kg/K)
 real(kind=8)    :: delt   ! potential T difference   (K)
 real(kind=8)    :: delq   ! humidity difference      (kg/kg)
 real(kind=8)    :: stable ! stability factor
 real(kind=8)    :: rdn    ! sqrt of neutral exchange coefficient (momentum) 
 real(kind=8)    :: rhn    ! sqrt of neutral exchange coefficient (heat)     
 real(kind=8)    :: ren    ! sqrt of neutral exchange coefficient (water)    
 real(kind=8)    :: rd     ! sqrt of exchange coefficient (momentum)         
 real(kind=8)    :: rh     ! sqrt of exchange coefficient (heat)             
 real(kind=8)    :: re     ! sqrt of exchange coefficient (water)            
 real(kind=8)    :: ustar  ! ustar             
 real(kind=8)    :: qstar  ! qstar             
 real(kind=8)    :: tstar  ! tstar             
 real(kind=8)    :: hol    ! H (at zbot) over L
 real(kind=8)    :: xsq    ! temporary variable                         
 real(kind=8)    :: xqq    ! temporary variable                         
 real(kind=8)    :: psimh  ! stability function at zbot (momentum)      
 real(kind=8)    :: psixh  ! stability function at zbot (heat and water)
 real(kind=8)    :: alz    ! ln(zbot/z10)
 real(kind=8)    :: u10n   ! 10m neutral wind 
 real(kind=8)    :: tau    ! stress at zbot
 real(kind=8)    :: cp     ! specific heat of moist air
 real(kind=8)    :: bn     ! exchange coef funct for interpolation
 real(kind=8)    :: bh     ! exchange coef funct for interpolation
 real(kind=8)    :: fac    ! interpolation factor
 real(kind=8)    :: ln0    ! log factor for interpolation
 real(kind=8)    :: ln3    ! log factor for interpolation
! luyx, 2013.6.7
 real(kind=8)    :: visa, rstar, zom, zot, zoq, alzm, &
                    alzt, alzq, psimhs, psixhs

!--- local functions --------------------------------
 real(kind=8)    :: Tk     ! dummy arg ~ temperature (K)
 real(kind=8)    :: qsat_h2o   ! the saturation humididty of air (kg/m^3)
 real(kind=8)    :: dqsatdt_h2o ! derivivative of qsat wrt surface temperature
 real(kind=8)    :: qsat_ice   ! the saturation humididty of air (kg/m^3)
 real(kind=8)    :: dqsatdt_ice ! derivivative of qsat wrt surface temperature
 real(kind=8)    :: Umps   ! wind velocity (m/s)
 real(kind=8)    :: cdn    ! function: neutral drag coeff at 10m
 real(kind=8)    :: psimhu ! unstable part of psimh
 real(kind=8)    :: psixhu ! unstable part of psimx
 real(kind=8)    :: xd     ! dummy argument  
 real(kind=8)    :: delt_Tice   ! T_now - T_previous  
 real(kind=8)    :: qsat, dqsatdt, psurf
 
!--------------wangln 2007-11-29 reference the Dice5---------------------
 qsat_h2o(Tk)   = 627572.4 / exp(5107.4/Tk)
 dqsatdt_h2o(Tk) = (5107.4 / Tk**2) * 627572.4 / exp(5107.4/Tk)
 qsat_ice(Tk)    = QQQ / exp( TTT / Tk )
 dqsatdt_ice(Tk) = ( TTT / Tk**2) * QQQ / exp ( TTT / Tk )
 cdn(Umps)  = 0.0027 / Umps + 0.000142 + 0.0000764 * Umps
!--------------wangln 2007-11-29 reference the Dice5---------------------

 psimhu(xd) = log((1.0+xd*(2.0+xd))*(1.0+xd*xd)/8.0) - 2.0*atan(xd) + 1.571
 psixhu(xd) = 2.0 * log((1.0 + xd*xd)/2.0)
 
!-------------------------------------------------------------------------------
! PURPOSE:
!   computes certain atm/ocn surface fluxes
!
! NOTES: 
!   o all fluxes are positive downward
!   o net heat flux = net sw + lw up + lw down + sen + lat
!   o here, tstar = <WT>/U*, and qstar = <WQ>/U*.
!   o wind speeds should all be above a minimum speed (eg. 1.0 m/s)
! 
! ASSUMPTIONS:
!   o The saturation humidity of air at T(K): qsat(T)  (kg/m^3)
!-------------------------------------------------------------------------------

  do j = jsc, jec
     do i = isc, iec
       if( ice % mask(i,j) ) then
          if( first_call ) then
            ice%previous_t_surf(i,j,:) = 0.0
            do k = 1, km
               ice%previous_t_surf(i,j,k) = ice%t_surf(i,j,k)
            enddo
            first_call = .false.
          endif 
          
        zbot  = atmos%atmos_z(i,j)        
        thbot = atmos%atmos_ptem(i,j)        
        qbot  = atmos%atmos_shum(i,j)        
        rbot  = atmos%atmos_dens(i,j)        
        tbot  = atmos%atmos_tbot(i,j)        

        ubot =  atmos%atmos_u(i,j)
        vbot =  atmos%atmos_v(i,j)
        psurf = atmos%atmos_p_surf(i,j)
!--------------------------------------------------Initialize--------------------------------------------------------------
        ice % atmos_tref(i,j,:) = 0.0;     ice % atmos_qref  (i,j,:) = 0.0
        ice % atmos_taux(i,j,:) = 0.0;     ice % atmos_tauy  (i,j,:) = 0.0
        ice % atmos_sen (i,j,:) = 0.0;     ice % atmos_latent(i,j,:) = 0.0
        ice % atmos_evap(i,j,:) = 0.0;     ice % atmos_swnet (i,j,:) = 0.0
        ice % dedt_surf (i,j,:) = 0.0;     ice % drdt_surf   (i,j,:) = 0.0
        ice % dhdt_surf (i,j,:) = 0.0;     ice % atmos_lwup  (i,j,:) = 0.0
        ice % atmos_tsurf(i,j)  = 0.0  
!--------------------------------------------------Initialize--------------------------------------------------------------
  
        do k = 1, km

           ts     = ice%t_surf(i,j,k)
           !!albedo = ice%albedo(i,j,k)

           !--- compute some needed quantities ---
           vmag   = max(umin, sqrt(ubot**2+vbot**2))
           thvbot = thbot * (1.0 + zvir * qbot) ! virtual temp (K)

!!=================FMS QSat Scheme======================================
           !!call escomp ( ts,  qsat )
           !!call descomp( ts, dqsatdt )
           !!ssq =  d622 * qsat / ( psurf - d378 * qsat ) 
           !!dssqdt =  d622 * dqsatdt / ( psurf - d378 * dqsatdt )
!!=================FMS QSat Scheme======================================
           
!!=================CCSM CSIM4 QSat Scheme=====================================
           ssq = qsat_ice(ts) / rbot
           dssqdt = dqsatdt_ice(ts) / rbot 
!!=================CCSM QSat Scheme=====================================

           delt   = thbot - ts                 ! pot temp diff (K)
           delq   = qbot - ssq                   ! spec hum dif (kg/kg)
           alz    = log(zbot/zref) 
           cp     = cpdair*(1.0 + cpvir*ssq) 
           ! luyx, 2013.6.7
           visa   = 1.326e-5*(1+6.542e-3*(thbot-273.16)+8.301e-6*(thbot-273.16)**2 &
                    -4.84e-9*(thbot-273.16)**3)

   
           !------------------------------------------------------------
           ! first estimate of Z/L and ustar, tstar and qstar
           !------------------------------------------------------------
           !--- neutral coefficients, z/L = 0.0 ---
           rdn = karman/log(zref/wzsice)
           rhn = rdn
           ren = rdn

           !--- ustar,tstar,qstar ----
           ustar = rdn * vmag
           tstar = rhn * delt
           qstar = ren * delq

           ! luyx, 2013.6.7
           if (ts.le.271.16) then
              zom = 1.e-4
           else
              zom = 8.e-4
           end if

           !--- compute stability & evaluate all stability functions ---
           do n = 1, 5

              ! luyx, 2013.6.7
              ! compute scalar roughness lengths based on the Andreas's (1987) theoretical model
              rstar = ustar*zom/visa
              if (rstar.le.0.135) then
                 zot = 1.250
                 zoq = 1.610
              elseif (rstar.gt.0.135.and.rstar.lt.2.5) then
                 zot = 0.149-0.550*log(rstar)
                 zoq = 0.351-0.628*log(rstar)
              elseif (rstar.ge.2.5.and.rstar.le.1000.) then
                 zot = 0.317-0.565*log(rstar)-0.183*(log(rstar))**2
                 zoq = 0.396-0.512*log(rstar)-0.180*(log(rstar))**2
              end if
              zot = zom*exp(zot)
              zoq = zom*exp(zoq)
 
              alzm = alz+log(wzsice/zom)
              alzt = alz+log(wzsice/zot)
              alzq = alz+log(wzsice/zoq)

              hol    = karman * g * zbot * (tstar/thvbot+qstar/(1./zvir+qbot)) / ustar**2
              hol    = sign( min(abs(hol),10.), hol )
              stable = 0.5 + sign(0.5 , hol)
              xsq    = max(sqrt(abs(1. - 16.*hol)) , 1.)
              xqq    = sqrt(xsq)
              ! luyx, 2013.6.7
              !psimh  = -5.*hol*stable + (1.-stable)*psimhu(xqq)
              !psixh  = -5.*hol*stable + (1.-stable)*psixhu(xqq)
              ! the Zeng et al. (1998) stability functions
              if (hol.gt.0.0 .and. hol.le.1.0) then
                 psimhs = -5.0*hol
              elseif (hol.gt.1.0) then
                 psimhs = -4.*log(hol)-hol-4.
              end if
              psixhs = psimhs
              psimh  = stable*psimhs + (1.-stable)*psimhu(xqq)
              psixh  = stable*psixhs + (1.-stable)*psixhu(xqq)

              !--- shift all coeffs to measurement height and stability ---
              ! luyx, 2013.6.7
              !rd = rdn / (1.+rdn/karman*(alz-psimh))
              !rh = rhn / (1.+rhn/karman*(alz-psixh))
              !re = ren / (1.+ren/karman*(alz-psixh))
              rd = rdn / (1.+rdn/karman*(alzm-psimh))
              rh = rhn / (1.+rhn/karman*(alzt-psixh))
              re = ren / (1.+ren/karman*(alzq-psixh))

              !--- update ustar, tstar, qstar w/ updated, shifted coeffs --
              ustar = rd * vmag
              tstar = rh * delt
              qstar = re * delq
           enddo 

           !------------------------------------------------------------
           ! compute the fluxes
           !------------------------------------------------------------
   
           tau = rbot * ustar * ustar 
       
!------------------wangln 2007-06-24 For drdt, dedt, dhdt--------------------
           dhdt =  cp * rbot * ustar * rh                      ! Sen  / delt & Tstar = rh * delt 
           dedt =  re * rbot * ustar * dssqdt  
           drdt =  4 * emissity * stebol * ( ts ** 3 ) 

           !--- momentum flux ---
           atmos%u_flux(i,j,k) = - tau * ubot / vmag  ! upward (+) 
           atmos%v_flux(i,j,k) = - tau * vbot / vmag  ! upward (+) 
!
! -------------------
           !--- heat flux ---
           atmos%t_flux(i,j,k)  = -  rbot * ustar * rh  * cp * delt     
           atmos%q_flux(i,j,k)  = -  rbot * ustar * re  * delq         
           swndf = atmos%atmos_swndf(i,j) * ( 1. - ice%albedo_nir_dif(i,j,k) )        
           swvdf = atmos%atmos_swvdf(i,j) * ( 1. - ice%albedo_vis_dif(i,j,k) )       
           swndr = atmos%atmos_swndr(i,j) * ( 1. - ice%albedo_nir_dir(i,j,k) )       
           swvdr = atmos%atmos_swvdr(i,j) * ( 1. - ice%albedo_vis_dir(i,j,k) )       
           atmos%sw_flux(i,j,k) = swndf + swvdf + swndr + swvdr

! wutw 2009-10-26 ==>
!! wutw 2009-10-19 add calculate total ice%albedo, including four band ==>
!           ice%albedo(i,j,k) = 1.0 - atmos%sw_flux(i,j,k) / (atmos%atmos_swndf(i,j)  + &
!                              atmos%atmos_swvdf(i,j) + atmos%atmos_swndr(i,j) + &
!                              atmos%atmos_swvdr(i,j) )
!! wutw 2009-10-19 add calculate total ice%albedo, including four band <==
! wutw 2009-10-26 <==
           
!-----------------------wangln 2008-09-08 accrodingwith CSIM4 in CCSM3-----------------------
           !!atmos%lw_flux(i,j,k) = atmos%lw_flux(i,j,k) -  emissity * stebol * ( ts ** 4 ) 
           atmos%lw_flux(i,j,k) = emissity * ( atmos%lw_flux(i,j,k) - stebol * ( ts ** 4 ) ) 
!-----------------------wangln 2008-09-08 accrodingwith CSIM4 in CCSM3-----------------------

           atmos%drdt(i,j,k) = drdt 
           atmos%dhdt(i,j,k) = dhdt 
           atmos%dedt(i,j,k) = dedt 

!------------------------wutw 20090805 ----------------------
!
            ice%u_10m(i,j,k) = ustar/karman*(alz-psimh)* ubot / vmag      ! 2009.08.05
            ice%v_10m(i,j,k) = ustar/karman*(alz-psimh)* vbot / vmag      ! 2009.08.05

!------------------wangln 2007-06-24 For drdt, dedt, dhdt--------------------
           bn = karman/rdn
           bh = karman/rh

           !--- interpolation factor for stable and unstable cases ---
           ln0 = log(1.0 + (ztref/zbot)*(exp(bn) - 1.0))
           ln3 = log(1.0 + (ztref/zbot)*(exp(bn - bh) - 1.0))
           fac = (ln0-ztref/zbot*(bn-bh))/bh*stable+(ln0-ln3)/bh*(1.0-stable)
           fac = min(max(fac,0.0),1.0)

           ice%atmos_tref(i,j,k) = ts + (tbot - ts)*fac
           ice%atmos_qref(i,j,k)= qbot-fac*delq   ! 2m reference height Humidity
    
           ice%previous_t_surf(i,j,k) = ice%t_surf(i,j,k)
           ice%atmos_sen(i,j,k)       = rbot * ustar * rh  * cp * delt

!-----------------------wangln 2008-09-08 accrodingwith CSIM4 in CCSM3-----------------------
           !!ice%atmos_lwup(i,j,k)      = - emissity * stebol * ( ts**4 ) 
           ice%atmos_lwup(i,j,k)      = - emissity * stebol * ( ts**4 ) - ( 1- emissity ) * atmos%lw_flux(i,j,k)  
!-----------------------wangln 2008-09-08 accrodingwith CSIM4 in CCSM3-----------------------

           ice%atmos_evap(i,j,k)      = rbot * ustar * re  * delq 
           ice%atmos_swnet(i,j,k)     =  swndr + swvdr + swndf + swvdf 
           ice%atmos_taux(i,j,k)      = tau * ubot / vmag  
           ice%atmos_tauy(i,j,k)      = tau * vbot / vmag 
           ice%dedt_surf(i,j,k)       = 0. 
           ice%drdt_surf(i,j,k)       = 0. 
           ice%dhdt_surf(i,j,k)       = 0. 
 
          enddo !K Count


           endif   !! Ice%mask(i,j) = .true.
        enddo !I count
     enddo !J Count
    
END Subroutine atmos_ice_flux_cpl
