module gather_data_mod 
    
   use mpmd_mod
   use mom_basic_mod,    only : num_cpu, imt_global, jmt_global, local_domain
   use cpl_mod,          only : global_sin, global_cos
   use cpl_mod,          only : nsfields, sendfields, send_to_cpl
   use ice_type_mod,     only : ice_data_type 
   use mpp_mod,          only : stdout
   use cpl_mod,          only : cpl_stop_now
   use cpl_mod,          only : sis_csim 
   use cpl_mod,          only : ex_ice_ocn 
   use cpl_mod,          only : dt_evp, slow_freq 

!-----------2007-11-04-------------------------------
   use ice_grid_mod,     only : km
!-----------2007-11-04-------------------------------

   implicit none
    
   public   :: gather_local_data
   private  :: transfer_3d_to_1d
   private  :: transfer_1d_to_3d
   private  :: compute_displs_counts 

contains
!------------------------------------------------------------------------------------------------------ 
 subroutine transfer_3d_to_1d( local_ice, isc, iec, jsc, jec, local_data, total_size )
 
    type (ice_data_type), intent(in)     :: local_ice
    integer, intent(in)                  :: isc, iec, jsc, jec
    real(kind=8), intent(out)            :: local_data( 0: ) 

    real(kind=8) :: ataux, atauy, otaux, otauy, u10m, v10m 
    
    integer,      intent(in)             :: total_size

    integer                              :: i, j, k, n, ierr

    n = 0
   
    local_data = 0.0
    do j = jsc, jec 
       do i = isc, iec
          
          local_data( n ) = local_ice%ifrac(i,j,1);           n = n + 1
          local_data( n ) = local_ice%atmos_tsurf(i,j);       n = n + 1
          local_data( n ) = local_ice%atmos_albedo(i,j,1);    n = n + 1
          local_data( n ) = local_ice%atmos_albedo(i,j,3);    n = n + 1
          local_data( n ) = local_ice%atmos_albedo(i,j,2);    n = n + 1
          local_data( n ) = local_ice%atmos_albedo(i,j,4);    n = n + 1
!---
          local_data( n ) = local_ice%atmos_taux(i,j,1);      n = n + 1
          local_data( n ) = local_ice%atmos_tauy(i,j,1);      n = n + 1

!          ataux = local_ice%atmos_taux(i,j,1)
!          atauy = local_ice%atmos_tauy(i,j,1)
!          local_data( n ) =  ataux *global_cos(i,j) + &
!                             atauy *global_sin(i,j)       ;      n = n + 1
!          local_data( n ) =  atauy *global_cos(i,j) - &
!                             ataux *global_sin(i,j)       ;      n = n + 1
!---

          local_data( n ) = local_ice%atmos_latent(i,j,1);    n = n + 1 
          local_data( n ) = local_ice%atmos_sen(i,j,1);       n = n + 1 
          local_data( n ) = local_ice%atmos_lwup(i,j,1);      n = n + 1 
          local_data( n ) = 0.0;                              n = n + 1 

          local_data( n ) = local_ice%atmos_swnet(i,j,1);     n = n + 1 
          local_data( n ) = local_ice%atmos_tref(i,j,1);      n = n + 1 
          local_data( n ) = local_ice%atmos_qref(i,j,1);      n = n + 1 

          local_data( n ) = local_ice%meltw(i,j,1);           n = n + 1

          local_data( n ) = local_ice%ocn_q(i,j,1);           n = n + 1
          local_data( n ) = local_ice%ocn_t(i,j,1);           n = n + 1
          local_data( n ) = local_ice%ocn_swnet(i,j,1);       n = n + 1 
          local_data( n ) = local_ice%flux_salt(i,j);         n = n + 1 
!---
! wtw
!         otaux = local_ice%ocn_taux(i,j)
!         otauy = local_ice%ocn_tauy(i,j)
!         local_ice%ocn_taux(i,j) =  otaux *global_cos(i,j) + &
!                                    otauy *global_sin(i,j)
!         local_ice%ocn_tauy(i,j) =  otauy *global_cos(i,j) - &
!                                    otaux *global_sin(i,j)

         local_data( n ) = local_ice%ocn_taux(i,j)    ; n = n + 1       ! downward (+) 
         local_data( n ) = local_ice%ocn_tauy(i,j)    ; n = n + 1       ! downward (+) 
!------

         local_data( n ) = local_ice%ocn_p(i,j);             n = n + 1

!         u10m = local_ice%u_10m(i,j,1)* global_cos(i,j) + &
!                local_ice%v_10m(i,j,1)* global_sin(i,j)
!         v10m = local_ice%v_10m(i,j,1)* global_cos(i,j) - &
!                local_ice%u_10m(i,j,1)* global_sin(i,j)

         local_data( n ) = local_ice%u_10m(i,j,1);             n = n + 1
         local_data( n ) = local_ice%v_10m(i,j,1);             n = n + 1
!         local_data( n ) = u10m;             n = n + 1
!         local_data( n ) = v10m;             n = n + 1

      enddo
   enddo

    end subroutine transfer_3d_to_1d 
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
    subroutine transfer_1d_to_3d( temp_data )
    
        real(kind=8), intent(in)  :: temp_data( 0:imt_global*jmt_global*nsfields-1 )   
 
        integer :: i, j, k, n, m
   
        k = 0
        sendfields = 0.0
        do n = 0, num_cpu-1
           do j = local_domain( n )%y_beg, local_domain( n )%y_end
              do i = local_domain( n )%x_beg, local_domain( n )%x_end
                 do m = 1, nsfields
                    sendfields( i, j, m ) = temp_data( k )
                    k = k + 1
                 enddo
              enddo
           enddo
        enddo

    end subroutine transfer_1d_to_3d      
               
         
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
    subroutine gather_local_data(  cpl_date, ice, isc,iec, jsc, jec  )
 
    include 'mpif.h'
     
       integer, intent(in)                  :: cpl_date(6) 
       type (ice_data_type), intent(in)   :: ice 
       integer, intent(in)                  :: isc, iec, jsc, jec
       
       integer  :: myid
       integer  :: gsize, ierr
       integer  :: i, j, k, n
       integer  :: total_size
       real(kind=8), allocatable :: temp_data( : ) 
       real(kind=8), allocatable :: global_data( :,: )

       integer :: displs( 0 : num_cpu-1 )  
       integer :: recvcounts( 0:num_cpu-1 )

       real(kind=8), allocatable :: local_data( : )
       character(len=3) :: cstep

    

       total_size = ( iec - isc + 1 ) * ( jec - jsc + 1 ) * nsfields  -1
       allocate( local_data( 0:total_size ), stat = ierr )
       if( ierr /= 0 ) then
           write(stdout(),*) '----Sub gather_local_data:  applied the Array : local_data is failed !'  
       else
           local_data = 0.0
       endif

       call transfer_3d_to_1d( ice, isc, iec, jsc, jec, local_data, total_size )

       if( master_task ) then

          allocate( global_data( imt_global, jmt_global ), stat = ierr )
          if( ierr /=0 ) then
              write(stdout(),*) '--Sub gather_local_data: Array applied Failed : Global_data '
          else
              global_data = 0.0
          endif
          call compute_displs_counts( nsfields, num_cpu, displs, recvcounts )  ! set the MPI_GATHERV parameter

       endif

       if( master_task ) then

          allocate( temp_data( 0:imt_global*jmt_global*nsfields-1 ), stat = ierr )
          if( ierr /=0 ) then
              write(stdout(),*) '--Sub gather_local_data: Array applied Failed : Temp_data '
          else
              temp_data = 0.0
          endif

          call MPI_GATHERV( local_data,  size( local_data ), MPI_REAL8, & 
               temp_data, recvcounts, displs, MPI_REAL8, 0, MPI_COMM_ICE, ierr )
          if( ierr == 0 ) then
              call transfer_1d_to_3d( temp_data )
              call send_to_cpl( cpl_date )
          endif

       else
          call MPI_GATHERV( local_data, size( local_data ), MPI_REAL8, & 
               0.0, recvcounts, displs, MPI_REAL8, 0, MPI_COMM_ICE, ierr )
       endif


    if ( master_task ) then 
        deallocate( temp_data ) 
        deallocate( global_data )
    endif 
   
    deallocate( local_data ) 

  end subroutine gather_local_data
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
  subroutine compute_displs_counts( nfact, gsize, displs, counts )

     integer, intent(in) :: nfact ! the number of the elements types
     integer, intent(in) :: gsize ! the total CPUs
     integer, intent(out) :: displs( 0:gsize-1 ) ! displacement parameter
     integer, intent(out) :: counts( 0:gsize-1 ) ! the MPI_GATHERV reciver data size for every CPU

     integer  :: k


           do k = 0, gsize-1
              counts( k ) = nfact * local_domain(k)%x_size * local_domain(k)%y_size
           enddo

           displs( 0 ) = 0
           do k = 1, gsize-1
              displs( k ) = displs( k-1 ) + counts( k -1 )
           enddo

     
  end subroutine compute_displs_counts
!------------------------------------------------------------------------------------------------------ 
!======================================================================================================
end module gather_data_mod


