 Subroutine ice_atmos_flux_cpl( Ice, isc, iec, jsc, jec)

 use ice_model_mod,    only :  ice_data_type
 use const_mod
 use ice_grid_mod,     only : km, sin_rot, cos_rot
 use fms_mod,          only : stdout, mpp_pe, mpp_root_pe

 implicit none

 type (ice_data_type),           intent(inout)   :: ice 
 integer,                        intent(in)      :: isc, iec, jsc, jec   
 real(kind=8), parameter :: T_ice0 = 271.36   ! 273.16 -1.8
 real(kind=8), parameter :: stefan = -5.67e-8 ! stefan_boltzmann Constant 

 real(kind=8) :: ts

!--- local variables --------------------------------
 integer :: i, j, k, n      ! vector index
 real(kind=8)    :: total_area  ! Ice area with Temperatur
 real(kind=8)    :: delt        !  Difference of Ice T_surf between t & t+1
 real(kind=8)    :: uv_area     ! Ice area with U/V
 real(kind=8)    :: ltheat = latvap + latice 
 real(kind=8)    :: per         ! every category ice area percent part_size(:,:2:km) / part_size(:,:1)

  do j = jsc, jec
     do i = isc, iec
        ice % ifrac(i,j,:) = 0.0 
        if( ice % mask(i,j) ) then
           do k = 2, km
           
              delt = ice%t_surf(i,j,k) - ice%previous_t_surf(i,j,k)
              ice%atmos_sen(i,j,k)     = ice%atmos_sen(i,j,k)   
              ice%atmos_evap(i,j,k)    = ice%atmos_evap(i,j,k) 
              ice%atmos_latent(i,j,k)  = ltheat * ice%atmos_evap(i,j,k) 
              if( ice % part_size(i,j,k)>= 1e-13 ) ice %ifrac(i,j,k) = ice % part_size(i,j,k)
 
           enddo !K Count

!-----------get teh average flux fro Atmosphere---------------------------------------         

           ice % atmos_taux (i,j,1) = 0.0;  ice % atmos_tauy  (i,j,1) = 0.0
           ice % atmos_swnet(i,j,1) = 0.0;  ice % atmos_lwup  (i,j,1) = 0.0
           ice % atmos_sen  (i,j,1) = 0.0;  ice % atmos_latent(i,j,1) = 0.0
           ice % atmos_evap (i,j,1) = 0.0;  ice % atmos_tsurf (i,j)   = 0.0
           ice % atmos_tref (i,j,1) = 0.0;  ice % atmos_qref  (i,j,1) = 0.0
           ice % atmos_albedo(i,j,1:4) = 0.0

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
           ice % u_10m(i,j,1) = 0.0;    ice % v_10m(i,j,1) = 0.0 
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

           ice % ifrac(i,j,1) = 0.0
       
           do k = 2, km 
              if( ice % ifrac(i,j,k) > 0.0 ) then

                 ice % ifrac       (i,j,1) = ice % ifrac       (i,j,1) + ice % ifrac       (i,j,k) 

                 ice % atmos_tsurf  (i,j)  = ice % atmos_tsurf (i,j)   + ice % ifrac(i,j,k) * ice % t_surf      (i,j,k)  
                 ice % atmos_tref  (i,j,1) = ice % atmos_tref  (i,j,1) + ice % ifrac(i,j,k) * ice % atmos_tref  (i,j,k)  
                 ice % atmos_qref  (i,j,1) = ice % atmos_qref  (i,j,1) + ice % ifrac(i,j,k) * ice % atmos_qref  (i,j,k)  
                 ice % atmos_swnet (i,j,1) = ice % atmos_swnet (i,j,1) + ice % ifrac(i,j,k) * ice % atmos_swnet (i,j,k)  
                 ice % atmos_lwup  (i,j,1) = ice % atmos_lwup  (i,j,1) + ice % ifrac(i,j,k) * ice % atmos_lwup  (i,j,k)  
                 ice % atmos_sen   (i,j,1) = ice % atmos_sen   (i,j,1) + ice % ifrac(i,j,k) * ice % atmos_sen   (i,j,k)  
                 ice % atmos_latent(i,j,1) = ice % atmos_latent(i,j,1) + ice % ifrac(i,j,k) * ice % atmos_latent(i,j,k)
                 ice % atmos_evap  (i,j,1) = ice % atmos_evap  (i,j,1) + ice % ifrac(i,j,k) * ice % atmos_evap  (i,j,k)  
                 ice % atmos_albedo(i,j,1) = ice % atmos_albedo(i,j,1) + ice % ifrac(i,j,k) * ice % albedo_vis_dir(i,j,k)
                 ice % atmos_albedo(i,j,2) = ice % atmos_albedo(i,j,2) + ice % ifrac(i,j,k) * ice % albedo_vis_dif(i,j,k)
                 ice % atmos_albedo(i,j,3) = ice % atmos_albedo(i,j,3) + ice % ifrac(i,j,k) * ice % albedo_nir_dir(i,j,k)
                 ice % atmos_albedo(i,j,4) = ice % atmos_albedo(i,j,4) + ice % ifrac(i,j,k) * ice % albedo_nir_dif(i,j,k)
                 ice % atmos_taux  (i,j,1) = ice % atmos_taux  (i,j,1) + ice % ifrac(i,j,k) * ice % atmos_taux  (i,j,k)  
                 ice % atmos_tauy  (i,j,1) = ice % atmos_tauy  (i,j,1) + ice % ifrac(i,j,k) * ice % atmos_tauy  (i,j,k)  

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
                 ice % u_10m  (i,j,1) = ice % u_10m  (i,j,1) + ice % ifrac(i,j,k) * ice % u_10m  (i,j,k)  
                 ice % v_10m  (i,j,1) = ice % v_10m  (i,j,1) + ice % ifrac(i,j,k) * ice % v_10m  (i,j,k)  
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

              endif
           enddo

           total_area = 0.0   
           total_area =  ice % ifrac   (i,j,1)
           if( total_area > 0 ) then
              ice % atmos_tsurf  (i,j)  = ice % atmos_tsurf (i,j  )  / total_area 
              ice % atmos_tref  (i,j,1) = ice % atmos_tref  (i,j,1)  / total_area
              ice % atmos_qref  (i,j,1) = ice % atmos_qref  (i,j,1)  / total_area
              ice % atmos_swnet (i,j,1) = ice % atmos_swnet (i,j,1)  / total_area 
              ice % atmos_lwup  (i,j,1) = ice % atmos_lwup  (i,j,1)  / total_area 
              ice % atmos_sen   (i,j,1) = ice % atmos_sen   (i,j,1)  / total_area 
              ice % atmos_latent(i,j,1) = ice % atmos_latent(i,j,1)  / total_area
              ice % atmos_evap  (i,j,1) = ice % atmos_evap  (i,j,1)  / total_area 
              ice % atmos_albedo(i,j,:) = ice % atmos_albedo(i,j,:)  / total_area
              ice % atmos_taux  (i,j,1) = ice % atmos_taux  (i,j,1)  / total_area 
              ice % atmos_tauy  (i,j,1) = ice % atmos_tauy  (i,j,1)  / total_area 

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
              ice % u_10m       (i,j,1) = ice % u_10m       (i,j,1)  / total_area 
              ice % v_10m       (i,j,1) = ice % v_10m       (i,j,1)  / total_area 
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

           else
              ice % atmos_tref  (i,j,1) = ice % atmos_tref  (i,j,1)  
              ice % atmos_tsurf  (i,j)  = ice % atmos_tsurf  (i,j) 
!----------------------------
!wtw          ice % atmos_albedo(i,j,:) = ice % atmos_albedo(i,j,1) 
!
              ice % atmos_albedo(i,j,:) = 0.06
!-------------------------------
              ice % atmos_taux  (i,j,1) = ice % atmos_taux  (i,j,1)  
              ice % atmos_tauy  (i,j,1) =  ice % atmos_tauy  (i,j,1) 
           endif
            
          endif !! ice%mask            

        enddo !I count
     enddo !J Count
    
END Subroutine ice_atmos_flux_cpl
!=============================================================================================================
 Subroutine ice_ocn_flux( Ice,  isc, iec, jsc, jec)

 use ice_model_mod,    only :  ice_data_type
 use const_mod
 use ice_grid_mod,     only : km, sin_rot, cos_rot
 use fms_mod,          only : stdout, mpp_pe, mpp_root_pe

 implicit none

 type (ice_data_type),           intent(inout)   :: ice 
 integer,                        intent(in)      :: isc, iec, jsc, jec   
 real(kind=8), parameter :: T_ice0 = 271.36   ! 273.16 -1.8
 real(kind=8), parameter :: stefan = -5.67e-8 ! stefan_boltzmann Constant 

 real(kind=8) :: ts

!--- local variables --------------------------------
 integer :: i, j, k, n      ! vector index
 real(kind=8)    :: total_area  ! Ice area with Temperatur
 real(kind=8)    :: delt        !  Difference of Ice T_surf between t & t+1
 real(kind=8)    :: uv_area     ! Ice area with U/V
 real(kind=8)    :: ltheat = latvap + latice 
 real(kind=8)    :: per         ! every category ice area percent part_size(:,:2:km) / part_size(:,:1)

        
  do j = jsc, jec
     do i = isc, iec
        Ice % ifrac(i,j,:) = 0.0
        if( ice % mask(i,j) ) then
           do k = 2, km
              if( ice%part_size(i,j,k) >= 1e-13 ) Ice % ifrac(i,j,k) = ice%part_size(i,j,k)
           enddo  
           Ice % ifrac(i,j,1) = 0.0
           do k = 2, km
              Ice % ifrac(i,j,1) = Ice % ifrac(i,j,1) + Ice % ifrac(i,j,k)
           enddo 
          
           Ice % ocn_q    (i,j,1) = 0.0;     Ice % ocn_t    (i,j,1) = 0.0
           Ice % meltw    (i,j,1) = 0.0;     Ice % ocn_swnet(i,j,1) = 0.0
           !!Ice % ocn_p    (i,j  ) = 0.0
           do k = 2, km
              if(  ice% ifrac(i,j,1) > 0.0 ) then
                 per = ice % ifrac(i,j,k) / ice % ifrac(i,j,1) 
                 Ice % ocn_q    (i,j,1) = Ice % ocn_q    (i,j,1) + Ice % ocn_q    (i,j,k) * per
                 Ice % ocn_t    (i,j,1) = Ice % ocn_t    (i,j,1) + Ice % ocn_t    (i,j,k) * per
                 Ice % meltw    (i,j,1) = Ice % meltw    (i,j,1) + Ice % meltw    (i,j,k) * per
                 Ice % ocn_swnet(i,j,1) = Ice % ocn_swnet(i,j,1) + Ice % ocn_swnet(i,j,k) * per
              endif
           enddo 
        endif  !! ice%mask
     enddo
  enddo
              
     

END Subroutine ice_ocn_flux
