module mom_basic_mod
! This module is to get the MOM basic information    
! -------wangln 2006-03-25---------------- 
 use mpp_mod,                  only: stdout

    implicit none

    type mom_info
         integer :: x_beg  !x-axis beginning position
         integer :: x_end  !x-axis end position
         integer :: y_beg  !y-axis beginning position
         integer :: y_end  !y-axis end position
         integer :: x_size !x-axis size
         integer :: y_size !y-axis size
    end type mom_info
    public    read_spec_parameter
    public    get_local_size
    public    free_local_domain 
    private   calculate_local_size 


    save
    
         integer, public     :: imt_global  ! i-axis size
         integer, public     :: jmt_global  ! j-axis size 
         integer, public     :: kmt_global   ! no of vertical levels
            
         integer, public     :: num_cpu     ! CPUs
         integer, public     :: x_cpu       ! i-axis CPUs 
         integer, public     :: y_cpu       ! y-axis CPUs 

         type(mom_info), public, allocatable :: local_domain( : )
   

contains
!======================================================================================================
!------------------------------------------------------------------------------------------------------ 

    subroutine read_spec_parameter()
!   this subroutine get the MOM global I/J axis size from the fiel--grid_spec by using the MPP IO sub
     include 'netcdf.inc'
     
     integer  :: ierr
     integer  :: rcode   ! netcdf routine return code
     integer  :: file_id ! netcdf file ID
     integer  :: vid     ! netcdf variable ID
     integer  :: did     ! netcdf dimension ID

       character(len=128)                        :: name
       character(len=128)                        :: grd_file
       integer                                   :: i, unit, ndim, nvar, natt, ntime, len

        rcode = nf_open('INPUT/grid_spec.nc',NF_OPEN,file_id)
        if( rcode /= 0 ) write(stdout(),*) '---subroutine read_spec_parameter: Not open the NC : INPUT/grid_spec.nc ! '
        rcode = nf_inq_dimid (file_id, 'grid_x_T', did)
        rcode = nf_inq_dimlen(file_id, did, imt_global )
        rcode = nf_inq_dimid (file_id, 'grid_y_T', did)
        rcode = nf_inq_dimlen(file_id, did, jmt_global )
        rcode = nf_inq_dimid (file_id, 'zt', did)
        rcode = nf_inq_dimlen(file_id, did, kmt_global )

       rcode = nf_close( file_id )

       !write(6,*) 'IMT, JMT=', imt_global, jmt_global 

    end subroutine read_spec_parameter

!------------------------------------------------------------------------------------------------------ 

    subroutine get_local_size( mpicom )
!----This subroutine to get the MOM local size by calculating with CPUs 
!----and the method is from the sub: mpp_define_layout2D


       include 'mpif.h'

       integer, intent(in) :: mpicom

       integer :: ierr
       integer :: i ,j, n
       integer, allocatable     :: i_global(:,:)    ! global index for local subdomain of i-axis
       integer, allocatable     :: j_global(:,:)    ! global index for local subdomain of j-axis

       call mpi_comm_size( mpicom, num_cpu, ierr )
       allocate( local_domain( 0:num_cpu-1 ), stat = ierr )
       if( ierr /= 0 )  then
           write(stdout(),*) '----Get_local_size: applied the local_domain is fail !----- ' 
       endif

!---------Calculating CPU number on each X/Y axis--------------------   
       x_cpu = nint( sqrt( float(num_cpu*imt_global)/jmt_global) )
       x_cpu = max( x_cpu, 1 )
       do while ( mod( num_cpu, x_cpu ) /= 0 )
          x_cpu = x_cpu - 1
       enddo
       y_cpu = num_cpu / x_cpu
       

!============wangln 2007-06-28 Because the SIS has different Parallel analyse scheme as the MOM============= 
       x_cpu = num_cpu
       y_cpu = 1 
       ! the Parallel aanalyse only accord as the Longitude butnot same as the mom within a grid(Lon & Lat)
!============wangln 2007-06-28 Because the SIS has different Parallel analyse scheme as the MOM============= 


       !write(6,*) ' X_CPU & Y_CPU= ', x_cpu, y_cpu
!---------Calculating CPU number on each X/Y axis--------------------   

       allocate( i_global( 0:x_cpu-1, 2), stat = ierr )  
       if( ierr /= 0 )  then
           write(stdout(),*) '----Get_local_size:  applied the i_global is failed !----- ' 
       else
           i_global = 0
       endif

       allocate( j_global( 0:y_cpu-1, 2), stat = ierr )  
       if( ierr /= 0 )  then
           write(stdout(),*) '----Get_local_size:  applied the j_global is failed !----- ' 
       else
           j_global = 0
       endif
  
       call calculate_local_size( y_cpu, j_global, 1, jmt_global )
       call calculate_local_size( x_cpu, i_global, 1, imt_global )

       n = 0
       do j = 0, y_cpu-1
          do i = 0, x_cpu-1
             local_domain( n )%x_beg = i_global( i, 1 )
             local_domain( n )%x_end = i_global( i, 2 )
             local_domain( n )%x_size = i_global( i, 2 ) - i_global( i, 1 ) +1
             local_domain( n )%y_beg = j_global( j, 1 )
             local_domain( n )%y_end = j_global( j, 2 )
             local_domain( n )%y_size = j_global( j, 2 ) - j_global( j, 1 ) +1
             n = n + 1
          enddo
       enddo
             
       deallocate( i_global )
       deallocate( j_global )
 
       return

   end subroutine get_local_size 

!------------------------------------------------------------------------------------------------------ 

   subroutine calculate_local_size( ncpu, global_size, global_begin, global_end )
!------This is to calculating the every local area position : x-beg/end, y-beg/end
!-------and the method is from the sub: mpp_define_domain1d

     integer, intent(in) :: ncpu
     integer, intent(in) :: global_begin 
     integer, intent(in) :: global_end 
     integer, intent(out) :: global_size( 0:ncpu-1, 2 )
  
     integer :: i, j,k, n, ierr
     integer :: is, ie, ndmax, imax, ndmirror
     integer :: isg, ieg
     logical :: symmetrize
     logical, allocatable :: mask(:)
     logical :: even, odd

     !statement functions
        even(n) = (mod(n,2).EQ.0)
        odd (n) = (mod(n,2).EQ.1)

     allocate( mask( 0:ncpu-1 ), stat = ierr )
     if( ierr /= 0 ) then 
         write(stdout(),*) '----Calculate_local_size:  applied Mask is failed !----- ' 
     else
         mask = .true.
     endif 

     isg = global_begin
     ieg = global_end
     is = global_begin

     n = 0
     do k = 0, ncpu-1

        symmetrize = ( even(ncpu) .AND. even(ieg-isg+1) ) .OR. (  odd(ncpu) .AND.  odd(ieg-isg+1) ) .OR. &
                     (  odd(ncpu) .AND. even(ieg-isg+1) .AND. ncpu.LT.(ieg-isg+1)/2 )
        if( k == 0 ) then
            imax = global_end 
            ndmax = ncpu
        endif
        if( k < ( (ncpu-1)/2 + 1 ) ) then

            ie = is + CEILING( real(imax-is+1)/( ndmax - k ) ) - 1
            ndmirror = ( ncpu-1 ) - k !mirror domain

            if( ndmirror > k .AND. symmetrize ) then !only for domains over the midpoint
               !mirror extents, the max(,) is to eliminate overlaps
               global_size( ndmirror, 1 ) = max( global_begin + global_end - ie, ie+1 )
               global_size( ndmirror, 2 ) = max( global_begin + global_end - is, ie+1 )
               imax = global_size( ndmirror, 1 ) - 1
               ndmax = ndmax - 1
            endif
                
        else
   
            if( symmetrize ) then
                is = global_size( k, 1 )  
                ie = global_size( k, 2 )  
            else
                ie = is + CEILING( REAL( imax-is+1 )/( ndmax-k ) ) - 1
            endif

        endif

        global_size( k, 1) = is 
        global_size( k, 2) = ie 
        
        if( mask(k) ) then
           n = n + 1
        endif
           is = ie +1
     enddo

     deallocate( mask )

     return

   end subroutine calculate_local_size 
!------------------------------------------------------------------------------------------------------ 

   subroutine free_local_domain()

      deallocate( local_domain ) 

   end subroutine free_local_domain 
!------------------------------------------------------------------------------------------------------ 

!=====================================================================================================
end module mom_basic_mod


