module  scatter_data_mod 
    
   use mpmd_mod
   use mom_basic_mod,    only : num_cpu, imt_global, jmt_global, local_domain
   use cpl_mod,          only : nrfields, recvfields, recv_from_cpl, cpl_stop_now, ex_ice_ocn, ex_ocn_ice
   use ice_model_mod,    only : ocean_ice_boundary_type, atmos_ice_boundary_type, land_ice_boundary_type 
   use mpp_mod,          only : stdout
   use ice_grid_mod,     only : km
   use mpp_mod,          only : mpp_sync

   implicit none
    
   public   :: scatter_force_field
   private  :: transfer_3d_to_1d
   private  :: transfer_1d_to_3d
   private  :: compute_displs_counts 

contains
!=====================================================================================================

    subroutine transfer_3d_to_1d( fields, temp_data )

        real(kind=8), intent(in)  :: fields( imt_global, jmt_global, nrfields ) 
        real(kind=8), intent(out) :: temp_data( 0:imt_global*jmt_global*nrfields-1 ) 

        integer   :: i, j, k, n, m 


        m = 0
        temp_data = 0.0
        do n = 0, num_cpu-1
           do j = local_domain( n )%y_beg, local_domain( n )%y_end
              do i = local_domain( n )%x_beg, local_domain( n )%x_end

                 do k = 1, nrfields 
                      temp_data( m ) = fields(i, j, k ); m = m + 1
                 enddo
              enddo
           enddo    
        enddo


    end subroutine transfer_3d_to_1d 

!------------------------------------------------------------------------------------------------------ 
   subroutine transfer_1d_to_3d( isc, iec, jsc, jec, total_size, local_data, local_ocean, local_atmos, local_land )

!---
! wtw
        use ice_grid_mod,     only : sin_rot, cos_rot
        real(kind=8) :: u,v 
!--   
        integer, intent(in)                          :: isc, iec, jsc, jec
        integer, intent(in)                          :: total_size
        real(kind=8), intent(in)                     :: local_data( 0:total_size )   
        
        type (ocean_ice_boundary_type), intent(inout)   :: local_ocean
        type (atmos_ice_boundary_type), intent(inout)   :: local_atmos
        type ( land_ice_boundary_type), intent(inout)   :: local_land

 
        integer :: i, j, k, n
        real(kind=8) :: ocn_temp
   
         
        n = 0
        do j = jsc, jec
           do i = isc, iec

              local_ocean%t(i,j)                 = local_data( n );   n = n + 1
              local_ocean%s(i,j)                 = local_data( n );   n = n + 1
              local_ocean%u(i,j)                 = local_data( n );   n = n + 1
              local_ocean%v(i,j)                 = local_data( n );   n = n + 1
 !--
 ! wtw  from lon/lat to ocean coordinate
 !
              u =  local_ocean%u(i,j)  
              v =  local_ocean%v(i,j) 
              local_ocean%u(i,j) =  u*cos_rot(i,j) - v*sin_rot(i,j)
              local_ocean%v(i,j) =  u*sin_rot(i,j) + v*cos_rot(i,j)
 !---
              local_ocean%sea_level(i,j)         = local_data( n );   n = n + 1
              local_ocean%frazil(i,j)            = local_data( n );   n = n + 1

              local_atmos%atmos_z(i,j)           = local_data( n );   n = n + 1
              local_atmos%atmos_u(i,j)           = local_data( n );   n = n + 1
              local_atmos%atmos_v(i,j)           = local_data( n );   n = n + 1

!
!              u =  local_atmos%atmos_u(i,j)
!              v =  local_atmos%atmos_v(i,j)
!              local_atmos%atmos_u(i,j) =  u*cos_rot(i,j) + v*sin_rot(i,j)
!              local_atmos%atmos_v(i,j) = -u*sin_rot(i,j) + v*cos_rot(i,j)
!
              local_atmos%atmos_ptem(i,j)        = local_data( n );   n = n + 1
              local_atmos%atmos_tbot(i,j)        = local_data( n );   n = n + 1
              local_atmos%atmos_shum(i,j)        = local_data( n );   n = n + 1
              local_atmos%atmos_dens(i,j)        = local_data( n );   n = n + 1
              local_atmos%atmos_pbot(i,j)        = local_data( n );   n = n + 1
              local_atmos%atmos_swndr(i,j)       = local_data( n );   n = n + 1
              local_atmos%atmos_swvdr(i,j)       = local_data( n );   n = n + 1
              local_atmos%atmos_swndf(i,j)       = local_data( n );   n = n + 1
              local_atmos%atmos_swvdf(i,j)       = local_data( n );   n = n + 1
              local_atmos%lprec(i,j,:)           = local_data( n );   n = n + 1
              local_atmos%fprec(i,j,:)           = local_data( n );   n = n + 1
              local_atmos%lw_flux(i,j,:)         = local_data( n );   n = n + 1
              local_atmos%p(i,j,:)               = local_data( n );   n = n + 1

              if( .not. ex_ice_ocn ) then
                  local_land%step_runoff(i,j)    = local_land%step_runoff(i,j)  + local_data( n );   n = n + 1
                  local_land%step_calving(i,j)   = local_land%step_calving(i,j) + local_data( n );   n = n + 1
              else if( ex_ice_ocn ) then
                  local_land%runoff(i,j)  =  local_land%step_runoff (i,j)  + local_data( n );   n = n + 1
                  local_land%calving(i,j) =  local_land%step_calving(i,j)  + local_data( n );   n = n + 1
                  local_land%step_runoff (i,j) = 0.0
                  local_land%step_calving(i,j) = 0.0
              endif

              local_atmos%atmos_p_surf(i,j)             = local_data( n );   n = n + 1

           enddo
       enddo


    end subroutine transfer_1d_to_3d      
               
         
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
    subroutine scatter_force_field(   ocean_force, atmos_force, land_force, isc, iec, jsc, jec, cpl_date  )
 
    include 'mpif.h'
     
       type (ocean_ice_boundary_type), intent(inout)   :: ocean_force
       type (atmos_ice_boundary_type), intent(inout)   :: atmos_force
       type ( land_ice_boundary_type), intent(inout)   :: land_force
       integer, intent(in)                             :: isc, iec, jsc, jec
       integer, intent(in)                             :: cpl_date(6)
       
       integer                   :: myid
       integer                   :: gsize, ierr
       integer                   :: i, j, k, n
       integer                   :: total_size

       integer                   :: displs( 0 : num_cpu-1 )  
       integer                   :: sendcounts( 0:num_cpu-1 )
       character(len=3)          :: cstep
       !!character(len=2)          :: cname

       real(kind=8), allocatable :: local_data( : )
       real(kind=8), allocatable :: temp_data( : ) 
    
      
      
       if( master_task ) then 
           allocate( temp_data( 0:imt_global*jmt_global*nrfields-1 ), stat = ierr )
           if( ierr /=0 ) then
               write(stdout(),*) '--Sub Scatter_force_field:  Array applied Failed : Temp_data '
           else
               temp_data = 0.0
           endif
       endif

       total_size = ( iec - isc + 1 ) * ( jec - jsc + 1 ) * nrfields -1
       allocate( local_data( 0:total_size ), stat = ierr )
       if( ierr /= 0 ) then
           write(stdout(),*) '----Sub Scatter_force_field:  applied the Array : local_data is failed !'  
       else
           local_data = 0.0
       endif

       
       call recv_from_cpl()
       if( master_task ) then
          call transfer_3d_to_1d( recvfields, temp_data ) ! wangln 2007-06-22
          call compute_displs_counts( nrfields, num_cpu, displs, sendcounts )  ! set the MPI_GATHERV parameter
       endif
 


       if( master_task ) then
           call MPI_SCATTERV( temp_data, sendcounts, displs, MPI_REAL8,  & 
                             local_data, size( local_data ), MPI_REAL8, 0, MPI_COMM_ICE, ierr ) 
       else 
           call MPI_SCATTERV( 0,         sendcounts, displs, MPI_REAL8,  & 
                             local_data, size( local_data ), MPI_REAL8, 0, MPI_COMM_ICE, ierr ) 
       endif
       
     call transfer_1d_to_3d( isc, iec, jsc, jec, total_size, local_data(0:total_size), ocean_force, atmos_force, land_force ) 

   
    deallocate( local_data ) 
    if( master_task )  deallocate( temp_data ) 

  end subroutine scatter_force_field
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
  subroutine compute_displs_counts( nfact, gsize, displs, counts )

     integer, intent(in) :: nfact ! the number of the elements types
     integer, intent(in) :: gsize ! the total CPUs
     integer, intent(out) :: displs( 0:gsize-1 ) ! displacement parameter
     integer, intent(out) :: counts( 0:gsize-1 ) ! the MPI_GATHERV reciver data size for every CPU

     integer  :: k


           do k = 0, gsize-1
              counts( k ) = nfact * local_domain(k)%x_size * local_domain(k)%y_size
           enddo

           displs( 0 ) = 0
           do k = 1, gsize-1
              displs( k ) = displs( k-1 ) + counts( k -1 )
           enddo

     
  end subroutine compute_displs_counts
!------------------------------------------------------------------------------------------------------ 
!======================================================================================================
end module scatter_data_mod 


