#include <misc.h>
#include <preproc.h>

#if (defined MOZART2)

module VOCemiss_initialize

  use shr_kind_mod , only : r8 => shr_kind_r8
  use clm_varctl   , only : nsrest
  use decompMod    , only : get_proc_clumps, get_clump_bounds
  use spmdMod      , only : masterproc ,  mpicom, MPI_REAL8
  use surfFileMod  , only : readVOCdata

  implicit none

  private
  save

!
! Public interfaces
!
  public  ::  initialize_VOC 

  real(r8), public   :: NORM_FAC
  real(r8), public   :: DAYS_BTW_M
!
! Private data
!
    real(r8), pointer :: AEF_ISOP(:)  
    real(r8), pointer :: AEF_MBOX(:)     
    real(r8), pointer :: AEF_MYRC(:)      
    real(r8), pointer :: AEF_SABI(:)      
    real(r8), pointer :: AEF_APIN(:)      
    real(r8), pointer :: AEF_LIMO(:)      
    real(r8), pointer :: AEF_CARE(:)      
    real(r8), pointer :: AEF_BPIN(:)      
    real(r8), pointer :: AEF_OCIM(:)      
    real(r8), pointer :: AEF_OMON(:)      
    real(r8), pointer :: AEF_MOHX(:)      
    real(r8), pointer :: AEF_ACET(:)      
    real(r8), pointer :: AEF_ETOH(:)      
    real(r8), pointer :: AEF_CH2O(:)      
    real(r8), pointer :: AEF_ALD2(:)     
    real(r8), pointer :: AEF_FAXX(:)     
    real(r8), pointer :: AEF_AAXX(:) 
    real(r8), pointer :: AEF_C2H4(:)     
    real(r8), pointer :: AEF_TOLU(:)      
    real(r8), pointer :: AEF_HCNX(:)     
    real(r8), pointer :: AEF_PRPE(:)   
    real(r8), pointer :: AEF_FARN(:)    
    real(r8), pointer :: AEF_BCAR(:)     
    real(r8), pointer :: AEF_OSQT(:)      

    CONTAINS
!==================================================================================

   subroutine initialize_VOC() 

   use clmtype
   use decompMod, only : get_proc_bounds
   use clm_varsur, only: pft, pctpft_lunit, AEF_SABINENE, AEF_OCIMENE,  AEF_MYRCENE, &
                         AEF_MBO, AEF_LIMONENE, AEF_ISOPRENE, AEF_CARENE,            &
                         AEF_BETA_PINENE, AEF_ALPHA_PINENE

!---------------------------Local workspace-----------------------------
   integer   :: p, g, i, j, m       ! indices
   integer   :: ARR_IND
   integer   :: ier                !error code

   real(r8)  :: FACTOR
   real(r8)  :: SPECIES2CARBON
   real(r8)  :: PAC_DAILY
   real(r8)  :: PHI
   real(r8)  :: AAA, BBB
   real(r8)  :: CMLAI 
   real(r8)  :: GAMMA_P_STANDARD
   real(r8)  :: GAMMA_T_LD_STANDARD
   real(r8)  :: GAMMA_T_LI_STANDARD
   real(r8)  :: GAMMA_SM_STANDARD 
   real(r8)  :: GAMMA_LAI_STANDARD
   real(r8)  :: GAMMA_AGE_STANDARD
   real(r8)  :: PT_15, T, R, CEO, CT1, E_OPT, T_OPT, CT2, X
   real(r8)  :: LDF, GAMMA_STANDARD

   real(r8)  :: PFT_EF_OMON(15)     
   real(r8)  :: PFT_EF_MOHX(15)     
   real(r8)  :: PFT_EF_ACET(15)     
   real(r8)  :: PFT_EF_BIDR(15)     
   real(r8)  :: PFT_EF_STRS(15)     
   real(r8)  :: PFT_EF_OTHR(15)     
   real(r8)  :: PFT_EF_APIN(15)     
   real(r8)  :: PFT_EF_MYRC(15)     
   real(r8)  :: PFT_EF_FARN(15)     
   real(r8)  :: PFT_EF_BCAR(15)     
   real(r8)  :: PFT_EF_OSQT(15)     

   real(r8)  :: EM_FRAC_ALD2(15)     
   real(r8)  :: EM_FRAC_ETOH(15)     
   real(r8)  :: EM_FRAC_FAXX(15)     
   real(r8)  :: EM_FRAC_AAXX(15)     
   real(r8)  :: EM_FRAC_CH2O(15)     

    integer  :: nc            ! clump index
    integer  :: nclumps       ! number of clumps on this processor
    integer  :: begp, endp    ! clump beginning and ending pft indices
    integer  :: begc, endc    ! clump beginning and ending column indices
    integer  :: begl, endl    ! clump beginning and ending landunit indices
    integer  :: begg, endg    ! clump beginning and ending gridcell indices

    integer  :: type_veg

   integer , pointer :: ivt(:)           !  vegetation type index
   integer , pointer :: pgridcell(:)     ! gridcell index of corresponding pft
   type(gridcell_type), pointer :: gptr  ! pointer to gridcell derived subtype

!-----------------------------------------------
    AEF_ISOP   => clm3%g%l%c%p%pef%AEF_ISOP
    AEF_MBOX   => clm3%g%l%c%p%pef%AEF_MBOX
    AEF_MYRC   => clm3%g%l%c%p%pef%AEF_MYRC
    AEF_SABI   => clm3%g%l%c%p%pef%AEF_SABI
    AEF_APIN   => clm3%g%l%c%p%pef%AEF_APIN
    AEF_LIMO   => clm3%g%l%c%p%pef%AEF_LIMO
    AEF_CARE   => clm3%g%l%c%p%pef%AEF_CARE
    AEF_BPIN   => clm3%g%l%c%p%pef%AEF_BPIN
    AEF_OCIM   => clm3%g%l%c%p%pef%AEF_OCIM
    AEF_OMON   => clm3%g%l%c%p%pef%AEF_OMON
    AEF_MOHX   => clm3%g%l%c%p%pef%AEF_MOHX
    AEF_ACET   => clm3%g%l%c%p%pef%AEF_ACET
    AEF_ETOH   => clm3%g%l%c%p%pef%AEF_ETOH
    AEF_CH2O   => clm3%g%l%c%p%pef%AEF_CH2O
    AEF_ALD2   => clm3%g%l%c%p%pef%AEF_ALD2
    AEF_FAXX   => clm3%g%l%c%p%pef%AEF_FAXX
    AEF_AAXX   => clm3%g%l%c%p%pef%AEF_AAXX
    AEF_C2H4   => clm3%g%l%c%p%pef%AEF_C2H4
    AEF_TOLU   => clm3%g%l%c%p%pef%AEF_TOLU
    AEF_HCNX   => clm3%g%l%c%p%pef%AEF_HCNX
    AEF_PRPE   => clm3%g%l%c%p%pef%AEF_PRPE
    AEF_FARN   => clm3%g%l%c%p%pef%AEF_FARN
    AEF_BCAR   => clm3%g%l%c%p%pef%AEF_BCAR
    AEF_OSQT   => clm3%g%l%c%p%pef%AEF_OSQT

    gptr       => clm3%g
    pgridcell  => clm3%g%l%c%p%gridcell
    ivt        => clm3%g%l%c%p%itype
!---------------------------------------
! begin to CALC_AEF 
!
      AEF_ISOP(:) = 0.0
      AEF_MBOX(:) = 0.0
      AEF_BPIN(:) = 0.0
      AEF_CARE(:) = 0.0
      AEF_LIMO(:) = 0.0
      AEF_OCIM(:) = 0.0
      AEF_SABI(:) = 0.0
      AEF_APIN(:) = 0.0
      AEF_MYRC(:) = 0.0
      AEF_OMON(:) = 0.0
      AEF_FARN(:) = 0.0
      AEF_BCAR(:) = 0.0
      AEF_OSQT(:) = 0.0
      AEF_MOHX(:) = 0.0
      AEF_ACET(:) = 0.0
      AEF_ETOH(:) = 0.0
      AEF_CH2O(:) = 0.0
      AEF_ALD2(:) = 0.0
      AEF_FAXX(:) = 0.0
      AEF_AAXX(:) = 0.0
      AEF_C2H4(:) = 0.0
      AEF_TOLU(:) = 0.0
      AEF_HCNX(:) = 0.0
      AEF_PRPE(:) = 0.0

      ! --------------------------------------------------------------------------------
      ! PFT-specific EFs from Table 2 in Guenther et al., 2012
      ! in ug compound/m2/h
      ! PFTs 1-15 in the table correspond to #2-16
      ! (i.e., excluding bare ground #1) in the above array.
      ! --------------------------------------------------------------------------------
      ! Compound Class EF1 EF2 EF3 EF4 EF5 EF6 EF7 EF8 EF9 EF10 EF11 EF12 EF13 EF14 EF15
      ! --------------------------------------------------------------------------------
      ! Other Monoterp 180 180 170 150 150 150 150 150 110 200  110  5    5    5    5
      ! Methanol       900 900 900 500 900 500 900 900 900 900  900  500  500  500  900
      ! Acetone        240 240 240 240 240 240 240 240 240 240  240  80   80   80   80
      ! Bidirect VOC   500 500 500 500 500 500 500 500 500 500  500  80   80   80   80
      ! Stress VOC     300 300 300 300 300 300 300 300 300 300  300  300  300  300  300
      ! Other VOC      140 140 140 140 140 140 140 140 140 140  140  140  140  140  140
      ! a-Pinene       500 500 510 600 400 600 400 400 200 300  200    2    2    2    2
      ! Myrcene         70  70  60  80  30  80  30  30  30  50   30  0.3  0.3  0.3  0.3
      ! a-Farnesene     40  40  40  60  40  60  40  40  40  40   40    3    3    3    4
      ! b-Carophyllene  80  80  80  60  40  60  40  40  50  50   50    1    1    1    4
      ! Other sesqt.   120 120 120 120 100 120 100 100 100 100  100    2    2    2    2
      ! --------------------------------------------------------------------------------

      ! One thing to note is these are net emissions to the canopy atmosphere
      ! but not net emissions to the above canopy atmosphere since they don't
      !  account for within-canopy deposition. Only an issue for OVOCs.

      PFT_EF_OMON = (/180.0_r8, 180.0_r8, 170.0_r8, 150.0_r8, 150.0_r8,      &
                      150.0_r8, 150.0_r8, 150.0_r8, 110.0_r8, 200.0_r8,      &
                      110.0_r8, 5.0_r8  , 5.0_r8  , 5.0_r8  , 5.0_r8/)

!zf      PFT_EF_OMON = (/180.0_r8, 180.0_r8, 170.0_r8, 150.0_r8, 150.0_r8,      &
!zf                      150.0_r8, 150.0_r8, 150.0_r8, 110.0_r8, 110.0_r8,      &
!zf                      110.0_r8, 5.0_r8  , 5.0_r8  , 5.0_r8  , 5.0_r8/)

      PFT_EF_MOHX = (/900.0_r8, 900.0_r8, 900.0_r8, 500.0_r8, 900.0_r8,      &
                      500.0_r8, 900.0_r8, 900.0_r8, 900.0_r8, 900.0_r8,      &
                      900.0_r8, 500.0_r8, 500.0_r8, 500.0_r8, 900.0_r8/)

      PFT_EF_ACET = (/240.0_r8, 240.0_r8, 240.0_r8, 240.0_r8, 240.0_r8,      &
                      240.0_r8, 240.0_r8, 240.0_r8, 240.0_r8, 240.0_r8,      &
                      240.0_r8, 80.0_r8 , 80.0_r8 , 80.0_r8 , 80.0_r8/)

!zf      PFT_EF_ACET = (/240.0_r8, 240.0_r8, 240.0_r8, 240.0_r8, 240.0_r8,      &
!zf                      240.0_r8, 240.0_r8, 240.0_r8,  90.0_r8,  90.0_r8,      &
!zf                       90.0_r8, 80.0_r8 , 80.0_r8 , 80.0_r8 , 80.0_r8/)

      PFT_EF_BIDR = (/500.0_r8, 500.0_r8, 500.0_r8, 500.0_r8, 500.0_r8,      &
                      500.0_r8, 500.0_r8, 500.0_r8, 500.0_r8, 500.0_r8,      &
                      500.0_r8, 80.0_r8 , 80.0_r8 , 80.0_r8 , 80.0_r8/)

      PFT_EF_STRS = (/300.0_r8, 300.0_r8, 300.0_r8, 300.0_r8, 300.0_r8,      &
                      300.0_r8, 300.0_r8, 300.0_r8, 300.0_r8, 300.0_r8,      &
                      300.0_r8, 300.0_r8, 300.0_r8, 300.0_r8, 300.0_r8/)

      PFT_EF_OTHR = (/140.0_r8, 140.0_r8, 140.0_r8, 140.0_r8, 140.0_r8,      &
                      140.0_r8, 140.0_r8, 140.0_r8, 140.0_r8, 140.0_r8,      &
                      140.0_r8, 140.0_r8, 140.0_r8, 140.0_r8, 140.0_r8/)

! ---> Now compute EFs for a-pinene and myrcene as well (dbm, 12/2012)


      PFT_EF_APIN = (/500.0_r8, 500.0_r8, 510.0_r8, 600.0_r8, 400.0_r8,      &
                      600.0_r8, 400.0_r8, 400.0_r8, 200.0_r8, 300.0_r8,      &
                      200.0_r8, 2.0_r8,   2.0_r8,   2.0_r8,   2.0_r8/)

!zf      PFT_EF_APIN = (/500.0_r8, 500.0_r8, 510.0_r8, 600.0_r8, 400.0_r8,      &
!zf                      600.0_r8, 400.0_r8, 400.0_r8, 200.0_r8, 200.0_r8,      &
!zf                      200.0_r8, 2.0_r8,   2.0_r8,   2.0_r8,   2.0_r8/)

      PFT_EF_MYRC = (/70.0_r8,  70.0_r8,  60.0_r8,  80.0_r8,  30.0_r8,       &
                      80.0_r8,  30.0_r8,  30.0_r8,  30.0_r8,  50.0_r8,       &
                      30.0_r8,  0.3_r8,   0.3_r8,   0.3_r8,   0.3_r8/)

!zf      PFT_EF_MYRC = (/70.0_r8,  70.0_r8,  60.0_r8,  80.0_r8,  30.0_r8,       &
!zf                      80.0_r8,  30.0_r8,  30.0_r8,  30.0_r8,  30.0_r8,       &
!zf                      30.0_r8,  0.3_r8,   0.3_r8,   0.3_r8,   0.3_r8/)

! <---

      PFT_EF_FARN = (/40.0_r8,  40.0_r8,  40.0_r8,  60.0_r8,  40.0_r8,       &
                      60.0_r8,  40.0_r8,  40.0_r8,  40.0_r8,  40.0_r8,       &
                      40.0_r8,  3.0_r8,   3.0_r8,   3.0_r8,   4.0_r8/)

      PFT_EF_BCAR = (/80.0_r8,  80.0_r8,  80.0_r8,  60.0_r8,  40.0_r8,       &
                      60.0_r8,  40.0_r8,  40.0_r8,  50.0_r8,  50.0_r8,       &
                      50.0_r8,  1.0_r8,   1.0_r8,   1.0_r8,   4.0_r8/)

      PFT_EF_OSQT = (/120.0_r8, 120.0_r8, 120.0_r8, 120.0_r8, 100.0_r8,      &
                      120.0_r8, 100.0_r8, 100.0_r8, 100.0_r8, 100.0_r8,      &
                      100.0_r8, 2.0_r8,   2.0_r8,   2.0_r8,   2.0_r8/)

      ! Other monoterpenes, methanol, acetone, MBO are each 100% of thier
      ! respective categories. The VOCs within the stress category each
      ! account for a specific fraction of emissions across all PFTs
      ! (ethene 58%, toluene 3%, HCN 1.5%). The VOCs within the
      ! other category also account for a given fraction of emissions
      ! across all PFTs (propene 48%, butene 24%, other alkenes 0.2%). But
      ! VOCs in the bidirectional category account for a different amount of
      ! the total flux for the different PFTs. So in this case we define a
      ! vector containing these fractions.

      ! Acetaldehyde: 40% of bidirectional category flux, except 25%
      ! for grasses and crops
      EM_FRAC_ALD2 = (/0.40_r8, 0.40_r8, 0.40_r8, 0.40_r8, 0.40_r8,      &
                       0.40_r8, 0.40_r8, 0.40_r8, 0.40_r8, 0.40_r8,      &
                       0.40_r8, 0.25_r8, 0.25_r8, 0.25_r8, 0.25_r8/)

      ! Ethanol: 40% of bidirectional category flux, except 25%
      ! for grasses and crops
      EM_FRAC_ETOH = (/0.40_r8, 0.40_r8, 0.40_r8, 0.40_r8, 0.40_r8,      &
                       0.40_r8, 0.40_r8, 0.40_r8, 0.40_r8, 0.40_r8,      &
                       0.40_r8, 0.25_r8, 0.25_r8, 0.25_r8, 0.25_r8/)

      ! Formic acid: 6% of bidirectional category flux, except 15%
      ! for grasses and crops
      EM_FRAC_FAXX = (/0.06_r8, 0.06_r8, 0.06_r8, 0.06_r8, 0.06_r8,      &
                       0.06_r8, 0.06_r8, 0.06_r8, 0.06_r8, 0.06_r8,      &
                       0.06_r8, 0.15_r8, 0.15_r8, 0.15_r8, 0.15_r8/)

      ! Acetic acid: 6% of bidirectional category flux, except 15%
      ! for grasses and crops
      EM_FRAC_AAXX = (/0.06_r8, 0.06_r8, 0.06_r8, 0.06_r8, 0.06_r8,      &
                       0.06_r8, 0.06_r8, 0.06_r8, 0.06_r8, 0.06_r8,      &
                       0.06_r8, 0.15_r8, 0.15_r8, 0.15_r8, 0.15_r8/)

      ! Formaldehyde: 8% of bidirectional category flux, except 20%
      ! for grasses and crops
      EM_FRAC_CH2O = (/0.08_r8, 0.08_r8, 0.08_r8, 0.08_r8, 0.08_r8,      &
                       0.08_r8, 0.08_r8, 0.08_r8, 0.08_r8, 0.08_r8,      &
                       0.08_r8, 0.20_r8, 0.20_r8, 0.20_r8, 0.20_r8/)

!-----------------------------------------------------------------------------
      ! Loop through plant types
    ! Initialize DGVM and reset DGVM time constant variables

    call get_proc_bounds(begg, endg, begl, endl, begc, endc, begp, endp)

    call readVOCdata()

!-----------------------------
       do p =  begp , endp
          g =  pgridcell(p)
          i =  gptr%ixy(g)
          j =  gptr%jxy(g)
          do m = 1, maxpatch_pft
#ifdef RICEWATER

             if (ivt(p) .eq. pft(i,j,m) .and. ivt(p).ge.1 .and. ivt(p).le.16) then
                if( ivt(p) == 16 ) then
                    type_veg = 15               
                else
                    type_veg = ivt(p)
                endif
#else
             if (ivt(p) .eq. pft(i,j,m) .and. ivt(p).ge.1 .and. ivt(p).le.15) then
                type_veg = ivt(p)
#endif
               AEF_ISOP(p) = AEF_ISOPRENE(i,j,1)
               AEF_MBOX(p) = AEF_MBO(i,j,1)
!??               AEF_APIN(p) = AEF_ALPHA_PINENE(i,j,1)
               AEF_BPIN(p) = AEF_BETA_PINENE(i,j,1)
               AEF_CARE(p) = AEF_CARENE(i,j,1)
               AEF_LIMO(p) = AEF_LIMONENE(i,j,1)
!??            AEF_MYRC(p) = AEF_MYRCENE(i,j,1)
               AEF_OCIM(p) = AEF_OCIMENE(i,j,1)
               AEF_SABI(p) = AEF_SABINENE(i,j,1)

               AEF_APIN(p) = AEF_APIN(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_APIN(type_veg)
               AEF_MYRC(p) = AEF_MYRC(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_MYRC(type_veg)
               AEF_OMON(p) = AEF_OMON(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_OMON(type_veg)
               AEF_FARN(p) = AEF_FARN(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_FARN(type_veg)
               AEF_BCAR(p) = AEF_BCAR(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_BCAR(type_veg)
               AEF_OSQT(p) = AEF_OSQT(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_OSQT(type_veg)
               AEF_MOHX(p) = AEF_MOHX(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_MOHX(type_veg)
               AEF_ACET(p) = AEF_ACET(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_ACET(type_veg)
               AEF_ETOH(p) = AEF_ETOH(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_BIDR(type_veg) * EM_FRAC_ETOH(type_veg)
               AEF_CH2O(p) = AEF_CH2O(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_BIDR(type_veg) * EM_FRAC_CH2O(type_veg)
               AEF_ALD2(p) = AEF_ALD2(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_BIDR(type_veg) * EM_FRAC_ALD2(type_veg)
               AEF_FAXX(p) = AEF_FAXX(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_BIDR(type_veg) * EM_FRAC_FAXX(type_veg)
               AEF_AAXX(p) = AEF_AAXX(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_BIDR(type_veg) * EM_FRAC_AAXX(type_veg)
               AEF_C2H4(p) = AEF_C2H4(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_STRS(type_veg) * 0.58_r8
               AEF_TOLU(p) = AEF_TOLU(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_STRS(type_veg) * 0.03_r8
               AEF_HCNX(p) = AEF_HCNX(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_STRS(type_veg) * 0.15_r8
               AEF_PRPE(p) = AEF_PRPE(p) + pctpft_lunit(i,j,m)*0.01 * PFT_EF_OTHR(type_veg) * 0.722_r8

!zf               AEF_ISOP(p) = AEF_ISOP(p)*2.0    !zftest
!zf               AEF_ACET(p) = AEF_ACET(p)*1.08   !zftest
!zf               AEF_C2H4(p) = AEF_C2H4(p)*1.8    !zftest
!zf               AEF_PRPE(p) = AEF_PRPE(p)*0.65   !zftest

!zf               AEF_APIN(p) = AEF_APIN(p)*1.45   !zftest
!zf               AEF_BPIN(p) = AEF_BPIN(p)*1.45   !zftest
!zf               AEF_LIMO(p) = AEF_LIMO(p)*1.45   !zftest
!zf               AEF_SABI(p) = AEF_SABI(p)*1.45   !zftest
!zf               AEF_MYRC(p) = AEF_MYRC(p)*1.45   !zftest
!zf               AEF_CARE(p) = AEF_CARE(p)*1.45   !zftest
!zf               AEF_OCIM(p) = AEF_OCIM(p)*1.45   !zftest
!zf               AEF_OMON(p) = AEF_OMON(p)*1.45   !zftest
             end if
          end do
       end do

!--------------------------------------------------------------------------------------
      ! Conversion factor from [ug compound/m2/hr] to [kg compound/m2/s]
      FACTOR = 1.0e-9_r8 / 3600.0_r8

      ! Loop over grid boxes

    do p =  begp , endp
       g = pgridcell(p)

         ! Convert AEF arrays to [kgC/m2/s]
         ! Multiply arrays by FACTOR and ratio [g C/g compound]
         ! NOTE: AEFs for ISOP, MBOX, BPIN, CARE, LIMO, OCIM, SABI
         ! are read from file in [kgC/m2/s], so no need to convert here
            AEF_APIN(p) = AEF_APIN(p) * FACTOR * 120.0_r8 / 136.234_r8
            AEF_MYRC(p) = AEF_MYRC(p) * FACTOR * 120.0_r8 / 136.234_r8
            AEF_OMON(p) = AEF_OMON(p) * FACTOR * 120.0_r8 / 136.234_r8

            ! Sesquiterpenes
            SPECIES2CARBON = 15._r8 * 12.01_r8 / ( 15._r8 * 12.01_r8 + 24._r8 * 1.01_r8 )
            AEF_FARN(p) = AEF_FARN(p) * FACTOR * SPECIES2CARBON
            AEF_BCAR(p) = AEF_BCAR(p) * FACTOR * SPECIES2CARBON
            AEF_OSQT(p) = AEF_OSQT(p) * FACTOR * SPECIES2CARBON

         AEF_ACET(p) = AEF_ACET(p) * FACTOR *  36.0_r8 /  58.079_r8
         AEF_ETOH(p) = AEF_ETOH(p) * FACTOR *  24.0_r8 /  46.068_r8
         AEF_ALD2(p) = AEF_ALD2(p) * FACTOR *  24.0_r8 /  44.053_r8
         AEF_C2H4(p) = AEF_C2H4(p) * FACTOR *  24.0_r8 /  28.053_r8
         AEF_TOLU(p) = AEF_TOLU(p) * FACTOR *  84.0_r8 /  92.138_r8
         AEF_PRPE(p) = AEF_PRPE(p) * FACTOR *  36.0_r8 /  42.080_r8

         ! Methanol, formaldehyde, formic acid, acetic acid, HCN are
         ! carried in kg, not kg C
         ! Convert AEF arrays to [kg/m2/s]
         AEF_MOHX(p) = AEF_MOHX(p) * FACTOR
         AEF_CH2O(p) = AEF_CH2O(p) * FACTOR
         AEF_FAXX(p) = AEF_FAXX(p) * FACTOR
         AEF_AAXX(p) = AEF_AAXX(p) * FACTOR
         AEF_HCNX(p) = AEF_HCNX(p) * FACTOR

   end do 

! end CALC_AEF
!------------------------------------------
! Calculate normalization factor

      ! -----------------
      ! GAMMA_P for standard conditions
      ! -----------------
      ! Based on Eq. 11b from Guenther et al., 2006
      ! Using standard conditions of phi = 0.6, solar angle of 60 deg,
      ! and P_daily = 400
      ! Note corrigendum for Eq. 11b in that paper, should be a
      ! minus sign before the 0.9.

      PAC_DAILY = 400.0_r8
      PHI       = 0.6_r8
      BBB       = 1.0_r8 + 0.0005_r8 *( PAC_DAILY - 400.0_r8 )
      AAA       = ( 2.46_r8 * BBB * PHI ) - ( 0.9_r8 * PHI**2 )
      ! sin(60) = 0.866
      GAMMA_P_STANDARD = 0.866_r8 * AAA

      ! -----------------
      ! GAMMA_T_LI for standard conditions
      ! -----------------
      ! gamma_t_li = EXP( Beta * ( T - T_Standard ) )
      ! This is 1.0 for T = T_Standard
      GAMMA_T_LI_STANDARD = 1.0_r8

      ! -----------------
      ! GAMMA_SM for standard conditions
      ! -----------------
      ! Standard condition is soil moisture = 0.3 m^3/m^3
      ! GAMMA_SM = 1.0 for all compounds under this condition
      GAMMA_SM_STANDARD = 1.0_r8

      ! -----------------
      ! GAMMA_LAI for standard conditions
      ! -----------------
      ! Standard condition is LAI = 5
      CMLAI = 5.0_r8
      GAMMA_LAI_STANDARD = 0.49_r8 * CMLAI / SQRT( 1.0_r8 + 0.2_r8 * CMLAI*CMLAI )

      ! -----------------
      ! GAMMA_AGE for standard conditions
      ! -----------------
      ! Standard condition is 0% new, 10% growing, 80% mature, 10% old foliage
      ! Isoprene uses A_NEW = 0.05_r8, A_GRO = 0.6_r8, A_MAT = 1._r8, A_OLD = 0.9_r8
      GAMMA_AGE_STANDARD = 0.1_r8*0.6_r8 + 0.8_r8*1.0_r8 + 0.1_r8*0.9_r8

      ! -----------------
      ! GAMMA_T_LD for standard conditions
      ! -----------------
      ! Standard condition is
      ! PT_15 = average leaf temp over past 24-240 hours = 297K
      ! T = air temperature = 303K
      PT_15 = 297.0_r8
      T     = 303.0_r8
      R     = 8.314e-3_r8
      ! parameters for isoprene
      CEO = 2.0_r8
      CT1 = 95.0_r8

      E_OPT = CEO * EXP( 0.08_r8 * ( PT_15  - 2.97e2_r8 ) )
      T_OPT = 3.13e2_r8 + ( 6.0e-1_r8 * ( PT_15 - 2.97e2_r8 ) )
      CT2   = 200.0_r8

      ! Variable related to temperature
      X     = ( 1.0_r8/T_OPT - 1.0_r8/T ) / R

      GAMMA_T_LD_STANDARD = E_OPT * CT2 * EXP( CT1 * X ) /        &
           ( CT2 - CT1 * ( 1.0_r8 - EXP( CT2 * X ) ) )

      ! -----------------
      ! Overall GAMMA_STANDARD
      ! -----------------
      ! LDF = 1_r8 for isoprene
      LDF = 1.0_r8
      GAMMA_STANDARD =                                                         &
           GAMMA_AGE_STANDARD * GAMMA_SM_STANDARD * GAMMA_LAI_STANDARD         &
           * ((1.0_r8 - LDF) * GAMMA_T_LI_STANDARD                             &
           + (LDF * GAMMA_P_STANDARD * GAMMA_T_LD_STANDARD))
      ! This ends up being 1.0101081.

      NORM_FAC = 1.0_r8 / GAMMA_STANDARD
!--------------------------------
! Set to 1 day since we know that LAI is updated every day.

      DAYS_BTW_M = 1

      return
  end subroutine initialize_VOC 

end module VOCemiss_initialize

#endif
