#include <misc.h>
#include <preproc.h>
 
#if (defined NIE_BLDAS_LST) 


module EnKFLST1DMod

 implicit none
  save
 
!
! !PUBLIC MEMBER FUNCTIONS:
 
  public :: EnKFLST1D
! !REVISION HISTORY:
! Created by Suping Nie
 
contains

subroutine EnKFLST1D(c,obs_in)  
!---------------------------------------------------------------
    use shr_kind_mod, only: r8 => shr_kind_r8
    use time_manager, only : get_step_size, get_nstep, get_curr_date  

    use CompMod
    use clmtype
    use clm_varpar    , only :  nrens
    use clm_varcon    , only :  max_tgd, min_tgd   
    use CompMod
    use clm_varcon, only : denice, denh2o, wimp
    use abortutils          , only : endrun

!  ----------------------------------------------------------------------
    implicit none
    integer , parameter   :: nrobs=1         
    integer , parameter   :: ndim=1          
    integer , intent(in)  :: c               

!  ----------------------------------------------------------------------
    real(r8), intent(in)    :: obs_in              
    real(r8), pointer :: t_grnd(:)              !ground surface temperature [K]
    real(r8), pointer :: en_t_grnd(:,:,:)       ! ensemble ground temperature (Kelvin)  

    real(r8)::AA(ndim,nrens),X_b(ndim),X_A(ndim) 
    real(r8),allocatable :: Psi(:,:),Psi_A(:,:),Psi_B(:,:),Psi_tem(:,:),HPsi_tran(:,:)
    real(r8),allocatable :: Psi_tran(:,:),Tem(:,:),TEM1(:,:),alpha(:),HPsi(:,:)
    real(r8),allocatable :: HXb(:,:)
    real(r8),allocatable :: FFT(:,:)

    integer::tru_order,nren_d
    real(r8)::H(nrobs,ndim),R(nrobs,nrobs),B(ndim,ndim),EPS
    real(r8)::TT(nrens,nrens)
    real(r8)::V(nrens,nrens),VV(nrens),PA,FF(ndim,nrens)
    real(r8)::YY_b(ndim)
    integer::i,j,s,k,L
!----------------------------------------------------------------------------------


!******************************************************************************************************!

    t_grnd          => clm3%g%l%c%ces%t_grnd
    en_t_grnd       => clm3%g%l%c%ces%en_t_grnd
    nren_d          =  nrens-1

    X_b     =  t_grnd(c)
    AA(:,:) =  en_t_grnd(c,:,:) 
    X_a   = 0.
    R=0.0
    H=0.0

    do i=1,1
      R(i,i)=1.0/(0.0001)**2.0 
      H(i,i)=1.0
    end do

    do i=1,nrens
      AA(:,i)=AA(:,i)-X_b(:) 
    end do

    do i=1,nrens
      do j=1,nrens
         TT(i,j)=0.0
         do k=1,ndim
           TT(i,j)=TT(i,j)+AA(k,i)*AA(k,j)  
         end do
      end do
    enddo

    EPS=0.00000001D01
    V =0.	
    call CJCBI(TT,nrens,EPS,V,L) 
    do i=1,nrens-1
	    s=i
        do j=i+1, nrens
		   if(TT(j,j)>TT(s,s))s=j
		end do
		PA=TT(i,i)
		TT(i,i)=TT(s,s)
		TT(s,s)=PA
		VV(:)=V(:,i) 
		V(:,i)=V(:,s)
		V(:,s)=VV(:)	  	  
    end do

    FF = 0.
    call BRMUL(AA,V,ndim,nrens,nrens,FF) 
	tru_order=nrens-nren_d
    do i=1,tru_order
      FF(:,i)=FF(:,i)/sqrt(TT(i,i))
	  AA(:,i)=FF(:,i)+X_b(:)
    end do

    allocate(FFT(ndim,tru_order))
    FFT = 0. 
    do i=1,tru_order
      FFT(:,i)=FF(:,i)
    end do

    allocate(Psi(ndim,tru_order))         
    allocate(Psi_tran(tru_order,ndim))   
    allocate(Psi_A(tru_order,tru_order)) 
    allocate(Psi_tem(tru_order,nrobs))
    allocate(TEM(tru_order,tru_order))
    allocate(HPsi(nrobs,tru_order))
    allocate(HXb(nrobs,1))
    allocate(HPsi_tran(tru_order,nrobs))
    allocate(Psi_B(tru_order,1))	       
    allocate(TEM1(nrobs,1)) 

    Psi = 0.
    Psi_tran = 0.
    Psi_A = 0.
    Psi_tem = 0.
    TEM = 0.
    HPsi = 0.
    HXb = 0.
    HPsi_tran = 0.
    Psi_B = 0.
    TEM1 = 0.

	Psi_A=0.0
	do i=1,tru_order
	  Psi_A(i,i)=1.0/TT(i,i)*(nrens-1)
	end do      

	YY_b(1)=obs_in 
    YY_b(2:ndim)=X_b(2:ndim)
    Psi_B=0.0
    Psi(1:ndim,:)=AA(1:ndim,:) 
    HPsi = 0.
    call BRMUL(H,Psi,nrobs,ndim,tru_order,HPsi) 
    call BRMUL(H,X_b(1:ndim),nrobs,ndim,1,HXb)  
	do i=1,tru_order
      HPsi(:,i)=HPsi(:,i)-HXb(:,1)           
	end do

	call TRANS_MATRIX(HPsi,HPsi_tran,nrobs,tru_order)	      
	call BRMUL(HPsi_tran,R,tru_order,nrobs,nrobs,Psi_tem)   
	call BRMUL(Psi_tem,HPsi,tru_order,nrobs,tru_order,TEM)  
	Psi_A=Psi_A+TEM                                         
	call BRMUL(H,X_b(1:ndim),nrobs,ndim,1,HXb)
    TEM1(1:nrobs,1)=YY_b(:)-HXb(1:nrobs,1)     
    call BRMUL(Psi_tem,TEM1,tru_order,nrobs,1,TEM(1:tru_order,1))  
    Psi_B(1:tru_order,1)=Psi_B(1:tru_order,1)+TEM(1:tru_order,1)
    deallocate(HPsi,HPsi_tran,Tem1)
    deallocate(Psi_tem,Psi,Psi_tran,HXb)
    deallocate(TEM) 

    allocate (alpha(tru_order))
    alpha = 0.
    call  AGAUS(Psi_A,Psi_b,tru_order,alpha,L)

    if(L==0)then         
      X_a(:)=X_b(:)+0
    else	            
      call BRMUL(FFT,alpha,ndim,tru_order,1,X_a)  
      X_a = X_a + X_b 
    endif
    deallocate(FFT)         
    deallocate(alpha,Psi_A,Psi_B)          

    t_grnd(c)=  X_a(1)  
    if(t_grnd(c).gt.max_tgd) t_grnd(c) = max_tgd
    if(t_grnd(c).lt.min_tgd) t_grnd(c) = min_tgd

!******************************************************************************************************!

end subroutine EnKFLST1D
end module EnKFLST1DMod

#endif
