#include <misc.h>
#include <preproc.h>

module initialize2Mod

#if(defined LANDUSE)||(defined RICEWATER)
!-----------------------------------------------------------------------
!BOP
!
! !MODULE: initialize2Mod
!
! !DESCRIPTION:
! Performs land model initialization for rice or landuse
!
! !PUBLIC TYPES:
  implicit none
  save
!
! !PUBLIC MEMBER FUNCTIONS:
  public :: initialize2
!
! !REVISION HISTORY:
! Created by zhangyw
!
!EOP
!
!-----------------------------------------------------------------------

contains

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: initialize2
!
! !INTERFACE:
  subroutine initialize2
!
! !DESCRIPTION:
! Land model initialization.
!
! !USES:
    use shr_kind_mod    , only : r8 => shr_kind_r8
    use initGridCellsMod, only : initGridCells,initGridCells2
    use clm_varpar      , only : lsmlon, lsmlat, maxpatch
#if (defined RICEWATER)
    use surfFileMod     , only : surfrdRicewater
#endif
#if (defined LANDUSE)
    use surfFileMod     , only : surfrdLUC
#endif
!
! !ARGUMENTS:
    implicit none
!
! !CALLED FROM:
! Created by zhangyw 2017.04.10
!
!EOP
!
! !LOCAL VARIABLES:
    integer  :: vegxy(lsmlon,lsmlat,maxpatch) !vegetation type
    real(r8) :: wtxy(lsmlon,lsmlat,maxpatch)  !subgrid weights
!-----------------------------------------------------------------------
#if (defined LANDUSE)
  call surfrdLUC (vegxy, wtxy)
#endif
#if (defined RICEWATER)
  call surfrdRicewater (vegxy, wtxy)
#endif

  call initGridCells2(vegxy, wtxy)

  end subroutine initialize2

#endif
end module initialize2Mod
