#include <misc.h>
#include <preproc.h>
#if ( defined SCAM )
#include <max.h>
#endif

module pftvarcon

!-----------------------------------------------------------------------
!BOP
!
! !MODULE: pftvarcon
!
! !DESCRIPTION:
! Module containing vegetation constants and method to
! eads and initialize vegetation (PFT) constants.
!
! !USES:
  use shr_kind_mod, only : r8 => shr_kind_r8
  use abortutils  , only : endrun
  use clm_varpar
#if ( defined SCAM )
  use scamMod, only :lsmpftfile
#endif
!
! !PUBLIC TYPES:
  implicit none
  save
!
! Vegetation type constants
!
  character(len=40) pftname(0:numpft) !PFT description

  integer ncorn                  ! value for corn
  integer nwheat                 ! value for wheat
  integer noveg                  ! value for not vegetated
  integer ntree                  ! value for last type of tree

  real(r8) dleaf(0:numpft)       ! characteristic leaf dimension (m)
  real(r8) c3psn(0:numpft)       ! photosynthetic pathway: 0. = c4, 1. = c3
  real(r8) vcmx25(0:numpft)      ! max rate of carboxylation at 25C (umol CO2/m**2/s)
  real(r8) mp(0:numpft)          ! slope of conductance-to-photosynthesis relationship
  real(r8) qe25(0:numpft)        ! quantum efficiency at 25C (umol CO2 / umol photon)
  real(r8) xl(0:numpft)          ! leaf/stem orientation index
  real(r8) rhol(0:numpft,numrad) ! leaf reflectance: 1=vis, 2=nir
  real(r8) rhos(0:numpft,numrad) ! stem reflectance: 1=vis, 2=nir
  real(r8) taul(0:numpft,numrad) ! leaf transmittance: 1=vis, 2=nir
  real(r8) taus(0:numpft,numrad) ! stem transmittance: 1=vis, 2=nir
  real(r8) z0mr(0:numpft)        ! ratio of momentum roughness length to canopy top height (-)
  real(r8) displar(0:numpft)     ! ratio of displacement height to canopy top height (-)
  real(r8) roota_par(0:numpft)   ! CLM rooting distribution parameter [1/m]
  real(r8) rootb_par(0:numpft)   ! CLM rooting distribution parameter [1/m]
  real(r8) crop(0:numpft)        ! crop pft: 0. = not crop, 1. = crop pft

  real(r8) tmax(0:numpft)        ! 
  real(r8) tmin(0:numpft)        ! 
  real(r8) topt(0:numpft)        ! 
!zhangyw 20071219
#if (defined AVIM )
  real(r8) bw(0:numpft)          ! 
  real(r8) vmax(0:numpft)        ! 
  real(r8) crg1(0:numpft)
  real(r8) crg2(0:numpft)
  real(r8) crg3(0:numpft)
  real(r8) crg4(0:numpft)
  real(r8) crm1(0:numpft)
  real(r8) crm2(0:numpft)
  real(r8) crm3(0:numpft)
  real(r8) crm4(0:numpft)
#if (defined PHENO2)
  real(r8) smpso(0:numpft)
  real(r8) smpsc(0:numpft)
#endif     !! end of PHENO2
#endif     !! end of AVIM  

  real(r8) sla(0:numpft)              ! specific leaf area [m2 leaf g-1 carbon]
  real(r8) pftpar(0:numpft,1:npftpar) ! the rest for use with DGVM
  real(r8) lm_sapl(0:numpft)
  real(r8) sm_sapl(0:numpft)
  real(r8) hm_sapl(0:numpft)
  real(r8) rm_sapl(0:numpft)
  logical  tree(0:numpft)
  logical  summergreen(0:numpft)
  logical  raingreen(0:numpft)

  real(r8), parameter :: reinickerp = 1.6 !parameter in allometric equation
  real(r8), parameter :: wooddens = 2.0e5 !wood density (gC/m3)
  real(r8), parameter :: latosa = 8.0e3   !ratio of leaf area to sapwood cross-sectional
                                          !area (Shinozaki et al 1964a,b)
  real(r8), parameter :: allom1 = 100.0   !parameters in allometric
  real(r8), parameter :: allom2 =  40.0
  real(r8), parameter :: allom3 =   0.5
!
! !PUBLIC MEMBER FUNCTIONS:
  public :: pftconrd ! Read and initialize vegetation (PFT) constants
!
! !REVISION HISTORY:
! Created by Sam Levis (put into module form by Mariana Vertenstein)
!
!! liwp 2016-12-29 added RICEWATER selection
#if (defined PHENO2)
#if (defined RICEWATER)
!! liwp 2016-01-04 move PFT contanst data reading from inputdata to the following lines
  data z0mr   (1:17)/0.055, 0.055, 0.055, 0.075, 0.075, 0.055, 0.055, 0.055,  &
                     0.120, 0.120, 0.120, 0.120, 0.120, 0.120, 0.120, 0.120, 0.120/
  data displar(1:17)/0.67, 0.67, 0.67, 0.67, 0.67, 0.67, 0.67, 0.67, &
                     0.68, 0.68, 0.68, 0.68, 0.68, 0.68, 0.68, 0.68,0.68/
  data dleaf  (1:17)/0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, &
                     0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04,0.04/
  data c3psn  (1:17)/1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, &
                     1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 1.0, 1.0, 1.0 /
#if (defined OFFLINE)
  data vcmx25 (1:17)/50. ,50. ,45. ,45. , 45. ,42. ,46. ,46. , &
                     48. ,45. ,40. ,42. , 45. ,30. ,72. ,70. ,70. /
#endif
!! liwp 2017-03-06 increase vcmx25 to increase PHT, NPP etc. in coupled models when rainfall is small
#if (defined COUP_CSM)  || (defined COUP_CAM)
  data vcmx25 (1:17)/52. ,56. ,45. ,49. , 45. ,45. ,45. ,48. , &
                     49. ,48. ,40. ,43. , 46. ,30. ,72. ,70. ,70. /
#endif
  data mp     (1:17)/9. , 9. , 9. , 9. , 9. , 9. , 9. , 9. , &
                     9. , 9. , 9. , 9. , 9. , 5. , 9. , 9. ,9. /
  data qe25(1:17)/0.06, 0.06, 0.06, 0.06, 0.06, 0.06, 0.06, 0.06, &
                  0.06, 0.06, 0.06, 0.06, 0.06, 0.04, 0.06, 0.06,0.06 /
  data rhol(1:17,1)/0.07, 0.07, 0.07, 0.10, 0.10, 0.10, 0.10, 0.10, &
                    0.07, 0.10, 0.10, 0.11, 0.11, 0.11, 0.11, 0.11,0.11 /
  data rhol(1:17,2)/0.35, 0.35, 0.35, 0.45, 0.45, 0.45, 0.45, 0.45, &
                    0.35, 0.45, 0.45, 0.35, 0.35, 0.35, 0.35, 0.35,0.35 /
  data rhos(1:17,1)/0.16, 0.16, 0.16, 0.16, 0.16, 0.16, 0.16, 0.16, &
                    0.16, 0.16, 0.16, 0.31, 0.31, 0.31, 0.31, 0.31,0.31 /
  data rhos(1:17,2)/0.39, 0.39, 0.39, 0.39, 0.39, 0.39, 0.39, 0.39, &
                    0.39, 0.39, 0.39, 0.53, 0.53, 0.53, 0.53, 0.53,0.53 /
  data taul(1:17,1)/0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, &
                    0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,0.05 /
  data taul(1:17,2)/0.10, 0.10, 0.10, 0.25, 0.25, 0.25, 0.25, 0.25, &
                    0.10, 0.25, 0.25, 0.34, 0.34, 0.34, 0.34, 0.34,0.34 /
  data taus(1:17,1)/0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001, &
                    0.001,0.001,0.001,0.120,0.120,0.120,0.120,0.120,0.120/
  data taus(1:17,2)/0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001, &
                    0.001,0.001,0.001,0.250,0.250,0.250,0.250,0.250,0.250/
  data xl  (1:17)  /0.01, 0.01, 0.01, 0.10, 0.10, 0.01, 0.25, 0.25, &
                    0.01, 0.25, 0.25,-0.30,-0.30,-0.30,-0.30,-0.30,-0.30 /
  data roota_par(1:17)/7.0, 7.0, 7.0, 7.0, 7.0, 6.0, 6.0, 6.0,  &
                       7.0, 7.0, 7.0,11.0,11.0,11.0, 6.0, 6.0,6.0 /
  data rootb_par(1:17)/2.0, 2.0, 2.0, 1.0, 1.0, 2.0, 2.0, 2.0,  &
                       1.5, 1.5, 1.5, 2.0, 2.0, 2.0, 3.0, 3.0,3.0 /
  data tmin(1:17)/273., 270., 270., 279., 272., 278., 270., 271., &
                  273., 270., 272., 270., 270., 278., 273., 272.,272. /
  data topt(1:17)/299., 295., 296., 303., 295., 303., 301., 293., &
                  293., 310., 294., 294., 298., 304., 305., 295.,295. /
  data tmax(1:17)/314., 309., 310., 320., 315., 321., 318., 313., &
                  310., 326., 310., 310., 316., 323., 323., 316.,316. /
  data smpso(1:17)/-66000,-66000,-66000,-66000,-66000,-45000,-45000,-35000, &
                   -83000,-83000,-83000,-74000,-74000,-74000,-74000,-74000, -74000 /
  data smpsc(1:17)/-255000,-255000,-255000,-255000,-255000,         &
                   -368000,-254000,-254000,-438000,-528000,-458000, &
                   -285000,-275000,-275000,-285000,-275000,-275000 /
#endif

#ifndef RICEWATER    !! no RICEWATER 
  data z0mr   (1:16)/0.055, 0.055, 0.055, 0.075, 0.075, 0.055, 0.055, 0.055,  & 
                     0.120, 0.120, 0.120, 0.120, 0.120, 0.120, 0.120, 0.120/ 
  data displar(1:16)/0.67, 0.67, 0.67, 0.67, 0.67, 0.67, 0.67, 0.67, &
                     0.68, 0.68, 0.68, 0.68, 0.68, 0.68, 0.68, 0.68 /
  data dleaf  (1:16)/0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, &
                     0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04 /
  data c3psn  (1:16)/1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, &
                     1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 1.0, 1.0 /
#if (defined OFFLINE)
  data vcmx25 (1:16)/50. ,50. ,45. ,45. , 45. ,42. ,46. ,46. , &
                     48. ,45. ,40. ,42. , 45. ,30. ,72. ,70. /
#endif
#if (defined COUP_CSM)  || (defined COUP_CAM)
  data vcmx25 (1:16)/52. ,56. ,45. ,49. , 45. ,42. ,46. ,48. , &
                     49. ,48. ,40. ,43. , 46. ,30. ,72. ,70. /
#endif
  data mp     (1:16)/9. , 9. , 9. , 9. , 9. , 9. , 9. , 9. , &
                     9. , 9. , 9. , 9. , 9. , 5. , 9. , 9. /
  data qe25(1:16)/0.06, 0.06, 0.06, 0.06, 0.06, 0.06, 0.06, 0.06, &
                  0.06, 0.06, 0.06, 0.06, 0.06, 0.04, 0.06, 0.06 /
  data rhol(1:16,1)/0.07, 0.07, 0.07, 0.10, 0.10, 0.10, 0.10, 0.10, &
                    0.07, 0.10, 0.10, 0.11, 0.11, 0.11, 0.11, 0.11 /
  data rhol(1:16,2)/0.35, 0.35, 0.35, 0.45, 0.45, 0.45, 0.45, 0.45, &
                    0.35, 0.45, 0.45, 0.35, 0.35, 0.35, 0.35, 0.35 /
  data rhos(1:16,1)/0.16, 0.16, 0.16, 0.16, 0.16, 0.16, 0.16, 0.16, &
                    0.16, 0.16, 0.16, 0.31, 0.31, 0.31, 0.31, 0.31 /
  data rhos(1:16,2)/0.39, 0.39, 0.39, 0.39, 0.39, 0.39, 0.39, 0.39, &
                    0.39, 0.39, 0.39, 0.53, 0.53, 0.53, 0.53, 0.53 /
  data taul(1:16,1)/0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, &
                    0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05 /
  data taul(1:16,2)/0.10, 0.10, 0.10, 0.25, 0.25, 0.25, 0.25, 0.25, &
                    0.10, 0.25, 0.25, 0.34, 0.34, 0.34, 0.34, 0.34 /
  data taus(1:16,1)/0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001, &
                    0.001,0.001,0.001,0.120,0.120,0.120,0.120,0.120 /
  data taus(1:16,2)/0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001, &
                    0.001,0.001,0.001,0.250,0.250,0.250,0.250,0.250 /
  data xl  (1:16)  /0.01, 0.01, 0.01, 0.10, 0.10, 0.01, 0.25, 0.25, &
                    0.01, 0.25, 0.25,-0.30,-0.30,-0.30,-0.30,-0.30 /
  data roota_par(1:16)/7.0, 7.0, 7.0, 7.0, 7.0, 6.0, 6.0, 6.0,  &
                       7.0, 7.0, 7.0,11.0,11.0,11.0, 6.0, 6.0/
  data rootb_par(1:16)/2.0, 2.0, 2.0, 1.0, 1.0, 2.0, 2.0, 2.0,  &
                       1.5, 1.5, 1.5, 2.0, 2.0, 2.0, 3.0, 3.0/
  data tmin(1:16)/273., 270., 270., 279., 272., 278., 270., 271., &
                  273., 270., 272., 270., 270., 278., 273., 272. /
  data topt(1:16)/299., 295., 296., 303., 295., 303., 301., 293., &
                  293., 310., 294., 294., 298., 304., 305., 295. /
  data tmax(1:16)/314., 309., 310., 320., 315., 321., 318., 313., &
                  310., 326., 310., 310., 316., 323., 323., 316. /
  data smpso(1:16)/-66000,-66000,-66000,-66000,-66000,-45000,-45000,-35000, &
                   -83000,-83000,-83000,-74000,-74000,-74000,-74000,-74000 /
  data smpsc(1:16)/-255000,-255000,-255000,-255000,-255000,         &
                   -368000,-254000,-254000,-438000,-528000,-458000, &
                   -285000,-275000,-275000,-285000,-275000 /

#endif       !! end of RICEWATER
#endif       !! end of PHENO2

#if (defined CLM3)
   data z0mr(1:16)/0.055,0.055,0.055,0.075,0.075,0.055,0.055,0.055,0.12,0.12,0.12,0.12,0.12,0.12,0.12,0.12/
   data displar(1:16)/0.67,0.67,0.67,0.67,0.67,0.67,0.67,0.67,0.68,0.68,0.68,0.68,0.68,0.68,0.68,0.68/
   data dleaf(1:16)/0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04/
   data c3psn(1:16)/1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,0.,1.,1./
   data vcmx25(1:16)/50.,55.,70.,90.,70.,65.,65.,65.,35.,50.,50.,70.,55.,40.,75.,50./
   data mp(1:16)/6.,6.,6.,9.,9.,9.,9.,9.,9.,9.,9.,9.,9.,5.,9.,9./
   data qe25(1:16)/0.06,0.06,0.06,0.06,0.06,0.06,0.06,0.06,0.06,0.06,0.06,0.06,0.06,0.04,0.06,0.06/
   data rhol(1:16,1)/0.07,0.07,0.07,0.1,0.1,0.1,0.1,0.1,0.07,0.1,0.1,0.11,0.11,0.11,0.11,0.11/
   data rhol(1:16,2)/0.35,0.35,0.35,0.45,0.45,0.45,0.45,0.45,0.35,0.45,0.45,0.58,0.58,0.58,0.58,0.58/
   data rhos(1:16,1)/0.16,0.16,0.16,0.16,0.16,0.16,0.16,0.16,0.16,0.16,0.16,0.36,0.36,0.36,0.36,0.36/
   data rhos(1:16,2)/0.39,0.39,0.39,0.39,0.39,0.39,0.39,0.39,0.39,0.39,0.39,0.58,0.58,0.58,0.58,0.58/
   data taul(1:16,1)/0.05,0.05,0.05,0.05,0.05,0.05,0.05,0.05,0.05,0.05,0.05,0.07,0.07,0.07,0.07,0.07/
   data taul(1:16,2)/0.1,0.1,0.1,0.25,0.25,0.25,0.25,0.25,0.1,0.25,0.25,0.25,0.25,0.25,0.25,0.25/
   data taus(1:16,1)/0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.22,0.22,0.22,0.22,0.22/
   data taus(1:16,2)/0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.001,0.38,0.38,0.38,0.38,0.38/
   data xl(1:16)/0.01,0.01,0.01,0.1,0.1,0.01,0.25,0.25,0.01,0.25,0.25,-0.3,-0.3,-0.3,-0.3,-0.3/
   data roota_par(1:16)/7.,7.,7.,7.,7.,6.,6.,6.,7.,7.,7.,11.,11.,11.,6.,6./
   data rootb_par(1:16)/2.,2.,2.,1.,1.,2.,2.,2.,1.5,1.5,1.5,2.,2.,2.,3.,3./
#endif       !! end of CLM3
!
!EOP
!-----------------------------------------------------------------------

contains

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: pftconrd
!
! !INTERFACE:
  subroutine pftconrd
!
! !DESCRIPTION:
! Read and initialize vegetation (PFT) constants
!
! !USES:
    use fileutils , only : opnfil, getfil, relavu, getavu
    use clm_varctl, only : fpftcon
#ifdef SPMD
    use spmdMod   , only : masterproc, mpicom, MPI_REAL8
#else
    use spmdMod   , only : masterproc
#endif
!
! !ARGUMENTS:
    implicit none
!
! !CALLED FROM:
! routine initialize in module initializeMod
!
! !REVISION HISTORY:
! Created by Gordon Bonan
!
!EOP
!
! !LOCAL VARIABLES:
    character(len=256) :: locfn ! local file name
    integer :: i,n              ! loop indices
    integer :: ier              ! error code
!-----------------------------------------------------------------------

    ! Set specific vegetation type values
#if (defined AVIM)
    ncorn  = 15
    nwheat = 16
    ntree  =  8  ! value for last type of tree
    noveg  =  0  ! value for non-vegetated
#else
    ncorn  = 15
    nwheat = 16
    ntree  =  8  ! value for last type of tree
    noveg  =  0  ! value for non-vegetated
#endif

    ! Assign unit number to file. Get local file.
    ! Open file and read PFT's.
    ! Close and release file.

#ifndef PHENO2    !! for PHENO1 or CLM3
    if (masterproc) then
       write (6,*) 'Attempting to read PFT physiological data .....'
       n = getavu()
#if ( defined SCAM )
       call getfil (lsmpftfile, locfn, 0)
#else
       call getfil (fpftcon, locfn, 0)
#endif
       call opnfil (locfn, n, 'f')
       do i = 1, numpft
#if (defined PHENO1 )
          read (n,*,iostat=ier)  pftname(i), &
               z0mr(i)   , displar(i), dleaf(i)  , c3psn(i)  , &
               vcmx25(i) , mp(i)     , qe25(i)   , rhol(i,1) , &
               rhol(i,2) , rhos(i,1) , rhos(i,2) , taul(i,1) , &
               taul(i,2) , taus(i,1) , taus(i,2) , xl(i)     , &
               roota_par(i), rootb_par(i)                    , &
               tmax(i), tmin(i), topt(i), bw(i)  , vmax(i)   , &
               crg1(i), crg2(i), crg3(i), crg4(i), &
               crm1(i), crm2(i), crm3(i), crm4(i)
          if (ier /= 0) then
             write(6,*)'pftconrd: error in reading in pft data'
             call endrun()
          end if
#endif
!#if (defined PHENO2 )
!          read (n,*,iostat=ier)  pftname(i), &
!               z0mr(i)   , displar(i), dleaf(i)  , c3psn(i)  , &
!               vcmx25(i) , mp(i)     , qe25(i)   , rhol(i,1) , &
!               rhol(i,2) , rhos(i,1) , rhos(i,2) , taul(i,1) , &
!               taul(i,2) , taus(i,1) , taus(i,2) , xl(i)     , &
!               roota_par(i), rootb_par(i)                    , &
!               tmin(i), topt(i), tmax(i), smpso(i), smpsc(i) 
!          if (ier /= 0) then
!             write(6,*)'pftconrd: error in reading in pft data'
!             call endrun()
!          end if
!#endif
#if (defined CLM3 )
          read (n,*,iostat=ier)  pftname(i), &
               z0mr(i)   , displar(i), dleaf(i)  , c3psn(i)  , &
               vcmx25(i) , mp(i)     , qe25(i)   , rhol(i,1) , &
               rhol(i,2) , rhos(i,1) , rhos(i,2) , taul(i,1) , &
               taul(i,2) , taus(i,1) , taus(i,2) , xl(i)     , &
               roota_par(i), rootb_par(i)                       
            if (ier /= 0) then
             write(6,*)'pftconrd: error in reading in pft data'
             call endrun()
          end if
#endif
       end do
       call relavu (n)
    end if     !! end if masterproc reading pft data
#endif         !! end of NON-PHENO2

    ! Set crop explicitly here (in future will be on pft dataset)

    if (masterproc) then
#if(defined AVIM)
       crop(:)  = 0
       crop(15) = 1
       crop(16) = 1
#else
       crop(:)  = 0
       crop(15) = 1
       crop(16) = 1
#endif
    end if

    ! Define PFT zero to be bare ground
    pftname(noveg) = 'not_vegetated'
    z0mr(noveg) = 0.
    displar(noveg) = 0.
    dleaf(noveg) = 0.
    c3psn(noveg) = 1.
    vcmx25(noveg) = 0.
    mp(noveg) = 9.
    qe25(noveg) = 0.
    rhol(noveg,1) = 0.
    rhol(noveg,2) = 0.
    rhos(noveg,1) = 0.
    rhos(noveg,2) = 0.
    taul(noveg,1) = 0.
    taul(noveg,2) = 0.
    taus(noveg,1) = 0.
    taus(noveg,2) = 0.
    xl(noveg) = 0.
    roota_par(noveg) = 0.
    rootb_par(noveg) = 0.
    crop(noveg) = 0.

!zhangyw
#if(defined AVIM )
    tmax(noveg) = 0.
    tmin(noveg) = 0.
    topt(noveg) = 0.
    bw(noveg) = 0.
    vmax(noveg) = 0.
    crg1(noveg) = 0.
    crg2(noveg) = 0.
    crg3(noveg) = 0.
    crg4(noveg) = 0.
    crm1(noveg) = 0.
    crm2(noveg) = 0.
    crm3(noveg) = 0.
    crm4(noveg) = 0.
#if (defined PHENO2)
    smpso(noveg)= 0.
    smpsc(noveg)= 0.
#endif    !! end of PHENO2
#endif    !! end of AVIM 

#ifdef SPMD
    call mpi_bcast (z0mr, size(z0mr), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (displar, size(displar), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (dleaf, size(dleaf), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (c3psn, size(c3psn), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (vcmx25, size(vcmx25), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (mp, size(mp), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (qe25, size(qe25), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (rhol, size(rhol), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (rhos, size(rhos), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (taul, size(taul), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (taus, size(taus), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (xl, size(xl), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (roota_par, size(roota_par), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (rootb_par, size(rootb_par), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (crop, size(crop), MPI_REAL8, 0, mpicom, ier)
#if(defined  AVIM )
!! liwp 2017-03-08 added the follwoing 3 lines for AVIM
    call mpi_bcast (tmax, size(tmax), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (tmin, size(tmin), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (topt, size(topt), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (bw  , size(bw)  , MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (vmax, size(vmax), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (crg1, size(crg1), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (crg2, size(crg2), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (crg3, size(crg3), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (crg4, size(crg4), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (crm1, size(crm1), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (crm2, size(crm2), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (crm3, size(crm3), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (crm4, size(crm4), MPI_REAL8, 0, mpicom, ier)
#if (defined PHENO2)
    call mpi_bcast (smpso, size(smpso), MPI_REAL8, 0, mpicom, ier)
    call mpi_bcast (smpsc, size(smpsc), MPI_REAL8, 0, mpicom, ier)
#endif    !! end of PHENO2 
#endif    !! end of AVIM 

#endif    !! end of SPMD

    if (masterproc) then
       write (6,*) 'Successfully read PFT physiological data'
    end if

  end subroutine pftconrd

end module pftvarcon
