module cpl_mod
    
   use mpmd_mod
   use mom_basic_mod,            only : imt_global, jmt_global
   use time_manager_mod,         only: time_type 
   use mpp_mod,                  only: stdout
   use mpp_mod,                  only : mpp_sync

   implicit none

   integer            :: coupled_freq 
   integer            :: ocean_step 
   integer            :: ocean_days, ocean_seconds
   integer            :: accumulate_time 
   !! logical            :: first_call = .true.
    
   integer, parameter :: nibuff       = 100   ! size of integer control buffe
   integer, parameter :: msgtype_d2oi = 51    ! message type for cpl->ocn init 
   integer, parameter :: msgtype_o2di = 60    ! message type for ocn->cpl init 
   integer, parameter :: msgtype_d2o  = 50    ! message type for cpl->ocn 
   integer, parameter :: msgtype_o2d  = 61     ! message type for ocn->cpl 
   
!--------------------wangln 2008-09-04 add the carbon flux---------------------
   !!integer, parameter :: nsfields     = 6     ! number of fields sent  to   coupler
   !!integer, parameter :: nrfields     = 12    ! number of fields recvd from coupler 
   integer :: nsfields          ! number of fields sent  to   coupler
   integer :: nrfields          ! number of fields recvd from coupler 
   logical :: carbon_switch     ! carbon flux switch true is On & false is off from the CPL 77 
!--------------------wangln 2008-09-04 add the carbon flux---------------------
   

   real(kind=8)                     :: tlast_coupled
   real(kind=8)                     :: wln_radian
    
   real(kind=8), public, allocatable :: sendfields( :,:,: ) 
   real(kind=8), public, allocatable :: recvfields( :,:,: ) 
   integer, public  :: ibuff( nibuff )
      
   logical, public       :: cpl_stop_now           ! flag id for stop_now flag
   logical, public       :: sis_csim               ! flag id for Sea Ice model type : True is SIS , False is CSIM  
   real(kind=8), public  :: dt_evp                 ! Time parameter for Dhdx & dhdy

   integer            :: cpl_task !  task id for coupler
   integer, private   :: cpl_ts                ! flag id for coupled_ts flag           
   integer, private   :: cpl_write_restart     ! flag id for write restart           
   integer, private   :: cpl_write_history     ! flag id for write history           
   integer, private   :: cpl_write_tavg        ! flag id for write tavg           
   integer, private   :: cpl_diag_global       ! flag id for computing diagnostics           
   integer, private   :: cpl_diag_transp       ! flag id for computing diagnostics

   real(kind=8), private, allocatable   :: xc_o(:,:)    ! longitude of grid center
   real(kind=8), private, allocatable   :: yc_o(:,:)    ! latitude of grid center
   real(kind=8), private, allocatable   :: xv_o(:,:,:)  ! longitude of grid corner 
   real(kind=8), private, allocatable   :: yv_o(:,:,:)  ! latitude of grid corner 
   real(kind=8), private, allocatable   :: area_o(:,:)  ! area of grid 
   integer     , private, allocatable   :: mask_o(:,:)  ! mask of grid center
   real(kind=8), private, allocatable   :: angle_o(:,:)  ! area of grid 

   real(kind=8), public, allocatable   :: global_sin(:,:)    ! Sin of the grid angle
   real(kind=8), public, allocatable   :: global_cos(:,:)    ! Cos of the grid angle 

contains
!======================================================================================================

!-----------------------------------------------------------------------------------------------------
   subroutine init_cpl_array()

     use mpp_mod, only : stdout
     integer  :: ierr


      write(stdout(),*) 'Cpl_Array Sub', imt_global, jmt_global 

!--------------------wangln 2008-09-04 add the carbon flux---------------------
      !!allocate( sendfields( imt_global, jmt_global, nsfields ), stat = ierr )
      !!if( ierr /= 0 ) then
      !!    write(stdout(),*) '---Sub init_cpl_array: applied the Array : sendfields is failed ! '
      !!else 
      !!    sendfields = 0.0
      !!endif

      !!allocate( recvfields( imt_global, jmt_global, nrfields ), stat = ierr )
      !!if( ierr /= 0 ) then
      !!    write(stdout(),*) '---Sub init_cpl_array: applied the Array : sendfields is failed ! '
      !!else
      !!    recvfields = 0.0
      !!endif
!--------------------wangln 2008-09-04 add the carbon flux---------------------

      allocate( xc_o( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then 
          write(stdout(),*) '---Sub init_cpl_array: applied the Array : Xc_o is failed ! '
      else
          xc_o = 0.0
      endif

      allocate( yc_o( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then 
          write(stdout(),*) '---Sub init_cpl_array: applied the Array : Yc_o is failed ! '
      else
          yc_o = 0.0
      endif

      allocate( xv_o( 4,imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then
          write(stdout(),*) '---Sub init_cpl_array: applied the Array : Xc_o is failed ! '
      else
          xv_o = 0.0
      endif

      allocate( yv_o( 4,imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then
          write(stdout(),*) '---Sub init_cpl_array: applied the Array : Yc_o is failed ! '
      else
          yv_o = 0.0
      endif

      allocate( area_o( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then
          write(stdout(),*) '---Sub init_cpl_array: applied the Array : Area_o is failed ! '
      else
          area_o = 0.0
      endif 

      allocate( angle_o( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then
          write(stdout(),*) '---Sub init_cpl_array: applied the Array : Angle_o is failed ! '
      else
          area_o = 0.0
      endif 

      allocate( mask_o( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then
          write(stdout(),*) '---Sub init_cpl_array: applied the Array : Mask_o is failed ! '
      else
          mask_o = 0
      endif


      allocate( global_sin( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then 
          write(stdout(),*) '---Sub init_cpl_array: applied the Array : global_sin is failed ! '
      else
          global_sin = 0.0
      endif
      allocate( global_cos( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then 
          write(stdout(),*) '---Sub init_cpl_array: applied the Array : global_cos is failed ! '
      else
          global_cos = 0.0
      endif 

   end subroutine init_cpl_array
   
!------------------------------------------------------------------------------------------------------

!!------------xujx added 2008-06-16-------------------------------------------------------------------
   subroutine init_global_sin_cos()

     integer   :: ierr

     allocate( global_sin( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then 
          write(stdout(),*) '---Sub init_global_sin_cos: applied the Array : global_sin is failed ! '
      else
          global_sin = 0.0
      endif
      allocate( global_cos( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then 
          write(stdout(),*) '---Sub init_global_sin_cos applied the Array : global_cos is failed ! '
      else
          global_cos = 0.0
      endif 
   end subroutine init_global_sin_cos
!!------------xujx added 2008-06-16-------------------------------------------------------------------

   subroutine read_global_info()

      include 'netcdf.inc'

      integer  :: i, j, k
      integer  :: ierr
      integer  :: rcode   ! netcdf routine return code
      integer  :: file_id ! netcdf file ID
      integer  :: vid     ! netcdf variable ID
      integer  :: did     ! netcdf dimension ID
      real(kind=8), allocatable  :: tmp( :,: )
      real(kind=8), allocatable  :: tmp_corner( :,: ,:)

      call init_cpl_array() ! Initial the Cpl Array

      allocate( tmp( imt_global, jmt_global ), stat = ierr )
      if( ierr /= 0 ) then
          write(*,*) '---Sub Read_global_info:  applied the Array : tmp is failed ! '
      else
          tmp = 0.0
      endif

      allocate( tmp_corner( imt_global, jmt_global, 4 ), stat = ierr )
      if( ierr /= 0 ) then
          write(*,*) '---Sub Read_global_info:  applied the Array : tmp_corner is failed ! '
      else 
          tmp_corner = 0.0
      endif

      rcode = nf_open('INPUT/grid_spec.nc',NF_OPEN,file_id)
      if( rcode /= 0 ) write(*,*) '---Sub Read_global_info:  Not open the NC : INPUT/grid_spec.nc ! '

!---------------------read the Corner----------------------------------
      rcode = nf_inq_varid( file_id, 'x_T', vid )
      if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  x_T Inq Variable Status=',rcode
      rcode = nf_get_var_double( file_id, vid, xc_o )
      if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  Xc_o Get Variable Status=',rcode

      rcode = nf_inq_varid( file_id, 'y_T', vid )
      if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  ryx_T Inq Variable Status=',rcode
      rcode = nf_get_var_double( file_id, vid, yc_o )
      if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  Yc_o Get Variable Status=',rcode

      rcode = nf_inq_varid( file_id, 'area_T', vid )
      if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  area_T Inq Variable Status=',rcode
      rcode = nf_get_var_double( file_id, vid, area_o )
      if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  area_o Get Variable Status=',rcode

      rcode = nf_inq_varid( file_id, 'wet', vid )
      if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  Mask Inq Variable Status=',rcode
      rcode = nf_get_var_double( file_id, vid, tmp )
      if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  Mask Get Variable Status=',rcode

       mask_o = 0
       where ( tmp > 0.0 )
            mask_o = 1
       end where

       rcode = nf_inq_varid( file_id, 'angle_T', vid )
       if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  Angle_T Inq Variable Status=',rcode
       rcode = nf_get_var_double( file_id, vid, tmp )
       if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  Angle Get Variable Status=',rcode

       global_sin = 1.0e-10
       global_cos = 1.0e-10
       wln_radian = 180./( 4.0*atan(1.0) )     
       tmp = tmp / wln_radian ! (RADIAN = 180.0/Pi )
       where ( mask_o == 1 )
           global_sin = sin( tmp )
           global_cos = cos( tmp )
       end where
       angle_o = tmp

       rcode = nf_inq_varid( file_id, 'x_vert_T', vid )
       if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  x_vert_T Inq Variable Status=',rcode
       rcode = nf_get_var_double( file_id, vid, tmp_corner )
       if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  X_vert_T Get Variable Status=',rcode


       do j = 1, jmt_global
          do i = 1, imt_global
             do k = 1, 4
                xv_o(k, i,j) = tmp_corner(i,j,k)
             enddo
          enddo
       enddo

       rcode = nf_inq_varid( file_id, 'y_vert_T', vid )
       if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  y_vert_T Inq Variable Status=',rcode
       rcode = nf_get_var_double( file_id, vid, tmp_corner )
       if( rcode /= 0 ) write(*,*) '----Sub Read_global_info:  y_vert_T Get Variable Status=',rcode

       do j = 1, jmt_global
          do i = 1, imt_global
             do k = 1, 4
                yv_o(k, i,j) = tmp_corner(i,j,k)
             enddo
          enddo
       enddo
  
       area_o =( area_o/6371000.0)/ 6371000.0   ! Translate it from the M^2 to Radius

       deallocate( tmp )
       deallocate( tmp_corner )


      rcode = nf_close( file_id)
      write(stdout(),*) 'Now Finish the subroutine read_global_info'


   end subroutine read_global_info


!-----------------------------------------------------------------------------------------------------
!-----------------------------------------------------------------------------------------------------
   subroutine init_cpl( step_time, cpl_time, cpl_date )
    
    implicit none
 
    include 'mpif.h'
    !use mpp_mod, only : stdout

    integer, intent(in) :: step_time  ! ocean step time (unit: s)
    integer, intent(in) :: cpl_time   ! ocean every coupled time (unit: s)
    integer, intent(in) :: cpl_date(6)

    integer  :: ier
    integer  :: mpistatus(MPI_STATUS_SIZE)
    !character(len= MPI_MAX_ERROR_STRING) :: error_str
    integer  :: ierr, error_class

    write(stdout(),*) '---------Now begin the Subroutine init_cpl---------------'

      coupled_freq = 86400 / cpl_time
      ocean_step = step_time
      accumulate_time = cpl_time / step_time
      write(stdout(),*) ' coupled_freq, ocean_step, accumulate_time=', coupled_freq, ocean_step, accumulate_time
      wln_radian = 180./( 4.0*atan(1.0) )     

      tlast_coupled = 0

      if( master_task ) then
         call MPI_RECV(ibuff, nibuff, MPI_INTEGER, cpl_task, msgtype_d2oi, MPI_COMM_WORLD, mpistatus, ier) 

!--------------------wangln 2008-09-04 add the carbon flux---------------------
         if( ibuff(77) == 1 ) then
             carbon_switch = .true.
             write(stdout(),*) '-------Carbon flux switch On-----------------'
         else
             carbon_switch = .false.
             write(stdout(),*) '-------Carbon flux switch Off-----------------'
         endif
         if( carbon_switch ) then
             nsfields =  7
!wtw             nrfields = 13
             nrfields = 14
         else 
             nsfields =  6
             nrfields = 12
         endif
                
         allocate( sendfields( imt_global, jmt_global, nsfields ), stat = ierr )
         if( ierr /= 0 ) then
             write(stdout(),*) '---Sub init_cpl_array: applied the Array : sendfields is failed ! '
         else 
             sendfields = 0.0
         endif

         allocate( recvfields( imt_global, jmt_global, nrfields ), stat = ierr )
         if( ierr /= 0 ) then
             write(stdout(),*) '---Sub init_cpl_array: applied the Array : sendfields is failed ! '
         else
             recvfields = 0.0
         endif

      endif


         call mpi_bcast ( nsfields, 1, MPI_INTEGER, 0, MPI_COMM_OCN, ierr)
         if( ierr /= MPI_SUCCESS) then
             write( stdout(), *) 'MPI BCAST ERROR: for nsfields in Init_CPL Subroutine'
         endif
         call mpi_bcast ( nrfields, 1, MPI_INTEGER, 0, MPI_COMM_OCN, ierr)
         if( ierr /= MPI_SUCCESS) then
             write( stdout(), *) 'MPI BCAST ERROR: for nrfields in Init_CPL Subroutine'
         endif
         call mpi_bcast ( carbon_switch, 1, MPI_LOGICAL, 0, MPI_COMM_OCN, ierr)
         if( ierr /= MPI_SUCCESS) then
             write( stdout(), *) 'MPI BCAST ERROR: for Carbon_Sitchin Init_CPL Subroutine'
         endif

!--------------------wangln 2008-09-04 add the carbon flux---------------------



      if( master_task ) then
          ibuff = 0

          ibuff(4) = cpl_date(1)*10000 + cpl_date(2)*100 + cpl_date(3)   
          ibuff(5) = cpl_date(4)*3600  + cpl_date(5)*60  + cpl_date(6)   

          ibuff(7) = imt_global 
          ibuff(8) = jmt_global 
          ibuff(9) = coupled_freq  ! Coupled Frequency
          call MPI_SEND(ibuff, nibuff, MPI_INTEGER, cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier)
      endif
      sis_csim = .true.
      if( master_task .and. ibuff( 100 ) == 1 ) sis_csim = .false. 

      if( master_task ) then
          call MPI_SEND( xc_o, imt_global*jmt_global, MPI_DOUBLE_PRECISION, cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier) 
          call MPI_SEND( yc_o, imt_global*jmt_global, MPI_DOUBLE_PRECISION, cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier) 
          call MPI_SEND( xv_o, 4*imt_global*jmt_global, MPI_DOUBLE_PRECISION, cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier) 
          call MPI_SEND( yv_o, 4*imt_global*jmt_global, MPI_DOUBLE_PRECISION, cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier) 
          call MPI_SEND( area_o, imt_global*jmt_global, MPI_DOUBLE_PRECISION, cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier) 
          call MPI_SEND( mask_o, imt_global*jmt_global, MPI_INTEGER, cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier) 
          call MPI_SEND( angle_o,imt_global*jmt_global, MPI_DOUBLE_PRECISION, cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier) 
      endif
      if( master_task ) then

         deallocate( xc_o )
         deallocate( yc_o )
         deallocate( xv_o )
         deallocate( yv_o )
         deallocate( area_o )
         deallocate( mask_o )
         deallocate( angle_o )

      endif

     !-----------wangln 2007-01-24------- Add the dt_evp Calculating method for Dhdx & Dhdy---------- 
      if( master_task ) dt_evp = cpl_time * step_time / 86400
     !-----------wangln 2007-01-24------- Add the dt_evp Calculating method for Dhdx & Dhdy---------- 

   end subroutine init_cpl


!-----------------------------------------------------------------------------------------------------
   subroutine recv_from_cpl()
     
    implicit none

    include 'mpif.h'

    integer  :: ier
    integer  :: mpistatus(MPI_STATUS_SIZE)
    integer  :: ierr, error_class
    

      if(  master_task ) then
         call MPI_RECV( ibuff, nibuff, MPI_INTEGER, cpl_task, msgtype_d2o, MPI_COMM_WORLD, mpistatus, ier)
         call MPI_RECV( recvfields, size(recvfields), MPI_DOUBLE_PRECISION, cpl_task, msgtype_d2o, &
                        MPI_COMM_WORLD, mpistatus, ier)
      endif
      
      call mpi_bcast(ibuff, nibuff, mpi_integer, 0, MPI_COMM_OCN, ier )
      
      cpl_stop_now =  .false.
      sis_csim = .true.
      if (ibuff(3) == 1) then
         cpl_stop_now = .true. 
         return
      endif 

   end subroutine recv_from_cpl 
     
!-----------------------------------------------------------------------------------------------------
   subroutine send_to_cpl( cpl_date )
     
    implicit none

    include 'mpif.h'

    integer, intent(in) :: cpl_date(6)
    integer  :: ier
    integer  :: mpistatus(MPI_STATUS_SIZE)
    integer  :: t_size 
    integer  :: ierr, error_class

      
    t_size = nsfields*imt_global*jmt_global
      if( master_task ) then
         
          ibuff(4) = cpl_date(1)*10000 + cpl_date(2)*100 + cpl_date(3)  
          ibuff(5) = cpl_date(4)*3600  + cpl_date(5)*60  + cpl_date(6)   

         ! Transfer the data from the MOM triangle Grid to the lon_lat grid
         ! gather the data which CPL needed from every CPU 
 
         call MPI_SEND(ibuff, nibuff, MPI_INTEGER, cpl_task, msgtype_o2d, MPI_COMM_WORLD, ier)  
         call MPI_SEND( sendfields, size(sendfields), MPI_DOUBLE_PRECISION, cpl_task, msgtype_o2d, MPI_COMM_WORLD, ier)

      endif

      tlast_coupled = 0

   end subroutine send_to_cpl 

!-----------------------------------------------------------------------------------------------------
  subroutine setup_mpi

   implicit none

   include 'mpif.h'   ! MPI Fortran include file
  
   integer    :: ierr

   call mpi_init ( ierr)
   !if( ierr == MPI_SUCCESS ) write( *,*) ' MPI_INIT is OK !'
   call mpi_coupled
   !write(6,*) ' Coupled Initialied is OK! '

   !-----set up the Ocn Flag
   !call MPI_COMM_DUP(MPI_COMM_WORLD, MPI_COMM_OCN, ierr)

   master_task = .true.
   
   !write(6,*) ' Setup_mpi is Ok!'

 end subroutine setup_mpi  
!-----------------------------------------------------------------------------------------------------
  subroutine mpi_coupled

    implicit none

    include 'mpif.h'   ! MPI Fortran include file

    logical          :: mpi_running
    character(len=3) :: cmodel   ! model name temporary
    integer          :: range(3) ! array for creating groups for each coupled model

    integer       :: MPI_GROUP_WORLD        ! group id for MPI_COMM_WORLD
    integer       :: MPI_GROUP_CPL          ! group of processors assigned to cpl
    integer       :: MPI_GROUP_ATM          ! group of processors assigned to atm
    integer       :: MPI_GROUP_OCN          ! group of processors assigned to ocn
    integer       :: MPI_GROUP_ICE          ! group of processors assigned to ice
    integer       :: MPI_GROUP_LND           ! group of processors assigned to lnd

    integer       :: n                           ! dummy loop counter
    integer       :: ierr                        ! error flag for MPI comms
    integer       :: nprocs_cpl                  ! total processor count
    integer       :: my_task_coupled             ! rank of process in coupled domain
    integer       :: cpl_rank_min, cpl_rank_max  ! processor range for each 
    integer       :: ocn_rank_min, ocn_rank_max  !  component model
    integer       :: atm_rank_min, atm_rank_max
    integer       :: ice_rank_min, ice_rank_max
    integer       :: lnd_rank_min, lnd_rank_max
    integer       :: atm_tid          ! task id for the process in each component
    integer       :: ice_tid          !   which communicates w coupler - these are
    integer       :: lnd_tid          !   defined as the lowest rank process
    integer       :: ocn_tid          !   in the processor group
    integer       :: MPI_COMM_ATM     ! MPI communicator to be used for
    integer       :: MPI_COMM_CPL     !   messages internal to the component model
    integer       :: MPI_COMM_ICE     !   as opposed to between models
    integer       :: MPI_COMM_LND      !   these must be defined by all models even though only used internally by each model
    
  
       !call mpi_initialized (mpi_running, ierr)
       
       !if( .not. mpi_running )  call mpi_init ( ierr)

       call MPI_COMM_RANK  (MPI_COMM_WORLD, my_task_coupled, ierr)
       !write(6,*) 'establish intra-model communicators, my mpi_comm_world rank is ',my_task_coupled

       call MPI_COMM_SIZE (MPI_COMM_WORLD, nprocs_cpl, ierr)
       
       atm_rank_min = nprocs_cpl
       atm_rank_max = 0
       ocn_rank_min = nprocs_cpl
       ocn_rank_max = 0
       ice_rank_min = nprocs_cpl
       ice_rank_max = 0
       lnd_rank_min = nprocs_cpl
       lnd_rank_max = 0
       cpl_rank_min = nprocs_cpl
       cpl_rank_max = 0

       !*** each processor broadcasts its model to all the processors in the coupled domain

       do n=0,nprocs_cpl-1
        if (n == my_task_coupled) then
          cmodel = 'ocn'
        else
          cmodel = 'unk'
        endif

        call MPI_BCAST(cmodel, 3, MPI_CHARACTER, n, MPI_COMM_WORLD, ierr)

         select case(cmodel)
        case ('atm')
          atm_rank_min = min(atm_rank_min, n)
          atm_rank_max = max(atm_rank_max, n)
        case ('ocn')
          ocn_rank_min = min(ocn_rank_min, n)
          ocn_rank_max = max(ocn_rank_max, n)
        case ('ice')
          ice_rank_min = min(ice_rank_min, n)
          ice_rank_max = max(ice_rank_max, n)
        case ('lnd')
          lnd_rank_min = min(lnd_rank_min, n)
          lnd_rank_max = max(lnd_rank_max, n)
        case ('cpl')
          cpl_rank_min = min(cpl_rank_min, n)
          cpl_rank_max = max(cpl_rank_max, n)
        case ('dud')
        case default
       !   write(6,*) 'Unknown model ' ,cmodel,' in coupled model domain'
       !   write(6,*) 'Model must be atm, cpl, ice, lnd, ocn'
          stop
        end select

      end do
       

      !     assume the first processor in each domain is the task that will communicate coupled model messages

      atm_tid  = atm_rank_min
      cpl_task = cpl_rank_min
      ice_tid  = ice_rank_min
      lnd_tid  = lnd_rank_min
      ocn_tid  = ocn_rank_min

      !     create subroup and communicators for each models internal
      !     communciations, note that MPI_COMM_CREATE must be called by
      !     all processes in MPI_COMM_WORLD so this must be done by all
      !     models consistently and in the same order.

      call MPI_COMM_GROUP(MPI_COMM_WORLD, MPI_GROUP_WORLD, ierr)
 !     write(6,*) 'MPI_COMM_GROUP: MPI_COMM_WORLD, MPI_GROUP_WORLD = ', MPI_COMM_WORLD, MPI_GROUP_WORLD

      range(3) = 1
      range(1) = atm_rank_min
      range(2) = atm_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_ATM, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_ATM= ', MPI_GROUP_WORLD, MPI_GROUP_ATM

      range(1) = ocn_rank_min
      range(2) = ocn_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_OCN, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_OCN= ', MPI_GROUP_WORLD, MPI_GROUP_OCN

      range(1) = ice_rank_min
      range(2) = ice_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_ICE, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_ICE= ', MPI_GROUP_WORLD, MPI_GROUP_ICE

      range(1) = lnd_rank_min
      range(2) = lnd_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_LND, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_LND= ', MPI_GROUP_WORLD, MPI_GROUP_LND

      range(1) = cpl_rank_min
      range(2) = cpl_rank_max 
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, MPI_GROUP_CPL, ierr)
 !     write(6,*) 'MPI_GROUP_RANGE_INCL: MPI_GROUP_WORLD, MPI_GROUP_CPL= ', MPI_GROUP_WORLD, MPI_GROUP_CPL

      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_ATM, MPI_COMM_ATM, ierr)
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_OCN, MPI_COMM_OCN, ierr)
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_ICE, MPI_COMM_ICE, ierr)
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_LND, MPI_COMM_LND, ierr)
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_CPL, MPI_COMM_CPL, ierr)

      MPI_COMM_OCN_GROUP = MPI_GROUP_OCN


      !-----set up the Ocn Flag

      !master_task = .true.
      
  end subroutine mpi_coupled
!-----------------------------------------------------------------------------------------------------
!======================================================================================================
end module cpl_mod


