module gather_data_mod 
    
   use mpmd_mod
   use mom_basic_mod,    only : num_cpu, imt_global, jmt_global, local_domain
   use cpl_mod,          only : global_sin, global_cos
   use cpl_mod,          only : nsfields, sendfields, send_to_cpl
   use ocean_types_mod,  only : ocean_data_type, ice_ocean_boundary_type 
   use mpp_mod,          only : stdout
   use cpl_mod,          only : cpl_stop_now
   use cpl_mod,          only : sis_csim 
   use cpl_mod,          only : dt_evp 
   use cpl_mod,          only : accumulate_time 
!--------------------wangln 2008-09-04 add the carbon flux----------------------
   use cpl_mod,          only : carbon_switch 
!--------------------wangln 2008-09-04 add the carbon flux----------------------

   implicit none
    
   public   :: gather_local_data
   private  :: transfer_3d_to_1d
   private  :: transfer_1d_to_3d
   private  :: compute_displs_counts 

   save
   logical  :: first_call = .true.

contains
!=====================================================================================================
!------------------------------------------------------------------------------------------------------ 
    subroutine transfer_3d_to_1d( local_ocean, isc, iec, jsc, jec, local_data, total_size )
 
        type (ocean_data_type), intent(in)   :: local_ocean
        integer, intent(in)                  :: isc, iec, jsc, jec
        integer,      intent(in)             :: total_size
        real(kind=8), intent(out)            :: local_data( 0:total_size ) 

        integer                              :: i, j, k, n, ierr
        real(kind=8)                         :: u(isc:iec,jsc:jec), v(isc:iec,jsc:jec) 


!wtw        write(stdout(),*) ' --------transfer_3d_to_1d accumulate_time=', accumulate_time
!wtw        write(stdout(),*) '--------transfer_3d_to_1d total_size=', total_size
        n = 0
        local_data = 0.0 
        do j = jsc, jec 
           do i = isc, iec
!----------------
!  WUTW
!
                local_ocean%t0_surf(i,j) = local_ocean%t_surf(i,j)
!----------
                local_data( n ) = local_ocean%t_surf(i,j);  n = n + 1 
                local_data( n ) = local_ocean%s_surf(i,j);  n = n + 1 

!  from the ocean coordinate to lat/lon.
!
                local_data( n ) = local_ocean%u_surf(i,j) * global_cos( i,j ) -  &
                                  local_ocean%v_surf(i,j) * global_sin( i,j ) ;   n = n + 1 
                local_data( n ) = local_ocean%v_surf(i,j) * global_cos( i,j ) +  &
                                  local_ocean%u_surf(i,j) * global_sin( i,j ) ;   n = n + 1 

                local_data( n ) = local_ocean%sea_lev(i,j); n = n + 1   ! Sea_level 
                local_data( n ) = local_ocean%frazil(i,j) / accumulate_time;  n = n + 1 

!--------------------wangln 2008-09-04 add the carbon flux----------------------
                if( carbon_switch ) then
                     local_data( n ) = local_ocean%carbon(i,j);  n = n + 1 
                endif
!--------------------wangln 2008-09-04 add the carbon flux----------------------

           enddo
       enddo

    end subroutine transfer_3d_to_1d 
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
    subroutine transfer_1d_to_3d( temp_data )
    
        real(kind=8), intent(in)  :: temp_data( 0:imt_global*jmt_global*nsfields-1 )   
 
        integer :: i, j, k, n, m
   
        k = 0
        sendfields = 0.0
        do n = 0, num_cpu-1
           do j = local_domain( n )%y_beg, local_domain( n )%y_end
              do i = local_domain( n )%x_beg, local_domain( n )%x_end
                 do m = 1, nsfields
                      sendfields( i, j, m ) = temp_data( k )
                      k = k + 1
                 enddo
              enddo
           enddo
        enddo

    end subroutine transfer_1d_to_3d      
               
         
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
    subroutine gather_local_data(  cpl_date, ocean_sfc, isc,iec, jsc, jec  )
 
    include 'mpif.h'
     
       integer, intent(in)                  :: cpl_date(6) 
       type (ocean_data_type), intent(in)   :: ocean_sfc
       integer, intent(in)                  :: isc, iec, jsc, jec
       
       integer  :: myid
       integer  :: gsize, ierr
       integer  :: i, j, k, n
       integer  :: total_size
       real(kind=8), allocatable :: temp_data( : ) 
       real(kind=8), allocatable :: global_data( :,: )

       integer :: displs( 0 : num_cpu-1 )  
       integer :: recvcounts( 0:num_cpu-1 )

       real(kind=8), allocatable :: local_data( : )
       character(len=3) :: cstep
       character(len=3) :: ccpu

    
!wtw       write(stdout(),*) '-----------gather_local_data: nsfields =', nsfields

       total_size = ( iec - isc + 1 ) * ( jec - jsc + 1 ) * nsfields  -1
       allocate( local_data( 0:total_size ), stat = ierr )
       if( ierr /= 0 ) then
           write(stdout(),*) '----Sub gather_local_data:  applied the Array : local_data is failed !'  
       else
           local_data = 0.0
       endif

       call transfer_3d_to_1d( ocean_sfc, isc, iec, jsc, jec, local_data, total_size )

       if( master_task ) then

          allocate( global_data( imt_global, jmt_global ), stat = ierr )
          if( ierr /=0 ) then
              write(stdout(),*) '--Sub gather_local_data: Array applied Failed : Global_data '
          else
              global_data = 0.0
          endif
          call compute_displs_counts( nsfields, num_cpu, displs, recvcounts )  ! set the MPI_GATHERV parameter

       endif

       if( master_task ) then

          allocate( temp_data( 0:imt_global*jmt_global*nsfields-1 ), stat = ierr )
          if( ierr /=0 ) then
              write(stdout(),*) '--Sub gather_local_data: Array applied Failed : Temp_data '
          else
              temp_data = 0.0
          endif

          call MPI_GATHERV( local_data,  size( local_data ), MPI_REAL8, & 
               temp_data, recvcounts, displs, MPI_REAL8, 0, MPI_COMM_OCN, ierr )
          if( ierr == 0 ) then

              call transfer_1d_to_3d( temp_data )
              call send_to_cpl( cpl_date )

          endif

      else
          call MPI_GATHERV( local_data, size( local_data ), MPI_REAL8, & 
               0.0, recvcounts, displs, MPI_REAL8, 0, MPI_COMM_OCN, ierr )
      endif


    if ( master_task ) then 
        deallocate( temp_data ) 
        deallocate( global_data )
    endif 
   
    deallocate( local_data ) 

  end subroutine gather_local_data
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
  subroutine compute_displs_counts( nfact, gsize, displs, counts )

     integer, intent(in) :: nfact ! the number of the elements types
     integer, intent(in) :: gsize ! the total CPUs
     integer, intent(out) :: displs( 0:gsize-1 ) ! displacement parameter
     integer, intent(out) :: counts( 0:gsize-1 ) ! the MPI_GATHERV reciver data size for every CPU

     integer  :: k


           do k = 0, gsize-1
              counts( k ) = nfact * local_domain(k)%x_size * local_domain(k)%y_size
           enddo

           displs( 0 ) = 0
           do k = 1, gsize-1
              displs( k ) = displs( k-1 ) + counts( k -1 )
           enddo

     
  end subroutine compute_displs_counts
!------------------------------------------------------------------------------------------------------ 
!======================================================================================================
end module gather_data_mod


