module  scatter_data_mod 
    
   use mpmd_mod
   use mom_basic_mod,    only : num_cpu, imt_global, jmt_global, local_domain
   use cpl_mod,          only : nrfields, recvfields, recv_from_cpl, cpl_stop_now 
   use ocean_types_mod,  only : ice_ocean_boundary_type 
   use cpl_mod,          only : global_sin, global_cos
   use mpp_mod,          only : stdout
   use mpp_mod,          only : mpp_sync
!--------------------wangln 2008-09-04 add the carbon flux----------------------
   use cpl_mod,          only : carbon_switch
!--------------------wangln 2008-09-04 add the carbon flux----------------------

   implicit none
    
   public   :: scatter_force_field
   private  :: transfer_3d_to_1d
   private  :: transfer_1d_to_3d
   private  :: compute_displs_counts 

contains
!=====================================================================================================

   subroutine transfer_3d_to_1d( fields, temp_data )
 
        real(kind=8), intent(in)  :: fields( imt_global, jmt_global, nrfields ) 
        real(kind=8), intent(out) :: temp_data( 0:imt_global*jmt_global*nrfields-1 ) 

        integer   :: i, j, k, n, m 


        m = 0
        temp_data = 0.0
        do n = 0, num_cpu-1
           do j = local_domain( n )%y_beg, local_domain( n )%y_end
              do i = local_domain( n )%x_beg, local_domain( n )%x_end
                 do k = 1, nrfields 
                      temp_data( m ) = fields(i, j, k ); m = m + 1
                 enddo
              enddo
           enddo    
        enddo


    end subroutine transfer_3d_to_1d 

!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
    subroutine transfer_1d_to_3d( isc, iec, jsc, jec, total_size, local_data, local_boundary )
    
        integer, intent(in)                          :: isc, iec, jsc, jec
        integer, intent(in)                          :: total_size
        real(kind=8), intent(in)                     :: local_data( 0:total_size )   
        type (ice_ocean_boundary_type), intent(inout)  :: local_boundary
 
        integer :: i, j, n
        real(kind=8) :: u_flux_a, v_flux_a 
         
        n = 0
        do j = jsc, jec
           do i = isc, iec

!-----------------
!
!               local_boundary%u_flux(i,j)    = local_data ( n );  n = n + 1
!               local_boundary%v_flux(i,j)    = local_data ( n );  n = n + 1
                u_flux_a = local_data ( n );  n = n + 1
                v_flux_a = local_data ( n );  n = n + 1
               local_boundary%u_flux(i,j)  = global_cos( i,j )*u_flux_a + &
                                             global_sin( i,j )*v_flux_a

               local_boundary%v_flux(i,j)  =-global_sin( i,j )*u_flux_a + &
                                             global_cos( i,j )*v_flux_a
!-----------------------------
                local_boundary%t_flux(i,j)    = local_data ( n );  n = n + 1
                local_boundary%q_flux(i,j)    = local_data ( n );  n = n + 1
                local_boundary%salt_flux(i,j) = local_data ( n );  n = n + 1
                local_boundary%lw_flux(i,j)   = local_data ( n );  n = n + 1
                local_boundary%sw_flux(i,j)   = local_data ( n );  n = n + 1
                local_boundary%lprec(i,j)     = local_data ( n );  n = n + 1
                local_boundary%fprec(i,j)     = local_data ( n );  n = n + 1
                local_boundary%runoff(i,j)    = local_data ( n );  n = n + 1
                local_boundary%calving(i,j)   = local_data ( n );  n = n + 1
                local_boundary%p(i,j)         = local_data ( n );  n = n + 1

!--------------------wangln 2008-09-04 add the carbon flux----------------------
                if( carbon_switch ) then
                    local_boundary%co2(i,j)= local_data( n ) * 1.e6;  n = n + 1
                endif
!--------------------wangln 2008-09-04 add the carbon flux----------------------
                 
                local_boundary%dEdt(i,j)   = local_data ( n );  n = n + 1

           enddo
       enddo

    end subroutine transfer_1d_to_3d      
               
         
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
    subroutine scatter_force_field(   force_field, isc, iec, jsc, jec  )
 
    include 'mpif.h'
     

       type (ice_ocean_boundary_type), intent(inout)   :: force_field
       integer, intent(in)                           :: isc, iec, jsc, jec
       
       integer                   :: myid
       integer                   :: gsize, ierr
       integer                   :: i, j, k, n
       integer                   :: total_size

       integer                   :: displs( 0 : num_cpu-1 )  
       integer                   :: sendcounts( 0:num_cpu-1 )
       character(len=3)          :: cstep

       real(kind=8), allocatable :: local_data( : )
       real(kind=8), allocatable :: temp_data( : ) 
    
      
      
       if( master_task ) then 
           allocate( temp_data( 0:imt_global*jmt_global*nrfields-1 ), stat = ierr )
           if( ierr /=0 ) then
               write(stdout(),*) '--Sub Scatter_force_field:  Array applied Failed : Temp_data '
           else
               temp_data = 0.0
           endif 
       endif

       total_size = ( iec - isc + 1 ) * ( jec - jsc + 1 ) * nrfields  -1
       allocate( local_data( 0:total_size ), stat = ierr )
       if( ierr /= 0 ) then
           write(stdout(),*) '----Sub Scatter_force_field:  applied the Array : local_data is failed !'  
       else
           local_data = 0.0
       endif

       
       call recv_from_cpl()
       if( master_task ) then
          call transfer_3d_to_1d( recvfields, temp_data )
          call compute_displs_counts( nrfields, num_cpu, displs, sendcounts )  ! set the MPI_GATHERV parameter
       endif

       if( master_task ) then
           call MPI_SCATTERV( temp_data, sendcounts, displs, MPI_REAL8,  & 
                             local_data, size( local_data ), MPI_REAL8, 0, MPI_COMM_OCN, ierr ) 
       else 
           call MPI_SCATTERV( 0,         sendcounts, displs, MPI_REAL8,  & 
                             local_data, size( local_data ), MPI_REAL8, 0, MPI_COMM_OCN, ierr ) 
       endif
       
       call transfer_1d_to_3d( isc, iec, jsc, jec, total_size, local_data(0:total_size), force_field ) 

   
    deallocate( local_data ) 
    if( master_task )  deallocate( temp_data ) 

  end subroutine scatter_force_field
!------------------------------------------------------------------------------------------------------ 
!------------------------------------------------------------------------------------------------------ 
  subroutine compute_displs_counts( nfact, gsize, displs, counts )

     integer, intent(in) :: nfact ! the number of the elements types
     integer, intent(in) :: gsize ! the total CPUs
     integer, intent(out) :: displs( 0:gsize-1 ) ! displacement parameter
     integer, intent(out) :: counts( 0:gsize-1 ) ! the MPI_GATHERV reciver data size for every CPU

     integer  :: k


           do k = 0, gsize-1
              counts( k ) = nfact * local_domain(k)%x_size * local_domain(k)%y_size
           enddo

           displs( 0 ) = 0
           do k = 1, gsize-1
              displs( k ) = displs( k-1 ) + counts( k -1 )
           enddo

     
  end subroutine compute_displs_counts
!------------------------------------------------------------------------------------------------------ 
!======================================================================================================
end module scatter_data_mod 


