!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Created by Song.Zhenya 21/05/2009                                 !!
!! Contact email="songroy@fio.org.cn"                                !!
!! This module get the waved-indueced mixing(bv)                     !!
!!                                                                   !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
module wave_vert_mix_mod

!use constants_mod,       only: epsln, pi
!use diag_manager_mod,    only: register_diag_field, register_static_field, send_data
!use field_manager_mod,   only: MODEL_OCEAN, parse, find_field_index, get_field_methods, method_type, get_field_info
use fms_mod,             only: open_namelist_file, check_nml_error, close_file, write_version_number
use fms_mod,             only: FATAL, stdout, stdlog
use mpp_mod,             only: mpp_error
use fms_mod,                  only: close_file, file_exist, uppercase
!
use ocean_types_mod,     only: ocean_grid_type, ocean_domain_type
use ocean_domains_mod,   only: get_local_indices
!use ocean_types_mod,     only: ocean_time_type, ocean_time_steps_type
!use ocean_types_mod,     only: ocean_velocity_type, ocean_prog_tracer_type
!use ocean_types_mod,     only: missing_value
!use ocean_workspace_mod, only: wrk1, wrk2
use mpp_mod,          only: mpp_pe
use mom_basic_mod,     only: imt_global, jmt_global, kmt_global
!
implicit none

private

public wave_vert_mix_init
public wave_vert_mix
logical,public :: bv_is_use = .FALSE.


integer, parameter :: bv_daily = 1
integer, parameter :: bv_monthly = 2
logical :: module_is_initialized = .FALSE.
logical, save :: first_call = .TRUE.
integer, save :: mon_0, day_0
real, allocatable :: bv(:,:,:)
integer :: bv_type


contains

!#######################################################################
! <SUBROUTINE NAME="wave_vert_mix_init">
!
! <DESCRIPTION>
! Initialization for the wave-induced vertical mixing module
! </DESCRIPTION>
!
subroutine wave_vert_mix_init
character * 8 :: bv_use, bv_data_type
namelist /wave_vert_mix_nml/ bv_use, bv_data_type
integer :: ioun, io_status, ierr

  if ( module_is_initialized ) then
    call mpp_error(FATAL,'==>Error from ocean_vert_mix_mod (ocean_vert_mix_init)" module already initialized')
  endif

  module_is_initialized = .TRUE.

  ioun = open_namelist_file()
  read  (ioun, wave_vert_mix_nml,iostat=io_status)
  write (stdout(),'(/)')
  write (stdout(), wave_vert_mix_nml)
  write (stdlog(), wave_vert_mix_nml)
  ierr = check_nml_error(io_status, 'wave_vert_mix_nml')
  call close_file(ioun)

 select case( uppercase(trim(bv_use)) )
  case( 'YES' )
     bv_is_use = .true.
  case( 'NO' )
     bv_is_use = .false.
  case default
     call mpp_error( FATAL, 'wave_ver_mix: wave_ver_mix_nml entry bv_use must be one of YES|NO.' )
  end select

 select case( uppercase(trim(bv_data_type)) )
  case( 'MONTHLY' )
     bv_type = bv_monthly
  case( 'DAILY' )
     bv_type = bv_daily
  case default
     call mpp_error( FATAL, 'wave_ver_mix: wave_ver_mix_nml entry bv must be one of MONTHLY|DAILY.' )
  end select

  allocate (bv(0:imt_global+1,0:jmt_global+1,40))
  bv = 0.0

end subroutine wave_vert_mix_init
! </SUBROUTINE> NAME="wave_vert_mix_init"


!#######################################################################
! <SUBROUTINE NAME="wave_vert_mix">
!
! <DESCRIPTION>
! Initialization for the wave-induced vertical mixing module
! </DESCRIPTION>
!
subroutine wave_vert_mix(mon,day,isd,ied,jsd,jed,nk,kmt,zw,tbv)
include 'netcdf.inc'
integer, parameter :: N = 40
integer, intent(in) :: isd, ied, jsd, jed, nk
integer, intent(in) :: kmt(isd:ied,jsd:jed)
integer, intent(in) :: mon, day
real, intent(out) :: tbv(isd:ied,jsd:jed,nk)
real, intent(in) :: zw(nk)

real :: bv_global(0:imt_global+1,0:jmt_global+1,N)
real * 4 :: temp_bv(imt_global,jmt_global,N)
real * 4 :: missing_value
logical :: read_bv_file = .TRUE.

logical :: new_month, new_day


character * 2 :: s_month, s_day
character * 64 :: bv_file
integer :: ncid,id_bv,rcode
integer :: start(3), nread(3)
integer :: i, j, k

bv_global = 0.0
temp_bv = 0.0

if(bv_is_use) then
	if(first_call) then
	  first_call = .FALSE.
	  mon_0 = mon
	  day_0 = day
	  read_bv_file = .TRUE.
	else
	  if(mon_0 .ne. mon) then
	  	new_month = .true.
	  	mon_0 = mon
	  endif
	  if(day_0 .ne. day) then
	  	new_day = .true.
	  	day_0 = day
	  endif
	endif
  if(bv_type == bv_monthly) then
    write(s_month,'(i2.2)') mon
    bv_file = "INPUT/bv/monthly/bv_mom_"//s_month//".nc"
    if(new_month) read_bv_file = .true.
  else if(bv_type == bv_daily) then
  	write(s_month,'(i2.2)') mon
  	write(s_day,'(i2.2)') day
  	bv_file = "INPUT/bv/daily/bv_mom_"//s_month//s_day//".nc"
  	if(new_day) read_bv_file = .true.
  else
  	write (stdout(),*)'There is no bv_file '
  endif
!wtw  write (stdout(),*)'bv_file = ', trim(bv_file)
  if(read_bv_file) then
    rcode = nf_open(trim(bv_file), NF_NOWRITE, ncid)
    if(rcode .ne. nf_noerr) call handle_err(rcode)
    rcode = nf_inq_varid(ncid, 'bv', id_bv )
    if(rcode .ne. nf_noerr) call handle_err(rcode)

    start = (/1, 1, 1/)
    nread = (/imt_global, jmt_global, N/)
    rcode = nf_get_vara_real(ncid, id_bv, start, nread, temp_bv(:,:,:))
    if(rcode .ne. nf_noerr) call handle_err(rcode)
!    write (stdout(),*)'bv maxval 1', maxval(temp_bv),size(bv)

    rcode = nf_close(ncid)
    if(rcode .ne. nf_noerr) call handle_err(rcode)

    !--- get the missing value -------------------------------------------
    missing_value = 1e5
    where(abs(temp_bv) >= missing_value) temp_bv = 0.0
    !write(11+mpp_pe(),'(f8.7)') tbv(:,:,1)
    write (stdout(),*)'bv maxval ', maxval(temp_bv)
    bv_global(1:imt_global,1:jmt_global,:) = temp_bv(:,:,:)
    bv_global(:,0,:) = bv_global(:,1,:)
    bv_global(:,jmt_global+1,:) = bv_global(:,jmt_global,:)
    bv_global(0,:,:) = bv_global(imt_global,:,:)
    bv_global(imt_global+1,:,:) = bv_global(1,:,:)
!    write(stdout(),*) "bv before vertical interp", temp_bv(isd,jsd,:),bv(isd,jsd,:),zw,kmt(isd,jsd)
    call interp1d(bv_global(isd:ied,jsd:jed,:),bv(isd:ied,jsd:jed,1:nk),isd,ied,jsd,jed,nk,kmt,zw,N)
!    write(stdout(),*) "bv after vertical interp", tbv(isd,jsd,:)
    read_bv_file = .FALSE.
  endif
else
  bv = 0.0
endif
  tbv = bv(isd:ied,jsd:jed,1:nk)

end subroutine wave_vert_mix
! </SUBROUTINE> NAME="wave_vert_mix"

subroutine Handle_err(status)
implicit none
include 'netcdf.inc'
integer :: status
if(status .ne. nf_noerr) then
	write (stdout(),*) nf_strerror(status)
	stop 'stopped'
endif

end subroutine Handle_err


end module wave_vert_mix_mod



